!***************************************************************************
! PES program for OH3 
!
! Authors: Shanyu Han, Antonio Gustavo Sampaio de Oliveira Filho, Yinan Shu
! Dec. 2021
!
! Reference for this potential energy surface:
!   "Semiclassical Trajectory Studies of Reactive and Nonreactive Scattering
!    of OH(A2Sigma+) by H2 Based on an Improved Full-Dimensional Ab Initio
!    Diabatic Potential Energy Matrix"
!   by Shanyu Han, Antonio Gustavo Sampaio de Oliveira Filho,
!      Yinan Shu, Donald G. Truhlar, and Hua Guo
!   ChemPhysChem 2022, 23, e202200039
!
!***************************************************************************

!===========================================================================
! *def oh3_pes
!  1. Main PES program for calculating:
!     energies and gradients for diabatic states and diabatic couplings
!     energies, gradients and non-adiabatic couplings for adiabatic states 
!  2. The input file: ****.xyz (**** is a file name, for example, OH3;) 
!     !!!THE UNIT SHOULD BE ANGSTROM !!!
!     the format of ****.xyz should be a standard xyz format that is readbale
!     by Molden with 3 hydrogen atoms first, the oxygen at last, for example: 
!     4
!    
!     H  X Y Z 
!     H  X Y Z
!     H  X Y Z
!     O  X Y Z
!  3. Executing the program with the input, the program will generate 2 output 
!     they correspond to: ****.out and ****.tmp
!     ****.out has a detailed print out of energies, gradients and nonadiabatic 
!     couplings
!     ****.tmp has a three line simple  print of diabatic states, diabatic
!     couplings and
!     adiabatic states in the following format
!     u(1,1) u(2,2) u(3,3)
!     u(1,2) u(1,3) u(2,3)
!     a(1)   a(2)   a(3)
!  4. The zero of energy is: -76.80042447 hartree
!
!  ==========================definition of variables========================
!
!  4. xyz: the xyz coordinate
!          unit: in angstroms
!          xyz is a 4 by 3 matrix
!  5.  u : the diabatic matrix
!          unit: in hartrees
!          u is a 3 by 3 matrix
!          notice u(i,j)=u(j,i) for i not equal to j 
!          the diabatic states are diagonal elements of u 
!          the diabatic couplings are off-diagonal elements of u 
!  6.  gu: the gradients of diabatic matrix w.r.t. the xyz coordinates
!          unit: in hartrees/angstrom
!          gu is a 4 by 3 by 3 by 3 matrix
!          "gu(:,:,i,j)" is the gradient of u(i,j)
!  7.  a : the adiabatic states
!          unit: in hartrees
!          a is a 3 element vector 
!  8.  ga: the gradient of adiabatic states w.r.t. the xyz coordinates
!          unit: in hartrees/angstrom
!          ga is a 4 by 3 by 3 matrix
!          "ga(:,:,1)" is the gradient of the ground adiabatic state
!          "ga(:,:,2)" is the gradient of the first excited adiabatic state
!          "ga(:,:,3)" is the gradient of the second excited adiabatic state
!  9.  f : the non-adiabatic couplings
!          unit: in angstrom**(-1)
!          f is a 4 by 3 by 3 by 3 matrix
!          "f(:,:,i,j)" is the non-adiabatic coupling of state i and j
!          f(:,:,i,j)=-f(:,:,j,i), and f(:,:,i,i)=0 
!  10. rv: the diabatic to adiabatic rotation matrix 
!===========================================================================


  subroutine pes(x,igrad,p,g,d)

  implicit none
  ! number of electronic state
  integer, parameter :: nstates=3
  integer, parameter :: natoms=4
  integer, intent(in) :: igrad
  double precision, intent(in) :: x(natoms,3)
  double precision, intent(out) :: p(nstates), g(nstates,natoms,3)
  double precision, intent(out) :: d(nstates,nstates,natoms,3) 

  character (len=1) :: symb
  integer :: iatom,idir,istate,jstate
  integer :: i,j,k,l
  real*8 :: u11, u22, u33
  real*8 :: u12, u13, u23
  real*8 :: s12, s23
  real*8 :: xyz(4,3)
  character(len=32)::fname, bname
  real*8 :: du11dx(4,3), du22dx(4,3), du33dx(4,3)
  real*8 :: du12dx(4,3), du13dx(4,3), du23dx(4,3)
  real*8 :: gu(4,3,3,3)
  real*8 :: u(3,3),rv(3,3),a(3)
  real*8 :: ga(4,3,3),f(4,3,3,3)
  real*8 :: tmpgu(3,3),tmpga(3,3)


   xyz=x

!compute diabatic states
  call u11_prepot_8
  call u11_pot_8(xyz,u11,du11dx)
  call u22_prepot_8
  call u22_pot_8(xyz,u22,du22dx)
  call u33_prepot_8
  call u33_pot_8(xyz,u33,du33dx)

!compute diabatic couplings
  call u12_order10
  call u13_order12
  call u23_order10
  call u12_dc_10(xyz,u12,s12)
  call u13_dc_12(xyz,u13)
  call u23_dc_10(xyz,u23,s23)

!compute the adiabatic states and eigenvectors
  u(1,1)=u11
  u(1,2)=u12
  u(1,3)=u13
  u(2,1)=u12
  u(2,2)=u22
  u(2,3)=u23
  u(3,1)=u13
  u(3,2)=u23
  u(3,3)=u33
  call dsyevj3(u,rv,a)
  call order(a,rv)


  p=(a+76.80042447)*27.211386

  if (igrad.ne.0) then 

  !compute diabatic couplings derivatives
  call u12_dvdx(xyz,u12,s12,du12dx)
  call u13_dvdx(xyz,u13,du13dx)
  call u23_dvdx(xyz,u23,s23,du23dx)

  !assign dudx to gu

  do i=1,4
  do j=1,3
  gu(i,j,1,1)=du11dx(i,j)
  gu(i,j,1,2)=du12dx(i,j)
  gu(i,j,1,3)=du13dx(i,j)
  gu(i,j,2,1)=du12dx(i,j)
  gu(i,j,2,2)=du22dx(i,j)
  gu(i,j,2,3)=du23dx(i,j)
  gu(i,j,3,1)=du13dx(i,j)
  gu(i,j,3,2)=du23dx(i,j)
  gu(i,j,3,3)=du33dx(i,j)
  enddo
  enddo

!initialize the ga and f matrices
  do i=1,4
  do j=1,3
  do k=1,3
  do l=1,3
  ga(i,j,k)=0.000
  f(i,j,k,l)=0.000
  enddo
  enddo
  enddo
  enddo

!compute the adiabatic states gradients,nonadiabatic coupling vectors
  do i=1,4
  do j=1,3
  tmpgu(1,1)=du11dx(i,j)
  tmpgu(1,2)=du12dx(i,j)
  tmpgu(1,3)=du13dx(i,j)
  tmpgu(2,1)=du12dx(i,j)
  tmpgu(2,2)=du22dx(i,j)
  tmpgu(2,3)=du23dx(i,j)
  tmpgu(3,1)=du13dx(i,j)
  tmpgu(3,2)=du23dx(i,j)
  tmpgu(3,3)=du33dx(i,j)
  !C^T*dU*C
  tmpga = matmul(transpose(rv), matmul(tmpgu, rv))
  !dV_i = (C^T*dU*C)_ii
  do k=1,3
  ga(i,j,k)=tmpga(k,k)
  enddo
  !F_ij = (C^T*dU*C)_ij / (V_j-V_i)
  do k=1,2
  do l=k+1,3
  f(i,j,k,l)=tmpga(k,l)/(a(l)-a(k))
  f(i,j,l,k)=-f(i,j,k,l)
  enddo
  enddo
  enddo
  enddo

    do iatom=1,4
    do idir=1,3
    do istate=1,3
      g(istate,iatom,idir)=ga(iatom,idir,istate)*27.211386
    enddo
    enddo
    enddo 

    do iatom=1,4
    do idir=1,3
    do istate=1,3
    do jstate=1,3
      d(istate,jstate,iatom,idir)=f(iatom,idir,istate,jstate)
    enddo
    enddo
    enddo
    enddo

  endif 

end

!===========================================================================
! *def DSYEVJ3(A, Q, W)
!  subroutine of diagonalizing a 3 by 3 matrix
!  A is the input matirx
!  Q is the eigenvector matrix
!  W is the eigenvalue
!===========================================================================
SUBROUTINE DSYEVJ3(A, Q, W)
      DOUBLE PRECISION A(3,3)
      DOUBLE PRECISION Q(3,3)
      DOUBLE PRECISION W(3)

      INTEGER          N
      PARAMETER        ( N = 3 )
    
      DOUBLE PRECISION SD, SO
      DOUBLE PRECISION S, C, T
      DOUBLE PRECISION G, H, Z, THETA
      DOUBLE PRECISION THRESH
      INTEGER          I, X, Y, R

      DO 10 X = 1, N
        Q(X,X) = 1.0D0
        DO 11, Y = 1, X-1
          Q(X, Y) = 0.0D0
          Q(Y, X) = 0.0D0
   11   CONTINUE
   10 CONTINUE

      DO 20 X = 1, N
        W(X) = A(X, X)
   20 CONTINUE

      SD = 0.0D0
      DO 30 X = 1, N
        SD = SD + ABS(W(X))
   30 CONTINUE
      SD = SD**2
 
      DO 40 I = 1, 50
        SO = 0.0D0
        DO 50 X = 1, N
          DO 51 Y = X+1, N
            SO = SO + ABS(A(X, Y))
   51     CONTINUE
   50   CONTINUE
        IF (SO .EQ. 0.0D0) THEN
          RETURN
        END IF

        IF (I .LT. 4) THEN
          THRESH = 0.2D0 * SO / N**2
        ELSE
          THRESH = 0.0D0
        END IF

        DO 60 X = 1, N
          DO 61 Y = X+1, N
            G = 100.0D0 * ( ABS(A(X, Y)) )
            IF ( I .GT. 4 .AND. ABS(W(X)) + G .EQ. ABS(W(X)) .AND. ABS(W(Y)) + G .EQ. ABS(W(Y)) ) THEN
              A(X, Y) = 0.0D0
            ELSE IF (ABS(A(X, Y)) .GT. THRESH) THEN
              H = W(Y) - W(X)
              IF ( ABS(H) + G .EQ. ABS(H) ) THEN
                T = A(X, Y) / H
              ELSE
                THETA = 0.5D0 * H / A(X, Y)
                IF (THETA .LT. 0.0D0) THEN
                  T = -1.0D0 / (SQRT(1.0D0 + THETA**2) - THETA)
                ELSE
                  T = 1.0D0 / (SQRT(1.0D0 + THETA**2) + THETA)
                END IF
              END IF

              C = 1.0D0 / SQRT( 1.0D0 + T**2 )
              S = T * C
              Z = T * A(X, Y)
              
              A(X, Y) = 0.0D0
              W(X)    = W(X) - Z
              W(Y)    = W(Y) + Z
              DO 70 R = 1, X-1
                T       = A(R, X)
                A(R, X) = C * T - S * A(R, Y)
                A(R, Y) = S * T + C * A(R, Y)
   70         CONTINUE
              DO 80, R = X+1, Y-1
                T       = A(X, R)
                A(X, R) = C * T - S * A(R, Y)
                A(R, Y) = S * T + C * A(R, Y)
   80         CONTINUE
              DO 90, R = Y+1, N
                T       = A(X, R)
                A(X, R) = C * T - S * A(Y, R)
                A(Y, R) = S * T + C * A(Y, R)
   90         CONTINUE

              DO 100, R = 1, N
                T       = Q(R, X)
                Q(R, X) = C * T - S * Q(R, Y)
                Q(R, Y) = S * T + C * Q(R, Y)
  100         CONTINUE
            END IF
   61     CONTINUE
   60   CONTINUE
   40 CONTINUE

      PRINT *, "DSYEVJ3: No convergence."
            
END SUBROUTINE

!===========================================================================
! *def order(a,rv)
!  subroutine of ordering three eigenstates
!  a is the eigenvalue
!  rv is the eigenvector
!===========================================================================
subroutine order(a,rv)
double precision a(3)
double precision rv(3,3)
double precision tmpv(3,3)
double precision s0, s1, s2
integer i, j

do i=1,3
do j=1,3
tmpv(i,j)=rv(i,j)
enddo
enddo

    if (a(1) < a(2)) then
      if (a(1) < a(3)) then
        if (a(2) < a(3)) then
          s0=a(1)
          s1=a(2)
          s2=a(3)
          do i=1,3
            rv(i,1)=tmpv(i,1)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,2)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,3)
          enddo
        else
          s0=a(1)
          s1=a(3)
          s2=a(2)
          do i=1,3
            rv(i,1)=tmpv(i,1)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,3)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,2)
          enddo
        end if
      else
          s0=a(3)
          s1=a(1)
          s2=a(2)
          do i=1,3
            rv(i,1)=tmpv(i,3)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,1)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,2)
          enddo
      end if
    else
      if (a(2) < a(3)) then
        if (a(1) < a(3)) then
          s0=a(2)
          s1=a(1)
          s2=a(3)
          do i=1,3
            rv(i,1)=tmpv(i,2)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,1)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,3)
          enddo
        else
          s0=a(2)
          s1=a(3)
          s2=a(1)
          do i=1,3
            rv(i,1)=tmpv(i,2)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,3)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,1)
          enddo
        end if
      else
          s0=a(3)
          s1=a(2)
          s2=a(1)
          do i=1,3
            rv(i,1)=tmpv(i,3)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,2)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,1)
          enddo
      end if
    end if

a(1)=s0
a(2)=s1
a(3)=s2
end subroutine
!===========================================================================
! *def u12_dvdx(xyz,u12,s12,du12dx)
!  main subtoutine of evaluating derivative of u12 w.r.t. the xyz coordinates
!  xyz is the coordinate
!  u12 is u12
!  s12 is u12 devided by the hyperbolic function
!  du12dx is the gradient of u12
!===========================================================================
subroutine u12_dvdx(xyz,u12,s12,du12dx)
implicit none
integer, parameter:: dp=kind(0.d0)   
integer :: i,j,k
real(dp) :: du12dx(4,3),xyz(4,3)
real(dp) :: u12,s12
real(dp) :: coord(12), r(6)
real(dp) :: dudx(12)
real(dp) :: dvdx(12), dvdr(6), drdx(6,12)
real(dp) :: dtdx(12)
real(dp) :: volume

! Initialize dVdX(12)
      do i=1,12
        dVdX(i)=0.0d0
      enddo

! Initialize dRdX(6,12)
      do i=1,6
        do j=1,12
          dRdX(i,j)=0.0d0
        enddo
      enddo

! Initialize dVdR(6)
      do i=1,6
        dVdR(i)=0.0d0
      enddo

! Assign xyz to coord
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        coord(k)=xyz(i,j)
      enddo
      enddo

! Compute distances r
r(1)=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r(2)=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r(3)=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r(4)=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r(5)=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r(6)=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

! Call EvdVdR to evaluate dVdR(6)
      Call EvU12O10dVdR(coord,r,dvdr)

! Call EvdRdX to evaluate dRdX(6,12)
      Call EvdRdX(coord,r,drdx)

! Calculate dVdX by using chain rule: dV/dXi=(dV/dRj)*(dRj/dXi), j=1 to 6
      do i=1,12
        do j=1,6
          dVdX(i)=dVdX(i) + dVdR(j)*dRdX(j,i)
        enddo
      enddo

      do i=1,12
          dVdX(i)=dVdX(i)/627.509474
      enddo

  call P_volume(xyz,volume)

! Now compute numerical dtdx
  call Num_dtdx(xyz,dtdx)

! Chain rule again, because U=tanh*V
! dUdX=dtanhdX*V+tanh*dVdX
do i=1,12
  dudx(i)=0
enddo

do i=1,12
  dudx(i)=dtdx(i)*s12+volume*dvdx(i)
enddo


! Assign dvdx back to du12dx
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        du12dx(i,j)=dudx(k)
      enddo
      enddo

      return

end subroutine u12_dvdx 

!===========================================================================
! *def EvU12O10dVdR(coord,r,dvdr)
!  subtoutine of evaluating derivative of u12 w.r.t. the 6 nuclear distances
!  coord is the xyz coordinate 
!  r is the nuclear distances
!  dvdr is the derivative of u12 w.r.t. the 6 nuclear distances
!===========================================================================
subroutine EvU12O10dVdR(coord,r,dvdr)
integer, parameter:: ddp=kind(0.d0)
integer :: i,j
real(ddp) :: coord(12),r(6),dvdr(6)
real(ddp) :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=1300
real(ddp) :: coeff(ncoeff)
real(ddp),dimension(0:32) :: rm
real(ddp),dimension(0:32) :: dm
real(ddp),dimension(0:1506) :: p
real(ddp),dimension(0:1506) :: dp
real(ddp) :: x(6),dmsdr(6,6)
real(ddp) :: dbm1(1506), dbm2(1300)

! Assign coefficients first
a1=0.610
a2=0.800
b1=1.300
b2=1.300
ra1=0.802
ra2=0.702
rb1=0.802
rb2=0.702
coeff(1)=-0.200894146381D+03
coeff(2)=0.928472731432D+03
coeff(3)=-0.261022613087D+03
coeff(4)=-0.539380483332D+03
coeff(5)=0.101994742299D+04
coeff(6)=0.647330515917D+03
coeff(7)=0.675393110624D+03
coeff(8)=0.899938890523D+03
coeff(9)=-0.235323238714D+04
coeff(10)=-0.102800493753D+04
coeff(11)=0.933981557311D+02
coeff(12)=-0.171628272374D+04
coeff(13)=0.201570736612D+03
coeff(14)=-0.426825593199D+02
coeff(15)=0.673922176155D+03
coeff(16)=-0.483927233291D+03
coeff(17)=0.867741279535D+03
coeff(18)=0.142255818055D+04
coeff(19)=-0.427373733305D+04
coeff(20)=-0.784412615099D+03
coeff(21)=-0.155530579244D+04
coeff(22)=-0.155883450796D+04
coeff(23)=-0.654639670041D+03
coeff(24)=0.181115660004D+04
coeff(25)=-0.244496279907D+04
coeff(26)=-0.165264596406D+04
coeff(27)=0.737035495930D+02
coeff(28)=0.141620626401D+04
coeff(29)=0.269991251659D+04
coeff(30)=-0.676458531477D+03
coeff(31)=-0.169154008032D+04
coeff(32)=-0.944404764593D+03
coeff(33)=0.340855836109D+04
coeff(34)=0.226704552355D+03
coeff(35)=0.246754054573D+04
coeff(36)=0.148288275501D+03
coeff(37)=0.233408841386D+04
coeff(38)=0.318853150581D+03
coeff(39)=0.238613613333D+04
coeff(40)=-0.876239994343D+03
coeff(41)=0.112504579845D+04
coeff(42)=0.506300363576D+03
coeff(43)=0.638915481375D+03
coeff(44)=-0.623598752472D+03
coeff(45)=0.449543177665D+03
coeff(46)=0.589381722221D+03
coeff(47)=-0.200968926514D+04
coeff(48)=-0.186832614839D+04
coeff(49)=-0.182206810242D+04
coeff(50)=0.212950142336D+04
coeff(51)=0.186957763652D+04
coeff(52)=0.989689773718D+03
coeff(53)=-0.104938016545D+04
coeff(54)=-0.623003253753D+03
coeff(55)=0.862418465912D+04
coeff(56)=-0.214380543403D+04
coeff(57)=0.235459109660D+04
coeff(58)=0.736192412255D+03
coeff(59)=0.325410137455D+03
coeff(60)=-0.205469280833D+03
coeff(61)=0.130184460755D+03
coeff(62)=-0.108472886090D+04
coeff(63)=0.399737276425D+03
coeff(64)=-0.167176651975D+03
coeff(65)=-0.141072947461D+04
coeff(66)=0.444092859290D+03
coeff(67)=0.123658740994D+04
coeff(68)=-0.226202667988D+03
coeff(69)=0.352138440560D+02
coeff(70)=-0.483844332811D+03
coeff(71)=0.264395043830D+04
coeff(72)=-0.183085592243D+04
coeff(73)=-0.107346947910D+04
coeff(74)=-0.734823867609D+03
coeff(75)=0.117033290131D+04
coeff(76)=-0.169258470030D+04
coeff(77)=-0.119438319169D+04
coeff(78)=0.788790342290D+02
coeff(79)=-0.133808310276D+04
coeff(80)=-0.142161213940D+04
coeff(81)=-0.993662708023D+03
coeff(82)=0.734629534896D+03
coeff(83)=-0.161178090757D+04
coeff(84)=-0.888189147482D+03
coeff(85)=-0.218372314811D+04
coeff(86)=0.343491747509D+04
coeff(87)=-0.236749781172D+03
coeff(88)=0.325758505660D+03
coeff(89)=-0.875218917296D+03
coeff(90)=-0.120235160216D+04
coeff(91)=0.683181195726D+03
coeff(92)=0.523282563065D+02
coeff(93)=0.129950054147D+03
coeff(94)=-0.134769267040D+04
coeff(95)=0.231629678179D+04
coeff(96)=-0.415227960771D+04
coeff(97)=-0.179195717636D+03
coeff(98)=0.765656978143D+03
coeff(99)=-0.965862571783D+03
coeff(100)=0.135201263341D+04
coeff(101)=-0.510678236717D+03
coeff(102)=-0.152720550294D+04
coeff(103)=0.133169086131D+03
coeff(104)=-0.220644036625D+03
coeff(105)=-0.163708057120D+03
coeff(106)=-0.703466042263D+03
coeff(107)=-0.162495506707D+04
coeff(108)=0.711166708732D+03
coeff(109)=-0.747350308009D+03
coeff(110)=0.190408564193D+04
coeff(111)=0.592613801639D+03
coeff(112)=0.521056675424D+04
coeff(113)=-0.280208388985D+04
coeff(114)=0.224666365548D+04
coeff(115)=0.725205768488D+03
coeff(116)=0.646228531058D+03
coeff(117)=-0.103535653021D+03
coeff(118)=-0.123554448405D+03
coeff(119)=-0.303225918119D+04
coeff(120)=0.805717049353D+03
coeff(121)=-0.355539982112D+03
coeff(122)=0.945988029053D+03
coeff(123)=-0.655811716051D+03
coeff(124)=0.583955330275D+03
coeff(125)=0.611619773338D+03
coeff(126)=-0.468866760034D+04
coeff(127)=0.157548388637D+03
coeff(128)=-0.729793420938D+03
coeff(129)=-0.145369942131D+04
coeff(130)=0.126799369428D+04
coeff(131)=-0.667404937619D+03
coeff(132)=0.914719526679D+03
coeff(133)=0.322150888689D+02
coeff(134)=-0.538667256852D+03
coeff(135)=-0.206725654102D+03
coeff(136)=-0.149140806794D+04
coeff(137)=-0.151596074474D+04
coeff(138)=0.141871203235D+03
coeff(139)=0.143592027921D+03
coeff(140)=0.143880424018D+04
coeff(141)=0.170738383011D+03
coeff(142)=0.343896560635D+03
coeff(143)=0.638029490904D+03
coeff(144)=-0.112636907440D+03
coeff(145)=0.214222335835D+04
coeff(146)=-0.523222254632D+03
coeff(147)=-0.140579762449D+03
coeff(148)=-0.351804416562D+03
coeff(149)=-0.888396679338D+03
coeff(150)=0.434711683128D+03
coeff(151)=0.867984744801D+03
coeff(152)=0.331629185490D+03
coeff(153)=0.521630120106D+03
coeff(154)=-0.542106566074D+03
coeff(155)=0.795716722331D+03
coeff(156)=0.710700065662D+03
coeff(157)=-0.152090367813D+04
coeff(158)=0.413206522727D+03
coeff(159)=-0.520312375797D+03
coeff(160)=-0.110915179801D+03
coeff(161)=0.204693037890D+03
coeff(162)=0.373821957710D+03
coeff(163)=0.133096503606D+04
coeff(164)=0.196348207409D+04
coeff(165)=0.288120718428D+03
coeff(166)=0.356128093066D+03
coeff(167)=0.331838064532D+03
coeff(168)=0.146026855084D+04
coeff(169)=-0.195553151498D+03
coeff(170)=-0.117756698159D+04
coeff(171)=-0.171550596151D+03
coeff(172)=-0.130759930834D+04
coeff(173)=-0.445191810623D+03
coeff(174)=-0.173737607500D+04
coeff(175)=0.687299557843D+03
coeff(176)=0.706403699237D+03
coeff(177)=-0.744779260936D+03
coeff(178)=-0.646267841442D+03
coeff(179)=-0.811729433516D+02
coeff(180)=-0.193988487167D+04
coeff(181)=0.633356171360D+03
coeff(182)=-0.867221391780D+03
coeff(183)=-0.125897020563D+04
coeff(184)=0.404060352679D+03
coeff(185)=-0.280889332955D+04
coeff(186)=0.240531094170D+03
coeff(187)=-0.216103417754D+01
coeff(188)=0.601430406874D+03
coeff(189)=-0.207029470562D+04
coeff(190)=-0.543435091641D+03
coeff(191)=-0.243916614522D+04
coeff(192)=0.109598685360D+04
coeff(193)=0.645162644262D+03
coeff(194)=0.655697219345D+03
coeff(195)=-0.604903522140D+03
coeff(196)=-0.107827290966D+04
coeff(197)=-0.191719074532D+03
coeff(198)=-0.302041540440D+03
coeff(199)=-0.355405487572D+03
coeff(200)=0.615023569188D+03
coeff(201)=0.124155541609D+04
coeff(202)=0.258695630266D+03
coeff(203)=-0.401480283518D+03
coeff(204)=0.100072827158D+04
coeff(205)=-0.102976277956D+04
coeff(206)=0.150123586377D+03
coeff(207)=-0.637287736160D+03
coeff(208)=-0.574794457263D+03
coeff(209)=-0.119233079810D+04
coeff(210)=-0.109843119367D+04
coeff(211)=-0.102908495195D+04
coeff(212)=-0.662602554218D+02
coeff(213)=0.489870096551D+03
coeff(214)=-0.751847472718D+03
coeff(215)=-0.776415366781D+03
coeff(216)=-0.527277102837D+03
coeff(217)=-0.403547176807D+02
coeff(218)=-0.739812735654D+01
coeff(219)=0.262775980485D+03
coeff(220)=0.392616254888D+02
coeff(221)=-0.812206587196D+03
coeff(222)=0.122742838024D+04
coeff(223)=-0.764260201236D+03
coeff(224)=-0.228614614402D+02
coeff(225)=0.518227998892D+03
coeff(226)=0.193496684816D+04
coeff(227)=-0.273066697453D+03
coeff(228)=-0.725380271752D+03
coeff(229)=0.369109280008D+03
coeff(230)=-0.554039003548D+02
coeff(231)=-0.450553396709D+03
coeff(232)=0.226272404697D+03
coeff(233)=0.765893207415D+03
coeff(234)=-0.110874166091D+04
coeff(235)=-0.215106244001D+04
coeff(236)=0.816626550797D+03
coeff(237)=-0.791092175399D+03
coeff(238)=0.388689940385D+03
coeff(239)=0.699668741983D+03
coeff(240)=-0.164500177961D+04
coeff(241)=-0.187038976801D+04
coeff(242)=0.600680605931D+03
coeff(243)=0.163282488546D+04
coeff(244)=0.299455560877D+04
coeff(245)=0.565737355058D+03
coeff(246)=-0.178734131557D+04
coeff(247)=-0.365618527190D+04
coeff(248)=0.147887247668D+04
coeff(249)=-0.235421917192D+03
coeff(250)=-0.813363491057D+03
coeff(251)=-0.969058058779D+03
coeff(252)=-0.215789755499D+04
coeff(253)=0.233340738590D+04
coeff(254)=-0.683119251331D+03
coeff(255)=0.578809963854D+02
coeff(256)=-0.915318425202D+03
coeff(257)=0.901603106348D+03
coeff(258)=0.828814047928D+03
coeff(259)=0.615349573822D+03
coeff(260)=0.527217977439D+03
coeff(261)=0.967204713220D+03
coeff(262)=0.238558368661D+03
coeff(263)=-0.479955026989D+03
coeff(264)=-0.403638464503D+03
coeff(265)=-0.102797104104D+04
coeff(266)=-0.827124260843D+03
coeff(267)=0.115387476210D+03
coeff(268)=-0.515873671862D+03
coeff(269)=-0.293002428734D+03
coeff(270)=0.508032401944D+03
coeff(271)=-0.103047877262D+04
coeff(272)=0.890738386657D+03
coeff(273)=0.828553053434D+03
coeff(274)=-0.375657226633D+03
coeff(275)=0.186456532104D+04
coeff(276)=0.984441155653D+03
coeff(277)=-0.877572070872D+03
coeff(278)=0.951474515645D+03
coeff(279)=-0.103451129958D+04
coeff(280)=0.282312445083D+03
coeff(281)=0.466225814165D+02
coeff(282)=0.154658379689D+04
coeff(283)=0.122789252503D+04
coeff(284)=0.260060004925D+03
coeff(285)=0.205501411752D+04
coeff(286)=0.110306699892D+04
coeff(287)=-0.917039012542D+03
coeff(288)=0.576611060955D+03
coeff(289)=0.415142439150D+03
coeff(290)=-0.141661903790D+04
coeff(291)=0.104650805634D+04
coeff(292)=-0.565009767167D+02
coeff(293)=0.706000553461D+03
coeff(294)=0.110389054648D+04
coeff(295)=-0.580662972080D+03
coeff(296)=0.164126287102D+04
coeff(297)=-0.117362142784D+03
coeff(298)=-0.149586544576D+04
coeff(299)=0.444078089560D+03
coeff(300)=0.714031242900D+03
coeff(301)=0.127569062970D+04
coeff(302)=-0.815004933142D+03
coeff(303)=-0.200408526616D+03
coeff(304)=-0.109337385682D+04
coeff(305)=-0.375115315414D+03
coeff(306)=-0.163747780822D+03
coeff(307)=-0.610925607666D+03
coeff(308)=0.159945093873D+04
coeff(309)=0.732222462123D+03
coeff(310)=0.734388032520D+03
coeff(311)=-0.997506333285D+03
coeff(312)=-0.287706522688D+03
coeff(313)=0.178956389419D+04
coeff(314)=0.711334699833D+03
coeff(315)=0.702902388406D+03
coeff(316)=0.265059183422D+03
coeff(317)=0.294521896644D+03
coeff(318)=-0.917895768273D+03
coeff(319)=0.173697825919D+04
coeff(320)=-0.538653526579D+02
coeff(321)=-0.118945502938D+03
coeff(322)=-0.497197798650D+03
coeff(323)=0.115094733875D+03
coeff(324)=0.807005375994D+03
coeff(325)=-0.109067760430D+04
coeff(326)=-0.997367502542D+02
coeff(327)=0.386789371762D+03
coeff(328)=-0.710112939863D+03
coeff(329)=0.594666995642D+03
coeff(330)=-0.212142037563D+03
coeff(331)=0.496222702748D+02
coeff(332)=0.150769143441D+03
coeff(333)=0.119752869441D+04
coeff(334)=-0.773015119993D+03
coeff(335)=0.104793566531D+03
coeff(336)=-0.303063663809D+03
coeff(337)=-0.123111783563D+04
coeff(338)=0.104113220732D+04
coeff(339)=-0.128125941323D+03
coeff(340)=-0.663444508785D+03
coeff(341)=0.144018364126D+04
coeff(342)=0.402697329492D+03
coeff(343)=0.480736871685D+03
coeff(344)=-0.831966474099D+03
coeff(345)=-0.152861557954D+04
coeff(346)=-0.166818456682D+02
coeff(347)=-0.114133211609D+04
coeff(348)=0.272323226168D+03
coeff(349)=0.277106837249D+03
coeff(350)=-0.360293667318D+03
coeff(351)=0.902991974160D+03
coeff(352)=0.187459087864D+04
coeff(353)=-0.485188980960D+03
coeff(354)=-0.332195464323D+03
coeff(355)=0.383494096550D+03
coeff(356)=-0.238605308839D+04
coeff(357)=0.459660093879D+03
coeff(358)=0.133821259985D+04
coeff(359)=0.134343031413D+04
coeff(360)=-0.127270918304D+04
coeff(361)=0.728823250699D+03
coeff(362)=-0.181795228771D+04
coeff(363)=-0.531202184175D+03
coeff(364)=-0.611068123602D+02
coeff(365)=0.111597848440D+04
coeff(366)=0.592271799368D+03
coeff(367)=-0.272340952710D+03
coeff(368)=0.291732395295D+03
coeff(369)=0.997992961141D+03
coeff(370)=0.752427070673D+03
coeff(371)=0.116879825892D+04
coeff(372)=-0.226492037080D+04
coeff(373)=0.706536608610D+02
coeff(374)=0.117211504329D+04
coeff(375)=0.164871862358D+04
coeff(376)=0.150177348742D+04
coeff(377)=0.122552880984D+04
coeff(378)=0.833942293197D+01
coeff(379)=-0.144562096311D+04
coeff(380)=0.108610304952D+04
coeff(381)=0.115845888720D+04
coeff(382)=-0.303114876624D+03
coeff(383)=-0.616893081338D+03
coeff(384)=0.229094909894D+03
coeff(385)=0.120724798904D+04
coeff(386)=0.114325641937D+04
coeff(387)=-0.601175020694D+03
coeff(388)=-0.106757585888D+04
coeff(389)=-0.130709707153D+04
coeff(390)=0.161883051938D+02
coeff(391)=-0.747973705109D+03
coeff(392)=0.175601704664D+04
coeff(393)=0.133209253299D+04
coeff(394)=-0.860039085035D+03
coeff(395)=-0.696526399350D+03
coeff(396)=0.558222931165D+03
coeff(397)=-0.964634988739D+03
coeff(398)=-0.108401713961D+04
coeff(399)=0.150355363499D+04
coeff(400)=0.195005627260D+04
coeff(401)=-0.119425105391D+03
coeff(402)=-0.963239464768D+03
coeff(403)=0.183169576071D+04
coeff(404)=0.861432942401D+03
coeff(405)=-0.823569238545D+03
coeff(406)=-0.156794901970D+04
coeff(407)=0.133151445989D+04
coeff(408)=0.674267200599D+03
coeff(409)=-0.645159550604D+03
coeff(410)=0.161649331238D+04
coeff(411)=0.732457298162D+03
coeff(412)=0.160560300437D+03
coeff(413)=-0.292196795980D+03
coeff(414)=-0.398542514294D+02
coeff(415)=0.202041467094D+04
coeff(416)=0.571721782022D+03
coeff(417)=-0.285549174304D+03
coeff(418)=0.307213237068D+02
coeff(419)=0.174105889266D+03
coeff(420)=-0.291680491586D+03
coeff(421)=-0.268685548781D+04
coeff(422)=-0.126505882604D+04
coeff(423)=0.385265788397D+03
coeff(424)=0.115016053641D+04
coeff(425)=-0.163453738874D+04
coeff(426)=0.936310771331D+01
coeff(427)=-0.175710163164D+03
coeff(428)=-0.170833884633D+04
coeff(429)=-0.153747078616D+03
coeff(430)=-0.511240193455D+03
coeff(431)=0.358595282594D+03
coeff(432)=-0.406710982140D+04
coeff(433)=0.141516174424D+04
coeff(434)=0.118306285456D+02
coeff(435)=0.366870943342D+03
coeff(436)=-0.590909258567D+03
coeff(437)=0.144739072535D+04
coeff(438)=0.328343416805D+03
coeff(439)=0.291703134667D+02
coeff(440)=0.333237942269D+03
coeff(441)=0.489995424913D+03
coeff(442)=-0.109229818515D+04
coeff(443)=0.319085386891D+04
coeff(444)=-0.272190042873D+03
coeff(445)=-0.424021824758D+03
coeff(446)=-0.170416402581D+04
coeff(447)=-0.110358676803D+04
coeff(448)=0.165430956456D+03
coeff(449)=-0.386023778652D+03
coeff(450)=-0.531851966314D+03
coeff(451)=-0.890017961352D+03
coeff(452)=-0.123308947404D+04
coeff(453)=0.254669121177D+03
coeff(454)=0.485681682270D+04
coeff(455)=-0.569509725803D+03
coeff(456)=0.711128996725D+03
coeff(457)=0.930661054566D+03
coeff(458)=0.141812201479D+04
coeff(459)=-0.492867496407D+02
coeff(460)=-0.104363883497D+04
coeff(461)=0.273694374072D+04
coeff(462)=0.220304340529D+03
coeff(463)=0.183300412904D+03
coeff(464)=0.664334681884D+03
coeff(465)=0.353027487652D+03
coeff(466)=-0.267288987777D+03
coeff(467)=-0.482241049121D+03
coeff(468)=-0.110217883940D+03
coeff(469)=0.322264660023D+03
coeff(470)=-0.217399469820D+03
coeff(471)=0.242575232684D+02
coeff(472)=-0.109734034213D+04
coeff(473)=-0.398257155806D+03
coeff(474)=-0.574697619066D+03
coeff(475)=-0.729462388898D+03
coeff(476)=-0.736623294831D+03
coeff(477)=0.334236097737D+02
coeff(478)=-0.126937524889D+04
coeff(479)=-0.119193622826D+04
coeff(480)=-0.870017491909D+03
coeff(481)=-0.681954378432D+03
coeff(482)=0.106167566292D+04
coeff(483)=0.118520710382D+04
coeff(484)=-0.919777051739D+03
coeff(485)=0.101054492837D+04
coeff(486)=0.931885890355D+03
coeff(487)=0.199666062577D+03
coeff(488)=0.191948779147D+03
coeff(489)=0.440614337404D+03
coeff(490)=0.155589770254D+04
coeff(491)=0.488336383020D+03
coeff(492)=-0.476504103188D+03
coeff(493)=-0.696991352970D+03
coeff(494)=-0.106328689179D+04
coeff(495)=0.497922173936D+03
coeff(496)=-0.476666575521D+03
coeff(497)=-0.129787759381D+04
coeff(498)=0.571043074522D+03
coeff(499)=-0.334729399903D+03
coeff(500)=-0.223179165024D+03
coeff(501)=0.445892726440D+03
coeff(502)=0.744070320545D+03
coeff(503)=-0.561246657194D+03
coeff(504)=-0.911481598494D+03
coeff(505)=-0.215654857975D+03
coeff(506)=-0.196356529005D+04
coeff(507)=-0.886295470882D+03
coeff(508)=0.432484657908D+03
coeff(509)=0.118116483286D+04
coeff(510)=0.288121004311D+03
coeff(511)=-0.273300884384D+03
coeff(512)=-0.259969247987D+03
coeff(513)=-0.657470456907D+00
coeff(514)=-0.693063655201D+02
coeff(515)=0.137726966176D+04
coeff(516)=0.468654649420D+03
coeff(517)=-0.166913198066D+04
coeff(518)=-0.759182341524D+03
coeff(519)=0.202100237163D+02
coeff(520)=-0.113903757439D+04
coeff(521)=-0.710507800932D+03
coeff(522)=-0.202666148906D+04
coeff(523)=0.533055799192D+03
coeff(524)=-0.196039560712D+03
coeff(525)=-0.409029103950D+03
coeff(526)=-0.123649522596D+04
coeff(527)=-0.130243677819D+04
coeff(528)=-0.434542640413D+03
coeff(529)=0.274808896035D+03
coeff(530)=0.407656503257D+03
coeff(531)=-0.688392082085D+03
coeff(532)=-0.459875362577D+03
coeff(533)=0.432578162687D+03
coeff(534)=0.105931172699D+04
coeff(535)=0.155156224614D+04
coeff(536)=-0.221859508320D+03
coeff(537)=-0.141118210955D+04
coeff(538)=-0.853604283865D+03
coeff(539)=-0.124270468709D+04
coeff(540)=0.133070209273D+04
coeff(541)=-0.452590412670D+03
coeff(542)=-0.183509101733D+03
coeff(543)=0.806928303707D+03
coeff(544)=-0.247773330985D+04
coeff(545)=-0.477528152949D+03
coeff(546)=-0.551733973202D+03
coeff(547)=0.181617550170D+04
coeff(548)=-0.216829867454D+04
coeff(549)=0.678477870304D+03
coeff(550)=0.910554148127D+03
coeff(551)=0.564110892140D+03
coeff(552)=0.482630352812D+03
coeff(553)=-0.109430716115D+04
coeff(554)=-0.247453220057D+04
coeff(555)=0.182404973935D+04
coeff(556)=0.402958113233D+03
coeff(557)=0.122165175710D+04
coeff(558)=0.542396357702D+03
coeff(559)=-0.404555791448D+02
coeff(560)=-0.898418720803D+03
coeff(561)=-0.104026267941D+03
coeff(562)=0.191178910774D+04
coeff(563)=0.481413517018D+03
coeff(564)=-0.176500661637D+04
coeff(565)=-0.477412927288D+03
coeff(566)=-0.106917580421D+04
coeff(567)=-0.272967117890D+03
coeff(568)=-0.415654891599D+02
coeff(569)=0.323727531398D+03
coeff(570)=-0.104436979345D+04
coeff(571)=0.918317346619D+03
coeff(572)=-0.667033790216D+03
coeff(573)=-0.435970202404D+03
coeff(574)=-0.365538033567D+03
coeff(575)=0.741866656973D+02
coeff(576)=-0.213753280023D+04
coeff(577)=0.113644402142D+04
coeff(578)=-0.631013525921D+03
coeff(579)=-0.276658416152D+04
coeff(580)=-0.151784147768D+03
coeff(581)=0.486728796989D+03
coeff(582)=-0.310236882771D+03
coeff(583)=0.381767791178D+03
coeff(584)=-0.648471686890D+03
coeff(585)=0.685319873424D+03
coeff(586)=-0.152892098213D+04
coeff(587)=-0.987710749220D+03
coeff(588)=0.110412207467D+03
coeff(589)=-0.104120923245D+04
coeff(590)=0.601688981211D+03
coeff(591)=0.102907866531D+04
coeff(592)=0.268539591175D+03
coeff(593)=0.142527339436D+04
coeff(594)=0.572688649201D+03
coeff(595)=-0.836020173812D+03
coeff(596)=0.379828586053D+03
coeff(597)=0.373927219535D+03
coeff(598)=0.188017519963D+03
coeff(599)=0.545351816777D+03
coeff(600)=-0.146526773826D+04
coeff(601)=-0.812996330025D+03
coeff(602)=0.167358059918D+04
coeff(603)=0.127391808967D+04
coeff(604)=0.784290292303D+03
coeff(605)=-0.450889307660D+03
coeff(606)=-0.378586398769D+03
coeff(607)=0.152213100397D+04
coeff(608)=0.328446964726D+03
coeff(609)=-0.582238241183D+03
coeff(610)=-0.887236493377D+03
coeff(611)=0.431346099543D+03
coeff(612)=-0.224972197644D+03
coeff(613)=0.962309764252D+03
coeff(614)=-0.215408013477D+03
coeff(615)=-0.409144714174D+03
coeff(616)=-0.559821532716D+03
coeff(617)=-0.834111975926D+03
coeff(618)=0.687762150873D+03
coeff(619)=0.895125541943D+03
coeff(620)=-0.473542723079D+03
coeff(621)=0.839273770121D+02
coeff(622)=-0.130814952490D+02
coeff(623)=-0.462232124995D+03
coeff(624)=-0.178183063527D+02
coeff(625)=0.167686010390D+04
coeff(626)=0.708275026070D+03
coeff(627)=0.126898063922D+02
coeff(628)=0.587628117112D+02
coeff(629)=0.557430997762D+03
coeff(630)=-0.324110747332D+03
coeff(631)=0.166302415601D+03
coeff(632)=-0.105443223548D+04
coeff(633)=0.448554201091D+03
coeff(634)=-0.174447193055D+03
coeff(635)=-0.468055913492D+03
coeff(636)=-0.417988013706D+03
coeff(637)=0.846787981095D+03
coeff(638)=-0.500475884073D+03
coeff(639)=-0.436114643411D+03
coeff(640)=0.239415826614D+03
coeff(641)=0.908231766583D+03
coeff(642)=0.264613351005D+03
coeff(643)=0.288614833348D+03
coeff(644)=-0.210219607069D+03
coeff(645)=0.106580587714D+04
coeff(646)=0.526409114921D+03
coeff(647)=-0.736249843968D+03
coeff(648)=-0.339503480133D+02
coeff(649)=0.603429707038D+03
coeff(650)=0.187694149708D+04
coeff(651)=0.222377875231D+03
coeff(652)=0.196411778222D+04
coeff(653)=-0.728277545729D+03
coeff(654)=-0.668068870282D+02
coeff(655)=0.743759509460D+03
coeff(656)=0.170009589387D+03
coeff(657)=-0.310818251140D+03
coeff(658)=-0.100649360668D+04
coeff(659)=0.102643247869D+04
coeff(660)=-0.864078998130D+03
coeff(661)=0.114877393754D+04
coeff(662)=0.196738365345D+04
coeff(663)=-0.134501632890D+04
coeff(664)=-0.945872693227D+03
coeff(665)=0.870835003943D+03
coeff(666)=0.121064432358D+04
coeff(667)=0.541968625873D+03
coeff(668)=0.426737159319D+03
coeff(669)=-0.542892752873D+03
coeff(670)=-0.378199814788D+03
coeff(671)=-0.145496008416D+04
coeff(672)=0.134261933989D+04
coeff(673)=0.434064280967D+01
coeff(674)=0.670163862814D+03
coeff(675)=-0.146073263958D+04
coeff(676)=0.428079867303D+03
coeff(677)=0.820360259395D+03
coeff(678)=-0.130304812150D+03
coeff(679)=-0.141022391841D+04
coeff(680)=-0.214232126866D+04
coeff(681)=-0.648977147044D+03
coeff(682)=0.136332703641D+04
coeff(683)=-0.611149219657D+03
coeff(684)=0.114886551935D+04
coeff(685)=0.112448354779D+04
coeff(686)=0.644345401165D+03
coeff(687)=-0.112295175333D+04
coeff(688)=0.553130893649D+03
coeff(689)=-0.142538496464D+04
coeff(690)=-0.627398969291D+03
coeff(691)=0.291344570937D+04
coeff(692)=-0.101294963862D+04
coeff(693)=0.154649595301D+04
coeff(694)=-0.123781545726D+04
coeff(695)=0.744022296821D+03
coeff(696)=-0.426493526811D+03
coeff(697)=-0.103445729493D+04
coeff(698)=-0.126418582284D+04
coeff(699)=-0.260144118684D+04
coeff(700)=-0.378539491072D+03
coeff(701)=-0.863605731108D+03
coeff(702)=-0.341205593745D+02
coeff(703)=-0.988450757460D+03
coeff(704)=0.401754972535D+03
coeff(705)=-0.114611135973D+03
coeff(706)=0.894044562449D+02
coeff(707)=-0.184638575268D+03
coeff(708)=-0.171528081185D+04
coeff(709)=0.840898735986D+03
coeff(710)=0.212964728845D+03
coeff(711)=-0.668835959301D+03
coeff(712)=0.433808563325D+03
coeff(713)=-0.740048058179D+03
coeff(714)=0.111829357105D+04
coeff(715)=-0.845759237421D+03
coeff(716)=-0.269656040685D+03
coeff(717)=0.442905594231D+03
coeff(718)=0.107572603149D+04
coeff(719)=-0.204645607594D+03
coeff(720)=-0.150026658364D+03
coeff(721)=-0.479738416366D+03
coeff(722)=-0.274090404112D+03
coeff(723)=0.176186699440D+04
coeff(724)=-0.123779714692D+04
coeff(725)=-0.131561864553D+04
coeff(726)=-0.121536957722D+03
coeff(727)=0.300689377521D+03
coeff(728)=0.105636961740D+03
coeff(729)=-0.952261995960D+03
coeff(730)=-0.966399108389D+03
coeff(731)=0.172820582872D+03
coeff(732)=-0.304698537128D+03
coeff(733)=-0.750588809057D+02
coeff(734)=0.164182917388D+04
coeff(735)=0.109080442129D+03
coeff(736)=0.154651572753D+04
coeff(737)=0.274782124870D+03
coeff(738)=-0.691976219372D+03
coeff(739)=0.559085800680D+03
coeff(740)=-0.738793649917D+03
coeff(741)=0.292917139543D+03
coeff(742)=-0.704812518161D+03
coeff(743)=0.448091620052D+03
coeff(744)=0.970506326084D+03
coeff(745)=0.194909985978D+03
coeff(746)=0.344731938163D+03
coeff(747)=-0.891954733555D+03
coeff(748)=0.289533176313D+03
coeff(749)=0.295280117292D+04
coeff(750)=-0.154317195008D+04
coeff(751)=0.930647584777D+03
coeff(752)=0.616158921579D+03
coeff(753)=0.242901914479D+03
coeff(754)=0.109287602336D+04
coeff(755)=-0.117455456733D+04
coeff(756)=0.109479856879D+04
coeff(757)=0.435321565221D+02
coeff(758)=-0.765361169878D+03
coeff(759)=0.109550292652D+03
coeff(760)=0.516290781232D+03
coeff(761)=-0.986592856670D+03
coeff(762)=0.987111834618D+03
coeff(763)=0.407065058944D+03
coeff(764)=-0.929354962849D+03
coeff(765)=-0.150055100616D+03
coeff(766)=-0.193921317597D+03
coeff(767)=0.730790282071D+03
coeff(768)=0.185521562506D+03
coeff(769)=0.393793989874D+03
coeff(770)=0.113364330708D+04
coeff(771)=0.103436177101D+04
coeff(772)=-0.733166267615D+03
coeff(773)=-0.895624248230D+03
coeff(774)=-0.629343594325D+03
coeff(775)=-0.130433353431D+04
coeff(776)=0.189706657317D+04
coeff(777)=-0.744178065026D+01
coeff(778)=-0.112378085191D+04
coeff(779)=-0.262027558607D+04
coeff(780)=0.270152357694D+04
coeff(781)=-0.255961426306D+03
coeff(782)=0.535559236440D+03
coeff(783)=-0.110709583167D+04
coeff(784)=-0.727169903046D+03
coeff(785)=-0.542142984231D+03
coeff(786)=-0.300722465910D+04
coeff(787)=0.329659537836D+03
coeff(788)=0.306592425402D+04
coeff(789)=-0.181658378789D+04
coeff(790)=0.431101830140D+03
coeff(791)=0.184943014079D+04
coeff(792)=0.125325656321D+04
coeff(793)=-0.149159516141D+04
coeff(794)=-0.146226487596D+04
coeff(795)=-0.212958297191D+03
coeff(796)=-0.359525903700D+03
coeff(797)=-0.253306322066D+03
coeff(798)=0.376958206517D+03
coeff(799)=0.100826406175D+03
coeff(800)=-0.509354356069D+03
coeff(801)=0.413470114876D+03
coeff(802)=-0.147625039436D+04
coeff(803)=0.578218903654D+03
coeff(804)=0.123801548392D+04
coeff(805)=0.112017428241D+04
coeff(806)=-0.554416247172D+03
coeff(807)=0.125895805095D+03
coeff(808)=0.535252336290D+03
coeff(809)=0.194289420478D+04
coeff(810)=-0.430942336608D+03
coeff(811)=0.120361347074D+04
coeff(812)=-0.116989894998D+04
coeff(813)=0.419112280870D+03
coeff(814)=-0.383876588539D+03
coeff(815)=0.649702036500D+03
coeff(816)=-0.941825304474D+03
coeff(817)=-0.297407533918D+02
coeff(818)=0.396621719189D+03
coeff(819)=-0.989297931327D+03
coeff(820)=-0.410794247322D+03
coeff(821)=0.508105281671D+03
coeff(822)=0.353380429993D+03
coeff(823)=0.193378235932D+03
coeff(824)=-0.847791152991D+03
coeff(825)=-0.417126272731D+03
coeff(826)=0.831868573031D+03
coeff(827)=-0.834561615517D+03
coeff(828)=0.256644195574D+03
coeff(829)=-0.131377219611D+04
coeff(830)=0.922209289912D+03
coeff(831)=0.116868294384D+04
coeff(832)=-0.681251615742D+03
coeff(833)=0.300537101820D+03
coeff(834)=0.191472355117D+03
coeff(835)=-0.316254731343D+02
coeff(836)=-0.897474883408D+03
coeff(837)=-0.646668001780D+03
coeff(838)=0.135048083407D+03
coeff(839)=0.415265009386D+03
coeff(840)=0.147812024152D+04
coeff(841)=-0.850768388694D+03
coeff(842)=0.866536792776D+03
coeff(843)=-0.974047404355D+03
coeff(844)=-0.126920527447D+04
coeff(845)=0.510945270788D+03
coeff(846)=-0.836347745877D+03
coeff(847)=0.162153350741D+04
coeff(848)=-0.535856305100D+03
coeff(849)=-0.867597775544D+02
coeff(850)=0.493535335014D+03
coeff(851)=-0.914292514554D+03
coeff(852)=-0.552243299454D+03
coeff(853)=-0.669639264347D+03
coeff(854)=0.656054444885D+03
coeff(855)=0.459281503320D+03
coeff(856)=-0.441517232875D+03
coeff(857)=-0.533490984669D+02
coeff(858)=0.149978211478D+03
coeff(859)=0.283067645823D+03
coeff(860)=-0.577097962282D+03
coeff(861)=0.436733143571D+03
coeff(862)=-0.456568155511D+03
coeff(863)=-0.102325802422D+04
coeff(864)=-0.807124513642D+03
coeff(865)=-0.251889274997D+03
coeff(866)=0.104195725379D+04
coeff(867)=0.425583468559D+03
coeff(868)=-0.278010818210D+03
coeff(869)=-0.419557591074D+03
coeff(870)=-0.950101664245D+03
coeff(871)=0.596168662854D+03
coeff(872)=0.538761559833D+03
coeff(873)=0.285298124915D+03
coeff(874)=-0.921581859458D+03
coeff(875)=0.619750776384D+03
coeff(876)=0.233365897608D+03
coeff(877)=-0.349144526689D+03
coeff(878)=0.173874291384D+03
coeff(879)=-0.905139731843D+03
coeff(880)=0.984698261413D+03
coeff(881)=0.123544538472D+04
coeff(882)=0.588244664573D+03
coeff(883)=0.269810835856D+02
coeff(884)=0.589598491891D+03
coeff(885)=0.664628668244D+03
coeff(886)=0.111127362414D+04
coeff(887)=-0.427896717813D+03
coeff(888)=-0.509821824566D+03
coeff(889)=-0.588038480143D+03
coeff(890)=-0.895584625021D+03
coeff(891)=0.937991991686D+03
coeff(892)=0.456730737708D+03
coeff(893)=0.406042473672D+03
coeff(894)=0.268200847684D+01
coeff(895)=-0.105193135930D+04
coeff(896)=-0.462713768808D+03
coeff(897)=0.518553215758D+03
coeff(898)=-0.523878120002D+03
coeff(899)=0.741321358233D+01
coeff(900)=0.118545736520D+03
coeff(901)=0.313186593724D+03
coeff(902)=0.341306746876D+03
coeff(903)=0.301472657276D+03
coeff(904)=-0.509263340806D+03
coeff(905)=0.210617629652D+03
coeff(906)=-0.116757072865D+04
coeff(907)=0.844523242843D+03
coeff(908)=0.136887715141D+04
coeff(909)=0.151091819768D+04
coeff(910)=0.769035680854D+03
coeff(911)=-0.609231319106D+03
coeff(912)=0.428391326365D+03
coeff(913)=0.108495754402D+04
coeff(914)=-0.992397172542D+01
coeff(915)=0.320532413721D+03
coeff(916)=-0.965252707472D+03
coeff(917)=-0.133460325470D+04
coeff(918)=0.115332681735D+04
coeff(919)=0.798677044685D+03
coeff(920)=0.110563483324D+04
coeff(921)=-0.350999716219D+03
coeff(922)=-0.272095808884D+03
coeff(923)=-0.787216187265D+03
coeff(924)=0.888971923311D+03
coeff(925)=0.141212883279D+04
coeff(926)=-0.141307057590D+04
coeff(927)=0.170771867825D+04
coeff(928)=0.313898175314D+03
coeff(929)=0.783562797437D+03
coeff(930)=-0.230233505270D+03
coeff(931)=0.263757825840D+03
coeff(932)=0.978477854417D+03
coeff(933)=0.483935175169D+03
coeff(934)=-0.226052178385D+03
coeff(935)=0.124649523057D+03
coeff(936)=0.614714008960D+03
coeff(937)=-0.396288191176D+03
coeff(938)=0.288683355363D+03
coeff(939)=0.592023663627D+03
coeff(940)=0.646648053027D+03
coeff(941)=0.750286657756D+03
coeff(942)=0.862602468382D+03
coeff(943)=0.288465014161D+03
coeff(944)=-0.114823695008D+03
coeff(945)=-0.166530771141D+04
coeff(946)=-0.128628503740D+04
coeff(947)=-0.631364517004D+03
coeff(948)=-0.481200474454D+03
coeff(949)=-0.746334216306D+03
coeff(950)=-0.312561013916D+03
coeff(951)=-0.146331241713D+04
coeff(952)=-0.219175753853D+02
coeff(953)=0.838621498478D+03
coeff(954)=0.544228907647D+03
coeff(955)=-0.333445099150D+03
coeff(956)=-0.105703402694D+04
coeff(957)=-0.695161929374D+03
coeff(958)=-0.240420733159D+02
coeff(959)=-0.174614042031D+03
coeff(960)=-0.105312649922D+03
coeff(961)=0.726268386366D+02
coeff(962)=-0.209751596255D+04
coeff(963)=0.162145609797D+04
coeff(964)=0.105957094267D+03
coeff(965)=0.106712202802D+04
coeff(966)=-0.631681863635D+03
coeff(967)=0.244027872165D+03
coeff(968)=-0.182569961830D+04
coeff(969)=-0.144021304825D+03
coeff(970)=-0.153325159420D+03
coeff(971)=0.915108033500D+03
coeff(972)=0.163592541694D+04
coeff(973)=0.547237873098D+03
coeff(974)=-0.107889158241D+04
coeff(975)=-0.190874349056D+03
coeff(976)=-0.943154000711D+03
coeff(977)=0.108986084341D+04
coeff(978)=-0.909283426483D+03
coeff(979)=0.275805966187D+03
coeff(980)=-0.606560009780D+02
coeff(981)=-0.259437627884D+03
coeff(982)=0.126843746440D+04
coeff(983)=-0.150937073318D+04
coeff(984)=0.114970709985D+04
coeff(985)=-0.567841275548D+03
coeff(986)=0.283610498527D+03
coeff(987)=0.712600606168D+03
coeff(988)=-0.267241782967D+03
coeff(989)=-0.568876307555D+02
coeff(990)=-0.133841484146D+04
coeff(991)=0.779739870483D+03
coeff(992)=0.365330487117D+03
coeff(993)=0.644485000970D+03
coeff(994)=-0.907971924625D+03
coeff(995)=0.283845057404D+03
coeff(996)=0.379116282709D+03
coeff(997)=0.968717451648D+03
coeff(998)=-0.564669735966D+03
coeff(999)=-0.438316231280D+03
coeff(1000)=-0.441003623068D+03
coeff(1001)=0.104346932162D+04
coeff(1002)=0.242710040410D+03
coeff(1003)=-0.104245635726D+04
coeff(1004)=0.117372700406D+04
coeff(1005)=-0.127126896861D+04
coeff(1006)=0.398674536588D+03
coeff(1007)=0.317116052587D+03
coeff(1008)=0.290513286965D+03
coeff(1009)=-0.135199446280D+04
coeff(1010)=0.717047179818D+02
coeff(1011)=-0.531200038958D+03
coeff(1012)=0.142433608264D+04
coeff(1013)=0.181286590005D+04
coeff(1014)=0.230541931196D+03
coeff(1015)=-0.105814438252D+04
coeff(1016)=-0.892070269124D+03
coeff(1017)=-0.940084485187D+02
coeff(1018)=0.110887189048D+03
coeff(1019)=0.209181254458D+03
coeff(1020)=-0.117439367693D+04
coeff(1021)=-0.908979742124D+03
coeff(1022)=0.151173939206D+04
coeff(1023)=-0.151317328772D+04
coeff(1024)=0.270384790163D+03
coeff(1025)=-0.113452961665D+04
coeff(1026)=0.124590581641D+04
coeff(1027)=0.207842337795D+03
coeff(1028)=0.246388161822D+02
coeff(1029)=-0.410661157288D+03
coeff(1030)=-0.118419605886D+02
coeff(1031)=0.689300649881D+03
coeff(1032)=0.575483274122D+03
coeff(1033)=-0.126388671499D+03
coeff(1034)=0.339145717082D+03
coeff(1035)=-0.438558634051D+03
coeff(1036)=0.124210493012D+04
coeff(1037)=0.541024017371D+02
coeff(1038)=-0.644489381247D+03
coeff(1039)=0.131581867264D+04
coeff(1040)=0.424746394752D+03
coeff(1041)=-0.111942931285D+04
coeff(1042)=-0.546812128074D+03
coeff(1043)=0.801962980907D+03
coeff(1044)=-0.759658903832D+03
coeff(1045)=-0.664527658943D+03
coeff(1046)=0.178952253219D+04
coeff(1047)=0.286838615321D+03
coeff(1048)=0.780361277773D+02
coeff(1049)=-0.154958191573D+04
coeff(1050)=-0.501284912702D+03
coeff(1051)=0.559405974860D+03
coeff(1052)=0.121692752118D+03
coeff(1053)=-0.283331576202D+03
coeff(1054)=0.325632871445D+03
coeff(1055)=-0.444219038648D+03
coeff(1056)=-0.125105143005D+02
coeff(1057)=-0.847470734714D+03
coeff(1058)=0.325438027632D+03
coeff(1059)=-0.473193223727D+03
coeff(1060)=-0.103719158409D+04
coeff(1061)=0.337821948790D+03
coeff(1062)=0.807314292183D+03
coeff(1063)=-0.715443226262D+03
coeff(1064)=0.129158332028D+04
coeff(1065)=0.175222198573D+03
coeff(1066)=0.350106317093D+03
coeff(1067)=-0.294734374839D+03
coeff(1068)=0.302847495379D+03
coeff(1069)=-0.349660543765D+03
coeff(1070)=0.433820572813D+03
coeff(1071)=0.445387557326D+03
coeff(1072)=-0.803645112521D+03
coeff(1073)=0.394408809956D+03
coeff(1074)=0.185651190567D+03
coeff(1075)=-0.177178524836D+04
coeff(1076)=0.412473610315D+03
coeff(1077)=-0.904887210113D+02
coeff(1078)=-0.845677886274D+03
coeff(1079)=0.810108811542D+03
coeff(1080)=-0.117257739071D+04
coeff(1081)=-0.134381550699D+04
coeff(1082)=-0.536600287806D+02
coeff(1083)=0.524784614299D+03
coeff(1084)=0.485832262128D+03
coeff(1085)=-0.488484583757D+03
coeff(1086)=-0.535956693347D+03
coeff(1087)=-0.447454648568D+03
coeff(1088)=-0.208988832778D+03
coeff(1089)=-0.119205806902D+04
coeff(1090)=0.299071382229D+03
coeff(1091)=-0.826175014462D+02
coeff(1092)=0.704611174873D+03
coeff(1093)=-0.168745994395D+03
coeff(1094)=0.726146481992D+02
coeff(1095)=-0.937229777391D+03
coeff(1096)=0.187608490239D+03
coeff(1097)=0.628854160940D+03
coeff(1098)=-0.996549696591D+03
coeff(1099)=-0.769460200506D+03
coeff(1100)=-0.387591332540D+03
coeff(1101)=0.144559985045D+04
coeff(1102)=-0.962130351985D+03
coeff(1103)=0.147885370940D+04
coeff(1104)=0.542123762827D+03
coeff(1105)=0.136512849210D+04
coeff(1106)=-0.127121185597D+04
coeff(1107)=0.951619656941D+03
coeff(1108)=-0.371643607022D+03
coeff(1109)=-0.456641561680D+03
coeff(1110)=-0.121468619499D+04
coeff(1111)=-0.300755262823D+03
coeff(1112)=-0.132172061196D+04
coeff(1113)=0.289032071780D+03
coeff(1114)=0.409659948437D+03
coeff(1115)=-0.131533943633D+04
coeff(1116)=-0.114890707449D+04
coeff(1117)=-0.483607821818D+03
coeff(1118)=-0.969074521943D+03
coeff(1119)=-0.490316124651D+03
coeff(1120)=-0.278424768500D+02
coeff(1121)=-0.441796229452D+03
coeff(1122)=0.963001362157D+03
coeff(1123)=0.372714116713D+01
coeff(1124)=0.558427058664D+03
coeff(1125)=-0.200309700913D+03
coeff(1126)=0.320466607449D+03
coeff(1127)=-0.796530331730D+03
coeff(1128)=-0.889629179493D+03
coeff(1129)=0.844592404607D+03
coeff(1130)=-0.586157398311D+03
coeff(1131)=-0.633823333257D+03
coeff(1132)=0.925710277074D+03
coeff(1133)=-0.231464596247D+03
coeff(1134)=-0.160575737949D+04
coeff(1135)=0.828672727513D+03
coeff(1136)=0.126360360184D+04
coeff(1137)=0.124676491569D+04
coeff(1138)=0.256946706711D+03
coeff(1139)=-0.742766733518D+03
coeff(1140)=-0.177149572871D+03
coeff(1141)=-0.627985310668D+03
coeff(1142)=0.335384927885D+03
coeff(1143)=-0.314165256213D+03
coeff(1144)=-0.155670589465D+04
coeff(1145)=-0.653887151793D+02
coeff(1146)=-0.501360088256D+03
coeff(1147)=0.441606424868D+03
coeff(1148)=-0.374882165177D+03
coeff(1149)=0.114083440242D+04
coeff(1150)=0.102037535577D+04
coeff(1151)=0.142390086141D+04
coeff(1152)=-0.709152731754D+03
coeff(1153)=0.168050750114D+04
coeff(1154)=-0.105928043189D+03
coeff(1155)=-0.996800114645D+02
coeff(1156)=-0.127459424296D+04
coeff(1157)=0.146559608982D+04
coeff(1158)=-0.180938100587D+04
coeff(1159)=0.597186255202D+03
coeff(1160)=-0.227020303104D+03
coeff(1161)=-0.701337143255D+03
coeff(1162)=0.117995683968D+04
coeff(1163)=0.893153497507D+03
coeff(1164)=-0.291529227035D+03
coeff(1165)=0.678616699636D+03
coeff(1166)=0.380228973581D+02
coeff(1167)=-0.145148604850D+03
coeff(1168)=0.815232464044D+02
coeff(1169)=-0.148195594398D+04
coeff(1170)=0.416662065914D+03
coeff(1171)=0.435571572134D+03
coeff(1172)=0.880405755598D+03
coeff(1173)=-0.507439168033D+03
coeff(1174)=0.463036208145D+03
coeff(1175)=0.110218263571D+03
coeff(1176)=-0.781111130955D+03
coeff(1177)=-0.152657829837D+03
coeff(1178)=0.106839181130D+04
coeff(1179)=0.163844137363D+04
coeff(1180)=0.135167796165D+04
coeff(1181)=0.758970760256D+03
coeff(1182)=0.619902088128D+03
coeff(1183)=0.471877252647D+03
coeff(1184)=0.449786906441D+03
coeff(1185)=-0.133512222720D+03
coeff(1186)=0.311931994116D+03
coeff(1187)=-0.105493416415D+04
coeff(1188)=0.103606954190D+04
coeff(1189)=-0.113000780247D+04
coeff(1190)=-0.375958582914D+03
coeff(1191)=0.536817987879D+03
coeff(1192)=0.150607505009D+04
coeff(1193)=-0.914627203058D+03
coeff(1194)=0.416025771015D+02
coeff(1195)=0.639094667177D+02
coeff(1196)=-0.584212183834D+02
coeff(1197)=-0.282348206881D+03
coeff(1198)=0.606235547326D+02
coeff(1199)=-0.551425162549D+03
coeff(1200)=0.541762739601D+03
coeff(1201)=0.325471388543D+02
coeff(1202)=-0.263213114026D+03
coeff(1203)=-0.633153770283D+03
coeff(1204)=0.387692613807D+03
coeff(1205)=0.501913327067D+03
coeff(1206)=-0.378536513970D+01
coeff(1207)=-0.667770587051D+02
coeff(1208)=-0.244261122061D+02
coeff(1209)=0.803909394313D+02
coeff(1210)=0.745867465159D+03
coeff(1211)=-0.845307917248D+03
coeff(1212)=0.852775965474D+03
coeff(1213)=-0.135620800399D+04
coeff(1214)=-0.535358098165D+02
coeff(1215)=0.766486281763D+03
coeff(1216)=-0.784622172043D+03
coeff(1217)=0.233665366596D+03
coeff(1218)=-0.657701455618D+02
coeff(1219)=-0.217221749827D+03
coeff(1220)=0.129436038672D+03
coeff(1221)=-0.293177233847D+03
coeff(1222)=0.195416693238D+03
coeff(1223)=0.365715914566D+02
coeff(1224)=-0.923172236915D+03
coeff(1225)=0.617395036136D+03
coeff(1226)=-0.119746249449D+04
coeff(1227)=-0.186009536405D+03
coeff(1228)=0.389860247659D+03
coeff(1229)=-0.619628810350D+03
coeff(1230)=-0.118423070495D+03
coeff(1231)=0.699562468520D+03
coeff(1232)=-0.312325217530D+03
coeff(1233)=0.349285941651D+03
coeff(1234)=-0.442442176062D+02
coeff(1235)=-0.553886207381D+02
coeff(1236)=0.575169687895D+03
coeff(1237)=0.305173949266D+03
coeff(1238)=0.722502112196D+03
coeff(1239)=0.917323942304D+02
coeff(1240)=-0.186387786804D+04
coeff(1241)=-0.141887166381D+03
coeff(1242)=0.139514722556D+04
coeff(1243)=0.871767009671D+03
coeff(1244)=-0.205526936836D+04
coeff(1245)=-0.573516056701D+03
coeff(1246)=-0.426932340609D+03
coeff(1247)=-0.578242229134D+02
coeff(1248)=-0.321381543341D+03
coeff(1249)=-0.720931460948D+03
coeff(1250)=-0.289711189197D+02
coeff(1251)=0.113669858936D+04
coeff(1252)=0.723705696869D+02
coeff(1253)=0.356817667452D+03
coeff(1254)=-0.641658374459D+03
coeff(1255)=-0.460350140182D+03
coeff(1256)=0.493760009139D+03
coeff(1257)=0.546456358422D+03
coeff(1258)=0.165422563510D+03
coeff(1259)=0.137588500168D+04
coeff(1260)=0.131809130624D+04
coeff(1261)=-0.928585535879D+03
coeff(1262)=0.303851764638D+03
coeff(1263)=0.181682484961D+02
coeff(1264)=-0.155382345206D+04
coeff(1265)=0.142866845562D+04
coeff(1266)=-0.141687897913D+04
coeff(1267)=-0.550195297286D+03
coeff(1268)=0.240699158743D+03
coeff(1269)=-0.113353921503D+03
coeff(1270)=0.686216585031D+03
coeff(1271)=-0.500241061804D+03
coeff(1272)=0.724222184257D+02
coeff(1273)=-0.628372397936D+03
coeff(1274)=-0.416807260988D+03
coeff(1275)=-0.853832216207D+03
coeff(1276)=0.424448323442D+03
coeff(1277)=0.579508347763D+02
coeff(1278)=0.595664081595D+03
coeff(1279)=0.593123002258D+03
coeff(1280)=-0.602844904630D+02
coeff(1281)=-0.104733576711D+04
coeff(1282)=-0.578334529939D+03
coeff(1283)=0.244615208757D+03
coeff(1284)=0.821744176743D+03
coeff(1285)=0.348023130542D+03
coeff(1286)=0.864866481770D+03
coeff(1287)=-0.577697850745D+03
coeff(1288)=-0.182769814570D+04
coeff(1289)=0.507666885877D+03
coeff(1290)=-0.151104842348D+03
coeff(1291)=-0.108945986646D+04
coeff(1292)=-0.421915481270D+03
coeff(1293)=0.529996658902D+03
coeff(1294)=0.780772142695D+03
coeff(1295)=-0.675774310053D+02
coeff(1296)=-0.153225343325D+03
coeff(1297)=0.140252305046D+03
coeff(1298)=0.116561264677D+03
coeff(1299)=-0.932888244019D+02
coeff(1300)=-0.547157825248D+01

! Calculate MEG
x(1)=exp(-((r(1)-ra1))/a1 -((r(1)-rb1)**2)/b1)
x(2)=exp(-((r(2)-ra1))/a1 -((r(2)-rb1)**2)/b1)
x(3)=exp(-((r(3)-ra2))/a2 -((r(3)-rb2)**2)/b2)
x(4)=exp(-((r(4)-ra1))/a1 -((r(4)-rb1)**2)/b1)
x(5)=exp(-((r(5)-ra2))/a2 -((r(5)-rb2)**2)/b2)
x(6)=exp(-((r(6)-ra2))/a2 -((r(6)-rb2)**2)/b2)

! Calculate dMEG/dr(6,6) for giving R(6)
! dmsdr(i,j)=0  i!=j
! Initialize dmsdr
      do i=1,6
        do j=1,6
          dmsdr(i,j)=0.0d0
        enddo
      enddo
dmsdr(1,1)=(-2.0d0*(r(1)-rb1)/b1-1/a1)*Exp(-(r(1)-ra1)/a1-((r(1)-rb1)**2.0d0)/b1)
dmsdr(2,2)=(-2.0d0*(r(2)-rb1)/b1-1/a1)*Exp(-(r(2)-ra1)/a1-((r(2)-rb1)**2.0d0)/b1)
dmsdr(3,3)=(-2.0d0*(r(3)-rb2)/b2-1/a2)*Exp(-(r(3)-ra2)/a2-((r(3)-rb2)**2.0d0)/b2)
dmsdr(4,4)=(-2.0d0*(r(4)-rb1)/b1-1/a1)*Exp(-(r(4)-ra1)/a1-((r(4)-rb1)**2.0d0)/b1)
dmsdr(5,5)=(-2.0d0*(r(5)-rb2)/b2-1/a2)*Exp(-(r(5)-ra2)/a2-((r(5)-rb2)**2.0d0)/b2)
dmsdr(6,6)=(-2.0d0*(r(6)-rb2)/b2-1/a2)*Exp(-(r(6)-ra2)/a2-((r(6)-rb2)**2.0d0)/b2)

rm(0) = 1.0D0
rm(1) = x(4)
rm(2) = x(2)
rm(3) = x(1)
rm(4) = x(6)
rm(5) = x(5)
rm(6) = x(3)
rm(7) = rm(1)*rm(2)
rm(8) = rm(1)*rm(3)
rm(9) = rm(2)*rm(3)
rm(10) = rm(3)*rm(4)
rm(11) = rm(2)*rm(5)
rm(12) = rm(1)*rm(6)
rm(13) = rm(4)*rm(5)
rm(14) = rm(4)*rm(6)
rm(15) = rm(5)*rm(6)
rm(16) = rm(1)*rm(9)
rm(17) = rm(1)*rm(10)
rm(18) = rm(2)*rm(10)
rm(19) = rm(1)*rm(11)
rm(20) = rm(3)*rm(11)
rm(21) = rm(2)*rm(12)
rm(22) = rm(3)*rm(12)
rm(23) = rm(2)*rm(13)
rm(24) = rm(3)*rm(13)
rm(25) = rm(1)*rm(14)
rm(26) = rm(3)*rm(14)
rm(27) = rm(1)*rm(15)
rm(28) = rm(2)*rm(15)
rm(29) = rm(4)*rm(15)
rm(30) = rm(2)*rm(24)
rm(31) = rm(1)*rm(26)
rm(32) = rm(1)*rm(28)

p(0) = rm(0)
p(1) = rm(1) + rm(2) + rm(3)
p(2) = rm(4) + rm(5) + rm(6)
p(3) = rm(7) + rm(8) + rm(9)
p(4) = rm(10) + rm(11) + rm(12)
p(5) = p(1)*p(2) - p(4)
p(6) = rm(13) + rm(14) + rm(15)
p(7) = p(1)*p(1) - p(3) - p(3)
p(8) = p(2)*p(2) - p(6) - p(6)
p(9) = rm(16)
p(10) = rm(17) + rm(18) + rm(19) + rm(20) + rm(21) + rm(22)
p(11) = p(2)*p(3) - p(10)
p(12) = rm(23) + rm(24) + rm(25) + rm(26) + rm(27) + rm(28)
p(13) = p(1)*p(6) - p(12)
p(14) = rm(29)
p(15) = p(1)*p(3) - p(9) - p(9) - p(9)
p(16) = p(1)*p(4) - p(10)
p(17) = p(2)*p(7) - p(16)
p(18) = p(2)*p(4) - p(12)
p(19) = p(1)*p(8) - p(18)
p(20) = p(2)*p(6) - p(14) - p(14) - p(14)
p(21) = p(1)*p(7) - p(15)
p(22) = p(2)*p(8) - p(20)
p(23) = p(9)*p(2)
p(24) = rm(30) + rm(31) + rm(32)
p(25) = p(3)*p(6) - p(24)
p(26) = p(14)*p(1)
p(27) = p(9)*p(1)
p(28) = p(3)*p(4) - p(23)
p(29) = p(1)*p(10) - p(23) - p(28) - p(23)
p(30) = p(1)*p(11) - p(23)
p(31) = p(1)*p(12) - p(25) - p(24) - p(24)
p(32) = p(1)*p(13) - p(25)
p(33) = p(4)*p(5) - p(25) - p(31)
p(34) = p(2)*p(11) - p(25)
p(35) = p(4)*p(6) - p(26)
p(36) = p(2)*p(12) - p(26) - p(35) - p(26)
p(37) = p(2)*p(13) - p(26)
p(38) = p(14)*p(2)
p(39) = p(3)*p(3) - p(27) - p(27)
p(40) = p(3)*p(7) - p(27)
p(41) = p(1)*p(16) - p(28)
p(42) = p(2)*p(21) - p(41)
p(43) = p(1)*p(18) - p(33)
p(44) = p(7)*p(8) - p(43)
p(45) = p(6)*p(6) - p(38) - p(38)
p(46) = p(2)*p(18) - p(35)
p(47) = p(1)*p(22) - p(46)
p(48) = p(6)*p(8) - p(38)
p(49) = p(1)*p(21) - p(40)
p(50) = p(2)*p(22) - p(48)
p(51) = p(9)*p(6)
p(52) = p(14)*p(3)
p(53) = p(9)*p(4)
p(54) = p(9)*p(5)
p(55) = p(1)*p(24) - p(51)
p(56) = p(3)*p(12) - p(51) - p(55) - p(51)
p(57) = p(3)*p(13) - p(51)
p(58) = p(14)*p(7)
p(59) = p(9)*p(8)
p(60) = p(2)*p(24) - p(52)
p(61) = p(4)*p(13) - p(58)
p(62) = p(6)*p(11) - p(52)
p(63) = p(14)*p(4)
p(64) = p(14)*p(5)
p(65) = p(9)*p(3)
p(66) = p(9)*p(7)
p(67) = p(3)*p(16) - p(53)
p(68) = p(4)*p(15) - p(54) - p(67)
p(69) = p(2)*p(39) - p(68)
p(70) = p(1)*p(29) - p(54) - p(68)
p(71) = p(7)*p(11) - p(53)
p(72) = p(1)*p(31) - p(56) - p(55)
p(73) = p(1)*p(32) - p(57)
p(74) = p(3)*p(18) - p(59)
p(75) = p(2)*p(29) - p(57) - p(55)
p(76) = p(1)*p(34) - p(59)
p(77) = p(6)*p(16) - p(58)
p(78) = p(1)*p(36) - p(62) - p(60)
p(79) = p(2)*p(32) - p(58)
p(80) = p(6)*p(12) - p(64) - p(63) - p(63)
p(81) = p(1)*p(45) - p(80)
p(82) = p(14)*p(6)
p(83) = p(2)*p(33) - p(61) - p(60)
p(84) = p(2)*p(34) - p(62)
p(85) = p(6)*p(18) - p(63)
p(86) = p(2)*p(36) - p(64) - p(80)
p(87) = p(8)*p(13) - p(63)
p(88) = p(14)*p(8)
p(89) = p(1)*p(39) - p(65)
p(90) = p(3)*p(21) - p(66)
p(91) = p(1)*p(41) - p(67)
p(92) = p(2)*p(49) - p(91)
p(93) = p(1)*p(43) - p(74)
p(94) = p(8)*p(21) - p(93)
p(95) = p(1)*p(46) - p(83)
p(96) = p(7)*p(22) - p(95)
p(97) = p(2)*p(45) - p(82)
p(98) = p(2)*p(46) - p(85)
p(99) = p(1)*p(50) - p(98)
p(100) = p(6)*p(22) - p(88)
p(101) = p(1)*p(49) - p(90)
p(102) = p(2)*p(50) - p(100)
p(103) = p(9)*p(14)
p(104) = p(9)*p(12)
p(105) = p(9)*p(13)
p(106) = p(14)*p(15)
p(107) = p(9)*p(20)
p(108) = p(14)*p(10)
p(109) = p(14)*p(11)
p(110) = p(9)*p(16)
p(111) = p(9)*p(10)
p(112) = p(9)*p(11)
p(113) = p(9)*p(17)
p(114) = p(3)*p(24) - p(104)
p(115) = p(7)*p(24) - p(105)
p(116) = p(3)*p(31) - p(104) - p(115)
p(117) = p(6)*p(39) - p(114)
p(118) = p(3)*p(32) - p(105)
p(119) = p(14)*p(21)
p(120) = p(9)*p(18)
p(121) = p(9)*p(19)
p(122) = p(4)*p(24) - p(103) - p(103) - p(103)
p(123) = p(1)*p(60) - p(107) - p(122)
p(124) = p(13)*p(16) - p(119)
p(125) = p(2)*p(56) - p(106) - p(124)
p(126) = p(4)*p(32) - p(119)
p(127) = p(1)*p(62) - p(107) - p(125)
p(128) = p(14)*p(16)
p(129) = p(14)*p(17)
p(130) = p(6)*p(24) - p(108)
p(131) = p(3)*p(45) - p(130)
p(132) = p(14)*p(12)
p(133) = p(14)*p(13)
p(134) = p(9)*p(22)
p(135) = p(8)*p(24) - p(109)
p(136) = p(13)*p(18) - p(128)
p(137) = p(6)*p(34) - p(109)
p(138) = p(14)*p(18)
p(139) = p(14)*p(19)
p(140) = p(9)*p(9)
p(141) = p(9)*p(15)
p(142) = p(9)*p(21)
p(143) = p(3)*p(41) - p(110)
p(144) = p(4)*p(39) - p(112)
p(145) = p(1)*p(68) - p(111) - p(144)
p(146) = p(1)*p(69) - p(112)
p(147) = p(1)*p(70) - p(113) - p(145)
p(148) = p(11)*p(21) - p(110)
p(149) = p(1)*p(72) - p(116) - p(115)
p(150) = p(1)*p(73) - p(118)
p(151) = p(3)*p(43) - p(120)
p(152) = p(15)*p(18) - p(121) - p(151)
p(153) = p(2)*p(69) - p(117)
p(154) = p(1)*p(75) - p(121) - p(152)
p(155) = p(7)*p(34) - p(120)
p(156) = p(6)*p(41) - p(119)
p(157) = p(1)*p(78) - p(125) - p(123)
p(158) = p(2)*p(73) - p(119)
p(159) = p(1)*p(80) - p(131) - p(130) - p(130)
p(160) = p(1)*p(81) - p(131)
p(161) = p(14)*p(14)
p(162) = p(3)*p(46) - p(134)
p(163) = p(2)*p(75) - p(126) - p(123)
p(164) = p(1)*p(84) - p(134)
p(165) = p(6)*p(43) - p(128)
p(166) = p(1)*p(86) - p(137) - p(135)
p(167) = p(8)*p(32) - p(128)
p(168) = p(4)*p(45) - p(133)
p(169) = p(2)*p(80) - p(132) - p(168)
p(170) = p(2)*p(81) - p(133)
p(171) = p(14)*p(20)
p(172) = p(2)*p(83) - p(136) - p(135)
p(173) = p(2)*p(84) - p(137)
p(174) = p(6)*p(46) - p(138)
p(175) = p(2)*p(86) - p(139) - p(169)
p(176) = p(13)*p(22) - p(138)
p(177) = p(14)*p(22)
p(178) = p(3)*p(39) - p(141)
p(179) = p(1)*p(89) - p(141) - p(178) - p(178)
p(180) = p(3)*p(49) - p(142)
p(181) = p(1)*p(91) - p(143)
p(182) = p(2)*p(101) - p(181)
p(183) = p(1)*p(93) - p(151)
p(184) = p(8)*p(49) - p(183)
p(185) = p(1)*p(95) - p(162)
p(186) = p(21)*p(22) - p(185)
p(187) = p(6)*p(45) - p(171)
p(188) = p(1)*p(98) - p(172)
p(189) = p(7)*p(50) - p(188)
p(190) = p(2)*p(97) - p(171) - p(187) - p(187)
p(191) = p(2)*p(98) - p(174)
p(192) = p(1)*p(102) - p(191)
p(193) = p(6)*p(50) - p(177)
p(194) = p(1)*p(101) - p(180)
p(195) = p(2)*p(102) - p(193)
p(196) = p(9)*p(26)
p(197) = p(9)*p(38)
p(198) = p(9)*p(24)
p(199) = p(9)*p(31)
p(200) = p(9)*p(25)
p(201) = p(9)*p(32)
p(202) = p(14)*p(39)
p(203) = p(14)*p(40)
p(204) = p(9)*p(35)
p(205) = p(9)*p(36)
p(206) = p(9)*p(37)
p(207) = p(14)*p(28)
p(208) = p(14)*p(29)
p(209) = p(14)*p(30)
p(210) = p(9)*p(45)
p(211) = p(14)*p(24)
p(212) = p(14)*p(25)
p(213) = p(9)*p(48)
p(214) = p(14)*p(33)
p(215) = p(14)*p(34)
p(216) = p(9)*p(41)
p(217) = p(9)*p(28)
p(218) = p(9)*p(23)
p(219) = p(9)*p(29)
p(220) = p(9)*p(30)
p(221) = p(9)*p(42)
p(222) = p(1)*p(114) - p(198)
p(223) = p(21)*p(24) - p(201)
p(224) = p(3)*p(72) - p(199) - p(223)
p(225) = p(1)*p(116) - p(199) - p(224)
p(226) = p(13)*p(39) - p(198)
p(227) = p(3)*p(73) - p(201)
p(228) = p(14)*p(49)
p(229) = p(9)*p(43)
p(230) = p(9)*p(33)
p(231) = p(9)*p(34)
p(232) = p(9)*p(44)
p(233) = p(16)*p(24) - p(196)
p(234) = p(2)*p(114) - p(202)
p(235) = p(1)*p(123) - p(205) - p(234)
p(236) = p(13)*p(41) - p(228)
p(237) = p(2)*p(116) - p(203) - p(236)
p(238) = p(16)*p(32) - p(228)
p(239) = p(6)*p(69) - p(202)
p(240) = p(4)*p(73) - p(228)
p(241) = p(11)*p(32) - p(196)
p(242) = p(14)*p(41)
p(243) = p(14)*p(42)
p(244) = p(1)*p(130) - p(210)
p(245) = p(4)*p(62) - p(213) - p(209)
p(246) = p(3)*p(81) - p(210)
p(247) = p(14)*p(31)
p(248) = p(14)*p(32)
p(249) = p(14)*p(26)
p(250) = p(9)*p(46)
p(251) = p(9)*p(47)
p(252) = p(18)*p(24) - p(197)
p(253) = p(1)*p(135) - p(213) - p(252)
p(254) = p(13)*p(43) - p(242)
p(255) = p(2)*p(125) - p(209) - p(245)
p(256) = p(18)*p(32) - p(242)
p(257) = p(13)*p(34) - p(197)
p(258) = p(14)*p(43)
p(259) = p(14)*p(44)
p(260) = p(2)*p(130) - p(211)
p(261) = p(4)*p(81) - p(248)
p(262) = p(11)*p(45) - p(211)
p(263) = p(14)*p(35)
p(264) = p(14)*p(36)
p(265) = p(14)*p(37)
p(266) = p(9)*p(50)
p(267) = p(22)*p(24) - p(215)
p(268) = p(13)*p(46) - p(258)
p(269) = p(6)*p(84) - p(215)
p(270) = p(14)*p(46)
p(271) = p(14)*p(47)
p(272) = p(9)*p(27)
p(273) = p(9)*p(39)
p(274) = p(9)*p(40)
p(275) = p(9)*p(49)
p(276) = p(3)*p(91) - p(216)
p(277) = p(16)*p(39) - p(218)
p(278) = p(1)*p(144) - p(217) - p(277)
p(279) = p(2)*p(178) - p(278)
p(280) = p(1)*p(145) - p(219) - p(278)
p(281) = p(7)*p(69) - p(218)
p(282) = p(1)*p(147) - p(221) - p(280)
p(283) = p(11)*p(49) - p(216)
p(284) = p(1)*p(149) - p(224) - p(223)
p(285) = p(1)*p(150) - p(227)
p(286) = p(3)*p(93) - p(229)
p(287) = p(18)*p(39) - p(231)
p(288) = p(1)*p(152) - p(230) - p(287)
p(289) = p(1)*p(153) - p(231)
p(290) = p(1)*p(154) - p(232) - p(288)
p(291) = p(21)*p(34) - p(229)
p(292) = p(6)*p(91) - p(228)
p(293) = p(1)*p(157) - p(237) - p(235)
p(294) = p(2)*p(150) - p(228)
p(295) = p(1)*p(159) - p(245) - p(244)
p(296) = p(1)*p(160) - p(246)
p(297) = p(3)*p(95) - p(250)
p(298) = p(2)*p(152) - p(238) - p(234)
p(299) = p(2)*p(153) - p(239)
p(300) = p(1)*p(163) - p(251) - p(298)
p(301) = p(7)*p(84) - p(250)
p(302) = p(6)*p(93) - p(242)
p(303) = p(1)*p(166) - p(255) - p(253)
p(304) = p(8)*p(73) - p(242)
p(305) = p(16)*p(45) - p(248)
p(306) = p(1)*p(169) - p(262) - p(260)
p(307) = p(2)*p(160) - p(248)
p(308) = p(14)*p(38)
p(309) = p(6)*p(80) - p(264) - p(263)
p(310) = p(1)*p(187) - p(309)
p(311) = p(14)*p(45)
p(312) = p(3)*p(98) - p(266)
p(313) = p(2)*p(163) - p(256) - p(253)
p(314) = p(1)*p(173) - p(266)
p(315) = p(6)*p(95) - p(258)
p(316) = p(1)*p(175) - p(269) - p(267)
p(317) = p(22)*p(32) - p(258)
p(318) = p(18)*p(45) - p(249)
p(319) = p(2)*p(169) - p(264) - p(309)
p(320) = p(8)*p(81) - p(249)
p(321) = p(14)*p(48)
p(322) = p(2)*p(172) - p(268) - p(267)
p(323) = p(2)*p(173) - p(269)
p(324) = p(6)*p(98) - p(270)
p(325) = p(2)*p(175) - p(271) - p(319)
p(326) = p(13)*p(50) - p(270)
p(327) = p(14)*p(50)
p(328) = p(1)*p(178) - p(273)
p(329) = p(21)*p(39) - p(272)
p(330) = p(3)*p(101) - p(275)
p(331) = p(1)*p(181) - p(276)
p(332) = p(2)*p(194) - p(331)
p(333) = p(1)*p(183) - p(286)
p(334) = p(8)*p(101) - p(333)
p(335) = p(1)*p(185) - p(297)
p(336) = p(22)*p(49) - p(335)
p(337) = p(1)*p(188) - p(312)
p(338) = p(21)*p(50) - p(337)
p(339) = p(2)*p(187) - p(311)
p(340) = p(1)*p(191) - p(322)
p(341) = p(7)*p(102) - p(340)
p(342) = p(22)*p(45) - p(308)
p(343) = p(2)*p(191) - p(324)
p(344) = p(1)*p(195) - p(343)
p(345) = p(6)*p(102) - p(327)
p(346) = p(1)*p(194) - p(330)
p(347) = p(2)*p(195) - p(345)
p(348) = p(9)*p(52)
p(349) = p(9)*p(58)
p(350) = p(9)*p(63)
p(351) = p(9)*p(64)
p(352) = p(9)*p(82)
p(353) = p(9)*p(88)
p(354) = p(9)*p(55)
p(355) = p(9)*p(72)
p(356) = p(9)*p(51)
p(357) = p(9)*p(56)
p(358) = p(9)*p(57)
p(359) = p(9)*p(73)
p(360) = p(14)*p(89)
p(361) = p(14)*p(90)
p(362) = p(9)*p(77)
p(363) = p(9)*p(60)
p(364) = p(9)*p(78)
p(365) = p(9)*p(61)
p(366) = p(9)*p(62)
p(367) = p(9)*p(79)
p(368) = p(14)*p(67)
p(369) = p(14)*p(68)
p(370) = p(14)*p(69)
p(371) = p(14)*p(70)
p(372) = p(14)*p(71)
p(373) = p(9)*p(80)
p(374) = p(9)*p(81)
p(375) = p(14)*p(55)
p(376) = p(14)*p(56)
p(377) = p(14)*p(57)
p(378) = p(14)*p(52)
p(379) = p(9)*p(85)
p(380) = p(9)*p(86)
p(381) = p(9)*p(87)
p(382) = p(14)*p(74)
p(383) = p(14)*p(75)
p(384) = p(14)*p(76)
p(385) = p(9)*p(97)
p(386) = p(14)*p(60)
p(387) = p(14)*p(61)
p(388) = p(14)*p(62)
p(389) = p(9)*p(100)
p(390) = p(14)*p(83)
p(391) = p(14)*p(84)
p(392) = p(9)*p(91)
p(393) = p(9)*p(67)
p(394) = p(9)*p(53)
p(395) = p(9)*p(68)
p(396) = p(9)*p(54)
p(397) = p(9)*p(69)
p(398) = p(9)*p(70)
p(399) = p(9)*p(71)
p(400) = p(9)*p(92)
p(401) = p(3)*p(114) - p(354)
p(402) = p(7)*p(114) - p(356)
p(403) = p(24)*p(49) - p(359)
p(404) = p(3)*p(149) - p(355) - p(403)
p(405) = p(1)*p(224) - p(355) - p(404)
p(406) = p(6)*p(178) - p(401)
p(407) = p(32)*p(39) - p(356)
p(408) = p(3)*p(150) - p(359)
p(409) = p(14)*p(101)
p(410) = p(9)*p(93)
p(411) = p(9)*p(74)
p(412) = p(9)*p(59)
p(413) = p(9)*p(75)
p(414) = p(9)*p(76)
p(415) = p(9)*p(94)
p(416) = p(24)*p(41) - p(349)
p(417) = p(4)*p(114) - p(348)
p(418) = p(1)*p(234) - p(363) - p(417)
p(419) = p(1)*p(235) - p(364) - p(418)
p(420) = p(13)*p(91) - p(409)
p(421) = p(2)*p(224) - p(361) - p(420)
p(422) = p(32)*p(41) - p(409)
p(423) = p(1)*p(237) - p(364) - p(421)
p(424) = p(16)*p(73) - p(409)
p(425) = p(13)*p(69) - p(348)
p(426) = p(4)*p(150) - p(409)
p(427) = p(11)*p(73) - p(349)
p(428) = p(14)*p(91)
p(429) = p(14)*p(92)
p(430) = p(3)*p(130) - p(373)
p(431) = p(7)*p(130) - p(374)
p(432) = p(3)*p(159) - p(373) - p(431)
p(433) = p(39)*p(45) - p(430)
p(434) = p(3)*p(160) - p(374)
p(435) = p(14)*p(72)
p(436) = p(14)*p(73)
p(437) = p(14)*p(58)
p(438) = p(9)*p(95)
p(439) = p(9)*p(83)
p(440) = p(9)*p(84)
p(441) = p(9)*p(96)
p(442) = p(24)*p(43) - p(350)
p(443) = p(8)*p(114) - p(370)
p(444) = p(1)*p(253) - p(380) - p(443)
p(445) = p(13)*p(93) - p(428)
p(446) = p(2)*p(237) - p(372) - p(432)
p(447) = p(32)*p(43) - p(428)
p(448) = p(6)*p(153) - p(370)
p(449) = p(18)*p(73) - p(428)
p(450) = p(32)*p(34) - p(350)
p(451) = p(14)*p(93)
p(452) = p(14)*p(94)
p(453) = p(4)*p(130) - p(352)
p(454) = p(1)*p(260) - p(385) - p(453)
p(455) = p(16)*p(81) - p(436)
p(456) = p(2)*p(245) - p(376) - p(455)
p(457) = p(4)*p(160) - p(436)
p(458) = p(11)*p(81) - p(352)
p(459) = p(14)*p(77)
p(460) = p(14)*p(78)
p(461) = p(14)*p(79)
p(462) = p(14)*p(63)
p(463) = p(14)*p(64)
p(464) = p(6)*p(130) - p(386)
p(465) = p(3)*p(187) - p(464)
p(466) = p(14)*p(80)
p(467) = p(14)*p(81)
p(468) = p(9)*p(98)
p(469) = p(9)*p(99)
p(470) = p(24)*p(46) - p(353)
p(471) = p(1)*p(267) - p(389) - p(470)
p(472) = p(13)*p(95) - p(451)
p(473) = p(2)*p(255) - p(384) - p(456)
p(474) = p(32)*p(46) - p(451)
p(475) = p(13)*p(84) - p(353)
p(476) = p(14)*p(95)
p(477) = p(14)*p(96)
p(478) = p(8)*p(130) - p(378)
p(479) = p(18)*p(81) - p(437)
p(480) = p(34)*p(45) - p(378)
p(481) = p(14)*p(85)
p(482) = p(14)*p(86)
p(483) = p(14)*p(87)
p(484) = p(9)*p(102)
p(485) = p(24)*p(50) - p(391)
p(486) = p(13)*p(98) - p(476)
p(487) = p(6)*p(173) - p(391)
p(488) = p(14)*p(98)
p(489) = p(14)*p(99)
p(490) = p(9)*p(65)
p(491) = p(9)*p(66)
p(492) = p(9)*p(89)
p(493) = p(9)*p(90)
p(494) = p(9)*p(101)
p(495) = p(3)*p(181) - p(392)
p(496) = p(39)*p(41) - p(394)
p(497) = p(4)*p(178) - p(397)
p(498) = p(1)*p(278) - p(395) - p(497)
p(499) = p(1)*p(279) - p(397)
p(500) = p(1)*p(280) - p(398) - p(498)
p(501) = p(21)*p(69) - p(394)
p(502) = p(1)*p(282) - p(400) - p(500)
p(503) = p(11)*p(101) - p(392)
p(504) = p(1)*p(284) - p(404) - p(403)
p(505) = p(1)*p(285) - p(408)
p(506) = p(3)*p(183) - p(410)
p(507) = p(39)*p(43) - p(412)
p(508) = p(1)*p(287) - p(411) - p(507)
p(509) = p(2)*p(279) - p(406)
p(510) = p(1)*p(288) - p(413) - p(508)
p(511) = p(7)*p(153) - p(412)
p(512) = p(1)*p(290) - p(415) - p(510)
p(513) = p(34)*p(49) - p(410)
p(514) = p(6)*p(181) - p(409)
p(515) = p(1)*p(293) - p(421) - p(419)
p(516) = p(2)*p(285) - p(409)
p(517) = p(1)*p(295) - p(432) - p(431)
p(518) = p(1)*p(296) - p(434)
p(519) = p(3)*p(185) - p(438)
p(520) = p(39)*p(46) - p(440)
p(521) = p(1)*p(298) - p(439) - p(520)
p(522) = p(1)*p(299) - p(440)
p(523) = p(1)*p(300) - p(441) - p(521)
p(524) = p(21)*p(84) - p(438)
p(525) = p(6)*p(183) - p(428)
p(526) = p(1)*p(303) - p(446) - p(444)
p(527) = p(8)*p(150) - p(428)
p(528) = p(41)*p(45) - p(436)
p(529) = p(1)*p(306) - p(456) - p(454)
p(530) = p(2)*p(296) - p(436)
p(531) = p(4)*p(169) - p(388) - p(478)
p(532) = p(1)*p(310) - p(465)
p(533) = p(14)*p(82)
p(534) = p(3)*p(188) - p(468)
p(535) = p(2)*p(298) - p(447) - p(443)
p(536) = p(2)*p(299) - p(448)
p(537) = p(1)*p(313) - p(469) - p(535)
p(538) = p(7)*p(173) - p(468)
p(539) = p(6)*p(185) - p(451)
p(540) = p(1)*p(316) - p(473) - p(471)
p(541) = p(22)*p(73) - p(451)
p(542) = p(43)*p(45) - p(437)
p(543) = p(1)*p(319) - p(480) - p(478)
p(544) = p(8)*p(160) - p(437)
p(545) = p(14)*p(88)
p(546) = p(4)*p(187) - p(467)
p(547) = p(2)*p(309) - p(466) - p(546)
p(548) = p(2)*p(310) - p(467)
p(549) = p(14)*p(97)
p(550) = p(3)*p(191) - p(484)
p(551) = p(2)*p(313) - p(474) - p(471)
p(552) = p(1)*p(323) - p(484)
p(553) = p(6)*p(188) - p(476)
p(554) = p(1)*p(325) - p(487) - p(485)
p(555) = p(32)*p(50) - p(476)
p(556) = p(45)*p(46) - p(462)
p(557) = p(2)*p(319) - p(482) - p(547)
p(558) = p(22)*p(81) - p(462)
p(559) = p(14)*p(100)
p(560) = p(2)*p(322) - p(486) - p(485)
p(561) = p(2)*p(323) - p(487)
p(562) = p(6)*p(191) - p(488)
p(563) = p(2)*p(325) - p(489) - p(557)
p(564) = p(13)*p(102) - p(488)
p(565) = p(14)*p(102)
p(566) = p(3)*p(178) - p(492)
p(567) = p(7)*p(178) - p(490)
p(568) = p(39)*p(49) - p(491)
p(569) = p(3)*p(194) - p(494)
p(570) = p(1)*p(331) - p(495)
p(571) = p(2)*p(346) - p(570)
p(572) = p(1)*p(333) - p(506)
p(573) = p(8)*p(194) - p(572)
p(574) = p(1)*p(335) - p(519)
p(575) = p(22)*p(101) - p(574)
p(576) = p(1)*p(337) - p(534)
p(577) = p(49)*p(50) - p(576)
p(578) = p(6)*p(187) - p(549)
p(579) = p(1)*p(340) - p(550)
p(580) = p(21)*p(102) - p(579)
p(581) = p(8)*p(187) - p(533)
p(582) = p(1)*p(343) - p(560)
p(583) = p(7)*p(195) - p(582)
p(584) = p(45)*p(50) - p(545)
p(585) = p(2)*p(343) - p(562)
p(586) = p(1)*p(347) - p(585)
p(587) = p(6)*p(195) - p(565)
p(588) = p(1)*p(346) - p(569)
p(589) = p(2)*p(347) - p(587)
p(590) = p(9)*p(103)
p(591) = p(9)*p(106)
p(592) = p(9)*p(119)
p(593) = p(9)*p(128)
p(594) = p(9)*p(108)
p(595) = p(9)*p(109)
p(596) = p(9)*p(129)
p(597) = p(9)*p(132)
p(598) = p(9)*p(133)
p(599) = p(9)*p(161)
p(600) = p(9)*p(138)
p(601) = p(9)*p(139)
p(602) = p(9)*p(171)
p(603) = p(9)*p(177)
p(604) = p(9)*p(114)
p(605) = p(9)*p(115)
p(606) = p(9)*p(149)
p(607) = p(9)*p(104)
p(608) = p(9)*p(116)
p(609) = p(9)*p(105)
p(610) = p(9)*p(117)
p(611) = p(9)*p(118)
p(612) = p(9)*p(150)
p(613) = p(14)*p(178)
p(614) = p(14)*p(179)
p(615) = p(14)*p(180)
p(616) = p(9)*p(156)
p(617) = p(9)*p(122)
p(618) = p(9)*p(123)
p(619) = p(9)*p(157)
p(620) = p(9)*p(124)
p(621) = p(9)*p(107)
p(622) = p(9)*p(125)
p(623) = p(9)*p(126)
p(624) = p(9)*p(127)
p(625) = p(9)*p(158)
p(626) = p(14)*p(143)
p(627) = p(14)*p(144)
p(628) = p(14)*p(145)
p(629) = p(14)*p(146)
p(630) = p(14)*p(147)
p(631) = p(14)*p(148)
p(632) = p(9)*p(130)
p(633) = p(9)*p(159)
p(634) = p(9)*p(131)
p(635) = p(9)*p(160)
p(636) = p(14)*p(114)
p(637) = p(14)*p(115)
p(638) = p(14)*p(116)
p(639) = p(14)*p(117)
p(640) = p(14)*p(118)
p(641) = p(14)*p(106)
p(642) = p(9)*p(165)
p(643) = p(9)*p(135)
p(644) = p(9)*p(166)
p(645) = p(9)*p(136)
p(646) = p(9)*p(137)
p(647) = p(9)*p(167)
p(648) = p(14)*p(151)
p(649) = p(14)*p(152)
p(650) = p(14)*p(153)
p(651) = p(14)*p(154)
p(652) = p(14)*p(155)
p(653) = p(9)*p(168)
p(654) = p(9)*p(169)
p(655) = p(9)*p(170)
p(656) = p(14)*p(122)
p(657) = p(14)*p(123)
p(658) = p(14)*p(124)
p(659) = p(14)*p(125)
p(660) = p(14)*p(126)
p(661) = p(14)*p(127)
p(662) = p(14)*p(108)
p(663) = p(14)*p(109)
p(664) = p(9)*p(187)
p(665) = p(14)*p(130)
p(666) = p(14)*p(131)
p(667) = p(9)*p(174)
p(668) = p(9)*p(175)
p(669) = p(9)*p(176)
p(670) = p(14)*p(162)
p(671) = p(14)*p(163)
p(672) = p(14)*p(164)
p(673) = p(9)*p(190)
p(674) = p(14)*p(135)
p(675) = p(14)*p(136)
p(676) = p(14)*p(137)
p(677) = p(9)*p(193)
p(678) = p(14)*p(172)
p(679) = p(14)*p(173)
p(680) = p(9)*p(181)
p(681) = p(9)*p(143)
p(682) = p(9)*p(110)
p(683) = p(9)*p(144)
p(684) = p(9)*p(111)
p(685) = p(9)*p(112)
p(686) = p(9)*p(145)
p(687) = p(9)*p(113)
p(688) = p(9)*p(146)
p(689) = p(9)*p(147)
p(690) = p(9)*p(148)
p(691) = p(9)*p(182)
p(692) = p(1)*p(401) - p(604)
p(693) = p(21)*p(114) - p(609)
p(694) = p(24)*p(101) - p(612)
p(695) = p(3)*p(284) - p(606) - p(694)
p(696) = p(1)*p(404) - p(606) - p(695)
p(697) = p(1)*p(405) - p(608) - p(696)
p(698) = p(13)*p(178) - p(604)
p(699) = p(39)*p(73) - p(609)
p(700) = p(3)*p(285) - p(612)
p(701) = p(14)*p(194)
p(702) = p(9)*p(183)
p(703) = p(9)*p(151)
p(704) = p(9)*p(120)
p(705) = p(9)*p(152)
p(706) = p(9)*p(121)
p(707) = p(9)*p(153)
p(708) = p(9)*p(154)
p(709) = p(9)*p(155)
p(710) = p(9)*p(184)
p(711) = p(24)*p(91) - p(592)
p(712) = p(1)*p(416) - p(616) - p(711)
p(713) = p(2)*p(401) - p(613)
p(714) = p(1)*p(418) - p(618) - p(713)
p(715) = p(1)*p(419) - p(619) - p(714)
p(716) = p(13)*p(181) - p(701)
p(717) = p(2)*p(404) - p(615) - p(716)
p(718) = p(32)*p(91) - p(701)
p(719) = p(1)*p(421) - p(619) - p(717)
p(720) = p(41)*p(73) - p(701)
p(721) = p(6)*p(279) - p(613)
p(722) = p(16)*p(150) - p(701)
p(723) = p(1)*p(425) - p(624) - p(721)
p(724) = p(4)*p(285) - p(701)
p(725) = p(11)*p(150) - p(592)
p(726) = p(14)*p(181)
p(727) = p(14)*p(182)
p(728) = p(1)*p(430) - p(632)
p(729) = p(21)*p(130) - p(635)
p(730) = p(3)*p(295) - p(633) - p(729)
p(731) = p(1)*p(432) - p(633) - p(730)
p(732) = p(39)*p(81) - p(632)
p(733) = p(3)*p(296) - p(635)
p(734) = p(14)*p(149)
p(735) = p(14)*p(150)
p(736) = p(14)*p(119)
p(737) = p(9)*p(185)
p(738) = p(9)*p(162)
p(739) = p(9)*p(134)
p(740) = p(9)*p(163)
p(741) = p(9)*p(164)
p(742) = p(9)*p(186)
p(743) = p(24)*p(93) - p(593)
p(744) = p(18)*p(114) - p(595)
p(745) = p(1)*p(443) - p(643) - p(744)
p(746) = p(1)*p(444) - p(644) - p(745)
p(747) = p(13)*p(183) - p(726)
p(748) = p(2)*p(421) - p(631) - p(730)
p(749) = p(32)*p(93) - p(726)
p(750) = p(1)*p(446) - p(644) - p(748)
p(751) = p(43)*p(73) - p(726)
p(752) = p(13)*p(153) - p(595)
p(753) = p(18)*p(150) - p(726)
p(754) = p(34)*p(73) - p(593)
p(755) = p(14)*p(183)
p(756) = p(14)*p(184)
p(757) = p(16)*p(130) - p(598)
p(758) = p(2)*p(430) - p(636)
p(759) = p(1)*p(454) - p(654) - p(758)
p(760) = p(41)*p(81) - p(735)
p(761) = p(2)*p(432) - p(638) - p(760)
p(762) = p(16)*p(160) - p(735)
p(763) = p(45)*p(69) - p(636)
p(764) = p(4)*p(296) - p(735)
p(765) = p(11)*p(160) - p(598)
p(766) = p(14)*p(156)
p(767) = p(14)*p(157)
p(768) = p(14)*p(158)
p(769) = p(14)*p(128)
p(770) = p(14)*p(129)
p(771) = p(1)*p(464) - p(664)
p(772) = p(4)*p(262) - p(673) - p(661)
p(773) = p(3)*p(310) - p(664)
p(774) = p(14)*p(159)
p(775) = p(14)*p(160)
p(776) = p(14)*p(132)
p(777) = p(14)*p(133)
p(778) = p(9)*p(188)
p(779) = p(9)*p(172)
p(780) = p(9)*p(173)
p(781) = p(9)*p(189)
p(782) = p(24)*p(95) - p(600)
p(783) = p(22)*p(114) - p(650)
p(784) = p(1)*p(471) - p(668) - p(783)
p(785) = p(13)*p(185) - p(755)
p(786) = p(2)*p(446) - p(652) - p(761)
p(787) = p(32)*p(95) - p(755)
p(788) = p(6)*p(299) - p(650)
p(789) = p(46)*p(73) - p(755)
p(790) = p(32)*p(84) - p(600)
p(791) = p(14)*p(185)
p(792) = p(14)*p(186)
p(793) = p(2)*p(453) - p(656) - p(771)
p(794) = p(1)*p(478) - p(673) - p(793)
p(795) = p(43)*p(81) - p(736)
p(796) = p(2)*p(456) - p(659) - p(772)
p(797) = p(18)*p(160) - p(736)
p(798) = p(1)*p(480) - p(673) - p(796)
p(799) = p(14)*p(165)
p(800) = p(14)*p(166)
p(801) = p(14)*p(167)
p(802) = p(14)*p(138)
p(803) = p(14)*p(139)
p(804) = p(2)*p(464) - p(665)
p(805) = p(4)*p(310) - p(775)
p(806) = p(11)*p(187) - p(665)
p(807) = p(14)*p(168)
p(808) = p(14)*p(169)
p(809) = p(14)*p(170)
p(810) = p(9)*p(191)
p(811) = p(9)*p(192)
p(812) = p(24)*p(98) - p(603)
p(813) = p(1)*p(485) - p(677) - p(812)
p(814) = p(13)*p(188) - p(791)
p(815) = p(2)*p(473) - p(672) - p(796)
p(816) = p(32)*p(98) - p(791)
p(817) = p(13)*p(173) - p(603)
p(818) = p(14)*p(188)
p(819) = p(14)*p(189)
p(820) = p(22)*p(130) - p(663)
p(821) = p(46)*p(81) - p(769)
p(822) = p(45)*p(84) - p(663)
p(823) = p(14)*p(174)
p(824) = p(14)*p(175)
p(825) = p(14)*p(176)
p(826) = p(9)*p(195)
p(827) = p(24)*p(102) - p(679)
p(828) = p(13)*p(191) - p(818)
p(829) = p(6)*p(323) - p(679)
p(830) = p(14)*p(191)
p(831) = p(14)*p(192)
p(832) = p(9)*p(140)
p(833) = p(9)*p(141)
p(834) = p(9)*p(178)
p(835) = p(9)*p(142)
p(836) = p(9)*p(179)
p(837) = p(9)*p(180)
p(838) = p(9)*p(194)
p(839) = p(3)*p(331) - p(680)
p(840) = p(39)*p(91) - p(682)
p(841) = p(16)*p(178) - p(685)
p(842) = p(1)*p(497) - p(683) - p(841)
p(843) = p(2)*p(566) - p(842)
p(844) = p(1)*p(498) - p(686) - p(842)
p(845) = p(7)*p(279) - p(685)
p(846) = p(1)*p(500) - p(689) - p(844)
p(847) = p(49)*p(69) - p(682)
p(848) = p(1)*p(502) - p(691) - p(846)
p(849) = p(11)*p(194) - p(680)
p(850) = p(1)*p(504) - p(695) - p(694)
p(851) = p(1)*p(505) - p(700)
p(852) = p(3)*p(333) - p(702)
p(853) = p(39)*p(93) - p(704)
p(854) = p(18)*p(178) - p(707)
p(855) = p(1)*p(508) - p(705) - p(854)
p(856) = p(1)*p(509) - p(707)
p(857) = p(1)*p(510) - p(708) - p(855)
p(858) = p(21)*p(153) - p(704)
p(859) = p(1)*p(512) - p(710) - p(857)
p(860) = p(34)*p(101) - p(702)
p(861) = p(6)*p(331) - p(701)
p(862) = p(1)*p(515) - p(717) - p(715)
p(863) = p(2)*p(505) - p(701)
p(864) = p(1)*p(517) - p(730) - p(729)
p(865) = p(1)*p(518) - p(733)
p(866) = p(3)*p(335) - p(737)
p(867) = p(39)*p(95) - p(739)
p(868) = p(1)*p(520) - p(738) - p(867)
p(869) = p(2)*p(509) - p(721)
p(870) = p(1)*p(521) - p(740) - p(868)
p(871) = p(7)*p(299) - p(739)
p(872) = p(1)*p(523) - p(742) - p(870)
p(873) = p(49)*p(84) - p(737)
p(874) = p(6)*p(333) - p(726)
p(875) = p(1)*p(526) - p(748) - p(746)
p(876) = p(8)*p(285) - p(726)
p(877) = p(45)*p(91) - p(735)
p(878) = p(1)*p(529) - p(761) - p(759)
p(879) = p(2)*p(518) - p(735)
p(880) = p(1)*p(531) - p(772) - p(771)
p(881) = p(1)*p(532) - p(773)
p(882) = p(14)*p(161)
p(883) = p(3)*p(337) - p(778)
p(884) = p(39)*p(98) - p(780)
p(885) = p(1)*p(535) - p(779) - p(884)
p(886) = p(1)*p(536) - p(780)
p(887) = p(1)*p(537) - p(781) - p(885)
p(888) = p(21)*p(173) - p(778)
p(889) = p(6)*p(335) - p(755)
p(890) = p(1)*p(540) - p(786) - p(784)
p(891) = p(22)*p(150) - p(755)
p(892) = p(45)*p(93) - p(736)
p(893) = p(1)*p(543) - p(796) - p(794)
p(894) = p(8)*p(296) - p(736)
p(895) = p(16)*p(187) - p(775)
p(896) = p(1)*p(547) - p(806) - p(804)
p(897) = p(2)*p(532) - p(775)
p(898) = p(14)*p(171)
p(899) = p(6)*p(309) - p(808) - p(807)
p(900) = p(1)*p(578) - p(899)
p(901) = p(14)*p(187)
p(902) = p(3)*p(340) - p(810)
p(903) = p(2)*p(535) - p(787) - p(783)
p(904) = p(2)*p(536) - p(788)
p(905) = p(1)*p(551) - p(811) - p(903)
p(906) = p(7)*p(323) - p(810)
p(907) = p(6)*p(337) - p(791)
p(908) = p(1)*p(554) - p(815) - p(813)
p(909) = p(50)*p(73) - p(791)
p(910) = p(45)*p(95) - p(769)
p(911) = p(1)*p(557) - p(822) - p(820)
p(912) = p(22)*p(160) - p(769)
p(913) = p(14)*p(177)
p(914) = p(18)*p(187) - p(777)
p(915) = p(2)*p(547) - p(808) - p(899)
p(916) = p(8)*p(310) - p(777)
p(917) = p(14)*p(190)
p(918) = p(3)*p(343) - p(826)
p(919) = p(2)*p(551) - p(816) - p(813)
p(920) = p(1)*p(561) - p(826)
p(921) = p(6)*p(340) - p(818)
p(922) = p(1)*p(563) - p(829) - p(827)
p(923) = p(32)*p(102) - p(818)
p(924) = p(45)*p(98) - p(802)
p(925) = p(2)*p(557) - p(824) - p(915)
p(926) = p(50)*p(81) - p(802)
p(927) = p(14)*p(193)
p(928) = p(2)*p(560) - p(828) - p(827)
p(929) = p(2)*p(561) - p(829)
p(930) = p(6)*p(343) - p(830)
p(931) = p(2)*p(563) - p(831) - p(925)
p(932) = p(13)*p(195) - p(830)
p(933) = p(14)*p(195)
p(934) = p(1)*p(566) - p(834)
p(935) = p(1)*p(567) - p(836) - p(934)
p(936) = p(39)*p(101) - p(835)
p(937) = p(3)*p(346) - p(838)
p(938) = p(1)*p(570) - p(839)
p(939) = p(2)*p(588) - p(938)
p(940) = p(1)*p(572) - p(852)
p(941) = p(8)*p(346) - p(940)
p(942) = p(1)*p(574) - p(866)
p(943) = p(22)*p(194) - p(942)
p(944) = p(1)*p(576) - p(883)
p(945) = p(50)*p(101) - p(944)
p(946) = p(1)*p(579) - p(902)
p(947) = p(49)*p(102) - p(946)
p(948) = p(2)*p(578) - p(901)
p(949) = p(1)*p(582) - p(918)
p(950) = p(21)*p(195) - p(949)
p(951) = p(2)*p(581) - p(917) - p(948)
p(952) = p(1)*p(585) - p(928)
p(953) = p(7)*p(347) - p(952)
p(954) = p(45)*p(102) - p(913)
p(955) = p(2)*p(585) - p(930)
p(956) = p(1)*p(589) - p(955)
p(957) = p(6)*p(347) - p(933)
p(958) = p(1)*p(588) - p(937)
p(959) = p(2)*p(589) - p(957)
p(960) = p(9)*p(196)
p(961) = p(9)*p(202)
p(962) = p(9)*p(203)
p(963) = p(9)*p(228)
p(964) = p(9)*p(242)
p(965) = p(9)*p(207)
p(966) = p(9)*p(197)
p(967) = p(9)*p(208)
p(968) = p(9)*p(209)
p(969) = p(9)*p(243)
p(970) = p(9)*p(211)
p(971) = p(9)*p(247)
p(972) = p(9)*p(212)
p(973) = p(9)*p(248)
p(974) = p(9)*p(249)
p(975) = p(9)*p(258)
p(976) = p(9)*p(214)
p(977) = p(9)*p(215)
p(978) = p(9)*p(259)
p(979) = p(9)*p(263)
p(980) = p(9)*p(264)
p(981) = p(9)*p(265)
p(982) = p(9)*p(308)
p(983) = p(9)*p(311)
p(984) = p(9)*p(270)
p(985) = p(9)*p(271)
p(986) = p(9)*p(321)
p(987) = p(9)*p(327)
p(988) = p(9)*p(222)
p(989) = p(9)*p(223)
p(990) = p(9)*p(284)
p(991) = p(9)*p(198)
p(992) = p(9)*p(199)
p(993) = p(9)*p(224)
p(994) = p(9)*p(200)
p(995) = p(9)*p(225)
p(996) = p(9)*p(201)
p(997) = p(9)*p(226)
p(998) = p(9)*p(227)
p(999) = p(9)*p(285)
p(1000) = p(14)*p(328)
p(1001) = p(14)*p(329)
p(1002) = p(14)*p(330)
p(1003) = p(9)*p(292)
p(1004) = p(9)*p(233)
p(1005) = p(9)*p(234)
p(1006) = p(9)*p(235)
p(1007) = p(9)*p(293)
p(1008) = p(9)*p(236)
p(1009) = p(9)*p(204)
p(1010) = p(9)*p(205)
p(1011) = p(9)*p(237)
p(1012) = p(9)*p(238)
p(1013) = p(9)*p(206)
p(1014) = p(9)*p(239)
p(1015) = p(9)*p(240)
p(1016) = p(9)*p(241)
p(1017) = p(9)*p(294)
p(1018) = p(14)*p(276)
p(1019) = p(14)*p(277)
p(1020) = p(14)*p(278)
p(1021) = p(14)*p(279)
p(1022) = p(14)*p(280)
p(1023) = p(14)*p(281)
p(1024) = p(14)*p(282)
p(1025) = p(14)*p(283)
p(1026) = p(9)*p(244)
p(1027) = p(9)*p(295)
p(1028) = p(9)*p(210)
p(1029) = p(9)*p(245)
p(1030) = p(9)*p(246)
p(1031) = p(9)*p(296)
p(1032) = p(14)*p(222)
p(1033) = p(14)*p(223)
p(1034) = p(14)*p(224)
p(1035) = p(14)*p(225)
p(1036) = p(14)*p(226)
p(1037) = p(14)*p(227)
p(1038) = p(14)*p(202)
p(1039) = p(14)*p(203)
p(1040) = p(9)*p(302)
p(1041) = p(9)*p(252)
p(1042) = p(9)*p(253)
p(1043) = p(9)*p(303)
p(1044) = p(9)*p(254)
p(1045) = p(9)*p(213)
p(1046) = p(9)*p(255)
p(1047) = p(9)*p(256)
p(1048) = p(9)*p(257)
p(1049) = p(9)*p(304)
p(1050) = p(14)*p(286)
p(1051) = p(14)*p(287)
p(1052) = p(14)*p(288)
p(1053) = p(14)*p(289)
p(1054) = p(14)*p(290)
p(1055) = p(14)*p(291)
p(1056) = p(9)*p(305)
p(1057) = p(9)*p(260)
p(1058) = p(9)*p(306)
p(1059) = p(9)*p(261)
p(1060) = p(9)*p(262)
p(1061) = p(9)*p(307)
p(1062) = p(14)*p(233)
p(1063) = p(14)*p(234)
p(1064) = p(14)*p(235)
p(1065) = p(14)*p(236)
p(1066) = p(14)*p(237)
p(1067) = p(14)*p(238)
p(1068) = p(14)*p(239)
p(1069) = p(14)*p(240)
p(1070) = p(14)*p(241)
p(1071) = p(14)*p(207)
p(1072) = p(14)*p(208)
p(1073) = p(14)*p(209)
p(1074) = p(9)*p(309)
p(1075) = p(9)*p(310)
p(1076) = p(14)*p(244)
p(1077) = p(14)*p(245)
p(1078) = p(14)*p(246)
p(1079) = p(14)*p(211)
p(1080) = p(14)*p(212)
p(1081) = p(9)*p(315)
p(1082) = p(9)*p(267)
p(1083) = p(9)*p(316)
p(1084) = p(9)*p(268)
p(1085) = p(9)*p(269)
p(1086) = p(9)*p(317)
p(1087) = p(14)*p(297)
p(1088) = p(14)*p(298)
p(1089) = p(14)*p(299)
p(1090) = p(14)*p(300)
p(1091) = p(14)*p(301)
p(1092) = p(9)*p(318)
p(1093) = p(9)*p(319)
p(1094) = p(9)*p(320)
p(1095) = p(14)*p(252)
p(1096) = p(14)*p(253)
p(1097) = p(14)*p(254)
p(1098) = p(14)*p(255)
p(1099) = p(14)*p(256)
p(1100) = p(14)*p(257)
p(1101) = p(14)*p(214)
p(1102) = p(14)*p(215)
p(1103) = p(9)*p(339)
p(1104) = p(14)*p(260)
p(1105) = p(14)*p(261)
p(1106) = p(14)*p(262)
p(1107) = p(9)*p(324)
p(1108) = p(9)*p(325)
p(1109) = p(9)*p(326)
p(1110) = p(14)*p(312)
p(1111) = p(14)*p(313)
p(1112) = p(14)*p(314)
p(1113) = p(9)*p(342)
p(1114) = p(14)*p(267)
p(1115) = p(14)*p(268)
p(1116) = p(14)*p(269)
p(1117) = p(9)*p(345)
p(1118) = p(14)*p(322)
p(1119) = p(14)*p(323)
p(1120) = p(9)*p(331)
p(1121) = p(9)*p(276)
p(1122) = p(9)*p(216)
p(1123) = p(9)*p(277)
p(1124) = p(9)*p(217)
p(1125) = p(9)*p(218)
p(1126) = p(9)*p(278)
p(1127) = p(9)*p(219)
p(1128) = p(9)*p(220)
p(1129) = p(9)*p(279)
p(1130) = p(9)*p(280)
p(1131) = p(9)*p(221)
p(1132) = p(9)*p(281)
p(1133) = p(9)*p(282)
p(1134) = p(9)*p(283)
p(1135) = p(9)*p(332)
p(1136) = p(3)*p(401) - p(988)
p(1137) = p(7)*p(401) - p(991)
p(1138) = p(49)*p(114) - p(996)
p(1139) = p(24)*p(194) - p(999)
p(1140) = p(3)*p(504) - p(990) - p(1139)
p(1141) = p(1)*p(695) - p(990) - p(1140)
p(1142) = p(1)*p(696) - p(993) - p(1141)
p(1143) = p(6)*p(566) - p(1136)
p(1144) = p(32)*p(178) - p(991)
p(1145) = p(39)*p(150) - p(996)
p(1146) = p(3)*p(505) - p(999)
p(1147) = p(14)*p(346)
p(1148) = p(9)*p(333)
p(1149) = p(9)*p(286)
p(1150) = p(9)*p(229)
p(1151) = p(9)*p(287)
p(1152) = p(9)*p(230)
p(1153) = p(9)*p(231)
p(1154) = p(9)*p(288)
p(1155) = p(9)*p(232)
p(1156) = p(9)*p(289)
p(1157) = p(9)*p(290)
p(1158) = p(9)*p(291)
p(1159) = p(9)*p(334)
p(1160) = p(24)*p(181) - p(963)
p(1161) = p(41)*p(114) - p(960)
p(1162) = p(4)*p(401) - p(961)
p(1163) = p(1)*p(713) - p(1005) - p(1162)
p(1164) = p(1)*p(714) - p(1006) - p(1163)
p(1165) = p(1)*p(715) - p(1007) - p(1164)
p(1166) = p(13)*p(331) - p(1147)
p(1167) = p(2)*p(695) - p(1002) - p(1166)
p(1168) = p(32)*p(181) - p(1147)
p(1169) = p(1)*p(717) - p(1007) - p(1167)
p(1170) = p(73)*p(91) - p(1147)
p(1171) = p(1)*p(719) - p(1011) - p(1169)
p(1172) = p(41)*p(150) - p(1147)
p(1173) = p(13)*p(279) - p(961)
p(1174) = p(16)*p(285) - p(1147)
p(1175) = p(69)*p(73) - p(960)
p(1176) = p(4)*p(505) - p(1147)
p(1177) = p(11)*p(285) - p(963)
p(1178) = p(14)*p(331)
p(1179) = p(14)*p(332)
p(1180) = p(3)*p(430) - p(1026)
p(1181) = p(7)*p(430) - p(1028)
p(1182) = p(49)*p(130) - p(1031)
p(1183) = p(3)*p(517) - p(1027) - p(1182)
p(1184) = p(1)*p(730) - p(1027) - p(1183)
p(1185) = p(45)*p(178) - p(1180)
p(1186) = p(39)*p(160) - p(1028)
p(1187) = p(3)*p(518) - p(1031)
p(1188) = p(14)*p(284)
p(1189) = p(14)*p(285)
p(1190) = p(14)*p(228)
p(1191) = p(9)*p(335)
p(1192) = p(9)*p(297)
p(1193) = p(9)*p(250)
p(1194) = p(9)*p(298)
p(1195) = p(9)*p(251)
p(1196) = p(9)*p(299)
p(1197) = p(9)*p(300)
p(1198) = p(9)*p(301)
p(1199) = p(9)*p(336)
p(1200) = p(24)*p(183) - p(964)
p(1201) = p(43)*p(114) - p(966)
p(1202) = p(8)*p(401) - p(1021)
p(1203) = p(1)*p(745) - p(1042) - p(1202)
p(1204) = p(1)*p(746) - p(1043) - p(1203)
p(1205) = p(13)*p(333) - p(1178)
p(1206) = p(2)*p(717) - p(1025) - p(1183)
p(1207) = p(32)*p(183) - p(1178)
p(1208) = p(1)*p(748) - p(1043) - p(1206)
p(1209) = p(73)*p(93) - p(1178)
p(1210) = p(6)*p(509) - p(1021)
p(1211) = p(43)*p(150) - p(1178)
p(1212) = p(32)*p(153) - p(966)
p(1213) = p(18)*p(285) - p(1178)
p(1214) = p(34)*p(150) - p(964)
p(1215) = p(14)*p(333)
p(1216) = p(14)*p(334)
p(1217) = p(41)*p(130) - p(973)
p(1218) = p(4)*p(430) - p(970)
p(1219) = p(1)*p(758) - p(1057) - p(1218)
p(1220) = p(1)*p(759) - p(1058) - p(1219)
p(1221) = p(81)*p(91) - p(1189)
p(1222) = p(2)*p(730) - p(1034) - p(1221)
p(1223) = p(41)*p(160) - p(1189)
p(1224) = p(1)*p(761) - p(1058) - p(1222)
p(1225) = p(16)*p(296) - p(1189)
p(1226) = p(69)*p(81) - p(970)
p(1227) = p(4)*p(518) - p(1189)
p(1228) = p(11)*p(296) - p(973)
p(1229) = p(14)*p(292)
p(1230) = p(14)*p(293)
p(1231) = p(14)*p(294)
p(1232) = p(14)*p(242)
p(1233) = p(14)*p(243)
p(1234) = p(3)*p(464) - p(1074)
p(1235) = p(7)*p(464) - p(1075)
p(1236) = p(3)*p(531) - p(1074) - p(1235)
p(1237) = p(39)*p(187) - p(1234)
p(1238) = p(3)*p(532) - p(1075)
p(1239) = p(14)*p(295)
p(1240) = p(14)*p(296)
p(1241) = p(14)*p(247)
p(1242) = p(14)*p(248)
p(1243) = p(14)*p(249)
p(1244) = p(9)*p(337)
p(1245) = p(9)*p(312)
p(1246) = p(9)*p(266)
p(1247) = p(9)*p(313)
p(1248) = p(9)*p(314)
p(1249) = p(9)*p(338)
p(1250) = p(24)*p(185) - p(975)
p(1251) = p(46)*p(114) - p(977)
p(1252) = p(1)*p(783) - p(1082) - p(1251)
p(1253) = p(1)*p(784) - p(1083) - p(1252)
p(1254) = p(13)*p(335) - p(1215)
p(1255) = p(2)*p(748) - p(1055) - p(1222)
p(1256) = p(32)*p(185) - p(1215)
p(1257) = p(1)*p(786) - p(1083) - p(1255)
p(1258) = p(73)*p(95) - p(1215)
p(1259) = p(13)*p(299) - p(977)
p(1260) = p(46)*p(150) - p(1215)
p(1261) = p(73)*p(84) - p(975)
p(1262) = p(14)*p(335)
p(1263) = p(14)*p(336)
p(1264) = p(43)*p(130) - p(974)
p(1265) = p(8)*p(430) - p(1038)
p(1266) = p(1)*p(794) - p(1093) - p(1265)
p(1267) = p(81)*p(93) - p(1190)
p(1268) = p(2)*p(761) - p(1066) - p(1236)
p(1269) = p(43)*p(160) - p(1190)
p(1270) = p(45)*p(153) - p(1038)
p(1271) = p(18)*p(296) - p(1190)
p(1272) = p(34)*p(160) - p(974)
p(1273) = p(14)*p(302)
p(1274) = p(14)*p(303)
p(1275) = p(14)*p(304)
p(1276) = p(14)*p(258)
p(1277) = p(14)*p(259)
p(1278) = p(4)*p(464) - p(983)
p(1279) = p(1)*p(804) - p(1103) - p(1278)
p(1280) = p(16)*p(310) - p(1240)
p(1281) = p(2)*p(772) - p(1077) - p(1280)
p(1282) = p(4)*p(532) - p(1240)
p(1283) = p(11)*p(310) - p(983)
p(1284) = p(14)*p(305)
p(1285) = p(14)*p(306)
p(1286) = p(14)*p(307)
p(1287) = p(14)*p(263)
p(1288) = p(14)*p(264)
p(1289) = p(14)*p(265)
p(1290) = p(6)*p(464) - p(1104)
p(1291) = p(3)*p(578) - p(1290)
p(1292) = p(14)*p(309)
p(1293) = p(14)*p(310)
p(1294) = p(9)*p(340)
p(1295) = p(9)*p(322)
p(1296) = p(9)*p(323)
p(1297) = p(9)*p(341)
p(1298) = p(24)*p(188) - p(984)
p(1299) = p(50)*p(114) - p(1089)
p(1300) = p(1)*p(813) - p(1108) - p(1299)
p(1301) = p(13)*p(337) - p(1262)
p(1302) = p(2)*p(786) - p(1091) - p(1268)
p(1303) = p(32)*p(188) - p(1262)
p(1304) = p(6)*p(536) - p(1089)
p(1305) = p(73)*p(98) - p(1262)
p(1306) = p(32)*p(173) - p(984)
p(1307) = p(14)*p(337)
p(1308) = p(14)*p(338)
p(1309) = p(46)*p(130) - p(982)
p(1310) = p(1)*p(820) - p(1113) - p(1309)
p(1311) = p(81)*p(95) - p(1232)
p(1312) = p(2)*p(796) - p(1098) - p(1281)
p(1313) = p(46)*p(160) - p(1232)
p(1314) = p(81)*p(84) - p(982)
p(1315) = p(14)*p(315)
p(1316) = p(14)*p(316)
p(1317) = p(14)*p(317)
p(1318) = p(14)*p(270)
p(1319) = p(14)*p(271)
p(1320) = p(8)*p(464) - p(1079)
p(1321) = p(18)*p(310) - p(1242)
p(1322) = p(34)*p(187) - p(1079)
p(1323) = p(14)*p(318)
p(1324) = p(14)*p(319)
p(1325) = p(14)*p(320)
p(1326) = p(9)*p(343)
p(1327) = p(9)*p(344)
p(1328) = p(24)*p(191) - p(987)
p(1329) = p(1)*p(827) - p(1117) - p(1328)
p(1330) = p(13)*p(340) - p(1307)
p(1331) = p(2)*p(815) - p(1112) - p(1312)
p(1332) = p(32)*p(191) - p(1307)
p(1333) = p(13)*p(323) - p(987)
p(1334) = p(14)*p(340)
p(1335) = p(14)*p(341)
p(1336) = p(50)*p(130) - p(1102)
p(1337) = p(81)*p(98) - p(1276)
p(1338) = p(45)*p(173) - p(1102)
p(1339) = p(14)*p(324)
p(1340) = p(14)*p(325)
p(1341) = p(14)*p(326)
p(1342) = p(9)*p(347)
p(1343) = p(24)*p(195) - p(1119)
p(1344) = p(13)*p(343) - p(1334)
p(1345) = p(6)*p(561) - p(1119)
p(1346) = p(14)*p(343)
p(1347) = p(14)*p(344)
p(1348) = p(9)*p(272)
p(1349) = p(9)*p(273)
p(1350) = p(9)*p(274)
p(1351) = p(9)*p(328)
p(1352) = p(9)*p(275)
p(1353) = p(9)*p(329)
p(1354) = p(9)*p(330)
p(1355) = p(9)*p(346)
p(1356) = p(3)*p(570) - p(1120)
p(1357) = p(39)*p(181) - p(1122)
p(1358) = p(41)*p(178) - p(1125)
p(1359) = p(4)*p(566) - p(1129)
p(1360) = p(1)*p(842) - p(1126) - p(1359)
p(1361) = p(1)*p(843) - p(1129)
p(1362) = p(1)*p(844) - p(1130) - p(1360)
p(1363) = p(21)*p(279) - p(1125)
p(1364) = p(1)*p(846) - p(1133) - p(1362)
p(1365) = p(69)*p(101) - p(1122)
p(1366) = p(1)*p(848) - p(1135) - p(1364)
p(1367) = p(11)*p(346) - p(1120)
p(1368) = p(1)*p(850) - p(1140) - p(1139)
p(1369) = p(1)*p(851) - p(1146)
p(1370) = p(3)*p(572) - p(1148)
p(1371) = p(39)*p(183) - p(1150)
p(1372) = p(43)*p(178) - p(1153)
p(1373) = p(1)*p(854) - p(1151) - p(1372)
p(1374) = p(2)*p(843) - p(1143)
p(1375) = p(1)*p(855) - p(1154) - p(1373)
p(1376) = p(7)*p(509) - p(1153)
p(1377) = p(1)*p(857) - p(1157) - p(1375)
p(1378) = p(49)*p(153) - p(1150)
p(1379) = p(1)*p(859) - p(1159) - p(1377)
p(1380) = p(34)*p(194) - p(1148)
p(1381) = p(6)*p(570) - p(1147)
p(1382) = p(1)*p(862) - p(1167) - p(1165)
p(1383) = p(2)*p(851) - p(1147)
p(1384) = p(1)*p(864) - p(1183) - p(1182)
p(1385) = p(1)*p(865) - p(1187)
p(1386) = p(3)*p(574) - p(1191)
p(1387) = p(39)*p(185) - p(1193)
p(1388) = p(46)*p(178) - p(1196)
p(1389) = p(1)*p(868) - p(1194) - p(1388)
p(1390) = p(1)*p(869) - p(1196)
p(1391) = p(1)*p(870) - p(1197) - p(1389)
p(1392) = p(21)*p(299) - p(1193)
p(1393) = p(1)*p(872) - p(1199) - p(1391)
p(1394) = p(84)*p(101) - p(1191)
p(1395) = p(6)*p(572) - p(1178)
p(1396) = p(1)*p(875) - p(1206) - p(1204)
p(1397) = p(8)*p(505) - p(1178)
p(1398) = p(45)*p(181) - p(1189)
p(1399) = p(1)*p(878) - p(1222) - p(1220)
p(1400) = p(2)*p(865) - p(1189)
p(1401) = p(1)*p(880) - p(1236) - p(1235)
p(1402) = p(1)*p(881) - p(1238)
p(1403) = p(3)*p(576) - p(1244)
p(1404) = p(39)*p(188) - p(1246)
p(1405) = p(1)*p(884) - p(1245) - p(1404)
p(1406) = p(2)*p(869) - p(1210)
p(1407) = p(1)*p(885) - p(1247) - p(1405)
p(1408) = p(7)*p(536) - p(1246)
p(1409) = p(1)*p(887) - p(1249) - p(1407)
p(1410) = p(49)*p(173) - p(1244)
p(1411) = p(6)*p(574) - p(1215)
p(1412) = p(1)*p(890) - p(1255) - p(1253)
p(1413) = p(22)*p(285) - p(1215)
p(1414) = p(45)*p(183) - p(1190)
p(1415) = p(1)*p(893) - p(1268) - p(1266)
p(1416) = p(8)*p(518) - p(1190)
p(1417) = p(41)*p(187) - p(1240)
p(1418) = p(1)*p(896) - p(1281) - p(1279)
p(1419) = p(2)*p(881) - p(1240)
p(1420) = p(14)*p(308)
p(1421) = p(4)*p(547) - p(1106) - p(1320)
p(1422) = p(1)*p(900) - p(1291)
p(1423) = p(14)*p(311)
p(1424) = p(3)*p(579) - p(1294)
p(1425) = p(39)*p(191) - p(1296)
p(1426) = p(1)*p(903) - p(1295) - p(1425)
p(1427) = p(1)*p(904) - p(1296)
p(1428) = p(1)*p(905) - p(1297) - p(1426)
p(1429) = p(21)*p(323) - p(1294)
p(1430) = p(6)*p(576) - p(1262)
p(1431) = p(1)*p(908) - p(1302) - p(1300)
p(1432) = p(50)*p(150) - p(1262)
p(1433) = p(45)*p(185) - p(1232)
p(1434) = p(1)*p(911) - p(1312) - p(1310)
p(1435) = p(22)*p(296) - p(1232)
p(1436) = p(43)*p(187) - p(1242)
p(1437) = p(1)*p(915) - p(1322) - p(1320)
p(1438) = p(8)*p(532) - p(1242)
p(1439) = p(14)*p(321)
p(1440) = p(4)*p(578) - p(1293)
p(1441) = p(2)*p(899) - p(1292) - p(1440)
p(1442) = p(2)*p(900) - p(1293)
p(1443) = p(14)*p(339)
p(1444) = p(3)*p(582) - p(1326)
p(1445) = p(2)*p(903) - p(1303) - p(1299)
p(1446) = p(2)*p(904) - p(1304)
p(1447) = p(1)*p(919) - p(1327) - p(1445)
p(1448) = p(7)*p(561) - p(1326)
p(1449) = p(6)*p(579) - p(1307)
p(1450) = p(1)*p(922) - p(1331) - p(1329)
p(1451) = p(73)*p(102) - p(1307)
p(1452) = p(45)*p(188) - p(1276)
p(1453) = p(1)*p(925) - p(1338) - p(1336)
p(1454) = p(50)*p(160) - p(1276)
p(1455) = p(14)*p(327)
p(1456) = p(46)*p(187) - p(1243)
p(1457) = p(2)*p(915) - p(1324) - p(1441)
p(1458) = p(22)*p(310) - p(1243)
p(1459) = p(14)*p(342)
p(1460) = p(3)*p(585) - p(1342)
p(1461) = p(2)*p(919) - p(1332) - p(1329)
p(1462) = p(1)*p(929) - p(1342)
p(1463) = p(6)*p(582) - p(1334)
p(1464) = p(1)*p(931) - p(1345) - p(1343)
p(1465) = p(32)*p(195) - p(1334)
p(1466) = p(45)*p(191) - p(1318)
p(1467) = p(2)*p(925) - p(1340) - p(1457)
p(1468) = p(81)*p(102) - p(1318)
p(1469) = p(14)*p(345)
p(1470) = p(2)*p(928) - p(1344) - p(1343)
p(1471) = p(2)*p(929) - p(1345)
p(1472) = p(6)*p(585) - p(1346)
p(1473) = p(2)*p(931) - p(1347) - p(1467)
p(1474) = p(13)*p(347) - p(1346)
p(1475) = p(14)*p(347)
p(1476) = p(3)*p(566) - p(1351)
p(1477) = p(7)*p(566) - p(1349)
p(1478) = p(49)*p(178) - p(1348)
p(1479) = p(39)*p(194) - p(1352)
p(1480) = p(3)*p(588) - p(1355)
p(1481) = p(1)*p(938) - p(1356)
p(1482) = p(2)*p(958) - p(1481)
p(1483) = p(1)*p(940) - p(1370)
p(1484) = p(8)*p(588) - p(1483)
p(1485) = p(1)*p(942) - p(1386)
p(1486) = p(22)*p(346) - p(1485)
p(1487) = p(1)*p(944) - p(1403)
p(1488) = p(50)*p(194) - p(1487)
p(1489) = p(1)*p(946) - p(1424)
p(1490) = p(101)*p(102) - p(1489)
p(1491) = p(6)*p(578) - p(1443)
p(1492) = p(1)*p(949) - p(1444)
p(1493) = p(49)*p(195) - p(1492)
p(1494) = p(8)*p(578) - p(1423)
p(1495) = p(1)*p(952) - p(1460)
p(1496) = p(21)*p(347) - p(1495)
p(1497) = p(50)*p(187) - p(1420)
p(1498) = p(1)*p(955) - p(1470)
p(1499) = p(7)*p(589) - p(1498)
p(1500) = p(45)*p(195) - p(1455)
p(1501) = p(2)*p(955) - p(1472)
p(1502) = p(1)*p(959) - p(1501)
p(1503) = p(6)*p(589) - p(1475)
p(1504) = p(1)*p(958) - p(1480)
p(1505) = p(2)*p(959) - p(1503)


do i=1,6
dm(0) = 0.0D0
dm(1) = dmsdr(i,4)
dm(2) = dmsdr(i,2)
dm(3) = dmsdr(i,1)
dm(4) = dmsdr(i,6)
dm(5) = dmsdr(i,5)
dm(6) = dmsdr(i,3)
dm(7) = dm(1)*rm(2) + rm(1)*dm(2)
dm(8) = dm(1)*rm(3) + rm(1)*dm(3)
dm(9) = dm(2)*rm(3) + rm(2)*dm(3)
dm(10) = dm(3)*rm(4) + rm(3)*dm(4)
dm(11) = dm(2)*rm(5) + rm(2)*dm(5)
dm(12) = dm(1)*rm(6) + rm(1)*dm(6)
dm(13) = dm(4)*rm(5) + rm(4)*dm(5)
dm(14) = dm(4)*rm(6) + rm(4)*dm(6)
dm(15) = dm(5)*rm(6) + rm(5)*dm(6)
dm(16) = dm(1)*rm(9) + rm(1)*dm(9)
dm(17) = dm(1)*rm(10) + rm(1)*dm(10)
dm(18) = dm(2)*rm(10) + rm(2)*dm(10)
dm(19) = dm(1)*rm(11) + rm(1)*dm(11)
dm(20) = dm(3)*rm(11) + rm(3)*dm(11)
dm(21) = dm(2)*rm(12) + rm(2)*dm(12)
dm(22) = dm(3)*rm(12) + rm(3)*dm(12)
dm(23) = dm(2)*rm(13) + rm(2)*dm(13)
dm(24) = dm(3)*rm(13) + rm(3)*dm(13)
dm(25) = dm(1)*rm(14) + rm(1)*dm(14)
dm(26) = dm(3)*rm(14) + rm(3)*dm(14)
dm(27) = dm(1)*rm(15) + rm(1)*dm(15)
dm(28) = dm(2)*rm(15) + rm(2)*dm(15)
dm(29) = dm(4)*rm(15) + rm(4)*dm(15)
dm(30) = dm(2)*rm(24) + rm(2)*dm(24)
dm(31) = dm(1)*rm(26) + rm(1)*dm(26)
dm(32) = dm(1)*rm(28) + rm(1)*dm(28)

dp(0) = dm(0)
dp(1) = dm(1) + dm(2) + dm(3)
dp(2) = dm(4) + dm(5) + dm(6)
dp(3) = dm(7) + dm(8) + dm(9)
dp(4) = dm(10) + dm(11) + dm(12)
dp(5) = dp(1)*p(2) + p(1)*dp(2) - dp(4)
dp(6) = dm(13) + dm(14) + dm(15)
dp(7) = dp(1)*p(1) + p(1)*dp(1) - dp(3) - dp(3)
dp(8) = dp(2)*p(2) + p(2)*dp(2) - dp(6) - dp(6)
dp(9) = dm(16)
dp(10) = dm(17) + dm(18) + dm(19) + dm(20) + dm(21) + dm(22)
dp(11) = dp(2)*p(3) + p(2)*dp(3) - dp(10)
dp(12) = dm(23) + dm(24) + dm(25) + dm(26) + dm(27) + dm(28)
dp(13) = dp(1)*p(6) + p(1)*dp(6) - dp(12)
dp(14) = dm(29)
dp(15) = dp(1)*p(3) + p(1)*dp(3) - dp(9) - dp(9) - dp(9)
dp(16) = dp(1)*p(4) + p(1)*dp(4) - dp(10)
dp(17) = dp(2)*p(7) + p(2)*dp(7) - dp(16)
dp(18) = dp(2)*p(4) + p(2)*dp(4) - dp(12)
dp(19) = dp(1)*p(8) + p(1)*dp(8) - dp(18)
dp(20) = dp(2)*p(6) + p(2)*dp(6) - dp(14) - dp(14) - dp(14)
dp(21) = dp(1)*p(7) + p(1)*dp(7) - dp(15)
dp(22) = dp(2)*p(8) + p(2)*dp(8) - dp(20)
dp(23) = dp(9)*p(2) + p(9)*dp(2)
dp(24) = dm(30) + dm(31) + dm(32)
dp(25) = dp(3)*p(6) + p(3)*dp(6) - dp(24)
dp(26) = dp(14)*p(1) + p(14)*dp(1)
dp(27) = dp(9)*p(1) + p(9)*dp(1)
dp(28) = dp(3)*p(4) + p(3)*dp(4) - dp(23)
dp(29) = dp(1)*p(10) + p(1)*dp(10) - dp(23) - dp(28) - dp(23)
dp(30) = dp(1)*p(11) + p(1)*dp(11) - dp(23)
dp(31) = dp(1)*p(12) + p(1)*dp(12) - dp(25) - dp(24) - dp(24)
dp(32) = dp(1)*p(13) + p(1)*dp(13) - dp(25)
dp(33) = dp(4)*p(5) + p(4)*dp(5) - dp(25) - dp(31)
dp(34) = dp(2)*p(11) + p(2)*dp(11) - dp(25)
dp(35) = dp(4)*p(6) + p(4)*dp(6) - dp(26)
dp(36) = dp(2)*p(12) + p(2)*dp(12) - dp(26) - dp(35) - dp(26)
dp(37) = dp(2)*p(13) + p(2)*dp(13) - dp(26)
dp(38) = dp(14)*p(2) + p(14)*dp(2)
dp(39) = dp(3)*p(3) + p(3)*dp(3) - dp(27) - dp(27)
dp(40) = dp(3)*p(7) + p(3)*dp(7) - dp(27)
dp(41) = dp(1)*p(16) + p(1)*dp(16) - dp(28)
dp(42) = dp(2)*p(21) + p(2)*dp(21) - dp(41)
dp(43) = dp(1)*p(18) + p(1)*dp(18) - dp(33)
dp(44) = dp(7)*p(8) + p(7)*dp(8) - dp(43)
dp(45) = dp(6)*p(6) + p(6)*dp(6) - dp(38) - dp(38)
dp(46) = dp(2)*p(18) + p(2)*dp(18) - dp(35)
dp(47) = dp(1)*p(22) + p(1)*dp(22) - dp(46)
dp(48) = dp(6)*p(8) + p(6)*dp(8) - dp(38)
dp(49) = dp(1)*p(21) + p(1)*dp(21) - dp(40)
dp(50) = dp(2)*p(22) + p(2)*dp(22) - dp(48)
dp(51) = dp(9)*p(6) + p(9)*dp(6)
dp(52) = dp(14)*p(3) + p(14)*dp(3)
dp(53) = dp(9)*p(4) + p(9)*dp(4)
dp(54) = dp(9)*p(5) + p(9)*dp(5)
dp(55) = dp(1)*p(24) + p(1)*dp(24) - dp(51)
dp(56) = dp(3)*p(12) + p(3)*dp(12) - dp(51) - dp(55) - dp(51)
dp(57) = dp(3)*p(13) + p(3)*dp(13) - dp(51)
dp(58) = dp(14)*p(7) + p(14)*dp(7)
dp(59) = dp(9)*p(8) + p(9)*dp(8)
dp(60) = dp(2)*p(24) + p(2)*dp(24) - dp(52)
dp(61) = dp(4)*p(13) + p(4)*dp(13) - dp(58)
dp(62) = dp(6)*p(11) + p(6)*dp(11) - dp(52)
dp(63) = dp(14)*p(4) + p(14)*dp(4)
dp(64) = dp(14)*p(5) + p(14)*dp(5)
dp(65) = dp(9)*p(3) + p(9)*dp(3)
dp(66) = dp(9)*p(7) + p(9)*dp(7)
dp(67) = dp(3)*p(16) + p(3)*dp(16) - dp(53)
dp(68) = dp(4)*p(15) + p(4)*dp(15) - dp(54) - dp(67)
dp(69) = dp(2)*p(39) + p(2)*dp(39) - dp(68)
dp(70) = dp(1)*p(29) + p(1)*dp(29) - dp(54) - dp(68)
dp(71) = dp(7)*p(11) + p(7)*dp(11) - dp(53)
dp(72) = dp(1)*p(31) + p(1)*dp(31) - dp(56) - dp(55)
dp(73) = dp(1)*p(32) + p(1)*dp(32) - dp(57)
dp(74) = dp(3)*p(18) + p(3)*dp(18) - dp(59)
dp(75) = dp(2)*p(29) + p(2)*dp(29) - dp(57) - dp(55)
dp(76) = dp(1)*p(34) + p(1)*dp(34) - dp(59)
dp(77) = dp(6)*p(16) + p(6)*dp(16) - dp(58)
dp(78) = dp(1)*p(36) + p(1)*dp(36) - dp(62) - dp(60)
dp(79) = dp(2)*p(32) + p(2)*dp(32) - dp(58)
dp(80) = dp(6)*p(12) + p(6)*dp(12) - dp(64) - dp(63) - dp(63)
dp(81) = dp(1)*p(45) + p(1)*dp(45) - dp(80)
dp(82) = dp(14)*p(6) + p(14)*dp(6)
dp(83) = dp(2)*p(33) + p(2)*dp(33) - dp(61) - dp(60)
dp(84) = dp(2)*p(34) + p(2)*dp(34) - dp(62)
dp(85) = dp(6)*p(18) + p(6)*dp(18) - dp(63)
dp(86) = dp(2)*p(36) + p(2)*dp(36) - dp(64) - dp(80)
dp(87) = dp(8)*p(13) + p(8)*dp(13) - dp(63)
dp(88) = dp(14)*p(8) + p(14)*dp(8)
dp(89) = dp(1)*p(39) + p(1)*dp(39) - dp(65)
dp(90) = dp(3)*p(21) + p(3)*dp(21) - dp(66)
dp(91) = dp(1)*p(41) + p(1)*dp(41) - dp(67)
dp(92) = dp(2)*p(49) + p(2)*dp(49) - dp(91)
dp(93) = dp(1)*p(43) + p(1)*dp(43) - dp(74)
dp(94) = dp(8)*p(21) + p(8)*dp(21) - dp(93)
dp(95) = dp(1)*p(46) + p(1)*dp(46) - dp(83)
dp(96) = dp(7)*p(22) + p(7)*dp(22) - dp(95)
dp(97) = dp(2)*p(45) + p(2)*dp(45) - dp(82)
dp(98) = dp(2)*p(46) + p(2)*dp(46) - dp(85)
dp(99) = dp(1)*p(50) + p(1)*dp(50) - dp(98)
dp(100) = dp(6)*p(22) + p(6)*dp(22) - dp(88)
dp(101) = dp(1)*p(49) + p(1)*dp(49) - dp(90)
dp(102) = dp(2)*p(50) + p(2)*dp(50) - dp(100)
dp(103) = dp(9)*p(14) + p(9)*dp(14)
dp(104) = dp(9)*p(12) + p(9)*dp(12)
dp(105) = dp(9)*p(13) + p(9)*dp(13)
dp(106) = dp(14)*p(15) + p(14)*dp(15)
dp(107) = dp(9)*p(20) + p(9)*dp(20)
dp(108) = dp(14)*p(10) + p(14)*dp(10)
dp(109) = dp(14)*p(11) + p(14)*dp(11)
dp(110) = dp(9)*p(16) + p(9)*dp(16)
dp(111) = dp(9)*p(10) + p(9)*dp(10)
dp(112) = dp(9)*p(11) + p(9)*dp(11)
dp(113) = dp(9)*p(17) + p(9)*dp(17)
dp(114) = dp(3)*p(24) + p(3)*dp(24) - dp(104)
dp(115) = dp(7)*p(24) + p(7)*dp(24) - dp(105)
dp(116) = dp(3)*p(31) + p(3)*dp(31) - dp(104) - dp(115)
dp(117) = dp(6)*p(39) + p(6)*dp(39) - dp(114)
dp(118) = dp(3)*p(32) + p(3)*dp(32) - dp(105)
dp(119) = dp(14)*p(21) + p(14)*dp(21)
dp(120) = dp(9)*p(18) + p(9)*dp(18)
dp(121) = dp(9)*p(19) + p(9)*dp(19)
dp(122) = dp(4)*p(24) + p(4)*dp(24) - dp(103) - dp(103) - dp(103)
dp(123) = dp(1)*p(60) + p(1)*dp(60) - dp(107) - dp(122)
dp(124) = dp(13)*p(16) + p(13)*dp(16) - dp(119)
dp(125) = dp(2)*p(56) + p(2)*dp(56) - dp(106) - dp(124)
dp(126) = dp(4)*p(32) + p(4)*dp(32) - dp(119)
dp(127) = dp(1)*p(62) + p(1)*dp(62) - dp(107) - dp(125)
dp(128) = dp(14)*p(16) + p(14)*dp(16)
dp(129) = dp(14)*p(17) + p(14)*dp(17)
dp(130) = dp(6)*p(24) + p(6)*dp(24) - dp(108)
dp(131) = dp(3)*p(45) + p(3)*dp(45) - dp(130)
dp(132) = dp(14)*p(12) + p(14)*dp(12)
dp(133) = dp(14)*p(13) + p(14)*dp(13)
dp(134) = dp(9)*p(22) + p(9)*dp(22)
dp(135) = dp(8)*p(24) + p(8)*dp(24) - dp(109)
dp(136) = dp(13)*p(18) + p(13)*dp(18) - dp(128)
dp(137) = dp(6)*p(34) + p(6)*dp(34) - dp(109)
dp(138) = dp(14)*p(18) + p(14)*dp(18)
dp(139) = dp(14)*p(19) + p(14)*dp(19)
dp(140) = dp(9)*p(9) + p(9)*dp(9)
dp(141) = dp(9)*p(15) + p(9)*dp(15)
dp(142) = dp(9)*p(21) + p(9)*dp(21)
dp(143) = dp(3)*p(41) + p(3)*dp(41) - dp(110)
dp(144) = dp(4)*p(39) + p(4)*dp(39) - dp(112)
dp(145) = dp(1)*p(68) + p(1)*dp(68) - dp(111) - dp(144)
dp(146) = dp(1)*p(69) + p(1)*dp(69) - dp(112)
dp(147) = dp(1)*p(70) + p(1)*dp(70) - dp(113) - dp(145)
dp(148) = dp(11)*p(21) + p(11)*dp(21) - dp(110)
dp(149) = dp(1)*p(72) + p(1)*dp(72) - dp(116) - dp(115)
dp(150) = dp(1)*p(73) + p(1)*dp(73) - dp(118)
dp(151) = dp(3)*p(43) + p(3)*dp(43) - dp(120)
dp(152) = dp(15)*p(18) + p(15)*dp(18) - dp(121) - dp(151)
dp(153) = dp(2)*p(69) + p(2)*dp(69) - dp(117)
dp(154) = dp(1)*p(75) + p(1)*dp(75) - dp(121) - dp(152)
dp(155) = dp(7)*p(34) + p(7)*dp(34) - dp(120)
dp(156) = dp(6)*p(41) + p(6)*dp(41) - dp(119)
dp(157) = dp(1)*p(78) + p(1)*dp(78) - dp(125) - dp(123)
dp(158) = dp(2)*p(73) + p(2)*dp(73) - dp(119)
dp(159) = dp(1)*p(80) + p(1)*dp(80) - dp(131) - dp(130) - dp(130)
dp(160) = dp(1)*p(81) + p(1)*dp(81) - dp(131)
dp(161) = dp(14)*p(14) + p(14)*dp(14)
dp(162) = dp(3)*p(46) + p(3)*dp(46) - dp(134)
dp(163) = dp(2)*p(75) + p(2)*dp(75) - dp(126) - dp(123)
dp(164) = dp(1)*p(84) + p(1)*dp(84) - dp(134)
dp(165) = dp(6)*p(43) + p(6)*dp(43) - dp(128)
dp(166) = dp(1)*p(86) + p(1)*dp(86) - dp(137) - dp(135)
dp(167) = dp(8)*p(32) + p(8)*dp(32) - dp(128)
dp(168) = dp(4)*p(45) + p(4)*dp(45) - dp(133)
dp(169) = dp(2)*p(80) + p(2)*dp(80) - dp(132) - dp(168)
dp(170) = dp(2)*p(81) + p(2)*dp(81) - dp(133)
dp(171) = dp(14)*p(20) + p(14)*dp(20)
dp(172) = dp(2)*p(83) + p(2)*dp(83) - dp(136) - dp(135)
dp(173) = dp(2)*p(84) + p(2)*dp(84) - dp(137)
dp(174) = dp(6)*p(46) + p(6)*dp(46) - dp(138)
dp(175) = dp(2)*p(86) + p(2)*dp(86) - dp(139) - dp(169)
dp(176) = dp(13)*p(22) + p(13)*dp(22) - dp(138)
dp(177) = dp(14)*p(22) + p(14)*dp(22)
dp(178) = dp(3)*p(39) + p(3)*dp(39) - dp(141)
dp(179) = dp(1)*p(89) + p(1)*dp(89) - dp(141) - dp(178) - dp(178)
dp(180) = dp(3)*p(49) + p(3)*dp(49) - dp(142)
dp(181) = dp(1)*p(91) + p(1)*dp(91) - dp(143)
dp(182) = dp(2)*p(101) + p(2)*dp(101) - dp(181)
dp(183) = dp(1)*p(93) + p(1)*dp(93) - dp(151)
dp(184) = dp(8)*p(49) + p(8)*dp(49) - dp(183)
dp(185) = dp(1)*p(95) + p(1)*dp(95) - dp(162)
dp(186) = dp(21)*p(22) + p(21)*dp(22) - dp(185)
dp(187) = dp(6)*p(45) + p(6)*dp(45) - dp(171)
dp(188) = dp(1)*p(98) + p(1)*dp(98) - dp(172)
dp(189) = dp(7)*p(50) + p(7)*dp(50) - dp(188)
dp(190) = dp(2)*p(97) + p(2)*dp(97) - dp(171) - dp(187) - dp(187)
dp(191) = dp(2)*p(98) + p(2)*dp(98) - dp(174)
dp(192) = dp(1)*p(102) + p(1)*dp(102) - dp(191)
dp(193) = dp(6)*p(50) + p(6)*dp(50) - dp(177)
dp(194) = dp(1)*p(101) + p(1)*dp(101) - dp(180)
dp(195) = dp(2)*p(102) + p(2)*dp(102) - dp(193)
dp(196) = dp(9)*p(26) + p(9)*dp(26)
dp(197) = dp(9)*p(38) + p(9)*dp(38)
dp(198) = dp(9)*p(24) + p(9)*dp(24)
dp(199) = dp(9)*p(31) + p(9)*dp(31)
dp(200) = dp(9)*p(25) + p(9)*dp(25)
dp(201) = dp(9)*p(32) + p(9)*dp(32)
dp(202) = dp(14)*p(39) + p(14)*dp(39)
dp(203) = dp(14)*p(40) + p(14)*dp(40)
dp(204) = dp(9)*p(35) + p(9)*dp(35)
dp(205) = dp(9)*p(36) + p(9)*dp(36)
dp(206) = dp(9)*p(37) + p(9)*dp(37)
dp(207) = dp(14)*p(28) + p(14)*dp(28)
dp(208) = dp(14)*p(29) + p(14)*dp(29)
dp(209) = dp(14)*p(30) + p(14)*dp(30)
dp(210) = dp(9)*p(45) + p(9)*dp(45)
dp(211) = dp(14)*p(24) + p(14)*dp(24)
dp(212) = dp(14)*p(25) + p(14)*dp(25)
dp(213) = dp(9)*p(48) + p(9)*dp(48)
dp(214) = dp(14)*p(33) + p(14)*dp(33)
dp(215) = dp(14)*p(34) + p(14)*dp(34)
dp(216) = dp(9)*p(41) + p(9)*dp(41)
dp(217) = dp(9)*p(28) + p(9)*dp(28)
dp(218) = dp(9)*p(23) + p(9)*dp(23)
dp(219) = dp(9)*p(29) + p(9)*dp(29)
dp(220) = dp(9)*p(30) + p(9)*dp(30)
dp(221) = dp(9)*p(42) + p(9)*dp(42)
dp(222) = dp(1)*p(114) + p(1)*dp(114) - dp(198)
dp(223) = dp(21)*p(24) + p(21)*dp(24) - dp(201)
dp(224) = dp(3)*p(72) + p(3)*dp(72) - dp(199) - dp(223)
dp(225) = dp(1)*p(116) + p(1)*dp(116) - dp(199) - dp(224)
dp(226) = dp(13)*p(39) + p(13)*dp(39) - dp(198)
dp(227) = dp(3)*p(73) + p(3)*dp(73) - dp(201)
dp(228) = dp(14)*p(49) + p(14)*dp(49)
dp(229) = dp(9)*p(43) + p(9)*dp(43)
dp(230) = dp(9)*p(33) + p(9)*dp(33)
dp(231) = dp(9)*p(34) + p(9)*dp(34)
dp(232) = dp(9)*p(44) + p(9)*dp(44)
dp(233) = dp(16)*p(24) + p(16)*dp(24) - dp(196)
dp(234) = dp(2)*p(114) + p(2)*dp(114) - dp(202)
dp(235) = dp(1)*p(123) + p(1)*dp(123) - dp(205) - dp(234)
dp(236) = dp(13)*p(41) + p(13)*dp(41) - dp(228)
dp(237) = dp(2)*p(116) + p(2)*dp(116) - dp(203) - dp(236)
dp(238) = dp(16)*p(32) + p(16)*dp(32) - dp(228)
dp(239) = dp(6)*p(69) + p(6)*dp(69) - dp(202)
dp(240) = dp(4)*p(73) + p(4)*dp(73) - dp(228)
dp(241) = dp(11)*p(32) + p(11)*dp(32) - dp(196)
dp(242) = dp(14)*p(41) + p(14)*dp(41)
dp(243) = dp(14)*p(42) + p(14)*dp(42)
dp(244) = dp(1)*p(130) + p(1)*dp(130) - dp(210)
dp(245) = dp(4)*p(62) + p(4)*dp(62) - dp(213) - dp(209)
dp(246) = dp(3)*p(81) + p(3)*dp(81) - dp(210)
dp(247) = dp(14)*p(31) + p(14)*dp(31)
dp(248) = dp(14)*p(32) + p(14)*dp(32)
dp(249) = dp(14)*p(26) + p(14)*dp(26)
dp(250) = dp(9)*p(46) + p(9)*dp(46)
dp(251) = dp(9)*p(47) + p(9)*dp(47)
dp(252) = dp(18)*p(24) + p(18)*dp(24) - dp(197)
dp(253) = dp(1)*p(135) + p(1)*dp(135) - dp(213) - dp(252)
dp(254) = dp(13)*p(43) + p(13)*dp(43) - dp(242)
dp(255) = dp(2)*p(125) + p(2)*dp(125) - dp(209) - dp(245)
dp(256) = dp(18)*p(32) + p(18)*dp(32) - dp(242)
dp(257) = dp(13)*p(34) + p(13)*dp(34) - dp(197)
dp(258) = dp(14)*p(43) + p(14)*dp(43)
dp(259) = dp(14)*p(44) + p(14)*dp(44)
dp(260) = dp(2)*p(130) + p(2)*dp(130) - dp(211)
dp(261) = dp(4)*p(81) + p(4)*dp(81) - dp(248)
dp(262) = dp(11)*p(45) + p(11)*dp(45) - dp(211)
dp(263) = dp(14)*p(35) + p(14)*dp(35)
dp(264) = dp(14)*p(36) + p(14)*dp(36)
dp(265) = dp(14)*p(37) + p(14)*dp(37)
dp(266) = dp(9)*p(50) + p(9)*dp(50)
dp(267) = dp(22)*p(24) + p(22)*dp(24) - dp(215)
dp(268) = dp(13)*p(46) + p(13)*dp(46) - dp(258)
dp(269) = dp(6)*p(84) + p(6)*dp(84) - dp(215)
dp(270) = dp(14)*p(46) + p(14)*dp(46)
dp(271) = dp(14)*p(47) + p(14)*dp(47)
dp(272) = dp(9)*p(27) + p(9)*dp(27)
dp(273) = dp(9)*p(39) + p(9)*dp(39)
dp(274) = dp(9)*p(40) + p(9)*dp(40)
dp(275) = dp(9)*p(49) + p(9)*dp(49)
dp(276) = dp(3)*p(91) + p(3)*dp(91) - dp(216)
dp(277) = dp(16)*p(39) + p(16)*dp(39) - dp(218)
dp(278) = dp(1)*p(144) + p(1)*dp(144) - dp(217) - dp(277)
dp(279) = dp(2)*p(178) + p(2)*dp(178) - dp(278)
dp(280) = dp(1)*p(145) + p(1)*dp(145) - dp(219) - dp(278)
dp(281) = dp(7)*p(69) + p(7)*dp(69) - dp(218)
dp(282) = dp(1)*p(147) + p(1)*dp(147) - dp(221) - dp(280)
dp(283) = dp(11)*p(49) + p(11)*dp(49) - dp(216)
dp(284) = dp(1)*p(149) + p(1)*dp(149) - dp(224) - dp(223)
dp(285) = dp(1)*p(150) + p(1)*dp(150) - dp(227)
dp(286) = dp(3)*p(93) + p(3)*dp(93) - dp(229)
dp(287) = dp(18)*p(39) + p(18)*dp(39) - dp(231)
dp(288) = dp(1)*p(152) + p(1)*dp(152) - dp(230) - dp(287)
dp(289) = dp(1)*p(153) + p(1)*dp(153) - dp(231)
dp(290) = dp(1)*p(154) + p(1)*dp(154) - dp(232) - dp(288)
dp(291) = dp(21)*p(34) + p(21)*dp(34) - dp(229)
dp(292) = dp(6)*p(91) + p(6)*dp(91) - dp(228)
dp(293) = dp(1)*p(157) + p(1)*dp(157) - dp(237) - dp(235)
dp(294) = dp(2)*p(150) + p(2)*dp(150) - dp(228)
dp(295) = dp(1)*p(159) + p(1)*dp(159) - dp(245) - dp(244)
dp(296) = dp(1)*p(160) + p(1)*dp(160) - dp(246)
dp(297) = dp(3)*p(95) + p(3)*dp(95) - dp(250)
dp(298) = dp(2)*p(152) + p(2)*dp(152) - dp(238) - dp(234)
dp(299) = dp(2)*p(153) + p(2)*dp(153) - dp(239)
dp(300) = dp(1)*p(163) + p(1)*dp(163) - dp(251) - dp(298)
dp(301) = dp(7)*p(84) + p(7)*dp(84) - dp(250)
dp(302) = dp(6)*p(93) + p(6)*dp(93) - dp(242)
dp(303) = dp(1)*p(166) + p(1)*dp(166) - dp(255) - dp(253)
dp(304) = dp(8)*p(73) + p(8)*dp(73) - dp(242)
dp(305) = dp(16)*p(45) + p(16)*dp(45) - dp(248)
dp(306) = dp(1)*p(169) + p(1)*dp(169) - dp(262) - dp(260)
dp(307) = dp(2)*p(160) + p(2)*dp(160) - dp(248)
dp(308) = dp(14)*p(38) + p(14)*dp(38)
dp(309) = dp(6)*p(80) + p(6)*dp(80) - dp(264) - dp(263)
dp(310) = dp(1)*p(187) + p(1)*dp(187) - dp(309)
dp(311) = dp(14)*p(45) + p(14)*dp(45)
dp(312) = dp(3)*p(98) + p(3)*dp(98) - dp(266)
dp(313) = dp(2)*p(163) + p(2)*dp(163) - dp(256) - dp(253)
dp(314) = dp(1)*p(173) + p(1)*dp(173) - dp(266)
dp(315) = dp(6)*p(95) + p(6)*dp(95) - dp(258)
dp(316) = dp(1)*p(175) + p(1)*dp(175) - dp(269) - dp(267)
dp(317) = dp(22)*p(32) + p(22)*dp(32) - dp(258)
dp(318) = dp(18)*p(45) + p(18)*dp(45) - dp(249)
dp(319) = dp(2)*p(169) + p(2)*dp(169) - dp(264) - dp(309)
dp(320) = dp(8)*p(81) + p(8)*dp(81) - dp(249)
dp(321) = dp(14)*p(48) + p(14)*dp(48)
dp(322) = dp(2)*p(172) + p(2)*dp(172) - dp(268) - dp(267)
dp(323) = dp(2)*p(173) + p(2)*dp(173) - dp(269)
dp(324) = dp(6)*p(98) + p(6)*dp(98) - dp(270)
dp(325) = dp(2)*p(175) + p(2)*dp(175) - dp(271) - dp(319)
dp(326) = dp(13)*p(50) + p(13)*dp(50) - dp(270)
dp(327) = dp(14)*p(50) + p(14)*dp(50)
dp(328) = dp(1)*p(178) + p(1)*dp(178) - dp(273)
dp(329) = dp(21)*p(39) + p(21)*dp(39) - dp(272)
dp(330) = dp(3)*p(101) + p(3)*dp(101) - dp(275)
dp(331) = dp(1)*p(181) + p(1)*dp(181) - dp(276)
dp(332) = dp(2)*p(194) + p(2)*dp(194) - dp(331)
dp(333) = dp(1)*p(183) + p(1)*dp(183) - dp(286)
dp(334) = dp(8)*p(101) + p(8)*dp(101) - dp(333)
dp(335) = dp(1)*p(185) + p(1)*dp(185) - dp(297)
dp(336) = dp(22)*p(49) + p(22)*dp(49) - dp(335)
dp(337) = dp(1)*p(188) + p(1)*dp(188) - dp(312)
dp(338) = dp(21)*p(50) + p(21)*dp(50) - dp(337)
dp(339) = dp(2)*p(187) + p(2)*dp(187) - dp(311)
dp(340) = dp(1)*p(191) + p(1)*dp(191) - dp(322)
dp(341) = dp(7)*p(102) + p(7)*dp(102) - dp(340)
dp(342) = dp(22)*p(45) + p(22)*dp(45) - dp(308)
dp(343) = dp(2)*p(191) + p(2)*dp(191) - dp(324)
dp(344) = dp(1)*p(195) + p(1)*dp(195) - dp(343)
dp(345) = dp(6)*p(102) + p(6)*dp(102) - dp(327)
dp(346) = dp(1)*p(194) + p(1)*dp(194) - dp(330)
dp(347) = dp(2)*p(195) + p(2)*dp(195) - dp(345)
dp(348) = dp(9)*p(52) + p(9)*dp(52)
dp(349) = dp(9)*p(58) + p(9)*dp(58)
dp(350) = dp(9)*p(63) + p(9)*dp(63)
dp(351) = dp(9)*p(64) + p(9)*dp(64)
dp(352) = dp(9)*p(82) + p(9)*dp(82)
dp(353) = dp(9)*p(88) + p(9)*dp(88)
dp(354) = dp(9)*p(55) + p(9)*dp(55)
dp(355) = dp(9)*p(72) + p(9)*dp(72)
dp(356) = dp(9)*p(51) + p(9)*dp(51)
dp(357) = dp(9)*p(56) + p(9)*dp(56)
dp(358) = dp(9)*p(57) + p(9)*dp(57)
dp(359) = dp(9)*p(73) + p(9)*dp(73)
dp(360) = dp(14)*p(89) + p(14)*dp(89)
dp(361) = dp(14)*p(90) + p(14)*dp(90)
dp(362) = dp(9)*p(77) + p(9)*dp(77)
dp(363) = dp(9)*p(60) + p(9)*dp(60)
dp(364) = dp(9)*p(78) + p(9)*dp(78)
dp(365) = dp(9)*p(61) + p(9)*dp(61)
dp(366) = dp(9)*p(62) + p(9)*dp(62)
dp(367) = dp(9)*p(79) + p(9)*dp(79)
dp(368) = dp(14)*p(67) + p(14)*dp(67)
dp(369) = dp(14)*p(68) + p(14)*dp(68)
dp(370) = dp(14)*p(69) + p(14)*dp(69)
dp(371) = dp(14)*p(70) + p(14)*dp(70)
dp(372) = dp(14)*p(71) + p(14)*dp(71)
dp(373) = dp(9)*p(80) + p(9)*dp(80)
dp(374) = dp(9)*p(81) + p(9)*dp(81)
dp(375) = dp(14)*p(55) + p(14)*dp(55)
dp(376) = dp(14)*p(56) + p(14)*dp(56)
dp(377) = dp(14)*p(57) + p(14)*dp(57)
dp(378) = dp(14)*p(52) + p(14)*dp(52)
dp(379) = dp(9)*p(85) + p(9)*dp(85)
dp(380) = dp(9)*p(86) + p(9)*dp(86)
dp(381) = dp(9)*p(87) + p(9)*dp(87)
dp(382) = dp(14)*p(74) + p(14)*dp(74)
dp(383) = dp(14)*p(75) + p(14)*dp(75)
dp(384) = dp(14)*p(76) + p(14)*dp(76)
dp(385) = dp(9)*p(97) + p(9)*dp(97)
dp(386) = dp(14)*p(60) + p(14)*dp(60)
dp(387) = dp(14)*p(61) + p(14)*dp(61)
dp(388) = dp(14)*p(62) + p(14)*dp(62)
dp(389) = dp(9)*p(100) + p(9)*dp(100)
dp(390) = dp(14)*p(83) + p(14)*dp(83)
dp(391) = dp(14)*p(84) + p(14)*dp(84)
dp(392) = dp(9)*p(91) + p(9)*dp(91)
dp(393) = dp(9)*p(67) + p(9)*dp(67)
dp(394) = dp(9)*p(53) + p(9)*dp(53)
dp(395) = dp(9)*p(68) + p(9)*dp(68)
dp(396) = dp(9)*p(54) + p(9)*dp(54)
dp(397) = dp(9)*p(69) + p(9)*dp(69)
dp(398) = dp(9)*p(70) + p(9)*dp(70)
dp(399) = dp(9)*p(71) + p(9)*dp(71)
dp(400) = dp(9)*p(92) + p(9)*dp(92)
dp(401) = dp(3)*p(114) + p(3)*dp(114) - dp(354)
dp(402) = dp(7)*p(114) + p(7)*dp(114) - dp(356)
dp(403) = dp(24)*p(49) + p(24)*dp(49) - dp(359)
dp(404) = dp(3)*p(149) + p(3)*dp(149) - dp(355) - dp(403)
dp(405) = dp(1)*p(224) + p(1)*dp(224) - dp(355) - dp(404)
dp(406) = dp(6)*p(178) + p(6)*dp(178) - dp(401)
dp(407) = dp(32)*p(39) + p(32)*dp(39) - dp(356)
dp(408) = dp(3)*p(150) + p(3)*dp(150) - dp(359)
dp(409) = dp(14)*p(101) + p(14)*dp(101)
dp(410) = dp(9)*p(93) + p(9)*dp(93)
dp(411) = dp(9)*p(74) + p(9)*dp(74)
dp(412) = dp(9)*p(59) + p(9)*dp(59)
dp(413) = dp(9)*p(75) + p(9)*dp(75)
dp(414) = dp(9)*p(76) + p(9)*dp(76)
dp(415) = dp(9)*p(94) + p(9)*dp(94)
dp(416) = dp(24)*p(41) + p(24)*dp(41) - dp(349)
dp(417) = dp(4)*p(114) + p(4)*dp(114) - dp(348)
dp(418) = dp(1)*p(234) + p(1)*dp(234) - dp(363) - dp(417)
dp(419) = dp(1)*p(235) + p(1)*dp(235) - dp(364) - dp(418)
dp(420) = dp(13)*p(91) + p(13)*dp(91) - dp(409)
dp(421) = dp(2)*p(224) + p(2)*dp(224) - dp(361) - dp(420)
dp(422) = dp(32)*p(41) + p(32)*dp(41) - dp(409)
dp(423) = dp(1)*p(237) + p(1)*dp(237) - dp(364) - dp(421)
dp(424) = dp(16)*p(73) + p(16)*dp(73) - dp(409)
dp(425) = dp(13)*p(69) + p(13)*dp(69) - dp(348)
dp(426) = dp(4)*p(150) + p(4)*dp(150) - dp(409)
dp(427) = dp(11)*p(73) + p(11)*dp(73) - dp(349)
dp(428) = dp(14)*p(91) + p(14)*dp(91)
dp(429) = dp(14)*p(92) + p(14)*dp(92)
dp(430) = dp(3)*p(130) + p(3)*dp(130) - dp(373)
dp(431) = dp(7)*p(130) + p(7)*dp(130) - dp(374)
dp(432) = dp(3)*p(159) + p(3)*dp(159) - dp(373) - dp(431)
dp(433) = dp(39)*p(45) + p(39)*dp(45) - dp(430)
dp(434) = dp(3)*p(160) + p(3)*dp(160) - dp(374)
dp(435) = dp(14)*p(72) + p(14)*dp(72)
dp(436) = dp(14)*p(73) + p(14)*dp(73)
dp(437) = dp(14)*p(58) + p(14)*dp(58)
dp(438) = dp(9)*p(95) + p(9)*dp(95)
dp(439) = dp(9)*p(83) + p(9)*dp(83)
dp(440) = dp(9)*p(84) + p(9)*dp(84)
dp(441) = dp(9)*p(96) + p(9)*dp(96)
dp(442) = dp(24)*p(43) + p(24)*dp(43) - dp(350)
dp(443) = dp(8)*p(114) + p(8)*dp(114) - dp(370)
dp(444) = dp(1)*p(253) + p(1)*dp(253) - dp(380) - dp(443)
dp(445) = dp(13)*p(93) + p(13)*dp(93) - dp(428)
dp(446) = dp(2)*p(237) + p(2)*dp(237) - dp(372) - dp(432)
dp(447) = dp(32)*p(43) + p(32)*dp(43) - dp(428)
dp(448) = dp(6)*p(153) + p(6)*dp(153) - dp(370)
dp(449) = dp(18)*p(73) + p(18)*dp(73) - dp(428)
dp(450) = dp(32)*p(34) + p(32)*dp(34) - dp(350)
dp(451) = dp(14)*p(93) + p(14)*dp(93)
dp(452) = dp(14)*p(94) + p(14)*dp(94)
dp(453) = dp(4)*p(130) + p(4)*dp(130) - dp(352)
dp(454) = dp(1)*p(260) + p(1)*dp(260) - dp(385) - dp(453)
dp(455) = dp(16)*p(81) + p(16)*dp(81) - dp(436)
dp(456) = dp(2)*p(245) + p(2)*dp(245) - dp(376) - dp(455)
dp(457) = dp(4)*p(160) + p(4)*dp(160) - dp(436)
dp(458) = dp(11)*p(81) + p(11)*dp(81) - dp(352)
dp(459) = dp(14)*p(77) + p(14)*dp(77)
dp(460) = dp(14)*p(78) + p(14)*dp(78)
dp(461) = dp(14)*p(79) + p(14)*dp(79)
dp(462) = dp(14)*p(63) + p(14)*dp(63)
dp(463) = dp(14)*p(64) + p(14)*dp(64)
dp(464) = dp(6)*p(130) + p(6)*dp(130) - dp(386)
dp(465) = dp(3)*p(187) + p(3)*dp(187) - dp(464)
dp(466) = dp(14)*p(80) + p(14)*dp(80)
dp(467) = dp(14)*p(81) + p(14)*dp(81)
dp(468) = dp(9)*p(98) + p(9)*dp(98)
dp(469) = dp(9)*p(99) + p(9)*dp(99)
dp(470) = dp(24)*p(46) + p(24)*dp(46) - dp(353)
dp(471) = dp(1)*p(267) + p(1)*dp(267) - dp(389) - dp(470)
dp(472) = dp(13)*p(95) + p(13)*dp(95) - dp(451)
dp(473) = dp(2)*p(255) + p(2)*dp(255) - dp(384) - dp(456)
dp(474) = dp(32)*p(46) + p(32)*dp(46) - dp(451)
dp(475) = dp(13)*p(84) + p(13)*dp(84) - dp(353)
dp(476) = dp(14)*p(95) + p(14)*dp(95)
dp(477) = dp(14)*p(96) + p(14)*dp(96)
dp(478) = dp(8)*p(130) + p(8)*dp(130) - dp(378)
dp(479) = dp(18)*p(81) + p(18)*dp(81) - dp(437)
dp(480) = dp(34)*p(45) + p(34)*dp(45) - dp(378)
dp(481) = dp(14)*p(85) + p(14)*dp(85)
dp(482) = dp(14)*p(86) + p(14)*dp(86)
dp(483) = dp(14)*p(87) + p(14)*dp(87)
dp(484) = dp(9)*p(102) + p(9)*dp(102)
dp(485) = dp(24)*p(50) + p(24)*dp(50) - dp(391)
dp(486) = dp(13)*p(98) + p(13)*dp(98) - dp(476)
dp(487) = dp(6)*p(173) + p(6)*dp(173) - dp(391)
dp(488) = dp(14)*p(98) + p(14)*dp(98)
dp(489) = dp(14)*p(99) + p(14)*dp(99)
dp(490) = dp(9)*p(65) + p(9)*dp(65)
dp(491) = dp(9)*p(66) + p(9)*dp(66)
dp(492) = dp(9)*p(89) + p(9)*dp(89)
dp(493) = dp(9)*p(90) + p(9)*dp(90)
dp(494) = dp(9)*p(101) + p(9)*dp(101)
dp(495) = dp(3)*p(181) + p(3)*dp(181) - dp(392)
dp(496) = dp(39)*p(41) + p(39)*dp(41) - dp(394)
dp(497) = dp(4)*p(178) + p(4)*dp(178) - dp(397)
dp(498) = dp(1)*p(278) + p(1)*dp(278) - dp(395) - dp(497)
dp(499) = dp(1)*p(279) + p(1)*dp(279) - dp(397)
dp(500) = dp(1)*p(280) + p(1)*dp(280) - dp(398) - dp(498)
dp(501) = dp(21)*p(69) + p(21)*dp(69) - dp(394)
dp(502) = dp(1)*p(282) + p(1)*dp(282) - dp(400) - dp(500)
dp(503) = dp(11)*p(101) + p(11)*dp(101) - dp(392)
dp(504) = dp(1)*p(284) + p(1)*dp(284) - dp(404) - dp(403)
dp(505) = dp(1)*p(285) + p(1)*dp(285) - dp(408)
dp(506) = dp(3)*p(183) + p(3)*dp(183) - dp(410)
dp(507) = dp(39)*p(43) + p(39)*dp(43) - dp(412)
dp(508) = dp(1)*p(287) + p(1)*dp(287) - dp(411) - dp(507)
dp(509) = dp(2)*p(279) + p(2)*dp(279) - dp(406)
dp(510) = dp(1)*p(288) + p(1)*dp(288) - dp(413) - dp(508)
dp(511) = dp(7)*p(153) + p(7)*dp(153) - dp(412)
dp(512) = dp(1)*p(290) + p(1)*dp(290) - dp(415) - dp(510)
dp(513) = dp(34)*p(49) + p(34)*dp(49) - dp(410)
dp(514) = dp(6)*p(181) + p(6)*dp(181) - dp(409)
dp(515) = dp(1)*p(293) + p(1)*dp(293) - dp(421) - dp(419)
dp(516) = dp(2)*p(285) + p(2)*dp(285) - dp(409)
dp(517) = dp(1)*p(295) + p(1)*dp(295) - dp(432) - dp(431)
dp(518) = dp(1)*p(296) + p(1)*dp(296) - dp(434)
dp(519) = dp(3)*p(185) + p(3)*dp(185) - dp(438)
dp(520) = dp(39)*p(46) + p(39)*dp(46) - dp(440)
dp(521) = dp(1)*p(298) + p(1)*dp(298) - dp(439) - dp(520)
dp(522) = dp(1)*p(299) + p(1)*dp(299) - dp(440)
dp(523) = dp(1)*p(300) + p(1)*dp(300) - dp(441) - dp(521)
dp(524) = dp(21)*p(84) + p(21)*dp(84) - dp(438)
dp(525) = dp(6)*p(183) + p(6)*dp(183) - dp(428)
dp(526) = dp(1)*p(303) + p(1)*dp(303) - dp(446) - dp(444)
dp(527) = dp(8)*p(150) + p(8)*dp(150) - dp(428)
dp(528) = dp(41)*p(45) + p(41)*dp(45) - dp(436)
dp(529) = dp(1)*p(306) + p(1)*dp(306) - dp(456) - dp(454)
dp(530) = dp(2)*p(296) + p(2)*dp(296) - dp(436)
dp(531) = dp(4)*p(169) + p(4)*dp(169) - dp(388) - dp(478)
dp(532) = dp(1)*p(310) + p(1)*dp(310) - dp(465)
dp(533) = dp(14)*p(82) + p(14)*dp(82)
dp(534) = dp(3)*p(188) + p(3)*dp(188) - dp(468)
dp(535) = dp(2)*p(298) + p(2)*dp(298) - dp(447) - dp(443)
dp(536) = dp(2)*p(299) + p(2)*dp(299) - dp(448)
dp(537) = dp(1)*p(313) + p(1)*dp(313) - dp(469) - dp(535)
dp(538) = dp(7)*p(173) + p(7)*dp(173) - dp(468)
dp(539) = dp(6)*p(185) + p(6)*dp(185) - dp(451)
dp(540) = dp(1)*p(316) + p(1)*dp(316) - dp(473) - dp(471)
dp(541) = dp(22)*p(73) + p(22)*dp(73) - dp(451)
dp(542) = dp(43)*p(45) + p(43)*dp(45) - dp(437)
dp(543) = dp(1)*p(319) + p(1)*dp(319) - dp(480) - dp(478)
dp(544) = dp(8)*p(160) + p(8)*dp(160) - dp(437)
dp(545) = dp(14)*p(88) + p(14)*dp(88)
dp(546) = dp(4)*p(187) + p(4)*dp(187) - dp(467)
dp(547) = dp(2)*p(309) + p(2)*dp(309) - dp(466) - dp(546)
dp(548) = dp(2)*p(310) + p(2)*dp(310) - dp(467)
dp(549) = dp(14)*p(97) + p(14)*dp(97)
dp(550) = dp(3)*p(191) + p(3)*dp(191) - dp(484)
dp(551) = dp(2)*p(313) + p(2)*dp(313) - dp(474) - dp(471)
dp(552) = dp(1)*p(323) + p(1)*dp(323) - dp(484)
dp(553) = dp(6)*p(188) + p(6)*dp(188) - dp(476)
dp(554) = dp(1)*p(325) + p(1)*dp(325) - dp(487) - dp(485)
dp(555) = dp(32)*p(50) + p(32)*dp(50) - dp(476)
dp(556) = dp(45)*p(46) + p(45)*dp(46) - dp(462)
dp(557) = dp(2)*p(319) + p(2)*dp(319) - dp(482) - dp(547)
dp(558) = dp(22)*p(81) + p(22)*dp(81) - dp(462)
dp(559) = dp(14)*p(100) + p(14)*dp(100)
dp(560) = dp(2)*p(322) + p(2)*dp(322) - dp(486) - dp(485)
dp(561) = dp(2)*p(323) + p(2)*dp(323) - dp(487)
dp(562) = dp(6)*p(191) + p(6)*dp(191) - dp(488)
dp(563) = dp(2)*p(325) + p(2)*dp(325) - dp(489) - dp(557)
dp(564) = dp(13)*p(102) + p(13)*dp(102) - dp(488)
dp(565) = dp(14)*p(102) + p(14)*dp(102)
dp(566) = dp(3)*p(178) + p(3)*dp(178) - dp(492)
dp(567) = dp(7)*p(178) + p(7)*dp(178) - dp(490)
dp(568) = dp(39)*p(49) + p(39)*dp(49) - dp(491)
dp(569) = dp(3)*p(194) + p(3)*dp(194) - dp(494)
dp(570) = dp(1)*p(331) + p(1)*dp(331) - dp(495)
dp(571) = dp(2)*p(346) + p(2)*dp(346) - dp(570)
dp(572) = dp(1)*p(333) + p(1)*dp(333) - dp(506)
dp(573) = dp(8)*p(194) + p(8)*dp(194) - dp(572)
dp(574) = dp(1)*p(335) + p(1)*dp(335) - dp(519)
dp(575) = dp(22)*p(101) + p(22)*dp(101) - dp(574)
dp(576) = dp(1)*p(337) + p(1)*dp(337) - dp(534)
dp(577) = dp(49)*p(50) + p(49)*dp(50) - dp(576)
dp(578) = dp(6)*p(187) + p(6)*dp(187) - dp(549)
dp(579) = dp(1)*p(340) + p(1)*dp(340) - dp(550)
dp(580) = dp(21)*p(102) + p(21)*dp(102) - dp(579)
dp(581) = dp(8)*p(187) + p(8)*dp(187) - dp(533)
dp(582) = dp(1)*p(343) + p(1)*dp(343) - dp(560)
dp(583) = dp(7)*p(195) + p(7)*dp(195) - dp(582)
dp(584) = dp(45)*p(50) + p(45)*dp(50) - dp(545)
dp(585) = dp(2)*p(343) + p(2)*dp(343) - dp(562)
dp(586) = dp(1)*p(347) + p(1)*dp(347) - dp(585)
dp(587) = dp(6)*p(195) + p(6)*dp(195) - dp(565)
dp(588) = dp(1)*p(346) + p(1)*dp(346) - dp(569)
dp(589) = dp(2)*p(347) + p(2)*dp(347) - dp(587)
dp(590) = dp(9)*p(103) + p(9)*dp(103)
dp(591) = dp(9)*p(106) + p(9)*dp(106)
dp(592) = dp(9)*p(119) + p(9)*dp(119)
dp(593) = dp(9)*p(128) + p(9)*dp(128)
dp(594) = dp(9)*p(108) + p(9)*dp(108)
dp(595) = dp(9)*p(109) + p(9)*dp(109)
dp(596) = dp(9)*p(129) + p(9)*dp(129)
dp(597) = dp(9)*p(132) + p(9)*dp(132)
dp(598) = dp(9)*p(133) + p(9)*dp(133)
dp(599) = dp(9)*p(161) + p(9)*dp(161)
dp(600) = dp(9)*p(138) + p(9)*dp(138)
dp(601) = dp(9)*p(139) + p(9)*dp(139)
dp(602) = dp(9)*p(171) + p(9)*dp(171)
dp(603) = dp(9)*p(177) + p(9)*dp(177)
dp(604) = dp(9)*p(114) + p(9)*dp(114)
dp(605) = dp(9)*p(115) + p(9)*dp(115)
dp(606) = dp(9)*p(149) + p(9)*dp(149)
dp(607) = dp(9)*p(104) + p(9)*dp(104)
dp(608) = dp(9)*p(116) + p(9)*dp(116)
dp(609) = dp(9)*p(105) + p(9)*dp(105)
dp(610) = dp(9)*p(117) + p(9)*dp(117)
dp(611) = dp(9)*p(118) + p(9)*dp(118)
dp(612) = dp(9)*p(150) + p(9)*dp(150)
dp(613) = dp(14)*p(178) + p(14)*dp(178)
dp(614) = dp(14)*p(179) + p(14)*dp(179)
dp(615) = dp(14)*p(180) + p(14)*dp(180)
dp(616) = dp(9)*p(156) + p(9)*dp(156)
dp(617) = dp(9)*p(122) + p(9)*dp(122)
dp(618) = dp(9)*p(123) + p(9)*dp(123)
dp(619) = dp(9)*p(157) + p(9)*dp(157)
dp(620) = dp(9)*p(124) + p(9)*dp(124)
dp(621) = dp(9)*p(107) + p(9)*dp(107)
dp(622) = dp(9)*p(125) + p(9)*dp(125)
dp(623) = dp(9)*p(126) + p(9)*dp(126)
dp(624) = dp(9)*p(127) + p(9)*dp(127)
dp(625) = dp(9)*p(158) + p(9)*dp(158)
dp(626) = dp(14)*p(143) + p(14)*dp(143)
dp(627) = dp(14)*p(144) + p(14)*dp(144)
dp(628) = dp(14)*p(145) + p(14)*dp(145)
dp(629) = dp(14)*p(146) + p(14)*dp(146)
dp(630) = dp(14)*p(147) + p(14)*dp(147)
dp(631) = dp(14)*p(148) + p(14)*dp(148)
dp(632) = dp(9)*p(130) + p(9)*dp(130)
dp(633) = dp(9)*p(159) + p(9)*dp(159)
dp(634) = dp(9)*p(131) + p(9)*dp(131)
dp(635) = dp(9)*p(160) + p(9)*dp(160)
dp(636) = dp(14)*p(114) + p(14)*dp(114)
dp(637) = dp(14)*p(115) + p(14)*dp(115)
dp(638) = dp(14)*p(116) + p(14)*dp(116)
dp(639) = dp(14)*p(117) + p(14)*dp(117)
dp(640) = dp(14)*p(118) + p(14)*dp(118)
dp(641) = dp(14)*p(106) + p(14)*dp(106)
dp(642) = dp(9)*p(165) + p(9)*dp(165)
dp(643) = dp(9)*p(135) + p(9)*dp(135)
dp(644) = dp(9)*p(166) + p(9)*dp(166)
dp(645) = dp(9)*p(136) + p(9)*dp(136)
dp(646) = dp(9)*p(137) + p(9)*dp(137)
dp(647) = dp(9)*p(167) + p(9)*dp(167)
dp(648) = dp(14)*p(151) + p(14)*dp(151)
dp(649) = dp(14)*p(152) + p(14)*dp(152)
dp(650) = dp(14)*p(153) + p(14)*dp(153)
dp(651) = dp(14)*p(154) + p(14)*dp(154)
dp(652) = dp(14)*p(155) + p(14)*dp(155)
dp(653) = dp(9)*p(168) + p(9)*dp(168)
dp(654) = dp(9)*p(169) + p(9)*dp(169)
dp(655) = dp(9)*p(170) + p(9)*dp(170)
dp(656) = dp(14)*p(122) + p(14)*dp(122)
dp(657) = dp(14)*p(123) + p(14)*dp(123)
dp(658) = dp(14)*p(124) + p(14)*dp(124)
dp(659) = dp(14)*p(125) + p(14)*dp(125)
dp(660) = dp(14)*p(126) + p(14)*dp(126)
dp(661) = dp(14)*p(127) + p(14)*dp(127)
dp(662) = dp(14)*p(108) + p(14)*dp(108)
dp(663) = dp(14)*p(109) + p(14)*dp(109)
dp(664) = dp(9)*p(187) + p(9)*dp(187)
dp(665) = dp(14)*p(130) + p(14)*dp(130)
dp(666) = dp(14)*p(131) + p(14)*dp(131)
dp(667) = dp(9)*p(174) + p(9)*dp(174)
dp(668) = dp(9)*p(175) + p(9)*dp(175)
dp(669) = dp(9)*p(176) + p(9)*dp(176)
dp(670) = dp(14)*p(162) + p(14)*dp(162)
dp(671) = dp(14)*p(163) + p(14)*dp(163)
dp(672) = dp(14)*p(164) + p(14)*dp(164)
dp(673) = dp(9)*p(190) + p(9)*dp(190)
dp(674) = dp(14)*p(135) + p(14)*dp(135)
dp(675) = dp(14)*p(136) + p(14)*dp(136)
dp(676) = dp(14)*p(137) + p(14)*dp(137)
dp(677) = dp(9)*p(193) + p(9)*dp(193)
dp(678) = dp(14)*p(172) + p(14)*dp(172)
dp(679) = dp(14)*p(173) + p(14)*dp(173)
dp(680) = dp(9)*p(181) + p(9)*dp(181)
dp(681) = dp(9)*p(143) + p(9)*dp(143)
dp(682) = dp(9)*p(110) + p(9)*dp(110)
dp(683) = dp(9)*p(144) + p(9)*dp(144)
dp(684) = dp(9)*p(111) + p(9)*dp(111)
dp(685) = dp(9)*p(112) + p(9)*dp(112)
dp(686) = dp(9)*p(145) + p(9)*dp(145)
dp(687) = dp(9)*p(113) + p(9)*dp(113)
dp(688) = dp(9)*p(146) + p(9)*dp(146)
dp(689) = dp(9)*p(147) + p(9)*dp(147)
dp(690) = dp(9)*p(148) + p(9)*dp(148)
dp(691) = dp(9)*p(182) + p(9)*dp(182)
dp(692) = dp(1)*p(401) + p(1)*dp(401) - dp(604)
dp(693) = dp(21)*p(114) + p(21)*dp(114) - dp(609)
dp(694) = dp(24)*p(101) + p(24)*dp(101) - dp(612)
dp(695) = dp(3)*p(284) + p(3)*dp(284) - dp(606) - dp(694)
dp(696) = dp(1)*p(404) + p(1)*dp(404) - dp(606) - dp(695)
dp(697) = dp(1)*p(405) + p(1)*dp(405) - dp(608) - dp(696)
dp(698) = dp(13)*p(178) + p(13)*dp(178) - dp(604)
dp(699) = dp(39)*p(73) + p(39)*dp(73) - dp(609)
dp(700) = dp(3)*p(285) + p(3)*dp(285) - dp(612)
dp(701) = dp(14)*p(194) + p(14)*dp(194)
dp(702) = dp(9)*p(183) + p(9)*dp(183)
dp(703) = dp(9)*p(151) + p(9)*dp(151)
dp(704) = dp(9)*p(120) + p(9)*dp(120)
dp(705) = dp(9)*p(152) + p(9)*dp(152)
dp(706) = dp(9)*p(121) + p(9)*dp(121)
dp(707) = dp(9)*p(153) + p(9)*dp(153)
dp(708) = dp(9)*p(154) + p(9)*dp(154)
dp(709) = dp(9)*p(155) + p(9)*dp(155)
dp(710) = dp(9)*p(184) + p(9)*dp(184)
dp(711) = dp(24)*p(91) + p(24)*dp(91) - dp(592)
dp(712) = dp(1)*p(416) + p(1)*dp(416) - dp(616) - dp(711)
dp(713) = dp(2)*p(401) + p(2)*dp(401) - dp(613)
dp(714) = dp(1)*p(418) + p(1)*dp(418) - dp(618) - dp(713)
dp(715) = dp(1)*p(419) + p(1)*dp(419) - dp(619) - dp(714)
dp(716) = dp(13)*p(181) + p(13)*dp(181) - dp(701)
dp(717) = dp(2)*p(404) + p(2)*dp(404) - dp(615) - dp(716)
dp(718) = dp(32)*p(91) + p(32)*dp(91) - dp(701)
dp(719) = dp(1)*p(421) + p(1)*dp(421) - dp(619) - dp(717)
dp(720) = dp(41)*p(73) + p(41)*dp(73) - dp(701)
dp(721) = dp(6)*p(279) + p(6)*dp(279) - dp(613)
dp(722) = dp(16)*p(150) + p(16)*dp(150) - dp(701)
dp(723) = dp(1)*p(425) + p(1)*dp(425) - dp(624) - dp(721)
dp(724) = dp(4)*p(285) + p(4)*dp(285) - dp(701)
dp(725) = dp(11)*p(150) + p(11)*dp(150) - dp(592)
dp(726) = dp(14)*p(181) + p(14)*dp(181)
dp(727) = dp(14)*p(182) + p(14)*dp(182)
dp(728) = dp(1)*p(430) + p(1)*dp(430) - dp(632)
dp(729) = dp(21)*p(130) + p(21)*dp(130) - dp(635)
dp(730) = dp(3)*p(295) + p(3)*dp(295) - dp(633) - dp(729)
dp(731) = dp(1)*p(432) + p(1)*dp(432) - dp(633) - dp(730)
dp(732) = dp(39)*p(81) + p(39)*dp(81) - dp(632)
dp(733) = dp(3)*p(296) + p(3)*dp(296) - dp(635)
dp(734) = dp(14)*p(149) + p(14)*dp(149)
dp(735) = dp(14)*p(150) + p(14)*dp(150)
dp(736) = dp(14)*p(119) + p(14)*dp(119)
dp(737) = dp(9)*p(185) + p(9)*dp(185)
dp(738) = dp(9)*p(162) + p(9)*dp(162)
dp(739) = dp(9)*p(134) + p(9)*dp(134)
dp(740) = dp(9)*p(163) + p(9)*dp(163)
dp(741) = dp(9)*p(164) + p(9)*dp(164)
dp(742) = dp(9)*p(186) + p(9)*dp(186)
dp(743) = dp(24)*p(93) + p(24)*dp(93) - dp(593)
dp(744) = dp(18)*p(114) + p(18)*dp(114) - dp(595)
dp(745) = dp(1)*p(443) + p(1)*dp(443) - dp(643) - dp(744)
dp(746) = dp(1)*p(444) + p(1)*dp(444) - dp(644) - dp(745)
dp(747) = dp(13)*p(183) + p(13)*dp(183) - dp(726)
dp(748) = dp(2)*p(421) + p(2)*dp(421) - dp(631) - dp(730)
dp(749) = dp(32)*p(93) + p(32)*dp(93) - dp(726)
dp(750) = dp(1)*p(446) + p(1)*dp(446) - dp(644) - dp(748)
dp(751) = dp(43)*p(73) + p(43)*dp(73) - dp(726)
dp(752) = dp(13)*p(153) + p(13)*dp(153) - dp(595)
dp(753) = dp(18)*p(150) + p(18)*dp(150) - dp(726)
dp(754) = dp(34)*p(73) + p(34)*dp(73) - dp(593)
dp(755) = dp(14)*p(183) + p(14)*dp(183)
dp(756) = dp(14)*p(184) + p(14)*dp(184)
dp(757) = dp(16)*p(130) + p(16)*dp(130) - dp(598)
dp(758) = dp(2)*p(430) + p(2)*dp(430) - dp(636)
dp(759) = dp(1)*p(454) + p(1)*dp(454) - dp(654) - dp(758)
dp(760) = dp(41)*p(81) + p(41)*dp(81) - dp(735)
dp(761) = dp(2)*p(432) + p(2)*dp(432) - dp(638) - dp(760)
dp(762) = dp(16)*p(160) + p(16)*dp(160) - dp(735)
dp(763) = dp(45)*p(69) + p(45)*dp(69) - dp(636)
dp(764) = dp(4)*p(296) + p(4)*dp(296) - dp(735)
dp(765) = dp(11)*p(160) + p(11)*dp(160) - dp(598)
dp(766) = dp(14)*p(156) + p(14)*dp(156)
dp(767) = dp(14)*p(157) + p(14)*dp(157)
dp(768) = dp(14)*p(158) + p(14)*dp(158)
dp(769) = dp(14)*p(128) + p(14)*dp(128)
dp(770) = dp(14)*p(129) + p(14)*dp(129)
dp(771) = dp(1)*p(464) + p(1)*dp(464) - dp(664)
dp(772) = dp(4)*p(262) + p(4)*dp(262) - dp(673) - dp(661)
dp(773) = dp(3)*p(310) + p(3)*dp(310) - dp(664)
dp(774) = dp(14)*p(159) + p(14)*dp(159)
dp(775) = dp(14)*p(160) + p(14)*dp(160)
dp(776) = dp(14)*p(132) + p(14)*dp(132)
dp(777) = dp(14)*p(133) + p(14)*dp(133)
dp(778) = dp(9)*p(188) + p(9)*dp(188)
dp(779) = dp(9)*p(172) + p(9)*dp(172)
dp(780) = dp(9)*p(173) + p(9)*dp(173)
dp(781) = dp(9)*p(189) + p(9)*dp(189)
dp(782) = dp(24)*p(95) + p(24)*dp(95) - dp(600)
dp(783) = dp(22)*p(114) + p(22)*dp(114) - dp(650)
dp(784) = dp(1)*p(471) + p(1)*dp(471) - dp(668) - dp(783)
dp(785) = dp(13)*p(185) + p(13)*dp(185) - dp(755)
dp(786) = dp(2)*p(446) + p(2)*dp(446) - dp(652) - dp(761)
dp(787) = dp(32)*p(95) + p(32)*dp(95) - dp(755)
dp(788) = dp(6)*p(299) + p(6)*dp(299) - dp(650)
dp(789) = dp(46)*p(73) + p(46)*dp(73) - dp(755)
dp(790) = dp(32)*p(84) + p(32)*dp(84) - dp(600)
dp(791) = dp(14)*p(185) + p(14)*dp(185)
dp(792) = dp(14)*p(186) + p(14)*dp(186)
dp(793) = dp(2)*p(453) + p(2)*dp(453) - dp(656) - dp(771)
dp(794) = dp(1)*p(478) + p(1)*dp(478) - dp(673) - dp(793)
dp(795) = dp(43)*p(81) + p(43)*dp(81) - dp(736)
dp(796) = dp(2)*p(456) + p(2)*dp(456) - dp(659) - dp(772)
dp(797) = dp(18)*p(160) + p(18)*dp(160) - dp(736)
dp(798) = dp(1)*p(480) + p(1)*dp(480) - dp(673) - dp(796)
dp(799) = dp(14)*p(165) + p(14)*dp(165)
dp(800) = dp(14)*p(166) + p(14)*dp(166)
dp(801) = dp(14)*p(167) + p(14)*dp(167)
dp(802) = dp(14)*p(138) + p(14)*dp(138)
dp(803) = dp(14)*p(139) + p(14)*dp(139)
dp(804) = dp(2)*p(464) + p(2)*dp(464) - dp(665)
dp(805) = dp(4)*p(310) + p(4)*dp(310) - dp(775)
dp(806) = dp(11)*p(187) + p(11)*dp(187) - dp(665)
dp(807) = dp(14)*p(168) + p(14)*dp(168)
dp(808) = dp(14)*p(169) + p(14)*dp(169)
dp(809) = dp(14)*p(170) + p(14)*dp(170)
dp(810) = dp(9)*p(191) + p(9)*dp(191)
dp(811) = dp(9)*p(192) + p(9)*dp(192)
dp(812) = dp(24)*p(98) + p(24)*dp(98) - dp(603)
dp(813) = dp(1)*p(485) + p(1)*dp(485) - dp(677) - dp(812)
dp(814) = dp(13)*p(188) + p(13)*dp(188) - dp(791)
dp(815) = dp(2)*p(473) + p(2)*dp(473) - dp(672) - dp(796)
dp(816) = dp(32)*p(98) + p(32)*dp(98) - dp(791)
dp(817) = dp(13)*p(173) + p(13)*dp(173) - dp(603)
dp(818) = dp(14)*p(188) + p(14)*dp(188)
dp(819) = dp(14)*p(189) + p(14)*dp(189)
dp(820) = dp(22)*p(130) + p(22)*dp(130) - dp(663)
dp(821) = dp(46)*p(81) + p(46)*dp(81) - dp(769)
dp(822) = dp(45)*p(84) + p(45)*dp(84) - dp(663)
dp(823) = dp(14)*p(174) + p(14)*dp(174)
dp(824) = dp(14)*p(175) + p(14)*dp(175)
dp(825) = dp(14)*p(176) + p(14)*dp(176)
dp(826) = dp(9)*p(195) + p(9)*dp(195)
dp(827) = dp(24)*p(102) + p(24)*dp(102) - dp(679)
dp(828) = dp(13)*p(191) + p(13)*dp(191) - dp(818)
dp(829) = dp(6)*p(323) + p(6)*dp(323) - dp(679)
dp(830) = dp(14)*p(191) + p(14)*dp(191)
dp(831) = dp(14)*p(192) + p(14)*dp(192)
dp(832) = dp(9)*p(140) + p(9)*dp(140)
dp(833) = dp(9)*p(141) + p(9)*dp(141)
dp(834) = dp(9)*p(178) + p(9)*dp(178)
dp(835) = dp(9)*p(142) + p(9)*dp(142)
dp(836) = dp(9)*p(179) + p(9)*dp(179)
dp(837) = dp(9)*p(180) + p(9)*dp(180)
dp(838) = dp(9)*p(194) + p(9)*dp(194)
dp(839) = dp(3)*p(331) + p(3)*dp(331) - dp(680)
dp(840) = dp(39)*p(91) + p(39)*dp(91) - dp(682)
dp(841) = dp(16)*p(178) + p(16)*dp(178) - dp(685)
dp(842) = dp(1)*p(497) + p(1)*dp(497) - dp(683) - dp(841)
dp(843) = dp(2)*p(566) + p(2)*dp(566) - dp(842)
dp(844) = dp(1)*p(498) + p(1)*dp(498) - dp(686) - dp(842)
dp(845) = dp(7)*p(279) + p(7)*dp(279) - dp(685)
dp(846) = dp(1)*p(500) + p(1)*dp(500) - dp(689) - dp(844)
dp(847) = dp(49)*p(69) + p(49)*dp(69) - dp(682)
dp(848) = dp(1)*p(502) + p(1)*dp(502) - dp(691) - dp(846)
dp(849) = dp(11)*p(194) + p(11)*dp(194) - dp(680)
dp(850) = dp(1)*p(504) + p(1)*dp(504) - dp(695) - dp(694)
dp(851) = dp(1)*p(505) + p(1)*dp(505) - dp(700)
dp(852) = dp(3)*p(333) + p(3)*dp(333) - dp(702)
dp(853) = dp(39)*p(93) + p(39)*dp(93) - dp(704)
dp(854) = dp(18)*p(178) + p(18)*dp(178) - dp(707)
dp(855) = dp(1)*p(508) + p(1)*dp(508) - dp(705) - dp(854)
dp(856) = dp(1)*p(509) + p(1)*dp(509) - dp(707)
dp(857) = dp(1)*p(510) + p(1)*dp(510) - dp(708) - dp(855)
dp(858) = dp(21)*p(153) + p(21)*dp(153) - dp(704)
dp(859) = dp(1)*p(512) + p(1)*dp(512) - dp(710) - dp(857)
dp(860) = dp(34)*p(101) + p(34)*dp(101) - dp(702)
dp(861) = dp(6)*p(331) + p(6)*dp(331) - dp(701)
dp(862) = dp(1)*p(515) + p(1)*dp(515) - dp(717) - dp(715)
dp(863) = dp(2)*p(505) + p(2)*dp(505) - dp(701)
dp(864) = dp(1)*p(517) + p(1)*dp(517) - dp(730) - dp(729)
dp(865) = dp(1)*p(518) + p(1)*dp(518) - dp(733)
dp(866) = dp(3)*p(335) + p(3)*dp(335) - dp(737)
dp(867) = dp(39)*p(95) + p(39)*dp(95) - dp(739)
dp(868) = dp(1)*p(520) + p(1)*dp(520) - dp(738) - dp(867)
dp(869) = dp(2)*p(509) + p(2)*dp(509) - dp(721)
dp(870) = dp(1)*p(521) + p(1)*dp(521) - dp(740) - dp(868)
dp(871) = dp(7)*p(299) + p(7)*dp(299) - dp(739)
dp(872) = dp(1)*p(523) + p(1)*dp(523) - dp(742) - dp(870)
dp(873) = dp(49)*p(84) + p(49)*dp(84) - dp(737)
dp(874) = dp(6)*p(333) + p(6)*dp(333) - dp(726)
dp(875) = dp(1)*p(526) + p(1)*dp(526) - dp(748) - dp(746)
dp(876) = dp(8)*p(285) + p(8)*dp(285) - dp(726)
dp(877) = dp(45)*p(91) + p(45)*dp(91) - dp(735)
dp(878) = dp(1)*p(529) + p(1)*dp(529) - dp(761) - dp(759)
dp(879) = dp(2)*p(518) + p(2)*dp(518) - dp(735)
dp(880) = dp(1)*p(531) + p(1)*dp(531) - dp(772) - dp(771)
dp(881) = dp(1)*p(532) + p(1)*dp(532) - dp(773)
dp(882) = dp(14)*p(161) + p(14)*dp(161)
dp(883) = dp(3)*p(337) + p(3)*dp(337) - dp(778)
dp(884) = dp(39)*p(98) + p(39)*dp(98) - dp(780)
dp(885) = dp(1)*p(535) + p(1)*dp(535) - dp(779) - dp(884)
dp(886) = dp(1)*p(536) + p(1)*dp(536) - dp(780)
dp(887) = dp(1)*p(537) + p(1)*dp(537) - dp(781) - dp(885)
dp(888) = dp(21)*p(173) + p(21)*dp(173) - dp(778)
dp(889) = dp(6)*p(335) + p(6)*dp(335) - dp(755)
dp(890) = dp(1)*p(540) + p(1)*dp(540) - dp(786) - dp(784)
dp(891) = dp(22)*p(150) + p(22)*dp(150) - dp(755)
dp(892) = dp(45)*p(93) + p(45)*dp(93) - dp(736)
dp(893) = dp(1)*p(543) + p(1)*dp(543) - dp(796) - dp(794)
dp(894) = dp(8)*p(296) + p(8)*dp(296) - dp(736)
dp(895) = dp(16)*p(187) + p(16)*dp(187) - dp(775)
dp(896) = dp(1)*p(547) + p(1)*dp(547) - dp(806) - dp(804)
dp(897) = dp(2)*p(532) + p(2)*dp(532) - dp(775)
dp(898) = dp(14)*p(171) + p(14)*dp(171)
dp(899) = dp(6)*p(309) + p(6)*dp(309) - dp(808) - dp(807)
dp(900) = dp(1)*p(578) + p(1)*dp(578) - dp(899)
dp(901) = dp(14)*p(187) + p(14)*dp(187)
dp(902) = dp(3)*p(340) + p(3)*dp(340) - dp(810)
dp(903) = dp(2)*p(535) + p(2)*dp(535) - dp(787) - dp(783)
dp(904) = dp(2)*p(536) + p(2)*dp(536) - dp(788)
dp(905) = dp(1)*p(551) + p(1)*dp(551) - dp(811) - dp(903)
dp(906) = dp(7)*p(323) + p(7)*dp(323) - dp(810)
dp(907) = dp(6)*p(337) + p(6)*dp(337) - dp(791)
dp(908) = dp(1)*p(554) + p(1)*dp(554) - dp(815) - dp(813)
dp(909) = dp(50)*p(73) + p(50)*dp(73) - dp(791)
dp(910) = dp(45)*p(95) + p(45)*dp(95) - dp(769)
dp(911) = dp(1)*p(557) + p(1)*dp(557) - dp(822) - dp(820)
dp(912) = dp(22)*p(160) + p(22)*dp(160) - dp(769)
dp(913) = dp(14)*p(177) + p(14)*dp(177)
dp(914) = dp(18)*p(187) + p(18)*dp(187) - dp(777)
dp(915) = dp(2)*p(547) + p(2)*dp(547) - dp(808) - dp(899)
dp(916) = dp(8)*p(310) + p(8)*dp(310) - dp(777)
dp(917) = dp(14)*p(190) + p(14)*dp(190)
dp(918) = dp(3)*p(343) + p(3)*dp(343) - dp(826)
dp(919) = dp(2)*p(551) + p(2)*dp(551) - dp(816) - dp(813)
dp(920) = dp(1)*p(561) + p(1)*dp(561) - dp(826)
dp(921) = dp(6)*p(340) + p(6)*dp(340) - dp(818)
dp(922) = dp(1)*p(563) + p(1)*dp(563) - dp(829) - dp(827)
dp(923) = dp(32)*p(102) + p(32)*dp(102) - dp(818)
dp(924) = dp(45)*p(98) + p(45)*dp(98) - dp(802)
dp(925) = dp(2)*p(557) + p(2)*dp(557) - dp(824) - dp(915)
dp(926) = dp(50)*p(81) + p(50)*dp(81) - dp(802)
dp(927) = dp(14)*p(193) + p(14)*dp(193)
dp(928) = dp(2)*p(560) + p(2)*dp(560) - dp(828) - dp(827)
dp(929) = dp(2)*p(561) + p(2)*dp(561) - dp(829)
dp(930) = dp(6)*p(343) + p(6)*dp(343) - dp(830)
dp(931) = dp(2)*p(563) + p(2)*dp(563) - dp(831) - dp(925)
dp(932) = dp(13)*p(195) + p(13)*dp(195) - dp(830)
dp(933) = dp(14)*p(195) + p(14)*dp(195)
dp(934) = dp(1)*p(566) + p(1)*dp(566) - dp(834)
dp(935) = dp(1)*p(567) + p(1)*dp(567) - dp(836) - dp(934)
dp(936) = dp(39)*p(101) + p(39)*dp(101) - dp(835)
dp(937) = dp(3)*p(346) + p(3)*dp(346) - dp(838)
dp(938) = dp(1)*p(570) + p(1)*dp(570) - dp(839)
dp(939) = dp(2)*p(588) + p(2)*dp(588) - dp(938)
dp(940) = dp(1)*p(572) + p(1)*dp(572) - dp(852)
dp(941) = dp(8)*p(346) + p(8)*dp(346) - dp(940)
dp(942) = dp(1)*p(574) + p(1)*dp(574) - dp(866)
dp(943) = dp(22)*p(194) + p(22)*dp(194) - dp(942)
dp(944) = dp(1)*p(576) + p(1)*dp(576) - dp(883)
dp(945) = dp(50)*p(101) + p(50)*dp(101) - dp(944)
dp(946) = dp(1)*p(579) + p(1)*dp(579) - dp(902)
dp(947) = dp(49)*p(102) + p(49)*dp(102) - dp(946)
dp(948) = dp(2)*p(578) + p(2)*dp(578) - dp(901)
dp(949) = dp(1)*p(582) + p(1)*dp(582) - dp(918)
dp(950) = dp(21)*p(195) + p(21)*dp(195) - dp(949)
dp(951) = dp(2)*p(581) + p(2)*dp(581) - dp(917) - dp(948)
dp(952) = dp(1)*p(585) + p(1)*dp(585) - dp(928)
dp(953) = dp(7)*p(347) + p(7)*dp(347) - dp(952)
dp(954) = dp(45)*p(102) + p(45)*dp(102) - dp(913)
dp(955) = dp(2)*p(585) + p(2)*dp(585) - dp(930)
dp(956) = dp(1)*p(589) + p(1)*dp(589) - dp(955)
dp(957) = dp(6)*p(347) + p(6)*dp(347) - dp(933)
dp(958) = dp(1)*p(588) + p(1)*dp(588) - dp(937)
dp(959) = dp(2)*p(589) + p(2)*dp(589) - dp(957)
dp(960) = dp(9)*p(196) + p(9)*dp(196)
dp(961) = dp(9)*p(202) + p(9)*dp(202)
dp(962) = dp(9)*p(203) + p(9)*dp(203)
dp(963) = dp(9)*p(228) + p(9)*dp(228)
dp(964) = dp(9)*p(242) + p(9)*dp(242)
dp(965) = dp(9)*p(207) + p(9)*dp(207)
dp(966) = dp(9)*p(197) + p(9)*dp(197)
dp(967) = dp(9)*p(208) + p(9)*dp(208)
dp(968) = dp(9)*p(209) + p(9)*dp(209)
dp(969) = dp(9)*p(243) + p(9)*dp(243)
dp(970) = dp(9)*p(211) + p(9)*dp(211)
dp(971) = dp(9)*p(247) + p(9)*dp(247)
dp(972) = dp(9)*p(212) + p(9)*dp(212)
dp(973) = dp(9)*p(248) + p(9)*dp(248)
dp(974) = dp(9)*p(249) + p(9)*dp(249)
dp(975) = dp(9)*p(258) + p(9)*dp(258)
dp(976) = dp(9)*p(214) + p(9)*dp(214)
dp(977) = dp(9)*p(215) + p(9)*dp(215)
dp(978) = dp(9)*p(259) + p(9)*dp(259)
dp(979) = dp(9)*p(263) + p(9)*dp(263)
dp(980) = dp(9)*p(264) + p(9)*dp(264)
dp(981) = dp(9)*p(265) + p(9)*dp(265)
dp(982) = dp(9)*p(308) + p(9)*dp(308)
dp(983) = dp(9)*p(311) + p(9)*dp(311)
dp(984) = dp(9)*p(270) + p(9)*dp(270)
dp(985) = dp(9)*p(271) + p(9)*dp(271)
dp(986) = dp(9)*p(321) + p(9)*dp(321)
dp(987) = dp(9)*p(327) + p(9)*dp(327)
dp(988) = dp(9)*p(222) + p(9)*dp(222)
dp(989) = dp(9)*p(223) + p(9)*dp(223)
dp(990) = dp(9)*p(284) + p(9)*dp(284)
dp(991) = dp(9)*p(198) + p(9)*dp(198)
dp(992) = dp(9)*p(199) + p(9)*dp(199)
dp(993) = dp(9)*p(224) + p(9)*dp(224)
dp(994) = dp(9)*p(200) + p(9)*dp(200)
dp(995) = dp(9)*p(225) + p(9)*dp(225)
dp(996) = dp(9)*p(201) + p(9)*dp(201)
dp(997) = dp(9)*p(226) + p(9)*dp(226)
dp(998) = dp(9)*p(227) + p(9)*dp(227)
dp(999) = dp(9)*p(285) + p(9)*dp(285)
dp(1000) = dp(14)*p(328) + p(14)*dp(328)
dp(1001) = dp(14)*p(329) + p(14)*dp(329)
dp(1002) = dp(14)*p(330) + p(14)*dp(330)
dp(1003) = dp(9)*p(292) + p(9)*dp(292)
dp(1004) = dp(9)*p(233) + p(9)*dp(233)
dp(1005) = dp(9)*p(234) + p(9)*dp(234)
dp(1006) = dp(9)*p(235) + p(9)*dp(235)
dp(1007) = dp(9)*p(293) + p(9)*dp(293)
dp(1008) = dp(9)*p(236) + p(9)*dp(236)
dp(1009) = dp(9)*p(204) + p(9)*dp(204)
dp(1010) = dp(9)*p(205) + p(9)*dp(205)
dp(1011) = dp(9)*p(237) + p(9)*dp(237)
dp(1012) = dp(9)*p(238) + p(9)*dp(238)
dp(1013) = dp(9)*p(206) + p(9)*dp(206)
dp(1014) = dp(9)*p(239) + p(9)*dp(239)
dp(1015) = dp(9)*p(240) + p(9)*dp(240)
dp(1016) = dp(9)*p(241) + p(9)*dp(241)
dp(1017) = dp(9)*p(294) + p(9)*dp(294)
dp(1018) = dp(14)*p(276) + p(14)*dp(276)
dp(1019) = dp(14)*p(277) + p(14)*dp(277)
dp(1020) = dp(14)*p(278) + p(14)*dp(278)
dp(1021) = dp(14)*p(279) + p(14)*dp(279)
dp(1022) = dp(14)*p(280) + p(14)*dp(280)
dp(1023) = dp(14)*p(281) + p(14)*dp(281)
dp(1024) = dp(14)*p(282) + p(14)*dp(282)
dp(1025) = dp(14)*p(283) + p(14)*dp(283)
dp(1026) = dp(9)*p(244) + p(9)*dp(244)
dp(1027) = dp(9)*p(295) + p(9)*dp(295)
dp(1028) = dp(9)*p(210) + p(9)*dp(210)
dp(1029) = dp(9)*p(245) + p(9)*dp(245)
dp(1030) = dp(9)*p(246) + p(9)*dp(246)
dp(1031) = dp(9)*p(296) + p(9)*dp(296)
dp(1032) = dp(14)*p(222) + p(14)*dp(222)
dp(1033) = dp(14)*p(223) + p(14)*dp(223)
dp(1034) = dp(14)*p(224) + p(14)*dp(224)
dp(1035) = dp(14)*p(225) + p(14)*dp(225)
dp(1036) = dp(14)*p(226) + p(14)*dp(226)
dp(1037) = dp(14)*p(227) + p(14)*dp(227)
dp(1038) = dp(14)*p(202) + p(14)*dp(202)
dp(1039) = dp(14)*p(203) + p(14)*dp(203)
dp(1040) = dp(9)*p(302) + p(9)*dp(302)
dp(1041) = dp(9)*p(252) + p(9)*dp(252)
dp(1042) = dp(9)*p(253) + p(9)*dp(253)
dp(1043) = dp(9)*p(303) + p(9)*dp(303)
dp(1044) = dp(9)*p(254) + p(9)*dp(254)
dp(1045) = dp(9)*p(213) + p(9)*dp(213)
dp(1046) = dp(9)*p(255) + p(9)*dp(255)
dp(1047) = dp(9)*p(256) + p(9)*dp(256)
dp(1048) = dp(9)*p(257) + p(9)*dp(257)
dp(1049) = dp(9)*p(304) + p(9)*dp(304)
dp(1050) = dp(14)*p(286) + p(14)*dp(286)
dp(1051) = dp(14)*p(287) + p(14)*dp(287)
dp(1052) = dp(14)*p(288) + p(14)*dp(288)
dp(1053) = dp(14)*p(289) + p(14)*dp(289)
dp(1054) = dp(14)*p(290) + p(14)*dp(290)
dp(1055) = dp(14)*p(291) + p(14)*dp(291)
dp(1056) = dp(9)*p(305) + p(9)*dp(305)
dp(1057) = dp(9)*p(260) + p(9)*dp(260)
dp(1058) = dp(9)*p(306) + p(9)*dp(306)
dp(1059) = dp(9)*p(261) + p(9)*dp(261)
dp(1060) = dp(9)*p(262) + p(9)*dp(262)
dp(1061) = dp(9)*p(307) + p(9)*dp(307)
dp(1062) = dp(14)*p(233) + p(14)*dp(233)
dp(1063) = dp(14)*p(234) + p(14)*dp(234)
dp(1064) = dp(14)*p(235) + p(14)*dp(235)
dp(1065) = dp(14)*p(236) + p(14)*dp(236)
dp(1066) = dp(14)*p(237) + p(14)*dp(237)
dp(1067) = dp(14)*p(238) + p(14)*dp(238)
dp(1068) = dp(14)*p(239) + p(14)*dp(239)
dp(1069) = dp(14)*p(240) + p(14)*dp(240)
dp(1070) = dp(14)*p(241) + p(14)*dp(241)
dp(1071) = dp(14)*p(207) + p(14)*dp(207)
dp(1072) = dp(14)*p(208) + p(14)*dp(208)
dp(1073) = dp(14)*p(209) + p(14)*dp(209)
dp(1074) = dp(9)*p(309) + p(9)*dp(309)
dp(1075) = dp(9)*p(310) + p(9)*dp(310)
dp(1076) = dp(14)*p(244) + p(14)*dp(244)
dp(1077) = dp(14)*p(245) + p(14)*dp(245)
dp(1078) = dp(14)*p(246) + p(14)*dp(246)
dp(1079) = dp(14)*p(211) + p(14)*dp(211)
dp(1080) = dp(14)*p(212) + p(14)*dp(212)
dp(1081) = dp(9)*p(315) + p(9)*dp(315)
dp(1082) = dp(9)*p(267) + p(9)*dp(267)
dp(1083) = dp(9)*p(316) + p(9)*dp(316)
dp(1084) = dp(9)*p(268) + p(9)*dp(268)
dp(1085) = dp(9)*p(269) + p(9)*dp(269)
dp(1086) = dp(9)*p(317) + p(9)*dp(317)
dp(1087) = dp(14)*p(297) + p(14)*dp(297)
dp(1088) = dp(14)*p(298) + p(14)*dp(298)
dp(1089) = dp(14)*p(299) + p(14)*dp(299)
dp(1090) = dp(14)*p(300) + p(14)*dp(300)
dp(1091) = dp(14)*p(301) + p(14)*dp(301)
dp(1092) = dp(9)*p(318) + p(9)*dp(318)
dp(1093) = dp(9)*p(319) + p(9)*dp(319)
dp(1094) = dp(9)*p(320) + p(9)*dp(320)
dp(1095) = dp(14)*p(252) + p(14)*dp(252)
dp(1096) = dp(14)*p(253) + p(14)*dp(253)
dp(1097) = dp(14)*p(254) + p(14)*dp(254)
dp(1098) = dp(14)*p(255) + p(14)*dp(255)
dp(1099) = dp(14)*p(256) + p(14)*dp(256)
dp(1100) = dp(14)*p(257) + p(14)*dp(257)
dp(1101) = dp(14)*p(214) + p(14)*dp(214)
dp(1102) = dp(14)*p(215) + p(14)*dp(215)
dp(1103) = dp(9)*p(339) + p(9)*dp(339)
dp(1104) = dp(14)*p(260) + p(14)*dp(260)
dp(1105) = dp(14)*p(261) + p(14)*dp(261)
dp(1106) = dp(14)*p(262) + p(14)*dp(262)
dp(1107) = dp(9)*p(324) + p(9)*dp(324)
dp(1108) = dp(9)*p(325) + p(9)*dp(325)
dp(1109) = dp(9)*p(326) + p(9)*dp(326)
dp(1110) = dp(14)*p(312) + p(14)*dp(312)
dp(1111) = dp(14)*p(313) + p(14)*dp(313)
dp(1112) = dp(14)*p(314) + p(14)*dp(314)
dp(1113) = dp(9)*p(342) + p(9)*dp(342)
dp(1114) = dp(14)*p(267) + p(14)*dp(267)
dp(1115) = dp(14)*p(268) + p(14)*dp(268)
dp(1116) = dp(14)*p(269) + p(14)*dp(269)
dp(1117) = dp(9)*p(345) + p(9)*dp(345)
dp(1118) = dp(14)*p(322) + p(14)*dp(322)
dp(1119) = dp(14)*p(323) + p(14)*dp(323)
dp(1120) = dp(9)*p(331) + p(9)*dp(331)
dp(1121) = dp(9)*p(276) + p(9)*dp(276)
dp(1122) = dp(9)*p(216) + p(9)*dp(216)
dp(1123) = dp(9)*p(277) + p(9)*dp(277)
dp(1124) = dp(9)*p(217) + p(9)*dp(217)
dp(1125) = dp(9)*p(218) + p(9)*dp(218)
dp(1126) = dp(9)*p(278) + p(9)*dp(278)
dp(1127) = dp(9)*p(219) + p(9)*dp(219)
dp(1128) = dp(9)*p(220) + p(9)*dp(220)
dp(1129) = dp(9)*p(279) + p(9)*dp(279)
dp(1130) = dp(9)*p(280) + p(9)*dp(280)
dp(1131) = dp(9)*p(221) + p(9)*dp(221)
dp(1132) = dp(9)*p(281) + p(9)*dp(281)
dp(1133) = dp(9)*p(282) + p(9)*dp(282)
dp(1134) = dp(9)*p(283) + p(9)*dp(283)
dp(1135) = dp(9)*p(332) + p(9)*dp(332)
dp(1136) = dp(3)*p(401) + p(3)*dp(401) - dp(988)
dp(1137) = dp(7)*p(401) + p(7)*dp(401) - dp(991)
dp(1138) = dp(49)*p(114) + p(49)*dp(114) - dp(996)
dp(1139) = dp(24)*p(194) + p(24)*dp(194) - dp(999)
dp(1140) = dp(3)*p(504) + p(3)*dp(504) - dp(990) - dp(1139)
dp(1141) = dp(1)*p(695) + p(1)*dp(695) - dp(990) - dp(1140)
dp(1142) = dp(1)*p(696) + p(1)*dp(696) - dp(993) - dp(1141)
dp(1143) = dp(6)*p(566) + p(6)*dp(566) - dp(1136)
dp(1144) = dp(32)*p(178) + p(32)*dp(178) - dp(991)
dp(1145) = dp(39)*p(150) + p(39)*dp(150) - dp(996)
dp(1146) = dp(3)*p(505) + p(3)*dp(505) - dp(999)
dp(1147) = dp(14)*p(346) + p(14)*dp(346)
dp(1148) = dp(9)*p(333) + p(9)*dp(333)
dp(1149) = dp(9)*p(286) + p(9)*dp(286)
dp(1150) = dp(9)*p(229) + p(9)*dp(229)
dp(1151) = dp(9)*p(287) + p(9)*dp(287)
dp(1152) = dp(9)*p(230) + p(9)*dp(230)
dp(1153) = dp(9)*p(231) + p(9)*dp(231)
dp(1154) = dp(9)*p(288) + p(9)*dp(288)
dp(1155) = dp(9)*p(232) + p(9)*dp(232)
dp(1156) = dp(9)*p(289) + p(9)*dp(289)
dp(1157) = dp(9)*p(290) + p(9)*dp(290)
dp(1158) = dp(9)*p(291) + p(9)*dp(291)
dp(1159) = dp(9)*p(334) + p(9)*dp(334)
dp(1160) = dp(24)*p(181) + p(24)*dp(181) - dp(963)
dp(1161) = dp(41)*p(114) + p(41)*dp(114) - dp(960)
dp(1162) = dp(4)*p(401) + p(4)*dp(401) - dp(961)
dp(1163) = dp(1)*p(713) + p(1)*dp(713) - dp(1005) - dp(1162)
dp(1164) = dp(1)*p(714) + p(1)*dp(714) - dp(1006) - dp(1163)
dp(1165) = dp(1)*p(715) + p(1)*dp(715) - dp(1007) - dp(1164)
dp(1166) = dp(13)*p(331) + p(13)*dp(331) - dp(1147)
dp(1167) = dp(2)*p(695) + p(2)*dp(695) - dp(1002) - dp(1166)
dp(1168) = dp(32)*p(181) + p(32)*dp(181) - dp(1147)
dp(1169) = dp(1)*p(717) + p(1)*dp(717) - dp(1007) - dp(1167)
dp(1170) = dp(73)*p(91) + p(73)*dp(91) - dp(1147)
dp(1171) = dp(1)*p(719) + p(1)*dp(719) - dp(1011) - dp(1169)
dp(1172) = dp(41)*p(150) + p(41)*dp(150) - dp(1147)
dp(1173) = dp(13)*p(279) + p(13)*dp(279) - dp(961)
dp(1174) = dp(16)*p(285) + p(16)*dp(285) - dp(1147)
dp(1175) = dp(69)*p(73) + p(69)*dp(73) - dp(960)
dp(1176) = dp(4)*p(505) + p(4)*dp(505) - dp(1147)
dp(1177) = dp(11)*p(285) + p(11)*dp(285) - dp(963)
dp(1178) = dp(14)*p(331) + p(14)*dp(331)
dp(1179) = dp(14)*p(332) + p(14)*dp(332)
dp(1180) = dp(3)*p(430) + p(3)*dp(430) - dp(1026)
dp(1181) = dp(7)*p(430) + p(7)*dp(430) - dp(1028)
dp(1182) = dp(49)*p(130) + p(49)*dp(130) - dp(1031)
dp(1183) = dp(3)*p(517) + p(3)*dp(517) - dp(1027) - dp(1182)
dp(1184) = dp(1)*p(730) + p(1)*dp(730) - dp(1027) - dp(1183)
dp(1185) = dp(45)*p(178) + p(45)*dp(178) - dp(1180)
dp(1186) = dp(39)*p(160) + p(39)*dp(160) - dp(1028)
dp(1187) = dp(3)*p(518) + p(3)*dp(518) - dp(1031)
dp(1188) = dp(14)*p(284) + p(14)*dp(284)
dp(1189) = dp(14)*p(285) + p(14)*dp(285)
dp(1190) = dp(14)*p(228) + p(14)*dp(228)
dp(1191) = dp(9)*p(335) + p(9)*dp(335)
dp(1192) = dp(9)*p(297) + p(9)*dp(297)
dp(1193) = dp(9)*p(250) + p(9)*dp(250)
dp(1194) = dp(9)*p(298) + p(9)*dp(298)
dp(1195) = dp(9)*p(251) + p(9)*dp(251)
dp(1196) = dp(9)*p(299) + p(9)*dp(299)
dp(1197) = dp(9)*p(300) + p(9)*dp(300)
dp(1198) = dp(9)*p(301) + p(9)*dp(301)
dp(1199) = dp(9)*p(336) + p(9)*dp(336)
dp(1200) = dp(24)*p(183) + p(24)*dp(183) - dp(964)
dp(1201) = dp(43)*p(114) + p(43)*dp(114) - dp(966)
dp(1202) = dp(8)*p(401) + p(8)*dp(401) - dp(1021)
dp(1203) = dp(1)*p(745) + p(1)*dp(745) - dp(1042) - dp(1202)
dp(1204) = dp(1)*p(746) + p(1)*dp(746) - dp(1043) - dp(1203)
dp(1205) = dp(13)*p(333) + p(13)*dp(333) - dp(1178)
dp(1206) = dp(2)*p(717) + p(2)*dp(717) - dp(1025) - dp(1183)
dp(1207) = dp(32)*p(183) + p(32)*dp(183) - dp(1178)
dp(1208) = dp(1)*p(748) + p(1)*dp(748) - dp(1043) - dp(1206)
dp(1209) = dp(73)*p(93) + p(73)*dp(93) - dp(1178)
dp(1210) = dp(6)*p(509) + p(6)*dp(509) - dp(1021)
dp(1211) = dp(43)*p(150) + p(43)*dp(150) - dp(1178)
dp(1212) = dp(32)*p(153) + p(32)*dp(153) - dp(966)
dp(1213) = dp(18)*p(285) + p(18)*dp(285) - dp(1178)
dp(1214) = dp(34)*p(150) + p(34)*dp(150) - dp(964)
dp(1215) = dp(14)*p(333) + p(14)*dp(333)
dp(1216) = dp(14)*p(334) + p(14)*dp(334)
dp(1217) = dp(41)*p(130) + p(41)*dp(130) - dp(973)
dp(1218) = dp(4)*p(430) + p(4)*dp(430) - dp(970)
dp(1219) = dp(1)*p(758) + p(1)*dp(758) - dp(1057) - dp(1218)
dp(1220) = dp(1)*p(759) + p(1)*dp(759) - dp(1058) - dp(1219)
dp(1221) = dp(81)*p(91) + p(81)*dp(91) - dp(1189)
dp(1222) = dp(2)*p(730) + p(2)*dp(730) - dp(1034) - dp(1221)
dp(1223) = dp(41)*p(160) + p(41)*dp(160) - dp(1189)
dp(1224) = dp(1)*p(761) + p(1)*dp(761) - dp(1058) - dp(1222)
dp(1225) = dp(16)*p(296) + p(16)*dp(296) - dp(1189)
dp(1226) = dp(69)*p(81) + p(69)*dp(81) - dp(970)
dp(1227) = dp(4)*p(518) + p(4)*dp(518) - dp(1189)
dp(1228) = dp(11)*p(296) + p(11)*dp(296) - dp(973)
dp(1229) = dp(14)*p(292) + p(14)*dp(292)
dp(1230) = dp(14)*p(293) + p(14)*dp(293)
dp(1231) = dp(14)*p(294) + p(14)*dp(294)
dp(1232) = dp(14)*p(242) + p(14)*dp(242)
dp(1233) = dp(14)*p(243) + p(14)*dp(243)
dp(1234) = dp(3)*p(464) + p(3)*dp(464) - dp(1074)
dp(1235) = dp(7)*p(464) + p(7)*dp(464) - dp(1075)
dp(1236) = dp(3)*p(531) + p(3)*dp(531) - dp(1074) - dp(1235)
dp(1237) = dp(39)*p(187) + p(39)*dp(187) - dp(1234)
dp(1238) = dp(3)*p(532) + p(3)*dp(532) - dp(1075)
dp(1239) = dp(14)*p(295) + p(14)*dp(295)
dp(1240) = dp(14)*p(296) + p(14)*dp(296)
dp(1241) = dp(14)*p(247) + p(14)*dp(247)
dp(1242) = dp(14)*p(248) + p(14)*dp(248)
dp(1243) = dp(14)*p(249) + p(14)*dp(249)
dp(1244) = dp(9)*p(337) + p(9)*dp(337)
dp(1245) = dp(9)*p(312) + p(9)*dp(312)
dp(1246) = dp(9)*p(266) + p(9)*dp(266)
dp(1247) = dp(9)*p(313) + p(9)*dp(313)
dp(1248) = dp(9)*p(314) + p(9)*dp(314)
dp(1249) = dp(9)*p(338) + p(9)*dp(338)
dp(1250) = dp(24)*p(185) + p(24)*dp(185) - dp(975)
dp(1251) = dp(46)*p(114) + p(46)*dp(114) - dp(977)
dp(1252) = dp(1)*p(783) + p(1)*dp(783) - dp(1082) - dp(1251)
dp(1253) = dp(1)*p(784) + p(1)*dp(784) - dp(1083) - dp(1252)
dp(1254) = dp(13)*p(335) + p(13)*dp(335) - dp(1215)
dp(1255) = dp(2)*p(748) + p(2)*dp(748) - dp(1055) - dp(1222)
dp(1256) = dp(32)*p(185) + p(32)*dp(185) - dp(1215)
dp(1257) = dp(1)*p(786) + p(1)*dp(786) - dp(1083) - dp(1255)
dp(1258) = dp(73)*p(95) + p(73)*dp(95) - dp(1215)
dp(1259) = dp(13)*p(299) + p(13)*dp(299) - dp(977)
dp(1260) = dp(46)*p(150) + p(46)*dp(150) - dp(1215)
dp(1261) = dp(73)*p(84) + p(73)*dp(84) - dp(975)
dp(1262) = dp(14)*p(335) + p(14)*dp(335)
dp(1263) = dp(14)*p(336) + p(14)*dp(336)
dp(1264) = dp(43)*p(130) + p(43)*dp(130) - dp(974)
dp(1265) = dp(8)*p(430) + p(8)*dp(430) - dp(1038)
dp(1266) = dp(1)*p(794) + p(1)*dp(794) - dp(1093) - dp(1265)
dp(1267) = dp(81)*p(93) + p(81)*dp(93) - dp(1190)
dp(1268) = dp(2)*p(761) + p(2)*dp(761) - dp(1066) - dp(1236)
dp(1269) = dp(43)*p(160) + p(43)*dp(160) - dp(1190)
dp(1270) = dp(45)*p(153) + p(45)*dp(153) - dp(1038)
dp(1271) = dp(18)*p(296) + p(18)*dp(296) - dp(1190)
dp(1272) = dp(34)*p(160) + p(34)*dp(160) - dp(974)
dp(1273) = dp(14)*p(302) + p(14)*dp(302)
dp(1274) = dp(14)*p(303) + p(14)*dp(303)
dp(1275) = dp(14)*p(304) + p(14)*dp(304)
dp(1276) = dp(14)*p(258) + p(14)*dp(258)
dp(1277) = dp(14)*p(259) + p(14)*dp(259)
dp(1278) = dp(4)*p(464) + p(4)*dp(464) - dp(983)
dp(1279) = dp(1)*p(804) + p(1)*dp(804) - dp(1103) - dp(1278)
dp(1280) = dp(16)*p(310) + p(16)*dp(310) - dp(1240)
dp(1281) = dp(2)*p(772) + p(2)*dp(772) - dp(1077) - dp(1280)
dp(1282) = dp(4)*p(532) + p(4)*dp(532) - dp(1240)
dp(1283) = dp(11)*p(310) + p(11)*dp(310) - dp(983)
dp(1284) = dp(14)*p(305) + p(14)*dp(305)
dp(1285) = dp(14)*p(306) + p(14)*dp(306)
dp(1286) = dp(14)*p(307) + p(14)*dp(307)
dp(1287) = dp(14)*p(263) + p(14)*dp(263)
dp(1288) = dp(14)*p(264) + p(14)*dp(264)
dp(1289) = dp(14)*p(265) + p(14)*dp(265)
dp(1290) = dp(6)*p(464) + p(6)*dp(464) - dp(1104)
dp(1291) = dp(3)*p(578) + p(3)*dp(578) - dp(1290)
dp(1292) = dp(14)*p(309) + p(14)*dp(309)
dp(1293) = dp(14)*p(310) + p(14)*dp(310)
dp(1294) = dp(9)*p(340) + p(9)*dp(340)
dp(1295) = dp(9)*p(322) + p(9)*dp(322)
dp(1296) = dp(9)*p(323) + p(9)*dp(323)
dp(1297) = dp(9)*p(341) + p(9)*dp(341)
dp(1298) = dp(24)*p(188) + p(24)*dp(188) - dp(984)
dp(1299) = dp(50)*p(114) + p(50)*dp(114) - dp(1089)
dp(1300) = dp(1)*p(813) + p(1)*dp(813) - dp(1108) - dp(1299)
dp(1301) = dp(13)*p(337) + p(13)*dp(337) - dp(1262)
dp(1302) = dp(2)*p(786) + p(2)*dp(786) - dp(1091) - dp(1268)
dp(1303) = dp(32)*p(188) + p(32)*dp(188) - dp(1262)
dp(1304) = dp(6)*p(536) + p(6)*dp(536) - dp(1089)
dp(1305) = dp(73)*p(98) + p(73)*dp(98) - dp(1262)
dp(1306) = dp(32)*p(173) + p(32)*dp(173) - dp(984)
dp(1307) = dp(14)*p(337) + p(14)*dp(337)
dp(1308) = dp(14)*p(338) + p(14)*dp(338)
dp(1309) = dp(46)*p(130) + p(46)*dp(130) - dp(982)
dp(1310) = dp(1)*p(820) + p(1)*dp(820) - dp(1113) - dp(1309)
dp(1311) = dp(81)*p(95) + p(81)*dp(95) - dp(1232)
dp(1312) = dp(2)*p(796) + p(2)*dp(796) - dp(1098) - dp(1281)
dp(1313) = dp(46)*p(160) + p(46)*dp(160) - dp(1232)
dp(1314) = dp(81)*p(84) + p(81)*dp(84) - dp(982)
dp(1315) = dp(14)*p(315) + p(14)*dp(315)
dp(1316) = dp(14)*p(316) + p(14)*dp(316)
dp(1317) = dp(14)*p(317) + p(14)*dp(317)
dp(1318) = dp(14)*p(270) + p(14)*dp(270)
dp(1319) = dp(14)*p(271) + p(14)*dp(271)
dp(1320) = dp(8)*p(464) + p(8)*dp(464) - dp(1079)
dp(1321) = dp(18)*p(310) + p(18)*dp(310) - dp(1242)
dp(1322) = dp(34)*p(187) + p(34)*dp(187) - dp(1079)
dp(1323) = dp(14)*p(318) + p(14)*dp(318)
dp(1324) = dp(14)*p(319) + p(14)*dp(319)
dp(1325) = dp(14)*p(320) + p(14)*dp(320)
dp(1326) = dp(9)*p(343) + p(9)*dp(343)
dp(1327) = dp(9)*p(344) + p(9)*dp(344)
dp(1328) = dp(24)*p(191) + p(24)*dp(191) - dp(987)
dp(1329) = dp(1)*p(827) + p(1)*dp(827) - dp(1117) - dp(1328)
dp(1330) = dp(13)*p(340) + p(13)*dp(340) - dp(1307)
dp(1331) = dp(2)*p(815) + p(2)*dp(815) - dp(1112) - dp(1312)
dp(1332) = dp(32)*p(191) + p(32)*dp(191) - dp(1307)
dp(1333) = dp(13)*p(323) + p(13)*dp(323) - dp(987)
dp(1334) = dp(14)*p(340) + p(14)*dp(340)
dp(1335) = dp(14)*p(341) + p(14)*dp(341)
dp(1336) = dp(50)*p(130) + p(50)*dp(130) - dp(1102)
dp(1337) = dp(81)*p(98) + p(81)*dp(98) - dp(1276)
dp(1338) = dp(45)*p(173) + p(45)*dp(173) - dp(1102)
dp(1339) = dp(14)*p(324) + p(14)*dp(324)
dp(1340) = dp(14)*p(325) + p(14)*dp(325)
dp(1341) = dp(14)*p(326) + p(14)*dp(326)
dp(1342) = dp(9)*p(347) + p(9)*dp(347)
dp(1343) = dp(24)*p(195) + p(24)*dp(195) - dp(1119)
dp(1344) = dp(13)*p(343) + p(13)*dp(343) - dp(1334)
dp(1345) = dp(6)*p(561) + p(6)*dp(561) - dp(1119)
dp(1346) = dp(14)*p(343) + p(14)*dp(343)
dp(1347) = dp(14)*p(344) + p(14)*dp(344)
dp(1348) = dp(9)*p(272) + p(9)*dp(272)
dp(1349) = dp(9)*p(273) + p(9)*dp(273)
dp(1350) = dp(9)*p(274) + p(9)*dp(274)
dp(1351) = dp(9)*p(328) + p(9)*dp(328)
dp(1352) = dp(9)*p(275) + p(9)*dp(275)
dp(1353) = dp(9)*p(329) + p(9)*dp(329)
dp(1354) = dp(9)*p(330) + p(9)*dp(330)
dp(1355) = dp(9)*p(346) + p(9)*dp(346)
dp(1356) = dp(3)*p(570) + p(3)*dp(570) - dp(1120)
dp(1357) = dp(39)*p(181) + p(39)*dp(181) - dp(1122)
dp(1358) = dp(41)*p(178) + p(41)*dp(178) - dp(1125)
dp(1359) = dp(4)*p(566) + p(4)*dp(566) - dp(1129)
dp(1360) = dp(1)*p(842) + p(1)*dp(842) - dp(1126) - dp(1359)
dp(1361) = dp(1)*p(843) + p(1)*dp(843) - dp(1129)
dp(1362) = dp(1)*p(844) + p(1)*dp(844) - dp(1130) - dp(1360)
dp(1363) = dp(21)*p(279) + p(21)*dp(279) - dp(1125)
dp(1364) = dp(1)*p(846) + p(1)*dp(846) - dp(1133) - dp(1362)
dp(1365) = dp(69)*p(101) + p(69)*dp(101) - dp(1122)
dp(1366) = dp(1)*p(848) + p(1)*dp(848) - dp(1135) - dp(1364)
dp(1367) = dp(11)*p(346) + p(11)*dp(346) - dp(1120)
dp(1368) = dp(1)*p(850) + p(1)*dp(850) - dp(1140) - dp(1139)
dp(1369) = dp(1)*p(851) + p(1)*dp(851) - dp(1146)
dp(1370) = dp(3)*p(572) + p(3)*dp(572) - dp(1148)
dp(1371) = dp(39)*p(183) + p(39)*dp(183) - dp(1150)
dp(1372) = dp(43)*p(178) + p(43)*dp(178) - dp(1153)
dp(1373) = dp(1)*p(854) + p(1)*dp(854) - dp(1151) - dp(1372)
dp(1374) = dp(2)*p(843) + p(2)*dp(843) - dp(1143)
dp(1375) = dp(1)*p(855) + p(1)*dp(855) - dp(1154) - dp(1373)
dp(1376) = dp(7)*p(509) + p(7)*dp(509) - dp(1153)
dp(1377) = dp(1)*p(857) + p(1)*dp(857) - dp(1157) - dp(1375)
dp(1378) = dp(49)*p(153) + p(49)*dp(153) - dp(1150)
dp(1379) = dp(1)*p(859) + p(1)*dp(859) - dp(1159) - dp(1377)
dp(1380) = dp(34)*p(194) + p(34)*dp(194) - dp(1148)
dp(1381) = dp(6)*p(570) + p(6)*dp(570) - dp(1147)
dp(1382) = dp(1)*p(862) + p(1)*dp(862) - dp(1167) - dp(1165)
dp(1383) = dp(2)*p(851) + p(2)*dp(851) - dp(1147)
dp(1384) = dp(1)*p(864) + p(1)*dp(864) - dp(1183) - dp(1182)
dp(1385) = dp(1)*p(865) + p(1)*dp(865) - dp(1187)
dp(1386) = dp(3)*p(574) + p(3)*dp(574) - dp(1191)
dp(1387) = dp(39)*p(185) + p(39)*dp(185) - dp(1193)
dp(1388) = dp(46)*p(178) + p(46)*dp(178) - dp(1196)
dp(1389) = dp(1)*p(868) + p(1)*dp(868) - dp(1194) - dp(1388)
dp(1390) = dp(1)*p(869) + p(1)*dp(869) - dp(1196)
dp(1391) = dp(1)*p(870) + p(1)*dp(870) - dp(1197) - dp(1389)
dp(1392) = dp(21)*p(299) + p(21)*dp(299) - dp(1193)
dp(1393) = dp(1)*p(872) + p(1)*dp(872) - dp(1199) - dp(1391)
dp(1394) = dp(84)*p(101) + p(84)*dp(101) - dp(1191)
dp(1395) = dp(6)*p(572) + p(6)*dp(572) - dp(1178)
dp(1396) = dp(1)*p(875) + p(1)*dp(875) - dp(1206) - dp(1204)
dp(1397) = dp(8)*p(505) + p(8)*dp(505) - dp(1178)
dp(1398) = dp(45)*p(181) + p(45)*dp(181) - dp(1189)
dp(1399) = dp(1)*p(878) + p(1)*dp(878) - dp(1222) - dp(1220)
dp(1400) = dp(2)*p(865) + p(2)*dp(865) - dp(1189)
dp(1401) = dp(1)*p(880) + p(1)*dp(880) - dp(1236) - dp(1235)
dp(1402) = dp(1)*p(881) + p(1)*dp(881) - dp(1238)
dp(1403) = dp(3)*p(576) + p(3)*dp(576) - dp(1244)
dp(1404) = dp(39)*p(188) + p(39)*dp(188) - dp(1246)
dp(1405) = dp(1)*p(884) + p(1)*dp(884) - dp(1245) - dp(1404)
dp(1406) = dp(2)*p(869) + p(2)*dp(869) - dp(1210)
dp(1407) = dp(1)*p(885) + p(1)*dp(885) - dp(1247) - dp(1405)
dp(1408) = dp(7)*p(536) + p(7)*dp(536) - dp(1246)
dp(1409) = dp(1)*p(887) + p(1)*dp(887) - dp(1249) - dp(1407)
dp(1410) = dp(49)*p(173) + p(49)*dp(173) - dp(1244)
dp(1411) = dp(6)*p(574) + p(6)*dp(574) - dp(1215)
dp(1412) = dp(1)*p(890) + p(1)*dp(890) - dp(1255) - dp(1253)
dp(1413) = dp(22)*p(285) + p(22)*dp(285) - dp(1215)
dp(1414) = dp(45)*p(183) + p(45)*dp(183) - dp(1190)
dp(1415) = dp(1)*p(893) + p(1)*dp(893) - dp(1268) - dp(1266)
dp(1416) = dp(8)*p(518) + p(8)*dp(518) - dp(1190)
dp(1417) = dp(41)*p(187) + p(41)*dp(187) - dp(1240)
dp(1418) = dp(1)*p(896) + p(1)*dp(896) - dp(1281) - dp(1279)
dp(1419) = dp(2)*p(881) + p(2)*dp(881) - dp(1240)
dp(1420) = dp(14)*p(308) + p(14)*dp(308)
dp(1421) = dp(4)*p(547) + p(4)*dp(547) - dp(1106) - dp(1320)
dp(1422) = dp(1)*p(900) + p(1)*dp(900) - dp(1291)
dp(1423) = dp(14)*p(311) + p(14)*dp(311)
dp(1424) = dp(3)*p(579) + p(3)*dp(579) - dp(1294)
dp(1425) = dp(39)*p(191) + p(39)*dp(191) - dp(1296)
dp(1426) = dp(1)*p(903) + p(1)*dp(903) - dp(1295) - dp(1425)
dp(1427) = dp(1)*p(904) + p(1)*dp(904) - dp(1296)
dp(1428) = dp(1)*p(905) + p(1)*dp(905) - dp(1297) - dp(1426)
dp(1429) = dp(21)*p(323) + p(21)*dp(323) - dp(1294)
dp(1430) = dp(6)*p(576) + p(6)*dp(576) - dp(1262)
dp(1431) = dp(1)*p(908) + p(1)*dp(908) - dp(1302) - dp(1300)
dp(1432) = dp(50)*p(150) + p(50)*dp(150) - dp(1262)
dp(1433) = dp(45)*p(185) + p(45)*dp(185) - dp(1232)
dp(1434) = dp(1)*p(911) + p(1)*dp(911) - dp(1312) - dp(1310)
dp(1435) = dp(22)*p(296) + p(22)*dp(296) - dp(1232)
dp(1436) = dp(43)*p(187) + p(43)*dp(187) - dp(1242)
dp(1437) = dp(1)*p(915) + p(1)*dp(915) - dp(1322) - dp(1320)
dp(1438) = dp(8)*p(532) + p(8)*dp(532) - dp(1242)
dp(1439) = dp(14)*p(321) + p(14)*dp(321)
dp(1440) = dp(4)*p(578) + p(4)*dp(578) - dp(1293)
dp(1441) = dp(2)*p(899) + p(2)*dp(899) - dp(1292) - dp(1440)
dp(1442) = dp(2)*p(900) + p(2)*dp(900) - dp(1293)
dp(1443) = dp(14)*p(339) + p(14)*dp(339)
dp(1444) = dp(3)*p(582) + p(3)*dp(582) - dp(1326)
dp(1445) = dp(2)*p(903) + p(2)*dp(903) - dp(1303) - dp(1299)
dp(1446) = dp(2)*p(904) + p(2)*dp(904) - dp(1304)
dp(1447) = dp(1)*p(919) + p(1)*dp(919) - dp(1327) - dp(1445)
dp(1448) = dp(7)*p(561) + p(7)*dp(561) - dp(1326)
dp(1449) = dp(6)*p(579) + p(6)*dp(579) - dp(1307)
dp(1450) = dp(1)*p(922) + p(1)*dp(922) - dp(1331) - dp(1329)
dp(1451) = dp(73)*p(102) + p(73)*dp(102) - dp(1307)
dp(1452) = dp(45)*p(188) + p(45)*dp(188) - dp(1276)
dp(1453) = dp(1)*p(925) + p(1)*dp(925) - dp(1338) - dp(1336)
dp(1454) = dp(50)*p(160) + p(50)*dp(160) - dp(1276)
dp(1455) = dp(14)*p(327) + p(14)*dp(327)
dp(1456) = dp(46)*p(187) + p(46)*dp(187) - dp(1243)
dp(1457) = dp(2)*p(915) + p(2)*dp(915) - dp(1324) - dp(1441)
dp(1458) = dp(22)*p(310) + p(22)*dp(310) - dp(1243)
dp(1459) = dp(14)*p(342) + p(14)*dp(342)
dp(1460) = dp(3)*p(585) + p(3)*dp(585) - dp(1342)
dp(1461) = dp(2)*p(919) + p(2)*dp(919) - dp(1332) - dp(1329)
dp(1462) = dp(1)*p(929) + p(1)*dp(929) - dp(1342)
dp(1463) = dp(6)*p(582) + p(6)*dp(582) - dp(1334)
dp(1464) = dp(1)*p(931) + p(1)*dp(931) - dp(1345) - dp(1343)
dp(1465) = dp(32)*p(195) + p(32)*dp(195) - dp(1334)
dp(1466) = dp(45)*p(191) + p(45)*dp(191) - dp(1318)
dp(1467) = dp(2)*p(925) + p(2)*dp(925) - dp(1340) - dp(1457)
dp(1468) = dp(81)*p(102) + p(81)*dp(102) - dp(1318)
dp(1469) = dp(14)*p(345) + p(14)*dp(345)
dp(1470) = dp(2)*p(928) + p(2)*dp(928) - dp(1344) - dp(1343)
dp(1471) = dp(2)*p(929) + p(2)*dp(929) - dp(1345)
dp(1472) = dp(6)*p(585) + p(6)*dp(585) - dp(1346)
dp(1473) = dp(2)*p(931) + p(2)*dp(931) - dp(1347) - dp(1467)
dp(1474) = dp(13)*p(347) + p(13)*dp(347) - dp(1346)
dp(1475) = dp(14)*p(347) + p(14)*dp(347)
dp(1476) = dp(3)*p(566) + p(3)*dp(566) - dp(1351)
dp(1477) = dp(7)*p(566) + p(7)*dp(566) - dp(1349)
dp(1478) = dp(49)*p(178) + p(49)*dp(178) - dp(1348)
dp(1479) = dp(39)*p(194) + p(39)*dp(194) - dp(1352)
dp(1480) = dp(3)*p(588) + p(3)*dp(588) - dp(1355)
dp(1481) = dp(1)*p(938) + p(1)*dp(938) - dp(1356)
dp(1482) = dp(2)*p(958) + p(2)*dp(958) - dp(1481)
dp(1483) = dp(1)*p(940) + p(1)*dp(940) - dp(1370)
dp(1484) = dp(8)*p(588) + p(8)*dp(588) - dp(1483)
dp(1485) = dp(1)*p(942) + p(1)*dp(942) - dp(1386)
dp(1486) = dp(22)*p(346) + p(22)*dp(346) - dp(1485)
dp(1487) = dp(1)*p(944) + p(1)*dp(944) - dp(1403)
dp(1488) = dp(50)*p(194) + p(50)*dp(194) - dp(1487)
dp(1489) = dp(1)*p(946) + p(1)*dp(946) - dp(1424)
dp(1490) = dp(101)*p(102) + p(101)*dp(102) - dp(1489)
dp(1491) = dp(6)*p(578) + p(6)*dp(578) - dp(1443)
dp(1492) = dp(1)*p(949) + p(1)*dp(949) - dp(1444)
dp(1493) = dp(49)*p(195) + p(49)*dp(195) - dp(1492)
dp(1494) = dp(8)*p(578) + p(8)*dp(578) - dp(1423)
dp(1495) = dp(1)*p(952) + p(1)*dp(952) - dp(1460)
dp(1496) = dp(21)*p(347) + p(21)*dp(347) - dp(1495)
dp(1497) = dp(50)*p(187) + p(50)*dp(187) - dp(1420)
dp(1498) = dp(1)*p(955) + p(1)*dp(955) - dp(1470)
dp(1499) = dp(7)*p(589) + p(7)*dp(589) - dp(1498)
dp(1500) = dp(45)*p(195) + p(45)*dp(195) - dp(1455)
dp(1501) = dp(2)*p(955) + p(2)*dp(955) - dp(1472)
dp(1502) = dp(1)*p(959) + p(1)*dp(959) - dp(1501)
dp(1503) = dp(6)*p(589) + p(6)*dp(589) - dp(1475)
dp(1504) = dp(1)*p(958) + p(1)*dp(958) - dp(1480)
dp(1505) = dp(2)*p(959) + p(2)*dp(959) - dp(1503)

do j=1,1506
  dbm1(j)=dp(j-1)
enddo

dbm2(1)=dbm1(4)
dbm2(2)=dbm1(10)
dbm2(3)=dbm1(11)
dbm2(4)=dbm1(12)
dbm2(5)=dbm1(13)
dbm2(6)=dbm1(15)
dbm2(7)=dbm1(16)
dbm2(8)=dbm1(24)
dbm2(9)=dbm1(25)
dbm2(10)=dbm1(26)
dbm2(11)=dbm1(27)
dbm2(12)=dbm1(28)
dbm2(13)=dbm1(29)
dbm2(14)=dbm1(30)
dbm2(15)=dbm1(31)
dbm2(16)=dbm1(32)
dbm2(17)=dbm1(34)
dbm2(18)=dbm1(35)
dbm2(19)=dbm1(36)
dbm2(20)=dbm1(37)
dbm2(21)=dbm1(39)
dbm2(22)=dbm1(40)
dbm2(23)=dbm1(41)
dbm2(24)=dbm1(52)
dbm2(25)=dbm1(53)
dbm2(26)=dbm1(54)
dbm2(27)=dbm1(55)
dbm2(28)=dbm1(56)
dbm2(29)=dbm1(57)
dbm2(30)=dbm1(58)
dbm2(31)=dbm1(59)
dbm2(32)=dbm1(60)
dbm2(33)=dbm1(61)
dbm2(34)=dbm1(62)
dbm2(35)=dbm1(63)
dbm2(36)=dbm1(64)
dbm2(37)=dbm1(65)
dbm2(38)=dbm1(66)
dbm2(39)=dbm1(67)
dbm2(40)=dbm1(68)
dbm2(41)=dbm1(69)
dbm2(42)=dbm1(70)
dbm2(43)=dbm1(71)
dbm2(44)=dbm1(72)
dbm2(45)=dbm1(73)
dbm2(46)=dbm1(75)
dbm2(47)=dbm1(76)
dbm2(48)=dbm1(77)
dbm2(49)=dbm1(78)
dbm2(50)=dbm1(79)
dbm2(51)=dbm1(81)
dbm2(52)=dbm1(83)
dbm2(53)=dbm1(84)
dbm2(54)=dbm1(85)
dbm2(55)=dbm1(86)
dbm2(56)=dbm1(87)
dbm2(57)=dbm1(89)
dbm2(58)=dbm1(90)
dbm2(59)=dbm1(91)
dbm2(60)=dbm1(104)
dbm2(61)=dbm1(105)
dbm2(62)=dbm1(106)
dbm2(63)=dbm1(107)
dbm2(64)=dbm1(108)
dbm2(65)=dbm1(109)
dbm2(66)=dbm1(110)
dbm2(67)=dbm1(111)
dbm2(68)=dbm1(112)
dbm2(69)=dbm1(113)
dbm2(70)=dbm1(114)
dbm2(71)=dbm1(115)
dbm2(72)=dbm1(116)
dbm2(73)=dbm1(117)
dbm2(74)=dbm1(118)
dbm2(75)=dbm1(119)
dbm2(76)=dbm1(120)
dbm2(77)=dbm1(121)
dbm2(78)=dbm1(122)
dbm2(79)=dbm1(123)
dbm2(80)=dbm1(124)
dbm2(81)=dbm1(125)
dbm2(82)=dbm1(126)
dbm2(83)=dbm1(127)
dbm2(84)=dbm1(128)
dbm2(85)=dbm1(129)
dbm2(86)=dbm1(130)
dbm2(87)=dbm1(131)
dbm2(88)=dbm1(132)
dbm2(89)=dbm1(133)
dbm2(90)=dbm1(134)
dbm2(91)=dbm1(135)
dbm2(92)=dbm1(136)
dbm2(93)=dbm1(137)
dbm2(94)=dbm1(138)
dbm2(95)=dbm1(139)
dbm2(96)=dbm1(140)
dbm2(97)=dbm1(141)
dbm2(98)=dbm1(142)
dbm2(99)=dbm1(143)
dbm2(100)=dbm1(144)
dbm2(101)=dbm1(145)
dbm2(102)=dbm1(146)
dbm2(103)=dbm1(147)
dbm2(104)=dbm1(148)
dbm2(105)=dbm1(149)
dbm2(106)=dbm1(150)
dbm2(107)=dbm1(152)
dbm2(108)=dbm1(153)
dbm2(109)=dbm1(154)
dbm2(110)=dbm1(155)
dbm2(111)=dbm1(156)
dbm2(112)=dbm1(157)
dbm2(113)=dbm1(158)
dbm2(114)=dbm1(160)
dbm2(115)=dbm1(162)
dbm2(116)=dbm1(163)
dbm2(117)=dbm1(164)
dbm2(118)=dbm1(165)
dbm2(119)=dbm1(166)
dbm2(120)=dbm1(167)
dbm2(121)=dbm1(169)
dbm2(122)=dbm1(170)
dbm2(123)=dbm1(172)
dbm2(124)=dbm1(173)
dbm2(125)=dbm1(174)
dbm2(126)=dbm1(175)
dbm2(127)=dbm1(176)
dbm2(128)=dbm1(178)
dbm2(129)=dbm1(179)
dbm2(130)=dbm1(180)
dbm2(131)=dbm1(181)
dbm2(132)=dbm1(197)
dbm2(133)=dbm1(198)
dbm2(134)=dbm1(199)
dbm2(135)=dbm1(200)
dbm2(136)=dbm1(201)
dbm2(137)=dbm1(202)
dbm2(138)=dbm1(203)
dbm2(139)=dbm1(204)
dbm2(140)=dbm1(205)
dbm2(141)=dbm1(206)
dbm2(142)=dbm1(207)
dbm2(143)=dbm1(208)
dbm2(144)=dbm1(209)
dbm2(145)=dbm1(210)
dbm2(146)=dbm1(211)
dbm2(147)=dbm1(212)
dbm2(148)=dbm1(213)
dbm2(149)=dbm1(214)
dbm2(150)=dbm1(215)
dbm2(151)=dbm1(216)
dbm2(152)=dbm1(217)
dbm2(153)=dbm1(218)
dbm2(154)=dbm1(219)
dbm2(155)=dbm1(220)
dbm2(156)=dbm1(221)
dbm2(157)=dbm1(222)
dbm2(158)=dbm1(223)
dbm2(159)=dbm1(224)
dbm2(160)=dbm1(225)
dbm2(161)=dbm1(226)
dbm2(162)=dbm1(227)
dbm2(163)=dbm1(228)
dbm2(164)=dbm1(229)
dbm2(165)=dbm1(230)
dbm2(166)=dbm1(231)
dbm2(167)=dbm1(232)
dbm2(168)=dbm1(233)
dbm2(169)=dbm1(234)
dbm2(170)=dbm1(235)
dbm2(171)=dbm1(236)
dbm2(172)=dbm1(237)
dbm2(173)=dbm1(238)
dbm2(174)=dbm1(239)
dbm2(175)=dbm1(240)
dbm2(176)=dbm1(241)
dbm2(177)=dbm1(242)
dbm2(178)=dbm1(243)
dbm2(179)=dbm1(244)
dbm2(180)=dbm1(245)
dbm2(181)=dbm1(246)
dbm2(182)=dbm1(247)
dbm2(183)=dbm1(248)
dbm2(184)=dbm1(249)
dbm2(185)=dbm1(250)
dbm2(186)=dbm1(251)
dbm2(187)=dbm1(252)
dbm2(188)=dbm1(253)
dbm2(189)=dbm1(254)
dbm2(190)=dbm1(255)
dbm2(191)=dbm1(256)
dbm2(192)=dbm1(257)
dbm2(193)=dbm1(258)
dbm2(194)=dbm1(259)
dbm2(195)=dbm1(260)
dbm2(196)=dbm1(261)
dbm2(197)=dbm1(262)
dbm2(198)=dbm1(263)
dbm2(199)=dbm1(264)
dbm2(200)=dbm1(265)
dbm2(201)=dbm1(266)
dbm2(202)=dbm1(267)
dbm2(203)=dbm1(268)
dbm2(204)=dbm1(269)
dbm2(205)=dbm1(270)
dbm2(206)=dbm1(271)
dbm2(207)=dbm1(272)
dbm2(208)=dbm1(273)
dbm2(209)=dbm1(274)
dbm2(210)=dbm1(275)
dbm2(211)=dbm1(276)
dbm2(212)=dbm1(277)
dbm2(213)=dbm1(278)
dbm2(214)=dbm1(279)
dbm2(215)=dbm1(280)
dbm2(216)=dbm1(281)
dbm2(217)=dbm1(282)
dbm2(218)=dbm1(283)
dbm2(219)=dbm1(284)
dbm2(220)=dbm1(285)
dbm2(221)=dbm1(287)
dbm2(222)=dbm1(288)
dbm2(223)=dbm1(289)
dbm2(224)=dbm1(290)
dbm2(225)=dbm1(291)
dbm2(226)=dbm1(292)
dbm2(227)=dbm1(293)
dbm2(228)=dbm1(294)
dbm2(229)=dbm1(296)
dbm2(230)=dbm1(298)
dbm2(231)=dbm1(299)
dbm2(232)=dbm1(300)
dbm2(233)=dbm1(301)
dbm2(234)=dbm1(302)
dbm2(235)=dbm1(303)
dbm2(236)=dbm1(304)
dbm2(237)=dbm1(306)
dbm2(238)=dbm1(307)
dbm2(239)=dbm1(309)
dbm2(240)=dbm1(310)
dbm2(241)=dbm1(312)
dbm2(242)=dbm1(313)
dbm2(243)=dbm1(314)
dbm2(244)=dbm1(315)
dbm2(245)=dbm1(316)
dbm2(246)=dbm1(317)
dbm2(247)=dbm1(319)
dbm2(248)=dbm1(320)
dbm2(249)=dbm1(322)
dbm2(250)=dbm1(323)
dbm2(251)=dbm1(324)
dbm2(252)=dbm1(325)
dbm2(253)=dbm1(326)
dbm2(254)=dbm1(328)
dbm2(255)=dbm1(329)
dbm2(256)=dbm1(330)
dbm2(257)=dbm1(331)
dbm2(258)=dbm1(349)
dbm2(259)=dbm1(350)
dbm2(260)=dbm1(351)
dbm2(261)=dbm1(352)
dbm2(262)=dbm1(353)
dbm2(263)=dbm1(354)
dbm2(264)=dbm1(355)
dbm2(265)=dbm1(356)
dbm2(266)=dbm1(357)
dbm2(267)=dbm1(358)
dbm2(268)=dbm1(359)
dbm2(269)=dbm1(360)
dbm2(270)=dbm1(361)
dbm2(271)=dbm1(362)
dbm2(272)=dbm1(363)
dbm2(273)=dbm1(364)
dbm2(274)=dbm1(365)
dbm2(275)=dbm1(366)
dbm2(276)=dbm1(367)
dbm2(277)=dbm1(368)
dbm2(278)=dbm1(369)
dbm2(279)=dbm1(370)
dbm2(280)=dbm1(371)
dbm2(281)=dbm1(372)
dbm2(282)=dbm1(373)
dbm2(283)=dbm1(374)
dbm2(284)=dbm1(375)
dbm2(285)=dbm1(376)
dbm2(286)=dbm1(377)
dbm2(287)=dbm1(378)
dbm2(288)=dbm1(379)
dbm2(289)=dbm1(380)
dbm2(290)=dbm1(381)
dbm2(291)=dbm1(382)
dbm2(292)=dbm1(383)
dbm2(293)=dbm1(384)
dbm2(294)=dbm1(385)
dbm2(295)=dbm1(386)
dbm2(296)=dbm1(387)
dbm2(297)=dbm1(388)
dbm2(298)=dbm1(389)
dbm2(299)=dbm1(390)
dbm2(300)=dbm1(391)
dbm2(301)=dbm1(392)
dbm2(302)=dbm1(393)
dbm2(303)=dbm1(394)
dbm2(304)=dbm1(395)
dbm2(305)=dbm1(396)
dbm2(306)=dbm1(397)
dbm2(307)=dbm1(398)
dbm2(308)=dbm1(399)
dbm2(309)=dbm1(400)
dbm2(310)=dbm1(401)
dbm2(311)=dbm1(402)
dbm2(312)=dbm1(403)
dbm2(313)=dbm1(404)
dbm2(314)=dbm1(405)
dbm2(315)=dbm1(406)
dbm2(316)=dbm1(407)
dbm2(317)=dbm1(408)
dbm2(318)=dbm1(409)
dbm2(319)=dbm1(410)
dbm2(320)=dbm1(411)
dbm2(321)=dbm1(412)
dbm2(322)=dbm1(413)
dbm2(323)=dbm1(414)
dbm2(324)=dbm1(415)
dbm2(325)=dbm1(416)
dbm2(326)=dbm1(417)
dbm2(327)=dbm1(418)
dbm2(328)=dbm1(419)
dbm2(329)=dbm1(420)
dbm2(330)=dbm1(421)
dbm2(331)=dbm1(422)
dbm2(332)=dbm1(423)
dbm2(333)=dbm1(424)
dbm2(334)=dbm1(425)
dbm2(335)=dbm1(426)
dbm2(336)=dbm1(427)
dbm2(337)=dbm1(428)
dbm2(338)=dbm1(429)
dbm2(339)=dbm1(430)
dbm2(340)=dbm1(431)
dbm2(341)=dbm1(432)
dbm2(342)=dbm1(433)
dbm2(343)=dbm1(434)
dbm2(344)=dbm1(435)
dbm2(345)=dbm1(436)
dbm2(346)=dbm1(437)
dbm2(347)=dbm1(438)
dbm2(348)=dbm1(439)
dbm2(349)=dbm1(440)
dbm2(350)=dbm1(441)
dbm2(351)=dbm1(442)
dbm2(352)=dbm1(443)
dbm2(353)=dbm1(444)
dbm2(354)=dbm1(445)
dbm2(355)=dbm1(446)
dbm2(356)=dbm1(447)
dbm2(357)=dbm1(448)
dbm2(358)=dbm1(449)
dbm2(359)=dbm1(450)
dbm2(360)=dbm1(451)
dbm2(361)=dbm1(452)
dbm2(362)=dbm1(453)
dbm2(363)=dbm1(454)
dbm2(364)=dbm1(455)
dbm2(365)=dbm1(456)
dbm2(366)=dbm1(457)
dbm2(367)=dbm1(458)
dbm2(368)=dbm1(459)
dbm2(369)=dbm1(460)
dbm2(370)=dbm1(461)
dbm2(371)=dbm1(462)
dbm2(372)=dbm1(463)
dbm2(373)=dbm1(464)
dbm2(374)=dbm1(465)
dbm2(375)=dbm1(466)
dbm2(376)=dbm1(467)
dbm2(377)=dbm1(468)
dbm2(378)=dbm1(469)
dbm2(379)=dbm1(470)
dbm2(380)=dbm1(471)
dbm2(381)=dbm1(472)
dbm2(382)=dbm1(473)
dbm2(383)=dbm1(474)
dbm2(384)=dbm1(475)
dbm2(385)=dbm1(476)
dbm2(386)=dbm1(477)
dbm2(387)=dbm1(478)
dbm2(388)=dbm1(479)
dbm2(389)=dbm1(480)
dbm2(390)=dbm1(481)
dbm2(391)=dbm1(482)
dbm2(392)=dbm1(483)
dbm2(393)=dbm1(484)
dbm2(394)=dbm1(485)
dbm2(395)=dbm1(486)
dbm2(396)=dbm1(487)
dbm2(397)=dbm1(488)
dbm2(398)=dbm1(489)
dbm2(399)=dbm1(490)
dbm2(400)=dbm1(491)
dbm2(401)=dbm1(492)
dbm2(402)=dbm1(493)
dbm2(403)=dbm1(494)
dbm2(404)=dbm1(495)
dbm2(405)=dbm1(496)
dbm2(406)=dbm1(497)
dbm2(407)=dbm1(498)
dbm2(408)=dbm1(499)
dbm2(409)=dbm1(500)
dbm2(410)=dbm1(501)
dbm2(411)=dbm1(502)
dbm2(412)=dbm1(503)
dbm2(413)=dbm1(504)
dbm2(414)=dbm1(505)
dbm2(415)=dbm1(507)
dbm2(416)=dbm1(508)
dbm2(417)=dbm1(509)
dbm2(418)=dbm1(510)
dbm2(419)=dbm1(511)
dbm2(420)=dbm1(512)
dbm2(421)=dbm1(513)
dbm2(422)=dbm1(514)
dbm2(423)=dbm1(515)
dbm2(424)=dbm1(516)
dbm2(425)=dbm1(518)
dbm2(426)=dbm1(520)
dbm2(427)=dbm1(521)
dbm2(428)=dbm1(522)
dbm2(429)=dbm1(523)
dbm2(430)=dbm1(524)
dbm2(431)=dbm1(525)
dbm2(432)=dbm1(526)
dbm2(433)=dbm1(527)
dbm2(434)=dbm1(529)
dbm2(435)=dbm1(530)
dbm2(436)=dbm1(532)
dbm2(437)=dbm1(534)
dbm2(438)=dbm1(535)
dbm2(439)=dbm1(536)
dbm2(440)=dbm1(537)
dbm2(441)=dbm1(538)
dbm2(442)=dbm1(539)
dbm2(443)=dbm1(540)
dbm2(444)=dbm1(541)
dbm2(445)=dbm1(543)
dbm2(446)=dbm1(544)
dbm2(447)=dbm1(546)
dbm2(448)=dbm1(547)
dbm2(449)=dbm1(548)
dbm2(450)=dbm1(550)
dbm2(451)=dbm1(551)
dbm2(452)=dbm1(552)
dbm2(453)=dbm1(553)
dbm2(454)=dbm1(554)
dbm2(455)=dbm1(555)
dbm2(456)=dbm1(557)
dbm2(457)=dbm1(558)
dbm2(458)=dbm1(560)
dbm2(459)=dbm1(561)
dbm2(460)=dbm1(562)
dbm2(461)=dbm1(563)
dbm2(462)=dbm1(564)
dbm2(463)=dbm1(566)
dbm2(464)=dbm1(567)
dbm2(465)=dbm1(568)
dbm2(466)=dbm1(569)
dbm2(467)=dbm1(570)
dbm2(468)=dbm1(591)
dbm2(469)=dbm1(592)
dbm2(470)=dbm1(593)
dbm2(471)=dbm1(594)
dbm2(472)=dbm1(595)
dbm2(473)=dbm1(596)
dbm2(474)=dbm1(597)
dbm2(475)=dbm1(598)
dbm2(476)=dbm1(599)
dbm2(477)=dbm1(600)
dbm2(478)=dbm1(601)
dbm2(479)=dbm1(602)
dbm2(480)=dbm1(603)
dbm2(481)=dbm1(604)
dbm2(482)=dbm1(605)
dbm2(483)=dbm1(606)
dbm2(484)=dbm1(607)
dbm2(485)=dbm1(608)
dbm2(486)=dbm1(609)
dbm2(487)=dbm1(610)
dbm2(488)=dbm1(611)
dbm2(489)=dbm1(612)
dbm2(490)=dbm1(613)
dbm2(491)=dbm1(614)
dbm2(492)=dbm1(615)
dbm2(493)=dbm1(616)
dbm2(494)=dbm1(617)
dbm2(495)=dbm1(618)
dbm2(496)=dbm1(619)
dbm2(497)=dbm1(620)
dbm2(498)=dbm1(621)
dbm2(499)=dbm1(622)
dbm2(500)=dbm1(623)
dbm2(501)=dbm1(624)
dbm2(502)=dbm1(625)
dbm2(503)=dbm1(626)
dbm2(504)=dbm1(627)
dbm2(505)=dbm1(628)
dbm2(506)=dbm1(629)
dbm2(507)=dbm1(630)
dbm2(508)=dbm1(631)
dbm2(509)=dbm1(632)
dbm2(510)=dbm1(633)
dbm2(511)=dbm1(634)
dbm2(512)=dbm1(635)
dbm2(513)=dbm1(636)
dbm2(514)=dbm1(637)
dbm2(515)=dbm1(638)
dbm2(516)=dbm1(639)
dbm2(517)=dbm1(640)
dbm2(518)=dbm1(641)
dbm2(519)=dbm1(642)
dbm2(520)=dbm1(643)
dbm2(521)=dbm1(644)
dbm2(522)=dbm1(645)
dbm2(523)=dbm1(646)
dbm2(524)=dbm1(647)
dbm2(525)=dbm1(648)
dbm2(526)=dbm1(649)
dbm2(527)=dbm1(650)
dbm2(528)=dbm1(651)
dbm2(529)=dbm1(652)
dbm2(530)=dbm1(653)
dbm2(531)=dbm1(654)
dbm2(532)=dbm1(655)
dbm2(533)=dbm1(656)
dbm2(534)=dbm1(657)
dbm2(535)=dbm1(658)
dbm2(536)=dbm1(659)
dbm2(537)=dbm1(660)
dbm2(538)=dbm1(661)
dbm2(539)=dbm1(662)
dbm2(540)=dbm1(663)
dbm2(541)=dbm1(664)
dbm2(542)=dbm1(665)
dbm2(543)=dbm1(666)
dbm2(544)=dbm1(667)
dbm2(545)=dbm1(668)
dbm2(546)=dbm1(669)
dbm2(547)=dbm1(670)
dbm2(548)=dbm1(671)
dbm2(549)=dbm1(672)
dbm2(550)=dbm1(673)
dbm2(551)=dbm1(674)
dbm2(552)=dbm1(675)
dbm2(553)=dbm1(676)
dbm2(554)=dbm1(677)
dbm2(555)=dbm1(678)
dbm2(556)=dbm1(679)
dbm2(557)=dbm1(680)
dbm2(558)=dbm1(681)
dbm2(559)=dbm1(682)
dbm2(560)=dbm1(683)
dbm2(561)=dbm1(684)
dbm2(562)=dbm1(685)
dbm2(563)=dbm1(686)
dbm2(564)=dbm1(687)
dbm2(565)=dbm1(688)
dbm2(566)=dbm1(689)
dbm2(567)=dbm1(690)
dbm2(568)=dbm1(691)
dbm2(569)=dbm1(692)
dbm2(570)=dbm1(693)
dbm2(571)=dbm1(694)
dbm2(572)=dbm1(695)
dbm2(573)=dbm1(696)
dbm2(574)=dbm1(697)
dbm2(575)=dbm1(698)
dbm2(576)=dbm1(699)
dbm2(577)=dbm1(700)
dbm2(578)=dbm1(701)
dbm2(579)=dbm1(702)
dbm2(580)=dbm1(703)
dbm2(581)=dbm1(704)
dbm2(582)=dbm1(705)
dbm2(583)=dbm1(706)
dbm2(584)=dbm1(707)
dbm2(585)=dbm1(708)
dbm2(586)=dbm1(709)
dbm2(587)=dbm1(710)
dbm2(588)=dbm1(711)
dbm2(589)=dbm1(712)
dbm2(590)=dbm1(713)
dbm2(591)=dbm1(714)
dbm2(592)=dbm1(715)
dbm2(593)=dbm1(716)
dbm2(594)=dbm1(717)
dbm2(595)=dbm1(718)
dbm2(596)=dbm1(719)
dbm2(597)=dbm1(720)
dbm2(598)=dbm1(721)
dbm2(599)=dbm1(722)
dbm2(600)=dbm1(723)
dbm2(601)=dbm1(724)
dbm2(602)=dbm1(725)
dbm2(603)=dbm1(726)
dbm2(604)=dbm1(727)
dbm2(605)=dbm1(728)
dbm2(606)=dbm1(729)
dbm2(607)=dbm1(730)
dbm2(608)=dbm1(731)
dbm2(609)=dbm1(732)
dbm2(610)=dbm1(733)
dbm2(611)=dbm1(734)
dbm2(612)=dbm1(735)
dbm2(613)=dbm1(736)
dbm2(614)=dbm1(737)
dbm2(615)=dbm1(738)
dbm2(616)=dbm1(739)
dbm2(617)=dbm1(740)
dbm2(618)=dbm1(741)
dbm2(619)=dbm1(742)
dbm2(620)=dbm1(743)
dbm2(621)=dbm1(744)
dbm2(622)=dbm1(745)
dbm2(623)=dbm1(746)
dbm2(624)=dbm1(747)
dbm2(625)=dbm1(748)
dbm2(626)=dbm1(749)
dbm2(627)=dbm1(750)
dbm2(628)=dbm1(751)
dbm2(629)=dbm1(752)
dbm2(630)=dbm1(753)
dbm2(631)=dbm1(754)
dbm2(632)=dbm1(755)
dbm2(633)=dbm1(756)
dbm2(634)=dbm1(757)
dbm2(635)=dbm1(758)
dbm2(636)=dbm1(759)
dbm2(637)=dbm1(760)
dbm2(638)=dbm1(761)
dbm2(639)=dbm1(762)
dbm2(640)=dbm1(763)
dbm2(641)=dbm1(764)
dbm2(642)=dbm1(765)
dbm2(643)=dbm1(766)
dbm2(644)=dbm1(767)
dbm2(645)=dbm1(768)
dbm2(646)=dbm1(769)
dbm2(647)=dbm1(770)
dbm2(648)=dbm1(771)
dbm2(649)=dbm1(772)
dbm2(650)=dbm1(773)
dbm2(651)=dbm1(774)
dbm2(652)=dbm1(775)
dbm2(653)=dbm1(776)
dbm2(654)=dbm1(777)
dbm2(655)=dbm1(778)
dbm2(656)=dbm1(779)
dbm2(657)=dbm1(780)
dbm2(658)=dbm1(781)
dbm2(659)=dbm1(782)
dbm2(660)=dbm1(783)
dbm2(661)=dbm1(784)
dbm2(662)=dbm1(785)
dbm2(663)=dbm1(786)
dbm2(664)=dbm1(787)
dbm2(665)=dbm1(788)
dbm2(666)=dbm1(789)
dbm2(667)=dbm1(790)
dbm2(668)=dbm1(791)
dbm2(669)=dbm1(792)
dbm2(670)=dbm1(793)
dbm2(671)=dbm1(794)
dbm2(672)=dbm1(795)
dbm2(673)=dbm1(796)
dbm2(674)=dbm1(797)
dbm2(675)=dbm1(798)
dbm2(676)=dbm1(799)
dbm2(677)=dbm1(800)
dbm2(678)=dbm1(801)
dbm2(679)=dbm1(802)
dbm2(680)=dbm1(803)
dbm2(681)=dbm1(804)
dbm2(682)=dbm1(805)
dbm2(683)=dbm1(806)
dbm2(684)=dbm1(807)
dbm2(685)=dbm1(808)
dbm2(686)=dbm1(809)
dbm2(687)=dbm1(810)
dbm2(688)=dbm1(811)
dbm2(689)=dbm1(812)
dbm2(690)=dbm1(813)
dbm2(691)=dbm1(814)
dbm2(692)=dbm1(815)
dbm2(693)=dbm1(816)
dbm2(694)=dbm1(817)
dbm2(695)=dbm1(818)
dbm2(696)=dbm1(819)
dbm2(697)=dbm1(820)
dbm2(698)=dbm1(821)
dbm2(699)=dbm1(822)
dbm2(700)=dbm1(823)
dbm2(701)=dbm1(824)
dbm2(702)=dbm1(825)
dbm2(703)=dbm1(826)
dbm2(704)=dbm1(827)
dbm2(705)=dbm1(828)
dbm2(706)=dbm1(829)
dbm2(707)=dbm1(830)
dbm2(708)=dbm1(831)
dbm2(709)=dbm1(832)
dbm2(710)=dbm1(833)
dbm2(711)=dbm1(834)
dbm2(712)=dbm1(835)
dbm2(713)=dbm1(836)
dbm2(714)=dbm1(837)
dbm2(715)=dbm1(838)
dbm2(716)=dbm1(839)
dbm2(717)=dbm1(840)
dbm2(718)=dbm1(841)
dbm2(719)=dbm1(842)
dbm2(720)=dbm1(843)
dbm2(721)=dbm1(844)
dbm2(722)=dbm1(845)
dbm2(723)=dbm1(846)
dbm2(724)=dbm1(847)
dbm2(725)=dbm1(848)
dbm2(726)=dbm1(849)
dbm2(727)=dbm1(850)
dbm2(728)=dbm1(851)
dbm2(729)=dbm1(853)
dbm2(730)=dbm1(854)
dbm2(731)=dbm1(855)
dbm2(732)=dbm1(856)
dbm2(733)=dbm1(857)
dbm2(734)=dbm1(858)
dbm2(735)=dbm1(859)
dbm2(736)=dbm1(860)
dbm2(737)=dbm1(861)
dbm2(738)=dbm1(862)
dbm2(739)=dbm1(863)
dbm2(740)=dbm1(865)
dbm2(741)=dbm1(867)
dbm2(742)=dbm1(868)
dbm2(743)=dbm1(869)
dbm2(744)=dbm1(870)
dbm2(745)=dbm1(871)
dbm2(746)=dbm1(872)
dbm2(747)=dbm1(873)
dbm2(748)=dbm1(874)
dbm2(749)=dbm1(875)
dbm2(750)=dbm1(876)
dbm2(751)=dbm1(878)
dbm2(752)=dbm1(879)
dbm2(753)=dbm1(881)
dbm2(754)=dbm1(883)
dbm2(755)=dbm1(884)
dbm2(756)=dbm1(885)
dbm2(757)=dbm1(886)
dbm2(758)=dbm1(887)
dbm2(759)=dbm1(888)
dbm2(760)=dbm1(889)
dbm2(761)=dbm1(890)
dbm2(762)=dbm1(891)
dbm2(763)=dbm1(893)
dbm2(764)=dbm1(894)
dbm2(765)=dbm1(896)
dbm2(766)=dbm1(897)
dbm2(767)=dbm1(899)
dbm2(768)=dbm1(900)
dbm2(769)=dbm1(902)
dbm2(770)=dbm1(903)
dbm2(771)=dbm1(904)
dbm2(772)=dbm1(905)
dbm2(773)=dbm1(906)
dbm2(774)=dbm1(907)
dbm2(775)=dbm1(908)
dbm2(776)=dbm1(909)
dbm2(777)=dbm1(911)
dbm2(778)=dbm1(912)
dbm2(779)=dbm1(914)
dbm2(780)=dbm1(915)
dbm2(781)=dbm1(916)
dbm2(782)=dbm1(918)
dbm2(783)=dbm1(919)
dbm2(784)=dbm1(920)
dbm2(785)=dbm1(921)
dbm2(786)=dbm1(922)
dbm2(787)=dbm1(923)
dbm2(788)=dbm1(925)
dbm2(789)=dbm1(926)
dbm2(790)=dbm1(928)
dbm2(791)=dbm1(929)
dbm2(792)=dbm1(930)
dbm2(793)=dbm1(931)
dbm2(794)=dbm1(932)
dbm2(795)=dbm1(934)
dbm2(796)=dbm1(935)
dbm2(797)=dbm1(936)
dbm2(798)=dbm1(937)
dbm2(799)=dbm1(938)
dbm2(800)=dbm1(961)
dbm2(801)=dbm1(962)
dbm2(802)=dbm1(963)
dbm2(803)=dbm1(964)
dbm2(804)=dbm1(965)
dbm2(805)=dbm1(966)
dbm2(806)=dbm1(967)
dbm2(807)=dbm1(968)
dbm2(808)=dbm1(969)
dbm2(809)=dbm1(970)
dbm2(810)=dbm1(971)
dbm2(811)=dbm1(972)
dbm2(812)=dbm1(973)
dbm2(813)=dbm1(974)
dbm2(814)=dbm1(975)
dbm2(815)=dbm1(976)
dbm2(816)=dbm1(977)
dbm2(817)=dbm1(978)
dbm2(818)=dbm1(979)
dbm2(819)=dbm1(980)
dbm2(820)=dbm1(981)
dbm2(821)=dbm1(982)
dbm2(822)=dbm1(983)
dbm2(823)=dbm1(984)
dbm2(824)=dbm1(985)
dbm2(825)=dbm1(986)
dbm2(826)=dbm1(987)
dbm2(827)=dbm1(988)
dbm2(828)=dbm1(989)
dbm2(829)=dbm1(990)
dbm2(830)=dbm1(991)
dbm2(831)=dbm1(992)
dbm2(832)=dbm1(993)
dbm2(833)=dbm1(994)
dbm2(834)=dbm1(995)
dbm2(835)=dbm1(996)
dbm2(836)=dbm1(997)
dbm2(837)=dbm1(998)
dbm2(838)=dbm1(999)
dbm2(839)=dbm1(1000)
dbm2(840)=dbm1(1001)
dbm2(841)=dbm1(1002)
dbm2(842)=dbm1(1003)
dbm2(843)=dbm1(1004)
dbm2(844)=dbm1(1005)
dbm2(845)=dbm1(1006)
dbm2(846)=dbm1(1007)
dbm2(847)=dbm1(1008)
dbm2(848)=dbm1(1009)
dbm2(849)=dbm1(1010)
dbm2(850)=dbm1(1011)
dbm2(851)=dbm1(1012)
dbm2(852)=dbm1(1013)
dbm2(853)=dbm1(1014)
dbm2(854)=dbm1(1015)
dbm2(855)=dbm1(1016)
dbm2(856)=dbm1(1017)
dbm2(857)=dbm1(1018)
dbm2(858)=dbm1(1019)
dbm2(859)=dbm1(1020)
dbm2(860)=dbm1(1021)
dbm2(861)=dbm1(1022)
dbm2(862)=dbm1(1023)
dbm2(863)=dbm1(1024)
dbm2(864)=dbm1(1025)
dbm2(865)=dbm1(1026)
dbm2(866)=dbm1(1027)
dbm2(867)=dbm1(1028)
dbm2(868)=dbm1(1029)
dbm2(869)=dbm1(1030)
dbm2(870)=dbm1(1031)
dbm2(871)=dbm1(1032)
dbm2(872)=dbm1(1033)
dbm2(873)=dbm1(1034)
dbm2(874)=dbm1(1035)
dbm2(875)=dbm1(1036)
dbm2(876)=dbm1(1037)
dbm2(877)=dbm1(1038)
dbm2(878)=dbm1(1039)
dbm2(879)=dbm1(1040)
dbm2(880)=dbm1(1041)
dbm2(881)=dbm1(1042)
dbm2(882)=dbm1(1043)
dbm2(883)=dbm1(1044)
dbm2(884)=dbm1(1045)
dbm2(885)=dbm1(1046)
dbm2(886)=dbm1(1047)
dbm2(887)=dbm1(1048)
dbm2(888)=dbm1(1049)
dbm2(889)=dbm1(1050)
dbm2(890)=dbm1(1051)
dbm2(891)=dbm1(1052)
dbm2(892)=dbm1(1053)
dbm2(893)=dbm1(1054)
dbm2(894)=dbm1(1055)
dbm2(895)=dbm1(1056)
dbm2(896)=dbm1(1057)
dbm2(897)=dbm1(1058)
dbm2(898)=dbm1(1059)
dbm2(899)=dbm1(1060)
dbm2(900)=dbm1(1061)
dbm2(901)=dbm1(1062)
dbm2(902)=dbm1(1063)
dbm2(903)=dbm1(1064)
dbm2(904)=dbm1(1065)
dbm2(905)=dbm1(1066)
dbm2(906)=dbm1(1067)
dbm2(907)=dbm1(1068)
dbm2(908)=dbm1(1069)
dbm2(909)=dbm1(1070)
dbm2(910)=dbm1(1071)
dbm2(911)=dbm1(1072)
dbm2(912)=dbm1(1073)
dbm2(913)=dbm1(1074)
dbm2(914)=dbm1(1075)
dbm2(915)=dbm1(1076)
dbm2(916)=dbm1(1077)
dbm2(917)=dbm1(1078)
dbm2(918)=dbm1(1079)
dbm2(919)=dbm1(1080)
dbm2(920)=dbm1(1081)
dbm2(921)=dbm1(1082)
dbm2(922)=dbm1(1083)
dbm2(923)=dbm1(1084)
dbm2(924)=dbm1(1085)
dbm2(925)=dbm1(1086)
dbm2(926)=dbm1(1087)
dbm2(927)=dbm1(1088)
dbm2(928)=dbm1(1089)
dbm2(929)=dbm1(1090)
dbm2(930)=dbm1(1091)
dbm2(931)=dbm1(1092)
dbm2(932)=dbm1(1093)
dbm2(933)=dbm1(1094)
dbm2(934)=dbm1(1095)
dbm2(935)=dbm1(1096)
dbm2(936)=dbm1(1097)
dbm2(937)=dbm1(1098)
dbm2(938)=dbm1(1099)
dbm2(939)=dbm1(1100)
dbm2(940)=dbm1(1101)
dbm2(941)=dbm1(1102)
dbm2(942)=dbm1(1103)
dbm2(943)=dbm1(1104)
dbm2(944)=dbm1(1105)
dbm2(945)=dbm1(1106)
dbm2(946)=dbm1(1107)
dbm2(947)=dbm1(1108)
dbm2(948)=dbm1(1109)
dbm2(949)=dbm1(1110)
dbm2(950)=dbm1(1111)
dbm2(951)=dbm1(1112)
dbm2(952)=dbm1(1113)
dbm2(953)=dbm1(1114)
dbm2(954)=dbm1(1115)
dbm2(955)=dbm1(1116)
dbm2(956)=dbm1(1117)
dbm2(957)=dbm1(1118)
dbm2(958)=dbm1(1119)
dbm2(959)=dbm1(1120)
dbm2(960)=dbm1(1121)
dbm2(961)=dbm1(1122)
dbm2(962)=dbm1(1123)
dbm2(963)=dbm1(1124)
dbm2(964)=dbm1(1125)
dbm2(965)=dbm1(1126)
dbm2(966)=dbm1(1127)
dbm2(967)=dbm1(1128)
dbm2(968)=dbm1(1129)
dbm2(969)=dbm1(1130)
dbm2(970)=dbm1(1131)
dbm2(971)=dbm1(1132)
dbm2(972)=dbm1(1133)
dbm2(973)=dbm1(1134)
dbm2(974)=dbm1(1135)
dbm2(975)=dbm1(1136)
dbm2(976)=dbm1(1137)
dbm2(977)=dbm1(1138)
dbm2(978)=dbm1(1139)
dbm2(979)=dbm1(1140)
dbm2(980)=dbm1(1141)
dbm2(981)=dbm1(1142)
dbm2(982)=dbm1(1143)
dbm2(983)=dbm1(1144)
dbm2(984)=dbm1(1145)
dbm2(985)=dbm1(1146)
dbm2(986)=dbm1(1147)
dbm2(987)=dbm1(1148)
dbm2(988)=dbm1(1149)
dbm2(989)=dbm1(1150)
dbm2(990)=dbm1(1151)
dbm2(991)=dbm1(1152)
dbm2(992)=dbm1(1153)
dbm2(993)=dbm1(1154)
dbm2(994)=dbm1(1155)
dbm2(995)=dbm1(1156)
dbm2(996)=dbm1(1157)
dbm2(997)=dbm1(1158)
dbm2(998)=dbm1(1159)
dbm2(999)=dbm1(1160)
dbm2(1000)=dbm1(1161)
dbm2(1001)=dbm1(1162)
dbm2(1002)=dbm1(1163)
dbm2(1003)=dbm1(1164)
dbm2(1004)=dbm1(1165)
dbm2(1005)=dbm1(1166)
dbm2(1006)=dbm1(1167)
dbm2(1007)=dbm1(1168)
dbm2(1008)=dbm1(1169)
dbm2(1009)=dbm1(1170)
dbm2(1010)=dbm1(1171)
dbm2(1011)=dbm1(1172)
dbm2(1012)=dbm1(1173)
dbm2(1013)=dbm1(1174)
dbm2(1014)=dbm1(1175)
dbm2(1015)=dbm1(1176)
dbm2(1016)=dbm1(1177)
dbm2(1017)=dbm1(1178)
dbm2(1018)=dbm1(1179)
dbm2(1019)=dbm1(1180)
dbm2(1020)=dbm1(1181)
dbm2(1021)=dbm1(1182)
dbm2(1022)=dbm1(1183)
dbm2(1023)=dbm1(1184)
dbm2(1024)=dbm1(1185)
dbm2(1025)=dbm1(1186)
dbm2(1026)=dbm1(1187)
dbm2(1027)=dbm1(1188)
dbm2(1028)=dbm1(1189)
dbm2(1029)=dbm1(1190)
dbm2(1030)=dbm1(1191)
dbm2(1031)=dbm1(1192)
dbm2(1032)=dbm1(1193)
dbm2(1033)=dbm1(1194)
dbm2(1034)=dbm1(1195)
dbm2(1035)=dbm1(1196)
dbm2(1036)=dbm1(1197)
dbm2(1037)=dbm1(1198)
dbm2(1038)=dbm1(1199)
dbm2(1039)=dbm1(1200)
dbm2(1040)=dbm1(1201)
dbm2(1041)=dbm1(1202)
dbm2(1042)=dbm1(1203)
dbm2(1043)=dbm1(1204)
dbm2(1044)=dbm1(1205)
dbm2(1045)=dbm1(1206)
dbm2(1046)=dbm1(1207)
dbm2(1047)=dbm1(1208)
dbm2(1048)=dbm1(1209)
dbm2(1049)=dbm1(1210)
dbm2(1050)=dbm1(1211)
dbm2(1051)=dbm1(1212)
dbm2(1052)=dbm1(1213)
dbm2(1053)=dbm1(1214)
dbm2(1054)=dbm1(1215)
dbm2(1055)=dbm1(1216)
dbm2(1056)=dbm1(1217)
dbm2(1057)=dbm1(1218)
dbm2(1058)=dbm1(1219)
dbm2(1059)=dbm1(1220)
dbm2(1060)=dbm1(1221)
dbm2(1061)=dbm1(1222)
dbm2(1062)=dbm1(1223)
dbm2(1063)=dbm1(1224)
dbm2(1064)=dbm1(1225)
dbm2(1065)=dbm1(1226)
dbm2(1066)=dbm1(1227)
dbm2(1067)=dbm1(1228)
dbm2(1068)=dbm1(1229)
dbm2(1069)=dbm1(1230)
dbm2(1070)=dbm1(1231)
dbm2(1071)=dbm1(1232)
dbm2(1072)=dbm1(1233)
dbm2(1073)=dbm1(1234)
dbm2(1074)=dbm1(1235)
dbm2(1075)=dbm1(1236)
dbm2(1076)=dbm1(1237)
dbm2(1077)=dbm1(1238)
dbm2(1078)=dbm1(1239)
dbm2(1079)=dbm1(1240)
dbm2(1080)=dbm1(1241)
dbm2(1081)=dbm1(1242)
dbm2(1082)=dbm1(1243)
dbm2(1083)=dbm1(1244)
dbm2(1084)=dbm1(1245)
dbm2(1085)=dbm1(1246)
dbm2(1086)=dbm1(1247)
dbm2(1087)=dbm1(1248)
dbm2(1088)=dbm1(1249)
dbm2(1089)=dbm1(1250)
dbm2(1090)=dbm1(1251)
dbm2(1091)=dbm1(1252)
dbm2(1092)=dbm1(1253)
dbm2(1093)=dbm1(1254)
dbm2(1094)=dbm1(1255)
dbm2(1095)=dbm1(1256)
dbm2(1096)=dbm1(1257)
dbm2(1097)=dbm1(1258)
dbm2(1098)=dbm1(1259)
dbm2(1099)=dbm1(1260)
dbm2(1100)=dbm1(1261)
dbm2(1101)=dbm1(1262)
dbm2(1102)=dbm1(1263)
dbm2(1103)=dbm1(1264)
dbm2(1104)=dbm1(1265)
dbm2(1105)=dbm1(1266)
dbm2(1106)=dbm1(1267)
dbm2(1107)=dbm1(1268)
dbm2(1108)=dbm1(1269)
dbm2(1109)=dbm1(1270)
dbm2(1110)=dbm1(1271)
dbm2(1111)=dbm1(1272)
dbm2(1112)=dbm1(1273)
dbm2(1113)=dbm1(1274)
dbm2(1114)=dbm1(1275)
dbm2(1115)=dbm1(1276)
dbm2(1116)=dbm1(1277)
dbm2(1117)=dbm1(1278)
dbm2(1118)=dbm1(1279)
dbm2(1119)=dbm1(1280)
dbm2(1120)=dbm1(1281)
dbm2(1121)=dbm1(1282)
dbm2(1122)=dbm1(1283)
dbm2(1123)=dbm1(1284)
dbm2(1124)=dbm1(1285)
dbm2(1125)=dbm1(1286)
dbm2(1126)=dbm1(1287)
dbm2(1127)=dbm1(1288)
dbm2(1128)=dbm1(1289)
dbm2(1129)=dbm1(1290)
dbm2(1130)=dbm1(1291)
dbm2(1131)=dbm1(1292)
dbm2(1132)=dbm1(1293)
dbm2(1133)=dbm1(1294)
dbm2(1134)=dbm1(1295)
dbm2(1135)=dbm1(1296)
dbm2(1136)=dbm1(1297)
dbm2(1137)=dbm1(1298)
dbm2(1138)=dbm1(1299)
dbm2(1139)=dbm1(1300)
dbm2(1140)=dbm1(1301)
dbm2(1141)=dbm1(1302)
dbm2(1142)=dbm1(1303)
dbm2(1143)=dbm1(1304)
dbm2(1144)=dbm1(1305)
dbm2(1145)=dbm1(1306)
dbm2(1146)=dbm1(1307)
dbm2(1147)=dbm1(1308)
dbm2(1148)=dbm1(1309)
dbm2(1149)=dbm1(1310)
dbm2(1150)=dbm1(1311)
dbm2(1151)=dbm1(1312)
dbm2(1152)=dbm1(1313)
dbm2(1153)=dbm1(1314)
dbm2(1154)=dbm1(1315)
dbm2(1155)=dbm1(1316)
dbm2(1156)=dbm1(1317)
dbm2(1157)=dbm1(1318)
dbm2(1158)=dbm1(1319)
dbm2(1159)=dbm1(1320)
dbm2(1160)=dbm1(1321)
dbm2(1161)=dbm1(1322)
dbm2(1162)=dbm1(1323)
dbm2(1163)=dbm1(1324)
dbm2(1164)=dbm1(1325)
dbm2(1165)=dbm1(1326)
dbm2(1166)=dbm1(1327)
dbm2(1167)=dbm1(1328)
dbm2(1168)=dbm1(1329)
dbm2(1169)=dbm1(1330)
dbm2(1170)=dbm1(1331)
dbm2(1171)=dbm1(1332)
dbm2(1172)=dbm1(1333)
dbm2(1173)=dbm1(1334)
dbm2(1174)=dbm1(1335)
dbm2(1175)=dbm1(1336)
dbm2(1176)=dbm1(1337)
dbm2(1177)=dbm1(1338)
dbm2(1178)=dbm1(1339)
dbm2(1179)=dbm1(1340)
dbm2(1180)=dbm1(1341)
dbm2(1181)=dbm1(1342)
dbm2(1182)=dbm1(1343)
dbm2(1183)=dbm1(1344)
dbm2(1184)=dbm1(1345)
dbm2(1185)=dbm1(1346)
dbm2(1186)=dbm1(1347)
dbm2(1187)=dbm1(1348)
dbm2(1188)=dbm1(1349)
dbm2(1189)=dbm1(1350)
dbm2(1190)=dbm1(1351)
dbm2(1191)=dbm1(1352)
dbm2(1192)=dbm1(1353)
dbm2(1193)=dbm1(1354)
dbm2(1194)=dbm1(1355)
dbm2(1195)=dbm1(1356)
dbm2(1196)=dbm1(1357)
dbm2(1197)=dbm1(1358)
dbm2(1198)=dbm1(1359)
dbm2(1199)=dbm1(1360)
dbm2(1200)=dbm1(1361)
dbm2(1201)=dbm1(1362)
dbm2(1202)=dbm1(1363)
dbm2(1203)=dbm1(1364)
dbm2(1204)=dbm1(1365)
dbm2(1205)=dbm1(1366)
dbm2(1206)=dbm1(1367)
dbm2(1207)=dbm1(1368)
dbm2(1208)=dbm1(1369)
dbm2(1209)=dbm1(1371)
dbm2(1210)=dbm1(1372)
dbm2(1211)=dbm1(1373)
dbm2(1212)=dbm1(1374)
dbm2(1213)=dbm1(1375)
dbm2(1214)=dbm1(1376)
dbm2(1215)=dbm1(1377)
dbm2(1216)=dbm1(1378)
dbm2(1217)=dbm1(1379)
dbm2(1218)=dbm1(1380)
dbm2(1219)=dbm1(1381)
dbm2(1220)=dbm1(1382)
dbm2(1221)=dbm1(1383)
dbm2(1222)=dbm1(1385)
dbm2(1223)=dbm1(1387)
dbm2(1224)=dbm1(1388)
dbm2(1225)=dbm1(1389)
dbm2(1226)=dbm1(1390)
dbm2(1227)=dbm1(1391)
dbm2(1228)=dbm1(1392)
dbm2(1229)=dbm1(1393)
dbm2(1230)=dbm1(1394)
dbm2(1231)=dbm1(1395)
dbm2(1232)=dbm1(1396)
dbm2(1233)=dbm1(1397)
dbm2(1234)=dbm1(1399)
dbm2(1235)=dbm1(1400)
dbm2(1236)=dbm1(1402)
dbm2(1237)=dbm1(1404)
dbm2(1238)=dbm1(1405)
dbm2(1239)=dbm1(1406)
dbm2(1240)=dbm1(1407)
dbm2(1241)=dbm1(1408)
dbm2(1242)=dbm1(1409)
dbm2(1243)=dbm1(1410)
dbm2(1244)=dbm1(1411)
dbm2(1245)=dbm1(1412)
dbm2(1246)=dbm1(1413)
dbm2(1247)=dbm1(1415)
dbm2(1248)=dbm1(1416)
dbm2(1249)=dbm1(1418)
dbm2(1250)=dbm1(1419)
dbm2(1251)=dbm1(1421)
dbm2(1252)=dbm1(1422)
dbm2(1253)=dbm1(1424)
dbm2(1254)=dbm1(1425)
dbm2(1255)=dbm1(1426)
dbm2(1256)=dbm1(1427)
dbm2(1257)=dbm1(1428)
dbm2(1258)=dbm1(1429)
dbm2(1259)=dbm1(1430)
dbm2(1260)=dbm1(1431)
dbm2(1261)=dbm1(1432)
dbm2(1262)=dbm1(1434)
dbm2(1263)=dbm1(1435)
dbm2(1264)=dbm1(1437)
dbm2(1265)=dbm1(1438)
dbm2(1266)=dbm1(1440)
dbm2(1267)=dbm1(1441)
dbm2(1268)=dbm1(1442)
dbm2(1269)=dbm1(1444)
dbm2(1270)=dbm1(1445)
dbm2(1271)=dbm1(1446)
dbm2(1272)=dbm1(1447)
dbm2(1273)=dbm1(1448)
dbm2(1274)=dbm1(1449)
dbm2(1275)=dbm1(1450)
dbm2(1276)=dbm1(1451)
dbm2(1277)=dbm1(1453)
dbm2(1278)=dbm1(1454)
dbm2(1279)=dbm1(1456)
dbm2(1280)=dbm1(1457)
dbm2(1281)=dbm1(1458)
dbm2(1282)=dbm1(1460)
dbm2(1283)=dbm1(1461)
dbm2(1284)=dbm1(1462)
dbm2(1285)=dbm1(1463)
dbm2(1286)=dbm1(1464)
dbm2(1287)=dbm1(1465)
dbm2(1288)=dbm1(1467)
dbm2(1289)=dbm1(1468)
dbm2(1290)=dbm1(1470)
dbm2(1291)=dbm1(1471)
dbm2(1292)=dbm1(1472)
dbm2(1293)=dbm1(1473)
dbm2(1294)=dbm1(1474)
dbm2(1295)=dbm1(1476)
dbm2(1296)=dbm1(1477)
dbm2(1297)=dbm1(1478)
dbm2(1298)=dbm1(1479)
dbm2(1299)=dbm1(1480)
dbm2(1300)=dbm1(1481)

dVdR(i)=dVdR(i) + dot_product(coeff,dbm2)

end do

end subroutine EvU12O10dVdR

!===========================================================================
! *def EvdRdX(coord,r,dvdr)
!  subtoutine of evaluating derivative of 6 nuclear distances w.r.t. xyz 
!  coord is the xyz coordinate 
!  r is the nuclear distances
!  drdx is the derivative of 6 nuclear distances w.r.t. xyz 
!===========================================================================
subroutine EvdRdX(coord,r,drdx)
integer, parameter:: dp=kind(0.d0)
real(dp) :: coord(12),r(6),drdx(6,12)

! Start to calculate the non-zero dRdX
! dr1dx
      dRdX(1,1)=(coord(1)-coord(4))/r(1)
      dRdX(1,2)=(coord(2)-coord(5))/r(1)
      dRdX(1,3)=(coord(3)-coord(6))/r(1)
      dRdX(1,4)=-dRdX(1,1)
      dRdX(1,5)=-dRdX(1,2)
      dRdX(1,6)=-dRdX(1,3)

! dr2dx
      dRdX(2,1)=(coord(1)-coord(7))/r(2)
      dRdX(2,2)=(coord(2)-coord(8))/r(2)
      dRdX(2,3)=(coord(3)-coord(9))/r(2)
      dRdX(2,7)=-dRdX(2,1)
      dRdX(2,8)=-dRdX(2,2)
      dRdX(2,9)=-dRdX(2,3)

! dr3dx
      dRdX(3,1)=(coord(1)-coord(10))/r(3)
      dRdX(3,2)=(coord(2)-coord(11))/r(3)
      dRdX(3,3)=(coord(3)-coord(12))/r(3)
      dRdX(3,10)=-dRdX(3,1)
      dRdX(3,11)=-dRdX(3,2)
      dRdX(3,12)=-dRdX(3,3)

! dr4dx
      dRdX(4,4)=(coord(4)-coord(7))/r(4)
      dRdX(4,5)=(coord(5)-coord(8))/r(4)
      dRdX(4,6)=(coord(6)-coord(9))/r(4)
      dRdX(4,7)=-dRdX(4,4)
      dRdX(4,8)=-dRdX(4,5)
      dRdX(4,9)=-dRdX(4,6)

! dr5dx
      dRdX(5,4)=(coord(4)-coord(10))/r(5)
      dRdX(5,5)=(coord(5)-coord(11))/r(5)
      dRdX(5,6)=(coord(6)-coord(12))/r(5)
      dRdX(5,10)=-dRdX(5,4)
      dRdX(5,11)=-dRdX(5,5)
      dRdX(5,12)=-dRdX(5,6)

! dr6dx
      dRdX(6,7)=(coord(7)-coord(10))/r(6)
      dRdX(6,8)=(coord(8)-coord(11))/r(6)
      dRdX(6,9)=(coord(9)-coord(12))/r(6)
      dRdX(6,10)=-dRdX(6,7)
      dRdX(6,11)=-dRdX(6,8)
      dRdX(6,12)=-dRdX(6,9)
! Finish the calculation of non-zero dRdX

      return

end subroutine EvdRdX

!===========================================================================
! *def Num_dtdx(xyz,dtdx)
!  subtoutine of evaluating derivative of hyperbolic function
!  coord is the xyz coordinate 
!  dtdx is the derivative of hyperbolic function
!===========================================================================
subroutine Num_dtdx(xyz,dtdx)
implicit none
integer, parameter:: dp=kind(0.d0)
integer :: i,j,k,m,n
real(dp) :: xyz(4,3)
real(dp) :: dtdx(12)
real(dp) :: xyz1(4,3), xyz2(4,3)
real(dp) :: volume1, volume2

  do i=1,12
    dtdx(i)=0.000
  enddo

  do i=1,4
  do j=1,3
    do m=1,4
    do n=1,3
      xyz1(m,n)=xyz(m,n)
      xyz2(m,n)=xyz(m,n)
    enddo
    enddo
    k=(i-1)*3+j
    xyz1(i,j)=xyz(i,j)+0.001
    call P_volume(xyz1,volume1)
    xyz2(i,j)=xyz(i,j)-0.001
    call P_volume(xyz2,volume2)
    dtdx(k)=(volume1-volume2)/0.002
  enddo
  enddo
end subroutine Num_dtdx

!===========================================================================
! *def P_volume(xyz,volume) 
!  subtoutine of evaluating hyperbolic function
!  coord is the xyz coordinate 
!  volume is the hyperbolic function
!===========================================================================
subroutine P_volume(xyz,volume)
implicit none
integer, parameter:: dp=kind(0.d0)
integer :: i,j,k
real(dp) :: volume
real(dp) :: xyz(4,3)
real(dp) :: rrel(3,3),cross(3)
  volume=0.0000
  do i=1,3
  do j=1,3
    rrel(i,j)=xyz(i,j)-xyz(4,j)
  enddo
  enddo
  cross(1)=rrel(2,2)*rrel(3,3)-rrel(2,3)*rrel(3,2)
  cross(2)=-rrel(2,1)*rrel(3,3)+rrel(2,3)*rrel(3,1)
  cross(3)=rrel(2,1)*rrel(3,2)-rrel(2,2)*rrel(3,1)
  do i=1,3
    volume=volume+rrel(1,i)*cross(i)
  enddo
  volume=tanh(15.4*volume)
end subroutine P_volume
!===========================================================================
! *def u13_dvdx(xyz,u13,du13dx)
!  main subtoutine of evaluating derivative of u13 w.r.t. the xyz coordinates
!  xyz is the coordinate
!  u13 is u13
!  du13dx is the gradient of u13
!===========================================================================
subroutine u13_dvdx(xyz,u13,du13dx)
implicit none
integer, parameter:: dp=kind(0.d0)   
integer :: i,j,k
real(dp) :: du13dx(4,3),xyz(4,3)
real(dp) :: u13
real(dp) :: coord(12), r(6)
real(dp) :: dudx(12)
real(dp) :: dvdx(12), dvdr(6), drdx(6,12)
real(dp) :: volume

! Initialize dVdX(12)
      do i=1,12
        dVdX(i)=0.0d0
      enddo

! Initialize dRdX(6,12)
      do i=1,6
        do j=1,12
          dRdX(i,j)=0.0d0
        enddo
      enddo

! Initialize dVdR(6)
      do i=1,6
        dVdR(i)=0.0d0
      enddo

! Assign xyz to coord
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        coord(k)=xyz(i,j)
      enddo
      enddo

! Compute distances r
r(1)=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r(2)=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r(3)=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r(4)=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r(5)=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r(6)=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

! Call EvdVdR to evaluate dVdR(6)
      Call EvU13O10dVdR(coord,r,dvdr)

! Call EvdRdX to evaluate dRdX(6,12)
      Call EvdRdX(coord,r,drdx)

! Calculate dVdX by using chain rule: dV/dXi=(dV/dRj)*(dRj/dXi), j=1 to 6
      do i=1,12
        do j=1,6
          dVdX(i)=dVdX(i) + dVdR(j)*dRdX(j,i)
        enddo
      enddo

      do i=1,12
        dudx(i)=dvdx(i)/627.509474
      enddo

! Assign dvdx back to du13dvdx
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        du13dx(i,j)=dudx(k)
      enddo
      enddo

      return

end subroutine u13_dvdx 

!===========================================================================
! *def EvU13O10dVdR(coord,r,dvdr)
!  subtoutine of evaluating derivative of u12 w.r.t. the 6 nuclear distances
!  coord is the xyz coordinate 
!  r is the nuclear distances
!  dvdr is the derivative of u13 w.r.t. the 6 nuclear distances
!===========================================================================
subroutine EvU13O10dVdR(coord,r,dvdr)
integer, parameter:: ddp=kind(0.d0)
integer :: i,j
real(ddp) :: coord(12),r(6),dvdr(6)
real(ddp) :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=3075
real(ddp) :: coeff(ncoeff)
real(ddp),dimension(0:32) :: rm
real(ddp),dimension(0:32) :: dm
real(ddp),dimension(0:3392) :: p
real(ddp),dimension(0:3392) :: dp
real(ddp) :: x(6),dmsdr(6,6)
real(ddp) :: dbm1(3393), dbm2(3075)

! Assign coefficients first
a1=1.2561573
a2=1.1028022
b1=1.75
b2=1.61
ra1=1.196078
ra2=0.9333333
rb1=1.196078
rb2=0.9333333
coeff(1)=-0.107805854956D+03
coeff(2)=0.556492382441D+04
coeff(3)=0.193434802916D+03
coeff(4)=-0.100413112592D+03
coeff(5)=-0.962913723109D+03
coeff(6)=0.258159989406D+03
coeff(7)=-0.594309510988D+02
coeff(8)=0.520242005173D+04
coeff(9)=-0.325756660001D+04
coeff(10)=0.155028183157D+04
coeff(11)=-0.260255253474D+04
coeff(12)=-0.769852905947D+04
coeff(13)=-0.422487957664D+04
coeff(14)=0.996340850356D+03
coeff(15)=-0.201351147418D+03
coeff(16)=-0.440848464034D+03
coeff(17)=0.378709216760D+04
coeff(18)=-0.201015182836D+04
coeff(19)=0.413325479981D+04
coeff(20)=0.977420605649D+03
coeff(21)=0.261126119687D+02
coeff(22)=0.424137062336D+04
coeff(23)=-0.130257967411D+04
coeff(24)=0.362748811845D+04
coeff(25)=0.910702300766D+04
coeff(26)=-0.853818568342D+04
coeff(27)=-0.402964477375D+04
coeff(28)=0.435320240770D+03
coeff(29)=0.146757593582D+04
coeff(30)=-0.261012671677D+04
coeff(31)=0.331796204693D+04
coeff(32)=-0.683541059650D+04
coeff(33)=0.174397680298D+04
coeff(34)=-0.146831465382D+04
coeff(35)=-0.531866907147D+04
coeff(36)=-0.315352428484D+04
coeff(37)=0.293780615249D+04
coeff(38)=0.543009742828D+04
coeff(39)=-0.109281730359D+04
coeff(40)=0.680089026261D+04
coeff(41)=0.586134744782D+04
coeff(42)=-0.192649448148D+04
coeff(43)=-0.196009392878D+04
coeff(44)=-0.221733872858D+04
coeff(45)=0.261076354681D+04
coeff(46)=0.350958320486D+04
coeff(47)=-0.725492063236D+04
coeff(48)=0.437475668292D+04
coeff(49)=-0.575222465416D+04
coeff(50)=0.523544545563D+04
coeff(51)=-0.677504733647D+04
coeff(52)=-0.595479172762D+04
coeff(53)=-0.978467077710D+04
coeff(54)=0.367036537086D+04
coeff(55)=-0.106628088225D+04
coeff(56)=-0.169786011672D+04
coeff(57)=0.731840470613D+04
coeff(58)=-0.217881043512D+04
coeff(59)=0.249813535804D+04
coeff(60)=-0.121358158841D+05
coeff(61)=0.587315627322D+03
coeff(62)=-0.112588193422D+05
coeff(63)=0.366443741831D+04
coeff(64)=-0.174656345670D+04
coeff(65)=-0.489330596378D+04
coeff(66)=-0.744617023483D+04
coeff(67)=0.260258849965D+04
coeff(68)=0.766626640423D+04
coeff(69)=-0.715425120454D+04
coeff(70)=0.178477512694D+04
coeff(71)=-0.523539693397D+04
coeff(72)=0.695288613120D+04
coeff(73)=-0.413313261058D+04
coeff(74)=0.768778023273D+04
coeff(75)=-0.648298681147D+04
coeff(76)=-0.971855162615D+03
coeff(77)=-0.261786514289D+04
coeff(78)=0.766713806339D+04
coeff(79)=-0.496827607375D+04
coeff(80)=0.478604062448D+04
coeff(81)=0.538645545516D+04
coeff(82)=-0.817662901213D+04
coeff(83)=0.568475291513D+04
coeff(84)=0.975304501171D+04
coeff(85)=-0.287070485044D+04
coeff(86)=0.195871548191D+04
coeff(87)=-0.282356694067D+04
coeff(88)=0.501334661052D+04
coeff(89)=0.103320853448D+05
coeff(90)=-0.613378598659D+04
coeff(91)=0.439531914634D+04
coeff(92)=-0.356568484445D+04
coeff(93)=-0.133234136221D+04
coeff(94)=0.201388847908D+04
coeff(95)=0.991982988395D+03
coeff(96)=-0.292683504915D+04
coeff(97)=0.333046176871D+04
coeff(98)=0.468142884976D+04
coeff(99)=0.296566086861D+04
coeff(100)=-0.913038723143D+04
coeff(101)=0.103228824928D+04
coeff(102)=-0.331738823198D+04
coeff(103)=0.582301539087D+03
coeff(104)=0.576779172857D+03
coeff(105)=0.517189552576D+04
coeff(106)=0.721242624197D+02
coeff(107)=0.276821240742D+04
coeff(108)=-0.485599635311D+04
coeff(109)=-0.124864038024D+03
coeff(110)=0.117483397565D+04
coeff(111)=0.249086866111D+03
coeff(112)=-0.316445055337D+03
coeff(113)=-0.299283762368D+04
coeff(114)=0.223301711435D+04
coeff(115)=0.131389743485D+04
coeff(116)=-0.382916965685D+04
coeff(117)=0.550752651717D+04
coeff(118)=-0.659518085813D+04
coeff(119)=0.472053660298D+04
coeff(120)=-0.780079561157D+04
coeff(121)=0.184901878700D+03
coeff(122)=0.264061490802D+04
coeff(123)=-0.129771772344D+04
coeff(124)=0.105086475123D+05
coeff(125)=0.132440226246D+03
coeff(126)=-0.330626294776D+04
coeff(127)=0.677809914060D+04
coeff(128)=-0.396533195638D+04
coeff(129)=-0.547422907894D+04
coeff(130)=-0.166675842740D+04
coeff(131)=0.100087439390D+04
coeff(132)=-0.167226435639D+05
coeff(133)=-0.344019824375D+04
coeff(134)=-0.290157920744D+03
coeff(135)=0.100531700656D+04
coeff(136)=-0.521312336969D+04
coeff(137)=0.779485039709D+04
coeff(138)=-0.272535477840D+04
coeff(139)=0.821518252484D+04
coeff(140)=0.125933647946D+05
coeff(141)=-0.563502583800D+03
coeff(142)=-0.546720959422D+03
coeff(143)=-0.119038392140D+05
coeff(144)=0.112696215519D+05
coeff(145)=-0.144432012129D+05
coeff(146)=0.839533759209D+03
coeff(147)=0.258853456422D+02
coeff(148)=0.136945760667D+05
coeff(149)=0.541250034911D+04
coeff(150)=-0.562836964249D+04
coeff(151)=-0.321945570780D+04
coeff(152)=0.180738230713D+04
coeff(153)=-0.514050452496D+04
coeff(154)=0.107969569798D+05
coeff(155)=-0.210734409400D+04
coeff(156)=0.791755929595D+03
coeff(157)=0.166708780882D+04
coeff(158)=0.313890348080D+02
coeff(159)=-0.496716993434D+04
coeff(160)=0.216253779217D+04
coeff(161)=0.325394759772D+04
coeff(162)=-0.102853700128D+05
coeff(163)=0.348283350173D+04
coeff(164)=-0.379526359077D+04
coeff(165)=0.448745720285D+04
coeff(166)=-0.392602418969D+04
coeff(167)=-0.929493835355D+04
coeff(168)=0.225866744943D+04
coeff(169)=-0.954364170265D+03
coeff(170)=-0.575931156204D+04
coeff(171)=0.841350791990D+03
coeff(172)=-0.660858556247D+04
coeff(173)=0.269801897969D+04
coeff(174)=-0.131429167515D+04
coeff(175)=0.435242013662D+04
coeff(176)=0.275914570173D+04
coeff(177)=-0.923975646121D+01
coeff(178)=0.146399645948D+05
coeff(179)=-0.524220145689D+04
coeff(180)=-0.264457809985D+04
coeff(181)=0.699675921073D+04
coeff(182)=0.337148753180D+04
coeff(183)=-0.105292055752D+05
coeff(184)=0.161021708980D+04
coeff(185)=0.831309307447D+03
coeff(186)=0.701965349716D+04
coeff(187)=0.295372599711D+04
coeff(188)=0.124195215561D+05
coeff(189)=-0.515151743495D+04
coeff(190)=-0.289975739423D+04
coeff(191)=-0.710478933714D+04
coeff(192)=-0.797019624127D+02
coeff(193)=-0.414712873298D+04
coeff(194)=-0.820076294612D+04
coeff(195)=0.399666969667D+04
coeff(196)=-0.146222798800D+04
coeff(197)=-0.340348128991D+04
coeff(198)=-0.965861882237D+02
coeff(199)=-0.311666311991D+04
coeff(200)=0.323083089884D+04
coeff(201)=-0.946283217606D+03
coeff(202)=-0.348855312076D+04
coeff(203)=0.228631194179D+04
coeff(204)=-0.449106214490D+03
coeff(205)=-0.672498152636D+02
coeff(206)=0.253664423823D+04
coeff(207)=-0.374873654012D+04
coeff(208)=-0.397050214251D+03
coeff(209)=-0.766757897677D+04
coeff(210)=0.405457142830D+03
coeff(211)=0.203412222135D+04
coeff(212)=0.118293756545D+04
coeff(213)=-0.312493852811D+04
coeff(214)=-0.476936785652D+04
coeff(215)=-0.202922841783D+04
coeff(216)=0.243379999089D+04
coeff(217)=0.352151429125D+04
coeff(218)=0.149933738827D+04
coeff(219)=-0.172113294992D+04
coeff(220)=-0.437766778899D+04
coeff(221)=-0.237423991738D+04
coeff(222)=0.254413392378D+04
coeff(223)=-0.359665480377D+03
coeff(224)=-0.666707777676D+04
coeff(225)=-0.325575259134D+04
coeff(226)=0.644735567268D+04
coeff(227)=0.257998835105D+04
coeff(228)=0.164090389148D+04
coeff(229)=-0.555772777942D+03
coeff(230)=-0.308398832034D+04
coeff(231)=-0.165965884572D+04
coeff(232)=0.981963769796D+04
coeff(233)=0.794025475702D+04
coeff(234)=-0.701902520620D+04
coeff(235)=0.351894951252D+03
coeff(236)=-0.563868506925D+04
coeff(237)=0.427832149093D+04
coeff(238)=0.261366412479D+04
coeff(239)=0.309089987127D+04
coeff(240)=0.286323876161D+04
coeff(241)=-0.235481870178D+04
coeff(242)=0.379177554378D+04
coeff(243)=0.478908110983D+04
coeff(244)=-0.210026897121D+04
coeff(245)=-0.132597157745D+04
coeff(246)=0.179569895337D+04
coeff(247)=0.318070051192D+04
coeff(248)=0.960507416584D+03
coeff(249)=0.505244563123D+04
coeff(250)=-0.292026768029D+04
coeff(251)=0.813322320306D+03
coeff(252)=0.113311573124D+04
coeff(253)=-0.445276250691D+04
coeff(254)=-0.674745426631D+04
coeff(255)=0.372414183187D+04
coeff(256)=-0.186437831344D+04
coeff(257)=-0.177550110678D+04
coeff(258)=-0.122210466174D+02
coeff(259)=-0.984844784985D+04
coeff(260)=-0.616804558993D+04
coeff(261)=0.695621482343D+04
coeff(262)=0.576544790814D+04
coeff(263)=0.645405082836D+04
coeff(264)=0.100733478455D+04
coeff(265)=0.143829905745D+04
coeff(266)=0.348235974021D+04
coeff(267)=-0.117882300464D+05
coeff(268)=-0.102152222171D+04
coeff(269)=0.102475729190D+05
coeff(270)=0.882153670199D+04
coeff(271)=-0.351910199592D+04
coeff(272)=-0.506938301030D+04
coeff(273)=0.535548505717D+04
coeff(274)=0.511995029741D+04
coeff(275)=0.471341436755D+04
coeff(276)=0.770532761675D+04
coeff(277)=0.238028739459D+04
coeff(278)=0.100768265568D+05
coeff(279)=0.198123464986D+04
coeff(280)=-0.130303589030D+05
coeff(281)=-0.439389428901D+04
coeff(282)=0.583340712936D+04
coeff(283)=0.530462094504D+04
coeff(284)=0.591193033623D+04
coeff(285)=-0.664089718831D+02
coeff(286)=-0.754163070070D+03
coeff(287)=0.333210856124D+04
coeff(288)=0.896174144519D+04
coeff(289)=0.115564069815D+05
coeff(290)=-0.960649263013D+03
coeff(291)=-0.591770325940D+04
coeff(292)=-0.694604725701D+04
coeff(293)=0.121627742272D+05
coeff(294)=-0.916591763387D+04
coeff(295)=0.599696004709D+03
coeff(296)=0.227961221925D+03
coeff(297)=-0.292648197440D+04
coeff(298)=0.469027185398D+04
coeff(299)=-0.863043022874D+04
coeff(300)=-0.579821316090D+04
coeff(301)=0.467595931517D+04
coeff(302)=0.386823495530D+04
coeff(303)=-0.452421697838D+04
coeff(304)=-0.711305318831D+03
coeff(305)=-0.450977699982D+04
coeff(306)=0.348599483089D+04
coeff(307)=-0.181466327006D+04
coeff(308)=0.324741922533D+04
coeff(309)=0.288574515362D+04
coeff(310)=-0.213282353252D+04
coeff(311)=-0.271454163222D+03
coeff(312)=0.241509371352D+04
coeff(313)=-0.129138476648D+04
coeff(314)=-0.327397001064D+04
coeff(315)=0.531138261779D+04
coeff(316)=-0.790705447704D+03
coeff(317)=0.215900010953D+04
coeff(318)=0.570444159462D+04
coeff(319)=-0.414041509505D+04
coeff(320)=-0.114860777723D+04
coeff(321)=0.690979866908D+04
coeff(322)=-0.203431603414D+04
coeff(323)=-0.123271047050D+05
coeff(324)=-0.801806086687D+03
coeff(325)=0.489991455472D+03
coeff(326)=-0.351812235040D+04
coeff(327)=-0.163488568805D+04
coeff(328)=-0.910142455268D+04
coeff(329)=-0.129455680708D+04
coeff(330)=0.385328367788D+04
coeff(331)=0.765097579182D+04
coeff(332)=0.318703046143D+04
coeff(333)=-0.159691866552D+04
coeff(334)=-0.342318706833D+04
coeff(335)=-0.999156540271D+04
coeff(336)=-0.522582186078D+04
coeff(337)=-0.199055986189D+04
coeff(338)=0.121903589022D+05
coeff(339)=0.457912364223D+03
coeff(340)=0.137753195774D+05
coeff(341)=-0.700796775791D+04
coeff(342)=-0.701841366467D+04
coeff(343)=0.737949598184D+04
coeff(344)=-0.374647692312D+04
coeff(345)=-0.180334840352D+04
coeff(346)=0.360256820856D+04
coeff(347)=-0.783058169425D+04
coeff(348)=0.549773392774D+04
coeff(349)=-0.972418203440D+04
coeff(350)=-0.879545397553D+04
coeff(351)=-0.338374445537D+04
coeff(352)=0.229807703766D+04
coeff(353)=0.129959425011D+04
coeff(354)=-0.292215696401D+04
coeff(355)=-0.786165993311D+04
coeff(356)=0.115130446144D+04
coeff(357)=-0.116271331660D+05
coeff(358)=0.507452785639D+04
coeff(359)=0.127791748619D+04
coeff(360)=0.113702500233D+04
coeff(361)=-0.121989538830D+04
coeff(362)=-0.409418849432D+04
coeff(363)=0.968009147926D+04
coeff(364)=-0.122620800929D+05
coeff(365)=-0.903257026874D+03
coeff(366)=0.771901317997D+03
coeff(367)=0.574072893465D+03
coeff(368)=-0.139571855118D+05
coeff(369)=-0.344066641805D+04
coeff(370)=0.149182817186D+04
coeff(371)=-0.474614973867D+04
coeff(372)=-0.440039636746D+04
coeff(373)=-0.203646165285D+04
coeff(374)=0.747973620420D+03
coeff(375)=-0.327331501414D+04
coeff(376)=-0.209504004354D+04
coeff(377)=0.864480648907D+04
coeff(378)=-0.160472325012D+04
coeff(379)=0.105222826678D+04
coeff(380)=0.324813977483D+04
coeff(381)=0.464161870216D+04
coeff(382)=0.192848890211D+04
coeff(383)=-0.571469394719D+03
coeff(384)=-0.362772377121D+04
coeff(385)=-0.633677399664D+04
coeff(386)=-0.533008920058D+04
coeff(387)=0.981406766016D+04
coeff(388)=0.734106707511D+04
coeff(389)=-0.220061794127D+04
coeff(390)=-0.119671448630D+04
coeff(391)=-0.177695877775D+04
coeff(392)=-0.319871416200D+04
coeff(393)=-0.162364660011D+03
coeff(394)=-0.519750323948D+04
coeff(395)=-0.172512792168D+04
coeff(396)=0.214456128923D+04
coeff(397)=0.900967485080D+04
coeff(398)=0.400882405844D+04
coeff(399)=0.213192890114D+04
coeff(400)=-0.587264688785D+04
coeff(401)=-0.326016787812D+04
coeff(402)=-0.259141339343D+04
coeff(403)=-0.449341783756D+04
coeff(404)=-0.610712178359D+02
coeff(405)=0.875833445985D+04
coeff(406)=0.116402371133D+04
coeff(407)=-0.160069009170D+04
coeff(408)=0.558411293881D+04
coeff(409)=-0.575078710957D+03
coeff(410)=0.250287793957D+03
coeff(411)=-0.157164971378D+02
coeff(412)=0.202194599765D+03
coeff(413)=-0.320951472298D+04
coeff(414)=0.489483718387D+04
coeff(415)=-0.157739291733D+04
coeff(416)=0.613466773446D+04
coeff(417)=-0.234782693399D+04
coeff(418)=-0.530677341710D+04
coeff(419)=-0.498874098015D+03
coeff(420)=-0.181942160903D+04
coeff(421)=-0.191920757991D+04
coeff(422)=-0.665426251681D+02
coeff(423)=-0.604431496801D+04
coeff(424)=-0.665096985136D+04
coeff(425)=0.473795854187D+04
coeff(426)=-0.392906895120D+04
coeff(427)=0.437113451267D+04
coeff(428)=0.745940117335D+04
coeff(429)=0.792554652285D+04
coeff(430)=0.471288257195D+03
coeff(431)=-0.440098288441D+04
coeff(432)=-0.856634152336D+02
coeff(433)=0.725112328074D+04
coeff(434)=0.184831707351D+04
coeff(435)=-0.898652549900D+03
coeff(436)=0.383895349557D+04
coeff(437)=-0.110342747033D+04
coeff(438)=0.420443701235D+04
coeff(439)=-0.984151260238D+04
coeff(440)=0.856329236011D+04
coeff(441)=0.544692751367D+03
coeff(442)=0.311983721722D+04
coeff(443)=-0.184961214930D+04
coeff(444)=0.374485423080D+04
coeff(445)=-0.116267342075D+05
coeff(446)=0.265267582132D+04
coeff(447)=0.373097069174D+04
coeff(448)=0.397343152033D+04
coeff(449)=-0.100574646960D+05
coeff(450)=-0.204500008905D+04
coeff(451)=-0.425640679841D+03
coeff(452)=-0.548362520617D+04
coeff(453)=0.465831258726D+04
coeff(454)=0.545468242256D+04
coeff(455)=0.153248782112D+04
coeff(456)=0.235964497787D+04
coeff(457)=0.967375565360D+03
coeff(458)=0.197886863766D+04
coeff(459)=-0.530386942561D+04
coeff(460)=-0.173772734583D+04
coeff(461)=0.350003096482D+03
coeff(462)=-0.334266697500D+04
coeff(463)=-0.760316096488D+02
coeff(464)=0.529192262132D+04
coeff(465)=0.560821550643D+04
coeff(466)=-0.163156133726D+04
coeff(467)=-0.211635427550D+04
coeff(468)=0.164393994416D+04
coeff(469)=-0.409116092794D+02
coeff(470)=-0.267291854454D+04
coeff(471)=-0.106587009389D+05
coeff(472)=0.622037982506D+04
coeff(473)=0.611403934736D+04
coeff(474)=0.159869686383D+04
coeff(475)=-0.116140335506D+05
coeff(476)=0.212918975708D+04
coeff(477)=0.156532694031D+03
coeff(478)=-0.699138630514D+03
coeff(479)=0.863460004145D+04
coeff(480)=0.225206485971D+04
coeff(481)=0.698803832451D+04
coeff(482)=0.141778956036D+04
coeff(483)=0.438010987475D+04
coeff(484)=0.318946851180D+04
coeff(485)=-0.320031186342D+04
coeff(486)=-0.246710244270D+04
coeff(487)=-0.807859559922D+04
coeff(488)=-0.238849278722D+04
coeff(489)=0.610917892595D+03
coeff(490)=0.782557519575D+04
coeff(491)=0.100209162980D+05
coeff(492)=0.282928641797D+04
coeff(493)=-0.113934042294D+05
coeff(494)=-0.556504143900D+04
coeff(495)=-0.404507944021D+04
coeff(496)=0.461636114906D+04
coeff(497)=0.551662065859D+04
coeff(498)=-0.190520348737D+04
coeff(499)=0.865922780710D+04
coeff(500)=0.456199010537D+04
coeff(501)=-0.524985380104D+04
coeff(502)=-0.529585468721D+04
coeff(503)=-0.848017804513D+04
coeff(504)=0.939332354249D+04
coeff(505)=0.933827322920D+04
coeff(506)=-0.470234562466D+04
coeff(507)=-0.403219744627D+04
coeff(508)=-0.975418841331D+04
coeff(509)=0.391443720377D+04
coeff(510)=0.683891168876D+03
coeff(511)=-0.251925677467D+04
coeff(512)=0.271139386401D+04
coeff(513)=0.367990253568D+04
coeff(514)=0.261012304015D+04
coeff(515)=0.153399778525D+03
coeff(516)=0.624230121787D+04
coeff(517)=-0.299046272458D+04
coeff(518)=0.125718754283D+04
coeff(519)=0.202129576680D+04
coeff(520)=-0.539619736835D+04
coeff(521)=0.152567208369D+04
coeff(522)=0.357142673430D+04
coeff(523)=-0.468052963933D+04
coeff(524)=0.406508401236D+04
coeff(525)=-0.110018018041D+01
coeff(526)=0.332935869890D+04
coeff(527)=0.336274078550D+04
coeff(528)=-0.521348222923D+04
coeff(529)=-0.138695069878D+04
coeff(530)=0.638975976646D+03
coeff(531)=-0.404384090170D+04
coeff(532)=0.150296784799D+04
coeff(533)=0.670600329633D+03
coeff(534)=0.910930394153D+03
coeff(535)=-0.784253707668D+04
coeff(536)=0.488983002390D+04
coeff(537)=-0.159993247503D+04
coeff(538)=-0.278515763215D+04
coeff(539)=-0.866203054620D+04
coeff(540)=0.357600624734D+04
coeff(541)=-0.755702075213D+04
coeff(542)=-0.108413177078D+04
coeff(543)=-0.253083124687D+03
coeff(544)=0.219159479112D+04
coeff(545)=0.135251232539D+04
coeff(546)=-0.268012532444D+04
coeff(547)=-0.478268653843D+04
coeff(548)=-0.281799705061D+03
coeff(549)=0.102025071098D+05
coeff(550)=0.254214645348D+04
coeff(551)=-0.731442052441D+04
coeff(552)=0.783932223551D+03
coeff(553)=-0.136201177337D+05
coeff(554)=0.773489867592D+04
coeff(555)=-0.661783943376D+04
coeff(556)=-0.152735567075D+04
coeff(557)=0.611968390277D+04
coeff(558)=0.893275817796D+03
coeff(559)=-0.134480171440D+04
coeff(560)=0.377210133454D+04
coeff(561)=0.553462891570D+04
coeff(562)=-0.480440462234D+04
coeff(563)=-0.699698714521D+04
coeff(564)=0.148017261796D+03
coeff(565)=-0.153964047013D+04
coeff(566)=-0.119606149687D+04
coeff(567)=0.677557687114D+04
coeff(568)=0.142452566985D+04
coeff(569)=-0.448453370198D+04
coeff(570)=-0.207455300192D+04
coeff(571)=0.800007744289D+04
coeff(572)=-0.311671062455D+03
coeff(573)=-0.100384946859D+04
coeff(574)=-0.432089884328D+04
coeff(575)=0.175387422469D+04
coeff(576)=0.531676532667D+04
coeff(577)=0.763661013983D+04
coeff(578)=-0.131055186640D+04
coeff(579)=-0.466758384958D+04
coeff(580)=-0.547918958318D+04
coeff(581)=0.645436043507D+04
coeff(582)=0.211647606214D+04
coeff(583)=0.283431692119D+04
coeff(584)=0.519250396573D+04
coeff(585)=0.886753353460D+03
coeff(586)=-0.105524171860D+05
coeff(587)=0.747317968705D+04
coeff(588)=0.284552041924D+04
coeff(589)=0.354329951765D+04
coeff(590)=-0.627365506391D+04
coeff(591)=-0.945053184258D+04
coeff(592)=-0.763908026006D+04
coeff(593)=0.912846768227D+04
coeff(594)=0.105563819637D+03
coeff(595)=0.239533569783D+03
coeff(596)=0.262395471362D+04
coeff(597)=-0.437738896210D+04
coeff(598)=0.106513369321D+05
coeff(599)=-0.245214257736D+03
coeff(600)=-0.207571727837D+04
coeff(601)=-0.629888508187D+04
coeff(602)=-0.441282731647D+04
coeff(603)=0.822347438984D+03
coeff(604)=0.267818768554D+04
coeff(605)=0.990183052076D+04
coeff(606)=0.173166610351D+05
coeff(607)=-0.472538913425D+04
coeff(608)=-0.688736832733D+04
coeff(609)=-0.282078776769D+04
coeff(610)=0.534853091597D+04
coeff(611)=-0.305820595119D+04
coeff(612)=-0.854243292263D+03
coeff(613)=0.104567205830D+05
coeff(614)=-0.257068050809D+04
coeff(615)=-0.212411210139D+04
coeff(616)=0.112611153158D+04
coeff(617)=-0.391523592919D+04
coeff(618)=0.242677687643D+04
coeff(619)=-0.159227059124D+04
coeff(620)=-0.615667957168D+04
coeff(621)=-0.662707627440D+04
coeff(622)=0.230504207607D+04
coeff(623)=0.315828497100D+04
coeff(624)=-0.648342940824D+04
coeff(625)=0.455575051858D+04
coeff(626)=0.145800972592D+04
coeff(627)=-0.279073472566D+04
coeff(628)=-0.292106498590D+04
coeff(629)=-0.472449254079D+04
coeff(630)=-0.102034796647D+04
coeff(631)=-0.339067930629D+04
coeff(632)=0.760783393282D+04
coeff(633)=-0.275743270627D+04
coeff(634)=-0.413015468570D+04
coeff(635)=-0.119612153698D+04
coeff(636)=0.886214723028D+04
coeff(637)=-0.437684947292D+04
coeff(638)=0.187681134406D+04
coeff(639)=-0.472771509100D+04
coeff(640)=0.214248446677D+04
coeff(641)=0.815398485566D+04
coeff(642)=0.307425085589D+04
coeff(643)=-0.166281061783D+04
coeff(644)=0.139126833380D+04
coeff(645)=0.730464094838D+03
coeff(646)=-0.673353923747D+04
coeff(647)=0.104491571244D+04
coeff(648)=-0.108357187980D+05
coeff(649)=-0.510571460248D+03
coeff(650)=-0.534143200711D+04
coeff(651)=-0.108357081465D+04
coeff(652)=0.692212078297D+04
coeff(653)=-0.328220902393D+04
coeff(654)=0.230385940559D+04
coeff(655)=-0.203034544268D+04
coeff(656)=-0.427649579650D+04
coeff(657)=-0.655505542391D+04
coeff(658)=-0.379140850200D+04
coeff(659)=0.283382428517D+04
coeff(660)=-0.467855577607D+04
coeff(661)=0.673033282704D+03
coeff(662)=0.639584583428D+04
coeff(663)=0.261943328455D+03
coeff(664)=0.215367771242D+04
coeff(665)=-0.459257213891D+04
coeff(666)=0.211517171096D+04
coeff(667)=0.302961365907D+04
coeff(668)=-0.323697443737D+03
coeff(669)=-0.409099545936D+04
coeff(670)=-0.816815742392D+03
coeff(671)=-0.192509302122D+04
coeff(672)=-0.858450465108D+04
coeff(673)=0.169993445359D+04
coeff(674)=-0.916532651536D+03
coeff(675)=0.687985768945D+04
coeff(676)=-0.887053192969D+04
coeff(677)=0.922143477453D+04
coeff(678)=0.113346023223D+05
coeff(679)=0.542708644029D+04
coeff(680)=-0.234260762613D+04
coeff(681)=-0.803920809982D+03
coeff(682)=0.396230878084D+04
coeff(683)=-0.436640904035D+04
coeff(684)=0.338577070966D+04
coeff(685)=-0.322043040794D+04
coeff(686)=0.200884056984D+04
coeff(687)=0.114711936775D+05
coeff(688)=0.172791070005D+04
coeff(689)=0.651789089802D+04
coeff(690)=-0.554811797954D+04
coeff(691)=0.316514891251D+04
coeff(692)=0.789658531588D+04
coeff(693)=0.733802195729D+04
coeff(694)=-0.770707753049D+03
coeff(695)=0.433639764932D+03
coeff(696)=-0.168373909630D+03
coeff(697)=0.116829516701D+04
coeff(698)=0.313305901413D+04
coeff(699)=-0.801825373574D+03
coeff(700)=0.670020516718D+04
coeff(701)=0.227108756524D+04
coeff(702)=-0.113724011779D+05
coeff(703)=-0.780354273824D+04
coeff(704)=0.207182621052D+04
coeff(705)=-0.446804197556D+03
coeff(706)=0.210597653540D+04
coeff(707)=-0.645281606368D+03
coeff(708)=0.151954866521D+04
coeff(709)=0.375192155267D+04
coeff(710)=-0.376696718032D+04
coeff(711)=0.621107320156D+04
coeff(712)=0.713558263832D+04
coeff(713)=-0.542053892297D+04
coeff(714)=-0.286557476753D+03
coeff(715)=-0.405688419539D+04
coeff(716)=0.109150355593D+04
coeff(717)=-0.237822982002D+04
coeff(718)=0.127856192923D+03
coeff(719)=0.254292276025D+04
coeff(720)=0.133724998223D+04
coeff(721)=-0.964615895757D+03
coeff(722)=-0.779681481279D+03
coeff(723)=-0.175014395815D+04
coeff(724)=-0.638673256553D+03
coeff(725)=-0.986311947402D+03
coeff(726)=-0.321348229949D+04
coeff(727)=0.503250518154D+03
coeff(728)=0.223899596957D+04
coeff(729)=0.270962426893D+04
coeff(730)=-0.599913094017D+03
coeff(731)=0.388467525003D+03
coeff(732)=-0.430442435991D+04
coeff(733)=0.265865777736D+04
coeff(734)=-0.350140710179D+04
coeff(735)=-0.303616816438D+03
coeff(736)=0.627955800705D+04
coeff(737)=0.238277609241D+03
coeff(738)=0.386155101263D+04
coeff(739)=0.270566325905D+04
coeff(740)=-0.827232549869D+04
coeff(741)=-0.322387901436D+03
coeff(742)=-0.459453233231D+03
coeff(743)=0.352631411905D+04
coeff(744)=-0.182137505393D+03
coeff(745)=0.515275625786D+04
coeff(746)=0.462595796809D+04
coeff(747)=-0.474029571434D+04
coeff(748)=-0.625981468858D+04
coeff(749)=-0.689469259164D+04
coeff(750)=-0.177505388688D+04
coeff(751)=0.914100397641D+04
coeff(752)=0.339735369683D+04
coeff(753)=-0.456349691849D+04
coeff(754)=-0.851353435472D+03
coeff(755)=0.394858777645D+04
coeff(756)=0.182362243362D+03
coeff(757)=0.520534498947D+04
coeff(758)=-0.814113513412D+04
coeff(759)=-0.177274196694D+04
coeff(760)=0.273466581089D+03
coeff(761)=0.331524171601D+04
coeff(762)=0.498306349233D+04
coeff(763)=-0.102467419378D+04
coeff(764)=0.581236108173D+03
coeff(765)=-0.301582879741D+04
coeff(766)=0.104433701401D+04
coeff(767)=-0.585108969635D+03
coeff(768)=-0.774725294492D+04
coeff(769)=-0.439514058041D+04
coeff(770)=-0.286196877509D+04
coeff(771)=-0.736656800865D+04
coeff(772)=0.303923566470D+03
coeff(773)=-0.967334193548D+04
coeff(774)=0.119872192234D+05
coeff(775)=0.173476435546D+04
coeff(776)=-0.224595771568D+04
coeff(777)=-0.872095484711D+04
coeff(778)=-0.103295574816D+04
coeff(779)=-0.951361262185D+03
coeff(780)=0.414803607935D+04
coeff(781)=0.197556282325D+04
coeff(782)=0.260107706235D+04
coeff(783)=0.125644165342D+04
coeff(784)=0.285487750641D+04
coeff(785)=-0.362810717590D+04
coeff(786)=0.546731302099D+03
coeff(787)=0.364639391725D+04
coeff(788)=0.173359459555D+03
coeff(789)=-0.791823016511D+02
coeff(790)=-0.180980735297D+04
coeff(791)=0.158664351829D+04
coeff(792)=-0.204144619172D+04
coeff(793)=-0.198433519839D+04
coeff(794)=-0.268904902638D+03
coeff(795)=0.668962212935D+04
coeff(796)=-0.232741398902D+04
coeff(797)=0.133824453394D+04
coeff(798)=0.227419148864D+04
coeff(799)=0.113705660248D+04
coeff(800)=-0.285638279647D+04
coeff(801)=-0.271572700567D+04
coeff(802)=0.238263817700D+04
coeff(803)=0.483200022816D+04
coeff(804)=-0.390782147567D+04
coeff(805)=-0.564439100772D+04
coeff(806)=0.645792646487D+04
coeff(807)=-0.259389919566D+04
coeff(808)=0.700631661681D+04
coeff(809)=0.220994865750D+04
coeff(810)=-0.537442958082D+04
coeff(811)=-0.129979626604D+05
coeff(812)=-0.349970251249D+04
coeff(813)=0.339827952231D+04
coeff(814)=-0.194465903189D+04
coeff(815)=-0.638621742840D+04
coeff(816)=0.820916050164D+04
coeff(817)=0.335629054765D+04
coeff(818)=-0.776108218461D+03
coeff(819)=-0.499923682955D+04
coeff(820)=-0.141990075613D+05
coeff(821)=-0.310810620384D+04
coeff(822)=-0.143493030106D+04
coeff(823)=-0.291660790881D+04
coeff(824)=0.109888867219D+04
coeff(825)=0.844309762256D+03
coeff(826)=0.184038096918D+04
coeff(827)=0.620179186042D+04
coeff(828)=0.570414585449D+04
coeff(829)=0.405416954774D+04
coeff(830)=-0.100723168224D+04
coeff(831)=-0.591705937405D+04
coeff(832)=0.362080911736D+04
coeff(833)=-0.163232241542D+04
coeff(834)=-0.103398441723D+05
coeff(835)=0.413099558467D+04
coeff(836)=-0.161823062999D+04
coeff(837)=-0.136344764328D+04
coeff(838)=0.585076949419D+04
coeff(839)=0.418139619576D+04
coeff(840)=0.668582948439D+04
coeff(841)=-0.102467876213D+04
coeff(842)=-0.763186118001D+04
coeff(843)=-0.463073970559D+04
coeff(844)=-0.151312889228D+04
coeff(845)=-0.131297688392D+04
coeff(846)=-0.542729000058D+04
coeff(847)=0.171881722410D+04
coeff(848)=0.679325315864D+04
coeff(849)=0.287596183545D+04
coeff(850)=0.601254798876D+04
coeff(851)=0.946792162420D+04
coeff(852)=-0.527084295119D+04
coeff(853)=-0.697650993545D+04
coeff(854)=0.119161529759D+04
coeff(855)=-0.615992308679D+04
coeff(856)=-0.812116840196D+04
coeff(857)=-0.722193171437D+04
coeff(858)=0.849588654398D+03
coeff(859)=-0.389067307675D+04
coeff(860)=0.719343194410D+03
coeff(861)=0.845879971174D+04
coeff(862)=-0.811212668024D+04
coeff(863)=-0.695743771305D+04
coeff(864)=-0.663663098450D+03
coeff(865)=0.354025562287D+04
coeff(866)=-0.505270416032D+04
coeff(867)=-0.134289107801D+04
coeff(868)=0.694948065547D+03
coeff(869)=0.953454377426D+03
coeff(870)=-0.503712086928D+04
coeff(871)=0.686364244264D+03
coeff(872)=0.257988482847D+04
coeff(873)=0.468833841491D+04
coeff(874)=0.993377617354D+03
coeff(875)=0.960835446959D+03
coeff(876)=-0.123666286125D+04
coeff(877)=0.239246827615D+04
coeff(878)=0.628522933075D+04
coeff(879)=0.195777237207D+04
coeff(880)=0.857307578668D+03
coeff(881)=-0.308623946052D+04
coeff(882)=0.599894922656D+04
coeff(883)=-0.170304014359D+04
coeff(884)=-0.714891238772D+04
coeff(885)=0.306433129920D+04
coeff(886)=-0.423059057304D+04
coeff(887)=-0.296393868940D+04
coeff(888)=-0.467675576038D+04
coeff(889)=-0.290207681496D+04
coeff(890)=0.561385396360D+04
coeff(891)=-0.652281958112D+04
coeff(892)=-0.133520102046D+03
coeff(893)=0.336682815023D+04
coeff(894)=-0.211437664795D+04
coeff(895)=-0.216017879049D+04
coeff(896)=-0.293113306321D+04
coeff(897)=-0.430061788331D+04
coeff(898)=0.355113999210D+04
coeff(899)=-0.444807746914D+04
coeff(900)=0.508560788558D+03
coeff(901)=0.611734691241D+04
coeff(902)=0.978799007953D+04
coeff(903)=-0.477271724306D+04
coeff(904)=-0.928816048485D+04
coeff(905)=0.439487279608D+04
coeff(906)=-0.259825249501D+04
coeff(907)=0.194300039477D+04
coeff(908)=0.397795783662D+04
coeff(909)=0.410024697680D+04
coeff(910)=-0.785823150597D+04
coeff(911)=0.131895795318D+05
coeff(912)=-0.871344386398D+04
coeff(913)=-0.155359558512D+05
coeff(914)=-0.405364932364D+04
coeff(915)=-0.872198047599D+03
coeff(916)=-0.628412810614D+04
coeff(917)=0.639538242471D+04
coeff(918)=-0.188431119343D+04
coeff(919)=-0.314626515048D+04
coeff(920)=-0.118459551514D+05
coeff(921)=-0.942870170220D+03
coeff(922)=-0.136593815375D+04
coeff(923)=-0.143262477808D+04
coeff(924)=0.985165785666D+03
coeff(925)=0.591940521630D+04
coeff(926)=0.515469406697D+04
coeff(927)=-0.567035428856D+04
coeff(928)=0.629580501037D+04
coeff(929)=0.108136279483D+04
coeff(930)=0.115454596502D+04
coeff(931)=-0.128499612405D+04
coeff(932)=-0.472463274368D+04
coeff(933)=0.320097720171D+04
coeff(934)=0.589585656992D+02
coeff(935)=0.372765603380D+04
coeff(936)=-0.326366481930D+04
coeff(937)=-0.726052419592D+03
coeff(938)=0.193761112928D+04
coeff(939)=0.275078722414D+04
coeff(940)=-0.248814762629D+04
coeff(941)=0.610412998208D+04
coeff(942)=-0.785162231229D+04
coeff(943)=-0.359550133602D+04
coeff(944)=-0.129301532079D+04
coeff(945)=-0.327241264019D+03
coeff(946)=0.724059311508D+04
coeff(947)=0.857415157827D+03
coeff(948)=-0.285823757913D+04
coeff(949)=0.252918019976D+04
coeff(950)=0.353090516922D+04
coeff(951)=0.171286713381D+04
coeff(952)=-0.346185113900D+03
coeff(953)=-0.420025734514D+04
coeff(954)=0.450040313832D+04
coeff(955)=-0.129962582372D+05
coeff(956)=0.112250767623D+05
coeff(957)=0.125378382543D+04
coeff(958)=0.148093899477D+04
coeff(959)=-0.108395556867D+04
coeff(960)=-0.141652531122D+04
coeff(961)=-0.369146332484D+04
coeff(962)=0.345680814425D+04
coeff(963)=0.387083894453D+04
coeff(964)=0.553563508621D+04
coeff(965)=-0.187485442933D+04
coeff(966)=-0.785632700125D+03
coeff(967)=-0.709803936818D+03
coeff(968)=-0.271782276909D+04
coeff(969)=0.594281396160D+03
coeff(970)=-0.279938675517D+04
coeff(971)=-0.265089500396D+04
coeff(972)=-0.298288184259D+04
coeff(973)=0.597177379528D+04
coeff(974)=0.159829333914D+04
coeff(975)=-0.387533997721D+04
coeff(976)=-0.491413466208D+04
coeff(977)=-0.817290321090D+03
coeff(978)=0.696344202568D+04
coeff(979)=-0.319404240382D+04
coeff(980)=0.245035838056D+04
coeff(981)=-0.447870473814D+04
coeff(982)=-0.476427973556D+04
coeff(983)=-0.970513929402D+02
coeff(984)=-0.309243143103D+04
coeff(985)=0.381237819542D+04
coeff(986)=-0.838855083624D+04
coeff(987)=0.127769660656D+02
coeff(988)=-0.405290008935D+04
coeff(989)=0.101047744347D+04
coeff(990)=-0.361362418130D+04
coeff(991)=0.298309067739D+04
coeff(992)=0.266529060876D+04
coeff(993)=0.302966848025D+04
coeff(994)=0.429631879942D+04
coeff(995)=0.383081380201D+04
coeff(996)=0.165139030789D+04
coeff(997)=-0.697347366153D+04
coeff(998)=0.364721933224D+04
coeff(999)=0.697011835779D+04
coeff(1000)=0.924248637202D+04
coeff(1001)=-0.660225750744D+04
coeff(1002)=-0.364110101131D+04
coeff(1003)=-0.207703732406D+04
coeff(1004)=-0.198309975358D+04
coeff(1005)=0.607059133251D+04
coeff(1006)=0.254465207033D+03
coeff(1007)=-0.985212037975D+03
coeff(1008)=-0.534330503852D+04
coeff(1009)=-0.387417140410D+04
coeff(1010)=0.109547169012D+05
coeff(1011)=0.289106240127D+03
coeff(1012)=0.467549616506D+04
coeff(1013)=0.546202906067D+04
coeff(1014)=-0.225671109132D+04
coeff(1015)=-0.134290804142D+04
coeff(1016)=0.269280898779D+04
coeff(1017)=0.260642166533D+04
coeff(1018)=-0.403278026650D+04
coeff(1019)=0.425644374383D+04
coeff(1020)=0.277441319745D+04
coeff(1021)=0.613281895815D+04
coeff(1022)=0.127349197300D+03
coeff(1023)=-0.756197138398D+04
coeff(1024)=-0.109675806004D+05
coeff(1025)=-0.732358640903D+04
coeff(1026)=0.707745297692D+04
coeff(1027)=0.568723294198D+03
coeff(1028)=-0.176239755148D+04
coeff(1029)=0.108278234716D+05
coeff(1030)=0.248410825274D+04
coeff(1031)=-0.446895630368D+04
coeff(1032)=0.176582284223D+04
coeff(1033)=0.346779381690D+03
coeff(1034)=0.373093400615D+04
coeff(1035)=0.539892518356D+04
coeff(1036)=-0.341344325676D+04
coeff(1037)=0.323776807156D+04
coeff(1038)=0.733346286354D+04
coeff(1039)=-0.642818105198D+04
coeff(1040)=-0.642888934322D+04
coeff(1041)=-0.251064482772D+04
coeff(1042)=0.605855409561D+04
coeff(1043)=-0.200431312927D+04
coeff(1044)=0.158018908739D+04
coeff(1045)=0.450136216492D+04
coeff(1046)=0.744567317717D+04
coeff(1047)=0.966428568588D+03
coeff(1048)=-0.478531000974D+04
coeff(1049)=-0.185934122253D+03
coeff(1050)=-0.220900136301D+04
coeff(1051)=0.246055898846D+04
coeff(1052)=0.401617037831D+04
coeff(1053)=-0.197538300053D+04
coeff(1054)=0.781390645928D+04
coeff(1055)=-0.558500200852D+04
coeff(1056)=-0.103038190318D+04
coeff(1057)=-0.422217948696D+04
coeff(1058)=0.253461923577D+04
coeff(1059)=-0.512618338912D+04
coeff(1060)=0.525414129663D+04
coeff(1061)=0.452189161244D+04
coeff(1062)=-0.253867745052D+04
coeff(1063)=0.116601861568D+04
coeff(1064)=0.211559916667D+03
coeff(1065)=0.159161970068D+04
coeff(1066)=0.983032601258D+04
coeff(1067)=0.460601351901D+04
coeff(1068)=0.416883535006D+04
coeff(1069)=0.646292313724D+03
coeff(1070)=0.839966637022D+03
coeff(1071)=-0.239236615647D+04
coeff(1072)=0.310530526333D+04
coeff(1073)=-0.329626564421D+04
coeff(1074)=-0.118564045253D+04
coeff(1075)=-0.199255897523D+04
coeff(1076)=0.615511785304D+04
coeff(1077)=0.457417831074D+03
coeff(1078)=0.100544174073D+05
coeff(1079)=-0.822138052118D+04
coeff(1080)=-0.323552538419D+04
coeff(1081)=0.743489991296D+04
coeff(1082)=-0.147897354584D+04
coeff(1083)=0.360667585945D+04
coeff(1084)=-0.304670502497D+04
coeff(1085)=-0.604646339331D+03
coeff(1086)=-0.105602443262D+04
coeff(1087)=0.141064306858D+05
coeff(1088)=0.169976572749D+04
coeff(1089)=-0.495222616753D+02
coeff(1090)=-0.655251243636D+03
coeff(1091)=-0.884615614665D+02
coeff(1092)=0.110976910442D+05
coeff(1093)=-0.746516512225D+04
coeff(1094)=0.533495155304D+03
coeff(1095)=-0.940366623712D+04
coeff(1096)=0.576150289482D+04
coeff(1097)=0.300882332264D+03
coeff(1098)=0.178976644449D+04
coeff(1099)=-0.978570889587D+04
coeff(1100)=-0.406427750381D+04
coeff(1101)=0.108764867609D+04
coeff(1102)=0.293047410406D+04
coeff(1103)=-0.365509535161D+04
coeff(1104)=0.208595945524D+03
coeff(1105)=-0.988931538584D+04
coeff(1106)=0.105340589852D+05
coeff(1107)=0.842748381769D+04
coeff(1108)=-0.974671423844D+02
coeff(1109)=0.367510001612D+04
coeff(1110)=0.530820222586D+04
coeff(1111)=0.360738351786D+04
coeff(1112)=0.211943615724D+04
coeff(1113)=0.166571993513D+04
coeff(1114)=0.460763360004D+04
coeff(1115)=0.541111749390D+04
coeff(1116)=-0.176004580223D+03
coeff(1117)=0.329340692865D+04
coeff(1118)=-0.661615612991D+03
coeff(1119)=0.706175656535D+03
coeff(1120)=-0.217473365145D+04
coeff(1121)=-0.924204405434D+04
coeff(1122)=0.261039440190D+04
coeff(1123)=0.221152529456D+04
coeff(1124)=0.518340126064D+04
coeff(1125)=0.793490096849D+04
coeff(1126)=-0.253684469761D+04
coeff(1127)=0.816077796629D+02
coeff(1128)=0.136120679431D+05
coeff(1129)=-0.177093715903D+04
coeff(1130)=0.275752903209D+03
coeff(1131)=0.519097658231D+04
coeff(1132)=0.232637408374D+04
coeff(1133)=0.305098646123D+04
coeff(1134)=-0.246439513793D+04
coeff(1135)=0.381075749083D+03
coeff(1136)=-0.132782248250D+04
coeff(1137)=0.208360149216D+04
coeff(1138)=-0.330059152522D+03
coeff(1139)=-0.114777104025D+04
coeff(1140)=-0.192177247123D+04
coeff(1141)=0.115875335722D+04
coeff(1142)=0.651198552723D+04
coeff(1143)=-0.354390261294D+04
coeff(1144)=-0.112783130936D+02
coeff(1145)=0.768178900378D+04
coeff(1146)=0.299767725149D+04
coeff(1147)=-0.535183126171D+03
coeff(1148)=-0.318406973261D+04
coeff(1149)=-0.865547103704D+04
coeff(1150)=-0.865365723557D+04
coeff(1151)=0.392643585907D+04
coeff(1152)=0.343087301238D+04
coeff(1153)=-0.421470935186D+03
coeff(1154)=-0.714124724622D+04
coeff(1155)=0.109394424051D+05
coeff(1156)=0.344427761421D+04
coeff(1157)=0.404718269546D+04
coeff(1158)=0.212663076996D+03
coeff(1159)=-0.462369788634D+04
coeff(1160)=-0.116676304036D+04
coeff(1161)=-0.327064077662D+04
coeff(1162)=0.109184565318D+05
coeff(1163)=0.189507373653D+04
coeff(1164)=-0.323949128813D+04
coeff(1165)=-0.188770793407D+04
coeff(1166)=0.774644363785D+04
coeff(1167)=0.656174678904D+04
coeff(1168)=-0.340858382807D+04
coeff(1169)=0.172559395339D+04
coeff(1170)=0.405575701012D+04
coeff(1171)=-0.611609620118D+04
coeff(1172)=0.141957131970D+04
coeff(1173)=0.265498732774D+04
coeff(1174)=-0.264801852964D+04
coeff(1175)=-0.128273555390D+05
coeff(1176)=0.195230907709D+04
coeff(1177)=0.861711704726D+03
coeff(1178)=-0.131032450196D+04
coeff(1179)=-0.295056491945D+04
coeff(1180)=-0.112547743132D+05
coeff(1181)=-0.794948562521D+04
coeff(1182)=0.152391973531D+04
coeff(1183)=0.895009352775D+02
coeff(1184)=-0.423600077380D+04
coeff(1185)=-0.981526927104D+04
coeff(1186)=-0.212713329495D+03
coeff(1187)=0.221237254413D+04
coeff(1188)=-0.243654139798D+04
coeff(1189)=0.916092315980D+04
coeff(1190)=0.565231087690D+04
coeff(1191)=0.636255698004D+04
coeff(1192)=-0.731107570418D+04
coeff(1193)=-0.433999150105D+04
coeff(1194)=0.629288629511D+04
coeff(1195)=-0.210396749445D+03
coeff(1196)=-0.898770983255D+04
coeff(1197)=-0.506113377716D+04
coeff(1198)=0.344491388982D+04
coeff(1199)=-0.257555136355D+04
coeff(1200)=-0.248826070222D+04
coeff(1201)=-0.589091069218D+04
coeff(1202)=-0.423689772725D+04
coeff(1203)=0.477125966426D+04
coeff(1204)=-0.156018858550D+03
coeff(1205)=-0.180196597355D+04
coeff(1206)=-0.931143229892D+03
coeff(1207)=0.225728069618D+04
coeff(1208)=-0.667484896620D+04
coeff(1209)=-0.430020131177D+03
coeff(1210)=-0.238873994655D+02
coeff(1211)=-0.400358528528D+04
coeff(1212)=0.141943424192D+04
coeff(1213)=0.456145966758D+04
coeff(1214)=-0.839768028371D+04
coeff(1215)=0.543589724276D+02
coeff(1216)=0.383260856742D+04
coeff(1217)=0.256507491551D+03
coeff(1218)=0.458477517473D+04
coeff(1219)=0.150835538952D+04
coeff(1220)=0.586627995689D+04
coeff(1221)=0.583853063866D+04
coeff(1222)=-0.397768016164D+04
coeff(1223)=0.455885915761D+04
coeff(1224)=-0.482716175588D+04
coeff(1225)=-0.764602811983D+04
coeff(1226)=0.464475557522D+04
coeff(1227)=0.750453330854D+04
coeff(1228)=-0.559336548227D+04
coeff(1229)=0.275782710016D+04
coeff(1230)=0.525966134472D+03
coeff(1231)=0.138190778596D+04
coeff(1232)=0.205381186736D+04
coeff(1233)=0.417049666136D+03
coeff(1234)=0.226173544889D+04
coeff(1235)=-0.186153044393D+04
coeff(1236)=0.234621096954D+04
coeff(1237)=0.348428675936D+04
coeff(1238)=-0.106907069000D+04
coeff(1239)=0.281544004787D+04
coeff(1240)=-0.987780407164D+04
coeff(1241)=0.471772574237D+04
coeff(1242)=-0.546514209700D+04
coeff(1243)=-0.300318389923D+04
coeff(1244)=-0.475755291575D+04
coeff(1245)=-0.622507694571D+03
coeff(1246)=-0.492447936827D+04
coeff(1247)=0.154589496381D+04
coeff(1248)=-0.586556312158D+04
coeff(1249)=-0.382546525645D+04
coeff(1250)=-0.439047911745D+04
coeff(1251)=-0.490063977063D+04
coeff(1252)=0.511215847277D+04
coeff(1253)=0.321805992646D+04
coeff(1254)=-0.437211573566D+03
coeff(1255)=0.209249843807D+04
coeff(1256)=0.582466316839D+03
coeff(1257)=-0.128228709780D+05
coeff(1258)=-0.324305834593D+04
coeff(1259)=0.390387160887D+04
coeff(1260)=0.412759623882D+04
coeff(1261)=-0.165009616996D+04
coeff(1262)=0.467340766290D+04
coeff(1263)=0.156808725331D+03
coeff(1264)=0.242657770631D+04
coeff(1265)=-0.159156218320D+04
coeff(1266)=-0.530906095380D+03
coeff(1267)=-0.269533565694D+04
coeff(1268)=0.632500144195D+04
coeff(1269)=-0.534386103031D+04
coeff(1270)=-0.725948657040D+04
coeff(1271)=0.566338801128D+04
coeff(1272)=-0.163138329881D+04
coeff(1273)=-0.166799858183D+04
coeff(1274)=0.357800886489D+03
coeff(1275)=-0.531036932056D+04
coeff(1276)=-0.450558189171D+04
coeff(1277)=-0.196465679536D+04
coeff(1278)=-0.219245461228D+04
coeff(1279)=-0.252145215376D+04
coeff(1280)=-0.869354844635D+03
coeff(1281)=0.413912773439D+04
coeff(1282)=0.585679964401D+04
coeff(1283)=0.233464530165D+04
coeff(1284)=0.545800447565D+04
coeff(1285)=-0.766160819489D+04
coeff(1286)=-0.570055515318D+04
coeff(1287)=-0.208725811325D+04
coeff(1288)=0.146561555773D+03
coeff(1289)=-0.487786815956D+04
coeff(1290)=-0.843526946567D+03
coeff(1291)=0.458564299547D+04
coeff(1292)=0.352300550187D+04
coeff(1293)=0.694459550864D+03
coeff(1294)=0.383356983851D+04
coeff(1295)=0.313316650052D+04
coeff(1296)=-0.503137949219D+04
coeff(1297)=-0.921335598421D+04
coeff(1298)=-0.114107178778D+04
coeff(1299)=0.403338142248D+04
coeff(1300)=0.235934276817D+04
coeff(1301)=-0.784127533284D+04
coeff(1302)=-0.116911898770D+04
coeff(1303)=-0.213013426449D+04
coeff(1304)=0.224120655229D+04
coeff(1305)=0.936810723073D+04
coeff(1306)=0.361889013459D+04
coeff(1307)=0.176023057069D+04
coeff(1308)=-0.102318418027D+04
coeff(1309)=-0.883822983261D+04
coeff(1310)=0.426737802393D+04
coeff(1311)=0.481921279463D+04
coeff(1312)=-0.546775052124D+04
coeff(1313)=0.382206447718D+04
coeff(1314)=0.407439168733D+04
coeff(1315)=-0.707971959337D+04
coeff(1316)=0.281218286642D+04
coeff(1317)=0.679188096630D+04
coeff(1318)=-0.269767556933D+04
coeff(1319)=0.254984424795D+04
coeff(1320)=0.363271527487D+04
coeff(1321)=0.339733191659D+04
coeff(1322)=0.528348772227D+04
coeff(1323)=-0.169260314240D+04
coeff(1324)=0.361983485828D+04
coeff(1325)=0.831050693951D+04
coeff(1326)=0.977922646829D+03
coeff(1327)=0.297909850154D+04
coeff(1328)=-0.485620462071D+04
coeff(1329)=-0.210204540006D+03
coeff(1330)=0.551219010957D+04
coeff(1331)=-0.551965179499D+04
coeff(1332)=0.740812510128D+04
coeff(1333)=-0.984669227592D+03
coeff(1334)=0.463690043442D+04
coeff(1335)=0.518687295662D+04
coeff(1336)=0.305610209008D+04
coeff(1337)=0.443167430076D+03
coeff(1338)=0.181377044601D+03
coeff(1339)=0.368990526919D+04
coeff(1340)=-0.278420758149D+04
coeff(1341)=-0.678785614207D+03
coeff(1342)=-0.290287416908D+04
coeff(1343)=-0.119036678366D+05
coeff(1344)=0.531723205695D+04
coeff(1345)=-0.103098678147D+05
coeff(1346)=-0.702106798656D+04
coeff(1347)=0.233657681636D+04
coeff(1348)=-0.411586036219D+04
coeff(1349)=-0.283549697528D+04
coeff(1350)=-0.270075359772D+04
coeff(1351)=0.466887349689D+04
coeff(1352)=0.344447894447D+04
coeff(1353)=0.187925138023D+04
coeff(1354)=0.533714411434D+04
coeff(1355)=-0.336076832799D+04
coeff(1356)=-0.314968702868D+04
coeff(1357)=-0.379339825855D+03
coeff(1358)=0.454273213898D+04
coeff(1359)=-0.296719531599D+04
coeff(1360)=-0.442195528356D+02
coeff(1361)=-0.595966400356D+04
coeff(1362)=-0.318615329685D+04
coeff(1363)=0.498563586090D+04
coeff(1364)=0.279295672742D+04
coeff(1365)=0.740213663526D+04
coeff(1366)=0.571620697418D+03
coeff(1367)=0.782333068791D+04
coeff(1368)=-0.372943609937D+04
coeff(1369)=-0.688290211194D+03
coeff(1370)=-0.188544682825D+04
coeff(1371)=0.306999235866D+04
coeff(1372)=0.302137654847D+04
coeff(1373)=-0.292958454255D+04
coeff(1374)=-0.172662893655D+04
coeff(1375)=0.510168846340D+04
coeff(1376)=-0.183783057904D+03
coeff(1377)=-0.120842419084D+05
coeff(1378)=-0.753601816822D+04
coeff(1379)=0.687722731659D+04
coeff(1380)=0.230256919123D+04
coeff(1381)=-0.164668336854D+04
coeff(1382)=0.303494134586D+04
coeff(1383)=0.292228349447D+04
coeff(1384)=0.507944660565D+04
coeff(1385)=-0.199630688820D+03
coeff(1386)=0.649820222412D+04
coeff(1387)=0.524455069908D+04
coeff(1388)=-0.533200360842D+04
coeff(1389)=0.459046588904D+04
coeff(1390)=-0.713704906691D+03
coeff(1391)=0.378330808793D+04
coeff(1392)=-0.518264963547D+03
coeff(1393)=0.335525533754D+04
coeff(1394)=-0.410072513157D+04
coeff(1395)=-0.104131908120D+05
coeff(1396)=-0.333742691874D+04
coeff(1397)=-0.395008045890D+04
coeff(1398)=0.787999005324D+04
coeff(1399)=0.585545077961D+02
coeff(1400)=-0.695975440622D+04
coeff(1401)=0.626402540067D+04
coeff(1402)=0.460453315766D+04
coeff(1403)=-0.390858819994D+03
coeff(1404)=-0.751201455893D+04
coeff(1405)=-0.450430526715D+04
coeff(1406)=0.628397742941D+04
coeff(1407)=0.234164117487D+04
coeff(1408)=0.262549325244D+04
coeff(1409)=0.311113942247D+03
coeff(1410)=-0.113154746189D+04
coeff(1411)=0.925370486919D+03
coeff(1412)=0.149502452278D+04
coeff(1413)=0.173386502391D+04
coeff(1414)=0.590692933917D+04
coeff(1415)=-0.116606250099D+04
coeff(1416)=-0.550953964165D+04
coeff(1417)=0.220329123421D+03
coeff(1418)=-0.505561925372D+04
coeff(1419)=0.275323385159D+04
coeff(1420)=0.156241365460D+05
coeff(1421)=-0.135360262873D+04
coeff(1422)=0.231843406649D+04
coeff(1423)=0.531235244924D+04
coeff(1424)=0.366028320867D+03
coeff(1425)=-0.534549698496D+04
coeff(1426)=-0.421016067155D+04
coeff(1427)=0.675715476947D+04
coeff(1428)=0.244701151387D+04
coeff(1429)=-0.699863619734D+03
coeff(1430)=-0.662021531466D+04
coeff(1431)=-0.423437075982D+04
coeff(1432)=-0.609196061512D+04
coeff(1433)=-0.508040160573D+04
coeff(1434)=-0.140825037794D+04
coeff(1435)=-0.477791251521D+04
coeff(1436)=-0.146729593426D+04
coeff(1437)=-0.377657040749D+03
coeff(1438)=-0.969568066210D+04
coeff(1439)=-0.775041461211D+04
coeff(1440)=0.611938916632D+04
coeff(1441)=0.106999887055D+04
coeff(1442)=0.183651787972D+04
coeff(1443)=0.654726391872D+04
coeff(1444)=0.470488873924D+03
coeff(1445)=-0.125710719286D+03
coeff(1446)=-0.382616572092D+04
coeff(1447)=0.300550131581D+04
coeff(1448)=0.551586974341D+04
coeff(1449)=0.219209121218D+04
coeff(1450)=0.277556131025D+02
coeff(1451)=0.366035481580D+03
coeff(1452)=0.238199438603D+04
coeff(1453)=0.256386272660D+02
coeff(1454)=-0.111128723796D+04
coeff(1455)=0.945591296366D+04
coeff(1456)=0.428343119295D+04
coeff(1457)=-0.715749201403D+04
coeff(1458)=0.219881486224D+03
coeff(1459)=0.329403435941D+04
coeff(1460)=-0.335175584888D+04
coeff(1461)=-0.206749038461D+04
coeff(1462)=0.594368258058D+04
coeff(1463)=0.479645793478D+04
coeff(1464)=0.943403035490D+03
coeff(1465)=0.489266533576D+04
coeff(1466)=-0.550297036336D+04
coeff(1467)=0.139514175182D+05
coeff(1468)=0.130690854316D+05
coeff(1469)=0.427670758765D+04
coeff(1470)=-0.184065504274D+04
coeff(1471)=-0.768783102734D+04
coeff(1472)=-0.136765206640D+03
coeff(1473)=0.713259028189D+04
coeff(1474)=0.252477859678D+04
coeff(1475)=0.163311417597D+04
coeff(1476)=0.684129449224D+02
coeff(1477)=-0.438019039036D+03
coeff(1478)=-0.178028566715D+04
coeff(1479)=0.287142621647D+04
coeff(1480)=-0.226725295163D+03
coeff(1481)=0.848749881608D+03
coeff(1482)=-0.417812383943D+04
coeff(1483)=-0.806853548701D+04
coeff(1484)=-0.407726801349D+04
coeff(1485)=0.112181370697D+05
coeff(1486)=-0.320935351340D+04
coeff(1487)=-0.155965050488D+04
coeff(1488)=-0.644472789303D+04
coeff(1489)=-0.217792325494D+04
coeff(1490)=0.465044851326D+04
coeff(1491)=-0.630475426034D+04
coeff(1492)=0.524463315398D+04
coeff(1493)=0.302339369999D+04
coeff(1494)=0.701262213087D+03
coeff(1495)=0.722463144284D+03
coeff(1496)=-0.159391551611D+05
coeff(1497)=0.528171054053D+04
coeff(1498)=0.429648782102D+04
coeff(1499)=-0.112167040384D+04
coeff(1500)=-0.601891043981D+04
coeff(1501)=0.690627377718D+04
coeff(1502)=0.521845216308D+04
coeff(1503)=0.797141341696D+04
coeff(1504)=0.670572856553D+03
coeff(1505)=0.292387973823D+04
coeff(1506)=0.611109566485D+04
coeff(1507)=0.699609734731D+04
coeff(1508)=-0.284570556417D+04
coeff(1509)=0.366782428600D+04
coeff(1510)=-0.123553820451D+05
coeff(1511)=-0.303130691183D+04
coeff(1512)=-0.302720207328D+02
coeff(1513)=0.322505441853D+04
coeff(1514)=0.767977499963D+04
coeff(1515)=-0.388793103185D+04
coeff(1516)=0.130724004556D+05
coeff(1517)=0.184281358200D+04
coeff(1518)=-0.118985930295D+05
coeff(1519)=0.103207791676D+04
coeff(1520)=0.421526960269D+04
coeff(1521)=0.428363234678D+03
coeff(1522)=0.793872535379D+03
coeff(1523)=-0.942139962288D+03
coeff(1524)=-0.235458594883D+04
coeff(1525)=-0.339789062608D+03
coeff(1526)=0.143003045302D+03
coeff(1527)=-0.968015286084D+03
coeff(1528)=-0.364974982411D+04
coeff(1529)=0.273931994536D+04
coeff(1530)=-0.123383061777D+05
coeff(1531)=0.284866328148D+04
coeff(1532)=-0.355152655117D+04
coeff(1533)=0.670663101301D+04
coeff(1534)=0.136169664399D+04
coeff(1535)=-0.835054280224D+04
coeff(1536)=-0.530326793691D+04
coeff(1537)=-0.449672032164D+03
coeff(1538)=0.110043536270D+04
coeff(1539)=0.419141796830D+04
coeff(1540)=-0.889780741300D+04
coeff(1541)=0.869695333094D+03
coeff(1542)=0.777393945670D+03
coeff(1543)=-0.122037820540D+04
coeff(1544)=-0.311483346138D+04
coeff(1545)=0.739994438765D+03
coeff(1546)=0.445316417310D+04
coeff(1547)=-0.240169875367D+04
coeff(1548)=0.193240266352D+04
coeff(1549)=-0.606941584521D+03
coeff(1550)=-0.801626493389D+04
coeff(1551)=-0.270530088196D+03
coeff(1552)=0.411030442928D+04
coeff(1553)=-0.100996618996D+04
coeff(1554)=0.318461804694D+04
coeff(1555)=-0.714451405522D+03
coeff(1556)=-0.205570347754D+04
coeff(1557)=0.104345483798D+04
coeff(1558)=0.149320462593D+04
coeff(1559)=0.521871811426D+04
coeff(1560)=-0.635336097696D+03
coeff(1561)=-0.149633259987D+04
coeff(1562)=0.318831390032D+04
coeff(1563)=0.116470081787D+04
coeff(1564)=-0.230190707084D+04
coeff(1565)=-0.338415442202D+04
coeff(1566)=0.657975469670D+04
coeff(1567)=0.712584464679D+04
coeff(1568)=0.228044642078D+03
coeff(1569)=0.970202935416D+04
coeff(1570)=0.527969981399D+04
coeff(1571)=0.257156470396D+04
coeff(1572)=-0.371358354093D+04
coeff(1573)=0.332542075050D+04
coeff(1574)=0.121132362788D+04
coeff(1575)=0.658482490341D+03
coeff(1576)=-0.374397764429D+04
coeff(1577)=-0.288429352250D+04
coeff(1578)=0.236517177188D+04
coeff(1579)=-0.461144478341D+04
coeff(1580)=-0.153509118242D+04
coeff(1581)=0.101261547429D+05
coeff(1582)=0.454198326761D+04
coeff(1583)=-0.426866870342D+04
coeff(1584)=0.213411341608D+04
coeff(1585)=0.187018015332D+03
coeff(1586)=-0.371822605060D+03
coeff(1587)=-0.605227363135D+04
coeff(1588)=-0.408317691867D+04
coeff(1589)=0.280140543217D+03
coeff(1590)=0.447382982200D+04
coeff(1591)=0.569430871271D+03
coeff(1592)=-0.259589836920D+04
coeff(1593)=-0.621778774035D+04
coeff(1594)=-0.450629417588D+04
coeff(1595)=0.985587430356D+03
coeff(1596)=0.185060469762D+03
coeff(1597)=0.351964808419D+04
coeff(1598)=0.345903811390D+03
coeff(1599)=0.235860627607D+04
coeff(1600)=-0.253552910003D+03
coeff(1601)=-0.788717024814D+04
coeff(1602)=-0.691309822501D+04
coeff(1603)=-0.318314342459D+04
coeff(1604)=-0.418446012478D+04
coeff(1605)=0.102291716971D+05
coeff(1606)=0.200975714494D+04
coeff(1607)=0.606346251947D+04
coeff(1608)=-0.854156027551D+04
coeff(1609)=-0.525235085901D+04
coeff(1610)=-0.103039248422D+05
coeff(1611)=-0.509684774468D+04
coeff(1612)=-0.105208174343D+04
coeff(1613)=0.553359591855D+03
coeff(1614)=0.338537856236D+04
coeff(1615)=-0.538031956833D+04
coeff(1616)=0.120821684210D+05
coeff(1617)=-0.306174639622D+04
coeff(1618)=-0.112452291615D+05
coeff(1619)=-0.657494670118D+04
coeff(1620)=-0.213609691626D+04
coeff(1621)=0.536131325365D+04
coeff(1622)=0.943078873534D+04
coeff(1623)=-0.350960160301D+04
coeff(1624)=0.653965611527D+04
coeff(1625)=0.812187058081D+04
coeff(1626)=0.653205334425D+04
coeff(1627)=-0.500232972370D+04
coeff(1628)=-0.848337045113D+04
coeff(1629)=-0.343989799573D+04
coeff(1630)=-0.570779722167D+04
coeff(1631)=-0.599290777050D+03
coeff(1632)=0.429350419747D+04
coeff(1633)=0.801771655278D+04
coeff(1634)=0.337474338964D+04
coeff(1635)=0.872224467648D+04
coeff(1636)=0.231541978133D+03
coeff(1637)=-0.671325549194D+04
coeff(1638)=0.480158384864D+04
coeff(1639)=-0.311991790124D+04
coeff(1640)=-0.632497019801D+03
coeff(1641)=0.621509003858D+04
coeff(1642)=0.257461753461D+04
coeff(1643)=-0.286828619827D+04
coeff(1644)=-0.856516978220D+04
coeff(1645)=0.693610602227D+04
coeff(1646)=0.140370963126D+03
coeff(1647)=-0.474766289367D+04
coeff(1648)=0.206485138236D+04
coeff(1649)=-0.418483736458D+04
coeff(1650)=-0.971373006524D+04
coeff(1651)=-0.721735693203D+04
coeff(1652)=0.335885974718D+04
coeff(1653)=-0.855508772034D+03
coeff(1654)=-0.440108430676D+04
coeff(1655)=0.106973624993D+04
coeff(1656)=0.292132495382D+04
coeff(1657)=0.161820448551D+04
coeff(1658)=0.223155608665D+03
coeff(1659)=-0.397693675999D+04
coeff(1660)=0.161155324026D+04
coeff(1661)=-0.905598703660D+04
coeff(1662)=0.341690572518D+03
coeff(1663)=-0.314966451844D+04
coeff(1664)=-0.253096823982D+04
coeff(1665)=-0.492657260739D+04
coeff(1666)=0.288900703328D+04
coeff(1667)=-0.794952444929D+03
coeff(1668)=-0.430407948577D+04
coeff(1669)=-0.341041591888D+04
coeff(1670)=-0.486153516824D+03
coeff(1671)=0.794803275646D+04
coeff(1672)=0.837344240377D+04
coeff(1673)=-0.109262989834D+04
coeff(1674)=0.325248417449D+04
coeff(1675)=0.380918523667D+04
coeff(1676)=0.168925025653D+05
coeff(1677)=0.329868733097D+04
coeff(1678)=0.560655400588D+04
coeff(1679)=0.639891518410D+03
coeff(1680)=-0.345799339857D+04
coeff(1681)=-0.687650682928D+03
coeff(1682)=0.261745688329D+04
coeff(1683)=0.489602167636D+04
coeff(1684)=0.319062813778D+04
coeff(1685)=0.317849354393D+04
coeff(1686)=-0.401873611424D+03
coeff(1687)=-0.402328718411D+04
coeff(1688)=-0.693247946716D+04
coeff(1689)=-0.545402847496D+04
coeff(1690)=0.718528687551D+03
coeff(1691)=-0.112029207632D+05
coeff(1692)=-0.109015208131D+05
coeff(1693)=0.991560457174D+04
coeff(1694)=0.341394176351D+04
coeff(1695)=0.613311087212D+04
coeff(1696)=-0.223759583481D+03
coeff(1697)=-0.383539647630D+04
coeff(1698)=-0.129793739096D+05
coeff(1699)=-0.940870950472D+04
coeff(1700)=0.318893194122D+04
coeff(1701)=0.376025745971D+04
coeff(1702)=-0.383734942043D+04
coeff(1703)=-0.246331579106D+04
coeff(1704)=0.165405630091D+04
coeff(1705)=0.384465924275D+04
coeff(1706)=-0.766593724604D+04
coeff(1707)=0.167199559851D+04
coeff(1708)=0.374844803590D+03
coeff(1709)=-0.905257077178D+04
coeff(1710)=0.208379952716D+04
coeff(1711)=0.735501289726D+04
coeff(1712)=-0.247591696260D+04
coeff(1713)=-0.236168643409D+04
coeff(1714)=0.371540339217D+04
coeff(1715)=-0.918202771482D+04
coeff(1716)=-0.188253934617D+04
coeff(1717)=-0.148749663114D+03
coeff(1718)=0.280901604410D+04
coeff(1719)=0.324748898501D+04
coeff(1720)=0.930892779089D+03
coeff(1721)=0.484862127602D+04
coeff(1722)=0.163508843623D+03
coeff(1723)=-0.447380827022D+04
coeff(1724)=-0.611561655777D+03
coeff(1725)=-0.254911948352D+04
coeff(1726)=0.764883156988D+04
coeff(1727)=0.856108315755D+04
coeff(1728)=-0.164667759135D+04
coeff(1729)=-0.173093682703D+03
coeff(1730)=-0.123695998904D+04
coeff(1731)=0.812058911781D+04
coeff(1732)=-0.473337774896D+03
coeff(1733)=-0.528917287816D+04
coeff(1734)=-0.797257215589D+04
coeff(1735)=-0.406433246747D+04
coeff(1736)=0.269025337172D+04
coeff(1737)=0.238040197178D+04
coeff(1738)=0.108365908013D+02
coeff(1739)=-0.706243548244D+03
coeff(1740)=0.201303879121D+04
coeff(1741)=-0.337234916572D+04
coeff(1742)=-0.610303758865D+04
coeff(1743)=-0.708809695804D+04
coeff(1744)=0.540021904386D+04
coeff(1745)=0.124287958636D+04
coeff(1746)=0.222783360766D+04
coeff(1747)=-0.257443630691D+04
coeff(1748)=-0.907481165316D+04
coeff(1749)=0.155006106858D+05
coeff(1750)=-0.624628775735D+04
coeff(1751)=-0.146460219163D+04
coeff(1752)=0.630466966640D+04
coeff(1753)=0.200027542669D+04
coeff(1754)=-0.774251241289D+04
coeff(1755)=-0.348853156051D+04
coeff(1756)=-0.458391353177D+04
coeff(1757)=-0.436477888362D+04
coeff(1758)=-0.115484029061D+03
coeff(1759)=0.270594670226D+04
coeff(1760)=0.254333324066D+04
coeff(1761)=-0.426559054681D+04
coeff(1762)=0.669590056693D+04
coeff(1763)=0.318810401033D+04
coeff(1764)=-0.196055288835D+04
coeff(1765)=0.637530011142D+04
coeff(1766)=0.725087908250D+04
coeff(1767)=0.271095141008D+04
coeff(1768)=-0.118785380183D+04
coeff(1769)=-0.636995965469D+04
coeff(1770)=-0.636076919813D+04
coeff(1771)=-0.439027522118D+04
coeff(1772)=-0.189635799701D+05
coeff(1773)=-0.152673827290D+05
coeff(1774)=-0.443187280787D+04
coeff(1775)=-0.537419941669D+04
coeff(1776)=0.515500027176D+04
coeff(1777)=0.444490396141D+04
coeff(1778)=0.126398252305D+03
coeff(1779)=0.322961442852D+04
coeff(1780)=0.307133746842D+04
coeff(1781)=-0.116310520645D+05
coeff(1782)=0.306408151415D+04
coeff(1783)=-0.225462560057D+04
coeff(1784)=-0.241845040713D+04
coeff(1785)=0.241807565854D+04
coeff(1786)=0.508232517514D+04
coeff(1787)=0.320398472435D+04
coeff(1788)=-0.372131363431D+04
coeff(1789)=-0.275236121532D+04
coeff(1790)=0.240804867905D+04
coeff(1791)=0.153854262615D+04
coeff(1792)=-0.451416732623D+03
coeff(1793)=0.127834773885D+05
coeff(1794)=-0.392258487939D+04
coeff(1795)=-0.452900157406D+03
coeff(1796)=-0.800060463197D+04
coeff(1797)=-0.466995188313D+04
coeff(1798)=0.271278955824D+03
coeff(1799)=-0.894142032090D+03
coeff(1800)=0.785845972382D+04
coeff(1801)=-0.363827758374D+04
coeff(1802)=-0.602618055019D+04
coeff(1803)=-0.126343751442D+04
coeff(1804)=-0.178891752771D+04
coeff(1805)=0.928561671641D+04
coeff(1806)=0.575617854295D+04
coeff(1807)=0.358339688611D+04
coeff(1808)=-0.514654214671D+04
coeff(1809)=0.134613352866D+05
coeff(1810)=0.430114109498D+04
coeff(1811)=0.144446711558D+05
coeff(1812)=-0.560080238022D+04
coeff(1813)=0.235005894601D+04
coeff(1814)=-0.619549983633D+04
coeff(1815)=-0.147405112041D+04
coeff(1816)=-0.570290044303D+04
coeff(1817)=0.522808345600D+04
coeff(1818)=0.970294967992D+04
coeff(1819)=-0.646528856004D+04
coeff(1820)=-0.287166248982D+02
coeff(1821)=0.361507273846D+04
coeff(1822)=0.308013940368D+03
coeff(1823)=0.634774141386D+04
coeff(1824)=-0.521983383043D+04
coeff(1825)=-0.568135736711D+04
coeff(1826)=-0.589269538587D+04
coeff(1827)=0.117707689479D+04
coeff(1828)=-0.640155846250D+04
coeff(1829)=0.244403201341D+03
coeff(1830)=-0.269247404070D+04
coeff(1831)=0.828556249539D+04
coeff(1832)=-0.683428223228D+04
coeff(1833)=-0.563643161495D+04
coeff(1834)=-0.490553473028D+03
coeff(1835)=0.888095556096D+04
coeff(1836)=0.658712474065D+04
coeff(1837)=-0.275575008034D+04
coeff(1838)=-0.388584606800D+04
coeff(1839)=-0.722312080478D+03
coeff(1840)=-0.818092049254D+04
coeff(1841)=-0.374255735816D+04
coeff(1842)=-0.413184957878D+04
coeff(1843)=0.976047972766D+04
coeff(1844)=0.302060159758D+02
coeff(1845)=-0.511856801217D+04
coeff(1846)=-0.572834441665D+04
coeff(1847)=-0.298847823232D+04
coeff(1848)=-0.767537539097D+04
coeff(1849)=0.335631320310D+04
coeff(1850)=0.848293136108D+04
coeff(1851)=0.605899211479D+04
coeff(1852)=0.689367040569D+03
coeff(1853)=0.117417722293D+04
coeff(1854)=-0.451957223727D+04
coeff(1855)=-0.148093441166D+04
coeff(1856)=-0.113234139681D+01
coeff(1857)=-0.517151166207D+04
coeff(1858)=-0.597332284059D+04
coeff(1859)=-0.632128310387D+04
coeff(1860)=-0.249399765153D+04
coeff(1861)=0.180848030316D+04
coeff(1862)=0.666061450116D+04
coeff(1863)=-0.162819118731D+04
coeff(1864)=0.623621793039D+03
coeff(1865)=-0.155991267938D+04
coeff(1866)=0.329421557497D+04
coeff(1867)=-0.129933638415D+04
coeff(1868)=-0.398245487150D+04
coeff(1869)=-0.843494388399D+04
coeff(1870)=-0.298920255608D+04
coeff(1871)=0.200324979770D+04
coeff(1872)=-0.714748381148D+04
coeff(1873)=0.226043787221D+03
coeff(1874)=0.211334177439D+04
coeff(1875)=-0.218773903112D+04
coeff(1876)=-0.421570564546D+04
coeff(1877)=0.266501155839D+04
coeff(1878)=0.108513133789D+05
coeff(1879)=-0.576171491254D+04
coeff(1880)=-0.860704305134D+04
coeff(1881)=0.507327703481D+04
coeff(1882)=0.314385150595D+04
coeff(1883)=-0.156659911785D+04
coeff(1884)=-0.447162020515D+04
coeff(1885)=0.564771109634D+04
coeff(1886)=0.447297524030D+04
coeff(1887)=-0.794243712678D+04
coeff(1888)=-0.232554322156D+03
coeff(1889)=-0.754610133600D+04
coeff(1890)=0.121539135213D+04
coeff(1891)=0.297154280761D+04
coeff(1892)=0.381914468496D+04
coeff(1893)=-0.570222890432D+04
coeff(1894)=-0.673177652479D+04
coeff(1895)=0.232226798736D+04
coeff(1896)=0.246445102993D+04
coeff(1897)=-0.568700736086D+04
coeff(1898)=-0.542123834641D+04
coeff(1899)=-0.402236364873D+04
coeff(1900)=0.332208946418D+04
coeff(1901)=0.830228436796D+04
coeff(1902)=-0.774213624481D+04
coeff(1903)=0.902945704718D+04
coeff(1904)=0.776959741250D+02
coeff(1905)=0.299885335938D+04
coeff(1906)=-0.990172364618D+03
coeff(1907)=0.417380587178D+04
coeff(1908)=-0.225704328190D+04
coeff(1909)=0.718021228549D+04
coeff(1910)=0.549565069132D+04
coeff(1911)=-0.364150243043D+04
coeff(1912)=0.113614128607D+05
coeff(1913)=0.865838645260D+03
coeff(1914)=-0.429423332172D+04
coeff(1915)=0.589664150177D+04
coeff(1916)=-0.347627655341D+03
coeff(1917)=0.288396419694D+04
coeff(1918)=-0.336492360115D+04
coeff(1919)=0.469947421650D+04
coeff(1920)=-0.230039019749D+03
coeff(1921)=0.137360492692D+04
coeff(1922)=0.693149293090D+04
coeff(1923)=-0.247732490504D+04
coeff(1924)=-0.194864931661D+04
coeff(1925)=-0.734181098581D+04
coeff(1926)=0.673258793574D+04
coeff(1927)=-0.706758837524D+03
coeff(1928)=-0.744536094771D+04
coeff(1929)=-0.290004913825D+04
coeff(1930)=-0.506125877936D+04
coeff(1931)=-0.354547996961D+04
coeff(1932)=-0.125968743856D+04
coeff(1933)=-0.191882486553D+04
coeff(1934)=0.270057117423D+04
coeff(1935)=-0.664133188041D+04
coeff(1936)=-0.399406758942D+04
coeff(1937)=0.296924289670D+04
coeff(1938)=0.454408091152D+04
coeff(1939)=0.213163516701D+04
coeff(1940)=-0.352443108409D+04
coeff(1941)=-0.413728216785D+04
coeff(1942)=-0.871257674002D+02
coeff(1943)=0.568541051948D+04
coeff(1944)=0.507244407248D+03
coeff(1945)=-0.335035953784D+04
coeff(1946)=0.639063105821D+04
coeff(1947)=0.690303564791D+04
coeff(1948)=0.343613007887D+04
coeff(1949)=-0.433968285381D+04
coeff(1950)=0.139512929427D+04
coeff(1951)=-0.327039782357D+04
coeff(1952)=0.198532582192D+04
coeff(1953)=0.202311703749D+04
coeff(1954)=-0.131290943355D+04
coeff(1955)=0.166730105750D+04
coeff(1956)=0.636890925273D+04
coeff(1957)=-0.119595017075D+04
coeff(1958)=-0.197379643102D+03
coeff(1959)=0.207431417683D+04
coeff(1960)=-0.734628098016D+04
coeff(1961)=-0.597246574410D+04
coeff(1962)=-0.416054670780D+04
coeff(1963)=0.100685081831D+04
coeff(1964)=-0.633320251778D+03
coeff(1965)=-0.850597430086D+04
coeff(1966)=-0.191450709783D+02
coeff(1967)=0.195996141026D+04
coeff(1968)=-0.731445856875D+04
coeff(1969)=0.276960719772D+04
coeff(1970)=-0.535478354970D+04
coeff(1971)=0.498788541945D+04
coeff(1972)=0.243044052830D+04
coeff(1973)=-0.124315946859D+04
coeff(1974)=-0.558831332912D+04
coeff(1975)=0.223134652313D+04
coeff(1976)=-0.780653689808D+04
coeff(1977)=-0.395476920373D+04
coeff(1978)=-0.373679531200D+03
coeff(1979)=-0.466268046957D+04
coeff(1980)=-0.779086445424D+04
coeff(1981)=0.109224481968D+05
coeff(1982)=0.131693193146D+04
coeff(1983)=0.115456766355D+05
coeff(1984)=0.292322958265D+04
coeff(1985)=-0.153568026718D+04
coeff(1986)=0.290103594845D+04
coeff(1987)=0.245924888088D+04
coeff(1988)=-0.806994447147D+03
coeff(1989)=0.835387468550D+04
coeff(1990)=0.827034015786D+04
coeff(1991)=-0.548234429694D+04
coeff(1992)=0.562678815449D+04
coeff(1993)=0.966366658574D+04
coeff(1994)=0.483261942469D+04
coeff(1995)=0.104370349227D+05
coeff(1996)=0.105152622258D+03
coeff(1997)=-0.569904795279D+04
coeff(1998)=-0.243721845999D+04
coeff(1999)=-0.558952360352D+04
coeff(2000)=-0.585578134878D+03
coeff(2001)=-0.419488204307D+04
coeff(2002)=-0.100730261381D+04
coeff(2003)=0.416951718689D+04
coeff(2004)=0.686451204024D+04
coeff(2005)=0.205881247203D+04
coeff(2006)=-0.480651230042D+04
coeff(2007)=-0.158119018335D+04
coeff(2008)=-0.662285250249D+04
coeff(2009)=0.660974620055D+04
coeff(2010)=0.223879682417D+04
coeff(2011)=0.448512011568D+04
coeff(2012)=-0.559361713826D+04
coeff(2013)=-0.541862809368D+04
coeff(2014)=0.650166035899D+04
coeff(2015)=0.133530849914D+04
coeff(2016)=-0.561880246410D+04
coeff(2017)=0.587415444283D+04
coeff(2018)=0.311890337339D+04
coeff(2019)=0.264334084876D+04
coeff(2020)=0.281862290927D+04
coeff(2021)=0.341148368487D+04
coeff(2022)=0.108794519643D+04
coeff(2023)=-0.492203085223D+04
coeff(2024)=0.155422283171D+04
coeff(2025)=0.227037634423D+04
coeff(2026)=0.131775728710D+03
coeff(2027)=-0.106065769624D+05
coeff(2028)=0.988094066793D+04
coeff(2029)=-0.275286445585D+04
coeff(2030)=-0.152995057709D+04
coeff(2031)=-0.408752237129D+04
coeff(2032)=-0.192373308637D+04
coeff(2033)=-0.356683374375D+03
coeff(2034)=0.698159909077D+03
coeff(2035)=0.107786292059D+05
coeff(2036)=0.293555865778D+03
coeff(2037)=-0.753344987900D+02
coeff(2038)=-0.104925335615D+05
coeff(2039)=0.314836744994D+04
coeff(2040)=0.111368466293D+05
coeff(2041)=0.707241411974D+04
coeff(2042)=-0.142003210581D+04
coeff(2043)=0.744895281616D+04
coeff(2044)=-0.953936060590D+04
coeff(2045)=-0.107678838419D+04
coeff(2046)=-0.551263134527D+03
coeff(2047)=-0.910052165316D+03
coeff(2048)=0.142144418596D+05
coeff(2049)=-0.797756059826D+04
coeff(2050)=-0.536521428573D+04
coeff(2051)=0.107161020251D+04
coeff(2052)=-0.368677599844D+04
coeff(2053)=-0.158325902067D+04
coeff(2054)=0.177702462627D+05
coeff(2055)=-0.357542719472D+04
coeff(2056)=-0.501465725727D+04
coeff(2057)=0.133017452359D+04
coeff(2058)=0.199911297887D+04
coeff(2059)=-0.514077055629D+04
coeff(2060)=-0.130357181669D+04
coeff(2061)=-0.115521942022D+04
coeff(2062)=-0.124778871548D+05
coeff(2063)=0.533433198246D+04
coeff(2064)=-0.306814928984D+04
coeff(2065)=0.806319178153D+04
coeff(2066)=-0.724193718699D+03
coeff(2067)=0.428988976516D+04
coeff(2068)=0.481340914223D+04
coeff(2069)=0.551208944845D+04
coeff(2070)=-0.251596729494D+04
coeff(2071)=-0.495771517530D+02
coeff(2072)=-0.648095102539D+04
coeff(2073)=-0.530961427605D+04
coeff(2074)=-0.165609003282D+04
coeff(2075)=0.416731771235D+04
coeff(2076)=0.418213110798D+04
coeff(2077)=0.436148562562D+04
coeff(2078)=0.918418603709D+04
coeff(2079)=-0.770440783218D+04
coeff(2080)=0.920806985176D+04
coeff(2081)=0.181040984405D+04
coeff(2082)=0.406829539863D+04
coeff(2083)=-0.529988404774D+04
coeff(2084)=-0.931368036866D+04
coeff(2085)=-0.969596495353D+04
coeff(2086)=0.416989353906D+04
coeff(2087)=0.733716009425D+04
coeff(2088)=0.110406162728D+04
coeff(2089)=0.823785315729D+04
coeff(2090)=0.528193560091D+04
coeff(2091)=-0.564446816977D+04
coeff(2092)=-0.343285059154D+03
coeff(2093)=0.276760496186D+04
coeff(2094)=-0.112949588919D+03
coeff(2095)=-0.195609113495D+04
coeff(2096)=0.148583381887D+04
coeff(2097)=-0.722918613184D+04
coeff(2098)=-0.356479774628D+04
coeff(2099)=-0.148723675905D+05
coeff(2100)=0.107261115348D+04
coeff(2101)=0.235747700238D+04
coeff(2102)=-0.359746266299D+04
coeff(2103)=0.377143148977D+03
coeff(2104)=-0.560683265922D+04
coeff(2105)=0.250584250175D+04
coeff(2106)=0.663320523308D+03
coeff(2107)=-0.248820882533D+04
coeff(2108)=0.448947573462D+04
coeff(2109)=0.296918988664D+04
coeff(2110)=-0.152325335140D+04
coeff(2111)=0.732554220551D+04
coeff(2112)=-0.255007913249D+04
coeff(2113)=-0.304014893503D+04
coeff(2114)=-0.154464209027D+04
coeff(2115)=0.532802285071D+04
coeff(2116)=0.547382526354D+03
coeff(2117)=0.359649899627D+03
coeff(2118)=-0.156310782776D+04
coeff(2119)=-0.491527898076D+04
coeff(2120)=0.181363402621D+04
coeff(2121)=-0.108103704225D+04
coeff(2122)=-0.428183037236D+04
coeff(2123)=0.164188843115D+05
coeff(2124)=0.144998140401D+04
coeff(2125)=-0.170797625190D+04
coeff(2126)=-0.464834897729D+04
coeff(2127)=-0.410622125672D+04
coeff(2128)=-0.342313273872D+04
coeff(2129)=0.325509036002D+03
coeff(2130)=0.104883519368D+04
coeff(2131)=0.396160710477D+04
coeff(2132)=0.976925223794D+04
coeff(2133)=0.669523027421D+04
coeff(2134)=0.628538590789D+04
coeff(2135)=-0.945040543625D+04
coeff(2136)=-0.467670014301D+04
coeff(2137)=-0.218689899997D+04
coeff(2138)=-0.561280041133D+04
coeff(2139)=0.604504282977D+04
coeff(2140)=-0.511878537301D+04
coeff(2141)=0.477304674079D+04
coeff(2142)=0.362058806551D+04
coeff(2143)=-0.332777196450D+04
coeff(2144)=0.607013805273D+03
coeff(2145)=-0.193279622789D+05
coeff(2146)=0.261627308591D+04
coeff(2147)=-0.671971586429D+04
coeff(2148)=0.180392109562D+04
coeff(2149)=0.351719587313D+04
coeff(2150)=0.536701032515D+04
coeff(2151)=-0.100036605938D+05
coeff(2152)=-0.728094259394D+04
coeff(2153)=-0.763355728014D+04
coeff(2154)=0.658033564846D+04
coeff(2155)=-0.100127686738D+04
coeff(2156)=-0.224715449670D+04
coeff(2157)=0.152819215930D+04
coeff(2158)=-0.326392863279D+04
coeff(2159)=-0.396205063596D+03
coeff(2160)=0.146842445620D+04
coeff(2161)=0.292960289445D+04
coeff(2162)=0.804677659705D+04
coeff(2163)=-0.411226063207D+04
coeff(2164)=0.175745331172D+03
coeff(2165)=0.306227080142D+04
coeff(2166)=-0.371321388605D+04
coeff(2167)=-0.813771894122D+04
coeff(2168)=0.390794372240D+04
coeff(2169)=0.348877565211D+04
coeff(2170)=0.522625174031D+04
coeff(2171)=-0.146988270491D+04
coeff(2172)=-0.105899874681D+05
coeff(2173)=0.595418186057D+03
coeff(2174)=-0.587286455688D+04
coeff(2175)=0.588106963212D+04
coeff(2176)=0.807874661403D+03
coeff(2177)=0.104580247464D+05
coeff(2178)=0.133058177035D+03
coeff(2179)=-0.191002285971D+04
coeff(2180)=0.259838966950D+04
coeff(2181)=0.661586368113D+04
coeff(2182)=-0.248312525277D+04
coeff(2183)=-0.113677905545D+04
coeff(2184)=-0.526602454110D+04
coeff(2185)=0.132174530382D+05
coeff(2186)=-0.329444459769D+04
coeff(2187)=-0.942812329602D+04
coeff(2188)=-0.515286707083D+04
coeff(2189)=0.112668736710D+05
coeff(2190)=-0.341833928177D+04
coeff(2191)=0.337406727501D+04
coeff(2192)=0.313827965688D+04
coeff(2193)=-0.550095052695D+04
coeff(2194)=-0.965028833402D+04
coeff(2195)=0.761863465026D+04
coeff(2196)=0.115033977012D+04
coeff(2197)=0.506107031518D+04
coeff(2198)=0.706904209963D+04
coeff(2199)=0.309541799716D+03
coeff(2200)=0.343916068150D+04
coeff(2201)=0.317044730433D+03
coeff(2202)=0.760192782512D+04
coeff(2203)=-0.460745523594D+04
coeff(2204)=0.398437599002D+04
coeff(2205)=-0.451013668109D+04
coeff(2206)=-0.324211087933D+04
coeff(2207)=-0.848014048688D+04
coeff(2208)=-0.439344992346D+04
coeff(2209)=-0.461040779488D+04
coeff(2210)=0.985123889023D+03
coeff(2211)=-0.125517150387D+04
coeff(2212)=-0.947015935007D+04
coeff(2213)=-0.813621066922D+03
coeff(2214)=0.230999385403D+04
coeff(2215)=0.738326209975D+04
coeff(2216)=-0.281203992644D+04
coeff(2217)=0.113065282578D+05
coeff(2218)=0.495184346413D+04
coeff(2219)=-0.291845665395D+04
coeff(2220)=0.601259450000D+04
coeff(2221)=-0.849511448248D+04
coeff(2222)=-0.303811988444D+04
coeff(2223)=-0.596949761512D+04
coeff(2224)=0.103011696784D+05
coeff(2225)=-0.117609106989D+04
coeff(2226)=-0.103186567835D+04
coeff(2227)=-0.266700309665D+04
coeff(2228)=-0.348962161080D+04
coeff(2229)=-0.574919137098D+04
coeff(2230)=0.185110264149D+04
coeff(2231)=-0.699625831850D+04
coeff(2232)=-0.753001122072D+04
coeff(2233)=0.274324972908D+04
coeff(2234)=-0.341024511191D+04
coeff(2235)=-0.650756183322D+04
coeff(2236)=0.595668625393D+04
coeff(2237)=-0.570252149192D+04
coeff(2238)=-0.481109811723D+03
coeff(2239)=0.233895851173D+04
coeff(2240)=0.509424503111D+04
coeff(2241)=0.774886091306D+04
coeff(2242)=0.914215000839D+04
coeff(2243)=-0.121342918166D+05
coeff(2244)=-0.810178657058D+04
coeff(2245)=0.383877049091D+04
coeff(2246)=0.133003773772D+05
coeff(2247)=0.521052038881D+04
coeff(2248)=-0.811141747024D+04
coeff(2249)=-0.133097765512D+05
coeff(2250)=-0.834081242501D+04
coeff(2251)=-0.835654516584D+04
coeff(2252)=0.398139858102D+04
coeff(2253)=-0.786940976740D+04
coeff(2254)=0.134785063045D+04
coeff(2255)=0.104362135306D+04
coeff(2256)=-0.340839565231D+03
coeff(2257)=-0.936086692576D+04
coeff(2258)=-0.127245241920D+05
coeff(2259)=0.350760469225D+04
coeff(2260)=-0.180251065015D+04
coeff(2261)=-0.469069344422D+04
coeff(2262)=-0.204801397528D+04
coeff(2263)=-0.896919266194D+03
coeff(2264)=0.411365635735D+03
coeff(2265)=0.643927008948D+04
coeff(2266)=0.620320108187D+03
coeff(2267)=0.507024901314D+04
coeff(2268)=-0.614321149636D+04
coeff(2269)=0.103645022878D+04
coeff(2270)=0.315228320166D+03
coeff(2271)=-0.157078815148D+04
coeff(2272)=0.893913734052D+04
coeff(2273)=-0.906390162451D+04
coeff(2274)=0.634449107059D+04
coeff(2275)=0.335274880756D+04
coeff(2276)=0.158295722800D+04
coeff(2277)=-0.100141689686D+04
coeff(2278)=0.230472700700D+04
coeff(2279)=0.250655154002D+04
coeff(2280)=-0.223922923859D+03
coeff(2281)=-0.696387646186D+04
coeff(2282)=0.803703350940D+03
coeff(2283)=0.625359700918D+04
coeff(2284)=-0.189491576741D+04
coeff(2285)=0.201917643793D+04
coeff(2286)=-0.678730814307D+04
coeff(2287)=0.635097455805D+04
coeff(2288)=0.380833996453D+04
coeff(2289)=-0.104253274549D+04
coeff(2290)=0.116490192781D+04
coeff(2291)=0.212778160737D+04
coeff(2292)=-0.691391988406D+04
coeff(2293)=-0.163734054179D+04
coeff(2294)=-0.138529568387D+04
coeff(2295)=0.740916763170D+04
coeff(2296)=-0.717295981100D+04
coeff(2297)=-0.313917991527D+04
coeff(2298)=-0.310760078268D+04
coeff(2299)=-0.542971875012D+04
coeff(2300)=0.354822753631D+04
coeff(2301)=0.467387992303D+04
coeff(2302)=0.927941776070D+03
coeff(2303)=0.404931472446D+04
coeff(2304)=0.513605548983D+04
coeff(2305)=-0.231924543268D+04
coeff(2306)=0.311345093977D+04
coeff(2307)=0.399633855262D+04
coeff(2308)=0.124268986328D+05
coeff(2309)=0.232497117105D+04
coeff(2310)=0.795891055137D+04
coeff(2311)=-0.163208336066D+04
coeff(2312)=0.252713672297D+04
coeff(2313)=0.296483761797D+04
coeff(2314)=-0.501303657264D+03
coeff(2315)=-0.706341412925D+02
coeff(2316)=-0.193438948628D+03
coeff(2317)=-0.211923826699D+03
coeff(2318)=-0.344891149458D+04
coeff(2319)=-0.207425065557D+04
coeff(2320)=0.181247316354D+04
coeff(2321)=-0.430885972228D+04
coeff(2322)=0.143698732250D+05
coeff(2323)=0.414380730928D+04
coeff(2324)=-0.716529174836D+03
coeff(2325)=-0.723640067444D+04
coeff(2326)=-0.382429868054D+04
coeff(2327)=0.623071719025D+04
coeff(2328)=0.248526150778D+04
coeff(2329)=-0.421625820199D+02
coeff(2330)=-0.874555773519D+04
coeff(2331)=-0.414741223656D+04
coeff(2332)=-0.478371120556D+04
coeff(2333)=0.914968210324D+04
coeff(2334)=0.262772720421D+04
coeff(2335)=0.164927581525D+04
coeff(2336)=0.255923689781D+04
coeff(2337)=-0.173537267429D+04
coeff(2338)=0.236963942264D+02
coeff(2339)=-0.901878158578D+03
coeff(2340)=-0.737189373990D+04
coeff(2341)=0.701829807766D+04
coeff(2342)=0.453233172963D+04
coeff(2343)=-0.105355225302D+03
coeff(2344)=-0.498464955832D+03
coeff(2345)=0.105751749839D+05
coeff(2346)=0.757769039757D+04
coeff(2347)=0.777753686001D+03
coeff(2348)=-0.492703752413D+03
coeff(2349)=0.737215624842D+04
coeff(2350)=-0.313458407326D+03
coeff(2351)=0.151570058092D+04
coeff(2352)=0.534020119754D+03
coeff(2353)=-0.484357180561D+04
coeff(2354)=0.181449890574D+04
coeff(2355)=0.864220009426D+04
coeff(2356)=-0.891785287831D+03
coeff(2357)=0.448831833828D+04
coeff(2358)=-0.565794714606D+04
coeff(2359)=-0.352478359296D+04
coeff(2360)=-0.128788186635D+05
coeff(2361)=-0.219092158892D+04
coeff(2362)=0.887636514835D+03
coeff(2363)=0.207767570367D+03
coeff(2364)=0.124827995498D+04
coeff(2365)=-0.807783812599D+04
coeff(2366)=0.293095722207D+04
coeff(2367)=0.274656830737D+04
coeff(2368)=0.934350578684D+04
coeff(2369)=-0.430042419061D+04
coeff(2370)=0.199556046040D+04
coeff(2371)=0.191308793761D+04
coeff(2372)=0.981611104688D+03
coeff(2373)=-0.290448803248D+04
coeff(2374)=-0.396048265390D+04
coeff(2375)=-0.704800510818D+04
coeff(2376)=0.155254726006D+04
coeff(2377)=0.140170413484D+04
coeff(2378)=-0.719901510703D+04
coeff(2379)=-0.556346037866D+04
coeff(2380)=0.249259725073D+04
coeff(2381)=-0.327529196707D+04
coeff(2382)=0.510303162681D+04
coeff(2383)=-0.655723043046D+04
coeff(2384)=0.258726135904D+04
coeff(2385)=0.199471322125D+04
coeff(2386)=0.197958728300D+04
coeff(2387)=0.161040962752D+05
coeff(2388)=-0.315409733391D+04
coeff(2389)=0.104641704337D+05
coeff(2390)=0.100093529853D+04
coeff(2391)=-0.609166604810D+04
coeff(2392)=-0.458107242594D+04
coeff(2393)=0.641410521700D+04
coeff(2394)=0.646253330630D+04
coeff(2395)=-0.353354897436D+04
coeff(2396)=-0.771019154551D+04
coeff(2397)=-0.101843684372D+05
coeff(2398)=0.817300218509D+04
coeff(2399)=-0.403472988797D+04
coeff(2400)=-0.120468074788D+05
coeff(2401)=0.615553556400D+04
coeff(2402)=-0.627328472753D+02
coeff(2403)=0.110901919330D+03
coeff(2404)=-0.499509434103D+04
coeff(2405)=-0.326981001935D+03
coeff(2406)=-0.140061637723D+04
coeff(2407)=0.520097024469D+04
coeff(2408)=-0.207545820775D+04
coeff(2409)=-0.205063361221D+04
coeff(2410)=0.263109109727D+04
coeff(2411)=-0.135647389838D+05
coeff(2412)=-0.425392927152D+04
coeff(2413)=-0.682309507236D+04
coeff(2414)=0.146411794661D+03
coeff(2415)=0.815722427399D+04
coeff(2416)=-0.627148119679D+04
coeff(2417)=-0.167044649105D+04
coeff(2418)=-0.317594691882D+04
coeff(2419)=0.363588082780D+03
coeff(2420)=-0.410974466262D+04
coeff(2421)=0.378158415269D+04
coeff(2422)=0.303968744512D+04
coeff(2423)=0.716505886668D+04
coeff(2424)=-0.631911049503D+04
coeff(2425)=0.443563061956D+04
coeff(2426)=0.747111142787D+04
coeff(2427)=-0.116622075493D+02
coeff(2428)=-0.236469276634D+04
coeff(2429)=0.709290358837D+04
coeff(2430)=-0.276695529004D+03
coeff(2431)=-0.489287028476D+04
coeff(2432)=-0.282086163289D+04
coeff(2433)=0.989963136751D+03
coeff(2434)=0.807214757730D+04
coeff(2435)=0.222937441697D+04
coeff(2436)=0.421650313600D+03
coeff(2437)=0.664157669983D+04
coeff(2438)=-0.145775896697D+03
coeff(2439)=-0.382273705463D+04
coeff(2440)=-0.524559955657D+04
coeff(2441)=-0.169617473983D+04
coeff(2442)=-0.115149673322D+05
coeff(2443)=-0.808865657474D+03
coeff(2444)=-0.520425700519D+04
coeff(2445)=0.415098542525D+04
coeff(2446)=-0.207692356661D+04
coeff(2447)=-0.658988373807D+04
coeff(2448)=0.782225031757D+04
coeff(2449)=-0.858493488885D+04
coeff(2450)=0.623728389068D+04
coeff(2451)=0.695554630454D+04
coeff(2452)=-0.313856078833D+04
coeff(2453)=0.979229826615D+03
coeff(2454)=0.505715840216D+04
coeff(2455)=-0.517546882346D+04
coeff(2456)=-0.616944196297D+03
coeff(2457)=-0.207779058938D+04
coeff(2458)=0.329022601175D+04
coeff(2459)=0.101129278635D+05
coeff(2460)=-0.319419751300D+03
coeff(2461)=0.112451655003D+05
coeff(2462)=-0.806874765046D+04
coeff(2463)=-0.325783332425D+04
coeff(2464)=-0.131109728884D+05
coeff(2465)=-0.747300769436D+04
coeff(2466)=-0.557219683383D+02
coeff(2467)=-0.120050731600D+04
coeff(2468)=0.624156967015D+04
coeff(2469)=-0.200970152921D+04
coeff(2470)=-0.231897145605D+04
coeff(2471)=0.799080764311D+02
coeff(2472)=0.477525598399D+04
coeff(2473)=-0.613978344127D+04
coeff(2474)=0.986628867534D+03
coeff(2475)=0.587959830950D+02
coeff(2476)=0.127352047031D+05
coeff(2477)=-0.154604148617D+04
coeff(2478)=0.879692901533D+03
coeff(2479)=-0.628663134357D+04
coeff(2480)=0.167200190336D+04
coeff(2481)=-0.800025881395D+04
coeff(2482)=0.751201003045D+04
coeff(2483)=-0.603711392068D+04
coeff(2484)=0.350345027056D+04
coeff(2485)=-0.266263268914D+04
coeff(2486)=0.405698373005D+04
coeff(2487)=0.690742945896D+04
coeff(2488)=0.242648950729D+04
coeff(2489)=-0.111973373228D+05
coeff(2490)=0.569718759609D+04
coeff(2491)=0.656727473807D+04
coeff(2492)=0.599783608820D+03
coeff(2493)=0.219484158302D+04
coeff(2494)=-0.689561175900D+03
coeff(2495)=-0.250272111788D+04
coeff(2496)=0.439018591439D+03
coeff(2497)=0.343743196314D+03
coeff(2498)=-0.198822299239D+04
coeff(2499)=0.521341713563D+04
coeff(2500)=-0.208282464355D+04
coeff(2501)=-0.267137551800D+04
coeff(2502)=0.148792149546D+05
coeff(2503)=-0.481027047932D+04
coeff(2504)=0.822473255488D+04
coeff(2505)=-0.461851763955D+04
coeff(2506)=-0.679881822993D+04
coeff(2507)=0.498564133518D+04
coeff(2508)=-0.125890765704D+04
coeff(2509)=-0.498423823829D+04
coeff(2510)=0.117128036202D+05
coeff(2511)=0.321161910005D+04
coeff(2512)=-0.778869381721D+04
coeff(2513)=-0.137945866764D+04
coeff(2514)=-0.462843642188D+04
coeff(2515)=0.256889766561D+04
coeff(2516)=-0.142222938727D+04
coeff(2517)=0.127324670544D+04
coeff(2518)=0.264972986640D+04
coeff(2519)=-0.503023755967D+04
coeff(2520)=0.104496978865D+05
coeff(2521)=-0.462109463780D+04
coeff(2522)=0.878418606682D+04
coeff(2523)=-0.976886478575D+04
coeff(2524)=0.663019265476D+04
coeff(2525)=-0.250624773592D+04
coeff(2526)=-0.119925534370D+04
coeff(2527)=0.196570061701D+04
coeff(2528)=0.141354206133D+04
coeff(2529)=-0.309204034696D+03
coeff(2530)=-0.597983161195D+03
coeff(2531)=0.426640000426D+04
coeff(2532)=-0.379617017928D+04
coeff(2533)=0.924786642898D+04
coeff(2534)=-0.506795579163D+04
coeff(2535)=-0.152301638841D+04
coeff(2536)=0.321721707576D+04
coeff(2537)=-0.295246799109D+04
coeff(2538)=-0.129823280862D+05
coeff(2539)=0.175456286899D+04
coeff(2540)=0.495383748823D+04
coeff(2541)=-0.155178111018D+04
coeff(2542)=-0.468773983439D+04
coeff(2543)=-0.946230214582D+04
coeff(2544)=-0.671887203876D+04
coeff(2545)=0.550226476533D+04
coeff(2546)=-0.227400040603D+03
coeff(2547)=0.236657886290D+04
coeff(2548)=-0.261343567276D+04
coeff(2549)=-0.487951378398D+04
coeff(2550)=0.198473394455D+04
coeff(2551)=0.188803416218D+04
coeff(2552)=-0.451267159500D+04
coeff(2553)=0.226150364986D+04
coeff(2554)=-0.311825623798D+04
coeff(2555)=-0.490315471020D+04
coeff(2556)=0.990583322562D+04
coeff(2557)=0.352359684934D+04
coeff(2558)=0.367165501706D+04
coeff(2559)=0.120446323638D+05
coeff(2560)=0.242233780973D+04
coeff(2561)=-0.371363808878D+04
coeff(2562)=-0.683580653626D+04
coeff(2563)=0.513484312670D+03
coeff(2564)=-0.159519055166D+04
coeff(2565)=-0.864573460377D+03
coeff(2566)=0.431124039201D+04
coeff(2567)=-0.754990096551D+04
coeff(2568)=0.307302557544D+03
coeff(2569)=0.790368199050D+04
coeff(2570)=0.638636287987D+04
coeff(2571)=-0.639570301612D+04
coeff(2572)=0.199407498973D+04
coeff(2573)=-0.328245161860D+04
coeff(2574)=0.928282637138D+04
coeff(2575)=0.360223888355D+04
coeff(2576)=-0.571864435631D+03
coeff(2577)=-0.345246443949D+04
coeff(2578)=0.133745112348D+03
coeff(2579)=0.782770559830D+03
coeff(2580)=-0.104033828782D+04
coeff(2581)=0.382414268427D+04
coeff(2582)=-0.320520965865D+04
coeff(2583)=0.628743985891D+04
coeff(2584)=-0.606865351245D+03
coeff(2585)=-0.160459229083D+05
coeff(2586)=-0.402139273924D+04
coeff(2587)=0.170412019742D+04
coeff(2588)=-0.167548891146D+04
coeff(2589)=0.295211220060D+04
coeff(2590)=-0.711172160825D+04
coeff(2591)=0.252377787916D+04
coeff(2592)=0.505377384533D+04
coeff(2593)=-0.911031629160D+04
coeff(2594)=0.109581598294D+04
coeff(2595)=0.104820020831D+05
coeff(2596)=-0.257568736504D+04
coeff(2597)=-0.756833784511D+03
coeff(2598)=0.574990561412D+03
coeff(2599)=0.271301519857D+04
coeff(2600)=-0.193146591011D+04
coeff(2601)=0.723223457048D+04
coeff(2602)=-0.345654251659D+04
coeff(2603)=0.759767127819D+03
coeff(2604)=0.837448752940D+03
coeff(2605)=0.492441758664D+04
coeff(2606)=0.100069971485D+05
coeff(2607)=-0.375455621478D+04
coeff(2608)=-0.750225277064D+04
coeff(2609)=0.178603244244D+04
coeff(2610)=-0.492309935307D+04
coeff(2611)=0.279168511371D+04
coeff(2612)=-0.688066025204D+04
coeff(2613)=0.742001944952D+04
coeff(2614)=0.459637291676D+04
coeff(2615)=0.968838214708D+03
coeff(2616)=0.638323949600D+03
coeff(2617)=0.659087948854D+04
coeff(2618)=0.166869022533D+04
coeff(2619)=-0.237205806744D+03
coeff(2620)=-0.303226361894D+04
coeff(2621)=-0.718738284558D+04
coeff(2622)=0.999672364676D+04
coeff(2623)=0.324978876538D+04
coeff(2624)=0.376783539084D+04
coeff(2625)=0.274966091146D+04
coeff(2626)=0.112509765305D+04
coeff(2627)=-0.285581749030D+04
coeff(2628)=-0.506735802031D+04
coeff(2629)=0.154072800410D+05
coeff(2630)=-0.197260755468D+04
coeff(2631)=-0.387460936216D+04
coeff(2632)=0.257930179094D+04
coeff(2633)=-0.286559519074D+04
coeff(2634)=0.307009710200D+04
coeff(2635)=0.145907296089D+05
coeff(2636)=0.368016010200D+04
coeff(2637)=0.851306187212D+04
coeff(2638)=0.383596243019D+04
coeff(2639)=0.335571301258D+04
coeff(2640)=-0.223807280433D+04
coeff(2641)=0.820693565714D+04
coeff(2642)=-0.978192595115D+04
coeff(2643)=0.177740908136D+04
coeff(2644)=0.364348123161D+04
coeff(2645)=-0.622777902863D+04
coeff(2646)=-0.174413608379D+02
coeff(2647)=-0.627888887497D+04
coeff(2648)=-0.161204086537D+04
coeff(2649)=0.337318230276D+04
coeff(2650)=0.842699604150D+04
coeff(2651)=-0.481287330029D+04
coeff(2652)=-0.288567938052D+04
coeff(2653)=0.287459381221D+04
coeff(2654)=-0.545307981804D+04
coeff(2655)=0.533177310497D+04
coeff(2656)=0.288552072913D+04
coeff(2657)=0.324048872373D+04
coeff(2658)=-0.618065803667D+04
coeff(2659)=-0.414528668943D+04
coeff(2660)=-0.891201224749D+04
coeff(2661)=-0.508870106120D+04
coeff(2662)=0.448811049614D+04
coeff(2663)=-0.833612157964D+04
coeff(2664)=-0.414921383575D+04
coeff(2665)=-0.108451538018D+04
coeff(2666)=0.136030580369D+05
coeff(2667)=-0.187702781527D+04
coeff(2668)=-0.243940151661D+03
coeff(2669)=-0.261537642105D+03
coeff(2670)=0.803618487712D+03
coeff(2671)=0.187568625225D+04
coeff(2672)=0.577380133217D+04
coeff(2673)=0.763631490622D+04
coeff(2674)=0.351016601970D+04
coeff(2675)=-0.668131094128D+02
coeff(2676)=0.227762286439D+03
coeff(2677)=-0.445359022695D+04
coeff(2678)=-0.115107819924D+05
coeff(2679)=-0.923815178370D+04
coeff(2680)=0.735622994303D+04
coeff(2681)=-0.508321525370D+04
coeff(2682)=-0.934995282353D+04
coeff(2683)=-0.791426230952D+04
coeff(2684)=-0.173929056258D+03
coeff(2685)=-0.764423379662D+04
coeff(2686)=-0.753920544556D+04
coeff(2687)=0.356092413232D+04
coeff(2688)=-0.135367775317D+04
coeff(2689)=-0.555105846941D+04
coeff(2690)=-0.316432100033D+04
coeff(2691)=-0.228289120448D+04
coeff(2692)=-0.381311543058D+04
coeff(2693)=0.430375898313D+04
coeff(2694)=-0.102948948085D+05
coeff(2695)=0.608406458518D+04
coeff(2696)=-0.696117398084D+04
coeff(2697)=-0.547126869688D+04
coeff(2698)=-0.563662784213D+04
coeff(2699)=0.256541357478D+04
coeff(2700)=0.108050695092D+04
coeff(2701)=0.261770526778D+04
coeff(2702)=-0.420317856353D+04
coeff(2703)=-0.301551082629D+04
coeff(2704)=-0.444491775495D+04
coeff(2705)=0.913026471741D+04
coeff(2706)=-0.325945058377D+04
coeff(2707)=-0.716011462109D+03
coeff(2708)=0.565818861289D+04
coeff(2709)=0.202666687986D+04
coeff(2710)=0.297621062994D+04
coeff(2711)=0.808021673352D+03
coeff(2712)=-0.127380045931D+05
coeff(2713)=-0.189952161154D+04
coeff(2714)=-0.768530038679D+04
coeff(2715)=-0.160276455890D+05
coeff(2716)=0.114701648895D+05
coeff(2717)=-0.454394960926D+04
coeff(2718)=-0.385410333397D+02
coeff(2719)=0.701674363953D+03
coeff(2720)=-0.209914183063D+04
coeff(2721)=0.329754342226D+04
coeff(2722)=0.362869465454D+04
coeff(2723)=-0.170697701533D+03
coeff(2724)=-0.237314007930D+04
coeff(2725)=0.491775410378D+04
coeff(2726)=0.112693741834D+05
coeff(2727)=-0.871215356689D+04
coeff(2728)=0.222973285776D+04
coeff(2729)=-0.443864073155D+04
coeff(2730)=-0.584647592529D+03
coeff(2731)=-0.333042110539D+04
coeff(2732)=-0.402029941073D+04
coeff(2733)=-0.539392898597D+04
coeff(2734)=-0.444996976150D+04
coeff(2735)=0.691903698832D+04
coeff(2736)=-0.581004642397D+04
coeff(2737)=0.102718780733D+05
coeff(2738)=0.949871269600D+04
coeff(2739)=0.109169904602D+05
coeff(2740)=0.239252710793D+03
coeff(2741)=-0.644196332485D+04
coeff(2742)=-0.287649529625D+04
coeff(2743)=0.219463490962D+04
coeff(2744)=0.164060768269D+04
coeff(2745)=-0.945266709634D+03
coeff(2746)=0.139518405889D+04
coeff(2747)=-0.133576308163D+04
coeff(2748)=-0.535188205426D+04
coeff(2749)=0.151721608734D+04
coeff(2750)=-0.349231124670D+04
coeff(2751)=0.965918151096D+04
coeff(2752)=-0.665173856572D+04
coeff(2753)=-0.266838362559D+04
coeff(2754)=-0.146491347052D+04
coeff(2755)=0.867508821750D+04
coeff(2756)=0.571493788708D+04
coeff(2757)=0.112153594458D+03
coeff(2758)=-0.519622748190D+04
coeff(2759)=0.248818314494D+04
coeff(2760)=0.772235918654D+04
coeff(2761)=-0.307399912432D+04
coeff(2762)=-0.341714111403D+04
coeff(2763)=-0.995092920642D+03
coeff(2764)=-0.112124565330D+05
coeff(2765)=-0.266252999684D+04
coeff(2766)=-0.490960925594D+04
coeff(2767)=-0.456425905617D+04
coeff(2768)=0.330844850505D+04
coeff(2769)=-0.525761915365D+04
coeff(2770)=0.800597895015D+04
coeff(2771)=0.142203879216D+05
coeff(2772)=0.540166716463D+04
coeff(2773)=0.516479358394D+04
coeff(2774)=0.119055487806D+04
coeff(2775)=-0.797399528936D+04
coeff(2776)=0.628249398030D+04
coeff(2777)=-0.140046606304D+04
coeff(2778)=0.550279383587D+03
coeff(2779)=0.429856963489D+04
coeff(2780)=0.503163310760D+03
coeff(2781)=0.507795504335D+03
coeff(2782)=0.125723951697D+05
coeff(2783)=-0.164417110812D+04
coeff(2784)=0.214439803000D+04
coeff(2785)=0.354058695452D+04
coeff(2786)=-0.131263472069D+05
coeff(2787)=0.339882660509D+02
coeff(2788)=0.891799314064D+04
coeff(2789)=0.663898538719D+04
coeff(2790)=0.490650406623D+04
coeff(2791)=0.200187481962D+04
coeff(2792)=0.536017295486D+04
coeff(2793)=-0.136506225999D+05
coeff(2794)=-0.610368236337D+04
coeff(2795)=0.336225869636D+04
coeff(2796)=0.529861519499D+04
coeff(2797)=-0.658122658218D+03
coeff(2798)=0.209921987438D+04
coeff(2799)=-0.753359747150D+04
coeff(2800)=-0.387251140826D+04
coeff(2801)=0.855342972594D+04
coeff(2802)=0.851721580001D+04
coeff(2803)=-0.719994047079D+04
coeff(2804)=0.140443149571D+04
coeff(2805)=0.562297992148D+04
coeff(2806)=-0.660937910290D+04
coeff(2807)=0.133424195634D+04
coeff(2808)=-0.971543314436D+04
coeff(2809)=0.956050284063D+04
coeff(2810)=0.490471125012D+04
coeff(2811)=-0.485463704025D+04
coeff(2812)=0.421138920669D+04
coeff(2813)=-0.462965062708D+04
coeff(2814)=0.600258838324D+04
coeff(2815)=-0.406595435891D+04
coeff(2816)=-0.217741238126D+04
coeff(2817)=-0.183614035364D+04
coeff(2818)=-0.405603422703D+03
coeff(2819)=-0.446893466336D+04
coeff(2820)=-0.573749539439D+04
coeff(2821)=0.162143576870D+05
coeff(2822)=-0.118447853539D+05
coeff(2823)=-0.281977576893D+04
coeff(2824)=-0.416107216562D+04
coeff(2825)=-0.232851081544D+04
coeff(2826)=0.140882821321D+05
coeff(2827)=0.110036998024D+05
coeff(2828)=0.104605892911D+05
coeff(2829)=-0.868082446451D+03
coeff(2830)=0.610732273849D+04
coeff(2831)=0.122776167787D+04
coeff(2832)=-0.515641825097D+04
coeff(2833)=-0.101210593212D+05
coeff(2834)=0.269910041475D+04
coeff(2835)=0.365401431508D+04
coeff(2836)=0.118811737689D+05
coeff(2837)=0.144490081507D+04
coeff(2838)=-0.173793700578D+04
coeff(2839)=-0.259060232757D+04
coeff(2840)=0.410972358445D+04
coeff(2841)=0.152225209608D+04
coeff(2842)=0.965136001993D+04
coeff(2843)=-0.804047261909D+03
coeff(2844)=0.169842531271D+05
coeff(2845)=-0.564044884227D+04
coeff(2846)=-0.758686477506D+03
coeff(2847)=-0.111619823657D+05
coeff(2848)=-0.808083945966D+03
coeff(2849)=-0.353651277464D+04
coeff(2850)=0.212582205013D+04
coeff(2851)=-0.694769900709D+04
coeff(2852)=-0.402825890148D+04
coeff(2853)=-0.157656855567D+04
coeff(2854)=-0.174082614868D+04
coeff(2855)=0.185597784172D+03
coeff(2856)=0.641186983495D+02
coeff(2857)=0.216031048655D+04
coeff(2858)=-0.367228462704D+03
coeff(2859)=-0.698181454172D+04
coeff(2860)=-0.402038509637D+04
coeff(2861)=0.635764948831D+04
coeff(2862)=-0.688457769229D+03
coeff(2863)=-0.679397320802D+04
coeff(2864)=0.209543441304D+03
coeff(2865)=-0.307630296893D+03
coeff(2866)=0.401549176962D+04
coeff(2867)=-0.265274199886D+03
coeff(2868)=-0.241443517942D+04
coeff(2869)=-0.103386428489D+05
coeff(2870)=-0.134599690878D+05
coeff(2871)=-0.653411817968D+04
coeff(2872)=0.339814098490D+04
coeff(2873)=0.705217858584D+04
coeff(2874)=0.153167175235D+04
coeff(2875)=-0.293538669243D+04
coeff(2876)=0.408185257938D+04
coeff(2877)=-0.894429590079D+03
coeff(2878)=-0.100797186151D+04
coeff(2879)=0.560515967928D+04
coeff(2880)=-0.525909343972D+04
coeff(2881)=-0.895929012913D+04
coeff(2882)=-0.856069044334D+04
coeff(2883)=-0.293035628958D+04
coeff(2884)=0.248884299494D+04
coeff(2885)=0.341109844477D+04
coeff(2886)=0.212396213004D+04
coeff(2887)=0.976765598821D+03
coeff(2888)=0.346947131732D+04
coeff(2889)=0.114503391872D+05
coeff(2890)=-0.169189934634D+04
coeff(2891)=-0.858902039533D+04
coeff(2892)=-0.335294684210D+04
coeff(2893)=-0.117799960145D+04
coeff(2894)=-0.271260684734D+03
coeff(2895)=0.786478223036D+03
coeff(2896)=0.479671366223D+04
coeff(2897)=0.278828916986D+04
coeff(2898)=0.554239081521D+04
coeff(2899)=0.904609376648D+03
coeff(2900)=0.360698096619D+04
coeff(2901)=-0.261644226231D+04
coeff(2902)=0.634817938550D+04
coeff(2903)=-0.196181541688D+03
coeff(2904)=0.558937011003D+04
coeff(2905)=0.365339376217D+04
coeff(2906)=0.622025730897D+03
coeff(2907)=0.320227311492D+03
coeff(2908)=0.521773791672D+04
coeff(2909)=-0.781787211592D+03
coeff(2910)=0.137017511505D+04
coeff(2911)=0.407419149972D+04
coeff(2912)=0.492142805140D+03
coeff(2913)=-0.141603824872D+05
coeff(2914)=0.567352888517D+04
coeff(2915)=0.904582668641D+04
coeff(2916)=-0.875921929400D+04
coeff(2917)=0.519463858314D+04
coeff(2918)=-0.259851210980D+04
coeff(2919)=0.760161688226D+03
coeff(2920)=-0.615740488490D+03
coeff(2921)=-0.165198838366D+04
coeff(2922)=0.319520210741D+03
coeff(2923)=-0.671372444928D+04
coeff(2924)=-0.951461813318D+03
coeff(2925)=0.398158783263D+04
coeff(2926)=-0.273952308082D+04
coeff(2927)=0.187619569205D+04
coeff(2928)=-0.552698532582D+04
coeff(2929)=-0.282386746269D+04
coeff(2930)=0.352227965219D+04
coeff(2931)=-0.215314689783D+04
coeff(2932)=-0.668980016924D+03
coeff(2933)=-0.215399438657D+03
coeff(2934)=0.156123034911D+04
coeff(2935)=-0.124977369837D+04
coeff(2936)=-0.311119566607D+04
coeff(2937)=0.539309974568D+04
coeff(2938)=-0.293072114342D+04
coeff(2939)=0.148041366150D+04
coeff(2940)=0.119507107659D+04
coeff(2941)=0.711930911678D+04
coeff(2942)=-0.575074553917D+04
coeff(2943)=-0.688530721090D+04
coeff(2944)=0.500513277248D+04
coeff(2945)=0.511188461417D+04
coeff(2946)=-0.135618493659D+04
coeff(2947)=-0.334033829885D+03
coeff(2948)=0.139442208575D+04
coeff(2949)=0.555721103232D+03
coeff(2950)=0.260178112016D+03
coeff(2951)=0.164092685134D+04
coeff(2952)=-0.563870085159D+02
coeff(2953)=0.349904970964D+04
coeff(2954)=0.313703324047D+04
coeff(2955)=-0.776736182639D+04
coeff(2956)=-0.327553802687D+03
coeff(2957)=0.648654139233D+04
coeff(2958)=-0.224501728600D+04
coeff(2959)=-0.541744827237D+04
coeff(2960)=-0.478905496321D+04
coeff(2961)=0.290255696183D+04
coeff(2962)=0.171640978749D+03
coeff(2963)=-0.165120895138D+04
coeff(2964)=0.274101293418D+04
coeff(2965)=-0.148192868409D+04
coeff(2966)=0.803775032658D+03
coeff(2967)=-0.115969022929D+05
coeff(2968)=-0.249750136414D+04
coeff(2969)=-0.178200452910D+04
coeff(2970)=0.425646031483D+04
coeff(2971)=-0.381033070607D+04
coeff(2972)=-0.974944559301D+02
coeff(2973)=0.587185570789D+04
coeff(2974)=-0.376082135851D+04
coeff(2975)=-0.429379648625D+03
coeff(2976)=0.717323949686D+04
coeff(2977)=-0.280699408010D+04
coeff(2978)=-0.286149402814D+03
coeff(2979)=0.404096927440D+04
coeff(2980)=-0.474667006459D+04
coeff(2981)=0.468015498679D+04
coeff(2982)=0.568989336897D+04
coeff(2983)=-0.334606451561D+04
coeff(2984)=0.923638150573D+04
coeff(2985)=0.645182659636D+04
coeff(2986)=0.310818942925D+04
coeff(2987)=0.233225538644D+04
coeff(2988)=-0.373516077781D+04
coeff(2989)=0.680561640275D+04
coeff(2990)=-0.315928262192D+04
coeff(2991)=-0.206064991676D+04
coeff(2992)=0.116747610721D+04
coeff(2993)=-0.169237032766D+04
coeff(2994)=-0.557232777659D+04
coeff(2995)=0.553266174159D+04
coeff(2996)=-0.586432181138D+04
coeff(2997)=0.393015616244D+04
coeff(2998)=0.984850954799D+03
coeff(2999)=-0.513111310098D+04
coeff(3000)=0.848852282333D+04
coeff(3001)=0.238143827175D+04
coeff(3002)=-0.374150590352D+04
coeff(3003)=-0.114972070316D+05
coeff(3004)=0.332955397006D+03
coeff(3005)=-0.601535653135D+04
coeff(3006)=-0.451113575253D+04
coeff(3007)=0.349269451060D+04
coeff(3008)=0.334516836208D+04
coeff(3009)=-0.705225115955D+03
coeff(3010)=0.191360399606D+04
coeff(3011)=-0.294002664057D+03
coeff(3012)=0.890265984020D+03
coeff(3013)=0.717274912531D+04
coeff(3014)=-0.513586611859D+04
coeff(3015)=0.689004316081D+04
coeff(3016)=-0.471759653228D+04
coeff(3017)=-0.300734700638D+04
coeff(3018)=-0.464754555446D+03
coeff(3019)=0.374631318959D+04
coeff(3020)=0.568258200909D+04
coeff(3021)=0.196184886603D+04
coeff(3022)=0.169034480368D+04
coeff(3023)=0.101126263644D+05
coeff(3024)=-0.722990168962D+02
coeff(3025)=0.383736836088D+04
coeff(3026)=-0.162998096490D+04
coeff(3027)=0.438989498772D+04
coeff(3028)=-0.971059709897D+04
coeff(3029)=-0.471394037434D+04
coeff(3030)=0.149653042499D+04
coeff(3031)=0.551860275077D+02
coeff(3032)=-0.753375029871D+04
coeff(3033)=0.305885206786D+04
coeff(3034)=-0.417631895861D+04
coeff(3035)=-0.655247282959D+03
coeff(3036)=-0.119762346100D+04
coeff(3037)=-0.252495367013D+04
coeff(3038)=-0.913836146132D+04
coeff(3039)=0.605832508668D+03
coeff(3040)=-0.296858490377D+04
coeff(3041)=0.223016906343D+03
coeff(3042)=-0.361004368083D+04
coeff(3043)=0.174381660202D+04
coeff(3044)=0.104224913650D+05
coeff(3045)=-0.125711995122D+04
coeff(3046)=-0.808755196175D+04
coeff(3047)=0.163530505692D+04
coeff(3048)=0.252412587721D+04
coeff(3049)=0.131389254890D+04
coeff(3050)=0.335992537393D+04
coeff(3051)=0.316123378276D+04
coeff(3052)=-0.152763655114D+04
coeff(3053)=0.420416018704D+04
coeff(3054)=0.582183718574D+04
coeff(3055)=0.420677872160D+04
coeff(3056)=0.166689028613D+04
coeff(3057)=-0.551745218857D+04
coeff(3058)=0.329283838135D+03
coeff(3059)=-0.335764278340D+03
coeff(3060)=-0.714636418404D+03
coeff(3061)=-0.177059458798D+04
coeff(3062)=-0.506911661227D+04
coeff(3063)=-0.382964491144D+03
coeff(3064)=-0.606620534751D+04
coeff(3065)=0.237362202899D+04
coeff(3066)=-0.200395081191D+04
coeff(3067)=-0.100401603384D+04
coeff(3068)=-0.100545548774D+04
coeff(3069)=0.380515603086D+04
coeff(3070)=0.762575524766D+03
coeff(3071)=-0.341312861784D+04
coeff(3072)=0.342299577716D+04
coeff(3073)=0.550185197452D+03
coeff(3074)=0.974246061586D+03
coeff(3075)=0.340299040498D+03

! Calculate MEG
x(1)=exp(-((r(1)-ra1))/a1 -((r(1)-rb1)**2)/b1)
x(2)=exp(-((r(2)-ra1))/a1 -((r(2)-rb1)**2)/b1)
x(3)=exp(-((r(3)-ra2))/a2 -((r(3)-rb2)**2)/b2)
x(4)=exp(-((r(4)-ra1))/a1 -((r(4)-rb1)**2)/b1)
x(5)=exp(-((r(5)-ra2))/a2 -((r(5)-rb2)**2)/b2)
x(6)=exp(-((r(6)-ra2))/a2 -((r(6)-rb2)**2)/b2)

! Calculate dMEG/dr(6,6) for giving R(6)
! dmsdr(i,j)=0  i!=j
! Initialize dmsdr
      do i=1,6
        do j=1,6
          dmsdr(i,j)=0.0d0
        enddo
      enddo
dmsdr(1,1)=(-2.0d0*(r(1)-rb1)/b1-1/a1)*Exp(-(r(1)-ra1)/a1-((r(1)-rb1)**2.0d0)/b1)
dmsdr(2,2)=(-2.0d0*(r(2)-rb1)/b1-1/a1)*Exp(-(r(2)-ra1)/a1-((r(2)-rb1)**2.0d0)/b1)
dmsdr(3,3)=(-2.0d0*(r(3)-rb2)/b2-1/a2)*Exp(-(r(3)-ra2)/a2-((r(3)-rb2)**2.0d0)/b2)
dmsdr(4,4)=(-2.0d0*(r(4)-rb1)/b1-1/a1)*Exp(-(r(4)-ra1)/a1-((r(4)-rb1)**2.0d0)/b1)
dmsdr(5,5)=(-2.0d0*(r(5)-rb2)/b2-1/a2)*Exp(-(r(5)-ra2)/a2-((r(5)-rb2)**2.0d0)/b2)
dmsdr(6,6)=(-2.0d0*(r(6)-rb2)/b2-1/a2)*Exp(-(r(6)-ra2)/a2-((r(6)-rb2)**2.0d0)/b2)

rm(0) = 1.0D0
rm(1) = x(4)
rm(2) = x(2)
rm(3) = x(1)
rm(4) = x(6)
rm(5) = x(5)
rm(6) = x(3)
rm(7) = rm(1)*rm(2)
rm(8) = rm(1)*rm(3)
rm(9) = rm(2)*rm(3)
rm(10) = rm(3)*rm(4)
rm(11) = rm(2)*rm(5)
rm(12) = rm(1)*rm(6)
rm(13) = rm(4)*rm(5)
rm(14) = rm(4)*rm(6)
rm(15) = rm(5)*rm(6)
rm(16) = rm(1)*rm(9)
rm(17) = rm(1)*rm(10)
rm(18) = rm(2)*rm(10)
rm(19) = rm(1)*rm(11)
rm(20) = rm(3)*rm(11)
rm(21) = rm(2)*rm(12)
rm(22) = rm(3)*rm(12)
rm(23) = rm(2)*rm(13)
rm(24) = rm(3)*rm(13)
rm(25) = rm(1)*rm(14)
rm(26) = rm(3)*rm(14)
rm(27) = rm(1)*rm(15)
rm(28) = rm(2)*rm(15)
rm(29) = rm(4)*rm(15)
rm(30) = rm(2)*rm(24)
rm(31) = rm(1)*rm(26)
rm(32) = rm(1)*rm(28)

p(0) = rm(0)
p(1) = rm(1) + rm(2) + rm(3)
p(2) = rm(4) + rm(5) + rm(6)
p(3) = rm(7) + rm(8) + rm(9)
p(4) = rm(10) + rm(11) + rm(12)
p(5) = p(1)*p(2) - p(4)
p(6) = rm(13) + rm(14) + rm(15)
p(7) = p(1)*p(1) - p(3) - p(3)
p(8) = p(2)*p(2) - p(6) - p(6)
p(9) = rm(16)
p(10) = rm(17) + rm(18) + rm(19) + rm(20) + rm(21) + rm(22)
p(11) = p(2)*p(3) - p(10)
p(12) = rm(23) + rm(24) + rm(25) + rm(26) + rm(27) + rm(28)
p(13) = p(1)*p(6) - p(12)
p(14) = rm(29)
p(15) = p(1)*p(3) - p(9) - p(9) - p(9)
p(16) = p(1)*p(4) - p(10)
p(17) = p(2)*p(7) - p(16)
p(18) = p(2)*p(4) - p(12)
p(19) = p(1)*p(8) - p(18)
p(20) = p(2)*p(6) - p(14) - p(14) - p(14)
p(21) = p(1)*p(7) - p(15)
p(22) = p(2)*p(8) - p(20)
p(23) = p(9)*p(2)
p(24) = rm(30) + rm(31) + rm(32)
p(25) = p(3)*p(6) - p(24)
p(26) = p(14)*p(1)
p(27) = p(9)*p(1)
p(28) = p(3)*p(4) - p(23)
p(29) = p(1)*p(10) - p(23) - p(28) - p(23)
p(30) = p(1)*p(11) - p(23)
p(31) = p(1)*p(12) - p(25) - p(24) - p(24)
p(32) = p(1)*p(13) - p(25)
p(33) = p(4)*p(5) - p(25) - p(31)
p(34) = p(2)*p(11) - p(25)
p(35) = p(4)*p(6) - p(26)
p(36) = p(2)*p(12) - p(26) - p(35) - p(26)
p(37) = p(2)*p(13) - p(26)
p(38) = p(14)*p(2)
p(39) = p(3)*p(3) - p(27) - p(27)
p(40) = p(3)*p(7) - p(27)
p(41) = p(1)*p(16) - p(28)
p(42) = p(2)*p(21) - p(41)
p(43) = p(1)*p(18) - p(33)
p(44) = p(7)*p(8) - p(43)
p(45) = p(6)*p(6) - p(38) - p(38)
p(46) = p(2)*p(18) - p(35)
p(47) = p(1)*p(22) - p(46)
p(48) = p(6)*p(8) - p(38)
p(49) = p(1)*p(21) - p(40)
p(50) = p(2)*p(22) - p(48)
p(51) = p(9)*p(6)
p(52) = p(14)*p(3)
p(53) = p(9)*p(4)
p(54) = p(9)*p(5)
p(55) = p(1)*p(24) - p(51)
p(56) = p(3)*p(12) - p(51) - p(55) - p(51)
p(57) = p(3)*p(13) - p(51)
p(58) = p(14)*p(7)
p(59) = p(9)*p(8)
p(60) = p(2)*p(24) - p(52)
p(61) = p(4)*p(13) - p(58)
p(62) = p(6)*p(11) - p(52)
p(63) = p(14)*p(4)
p(64) = p(14)*p(5)
p(65) = p(9)*p(3)
p(66) = p(9)*p(7)
p(67) = p(3)*p(16) - p(53)
p(68) = p(4)*p(15) - p(54) - p(67)
p(69) = p(2)*p(39) - p(68)
p(70) = p(1)*p(29) - p(54) - p(68)
p(71) = p(7)*p(11) - p(53)
p(72) = p(1)*p(31) - p(56) - p(55)
p(73) = p(1)*p(32) - p(57)
p(74) = p(3)*p(18) - p(59)
p(75) = p(2)*p(29) - p(57) - p(55)
p(76) = p(1)*p(34) - p(59)
p(77) = p(6)*p(16) - p(58)
p(78) = p(1)*p(36) - p(62) - p(60)
p(79) = p(2)*p(32) - p(58)
p(80) = p(6)*p(12) - p(64) - p(63) - p(63)
p(81) = p(1)*p(45) - p(80)
p(82) = p(14)*p(6)
p(83) = p(2)*p(33) - p(61) - p(60)
p(84) = p(2)*p(34) - p(62)
p(85) = p(6)*p(18) - p(63)
p(86) = p(2)*p(36) - p(64) - p(80)
p(87) = p(8)*p(13) - p(63)
p(88) = p(14)*p(8)
p(89) = p(1)*p(39) - p(65)
p(90) = p(3)*p(21) - p(66)
p(91) = p(1)*p(41) - p(67)
p(92) = p(2)*p(49) - p(91)
p(93) = p(1)*p(43) - p(74)
p(94) = p(8)*p(21) - p(93)
p(95) = p(1)*p(46) - p(83)
p(96) = p(7)*p(22) - p(95)
p(97) = p(2)*p(45) - p(82)
p(98) = p(2)*p(46) - p(85)
p(99) = p(1)*p(50) - p(98)
p(100) = p(6)*p(22) - p(88)
p(101) = p(1)*p(49) - p(90)
p(102) = p(2)*p(50) - p(100)
p(103) = p(9)*p(14)
p(104) = p(9)*p(12)
p(105) = p(9)*p(13)
p(106) = p(14)*p(15)
p(107) = p(9)*p(20)
p(108) = p(14)*p(10)
p(109) = p(14)*p(11)
p(110) = p(9)*p(16)
p(111) = p(9)*p(10)
p(112) = p(9)*p(11)
p(113) = p(9)*p(17)
p(114) = p(3)*p(24) - p(104)
p(115) = p(7)*p(24) - p(105)
p(116) = p(3)*p(31) - p(104) - p(115)
p(117) = p(6)*p(39) - p(114)
p(118) = p(3)*p(32) - p(105)
p(119) = p(14)*p(21)
p(120) = p(9)*p(18)
p(121) = p(9)*p(19)
p(122) = p(4)*p(24) - p(103) - p(103) - p(103)
p(123) = p(1)*p(60) - p(107) - p(122)
p(124) = p(13)*p(16) - p(119)
p(125) = p(2)*p(56) - p(106) - p(124)
p(126) = p(4)*p(32) - p(119)
p(127) = p(1)*p(62) - p(107) - p(125)
p(128) = p(14)*p(16)
p(129) = p(14)*p(17)
p(130) = p(6)*p(24) - p(108)
p(131) = p(3)*p(45) - p(130)
p(132) = p(14)*p(12)
p(133) = p(14)*p(13)
p(134) = p(9)*p(22)
p(135) = p(8)*p(24) - p(109)
p(136) = p(13)*p(18) - p(128)
p(137) = p(6)*p(34) - p(109)
p(138) = p(14)*p(18)
p(139) = p(14)*p(19)
p(140) = p(9)*p(9)
p(141) = p(9)*p(15)
p(142) = p(9)*p(21)
p(143) = p(3)*p(41) - p(110)
p(144) = p(4)*p(39) - p(112)
p(145) = p(1)*p(68) - p(111) - p(144)
p(146) = p(1)*p(69) - p(112)
p(147) = p(1)*p(70) - p(113) - p(145)
p(148) = p(11)*p(21) - p(110)
p(149) = p(1)*p(72) - p(116) - p(115)
p(150) = p(1)*p(73) - p(118)
p(151) = p(3)*p(43) - p(120)
p(152) = p(15)*p(18) - p(121) - p(151)
p(153) = p(2)*p(69) - p(117)
p(154) = p(1)*p(75) - p(121) - p(152)
p(155) = p(7)*p(34) - p(120)
p(156) = p(6)*p(41) - p(119)
p(157) = p(1)*p(78) - p(125) - p(123)
p(158) = p(2)*p(73) - p(119)
p(159) = p(1)*p(80) - p(131) - p(130) - p(130)
p(160) = p(1)*p(81) - p(131)
p(161) = p(14)*p(14)
p(162) = p(3)*p(46) - p(134)
p(163) = p(2)*p(75) - p(126) - p(123)
p(164) = p(1)*p(84) - p(134)
p(165) = p(6)*p(43) - p(128)
p(166) = p(1)*p(86) - p(137) - p(135)
p(167) = p(8)*p(32) - p(128)
p(168) = p(4)*p(45) - p(133)
p(169) = p(2)*p(80) - p(132) - p(168)
p(170) = p(2)*p(81) - p(133)
p(171) = p(14)*p(20)
p(172) = p(2)*p(83) - p(136) - p(135)
p(173) = p(2)*p(84) - p(137)
p(174) = p(6)*p(46) - p(138)
p(175) = p(2)*p(86) - p(139) - p(169)
p(176) = p(13)*p(22) - p(138)
p(177) = p(14)*p(22)
p(178) = p(3)*p(39) - p(141)
p(179) = p(1)*p(89) - p(141) - p(178) - p(178)
p(180) = p(3)*p(49) - p(142)
p(181) = p(1)*p(91) - p(143)
p(182) = p(2)*p(101) - p(181)
p(183) = p(1)*p(93) - p(151)
p(184) = p(8)*p(49) - p(183)
p(185) = p(1)*p(95) - p(162)
p(186) = p(21)*p(22) - p(185)
p(187) = p(6)*p(45) - p(171)
p(188) = p(1)*p(98) - p(172)
p(189) = p(7)*p(50) - p(188)
p(190) = p(2)*p(97) - p(171) - p(187) - p(187)
p(191) = p(2)*p(98) - p(174)
p(192) = p(1)*p(102) - p(191)
p(193) = p(6)*p(50) - p(177)
p(194) = p(1)*p(101) - p(180)
p(195) = p(2)*p(102) - p(193)
p(196) = p(9)*p(26)
p(197) = p(9)*p(38)
p(198) = p(9)*p(24)
p(199) = p(9)*p(31)
p(200) = p(9)*p(25)
p(201) = p(9)*p(32)
p(202) = p(14)*p(39)
p(203) = p(14)*p(40)
p(204) = p(9)*p(35)
p(205) = p(9)*p(36)
p(206) = p(9)*p(37)
p(207) = p(14)*p(28)
p(208) = p(14)*p(29)
p(209) = p(14)*p(30)
p(210) = p(9)*p(45)
p(211) = p(14)*p(24)
p(212) = p(14)*p(25)
p(213) = p(9)*p(48)
p(214) = p(14)*p(33)
p(215) = p(14)*p(34)
p(216) = p(9)*p(41)
p(217) = p(9)*p(28)
p(218) = p(9)*p(23)
p(219) = p(9)*p(29)
p(220) = p(9)*p(30)
p(221) = p(9)*p(42)
p(222) = p(1)*p(114) - p(198)
p(223) = p(21)*p(24) - p(201)
p(224) = p(3)*p(72) - p(199) - p(223)
p(225) = p(1)*p(116) - p(199) - p(224)
p(226) = p(13)*p(39) - p(198)
p(227) = p(3)*p(73) - p(201)
p(228) = p(14)*p(49)
p(229) = p(9)*p(43)
p(230) = p(9)*p(33)
p(231) = p(9)*p(34)
p(232) = p(9)*p(44)
p(233) = p(16)*p(24) - p(196)
p(234) = p(2)*p(114) - p(202)
p(235) = p(1)*p(123) - p(205) - p(234)
p(236) = p(13)*p(41) - p(228)
p(237) = p(2)*p(116) - p(203) - p(236)
p(238) = p(16)*p(32) - p(228)
p(239) = p(6)*p(69) - p(202)
p(240) = p(4)*p(73) - p(228)
p(241) = p(11)*p(32) - p(196)
p(242) = p(14)*p(41)
p(243) = p(14)*p(42)
p(244) = p(1)*p(130) - p(210)
p(245) = p(4)*p(62) - p(213) - p(209)
p(246) = p(3)*p(81) - p(210)
p(247) = p(14)*p(31)
p(248) = p(14)*p(32)
p(249) = p(14)*p(26)
p(250) = p(9)*p(46)
p(251) = p(9)*p(47)
p(252) = p(18)*p(24) - p(197)
p(253) = p(1)*p(135) - p(213) - p(252)
p(254) = p(13)*p(43) - p(242)
p(255) = p(2)*p(125) - p(209) - p(245)
p(256) = p(18)*p(32) - p(242)
p(257) = p(13)*p(34) - p(197)
p(258) = p(14)*p(43)
p(259) = p(14)*p(44)
p(260) = p(2)*p(130) - p(211)
p(261) = p(4)*p(81) - p(248)
p(262) = p(11)*p(45) - p(211)
p(263) = p(14)*p(35)
p(264) = p(14)*p(36)
p(265) = p(14)*p(37)
p(266) = p(9)*p(50)
p(267) = p(22)*p(24) - p(215)
p(268) = p(13)*p(46) - p(258)
p(269) = p(6)*p(84) - p(215)
p(270) = p(14)*p(46)
p(271) = p(14)*p(47)
p(272) = p(9)*p(27)
p(273) = p(9)*p(39)
p(274) = p(9)*p(40)
p(275) = p(9)*p(49)
p(276) = p(3)*p(91) - p(216)
p(277) = p(16)*p(39) - p(218)
p(278) = p(1)*p(144) - p(217) - p(277)
p(279) = p(2)*p(178) - p(278)
p(280) = p(1)*p(145) - p(219) - p(278)
p(281) = p(7)*p(69) - p(218)
p(282) = p(1)*p(147) - p(221) - p(280)
p(283) = p(11)*p(49) - p(216)
p(284) = p(1)*p(149) - p(224) - p(223)
p(285) = p(1)*p(150) - p(227)
p(286) = p(3)*p(93) - p(229)
p(287) = p(18)*p(39) - p(231)
p(288) = p(1)*p(152) - p(230) - p(287)
p(289) = p(1)*p(153) - p(231)
p(290) = p(1)*p(154) - p(232) - p(288)
p(291) = p(21)*p(34) - p(229)
p(292) = p(6)*p(91) - p(228)
p(293) = p(1)*p(157) - p(237) - p(235)
p(294) = p(2)*p(150) - p(228)
p(295) = p(1)*p(159) - p(245) - p(244)
p(296) = p(1)*p(160) - p(246)
p(297) = p(3)*p(95) - p(250)
p(298) = p(2)*p(152) - p(238) - p(234)
p(299) = p(2)*p(153) - p(239)
p(300) = p(1)*p(163) - p(251) - p(298)
p(301) = p(7)*p(84) - p(250)
p(302) = p(6)*p(93) - p(242)
p(303) = p(1)*p(166) - p(255) - p(253)
p(304) = p(8)*p(73) - p(242)
p(305) = p(16)*p(45) - p(248)
p(306) = p(1)*p(169) - p(262) - p(260)
p(307) = p(2)*p(160) - p(248)
p(308) = p(14)*p(38)
p(309) = p(6)*p(80) - p(264) - p(263)
p(310) = p(1)*p(187) - p(309)
p(311) = p(14)*p(45)
p(312) = p(3)*p(98) - p(266)
p(313) = p(2)*p(163) - p(256) - p(253)
p(314) = p(1)*p(173) - p(266)
p(315) = p(6)*p(95) - p(258)
p(316) = p(1)*p(175) - p(269) - p(267)
p(317) = p(22)*p(32) - p(258)
p(318) = p(18)*p(45) - p(249)
p(319) = p(2)*p(169) - p(264) - p(309)
p(320) = p(8)*p(81) - p(249)
p(321) = p(14)*p(48)
p(322) = p(2)*p(172) - p(268) - p(267)
p(323) = p(2)*p(173) - p(269)
p(324) = p(6)*p(98) - p(270)
p(325) = p(2)*p(175) - p(271) - p(319)
p(326) = p(13)*p(50) - p(270)
p(327) = p(14)*p(50)
p(328) = p(1)*p(178) - p(273)
p(329) = p(21)*p(39) - p(272)
p(330) = p(3)*p(101) - p(275)
p(331) = p(1)*p(181) - p(276)
p(332) = p(2)*p(194) - p(331)
p(333) = p(1)*p(183) - p(286)
p(334) = p(8)*p(101) - p(333)
p(335) = p(1)*p(185) - p(297)
p(336) = p(22)*p(49) - p(335)
p(337) = p(1)*p(188) - p(312)
p(338) = p(21)*p(50) - p(337)
p(339) = p(2)*p(187) - p(311)
p(340) = p(1)*p(191) - p(322)
p(341) = p(7)*p(102) - p(340)
p(342) = p(22)*p(45) - p(308)
p(343) = p(2)*p(191) - p(324)
p(344) = p(1)*p(195) - p(343)
p(345) = p(6)*p(102) - p(327)
p(346) = p(1)*p(194) - p(330)
p(347) = p(2)*p(195) - p(345)
p(348) = p(9)*p(52)
p(349) = p(9)*p(58)
p(350) = p(9)*p(63)
p(351) = p(9)*p(64)
p(352) = p(9)*p(82)
p(353) = p(9)*p(88)
p(354) = p(9)*p(55)
p(355) = p(9)*p(72)
p(356) = p(9)*p(51)
p(357) = p(9)*p(56)
p(358) = p(9)*p(57)
p(359) = p(9)*p(73)
p(360) = p(14)*p(89)
p(361) = p(14)*p(90)
p(362) = p(9)*p(77)
p(363) = p(9)*p(60)
p(364) = p(9)*p(78)
p(365) = p(9)*p(61)
p(366) = p(9)*p(62)
p(367) = p(9)*p(79)
p(368) = p(14)*p(67)
p(369) = p(14)*p(68)
p(370) = p(14)*p(69)
p(371) = p(14)*p(70)
p(372) = p(14)*p(71)
p(373) = p(9)*p(80)
p(374) = p(9)*p(81)
p(375) = p(14)*p(55)
p(376) = p(14)*p(56)
p(377) = p(14)*p(57)
p(378) = p(14)*p(52)
p(379) = p(9)*p(85)
p(380) = p(9)*p(86)
p(381) = p(9)*p(87)
p(382) = p(14)*p(74)
p(383) = p(14)*p(75)
p(384) = p(14)*p(76)
p(385) = p(9)*p(97)
p(386) = p(14)*p(60)
p(387) = p(14)*p(61)
p(388) = p(14)*p(62)
p(389) = p(9)*p(100)
p(390) = p(14)*p(83)
p(391) = p(14)*p(84)
p(392) = p(9)*p(91)
p(393) = p(9)*p(67)
p(394) = p(9)*p(53)
p(395) = p(9)*p(68)
p(396) = p(9)*p(54)
p(397) = p(9)*p(69)
p(398) = p(9)*p(70)
p(399) = p(9)*p(71)
p(400) = p(9)*p(92)
p(401) = p(3)*p(114) - p(354)
p(402) = p(7)*p(114) - p(356)
p(403) = p(24)*p(49) - p(359)
p(404) = p(3)*p(149) - p(355) - p(403)
p(405) = p(1)*p(224) - p(355) - p(404)
p(406) = p(6)*p(178) - p(401)
p(407) = p(32)*p(39) - p(356)
p(408) = p(3)*p(150) - p(359)
p(409) = p(14)*p(101)
p(410) = p(9)*p(93)
p(411) = p(9)*p(74)
p(412) = p(9)*p(59)
p(413) = p(9)*p(75)
p(414) = p(9)*p(76)
p(415) = p(9)*p(94)
p(416) = p(24)*p(41) - p(349)
p(417) = p(4)*p(114) - p(348)
p(418) = p(1)*p(234) - p(363) - p(417)
p(419) = p(1)*p(235) - p(364) - p(418)
p(420) = p(13)*p(91) - p(409)
p(421) = p(2)*p(224) - p(361) - p(420)
p(422) = p(32)*p(41) - p(409)
p(423) = p(1)*p(237) - p(364) - p(421)
p(424) = p(16)*p(73) - p(409)
p(425) = p(13)*p(69) - p(348)
p(426) = p(4)*p(150) - p(409)
p(427) = p(11)*p(73) - p(349)
p(428) = p(14)*p(91)
p(429) = p(14)*p(92)
p(430) = p(3)*p(130) - p(373)
p(431) = p(7)*p(130) - p(374)
p(432) = p(3)*p(159) - p(373) - p(431)
p(433) = p(39)*p(45) - p(430)
p(434) = p(3)*p(160) - p(374)
p(435) = p(14)*p(72)
p(436) = p(14)*p(73)
p(437) = p(14)*p(58)
p(438) = p(9)*p(95)
p(439) = p(9)*p(83)
p(440) = p(9)*p(84)
p(441) = p(9)*p(96)
p(442) = p(24)*p(43) - p(350)
p(443) = p(8)*p(114) - p(370)
p(444) = p(1)*p(253) - p(380) - p(443)
p(445) = p(13)*p(93) - p(428)
p(446) = p(2)*p(237) - p(372) - p(432)
p(447) = p(32)*p(43) - p(428)
p(448) = p(6)*p(153) - p(370)
p(449) = p(18)*p(73) - p(428)
p(450) = p(32)*p(34) - p(350)
p(451) = p(14)*p(93)
p(452) = p(14)*p(94)
p(453) = p(4)*p(130) - p(352)
p(454) = p(1)*p(260) - p(385) - p(453)
p(455) = p(16)*p(81) - p(436)
p(456) = p(2)*p(245) - p(376) - p(455)
p(457) = p(4)*p(160) - p(436)
p(458) = p(11)*p(81) - p(352)
p(459) = p(14)*p(77)
p(460) = p(14)*p(78)
p(461) = p(14)*p(79)
p(462) = p(14)*p(63)
p(463) = p(14)*p(64)
p(464) = p(6)*p(130) - p(386)
p(465) = p(3)*p(187) - p(464)
p(466) = p(14)*p(80)
p(467) = p(14)*p(81)
p(468) = p(9)*p(98)
p(469) = p(9)*p(99)
p(470) = p(24)*p(46) - p(353)
p(471) = p(1)*p(267) - p(389) - p(470)
p(472) = p(13)*p(95) - p(451)
p(473) = p(2)*p(255) - p(384) - p(456)
p(474) = p(32)*p(46) - p(451)
p(475) = p(13)*p(84) - p(353)
p(476) = p(14)*p(95)
p(477) = p(14)*p(96)
p(478) = p(8)*p(130) - p(378)
p(479) = p(18)*p(81) - p(437)
p(480) = p(34)*p(45) - p(378)
p(481) = p(14)*p(85)
p(482) = p(14)*p(86)
p(483) = p(14)*p(87)
p(484) = p(9)*p(102)
p(485) = p(24)*p(50) - p(391)
p(486) = p(13)*p(98) - p(476)
p(487) = p(6)*p(173) - p(391)
p(488) = p(14)*p(98)
p(489) = p(14)*p(99)
p(490) = p(9)*p(65)
p(491) = p(9)*p(66)
p(492) = p(9)*p(89)
p(493) = p(9)*p(90)
p(494) = p(9)*p(101)
p(495) = p(3)*p(181) - p(392)
p(496) = p(39)*p(41) - p(394)
p(497) = p(4)*p(178) - p(397)
p(498) = p(1)*p(278) - p(395) - p(497)
p(499) = p(1)*p(279) - p(397)
p(500) = p(1)*p(280) - p(398) - p(498)
p(501) = p(21)*p(69) - p(394)
p(502) = p(1)*p(282) - p(400) - p(500)
p(503) = p(11)*p(101) - p(392)
p(504) = p(1)*p(284) - p(404) - p(403)
p(505) = p(1)*p(285) - p(408)
p(506) = p(3)*p(183) - p(410)
p(507) = p(39)*p(43) - p(412)
p(508) = p(1)*p(287) - p(411) - p(507)
p(509) = p(2)*p(279) - p(406)
p(510) = p(1)*p(288) - p(413) - p(508)
p(511) = p(7)*p(153) - p(412)
p(512) = p(1)*p(290) - p(415) - p(510)
p(513) = p(34)*p(49) - p(410)
p(514) = p(6)*p(181) - p(409)
p(515) = p(1)*p(293) - p(421) - p(419)
p(516) = p(2)*p(285) - p(409)
p(517) = p(1)*p(295) - p(432) - p(431)
p(518) = p(1)*p(296) - p(434)
p(519) = p(3)*p(185) - p(438)
p(520) = p(39)*p(46) - p(440)
p(521) = p(1)*p(298) - p(439) - p(520)
p(522) = p(1)*p(299) - p(440)
p(523) = p(1)*p(300) - p(441) - p(521)
p(524) = p(21)*p(84) - p(438)
p(525) = p(6)*p(183) - p(428)
p(526) = p(1)*p(303) - p(446) - p(444)
p(527) = p(8)*p(150) - p(428)
p(528) = p(41)*p(45) - p(436)
p(529) = p(1)*p(306) - p(456) - p(454)
p(530) = p(2)*p(296) - p(436)
p(531) = p(4)*p(169) - p(388) - p(478)
p(532) = p(1)*p(310) - p(465)
p(533) = p(14)*p(82)
p(534) = p(3)*p(188) - p(468)
p(535) = p(2)*p(298) - p(447) - p(443)
p(536) = p(2)*p(299) - p(448)
p(537) = p(1)*p(313) - p(469) - p(535)
p(538) = p(7)*p(173) - p(468)
p(539) = p(6)*p(185) - p(451)
p(540) = p(1)*p(316) - p(473) - p(471)
p(541) = p(22)*p(73) - p(451)
p(542) = p(43)*p(45) - p(437)
p(543) = p(1)*p(319) - p(480) - p(478)
p(544) = p(8)*p(160) - p(437)
p(545) = p(14)*p(88)
p(546) = p(4)*p(187) - p(467)
p(547) = p(2)*p(309) - p(466) - p(546)
p(548) = p(2)*p(310) - p(467)
p(549) = p(14)*p(97)
p(550) = p(3)*p(191) - p(484)
p(551) = p(2)*p(313) - p(474) - p(471)
p(552) = p(1)*p(323) - p(484)
p(553) = p(6)*p(188) - p(476)
p(554) = p(1)*p(325) - p(487) - p(485)
p(555) = p(32)*p(50) - p(476)
p(556) = p(45)*p(46) - p(462)
p(557) = p(2)*p(319) - p(482) - p(547)
p(558) = p(22)*p(81) - p(462)
p(559) = p(14)*p(100)
p(560) = p(2)*p(322) - p(486) - p(485)
p(561) = p(2)*p(323) - p(487)
p(562) = p(6)*p(191) - p(488)
p(563) = p(2)*p(325) - p(489) - p(557)
p(564) = p(13)*p(102) - p(488)
p(565) = p(14)*p(102)
p(566) = p(3)*p(178) - p(492)
p(567) = p(7)*p(178) - p(490)
p(568) = p(39)*p(49) - p(491)
p(569) = p(3)*p(194) - p(494)
p(570) = p(1)*p(331) - p(495)
p(571) = p(2)*p(346) - p(570)
p(572) = p(1)*p(333) - p(506)
p(573) = p(8)*p(194) - p(572)
p(574) = p(1)*p(335) - p(519)
p(575) = p(22)*p(101) - p(574)
p(576) = p(1)*p(337) - p(534)
p(577) = p(49)*p(50) - p(576)
p(578) = p(6)*p(187) - p(549)
p(579) = p(1)*p(340) - p(550)
p(580) = p(21)*p(102) - p(579)
p(581) = p(8)*p(187) - p(533)
p(582) = p(1)*p(343) - p(560)
p(583) = p(7)*p(195) - p(582)
p(584) = p(45)*p(50) - p(545)
p(585) = p(2)*p(343) - p(562)
p(586) = p(1)*p(347) - p(585)
p(587) = p(6)*p(195) - p(565)
p(588) = p(1)*p(346) - p(569)
p(589) = p(2)*p(347) - p(587)
p(590) = p(9)*p(103)
p(591) = p(9)*p(106)
p(592) = p(9)*p(119)
p(593) = p(9)*p(128)
p(594) = p(9)*p(108)
p(595) = p(9)*p(109)
p(596) = p(9)*p(129)
p(597) = p(9)*p(132)
p(598) = p(9)*p(133)
p(599) = p(9)*p(161)
p(600) = p(9)*p(138)
p(601) = p(9)*p(139)
p(602) = p(9)*p(171)
p(603) = p(9)*p(177)
p(604) = p(9)*p(114)
p(605) = p(9)*p(115)
p(606) = p(9)*p(149)
p(607) = p(9)*p(104)
p(608) = p(9)*p(116)
p(609) = p(9)*p(105)
p(610) = p(9)*p(117)
p(611) = p(9)*p(118)
p(612) = p(9)*p(150)
p(613) = p(14)*p(178)
p(614) = p(14)*p(179)
p(615) = p(14)*p(180)
p(616) = p(9)*p(156)
p(617) = p(9)*p(122)
p(618) = p(9)*p(123)
p(619) = p(9)*p(157)
p(620) = p(9)*p(124)
p(621) = p(9)*p(107)
p(622) = p(9)*p(125)
p(623) = p(9)*p(126)
p(624) = p(9)*p(127)
p(625) = p(9)*p(158)
p(626) = p(14)*p(143)
p(627) = p(14)*p(144)
p(628) = p(14)*p(145)
p(629) = p(14)*p(146)
p(630) = p(14)*p(147)
p(631) = p(14)*p(148)
p(632) = p(9)*p(130)
p(633) = p(9)*p(159)
p(634) = p(9)*p(131)
p(635) = p(9)*p(160)
p(636) = p(14)*p(114)
p(637) = p(14)*p(115)
p(638) = p(14)*p(116)
p(639) = p(14)*p(117)
p(640) = p(14)*p(118)
p(641) = p(14)*p(106)
p(642) = p(9)*p(165)
p(643) = p(9)*p(135)
p(644) = p(9)*p(166)
p(645) = p(9)*p(136)
p(646) = p(9)*p(137)
p(647) = p(9)*p(167)
p(648) = p(14)*p(151)
p(649) = p(14)*p(152)
p(650) = p(14)*p(153)
p(651) = p(14)*p(154)
p(652) = p(14)*p(155)
p(653) = p(9)*p(168)
p(654) = p(9)*p(169)
p(655) = p(9)*p(170)
p(656) = p(14)*p(122)
p(657) = p(14)*p(123)
p(658) = p(14)*p(124)
p(659) = p(14)*p(125)
p(660) = p(14)*p(126)
p(661) = p(14)*p(127)
p(662) = p(14)*p(108)
p(663) = p(14)*p(109)
p(664) = p(9)*p(187)
p(665) = p(14)*p(130)
p(666) = p(14)*p(131)
p(667) = p(9)*p(174)
p(668) = p(9)*p(175)
p(669) = p(9)*p(176)
p(670) = p(14)*p(162)
p(671) = p(14)*p(163)
p(672) = p(14)*p(164)
p(673) = p(9)*p(190)
p(674) = p(14)*p(135)
p(675) = p(14)*p(136)
p(676) = p(14)*p(137)
p(677) = p(9)*p(193)
p(678) = p(14)*p(172)
p(679) = p(14)*p(173)
p(680) = p(9)*p(181)
p(681) = p(9)*p(143)
p(682) = p(9)*p(110)
p(683) = p(9)*p(144)
p(684) = p(9)*p(111)
p(685) = p(9)*p(112)
p(686) = p(9)*p(145)
p(687) = p(9)*p(113)
p(688) = p(9)*p(146)
p(689) = p(9)*p(147)
p(690) = p(9)*p(148)
p(691) = p(9)*p(182)
p(692) = p(1)*p(401) - p(604)
p(693) = p(21)*p(114) - p(609)
p(694) = p(24)*p(101) - p(612)
p(695) = p(3)*p(284) - p(606) - p(694)
p(696) = p(1)*p(404) - p(606) - p(695)
p(697) = p(1)*p(405) - p(608) - p(696)
p(698) = p(13)*p(178) - p(604)
p(699) = p(39)*p(73) - p(609)
p(700) = p(3)*p(285) - p(612)
p(701) = p(14)*p(194)
p(702) = p(9)*p(183)
p(703) = p(9)*p(151)
p(704) = p(9)*p(120)
p(705) = p(9)*p(152)
p(706) = p(9)*p(121)
p(707) = p(9)*p(153)
p(708) = p(9)*p(154)
p(709) = p(9)*p(155)
p(710) = p(9)*p(184)
p(711) = p(24)*p(91) - p(592)
p(712) = p(1)*p(416) - p(616) - p(711)
p(713) = p(2)*p(401) - p(613)
p(714) = p(1)*p(418) - p(618) - p(713)
p(715) = p(1)*p(419) - p(619) - p(714)
p(716) = p(13)*p(181) - p(701)
p(717) = p(2)*p(404) - p(615) - p(716)
p(718) = p(32)*p(91) - p(701)
p(719) = p(1)*p(421) - p(619) - p(717)
p(720) = p(41)*p(73) - p(701)
p(721) = p(6)*p(279) - p(613)
p(722) = p(16)*p(150) - p(701)
p(723) = p(1)*p(425) - p(624) - p(721)
p(724) = p(4)*p(285) - p(701)
p(725) = p(11)*p(150) - p(592)
p(726) = p(14)*p(181)
p(727) = p(14)*p(182)
p(728) = p(1)*p(430) - p(632)
p(729) = p(21)*p(130) - p(635)
p(730) = p(3)*p(295) - p(633) - p(729)
p(731) = p(1)*p(432) - p(633) - p(730)
p(732) = p(39)*p(81) - p(632)
p(733) = p(3)*p(296) - p(635)
p(734) = p(14)*p(149)
p(735) = p(14)*p(150)
p(736) = p(14)*p(119)
p(737) = p(9)*p(185)
p(738) = p(9)*p(162)
p(739) = p(9)*p(134)
p(740) = p(9)*p(163)
p(741) = p(9)*p(164)
p(742) = p(9)*p(186)
p(743) = p(24)*p(93) - p(593)
p(744) = p(18)*p(114) - p(595)
p(745) = p(1)*p(443) - p(643) - p(744)
p(746) = p(1)*p(444) - p(644) - p(745)
p(747) = p(13)*p(183) - p(726)
p(748) = p(2)*p(421) - p(631) - p(730)
p(749) = p(32)*p(93) - p(726)
p(750) = p(1)*p(446) - p(644) - p(748)
p(751) = p(43)*p(73) - p(726)
p(752) = p(13)*p(153) - p(595)
p(753) = p(18)*p(150) - p(726)
p(754) = p(34)*p(73) - p(593)
p(755) = p(14)*p(183)
p(756) = p(14)*p(184)
p(757) = p(16)*p(130) - p(598)
p(758) = p(2)*p(430) - p(636)
p(759) = p(1)*p(454) - p(654) - p(758)
p(760) = p(41)*p(81) - p(735)
p(761) = p(2)*p(432) - p(638) - p(760)
p(762) = p(16)*p(160) - p(735)
p(763) = p(45)*p(69) - p(636)
p(764) = p(4)*p(296) - p(735)
p(765) = p(11)*p(160) - p(598)
p(766) = p(14)*p(156)
p(767) = p(14)*p(157)
p(768) = p(14)*p(158)
p(769) = p(14)*p(128)
p(770) = p(14)*p(129)
p(771) = p(1)*p(464) - p(664)
p(772) = p(4)*p(262) - p(673) - p(661)
p(773) = p(3)*p(310) - p(664)
p(774) = p(14)*p(159)
p(775) = p(14)*p(160)
p(776) = p(14)*p(132)
p(777) = p(14)*p(133)
p(778) = p(9)*p(188)
p(779) = p(9)*p(172)
p(780) = p(9)*p(173)
p(781) = p(9)*p(189)
p(782) = p(24)*p(95) - p(600)
p(783) = p(22)*p(114) - p(650)
p(784) = p(1)*p(471) - p(668) - p(783)
p(785) = p(13)*p(185) - p(755)
p(786) = p(2)*p(446) - p(652) - p(761)
p(787) = p(32)*p(95) - p(755)
p(788) = p(6)*p(299) - p(650)
p(789) = p(46)*p(73) - p(755)
p(790) = p(32)*p(84) - p(600)
p(791) = p(14)*p(185)
p(792) = p(14)*p(186)
p(793) = p(2)*p(453) - p(656) - p(771)
p(794) = p(1)*p(478) - p(673) - p(793)
p(795) = p(43)*p(81) - p(736)
p(796) = p(2)*p(456) - p(659) - p(772)
p(797) = p(18)*p(160) - p(736)
p(798) = p(1)*p(480) - p(673) - p(796)
p(799) = p(14)*p(165)
p(800) = p(14)*p(166)
p(801) = p(14)*p(167)
p(802) = p(14)*p(138)
p(803) = p(14)*p(139)
p(804) = p(2)*p(464) - p(665)
p(805) = p(4)*p(310) - p(775)
p(806) = p(11)*p(187) - p(665)
p(807) = p(14)*p(168)
p(808) = p(14)*p(169)
p(809) = p(14)*p(170)
p(810) = p(9)*p(191)
p(811) = p(9)*p(192)
p(812) = p(24)*p(98) - p(603)
p(813) = p(1)*p(485) - p(677) - p(812)
p(814) = p(13)*p(188) - p(791)
p(815) = p(2)*p(473) - p(672) - p(796)
p(816) = p(32)*p(98) - p(791)
p(817) = p(13)*p(173) - p(603)
p(818) = p(14)*p(188)
p(819) = p(14)*p(189)
p(820) = p(22)*p(130) - p(663)
p(821) = p(46)*p(81) - p(769)
p(822) = p(45)*p(84) - p(663)
p(823) = p(14)*p(174)
p(824) = p(14)*p(175)
p(825) = p(14)*p(176)
p(826) = p(9)*p(195)
p(827) = p(24)*p(102) - p(679)
p(828) = p(13)*p(191) - p(818)
p(829) = p(6)*p(323) - p(679)
p(830) = p(14)*p(191)
p(831) = p(14)*p(192)
p(832) = p(9)*p(140)
p(833) = p(9)*p(141)
p(834) = p(9)*p(178)
p(835) = p(9)*p(142)
p(836) = p(9)*p(179)
p(837) = p(9)*p(180)
p(838) = p(9)*p(194)
p(839) = p(3)*p(331) - p(680)
p(840) = p(39)*p(91) - p(682)
p(841) = p(16)*p(178) - p(685)
p(842) = p(1)*p(497) - p(683) - p(841)
p(843) = p(2)*p(566) - p(842)
p(844) = p(1)*p(498) - p(686) - p(842)
p(845) = p(7)*p(279) - p(685)
p(846) = p(1)*p(500) - p(689) - p(844)
p(847) = p(49)*p(69) - p(682)
p(848) = p(1)*p(502) - p(691) - p(846)
p(849) = p(11)*p(194) - p(680)
p(850) = p(1)*p(504) - p(695) - p(694)
p(851) = p(1)*p(505) - p(700)
p(852) = p(3)*p(333) - p(702)
p(853) = p(39)*p(93) - p(704)
p(854) = p(18)*p(178) - p(707)
p(855) = p(1)*p(508) - p(705) - p(854)
p(856) = p(1)*p(509) - p(707)
p(857) = p(1)*p(510) - p(708) - p(855)
p(858) = p(21)*p(153) - p(704)
p(859) = p(1)*p(512) - p(710) - p(857)
p(860) = p(34)*p(101) - p(702)
p(861) = p(6)*p(331) - p(701)
p(862) = p(1)*p(515) - p(717) - p(715)
p(863) = p(2)*p(505) - p(701)
p(864) = p(1)*p(517) - p(730) - p(729)
p(865) = p(1)*p(518) - p(733)
p(866) = p(3)*p(335) - p(737)
p(867) = p(39)*p(95) - p(739)
p(868) = p(1)*p(520) - p(738) - p(867)
p(869) = p(2)*p(509) - p(721)
p(870) = p(1)*p(521) - p(740) - p(868)
p(871) = p(7)*p(299) - p(739)
p(872) = p(1)*p(523) - p(742) - p(870)
p(873) = p(49)*p(84) - p(737)
p(874) = p(6)*p(333) - p(726)
p(875) = p(1)*p(526) - p(748) - p(746)
p(876) = p(8)*p(285) - p(726)
p(877) = p(45)*p(91) - p(735)
p(878) = p(1)*p(529) - p(761) - p(759)
p(879) = p(2)*p(518) - p(735)
p(880) = p(1)*p(531) - p(772) - p(771)
p(881) = p(1)*p(532) - p(773)
p(882) = p(14)*p(161)
p(883) = p(3)*p(337) - p(778)
p(884) = p(39)*p(98) - p(780)
p(885) = p(1)*p(535) - p(779) - p(884)
p(886) = p(1)*p(536) - p(780)
p(887) = p(1)*p(537) - p(781) - p(885)
p(888) = p(21)*p(173) - p(778)
p(889) = p(6)*p(335) - p(755)
p(890) = p(1)*p(540) - p(786) - p(784)
p(891) = p(22)*p(150) - p(755)
p(892) = p(45)*p(93) - p(736)
p(893) = p(1)*p(543) - p(796) - p(794)
p(894) = p(8)*p(296) - p(736)
p(895) = p(16)*p(187) - p(775)
p(896) = p(1)*p(547) - p(806) - p(804)
p(897) = p(2)*p(532) - p(775)
p(898) = p(14)*p(171)
p(899) = p(6)*p(309) - p(808) - p(807)
p(900) = p(1)*p(578) - p(899)
p(901) = p(14)*p(187)
p(902) = p(3)*p(340) - p(810)
p(903) = p(2)*p(535) - p(787) - p(783)
p(904) = p(2)*p(536) - p(788)
p(905) = p(1)*p(551) - p(811) - p(903)
p(906) = p(7)*p(323) - p(810)
p(907) = p(6)*p(337) - p(791)
p(908) = p(1)*p(554) - p(815) - p(813)
p(909) = p(50)*p(73) - p(791)
p(910) = p(45)*p(95) - p(769)
p(911) = p(1)*p(557) - p(822) - p(820)
p(912) = p(22)*p(160) - p(769)
p(913) = p(14)*p(177)
p(914) = p(18)*p(187) - p(777)
p(915) = p(2)*p(547) - p(808) - p(899)
p(916) = p(8)*p(310) - p(777)
p(917) = p(14)*p(190)
p(918) = p(3)*p(343) - p(826)
p(919) = p(2)*p(551) - p(816) - p(813)
p(920) = p(1)*p(561) - p(826)
p(921) = p(6)*p(340) - p(818)
p(922) = p(1)*p(563) - p(829) - p(827)
p(923) = p(32)*p(102) - p(818)
p(924) = p(45)*p(98) - p(802)
p(925) = p(2)*p(557) - p(824) - p(915)
p(926) = p(50)*p(81) - p(802)
p(927) = p(14)*p(193)
p(928) = p(2)*p(560) - p(828) - p(827)
p(929) = p(2)*p(561) - p(829)
p(930) = p(6)*p(343) - p(830)
p(931) = p(2)*p(563) - p(831) - p(925)
p(932) = p(13)*p(195) - p(830)
p(933) = p(14)*p(195)
p(934) = p(1)*p(566) - p(834)
p(935) = p(1)*p(567) - p(836) - p(934)
p(936) = p(39)*p(101) - p(835)
p(937) = p(3)*p(346) - p(838)
p(938) = p(1)*p(570) - p(839)
p(939) = p(2)*p(588) - p(938)
p(940) = p(1)*p(572) - p(852)
p(941) = p(8)*p(346) - p(940)
p(942) = p(1)*p(574) - p(866)
p(943) = p(22)*p(194) - p(942)
p(944) = p(1)*p(576) - p(883)
p(945) = p(50)*p(101) - p(944)
p(946) = p(1)*p(579) - p(902)
p(947) = p(49)*p(102) - p(946)
p(948) = p(2)*p(578) - p(901)
p(949) = p(1)*p(582) - p(918)
p(950) = p(21)*p(195) - p(949)
p(951) = p(2)*p(581) - p(917) - p(948)
p(952) = p(1)*p(585) - p(928)
p(953) = p(7)*p(347) - p(952)
p(954) = p(45)*p(102) - p(913)
p(955) = p(2)*p(585) - p(930)
p(956) = p(1)*p(589) - p(955)
p(957) = p(6)*p(347) - p(933)
p(958) = p(1)*p(588) - p(937)
p(959) = p(2)*p(589) - p(957)
p(960) = p(9)*p(196)
p(961) = p(9)*p(202)
p(962) = p(9)*p(203)
p(963) = p(9)*p(228)
p(964) = p(9)*p(242)
p(965) = p(9)*p(207)
p(966) = p(9)*p(197)
p(967) = p(9)*p(208)
p(968) = p(9)*p(209)
p(969) = p(9)*p(243)
p(970) = p(9)*p(211)
p(971) = p(9)*p(247)
p(972) = p(9)*p(212)
p(973) = p(9)*p(248)
p(974) = p(9)*p(249)
p(975) = p(9)*p(258)
p(976) = p(9)*p(214)
p(977) = p(9)*p(215)
p(978) = p(9)*p(259)
p(979) = p(9)*p(263)
p(980) = p(9)*p(264)
p(981) = p(9)*p(265)
p(982) = p(9)*p(308)
p(983) = p(9)*p(311)
p(984) = p(9)*p(270)
p(985) = p(9)*p(271)
p(986) = p(9)*p(321)
p(987) = p(9)*p(327)
p(988) = p(9)*p(222)
p(989) = p(9)*p(223)
p(990) = p(9)*p(284)
p(991) = p(9)*p(198)
p(992) = p(9)*p(199)
p(993) = p(9)*p(224)
p(994) = p(9)*p(200)
p(995) = p(9)*p(225)
p(996) = p(9)*p(201)
p(997) = p(9)*p(226)
p(998) = p(9)*p(227)
p(999) = p(9)*p(285)
p(1000) = p(14)*p(328)
p(1001) = p(14)*p(329)
p(1002) = p(14)*p(330)
p(1003) = p(9)*p(292)
p(1004) = p(9)*p(233)
p(1005) = p(9)*p(234)
p(1006) = p(9)*p(235)
p(1007) = p(9)*p(293)
p(1008) = p(9)*p(236)
p(1009) = p(9)*p(204)
p(1010) = p(9)*p(205)
p(1011) = p(9)*p(237)
p(1012) = p(9)*p(238)
p(1013) = p(9)*p(206)
p(1014) = p(9)*p(239)
p(1015) = p(9)*p(240)
p(1016) = p(9)*p(241)
p(1017) = p(9)*p(294)
p(1018) = p(14)*p(276)
p(1019) = p(14)*p(277)
p(1020) = p(14)*p(278)
p(1021) = p(14)*p(279)
p(1022) = p(14)*p(280)
p(1023) = p(14)*p(281)
p(1024) = p(14)*p(282)
p(1025) = p(14)*p(283)
p(1026) = p(9)*p(244)
p(1027) = p(9)*p(295)
p(1028) = p(9)*p(210)
p(1029) = p(9)*p(245)
p(1030) = p(9)*p(246)
p(1031) = p(9)*p(296)
p(1032) = p(14)*p(222)
p(1033) = p(14)*p(223)
p(1034) = p(14)*p(224)
p(1035) = p(14)*p(225)
p(1036) = p(14)*p(226)
p(1037) = p(14)*p(227)
p(1038) = p(14)*p(202)
p(1039) = p(14)*p(203)
p(1040) = p(9)*p(302)
p(1041) = p(9)*p(252)
p(1042) = p(9)*p(253)
p(1043) = p(9)*p(303)
p(1044) = p(9)*p(254)
p(1045) = p(9)*p(213)
p(1046) = p(9)*p(255)
p(1047) = p(9)*p(256)
p(1048) = p(9)*p(257)
p(1049) = p(9)*p(304)
p(1050) = p(14)*p(286)
p(1051) = p(14)*p(287)
p(1052) = p(14)*p(288)
p(1053) = p(14)*p(289)
p(1054) = p(14)*p(290)
p(1055) = p(14)*p(291)
p(1056) = p(9)*p(305)
p(1057) = p(9)*p(260)
p(1058) = p(9)*p(306)
p(1059) = p(9)*p(261)
p(1060) = p(9)*p(262)
p(1061) = p(9)*p(307)
p(1062) = p(14)*p(233)
p(1063) = p(14)*p(234)
p(1064) = p(14)*p(235)
p(1065) = p(14)*p(236)
p(1066) = p(14)*p(237)
p(1067) = p(14)*p(238)
p(1068) = p(14)*p(239)
p(1069) = p(14)*p(240)
p(1070) = p(14)*p(241)
p(1071) = p(14)*p(207)
p(1072) = p(14)*p(208)
p(1073) = p(14)*p(209)
p(1074) = p(9)*p(309)
p(1075) = p(9)*p(310)
p(1076) = p(14)*p(244)
p(1077) = p(14)*p(245)
p(1078) = p(14)*p(246)
p(1079) = p(14)*p(211)
p(1080) = p(14)*p(212)
p(1081) = p(9)*p(315)
p(1082) = p(9)*p(267)
p(1083) = p(9)*p(316)
p(1084) = p(9)*p(268)
p(1085) = p(9)*p(269)
p(1086) = p(9)*p(317)
p(1087) = p(14)*p(297)
p(1088) = p(14)*p(298)
p(1089) = p(14)*p(299)
p(1090) = p(14)*p(300)
p(1091) = p(14)*p(301)
p(1092) = p(9)*p(318)
p(1093) = p(9)*p(319)
p(1094) = p(9)*p(320)
p(1095) = p(14)*p(252)
p(1096) = p(14)*p(253)
p(1097) = p(14)*p(254)
p(1098) = p(14)*p(255)
p(1099) = p(14)*p(256)
p(1100) = p(14)*p(257)
p(1101) = p(14)*p(214)
p(1102) = p(14)*p(215)
p(1103) = p(9)*p(339)
p(1104) = p(14)*p(260)
p(1105) = p(14)*p(261)
p(1106) = p(14)*p(262)
p(1107) = p(9)*p(324)
p(1108) = p(9)*p(325)
p(1109) = p(9)*p(326)
p(1110) = p(14)*p(312)
p(1111) = p(14)*p(313)
p(1112) = p(14)*p(314)
p(1113) = p(9)*p(342)
p(1114) = p(14)*p(267)
p(1115) = p(14)*p(268)
p(1116) = p(14)*p(269)
p(1117) = p(9)*p(345)
p(1118) = p(14)*p(322)
p(1119) = p(14)*p(323)
p(1120) = p(9)*p(331)
p(1121) = p(9)*p(276)
p(1122) = p(9)*p(216)
p(1123) = p(9)*p(277)
p(1124) = p(9)*p(217)
p(1125) = p(9)*p(218)
p(1126) = p(9)*p(278)
p(1127) = p(9)*p(219)
p(1128) = p(9)*p(220)
p(1129) = p(9)*p(279)
p(1130) = p(9)*p(280)
p(1131) = p(9)*p(221)
p(1132) = p(9)*p(281)
p(1133) = p(9)*p(282)
p(1134) = p(9)*p(283)
p(1135) = p(9)*p(332)
p(1136) = p(3)*p(401) - p(988)
p(1137) = p(7)*p(401) - p(991)
p(1138) = p(49)*p(114) - p(996)
p(1139) = p(24)*p(194) - p(999)
p(1140) = p(3)*p(504) - p(990) - p(1139)
p(1141) = p(1)*p(695) - p(990) - p(1140)
p(1142) = p(1)*p(696) - p(993) - p(1141)
p(1143) = p(6)*p(566) - p(1136)
p(1144) = p(32)*p(178) - p(991)
p(1145) = p(39)*p(150) - p(996)
p(1146) = p(3)*p(505) - p(999)
p(1147) = p(14)*p(346)
p(1148) = p(9)*p(333)
p(1149) = p(9)*p(286)
p(1150) = p(9)*p(229)
p(1151) = p(9)*p(287)
p(1152) = p(9)*p(230)
p(1153) = p(9)*p(231)
p(1154) = p(9)*p(288)
p(1155) = p(9)*p(232)
p(1156) = p(9)*p(289)
p(1157) = p(9)*p(290)
p(1158) = p(9)*p(291)
p(1159) = p(9)*p(334)
p(1160) = p(24)*p(181) - p(963)
p(1161) = p(41)*p(114) - p(960)
p(1162) = p(4)*p(401) - p(961)
p(1163) = p(1)*p(713) - p(1005) - p(1162)
p(1164) = p(1)*p(714) - p(1006) - p(1163)
p(1165) = p(1)*p(715) - p(1007) - p(1164)
p(1166) = p(13)*p(331) - p(1147)
p(1167) = p(2)*p(695) - p(1002) - p(1166)
p(1168) = p(32)*p(181) - p(1147)
p(1169) = p(1)*p(717) - p(1007) - p(1167)
p(1170) = p(73)*p(91) - p(1147)
p(1171) = p(1)*p(719) - p(1011) - p(1169)
p(1172) = p(41)*p(150) - p(1147)
p(1173) = p(13)*p(279) - p(961)
p(1174) = p(16)*p(285) - p(1147)
p(1175) = p(69)*p(73) - p(960)
p(1176) = p(4)*p(505) - p(1147)
p(1177) = p(11)*p(285) - p(963)
p(1178) = p(14)*p(331)
p(1179) = p(14)*p(332)
p(1180) = p(3)*p(430) - p(1026)
p(1181) = p(7)*p(430) - p(1028)
p(1182) = p(49)*p(130) - p(1031)
p(1183) = p(3)*p(517) - p(1027) - p(1182)
p(1184) = p(1)*p(730) - p(1027) - p(1183)
p(1185) = p(45)*p(178) - p(1180)
p(1186) = p(39)*p(160) - p(1028)
p(1187) = p(3)*p(518) - p(1031)
p(1188) = p(14)*p(284)
p(1189) = p(14)*p(285)
p(1190) = p(14)*p(228)
p(1191) = p(9)*p(335)
p(1192) = p(9)*p(297)
p(1193) = p(9)*p(250)
p(1194) = p(9)*p(298)
p(1195) = p(9)*p(251)
p(1196) = p(9)*p(299)
p(1197) = p(9)*p(300)
p(1198) = p(9)*p(301)
p(1199) = p(9)*p(336)
p(1200) = p(24)*p(183) - p(964)
p(1201) = p(43)*p(114) - p(966)
p(1202) = p(8)*p(401) - p(1021)
p(1203) = p(1)*p(745) - p(1042) - p(1202)
p(1204) = p(1)*p(746) - p(1043) - p(1203)
p(1205) = p(13)*p(333) - p(1178)
p(1206) = p(2)*p(717) - p(1025) - p(1183)
p(1207) = p(32)*p(183) - p(1178)
p(1208) = p(1)*p(748) - p(1043) - p(1206)
p(1209) = p(73)*p(93) - p(1178)
p(1210) = p(6)*p(509) - p(1021)
p(1211) = p(43)*p(150) - p(1178)
p(1212) = p(32)*p(153) - p(966)
p(1213) = p(18)*p(285) - p(1178)
p(1214) = p(34)*p(150) - p(964)
p(1215) = p(14)*p(333)
p(1216) = p(14)*p(334)
p(1217) = p(41)*p(130) - p(973)
p(1218) = p(4)*p(430) - p(970)
p(1219) = p(1)*p(758) - p(1057) - p(1218)
p(1220) = p(1)*p(759) - p(1058) - p(1219)
p(1221) = p(81)*p(91) - p(1189)
p(1222) = p(2)*p(730) - p(1034) - p(1221)
p(1223) = p(41)*p(160) - p(1189)
p(1224) = p(1)*p(761) - p(1058) - p(1222)
p(1225) = p(16)*p(296) - p(1189)
p(1226) = p(69)*p(81) - p(970)
p(1227) = p(4)*p(518) - p(1189)
p(1228) = p(11)*p(296) - p(973)
p(1229) = p(14)*p(292)
p(1230) = p(14)*p(293)
p(1231) = p(14)*p(294)
p(1232) = p(14)*p(242)
p(1233) = p(14)*p(243)
p(1234) = p(3)*p(464) - p(1074)
p(1235) = p(7)*p(464) - p(1075)
p(1236) = p(3)*p(531) - p(1074) - p(1235)
p(1237) = p(39)*p(187) - p(1234)
p(1238) = p(3)*p(532) - p(1075)
p(1239) = p(14)*p(295)
p(1240) = p(14)*p(296)
p(1241) = p(14)*p(247)
p(1242) = p(14)*p(248)
p(1243) = p(14)*p(249)
p(1244) = p(9)*p(337)
p(1245) = p(9)*p(312)
p(1246) = p(9)*p(266)
p(1247) = p(9)*p(313)
p(1248) = p(9)*p(314)
p(1249) = p(9)*p(338)
p(1250) = p(24)*p(185) - p(975)
p(1251) = p(46)*p(114) - p(977)
p(1252) = p(1)*p(783) - p(1082) - p(1251)
p(1253) = p(1)*p(784) - p(1083) - p(1252)
p(1254) = p(13)*p(335) - p(1215)
p(1255) = p(2)*p(748) - p(1055) - p(1222)
p(1256) = p(32)*p(185) - p(1215)
p(1257) = p(1)*p(786) - p(1083) - p(1255)
p(1258) = p(73)*p(95) - p(1215)
p(1259) = p(13)*p(299) - p(977)
p(1260) = p(46)*p(150) - p(1215)
p(1261) = p(73)*p(84) - p(975)
p(1262) = p(14)*p(335)
p(1263) = p(14)*p(336)
p(1264) = p(43)*p(130) - p(974)
p(1265) = p(8)*p(430) - p(1038)
p(1266) = p(1)*p(794) - p(1093) - p(1265)
p(1267) = p(81)*p(93) - p(1190)
p(1268) = p(2)*p(761) - p(1066) - p(1236)
p(1269) = p(43)*p(160) - p(1190)
p(1270) = p(45)*p(153) - p(1038)
p(1271) = p(18)*p(296) - p(1190)
p(1272) = p(34)*p(160) - p(974)
p(1273) = p(14)*p(302)
p(1274) = p(14)*p(303)
p(1275) = p(14)*p(304)
p(1276) = p(14)*p(258)
p(1277) = p(14)*p(259)
p(1278) = p(4)*p(464) - p(983)
p(1279) = p(1)*p(804) - p(1103) - p(1278)
p(1280) = p(16)*p(310) - p(1240)
p(1281) = p(2)*p(772) - p(1077) - p(1280)
p(1282) = p(4)*p(532) - p(1240)
p(1283) = p(11)*p(310) - p(983)
p(1284) = p(14)*p(305)
p(1285) = p(14)*p(306)
p(1286) = p(14)*p(307)
p(1287) = p(14)*p(263)
p(1288) = p(14)*p(264)
p(1289) = p(14)*p(265)
p(1290) = p(6)*p(464) - p(1104)
p(1291) = p(3)*p(578) - p(1290)
p(1292) = p(14)*p(309)
p(1293) = p(14)*p(310)
p(1294) = p(9)*p(340)
p(1295) = p(9)*p(322)
p(1296) = p(9)*p(323)
p(1297) = p(9)*p(341)
p(1298) = p(24)*p(188) - p(984)
p(1299) = p(50)*p(114) - p(1089)
p(1300) = p(1)*p(813) - p(1108) - p(1299)
p(1301) = p(13)*p(337) - p(1262)
p(1302) = p(2)*p(786) - p(1091) - p(1268)
p(1303) = p(32)*p(188) - p(1262)
p(1304) = p(6)*p(536) - p(1089)
p(1305) = p(73)*p(98) - p(1262)
p(1306) = p(32)*p(173) - p(984)
p(1307) = p(14)*p(337)
p(1308) = p(14)*p(338)
p(1309) = p(46)*p(130) - p(982)
p(1310) = p(1)*p(820) - p(1113) - p(1309)
p(1311) = p(81)*p(95) - p(1232)
p(1312) = p(2)*p(796) - p(1098) - p(1281)
p(1313) = p(46)*p(160) - p(1232)
p(1314) = p(81)*p(84) - p(982)
p(1315) = p(14)*p(315)
p(1316) = p(14)*p(316)
p(1317) = p(14)*p(317)
p(1318) = p(14)*p(270)
p(1319) = p(14)*p(271)
p(1320) = p(8)*p(464) - p(1079)
p(1321) = p(18)*p(310) - p(1242)
p(1322) = p(34)*p(187) - p(1079)
p(1323) = p(14)*p(318)
p(1324) = p(14)*p(319)
p(1325) = p(14)*p(320)
p(1326) = p(9)*p(343)
p(1327) = p(9)*p(344)
p(1328) = p(24)*p(191) - p(987)
p(1329) = p(1)*p(827) - p(1117) - p(1328)
p(1330) = p(13)*p(340) - p(1307)
p(1331) = p(2)*p(815) - p(1112) - p(1312)
p(1332) = p(32)*p(191) - p(1307)
p(1333) = p(13)*p(323) - p(987)
p(1334) = p(14)*p(340)
p(1335) = p(14)*p(341)
p(1336) = p(50)*p(130) - p(1102)
p(1337) = p(81)*p(98) - p(1276)
p(1338) = p(45)*p(173) - p(1102)
p(1339) = p(14)*p(324)
p(1340) = p(14)*p(325)
p(1341) = p(14)*p(326)
p(1342) = p(9)*p(347)
p(1343) = p(24)*p(195) - p(1119)
p(1344) = p(13)*p(343) - p(1334)
p(1345) = p(6)*p(561) - p(1119)
p(1346) = p(14)*p(343)
p(1347) = p(14)*p(344)
p(1348) = p(9)*p(272)
p(1349) = p(9)*p(273)
p(1350) = p(9)*p(274)
p(1351) = p(9)*p(328)
p(1352) = p(9)*p(275)
p(1353) = p(9)*p(329)
p(1354) = p(9)*p(330)
p(1355) = p(9)*p(346)
p(1356) = p(3)*p(570) - p(1120)
p(1357) = p(39)*p(181) - p(1122)
p(1358) = p(41)*p(178) - p(1125)
p(1359) = p(4)*p(566) - p(1129)
p(1360) = p(1)*p(842) - p(1126) - p(1359)
p(1361) = p(1)*p(843) - p(1129)
p(1362) = p(1)*p(844) - p(1130) - p(1360)
p(1363) = p(21)*p(279) - p(1125)
p(1364) = p(1)*p(846) - p(1133) - p(1362)
p(1365) = p(69)*p(101) - p(1122)
p(1366) = p(1)*p(848) - p(1135) - p(1364)
p(1367) = p(11)*p(346) - p(1120)
p(1368) = p(1)*p(850) - p(1140) - p(1139)
p(1369) = p(1)*p(851) - p(1146)
p(1370) = p(3)*p(572) - p(1148)
p(1371) = p(39)*p(183) - p(1150)
p(1372) = p(43)*p(178) - p(1153)
p(1373) = p(1)*p(854) - p(1151) - p(1372)
p(1374) = p(2)*p(843) - p(1143)
p(1375) = p(1)*p(855) - p(1154) - p(1373)
p(1376) = p(7)*p(509) - p(1153)
p(1377) = p(1)*p(857) - p(1157) - p(1375)
p(1378) = p(49)*p(153) - p(1150)
p(1379) = p(1)*p(859) - p(1159) - p(1377)
p(1380) = p(34)*p(194) - p(1148)
p(1381) = p(6)*p(570) - p(1147)
p(1382) = p(1)*p(862) - p(1167) - p(1165)
p(1383) = p(2)*p(851) - p(1147)
p(1384) = p(1)*p(864) - p(1183) - p(1182)
p(1385) = p(1)*p(865) - p(1187)
p(1386) = p(3)*p(574) - p(1191)
p(1387) = p(39)*p(185) - p(1193)
p(1388) = p(46)*p(178) - p(1196)
p(1389) = p(1)*p(868) - p(1194) - p(1388)
p(1390) = p(1)*p(869) - p(1196)
p(1391) = p(1)*p(870) - p(1197) - p(1389)
p(1392) = p(21)*p(299) - p(1193)
p(1393) = p(1)*p(872) - p(1199) - p(1391)
p(1394) = p(84)*p(101) - p(1191)
p(1395) = p(6)*p(572) - p(1178)
p(1396) = p(1)*p(875) - p(1206) - p(1204)
p(1397) = p(8)*p(505) - p(1178)
p(1398) = p(45)*p(181) - p(1189)
p(1399) = p(1)*p(878) - p(1222) - p(1220)
p(1400) = p(2)*p(865) - p(1189)
p(1401) = p(1)*p(880) - p(1236) - p(1235)
p(1402) = p(1)*p(881) - p(1238)
p(1403) = p(3)*p(576) - p(1244)
p(1404) = p(39)*p(188) - p(1246)
p(1405) = p(1)*p(884) - p(1245) - p(1404)
p(1406) = p(2)*p(869) - p(1210)
p(1407) = p(1)*p(885) - p(1247) - p(1405)
p(1408) = p(7)*p(536) - p(1246)
p(1409) = p(1)*p(887) - p(1249) - p(1407)
p(1410) = p(49)*p(173) - p(1244)
p(1411) = p(6)*p(574) - p(1215)
p(1412) = p(1)*p(890) - p(1255) - p(1253)
p(1413) = p(22)*p(285) - p(1215)
p(1414) = p(45)*p(183) - p(1190)
p(1415) = p(1)*p(893) - p(1268) - p(1266)
p(1416) = p(8)*p(518) - p(1190)
p(1417) = p(41)*p(187) - p(1240)
p(1418) = p(1)*p(896) - p(1281) - p(1279)
p(1419) = p(2)*p(881) - p(1240)
p(1420) = p(14)*p(308)
p(1421) = p(4)*p(547) - p(1106) - p(1320)
p(1422) = p(1)*p(900) - p(1291)
p(1423) = p(14)*p(311)
p(1424) = p(3)*p(579) - p(1294)
p(1425) = p(39)*p(191) - p(1296)
p(1426) = p(1)*p(903) - p(1295) - p(1425)
p(1427) = p(1)*p(904) - p(1296)
p(1428) = p(1)*p(905) - p(1297) - p(1426)
p(1429) = p(21)*p(323) - p(1294)
p(1430) = p(6)*p(576) - p(1262)
p(1431) = p(1)*p(908) - p(1302) - p(1300)
p(1432) = p(50)*p(150) - p(1262)
p(1433) = p(45)*p(185) - p(1232)
p(1434) = p(1)*p(911) - p(1312) - p(1310)
p(1435) = p(22)*p(296) - p(1232)
p(1436) = p(43)*p(187) - p(1242)
p(1437) = p(1)*p(915) - p(1322) - p(1320)
p(1438) = p(8)*p(532) - p(1242)
p(1439) = p(14)*p(321)
p(1440) = p(4)*p(578) - p(1293)
p(1441) = p(2)*p(899) - p(1292) - p(1440)
p(1442) = p(2)*p(900) - p(1293)
p(1443) = p(14)*p(339)
p(1444) = p(3)*p(582) - p(1326)
p(1445) = p(2)*p(903) - p(1303) - p(1299)
p(1446) = p(2)*p(904) - p(1304)
p(1447) = p(1)*p(919) - p(1327) - p(1445)
p(1448) = p(7)*p(561) - p(1326)
p(1449) = p(6)*p(579) - p(1307)
p(1450) = p(1)*p(922) - p(1331) - p(1329)
p(1451) = p(73)*p(102) - p(1307)
p(1452) = p(45)*p(188) - p(1276)
p(1453) = p(1)*p(925) - p(1338) - p(1336)
p(1454) = p(50)*p(160) - p(1276)
p(1455) = p(14)*p(327)
p(1456) = p(46)*p(187) - p(1243)
p(1457) = p(2)*p(915) - p(1324) - p(1441)
p(1458) = p(22)*p(310) - p(1243)
p(1459) = p(14)*p(342)
p(1460) = p(3)*p(585) - p(1342)
p(1461) = p(2)*p(919) - p(1332) - p(1329)
p(1462) = p(1)*p(929) - p(1342)
p(1463) = p(6)*p(582) - p(1334)
p(1464) = p(1)*p(931) - p(1345) - p(1343)
p(1465) = p(32)*p(195) - p(1334)
p(1466) = p(45)*p(191) - p(1318)
p(1467) = p(2)*p(925) - p(1340) - p(1457)
p(1468) = p(81)*p(102) - p(1318)
p(1469) = p(14)*p(345)
p(1470) = p(2)*p(928) - p(1344) - p(1343)
p(1471) = p(2)*p(929) - p(1345)
p(1472) = p(6)*p(585) - p(1346)
p(1473) = p(2)*p(931) - p(1347) - p(1467)
p(1474) = p(13)*p(347) - p(1346)
p(1475) = p(14)*p(347)
p(1476) = p(3)*p(566) - p(1351)
p(1477) = p(7)*p(566) - p(1349)
p(1478) = p(49)*p(178) - p(1348)
p(1479) = p(39)*p(194) - p(1352)
p(1480) = p(3)*p(588) - p(1355)
p(1481) = p(1)*p(938) - p(1356)
p(1482) = p(2)*p(958) - p(1481)
p(1483) = p(1)*p(940) - p(1370)
p(1484) = p(8)*p(588) - p(1483)
p(1485) = p(1)*p(942) - p(1386)
p(1486) = p(22)*p(346) - p(1485)
p(1487) = p(1)*p(944) - p(1403)
p(1488) = p(50)*p(194) - p(1487)
p(1489) = p(1)*p(946) - p(1424)
p(1490) = p(101)*p(102) - p(1489)
p(1491) = p(6)*p(578) - p(1443)
p(1492) = p(1)*p(949) - p(1444)
p(1493) = p(49)*p(195) - p(1492)
p(1494) = p(8)*p(578) - p(1423)
p(1495) = p(1)*p(952) - p(1460)
p(1496) = p(21)*p(347) - p(1495)
p(1497) = p(50)*p(187) - p(1420)
p(1498) = p(1)*p(955) - p(1470)
p(1499) = p(7)*p(589) - p(1498)
p(1500) = p(45)*p(195) - p(1455)
p(1501) = p(2)*p(955) - p(1472)
p(1502) = p(1)*p(959) - p(1501)
p(1503) = p(6)*p(589) - p(1475)
p(1504) = p(1)*p(958) - p(1480)
p(1505) = p(2)*p(959) - p(1503)
p(1506) = p(9)*p(348)
p(1507) = p(9)*p(349)
p(1508) = p(9)*p(360)
p(1509) = p(9)*p(361)
p(1510) = p(9)*p(409)
p(1511) = p(9)*p(428)
p(1512) = p(9)*p(368)
p(1513) = p(9)*p(350)
p(1514) = p(9)*p(369)
p(1515) = p(9)*p(351)
p(1516) = p(9)*p(370)
p(1517) = p(9)*p(371)
p(1518) = p(9)*p(372)
p(1519) = p(9)*p(429)
p(1520) = p(9)*p(375)
p(1521) = p(9)*p(435)
p(1522) = p(9)*p(352)
p(1523) = p(9)*p(376)
p(1524) = p(9)*p(377)
p(1525) = p(9)*p(436)
p(1526) = p(9)*p(378)
p(1527) = p(9)*p(437)
p(1528) = p(9)*p(451)
p(1529) = p(9)*p(382)
p(1530) = p(9)*p(353)
p(1531) = p(9)*p(383)
p(1532) = p(9)*p(384)
p(1533) = p(9)*p(452)
p(1534) = p(9)*p(459)
p(1535) = p(9)*p(386)
p(1536) = p(9)*p(460)
p(1537) = p(9)*p(387)
p(1538) = p(9)*p(388)
p(1539) = p(9)*p(461)
p(1540) = p(9)*p(462)
p(1541) = p(9)*p(463)
p(1542) = p(9)*p(466)
p(1543) = p(9)*p(467)
p(1544) = p(9)*p(533)
p(1545) = p(9)*p(476)
p(1546) = p(9)*p(390)
p(1547) = p(9)*p(391)
p(1548) = p(9)*p(477)
p(1549) = p(9)*p(481)
p(1550) = p(9)*p(482)
p(1551) = p(9)*p(483)
p(1552) = p(9)*p(545)
p(1553) = p(9)*p(549)
p(1554) = p(9)*p(488)
p(1555) = p(9)*p(489)
p(1556) = p(9)*p(559)
p(1557) = p(9)*p(565)
p(1558) = p(9)*p(401)
p(1559) = p(9)*p(402)
p(1560) = p(9)*p(403)
p(1561) = p(9)*p(504)
p(1562) = p(9)*p(354)
p(1563) = p(9)*p(355)
p(1564) = p(9)*p(404)
p(1565) = p(9)*p(356)
p(1566) = p(9)*p(357)
p(1567) = p(9)*p(405)
p(1568) = p(9)*p(358)
p(1569) = p(9)*p(406)
p(1570) = p(9)*p(359)
p(1571) = p(9)*p(407)
p(1572) = p(9)*p(408)
p(1573) = p(9)*p(505)
p(1574) = p(14)*p(566)
p(1575) = p(14)*p(567)
p(1576) = p(14)*p(568)
p(1577) = p(14)*p(569)
p(1578) = p(9)*p(514)
p(1579) = p(9)*p(416)
p(1580) = p(9)*p(417)
p(1581) = p(9)*p(418)
p(1582) = p(9)*p(419)
p(1583) = p(9)*p(515)
p(1584) = p(9)*p(420)
p(1585) = p(9)*p(362)
p(1586) = p(9)*p(363)
p(1587) = p(9)*p(364)
p(1588) = p(9)*p(421)
p(1589) = p(9)*p(422)
p(1590) = p(9)*p(365)
p(1591) = p(9)*p(366)
p(1592) = p(9)*p(423)
p(1593) = p(9)*p(424)
p(1594) = p(9)*p(367)
p(1595) = p(9)*p(425)
p(1596) = p(9)*p(426)
p(1597) = p(9)*p(427)
p(1598) = p(9)*p(516)
p(1599) = p(14)*p(495)
p(1600) = p(14)*p(496)
p(1601) = p(14)*p(497)
p(1602) = p(14)*p(498)
p(1603) = p(14)*p(499)
p(1604) = p(14)*p(500)
p(1605) = p(14)*p(501)
p(1606) = p(14)*p(502)
p(1607) = p(14)*p(503)
p(1608) = p(9)*p(430)
p(1609) = p(9)*p(431)
p(1610) = p(9)*p(517)
p(1611) = p(9)*p(373)
p(1612) = p(9)*p(432)
p(1613) = p(9)*p(374)
p(1614) = p(9)*p(433)
p(1615) = p(9)*p(434)
p(1616) = p(9)*p(518)
p(1617) = p(14)*p(401)
p(1618) = p(14)*p(402)
p(1619) = p(14)*p(403)
p(1620) = p(14)*p(404)
p(1621) = p(14)*p(405)
p(1622) = p(14)*p(406)
p(1623) = p(14)*p(407)
p(1624) = p(14)*p(408)
p(1625) = p(14)*p(360)
p(1626) = p(14)*p(361)
p(1627) = p(9)*p(525)
p(1628) = p(9)*p(442)
p(1629) = p(9)*p(443)
p(1630) = p(9)*p(444)
p(1631) = p(9)*p(526)
p(1632) = p(9)*p(445)
p(1633) = p(9)*p(379)
p(1634) = p(9)*p(380)
p(1635) = p(9)*p(446)
p(1636) = p(9)*p(447)
p(1637) = p(9)*p(381)
p(1638) = p(9)*p(448)
p(1639) = p(9)*p(449)
p(1640) = p(9)*p(450)
p(1641) = p(9)*p(527)
p(1642) = p(14)*p(506)
p(1643) = p(14)*p(507)
p(1644) = p(14)*p(508)
p(1645) = p(14)*p(509)
p(1646) = p(14)*p(510)
p(1647) = p(14)*p(511)
p(1648) = p(14)*p(512)
p(1649) = p(14)*p(513)
p(1650) = p(9)*p(528)
p(1651) = p(9)*p(453)
p(1652) = p(9)*p(454)
p(1653) = p(9)*p(529)
p(1654) = p(9)*p(455)
p(1655) = p(9)*p(385)
p(1656) = p(9)*p(456)
p(1657) = p(9)*p(457)
p(1658) = p(9)*p(458)
p(1659) = p(9)*p(530)
p(1660) = p(14)*p(416)
p(1661) = p(14)*p(417)
p(1662) = p(14)*p(418)
p(1663) = p(14)*p(419)
p(1664) = p(14)*p(420)
p(1665) = p(14)*p(421)
p(1666) = p(14)*p(422)
p(1667) = p(14)*p(423)
p(1668) = p(14)*p(424)
p(1669) = p(14)*p(425)
p(1670) = p(14)*p(426)
p(1671) = p(14)*p(427)
p(1672) = p(14)*p(368)
p(1673) = p(14)*p(369)
p(1674) = p(14)*p(370)
p(1675) = p(14)*p(371)
p(1676) = p(14)*p(372)
p(1677) = p(9)*p(464)
p(1678) = p(9)*p(531)
p(1679) = p(9)*p(465)
p(1680) = p(9)*p(532)
p(1681) = p(14)*p(430)
p(1682) = p(14)*p(431)
p(1683) = p(14)*p(432)
p(1684) = p(14)*p(433)
p(1685) = p(14)*p(434)
p(1686) = p(14)*p(375)
p(1687) = p(14)*p(376)
p(1688) = p(14)*p(377)
p(1689) = p(14)*p(378)
p(1690) = p(9)*p(539)
p(1691) = p(9)*p(470)
p(1692) = p(9)*p(471)
p(1693) = p(9)*p(540)
p(1694) = p(9)*p(472)
p(1695) = p(9)*p(389)
p(1696) = p(9)*p(473)
p(1697) = p(9)*p(474)
p(1698) = p(9)*p(475)
p(1699) = p(9)*p(541)
p(1700) = p(14)*p(519)
p(1701) = p(14)*p(520)
p(1702) = p(14)*p(521)
p(1703) = p(14)*p(522)
p(1704) = p(14)*p(523)
p(1705) = p(14)*p(524)
p(1706) = p(9)*p(542)
p(1707) = p(9)*p(478)
p(1708) = p(9)*p(543)
p(1709) = p(9)*p(479)
p(1710) = p(9)*p(480)
p(1711) = p(9)*p(544)
p(1712) = p(14)*p(442)
p(1713) = p(14)*p(443)
p(1714) = p(14)*p(444)
p(1715) = p(14)*p(445)
p(1716) = p(14)*p(446)
p(1717) = p(14)*p(447)
p(1718) = p(14)*p(448)
p(1719) = p(14)*p(449)
p(1720) = p(14)*p(450)
p(1721) = p(14)*p(382)
p(1722) = p(14)*p(383)
p(1723) = p(14)*p(384)
p(1724) = p(9)*p(546)
p(1725) = p(9)*p(547)
p(1726) = p(9)*p(548)
p(1727) = p(14)*p(453)
p(1728) = p(14)*p(454)
p(1729) = p(14)*p(455)
p(1730) = p(14)*p(456)
p(1731) = p(14)*p(457)
p(1732) = p(14)*p(458)
p(1733) = p(14)*p(386)
p(1734) = p(14)*p(387)
p(1735) = p(14)*p(388)
p(1736) = p(9)*p(578)
p(1737) = p(14)*p(464)
p(1738) = p(14)*p(465)
p(1739) = p(9)*p(553)
p(1740) = p(9)*p(485)
p(1741) = p(9)*p(554)
p(1742) = p(9)*p(486)
p(1743) = p(9)*p(487)
p(1744) = p(9)*p(555)
p(1745) = p(14)*p(534)
p(1746) = p(14)*p(535)
p(1747) = p(14)*p(536)
p(1748) = p(14)*p(537)
p(1749) = p(14)*p(538)
p(1750) = p(9)*p(556)
p(1751) = p(9)*p(557)
p(1752) = p(9)*p(558)
p(1753) = p(14)*p(470)
p(1754) = p(14)*p(471)
p(1755) = p(14)*p(472)
p(1756) = p(14)*p(473)
p(1757) = p(14)*p(474)
p(1758) = p(14)*p(475)
p(1759) = p(14)*p(390)
p(1760) = p(14)*p(391)
p(1761) = p(9)*p(581)
p(1762) = p(14)*p(478)
p(1763) = p(14)*p(479)
p(1764) = p(14)*p(480)
p(1765) = p(9)*p(562)
p(1766) = p(9)*p(563)
p(1767) = p(9)*p(564)
p(1768) = p(14)*p(550)
p(1769) = p(14)*p(551)
p(1770) = p(14)*p(552)
p(1771) = p(9)*p(584)
p(1772) = p(14)*p(485)
p(1773) = p(14)*p(486)
p(1774) = p(14)*p(487)
p(1775) = p(9)*p(587)
p(1776) = p(14)*p(560)
p(1777) = p(14)*p(561)
p(1778) = p(9)*p(570)
p(1779) = p(9)*p(495)
p(1780) = p(9)*p(392)
p(1781) = p(9)*p(496)
p(1782) = p(9)*p(393)
p(1783) = p(9)*p(394)
p(1784) = p(9)*p(497)
p(1785) = p(9)*p(395)
p(1786) = p(9)*p(396)
p(1787) = p(9)*p(397)
p(1788) = p(9)*p(498)
p(1789) = p(9)*p(398)
p(1790) = p(9)*p(399)
p(1791) = p(9)*p(499)
p(1792) = p(9)*p(500)
p(1793) = p(9)*p(400)
p(1794) = p(9)*p(501)
p(1795) = p(9)*p(502)
p(1796) = p(9)*p(503)
p(1797) = p(9)*p(571)
p(1798) = p(1)*p(1136) - p(1558)
p(1799) = p(21)*p(401) - p(1565)
p(1800) = p(101)*p(114) - p(1570)
p(1801) = p(24)*p(346) - p(1573)
p(1802) = p(3)*p(850) - p(1561) - p(1801)
p(1803) = p(1)*p(1140) - p(1561) - p(1802)
p(1804) = p(1)*p(1141) - p(1564) - p(1803)
p(1805) = p(1)*p(1142) - p(1567) - p(1804)
p(1806) = p(13)*p(566) - p(1558)
p(1807) = p(73)*p(178) - p(1565)
p(1808) = p(39)*p(285) - p(1570)
p(1809) = p(3)*p(851) - p(1573)
p(1810) = p(14)*p(588)
p(1811) = p(9)*p(572)
p(1812) = p(9)*p(506)
p(1813) = p(9)*p(410)
p(1814) = p(9)*p(507)
p(1815) = p(9)*p(411)
p(1816) = p(9)*p(412)
p(1817) = p(9)*p(508)
p(1818) = p(9)*p(413)
p(1819) = p(9)*p(414)
p(1820) = p(9)*p(509)
p(1821) = p(9)*p(510)
p(1822) = p(9)*p(415)
p(1823) = p(9)*p(511)
p(1824) = p(9)*p(512)
p(1825) = p(9)*p(513)
p(1826) = p(9)*p(573)
p(1827) = p(24)*p(331) - p(1510)
p(1828) = p(91)*p(114) - p(1507)
p(1829) = p(16)*p(401) - p(1506)
p(1830) = p(2)*p(1136) - p(1574)
p(1831) = p(1)*p(1163) - p(1581) - p(1830)
p(1832) = p(1)*p(1164) - p(1582) - p(1831)
p(1833) = p(1)*p(1165) - p(1583) - p(1832)
p(1834) = p(13)*p(570) - p(1810)
p(1835) = p(2)*p(1140) - p(1577) - p(1834)
p(1836) = p(32)*p(331) - p(1810)
p(1837) = p(1)*p(1167) - p(1583) - p(1835)
p(1838) = p(73)*p(181) - p(1810)
p(1839) = p(1)*p(1169) - p(1588) - p(1837)
p(1840) = p(91)*p(150) - p(1810)
p(1841) = p(6)*p(843) - p(1574)
p(1842) = p(41)*p(285) - p(1810)
p(1843) = p(32)*p(279) - p(1506)
p(1844) = p(16)*p(505) - p(1810)
p(1845) = p(69)*p(150) - p(1507)
p(1846) = p(4)*p(851) - p(1810)
p(1847) = p(11)*p(505) - p(1510)
p(1848) = p(14)*p(570)
p(1849) = p(14)*p(571)
p(1850) = p(1)*p(1180) - p(1608)
p(1851) = p(21)*p(430) - p(1613)
p(1852) = p(101)*p(130) - p(1616)
p(1853) = p(3)*p(864) - p(1610) - p(1852)
p(1854) = p(1)*p(1183) - p(1610) - p(1853)
p(1855) = p(1)*p(1184) - p(1612) - p(1854)
p(1856) = p(81)*p(178) - p(1608)
p(1857) = p(39)*p(296) - p(1613)
p(1858) = p(3)*p(865) - p(1616)
p(1859) = p(14)*p(504)
p(1860) = p(14)*p(505)
p(1861) = p(14)*p(409)
p(1862) = p(9)*p(574)
p(1863) = p(9)*p(519)
p(1864) = p(9)*p(438)
p(1865) = p(9)*p(520)
p(1866) = p(9)*p(439)
p(1867) = p(9)*p(440)
p(1868) = p(9)*p(521)
p(1869) = p(9)*p(441)
p(1870) = p(9)*p(522)
p(1871) = p(9)*p(523)
p(1872) = p(9)*p(524)
p(1873) = p(9)*p(575)
p(1874) = p(24)*p(333) - p(1511)
p(1875) = p(93)*p(114) - p(1513)
p(1876) = p(18)*p(401) - p(1516)
p(1877) = p(1)*p(1202) - p(1629) - p(1876)
p(1878) = p(1)*p(1203) - p(1630) - p(1877)
p(1879) = p(1)*p(1204) - p(1631) - p(1878)
p(1880) = p(13)*p(572) - p(1848)
p(1881) = p(2)*p(1167) - p(1607) - p(1853)
p(1882) = p(32)*p(333) - p(1848)
p(1883) = p(1)*p(1206) - p(1631) - p(1881)
p(1884) = p(73)*p(183) - p(1848)
p(1885) = p(1)*p(1208) - p(1635) - p(1883)
p(1886) = p(93)*p(150) - p(1848)
p(1887) = p(13)*p(509) - p(1516)
p(1888) = p(43)*p(285) - p(1848)
p(1889) = p(73)*p(153) - p(1513)
p(1890) = p(18)*p(505) - p(1848)
p(1891) = p(34)*p(285) - p(1511)
p(1892) = p(14)*p(572)
p(1893) = p(14)*p(573)
p(1894) = p(91)*p(130) - p(1525)
p(1895) = p(16)*p(430) - p(1522)
p(1896) = p(2)*p(1180) - p(1617)
p(1897) = p(1)*p(1219) - p(1652) - p(1896)
p(1898) = p(1)*p(1220) - p(1653) - p(1897)
p(1899) = p(81)*p(181) - p(1860)
p(1900) = p(2)*p(1183) - p(1620) - p(1899)
p(1901) = p(91)*p(160) - p(1860)
p(1902) = p(1)*p(1222) - p(1653) - p(1900)
p(1903) = p(41)*p(296) - p(1860)
p(1904) = p(45)*p(279) - p(1617)
p(1905) = p(16)*p(518) - p(1860)
p(1906) = p(69)*p(160) - p(1522)
p(1907) = p(4)*p(865) - p(1860)
p(1908) = p(11)*p(518) - p(1525)
p(1909) = p(14)*p(514)
p(1910) = p(14)*p(515)
p(1911) = p(14)*p(516)
p(1912) = p(14)*p(428)
p(1913) = p(14)*p(429)
p(1914) = p(1)*p(1234) - p(1677)
p(1915) = p(21)*p(464) - p(1680)
p(1916) = p(3)*p(880) - p(1678) - p(1915)
p(1917) = p(1)*p(1236) - p(1678) - p(1916)
p(1918) = p(39)*p(310) - p(1677)
p(1919) = p(3)*p(881) - p(1680)
p(1920) = p(14)*p(517)
p(1921) = p(14)*p(518)
p(1922) = p(14)*p(435)
p(1923) = p(14)*p(436)
p(1924) = p(14)*p(437)
p(1925) = p(9)*p(576)
p(1926) = p(9)*p(534)
p(1927) = p(9)*p(468)
p(1928) = p(9)*p(535)
p(1929) = p(9)*p(469)
p(1930) = p(9)*p(536)
p(1931) = p(9)*p(537)
p(1932) = p(9)*p(538)
p(1933) = p(9)*p(577)
p(1934) = p(24)*p(335) - p(1528)
p(1935) = p(95)*p(114) - p(1530)
p(1936) = p(22)*p(401) - p(1645)
p(1937) = p(1)*p(1252) - p(1692) - p(1936)
p(1938) = p(1)*p(1253) - p(1693) - p(1937)
p(1939) = p(13)*p(574) - p(1892)
p(1940) = p(2)*p(1206) - p(1649) - p(1900)
p(1941) = p(32)*p(335) - p(1892)
p(1942) = p(1)*p(1255) - p(1693) - p(1940)
p(1943) = p(73)*p(185) - p(1892)
p(1944) = p(6)*p(869) - p(1645)
p(1945) = p(95)*p(150) - p(1892)
p(1946) = p(32)*p(299) - p(1530)
p(1947) = p(46)*p(285) - p(1892)
p(1948) = p(84)*p(150) - p(1528)
p(1949) = p(14)*p(574)
p(1950) = p(14)*p(575)
p(1951) = p(93)*p(130) - p(1527)
p(1952) = p(18)*p(430) - p(1526)
p(1953) = p(1)*p(1265) - p(1707) - p(1952)
p(1954) = p(1)*p(1266) - p(1708) - p(1953)
p(1955) = p(81)*p(183) - p(1861)
p(1956) = p(2)*p(1222) - p(1665) - p(1916)
p(1957) = p(93)*p(160) - p(1861)
p(1958) = p(1)*p(1268) - p(1708) - p(1956)
p(1959) = p(43)*p(296) - p(1861)
p(1960) = p(81)*p(153) - p(1526)
p(1961) = p(18)*p(518) - p(1861)
p(1962) = p(34)*p(296) - p(1527)
p(1963) = p(14)*p(525)
p(1964) = p(14)*p(526)
p(1965) = p(14)*p(527)
p(1966) = p(14)*p(451)
p(1967) = p(14)*p(452)
p(1968) = p(16)*p(464) - p(1543)
p(1969) = p(2)*p(1234) - p(1681)
p(1970) = p(1)*p(1279) - p(1725) - p(1969)
p(1971) = p(41)*p(310) - p(1921)
p(1972) = p(2)*p(1236) - p(1683) - p(1971)
p(1973) = p(16)*p(532) - p(1921)
p(1974) = p(69)*p(187) - p(1681)
p(1975) = p(4)*p(881) - p(1921)
p(1976) = p(11)*p(532) - p(1543)
p(1977) = p(14)*p(528)
p(1978) = p(14)*p(529)
p(1979) = p(14)*p(530)
p(1980) = p(14)*p(459)
p(1981) = p(14)*p(460)
p(1982) = p(14)*p(461)
p(1983) = p(14)*p(462)
p(1984) = p(14)*p(463)
p(1985) = p(1)*p(1290) - p(1736)
p(1986) = p(4)*p(806) - p(1761) - p(1732)
p(1987) = p(3)*p(900) - p(1736)
p(1988) = p(14)*p(531)
p(1989) = p(14)*p(532)
p(1990) = p(14)*p(466)
p(1991) = p(14)*p(467)
p(1992) = p(9)*p(579)
p(1993) = p(9)*p(550)
p(1994) = p(9)*p(484)
p(1995) = p(9)*p(551)
p(1996) = p(9)*p(552)
p(1997) = p(9)*p(580)
p(1998) = p(24)*p(337) - p(1545)
p(1999) = p(98)*p(114) - p(1547)
p(2000) = p(1)*p(1299) - p(1740) - p(1999)
p(2001) = p(1)*p(1300) - p(1741) - p(2000)
p(2002) = p(13)*p(576) - p(1949)
p(2003) = p(2)*p(1255) - p(1705) - p(1956)
p(2004) = p(32)*p(337) - p(1949)
p(2005) = p(1)*p(1302) - p(1741) - p(2003)
p(2006) = p(73)*p(188) - p(1949)
p(2007) = p(13)*p(536) - p(1547)
p(2008) = p(98)*p(150) - p(1949)
p(2009) = p(73)*p(173) - p(1545)
p(2010) = p(14)*p(576)
p(2011) = p(14)*p(577)
p(2012) = p(95)*p(130) - p(1540)
p(2013) = p(22)*p(430) - p(1674)
p(2014) = p(1)*p(1310) - p(1751) - p(2013)
p(2015) = p(81)*p(185) - p(1912)
p(2016) = p(2)*p(1268) - p(1716) - p(1972)
p(2017) = p(95)*p(160) - p(1912)
p(2018) = p(45)*p(299) - p(1674)
p(2019) = p(46)*p(296) - p(1912)
p(2020) = p(84)*p(160) - p(1540)
p(2021) = p(14)*p(539)
p(2022) = p(14)*p(540)
p(2023) = p(14)*p(541)
p(2024) = p(14)*p(476)
p(2025) = p(14)*p(477)
p(2026) = p(18)*p(464) - p(1544)
p(2027) = p(1)*p(1320) - p(1761) - p(2026)
p(2028) = p(43)*p(310) - p(1923)
p(2029) = p(2)*p(1281) - p(1730) - p(1986)
p(2030) = p(18)*p(532) - p(1923)
p(2031) = p(34)*p(310) - p(1544)
p(2032) = p(14)*p(542)
p(2033) = p(14)*p(543)
p(2034) = p(14)*p(544)
p(2035) = p(14)*p(481)
p(2036) = p(14)*p(482)
p(2037) = p(14)*p(483)
p(2038) = p(2)*p(1290) - p(1737)
p(2039) = p(4)*p(900) - p(1989)
p(2040) = p(11)*p(578) - p(1737)
p(2041) = p(14)*p(546)
p(2042) = p(14)*p(547)
p(2043) = p(14)*p(548)
p(2044) = p(9)*p(582)
p(2045) = p(9)*p(560)
p(2046) = p(9)*p(561)
p(2047) = p(9)*p(583)
p(2048) = p(24)*p(340) - p(1554)
p(2049) = p(102)*p(114) - p(1747)
p(2050) = p(1)*p(1329) - p(1766) - p(2049)
p(2051) = p(13)*p(579) - p(2010)
p(2052) = p(2)*p(1302) - p(1749) - p(2016)
p(2053) = p(32)*p(340) - p(2010)
p(2054) = p(6)*p(904) - p(1747)
p(2055) = p(73)*p(191) - p(2010)
p(2056) = p(32)*p(323) - p(1554)
p(2057) = p(14)*p(579)
p(2058) = p(14)*p(580)
p(2059) = p(98)*p(130) - p(1552)
p(2060) = p(1)*p(1336) - p(1771) - p(2059)
p(2061) = p(81)*p(188) - p(1966)
p(2062) = p(2)*p(1312) - p(1756) - p(2029)
p(2063) = p(98)*p(160) - p(1966)
p(2064) = p(81)*p(173) - p(1552)
p(2065) = p(14)*p(553)
p(2066) = p(14)*p(554)
p(2067) = p(14)*p(555)
p(2068) = p(14)*p(488)
p(2069) = p(14)*p(489)
p(2070) = p(22)*p(464) - p(1689)
p(2071) = p(46)*p(310) - p(1924)
p(2072) = p(84)*p(187) - p(1689)
p(2073) = p(14)*p(556)
p(2074) = p(14)*p(557)
p(2075) = p(14)*p(558)
p(2076) = p(9)*p(585)
p(2077) = p(9)*p(586)
p(2078) = p(24)*p(343) - p(1557)
p(2079) = p(1)*p(1343) - p(1775) - p(2078)
p(2080) = p(13)*p(582) - p(2057)
p(2081) = p(2)*p(1331) - p(1770) - p(2062)
p(2082) = p(32)*p(343) - p(2057)
p(2083) = p(13)*p(561) - p(1557)
p(2084) = p(14)*p(582)
p(2085) = p(14)*p(583)
p(2086) = p(102)*p(130) - p(1760)
p(2087) = p(81)*p(191) - p(2024)
p(2088) = p(45)*p(323) - p(1760)
p(2089) = p(14)*p(562)
p(2090) = p(14)*p(563)
p(2091) = p(14)*p(564)
p(2092) = p(9)*p(589)
p(2093) = p(24)*p(347) - p(1777)
p(2094) = p(13)*p(585) - p(2084)
p(2095) = p(6)*p(929) - p(1777)
p(2096) = p(14)*p(585)
p(2097) = p(14)*p(586)
p(2098) = p(9)*p(490)
p(2099) = p(9)*p(491)
p(2100) = p(9)*p(492)
p(2101) = p(9)*p(566)
p(2102) = p(9)*p(493)
p(2103) = p(9)*p(567)
p(2104) = p(9)*p(494)
p(2105) = p(9)*p(568)
p(2106) = p(9)*p(569)
p(2107) = p(9)*p(588)
p(2108) = p(3)*p(938) - p(1778)
p(2109) = p(39)*p(331) - p(1780)
p(2110) = p(91)*p(178) - p(1783)
p(2111) = p(16)*p(566) - p(1787)
p(2112) = p(1)*p(1359) - p(1784) - p(2111)
p(2113) = p(2)*p(1476) - p(2112)
p(2114) = p(1)*p(1360) - p(1788) - p(2112)
p(2115) = p(7)*p(843) - p(1787)
p(2116) = p(1)*p(1362) - p(1792) - p(2114)
p(2117) = p(49)*p(279) - p(1783)
p(2118) = p(1)*p(1364) - p(1795) - p(2116)
p(2119) = p(69)*p(194) - p(1780)
p(2120) = p(1)*p(1366) - p(1797) - p(2118)
p(2121) = p(11)*p(588) - p(1778)
p(2122) = p(1)*p(1368) - p(1802) - p(1801)
p(2123) = p(1)*p(1369) - p(1809)
p(2124) = p(3)*p(940) - p(1811)
p(2125) = p(39)*p(333) - p(1813)
p(2126) = p(93)*p(178) - p(1816)
p(2127) = p(18)*p(566) - p(1820)
p(2128) = p(1)*p(1373) - p(1817) - p(2127)
p(2129) = p(1)*p(1374) - p(1820)
p(2130) = p(1)*p(1375) - p(1821) - p(2128)
p(2131) = p(21)*p(509) - p(1816)
p(2132) = p(1)*p(1377) - p(1824) - p(2130)
p(2133) = p(101)*p(153) - p(1813)
p(2134) = p(1)*p(1379) - p(1826) - p(2132)
p(2135) = p(34)*p(346) - p(1811)
p(2136) = p(6)*p(938) - p(1810)
p(2137) = p(1)*p(1382) - p(1835) - p(1833)
p(2138) = p(2)*p(1369) - p(1810)
p(2139) = p(1)*p(1384) - p(1853) - p(1852)
p(2140) = p(1)*p(1385) - p(1858)
p(2141) = p(3)*p(942) - p(1862)
p(2142) = p(39)*p(335) - p(1864)
p(2143) = p(95)*p(178) - p(1867)
p(2144) = p(1)*p(1388) - p(1865) - p(2143)
p(2145) = p(2)*p(1374) - p(1841)
p(2146) = p(1)*p(1389) - p(1868) - p(2144)
p(2147) = p(7)*p(869) - p(1867)
p(2148) = p(1)*p(1391) - p(1871) - p(2146)
p(2149) = p(49)*p(299) - p(1864)
p(2150) = p(1)*p(1393) - p(1873) - p(2148)
p(2151) = p(84)*p(194) - p(1862)
p(2152) = p(6)*p(940) - p(1848)
p(2153) = p(1)*p(1396) - p(1881) - p(1879)
p(2154) = p(8)*p(851) - p(1848)
p(2155) = p(45)*p(331) - p(1860)
p(2156) = p(1)*p(1399) - p(1900) - p(1898)
p(2157) = p(2)*p(1385) - p(1860)
p(2158) = p(1)*p(1401) - p(1916) - p(1915)
p(2159) = p(1)*p(1402) - p(1919)
p(2160) = p(3)*p(944) - p(1925)
p(2161) = p(39)*p(337) - p(1927)
p(2162) = p(98)*p(178) - p(1930)
p(2163) = p(1)*p(1405) - p(1928) - p(2162)
p(2164) = p(1)*p(1406) - p(1930)
p(2165) = p(1)*p(1407) - p(1931) - p(2163)
p(2166) = p(21)*p(536) - p(1927)
p(2167) = p(1)*p(1409) - p(1933) - p(2165)
p(2168) = p(101)*p(173) - p(1925)
p(2169) = p(6)*p(942) - p(1892)
p(2170) = p(1)*p(1412) - p(1940) - p(1938)
p(2171) = p(22)*p(505) - p(1892)
p(2172) = p(45)*p(333) - p(1861)
p(2173) = p(1)*p(1415) - p(1956) - p(1954)
p(2174) = p(8)*p(865) - p(1861)
p(2175) = p(91)*p(187) - p(1921)
p(2176) = p(1)*p(1418) - p(1972) - p(1970)
p(2177) = p(2)*p(1402) - p(1921)
p(2178) = p(1)*p(1421) - p(1986) - p(1985)
p(2179) = p(1)*p(1422) - p(1987)
p(2180) = p(14)*p(533)
p(2181) = p(3)*p(946) - p(1992)
p(2182) = p(39)*p(340) - p(1994)
p(2183) = p(1)*p(1425) - p(1993) - p(2182)
p(2184) = p(2)*p(1406) - p(1944)
p(2185) = p(1)*p(1426) - p(1995) - p(2183)
p(2186) = p(7)*p(904) - p(1994)
p(2187) = p(1)*p(1428) - p(1997) - p(2185)
p(2188) = p(49)*p(323) - p(1992)
p(2189) = p(6)*p(944) - p(1949)
p(2190) = p(1)*p(1431) - p(2003) - p(2001)
p(2191) = p(50)*p(285) - p(1949)
p(2192) = p(45)*p(335) - p(1912)
p(2193) = p(1)*p(1434) - p(2016) - p(2014)
p(2194) = p(22)*p(518) - p(1912)
p(2195) = p(93)*p(187) - p(1923)
p(2196) = p(1)*p(1437) - p(2029) - p(2027)
p(2197) = p(8)*p(881) - p(1923)
p(2198) = p(14)*p(545)
p(2199) = p(16)*p(578) - p(1989)
p(2200) = p(1)*p(1441) - p(2040) - p(2038)
p(2201) = p(2)*p(1422) - p(1989)
p(2202) = p(14)*p(549)
p(2203) = p(6)*p(899) - p(2042) - p(2041)
p(2204) = p(1)*p(1491) - p(2203)
p(2205) = p(14)*p(578)
p(2206) = p(3)*p(949) - p(2044)
p(2207) = p(39)*p(343) - p(2046)
p(2208) = p(1)*p(1445) - p(2045) - p(2207)
p(2209) = p(1)*p(1446) - p(2046)
p(2210) = p(1)*p(1447) - p(2047) - p(2208)
p(2211) = p(21)*p(561) - p(2044)
p(2212) = p(6)*p(946) - p(2010)
p(2213) = p(1)*p(1450) - p(2052) - p(2050)
p(2214) = p(102)*p(150) - p(2010)
p(2215) = p(45)*p(337) - p(1966)
p(2216) = p(1)*p(1453) - p(2062) - p(2060)
p(2217) = p(50)*p(296) - p(1966)
p(2218) = p(95)*p(187) - p(1924)
p(2219) = p(1)*p(1457) - p(2072) - p(2070)
p(2220) = p(22)*p(532) - p(1924)
p(2221) = p(14)*p(559)
p(2222) = p(18)*p(578) - p(1991)
p(2223) = p(2)*p(1441) - p(2042) - p(2203)
p(2224) = p(8)*p(900) - p(1991)
p(2225) = p(14)*p(581)
p(2226) = p(3)*p(952) - p(2076)
p(2227) = p(2)*p(1445) - p(2053) - p(2049)
p(2228) = p(2)*p(1446) - p(2054)
p(2229) = p(1)*p(1461) - p(2077) - p(2227)
p(2230) = p(7)*p(929) - p(2076)
p(2231) = p(6)*p(949) - p(2057)
p(2232) = p(1)*p(1464) - p(2081) - p(2079)
p(2233) = p(73)*p(195) - p(2057)
p(2234) = p(45)*p(340) - p(2024)
p(2235) = p(1)*p(1467) - p(2088) - p(2086)
p(2236) = p(102)*p(160) - p(2024)
p(2237) = p(14)*p(565)
p(2238) = p(98)*p(187) - p(1983)
p(2239) = p(2)*p(1457) - p(2074) - p(2223)
p(2240) = p(50)*p(310) - p(1983)
p(2241) = p(14)*p(584)
p(2242) = p(3)*p(955) - p(2092)
p(2243) = p(2)*p(1461) - p(2082) - p(2079)
p(2244) = p(1)*p(1471) - p(2092)
p(2245) = p(6)*p(952) - p(2084)
p(2246) = p(1)*p(1473) - p(2095) - p(2093)
p(2247) = p(32)*p(347) - p(2084)
p(2248) = p(45)*p(343) - p(2068)
p(2249) = p(2)*p(1467) - p(2090) - p(2239)
p(2250) = p(81)*p(195) - p(2068)
p(2251) = p(14)*p(587)
p(2252) = p(2)*p(1470) - p(2094) - p(2093)
p(2253) = p(2)*p(1471) - p(2095)
p(2254) = p(6)*p(955) - p(2096)
p(2255) = p(2)*p(1473) - p(2097) - p(2249)
p(2256) = p(13)*p(589) - p(2096)
p(2257) = p(14)*p(589)
p(2258) = p(1)*p(1476) - p(2101)
p(2259) = p(21)*p(566) - p(2098)
p(2260) = p(101)*p(178) - p(2099)
p(2261) = p(39)*p(346) - p(2104)
p(2262) = p(3)*p(958) - p(2107)
p(2263) = p(1)*p(1481) - p(2108)
p(2264) = p(2)*p(1504) - p(2263)
p(2265) = p(1)*p(1483) - p(2124)
p(2266) = p(8)*p(958) - p(2265)
p(2267) = p(1)*p(1485) - p(2141)
p(2268) = p(22)*p(588) - p(2267)
p(2269) = p(1)*p(1487) - p(2160)
p(2270) = p(50)*p(346) - p(2269)
p(2271) = p(1)*p(1489) - p(2181)
p(2272) = p(102)*p(194) - p(2271)
p(2273) = p(1)*p(1492) - p(2206)
p(2274) = p(101)*p(195) - p(2273)
p(2275) = p(2)*p(1491) - p(2205)
p(2276) = p(1)*p(1495) - p(2226)
p(2277) = p(49)*p(347) - p(2276)
p(2278) = p(22)*p(578) - p(2180)
p(2279) = p(1)*p(1498) - p(2242)
p(2280) = p(21)*p(589) - p(2279)
p(2281) = p(102)*p(187) - p(2198)
p(2282) = p(1)*p(1501) - p(2252)
p(2283) = p(7)*p(959) - p(2282)
p(2284) = p(45)*p(347) - p(2237)
p(2285) = p(2)*p(1501) - p(2254)
p(2286) = p(1)*p(1505) - p(2285)
p(2287) = p(6)*p(959) - p(2257)
p(2288) = p(1)*p(1504) - p(2262)
p(2289) = p(2)*p(1505) - p(2287)
p(2290) = p(9)*p(590)
p(2291) = p(9)*p(591)
p(2292) = p(9)*p(613)
p(2293) = p(9)*p(592)
p(2294) = p(9)*p(614)
p(2295) = p(9)*p(615)
p(2296) = p(9)*p(701)
p(2297) = p(9)*p(726)
p(2298) = p(9)*p(626)
p(2299) = p(9)*p(593)
p(2300) = p(9)*p(627)
p(2301) = p(9)*p(594)
p(2302) = p(9)*p(595)
p(2303) = p(9)*p(628)
p(2304) = p(9)*p(596)
p(2305) = p(9)*p(629)
p(2306) = p(9)*p(630)
p(2307) = p(9)*p(631)
p(2308) = p(9)*p(727)
p(2309) = p(9)*p(636)
p(2310) = p(9)*p(637)
p(2311) = p(9)*p(734)
p(2312) = p(9)*p(597)
p(2313) = p(9)*p(638)
p(2314) = p(9)*p(598)
p(2315) = p(9)*p(639)
p(2316) = p(9)*p(640)
p(2317) = p(9)*p(735)
p(2318) = p(9)*p(599)
p(2319) = p(9)*p(641)
p(2320) = p(9)*p(736)
p(2321) = p(9)*p(755)
p(2322) = p(9)*p(648)
p(2323) = p(9)*p(600)
p(2324) = p(9)*p(649)
p(2325) = p(9)*p(601)
p(2326) = p(9)*p(650)
p(2327) = p(9)*p(651)
p(2328) = p(9)*p(652)
p(2329) = p(9)*p(756)
p(2330) = p(9)*p(766)
p(2331) = p(9)*p(656)
p(2332) = p(9)*p(657)
p(2333) = p(9)*p(767)
p(2334) = p(9)*p(658)
p(2335) = p(9)*p(602)
p(2336) = p(9)*p(659)
p(2337) = p(9)*p(660)
p(2338) = p(9)*p(661)
p(2339) = p(9)*p(768)
p(2340) = p(9)*p(769)
p(2341) = p(9)*p(662)
p(2342) = p(9)*p(663)
p(2343) = p(9)*p(770)
p(2344) = p(9)*p(665)
p(2345) = p(9)*p(774)
p(2346) = p(9)*p(666)
p(2347) = p(9)*p(775)
p(2348) = p(9)*p(776)
p(2349) = p(9)*p(777)
p(2350) = p(9)*p(882)
p(2351) = p(9)*p(791)
p(2352) = p(9)*p(670)
p(2353) = p(9)*p(603)
p(2354) = p(9)*p(671)
p(2355) = p(9)*p(672)
p(2356) = p(9)*p(792)
p(2357) = p(9)*p(799)
p(2358) = p(9)*p(674)
p(2359) = p(9)*p(800)
p(2360) = p(9)*p(675)
p(2361) = p(9)*p(676)
p(2362) = p(9)*p(801)
p(2363) = p(9)*p(802)
p(2364) = p(9)*p(803)
p(2365) = p(9)*p(807)
p(2366) = p(9)*p(808)
p(2367) = p(9)*p(809)
p(2368) = p(9)*p(898)
p(2369) = p(9)*p(901)
p(2370) = p(9)*p(818)
p(2371) = p(9)*p(678)
p(2372) = p(9)*p(679)
p(2373) = p(9)*p(819)
p(2374) = p(9)*p(823)
p(2375) = p(9)*p(824)
p(2376) = p(9)*p(825)
p(2377) = p(9)*p(913)
p(2378) = p(9)*p(917)
p(2379) = p(9)*p(830)
p(2380) = p(9)*p(831)
p(2381) = p(9)*p(927)
p(2382) = p(9)*p(933)
p(2383) = p(9)*p(692)
p(2384) = p(9)*p(693)
p(2385) = p(9)*p(694)
p(2386) = p(9)*p(850)
p(2387) = p(9)*p(604)
p(2388) = p(9)*p(605)
p(2389) = p(9)*p(606)
p(2390) = p(9)*p(695)
p(2391) = p(9)*p(607)
p(2392) = p(9)*p(608)
p(2393) = p(9)*p(696)
p(2394) = p(9)*p(609)
p(2395) = p(9)*p(610)
p(2396) = p(9)*p(697)
p(2397) = p(9)*p(611)
p(2398) = p(9)*p(698)
p(2399) = p(9)*p(612)
p(2400) = p(9)*p(699)
p(2401) = p(9)*p(700)
p(2402) = p(9)*p(851)
p(2403) = p(14)*p(934)
p(2404) = p(14)*p(935)
p(2405) = p(14)*p(936)
p(2406) = p(14)*p(937)
p(2407) = p(9)*p(861)
p(2408) = p(9)*p(711)
p(2409) = p(9)*p(712)
p(2410) = p(9)*p(713)
p(2411) = p(9)*p(714)
p(2412) = p(9)*p(715)
p(2413) = p(9)*p(862)
p(2414) = p(9)*p(716)
p(2415) = p(9)*p(616)
p(2416) = p(9)*p(617)
p(2417) = p(9)*p(618)
p(2418) = p(9)*p(619)
p(2419) = p(9)*p(717)
p(2420) = p(9)*p(718)
p(2421) = p(9)*p(620)
p(2422) = p(9)*p(621)
p(2423) = p(9)*p(622)
p(2424) = p(9)*p(719)
p(2425) = p(9)*p(720)
p(2426) = p(9)*p(623)
p(2427) = p(9)*p(624)
p(2428) = p(9)*p(721)
p(2429) = p(9)*p(722)
p(2430) = p(9)*p(625)
p(2431) = p(9)*p(723)
p(2432) = p(9)*p(724)
p(2433) = p(9)*p(725)
p(2434) = p(9)*p(863)
p(2435) = p(14)*p(839)
p(2436) = p(14)*p(840)
p(2437) = p(14)*p(841)
p(2438) = p(14)*p(842)
p(2439) = p(14)*p(843)
p(2440) = p(14)*p(844)
p(2441) = p(14)*p(845)
p(2442) = p(14)*p(846)
p(2443) = p(14)*p(847)
p(2444) = p(14)*p(848)
p(2445) = p(14)*p(849)
p(2446) = p(9)*p(728)
p(2447) = p(9)*p(729)
p(2448) = p(9)*p(864)
p(2449) = p(9)*p(632)
p(2450) = p(9)*p(633)
p(2451) = p(9)*p(730)
p(2452) = p(9)*p(634)
p(2453) = p(9)*p(731)
p(2454) = p(9)*p(635)
p(2455) = p(9)*p(732)
p(2456) = p(9)*p(733)
p(2457) = p(9)*p(865)
p(2458) = p(14)*p(692)
p(2459) = p(14)*p(693)
p(2460) = p(14)*p(694)
p(2461) = p(14)*p(695)
p(2462) = p(14)*p(696)
p(2463) = p(14)*p(697)
p(2464) = p(14)*p(698)
p(2465) = p(14)*p(699)
p(2466) = p(14)*p(700)
p(2467) = p(14)*p(613)
p(2468) = p(14)*p(614)
p(2469) = p(14)*p(615)
p(2470) = p(9)*p(874)
p(2471) = p(9)*p(743)
p(2472) = p(9)*p(744)
p(2473) = p(9)*p(745)
p(2474) = p(9)*p(746)
p(2475) = p(9)*p(875)
p(2476) = p(9)*p(747)
p(2477) = p(9)*p(642)
p(2478) = p(9)*p(643)
p(2479) = p(9)*p(644)
p(2480) = p(9)*p(748)
p(2481) = p(9)*p(749)
p(2482) = p(9)*p(645)
p(2483) = p(9)*p(646)
p(2484) = p(9)*p(750)
p(2485) = p(9)*p(751)
p(2486) = p(9)*p(647)
p(2487) = p(9)*p(752)
p(2488) = p(9)*p(753)
p(2489) = p(9)*p(754)
p(2490) = p(9)*p(876)
p(2491) = p(14)*p(852)
p(2492) = p(14)*p(853)
p(2493) = p(14)*p(854)
p(2494) = p(14)*p(855)
p(2495) = p(14)*p(856)
p(2496) = p(14)*p(857)
p(2497) = p(14)*p(858)
p(2498) = p(14)*p(859)
p(2499) = p(14)*p(860)
p(2500) = p(9)*p(877)
p(2501) = p(9)*p(757)
p(2502) = p(9)*p(758)
p(2503) = p(9)*p(759)
p(2504) = p(9)*p(878)
p(2505) = p(9)*p(760)
p(2506) = p(9)*p(653)
p(2507) = p(9)*p(654)
p(2508) = p(9)*p(761)
p(2509) = p(9)*p(762)
p(2510) = p(9)*p(655)
p(2511) = p(9)*p(763)
p(2512) = p(9)*p(764)
p(2513) = p(9)*p(765)
p(2514) = p(9)*p(879)
p(2515) = p(14)*p(711)
p(2516) = p(14)*p(712)
p(2517) = p(14)*p(713)
p(2518) = p(14)*p(714)
p(2519) = p(14)*p(715)
p(2520) = p(14)*p(716)
p(2521) = p(14)*p(717)
p(2522) = p(14)*p(718)
p(2523) = p(14)*p(719)
p(2524) = p(14)*p(720)
p(2525) = p(14)*p(721)
p(2526) = p(14)*p(722)
p(2527) = p(14)*p(723)
p(2528) = p(14)*p(724)
p(2529) = p(14)*p(725)
p(2530) = p(14)*p(626)
p(2531) = p(14)*p(627)
p(2532) = p(14)*p(628)
p(2533) = p(14)*p(629)
p(2534) = p(14)*p(630)
p(2535) = p(14)*p(631)
p(2536) = p(9)*p(771)
p(2537) = p(9)*p(880)
p(2538) = p(9)*p(664)
p(2539) = p(9)*p(772)
p(2540) = p(9)*p(773)
p(2541) = p(9)*p(881)
p(2542) = p(14)*p(728)
p(2543) = p(14)*p(729)
p(2544) = p(14)*p(730)
p(2545) = p(14)*p(731)
p(2546) = p(14)*p(732)
p(2547) = p(14)*p(733)
p(2548) = p(14)*p(636)
p(2549) = p(14)*p(637)
p(2550) = p(14)*p(638)
p(2551) = p(14)*p(639)
p(2552) = p(14)*p(640)
p(2553) = p(14)*p(641)
p(2554) = p(9)*p(889)
p(2555) = p(9)*p(782)
p(2556) = p(9)*p(783)
p(2557) = p(9)*p(784)
p(2558) = p(9)*p(890)
p(2559) = p(9)*p(785)
p(2560) = p(9)*p(667)
p(2561) = p(9)*p(668)
p(2562) = p(9)*p(786)
p(2563) = p(9)*p(787)
p(2564) = p(9)*p(669)
p(2565) = p(9)*p(788)
p(2566) = p(9)*p(789)
p(2567) = p(9)*p(790)
p(2568) = p(9)*p(891)
p(2569) = p(14)*p(866)
p(2570) = p(14)*p(867)
p(2571) = p(14)*p(868)
p(2572) = p(14)*p(869)
p(2573) = p(14)*p(870)
p(2574) = p(14)*p(871)
p(2575) = p(14)*p(872)
p(2576) = p(14)*p(873)
p(2577) = p(9)*p(892)
p(2578) = p(9)*p(793)
p(2579) = p(9)*p(794)
p(2580) = p(9)*p(893)
p(2581) = p(9)*p(795)
p(2582) = p(9)*p(673)
p(2583) = p(9)*p(796)
p(2584) = p(9)*p(797)
p(2585) = p(9)*p(798)
p(2586) = p(9)*p(894)
p(2587) = p(14)*p(743)
p(2588) = p(14)*p(744)
p(2589) = p(14)*p(745)
p(2590) = p(14)*p(746)
p(2591) = p(14)*p(747)
p(2592) = p(14)*p(748)
p(2593) = p(14)*p(749)
p(2594) = p(14)*p(750)
p(2595) = p(14)*p(751)
p(2596) = p(14)*p(752)
p(2597) = p(14)*p(753)
p(2598) = p(14)*p(754)
p(2599) = p(14)*p(648)
p(2600) = p(14)*p(649)
p(2601) = p(14)*p(650)
p(2602) = p(14)*p(651)
p(2603) = p(14)*p(652)
p(2604) = p(9)*p(895)
p(2605) = p(9)*p(804)
p(2606) = p(9)*p(896)
p(2607) = p(9)*p(805)
p(2608) = p(9)*p(806)
p(2609) = p(9)*p(897)
p(2610) = p(14)*p(757)
p(2611) = p(14)*p(758)
p(2612) = p(14)*p(759)
p(2613) = p(14)*p(760)
p(2614) = p(14)*p(761)
p(2615) = p(14)*p(762)
p(2616) = p(14)*p(763)
p(2617) = p(14)*p(764)
p(2618) = p(14)*p(765)
p(2619) = p(14)*p(656)
p(2620) = p(14)*p(657)
p(2621) = p(14)*p(658)
p(2622) = p(14)*p(659)
p(2623) = p(14)*p(660)
p(2624) = p(14)*p(661)
p(2625) = p(14)*p(662)
p(2626) = p(14)*p(663)
p(2627) = p(9)*p(899)
p(2628) = p(9)*p(900)
p(2629) = p(14)*p(771)
p(2630) = p(14)*p(772)
p(2631) = p(14)*p(773)
p(2632) = p(14)*p(665)
p(2633) = p(14)*p(666)
p(2634) = p(9)*p(907)
p(2635) = p(9)*p(812)
p(2636) = p(9)*p(813)
p(2637) = p(9)*p(908)
p(2638) = p(9)*p(814)
p(2639) = p(9)*p(677)
p(2640) = p(9)*p(815)
p(2641) = p(9)*p(816)
p(2642) = p(9)*p(817)
p(2643) = p(9)*p(909)
p(2644) = p(14)*p(883)
p(2645) = p(14)*p(884)
p(2646) = p(14)*p(885)
p(2647) = p(14)*p(886)
p(2648) = p(14)*p(887)
p(2649) = p(14)*p(888)
p(2650) = p(9)*p(910)
p(2651) = p(9)*p(820)
p(2652) = p(9)*p(911)
p(2653) = p(9)*p(821)
p(2654) = p(9)*p(822)
p(2655) = p(9)*p(912)
p(2656) = p(14)*p(782)
p(2657) = p(14)*p(783)
p(2658) = p(14)*p(784)
p(2659) = p(14)*p(785)
p(2660) = p(14)*p(786)
p(2661) = p(14)*p(787)
p(2662) = p(14)*p(788)
p(2663) = p(14)*p(789)
p(2664) = p(14)*p(790)
p(2665) = p(14)*p(670)
p(2666) = p(14)*p(671)
p(2667) = p(14)*p(672)
p(2668) = p(9)*p(914)
p(2669) = p(9)*p(915)
p(2670) = p(9)*p(916)
p(2671) = p(14)*p(793)
p(2672) = p(14)*p(794)
p(2673) = p(14)*p(795)
p(2674) = p(14)*p(796)
p(2675) = p(14)*p(797)
p(2676) = p(14)*p(798)
p(2677) = p(14)*p(674)
p(2678) = p(14)*p(675)
p(2679) = p(14)*p(676)
p(2680) = p(9)*p(948)
p(2681) = p(14)*p(804)
p(2682) = p(14)*p(805)
p(2683) = p(14)*p(806)
p(2684) = p(9)*p(921)
p(2685) = p(9)*p(827)
p(2686) = p(9)*p(922)
p(2687) = p(9)*p(828)
p(2688) = p(9)*p(829)
p(2689) = p(9)*p(923)
p(2690) = p(14)*p(902)
p(2691) = p(14)*p(903)
p(2692) = p(14)*p(904)
p(2693) = p(14)*p(905)
p(2694) = p(14)*p(906)
p(2695) = p(9)*p(924)
p(2696) = p(9)*p(925)
p(2697) = p(9)*p(926)
p(2698) = p(14)*p(812)
p(2699) = p(14)*p(813)
p(2700) = p(14)*p(814)
p(2701) = p(14)*p(815)
p(2702) = p(14)*p(816)
p(2703) = p(14)*p(817)
p(2704) = p(14)*p(678)
p(2705) = p(14)*p(679)
p(2706) = p(9)*p(951)
p(2707) = p(14)*p(820)
p(2708) = p(14)*p(821)
p(2709) = p(14)*p(822)
p(2710) = p(9)*p(930)
p(2711) = p(9)*p(931)
p(2712) = p(9)*p(932)
p(2713) = p(14)*p(918)
p(2714) = p(14)*p(919)
p(2715) = p(14)*p(920)
p(2716) = p(9)*p(954)
p(2717) = p(14)*p(827)
p(2718) = p(14)*p(828)
p(2719) = p(14)*p(829)
p(2720) = p(9)*p(957)
p(2721) = p(14)*p(928)
p(2722) = p(14)*p(929)
p(2723) = p(9)*p(938)
p(2724) = p(9)*p(839)
p(2725) = p(9)*p(680)
p(2726) = p(9)*p(840)
p(2727) = p(9)*p(681)
p(2728) = p(9)*p(682)
p(2729) = p(9)*p(841)
p(2730) = p(9)*p(683)
p(2731) = p(9)*p(684)
p(2732) = p(9)*p(685)
p(2733) = p(9)*p(842)
p(2734) = p(9)*p(686)
p(2735) = p(9)*p(687)
p(2736) = p(9)*p(688)
p(2737) = p(9)*p(843)
p(2738) = p(9)*p(844)
p(2739) = p(9)*p(689)
p(2740) = p(9)*p(690)
p(2741) = p(9)*p(845)
p(2742) = p(9)*p(846)
p(2743) = p(9)*p(691)
p(2744) = p(9)*p(847)
p(2745) = p(9)*p(848)
p(2746) = p(9)*p(849)
p(2747) = p(9)*p(939)
p(2748) = p(3)*p(1136) - p(2383)
p(2749) = p(7)*p(1136) - p(2387)
p(2750) = p(49)*p(401) - p(2394)
p(2751) = p(114)*p(194) - p(2399)
p(2752) = p(24)*p(588) - p(2402)
p(2753) = p(3)*p(1368) - p(2386) - p(2752)
p(2754) = p(1)*p(1802) - p(2386) - p(2753)
p(2755) = p(1)*p(1803) - p(2390) - p(2754)
p(2756) = p(1)*p(1804) - p(2393) - p(2755)
p(2757) = p(6)*p(1476) - p(2748)
p(2758) = p(32)*p(566) - p(2387)
p(2759) = p(150)*p(178) - p(2394)
p(2760) = p(39)*p(505) - p(2399)
p(2761) = p(3)*p(1369) - p(2402)
p(2762) = p(14)*p(958)
p(2763) = p(9)*p(940)
p(2764) = p(9)*p(852)
p(2765) = p(9)*p(702)
p(2766) = p(9)*p(853)
p(2767) = p(9)*p(703)
p(2768) = p(9)*p(704)
p(2769) = p(9)*p(854)
p(2770) = p(9)*p(705)
p(2771) = p(9)*p(706)
p(2772) = p(9)*p(707)
p(2773) = p(9)*p(855)
p(2774) = p(9)*p(708)
p(2775) = p(9)*p(709)
p(2776) = p(9)*p(856)
p(2777) = p(9)*p(857)
p(2778) = p(9)*p(710)
p(2779) = p(9)*p(858)
p(2780) = p(9)*p(859)
p(2781) = p(9)*p(860)
p(2782) = p(9)*p(941)
p(2783) = p(24)*p(570) - p(2296)
p(2784) = p(114)*p(181) - p(2293)
p(2785) = p(1)*p(1828) - p(2408) - p(2784)
p(2786) = p(4)*p(1136) - p(2292)
p(2787) = p(1)*p(1830) - p(2410) - p(2786)
p(2788) = p(1)*p(1831) - p(2411) - p(2787)
p(2789) = p(1)*p(1832) - p(2412) - p(2788)
p(2790) = p(1)*p(1833) - p(2413) - p(2789)
p(2791) = p(13)*p(938) - p(2762)
p(2792) = p(2)*p(1802) - p(2406) - p(2791)
p(2793) = p(32)*p(570) - p(2762)
p(2794) = p(1)*p(1835) - p(2413) - p(2792)
p(2795) = p(73)*p(331) - p(2762)
p(2796) = p(1)*p(1837) - p(2419) - p(2794)
p(2797) = p(150)*p(181) - p(2762)
p(2798) = p(1)*p(1839) - p(2424) - p(2796)
p(2799) = p(91)*p(285) - p(2762)
p(2800) = p(13)*p(843) - p(2292)
p(2801) = p(41)*p(505) - p(2762)
p(2802) = p(1)*p(1843) - p(2431) - p(2800)
p(2803) = p(16)*p(851) - p(2762)
p(2804) = p(69)*p(285) - p(2293)
p(2805) = p(4)*p(1369) - p(2762)
p(2806) = p(11)*p(851) - p(2296)
p(2807) = p(14)*p(938)
p(2808) = p(14)*p(939)
p(2809) = p(3)*p(1180) - p(2446)
p(2810) = p(7)*p(1180) - p(2449)
p(2811) = p(49)*p(430) - p(2454)
p(2812) = p(130)*p(194) - p(2457)
p(2813) = p(3)*p(1384) - p(2448) - p(2812)
p(2814) = p(1)*p(1853) - p(2448) - p(2813)
p(2815) = p(1)*p(1854) - p(2451) - p(2814)
p(2816) = p(45)*p(566) - p(2809)
p(2817) = p(160)*p(178) - p(2449)
p(2818) = p(39)*p(518) - p(2454)
p(2819) = p(3)*p(1385) - p(2457)
p(2820) = p(14)*p(850)
p(2821) = p(14)*p(851)
p(2822) = p(14)*p(701)
p(2823) = p(9)*p(942)
p(2824) = p(9)*p(866)
p(2825) = p(9)*p(737)
p(2826) = p(9)*p(867)
p(2827) = p(9)*p(738)
p(2828) = p(9)*p(739)
p(2829) = p(9)*p(868)
p(2830) = p(9)*p(740)
p(2831) = p(9)*p(741)
p(2832) = p(9)*p(869)
p(2833) = p(9)*p(870)
p(2834) = p(9)*p(742)
p(2835) = p(9)*p(871)
p(2836) = p(9)*p(872)
p(2837) = p(9)*p(873)
p(2838) = p(9)*p(943)
p(2839) = p(24)*p(572) - p(2297)
p(2840) = p(114)*p(183) - p(2299)
p(2841) = p(43)*p(401) - p(2302)
p(2842) = p(8)*p(1136) - p(2439)
p(2843) = p(1)*p(1877) - p(2473) - p(2842)
p(2844) = p(1)*p(1878) - p(2474) - p(2843)
p(2845) = p(1)*p(1879) - p(2475) - p(2844)
p(2846) = p(13)*p(940) - p(2807)
p(2847) = p(2)*p(1835) - p(2445) - p(2813)
p(2848) = p(32)*p(572) - p(2807)
p(2849) = p(1)*p(1881) - p(2475) - p(2847)
p(2850) = p(73)*p(333) - p(2807)
p(2851) = p(1)*p(1883) - p(2480) - p(2849)
p(2852) = p(150)*p(183) - p(2807)
p(2853) = p(6)*p(1374) - p(2439)
p(2854) = p(93)*p(285) - p(2807)
p(2855) = p(32)*p(509) - p(2302)
p(2856) = p(43)*p(505) - p(2807)
p(2857) = p(150)*p(153) - p(2299)
p(2858) = p(18)*p(851) - p(2807)
p(2859) = p(34)*p(505) - p(2297)
p(2860) = p(14)*p(940)
p(2861) = p(14)*p(941)
p(2862) = p(130)*p(181) - p(2317)
p(2863) = p(41)*p(430) - p(2314)
p(2864) = p(4)*p(1180) - p(2309)
p(2865) = p(1)*p(1896) - p(2502) - p(2864)
p(2866) = p(1)*p(1897) - p(2503) - p(2865)
p(2867) = p(1)*p(1898) - p(2504) - p(2866)
p(2868) = p(81)*p(331) - p(2821)
p(2869) = p(2)*p(1853) - p(2461) - p(2868)
p(2870) = p(160)*p(181) - p(2821)
p(2871) = p(1)*p(1900) - p(2504) - p(2869)
p(2872) = p(91)*p(296) - p(2821)
p(2873) = p(1)*p(1902) - p(2508) - p(2871)
p(2874) = p(41)*p(518) - p(2821)
p(2875) = p(81)*p(279) - p(2309)
p(2876) = p(16)*p(865) - p(2821)
p(2877) = p(69)*p(296) - p(2314)
p(2878) = p(4)*p(1385) - p(2821)
p(2879) = p(11)*p(865) - p(2317)
p(2880) = p(14)*p(861)
p(2881) = p(14)*p(862)
p(2882) = p(14)*p(863)
p(2883) = p(14)*p(726)
p(2884) = p(14)*p(727)
p(2885) = p(3)*p(1234) - p(2536)
p(2886) = p(7)*p(1234) - p(2538)
p(2887) = p(49)*p(464) - p(2541)
p(2888) = p(3)*p(1401) - p(2537) - p(2887)
p(2889) = p(1)*p(1916) - p(2537) - p(2888)
p(2890) = p(178)*p(187) - p(2885)
p(2891) = p(39)*p(532) - p(2538)
p(2892) = p(3)*p(1402) - p(2541)
p(2893) = p(14)*p(864)
p(2894) = p(14)*p(865)
p(2895) = p(14)*p(734)
p(2896) = p(14)*p(735)
p(2897) = p(14)*p(736)
p(2898) = p(9)*p(944)
p(2899) = p(9)*p(883)
p(2900) = p(9)*p(778)
p(2901) = p(9)*p(884)
p(2902) = p(9)*p(779)
p(2903) = p(9)*p(780)
p(2904) = p(9)*p(885)
p(2905) = p(9)*p(781)
p(2906) = p(9)*p(886)
p(2907) = p(9)*p(887)
p(2908) = p(9)*p(888)
p(2909) = p(9)*p(945)
p(2910) = p(24)*p(574) - p(2321)
p(2911) = p(114)*p(185) - p(2323)
p(2912) = p(46)*p(401) - p(2326)
p(2913) = p(1)*p(1936) - p(2556) - p(2912)
p(2914) = p(1)*p(1937) - p(2557) - p(2913)
p(2915) = p(1)*p(1938) - p(2558) - p(2914)
p(2916) = p(13)*p(942) - p(2860)
p(2917) = p(2)*p(1881) - p(2499) - p(2869)
p(2918) = p(32)*p(574) - p(2860)
p(2919) = p(1)*p(1940) - p(2558) - p(2917)
p(2920) = p(73)*p(335) - p(2860)
p(2921) = p(1)*p(1942) - p(2562) - p(2919)
p(2922) = p(150)*p(185) - p(2860)
p(2923) = p(13)*p(869) - p(2326)
p(2924) = p(95)*p(285) - p(2860)
p(2925) = p(73)*p(299) - p(2323)
p(2926) = p(46)*p(505) - p(2860)
p(2927) = p(84)*p(285) - p(2321)
p(2928) = p(14)*p(942)
p(2929) = p(14)*p(943)
p(2930) = p(130)*p(183) - p(2320)
p(2931) = p(1)*p(1951) - p(2577) - p(2930)
p(2932) = p(8)*p(1180) - p(2467)
p(2933) = p(1)*p(1953) - p(2579) - p(2932)
p(2934) = p(1)*p(1954) - p(2580) - p(2933)
p(2935) = p(81)*p(333) - p(2822)
p(2936) = p(2)*p(1900) - p(2521) - p(2888)
p(2937) = p(160)*p(183) - p(2822)
p(2938) = p(1)*p(1956) - p(2580) - p(2936)
p(2939) = p(93)*p(296) - p(2822)
p(2940) = p(45)*p(509) - p(2467)
p(2941) = p(43)*p(518) - p(2822)
p(2942) = p(1)*p(1960) - p(2585) - p(2940)
p(2943) = p(18)*p(865) - p(2822)
p(2944) = p(34)*p(518) - p(2320)
p(2945) = p(14)*p(874)
p(2946) = p(14)*p(875)
p(2947) = p(14)*p(876)
p(2948) = p(14)*p(755)
p(2949) = p(14)*p(756)
p(2950) = p(41)*p(464) - p(2347)
p(2951) = p(4)*p(1234) - p(2344)
p(2952) = p(1)*p(1969) - p(2605) - p(2951)
p(2953) = p(1)*p(1970) - p(2606) - p(2952)
p(2954) = p(91)*p(310) - p(2894)
p(2955) = p(2)*p(1916) - p(2544) - p(2954)
p(2956) = p(41)*p(532) - p(2894)
p(2957) = p(1)*p(1972) - p(2606) - p(2955)
p(2958) = p(16)*p(881) - p(2894)
p(2959) = p(69)*p(310) - p(2344)
p(2960) = p(4)*p(1402) - p(2894)
p(2961) = p(11)*p(881) - p(2347)
p(2962) = p(14)*p(877)
p(2963) = p(14)*p(878)
p(2964) = p(14)*p(879)
p(2965) = p(14)*p(766)
p(2966) = p(14)*p(767)
p(2967) = p(14)*p(768)
p(2968) = p(14)*p(769)
p(2969) = p(14)*p(770)
p(2970) = p(3)*p(1290) - p(2627)
p(2971) = p(7)*p(1290) - p(2628)
p(2972) = p(3)*p(1421) - p(2627) - p(2971)
p(2973) = p(39)*p(578) - p(2970)
p(2974) = p(3)*p(1422) - p(2628)
p(2975) = p(14)*p(880)
p(2976) = p(14)*p(881)
p(2977) = p(14)*p(774)
p(2978) = p(14)*p(775)
p(2979) = p(14)*p(776)
p(2980) = p(14)*p(777)
p(2981) = p(9)*p(946)
p(2982) = p(9)*p(902)
p(2983) = p(9)*p(810)
p(2984) = p(9)*p(903)
p(2985) = p(9)*p(811)
p(2986) = p(9)*p(904)
p(2987) = p(9)*p(905)
p(2988) = p(9)*p(906)
p(2989) = p(9)*p(947)
p(2990) = p(24)*p(576) - p(2351)
p(2991) = p(114)*p(188) - p(2353)
p(2992) = p(50)*p(401) - p(2572)
p(2993) = p(1)*p(2000) - p(2636) - p(2992)
p(2994) = p(1)*p(2001) - p(2637) - p(2993)
p(2995) = p(13)*p(944) - p(2928)
p(2996) = p(2)*p(1940) - p(2576) - p(2936)
p(2997) = p(32)*p(576) - p(2928)
p(2998) = p(1)*p(2003) - p(2637) - p(2996)
p(2999) = p(73)*p(337) - p(2928)
p(3000) = p(6)*p(1406) - p(2572)
p(3001) = p(150)*p(188) - p(2928)
p(3002) = p(32)*p(536) - p(2353)
p(3003) = p(98)*p(285) - p(2928)
p(3004) = p(150)*p(173) - p(2351)
p(3005) = p(14)*p(944)
p(3006) = p(14)*p(945)
p(3007) = p(130)*p(185) - p(2340)
p(3008) = p(46)*p(430) - p(2342)
p(3009) = p(1)*p(2013) - p(2651) - p(3008)
p(3010) = p(1)*p(2014) - p(2652) - p(3009)
p(3011) = p(81)*p(335) - p(2883)
p(3012) = p(2)*p(1956) - p(2592) - p(2955)
p(3013) = p(160)*p(185) - p(2883)
p(3014) = p(1)*p(2016) - p(2652) - p(3012)
p(3015) = p(95)*p(296) - p(2883)
p(3016) = p(81)*p(299) - p(2342)
p(3017) = p(46)*p(518) - p(2883)
p(3018) = p(84)*p(296) - p(2340)
p(3019) = p(14)*p(889)
p(3020) = p(14)*p(890)
p(3021) = p(14)*p(891)
p(3022) = p(14)*p(791)
p(3023) = p(14)*p(792)
p(3024) = p(43)*p(464) - p(2349)
p(3025) = p(8)*p(1234) - p(2548)
p(3026) = p(1)*p(2027) - p(2669) - p(3025)
p(3027) = p(93)*p(310) - p(2896)
p(3028) = p(2)*p(1972) - p(2614) - p(2972)
p(3029) = p(43)*p(532) - p(2896)
p(3030) = p(153)*p(187) - p(2548)
p(3031) = p(18)*p(881) - p(2896)
p(3032) = p(34)*p(532) - p(2349)
p(3033) = p(14)*p(892)
p(3034) = p(14)*p(893)
p(3035) = p(14)*p(894)
p(3036) = p(14)*p(799)
p(3037) = p(14)*p(800)
p(3038) = p(14)*p(801)
p(3039) = p(14)*p(802)
p(3040) = p(14)*p(803)
p(3041) = p(4)*p(1290) - p(2369)
p(3042) = p(1)*p(2038) - p(2680) - p(3041)
p(3043) = p(16)*p(900) - p(2976)
p(3044) = p(2)*p(1986) - p(2630) - p(3043)
p(3045) = p(4)*p(1422) - p(2976)
p(3046) = p(11)*p(900) - p(2369)
p(3047) = p(14)*p(895)
p(3048) = p(14)*p(896)
p(3049) = p(14)*p(897)
p(3050) = p(14)*p(807)
p(3051) = p(14)*p(808)
p(3052) = p(14)*p(809)
p(3053) = p(6)*p(1290) - p(2681)
p(3054) = p(3)*p(1491) - p(3053)
p(3055) = p(14)*p(899)
p(3056) = p(14)*p(900)
p(3057) = p(9)*p(949)
p(3058) = p(9)*p(918)
p(3059) = p(9)*p(826)
p(3060) = p(9)*p(919)
p(3061) = p(9)*p(920)
p(3062) = p(9)*p(950)
p(3063) = p(24)*p(579) - p(2370)
p(3064) = p(114)*p(191) - p(2372)
p(3065) = p(1)*p(2049) - p(2685) - p(3064)
p(3066) = p(1)*p(2050) - p(2686) - p(3065)
p(3067) = p(13)*p(946) - p(3005)
p(3068) = p(2)*p(2003) - p(2649) - p(3012)
p(3069) = p(32)*p(579) - p(3005)
p(3070) = p(1)*p(2052) - p(2686) - p(3068)
p(3071) = p(73)*p(340) - p(3005)
p(3072) = p(13)*p(904) - p(2372)
p(3073) = p(150)*p(191) - p(3005)
p(3074) = p(73)*p(323) - p(2370)
p(3075) = p(14)*p(946)
p(3076) = p(14)*p(947)
p(3077) = p(130)*p(188) - p(2363)
p(3078) = p(50)*p(430) - p(2601)
p(3079) = p(1)*p(2060) - p(2696) - p(3078)
p(3080) = p(81)*p(337) - p(2948)
p(3081) = p(2)*p(2016) - p(2660) - p(3028)
p(3082) = p(160)*p(188) - p(2948)
p(3083) = p(45)*p(536) - p(2601)
p(3084) = p(98)*p(296) - p(2948)
p(3085) = p(160)*p(173) - p(2363)
p(3086) = p(14)*p(907)
p(3087) = p(14)*p(908)
p(3088) = p(14)*p(909)
p(3089) = p(14)*p(818)
p(3090) = p(14)*p(819)
p(3091) = p(2)*p(2026) - p(2671) - p(3041)
p(3092) = p(1)*p(2070) - p(2706) - p(3091)
p(3093) = p(95)*p(310) - p(2897)
p(3094) = p(2)*p(2029) - p(2674) - p(3044)
p(3095) = p(46)*p(532) - p(2897)
p(3096) = p(1)*p(2072) - p(2706) - p(3094)
p(3097) = p(14)*p(910)
p(3098) = p(14)*p(911)
p(3099) = p(14)*p(912)
p(3100) = p(14)*p(823)
p(3101) = p(14)*p(824)
p(3102) = p(14)*p(825)
p(3103) = p(8)*p(1290) - p(2632)
p(3104) = p(18)*p(900) - p(2978)
p(3105) = p(34)*p(578) - p(2632)
p(3106) = p(14)*p(914)
p(3107) = p(14)*p(915)
p(3108) = p(14)*p(916)
p(3109) = p(9)*p(952)
p(3110) = p(9)*p(928)
p(3111) = p(9)*p(929)
p(3112) = p(9)*p(953)
p(3113) = p(24)*p(582) - p(2379)
p(3114) = p(114)*p(195) - p(2692)
p(3115) = p(1)*p(2079) - p(2711) - p(3114)
p(3116) = p(13)*p(949) - p(3075)
p(3117) = p(2)*p(2052) - p(2694) - p(3081)
p(3118) = p(32)*p(582) - p(3075)
p(3119) = p(6)*p(1446) - p(2692)
p(3120) = p(73)*p(343) - p(3075)
p(3121) = p(32)*p(561) - p(2379)
p(3122) = p(14)*p(949)
p(3123) = p(14)*p(950)
p(3124) = p(130)*p(191) - p(2377)
p(3125) = p(1)*p(2086) - p(2716) - p(3124)
p(3126) = p(81)*p(340) - p(3022)
p(3127) = p(2)*p(2062) - p(2701) - p(3094)
p(3128) = p(160)*p(191) - p(3022)
p(3129) = p(81)*p(323) - p(2377)
p(3130) = p(14)*p(921)
p(3131) = p(14)*p(922)
p(3132) = p(14)*p(923)
p(3133) = p(14)*p(830)
p(3134) = p(14)*p(831)
p(3135) = p(50)*p(464) - p(2626)
p(3136) = p(98)*p(310) - p(2968)
p(3137) = p(173)*p(187) - p(2626)
p(3138) = p(14)*p(924)
p(3139) = p(14)*p(925)
p(3140) = p(14)*p(926)
p(3141) = p(9)*p(955)
p(3142) = p(9)*p(956)
p(3143) = p(24)*p(585) - p(2382)
p(3144) = p(1)*p(2093) - p(2720) - p(3143)
p(3145) = p(13)*p(952) - p(3122)
p(3146) = p(2)*p(2081) - p(2715) - p(3127)
p(3147) = p(32)*p(585) - p(3122)
p(3148) = p(13)*p(929) - p(2382)
p(3149) = p(14)*p(952)
p(3150) = p(14)*p(953)
p(3151) = p(130)*p(195) - p(2705)
p(3152) = p(81)*p(343) - p(3089)
p(3153) = p(45)*p(561) - p(2705)
p(3154) = p(14)*p(930)
p(3155) = p(14)*p(931)
p(3156) = p(14)*p(932)
p(3157) = p(9)*p(959)
p(3158) = p(24)*p(589) - p(2722)
p(3159) = p(13)*p(955) - p(3149)
p(3160) = p(6)*p(1471) - p(2722)
p(3161) = p(14)*p(955)
p(3162) = p(14)*p(956)
p(3163) = p(9)*p(832)
p(3164) = p(9)*p(833)
p(3165) = p(9)*p(834)
p(3166) = p(9)*p(835)
p(3167) = p(9)*p(836)
p(3168) = p(9)*p(934)
p(3169) = p(9)*p(837)
p(3170) = p(9)*p(935)
p(3171) = p(9)*p(838)
p(3172) = p(9)*p(936)
p(3173) = p(9)*p(937)
p(3174) = p(9)*p(958)
p(3175) = p(3)*p(1481) - p(2723)
p(3176) = p(39)*p(570) - p(2725)
p(3177) = p(178)*p(181) - p(2728)
p(3178) = p(41)*p(566) - p(2732)
p(3179) = p(4)*p(1476) - p(2737)
p(3180) = p(1)*p(2112) - p(2733) - p(3179)
p(3181) = p(1)*p(2113) - p(2737)
p(3182) = p(1)*p(2114) - p(2738) - p(3180)
p(3183) = p(21)*p(843) - p(2732)
p(3184) = p(1)*p(2116) - p(2742) - p(3182)
p(3185) = p(101)*p(279) - p(2728)
p(3186) = p(1)*p(2118) - p(2745) - p(3184)
p(3187) = p(69)*p(346) - p(2725)
p(3188) = p(1)*p(2120) - p(2747) - p(3186)
p(3189) = p(11)*p(958) - p(2723)
p(3190) = p(1)*p(2122) - p(2753) - p(2752)
p(3191) = p(1)*p(2123) - p(2761)
p(3192) = p(3)*p(1483) - p(2763)
p(3193) = p(39)*p(572) - p(2765)
p(3194) = p(178)*p(183) - p(2768)
p(3195) = p(43)*p(566) - p(2772)
p(3196) = p(1)*p(2127) - p(2769) - p(3195)
p(3197) = p(2)*p(2113) - p(2757)
p(3198) = p(1)*p(2128) - p(2773) - p(3196)
p(3199) = p(7)*p(1374) - p(2772)
p(3200) = p(1)*p(2130) - p(2777) - p(3198)
p(3201) = p(49)*p(509) - p(2768)
p(3202) = p(1)*p(2132) - p(2780) - p(3200)
p(3203) = p(153)*p(194) - p(2765)
p(3204) = p(1)*p(2134) - p(2782) - p(3202)
p(3205) = p(34)*p(588) - p(2763)
p(3206) = p(6)*p(1481) - p(2762)
p(3207) = p(1)*p(2137) - p(2792) - p(2790)
p(3208) = p(2)*p(2123) - p(2762)
p(3209) = p(1)*p(2139) - p(2813) - p(2812)
p(3210) = p(1)*p(2140) - p(2819)
p(3211) = p(3)*p(1485) - p(2823)
p(3212) = p(39)*p(574) - p(2825)
p(3213) = p(178)*p(185) - p(2828)
p(3214) = p(46)*p(566) - p(2832)
p(3215) = p(1)*p(2144) - p(2829) - p(3214)
p(3216) = p(1)*p(2145) - p(2832)
p(3217) = p(1)*p(2146) - p(2833) - p(3215)
p(3218) = p(21)*p(869) - p(2828)
p(3219) = p(1)*p(2148) - p(2836) - p(3217)
p(3220) = p(101)*p(299) - p(2825)
p(3221) = p(1)*p(2150) - p(2838) - p(3219)
p(3222) = p(84)*p(346) - p(2823)
p(3223) = p(6)*p(1483) - p(2807)
p(3224) = p(1)*p(2153) - p(2847) - p(2845)
p(3225) = p(8)*p(1369) - p(2807)
p(3226) = p(45)*p(570) - p(2821)
p(3227) = p(1)*p(2156) - p(2869) - p(2867)
p(3228) = p(2)*p(2140) - p(2821)
p(3229) = p(1)*p(2158) - p(2888) - p(2887)
p(3230) = p(1)*p(2159) - p(2892)
p(3231) = p(3)*p(1487) - p(2898)
p(3232) = p(39)*p(576) - p(2900)
p(3233) = p(178)*p(188) - p(2903)
p(3234) = p(1)*p(2162) - p(2901) - p(3233)
p(3235) = p(2)*p(2145) - p(2853)
p(3236) = p(1)*p(2163) - p(2904) - p(3234)
p(3237) = p(7)*p(1406) - p(2903)
p(3238) = p(1)*p(2165) - p(2907) - p(3236)
p(3239) = p(49)*p(536) - p(2900)
p(3240) = p(1)*p(2167) - p(2909) - p(3238)
p(3241) = p(173)*p(194) - p(2898)
p(3242) = p(6)*p(1485) - p(2860)
p(3243) = p(1)*p(2170) - p(2917) - p(2915)
p(3244) = p(22)*p(851) - p(2860)
p(3245) = p(45)*p(572) - p(2822)
p(3246) = p(1)*p(2173) - p(2936) - p(2934)
p(3247) = p(8)*p(1385) - p(2822)
p(3248) = p(181)*p(187) - p(2894)
p(3249) = p(1)*p(2176) - p(2955) - p(2953)
p(3250) = p(2)*p(2159) - p(2894)
p(3251) = p(1)*p(2178) - p(2972) - p(2971)
p(3252) = p(1)*p(2179) - p(2974)
p(3253) = p(14)*p(882)
p(3254) = p(3)*p(1489) - p(2981)
p(3255) = p(39)*p(579) - p(2983)
p(3256) = p(178)*p(191) - p(2986)
p(3257) = p(1)*p(2183) - p(2984) - p(3256)
p(3258) = p(1)*p(2184) - p(2986)
p(3259) = p(1)*p(2185) - p(2987) - p(3257)
p(3260) = p(21)*p(904) - p(2983)
p(3261) = p(1)*p(2187) - p(2989) - p(3259)
p(3262) = p(101)*p(323) - p(2981)
p(3263) = p(6)*p(1487) - p(2928)
p(3264) = p(1)*p(2190) - p(2996) - p(2994)
p(3265) = p(50)*p(505) - p(2928)
p(3266) = p(45)*p(574) - p(2883)
p(3267) = p(1)*p(2193) - p(3012) - p(3010)
p(3268) = p(22)*p(865) - p(2883)
p(3269) = p(183)*p(187) - p(2896)
p(3270) = p(1)*p(2196) - p(3028) - p(3026)
p(3271) = p(8)*p(1402) - p(2896)
p(3272) = p(41)*p(578) - p(2976)
p(3273) = p(1)*p(2200) - p(3044) - p(3042)
p(3274) = p(2)*p(2179) - p(2976)
p(3275) = p(14)*p(898)
p(3276) = p(4)*p(1441) - p(2683) - p(3103)
p(3277) = p(1)*p(2204) - p(3054)
p(3278) = p(14)*p(901)
p(3279) = p(3)*p(1492) - p(3057)
p(3280) = p(39)*p(582) - p(3059)
p(3281) = p(1)*p(2207) - p(3058) - p(3280)
p(3282) = p(2)*p(2184) - p(3000)
p(3283) = p(1)*p(2208) - p(3060) - p(3281)
p(3284) = p(7)*p(1446) - p(3059)
p(3285) = p(1)*p(2210) - p(3062) - p(3283)
p(3286) = p(49)*p(561) - p(3057)
p(3287) = p(6)*p(1489) - p(3005)
p(3288) = p(1)*p(2213) - p(3068) - p(3066)
p(3289) = p(102)*p(285) - p(3005)
p(3290) = p(45)*p(576) - p(2948)
p(3291) = p(1)*p(2216) - p(3081) - p(3079)
p(3292) = p(50)*p(518) - p(2948)
p(3293) = p(185)*p(187) - p(2897)
p(3294) = p(1)*p(2219) - p(3094) - p(3092)
p(3295) = p(22)*p(881) - p(2897)
p(3296) = p(14)*p(913)
p(3297) = p(43)*p(578) - p(2978)
p(3298) = p(1)*p(2223) - p(3105) - p(3103)
p(3299) = p(8)*p(1422) - p(2978)
p(3300) = p(14)*p(917)
p(3301) = p(4)*p(1491) - p(3056)
p(3302) = p(2)*p(2203) - p(3055) - p(3301)
p(3303) = p(2)*p(2204) - p(3056)
p(3304) = p(14)*p(948)
p(3305) = p(3)*p(1495) - p(3109)
p(3306) = p(39)*p(585) - p(3111)
p(3307) = p(1)*p(2227) - p(3110) - p(3306)
p(3308) = p(1)*p(2228) - p(3111)
p(3309) = p(1)*p(2229) - p(3112) - p(3307)
p(3310) = p(21)*p(929) - p(3109)
p(3311) = p(6)*p(1492) - p(3075)
p(3312) = p(1)*p(2232) - p(3117) - p(3115)
p(3313) = p(150)*p(195) - p(3075)
p(3314) = p(45)*p(579) - p(3022)
p(3315) = p(1)*p(2235) - p(3127) - p(3125)
p(3316) = p(102)*p(296) - p(3022)
p(3317) = p(187)*p(188) - p(2968)
p(3318) = p(1)*p(2239) - p(3137) - p(3135)
p(3319) = p(50)*p(532) - p(2968)
p(3320) = p(14)*p(927)
p(3321) = p(46)*p(578) - p(2980)
p(3322) = p(2)*p(2223) - p(3107) - p(3302)
p(3323) = p(22)*p(900) - p(2980)
p(3324) = p(14)*p(951)
p(3325) = p(3)*p(1498) - p(3141)
p(3326) = p(2)*p(2227) - p(3118) - p(3114)
p(3327) = p(2)*p(2228) - p(3119)
p(3328) = p(1)*p(2243) - p(3142) - p(3326)
p(3329) = p(7)*p(1471) - p(3141)
p(3330) = p(6)*p(1495) - p(3122)
p(3331) = p(1)*p(2246) - p(3146) - p(3144)
p(3332) = p(73)*p(347) - p(3122)
p(3333) = p(45)*p(582) - p(3089)
p(3334) = p(1)*p(2249) - p(3153) - p(3151)
p(3335) = p(160)*p(195) - p(3089)
p(3336) = p(14)*p(933)
p(3337) = p(187)*p(191) - p(3039)
p(3338) = p(2)*p(2239) - p(3139) - p(3322)
p(3339) = p(102)*p(310) - p(3039)
p(3340) = p(14)*p(954)
p(3341) = p(3)*p(1501) - p(3157)
p(3342) = p(2)*p(2243) - p(3147) - p(3144)
p(3343) = p(1)*p(2253) - p(3157)
p(3344) = p(6)*p(1498) - p(3149)
p(3345) = p(1)*p(2255) - p(3160) - p(3158)
p(3346) = p(32)*p(589) - p(3149)
p(3347) = p(45)*p(585) - p(3133)
p(3348) = p(2)*p(2249) - p(3155) - p(3338)
p(3349) = p(81)*p(347) - p(3133)
p(3350) = p(14)*p(957)
p(3351) = p(2)*p(2252) - p(3159) - p(3158)
p(3352) = p(2)*p(2253) - p(3160)
p(3353) = p(6)*p(1501) - p(3161)
p(3354) = p(2)*p(2255) - p(3162) - p(3348)
p(3355) = p(13)*p(959) - p(3161)
p(3356) = p(14)*p(959)
p(3357) = p(3)*p(1476) - p(3168)
p(3358) = p(7)*p(1476) - p(3165)
p(3359) = p(1)*p(2259) - p(3170) - p(3358)
p(3360) = p(178)*p(194) - p(3166)
p(3361) = p(39)*p(588) - p(3171)
p(3362) = p(3)*p(1504) - p(3174)
p(3363) = p(1)*p(2263) - p(3175)
p(3364) = p(2)*p(2288) - p(3363)
p(3365) = p(1)*p(2265) - p(3192)
p(3366) = p(8)*p(1504) - p(3365)
p(3367) = p(1)*p(2267) - p(3211)
p(3368) = p(22)*p(958) - p(3367)
p(3369) = p(1)*p(2269) - p(3231)
p(3370) = p(50)*p(588) - p(3369)
p(3371) = p(1)*p(2271) - p(3254)
p(3372) = p(102)*p(346) - p(3371)
p(3373) = p(1)*p(2273) - p(3279)
p(3374) = p(194)*p(195) - p(3373)
p(3375) = p(6)*p(1491) - p(3304)
p(3376) = p(1)*p(2276) - p(3305)
p(3377) = p(101)*p(347) - p(3376)
p(3378) = p(8)*p(1491) - p(3278)
p(3379) = p(1)*p(2279) - p(3325)
p(3380) = p(49)*p(589) - p(3379)
p(3381) = p(2)*p(2278) - p(3324) - p(3378)
p(3382) = p(1)*p(2282) - p(3341)
p(3383) = p(21)*p(959) - p(3382)
p(3384) = p(187)*p(195) - p(3296)
p(3385) = p(1)*p(2285) - p(3351)
p(3386) = p(7)*p(1505) - p(3385)
p(3387) = p(45)*p(589) - p(3336)
p(3388) = p(2)*p(2285) - p(3353)
p(3389) = p(1)*p(2289) - p(3388)
p(3390) = p(6)*p(1505) - p(3356)
p(3391) = p(1)*p(2288) - p(3362)
p(3392) = p(2)*p(2289) - p(3390)

do i=1,6
dm(0) = 0.0D0
dm(1) = dmsdr(i,4)
dm(2) = dmsdr(i,2)
dm(3) = dmsdr(i,1)
dm(4) = dmsdr(i,6)
dm(5) = dmsdr(i,5)
dm(6) = dmsdr(i,3)
dm(7) = dm(1)*rm(2) + rm(1)*dm(2)
dm(8) = dm(1)*rm(3) + rm(1)*dm(3)
dm(9) = dm(2)*rm(3) + rm(2)*dm(3)
dm(10) = dm(3)*rm(4) + rm(3)*dm(4)
dm(11) = dm(2)*rm(5) + rm(2)*dm(5)
dm(12) = dm(1)*rm(6) + rm(1)*dm(6)
dm(13) = dm(4)*rm(5) + rm(4)*dm(5)
dm(14) = dm(4)*rm(6) + rm(4)*dm(6)
dm(15) = dm(5)*rm(6) + rm(5)*dm(6)
dm(16) = dm(1)*rm(9) + rm(1)*dm(9)
dm(17) = dm(1)*rm(10) + rm(1)*dm(10)
dm(18) = dm(2)*rm(10) + rm(2)*dm(10)
dm(19) = dm(1)*rm(11) + rm(1)*dm(11)
dm(20) = dm(3)*rm(11) + rm(3)*dm(11)
dm(21) = dm(2)*rm(12) + rm(2)*dm(12)
dm(22) = dm(3)*rm(12) + rm(3)*dm(12)
dm(23) = dm(2)*rm(13) + rm(2)*dm(13)
dm(24) = dm(3)*rm(13) + rm(3)*dm(13)
dm(25) = dm(1)*rm(14) + rm(1)*dm(14)
dm(26) = dm(3)*rm(14) + rm(3)*dm(14)
dm(27) = dm(1)*rm(15) + rm(1)*dm(15)
dm(28) = dm(2)*rm(15) + rm(2)*dm(15)
dm(29) = dm(4)*rm(15) + rm(4)*dm(15)
dm(30) = dm(2)*rm(24) + rm(2)*dm(24)
dm(31) = dm(1)*rm(26) + rm(1)*dm(26)
dm(32) = dm(1)*rm(28) + rm(1)*dm(28)

dp(0) = dm(0)
dp(1) = dm(1) + dm(2) + dm(3)
dp(2) = dm(4) + dm(5) + dm(6)
dp(3) = dm(7) + dm(8) + dm(9)
dp(4) = dm(10) + dm(11) + dm(12)
dp(5) = dp(1)*p(2) + p(1)*dp(2) - dp(4)
dp(6) = dm(13) + dm(14) + dm(15)
dp(7) = dp(1)*p(1) + p(1)*dp(1) - dp(3) - dp(3)
dp(8) = dp(2)*p(2) + p(2)*dp(2) - dp(6) - dp(6)
dp(9) = dm(16)
dp(10) = dm(17) + dm(18) + dm(19) + dm(20) + dm(21) + dm(22)
dp(11) = dp(2)*p(3) + p(2)*dp(3) - dp(10)
dp(12) = dm(23) + dm(24) + dm(25) + dm(26) + dm(27) + dm(28)
dp(13) = dp(1)*p(6) + p(1)*dp(6) - dp(12)
dp(14) = dm(29)
dp(15) = dp(1)*p(3) + p(1)*dp(3) - dp(9) - dp(9) - dp(9)
dp(16) = dp(1)*p(4) + p(1)*dp(4) - dp(10)
dp(17) = dp(2)*p(7) + p(2)*dp(7) - dp(16)
dp(18) = dp(2)*p(4) + p(2)*dp(4) - dp(12)
dp(19) = dp(1)*p(8) + p(1)*dp(8) - dp(18)
dp(20) = dp(2)*p(6) + p(2)*dp(6) - dp(14) - dp(14) - dp(14)
dp(21) = dp(1)*p(7) + p(1)*dp(7) - dp(15)
dp(22) = dp(2)*p(8) + p(2)*dp(8) - dp(20)
dp(23) = dp(9)*p(2) + p(9)*dp(2)
dp(24) = dm(30) + dm(31) + dm(32)
dp(25) = dp(3)*p(6) + p(3)*dp(6) - dp(24)
dp(26) = dp(14)*p(1) + p(14)*dp(1)
dp(27) = dp(9)*p(1) + p(9)*dp(1)
dp(28) = dp(3)*p(4) + p(3)*dp(4) - dp(23)
dp(29) = dp(1)*p(10) + p(1)*dp(10) - dp(23) - dp(28) - dp(23)
dp(30) = dp(1)*p(11) + p(1)*dp(11) - dp(23)
dp(31) = dp(1)*p(12) + p(1)*dp(12) - dp(25) - dp(24) - dp(24)
dp(32) = dp(1)*p(13) + p(1)*dp(13) - dp(25)
dp(33) = dp(4)*p(5) + p(4)*dp(5) - dp(25) - dp(31)
dp(34) = dp(2)*p(11) + p(2)*dp(11) - dp(25)
dp(35) = dp(4)*p(6) + p(4)*dp(6) - dp(26)
dp(36) = dp(2)*p(12) + p(2)*dp(12) - dp(26) - dp(35) - dp(26)
dp(37) = dp(2)*p(13) + p(2)*dp(13) - dp(26)
dp(38) = dp(14)*p(2) + p(14)*dp(2)
dp(39) = dp(3)*p(3) + p(3)*dp(3) - dp(27) - dp(27)
dp(40) = dp(3)*p(7) + p(3)*dp(7) - dp(27)
dp(41) = dp(1)*p(16) + p(1)*dp(16) - dp(28)
dp(42) = dp(2)*p(21) + p(2)*dp(21) - dp(41)
dp(43) = dp(1)*p(18) + p(1)*dp(18) - dp(33)
dp(44) = dp(7)*p(8) + p(7)*dp(8) - dp(43)
dp(45) = dp(6)*p(6) + p(6)*dp(6) - dp(38) - dp(38)
dp(46) = dp(2)*p(18) + p(2)*dp(18) - dp(35)
dp(47) = dp(1)*p(22) + p(1)*dp(22) - dp(46)
dp(48) = dp(6)*p(8) + p(6)*dp(8) - dp(38)
dp(49) = dp(1)*p(21) + p(1)*dp(21) - dp(40)
dp(50) = dp(2)*p(22) + p(2)*dp(22) - dp(48)
dp(51) = dp(9)*p(6) + p(9)*dp(6)
dp(52) = dp(14)*p(3) + p(14)*dp(3)
dp(53) = dp(9)*p(4) + p(9)*dp(4)
dp(54) = dp(9)*p(5) + p(9)*dp(5)
dp(55) = dp(1)*p(24) + p(1)*dp(24) - dp(51)
dp(56) = dp(3)*p(12) + p(3)*dp(12) - dp(51) - dp(55) - dp(51)
dp(57) = dp(3)*p(13) + p(3)*dp(13) - dp(51)
dp(58) = dp(14)*p(7) + p(14)*dp(7)
dp(59) = dp(9)*p(8) + p(9)*dp(8)
dp(60) = dp(2)*p(24) + p(2)*dp(24) - dp(52)
dp(61) = dp(4)*p(13) + p(4)*dp(13) - dp(58)
dp(62) = dp(6)*p(11) + p(6)*dp(11) - dp(52)
dp(63) = dp(14)*p(4) + p(14)*dp(4)
dp(64) = dp(14)*p(5) + p(14)*dp(5)
dp(65) = dp(9)*p(3) + p(9)*dp(3)
dp(66) = dp(9)*p(7) + p(9)*dp(7)
dp(67) = dp(3)*p(16) + p(3)*dp(16) - dp(53)
dp(68) = dp(4)*p(15) + p(4)*dp(15) - dp(54) - dp(67)
dp(69) = dp(2)*p(39) + p(2)*dp(39) - dp(68)
dp(70) = dp(1)*p(29) + p(1)*dp(29) - dp(54) - dp(68)
dp(71) = dp(7)*p(11) + p(7)*dp(11) - dp(53)
dp(72) = dp(1)*p(31) + p(1)*dp(31) - dp(56) - dp(55)
dp(73) = dp(1)*p(32) + p(1)*dp(32) - dp(57)
dp(74) = dp(3)*p(18) + p(3)*dp(18) - dp(59)
dp(75) = dp(2)*p(29) + p(2)*dp(29) - dp(57) - dp(55)
dp(76) = dp(1)*p(34) + p(1)*dp(34) - dp(59)
dp(77) = dp(6)*p(16) + p(6)*dp(16) - dp(58)
dp(78) = dp(1)*p(36) + p(1)*dp(36) - dp(62) - dp(60)
dp(79) = dp(2)*p(32) + p(2)*dp(32) - dp(58)
dp(80) = dp(6)*p(12) + p(6)*dp(12) - dp(64) - dp(63) - dp(63)
dp(81) = dp(1)*p(45) + p(1)*dp(45) - dp(80)
dp(82) = dp(14)*p(6) + p(14)*dp(6)
dp(83) = dp(2)*p(33) + p(2)*dp(33) - dp(61) - dp(60)
dp(84) = dp(2)*p(34) + p(2)*dp(34) - dp(62)
dp(85) = dp(6)*p(18) + p(6)*dp(18) - dp(63)
dp(86) = dp(2)*p(36) + p(2)*dp(36) - dp(64) - dp(80)
dp(87) = dp(8)*p(13) + p(8)*dp(13) - dp(63)
dp(88) = dp(14)*p(8) + p(14)*dp(8)
dp(89) = dp(1)*p(39) + p(1)*dp(39) - dp(65)
dp(90) = dp(3)*p(21) + p(3)*dp(21) - dp(66)
dp(91) = dp(1)*p(41) + p(1)*dp(41) - dp(67)
dp(92) = dp(2)*p(49) + p(2)*dp(49) - dp(91)
dp(93) = dp(1)*p(43) + p(1)*dp(43) - dp(74)
dp(94) = dp(8)*p(21) + p(8)*dp(21) - dp(93)
dp(95) = dp(1)*p(46) + p(1)*dp(46) - dp(83)
dp(96) = dp(7)*p(22) + p(7)*dp(22) - dp(95)
dp(97) = dp(2)*p(45) + p(2)*dp(45) - dp(82)
dp(98) = dp(2)*p(46) + p(2)*dp(46) - dp(85)
dp(99) = dp(1)*p(50) + p(1)*dp(50) - dp(98)
dp(100) = dp(6)*p(22) + p(6)*dp(22) - dp(88)
dp(101) = dp(1)*p(49) + p(1)*dp(49) - dp(90)
dp(102) = dp(2)*p(50) + p(2)*dp(50) - dp(100)
dp(103) = dp(9)*p(14) + p(9)*dp(14)
dp(104) = dp(9)*p(12) + p(9)*dp(12)
dp(105) = dp(9)*p(13) + p(9)*dp(13)
dp(106) = dp(14)*p(15) + p(14)*dp(15)
dp(107) = dp(9)*p(20) + p(9)*dp(20)
dp(108) = dp(14)*p(10) + p(14)*dp(10)
dp(109) = dp(14)*p(11) + p(14)*dp(11)
dp(110) = dp(9)*p(16) + p(9)*dp(16)
dp(111) = dp(9)*p(10) + p(9)*dp(10)
dp(112) = dp(9)*p(11) + p(9)*dp(11)
dp(113) = dp(9)*p(17) + p(9)*dp(17)
dp(114) = dp(3)*p(24) + p(3)*dp(24) - dp(104)
dp(115) = dp(7)*p(24) + p(7)*dp(24) - dp(105)
dp(116) = dp(3)*p(31) + p(3)*dp(31) - dp(104) - dp(115)
dp(117) = dp(6)*p(39) + p(6)*dp(39) - dp(114)
dp(118) = dp(3)*p(32) + p(3)*dp(32) - dp(105)
dp(119) = dp(14)*p(21) + p(14)*dp(21)
dp(120) = dp(9)*p(18) + p(9)*dp(18)
dp(121) = dp(9)*p(19) + p(9)*dp(19)
dp(122) = dp(4)*p(24) + p(4)*dp(24) - dp(103) - dp(103) - dp(103)
dp(123) = dp(1)*p(60) + p(1)*dp(60) - dp(107) - dp(122)
dp(124) = dp(13)*p(16) + p(13)*dp(16) - dp(119)
dp(125) = dp(2)*p(56) + p(2)*dp(56) - dp(106) - dp(124)
dp(126) = dp(4)*p(32) + p(4)*dp(32) - dp(119)
dp(127) = dp(1)*p(62) + p(1)*dp(62) - dp(107) - dp(125)
dp(128) = dp(14)*p(16) + p(14)*dp(16)
dp(129) = dp(14)*p(17) + p(14)*dp(17)
dp(130) = dp(6)*p(24) + p(6)*dp(24) - dp(108)
dp(131) = dp(3)*p(45) + p(3)*dp(45) - dp(130)
dp(132) = dp(14)*p(12) + p(14)*dp(12)
dp(133) = dp(14)*p(13) + p(14)*dp(13)
dp(134) = dp(9)*p(22) + p(9)*dp(22)
dp(135) = dp(8)*p(24) + p(8)*dp(24) - dp(109)
dp(136) = dp(13)*p(18) + p(13)*dp(18) - dp(128)
dp(137) = dp(6)*p(34) + p(6)*dp(34) - dp(109)
dp(138) = dp(14)*p(18) + p(14)*dp(18)
dp(139) = dp(14)*p(19) + p(14)*dp(19)
dp(140) = dp(9)*p(9) + p(9)*dp(9)
dp(141) = dp(9)*p(15) + p(9)*dp(15)
dp(142) = dp(9)*p(21) + p(9)*dp(21)
dp(143) = dp(3)*p(41) + p(3)*dp(41) - dp(110)
dp(144) = dp(4)*p(39) + p(4)*dp(39) - dp(112)
dp(145) = dp(1)*p(68) + p(1)*dp(68) - dp(111) - dp(144)
dp(146) = dp(1)*p(69) + p(1)*dp(69) - dp(112)
dp(147) = dp(1)*p(70) + p(1)*dp(70) - dp(113) - dp(145)
dp(148) = dp(11)*p(21) + p(11)*dp(21) - dp(110)
dp(149) = dp(1)*p(72) + p(1)*dp(72) - dp(116) - dp(115)
dp(150) = dp(1)*p(73) + p(1)*dp(73) - dp(118)
dp(151) = dp(3)*p(43) + p(3)*dp(43) - dp(120)
dp(152) = dp(15)*p(18) + p(15)*dp(18) - dp(121) - dp(151)
dp(153) = dp(2)*p(69) + p(2)*dp(69) - dp(117)
dp(154) = dp(1)*p(75) + p(1)*dp(75) - dp(121) - dp(152)
dp(155) = dp(7)*p(34) + p(7)*dp(34) - dp(120)
dp(156) = dp(6)*p(41) + p(6)*dp(41) - dp(119)
dp(157) = dp(1)*p(78) + p(1)*dp(78) - dp(125) - dp(123)
dp(158) = dp(2)*p(73) + p(2)*dp(73) - dp(119)
dp(159) = dp(1)*p(80) + p(1)*dp(80) - dp(131) - dp(130) - dp(130)
dp(160) = dp(1)*p(81) + p(1)*dp(81) - dp(131)
dp(161) = dp(14)*p(14) + p(14)*dp(14)
dp(162) = dp(3)*p(46) + p(3)*dp(46) - dp(134)
dp(163) = dp(2)*p(75) + p(2)*dp(75) - dp(126) - dp(123)
dp(164) = dp(1)*p(84) + p(1)*dp(84) - dp(134)
dp(165) = dp(6)*p(43) + p(6)*dp(43) - dp(128)
dp(166) = dp(1)*p(86) + p(1)*dp(86) - dp(137) - dp(135)
dp(167) = dp(8)*p(32) + p(8)*dp(32) - dp(128)
dp(168) = dp(4)*p(45) + p(4)*dp(45) - dp(133)
dp(169) = dp(2)*p(80) + p(2)*dp(80) - dp(132) - dp(168)
dp(170) = dp(2)*p(81) + p(2)*dp(81) - dp(133)
dp(171) = dp(14)*p(20) + p(14)*dp(20)
dp(172) = dp(2)*p(83) + p(2)*dp(83) - dp(136) - dp(135)
dp(173) = dp(2)*p(84) + p(2)*dp(84) - dp(137)
dp(174) = dp(6)*p(46) + p(6)*dp(46) - dp(138)
dp(175) = dp(2)*p(86) + p(2)*dp(86) - dp(139) - dp(169)
dp(176) = dp(13)*p(22) + p(13)*dp(22) - dp(138)
dp(177) = dp(14)*p(22) + p(14)*dp(22)
dp(178) = dp(3)*p(39) + p(3)*dp(39) - dp(141)
dp(179) = dp(1)*p(89) + p(1)*dp(89) - dp(141) - dp(178) - dp(178)
dp(180) = dp(3)*p(49) + p(3)*dp(49) - dp(142)
dp(181) = dp(1)*p(91) + p(1)*dp(91) - dp(143)
dp(182) = dp(2)*p(101) + p(2)*dp(101) - dp(181)
dp(183) = dp(1)*p(93) + p(1)*dp(93) - dp(151)
dp(184) = dp(8)*p(49) + p(8)*dp(49) - dp(183)
dp(185) = dp(1)*p(95) + p(1)*dp(95) - dp(162)
dp(186) = dp(21)*p(22) + p(21)*dp(22) - dp(185)
dp(187) = dp(6)*p(45) + p(6)*dp(45) - dp(171)
dp(188) = dp(1)*p(98) + p(1)*dp(98) - dp(172)
dp(189) = dp(7)*p(50) + p(7)*dp(50) - dp(188)
dp(190) = dp(2)*p(97) + p(2)*dp(97) - dp(171) - dp(187) - dp(187)
dp(191) = dp(2)*p(98) + p(2)*dp(98) - dp(174)
dp(192) = dp(1)*p(102) + p(1)*dp(102) - dp(191)
dp(193) = dp(6)*p(50) + p(6)*dp(50) - dp(177)
dp(194) = dp(1)*p(101) + p(1)*dp(101) - dp(180)
dp(195) = dp(2)*p(102) + p(2)*dp(102) - dp(193)
dp(196) = dp(9)*p(26) + p(9)*dp(26)
dp(197) = dp(9)*p(38) + p(9)*dp(38)
dp(198) = dp(9)*p(24) + p(9)*dp(24)
dp(199) = dp(9)*p(31) + p(9)*dp(31)
dp(200) = dp(9)*p(25) + p(9)*dp(25)
dp(201) = dp(9)*p(32) + p(9)*dp(32)
dp(202) = dp(14)*p(39) + p(14)*dp(39)
dp(203) = dp(14)*p(40) + p(14)*dp(40)
dp(204) = dp(9)*p(35) + p(9)*dp(35)
dp(205) = dp(9)*p(36) + p(9)*dp(36)
dp(206) = dp(9)*p(37) + p(9)*dp(37)
dp(207) = dp(14)*p(28) + p(14)*dp(28)
dp(208) = dp(14)*p(29) + p(14)*dp(29)
dp(209) = dp(14)*p(30) + p(14)*dp(30)
dp(210) = dp(9)*p(45) + p(9)*dp(45)
dp(211) = dp(14)*p(24) + p(14)*dp(24)
dp(212) = dp(14)*p(25) + p(14)*dp(25)
dp(213) = dp(9)*p(48) + p(9)*dp(48)
dp(214) = dp(14)*p(33) + p(14)*dp(33)
dp(215) = dp(14)*p(34) + p(14)*dp(34)
dp(216) = dp(9)*p(41) + p(9)*dp(41)
dp(217) = dp(9)*p(28) + p(9)*dp(28)
dp(218) = dp(9)*p(23) + p(9)*dp(23)
dp(219) = dp(9)*p(29) + p(9)*dp(29)
dp(220) = dp(9)*p(30) + p(9)*dp(30)
dp(221) = dp(9)*p(42) + p(9)*dp(42)
dp(222) = dp(1)*p(114) + p(1)*dp(114) - dp(198)
dp(223) = dp(21)*p(24) + p(21)*dp(24) - dp(201)
dp(224) = dp(3)*p(72) + p(3)*dp(72) - dp(199) - dp(223)
dp(225) = dp(1)*p(116) + p(1)*dp(116) - dp(199) - dp(224)
dp(226) = dp(13)*p(39) + p(13)*dp(39) - dp(198)
dp(227) = dp(3)*p(73) + p(3)*dp(73) - dp(201)
dp(228) = dp(14)*p(49) + p(14)*dp(49)
dp(229) = dp(9)*p(43) + p(9)*dp(43)
dp(230) = dp(9)*p(33) + p(9)*dp(33)
dp(231) = dp(9)*p(34) + p(9)*dp(34)
dp(232) = dp(9)*p(44) + p(9)*dp(44)
dp(233) = dp(16)*p(24) + p(16)*dp(24) - dp(196)
dp(234) = dp(2)*p(114) + p(2)*dp(114) - dp(202)
dp(235) = dp(1)*p(123) + p(1)*dp(123) - dp(205) - dp(234)
dp(236) = dp(13)*p(41) + p(13)*dp(41) - dp(228)
dp(237) = dp(2)*p(116) + p(2)*dp(116) - dp(203) - dp(236)
dp(238) = dp(16)*p(32) + p(16)*dp(32) - dp(228)
dp(239) = dp(6)*p(69) + p(6)*dp(69) - dp(202)
dp(240) = dp(4)*p(73) + p(4)*dp(73) - dp(228)
dp(241) = dp(11)*p(32) + p(11)*dp(32) - dp(196)
dp(242) = dp(14)*p(41) + p(14)*dp(41)
dp(243) = dp(14)*p(42) + p(14)*dp(42)
dp(244) = dp(1)*p(130) + p(1)*dp(130) - dp(210)
dp(245) = dp(4)*p(62) + p(4)*dp(62) - dp(213) - dp(209)
dp(246) = dp(3)*p(81) + p(3)*dp(81) - dp(210)
dp(247) = dp(14)*p(31) + p(14)*dp(31)
dp(248) = dp(14)*p(32) + p(14)*dp(32)
dp(249) = dp(14)*p(26) + p(14)*dp(26)
dp(250) = dp(9)*p(46) + p(9)*dp(46)
dp(251) = dp(9)*p(47) + p(9)*dp(47)
dp(252) = dp(18)*p(24) + p(18)*dp(24) - dp(197)
dp(253) = dp(1)*p(135) + p(1)*dp(135) - dp(213) - dp(252)
dp(254) = dp(13)*p(43) + p(13)*dp(43) - dp(242)
dp(255) = dp(2)*p(125) + p(2)*dp(125) - dp(209) - dp(245)
dp(256) = dp(18)*p(32) + p(18)*dp(32) - dp(242)
dp(257) = dp(13)*p(34) + p(13)*dp(34) - dp(197)
dp(258) = dp(14)*p(43) + p(14)*dp(43)
dp(259) = dp(14)*p(44) + p(14)*dp(44)
dp(260) = dp(2)*p(130) + p(2)*dp(130) - dp(211)
dp(261) = dp(4)*p(81) + p(4)*dp(81) - dp(248)
dp(262) = dp(11)*p(45) + p(11)*dp(45) - dp(211)
dp(263) = dp(14)*p(35) + p(14)*dp(35)
dp(264) = dp(14)*p(36) + p(14)*dp(36)
dp(265) = dp(14)*p(37) + p(14)*dp(37)
dp(266) = dp(9)*p(50) + p(9)*dp(50)
dp(267) = dp(22)*p(24) + p(22)*dp(24) - dp(215)
dp(268) = dp(13)*p(46) + p(13)*dp(46) - dp(258)
dp(269) = dp(6)*p(84) + p(6)*dp(84) - dp(215)
dp(270) = dp(14)*p(46) + p(14)*dp(46)
dp(271) = dp(14)*p(47) + p(14)*dp(47)
dp(272) = dp(9)*p(27) + p(9)*dp(27)
dp(273) = dp(9)*p(39) + p(9)*dp(39)
dp(274) = dp(9)*p(40) + p(9)*dp(40)
dp(275) = dp(9)*p(49) + p(9)*dp(49)
dp(276) = dp(3)*p(91) + p(3)*dp(91) - dp(216)
dp(277) = dp(16)*p(39) + p(16)*dp(39) - dp(218)
dp(278) = dp(1)*p(144) + p(1)*dp(144) - dp(217) - dp(277)
dp(279) = dp(2)*p(178) + p(2)*dp(178) - dp(278)
dp(280) = dp(1)*p(145) + p(1)*dp(145) - dp(219) - dp(278)
dp(281) = dp(7)*p(69) + p(7)*dp(69) - dp(218)
dp(282) = dp(1)*p(147) + p(1)*dp(147) - dp(221) - dp(280)
dp(283) = dp(11)*p(49) + p(11)*dp(49) - dp(216)
dp(284) = dp(1)*p(149) + p(1)*dp(149) - dp(224) - dp(223)
dp(285) = dp(1)*p(150) + p(1)*dp(150) - dp(227)
dp(286) = dp(3)*p(93) + p(3)*dp(93) - dp(229)
dp(287) = dp(18)*p(39) + p(18)*dp(39) - dp(231)
dp(288) = dp(1)*p(152) + p(1)*dp(152) - dp(230) - dp(287)
dp(289) = dp(1)*p(153) + p(1)*dp(153) - dp(231)
dp(290) = dp(1)*p(154) + p(1)*dp(154) - dp(232) - dp(288)
dp(291) = dp(21)*p(34) + p(21)*dp(34) - dp(229)
dp(292) = dp(6)*p(91) + p(6)*dp(91) - dp(228)
dp(293) = dp(1)*p(157) + p(1)*dp(157) - dp(237) - dp(235)
dp(294) = dp(2)*p(150) + p(2)*dp(150) - dp(228)
dp(295) = dp(1)*p(159) + p(1)*dp(159) - dp(245) - dp(244)
dp(296) = dp(1)*p(160) + p(1)*dp(160) - dp(246)
dp(297) = dp(3)*p(95) + p(3)*dp(95) - dp(250)
dp(298) = dp(2)*p(152) + p(2)*dp(152) - dp(238) - dp(234)
dp(299) = dp(2)*p(153) + p(2)*dp(153) - dp(239)
dp(300) = dp(1)*p(163) + p(1)*dp(163) - dp(251) - dp(298)
dp(301) = dp(7)*p(84) + p(7)*dp(84) - dp(250)
dp(302) = dp(6)*p(93) + p(6)*dp(93) - dp(242)
dp(303) = dp(1)*p(166) + p(1)*dp(166) - dp(255) - dp(253)
dp(304) = dp(8)*p(73) + p(8)*dp(73) - dp(242)
dp(305) = dp(16)*p(45) + p(16)*dp(45) - dp(248)
dp(306) = dp(1)*p(169) + p(1)*dp(169) - dp(262) - dp(260)
dp(307) = dp(2)*p(160) + p(2)*dp(160) - dp(248)
dp(308) = dp(14)*p(38) + p(14)*dp(38)
dp(309) = dp(6)*p(80) + p(6)*dp(80) - dp(264) - dp(263)
dp(310) = dp(1)*p(187) + p(1)*dp(187) - dp(309)
dp(311) = dp(14)*p(45) + p(14)*dp(45)
dp(312) = dp(3)*p(98) + p(3)*dp(98) - dp(266)
dp(313) = dp(2)*p(163) + p(2)*dp(163) - dp(256) - dp(253)
dp(314) = dp(1)*p(173) + p(1)*dp(173) - dp(266)
dp(315) = dp(6)*p(95) + p(6)*dp(95) - dp(258)
dp(316) = dp(1)*p(175) + p(1)*dp(175) - dp(269) - dp(267)
dp(317) = dp(22)*p(32) + p(22)*dp(32) - dp(258)
dp(318) = dp(18)*p(45) + p(18)*dp(45) - dp(249)
dp(319) = dp(2)*p(169) + p(2)*dp(169) - dp(264) - dp(309)
dp(320) = dp(8)*p(81) + p(8)*dp(81) - dp(249)
dp(321) = dp(14)*p(48) + p(14)*dp(48)
dp(322) = dp(2)*p(172) + p(2)*dp(172) - dp(268) - dp(267)
dp(323) = dp(2)*p(173) + p(2)*dp(173) - dp(269)
dp(324) = dp(6)*p(98) + p(6)*dp(98) - dp(270)
dp(325) = dp(2)*p(175) + p(2)*dp(175) - dp(271) - dp(319)
dp(326) = dp(13)*p(50) + p(13)*dp(50) - dp(270)
dp(327) = dp(14)*p(50) + p(14)*dp(50)
dp(328) = dp(1)*p(178) + p(1)*dp(178) - dp(273)
dp(329) = dp(21)*p(39) + p(21)*dp(39) - dp(272)
dp(330) = dp(3)*p(101) + p(3)*dp(101) - dp(275)
dp(331) = dp(1)*p(181) + p(1)*dp(181) - dp(276)
dp(332) = dp(2)*p(194) + p(2)*dp(194) - dp(331)
dp(333) = dp(1)*p(183) + p(1)*dp(183) - dp(286)
dp(334) = dp(8)*p(101) + p(8)*dp(101) - dp(333)
dp(335) = dp(1)*p(185) + p(1)*dp(185) - dp(297)
dp(336) = dp(22)*p(49) + p(22)*dp(49) - dp(335)
dp(337) = dp(1)*p(188) + p(1)*dp(188) - dp(312)
dp(338) = dp(21)*p(50) + p(21)*dp(50) - dp(337)
dp(339) = dp(2)*p(187) + p(2)*dp(187) - dp(311)
dp(340) = dp(1)*p(191) + p(1)*dp(191) - dp(322)
dp(341) = dp(7)*p(102) + p(7)*dp(102) - dp(340)
dp(342) = dp(22)*p(45) + p(22)*dp(45) - dp(308)
dp(343) = dp(2)*p(191) + p(2)*dp(191) - dp(324)
dp(344) = dp(1)*p(195) + p(1)*dp(195) - dp(343)
dp(345) = dp(6)*p(102) + p(6)*dp(102) - dp(327)
dp(346) = dp(1)*p(194) + p(1)*dp(194) - dp(330)
dp(347) = dp(2)*p(195) + p(2)*dp(195) - dp(345)
dp(348) = dp(9)*p(52) + p(9)*dp(52)
dp(349) = dp(9)*p(58) + p(9)*dp(58)
dp(350) = dp(9)*p(63) + p(9)*dp(63)
dp(351) = dp(9)*p(64) + p(9)*dp(64)
dp(352) = dp(9)*p(82) + p(9)*dp(82)
dp(353) = dp(9)*p(88) + p(9)*dp(88)
dp(354) = dp(9)*p(55) + p(9)*dp(55)
dp(355) = dp(9)*p(72) + p(9)*dp(72)
dp(356) = dp(9)*p(51) + p(9)*dp(51)
dp(357) = dp(9)*p(56) + p(9)*dp(56)
dp(358) = dp(9)*p(57) + p(9)*dp(57)
dp(359) = dp(9)*p(73) + p(9)*dp(73)
dp(360) = dp(14)*p(89) + p(14)*dp(89)
dp(361) = dp(14)*p(90) + p(14)*dp(90)
dp(362) = dp(9)*p(77) + p(9)*dp(77)
dp(363) = dp(9)*p(60) + p(9)*dp(60)
dp(364) = dp(9)*p(78) + p(9)*dp(78)
dp(365) = dp(9)*p(61) + p(9)*dp(61)
dp(366) = dp(9)*p(62) + p(9)*dp(62)
dp(367) = dp(9)*p(79) + p(9)*dp(79)
dp(368) = dp(14)*p(67) + p(14)*dp(67)
dp(369) = dp(14)*p(68) + p(14)*dp(68)
dp(370) = dp(14)*p(69) + p(14)*dp(69)
dp(371) = dp(14)*p(70) + p(14)*dp(70)
dp(372) = dp(14)*p(71) + p(14)*dp(71)
dp(373) = dp(9)*p(80) + p(9)*dp(80)
dp(374) = dp(9)*p(81) + p(9)*dp(81)
dp(375) = dp(14)*p(55) + p(14)*dp(55)
dp(376) = dp(14)*p(56) + p(14)*dp(56)
dp(377) = dp(14)*p(57) + p(14)*dp(57)
dp(378) = dp(14)*p(52) + p(14)*dp(52)
dp(379) = dp(9)*p(85) + p(9)*dp(85)
dp(380) = dp(9)*p(86) + p(9)*dp(86)
dp(381) = dp(9)*p(87) + p(9)*dp(87)
dp(382) = dp(14)*p(74) + p(14)*dp(74)
dp(383) = dp(14)*p(75) + p(14)*dp(75)
dp(384) = dp(14)*p(76) + p(14)*dp(76)
dp(385) = dp(9)*p(97) + p(9)*dp(97)
dp(386) = dp(14)*p(60) + p(14)*dp(60)
dp(387) = dp(14)*p(61) + p(14)*dp(61)
dp(388) = dp(14)*p(62) + p(14)*dp(62)
dp(389) = dp(9)*p(100) + p(9)*dp(100)
dp(390) = dp(14)*p(83) + p(14)*dp(83)
dp(391) = dp(14)*p(84) + p(14)*dp(84)
dp(392) = dp(9)*p(91) + p(9)*dp(91)
dp(393) = dp(9)*p(67) + p(9)*dp(67)
dp(394) = dp(9)*p(53) + p(9)*dp(53)
dp(395) = dp(9)*p(68) + p(9)*dp(68)
dp(396) = dp(9)*p(54) + p(9)*dp(54)
dp(397) = dp(9)*p(69) + p(9)*dp(69)
dp(398) = dp(9)*p(70) + p(9)*dp(70)
dp(399) = dp(9)*p(71) + p(9)*dp(71)
dp(400) = dp(9)*p(92) + p(9)*dp(92)
dp(401) = dp(3)*p(114) + p(3)*dp(114) - dp(354)
dp(402) = dp(7)*p(114) + p(7)*dp(114) - dp(356)
dp(403) = dp(24)*p(49) + p(24)*dp(49) - dp(359)
dp(404) = dp(3)*p(149) + p(3)*dp(149) - dp(355) - dp(403)
dp(405) = dp(1)*p(224) + p(1)*dp(224) - dp(355) - dp(404)
dp(406) = dp(6)*p(178) + p(6)*dp(178) - dp(401)
dp(407) = dp(32)*p(39) + p(32)*dp(39) - dp(356)
dp(408) = dp(3)*p(150) + p(3)*dp(150) - dp(359)
dp(409) = dp(14)*p(101) + p(14)*dp(101)
dp(410) = dp(9)*p(93) + p(9)*dp(93)
dp(411) = dp(9)*p(74) + p(9)*dp(74)
dp(412) = dp(9)*p(59) + p(9)*dp(59)
dp(413) = dp(9)*p(75) + p(9)*dp(75)
dp(414) = dp(9)*p(76) + p(9)*dp(76)
dp(415) = dp(9)*p(94) + p(9)*dp(94)
dp(416) = dp(24)*p(41) + p(24)*dp(41) - dp(349)
dp(417) = dp(4)*p(114) + p(4)*dp(114) - dp(348)
dp(418) = dp(1)*p(234) + p(1)*dp(234) - dp(363) - dp(417)
dp(419) = dp(1)*p(235) + p(1)*dp(235) - dp(364) - dp(418)
dp(420) = dp(13)*p(91) + p(13)*dp(91) - dp(409)
dp(421) = dp(2)*p(224) + p(2)*dp(224) - dp(361) - dp(420)
dp(422) = dp(32)*p(41) + p(32)*dp(41) - dp(409)
dp(423) = dp(1)*p(237) + p(1)*dp(237) - dp(364) - dp(421)
dp(424) = dp(16)*p(73) + p(16)*dp(73) - dp(409)
dp(425) = dp(13)*p(69) + p(13)*dp(69) - dp(348)
dp(426) = dp(4)*p(150) + p(4)*dp(150) - dp(409)
dp(427) = dp(11)*p(73) + p(11)*dp(73) - dp(349)
dp(428) = dp(14)*p(91) + p(14)*dp(91)
dp(429) = dp(14)*p(92) + p(14)*dp(92)
dp(430) = dp(3)*p(130) + p(3)*dp(130) - dp(373)
dp(431) = dp(7)*p(130) + p(7)*dp(130) - dp(374)
dp(432) = dp(3)*p(159) + p(3)*dp(159) - dp(373) - dp(431)
dp(433) = dp(39)*p(45) + p(39)*dp(45) - dp(430)
dp(434) = dp(3)*p(160) + p(3)*dp(160) - dp(374)
dp(435) = dp(14)*p(72) + p(14)*dp(72)
dp(436) = dp(14)*p(73) + p(14)*dp(73)
dp(437) = dp(14)*p(58) + p(14)*dp(58)
dp(438) = dp(9)*p(95) + p(9)*dp(95)
dp(439) = dp(9)*p(83) + p(9)*dp(83)
dp(440) = dp(9)*p(84) + p(9)*dp(84)
dp(441) = dp(9)*p(96) + p(9)*dp(96)
dp(442) = dp(24)*p(43) + p(24)*dp(43) - dp(350)
dp(443) = dp(8)*p(114) + p(8)*dp(114) - dp(370)
dp(444) = dp(1)*p(253) + p(1)*dp(253) - dp(380) - dp(443)
dp(445) = dp(13)*p(93) + p(13)*dp(93) - dp(428)
dp(446) = dp(2)*p(237) + p(2)*dp(237) - dp(372) - dp(432)
dp(447) = dp(32)*p(43) + p(32)*dp(43) - dp(428)
dp(448) = dp(6)*p(153) + p(6)*dp(153) - dp(370)
dp(449) = dp(18)*p(73) + p(18)*dp(73) - dp(428)
dp(450) = dp(32)*p(34) + p(32)*dp(34) - dp(350)
dp(451) = dp(14)*p(93) + p(14)*dp(93)
dp(452) = dp(14)*p(94) + p(14)*dp(94)
dp(453) = dp(4)*p(130) + p(4)*dp(130) - dp(352)
dp(454) = dp(1)*p(260) + p(1)*dp(260) - dp(385) - dp(453)
dp(455) = dp(16)*p(81) + p(16)*dp(81) - dp(436)
dp(456) = dp(2)*p(245) + p(2)*dp(245) - dp(376) - dp(455)
dp(457) = dp(4)*p(160) + p(4)*dp(160) - dp(436)
dp(458) = dp(11)*p(81) + p(11)*dp(81) - dp(352)
dp(459) = dp(14)*p(77) + p(14)*dp(77)
dp(460) = dp(14)*p(78) + p(14)*dp(78)
dp(461) = dp(14)*p(79) + p(14)*dp(79)
dp(462) = dp(14)*p(63) + p(14)*dp(63)
dp(463) = dp(14)*p(64) + p(14)*dp(64)
dp(464) = dp(6)*p(130) + p(6)*dp(130) - dp(386)
dp(465) = dp(3)*p(187) + p(3)*dp(187) - dp(464)
dp(466) = dp(14)*p(80) + p(14)*dp(80)
dp(467) = dp(14)*p(81) + p(14)*dp(81)
dp(468) = dp(9)*p(98) + p(9)*dp(98)
dp(469) = dp(9)*p(99) + p(9)*dp(99)
dp(470) = dp(24)*p(46) + p(24)*dp(46) - dp(353)
dp(471) = dp(1)*p(267) + p(1)*dp(267) - dp(389) - dp(470)
dp(472) = dp(13)*p(95) + p(13)*dp(95) - dp(451)
dp(473) = dp(2)*p(255) + p(2)*dp(255) - dp(384) - dp(456)
dp(474) = dp(32)*p(46) + p(32)*dp(46) - dp(451)
dp(475) = dp(13)*p(84) + p(13)*dp(84) - dp(353)
dp(476) = dp(14)*p(95) + p(14)*dp(95)
dp(477) = dp(14)*p(96) + p(14)*dp(96)
dp(478) = dp(8)*p(130) + p(8)*dp(130) - dp(378)
dp(479) = dp(18)*p(81) + p(18)*dp(81) - dp(437)
dp(480) = dp(34)*p(45) + p(34)*dp(45) - dp(378)
dp(481) = dp(14)*p(85) + p(14)*dp(85)
dp(482) = dp(14)*p(86) + p(14)*dp(86)
dp(483) = dp(14)*p(87) + p(14)*dp(87)
dp(484) = dp(9)*p(102) + p(9)*dp(102)
dp(485) = dp(24)*p(50) + p(24)*dp(50) - dp(391)
dp(486) = dp(13)*p(98) + p(13)*dp(98) - dp(476)
dp(487) = dp(6)*p(173) + p(6)*dp(173) - dp(391)
dp(488) = dp(14)*p(98) + p(14)*dp(98)
dp(489) = dp(14)*p(99) + p(14)*dp(99)
dp(490) = dp(9)*p(65) + p(9)*dp(65)
dp(491) = dp(9)*p(66) + p(9)*dp(66)
dp(492) = dp(9)*p(89) + p(9)*dp(89)
dp(493) = dp(9)*p(90) + p(9)*dp(90)
dp(494) = dp(9)*p(101) + p(9)*dp(101)
dp(495) = dp(3)*p(181) + p(3)*dp(181) - dp(392)
dp(496) = dp(39)*p(41) + p(39)*dp(41) - dp(394)
dp(497) = dp(4)*p(178) + p(4)*dp(178) - dp(397)
dp(498) = dp(1)*p(278) + p(1)*dp(278) - dp(395) - dp(497)
dp(499) = dp(1)*p(279) + p(1)*dp(279) - dp(397)
dp(500) = dp(1)*p(280) + p(1)*dp(280) - dp(398) - dp(498)
dp(501) = dp(21)*p(69) + p(21)*dp(69) - dp(394)
dp(502) = dp(1)*p(282) + p(1)*dp(282) - dp(400) - dp(500)
dp(503) = dp(11)*p(101) + p(11)*dp(101) - dp(392)
dp(504) = dp(1)*p(284) + p(1)*dp(284) - dp(404) - dp(403)
dp(505) = dp(1)*p(285) + p(1)*dp(285) - dp(408)
dp(506) = dp(3)*p(183) + p(3)*dp(183) - dp(410)
dp(507) = dp(39)*p(43) + p(39)*dp(43) - dp(412)
dp(508) = dp(1)*p(287) + p(1)*dp(287) - dp(411) - dp(507)
dp(509) = dp(2)*p(279) + p(2)*dp(279) - dp(406)
dp(510) = dp(1)*p(288) + p(1)*dp(288) - dp(413) - dp(508)
dp(511) = dp(7)*p(153) + p(7)*dp(153) - dp(412)
dp(512) = dp(1)*p(290) + p(1)*dp(290) - dp(415) - dp(510)
dp(513) = dp(34)*p(49) + p(34)*dp(49) - dp(410)
dp(514) = dp(6)*p(181) + p(6)*dp(181) - dp(409)
dp(515) = dp(1)*p(293) + p(1)*dp(293) - dp(421) - dp(419)
dp(516) = dp(2)*p(285) + p(2)*dp(285) - dp(409)
dp(517) = dp(1)*p(295) + p(1)*dp(295) - dp(432) - dp(431)
dp(518) = dp(1)*p(296) + p(1)*dp(296) - dp(434)
dp(519) = dp(3)*p(185) + p(3)*dp(185) - dp(438)
dp(520) = dp(39)*p(46) + p(39)*dp(46) - dp(440)
dp(521) = dp(1)*p(298) + p(1)*dp(298) - dp(439) - dp(520)
dp(522) = dp(1)*p(299) + p(1)*dp(299) - dp(440)
dp(523) = dp(1)*p(300) + p(1)*dp(300) - dp(441) - dp(521)
dp(524) = dp(21)*p(84) + p(21)*dp(84) - dp(438)
dp(525) = dp(6)*p(183) + p(6)*dp(183) - dp(428)
dp(526) = dp(1)*p(303) + p(1)*dp(303) - dp(446) - dp(444)
dp(527) = dp(8)*p(150) + p(8)*dp(150) - dp(428)
dp(528) = dp(41)*p(45) + p(41)*dp(45) - dp(436)
dp(529) = dp(1)*p(306) + p(1)*dp(306) - dp(456) - dp(454)
dp(530) = dp(2)*p(296) + p(2)*dp(296) - dp(436)
dp(531) = dp(4)*p(169) + p(4)*dp(169) - dp(388) - dp(478)
dp(532) = dp(1)*p(310) + p(1)*dp(310) - dp(465)
dp(533) = dp(14)*p(82) + p(14)*dp(82)
dp(534) = dp(3)*p(188) + p(3)*dp(188) - dp(468)
dp(535) = dp(2)*p(298) + p(2)*dp(298) - dp(447) - dp(443)
dp(536) = dp(2)*p(299) + p(2)*dp(299) - dp(448)
dp(537) = dp(1)*p(313) + p(1)*dp(313) - dp(469) - dp(535)
dp(538) = dp(7)*p(173) + p(7)*dp(173) - dp(468)
dp(539) = dp(6)*p(185) + p(6)*dp(185) - dp(451)
dp(540) = dp(1)*p(316) + p(1)*dp(316) - dp(473) - dp(471)
dp(541) = dp(22)*p(73) + p(22)*dp(73) - dp(451)
dp(542) = dp(43)*p(45) + p(43)*dp(45) - dp(437)
dp(543) = dp(1)*p(319) + p(1)*dp(319) - dp(480) - dp(478)
dp(544) = dp(8)*p(160) + p(8)*dp(160) - dp(437)
dp(545) = dp(14)*p(88) + p(14)*dp(88)
dp(546) = dp(4)*p(187) + p(4)*dp(187) - dp(467)
dp(547) = dp(2)*p(309) + p(2)*dp(309) - dp(466) - dp(546)
dp(548) = dp(2)*p(310) + p(2)*dp(310) - dp(467)
dp(549) = dp(14)*p(97) + p(14)*dp(97)
dp(550) = dp(3)*p(191) + p(3)*dp(191) - dp(484)
dp(551) = dp(2)*p(313) + p(2)*dp(313) - dp(474) - dp(471)
dp(552) = dp(1)*p(323) + p(1)*dp(323) - dp(484)
dp(553) = dp(6)*p(188) + p(6)*dp(188) - dp(476)
dp(554) = dp(1)*p(325) + p(1)*dp(325) - dp(487) - dp(485)
dp(555) = dp(32)*p(50) + p(32)*dp(50) - dp(476)
dp(556) = dp(45)*p(46) + p(45)*dp(46) - dp(462)
dp(557) = dp(2)*p(319) + p(2)*dp(319) - dp(482) - dp(547)
dp(558) = dp(22)*p(81) + p(22)*dp(81) - dp(462)
dp(559) = dp(14)*p(100) + p(14)*dp(100)
dp(560) = dp(2)*p(322) + p(2)*dp(322) - dp(486) - dp(485)
dp(561) = dp(2)*p(323) + p(2)*dp(323) - dp(487)
dp(562) = dp(6)*p(191) + p(6)*dp(191) - dp(488)
dp(563) = dp(2)*p(325) + p(2)*dp(325) - dp(489) - dp(557)
dp(564) = dp(13)*p(102) + p(13)*dp(102) - dp(488)
dp(565) = dp(14)*p(102) + p(14)*dp(102)
dp(566) = dp(3)*p(178) + p(3)*dp(178) - dp(492)
dp(567) = dp(7)*p(178) + p(7)*dp(178) - dp(490)
dp(568) = dp(39)*p(49) + p(39)*dp(49) - dp(491)
dp(569) = dp(3)*p(194) + p(3)*dp(194) - dp(494)
dp(570) = dp(1)*p(331) + p(1)*dp(331) - dp(495)
dp(571) = dp(2)*p(346) + p(2)*dp(346) - dp(570)
dp(572) = dp(1)*p(333) + p(1)*dp(333) - dp(506)
dp(573) = dp(8)*p(194) + p(8)*dp(194) - dp(572)
dp(574) = dp(1)*p(335) + p(1)*dp(335) - dp(519)
dp(575) = dp(22)*p(101) + p(22)*dp(101) - dp(574)
dp(576) = dp(1)*p(337) + p(1)*dp(337) - dp(534)
dp(577) = dp(49)*p(50) + p(49)*dp(50) - dp(576)
dp(578) = dp(6)*p(187) + p(6)*dp(187) - dp(549)
dp(579) = dp(1)*p(340) + p(1)*dp(340) - dp(550)
dp(580) = dp(21)*p(102) + p(21)*dp(102) - dp(579)
dp(581) = dp(8)*p(187) + p(8)*dp(187) - dp(533)
dp(582) = dp(1)*p(343) + p(1)*dp(343) - dp(560)
dp(583) = dp(7)*p(195) + p(7)*dp(195) - dp(582)
dp(584) = dp(45)*p(50) + p(45)*dp(50) - dp(545)
dp(585) = dp(2)*p(343) + p(2)*dp(343) - dp(562)
dp(586) = dp(1)*p(347) + p(1)*dp(347) - dp(585)
dp(587) = dp(6)*p(195) + p(6)*dp(195) - dp(565)
dp(588) = dp(1)*p(346) + p(1)*dp(346) - dp(569)
dp(589) = dp(2)*p(347) + p(2)*dp(347) - dp(587)
dp(590) = dp(9)*p(103) + p(9)*dp(103)
dp(591) = dp(9)*p(106) + p(9)*dp(106)
dp(592) = dp(9)*p(119) + p(9)*dp(119)
dp(593) = dp(9)*p(128) + p(9)*dp(128)
dp(594) = dp(9)*p(108) + p(9)*dp(108)
dp(595) = dp(9)*p(109) + p(9)*dp(109)
dp(596) = dp(9)*p(129) + p(9)*dp(129)
dp(597) = dp(9)*p(132) + p(9)*dp(132)
dp(598) = dp(9)*p(133) + p(9)*dp(133)
dp(599) = dp(9)*p(161) + p(9)*dp(161)
dp(600) = dp(9)*p(138) + p(9)*dp(138)
dp(601) = dp(9)*p(139) + p(9)*dp(139)
dp(602) = dp(9)*p(171) + p(9)*dp(171)
dp(603) = dp(9)*p(177) + p(9)*dp(177)
dp(604) = dp(9)*p(114) + p(9)*dp(114)
dp(605) = dp(9)*p(115) + p(9)*dp(115)
dp(606) = dp(9)*p(149) + p(9)*dp(149)
dp(607) = dp(9)*p(104) + p(9)*dp(104)
dp(608) = dp(9)*p(116) + p(9)*dp(116)
dp(609) = dp(9)*p(105) + p(9)*dp(105)
dp(610) = dp(9)*p(117) + p(9)*dp(117)
dp(611) = dp(9)*p(118) + p(9)*dp(118)
dp(612) = dp(9)*p(150) + p(9)*dp(150)
dp(613) = dp(14)*p(178) + p(14)*dp(178)
dp(614) = dp(14)*p(179) + p(14)*dp(179)
dp(615) = dp(14)*p(180) + p(14)*dp(180)
dp(616) = dp(9)*p(156) + p(9)*dp(156)
dp(617) = dp(9)*p(122) + p(9)*dp(122)
dp(618) = dp(9)*p(123) + p(9)*dp(123)
dp(619) = dp(9)*p(157) + p(9)*dp(157)
dp(620) = dp(9)*p(124) + p(9)*dp(124)
dp(621) = dp(9)*p(107) + p(9)*dp(107)
dp(622) = dp(9)*p(125) + p(9)*dp(125)
dp(623) = dp(9)*p(126) + p(9)*dp(126)
dp(624) = dp(9)*p(127) + p(9)*dp(127)
dp(625) = dp(9)*p(158) + p(9)*dp(158)
dp(626) = dp(14)*p(143) + p(14)*dp(143)
dp(627) = dp(14)*p(144) + p(14)*dp(144)
dp(628) = dp(14)*p(145) + p(14)*dp(145)
dp(629) = dp(14)*p(146) + p(14)*dp(146)
dp(630) = dp(14)*p(147) + p(14)*dp(147)
dp(631) = dp(14)*p(148) + p(14)*dp(148)
dp(632) = dp(9)*p(130) + p(9)*dp(130)
dp(633) = dp(9)*p(159) + p(9)*dp(159)
dp(634) = dp(9)*p(131) + p(9)*dp(131)
dp(635) = dp(9)*p(160) + p(9)*dp(160)
dp(636) = dp(14)*p(114) + p(14)*dp(114)
dp(637) = dp(14)*p(115) + p(14)*dp(115)
dp(638) = dp(14)*p(116) + p(14)*dp(116)
dp(639) = dp(14)*p(117) + p(14)*dp(117)
dp(640) = dp(14)*p(118) + p(14)*dp(118)
dp(641) = dp(14)*p(106) + p(14)*dp(106)
dp(642) = dp(9)*p(165) + p(9)*dp(165)
dp(643) = dp(9)*p(135) + p(9)*dp(135)
dp(644) = dp(9)*p(166) + p(9)*dp(166)
dp(645) = dp(9)*p(136) + p(9)*dp(136)
dp(646) = dp(9)*p(137) + p(9)*dp(137)
dp(647) = dp(9)*p(167) + p(9)*dp(167)
dp(648) = dp(14)*p(151) + p(14)*dp(151)
dp(649) = dp(14)*p(152) + p(14)*dp(152)
dp(650) = dp(14)*p(153) + p(14)*dp(153)
dp(651) = dp(14)*p(154) + p(14)*dp(154)
dp(652) = dp(14)*p(155) + p(14)*dp(155)
dp(653) = dp(9)*p(168) + p(9)*dp(168)
dp(654) = dp(9)*p(169) + p(9)*dp(169)
dp(655) = dp(9)*p(170) + p(9)*dp(170)
dp(656) = dp(14)*p(122) + p(14)*dp(122)
dp(657) = dp(14)*p(123) + p(14)*dp(123)
dp(658) = dp(14)*p(124) + p(14)*dp(124)
dp(659) = dp(14)*p(125) + p(14)*dp(125)
dp(660) = dp(14)*p(126) + p(14)*dp(126)
dp(661) = dp(14)*p(127) + p(14)*dp(127)
dp(662) = dp(14)*p(108) + p(14)*dp(108)
dp(663) = dp(14)*p(109) + p(14)*dp(109)
dp(664) = dp(9)*p(187) + p(9)*dp(187)
dp(665) = dp(14)*p(130) + p(14)*dp(130)
dp(666) = dp(14)*p(131) + p(14)*dp(131)
dp(667) = dp(9)*p(174) + p(9)*dp(174)
dp(668) = dp(9)*p(175) + p(9)*dp(175)
dp(669) = dp(9)*p(176) + p(9)*dp(176)
dp(670) = dp(14)*p(162) + p(14)*dp(162)
dp(671) = dp(14)*p(163) + p(14)*dp(163)
dp(672) = dp(14)*p(164) + p(14)*dp(164)
dp(673) = dp(9)*p(190) + p(9)*dp(190)
dp(674) = dp(14)*p(135) + p(14)*dp(135)
dp(675) = dp(14)*p(136) + p(14)*dp(136)
dp(676) = dp(14)*p(137) + p(14)*dp(137)
dp(677) = dp(9)*p(193) + p(9)*dp(193)
dp(678) = dp(14)*p(172) + p(14)*dp(172)
dp(679) = dp(14)*p(173) + p(14)*dp(173)
dp(680) = dp(9)*p(181) + p(9)*dp(181)
dp(681) = dp(9)*p(143) + p(9)*dp(143)
dp(682) = dp(9)*p(110) + p(9)*dp(110)
dp(683) = dp(9)*p(144) + p(9)*dp(144)
dp(684) = dp(9)*p(111) + p(9)*dp(111)
dp(685) = dp(9)*p(112) + p(9)*dp(112)
dp(686) = dp(9)*p(145) + p(9)*dp(145)
dp(687) = dp(9)*p(113) + p(9)*dp(113)
dp(688) = dp(9)*p(146) + p(9)*dp(146)
dp(689) = dp(9)*p(147) + p(9)*dp(147)
dp(690) = dp(9)*p(148) + p(9)*dp(148)
dp(691) = dp(9)*p(182) + p(9)*dp(182)
dp(692) = dp(1)*p(401) + p(1)*dp(401) - dp(604)
dp(693) = dp(21)*p(114) + p(21)*dp(114) - dp(609)
dp(694) = dp(24)*p(101) + p(24)*dp(101) - dp(612)
dp(695) = dp(3)*p(284) + p(3)*dp(284) - dp(606) - dp(694)
dp(696) = dp(1)*p(404) + p(1)*dp(404) - dp(606) - dp(695)
dp(697) = dp(1)*p(405) + p(1)*dp(405) - dp(608) - dp(696)
dp(698) = dp(13)*p(178) + p(13)*dp(178) - dp(604)
dp(699) = dp(39)*p(73) + p(39)*dp(73) - dp(609)
dp(700) = dp(3)*p(285) + p(3)*dp(285) - dp(612)
dp(701) = dp(14)*p(194) + p(14)*dp(194)
dp(702) = dp(9)*p(183) + p(9)*dp(183)
dp(703) = dp(9)*p(151) + p(9)*dp(151)
dp(704) = dp(9)*p(120) + p(9)*dp(120)
dp(705) = dp(9)*p(152) + p(9)*dp(152)
dp(706) = dp(9)*p(121) + p(9)*dp(121)
dp(707) = dp(9)*p(153) + p(9)*dp(153)
dp(708) = dp(9)*p(154) + p(9)*dp(154)
dp(709) = dp(9)*p(155) + p(9)*dp(155)
dp(710) = dp(9)*p(184) + p(9)*dp(184)
dp(711) = dp(24)*p(91) + p(24)*dp(91) - dp(592)
dp(712) = dp(1)*p(416) + p(1)*dp(416) - dp(616) - dp(711)
dp(713) = dp(2)*p(401) + p(2)*dp(401) - dp(613)
dp(714) = dp(1)*p(418) + p(1)*dp(418) - dp(618) - dp(713)
dp(715) = dp(1)*p(419) + p(1)*dp(419) - dp(619) - dp(714)
dp(716) = dp(13)*p(181) + p(13)*dp(181) - dp(701)
dp(717) = dp(2)*p(404) + p(2)*dp(404) - dp(615) - dp(716)
dp(718) = dp(32)*p(91) + p(32)*dp(91) - dp(701)
dp(719) = dp(1)*p(421) + p(1)*dp(421) - dp(619) - dp(717)
dp(720) = dp(41)*p(73) + p(41)*dp(73) - dp(701)
dp(721) = dp(6)*p(279) + p(6)*dp(279) - dp(613)
dp(722) = dp(16)*p(150) + p(16)*dp(150) - dp(701)
dp(723) = dp(1)*p(425) + p(1)*dp(425) - dp(624) - dp(721)
dp(724) = dp(4)*p(285) + p(4)*dp(285) - dp(701)
dp(725) = dp(11)*p(150) + p(11)*dp(150) - dp(592)
dp(726) = dp(14)*p(181) + p(14)*dp(181)
dp(727) = dp(14)*p(182) + p(14)*dp(182)
dp(728) = dp(1)*p(430) + p(1)*dp(430) - dp(632)
dp(729) = dp(21)*p(130) + p(21)*dp(130) - dp(635)
dp(730) = dp(3)*p(295) + p(3)*dp(295) - dp(633) - dp(729)
dp(731) = dp(1)*p(432) + p(1)*dp(432) - dp(633) - dp(730)
dp(732) = dp(39)*p(81) + p(39)*dp(81) - dp(632)
dp(733) = dp(3)*p(296) + p(3)*dp(296) - dp(635)
dp(734) = dp(14)*p(149) + p(14)*dp(149)
dp(735) = dp(14)*p(150) + p(14)*dp(150)
dp(736) = dp(14)*p(119) + p(14)*dp(119)
dp(737) = dp(9)*p(185) + p(9)*dp(185)
dp(738) = dp(9)*p(162) + p(9)*dp(162)
dp(739) = dp(9)*p(134) + p(9)*dp(134)
dp(740) = dp(9)*p(163) + p(9)*dp(163)
dp(741) = dp(9)*p(164) + p(9)*dp(164)
dp(742) = dp(9)*p(186) + p(9)*dp(186)
dp(743) = dp(24)*p(93) + p(24)*dp(93) - dp(593)
dp(744) = dp(18)*p(114) + p(18)*dp(114) - dp(595)
dp(745) = dp(1)*p(443) + p(1)*dp(443) - dp(643) - dp(744)
dp(746) = dp(1)*p(444) + p(1)*dp(444) - dp(644) - dp(745)
dp(747) = dp(13)*p(183) + p(13)*dp(183) - dp(726)
dp(748) = dp(2)*p(421) + p(2)*dp(421) - dp(631) - dp(730)
dp(749) = dp(32)*p(93) + p(32)*dp(93) - dp(726)
dp(750) = dp(1)*p(446) + p(1)*dp(446) - dp(644) - dp(748)
dp(751) = dp(43)*p(73) + p(43)*dp(73) - dp(726)
dp(752) = dp(13)*p(153) + p(13)*dp(153) - dp(595)
dp(753) = dp(18)*p(150) + p(18)*dp(150) - dp(726)
dp(754) = dp(34)*p(73) + p(34)*dp(73) - dp(593)
dp(755) = dp(14)*p(183) + p(14)*dp(183)
dp(756) = dp(14)*p(184) + p(14)*dp(184)
dp(757) = dp(16)*p(130) + p(16)*dp(130) - dp(598)
dp(758) = dp(2)*p(430) + p(2)*dp(430) - dp(636)
dp(759) = dp(1)*p(454) + p(1)*dp(454) - dp(654) - dp(758)
dp(760) = dp(41)*p(81) + p(41)*dp(81) - dp(735)
dp(761) = dp(2)*p(432) + p(2)*dp(432) - dp(638) - dp(760)
dp(762) = dp(16)*p(160) + p(16)*dp(160) - dp(735)
dp(763) = dp(45)*p(69) + p(45)*dp(69) - dp(636)
dp(764) = dp(4)*p(296) + p(4)*dp(296) - dp(735)
dp(765) = dp(11)*p(160) + p(11)*dp(160) - dp(598)
dp(766) = dp(14)*p(156) + p(14)*dp(156)
dp(767) = dp(14)*p(157) + p(14)*dp(157)
dp(768) = dp(14)*p(158) + p(14)*dp(158)
dp(769) = dp(14)*p(128) + p(14)*dp(128)
dp(770) = dp(14)*p(129) + p(14)*dp(129)
dp(771) = dp(1)*p(464) + p(1)*dp(464) - dp(664)
dp(772) = dp(4)*p(262) + p(4)*dp(262) - dp(673) - dp(661)
dp(773) = dp(3)*p(310) + p(3)*dp(310) - dp(664)
dp(774) = dp(14)*p(159) + p(14)*dp(159)
dp(775) = dp(14)*p(160) + p(14)*dp(160)
dp(776) = dp(14)*p(132) + p(14)*dp(132)
dp(777) = dp(14)*p(133) + p(14)*dp(133)
dp(778) = dp(9)*p(188) + p(9)*dp(188)
dp(779) = dp(9)*p(172) + p(9)*dp(172)
dp(780) = dp(9)*p(173) + p(9)*dp(173)
dp(781) = dp(9)*p(189) + p(9)*dp(189)
dp(782) = dp(24)*p(95) + p(24)*dp(95) - dp(600)
dp(783) = dp(22)*p(114) + p(22)*dp(114) - dp(650)
dp(784) = dp(1)*p(471) + p(1)*dp(471) - dp(668) - dp(783)
dp(785) = dp(13)*p(185) + p(13)*dp(185) - dp(755)
dp(786) = dp(2)*p(446) + p(2)*dp(446) - dp(652) - dp(761)
dp(787) = dp(32)*p(95) + p(32)*dp(95) - dp(755)
dp(788) = dp(6)*p(299) + p(6)*dp(299) - dp(650)
dp(789) = dp(46)*p(73) + p(46)*dp(73) - dp(755)
dp(790) = dp(32)*p(84) + p(32)*dp(84) - dp(600)
dp(791) = dp(14)*p(185) + p(14)*dp(185)
dp(792) = dp(14)*p(186) + p(14)*dp(186)
dp(793) = dp(2)*p(453) + p(2)*dp(453) - dp(656) - dp(771)
dp(794) = dp(1)*p(478) + p(1)*dp(478) - dp(673) - dp(793)
dp(795) = dp(43)*p(81) + p(43)*dp(81) - dp(736)
dp(796) = dp(2)*p(456) + p(2)*dp(456) - dp(659) - dp(772)
dp(797) = dp(18)*p(160) + p(18)*dp(160) - dp(736)
dp(798) = dp(1)*p(480) + p(1)*dp(480) - dp(673) - dp(796)
dp(799) = dp(14)*p(165) + p(14)*dp(165)
dp(800) = dp(14)*p(166) + p(14)*dp(166)
dp(801) = dp(14)*p(167) + p(14)*dp(167)
dp(802) = dp(14)*p(138) + p(14)*dp(138)
dp(803) = dp(14)*p(139) + p(14)*dp(139)
dp(804) = dp(2)*p(464) + p(2)*dp(464) - dp(665)
dp(805) = dp(4)*p(310) + p(4)*dp(310) - dp(775)
dp(806) = dp(11)*p(187) + p(11)*dp(187) - dp(665)
dp(807) = dp(14)*p(168) + p(14)*dp(168)
dp(808) = dp(14)*p(169) + p(14)*dp(169)
dp(809) = dp(14)*p(170) + p(14)*dp(170)
dp(810) = dp(9)*p(191) + p(9)*dp(191)
dp(811) = dp(9)*p(192) + p(9)*dp(192)
dp(812) = dp(24)*p(98) + p(24)*dp(98) - dp(603)
dp(813) = dp(1)*p(485) + p(1)*dp(485) - dp(677) - dp(812)
dp(814) = dp(13)*p(188) + p(13)*dp(188) - dp(791)
dp(815) = dp(2)*p(473) + p(2)*dp(473) - dp(672) - dp(796)
dp(816) = dp(32)*p(98) + p(32)*dp(98) - dp(791)
dp(817) = dp(13)*p(173) + p(13)*dp(173) - dp(603)
dp(818) = dp(14)*p(188) + p(14)*dp(188)
dp(819) = dp(14)*p(189) + p(14)*dp(189)
dp(820) = dp(22)*p(130) + p(22)*dp(130) - dp(663)
dp(821) = dp(46)*p(81) + p(46)*dp(81) - dp(769)
dp(822) = dp(45)*p(84) + p(45)*dp(84) - dp(663)
dp(823) = dp(14)*p(174) + p(14)*dp(174)
dp(824) = dp(14)*p(175) + p(14)*dp(175)
dp(825) = dp(14)*p(176) + p(14)*dp(176)
dp(826) = dp(9)*p(195) + p(9)*dp(195)
dp(827) = dp(24)*p(102) + p(24)*dp(102) - dp(679)
dp(828) = dp(13)*p(191) + p(13)*dp(191) - dp(818)
dp(829) = dp(6)*p(323) + p(6)*dp(323) - dp(679)
dp(830) = dp(14)*p(191) + p(14)*dp(191)
dp(831) = dp(14)*p(192) + p(14)*dp(192)
dp(832) = dp(9)*p(140) + p(9)*dp(140)
dp(833) = dp(9)*p(141) + p(9)*dp(141)
dp(834) = dp(9)*p(178) + p(9)*dp(178)
dp(835) = dp(9)*p(142) + p(9)*dp(142)
dp(836) = dp(9)*p(179) + p(9)*dp(179)
dp(837) = dp(9)*p(180) + p(9)*dp(180)
dp(838) = dp(9)*p(194) + p(9)*dp(194)
dp(839) = dp(3)*p(331) + p(3)*dp(331) - dp(680)
dp(840) = dp(39)*p(91) + p(39)*dp(91) - dp(682)
dp(841) = dp(16)*p(178) + p(16)*dp(178) - dp(685)
dp(842) = dp(1)*p(497) + p(1)*dp(497) - dp(683) - dp(841)
dp(843) = dp(2)*p(566) + p(2)*dp(566) - dp(842)
dp(844) = dp(1)*p(498) + p(1)*dp(498) - dp(686) - dp(842)
dp(845) = dp(7)*p(279) + p(7)*dp(279) - dp(685)
dp(846) = dp(1)*p(500) + p(1)*dp(500) - dp(689) - dp(844)
dp(847) = dp(49)*p(69) + p(49)*dp(69) - dp(682)
dp(848) = dp(1)*p(502) + p(1)*dp(502) - dp(691) - dp(846)
dp(849) = dp(11)*p(194) + p(11)*dp(194) - dp(680)
dp(850) = dp(1)*p(504) + p(1)*dp(504) - dp(695) - dp(694)
dp(851) = dp(1)*p(505) + p(1)*dp(505) - dp(700)
dp(852) = dp(3)*p(333) + p(3)*dp(333) - dp(702)
dp(853) = dp(39)*p(93) + p(39)*dp(93) - dp(704)
dp(854) = dp(18)*p(178) + p(18)*dp(178) - dp(707)
dp(855) = dp(1)*p(508) + p(1)*dp(508) - dp(705) - dp(854)
dp(856) = dp(1)*p(509) + p(1)*dp(509) - dp(707)
dp(857) = dp(1)*p(510) + p(1)*dp(510) - dp(708) - dp(855)
dp(858) = dp(21)*p(153) + p(21)*dp(153) - dp(704)
dp(859) = dp(1)*p(512) + p(1)*dp(512) - dp(710) - dp(857)
dp(860) = dp(34)*p(101) + p(34)*dp(101) - dp(702)
dp(861) = dp(6)*p(331) + p(6)*dp(331) - dp(701)
dp(862) = dp(1)*p(515) + p(1)*dp(515) - dp(717) - dp(715)
dp(863) = dp(2)*p(505) + p(2)*dp(505) - dp(701)
dp(864) = dp(1)*p(517) + p(1)*dp(517) - dp(730) - dp(729)
dp(865) = dp(1)*p(518) + p(1)*dp(518) - dp(733)
dp(866) = dp(3)*p(335) + p(3)*dp(335) - dp(737)
dp(867) = dp(39)*p(95) + p(39)*dp(95) - dp(739)
dp(868) = dp(1)*p(520) + p(1)*dp(520) - dp(738) - dp(867)
dp(869) = dp(2)*p(509) + p(2)*dp(509) - dp(721)
dp(870) = dp(1)*p(521) + p(1)*dp(521) - dp(740) - dp(868)
dp(871) = dp(7)*p(299) + p(7)*dp(299) - dp(739)
dp(872) = dp(1)*p(523) + p(1)*dp(523) - dp(742) - dp(870)
dp(873) = dp(49)*p(84) + p(49)*dp(84) - dp(737)
dp(874) = dp(6)*p(333) + p(6)*dp(333) - dp(726)
dp(875) = dp(1)*p(526) + p(1)*dp(526) - dp(748) - dp(746)
dp(876) = dp(8)*p(285) + p(8)*dp(285) - dp(726)
dp(877) = dp(45)*p(91) + p(45)*dp(91) - dp(735)
dp(878) = dp(1)*p(529) + p(1)*dp(529) - dp(761) - dp(759)
dp(879) = dp(2)*p(518) + p(2)*dp(518) - dp(735)
dp(880) = dp(1)*p(531) + p(1)*dp(531) - dp(772) - dp(771)
dp(881) = dp(1)*p(532) + p(1)*dp(532) - dp(773)
dp(882) = dp(14)*p(161) + p(14)*dp(161)
dp(883) = dp(3)*p(337) + p(3)*dp(337) - dp(778)
dp(884) = dp(39)*p(98) + p(39)*dp(98) - dp(780)
dp(885) = dp(1)*p(535) + p(1)*dp(535) - dp(779) - dp(884)
dp(886) = dp(1)*p(536) + p(1)*dp(536) - dp(780)
dp(887) = dp(1)*p(537) + p(1)*dp(537) - dp(781) - dp(885)
dp(888) = dp(21)*p(173) + p(21)*dp(173) - dp(778)
dp(889) = dp(6)*p(335) + p(6)*dp(335) - dp(755)
dp(890) = dp(1)*p(540) + p(1)*dp(540) - dp(786) - dp(784)
dp(891) = dp(22)*p(150) + p(22)*dp(150) - dp(755)
dp(892) = dp(45)*p(93) + p(45)*dp(93) - dp(736)
dp(893) = dp(1)*p(543) + p(1)*dp(543) - dp(796) - dp(794)
dp(894) = dp(8)*p(296) + p(8)*dp(296) - dp(736)
dp(895) = dp(16)*p(187) + p(16)*dp(187) - dp(775)
dp(896) = dp(1)*p(547) + p(1)*dp(547) - dp(806) - dp(804)
dp(897) = dp(2)*p(532) + p(2)*dp(532) - dp(775)
dp(898) = dp(14)*p(171) + p(14)*dp(171)
dp(899) = dp(6)*p(309) + p(6)*dp(309) - dp(808) - dp(807)
dp(900) = dp(1)*p(578) + p(1)*dp(578) - dp(899)
dp(901) = dp(14)*p(187) + p(14)*dp(187)
dp(902) = dp(3)*p(340) + p(3)*dp(340) - dp(810)
dp(903) = dp(2)*p(535) + p(2)*dp(535) - dp(787) - dp(783)
dp(904) = dp(2)*p(536) + p(2)*dp(536) - dp(788)
dp(905) = dp(1)*p(551) + p(1)*dp(551) - dp(811) - dp(903)
dp(906) = dp(7)*p(323) + p(7)*dp(323) - dp(810)
dp(907) = dp(6)*p(337) + p(6)*dp(337) - dp(791)
dp(908) = dp(1)*p(554) + p(1)*dp(554) - dp(815) - dp(813)
dp(909) = dp(50)*p(73) + p(50)*dp(73) - dp(791)
dp(910) = dp(45)*p(95) + p(45)*dp(95) - dp(769)
dp(911) = dp(1)*p(557) + p(1)*dp(557) - dp(822) - dp(820)
dp(912) = dp(22)*p(160) + p(22)*dp(160) - dp(769)
dp(913) = dp(14)*p(177) + p(14)*dp(177)
dp(914) = dp(18)*p(187) + p(18)*dp(187) - dp(777)
dp(915) = dp(2)*p(547) + p(2)*dp(547) - dp(808) - dp(899)
dp(916) = dp(8)*p(310) + p(8)*dp(310) - dp(777)
dp(917) = dp(14)*p(190) + p(14)*dp(190)
dp(918) = dp(3)*p(343) + p(3)*dp(343) - dp(826)
dp(919) = dp(2)*p(551) + p(2)*dp(551) - dp(816) - dp(813)
dp(920) = dp(1)*p(561) + p(1)*dp(561) - dp(826)
dp(921) = dp(6)*p(340) + p(6)*dp(340) - dp(818)
dp(922) = dp(1)*p(563) + p(1)*dp(563) - dp(829) - dp(827)
dp(923) = dp(32)*p(102) + p(32)*dp(102) - dp(818)
dp(924) = dp(45)*p(98) + p(45)*dp(98) - dp(802)
dp(925) = dp(2)*p(557) + p(2)*dp(557) - dp(824) - dp(915)
dp(926) = dp(50)*p(81) + p(50)*dp(81) - dp(802)
dp(927) = dp(14)*p(193) + p(14)*dp(193)
dp(928) = dp(2)*p(560) + p(2)*dp(560) - dp(828) - dp(827)
dp(929) = dp(2)*p(561) + p(2)*dp(561) - dp(829)
dp(930) = dp(6)*p(343) + p(6)*dp(343) - dp(830)
dp(931) = dp(2)*p(563) + p(2)*dp(563) - dp(831) - dp(925)
dp(932) = dp(13)*p(195) + p(13)*dp(195) - dp(830)
dp(933) = dp(14)*p(195) + p(14)*dp(195)
dp(934) = dp(1)*p(566) + p(1)*dp(566) - dp(834)
dp(935) = dp(1)*p(567) + p(1)*dp(567) - dp(836) - dp(934)
dp(936) = dp(39)*p(101) + p(39)*dp(101) - dp(835)
dp(937) = dp(3)*p(346) + p(3)*dp(346) - dp(838)
dp(938) = dp(1)*p(570) + p(1)*dp(570) - dp(839)
dp(939) = dp(2)*p(588) + p(2)*dp(588) - dp(938)
dp(940) = dp(1)*p(572) + p(1)*dp(572) - dp(852)
dp(941) = dp(8)*p(346) + p(8)*dp(346) - dp(940)
dp(942) = dp(1)*p(574) + p(1)*dp(574) - dp(866)
dp(943) = dp(22)*p(194) + p(22)*dp(194) - dp(942)
dp(944) = dp(1)*p(576) + p(1)*dp(576) - dp(883)
dp(945) = dp(50)*p(101) + p(50)*dp(101) - dp(944)
dp(946) = dp(1)*p(579) + p(1)*dp(579) - dp(902)
dp(947) = dp(49)*p(102) + p(49)*dp(102) - dp(946)
dp(948) = dp(2)*p(578) + p(2)*dp(578) - dp(901)
dp(949) = dp(1)*p(582) + p(1)*dp(582) - dp(918)
dp(950) = dp(21)*p(195) + p(21)*dp(195) - dp(949)
dp(951) = dp(2)*p(581) + p(2)*dp(581) - dp(917) - dp(948)
dp(952) = dp(1)*p(585) + p(1)*dp(585) - dp(928)
dp(953) = dp(7)*p(347) + p(7)*dp(347) - dp(952)
dp(954) = dp(45)*p(102) + p(45)*dp(102) - dp(913)
dp(955) = dp(2)*p(585) + p(2)*dp(585) - dp(930)
dp(956) = dp(1)*p(589) + p(1)*dp(589) - dp(955)
dp(957) = dp(6)*p(347) + p(6)*dp(347) - dp(933)
dp(958) = dp(1)*p(588) + p(1)*dp(588) - dp(937)
dp(959) = dp(2)*p(589) + p(2)*dp(589) - dp(957)
dp(960) = dp(9)*p(196) + p(9)*dp(196)
dp(961) = dp(9)*p(202) + p(9)*dp(202)
dp(962) = dp(9)*p(203) + p(9)*dp(203)
dp(963) = dp(9)*p(228) + p(9)*dp(228)
dp(964) = dp(9)*p(242) + p(9)*dp(242)
dp(965) = dp(9)*p(207) + p(9)*dp(207)
dp(966) = dp(9)*p(197) + p(9)*dp(197)
dp(967) = dp(9)*p(208) + p(9)*dp(208)
dp(968) = dp(9)*p(209) + p(9)*dp(209)
dp(969) = dp(9)*p(243) + p(9)*dp(243)
dp(970) = dp(9)*p(211) + p(9)*dp(211)
dp(971) = dp(9)*p(247) + p(9)*dp(247)
dp(972) = dp(9)*p(212) + p(9)*dp(212)
dp(973) = dp(9)*p(248) + p(9)*dp(248)
dp(974) = dp(9)*p(249) + p(9)*dp(249)
dp(975) = dp(9)*p(258) + p(9)*dp(258)
dp(976) = dp(9)*p(214) + p(9)*dp(214)
dp(977) = dp(9)*p(215) + p(9)*dp(215)
dp(978) = dp(9)*p(259) + p(9)*dp(259)
dp(979) = dp(9)*p(263) + p(9)*dp(263)
dp(980) = dp(9)*p(264) + p(9)*dp(264)
dp(981) = dp(9)*p(265) + p(9)*dp(265)
dp(982) = dp(9)*p(308) + p(9)*dp(308)
dp(983) = dp(9)*p(311) + p(9)*dp(311)
dp(984) = dp(9)*p(270) + p(9)*dp(270)
dp(985) = dp(9)*p(271) + p(9)*dp(271)
dp(986) = dp(9)*p(321) + p(9)*dp(321)
dp(987) = dp(9)*p(327) + p(9)*dp(327)
dp(988) = dp(9)*p(222) + p(9)*dp(222)
dp(989) = dp(9)*p(223) + p(9)*dp(223)
dp(990) = dp(9)*p(284) + p(9)*dp(284)
dp(991) = dp(9)*p(198) + p(9)*dp(198)
dp(992) = dp(9)*p(199) + p(9)*dp(199)
dp(993) = dp(9)*p(224) + p(9)*dp(224)
dp(994) = dp(9)*p(200) + p(9)*dp(200)
dp(995) = dp(9)*p(225) + p(9)*dp(225)
dp(996) = dp(9)*p(201) + p(9)*dp(201)
dp(997) = dp(9)*p(226) + p(9)*dp(226)
dp(998) = dp(9)*p(227) + p(9)*dp(227)
dp(999) = dp(9)*p(285) + p(9)*dp(285)
dp(1000) = dp(14)*p(328) + p(14)*dp(328)
dp(1001) = dp(14)*p(329) + p(14)*dp(329)
dp(1002) = dp(14)*p(330) + p(14)*dp(330)
dp(1003) = dp(9)*p(292) + p(9)*dp(292)
dp(1004) = dp(9)*p(233) + p(9)*dp(233)
dp(1005) = dp(9)*p(234) + p(9)*dp(234)
dp(1006) = dp(9)*p(235) + p(9)*dp(235)
dp(1007) = dp(9)*p(293) + p(9)*dp(293)
dp(1008) = dp(9)*p(236) + p(9)*dp(236)
dp(1009) = dp(9)*p(204) + p(9)*dp(204)
dp(1010) = dp(9)*p(205) + p(9)*dp(205)
dp(1011) = dp(9)*p(237) + p(9)*dp(237)
dp(1012) = dp(9)*p(238) + p(9)*dp(238)
dp(1013) = dp(9)*p(206) + p(9)*dp(206)
dp(1014) = dp(9)*p(239) + p(9)*dp(239)
dp(1015) = dp(9)*p(240) + p(9)*dp(240)
dp(1016) = dp(9)*p(241) + p(9)*dp(241)
dp(1017) = dp(9)*p(294) + p(9)*dp(294)
dp(1018) = dp(14)*p(276) + p(14)*dp(276)
dp(1019) = dp(14)*p(277) + p(14)*dp(277)
dp(1020) = dp(14)*p(278) + p(14)*dp(278)
dp(1021) = dp(14)*p(279) + p(14)*dp(279)
dp(1022) = dp(14)*p(280) + p(14)*dp(280)
dp(1023) = dp(14)*p(281) + p(14)*dp(281)
dp(1024) = dp(14)*p(282) + p(14)*dp(282)
dp(1025) = dp(14)*p(283) + p(14)*dp(283)
dp(1026) = dp(9)*p(244) + p(9)*dp(244)
dp(1027) = dp(9)*p(295) + p(9)*dp(295)
dp(1028) = dp(9)*p(210) + p(9)*dp(210)
dp(1029) = dp(9)*p(245) + p(9)*dp(245)
dp(1030) = dp(9)*p(246) + p(9)*dp(246)
dp(1031) = dp(9)*p(296) + p(9)*dp(296)
dp(1032) = dp(14)*p(222) + p(14)*dp(222)
dp(1033) = dp(14)*p(223) + p(14)*dp(223)
dp(1034) = dp(14)*p(224) + p(14)*dp(224)
dp(1035) = dp(14)*p(225) + p(14)*dp(225)
dp(1036) = dp(14)*p(226) + p(14)*dp(226)
dp(1037) = dp(14)*p(227) + p(14)*dp(227)
dp(1038) = dp(14)*p(202) + p(14)*dp(202)
dp(1039) = dp(14)*p(203) + p(14)*dp(203)
dp(1040) = dp(9)*p(302) + p(9)*dp(302)
dp(1041) = dp(9)*p(252) + p(9)*dp(252)
dp(1042) = dp(9)*p(253) + p(9)*dp(253)
dp(1043) = dp(9)*p(303) + p(9)*dp(303)
dp(1044) = dp(9)*p(254) + p(9)*dp(254)
dp(1045) = dp(9)*p(213) + p(9)*dp(213)
dp(1046) = dp(9)*p(255) + p(9)*dp(255)
dp(1047) = dp(9)*p(256) + p(9)*dp(256)
dp(1048) = dp(9)*p(257) + p(9)*dp(257)
dp(1049) = dp(9)*p(304) + p(9)*dp(304)
dp(1050) = dp(14)*p(286) + p(14)*dp(286)
dp(1051) = dp(14)*p(287) + p(14)*dp(287)
dp(1052) = dp(14)*p(288) + p(14)*dp(288)
dp(1053) = dp(14)*p(289) + p(14)*dp(289)
dp(1054) = dp(14)*p(290) + p(14)*dp(290)
dp(1055) = dp(14)*p(291) + p(14)*dp(291)
dp(1056) = dp(9)*p(305) + p(9)*dp(305)
dp(1057) = dp(9)*p(260) + p(9)*dp(260)
dp(1058) = dp(9)*p(306) + p(9)*dp(306)
dp(1059) = dp(9)*p(261) + p(9)*dp(261)
dp(1060) = dp(9)*p(262) + p(9)*dp(262)
dp(1061) = dp(9)*p(307) + p(9)*dp(307)
dp(1062) = dp(14)*p(233) + p(14)*dp(233)
dp(1063) = dp(14)*p(234) + p(14)*dp(234)
dp(1064) = dp(14)*p(235) + p(14)*dp(235)
dp(1065) = dp(14)*p(236) + p(14)*dp(236)
dp(1066) = dp(14)*p(237) + p(14)*dp(237)
dp(1067) = dp(14)*p(238) + p(14)*dp(238)
dp(1068) = dp(14)*p(239) + p(14)*dp(239)
dp(1069) = dp(14)*p(240) + p(14)*dp(240)
dp(1070) = dp(14)*p(241) + p(14)*dp(241)
dp(1071) = dp(14)*p(207) + p(14)*dp(207)
dp(1072) = dp(14)*p(208) + p(14)*dp(208)
dp(1073) = dp(14)*p(209) + p(14)*dp(209)
dp(1074) = dp(9)*p(309) + p(9)*dp(309)
dp(1075) = dp(9)*p(310) + p(9)*dp(310)
dp(1076) = dp(14)*p(244) + p(14)*dp(244)
dp(1077) = dp(14)*p(245) + p(14)*dp(245)
dp(1078) = dp(14)*p(246) + p(14)*dp(246)
dp(1079) = dp(14)*p(211) + p(14)*dp(211)
dp(1080) = dp(14)*p(212) + p(14)*dp(212)
dp(1081) = dp(9)*p(315) + p(9)*dp(315)
dp(1082) = dp(9)*p(267) + p(9)*dp(267)
dp(1083) = dp(9)*p(316) + p(9)*dp(316)
dp(1084) = dp(9)*p(268) + p(9)*dp(268)
dp(1085) = dp(9)*p(269) + p(9)*dp(269)
dp(1086) = dp(9)*p(317) + p(9)*dp(317)
dp(1087) = dp(14)*p(297) + p(14)*dp(297)
dp(1088) = dp(14)*p(298) + p(14)*dp(298)
dp(1089) = dp(14)*p(299) + p(14)*dp(299)
dp(1090) = dp(14)*p(300) + p(14)*dp(300)
dp(1091) = dp(14)*p(301) + p(14)*dp(301)
dp(1092) = dp(9)*p(318) + p(9)*dp(318)
dp(1093) = dp(9)*p(319) + p(9)*dp(319)
dp(1094) = dp(9)*p(320) + p(9)*dp(320)
dp(1095) = dp(14)*p(252) + p(14)*dp(252)
dp(1096) = dp(14)*p(253) + p(14)*dp(253)
dp(1097) = dp(14)*p(254) + p(14)*dp(254)
dp(1098) = dp(14)*p(255) + p(14)*dp(255)
dp(1099) = dp(14)*p(256) + p(14)*dp(256)
dp(1100) = dp(14)*p(257) + p(14)*dp(257)
dp(1101) = dp(14)*p(214) + p(14)*dp(214)
dp(1102) = dp(14)*p(215) + p(14)*dp(215)
dp(1103) = dp(9)*p(339) + p(9)*dp(339)
dp(1104) = dp(14)*p(260) + p(14)*dp(260)
dp(1105) = dp(14)*p(261) + p(14)*dp(261)
dp(1106) = dp(14)*p(262) + p(14)*dp(262)
dp(1107) = dp(9)*p(324) + p(9)*dp(324)
dp(1108) = dp(9)*p(325) + p(9)*dp(325)
dp(1109) = dp(9)*p(326) + p(9)*dp(326)
dp(1110) = dp(14)*p(312) + p(14)*dp(312)
dp(1111) = dp(14)*p(313) + p(14)*dp(313)
dp(1112) = dp(14)*p(314) + p(14)*dp(314)
dp(1113) = dp(9)*p(342) + p(9)*dp(342)
dp(1114) = dp(14)*p(267) + p(14)*dp(267)
dp(1115) = dp(14)*p(268) + p(14)*dp(268)
dp(1116) = dp(14)*p(269) + p(14)*dp(269)
dp(1117) = dp(9)*p(345) + p(9)*dp(345)
dp(1118) = dp(14)*p(322) + p(14)*dp(322)
dp(1119) = dp(14)*p(323) + p(14)*dp(323)
dp(1120) = dp(9)*p(331) + p(9)*dp(331)
dp(1121) = dp(9)*p(276) + p(9)*dp(276)
dp(1122) = dp(9)*p(216) + p(9)*dp(216)
dp(1123) = dp(9)*p(277) + p(9)*dp(277)
dp(1124) = dp(9)*p(217) + p(9)*dp(217)
dp(1125) = dp(9)*p(218) + p(9)*dp(218)
dp(1126) = dp(9)*p(278) + p(9)*dp(278)
dp(1127) = dp(9)*p(219) + p(9)*dp(219)
dp(1128) = dp(9)*p(220) + p(9)*dp(220)
dp(1129) = dp(9)*p(279) + p(9)*dp(279)
dp(1130) = dp(9)*p(280) + p(9)*dp(280)
dp(1131) = dp(9)*p(221) + p(9)*dp(221)
dp(1132) = dp(9)*p(281) + p(9)*dp(281)
dp(1133) = dp(9)*p(282) + p(9)*dp(282)
dp(1134) = dp(9)*p(283) + p(9)*dp(283)
dp(1135) = dp(9)*p(332) + p(9)*dp(332)
dp(1136) = dp(3)*p(401) + p(3)*dp(401) - dp(988)
dp(1137) = dp(7)*p(401) + p(7)*dp(401) - dp(991)
dp(1138) = dp(49)*p(114) + p(49)*dp(114) - dp(996)
dp(1139) = dp(24)*p(194) + p(24)*dp(194) - dp(999)
dp(1140) = dp(3)*p(504) + p(3)*dp(504) - dp(990) - dp(1139)
dp(1141) = dp(1)*p(695) + p(1)*dp(695) - dp(990) - dp(1140)
dp(1142) = dp(1)*p(696) + p(1)*dp(696) - dp(993) - dp(1141)
dp(1143) = dp(6)*p(566) + p(6)*dp(566) - dp(1136)
dp(1144) = dp(32)*p(178) + p(32)*dp(178) - dp(991)
dp(1145) = dp(39)*p(150) + p(39)*dp(150) - dp(996)
dp(1146) = dp(3)*p(505) + p(3)*dp(505) - dp(999)
dp(1147) = dp(14)*p(346) + p(14)*dp(346)
dp(1148) = dp(9)*p(333) + p(9)*dp(333)
dp(1149) = dp(9)*p(286) + p(9)*dp(286)
dp(1150) = dp(9)*p(229) + p(9)*dp(229)
dp(1151) = dp(9)*p(287) + p(9)*dp(287)
dp(1152) = dp(9)*p(230) + p(9)*dp(230)
dp(1153) = dp(9)*p(231) + p(9)*dp(231)
dp(1154) = dp(9)*p(288) + p(9)*dp(288)
dp(1155) = dp(9)*p(232) + p(9)*dp(232)
dp(1156) = dp(9)*p(289) + p(9)*dp(289)
dp(1157) = dp(9)*p(290) + p(9)*dp(290)
dp(1158) = dp(9)*p(291) + p(9)*dp(291)
dp(1159) = dp(9)*p(334) + p(9)*dp(334)
dp(1160) = dp(24)*p(181) + p(24)*dp(181) - dp(963)
dp(1161) = dp(41)*p(114) + p(41)*dp(114) - dp(960)
dp(1162) = dp(4)*p(401) + p(4)*dp(401) - dp(961)
dp(1163) = dp(1)*p(713) + p(1)*dp(713) - dp(1005) - dp(1162)
dp(1164) = dp(1)*p(714) + p(1)*dp(714) - dp(1006) - dp(1163)
dp(1165) = dp(1)*p(715) + p(1)*dp(715) - dp(1007) - dp(1164)
dp(1166) = dp(13)*p(331) + p(13)*dp(331) - dp(1147)
dp(1167) = dp(2)*p(695) + p(2)*dp(695) - dp(1002) - dp(1166)
dp(1168) = dp(32)*p(181) + p(32)*dp(181) - dp(1147)
dp(1169) = dp(1)*p(717) + p(1)*dp(717) - dp(1007) - dp(1167)
dp(1170) = dp(73)*p(91) + p(73)*dp(91) - dp(1147)
dp(1171) = dp(1)*p(719) + p(1)*dp(719) - dp(1011) - dp(1169)
dp(1172) = dp(41)*p(150) + p(41)*dp(150) - dp(1147)
dp(1173) = dp(13)*p(279) + p(13)*dp(279) - dp(961)
dp(1174) = dp(16)*p(285) + p(16)*dp(285) - dp(1147)
dp(1175) = dp(69)*p(73) + p(69)*dp(73) - dp(960)
dp(1176) = dp(4)*p(505) + p(4)*dp(505) - dp(1147)
dp(1177) = dp(11)*p(285) + p(11)*dp(285) - dp(963)
dp(1178) = dp(14)*p(331) + p(14)*dp(331)
dp(1179) = dp(14)*p(332) + p(14)*dp(332)
dp(1180) = dp(3)*p(430) + p(3)*dp(430) - dp(1026)
dp(1181) = dp(7)*p(430) + p(7)*dp(430) - dp(1028)
dp(1182) = dp(49)*p(130) + p(49)*dp(130) - dp(1031)
dp(1183) = dp(3)*p(517) + p(3)*dp(517) - dp(1027) - dp(1182)
dp(1184) = dp(1)*p(730) + p(1)*dp(730) - dp(1027) - dp(1183)
dp(1185) = dp(45)*p(178) + p(45)*dp(178) - dp(1180)
dp(1186) = dp(39)*p(160) + p(39)*dp(160) - dp(1028)
dp(1187) = dp(3)*p(518) + p(3)*dp(518) - dp(1031)
dp(1188) = dp(14)*p(284) + p(14)*dp(284)
dp(1189) = dp(14)*p(285) + p(14)*dp(285)
dp(1190) = dp(14)*p(228) + p(14)*dp(228)
dp(1191) = dp(9)*p(335) + p(9)*dp(335)
dp(1192) = dp(9)*p(297) + p(9)*dp(297)
dp(1193) = dp(9)*p(250) + p(9)*dp(250)
dp(1194) = dp(9)*p(298) + p(9)*dp(298)
dp(1195) = dp(9)*p(251) + p(9)*dp(251)
dp(1196) = dp(9)*p(299) + p(9)*dp(299)
dp(1197) = dp(9)*p(300) + p(9)*dp(300)
dp(1198) = dp(9)*p(301) + p(9)*dp(301)
dp(1199) = dp(9)*p(336) + p(9)*dp(336)
dp(1200) = dp(24)*p(183) + p(24)*dp(183) - dp(964)
dp(1201) = dp(43)*p(114) + p(43)*dp(114) - dp(966)
dp(1202) = dp(8)*p(401) + p(8)*dp(401) - dp(1021)
dp(1203) = dp(1)*p(745) + p(1)*dp(745) - dp(1042) - dp(1202)
dp(1204) = dp(1)*p(746) + p(1)*dp(746) - dp(1043) - dp(1203)
dp(1205) = dp(13)*p(333) + p(13)*dp(333) - dp(1178)
dp(1206) = dp(2)*p(717) + p(2)*dp(717) - dp(1025) - dp(1183)
dp(1207) = dp(32)*p(183) + p(32)*dp(183) - dp(1178)
dp(1208) = dp(1)*p(748) + p(1)*dp(748) - dp(1043) - dp(1206)
dp(1209) = dp(73)*p(93) + p(73)*dp(93) - dp(1178)
dp(1210) = dp(6)*p(509) + p(6)*dp(509) - dp(1021)
dp(1211) = dp(43)*p(150) + p(43)*dp(150) - dp(1178)
dp(1212) = dp(32)*p(153) + p(32)*dp(153) - dp(966)
dp(1213) = dp(18)*p(285) + p(18)*dp(285) - dp(1178)
dp(1214) = dp(34)*p(150) + p(34)*dp(150) - dp(964)
dp(1215) = dp(14)*p(333) + p(14)*dp(333)
dp(1216) = dp(14)*p(334) + p(14)*dp(334)
dp(1217) = dp(41)*p(130) + p(41)*dp(130) - dp(973)
dp(1218) = dp(4)*p(430) + p(4)*dp(430) - dp(970)
dp(1219) = dp(1)*p(758) + p(1)*dp(758) - dp(1057) - dp(1218)
dp(1220) = dp(1)*p(759) + p(1)*dp(759) - dp(1058) - dp(1219)
dp(1221) = dp(81)*p(91) + p(81)*dp(91) - dp(1189)
dp(1222) = dp(2)*p(730) + p(2)*dp(730) - dp(1034) - dp(1221)
dp(1223) = dp(41)*p(160) + p(41)*dp(160) - dp(1189)
dp(1224) = dp(1)*p(761) + p(1)*dp(761) - dp(1058) - dp(1222)
dp(1225) = dp(16)*p(296) + p(16)*dp(296) - dp(1189)
dp(1226) = dp(69)*p(81) + p(69)*dp(81) - dp(970)
dp(1227) = dp(4)*p(518) + p(4)*dp(518) - dp(1189)
dp(1228) = dp(11)*p(296) + p(11)*dp(296) - dp(973)
dp(1229) = dp(14)*p(292) + p(14)*dp(292)
dp(1230) = dp(14)*p(293) + p(14)*dp(293)
dp(1231) = dp(14)*p(294) + p(14)*dp(294)
dp(1232) = dp(14)*p(242) + p(14)*dp(242)
dp(1233) = dp(14)*p(243) + p(14)*dp(243)
dp(1234) = dp(3)*p(464) + p(3)*dp(464) - dp(1074)
dp(1235) = dp(7)*p(464) + p(7)*dp(464) - dp(1075)
dp(1236) = dp(3)*p(531) + p(3)*dp(531) - dp(1074) - dp(1235)
dp(1237) = dp(39)*p(187) + p(39)*dp(187) - dp(1234)
dp(1238) = dp(3)*p(532) + p(3)*dp(532) - dp(1075)
dp(1239) = dp(14)*p(295) + p(14)*dp(295)
dp(1240) = dp(14)*p(296) + p(14)*dp(296)
dp(1241) = dp(14)*p(247) + p(14)*dp(247)
dp(1242) = dp(14)*p(248) + p(14)*dp(248)
dp(1243) = dp(14)*p(249) + p(14)*dp(249)
dp(1244) = dp(9)*p(337) + p(9)*dp(337)
dp(1245) = dp(9)*p(312) + p(9)*dp(312)
dp(1246) = dp(9)*p(266) + p(9)*dp(266)
dp(1247) = dp(9)*p(313) + p(9)*dp(313)
dp(1248) = dp(9)*p(314) + p(9)*dp(314)
dp(1249) = dp(9)*p(338) + p(9)*dp(338)
dp(1250) = dp(24)*p(185) + p(24)*dp(185) - dp(975)
dp(1251) = dp(46)*p(114) + p(46)*dp(114) - dp(977)
dp(1252) = dp(1)*p(783) + p(1)*dp(783) - dp(1082) - dp(1251)
dp(1253) = dp(1)*p(784) + p(1)*dp(784) - dp(1083) - dp(1252)
dp(1254) = dp(13)*p(335) + p(13)*dp(335) - dp(1215)
dp(1255) = dp(2)*p(748) + p(2)*dp(748) - dp(1055) - dp(1222)
dp(1256) = dp(32)*p(185) + p(32)*dp(185) - dp(1215)
dp(1257) = dp(1)*p(786) + p(1)*dp(786) - dp(1083) - dp(1255)
dp(1258) = dp(73)*p(95) + p(73)*dp(95) - dp(1215)
dp(1259) = dp(13)*p(299) + p(13)*dp(299) - dp(977)
dp(1260) = dp(46)*p(150) + p(46)*dp(150) - dp(1215)
dp(1261) = dp(73)*p(84) + p(73)*dp(84) - dp(975)
dp(1262) = dp(14)*p(335) + p(14)*dp(335)
dp(1263) = dp(14)*p(336) + p(14)*dp(336)
dp(1264) = dp(43)*p(130) + p(43)*dp(130) - dp(974)
dp(1265) = dp(8)*p(430) + p(8)*dp(430) - dp(1038)
dp(1266) = dp(1)*p(794) + p(1)*dp(794) - dp(1093) - dp(1265)
dp(1267) = dp(81)*p(93) + p(81)*dp(93) - dp(1190)
dp(1268) = dp(2)*p(761) + p(2)*dp(761) - dp(1066) - dp(1236)
dp(1269) = dp(43)*p(160) + p(43)*dp(160) - dp(1190)
dp(1270) = dp(45)*p(153) + p(45)*dp(153) - dp(1038)
dp(1271) = dp(18)*p(296) + p(18)*dp(296) - dp(1190)
dp(1272) = dp(34)*p(160) + p(34)*dp(160) - dp(974)
dp(1273) = dp(14)*p(302) + p(14)*dp(302)
dp(1274) = dp(14)*p(303) + p(14)*dp(303)
dp(1275) = dp(14)*p(304) + p(14)*dp(304)
dp(1276) = dp(14)*p(258) + p(14)*dp(258)
dp(1277) = dp(14)*p(259) + p(14)*dp(259)
dp(1278) = dp(4)*p(464) + p(4)*dp(464) - dp(983)
dp(1279) = dp(1)*p(804) + p(1)*dp(804) - dp(1103) - dp(1278)
dp(1280) = dp(16)*p(310) + p(16)*dp(310) - dp(1240)
dp(1281) = dp(2)*p(772) + p(2)*dp(772) - dp(1077) - dp(1280)
dp(1282) = dp(4)*p(532) + p(4)*dp(532) - dp(1240)
dp(1283) = dp(11)*p(310) + p(11)*dp(310) - dp(983)
dp(1284) = dp(14)*p(305) + p(14)*dp(305)
dp(1285) = dp(14)*p(306) + p(14)*dp(306)
dp(1286) = dp(14)*p(307) + p(14)*dp(307)
dp(1287) = dp(14)*p(263) + p(14)*dp(263)
dp(1288) = dp(14)*p(264) + p(14)*dp(264)
dp(1289) = dp(14)*p(265) + p(14)*dp(265)
dp(1290) = dp(6)*p(464) + p(6)*dp(464) - dp(1104)
dp(1291) = dp(3)*p(578) + p(3)*dp(578) - dp(1290)
dp(1292) = dp(14)*p(309) + p(14)*dp(309)
dp(1293) = dp(14)*p(310) + p(14)*dp(310)
dp(1294) = dp(9)*p(340) + p(9)*dp(340)
dp(1295) = dp(9)*p(322) + p(9)*dp(322)
dp(1296) = dp(9)*p(323) + p(9)*dp(323)
dp(1297) = dp(9)*p(341) + p(9)*dp(341)
dp(1298) = dp(24)*p(188) + p(24)*dp(188) - dp(984)
dp(1299) = dp(50)*p(114) + p(50)*dp(114) - dp(1089)
dp(1300) = dp(1)*p(813) + p(1)*dp(813) - dp(1108) - dp(1299)
dp(1301) = dp(13)*p(337) + p(13)*dp(337) - dp(1262)
dp(1302) = dp(2)*p(786) + p(2)*dp(786) - dp(1091) - dp(1268)
dp(1303) = dp(32)*p(188) + p(32)*dp(188) - dp(1262)
dp(1304) = dp(6)*p(536) + p(6)*dp(536) - dp(1089)
dp(1305) = dp(73)*p(98) + p(73)*dp(98) - dp(1262)
dp(1306) = dp(32)*p(173) + p(32)*dp(173) - dp(984)
dp(1307) = dp(14)*p(337) + p(14)*dp(337)
dp(1308) = dp(14)*p(338) + p(14)*dp(338)
dp(1309) = dp(46)*p(130) + p(46)*dp(130) - dp(982)
dp(1310) = dp(1)*p(820) + p(1)*dp(820) - dp(1113) - dp(1309)
dp(1311) = dp(81)*p(95) + p(81)*dp(95) - dp(1232)
dp(1312) = dp(2)*p(796) + p(2)*dp(796) - dp(1098) - dp(1281)
dp(1313) = dp(46)*p(160) + p(46)*dp(160) - dp(1232)
dp(1314) = dp(81)*p(84) + p(81)*dp(84) - dp(982)
dp(1315) = dp(14)*p(315) + p(14)*dp(315)
dp(1316) = dp(14)*p(316) + p(14)*dp(316)
dp(1317) = dp(14)*p(317) + p(14)*dp(317)
dp(1318) = dp(14)*p(270) + p(14)*dp(270)
dp(1319) = dp(14)*p(271) + p(14)*dp(271)
dp(1320) = dp(8)*p(464) + p(8)*dp(464) - dp(1079)
dp(1321) = dp(18)*p(310) + p(18)*dp(310) - dp(1242)
dp(1322) = dp(34)*p(187) + p(34)*dp(187) - dp(1079)
dp(1323) = dp(14)*p(318) + p(14)*dp(318)
dp(1324) = dp(14)*p(319) + p(14)*dp(319)
dp(1325) = dp(14)*p(320) + p(14)*dp(320)
dp(1326) = dp(9)*p(343) + p(9)*dp(343)
dp(1327) = dp(9)*p(344) + p(9)*dp(344)
dp(1328) = dp(24)*p(191) + p(24)*dp(191) - dp(987)
dp(1329) = dp(1)*p(827) + p(1)*dp(827) - dp(1117) - dp(1328)
dp(1330) = dp(13)*p(340) + p(13)*dp(340) - dp(1307)
dp(1331) = dp(2)*p(815) + p(2)*dp(815) - dp(1112) - dp(1312)
dp(1332) = dp(32)*p(191) + p(32)*dp(191) - dp(1307)
dp(1333) = dp(13)*p(323) + p(13)*dp(323) - dp(987)
dp(1334) = dp(14)*p(340) + p(14)*dp(340)
dp(1335) = dp(14)*p(341) + p(14)*dp(341)
dp(1336) = dp(50)*p(130) + p(50)*dp(130) - dp(1102)
dp(1337) = dp(81)*p(98) + p(81)*dp(98) - dp(1276)
dp(1338) = dp(45)*p(173) + p(45)*dp(173) - dp(1102)
dp(1339) = dp(14)*p(324) + p(14)*dp(324)
dp(1340) = dp(14)*p(325) + p(14)*dp(325)
dp(1341) = dp(14)*p(326) + p(14)*dp(326)
dp(1342) = dp(9)*p(347) + p(9)*dp(347)
dp(1343) = dp(24)*p(195) + p(24)*dp(195) - dp(1119)
dp(1344) = dp(13)*p(343) + p(13)*dp(343) - dp(1334)
dp(1345) = dp(6)*p(561) + p(6)*dp(561) - dp(1119)
dp(1346) = dp(14)*p(343) + p(14)*dp(343)
dp(1347) = dp(14)*p(344) + p(14)*dp(344)
dp(1348) = dp(9)*p(272) + p(9)*dp(272)
dp(1349) = dp(9)*p(273) + p(9)*dp(273)
dp(1350) = dp(9)*p(274) + p(9)*dp(274)
dp(1351) = dp(9)*p(328) + p(9)*dp(328)
dp(1352) = dp(9)*p(275) + p(9)*dp(275)
dp(1353) = dp(9)*p(329) + p(9)*dp(329)
dp(1354) = dp(9)*p(330) + p(9)*dp(330)
dp(1355) = dp(9)*p(346) + p(9)*dp(346)
dp(1356) = dp(3)*p(570) + p(3)*dp(570) - dp(1120)
dp(1357) = dp(39)*p(181) + p(39)*dp(181) - dp(1122)
dp(1358) = dp(41)*p(178) + p(41)*dp(178) - dp(1125)
dp(1359) = dp(4)*p(566) + p(4)*dp(566) - dp(1129)
dp(1360) = dp(1)*p(842) + p(1)*dp(842) - dp(1126) - dp(1359)
dp(1361) = dp(1)*p(843) + p(1)*dp(843) - dp(1129)
dp(1362) = dp(1)*p(844) + p(1)*dp(844) - dp(1130) - dp(1360)
dp(1363) = dp(21)*p(279) + p(21)*dp(279) - dp(1125)
dp(1364) = dp(1)*p(846) + p(1)*dp(846) - dp(1133) - dp(1362)
dp(1365) = dp(69)*p(101) + p(69)*dp(101) - dp(1122)
dp(1366) = dp(1)*p(848) + p(1)*dp(848) - dp(1135) - dp(1364)
dp(1367) = dp(11)*p(346) + p(11)*dp(346) - dp(1120)
dp(1368) = dp(1)*p(850) + p(1)*dp(850) - dp(1140) - dp(1139)
dp(1369) = dp(1)*p(851) + p(1)*dp(851) - dp(1146)
dp(1370) = dp(3)*p(572) + p(3)*dp(572) - dp(1148)
dp(1371) = dp(39)*p(183) + p(39)*dp(183) - dp(1150)
dp(1372) = dp(43)*p(178) + p(43)*dp(178) - dp(1153)
dp(1373) = dp(1)*p(854) + p(1)*dp(854) - dp(1151) - dp(1372)
dp(1374) = dp(2)*p(843) + p(2)*dp(843) - dp(1143)
dp(1375) = dp(1)*p(855) + p(1)*dp(855) - dp(1154) - dp(1373)
dp(1376) = dp(7)*p(509) + p(7)*dp(509) - dp(1153)
dp(1377) = dp(1)*p(857) + p(1)*dp(857) - dp(1157) - dp(1375)
dp(1378) = dp(49)*p(153) + p(49)*dp(153) - dp(1150)
dp(1379) = dp(1)*p(859) + p(1)*dp(859) - dp(1159) - dp(1377)
dp(1380) = dp(34)*p(194) + p(34)*dp(194) - dp(1148)
dp(1381) = dp(6)*p(570) + p(6)*dp(570) - dp(1147)
dp(1382) = dp(1)*p(862) + p(1)*dp(862) - dp(1167) - dp(1165)
dp(1383) = dp(2)*p(851) + p(2)*dp(851) - dp(1147)
dp(1384) = dp(1)*p(864) + p(1)*dp(864) - dp(1183) - dp(1182)
dp(1385) = dp(1)*p(865) + p(1)*dp(865) - dp(1187)
dp(1386) = dp(3)*p(574) + p(3)*dp(574) - dp(1191)
dp(1387) = dp(39)*p(185) + p(39)*dp(185) - dp(1193)
dp(1388) = dp(46)*p(178) + p(46)*dp(178) - dp(1196)
dp(1389) = dp(1)*p(868) + p(1)*dp(868) - dp(1194) - dp(1388)
dp(1390) = dp(1)*p(869) + p(1)*dp(869) - dp(1196)
dp(1391) = dp(1)*p(870) + p(1)*dp(870) - dp(1197) - dp(1389)
dp(1392) = dp(21)*p(299) + p(21)*dp(299) - dp(1193)
dp(1393) = dp(1)*p(872) + p(1)*dp(872) - dp(1199) - dp(1391)
dp(1394) = dp(84)*p(101) + p(84)*dp(101) - dp(1191)
dp(1395) = dp(6)*p(572) + p(6)*dp(572) - dp(1178)
dp(1396) = dp(1)*p(875) + p(1)*dp(875) - dp(1206) - dp(1204)
dp(1397) = dp(8)*p(505) + p(8)*dp(505) - dp(1178)
dp(1398) = dp(45)*p(181) + p(45)*dp(181) - dp(1189)
dp(1399) = dp(1)*p(878) + p(1)*dp(878) - dp(1222) - dp(1220)
dp(1400) = dp(2)*p(865) + p(2)*dp(865) - dp(1189)
dp(1401) = dp(1)*p(880) + p(1)*dp(880) - dp(1236) - dp(1235)
dp(1402) = dp(1)*p(881) + p(1)*dp(881) - dp(1238)
dp(1403) = dp(3)*p(576) + p(3)*dp(576) - dp(1244)
dp(1404) = dp(39)*p(188) + p(39)*dp(188) - dp(1246)
dp(1405) = dp(1)*p(884) + p(1)*dp(884) - dp(1245) - dp(1404)
dp(1406) = dp(2)*p(869) + p(2)*dp(869) - dp(1210)
dp(1407) = dp(1)*p(885) + p(1)*dp(885) - dp(1247) - dp(1405)
dp(1408) = dp(7)*p(536) + p(7)*dp(536) - dp(1246)
dp(1409) = dp(1)*p(887) + p(1)*dp(887) - dp(1249) - dp(1407)
dp(1410) = dp(49)*p(173) + p(49)*dp(173) - dp(1244)
dp(1411) = dp(6)*p(574) + p(6)*dp(574) - dp(1215)
dp(1412) = dp(1)*p(890) + p(1)*dp(890) - dp(1255) - dp(1253)
dp(1413) = dp(22)*p(285) + p(22)*dp(285) - dp(1215)
dp(1414) = dp(45)*p(183) + p(45)*dp(183) - dp(1190)
dp(1415) = dp(1)*p(893) + p(1)*dp(893) - dp(1268) - dp(1266)
dp(1416) = dp(8)*p(518) + p(8)*dp(518) - dp(1190)
dp(1417) = dp(41)*p(187) + p(41)*dp(187) - dp(1240)
dp(1418) = dp(1)*p(896) + p(1)*dp(896) - dp(1281) - dp(1279)
dp(1419) = dp(2)*p(881) + p(2)*dp(881) - dp(1240)
dp(1420) = dp(14)*p(308) + p(14)*dp(308)
dp(1421) = dp(4)*p(547) + p(4)*dp(547) - dp(1106) - dp(1320)
dp(1422) = dp(1)*p(900) + p(1)*dp(900) - dp(1291)
dp(1423) = dp(14)*p(311) + p(14)*dp(311)
dp(1424) = dp(3)*p(579) + p(3)*dp(579) - dp(1294)
dp(1425) = dp(39)*p(191) + p(39)*dp(191) - dp(1296)
dp(1426) = dp(1)*p(903) + p(1)*dp(903) - dp(1295) - dp(1425)
dp(1427) = dp(1)*p(904) + p(1)*dp(904) - dp(1296)
dp(1428) = dp(1)*p(905) + p(1)*dp(905) - dp(1297) - dp(1426)
dp(1429) = dp(21)*p(323) + p(21)*dp(323) - dp(1294)
dp(1430) = dp(6)*p(576) + p(6)*dp(576) - dp(1262)
dp(1431) = dp(1)*p(908) + p(1)*dp(908) - dp(1302) - dp(1300)
dp(1432) = dp(50)*p(150) + p(50)*dp(150) - dp(1262)
dp(1433) = dp(45)*p(185) + p(45)*dp(185) - dp(1232)
dp(1434) = dp(1)*p(911) + p(1)*dp(911) - dp(1312) - dp(1310)
dp(1435) = dp(22)*p(296) + p(22)*dp(296) - dp(1232)
dp(1436) = dp(43)*p(187) + p(43)*dp(187) - dp(1242)
dp(1437) = dp(1)*p(915) + p(1)*dp(915) - dp(1322) - dp(1320)
dp(1438) = dp(8)*p(532) + p(8)*dp(532) - dp(1242)
dp(1439) = dp(14)*p(321) + p(14)*dp(321)
dp(1440) = dp(4)*p(578) + p(4)*dp(578) - dp(1293)
dp(1441) = dp(2)*p(899) + p(2)*dp(899) - dp(1292) - dp(1440)
dp(1442) = dp(2)*p(900) + p(2)*dp(900) - dp(1293)
dp(1443) = dp(14)*p(339) + p(14)*dp(339)
dp(1444) = dp(3)*p(582) + p(3)*dp(582) - dp(1326)
dp(1445) = dp(2)*p(903) + p(2)*dp(903) - dp(1303) - dp(1299)
dp(1446) = dp(2)*p(904) + p(2)*dp(904) - dp(1304)
dp(1447) = dp(1)*p(919) + p(1)*dp(919) - dp(1327) - dp(1445)
dp(1448) = dp(7)*p(561) + p(7)*dp(561) - dp(1326)
dp(1449) = dp(6)*p(579) + p(6)*dp(579) - dp(1307)
dp(1450) = dp(1)*p(922) + p(1)*dp(922) - dp(1331) - dp(1329)
dp(1451) = dp(73)*p(102) + p(73)*dp(102) - dp(1307)
dp(1452) = dp(45)*p(188) + p(45)*dp(188) - dp(1276)
dp(1453) = dp(1)*p(925) + p(1)*dp(925) - dp(1338) - dp(1336)
dp(1454) = dp(50)*p(160) + p(50)*dp(160) - dp(1276)
dp(1455) = dp(14)*p(327) + p(14)*dp(327)
dp(1456) = dp(46)*p(187) + p(46)*dp(187) - dp(1243)
dp(1457) = dp(2)*p(915) + p(2)*dp(915) - dp(1324) - dp(1441)
dp(1458) = dp(22)*p(310) + p(22)*dp(310) - dp(1243)
dp(1459) = dp(14)*p(342) + p(14)*dp(342)
dp(1460) = dp(3)*p(585) + p(3)*dp(585) - dp(1342)
dp(1461) = dp(2)*p(919) + p(2)*dp(919) - dp(1332) - dp(1329)
dp(1462) = dp(1)*p(929) + p(1)*dp(929) - dp(1342)
dp(1463) = dp(6)*p(582) + p(6)*dp(582) - dp(1334)
dp(1464) = dp(1)*p(931) + p(1)*dp(931) - dp(1345) - dp(1343)
dp(1465) = dp(32)*p(195) + p(32)*dp(195) - dp(1334)
dp(1466) = dp(45)*p(191) + p(45)*dp(191) - dp(1318)
dp(1467) = dp(2)*p(925) + p(2)*dp(925) - dp(1340) - dp(1457)
dp(1468) = dp(81)*p(102) + p(81)*dp(102) - dp(1318)
dp(1469) = dp(14)*p(345) + p(14)*dp(345)
dp(1470) = dp(2)*p(928) + p(2)*dp(928) - dp(1344) - dp(1343)
dp(1471) = dp(2)*p(929) + p(2)*dp(929) - dp(1345)
dp(1472) = dp(6)*p(585) + p(6)*dp(585) - dp(1346)
dp(1473) = dp(2)*p(931) + p(2)*dp(931) - dp(1347) - dp(1467)
dp(1474) = dp(13)*p(347) + p(13)*dp(347) - dp(1346)
dp(1475) = dp(14)*p(347) + p(14)*dp(347)
dp(1476) = dp(3)*p(566) + p(3)*dp(566) - dp(1351)
dp(1477) = dp(7)*p(566) + p(7)*dp(566) - dp(1349)
dp(1478) = dp(49)*p(178) + p(49)*dp(178) - dp(1348)
dp(1479) = dp(39)*p(194) + p(39)*dp(194) - dp(1352)
dp(1480) = dp(3)*p(588) + p(3)*dp(588) - dp(1355)
dp(1481) = dp(1)*p(938) + p(1)*dp(938) - dp(1356)
dp(1482) = dp(2)*p(958) + p(2)*dp(958) - dp(1481)
dp(1483) = dp(1)*p(940) + p(1)*dp(940) - dp(1370)
dp(1484) = dp(8)*p(588) + p(8)*dp(588) - dp(1483)
dp(1485) = dp(1)*p(942) + p(1)*dp(942) - dp(1386)
dp(1486) = dp(22)*p(346) + p(22)*dp(346) - dp(1485)
dp(1487) = dp(1)*p(944) + p(1)*dp(944) - dp(1403)
dp(1488) = dp(50)*p(194) + p(50)*dp(194) - dp(1487)
dp(1489) = dp(1)*p(946) + p(1)*dp(946) - dp(1424)
dp(1490) = dp(101)*p(102) + p(101)*dp(102) - dp(1489)
dp(1491) = dp(6)*p(578) + p(6)*dp(578) - dp(1443)
dp(1492) = dp(1)*p(949) + p(1)*dp(949) - dp(1444)
dp(1493) = dp(49)*p(195) + p(49)*dp(195) - dp(1492)
dp(1494) = dp(8)*p(578) + p(8)*dp(578) - dp(1423)
dp(1495) = dp(1)*p(952) + p(1)*dp(952) - dp(1460)
dp(1496) = dp(21)*p(347) + p(21)*dp(347) - dp(1495)
dp(1497) = dp(50)*p(187) + p(50)*dp(187) - dp(1420)
dp(1498) = dp(1)*p(955) + p(1)*dp(955) - dp(1470)
dp(1499) = dp(7)*p(589) + p(7)*dp(589) - dp(1498)
dp(1500) = dp(45)*p(195) + p(45)*dp(195) - dp(1455)
dp(1501) = dp(2)*p(955) + p(2)*dp(955) - dp(1472)
dp(1502) = dp(1)*p(959) + p(1)*dp(959) - dp(1501)
dp(1503) = dp(6)*p(589) + p(6)*dp(589) - dp(1475)
dp(1504) = dp(1)*p(958) + p(1)*dp(958) - dp(1480)
dp(1505) = dp(2)*p(959) + p(2)*dp(959) - dp(1503)
dp(1506) = dp(9)*p(348) + p(9)*dp(348)
dp(1507) = dp(9)*p(349) + p(9)*dp(349)
dp(1508) = dp(9)*p(360) + p(9)*dp(360)
dp(1509) = dp(9)*p(361) + p(9)*dp(361)
dp(1510) = dp(9)*p(409) + p(9)*dp(409)
dp(1511) = dp(9)*p(428) + p(9)*dp(428)
dp(1512) = dp(9)*p(368) + p(9)*dp(368)
dp(1513) = dp(9)*p(350) + p(9)*dp(350)
dp(1514) = dp(9)*p(369) + p(9)*dp(369)
dp(1515) = dp(9)*p(351) + p(9)*dp(351)
dp(1516) = dp(9)*p(370) + p(9)*dp(370)
dp(1517) = dp(9)*p(371) + p(9)*dp(371)
dp(1518) = dp(9)*p(372) + p(9)*dp(372)
dp(1519) = dp(9)*p(429) + p(9)*dp(429)
dp(1520) = dp(9)*p(375) + p(9)*dp(375)
dp(1521) = dp(9)*p(435) + p(9)*dp(435)
dp(1522) = dp(9)*p(352) + p(9)*dp(352)
dp(1523) = dp(9)*p(376) + p(9)*dp(376)
dp(1524) = dp(9)*p(377) + p(9)*dp(377)
dp(1525) = dp(9)*p(436) + p(9)*dp(436)
dp(1526) = dp(9)*p(378) + p(9)*dp(378)
dp(1527) = dp(9)*p(437) + p(9)*dp(437)
dp(1528) = dp(9)*p(451) + p(9)*dp(451)
dp(1529) = dp(9)*p(382) + p(9)*dp(382)
dp(1530) = dp(9)*p(353) + p(9)*dp(353)
dp(1531) = dp(9)*p(383) + p(9)*dp(383)
dp(1532) = dp(9)*p(384) + p(9)*dp(384)
dp(1533) = dp(9)*p(452) + p(9)*dp(452)
dp(1534) = dp(9)*p(459) + p(9)*dp(459)
dp(1535) = dp(9)*p(386) + p(9)*dp(386)
dp(1536) = dp(9)*p(460) + p(9)*dp(460)
dp(1537) = dp(9)*p(387) + p(9)*dp(387)
dp(1538) = dp(9)*p(388) + p(9)*dp(388)
dp(1539) = dp(9)*p(461) + p(9)*dp(461)
dp(1540) = dp(9)*p(462) + p(9)*dp(462)
dp(1541) = dp(9)*p(463) + p(9)*dp(463)
dp(1542) = dp(9)*p(466) + p(9)*dp(466)
dp(1543) = dp(9)*p(467) + p(9)*dp(467)
dp(1544) = dp(9)*p(533) + p(9)*dp(533)
dp(1545) = dp(9)*p(476) + p(9)*dp(476)
dp(1546) = dp(9)*p(390) + p(9)*dp(390)
dp(1547) = dp(9)*p(391) + p(9)*dp(391)
dp(1548) = dp(9)*p(477) + p(9)*dp(477)
dp(1549) = dp(9)*p(481) + p(9)*dp(481)
dp(1550) = dp(9)*p(482) + p(9)*dp(482)
dp(1551) = dp(9)*p(483) + p(9)*dp(483)
dp(1552) = dp(9)*p(545) + p(9)*dp(545)
dp(1553) = dp(9)*p(549) + p(9)*dp(549)
dp(1554) = dp(9)*p(488) + p(9)*dp(488)
dp(1555) = dp(9)*p(489) + p(9)*dp(489)
dp(1556) = dp(9)*p(559) + p(9)*dp(559)
dp(1557) = dp(9)*p(565) + p(9)*dp(565)
dp(1558) = dp(9)*p(401) + p(9)*dp(401)
dp(1559) = dp(9)*p(402) + p(9)*dp(402)
dp(1560) = dp(9)*p(403) + p(9)*dp(403)
dp(1561) = dp(9)*p(504) + p(9)*dp(504)
dp(1562) = dp(9)*p(354) + p(9)*dp(354)
dp(1563) = dp(9)*p(355) + p(9)*dp(355)
dp(1564) = dp(9)*p(404) + p(9)*dp(404)
dp(1565) = dp(9)*p(356) + p(9)*dp(356)
dp(1566) = dp(9)*p(357) + p(9)*dp(357)
dp(1567) = dp(9)*p(405) + p(9)*dp(405)
dp(1568) = dp(9)*p(358) + p(9)*dp(358)
dp(1569) = dp(9)*p(406) + p(9)*dp(406)
dp(1570) = dp(9)*p(359) + p(9)*dp(359)
dp(1571) = dp(9)*p(407) + p(9)*dp(407)
dp(1572) = dp(9)*p(408) + p(9)*dp(408)
dp(1573) = dp(9)*p(505) + p(9)*dp(505)
dp(1574) = dp(14)*p(566) + p(14)*dp(566)
dp(1575) = dp(14)*p(567) + p(14)*dp(567)
dp(1576) = dp(14)*p(568) + p(14)*dp(568)
dp(1577) = dp(14)*p(569) + p(14)*dp(569)
dp(1578) = dp(9)*p(514) + p(9)*dp(514)
dp(1579) = dp(9)*p(416) + p(9)*dp(416)
dp(1580) = dp(9)*p(417) + p(9)*dp(417)
dp(1581) = dp(9)*p(418) + p(9)*dp(418)
dp(1582) = dp(9)*p(419) + p(9)*dp(419)
dp(1583) = dp(9)*p(515) + p(9)*dp(515)
dp(1584) = dp(9)*p(420) + p(9)*dp(420)
dp(1585) = dp(9)*p(362) + p(9)*dp(362)
dp(1586) = dp(9)*p(363) + p(9)*dp(363)
dp(1587) = dp(9)*p(364) + p(9)*dp(364)
dp(1588) = dp(9)*p(421) + p(9)*dp(421)
dp(1589) = dp(9)*p(422) + p(9)*dp(422)
dp(1590) = dp(9)*p(365) + p(9)*dp(365)
dp(1591) = dp(9)*p(366) + p(9)*dp(366)
dp(1592) = dp(9)*p(423) + p(9)*dp(423)
dp(1593) = dp(9)*p(424) + p(9)*dp(424)
dp(1594) = dp(9)*p(367) + p(9)*dp(367)
dp(1595) = dp(9)*p(425) + p(9)*dp(425)
dp(1596) = dp(9)*p(426) + p(9)*dp(426)
dp(1597) = dp(9)*p(427) + p(9)*dp(427)
dp(1598) = dp(9)*p(516) + p(9)*dp(516)
dp(1599) = dp(14)*p(495) + p(14)*dp(495)
dp(1600) = dp(14)*p(496) + p(14)*dp(496)
dp(1601) = dp(14)*p(497) + p(14)*dp(497)
dp(1602) = dp(14)*p(498) + p(14)*dp(498)
dp(1603) = dp(14)*p(499) + p(14)*dp(499)
dp(1604) = dp(14)*p(500) + p(14)*dp(500)
dp(1605) = dp(14)*p(501) + p(14)*dp(501)
dp(1606) = dp(14)*p(502) + p(14)*dp(502)
dp(1607) = dp(14)*p(503) + p(14)*dp(503)
dp(1608) = dp(9)*p(430) + p(9)*dp(430)
dp(1609) = dp(9)*p(431) + p(9)*dp(431)
dp(1610) = dp(9)*p(517) + p(9)*dp(517)
dp(1611) = dp(9)*p(373) + p(9)*dp(373)
dp(1612) = dp(9)*p(432) + p(9)*dp(432)
dp(1613) = dp(9)*p(374) + p(9)*dp(374)
dp(1614) = dp(9)*p(433) + p(9)*dp(433)
dp(1615) = dp(9)*p(434) + p(9)*dp(434)
dp(1616) = dp(9)*p(518) + p(9)*dp(518)
dp(1617) = dp(14)*p(401) + p(14)*dp(401)
dp(1618) = dp(14)*p(402) + p(14)*dp(402)
dp(1619) = dp(14)*p(403) + p(14)*dp(403)
dp(1620) = dp(14)*p(404) + p(14)*dp(404)
dp(1621) = dp(14)*p(405) + p(14)*dp(405)
dp(1622) = dp(14)*p(406) + p(14)*dp(406)
dp(1623) = dp(14)*p(407) + p(14)*dp(407)
dp(1624) = dp(14)*p(408) + p(14)*dp(408)
dp(1625) = dp(14)*p(360) + p(14)*dp(360)
dp(1626) = dp(14)*p(361) + p(14)*dp(361)
dp(1627) = dp(9)*p(525) + p(9)*dp(525)
dp(1628) = dp(9)*p(442) + p(9)*dp(442)
dp(1629) = dp(9)*p(443) + p(9)*dp(443)
dp(1630) = dp(9)*p(444) + p(9)*dp(444)
dp(1631) = dp(9)*p(526) + p(9)*dp(526)
dp(1632) = dp(9)*p(445) + p(9)*dp(445)
dp(1633) = dp(9)*p(379) + p(9)*dp(379)
dp(1634) = dp(9)*p(380) + p(9)*dp(380)
dp(1635) = dp(9)*p(446) + p(9)*dp(446)
dp(1636) = dp(9)*p(447) + p(9)*dp(447)
dp(1637) = dp(9)*p(381) + p(9)*dp(381)
dp(1638) = dp(9)*p(448) + p(9)*dp(448)
dp(1639) = dp(9)*p(449) + p(9)*dp(449)
dp(1640) = dp(9)*p(450) + p(9)*dp(450)
dp(1641) = dp(9)*p(527) + p(9)*dp(527)
dp(1642) = dp(14)*p(506) + p(14)*dp(506)
dp(1643) = dp(14)*p(507) + p(14)*dp(507)
dp(1644) = dp(14)*p(508) + p(14)*dp(508)
dp(1645) = dp(14)*p(509) + p(14)*dp(509)
dp(1646) = dp(14)*p(510) + p(14)*dp(510)
dp(1647) = dp(14)*p(511) + p(14)*dp(511)
dp(1648) = dp(14)*p(512) + p(14)*dp(512)
dp(1649) = dp(14)*p(513) + p(14)*dp(513)
dp(1650) = dp(9)*p(528) + p(9)*dp(528)
dp(1651) = dp(9)*p(453) + p(9)*dp(453)
dp(1652) = dp(9)*p(454) + p(9)*dp(454)
dp(1653) = dp(9)*p(529) + p(9)*dp(529)
dp(1654) = dp(9)*p(455) + p(9)*dp(455)
dp(1655) = dp(9)*p(385) + p(9)*dp(385)
dp(1656) = dp(9)*p(456) + p(9)*dp(456)
dp(1657) = dp(9)*p(457) + p(9)*dp(457)
dp(1658) = dp(9)*p(458) + p(9)*dp(458)
dp(1659) = dp(9)*p(530) + p(9)*dp(530)
dp(1660) = dp(14)*p(416) + p(14)*dp(416)
dp(1661) = dp(14)*p(417) + p(14)*dp(417)
dp(1662) = dp(14)*p(418) + p(14)*dp(418)
dp(1663) = dp(14)*p(419) + p(14)*dp(419)
dp(1664) = dp(14)*p(420) + p(14)*dp(420)
dp(1665) = dp(14)*p(421) + p(14)*dp(421)
dp(1666) = dp(14)*p(422) + p(14)*dp(422)
dp(1667) = dp(14)*p(423) + p(14)*dp(423)
dp(1668) = dp(14)*p(424) + p(14)*dp(424)
dp(1669) = dp(14)*p(425) + p(14)*dp(425)
dp(1670) = dp(14)*p(426) + p(14)*dp(426)
dp(1671) = dp(14)*p(427) + p(14)*dp(427)
dp(1672) = dp(14)*p(368) + p(14)*dp(368)
dp(1673) = dp(14)*p(369) + p(14)*dp(369)
dp(1674) = dp(14)*p(370) + p(14)*dp(370)
dp(1675) = dp(14)*p(371) + p(14)*dp(371)
dp(1676) = dp(14)*p(372) + p(14)*dp(372)
dp(1677) = dp(9)*p(464) + p(9)*dp(464)
dp(1678) = dp(9)*p(531) + p(9)*dp(531)
dp(1679) = dp(9)*p(465) + p(9)*dp(465)
dp(1680) = dp(9)*p(532) + p(9)*dp(532)
dp(1681) = dp(14)*p(430) + p(14)*dp(430)
dp(1682) = dp(14)*p(431) + p(14)*dp(431)
dp(1683) = dp(14)*p(432) + p(14)*dp(432)
dp(1684) = dp(14)*p(433) + p(14)*dp(433)
dp(1685) = dp(14)*p(434) + p(14)*dp(434)
dp(1686) = dp(14)*p(375) + p(14)*dp(375)
dp(1687) = dp(14)*p(376) + p(14)*dp(376)
dp(1688) = dp(14)*p(377) + p(14)*dp(377)
dp(1689) = dp(14)*p(378) + p(14)*dp(378)
dp(1690) = dp(9)*p(539) + p(9)*dp(539)
dp(1691) = dp(9)*p(470) + p(9)*dp(470)
dp(1692) = dp(9)*p(471) + p(9)*dp(471)
dp(1693) = dp(9)*p(540) + p(9)*dp(540)
dp(1694) = dp(9)*p(472) + p(9)*dp(472)
dp(1695) = dp(9)*p(389) + p(9)*dp(389)
dp(1696) = dp(9)*p(473) + p(9)*dp(473)
dp(1697) = dp(9)*p(474) + p(9)*dp(474)
dp(1698) = dp(9)*p(475) + p(9)*dp(475)
dp(1699) = dp(9)*p(541) + p(9)*dp(541)
dp(1700) = dp(14)*p(519) + p(14)*dp(519)
dp(1701) = dp(14)*p(520) + p(14)*dp(520)
dp(1702) = dp(14)*p(521) + p(14)*dp(521)
dp(1703) = dp(14)*p(522) + p(14)*dp(522)
dp(1704) = dp(14)*p(523) + p(14)*dp(523)
dp(1705) = dp(14)*p(524) + p(14)*dp(524)
dp(1706) = dp(9)*p(542) + p(9)*dp(542)
dp(1707) = dp(9)*p(478) + p(9)*dp(478)
dp(1708) = dp(9)*p(543) + p(9)*dp(543)
dp(1709) = dp(9)*p(479) + p(9)*dp(479)
dp(1710) = dp(9)*p(480) + p(9)*dp(480)
dp(1711) = dp(9)*p(544) + p(9)*dp(544)
dp(1712) = dp(14)*p(442) + p(14)*dp(442)
dp(1713) = dp(14)*p(443) + p(14)*dp(443)
dp(1714) = dp(14)*p(444) + p(14)*dp(444)
dp(1715) = dp(14)*p(445) + p(14)*dp(445)
dp(1716) = dp(14)*p(446) + p(14)*dp(446)
dp(1717) = dp(14)*p(447) + p(14)*dp(447)
dp(1718) = dp(14)*p(448) + p(14)*dp(448)
dp(1719) = dp(14)*p(449) + p(14)*dp(449)
dp(1720) = dp(14)*p(450) + p(14)*dp(450)
dp(1721) = dp(14)*p(382) + p(14)*dp(382)
dp(1722) = dp(14)*p(383) + p(14)*dp(383)
dp(1723) = dp(14)*p(384) + p(14)*dp(384)
dp(1724) = dp(9)*p(546) + p(9)*dp(546)
dp(1725) = dp(9)*p(547) + p(9)*dp(547)
dp(1726) = dp(9)*p(548) + p(9)*dp(548)
dp(1727) = dp(14)*p(453) + p(14)*dp(453)
dp(1728) = dp(14)*p(454) + p(14)*dp(454)
dp(1729) = dp(14)*p(455) + p(14)*dp(455)
dp(1730) = dp(14)*p(456) + p(14)*dp(456)
dp(1731) = dp(14)*p(457) + p(14)*dp(457)
dp(1732) = dp(14)*p(458) + p(14)*dp(458)
dp(1733) = dp(14)*p(386) + p(14)*dp(386)
dp(1734) = dp(14)*p(387) + p(14)*dp(387)
dp(1735) = dp(14)*p(388) + p(14)*dp(388)
dp(1736) = dp(9)*p(578) + p(9)*dp(578)
dp(1737) = dp(14)*p(464) + p(14)*dp(464)
dp(1738) = dp(14)*p(465) + p(14)*dp(465)
dp(1739) = dp(9)*p(553) + p(9)*dp(553)
dp(1740) = dp(9)*p(485) + p(9)*dp(485)
dp(1741) = dp(9)*p(554) + p(9)*dp(554)
dp(1742) = dp(9)*p(486) + p(9)*dp(486)
dp(1743) = dp(9)*p(487) + p(9)*dp(487)
dp(1744) = dp(9)*p(555) + p(9)*dp(555)
dp(1745) = dp(14)*p(534) + p(14)*dp(534)
dp(1746) = dp(14)*p(535) + p(14)*dp(535)
dp(1747) = dp(14)*p(536) + p(14)*dp(536)
dp(1748) = dp(14)*p(537) + p(14)*dp(537)
dp(1749) = dp(14)*p(538) + p(14)*dp(538)
dp(1750) = dp(9)*p(556) + p(9)*dp(556)
dp(1751) = dp(9)*p(557) + p(9)*dp(557)
dp(1752) = dp(9)*p(558) + p(9)*dp(558)
dp(1753) = dp(14)*p(470) + p(14)*dp(470)
dp(1754) = dp(14)*p(471) + p(14)*dp(471)
dp(1755) = dp(14)*p(472) + p(14)*dp(472)
dp(1756) = dp(14)*p(473) + p(14)*dp(473)
dp(1757) = dp(14)*p(474) + p(14)*dp(474)
dp(1758) = dp(14)*p(475) + p(14)*dp(475)
dp(1759) = dp(14)*p(390) + p(14)*dp(390)
dp(1760) = dp(14)*p(391) + p(14)*dp(391)
dp(1761) = dp(9)*p(581) + p(9)*dp(581)
dp(1762) = dp(14)*p(478) + p(14)*dp(478)
dp(1763) = dp(14)*p(479) + p(14)*dp(479)
dp(1764) = dp(14)*p(480) + p(14)*dp(480)
dp(1765) = dp(9)*p(562) + p(9)*dp(562)
dp(1766) = dp(9)*p(563) + p(9)*dp(563)
dp(1767) = dp(9)*p(564) + p(9)*dp(564)
dp(1768) = dp(14)*p(550) + p(14)*dp(550)
dp(1769) = dp(14)*p(551) + p(14)*dp(551)
dp(1770) = dp(14)*p(552) + p(14)*dp(552)
dp(1771) = dp(9)*p(584) + p(9)*dp(584)
dp(1772) = dp(14)*p(485) + p(14)*dp(485)
dp(1773) = dp(14)*p(486) + p(14)*dp(486)
dp(1774) = dp(14)*p(487) + p(14)*dp(487)
dp(1775) = dp(9)*p(587) + p(9)*dp(587)
dp(1776) = dp(14)*p(560) + p(14)*dp(560)
dp(1777) = dp(14)*p(561) + p(14)*dp(561)
dp(1778) = dp(9)*p(570) + p(9)*dp(570)
dp(1779) = dp(9)*p(495) + p(9)*dp(495)
dp(1780) = dp(9)*p(392) + p(9)*dp(392)
dp(1781) = dp(9)*p(496) + p(9)*dp(496)
dp(1782) = dp(9)*p(393) + p(9)*dp(393)
dp(1783) = dp(9)*p(394) + p(9)*dp(394)
dp(1784) = dp(9)*p(497) + p(9)*dp(497)
dp(1785) = dp(9)*p(395) + p(9)*dp(395)
dp(1786) = dp(9)*p(396) + p(9)*dp(396)
dp(1787) = dp(9)*p(397) + p(9)*dp(397)
dp(1788) = dp(9)*p(498) + p(9)*dp(498)
dp(1789) = dp(9)*p(398) + p(9)*dp(398)
dp(1790) = dp(9)*p(399) + p(9)*dp(399)
dp(1791) = dp(9)*p(499) + p(9)*dp(499)
dp(1792) = dp(9)*p(500) + p(9)*dp(500)
dp(1793) = dp(9)*p(400) + p(9)*dp(400)
dp(1794) = dp(9)*p(501) + p(9)*dp(501)
dp(1795) = dp(9)*p(502) + p(9)*dp(502)
dp(1796) = dp(9)*p(503) + p(9)*dp(503)
dp(1797) = dp(9)*p(571) + p(9)*dp(571)
dp(1798) = dp(1)*p(1136) + p(1)*dp(1136) - dp(1558)
dp(1799) = dp(21)*p(401) + p(21)*dp(401) - dp(1565)
dp(1800) = dp(101)*p(114) + p(101)*dp(114) - dp(1570)
dp(1801) = dp(24)*p(346) + p(24)*dp(346) - dp(1573)
dp(1802) = dp(3)*p(850) + p(3)*dp(850) - dp(1561) - dp(1801)
dp(1803) = dp(1)*p(1140) + p(1)*dp(1140) - dp(1561) - dp(1802)
dp(1804) = dp(1)*p(1141) + p(1)*dp(1141) - dp(1564) - dp(1803)
dp(1805) = dp(1)*p(1142) + p(1)*dp(1142) - dp(1567) - dp(1804)
dp(1806) = dp(13)*p(566) + p(13)*dp(566) - dp(1558)
dp(1807) = dp(73)*p(178) + p(73)*dp(178) - dp(1565)
dp(1808) = dp(39)*p(285) + p(39)*dp(285) - dp(1570)
dp(1809) = dp(3)*p(851) + p(3)*dp(851) - dp(1573)
dp(1810) = dp(14)*p(588) + p(14)*dp(588)
dp(1811) = dp(9)*p(572) + p(9)*dp(572)
dp(1812) = dp(9)*p(506) + p(9)*dp(506)
dp(1813) = dp(9)*p(410) + p(9)*dp(410)
dp(1814) = dp(9)*p(507) + p(9)*dp(507)
dp(1815) = dp(9)*p(411) + p(9)*dp(411)
dp(1816) = dp(9)*p(412) + p(9)*dp(412)
dp(1817) = dp(9)*p(508) + p(9)*dp(508)
dp(1818) = dp(9)*p(413) + p(9)*dp(413)
dp(1819) = dp(9)*p(414) + p(9)*dp(414)
dp(1820) = dp(9)*p(509) + p(9)*dp(509)
dp(1821) = dp(9)*p(510) + p(9)*dp(510)
dp(1822) = dp(9)*p(415) + p(9)*dp(415)
dp(1823) = dp(9)*p(511) + p(9)*dp(511)
dp(1824) = dp(9)*p(512) + p(9)*dp(512)
dp(1825) = dp(9)*p(513) + p(9)*dp(513)
dp(1826) = dp(9)*p(573) + p(9)*dp(573)
dp(1827) = dp(24)*p(331) + p(24)*dp(331) - dp(1510)
dp(1828) = dp(91)*p(114) + p(91)*dp(114) - dp(1507)
dp(1829) = dp(16)*p(401) + p(16)*dp(401) - dp(1506)
dp(1830) = dp(2)*p(1136) + p(2)*dp(1136) - dp(1574)
dp(1831) = dp(1)*p(1163) + p(1)*dp(1163) - dp(1581) - dp(1830)
dp(1832) = dp(1)*p(1164) + p(1)*dp(1164) - dp(1582) - dp(1831)
dp(1833) = dp(1)*p(1165) + p(1)*dp(1165) - dp(1583) - dp(1832)
dp(1834) = dp(13)*p(570) + p(13)*dp(570) - dp(1810)
dp(1835) = dp(2)*p(1140) + p(2)*dp(1140) - dp(1577) - dp(1834)
dp(1836) = dp(32)*p(331) + p(32)*dp(331) - dp(1810)
dp(1837) = dp(1)*p(1167) + p(1)*dp(1167) - dp(1583) - dp(1835)
dp(1838) = dp(73)*p(181) + p(73)*dp(181) - dp(1810)
dp(1839) = dp(1)*p(1169) + p(1)*dp(1169) - dp(1588) - dp(1837)
dp(1840) = dp(91)*p(150) + p(91)*dp(150) - dp(1810)
dp(1841) = dp(6)*p(843) + p(6)*dp(843) - dp(1574)
dp(1842) = dp(41)*p(285) + p(41)*dp(285) - dp(1810)
dp(1843) = dp(32)*p(279) + p(32)*dp(279) - dp(1506)
dp(1844) = dp(16)*p(505) + p(16)*dp(505) - dp(1810)
dp(1845) = dp(69)*p(150) + p(69)*dp(150) - dp(1507)
dp(1846) = dp(4)*p(851) + p(4)*dp(851) - dp(1810)
dp(1847) = dp(11)*p(505) + p(11)*dp(505) - dp(1510)
dp(1848) = dp(14)*p(570) + p(14)*dp(570)
dp(1849) = dp(14)*p(571) + p(14)*dp(571)
dp(1850) = dp(1)*p(1180) + p(1)*dp(1180) - dp(1608)
dp(1851) = dp(21)*p(430) + p(21)*dp(430) - dp(1613)
dp(1852) = dp(101)*p(130) + p(101)*dp(130) - dp(1616)
dp(1853) = dp(3)*p(864) + p(3)*dp(864) - dp(1610) - dp(1852)
dp(1854) = dp(1)*p(1183) + p(1)*dp(1183) - dp(1610) - dp(1853)
dp(1855) = dp(1)*p(1184) + p(1)*dp(1184) - dp(1612) - dp(1854)
dp(1856) = dp(81)*p(178) + p(81)*dp(178) - dp(1608)
dp(1857) = dp(39)*p(296) + p(39)*dp(296) - dp(1613)
dp(1858) = dp(3)*p(865) + p(3)*dp(865) - dp(1616)
dp(1859) = dp(14)*p(504) + p(14)*dp(504)
dp(1860) = dp(14)*p(505) + p(14)*dp(505)
dp(1861) = dp(14)*p(409) + p(14)*dp(409)
dp(1862) = dp(9)*p(574) + p(9)*dp(574)
dp(1863) = dp(9)*p(519) + p(9)*dp(519)
dp(1864) = dp(9)*p(438) + p(9)*dp(438)
dp(1865) = dp(9)*p(520) + p(9)*dp(520)
dp(1866) = dp(9)*p(439) + p(9)*dp(439)
dp(1867) = dp(9)*p(440) + p(9)*dp(440)
dp(1868) = dp(9)*p(521) + p(9)*dp(521)
dp(1869) = dp(9)*p(441) + p(9)*dp(441)
dp(1870) = dp(9)*p(522) + p(9)*dp(522)
dp(1871) = dp(9)*p(523) + p(9)*dp(523)
dp(1872) = dp(9)*p(524) + p(9)*dp(524)
dp(1873) = dp(9)*p(575) + p(9)*dp(575)
dp(1874) = dp(24)*p(333) + p(24)*dp(333) - dp(1511)
dp(1875) = dp(93)*p(114) + p(93)*dp(114) - dp(1513)
dp(1876) = dp(18)*p(401) + p(18)*dp(401) - dp(1516)
dp(1877) = dp(1)*p(1202) + p(1)*dp(1202) - dp(1629) - dp(1876)
dp(1878) = dp(1)*p(1203) + p(1)*dp(1203) - dp(1630) - dp(1877)
dp(1879) = dp(1)*p(1204) + p(1)*dp(1204) - dp(1631) - dp(1878)
dp(1880) = dp(13)*p(572) + p(13)*dp(572) - dp(1848)
dp(1881) = dp(2)*p(1167) + p(2)*dp(1167) - dp(1607) - dp(1853)
dp(1882) = dp(32)*p(333) + p(32)*dp(333) - dp(1848)
dp(1883) = dp(1)*p(1206) + p(1)*dp(1206) - dp(1631) - dp(1881)
dp(1884) = dp(73)*p(183) + p(73)*dp(183) - dp(1848)
dp(1885) = dp(1)*p(1208) + p(1)*dp(1208) - dp(1635) - dp(1883)
dp(1886) = dp(93)*p(150) + p(93)*dp(150) - dp(1848)
dp(1887) = dp(13)*p(509) + p(13)*dp(509) - dp(1516)
dp(1888) = dp(43)*p(285) + p(43)*dp(285) - dp(1848)
dp(1889) = dp(73)*p(153) + p(73)*dp(153) - dp(1513)
dp(1890) = dp(18)*p(505) + p(18)*dp(505) - dp(1848)
dp(1891) = dp(34)*p(285) + p(34)*dp(285) - dp(1511)
dp(1892) = dp(14)*p(572) + p(14)*dp(572)
dp(1893) = dp(14)*p(573) + p(14)*dp(573)
dp(1894) = dp(91)*p(130) + p(91)*dp(130) - dp(1525)
dp(1895) = dp(16)*p(430) + p(16)*dp(430) - dp(1522)
dp(1896) = dp(2)*p(1180) + p(2)*dp(1180) - dp(1617)
dp(1897) = dp(1)*p(1219) + p(1)*dp(1219) - dp(1652) - dp(1896)
dp(1898) = dp(1)*p(1220) + p(1)*dp(1220) - dp(1653) - dp(1897)
dp(1899) = dp(81)*p(181) + p(81)*dp(181) - dp(1860)
dp(1900) = dp(2)*p(1183) + p(2)*dp(1183) - dp(1620) - dp(1899)
dp(1901) = dp(91)*p(160) + p(91)*dp(160) - dp(1860)
dp(1902) = dp(1)*p(1222) + p(1)*dp(1222) - dp(1653) - dp(1900)
dp(1903) = dp(41)*p(296) + p(41)*dp(296) - dp(1860)
dp(1904) = dp(45)*p(279) + p(45)*dp(279) - dp(1617)
dp(1905) = dp(16)*p(518) + p(16)*dp(518) - dp(1860)
dp(1906) = dp(69)*p(160) + p(69)*dp(160) - dp(1522)
dp(1907) = dp(4)*p(865) + p(4)*dp(865) - dp(1860)
dp(1908) = dp(11)*p(518) + p(11)*dp(518) - dp(1525)
dp(1909) = dp(14)*p(514) + p(14)*dp(514)
dp(1910) = dp(14)*p(515) + p(14)*dp(515)
dp(1911) = dp(14)*p(516) + p(14)*dp(516)
dp(1912) = dp(14)*p(428) + p(14)*dp(428)
dp(1913) = dp(14)*p(429) + p(14)*dp(429)
dp(1914) = dp(1)*p(1234) + p(1)*dp(1234) - dp(1677)
dp(1915) = dp(21)*p(464) + p(21)*dp(464) - dp(1680)
dp(1916) = dp(3)*p(880) + p(3)*dp(880) - dp(1678) - dp(1915)
dp(1917) = dp(1)*p(1236) + p(1)*dp(1236) - dp(1678) - dp(1916)
dp(1918) = dp(39)*p(310) + p(39)*dp(310) - dp(1677)
dp(1919) = dp(3)*p(881) + p(3)*dp(881) - dp(1680)
dp(1920) = dp(14)*p(517) + p(14)*dp(517)
dp(1921) = dp(14)*p(518) + p(14)*dp(518)
dp(1922) = dp(14)*p(435) + p(14)*dp(435)
dp(1923) = dp(14)*p(436) + p(14)*dp(436)
dp(1924) = dp(14)*p(437) + p(14)*dp(437)
dp(1925) = dp(9)*p(576) + p(9)*dp(576)
dp(1926) = dp(9)*p(534) + p(9)*dp(534)
dp(1927) = dp(9)*p(468) + p(9)*dp(468)
dp(1928) = dp(9)*p(535) + p(9)*dp(535)
dp(1929) = dp(9)*p(469) + p(9)*dp(469)
dp(1930) = dp(9)*p(536) + p(9)*dp(536)
dp(1931) = dp(9)*p(537) + p(9)*dp(537)
dp(1932) = dp(9)*p(538) + p(9)*dp(538)
dp(1933) = dp(9)*p(577) + p(9)*dp(577)
dp(1934) = dp(24)*p(335) + p(24)*dp(335) - dp(1528)
dp(1935) = dp(95)*p(114) + p(95)*dp(114) - dp(1530)
dp(1936) = dp(22)*p(401) + p(22)*dp(401) - dp(1645)
dp(1937) = dp(1)*p(1252) + p(1)*dp(1252) - dp(1692) - dp(1936)
dp(1938) = dp(1)*p(1253) + p(1)*dp(1253) - dp(1693) - dp(1937)
dp(1939) = dp(13)*p(574) + p(13)*dp(574) - dp(1892)
dp(1940) = dp(2)*p(1206) + p(2)*dp(1206) - dp(1649) - dp(1900)
dp(1941) = dp(32)*p(335) + p(32)*dp(335) - dp(1892)
dp(1942) = dp(1)*p(1255) + p(1)*dp(1255) - dp(1693) - dp(1940)
dp(1943) = dp(73)*p(185) + p(73)*dp(185) - dp(1892)
dp(1944) = dp(6)*p(869) + p(6)*dp(869) - dp(1645)
dp(1945) = dp(95)*p(150) + p(95)*dp(150) - dp(1892)
dp(1946) = dp(32)*p(299) + p(32)*dp(299) - dp(1530)
dp(1947) = dp(46)*p(285) + p(46)*dp(285) - dp(1892)
dp(1948) = dp(84)*p(150) + p(84)*dp(150) - dp(1528)
dp(1949) = dp(14)*p(574) + p(14)*dp(574)
dp(1950) = dp(14)*p(575) + p(14)*dp(575)
dp(1951) = dp(93)*p(130) + p(93)*dp(130) - dp(1527)
dp(1952) = dp(18)*p(430) + p(18)*dp(430) - dp(1526)
dp(1953) = dp(1)*p(1265) + p(1)*dp(1265) - dp(1707) - dp(1952)
dp(1954) = dp(1)*p(1266) + p(1)*dp(1266) - dp(1708) - dp(1953)
dp(1955) = dp(81)*p(183) + p(81)*dp(183) - dp(1861)
dp(1956) = dp(2)*p(1222) + p(2)*dp(1222) - dp(1665) - dp(1916)
dp(1957) = dp(93)*p(160) + p(93)*dp(160) - dp(1861)
dp(1958) = dp(1)*p(1268) + p(1)*dp(1268) - dp(1708) - dp(1956)
dp(1959) = dp(43)*p(296) + p(43)*dp(296) - dp(1861)
dp(1960) = dp(81)*p(153) + p(81)*dp(153) - dp(1526)
dp(1961) = dp(18)*p(518) + p(18)*dp(518) - dp(1861)
dp(1962) = dp(34)*p(296) + p(34)*dp(296) - dp(1527)
dp(1963) = dp(14)*p(525) + p(14)*dp(525)
dp(1964) = dp(14)*p(526) + p(14)*dp(526)
dp(1965) = dp(14)*p(527) + p(14)*dp(527)
dp(1966) = dp(14)*p(451) + p(14)*dp(451)
dp(1967) = dp(14)*p(452) + p(14)*dp(452)
dp(1968) = dp(16)*p(464) + p(16)*dp(464) - dp(1543)
dp(1969) = dp(2)*p(1234) + p(2)*dp(1234) - dp(1681)
dp(1970) = dp(1)*p(1279) + p(1)*dp(1279) - dp(1725) - dp(1969)
dp(1971) = dp(41)*p(310) + p(41)*dp(310) - dp(1921)
dp(1972) = dp(2)*p(1236) + p(2)*dp(1236) - dp(1683) - dp(1971)
dp(1973) = dp(16)*p(532) + p(16)*dp(532) - dp(1921)
dp(1974) = dp(69)*p(187) + p(69)*dp(187) - dp(1681)
dp(1975) = dp(4)*p(881) + p(4)*dp(881) - dp(1921)
dp(1976) = dp(11)*p(532) + p(11)*dp(532) - dp(1543)
dp(1977) = dp(14)*p(528) + p(14)*dp(528)
dp(1978) = dp(14)*p(529) + p(14)*dp(529)
dp(1979) = dp(14)*p(530) + p(14)*dp(530)
dp(1980) = dp(14)*p(459) + p(14)*dp(459)
dp(1981) = dp(14)*p(460) + p(14)*dp(460)
dp(1982) = dp(14)*p(461) + p(14)*dp(461)
dp(1983) = dp(14)*p(462) + p(14)*dp(462)
dp(1984) = dp(14)*p(463) + p(14)*dp(463)
dp(1985) = dp(1)*p(1290) + p(1)*dp(1290) - dp(1736)
dp(1986) = dp(4)*p(806) + p(4)*dp(806) - dp(1761) - dp(1732)
dp(1987) = dp(3)*p(900) + p(3)*dp(900) - dp(1736)
dp(1988) = dp(14)*p(531) + p(14)*dp(531)
dp(1989) = dp(14)*p(532) + p(14)*dp(532)
dp(1990) = dp(14)*p(466) + p(14)*dp(466)
dp(1991) = dp(14)*p(467) + p(14)*dp(467)
dp(1992) = dp(9)*p(579) + p(9)*dp(579)
dp(1993) = dp(9)*p(550) + p(9)*dp(550)
dp(1994) = dp(9)*p(484) + p(9)*dp(484)
dp(1995) = dp(9)*p(551) + p(9)*dp(551)
dp(1996) = dp(9)*p(552) + p(9)*dp(552)
dp(1997) = dp(9)*p(580) + p(9)*dp(580)
dp(1998) = dp(24)*p(337) + p(24)*dp(337) - dp(1545)
dp(1999) = dp(98)*p(114) + p(98)*dp(114) - dp(1547)
dp(2000) = dp(1)*p(1299) + p(1)*dp(1299) - dp(1740) - dp(1999)
dp(2001) = dp(1)*p(1300) + p(1)*dp(1300) - dp(1741) - dp(2000)
dp(2002) = dp(13)*p(576) + p(13)*dp(576) - dp(1949)
dp(2003) = dp(2)*p(1255) + p(2)*dp(1255) - dp(1705) - dp(1956)
dp(2004) = dp(32)*p(337) + p(32)*dp(337) - dp(1949)
dp(2005) = dp(1)*p(1302) + p(1)*dp(1302) - dp(1741) - dp(2003)
dp(2006) = dp(73)*p(188) + p(73)*dp(188) - dp(1949)
dp(2007) = dp(13)*p(536) + p(13)*dp(536) - dp(1547)
dp(2008) = dp(98)*p(150) + p(98)*dp(150) - dp(1949)
dp(2009) = dp(73)*p(173) + p(73)*dp(173) - dp(1545)
dp(2010) = dp(14)*p(576) + p(14)*dp(576)
dp(2011) = dp(14)*p(577) + p(14)*dp(577)
dp(2012) = dp(95)*p(130) + p(95)*dp(130) - dp(1540)
dp(2013) = dp(22)*p(430) + p(22)*dp(430) - dp(1674)
dp(2014) = dp(1)*p(1310) + p(1)*dp(1310) - dp(1751) - dp(2013)
dp(2015) = dp(81)*p(185) + p(81)*dp(185) - dp(1912)
dp(2016) = dp(2)*p(1268) + p(2)*dp(1268) - dp(1716) - dp(1972)
dp(2017) = dp(95)*p(160) + p(95)*dp(160) - dp(1912)
dp(2018) = dp(45)*p(299) + p(45)*dp(299) - dp(1674)
dp(2019) = dp(46)*p(296) + p(46)*dp(296) - dp(1912)
dp(2020) = dp(84)*p(160) + p(84)*dp(160) - dp(1540)
dp(2021) = dp(14)*p(539) + p(14)*dp(539)
dp(2022) = dp(14)*p(540) + p(14)*dp(540)
dp(2023) = dp(14)*p(541) + p(14)*dp(541)
dp(2024) = dp(14)*p(476) + p(14)*dp(476)
dp(2025) = dp(14)*p(477) + p(14)*dp(477)
dp(2026) = dp(18)*p(464) + p(18)*dp(464) - dp(1544)
dp(2027) = dp(1)*p(1320) + p(1)*dp(1320) - dp(1761) - dp(2026)
dp(2028) = dp(43)*p(310) + p(43)*dp(310) - dp(1923)
dp(2029) = dp(2)*p(1281) + p(2)*dp(1281) - dp(1730) - dp(1986)
dp(2030) = dp(18)*p(532) + p(18)*dp(532) - dp(1923)
dp(2031) = dp(34)*p(310) + p(34)*dp(310) - dp(1544)
dp(2032) = dp(14)*p(542) + p(14)*dp(542)
dp(2033) = dp(14)*p(543) + p(14)*dp(543)
dp(2034) = dp(14)*p(544) + p(14)*dp(544)
dp(2035) = dp(14)*p(481) + p(14)*dp(481)
dp(2036) = dp(14)*p(482) + p(14)*dp(482)
dp(2037) = dp(14)*p(483) + p(14)*dp(483)
dp(2038) = dp(2)*p(1290) + p(2)*dp(1290) - dp(1737)
dp(2039) = dp(4)*p(900) + p(4)*dp(900) - dp(1989)
dp(2040) = dp(11)*p(578) + p(11)*dp(578) - dp(1737)
dp(2041) = dp(14)*p(546) + p(14)*dp(546)
dp(2042) = dp(14)*p(547) + p(14)*dp(547)
dp(2043) = dp(14)*p(548) + p(14)*dp(548)
dp(2044) = dp(9)*p(582) + p(9)*dp(582)
dp(2045) = dp(9)*p(560) + p(9)*dp(560)
dp(2046) = dp(9)*p(561) + p(9)*dp(561)
dp(2047) = dp(9)*p(583) + p(9)*dp(583)
dp(2048) = dp(24)*p(340) + p(24)*dp(340) - dp(1554)
dp(2049) = dp(102)*p(114) + p(102)*dp(114) - dp(1747)
dp(2050) = dp(1)*p(1329) + p(1)*dp(1329) - dp(1766) - dp(2049)
dp(2051) = dp(13)*p(579) + p(13)*dp(579) - dp(2010)
dp(2052) = dp(2)*p(1302) + p(2)*dp(1302) - dp(1749) - dp(2016)
dp(2053) = dp(32)*p(340) + p(32)*dp(340) - dp(2010)
dp(2054) = dp(6)*p(904) + p(6)*dp(904) - dp(1747)
dp(2055) = dp(73)*p(191) + p(73)*dp(191) - dp(2010)
dp(2056) = dp(32)*p(323) + p(32)*dp(323) - dp(1554)
dp(2057) = dp(14)*p(579) + p(14)*dp(579)
dp(2058) = dp(14)*p(580) + p(14)*dp(580)
dp(2059) = dp(98)*p(130) + p(98)*dp(130) - dp(1552)
dp(2060) = dp(1)*p(1336) + p(1)*dp(1336) - dp(1771) - dp(2059)
dp(2061) = dp(81)*p(188) + p(81)*dp(188) - dp(1966)
dp(2062) = dp(2)*p(1312) + p(2)*dp(1312) - dp(1756) - dp(2029)
dp(2063) = dp(98)*p(160) + p(98)*dp(160) - dp(1966)
dp(2064) = dp(81)*p(173) + p(81)*dp(173) - dp(1552)
dp(2065) = dp(14)*p(553) + p(14)*dp(553)
dp(2066) = dp(14)*p(554) + p(14)*dp(554)
dp(2067) = dp(14)*p(555) + p(14)*dp(555)
dp(2068) = dp(14)*p(488) + p(14)*dp(488)
dp(2069) = dp(14)*p(489) + p(14)*dp(489)
dp(2070) = dp(22)*p(464) + p(22)*dp(464) - dp(1689)
dp(2071) = dp(46)*p(310) + p(46)*dp(310) - dp(1924)
dp(2072) = dp(84)*p(187) + p(84)*dp(187) - dp(1689)
dp(2073) = dp(14)*p(556) + p(14)*dp(556)
dp(2074) = dp(14)*p(557) + p(14)*dp(557)
dp(2075) = dp(14)*p(558) + p(14)*dp(558)
dp(2076) = dp(9)*p(585) + p(9)*dp(585)
dp(2077) = dp(9)*p(586) + p(9)*dp(586)
dp(2078) = dp(24)*p(343) + p(24)*dp(343) - dp(1557)
dp(2079) = dp(1)*p(1343) + p(1)*dp(1343) - dp(1775) - dp(2078)
dp(2080) = dp(13)*p(582) + p(13)*dp(582) - dp(2057)
dp(2081) = dp(2)*p(1331) + p(2)*dp(1331) - dp(1770) - dp(2062)
dp(2082) = dp(32)*p(343) + p(32)*dp(343) - dp(2057)
dp(2083) = dp(13)*p(561) + p(13)*dp(561) - dp(1557)
dp(2084) = dp(14)*p(582) + p(14)*dp(582)
dp(2085) = dp(14)*p(583) + p(14)*dp(583)
dp(2086) = dp(102)*p(130) + p(102)*dp(130) - dp(1760)
dp(2087) = dp(81)*p(191) + p(81)*dp(191) - dp(2024)
dp(2088) = dp(45)*p(323) + p(45)*dp(323) - dp(1760)
dp(2089) = dp(14)*p(562) + p(14)*dp(562)
dp(2090) = dp(14)*p(563) + p(14)*dp(563)
dp(2091) = dp(14)*p(564) + p(14)*dp(564)
dp(2092) = dp(9)*p(589) + p(9)*dp(589)
dp(2093) = dp(24)*p(347) + p(24)*dp(347) - dp(1777)
dp(2094) = dp(13)*p(585) + p(13)*dp(585) - dp(2084)
dp(2095) = dp(6)*p(929) + p(6)*dp(929) - dp(1777)
dp(2096) = dp(14)*p(585) + p(14)*dp(585)
dp(2097) = dp(14)*p(586) + p(14)*dp(586)
dp(2098) = dp(9)*p(490) + p(9)*dp(490)
dp(2099) = dp(9)*p(491) + p(9)*dp(491)
dp(2100) = dp(9)*p(492) + p(9)*dp(492)
dp(2101) = dp(9)*p(566) + p(9)*dp(566)
dp(2102) = dp(9)*p(493) + p(9)*dp(493)
dp(2103) = dp(9)*p(567) + p(9)*dp(567)
dp(2104) = dp(9)*p(494) + p(9)*dp(494)
dp(2105) = dp(9)*p(568) + p(9)*dp(568)
dp(2106) = dp(9)*p(569) + p(9)*dp(569)
dp(2107) = dp(9)*p(588) + p(9)*dp(588)
dp(2108) = dp(3)*p(938) + p(3)*dp(938) - dp(1778)
dp(2109) = dp(39)*p(331) + p(39)*dp(331) - dp(1780)
dp(2110) = dp(91)*p(178) + p(91)*dp(178) - dp(1783)
dp(2111) = dp(16)*p(566) + p(16)*dp(566) - dp(1787)
dp(2112) = dp(1)*p(1359) + p(1)*dp(1359) - dp(1784) - dp(2111)
dp(2113) = dp(2)*p(1476) + p(2)*dp(1476) - dp(2112)
dp(2114) = dp(1)*p(1360) + p(1)*dp(1360) - dp(1788) - dp(2112)
dp(2115) = dp(7)*p(843) + p(7)*dp(843) - dp(1787)
dp(2116) = dp(1)*p(1362) + p(1)*dp(1362) - dp(1792) - dp(2114)
dp(2117) = dp(49)*p(279) + p(49)*dp(279) - dp(1783)
dp(2118) = dp(1)*p(1364) + p(1)*dp(1364) - dp(1795) - dp(2116)
dp(2119) = dp(69)*p(194) + p(69)*dp(194) - dp(1780)
dp(2120) = dp(1)*p(1366) + p(1)*dp(1366) - dp(1797) - dp(2118)
dp(2121) = dp(11)*p(588) + p(11)*dp(588) - dp(1778)
dp(2122) = dp(1)*p(1368) + p(1)*dp(1368) - dp(1802) - dp(1801)
dp(2123) = dp(1)*p(1369) + p(1)*dp(1369) - dp(1809)
dp(2124) = dp(3)*p(940) + p(3)*dp(940) - dp(1811)
dp(2125) = dp(39)*p(333) + p(39)*dp(333) - dp(1813)
dp(2126) = dp(93)*p(178) + p(93)*dp(178) - dp(1816)
dp(2127) = dp(18)*p(566) + p(18)*dp(566) - dp(1820)
dp(2128) = dp(1)*p(1373) + p(1)*dp(1373) - dp(1817) - dp(2127)
dp(2129) = dp(1)*p(1374) + p(1)*dp(1374) - dp(1820)
dp(2130) = dp(1)*p(1375) + p(1)*dp(1375) - dp(1821) - dp(2128)
dp(2131) = dp(21)*p(509) + p(21)*dp(509) - dp(1816)
dp(2132) = dp(1)*p(1377) + p(1)*dp(1377) - dp(1824) - dp(2130)
dp(2133) = dp(101)*p(153) + p(101)*dp(153) - dp(1813)
dp(2134) = dp(1)*p(1379) + p(1)*dp(1379) - dp(1826) - dp(2132)
dp(2135) = dp(34)*p(346) + p(34)*dp(346) - dp(1811)
dp(2136) = dp(6)*p(938) + p(6)*dp(938) - dp(1810)
dp(2137) = dp(1)*p(1382) + p(1)*dp(1382) - dp(1835) - dp(1833)
dp(2138) = dp(2)*p(1369) + p(2)*dp(1369) - dp(1810)
dp(2139) = dp(1)*p(1384) + p(1)*dp(1384) - dp(1853) - dp(1852)
dp(2140) = dp(1)*p(1385) + p(1)*dp(1385) - dp(1858)
dp(2141) = dp(3)*p(942) + p(3)*dp(942) - dp(1862)
dp(2142) = dp(39)*p(335) + p(39)*dp(335) - dp(1864)
dp(2143) = dp(95)*p(178) + p(95)*dp(178) - dp(1867)
dp(2144) = dp(1)*p(1388) + p(1)*dp(1388) - dp(1865) - dp(2143)
dp(2145) = dp(2)*p(1374) + p(2)*dp(1374) - dp(1841)
dp(2146) = dp(1)*p(1389) + p(1)*dp(1389) - dp(1868) - dp(2144)
dp(2147) = dp(7)*p(869) + p(7)*dp(869) - dp(1867)
dp(2148) = dp(1)*p(1391) + p(1)*dp(1391) - dp(1871) - dp(2146)
dp(2149) = dp(49)*p(299) + p(49)*dp(299) - dp(1864)
dp(2150) = dp(1)*p(1393) + p(1)*dp(1393) - dp(1873) - dp(2148)
dp(2151) = dp(84)*p(194) + p(84)*dp(194) - dp(1862)
dp(2152) = dp(6)*p(940) + p(6)*dp(940) - dp(1848)
dp(2153) = dp(1)*p(1396) + p(1)*dp(1396) - dp(1881) - dp(1879)
dp(2154) = dp(8)*p(851) + p(8)*dp(851) - dp(1848)
dp(2155) = dp(45)*p(331) + p(45)*dp(331) - dp(1860)
dp(2156) = dp(1)*p(1399) + p(1)*dp(1399) - dp(1900) - dp(1898)
dp(2157) = dp(2)*p(1385) + p(2)*dp(1385) - dp(1860)
dp(2158) = dp(1)*p(1401) + p(1)*dp(1401) - dp(1916) - dp(1915)
dp(2159) = dp(1)*p(1402) + p(1)*dp(1402) - dp(1919)
dp(2160) = dp(3)*p(944) + p(3)*dp(944) - dp(1925)
dp(2161) = dp(39)*p(337) + p(39)*dp(337) - dp(1927)
dp(2162) = dp(98)*p(178) + p(98)*dp(178) - dp(1930)
dp(2163) = dp(1)*p(1405) + p(1)*dp(1405) - dp(1928) - dp(2162)
dp(2164) = dp(1)*p(1406) + p(1)*dp(1406) - dp(1930)
dp(2165) = dp(1)*p(1407) + p(1)*dp(1407) - dp(1931) - dp(2163)
dp(2166) = dp(21)*p(536) + p(21)*dp(536) - dp(1927)
dp(2167) = dp(1)*p(1409) + p(1)*dp(1409) - dp(1933) - dp(2165)
dp(2168) = dp(101)*p(173) + p(101)*dp(173) - dp(1925)
dp(2169) = dp(6)*p(942) + p(6)*dp(942) - dp(1892)
dp(2170) = dp(1)*p(1412) + p(1)*dp(1412) - dp(1940) - dp(1938)
dp(2171) = dp(22)*p(505) + p(22)*dp(505) - dp(1892)
dp(2172) = dp(45)*p(333) + p(45)*dp(333) - dp(1861)
dp(2173) = dp(1)*p(1415) + p(1)*dp(1415) - dp(1956) - dp(1954)
dp(2174) = dp(8)*p(865) + p(8)*dp(865) - dp(1861)
dp(2175) = dp(91)*p(187) + p(91)*dp(187) - dp(1921)
dp(2176) = dp(1)*p(1418) + p(1)*dp(1418) - dp(1972) - dp(1970)
dp(2177) = dp(2)*p(1402) + p(2)*dp(1402) - dp(1921)
dp(2178) = dp(1)*p(1421) + p(1)*dp(1421) - dp(1986) - dp(1985)
dp(2179) = dp(1)*p(1422) + p(1)*dp(1422) - dp(1987)
dp(2180) = dp(14)*p(533) + p(14)*dp(533)
dp(2181) = dp(3)*p(946) + p(3)*dp(946) - dp(1992)
dp(2182) = dp(39)*p(340) + p(39)*dp(340) - dp(1994)
dp(2183) = dp(1)*p(1425) + p(1)*dp(1425) - dp(1993) - dp(2182)
dp(2184) = dp(2)*p(1406) + p(2)*dp(1406) - dp(1944)
dp(2185) = dp(1)*p(1426) + p(1)*dp(1426) - dp(1995) - dp(2183)
dp(2186) = dp(7)*p(904) + p(7)*dp(904) - dp(1994)
dp(2187) = dp(1)*p(1428) + p(1)*dp(1428) - dp(1997) - dp(2185)
dp(2188) = dp(49)*p(323) + p(49)*dp(323) - dp(1992)
dp(2189) = dp(6)*p(944) + p(6)*dp(944) - dp(1949)
dp(2190) = dp(1)*p(1431) + p(1)*dp(1431) - dp(2003) - dp(2001)
dp(2191) = dp(50)*p(285) + p(50)*dp(285) - dp(1949)
dp(2192) = dp(45)*p(335) + p(45)*dp(335) - dp(1912)
dp(2193) = dp(1)*p(1434) + p(1)*dp(1434) - dp(2016) - dp(2014)
dp(2194) = dp(22)*p(518) + p(22)*dp(518) - dp(1912)
dp(2195) = dp(93)*p(187) + p(93)*dp(187) - dp(1923)
dp(2196) = dp(1)*p(1437) + p(1)*dp(1437) - dp(2029) - dp(2027)
dp(2197) = dp(8)*p(881) + p(8)*dp(881) - dp(1923)
dp(2198) = dp(14)*p(545) + p(14)*dp(545)
dp(2199) = dp(16)*p(578) + p(16)*dp(578) - dp(1989)
dp(2200) = dp(1)*p(1441) + p(1)*dp(1441) - dp(2040) - dp(2038)
dp(2201) = dp(2)*p(1422) + p(2)*dp(1422) - dp(1989)
dp(2202) = dp(14)*p(549) + p(14)*dp(549)
dp(2203) = dp(6)*p(899) + p(6)*dp(899) - dp(2042) - dp(2041)
dp(2204) = dp(1)*p(1491) + p(1)*dp(1491) - dp(2203)
dp(2205) = dp(14)*p(578) + p(14)*dp(578)
dp(2206) = dp(3)*p(949) + p(3)*dp(949) - dp(2044)
dp(2207) = dp(39)*p(343) + p(39)*dp(343) - dp(2046)
dp(2208) = dp(1)*p(1445) + p(1)*dp(1445) - dp(2045) - dp(2207)
dp(2209) = dp(1)*p(1446) + p(1)*dp(1446) - dp(2046)
dp(2210) = dp(1)*p(1447) + p(1)*dp(1447) - dp(2047) - dp(2208)
dp(2211) = dp(21)*p(561) + p(21)*dp(561) - dp(2044)
dp(2212) = dp(6)*p(946) + p(6)*dp(946) - dp(2010)
dp(2213) = dp(1)*p(1450) + p(1)*dp(1450) - dp(2052) - dp(2050)
dp(2214) = dp(102)*p(150) + p(102)*dp(150) - dp(2010)
dp(2215) = dp(45)*p(337) + p(45)*dp(337) - dp(1966)
dp(2216) = dp(1)*p(1453) + p(1)*dp(1453) - dp(2062) - dp(2060)
dp(2217) = dp(50)*p(296) + p(50)*dp(296) - dp(1966)
dp(2218) = dp(95)*p(187) + p(95)*dp(187) - dp(1924)
dp(2219) = dp(1)*p(1457) + p(1)*dp(1457) - dp(2072) - dp(2070)
dp(2220) = dp(22)*p(532) + p(22)*dp(532) - dp(1924)
dp(2221) = dp(14)*p(559) + p(14)*dp(559)
dp(2222) = dp(18)*p(578) + p(18)*dp(578) - dp(1991)
dp(2223) = dp(2)*p(1441) + p(2)*dp(1441) - dp(2042) - dp(2203)
dp(2224) = dp(8)*p(900) + p(8)*dp(900) - dp(1991)
dp(2225) = dp(14)*p(581) + p(14)*dp(581)
dp(2226) = dp(3)*p(952) + p(3)*dp(952) - dp(2076)
dp(2227) = dp(2)*p(1445) + p(2)*dp(1445) - dp(2053) - dp(2049)
dp(2228) = dp(2)*p(1446) + p(2)*dp(1446) - dp(2054)
dp(2229) = dp(1)*p(1461) + p(1)*dp(1461) - dp(2077) - dp(2227)
dp(2230) = dp(7)*p(929) + p(7)*dp(929) - dp(2076)
dp(2231) = dp(6)*p(949) + p(6)*dp(949) - dp(2057)
dp(2232) = dp(1)*p(1464) + p(1)*dp(1464) - dp(2081) - dp(2079)
dp(2233) = dp(73)*p(195) + p(73)*dp(195) - dp(2057)
dp(2234) = dp(45)*p(340) + p(45)*dp(340) - dp(2024)
dp(2235) = dp(1)*p(1467) + p(1)*dp(1467) - dp(2088) - dp(2086)
dp(2236) = dp(102)*p(160) + p(102)*dp(160) - dp(2024)
dp(2237) = dp(14)*p(565) + p(14)*dp(565)
dp(2238) = dp(98)*p(187) + p(98)*dp(187) - dp(1983)
dp(2239) = dp(2)*p(1457) + p(2)*dp(1457) - dp(2074) - dp(2223)
dp(2240) = dp(50)*p(310) + p(50)*dp(310) - dp(1983)
dp(2241) = dp(14)*p(584) + p(14)*dp(584)
dp(2242) = dp(3)*p(955) + p(3)*dp(955) - dp(2092)
dp(2243) = dp(2)*p(1461) + p(2)*dp(1461) - dp(2082) - dp(2079)
dp(2244) = dp(1)*p(1471) + p(1)*dp(1471) - dp(2092)
dp(2245) = dp(6)*p(952) + p(6)*dp(952) - dp(2084)
dp(2246) = dp(1)*p(1473) + p(1)*dp(1473) - dp(2095) - dp(2093)
dp(2247) = dp(32)*p(347) + p(32)*dp(347) - dp(2084)
dp(2248) = dp(45)*p(343) + p(45)*dp(343) - dp(2068)
dp(2249) = dp(2)*p(1467) + p(2)*dp(1467) - dp(2090) - dp(2239)
dp(2250) = dp(81)*p(195) + p(81)*dp(195) - dp(2068)
dp(2251) = dp(14)*p(587) + p(14)*dp(587)
dp(2252) = dp(2)*p(1470) + p(2)*dp(1470) - dp(2094) - dp(2093)
dp(2253) = dp(2)*p(1471) + p(2)*dp(1471) - dp(2095)
dp(2254) = dp(6)*p(955) + p(6)*dp(955) - dp(2096)
dp(2255) = dp(2)*p(1473) + p(2)*dp(1473) - dp(2097) - dp(2249)
dp(2256) = dp(13)*p(589) + p(13)*dp(589) - dp(2096)
dp(2257) = dp(14)*p(589) + p(14)*dp(589)
dp(2258) = dp(1)*p(1476) + p(1)*dp(1476) - dp(2101)
dp(2259) = dp(21)*p(566) + p(21)*dp(566) - dp(2098)
dp(2260) = dp(101)*p(178) + p(101)*dp(178) - dp(2099)
dp(2261) = dp(39)*p(346) + p(39)*dp(346) - dp(2104)
dp(2262) = dp(3)*p(958) + p(3)*dp(958) - dp(2107)
dp(2263) = dp(1)*p(1481) + p(1)*dp(1481) - dp(2108)
dp(2264) = dp(2)*p(1504) + p(2)*dp(1504) - dp(2263)
dp(2265) = dp(1)*p(1483) + p(1)*dp(1483) - dp(2124)
dp(2266) = dp(8)*p(958) + p(8)*dp(958) - dp(2265)
dp(2267) = dp(1)*p(1485) + p(1)*dp(1485) - dp(2141)
dp(2268) = dp(22)*p(588) + p(22)*dp(588) - dp(2267)
dp(2269) = dp(1)*p(1487) + p(1)*dp(1487) - dp(2160)
dp(2270) = dp(50)*p(346) + p(50)*dp(346) - dp(2269)
dp(2271) = dp(1)*p(1489) + p(1)*dp(1489) - dp(2181)
dp(2272) = dp(102)*p(194) + p(102)*dp(194) - dp(2271)
dp(2273) = dp(1)*p(1492) + p(1)*dp(1492) - dp(2206)
dp(2274) = dp(101)*p(195) + p(101)*dp(195) - dp(2273)
dp(2275) = dp(2)*p(1491) + p(2)*dp(1491) - dp(2205)
dp(2276) = dp(1)*p(1495) + p(1)*dp(1495) - dp(2226)
dp(2277) = dp(49)*p(347) + p(49)*dp(347) - dp(2276)
dp(2278) = dp(22)*p(578) + p(22)*dp(578) - dp(2180)
dp(2279) = dp(1)*p(1498) + p(1)*dp(1498) - dp(2242)
dp(2280) = dp(21)*p(589) + p(21)*dp(589) - dp(2279)
dp(2281) = dp(102)*p(187) + p(102)*dp(187) - dp(2198)
dp(2282) = dp(1)*p(1501) + p(1)*dp(1501) - dp(2252)
dp(2283) = dp(7)*p(959) + p(7)*dp(959) - dp(2282)
dp(2284) = dp(45)*p(347) + p(45)*dp(347) - dp(2237)
dp(2285) = dp(2)*p(1501) + p(2)*dp(1501) - dp(2254)
dp(2286) = dp(1)*p(1505) + p(1)*dp(1505) - dp(2285)
dp(2287) = dp(6)*p(959) + p(6)*dp(959) - dp(2257)
dp(2288) = dp(1)*p(1504) + p(1)*dp(1504) - dp(2262)
dp(2289) = dp(2)*p(1505) + p(2)*dp(1505) - dp(2287)
dp(2290) = dp(9)*p(590) + p(9)*dp(590)
dp(2291) = dp(9)*p(591) + p(9)*dp(591)
dp(2292) = dp(9)*p(613) + p(9)*dp(613)
dp(2293) = dp(9)*p(592) + p(9)*dp(592)
dp(2294) = dp(9)*p(614) + p(9)*dp(614)
dp(2295) = dp(9)*p(615) + p(9)*dp(615)
dp(2296) = dp(9)*p(701) + p(9)*dp(701)
dp(2297) = dp(9)*p(726) + p(9)*dp(726)
dp(2298) = dp(9)*p(626) + p(9)*dp(626)
dp(2299) = dp(9)*p(593) + p(9)*dp(593)
dp(2300) = dp(9)*p(627) + p(9)*dp(627)
dp(2301) = dp(9)*p(594) + p(9)*dp(594)
dp(2302) = dp(9)*p(595) + p(9)*dp(595)
dp(2303) = dp(9)*p(628) + p(9)*dp(628)
dp(2304) = dp(9)*p(596) + p(9)*dp(596)
dp(2305) = dp(9)*p(629) + p(9)*dp(629)
dp(2306) = dp(9)*p(630) + p(9)*dp(630)
dp(2307) = dp(9)*p(631) + p(9)*dp(631)
dp(2308) = dp(9)*p(727) + p(9)*dp(727)
dp(2309) = dp(9)*p(636) + p(9)*dp(636)
dp(2310) = dp(9)*p(637) + p(9)*dp(637)
dp(2311) = dp(9)*p(734) + p(9)*dp(734)
dp(2312) = dp(9)*p(597) + p(9)*dp(597)
dp(2313) = dp(9)*p(638) + p(9)*dp(638)
dp(2314) = dp(9)*p(598) + p(9)*dp(598)
dp(2315) = dp(9)*p(639) + p(9)*dp(639)
dp(2316) = dp(9)*p(640) + p(9)*dp(640)
dp(2317) = dp(9)*p(735) + p(9)*dp(735)
dp(2318) = dp(9)*p(599) + p(9)*dp(599)
dp(2319) = dp(9)*p(641) + p(9)*dp(641)
dp(2320) = dp(9)*p(736) + p(9)*dp(736)
dp(2321) = dp(9)*p(755) + p(9)*dp(755)
dp(2322) = dp(9)*p(648) + p(9)*dp(648)
dp(2323) = dp(9)*p(600) + p(9)*dp(600)
dp(2324) = dp(9)*p(649) + p(9)*dp(649)
dp(2325) = dp(9)*p(601) + p(9)*dp(601)
dp(2326) = dp(9)*p(650) + p(9)*dp(650)
dp(2327) = dp(9)*p(651) + p(9)*dp(651)
dp(2328) = dp(9)*p(652) + p(9)*dp(652)
dp(2329) = dp(9)*p(756) + p(9)*dp(756)
dp(2330) = dp(9)*p(766) + p(9)*dp(766)
dp(2331) = dp(9)*p(656) + p(9)*dp(656)
dp(2332) = dp(9)*p(657) + p(9)*dp(657)
dp(2333) = dp(9)*p(767) + p(9)*dp(767)
dp(2334) = dp(9)*p(658) + p(9)*dp(658)
dp(2335) = dp(9)*p(602) + p(9)*dp(602)
dp(2336) = dp(9)*p(659) + p(9)*dp(659)
dp(2337) = dp(9)*p(660) + p(9)*dp(660)
dp(2338) = dp(9)*p(661) + p(9)*dp(661)
dp(2339) = dp(9)*p(768) + p(9)*dp(768)
dp(2340) = dp(9)*p(769) + p(9)*dp(769)
dp(2341) = dp(9)*p(662) + p(9)*dp(662)
dp(2342) = dp(9)*p(663) + p(9)*dp(663)
dp(2343) = dp(9)*p(770) + p(9)*dp(770)
dp(2344) = dp(9)*p(665) + p(9)*dp(665)
dp(2345) = dp(9)*p(774) + p(9)*dp(774)
dp(2346) = dp(9)*p(666) + p(9)*dp(666)
dp(2347) = dp(9)*p(775) + p(9)*dp(775)
dp(2348) = dp(9)*p(776) + p(9)*dp(776)
dp(2349) = dp(9)*p(777) + p(9)*dp(777)
dp(2350) = dp(9)*p(882) + p(9)*dp(882)
dp(2351) = dp(9)*p(791) + p(9)*dp(791)
dp(2352) = dp(9)*p(670) + p(9)*dp(670)
dp(2353) = dp(9)*p(603) + p(9)*dp(603)
dp(2354) = dp(9)*p(671) + p(9)*dp(671)
dp(2355) = dp(9)*p(672) + p(9)*dp(672)
dp(2356) = dp(9)*p(792) + p(9)*dp(792)
dp(2357) = dp(9)*p(799) + p(9)*dp(799)
dp(2358) = dp(9)*p(674) + p(9)*dp(674)
dp(2359) = dp(9)*p(800) + p(9)*dp(800)
dp(2360) = dp(9)*p(675) + p(9)*dp(675)
dp(2361) = dp(9)*p(676) + p(9)*dp(676)
dp(2362) = dp(9)*p(801) + p(9)*dp(801)
dp(2363) = dp(9)*p(802) + p(9)*dp(802)
dp(2364) = dp(9)*p(803) + p(9)*dp(803)
dp(2365) = dp(9)*p(807) + p(9)*dp(807)
dp(2366) = dp(9)*p(808) + p(9)*dp(808)
dp(2367) = dp(9)*p(809) + p(9)*dp(809)
dp(2368) = dp(9)*p(898) + p(9)*dp(898)
dp(2369) = dp(9)*p(901) + p(9)*dp(901)
dp(2370) = dp(9)*p(818) + p(9)*dp(818)
dp(2371) = dp(9)*p(678) + p(9)*dp(678)
dp(2372) = dp(9)*p(679) + p(9)*dp(679)
dp(2373) = dp(9)*p(819) + p(9)*dp(819)
dp(2374) = dp(9)*p(823) + p(9)*dp(823)
dp(2375) = dp(9)*p(824) + p(9)*dp(824)
dp(2376) = dp(9)*p(825) + p(9)*dp(825)
dp(2377) = dp(9)*p(913) + p(9)*dp(913)
dp(2378) = dp(9)*p(917) + p(9)*dp(917)
dp(2379) = dp(9)*p(830) + p(9)*dp(830)
dp(2380) = dp(9)*p(831) + p(9)*dp(831)
dp(2381) = dp(9)*p(927) + p(9)*dp(927)
dp(2382) = dp(9)*p(933) + p(9)*dp(933)
dp(2383) = dp(9)*p(692) + p(9)*dp(692)
dp(2384) = dp(9)*p(693) + p(9)*dp(693)
dp(2385) = dp(9)*p(694) + p(9)*dp(694)
dp(2386) = dp(9)*p(850) + p(9)*dp(850)
dp(2387) = dp(9)*p(604) + p(9)*dp(604)
dp(2388) = dp(9)*p(605) + p(9)*dp(605)
dp(2389) = dp(9)*p(606) + p(9)*dp(606)
dp(2390) = dp(9)*p(695) + p(9)*dp(695)
dp(2391) = dp(9)*p(607) + p(9)*dp(607)
dp(2392) = dp(9)*p(608) + p(9)*dp(608)
dp(2393) = dp(9)*p(696) + p(9)*dp(696)
dp(2394) = dp(9)*p(609) + p(9)*dp(609)
dp(2395) = dp(9)*p(610) + p(9)*dp(610)
dp(2396) = dp(9)*p(697) + p(9)*dp(697)
dp(2397) = dp(9)*p(611) + p(9)*dp(611)
dp(2398) = dp(9)*p(698) + p(9)*dp(698)
dp(2399) = dp(9)*p(612) + p(9)*dp(612)
dp(2400) = dp(9)*p(699) + p(9)*dp(699)
dp(2401) = dp(9)*p(700) + p(9)*dp(700)
dp(2402) = dp(9)*p(851) + p(9)*dp(851)
dp(2403) = dp(14)*p(934) + p(14)*dp(934)
dp(2404) = dp(14)*p(935) + p(14)*dp(935)
dp(2405) = dp(14)*p(936) + p(14)*dp(936)
dp(2406) = dp(14)*p(937) + p(14)*dp(937)
dp(2407) = dp(9)*p(861) + p(9)*dp(861)
dp(2408) = dp(9)*p(711) + p(9)*dp(711)
dp(2409) = dp(9)*p(712) + p(9)*dp(712)
dp(2410) = dp(9)*p(713) + p(9)*dp(713)
dp(2411) = dp(9)*p(714) + p(9)*dp(714)
dp(2412) = dp(9)*p(715) + p(9)*dp(715)
dp(2413) = dp(9)*p(862) + p(9)*dp(862)
dp(2414) = dp(9)*p(716) + p(9)*dp(716)
dp(2415) = dp(9)*p(616) + p(9)*dp(616)
dp(2416) = dp(9)*p(617) + p(9)*dp(617)
dp(2417) = dp(9)*p(618) + p(9)*dp(618)
dp(2418) = dp(9)*p(619) + p(9)*dp(619)
dp(2419) = dp(9)*p(717) + p(9)*dp(717)
dp(2420) = dp(9)*p(718) + p(9)*dp(718)
dp(2421) = dp(9)*p(620) + p(9)*dp(620)
dp(2422) = dp(9)*p(621) + p(9)*dp(621)
dp(2423) = dp(9)*p(622) + p(9)*dp(622)
dp(2424) = dp(9)*p(719) + p(9)*dp(719)
dp(2425) = dp(9)*p(720) + p(9)*dp(720)
dp(2426) = dp(9)*p(623) + p(9)*dp(623)
dp(2427) = dp(9)*p(624) + p(9)*dp(624)
dp(2428) = dp(9)*p(721) + p(9)*dp(721)
dp(2429) = dp(9)*p(722) + p(9)*dp(722)
dp(2430) = dp(9)*p(625) + p(9)*dp(625)
dp(2431) = dp(9)*p(723) + p(9)*dp(723)
dp(2432) = dp(9)*p(724) + p(9)*dp(724)
dp(2433) = dp(9)*p(725) + p(9)*dp(725)
dp(2434) = dp(9)*p(863) + p(9)*dp(863)
dp(2435) = dp(14)*p(839) + p(14)*dp(839)
dp(2436) = dp(14)*p(840) + p(14)*dp(840)
dp(2437) = dp(14)*p(841) + p(14)*dp(841)
dp(2438) = dp(14)*p(842) + p(14)*dp(842)
dp(2439) = dp(14)*p(843) + p(14)*dp(843)
dp(2440) = dp(14)*p(844) + p(14)*dp(844)
dp(2441) = dp(14)*p(845) + p(14)*dp(845)
dp(2442) = dp(14)*p(846) + p(14)*dp(846)
dp(2443) = dp(14)*p(847) + p(14)*dp(847)
dp(2444) = dp(14)*p(848) + p(14)*dp(848)
dp(2445) = dp(14)*p(849) + p(14)*dp(849)
dp(2446) = dp(9)*p(728) + p(9)*dp(728)
dp(2447) = dp(9)*p(729) + p(9)*dp(729)
dp(2448) = dp(9)*p(864) + p(9)*dp(864)
dp(2449) = dp(9)*p(632) + p(9)*dp(632)
dp(2450) = dp(9)*p(633) + p(9)*dp(633)
dp(2451) = dp(9)*p(730) + p(9)*dp(730)
dp(2452) = dp(9)*p(634) + p(9)*dp(634)
dp(2453) = dp(9)*p(731) + p(9)*dp(731)
dp(2454) = dp(9)*p(635) + p(9)*dp(635)
dp(2455) = dp(9)*p(732) + p(9)*dp(732)
dp(2456) = dp(9)*p(733) + p(9)*dp(733)
dp(2457) = dp(9)*p(865) + p(9)*dp(865)
dp(2458) = dp(14)*p(692) + p(14)*dp(692)
dp(2459) = dp(14)*p(693) + p(14)*dp(693)
dp(2460) = dp(14)*p(694) + p(14)*dp(694)
dp(2461) = dp(14)*p(695) + p(14)*dp(695)
dp(2462) = dp(14)*p(696) + p(14)*dp(696)
dp(2463) = dp(14)*p(697) + p(14)*dp(697)
dp(2464) = dp(14)*p(698) + p(14)*dp(698)
dp(2465) = dp(14)*p(699) + p(14)*dp(699)
dp(2466) = dp(14)*p(700) + p(14)*dp(700)
dp(2467) = dp(14)*p(613) + p(14)*dp(613)
dp(2468) = dp(14)*p(614) + p(14)*dp(614)
dp(2469) = dp(14)*p(615) + p(14)*dp(615)
dp(2470) = dp(9)*p(874) + p(9)*dp(874)
dp(2471) = dp(9)*p(743) + p(9)*dp(743)
dp(2472) = dp(9)*p(744) + p(9)*dp(744)
dp(2473) = dp(9)*p(745) + p(9)*dp(745)
dp(2474) = dp(9)*p(746) + p(9)*dp(746)
dp(2475) = dp(9)*p(875) + p(9)*dp(875)
dp(2476) = dp(9)*p(747) + p(9)*dp(747)
dp(2477) = dp(9)*p(642) + p(9)*dp(642)
dp(2478) = dp(9)*p(643) + p(9)*dp(643)
dp(2479) = dp(9)*p(644) + p(9)*dp(644)
dp(2480) = dp(9)*p(748) + p(9)*dp(748)
dp(2481) = dp(9)*p(749) + p(9)*dp(749)
dp(2482) = dp(9)*p(645) + p(9)*dp(645)
dp(2483) = dp(9)*p(646) + p(9)*dp(646)
dp(2484) = dp(9)*p(750) + p(9)*dp(750)
dp(2485) = dp(9)*p(751) + p(9)*dp(751)
dp(2486) = dp(9)*p(647) + p(9)*dp(647)
dp(2487) = dp(9)*p(752) + p(9)*dp(752)
dp(2488) = dp(9)*p(753) + p(9)*dp(753)
dp(2489) = dp(9)*p(754) + p(9)*dp(754)
dp(2490) = dp(9)*p(876) + p(9)*dp(876)
dp(2491) = dp(14)*p(852) + p(14)*dp(852)
dp(2492) = dp(14)*p(853) + p(14)*dp(853)
dp(2493) = dp(14)*p(854) + p(14)*dp(854)
dp(2494) = dp(14)*p(855) + p(14)*dp(855)
dp(2495) = dp(14)*p(856) + p(14)*dp(856)
dp(2496) = dp(14)*p(857) + p(14)*dp(857)
dp(2497) = dp(14)*p(858) + p(14)*dp(858)
dp(2498) = dp(14)*p(859) + p(14)*dp(859)
dp(2499) = dp(14)*p(860) + p(14)*dp(860)
dp(2500) = dp(9)*p(877) + p(9)*dp(877)
dp(2501) = dp(9)*p(757) + p(9)*dp(757)
dp(2502) = dp(9)*p(758) + p(9)*dp(758)
dp(2503) = dp(9)*p(759) + p(9)*dp(759)
dp(2504) = dp(9)*p(878) + p(9)*dp(878)
dp(2505) = dp(9)*p(760) + p(9)*dp(760)
dp(2506) = dp(9)*p(653) + p(9)*dp(653)
dp(2507) = dp(9)*p(654) + p(9)*dp(654)
dp(2508) = dp(9)*p(761) + p(9)*dp(761)
dp(2509) = dp(9)*p(762) + p(9)*dp(762)
dp(2510) = dp(9)*p(655) + p(9)*dp(655)
dp(2511) = dp(9)*p(763) + p(9)*dp(763)
dp(2512) = dp(9)*p(764) + p(9)*dp(764)
dp(2513) = dp(9)*p(765) + p(9)*dp(765)
dp(2514) = dp(9)*p(879) + p(9)*dp(879)
dp(2515) = dp(14)*p(711) + p(14)*dp(711)
dp(2516) = dp(14)*p(712) + p(14)*dp(712)
dp(2517) = dp(14)*p(713) + p(14)*dp(713)
dp(2518) = dp(14)*p(714) + p(14)*dp(714)
dp(2519) = dp(14)*p(715) + p(14)*dp(715)
dp(2520) = dp(14)*p(716) + p(14)*dp(716)
dp(2521) = dp(14)*p(717) + p(14)*dp(717)
dp(2522) = dp(14)*p(718) + p(14)*dp(718)
dp(2523) = dp(14)*p(719) + p(14)*dp(719)
dp(2524) = dp(14)*p(720) + p(14)*dp(720)
dp(2525) = dp(14)*p(721) + p(14)*dp(721)
dp(2526) = dp(14)*p(722) + p(14)*dp(722)
dp(2527) = dp(14)*p(723) + p(14)*dp(723)
dp(2528) = dp(14)*p(724) + p(14)*dp(724)
dp(2529) = dp(14)*p(725) + p(14)*dp(725)
dp(2530) = dp(14)*p(626) + p(14)*dp(626)
dp(2531) = dp(14)*p(627) + p(14)*dp(627)
dp(2532) = dp(14)*p(628) + p(14)*dp(628)
dp(2533) = dp(14)*p(629) + p(14)*dp(629)
dp(2534) = dp(14)*p(630) + p(14)*dp(630)
dp(2535) = dp(14)*p(631) + p(14)*dp(631)
dp(2536) = dp(9)*p(771) + p(9)*dp(771)
dp(2537) = dp(9)*p(880) + p(9)*dp(880)
dp(2538) = dp(9)*p(664) + p(9)*dp(664)
dp(2539) = dp(9)*p(772) + p(9)*dp(772)
dp(2540) = dp(9)*p(773) + p(9)*dp(773)
dp(2541) = dp(9)*p(881) + p(9)*dp(881)
dp(2542) = dp(14)*p(728) + p(14)*dp(728)
dp(2543) = dp(14)*p(729) + p(14)*dp(729)
dp(2544) = dp(14)*p(730) + p(14)*dp(730)
dp(2545) = dp(14)*p(731) + p(14)*dp(731)
dp(2546) = dp(14)*p(732) + p(14)*dp(732)
dp(2547) = dp(14)*p(733) + p(14)*dp(733)
dp(2548) = dp(14)*p(636) + p(14)*dp(636)
dp(2549) = dp(14)*p(637) + p(14)*dp(637)
dp(2550) = dp(14)*p(638) + p(14)*dp(638)
dp(2551) = dp(14)*p(639) + p(14)*dp(639)
dp(2552) = dp(14)*p(640) + p(14)*dp(640)
dp(2553) = dp(14)*p(641) + p(14)*dp(641)
dp(2554) = dp(9)*p(889) + p(9)*dp(889)
dp(2555) = dp(9)*p(782) + p(9)*dp(782)
dp(2556) = dp(9)*p(783) + p(9)*dp(783)
dp(2557) = dp(9)*p(784) + p(9)*dp(784)
dp(2558) = dp(9)*p(890) + p(9)*dp(890)
dp(2559) = dp(9)*p(785) + p(9)*dp(785)
dp(2560) = dp(9)*p(667) + p(9)*dp(667)
dp(2561) = dp(9)*p(668) + p(9)*dp(668)
dp(2562) = dp(9)*p(786) + p(9)*dp(786)
dp(2563) = dp(9)*p(787) + p(9)*dp(787)
dp(2564) = dp(9)*p(669) + p(9)*dp(669)
dp(2565) = dp(9)*p(788) + p(9)*dp(788)
dp(2566) = dp(9)*p(789) + p(9)*dp(789)
dp(2567) = dp(9)*p(790) + p(9)*dp(790)
dp(2568) = dp(9)*p(891) + p(9)*dp(891)
dp(2569) = dp(14)*p(866) + p(14)*dp(866)
dp(2570) = dp(14)*p(867) + p(14)*dp(867)
dp(2571) = dp(14)*p(868) + p(14)*dp(868)
dp(2572) = dp(14)*p(869) + p(14)*dp(869)
dp(2573) = dp(14)*p(870) + p(14)*dp(870)
dp(2574) = dp(14)*p(871) + p(14)*dp(871)
dp(2575) = dp(14)*p(872) + p(14)*dp(872)
dp(2576) = dp(14)*p(873) + p(14)*dp(873)
dp(2577) = dp(9)*p(892) + p(9)*dp(892)
dp(2578) = dp(9)*p(793) + p(9)*dp(793)
dp(2579) = dp(9)*p(794) + p(9)*dp(794)
dp(2580) = dp(9)*p(893) + p(9)*dp(893)
dp(2581) = dp(9)*p(795) + p(9)*dp(795)
dp(2582) = dp(9)*p(673) + p(9)*dp(673)
dp(2583) = dp(9)*p(796) + p(9)*dp(796)
dp(2584) = dp(9)*p(797) + p(9)*dp(797)
dp(2585) = dp(9)*p(798) + p(9)*dp(798)
dp(2586) = dp(9)*p(894) + p(9)*dp(894)
dp(2587) = dp(14)*p(743) + p(14)*dp(743)
dp(2588) = dp(14)*p(744) + p(14)*dp(744)
dp(2589) = dp(14)*p(745) + p(14)*dp(745)
dp(2590) = dp(14)*p(746) + p(14)*dp(746)
dp(2591) = dp(14)*p(747) + p(14)*dp(747)
dp(2592) = dp(14)*p(748) + p(14)*dp(748)
dp(2593) = dp(14)*p(749) + p(14)*dp(749)
dp(2594) = dp(14)*p(750) + p(14)*dp(750)
dp(2595) = dp(14)*p(751) + p(14)*dp(751)
dp(2596) = dp(14)*p(752) + p(14)*dp(752)
dp(2597) = dp(14)*p(753) + p(14)*dp(753)
dp(2598) = dp(14)*p(754) + p(14)*dp(754)
dp(2599) = dp(14)*p(648) + p(14)*dp(648)
dp(2600) = dp(14)*p(649) + p(14)*dp(649)
dp(2601) = dp(14)*p(650) + p(14)*dp(650)
dp(2602) = dp(14)*p(651) + p(14)*dp(651)
dp(2603) = dp(14)*p(652) + p(14)*dp(652)
dp(2604) = dp(9)*p(895) + p(9)*dp(895)
dp(2605) = dp(9)*p(804) + p(9)*dp(804)
dp(2606) = dp(9)*p(896) + p(9)*dp(896)
dp(2607) = dp(9)*p(805) + p(9)*dp(805)
dp(2608) = dp(9)*p(806) + p(9)*dp(806)
dp(2609) = dp(9)*p(897) + p(9)*dp(897)
dp(2610) = dp(14)*p(757) + p(14)*dp(757)
dp(2611) = dp(14)*p(758) + p(14)*dp(758)
dp(2612) = dp(14)*p(759) + p(14)*dp(759)
dp(2613) = dp(14)*p(760) + p(14)*dp(760)
dp(2614) = dp(14)*p(761) + p(14)*dp(761)
dp(2615) = dp(14)*p(762) + p(14)*dp(762)
dp(2616) = dp(14)*p(763) + p(14)*dp(763)
dp(2617) = dp(14)*p(764) + p(14)*dp(764)
dp(2618) = dp(14)*p(765) + p(14)*dp(765)
dp(2619) = dp(14)*p(656) + p(14)*dp(656)
dp(2620) = dp(14)*p(657) + p(14)*dp(657)
dp(2621) = dp(14)*p(658) + p(14)*dp(658)
dp(2622) = dp(14)*p(659) + p(14)*dp(659)
dp(2623) = dp(14)*p(660) + p(14)*dp(660)
dp(2624) = dp(14)*p(661) + p(14)*dp(661)
dp(2625) = dp(14)*p(662) + p(14)*dp(662)
dp(2626) = dp(14)*p(663) + p(14)*dp(663)
dp(2627) = dp(9)*p(899) + p(9)*dp(899)
dp(2628) = dp(9)*p(900) + p(9)*dp(900)
dp(2629) = dp(14)*p(771) + p(14)*dp(771)
dp(2630) = dp(14)*p(772) + p(14)*dp(772)
dp(2631) = dp(14)*p(773) + p(14)*dp(773)
dp(2632) = dp(14)*p(665) + p(14)*dp(665)
dp(2633) = dp(14)*p(666) + p(14)*dp(666)
dp(2634) = dp(9)*p(907) + p(9)*dp(907)
dp(2635) = dp(9)*p(812) + p(9)*dp(812)
dp(2636) = dp(9)*p(813) + p(9)*dp(813)
dp(2637) = dp(9)*p(908) + p(9)*dp(908)
dp(2638) = dp(9)*p(814) + p(9)*dp(814)
dp(2639) = dp(9)*p(677) + p(9)*dp(677)
dp(2640) = dp(9)*p(815) + p(9)*dp(815)
dp(2641) = dp(9)*p(816) + p(9)*dp(816)
dp(2642) = dp(9)*p(817) + p(9)*dp(817)
dp(2643) = dp(9)*p(909) + p(9)*dp(909)
dp(2644) = dp(14)*p(883) + p(14)*dp(883)
dp(2645) = dp(14)*p(884) + p(14)*dp(884)
dp(2646) = dp(14)*p(885) + p(14)*dp(885)
dp(2647) = dp(14)*p(886) + p(14)*dp(886)
dp(2648) = dp(14)*p(887) + p(14)*dp(887)
dp(2649) = dp(14)*p(888) + p(14)*dp(888)
dp(2650) = dp(9)*p(910) + p(9)*dp(910)
dp(2651) = dp(9)*p(820) + p(9)*dp(820)
dp(2652) = dp(9)*p(911) + p(9)*dp(911)
dp(2653) = dp(9)*p(821) + p(9)*dp(821)
dp(2654) = dp(9)*p(822) + p(9)*dp(822)
dp(2655) = dp(9)*p(912) + p(9)*dp(912)
dp(2656) = dp(14)*p(782) + p(14)*dp(782)
dp(2657) = dp(14)*p(783) + p(14)*dp(783)
dp(2658) = dp(14)*p(784) + p(14)*dp(784)
dp(2659) = dp(14)*p(785) + p(14)*dp(785)
dp(2660) = dp(14)*p(786) + p(14)*dp(786)
dp(2661) = dp(14)*p(787) + p(14)*dp(787)
dp(2662) = dp(14)*p(788) + p(14)*dp(788)
dp(2663) = dp(14)*p(789) + p(14)*dp(789)
dp(2664) = dp(14)*p(790) + p(14)*dp(790)
dp(2665) = dp(14)*p(670) + p(14)*dp(670)
dp(2666) = dp(14)*p(671) + p(14)*dp(671)
dp(2667) = dp(14)*p(672) + p(14)*dp(672)
dp(2668) = dp(9)*p(914) + p(9)*dp(914)
dp(2669) = dp(9)*p(915) + p(9)*dp(915)
dp(2670) = dp(9)*p(916) + p(9)*dp(916)
dp(2671) = dp(14)*p(793) + p(14)*dp(793)
dp(2672) = dp(14)*p(794) + p(14)*dp(794)
dp(2673) = dp(14)*p(795) + p(14)*dp(795)
dp(2674) = dp(14)*p(796) + p(14)*dp(796)
dp(2675) = dp(14)*p(797) + p(14)*dp(797)
dp(2676) = dp(14)*p(798) + p(14)*dp(798)
dp(2677) = dp(14)*p(674) + p(14)*dp(674)
dp(2678) = dp(14)*p(675) + p(14)*dp(675)
dp(2679) = dp(14)*p(676) + p(14)*dp(676)
dp(2680) = dp(9)*p(948) + p(9)*dp(948)
dp(2681) = dp(14)*p(804) + p(14)*dp(804)
dp(2682) = dp(14)*p(805) + p(14)*dp(805)
dp(2683) = dp(14)*p(806) + p(14)*dp(806)
dp(2684) = dp(9)*p(921) + p(9)*dp(921)
dp(2685) = dp(9)*p(827) + p(9)*dp(827)
dp(2686) = dp(9)*p(922) + p(9)*dp(922)
dp(2687) = dp(9)*p(828) + p(9)*dp(828)
dp(2688) = dp(9)*p(829) + p(9)*dp(829)
dp(2689) = dp(9)*p(923) + p(9)*dp(923)
dp(2690) = dp(14)*p(902) + p(14)*dp(902)
dp(2691) = dp(14)*p(903) + p(14)*dp(903)
dp(2692) = dp(14)*p(904) + p(14)*dp(904)
dp(2693) = dp(14)*p(905) + p(14)*dp(905)
dp(2694) = dp(14)*p(906) + p(14)*dp(906)
dp(2695) = dp(9)*p(924) + p(9)*dp(924)
dp(2696) = dp(9)*p(925) + p(9)*dp(925)
dp(2697) = dp(9)*p(926) + p(9)*dp(926)
dp(2698) = dp(14)*p(812) + p(14)*dp(812)
dp(2699) = dp(14)*p(813) + p(14)*dp(813)
dp(2700) = dp(14)*p(814) + p(14)*dp(814)
dp(2701) = dp(14)*p(815) + p(14)*dp(815)
dp(2702) = dp(14)*p(816) + p(14)*dp(816)
dp(2703) = dp(14)*p(817) + p(14)*dp(817)
dp(2704) = dp(14)*p(678) + p(14)*dp(678)
dp(2705) = dp(14)*p(679) + p(14)*dp(679)
dp(2706) = dp(9)*p(951) + p(9)*dp(951)
dp(2707) = dp(14)*p(820) + p(14)*dp(820)
dp(2708) = dp(14)*p(821) + p(14)*dp(821)
dp(2709) = dp(14)*p(822) + p(14)*dp(822)
dp(2710) = dp(9)*p(930) + p(9)*dp(930)
dp(2711) = dp(9)*p(931) + p(9)*dp(931)
dp(2712) = dp(9)*p(932) + p(9)*dp(932)
dp(2713) = dp(14)*p(918) + p(14)*dp(918)
dp(2714) = dp(14)*p(919) + p(14)*dp(919)
dp(2715) = dp(14)*p(920) + p(14)*dp(920)
dp(2716) = dp(9)*p(954) + p(9)*dp(954)
dp(2717) = dp(14)*p(827) + p(14)*dp(827)
dp(2718) = dp(14)*p(828) + p(14)*dp(828)
dp(2719) = dp(14)*p(829) + p(14)*dp(829)
dp(2720) = dp(9)*p(957) + p(9)*dp(957)
dp(2721) = dp(14)*p(928) + p(14)*dp(928)
dp(2722) = dp(14)*p(929) + p(14)*dp(929)
dp(2723) = dp(9)*p(938) + p(9)*dp(938)
dp(2724) = dp(9)*p(839) + p(9)*dp(839)
dp(2725) = dp(9)*p(680) + p(9)*dp(680)
dp(2726) = dp(9)*p(840) + p(9)*dp(840)
dp(2727) = dp(9)*p(681) + p(9)*dp(681)
dp(2728) = dp(9)*p(682) + p(9)*dp(682)
dp(2729) = dp(9)*p(841) + p(9)*dp(841)
dp(2730) = dp(9)*p(683) + p(9)*dp(683)
dp(2731) = dp(9)*p(684) + p(9)*dp(684)
dp(2732) = dp(9)*p(685) + p(9)*dp(685)
dp(2733) = dp(9)*p(842) + p(9)*dp(842)
dp(2734) = dp(9)*p(686) + p(9)*dp(686)
dp(2735) = dp(9)*p(687) + p(9)*dp(687)
dp(2736) = dp(9)*p(688) + p(9)*dp(688)
dp(2737) = dp(9)*p(843) + p(9)*dp(843)
dp(2738) = dp(9)*p(844) + p(9)*dp(844)
dp(2739) = dp(9)*p(689) + p(9)*dp(689)
dp(2740) = dp(9)*p(690) + p(9)*dp(690)
dp(2741) = dp(9)*p(845) + p(9)*dp(845)
dp(2742) = dp(9)*p(846) + p(9)*dp(846)
dp(2743) = dp(9)*p(691) + p(9)*dp(691)
dp(2744) = dp(9)*p(847) + p(9)*dp(847)
dp(2745) = dp(9)*p(848) + p(9)*dp(848)
dp(2746) = dp(9)*p(849) + p(9)*dp(849)
dp(2747) = dp(9)*p(939) + p(9)*dp(939)
dp(2748) = dp(3)*p(1136) + p(3)*dp(1136) - dp(2383)
dp(2749) = dp(7)*p(1136) + p(7)*dp(1136) - dp(2387)
dp(2750) = dp(49)*p(401) + p(49)*dp(401) - dp(2394)
dp(2751) = dp(114)*p(194) + p(114)*dp(194) - dp(2399)
dp(2752) = dp(24)*p(588) + p(24)*dp(588) - dp(2402)
dp(2753) = dp(3)*p(1368) + p(3)*dp(1368) - dp(2386) - dp(2752)
dp(2754) = dp(1)*p(1802) + p(1)*dp(1802) - dp(2386) - dp(2753)
dp(2755) = dp(1)*p(1803) + p(1)*dp(1803) - dp(2390) - dp(2754)
dp(2756) = dp(1)*p(1804) + p(1)*dp(1804) - dp(2393) - dp(2755)
dp(2757) = dp(6)*p(1476) + p(6)*dp(1476) - dp(2748)
dp(2758) = dp(32)*p(566) + p(32)*dp(566) - dp(2387)
dp(2759) = dp(150)*p(178) + p(150)*dp(178) - dp(2394)
dp(2760) = dp(39)*p(505) + p(39)*dp(505) - dp(2399)
dp(2761) = dp(3)*p(1369) + p(3)*dp(1369) - dp(2402)
dp(2762) = dp(14)*p(958) + p(14)*dp(958)
dp(2763) = dp(9)*p(940) + p(9)*dp(940)
dp(2764) = dp(9)*p(852) + p(9)*dp(852)
dp(2765) = dp(9)*p(702) + p(9)*dp(702)
dp(2766) = dp(9)*p(853) + p(9)*dp(853)
dp(2767) = dp(9)*p(703) + p(9)*dp(703)
dp(2768) = dp(9)*p(704) + p(9)*dp(704)
dp(2769) = dp(9)*p(854) + p(9)*dp(854)
dp(2770) = dp(9)*p(705) + p(9)*dp(705)
dp(2771) = dp(9)*p(706) + p(9)*dp(706)
dp(2772) = dp(9)*p(707) + p(9)*dp(707)
dp(2773) = dp(9)*p(855) + p(9)*dp(855)
dp(2774) = dp(9)*p(708) + p(9)*dp(708)
dp(2775) = dp(9)*p(709) + p(9)*dp(709)
dp(2776) = dp(9)*p(856) + p(9)*dp(856)
dp(2777) = dp(9)*p(857) + p(9)*dp(857)
dp(2778) = dp(9)*p(710) + p(9)*dp(710)
dp(2779) = dp(9)*p(858) + p(9)*dp(858)
dp(2780) = dp(9)*p(859) + p(9)*dp(859)
dp(2781) = dp(9)*p(860) + p(9)*dp(860)
dp(2782) = dp(9)*p(941) + p(9)*dp(941)
dp(2783) = dp(24)*p(570) + p(24)*dp(570) - dp(2296)
dp(2784) = dp(114)*p(181) + p(114)*dp(181) - dp(2293)
dp(2785) = dp(1)*p(1828) + p(1)*dp(1828) - dp(2408) - dp(2784)
dp(2786) = dp(4)*p(1136) + p(4)*dp(1136) - dp(2292)
dp(2787) = dp(1)*p(1830) + p(1)*dp(1830) - dp(2410) - dp(2786)
dp(2788) = dp(1)*p(1831) + p(1)*dp(1831) - dp(2411) - dp(2787)
dp(2789) = dp(1)*p(1832) + p(1)*dp(1832) - dp(2412) - dp(2788)
dp(2790) = dp(1)*p(1833) + p(1)*dp(1833) - dp(2413) - dp(2789)
dp(2791) = dp(13)*p(938) + p(13)*dp(938) - dp(2762)
dp(2792) = dp(2)*p(1802) + p(2)*dp(1802) - dp(2406) - dp(2791)
dp(2793) = dp(32)*p(570) + p(32)*dp(570) - dp(2762)
dp(2794) = dp(1)*p(1835) + p(1)*dp(1835) - dp(2413) - dp(2792)
dp(2795) = dp(73)*p(331) + p(73)*dp(331) - dp(2762)
dp(2796) = dp(1)*p(1837) + p(1)*dp(1837) - dp(2419) - dp(2794)
dp(2797) = dp(150)*p(181) + p(150)*dp(181) - dp(2762)
dp(2798) = dp(1)*p(1839) + p(1)*dp(1839) - dp(2424) - dp(2796)
dp(2799) = dp(91)*p(285) + p(91)*dp(285) - dp(2762)
dp(2800) = dp(13)*p(843) + p(13)*dp(843) - dp(2292)
dp(2801) = dp(41)*p(505) + p(41)*dp(505) - dp(2762)
dp(2802) = dp(1)*p(1843) + p(1)*dp(1843) - dp(2431) - dp(2800)
dp(2803) = dp(16)*p(851) + p(16)*dp(851) - dp(2762)
dp(2804) = dp(69)*p(285) + p(69)*dp(285) - dp(2293)
dp(2805) = dp(4)*p(1369) + p(4)*dp(1369) - dp(2762)
dp(2806) = dp(11)*p(851) + p(11)*dp(851) - dp(2296)
dp(2807) = dp(14)*p(938) + p(14)*dp(938)
dp(2808) = dp(14)*p(939) + p(14)*dp(939)
dp(2809) = dp(3)*p(1180) + p(3)*dp(1180) - dp(2446)
dp(2810) = dp(7)*p(1180) + p(7)*dp(1180) - dp(2449)
dp(2811) = dp(49)*p(430) + p(49)*dp(430) - dp(2454)
dp(2812) = dp(130)*p(194) + p(130)*dp(194) - dp(2457)
dp(2813) = dp(3)*p(1384) + p(3)*dp(1384) - dp(2448) - dp(2812)
dp(2814) = dp(1)*p(1853) + p(1)*dp(1853) - dp(2448) - dp(2813)
dp(2815) = dp(1)*p(1854) + p(1)*dp(1854) - dp(2451) - dp(2814)
dp(2816) = dp(45)*p(566) + p(45)*dp(566) - dp(2809)
dp(2817) = dp(160)*p(178) + p(160)*dp(178) - dp(2449)
dp(2818) = dp(39)*p(518) + p(39)*dp(518) - dp(2454)
dp(2819) = dp(3)*p(1385) + p(3)*dp(1385) - dp(2457)
dp(2820) = dp(14)*p(850) + p(14)*dp(850)
dp(2821) = dp(14)*p(851) + p(14)*dp(851)
dp(2822) = dp(14)*p(701) + p(14)*dp(701)
dp(2823) = dp(9)*p(942) + p(9)*dp(942)
dp(2824) = dp(9)*p(866) + p(9)*dp(866)
dp(2825) = dp(9)*p(737) + p(9)*dp(737)
dp(2826) = dp(9)*p(867) + p(9)*dp(867)
dp(2827) = dp(9)*p(738) + p(9)*dp(738)
dp(2828) = dp(9)*p(739) + p(9)*dp(739)
dp(2829) = dp(9)*p(868) + p(9)*dp(868)
dp(2830) = dp(9)*p(740) + p(9)*dp(740)
dp(2831) = dp(9)*p(741) + p(9)*dp(741)
dp(2832) = dp(9)*p(869) + p(9)*dp(869)
dp(2833) = dp(9)*p(870) + p(9)*dp(870)
dp(2834) = dp(9)*p(742) + p(9)*dp(742)
dp(2835) = dp(9)*p(871) + p(9)*dp(871)
dp(2836) = dp(9)*p(872) + p(9)*dp(872)
dp(2837) = dp(9)*p(873) + p(9)*dp(873)
dp(2838) = dp(9)*p(943) + p(9)*dp(943)
dp(2839) = dp(24)*p(572) + p(24)*dp(572) - dp(2297)
dp(2840) = dp(114)*p(183) + p(114)*dp(183) - dp(2299)
dp(2841) = dp(43)*p(401) + p(43)*dp(401) - dp(2302)
dp(2842) = dp(8)*p(1136) + p(8)*dp(1136) - dp(2439)
dp(2843) = dp(1)*p(1877) + p(1)*dp(1877) - dp(2473) - dp(2842)
dp(2844) = dp(1)*p(1878) + p(1)*dp(1878) - dp(2474) - dp(2843)
dp(2845) = dp(1)*p(1879) + p(1)*dp(1879) - dp(2475) - dp(2844)
dp(2846) = dp(13)*p(940) + p(13)*dp(940) - dp(2807)
dp(2847) = dp(2)*p(1835) + p(2)*dp(1835) - dp(2445) - dp(2813)
dp(2848) = dp(32)*p(572) + p(32)*dp(572) - dp(2807)
dp(2849) = dp(1)*p(1881) + p(1)*dp(1881) - dp(2475) - dp(2847)
dp(2850) = dp(73)*p(333) + p(73)*dp(333) - dp(2807)
dp(2851) = dp(1)*p(1883) + p(1)*dp(1883) - dp(2480) - dp(2849)
dp(2852) = dp(150)*p(183) + p(150)*dp(183) - dp(2807)
dp(2853) = dp(6)*p(1374) + p(6)*dp(1374) - dp(2439)
dp(2854) = dp(93)*p(285) + p(93)*dp(285) - dp(2807)
dp(2855) = dp(32)*p(509) + p(32)*dp(509) - dp(2302)
dp(2856) = dp(43)*p(505) + p(43)*dp(505) - dp(2807)
dp(2857) = dp(150)*p(153) + p(150)*dp(153) - dp(2299)
dp(2858) = dp(18)*p(851) + p(18)*dp(851) - dp(2807)
dp(2859) = dp(34)*p(505) + p(34)*dp(505) - dp(2297)
dp(2860) = dp(14)*p(940) + p(14)*dp(940)
dp(2861) = dp(14)*p(941) + p(14)*dp(941)
dp(2862) = dp(130)*p(181) + p(130)*dp(181) - dp(2317)
dp(2863) = dp(41)*p(430) + p(41)*dp(430) - dp(2314)
dp(2864) = dp(4)*p(1180) + p(4)*dp(1180) - dp(2309)
dp(2865) = dp(1)*p(1896) + p(1)*dp(1896) - dp(2502) - dp(2864)
dp(2866) = dp(1)*p(1897) + p(1)*dp(1897) - dp(2503) - dp(2865)
dp(2867) = dp(1)*p(1898) + p(1)*dp(1898) - dp(2504) - dp(2866)
dp(2868) = dp(81)*p(331) + p(81)*dp(331) - dp(2821)
dp(2869) = dp(2)*p(1853) + p(2)*dp(1853) - dp(2461) - dp(2868)
dp(2870) = dp(160)*p(181) + p(160)*dp(181) - dp(2821)
dp(2871) = dp(1)*p(1900) + p(1)*dp(1900) - dp(2504) - dp(2869)
dp(2872) = dp(91)*p(296) + p(91)*dp(296) - dp(2821)
dp(2873) = dp(1)*p(1902) + p(1)*dp(1902) - dp(2508) - dp(2871)
dp(2874) = dp(41)*p(518) + p(41)*dp(518) - dp(2821)
dp(2875) = dp(81)*p(279) + p(81)*dp(279) - dp(2309)
dp(2876) = dp(16)*p(865) + p(16)*dp(865) - dp(2821)
dp(2877) = dp(69)*p(296) + p(69)*dp(296) - dp(2314)
dp(2878) = dp(4)*p(1385) + p(4)*dp(1385) - dp(2821)
dp(2879) = dp(11)*p(865) + p(11)*dp(865) - dp(2317)
dp(2880) = dp(14)*p(861) + p(14)*dp(861)
dp(2881) = dp(14)*p(862) + p(14)*dp(862)
dp(2882) = dp(14)*p(863) + p(14)*dp(863)
dp(2883) = dp(14)*p(726) + p(14)*dp(726)
dp(2884) = dp(14)*p(727) + p(14)*dp(727)
dp(2885) = dp(3)*p(1234) + p(3)*dp(1234) - dp(2536)
dp(2886) = dp(7)*p(1234) + p(7)*dp(1234) - dp(2538)
dp(2887) = dp(49)*p(464) + p(49)*dp(464) - dp(2541)
dp(2888) = dp(3)*p(1401) + p(3)*dp(1401) - dp(2537) - dp(2887)
dp(2889) = dp(1)*p(1916) + p(1)*dp(1916) - dp(2537) - dp(2888)
dp(2890) = dp(178)*p(187) + p(178)*dp(187) - dp(2885)
dp(2891) = dp(39)*p(532) + p(39)*dp(532) - dp(2538)
dp(2892) = dp(3)*p(1402) + p(3)*dp(1402) - dp(2541)
dp(2893) = dp(14)*p(864) + p(14)*dp(864)
dp(2894) = dp(14)*p(865) + p(14)*dp(865)
dp(2895) = dp(14)*p(734) + p(14)*dp(734)
dp(2896) = dp(14)*p(735) + p(14)*dp(735)
dp(2897) = dp(14)*p(736) + p(14)*dp(736)
dp(2898) = dp(9)*p(944) + p(9)*dp(944)
dp(2899) = dp(9)*p(883) + p(9)*dp(883)
dp(2900) = dp(9)*p(778) + p(9)*dp(778)
dp(2901) = dp(9)*p(884) + p(9)*dp(884)
dp(2902) = dp(9)*p(779) + p(9)*dp(779)
dp(2903) = dp(9)*p(780) + p(9)*dp(780)
dp(2904) = dp(9)*p(885) + p(9)*dp(885)
dp(2905) = dp(9)*p(781) + p(9)*dp(781)
dp(2906) = dp(9)*p(886) + p(9)*dp(886)
dp(2907) = dp(9)*p(887) + p(9)*dp(887)
dp(2908) = dp(9)*p(888) + p(9)*dp(888)
dp(2909) = dp(9)*p(945) + p(9)*dp(945)
dp(2910) = dp(24)*p(574) + p(24)*dp(574) - dp(2321)
dp(2911) = dp(114)*p(185) + p(114)*dp(185) - dp(2323)
dp(2912) = dp(46)*p(401) + p(46)*dp(401) - dp(2326)
dp(2913) = dp(1)*p(1936) + p(1)*dp(1936) - dp(2556) - dp(2912)
dp(2914) = dp(1)*p(1937) + p(1)*dp(1937) - dp(2557) - dp(2913)
dp(2915) = dp(1)*p(1938) + p(1)*dp(1938) - dp(2558) - dp(2914)
dp(2916) = dp(13)*p(942) + p(13)*dp(942) - dp(2860)
dp(2917) = dp(2)*p(1881) + p(2)*dp(1881) - dp(2499) - dp(2869)
dp(2918) = dp(32)*p(574) + p(32)*dp(574) - dp(2860)
dp(2919) = dp(1)*p(1940) + p(1)*dp(1940) - dp(2558) - dp(2917)
dp(2920) = dp(73)*p(335) + p(73)*dp(335) - dp(2860)
dp(2921) = dp(1)*p(1942) + p(1)*dp(1942) - dp(2562) - dp(2919)
dp(2922) = dp(150)*p(185) + p(150)*dp(185) - dp(2860)
dp(2923) = dp(13)*p(869) + p(13)*dp(869) - dp(2326)
dp(2924) = dp(95)*p(285) + p(95)*dp(285) - dp(2860)
dp(2925) = dp(73)*p(299) + p(73)*dp(299) - dp(2323)
dp(2926) = dp(46)*p(505) + p(46)*dp(505) - dp(2860)
dp(2927) = dp(84)*p(285) + p(84)*dp(285) - dp(2321)
dp(2928) = dp(14)*p(942) + p(14)*dp(942)
dp(2929) = dp(14)*p(943) + p(14)*dp(943)
dp(2930) = dp(130)*p(183) + p(130)*dp(183) - dp(2320)
dp(2931) = dp(1)*p(1951) + p(1)*dp(1951) - dp(2577) - dp(2930)
dp(2932) = dp(8)*p(1180) + p(8)*dp(1180) - dp(2467)
dp(2933) = dp(1)*p(1953) + p(1)*dp(1953) - dp(2579) - dp(2932)
dp(2934) = dp(1)*p(1954) + p(1)*dp(1954) - dp(2580) - dp(2933)
dp(2935) = dp(81)*p(333) + p(81)*dp(333) - dp(2822)
dp(2936) = dp(2)*p(1900) + p(2)*dp(1900) - dp(2521) - dp(2888)
dp(2937) = dp(160)*p(183) + p(160)*dp(183) - dp(2822)
dp(2938) = dp(1)*p(1956) + p(1)*dp(1956) - dp(2580) - dp(2936)
dp(2939) = dp(93)*p(296) + p(93)*dp(296) - dp(2822)
dp(2940) = dp(45)*p(509) + p(45)*dp(509) - dp(2467)
dp(2941) = dp(43)*p(518) + p(43)*dp(518) - dp(2822)
dp(2942) = dp(1)*p(1960) + p(1)*dp(1960) - dp(2585) - dp(2940)
dp(2943) = dp(18)*p(865) + p(18)*dp(865) - dp(2822)
dp(2944) = dp(34)*p(518) + p(34)*dp(518) - dp(2320)
dp(2945) = dp(14)*p(874) + p(14)*dp(874)
dp(2946) = dp(14)*p(875) + p(14)*dp(875)
dp(2947) = dp(14)*p(876) + p(14)*dp(876)
dp(2948) = dp(14)*p(755) + p(14)*dp(755)
dp(2949) = dp(14)*p(756) + p(14)*dp(756)
dp(2950) = dp(41)*p(464) + p(41)*dp(464) - dp(2347)
dp(2951) = dp(4)*p(1234) + p(4)*dp(1234) - dp(2344)
dp(2952) = dp(1)*p(1969) + p(1)*dp(1969) - dp(2605) - dp(2951)
dp(2953) = dp(1)*p(1970) + p(1)*dp(1970) - dp(2606) - dp(2952)
dp(2954) = dp(91)*p(310) + p(91)*dp(310) - dp(2894)
dp(2955) = dp(2)*p(1916) + p(2)*dp(1916) - dp(2544) - dp(2954)
dp(2956) = dp(41)*p(532) + p(41)*dp(532) - dp(2894)
dp(2957) = dp(1)*p(1972) + p(1)*dp(1972) - dp(2606) - dp(2955)
dp(2958) = dp(16)*p(881) + p(16)*dp(881) - dp(2894)
dp(2959) = dp(69)*p(310) + p(69)*dp(310) - dp(2344)
dp(2960) = dp(4)*p(1402) + p(4)*dp(1402) - dp(2894)
dp(2961) = dp(11)*p(881) + p(11)*dp(881) - dp(2347)
dp(2962) = dp(14)*p(877) + p(14)*dp(877)
dp(2963) = dp(14)*p(878) + p(14)*dp(878)
dp(2964) = dp(14)*p(879) + p(14)*dp(879)
dp(2965) = dp(14)*p(766) + p(14)*dp(766)
dp(2966) = dp(14)*p(767) + p(14)*dp(767)
dp(2967) = dp(14)*p(768) + p(14)*dp(768)
dp(2968) = dp(14)*p(769) + p(14)*dp(769)
dp(2969) = dp(14)*p(770) + p(14)*dp(770)
dp(2970) = dp(3)*p(1290) + p(3)*dp(1290) - dp(2627)
dp(2971) = dp(7)*p(1290) + p(7)*dp(1290) - dp(2628)
dp(2972) = dp(3)*p(1421) + p(3)*dp(1421) - dp(2627) - dp(2971)
dp(2973) = dp(39)*p(578) + p(39)*dp(578) - dp(2970)
dp(2974) = dp(3)*p(1422) + p(3)*dp(1422) - dp(2628)
dp(2975) = dp(14)*p(880) + p(14)*dp(880)
dp(2976) = dp(14)*p(881) + p(14)*dp(881)
dp(2977) = dp(14)*p(774) + p(14)*dp(774)
dp(2978) = dp(14)*p(775) + p(14)*dp(775)
dp(2979) = dp(14)*p(776) + p(14)*dp(776)
dp(2980) = dp(14)*p(777) + p(14)*dp(777)
dp(2981) = dp(9)*p(946) + p(9)*dp(946)
dp(2982) = dp(9)*p(902) + p(9)*dp(902)
dp(2983) = dp(9)*p(810) + p(9)*dp(810)
dp(2984) = dp(9)*p(903) + p(9)*dp(903)
dp(2985) = dp(9)*p(811) + p(9)*dp(811)
dp(2986) = dp(9)*p(904) + p(9)*dp(904)
dp(2987) = dp(9)*p(905) + p(9)*dp(905)
dp(2988) = dp(9)*p(906) + p(9)*dp(906)
dp(2989) = dp(9)*p(947) + p(9)*dp(947)
dp(2990) = dp(24)*p(576) + p(24)*dp(576) - dp(2351)
dp(2991) = dp(114)*p(188) + p(114)*dp(188) - dp(2353)
dp(2992) = dp(50)*p(401) + p(50)*dp(401) - dp(2572)
dp(2993) = dp(1)*p(2000) + p(1)*dp(2000) - dp(2636) - dp(2992)
dp(2994) = dp(1)*p(2001) + p(1)*dp(2001) - dp(2637) - dp(2993)
dp(2995) = dp(13)*p(944) + p(13)*dp(944) - dp(2928)
dp(2996) = dp(2)*p(1940) + p(2)*dp(1940) - dp(2576) - dp(2936)
dp(2997) = dp(32)*p(576) + p(32)*dp(576) - dp(2928)
dp(2998) = dp(1)*p(2003) + p(1)*dp(2003) - dp(2637) - dp(2996)
dp(2999) = dp(73)*p(337) + p(73)*dp(337) - dp(2928)
dp(3000) = dp(6)*p(1406) + p(6)*dp(1406) - dp(2572)
dp(3001) = dp(150)*p(188) + p(150)*dp(188) - dp(2928)
dp(3002) = dp(32)*p(536) + p(32)*dp(536) - dp(2353)
dp(3003) = dp(98)*p(285) + p(98)*dp(285) - dp(2928)
dp(3004) = dp(150)*p(173) + p(150)*dp(173) - dp(2351)
dp(3005) = dp(14)*p(944) + p(14)*dp(944)
dp(3006) = dp(14)*p(945) + p(14)*dp(945)
dp(3007) = dp(130)*p(185) + p(130)*dp(185) - dp(2340)
dp(3008) = dp(46)*p(430) + p(46)*dp(430) - dp(2342)
dp(3009) = dp(1)*p(2013) + p(1)*dp(2013) - dp(2651) - dp(3008)
dp(3010) = dp(1)*p(2014) + p(1)*dp(2014) - dp(2652) - dp(3009)
dp(3011) = dp(81)*p(335) + p(81)*dp(335) - dp(2883)
dp(3012) = dp(2)*p(1956) + p(2)*dp(1956) - dp(2592) - dp(2955)
dp(3013) = dp(160)*p(185) + p(160)*dp(185) - dp(2883)
dp(3014) = dp(1)*p(2016) + p(1)*dp(2016) - dp(2652) - dp(3012)
dp(3015) = dp(95)*p(296) + p(95)*dp(296) - dp(2883)
dp(3016) = dp(81)*p(299) + p(81)*dp(299) - dp(2342)
dp(3017) = dp(46)*p(518) + p(46)*dp(518) - dp(2883)
dp(3018) = dp(84)*p(296) + p(84)*dp(296) - dp(2340)
dp(3019) = dp(14)*p(889) + p(14)*dp(889)
dp(3020) = dp(14)*p(890) + p(14)*dp(890)
dp(3021) = dp(14)*p(891) + p(14)*dp(891)
dp(3022) = dp(14)*p(791) + p(14)*dp(791)
dp(3023) = dp(14)*p(792) + p(14)*dp(792)
dp(3024) = dp(43)*p(464) + p(43)*dp(464) - dp(2349)
dp(3025) = dp(8)*p(1234) + p(8)*dp(1234) - dp(2548)
dp(3026) = dp(1)*p(2027) + p(1)*dp(2027) - dp(2669) - dp(3025)
dp(3027) = dp(93)*p(310) + p(93)*dp(310) - dp(2896)
dp(3028) = dp(2)*p(1972) + p(2)*dp(1972) - dp(2614) - dp(2972)
dp(3029) = dp(43)*p(532) + p(43)*dp(532) - dp(2896)
dp(3030) = dp(153)*p(187) + p(153)*dp(187) - dp(2548)
dp(3031) = dp(18)*p(881) + p(18)*dp(881) - dp(2896)
dp(3032) = dp(34)*p(532) + p(34)*dp(532) - dp(2349)
dp(3033) = dp(14)*p(892) + p(14)*dp(892)
dp(3034) = dp(14)*p(893) + p(14)*dp(893)
dp(3035) = dp(14)*p(894) + p(14)*dp(894)
dp(3036) = dp(14)*p(799) + p(14)*dp(799)
dp(3037) = dp(14)*p(800) + p(14)*dp(800)
dp(3038) = dp(14)*p(801) + p(14)*dp(801)
dp(3039) = dp(14)*p(802) + p(14)*dp(802)
dp(3040) = dp(14)*p(803) + p(14)*dp(803)
dp(3041) = dp(4)*p(1290) + p(4)*dp(1290) - dp(2369)
dp(3042) = dp(1)*p(2038) + p(1)*dp(2038) - dp(2680) - dp(3041)
dp(3043) = dp(16)*p(900) + p(16)*dp(900) - dp(2976)
dp(3044) = dp(2)*p(1986) + p(2)*dp(1986) - dp(2630) - dp(3043)
dp(3045) = dp(4)*p(1422) + p(4)*dp(1422) - dp(2976)
dp(3046) = dp(11)*p(900) + p(11)*dp(900) - dp(2369)
dp(3047) = dp(14)*p(895) + p(14)*dp(895)
dp(3048) = dp(14)*p(896) + p(14)*dp(896)
dp(3049) = dp(14)*p(897) + p(14)*dp(897)
dp(3050) = dp(14)*p(807) + p(14)*dp(807)
dp(3051) = dp(14)*p(808) + p(14)*dp(808)
dp(3052) = dp(14)*p(809) + p(14)*dp(809)
dp(3053) = dp(6)*p(1290) + p(6)*dp(1290) - dp(2681)
dp(3054) = dp(3)*p(1491) + p(3)*dp(1491) - dp(3053)
dp(3055) = dp(14)*p(899) + p(14)*dp(899)
dp(3056) = dp(14)*p(900) + p(14)*dp(900)
dp(3057) = dp(9)*p(949) + p(9)*dp(949)
dp(3058) = dp(9)*p(918) + p(9)*dp(918)
dp(3059) = dp(9)*p(826) + p(9)*dp(826)
dp(3060) = dp(9)*p(919) + p(9)*dp(919)
dp(3061) = dp(9)*p(920) + p(9)*dp(920)
dp(3062) = dp(9)*p(950) + p(9)*dp(950)
dp(3063) = dp(24)*p(579) + p(24)*dp(579) - dp(2370)
dp(3064) = dp(114)*p(191) + p(114)*dp(191) - dp(2372)
dp(3065) = dp(1)*p(2049) + p(1)*dp(2049) - dp(2685) - dp(3064)
dp(3066) = dp(1)*p(2050) + p(1)*dp(2050) - dp(2686) - dp(3065)
dp(3067) = dp(13)*p(946) + p(13)*dp(946) - dp(3005)
dp(3068) = dp(2)*p(2003) + p(2)*dp(2003) - dp(2649) - dp(3012)
dp(3069) = dp(32)*p(579) + p(32)*dp(579) - dp(3005)
dp(3070) = dp(1)*p(2052) + p(1)*dp(2052) - dp(2686) - dp(3068)
dp(3071) = dp(73)*p(340) + p(73)*dp(340) - dp(3005)
dp(3072) = dp(13)*p(904) + p(13)*dp(904) - dp(2372)
dp(3073) = dp(150)*p(191) + p(150)*dp(191) - dp(3005)
dp(3074) = dp(73)*p(323) + p(73)*dp(323) - dp(2370)
dp(3075) = dp(14)*p(946) + p(14)*dp(946)
dp(3076) = dp(14)*p(947) + p(14)*dp(947)
dp(3077) = dp(130)*p(188) + p(130)*dp(188) - dp(2363)
dp(3078) = dp(50)*p(430) + p(50)*dp(430) - dp(2601)
dp(3079) = dp(1)*p(2060) + p(1)*dp(2060) - dp(2696) - dp(3078)
dp(3080) = dp(81)*p(337) + p(81)*dp(337) - dp(2948)
dp(3081) = dp(2)*p(2016) + p(2)*dp(2016) - dp(2660) - dp(3028)
dp(3082) = dp(160)*p(188) + p(160)*dp(188) - dp(2948)
dp(3083) = dp(45)*p(536) + p(45)*dp(536) - dp(2601)
dp(3084) = dp(98)*p(296) + p(98)*dp(296) - dp(2948)
dp(3085) = dp(160)*p(173) + p(160)*dp(173) - dp(2363)
dp(3086) = dp(14)*p(907) + p(14)*dp(907)
dp(3087) = dp(14)*p(908) + p(14)*dp(908)
dp(3088) = dp(14)*p(909) + p(14)*dp(909)
dp(3089) = dp(14)*p(818) + p(14)*dp(818)
dp(3090) = dp(14)*p(819) + p(14)*dp(819)
dp(3091) = dp(2)*p(2026) + p(2)*dp(2026) - dp(2671) - dp(3041)
dp(3092) = dp(1)*p(2070) + p(1)*dp(2070) - dp(2706) - dp(3091)
dp(3093) = dp(95)*p(310) + p(95)*dp(310) - dp(2897)
dp(3094) = dp(2)*p(2029) + p(2)*dp(2029) - dp(2674) - dp(3044)
dp(3095) = dp(46)*p(532) + p(46)*dp(532) - dp(2897)
dp(3096) = dp(1)*p(2072) + p(1)*dp(2072) - dp(2706) - dp(3094)
dp(3097) = dp(14)*p(910) + p(14)*dp(910)
dp(3098) = dp(14)*p(911) + p(14)*dp(911)
dp(3099) = dp(14)*p(912) + p(14)*dp(912)
dp(3100) = dp(14)*p(823) + p(14)*dp(823)
dp(3101) = dp(14)*p(824) + p(14)*dp(824)
dp(3102) = dp(14)*p(825) + p(14)*dp(825)
dp(3103) = dp(8)*p(1290) + p(8)*dp(1290) - dp(2632)
dp(3104) = dp(18)*p(900) + p(18)*dp(900) - dp(2978)
dp(3105) = dp(34)*p(578) + p(34)*dp(578) - dp(2632)
dp(3106) = dp(14)*p(914) + p(14)*dp(914)
dp(3107) = dp(14)*p(915) + p(14)*dp(915)
dp(3108) = dp(14)*p(916) + p(14)*dp(916)
dp(3109) = dp(9)*p(952) + p(9)*dp(952)
dp(3110) = dp(9)*p(928) + p(9)*dp(928)
dp(3111) = dp(9)*p(929) + p(9)*dp(929)
dp(3112) = dp(9)*p(953) + p(9)*dp(953)
dp(3113) = dp(24)*p(582) + p(24)*dp(582) - dp(2379)
dp(3114) = dp(114)*p(195) + p(114)*dp(195) - dp(2692)
dp(3115) = dp(1)*p(2079) + p(1)*dp(2079) - dp(2711) - dp(3114)
dp(3116) = dp(13)*p(949) + p(13)*dp(949) - dp(3075)
dp(3117) = dp(2)*p(2052) + p(2)*dp(2052) - dp(2694) - dp(3081)
dp(3118) = dp(32)*p(582) + p(32)*dp(582) - dp(3075)
dp(3119) = dp(6)*p(1446) + p(6)*dp(1446) - dp(2692)
dp(3120) = dp(73)*p(343) + p(73)*dp(343) - dp(3075)
dp(3121) = dp(32)*p(561) + p(32)*dp(561) - dp(2379)
dp(3122) = dp(14)*p(949) + p(14)*dp(949)
dp(3123) = dp(14)*p(950) + p(14)*dp(950)
dp(3124) = dp(130)*p(191) + p(130)*dp(191) - dp(2377)
dp(3125) = dp(1)*p(2086) + p(1)*dp(2086) - dp(2716) - dp(3124)
dp(3126) = dp(81)*p(340) + p(81)*dp(340) - dp(3022)
dp(3127) = dp(2)*p(2062) + p(2)*dp(2062) - dp(2701) - dp(3094)
dp(3128) = dp(160)*p(191) + p(160)*dp(191) - dp(3022)
dp(3129) = dp(81)*p(323) + p(81)*dp(323) - dp(2377)
dp(3130) = dp(14)*p(921) + p(14)*dp(921)
dp(3131) = dp(14)*p(922) + p(14)*dp(922)
dp(3132) = dp(14)*p(923) + p(14)*dp(923)
dp(3133) = dp(14)*p(830) + p(14)*dp(830)
dp(3134) = dp(14)*p(831) + p(14)*dp(831)
dp(3135) = dp(50)*p(464) + p(50)*dp(464) - dp(2626)
dp(3136) = dp(98)*p(310) + p(98)*dp(310) - dp(2968)
dp(3137) = dp(173)*p(187) + p(173)*dp(187) - dp(2626)
dp(3138) = dp(14)*p(924) + p(14)*dp(924)
dp(3139) = dp(14)*p(925) + p(14)*dp(925)
dp(3140) = dp(14)*p(926) + p(14)*dp(926)
dp(3141) = dp(9)*p(955) + p(9)*dp(955)
dp(3142) = dp(9)*p(956) + p(9)*dp(956)
dp(3143) = dp(24)*p(585) + p(24)*dp(585) - dp(2382)
dp(3144) = dp(1)*p(2093) + p(1)*dp(2093) - dp(2720) - dp(3143)
dp(3145) = dp(13)*p(952) + p(13)*dp(952) - dp(3122)
dp(3146) = dp(2)*p(2081) + p(2)*dp(2081) - dp(2715) - dp(3127)
dp(3147) = dp(32)*p(585) + p(32)*dp(585) - dp(3122)
dp(3148) = dp(13)*p(929) + p(13)*dp(929) - dp(2382)
dp(3149) = dp(14)*p(952) + p(14)*dp(952)
dp(3150) = dp(14)*p(953) + p(14)*dp(953)
dp(3151) = dp(130)*p(195) + p(130)*dp(195) - dp(2705)
dp(3152) = dp(81)*p(343) + p(81)*dp(343) - dp(3089)
dp(3153) = dp(45)*p(561) + p(45)*dp(561) - dp(2705)
dp(3154) = dp(14)*p(930) + p(14)*dp(930)
dp(3155) = dp(14)*p(931) + p(14)*dp(931)
dp(3156) = dp(14)*p(932) + p(14)*dp(932)
dp(3157) = dp(9)*p(959) + p(9)*dp(959)
dp(3158) = dp(24)*p(589) + p(24)*dp(589) - dp(2722)
dp(3159) = dp(13)*p(955) + p(13)*dp(955) - dp(3149)
dp(3160) = dp(6)*p(1471) + p(6)*dp(1471) - dp(2722)
dp(3161) = dp(14)*p(955) + p(14)*dp(955)
dp(3162) = dp(14)*p(956) + p(14)*dp(956)
dp(3163) = dp(9)*p(832) + p(9)*dp(832)
dp(3164) = dp(9)*p(833) + p(9)*dp(833)
dp(3165) = dp(9)*p(834) + p(9)*dp(834)
dp(3166) = dp(9)*p(835) + p(9)*dp(835)
dp(3167) = dp(9)*p(836) + p(9)*dp(836)
dp(3168) = dp(9)*p(934) + p(9)*dp(934)
dp(3169) = dp(9)*p(837) + p(9)*dp(837)
dp(3170) = dp(9)*p(935) + p(9)*dp(935)
dp(3171) = dp(9)*p(838) + p(9)*dp(838)
dp(3172) = dp(9)*p(936) + p(9)*dp(936)
dp(3173) = dp(9)*p(937) + p(9)*dp(937)
dp(3174) = dp(9)*p(958) + p(9)*dp(958)
dp(3175) = dp(3)*p(1481) + p(3)*dp(1481) - dp(2723)
dp(3176) = dp(39)*p(570) + p(39)*dp(570) - dp(2725)
dp(3177) = dp(178)*p(181) + p(178)*dp(181) - dp(2728)
dp(3178) = dp(41)*p(566) + p(41)*dp(566) - dp(2732)
dp(3179) = dp(4)*p(1476) + p(4)*dp(1476) - dp(2737)
dp(3180) = dp(1)*p(2112) + p(1)*dp(2112) - dp(2733) - dp(3179)
dp(3181) = dp(1)*p(2113) + p(1)*dp(2113) - dp(2737)
dp(3182) = dp(1)*p(2114) + p(1)*dp(2114) - dp(2738) - dp(3180)
dp(3183) = dp(21)*p(843) + p(21)*dp(843) - dp(2732)
dp(3184) = dp(1)*p(2116) + p(1)*dp(2116) - dp(2742) - dp(3182)
dp(3185) = dp(101)*p(279) + p(101)*dp(279) - dp(2728)
dp(3186) = dp(1)*p(2118) + p(1)*dp(2118) - dp(2745) - dp(3184)
dp(3187) = dp(69)*p(346) + p(69)*dp(346) - dp(2725)
dp(3188) = dp(1)*p(2120) + p(1)*dp(2120) - dp(2747) - dp(3186)
dp(3189) = dp(11)*p(958) + p(11)*dp(958) - dp(2723)
dp(3190) = dp(1)*p(2122) + p(1)*dp(2122) - dp(2753) - dp(2752)
dp(3191) = dp(1)*p(2123) + p(1)*dp(2123) - dp(2761)
dp(3192) = dp(3)*p(1483) + p(3)*dp(1483) - dp(2763)
dp(3193) = dp(39)*p(572) + p(39)*dp(572) - dp(2765)
dp(3194) = dp(178)*p(183) + p(178)*dp(183) - dp(2768)
dp(3195) = dp(43)*p(566) + p(43)*dp(566) - dp(2772)
dp(3196) = dp(1)*p(2127) + p(1)*dp(2127) - dp(2769) - dp(3195)
dp(3197) = dp(2)*p(2113) + p(2)*dp(2113) - dp(2757)
dp(3198) = dp(1)*p(2128) + p(1)*dp(2128) - dp(2773) - dp(3196)
dp(3199) = dp(7)*p(1374) + p(7)*dp(1374) - dp(2772)
dp(3200) = dp(1)*p(2130) + p(1)*dp(2130) - dp(2777) - dp(3198)
dp(3201) = dp(49)*p(509) + p(49)*dp(509) - dp(2768)
dp(3202) = dp(1)*p(2132) + p(1)*dp(2132) - dp(2780) - dp(3200)
dp(3203) = dp(153)*p(194) + p(153)*dp(194) - dp(2765)
dp(3204) = dp(1)*p(2134) + p(1)*dp(2134) - dp(2782) - dp(3202)
dp(3205) = dp(34)*p(588) + p(34)*dp(588) - dp(2763)
dp(3206) = dp(6)*p(1481) + p(6)*dp(1481) - dp(2762)
dp(3207) = dp(1)*p(2137) + p(1)*dp(2137) - dp(2792) - dp(2790)
dp(3208) = dp(2)*p(2123) + p(2)*dp(2123) - dp(2762)
dp(3209) = dp(1)*p(2139) + p(1)*dp(2139) - dp(2813) - dp(2812)
dp(3210) = dp(1)*p(2140) + p(1)*dp(2140) - dp(2819)
dp(3211) = dp(3)*p(1485) + p(3)*dp(1485) - dp(2823)
dp(3212) = dp(39)*p(574) + p(39)*dp(574) - dp(2825)
dp(3213) = dp(178)*p(185) + p(178)*dp(185) - dp(2828)
dp(3214) = dp(46)*p(566) + p(46)*dp(566) - dp(2832)
dp(3215) = dp(1)*p(2144) + p(1)*dp(2144) - dp(2829) - dp(3214)
dp(3216) = dp(1)*p(2145) + p(1)*dp(2145) - dp(2832)
dp(3217) = dp(1)*p(2146) + p(1)*dp(2146) - dp(2833) - dp(3215)
dp(3218) = dp(21)*p(869) + p(21)*dp(869) - dp(2828)
dp(3219) = dp(1)*p(2148) + p(1)*dp(2148) - dp(2836) - dp(3217)
dp(3220) = dp(101)*p(299) + p(101)*dp(299) - dp(2825)
dp(3221) = dp(1)*p(2150) + p(1)*dp(2150) - dp(2838) - dp(3219)
dp(3222) = dp(84)*p(346) + p(84)*dp(346) - dp(2823)
dp(3223) = dp(6)*p(1483) + p(6)*dp(1483) - dp(2807)
dp(3224) = dp(1)*p(2153) + p(1)*dp(2153) - dp(2847) - dp(2845)
dp(3225) = dp(8)*p(1369) + p(8)*dp(1369) - dp(2807)
dp(3226) = dp(45)*p(570) + p(45)*dp(570) - dp(2821)
dp(3227) = dp(1)*p(2156) + p(1)*dp(2156) - dp(2869) - dp(2867)
dp(3228) = dp(2)*p(2140) + p(2)*dp(2140) - dp(2821)
dp(3229) = dp(1)*p(2158) + p(1)*dp(2158) - dp(2888) - dp(2887)
dp(3230) = dp(1)*p(2159) + p(1)*dp(2159) - dp(2892)
dp(3231) = dp(3)*p(1487) + p(3)*dp(1487) - dp(2898)
dp(3232) = dp(39)*p(576) + p(39)*dp(576) - dp(2900)
dp(3233) = dp(178)*p(188) + p(178)*dp(188) - dp(2903)
dp(3234) = dp(1)*p(2162) + p(1)*dp(2162) - dp(2901) - dp(3233)
dp(3235) = dp(2)*p(2145) + p(2)*dp(2145) - dp(2853)
dp(3236) = dp(1)*p(2163) + p(1)*dp(2163) - dp(2904) - dp(3234)
dp(3237) = dp(7)*p(1406) + p(7)*dp(1406) - dp(2903)
dp(3238) = dp(1)*p(2165) + p(1)*dp(2165) - dp(2907) - dp(3236)
dp(3239) = dp(49)*p(536) + p(49)*dp(536) - dp(2900)
dp(3240) = dp(1)*p(2167) + p(1)*dp(2167) - dp(2909) - dp(3238)
dp(3241) = dp(173)*p(194) + p(173)*dp(194) - dp(2898)
dp(3242) = dp(6)*p(1485) + p(6)*dp(1485) - dp(2860)
dp(3243) = dp(1)*p(2170) + p(1)*dp(2170) - dp(2917) - dp(2915)
dp(3244) = dp(22)*p(851) + p(22)*dp(851) - dp(2860)
dp(3245) = dp(45)*p(572) + p(45)*dp(572) - dp(2822)
dp(3246) = dp(1)*p(2173) + p(1)*dp(2173) - dp(2936) - dp(2934)
dp(3247) = dp(8)*p(1385) + p(8)*dp(1385) - dp(2822)
dp(3248) = dp(181)*p(187) + p(181)*dp(187) - dp(2894)
dp(3249) = dp(1)*p(2176) + p(1)*dp(2176) - dp(2955) - dp(2953)
dp(3250) = dp(2)*p(2159) + p(2)*dp(2159) - dp(2894)
dp(3251) = dp(1)*p(2178) + p(1)*dp(2178) - dp(2972) - dp(2971)
dp(3252) = dp(1)*p(2179) + p(1)*dp(2179) - dp(2974)
dp(3253) = dp(14)*p(882) + p(14)*dp(882)
dp(3254) = dp(3)*p(1489) + p(3)*dp(1489) - dp(2981)
dp(3255) = dp(39)*p(579) + p(39)*dp(579) - dp(2983)
dp(3256) = dp(178)*p(191) + p(178)*dp(191) - dp(2986)
dp(3257) = dp(1)*p(2183) + p(1)*dp(2183) - dp(2984) - dp(3256)
dp(3258) = dp(1)*p(2184) + p(1)*dp(2184) - dp(2986)
dp(3259) = dp(1)*p(2185) + p(1)*dp(2185) - dp(2987) - dp(3257)
dp(3260) = dp(21)*p(904) + p(21)*dp(904) - dp(2983)
dp(3261) = dp(1)*p(2187) + p(1)*dp(2187) - dp(2989) - dp(3259)
dp(3262) = dp(101)*p(323) + p(101)*dp(323) - dp(2981)
dp(3263) = dp(6)*p(1487) + p(6)*dp(1487) - dp(2928)
dp(3264) = dp(1)*p(2190) + p(1)*dp(2190) - dp(2996) - dp(2994)
dp(3265) = dp(50)*p(505) + p(50)*dp(505) - dp(2928)
dp(3266) = dp(45)*p(574) + p(45)*dp(574) - dp(2883)
dp(3267) = dp(1)*p(2193) + p(1)*dp(2193) - dp(3012) - dp(3010)
dp(3268) = dp(22)*p(865) + p(22)*dp(865) - dp(2883)
dp(3269) = dp(183)*p(187) + p(183)*dp(187) - dp(2896)
dp(3270) = dp(1)*p(2196) + p(1)*dp(2196) - dp(3028) - dp(3026)
dp(3271) = dp(8)*p(1402) + p(8)*dp(1402) - dp(2896)
dp(3272) = dp(41)*p(578) + p(41)*dp(578) - dp(2976)
dp(3273) = dp(1)*p(2200) + p(1)*dp(2200) - dp(3044) - dp(3042)
dp(3274) = dp(2)*p(2179) + p(2)*dp(2179) - dp(2976)
dp(3275) = dp(14)*p(898) + p(14)*dp(898)
dp(3276) = dp(4)*p(1441) + p(4)*dp(1441) - dp(2683) - dp(3103)
dp(3277) = dp(1)*p(2204) + p(1)*dp(2204) - dp(3054)
dp(3278) = dp(14)*p(901) + p(14)*dp(901)
dp(3279) = dp(3)*p(1492) + p(3)*dp(1492) - dp(3057)
dp(3280) = dp(39)*p(582) + p(39)*dp(582) - dp(3059)
dp(3281) = dp(1)*p(2207) + p(1)*dp(2207) - dp(3058) - dp(3280)
dp(3282) = dp(2)*p(2184) + p(2)*dp(2184) - dp(3000)
dp(3283) = dp(1)*p(2208) + p(1)*dp(2208) - dp(3060) - dp(3281)
dp(3284) = dp(7)*p(1446) + p(7)*dp(1446) - dp(3059)
dp(3285) = dp(1)*p(2210) + p(1)*dp(2210) - dp(3062) - dp(3283)
dp(3286) = dp(49)*p(561) + p(49)*dp(561) - dp(3057)
dp(3287) = dp(6)*p(1489) + p(6)*dp(1489) - dp(3005)
dp(3288) = dp(1)*p(2213) + p(1)*dp(2213) - dp(3068) - dp(3066)
dp(3289) = dp(102)*p(285) + p(102)*dp(285) - dp(3005)
dp(3290) = dp(45)*p(576) + p(45)*dp(576) - dp(2948)
dp(3291) = dp(1)*p(2216) + p(1)*dp(2216) - dp(3081) - dp(3079)
dp(3292) = dp(50)*p(518) + p(50)*dp(518) - dp(2948)
dp(3293) = dp(185)*p(187) + p(185)*dp(187) - dp(2897)
dp(3294) = dp(1)*p(2219) + p(1)*dp(2219) - dp(3094) - dp(3092)
dp(3295) = dp(22)*p(881) + p(22)*dp(881) - dp(2897)
dp(3296) = dp(14)*p(913) + p(14)*dp(913)
dp(3297) = dp(43)*p(578) + p(43)*dp(578) - dp(2978)
dp(3298) = dp(1)*p(2223) + p(1)*dp(2223) - dp(3105) - dp(3103)
dp(3299) = dp(8)*p(1422) + p(8)*dp(1422) - dp(2978)
dp(3300) = dp(14)*p(917) + p(14)*dp(917)
dp(3301) = dp(4)*p(1491) + p(4)*dp(1491) - dp(3056)
dp(3302) = dp(2)*p(2203) + p(2)*dp(2203) - dp(3055) - dp(3301)
dp(3303) = dp(2)*p(2204) + p(2)*dp(2204) - dp(3056)
dp(3304) = dp(14)*p(948) + p(14)*dp(948)
dp(3305) = dp(3)*p(1495) + p(3)*dp(1495) - dp(3109)
dp(3306) = dp(39)*p(585) + p(39)*dp(585) - dp(3111)
dp(3307) = dp(1)*p(2227) + p(1)*dp(2227) - dp(3110) - dp(3306)
dp(3308) = dp(1)*p(2228) + p(1)*dp(2228) - dp(3111)
dp(3309) = dp(1)*p(2229) + p(1)*dp(2229) - dp(3112) - dp(3307)
dp(3310) = dp(21)*p(929) + p(21)*dp(929) - dp(3109)
dp(3311) = dp(6)*p(1492) + p(6)*dp(1492) - dp(3075)
dp(3312) = dp(1)*p(2232) + p(1)*dp(2232) - dp(3117) - dp(3115)
dp(3313) = dp(150)*p(195) + p(150)*dp(195) - dp(3075)
dp(3314) = dp(45)*p(579) + p(45)*dp(579) - dp(3022)
dp(3315) = dp(1)*p(2235) + p(1)*dp(2235) - dp(3127) - dp(3125)
dp(3316) = dp(102)*p(296) + p(102)*dp(296) - dp(3022)
dp(3317) = dp(187)*p(188) + p(187)*dp(188) - dp(2968)
dp(3318) = dp(1)*p(2239) + p(1)*dp(2239) - dp(3137) - dp(3135)
dp(3319) = dp(50)*p(532) + p(50)*dp(532) - dp(2968)
dp(3320) = dp(14)*p(927) + p(14)*dp(927)
dp(3321) = dp(46)*p(578) + p(46)*dp(578) - dp(2980)
dp(3322) = dp(2)*p(2223) + p(2)*dp(2223) - dp(3107) - dp(3302)
dp(3323) = dp(22)*p(900) + p(22)*dp(900) - dp(2980)
dp(3324) = dp(14)*p(951) + p(14)*dp(951)
dp(3325) = dp(3)*p(1498) + p(3)*dp(1498) - dp(3141)
dp(3326) = dp(2)*p(2227) + p(2)*dp(2227) - dp(3118) - dp(3114)
dp(3327) = dp(2)*p(2228) + p(2)*dp(2228) - dp(3119)
dp(3328) = dp(1)*p(2243) + p(1)*dp(2243) - dp(3142) - dp(3326)
dp(3329) = dp(7)*p(1471) + p(7)*dp(1471) - dp(3141)
dp(3330) = dp(6)*p(1495) + p(6)*dp(1495) - dp(3122)
dp(3331) = dp(1)*p(2246) + p(1)*dp(2246) - dp(3146) - dp(3144)
dp(3332) = dp(73)*p(347) + p(73)*dp(347) - dp(3122)
dp(3333) = dp(45)*p(582) + p(45)*dp(582) - dp(3089)
dp(3334) = dp(1)*p(2249) + p(1)*dp(2249) - dp(3153) - dp(3151)
dp(3335) = dp(160)*p(195) + p(160)*dp(195) - dp(3089)
dp(3336) = dp(14)*p(933) + p(14)*dp(933)
dp(3337) = dp(187)*p(191) + p(187)*dp(191) - dp(3039)
dp(3338) = dp(2)*p(2239) + p(2)*dp(2239) - dp(3139) - dp(3322)
dp(3339) = dp(102)*p(310) + p(102)*dp(310) - dp(3039)
dp(3340) = dp(14)*p(954) + p(14)*dp(954)
dp(3341) = dp(3)*p(1501) + p(3)*dp(1501) - dp(3157)
dp(3342) = dp(2)*p(2243) + p(2)*dp(2243) - dp(3147) - dp(3144)
dp(3343) = dp(1)*p(2253) + p(1)*dp(2253) - dp(3157)
dp(3344) = dp(6)*p(1498) + p(6)*dp(1498) - dp(3149)
dp(3345) = dp(1)*p(2255) + p(1)*dp(2255) - dp(3160) - dp(3158)
dp(3346) = dp(32)*p(589) + p(32)*dp(589) - dp(3149)
dp(3347) = dp(45)*p(585) + p(45)*dp(585) - dp(3133)
dp(3348) = dp(2)*p(2249) + p(2)*dp(2249) - dp(3155) - dp(3338)
dp(3349) = dp(81)*p(347) + p(81)*dp(347) - dp(3133)
dp(3350) = dp(14)*p(957) + p(14)*dp(957)
dp(3351) = dp(2)*p(2252) + p(2)*dp(2252) - dp(3159) - dp(3158)
dp(3352) = dp(2)*p(2253) + p(2)*dp(2253) - dp(3160)
dp(3353) = dp(6)*p(1501) + p(6)*dp(1501) - dp(3161)
dp(3354) = dp(2)*p(2255) + p(2)*dp(2255) - dp(3162) - dp(3348)
dp(3355) = dp(13)*p(959) + p(13)*dp(959) - dp(3161)
dp(3356) = dp(14)*p(959) + p(14)*dp(959)
dp(3357) = dp(3)*p(1476) + p(3)*dp(1476) - dp(3168)
dp(3358) = dp(7)*p(1476) + p(7)*dp(1476) - dp(3165)
dp(3359) = dp(1)*p(2259) + p(1)*dp(2259) - dp(3170) - dp(3358)
dp(3360) = dp(178)*p(194) + p(178)*dp(194) - dp(3166)
dp(3361) = dp(39)*p(588) + p(39)*dp(588) - dp(3171)
dp(3362) = dp(3)*p(1504) + p(3)*dp(1504) - dp(3174)
dp(3363) = dp(1)*p(2263) + p(1)*dp(2263) - dp(3175)
dp(3364) = dp(2)*p(2288) + p(2)*dp(2288) - dp(3363)
dp(3365) = dp(1)*p(2265) + p(1)*dp(2265) - dp(3192)
dp(3366) = dp(8)*p(1504) + p(8)*dp(1504) - dp(3365)
dp(3367) = dp(1)*p(2267) + p(1)*dp(2267) - dp(3211)
dp(3368) = dp(22)*p(958) + p(22)*dp(958) - dp(3367)
dp(3369) = dp(1)*p(2269) + p(1)*dp(2269) - dp(3231)
dp(3370) = dp(50)*p(588) + p(50)*dp(588) - dp(3369)
dp(3371) = dp(1)*p(2271) + p(1)*dp(2271) - dp(3254)
dp(3372) = dp(102)*p(346) + p(102)*dp(346) - dp(3371)
dp(3373) = dp(1)*p(2273) + p(1)*dp(2273) - dp(3279)
dp(3374) = dp(194)*p(195) + p(194)*dp(195) - dp(3373)
dp(3375) = dp(6)*p(1491) + p(6)*dp(1491) - dp(3304)
dp(3376) = dp(1)*p(2276) + p(1)*dp(2276) - dp(3305)
dp(3377) = dp(101)*p(347) + p(101)*dp(347) - dp(3376)
dp(3378) = dp(8)*p(1491) + p(8)*dp(1491) - dp(3278)
dp(3379) = dp(1)*p(2279) + p(1)*dp(2279) - dp(3325)
dp(3380) = dp(49)*p(589) + p(49)*dp(589) - dp(3379)
dp(3381) = dp(2)*p(2278) + p(2)*dp(2278) - dp(3324) - dp(3378)
dp(3382) = dp(1)*p(2282) + p(1)*dp(2282) - dp(3341)
dp(3383) = dp(21)*p(959) + p(21)*dp(959) - dp(3382)
dp(3384) = dp(187)*p(195) + p(187)*dp(195) - dp(3296)
dp(3385) = dp(1)*p(2285) + p(1)*dp(2285) - dp(3351)
dp(3386) = dp(7)*p(1505) + p(7)*dp(1505) - dp(3385)
dp(3387) = dp(45)*p(589) + p(45)*dp(589) - dp(3336)
dp(3388) = dp(2)*p(2285) + p(2)*dp(2285) - dp(3353)
dp(3389) = dp(1)*p(2289) + p(1)*dp(2289) - dp(3388)
dp(3390) = dp(6)*p(1505) + p(6)*dp(1505) - dp(3356)
dp(3391) = dp(1)*p(2288) + p(1)*dp(2288) - dp(3362)
dp(3392) = dp(2)*p(2289) + p(2)*dp(2289) - dp(3390)

do j=1,3393
  dbm1(j)=dp(j-1)
enddo

dbm2(1)=dbm1(4)
dbm2(2)=dbm1(10)
dbm2(3)=dbm1(11)
dbm2(4)=dbm1(12)
dbm2(5)=dbm1(13)
dbm2(6)=dbm1(15)
dbm2(7)=dbm1(16)
dbm2(8)=dbm1(24)
dbm2(9)=dbm1(25)
dbm2(10)=dbm1(26)
dbm2(11)=dbm1(27)
dbm2(12)=dbm1(28)
dbm2(13)=dbm1(29)
dbm2(14)=dbm1(30)
dbm2(15)=dbm1(31)
dbm2(16)=dbm1(32)
dbm2(17)=dbm1(34)
dbm2(18)=dbm1(35)
dbm2(19)=dbm1(36)
dbm2(20)=dbm1(37)
dbm2(21)=dbm1(39)
dbm2(22)=dbm1(40)
dbm2(23)=dbm1(41)
dbm2(24)=dbm1(52)
dbm2(25)=dbm1(53)
dbm2(26)=dbm1(54)
dbm2(27)=dbm1(55)
dbm2(28)=dbm1(56)
dbm2(29)=dbm1(57)
dbm2(30)=dbm1(58)
dbm2(31)=dbm1(59)
dbm2(32)=dbm1(60)
dbm2(33)=dbm1(61)
dbm2(34)=dbm1(62)
dbm2(35)=dbm1(63)
dbm2(36)=dbm1(64)
dbm2(37)=dbm1(65)
dbm2(38)=dbm1(66)
dbm2(39)=dbm1(67)
dbm2(40)=dbm1(68)
dbm2(41)=dbm1(69)
dbm2(42)=dbm1(70)
dbm2(43)=dbm1(71)
dbm2(44)=dbm1(72)
dbm2(45)=dbm1(73)
dbm2(46)=dbm1(75)
dbm2(47)=dbm1(76)
dbm2(48)=dbm1(77)
dbm2(49)=dbm1(78)
dbm2(50)=dbm1(79)
dbm2(51)=dbm1(81)
dbm2(52)=dbm1(83)
dbm2(53)=dbm1(84)
dbm2(54)=dbm1(85)
dbm2(55)=dbm1(86)
dbm2(56)=dbm1(87)
dbm2(57)=dbm1(89)
dbm2(58)=dbm1(90)
dbm2(59)=dbm1(91)
dbm2(60)=dbm1(104)
dbm2(61)=dbm1(105)
dbm2(62)=dbm1(106)
dbm2(63)=dbm1(107)
dbm2(64)=dbm1(108)
dbm2(65)=dbm1(109)
dbm2(66)=dbm1(110)
dbm2(67)=dbm1(111)
dbm2(68)=dbm1(112)
dbm2(69)=dbm1(113)
dbm2(70)=dbm1(114)
dbm2(71)=dbm1(115)
dbm2(72)=dbm1(116)
dbm2(73)=dbm1(117)
dbm2(74)=dbm1(118)
dbm2(75)=dbm1(119)
dbm2(76)=dbm1(120)
dbm2(77)=dbm1(121)
dbm2(78)=dbm1(122)
dbm2(79)=dbm1(123)
dbm2(80)=dbm1(124)
dbm2(81)=dbm1(125)
dbm2(82)=dbm1(126)
dbm2(83)=dbm1(127)
dbm2(84)=dbm1(128)
dbm2(85)=dbm1(129)
dbm2(86)=dbm1(130)
dbm2(87)=dbm1(131)
dbm2(88)=dbm1(132)
dbm2(89)=dbm1(133)
dbm2(90)=dbm1(134)
dbm2(91)=dbm1(135)
dbm2(92)=dbm1(136)
dbm2(93)=dbm1(137)
dbm2(94)=dbm1(138)
dbm2(95)=dbm1(139)
dbm2(96)=dbm1(140)
dbm2(97)=dbm1(141)
dbm2(98)=dbm1(142)
dbm2(99)=dbm1(143)
dbm2(100)=dbm1(144)
dbm2(101)=dbm1(145)
dbm2(102)=dbm1(146)
dbm2(103)=dbm1(147)
dbm2(104)=dbm1(148)
dbm2(105)=dbm1(149)
dbm2(106)=dbm1(150)
dbm2(107)=dbm1(152)
dbm2(108)=dbm1(153)
dbm2(109)=dbm1(154)
dbm2(110)=dbm1(155)
dbm2(111)=dbm1(156)
dbm2(112)=dbm1(157)
dbm2(113)=dbm1(158)
dbm2(114)=dbm1(160)
dbm2(115)=dbm1(162)
dbm2(116)=dbm1(163)
dbm2(117)=dbm1(164)
dbm2(118)=dbm1(165)
dbm2(119)=dbm1(166)
dbm2(120)=dbm1(167)
dbm2(121)=dbm1(169)
dbm2(122)=dbm1(170)
dbm2(123)=dbm1(172)
dbm2(124)=dbm1(173)
dbm2(125)=dbm1(174)
dbm2(126)=dbm1(175)
dbm2(127)=dbm1(176)
dbm2(128)=dbm1(178)
dbm2(129)=dbm1(179)
dbm2(130)=dbm1(180)
dbm2(131)=dbm1(181)
dbm2(132)=dbm1(197)
dbm2(133)=dbm1(198)
dbm2(134)=dbm1(199)
dbm2(135)=dbm1(200)
dbm2(136)=dbm1(201)
dbm2(137)=dbm1(202)
dbm2(138)=dbm1(203)
dbm2(139)=dbm1(204)
dbm2(140)=dbm1(205)
dbm2(141)=dbm1(206)
dbm2(142)=dbm1(207)
dbm2(143)=dbm1(208)
dbm2(144)=dbm1(209)
dbm2(145)=dbm1(210)
dbm2(146)=dbm1(211)
dbm2(147)=dbm1(212)
dbm2(148)=dbm1(213)
dbm2(149)=dbm1(214)
dbm2(150)=dbm1(215)
dbm2(151)=dbm1(216)
dbm2(152)=dbm1(217)
dbm2(153)=dbm1(218)
dbm2(154)=dbm1(219)
dbm2(155)=dbm1(220)
dbm2(156)=dbm1(221)
dbm2(157)=dbm1(222)
dbm2(158)=dbm1(223)
dbm2(159)=dbm1(224)
dbm2(160)=dbm1(225)
dbm2(161)=dbm1(226)
dbm2(162)=dbm1(227)
dbm2(163)=dbm1(228)
dbm2(164)=dbm1(229)
dbm2(165)=dbm1(230)
dbm2(166)=dbm1(231)
dbm2(167)=dbm1(232)
dbm2(168)=dbm1(233)
dbm2(169)=dbm1(234)
dbm2(170)=dbm1(235)
dbm2(171)=dbm1(236)
dbm2(172)=dbm1(237)
dbm2(173)=dbm1(238)
dbm2(174)=dbm1(239)
dbm2(175)=dbm1(240)
dbm2(176)=dbm1(241)
dbm2(177)=dbm1(242)
dbm2(178)=dbm1(243)
dbm2(179)=dbm1(244)
dbm2(180)=dbm1(245)
dbm2(181)=dbm1(246)
dbm2(182)=dbm1(247)
dbm2(183)=dbm1(248)
dbm2(184)=dbm1(249)
dbm2(185)=dbm1(250)
dbm2(186)=dbm1(251)
dbm2(187)=dbm1(252)
dbm2(188)=dbm1(253)
dbm2(189)=dbm1(254)
dbm2(190)=dbm1(255)
dbm2(191)=dbm1(256)
dbm2(192)=dbm1(257)
dbm2(193)=dbm1(258)
dbm2(194)=dbm1(259)
dbm2(195)=dbm1(260)
dbm2(196)=dbm1(261)
dbm2(197)=dbm1(262)
dbm2(198)=dbm1(263)
dbm2(199)=dbm1(264)
dbm2(200)=dbm1(265)
dbm2(201)=dbm1(266)
dbm2(202)=dbm1(267)
dbm2(203)=dbm1(268)
dbm2(204)=dbm1(269)
dbm2(205)=dbm1(270)
dbm2(206)=dbm1(271)
dbm2(207)=dbm1(272)
dbm2(208)=dbm1(273)
dbm2(209)=dbm1(274)
dbm2(210)=dbm1(275)
dbm2(211)=dbm1(276)
dbm2(212)=dbm1(277)
dbm2(213)=dbm1(278)
dbm2(214)=dbm1(279)
dbm2(215)=dbm1(280)
dbm2(216)=dbm1(281)
dbm2(217)=dbm1(282)
dbm2(218)=dbm1(283)
dbm2(219)=dbm1(284)
dbm2(220)=dbm1(285)
dbm2(221)=dbm1(287)
dbm2(222)=dbm1(288)
dbm2(223)=dbm1(289)
dbm2(224)=dbm1(290)
dbm2(225)=dbm1(291)
dbm2(226)=dbm1(292)
dbm2(227)=dbm1(293)
dbm2(228)=dbm1(294)
dbm2(229)=dbm1(296)
dbm2(230)=dbm1(298)
dbm2(231)=dbm1(299)
dbm2(232)=dbm1(300)
dbm2(233)=dbm1(301)
dbm2(234)=dbm1(302)
dbm2(235)=dbm1(303)
dbm2(236)=dbm1(304)
dbm2(237)=dbm1(306)
dbm2(238)=dbm1(307)
dbm2(239)=dbm1(309)
dbm2(240)=dbm1(310)
dbm2(241)=dbm1(312)
dbm2(242)=dbm1(313)
dbm2(243)=dbm1(314)
dbm2(244)=dbm1(315)
dbm2(245)=dbm1(316)
dbm2(246)=dbm1(317)
dbm2(247)=dbm1(319)
dbm2(248)=dbm1(320)
dbm2(249)=dbm1(322)
dbm2(250)=dbm1(323)
dbm2(251)=dbm1(324)
dbm2(252)=dbm1(325)
dbm2(253)=dbm1(326)
dbm2(254)=dbm1(328)
dbm2(255)=dbm1(329)
dbm2(256)=dbm1(330)
dbm2(257)=dbm1(331)
dbm2(258)=dbm1(349)
dbm2(259)=dbm1(350)
dbm2(260)=dbm1(351)
dbm2(261)=dbm1(352)
dbm2(262)=dbm1(353)
dbm2(263)=dbm1(354)
dbm2(264)=dbm1(355)
dbm2(265)=dbm1(356)
dbm2(266)=dbm1(357)
dbm2(267)=dbm1(358)
dbm2(268)=dbm1(359)
dbm2(269)=dbm1(360)
dbm2(270)=dbm1(361)
dbm2(271)=dbm1(362)
dbm2(272)=dbm1(363)
dbm2(273)=dbm1(364)
dbm2(274)=dbm1(365)
dbm2(275)=dbm1(366)
dbm2(276)=dbm1(367)
dbm2(277)=dbm1(368)
dbm2(278)=dbm1(369)
dbm2(279)=dbm1(370)
dbm2(280)=dbm1(371)
dbm2(281)=dbm1(372)
dbm2(282)=dbm1(373)
dbm2(283)=dbm1(374)
dbm2(284)=dbm1(375)
dbm2(285)=dbm1(376)
dbm2(286)=dbm1(377)
dbm2(287)=dbm1(378)
dbm2(288)=dbm1(379)
dbm2(289)=dbm1(380)
dbm2(290)=dbm1(381)
dbm2(291)=dbm1(382)
dbm2(292)=dbm1(383)
dbm2(293)=dbm1(384)
dbm2(294)=dbm1(385)
dbm2(295)=dbm1(386)
dbm2(296)=dbm1(387)
dbm2(297)=dbm1(388)
dbm2(298)=dbm1(389)
dbm2(299)=dbm1(390)
dbm2(300)=dbm1(391)
dbm2(301)=dbm1(392)
dbm2(302)=dbm1(393)
dbm2(303)=dbm1(394)
dbm2(304)=dbm1(395)
dbm2(305)=dbm1(396)
dbm2(306)=dbm1(397)
dbm2(307)=dbm1(398)
dbm2(308)=dbm1(399)
dbm2(309)=dbm1(400)
dbm2(310)=dbm1(401)
dbm2(311)=dbm1(402)
dbm2(312)=dbm1(403)
dbm2(313)=dbm1(404)
dbm2(314)=dbm1(405)
dbm2(315)=dbm1(406)
dbm2(316)=dbm1(407)
dbm2(317)=dbm1(408)
dbm2(318)=dbm1(409)
dbm2(319)=dbm1(410)
dbm2(320)=dbm1(411)
dbm2(321)=dbm1(412)
dbm2(322)=dbm1(413)
dbm2(323)=dbm1(414)
dbm2(324)=dbm1(415)
dbm2(325)=dbm1(416)
dbm2(326)=dbm1(417)
dbm2(327)=dbm1(418)
dbm2(328)=dbm1(419)
dbm2(329)=dbm1(420)
dbm2(330)=dbm1(421)
dbm2(331)=dbm1(422)
dbm2(332)=dbm1(423)
dbm2(333)=dbm1(424)
dbm2(334)=dbm1(425)
dbm2(335)=dbm1(426)
dbm2(336)=dbm1(427)
dbm2(337)=dbm1(428)
dbm2(338)=dbm1(429)
dbm2(339)=dbm1(430)
dbm2(340)=dbm1(431)
dbm2(341)=dbm1(432)
dbm2(342)=dbm1(433)
dbm2(343)=dbm1(434)
dbm2(344)=dbm1(435)
dbm2(345)=dbm1(436)
dbm2(346)=dbm1(437)
dbm2(347)=dbm1(438)
dbm2(348)=dbm1(439)
dbm2(349)=dbm1(440)
dbm2(350)=dbm1(441)
dbm2(351)=dbm1(442)
dbm2(352)=dbm1(443)
dbm2(353)=dbm1(444)
dbm2(354)=dbm1(445)
dbm2(355)=dbm1(446)
dbm2(356)=dbm1(447)
dbm2(357)=dbm1(448)
dbm2(358)=dbm1(449)
dbm2(359)=dbm1(450)
dbm2(360)=dbm1(451)
dbm2(361)=dbm1(452)
dbm2(362)=dbm1(453)
dbm2(363)=dbm1(454)
dbm2(364)=dbm1(455)
dbm2(365)=dbm1(456)
dbm2(366)=dbm1(457)
dbm2(367)=dbm1(458)
dbm2(368)=dbm1(459)
dbm2(369)=dbm1(460)
dbm2(370)=dbm1(461)
dbm2(371)=dbm1(462)
dbm2(372)=dbm1(463)
dbm2(373)=dbm1(464)
dbm2(374)=dbm1(465)
dbm2(375)=dbm1(466)
dbm2(376)=dbm1(467)
dbm2(377)=dbm1(468)
dbm2(378)=dbm1(469)
dbm2(379)=dbm1(470)
dbm2(380)=dbm1(471)
dbm2(381)=dbm1(472)
dbm2(382)=dbm1(473)
dbm2(383)=dbm1(474)
dbm2(384)=dbm1(475)
dbm2(385)=dbm1(476)
dbm2(386)=dbm1(477)
dbm2(387)=dbm1(478)
dbm2(388)=dbm1(479)
dbm2(389)=dbm1(480)
dbm2(390)=dbm1(481)
dbm2(391)=dbm1(482)
dbm2(392)=dbm1(483)
dbm2(393)=dbm1(484)
dbm2(394)=dbm1(485)
dbm2(395)=dbm1(486)
dbm2(396)=dbm1(487)
dbm2(397)=dbm1(488)
dbm2(398)=dbm1(489)
dbm2(399)=dbm1(490)
dbm2(400)=dbm1(491)
dbm2(401)=dbm1(492)
dbm2(402)=dbm1(493)
dbm2(403)=dbm1(494)
dbm2(404)=dbm1(495)
dbm2(405)=dbm1(496)
dbm2(406)=dbm1(497)
dbm2(407)=dbm1(498)
dbm2(408)=dbm1(499)
dbm2(409)=dbm1(500)
dbm2(410)=dbm1(501)
dbm2(411)=dbm1(502)
dbm2(412)=dbm1(503)
dbm2(413)=dbm1(504)
dbm2(414)=dbm1(505)
dbm2(415)=dbm1(507)
dbm2(416)=dbm1(508)
dbm2(417)=dbm1(509)
dbm2(418)=dbm1(510)
dbm2(419)=dbm1(511)
dbm2(420)=dbm1(512)
dbm2(421)=dbm1(513)
dbm2(422)=dbm1(514)
dbm2(423)=dbm1(515)
dbm2(424)=dbm1(516)
dbm2(425)=dbm1(518)
dbm2(426)=dbm1(520)
dbm2(427)=dbm1(521)
dbm2(428)=dbm1(522)
dbm2(429)=dbm1(523)
dbm2(430)=dbm1(524)
dbm2(431)=dbm1(525)
dbm2(432)=dbm1(526)
dbm2(433)=dbm1(527)
dbm2(434)=dbm1(529)
dbm2(435)=dbm1(530)
dbm2(436)=dbm1(532)
dbm2(437)=dbm1(534)
dbm2(438)=dbm1(535)
dbm2(439)=dbm1(536)
dbm2(440)=dbm1(537)
dbm2(441)=dbm1(538)
dbm2(442)=dbm1(539)
dbm2(443)=dbm1(540)
dbm2(444)=dbm1(541)
dbm2(445)=dbm1(543)
dbm2(446)=dbm1(544)
dbm2(447)=dbm1(546)
dbm2(448)=dbm1(547)
dbm2(449)=dbm1(548)
dbm2(450)=dbm1(550)
dbm2(451)=dbm1(551)
dbm2(452)=dbm1(552)
dbm2(453)=dbm1(553)
dbm2(454)=dbm1(554)
dbm2(455)=dbm1(555)
dbm2(456)=dbm1(557)
dbm2(457)=dbm1(558)
dbm2(458)=dbm1(560)
dbm2(459)=dbm1(561)
dbm2(460)=dbm1(562)
dbm2(461)=dbm1(563)
dbm2(462)=dbm1(564)
dbm2(463)=dbm1(566)
dbm2(464)=dbm1(567)
dbm2(465)=dbm1(568)
dbm2(466)=dbm1(569)
dbm2(467)=dbm1(570)
dbm2(468)=dbm1(591)
dbm2(469)=dbm1(592)
dbm2(470)=dbm1(593)
dbm2(471)=dbm1(594)
dbm2(472)=dbm1(595)
dbm2(473)=dbm1(596)
dbm2(474)=dbm1(597)
dbm2(475)=dbm1(598)
dbm2(476)=dbm1(599)
dbm2(477)=dbm1(600)
dbm2(478)=dbm1(601)
dbm2(479)=dbm1(602)
dbm2(480)=dbm1(603)
dbm2(481)=dbm1(604)
dbm2(482)=dbm1(605)
dbm2(483)=dbm1(606)
dbm2(484)=dbm1(607)
dbm2(485)=dbm1(608)
dbm2(486)=dbm1(609)
dbm2(487)=dbm1(610)
dbm2(488)=dbm1(611)
dbm2(489)=dbm1(612)
dbm2(490)=dbm1(613)
dbm2(491)=dbm1(614)
dbm2(492)=dbm1(615)
dbm2(493)=dbm1(616)
dbm2(494)=dbm1(617)
dbm2(495)=dbm1(618)
dbm2(496)=dbm1(619)
dbm2(497)=dbm1(620)
dbm2(498)=dbm1(621)
dbm2(499)=dbm1(622)
dbm2(500)=dbm1(623)
dbm2(501)=dbm1(624)
dbm2(502)=dbm1(625)
dbm2(503)=dbm1(626)
dbm2(504)=dbm1(627)
dbm2(505)=dbm1(628)
dbm2(506)=dbm1(629)
dbm2(507)=dbm1(630)
dbm2(508)=dbm1(631)
dbm2(509)=dbm1(632)
dbm2(510)=dbm1(633)
dbm2(511)=dbm1(634)
dbm2(512)=dbm1(635)
dbm2(513)=dbm1(636)
dbm2(514)=dbm1(637)
dbm2(515)=dbm1(638)
dbm2(516)=dbm1(639)
dbm2(517)=dbm1(640)
dbm2(518)=dbm1(641)
dbm2(519)=dbm1(642)
dbm2(520)=dbm1(643)
dbm2(521)=dbm1(644)
dbm2(522)=dbm1(645)
dbm2(523)=dbm1(646)
dbm2(524)=dbm1(647)
dbm2(525)=dbm1(648)
dbm2(526)=dbm1(649)
dbm2(527)=dbm1(650)
dbm2(528)=dbm1(651)
dbm2(529)=dbm1(652)
dbm2(530)=dbm1(653)
dbm2(531)=dbm1(654)
dbm2(532)=dbm1(655)
dbm2(533)=dbm1(656)
dbm2(534)=dbm1(657)
dbm2(535)=dbm1(658)
dbm2(536)=dbm1(659)
dbm2(537)=dbm1(660)
dbm2(538)=dbm1(661)
dbm2(539)=dbm1(662)
dbm2(540)=dbm1(663)
dbm2(541)=dbm1(664)
dbm2(542)=dbm1(665)
dbm2(543)=dbm1(666)
dbm2(544)=dbm1(667)
dbm2(545)=dbm1(668)
dbm2(546)=dbm1(669)
dbm2(547)=dbm1(670)
dbm2(548)=dbm1(671)
dbm2(549)=dbm1(672)
dbm2(550)=dbm1(673)
dbm2(551)=dbm1(674)
dbm2(552)=dbm1(675)
dbm2(553)=dbm1(676)
dbm2(554)=dbm1(677)
dbm2(555)=dbm1(678)
dbm2(556)=dbm1(679)
dbm2(557)=dbm1(680)
dbm2(558)=dbm1(681)
dbm2(559)=dbm1(682)
dbm2(560)=dbm1(683)
dbm2(561)=dbm1(684)
dbm2(562)=dbm1(685)
dbm2(563)=dbm1(686)
dbm2(564)=dbm1(687)
dbm2(565)=dbm1(688)
dbm2(566)=dbm1(689)
dbm2(567)=dbm1(690)
dbm2(568)=dbm1(691)
dbm2(569)=dbm1(692)
dbm2(570)=dbm1(693)
dbm2(571)=dbm1(694)
dbm2(572)=dbm1(695)
dbm2(573)=dbm1(696)
dbm2(574)=dbm1(697)
dbm2(575)=dbm1(698)
dbm2(576)=dbm1(699)
dbm2(577)=dbm1(700)
dbm2(578)=dbm1(701)
dbm2(579)=dbm1(702)
dbm2(580)=dbm1(703)
dbm2(581)=dbm1(704)
dbm2(582)=dbm1(705)
dbm2(583)=dbm1(706)
dbm2(584)=dbm1(707)
dbm2(585)=dbm1(708)
dbm2(586)=dbm1(709)
dbm2(587)=dbm1(710)
dbm2(588)=dbm1(711)
dbm2(589)=dbm1(712)
dbm2(590)=dbm1(713)
dbm2(591)=dbm1(714)
dbm2(592)=dbm1(715)
dbm2(593)=dbm1(716)
dbm2(594)=dbm1(717)
dbm2(595)=dbm1(718)
dbm2(596)=dbm1(719)
dbm2(597)=dbm1(720)
dbm2(598)=dbm1(721)
dbm2(599)=dbm1(722)
dbm2(600)=dbm1(723)
dbm2(601)=dbm1(724)
dbm2(602)=dbm1(725)
dbm2(603)=dbm1(726)
dbm2(604)=dbm1(727)
dbm2(605)=dbm1(728)
dbm2(606)=dbm1(729)
dbm2(607)=dbm1(730)
dbm2(608)=dbm1(731)
dbm2(609)=dbm1(732)
dbm2(610)=dbm1(733)
dbm2(611)=dbm1(734)
dbm2(612)=dbm1(735)
dbm2(613)=dbm1(736)
dbm2(614)=dbm1(737)
dbm2(615)=dbm1(738)
dbm2(616)=dbm1(739)
dbm2(617)=dbm1(740)
dbm2(618)=dbm1(741)
dbm2(619)=dbm1(742)
dbm2(620)=dbm1(743)
dbm2(621)=dbm1(744)
dbm2(622)=dbm1(745)
dbm2(623)=dbm1(746)
dbm2(624)=dbm1(747)
dbm2(625)=dbm1(748)
dbm2(626)=dbm1(749)
dbm2(627)=dbm1(750)
dbm2(628)=dbm1(751)
dbm2(629)=dbm1(752)
dbm2(630)=dbm1(753)
dbm2(631)=dbm1(754)
dbm2(632)=dbm1(755)
dbm2(633)=dbm1(756)
dbm2(634)=dbm1(757)
dbm2(635)=dbm1(758)
dbm2(636)=dbm1(759)
dbm2(637)=dbm1(760)
dbm2(638)=dbm1(761)
dbm2(639)=dbm1(762)
dbm2(640)=dbm1(763)
dbm2(641)=dbm1(764)
dbm2(642)=dbm1(765)
dbm2(643)=dbm1(766)
dbm2(644)=dbm1(767)
dbm2(645)=dbm1(768)
dbm2(646)=dbm1(769)
dbm2(647)=dbm1(770)
dbm2(648)=dbm1(771)
dbm2(649)=dbm1(772)
dbm2(650)=dbm1(773)
dbm2(651)=dbm1(774)
dbm2(652)=dbm1(775)
dbm2(653)=dbm1(776)
dbm2(654)=dbm1(777)
dbm2(655)=dbm1(778)
dbm2(656)=dbm1(779)
dbm2(657)=dbm1(780)
dbm2(658)=dbm1(781)
dbm2(659)=dbm1(782)
dbm2(660)=dbm1(783)
dbm2(661)=dbm1(784)
dbm2(662)=dbm1(785)
dbm2(663)=dbm1(786)
dbm2(664)=dbm1(787)
dbm2(665)=dbm1(788)
dbm2(666)=dbm1(789)
dbm2(667)=dbm1(790)
dbm2(668)=dbm1(791)
dbm2(669)=dbm1(792)
dbm2(670)=dbm1(793)
dbm2(671)=dbm1(794)
dbm2(672)=dbm1(795)
dbm2(673)=dbm1(796)
dbm2(674)=dbm1(797)
dbm2(675)=dbm1(798)
dbm2(676)=dbm1(799)
dbm2(677)=dbm1(800)
dbm2(678)=dbm1(801)
dbm2(679)=dbm1(802)
dbm2(680)=dbm1(803)
dbm2(681)=dbm1(804)
dbm2(682)=dbm1(805)
dbm2(683)=dbm1(806)
dbm2(684)=dbm1(807)
dbm2(685)=dbm1(808)
dbm2(686)=dbm1(809)
dbm2(687)=dbm1(810)
dbm2(688)=dbm1(811)
dbm2(689)=dbm1(812)
dbm2(690)=dbm1(813)
dbm2(691)=dbm1(814)
dbm2(692)=dbm1(815)
dbm2(693)=dbm1(816)
dbm2(694)=dbm1(817)
dbm2(695)=dbm1(818)
dbm2(696)=dbm1(819)
dbm2(697)=dbm1(820)
dbm2(698)=dbm1(821)
dbm2(699)=dbm1(822)
dbm2(700)=dbm1(823)
dbm2(701)=dbm1(824)
dbm2(702)=dbm1(825)
dbm2(703)=dbm1(826)
dbm2(704)=dbm1(827)
dbm2(705)=dbm1(828)
dbm2(706)=dbm1(829)
dbm2(707)=dbm1(830)
dbm2(708)=dbm1(831)
dbm2(709)=dbm1(832)
dbm2(710)=dbm1(833)
dbm2(711)=dbm1(834)
dbm2(712)=dbm1(835)
dbm2(713)=dbm1(836)
dbm2(714)=dbm1(837)
dbm2(715)=dbm1(838)
dbm2(716)=dbm1(839)
dbm2(717)=dbm1(840)
dbm2(718)=dbm1(841)
dbm2(719)=dbm1(842)
dbm2(720)=dbm1(843)
dbm2(721)=dbm1(844)
dbm2(722)=dbm1(845)
dbm2(723)=dbm1(846)
dbm2(724)=dbm1(847)
dbm2(725)=dbm1(848)
dbm2(726)=dbm1(849)
dbm2(727)=dbm1(850)
dbm2(728)=dbm1(851)
dbm2(729)=dbm1(853)
dbm2(730)=dbm1(854)
dbm2(731)=dbm1(855)
dbm2(732)=dbm1(856)
dbm2(733)=dbm1(857)
dbm2(734)=dbm1(858)
dbm2(735)=dbm1(859)
dbm2(736)=dbm1(860)
dbm2(737)=dbm1(861)
dbm2(738)=dbm1(862)
dbm2(739)=dbm1(863)
dbm2(740)=dbm1(865)
dbm2(741)=dbm1(867)
dbm2(742)=dbm1(868)
dbm2(743)=dbm1(869)
dbm2(744)=dbm1(870)
dbm2(745)=dbm1(871)
dbm2(746)=dbm1(872)
dbm2(747)=dbm1(873)
dbm2(748)=dbm1(874)
dbm2(749)=dbm1(875)
dbm2(750)=dbm1(876)
dbm2(751)=dbm1(878)
dbm2(752)=dbm1(879)
dbm2(753)=dbm1(881)
dbm2(754)=dbm1(883)
dbm2(755)=dbm1(884)
dbm2(756)=dbm1(885)
dbm2(757)=dbm1(886)
dbm2(758)=dbm1(887)
dbm2(759)=dbm1(888)
dbm2(760)=dbm1(889)
dbm2(761)=dbm1(890)
dbm2(762)=dbm1(891)
dbm2(763)=dbm1(893)
dbm2(764)=dbm1(894)
dbm2(765)=dbm1(896)
dbm2(766)=dbm1(897)
dbm2(767)=dbm1(899)
dbm2(768)=dbm1(900)
dbm2(769)=dbm1(902)
dbm2(770)=dbm1(903)
dbm2(771)=dbm1(904)
dbm2(772)=dbm1(905)
dbm2(773)=dbm1(906)
dbm2(774)=dbm1(907)
dbm2(775)=dbm1(908)
dbm2(776)=dbm1(909)
dbm2(777)=dbm1(911)
dbm2(778)=dbm1(912)
dbm2(779)=dbm1(914)
dbm2(780)=dbm1(915)
dbm2(781)=dbm1(916)
dbm2(782)=dbm1(918)
dbm2(783)=dbm1(919)
dbm2(784)=dbm1(920)
dbm2(785)=dbm1(921)
dbm2(786)=dbm1(922)
dbm2(787)=dbm1(923)
dbm2(788)=dbm1(925)
dbm2(789)=dbm1(926)
dbm2(790)=dbm1(928)
dbm2(791)=dbm1(929)
dbm2(792)=dbm1(930)
dbm2(793)=dbm1(931)
dbm2(794)=dbm1(932)
dbm2(795)=dbm1(934)
dbm2(796)=dbm1(935)
dbm2(797)=dbm1(936)
dbm2(798)=dbm1(937)
dbm2(799)=dbm1(938)
dbm2(800)=dbm1(961)
dbm2(801)=dbm1(962)
dbm2(802)=dbm1(963)
dbm2(803)=dbm1(964)
dbm2(804)=dbm1(965)
dbm2(805)=dbm1(966)
dbm2(806)=dbm1(967)
dbm2(807)=dbm1(968)
dbm2(808)=dbm1(969)
dbm2(809)=dbm1(970)
dbm2(810)=dbm1(971)
dbm2(811)=dbm1(972)
dbm2(812)=dbm1(973)
dbm2(813)=dbm1(974)
dbm2(814)=dbm1(975)
dbm2(815)=dbm1(976)
dbm2(816)=dbm1(977)
dbm2(817)=dbm1(978)
dbm2(818)=dbm1(979)
dbm2(819)=dbm1(980)
dbm2(820)=dbm1(981)
dbm2(821)=dbm1(982)
dbm2(822)=dbm1(983)
dbm2(823)=dbm1(984)
dbm2(824)=dbm1(985)
dbm2(825)=dbm1(986)
dbm2(826)=dbm1(987)
dbm2(827)=dbm1(988)
dbm2(828)=dbm1(989)
dbm2(829)=dbm1(990)
dbm2(830)=dbm1(991)
dbm2(831)=dbm1(992)
dbm2(832)=dbm1(993)
dbm2(833)=dbm1(994)
dbm2(834)=dbm1(995)
dbm2(835)=dbm1(996)
dbm2(836)=dbm1(997)
dbm2(837)=dbm1(998)
dbm2(838)=dbm1(999)
dbm2(839)=dbm1(1000)
dbm2(840)=dbm1(1001)
dbm2(841)=dbm1(1002)
dbm2(842)=dbm1(1003)
dbm2(843)=dbm1(1004)
dbm2(844)=dbm1(1005)
dbm2(845)=dbm1(1006)
dbm2(846)=dbm1(1007)
dbm2(847)=dbm1(1008)
dbm2(848)=dbm1(1009)
dbm2(849)=dbm1(1010)
dbm2(850)=dbm1(1011)
dbm2(851)=dbm1(1012)
dbm2(852)=dbm1(1013)
dbm2(853)=dbm1(1014)
dbm2(854)=dbm1(1015)
dbm2(855)=dbm1(1016)
dbm2(856)=dbm1(1017)
dbm2(857)=dbm1(1018)
dbm2(858)=dbm1(1019)
dbm2(859)=dbm1(1020)
dbm2(860)=dbm1(1021)
dbm2(861)=dbm1(1022)
dbm2(862)=dbm1(1023)
dbm2(863)=dbm1(1024)
dbm2(864)=dbm1(1025)
dbm2(865)=dbm1(1026)
dbm2(866)=dbm1(1027)
dbm2(867)=dbm1(1028)
dbm2(868)=dbm1(1029)
dbm2(869)=dbm1(1030)
dbm2(870)=dbm1(1031)
dbm2(871)=dbm1(1032)
dbm2(872)=dbm1(1033)
dbm2(873)=dbm1(1034)
dbm2(874)=dbm1(1035)
dbm2(875)=dbm1(1036)
dbm2(876)=dbm1(1037)
dbm2(877)=dbm1(1038)
dbm2(878)=dbm1(1039)
dbm2(879)=dbm1(1040)
dbm2(880)=dbm1(1041)
dbm2(881)=dbm1(1042)
dbm2(882)=dbm1(1043)
dbm2(883)=dbm1(1044)
dbm2(884)=dbm1(1045)
dbm2(885)=dbm1(1046)
dbm2(886)=dbm1(1047)
dbm2(887)=dbm1(1048)
dbm2(888)=dbm1(1049)
dbm2(889)=dbm1(1050)
dbm2(890)=dbm1(1051)
dbm2(891)=dbm1(1052)
dbm2(892)=dbm1(1053)
dbm2(893)=dbm1(1054)
dbm2(894)=dbm1(1055)
dbm2(895)=dbm1(1056)
dbm2(896)=dbm1(1057)
dbm2(897)=dbm1(1058)
dbm2(898)=dbm1(1059)
dbm2(899)=dbm1(1060)
dbm2(900)=dbm1(1061)
dbm2(901)=dbm1(1062)
dbm2(902)=dbm1(1063)
dbm2(903)=dbm1(1064)
dbm2(904)=dbm1(1065)
dbm2(905)=dbm1(1066)
dbm2(906)=dbm1(1067)
dbm2(907)=dbm1(1068)
dbm2(908)=dbm1(1069)
dbm2(909)=dbm1(1070)
dbm2(910)=dbm1(1071)
dbm2(911)=dbm1(1072)
dbm2(912)=dbm1(1073)
dbm2(913)=dbm1(1074)
dbm2(914)=dbm1(1075)
dbm2(915)=dbm1(1076)
dbm2(916)=dbm1(1077)
dbm2(917)=dbm1(1078)
dbm2(918)=dbm1(1079)
dbm2(919)=dbm1(1080)
dbm2(920)=dbm1(1081)
dbm2(921)=dbm1(1082)
dbm2(922)=dbm1(1083)
dbm2(923)=dbm1(1084)
dbm2(924)=dbm1(1085)
dbm2(925)=dbm1(1086)
dbm2(926)=dbm1(1087)
dbm2(927)=dbm1(1088)
dbm2(928)=dbm1(1089)
dbm2(929)=dbm1(1090)
dbm2(930)=dbm1(1091)
dbm2(931)=dbm1(1092)
dbm2(932)=dbm1(1093)
dbm2(933)=dbm1(1094)
dbm2(934)=dbm1(1095)
dbm2(935)=dbm1(1096)
dbm2(936)=dbm1(1097)
dbm2(937)=dbm1(1098)
dbm2(938)=dbm1(1099)
dbm2(939)=dbm1(1100)
dbm2(940)=dbm1(1101)
dbm2(941)=dbm1(1102)
dbm2(942)=dbm1(1103)
dbm2(943)=dbm1(1104)
dbm2(944)=dbm1(1105)
dbm2(945)=dbm1(1106)
dbm2(946)=dbm1(1107)
dbm2(947)=dbm1(1108)
dbm2(948)=dbm1(1109)
dbm2(949)=dbm1(1110)
dbm2(950)=dbm1(1111)
dbm2(951)=dbm1(1112)
dbm2(952)=dbm1(1113)
dbm2(953)=dbm1(1114)
dbm2(954)=dbm1(1115)
dbm2(955)=dbm1(1116)
dbm2(956)=dbm1(1117)
dbm2(957)=dbm1(1118)
dbm2(958)=dbm1(1119)
dbm2(959)=dbm1(1120)
dbm2(960)=dbm1(1121)
dbm2(961)=dbm1(1122)
dbm2(962)=dbm1(1123)
dbm2(963)=dbm1(1124)
dbm2(964)=dbm1(1125)
dbm2(965)=dbm1(1126)
dbm2(966)=dbm1(1127)
dbm2(967)=dbm1(1128)
dbm2(968)=dbm1(1129)
dbm2(969)=dbm1(1130)
dbm2(970)=dbm1(1131)
dbm2(971)=dbm1(1132)
dbm2(972)=dbm1(1133)
dbm2(973)=dbm1(1134)
dbm2(974)=dbm1(1135)
dbm2(975)=dbm1(1136)
dbm2(976)=dbm1(1137)
dbm2(977)=dbm1(1138)
dbm2(978)=dbm1(1139)
dbm2(979)=dbm1(1140)
dbm2(980)=dbm1(1141)
dbm2(981)=dbm1(1142)
dbm2(982)=dbm1(1143)
dbm2(983)=dbm1(1144)
dbm2(984)=dbm1(1145)
dbm2(985)=dbm1(1146)
dbm2(986)=dbm1(1147)
dbm2(987)=dbm1(1148)
dbm2(988)=dbm1(1149)
dbm2(989)=dbm1(1150)
dbm2(990)=dbm1(1151)
dbm2(991)=dbm1(1152)
dbm2(992)=dbm1(1153)
dbm2(993)=dbm1(1154)
dbm2(994)=dbm1(1155)
dbm2(995)=dbm1(1156)
dbm2(996)=dbm1(1157)
dbm2(997)=dbm1(1158)
dbm2(998)=dbm1(1159)
dbm2(999)=dbm1(1160)
dbm2(1000)=dbm1(1161)
dbm2(1001)=dbm1(1162)
dbm2(1002)=dbm1(1163)
dbm2(1003)=dbm1(1164)
dbm2(1004)=dbm1(1165)
dbm2(1005)=dbm1(1166)
dbm2(1006)=dbm1(1167)
dbm2(1007)=dbm1(1168)
dbm2(1008)=dbm1(1169)
dbm2(1009)=dbm1(1170)
dbm2(1010)=dbm1(1171)
dbm2(1011)=dbm1(1172)
dbm2(1012)=dbm1(1173)
dbm2(1013)=dbm1(1174)
dbm2(1014)=dbm1(1175)
dbm2(1015)=dbm1(1176)
dbm2(1016)=dbm1(1177)
dbm2(1017)=dbm1(1178)
dbm2(1018)=dbm1(1179)
dbm2(1019)=dbm1(1180)
dbm2(1020)=dbm1(1181)
dbm2(1021)=dbm1(1182)
dbm2(1022)=dbm1(1183)
dbm2(1023)=dbm1(1184)
dbm2(1024)=dbm1(1185)
dbm2(1025)=dbm1(1186)
dbm2(1026)=dbm1(1187)
dbm2(1027)=dbm1(1188)
dbm2(1028)=dbm1(1189)
dbm2(1029)=dbm1(1190)
dbm2(1030)=dbm1(1191)
dbm2(1031)=dbm1(1192)
dbm2(1032)=dbm1(1193)
dbm2(1033)=dbm1(1194)
dbm2(1034)=dbm1(1195)
dbm2(1035)=dbm1(1196)
dbm2(1036)=dbm1(1197)
dbm2(1037)=dbm1(1198)
dbm2(1038)=dbm1(1199)
dbm2(1039)=dbm1(1200)
dbm2(1040)=dbm1(1201)
dbm2(1041)=dbm1(1202)
dbm2(1042)=dbm1(1203)
dbm2(1043)=dbm1(1204)
dbm2(1044)=dbm1(1205)
dbm2(1045)=dbm1(1206)
dbm2(1046)=dbm1(1207)
dbm2(1047)=dbm1(1208)
dbm2(1048)=dbm1(1209)
dbm2(1049)=dbm1(1210)
dbm2(1050)=dbm1(1211)
dbm2(1051)=dbm1(1212)
dbm2(1052)=dbm1(1213)
dbm2(1053)=dbm1(1214)
dbm2(1054)=dbm1(1215)
dbm2(1055)=dbm1(1216)
dbm2(1056)=dbm1(1217)
dbm2(1057)=dbm1(1218)
dbm2(1058)=dbm1(1219)
dbm2(1059)=dbm1(1220)
dbm2(1060)=dbm1(1221)
dbm2(1061)=dbm1(1222)
dbm2(1062)=dbm1(1223)
dbm2(1063)=dbm1(1224)
dbm2(1064)=dbm1(1225)
dbm2(1065)=dbm1(1226)
dbm2(1066)=dbm1(1227)
dbm2(1067)=dbm1(1228)
dbm2(1068)=dbm1(1229)
dbm2(1069)=dbm1(1230)
dbm2(1070)=dbm1(1231)
dbm2(1071)=dbm1(1232)
dbm2(1072)=dbm1(1233)
dbm2(1073)=dbm1(1234)
dbm2(1074)=dbm1(1235)
dbm2(1075)=dbm1(1236)
dbm2(1076)=dbm1(1237)
dbm2(1077)=dbm1(1238)
dbm2(1078)=dbm1(1239)
dbm2(1079)=dbm1(1240)
dbm2(1080)=dbm1(1241)
dbm2(1081)=dbm1(1242)
dbm2(1082)=dbm1(1243)
dbm2(1083)=dbm1(1244)
dbm2(1084)=dbm1(1245)
dbm2(1085)=dbm1(1246)
dbm2(1086)=dbm1(1247)
dbm2(1087)=dbm1(1248)
dbm2(1088)=dbm1(1249)
dbm2(1089)=dbm1(1250)
dbm2(1090)=dbm1(1251)
dbm2(1091)=dbm1(1252)
dbm2(1092)=dbm1(1253)
dbm2(1093)=dbm1(1254)
dbm2(1094)=dbm1(1255)
dbm2(1095)=dbm1(1256)
dbm2(1096)=dbm1(1257)
dbm2(1097)=dbm1(1258)
dbm2(1098)=dbm1(1259)
dbm2(1099)=dbm1(1260)
dbm2(1100)=dbm1(1261)
dbm2(1101)=dbm1(1262)
dbm2(1102)=dbm1(1263)
dbm2(1103)=dbm1(1264)
dbm2(1104)=dbm1(1265)
dbm2(1105)=dbm1(1266)
dbm2(1106)=dbm1(1267)
dbm2(1107)=dbm1(1268)
dbm2(1108)=dbm1(1269)
dbm2(1109)=dbm1(1270)
dbm2(1110)=dbm1(1271)
dbm2(1111)=dbm1(1272)
dbm2(1112)=dbm1(1273)
dbm2(1113)=dbm1(1274)
dbm2(1114)=dbm1(1275)
dbm2(1115)=dbm1(1276)
dbm2(1116)=dbm1(1277)
dbm2(1117)=dbm1(1278)
dbm2(1118)=dbm1(1279)
dbm2(1119)=dbm1(1280)
dbm2(1120)=dbm1(1281)
dbm2(1121)=dbm1(1282)
dbm2(1122)=dbm1(1283)
dbm2(1123)=dbm1(1284)
dbm2(1124)=dbm1(1285)
dbm2(1125)=dbm1(1286)
dbm2(1126)=dbm1(1287)
dbm2(1127)=dbm1(1288)
dbm2(1128)=dbm1(1289)
dbm2(1129)=dbm1(1290)
dbm2(1130)=dbm1(1291)
dbm2(1131)=dbm1(1292)
dbm2(1132)=dbm1(1293)
dbm2(1133)=dbm1(1294)
dbm2(1134)=dbm1(1295)
dbm2(1135)=dbm1(1296)
dbm2(1136)=dbm1(1297)
dbm2(1137)=dbm1(1298)
dbm2(1138)=dbm1(1299)
dbm2(1139)=dbm1(1300)
dbm2(1140)=dbm1(1301)
dbm2(1141)=dbm1(1302)
dbm2(1142)=dbm1(1303)
dbm2(1143)=dbm1(1304)
dbm2(1144)=dbm1(1305)
dbm2(1145)=dbm1(1306)
dbm2(1146)=dbm1(1307)
dbm2(1147)=dbm1(1308)
dbm2(1148)=dbm1(1309)
dbm2(1149)=dbm1(1310)
dbm2(1150)=dbm1(1311)
dbm2(1151)=dbm1(1312)
dbm2(1152)=dbm1(1313)
dbm2(1153)=dbm1(1314)
dbm2(1154)=dbm1(1315)
dbm2(1155)=dbm1(1316)
dbm2(1156)=dbm1(1317)
dbm2(1157)=dbm1(1318)
dbm2(1158)=dbm1(1319)
dbm2(1159)=dbm1(1320)
dbm2(1160)=dbm1(1321)
dbm2(1161)=dbm1(1322)
dbm2(1162)=dbm1(1323)
dbm2(1163)=dbm1(1324)
dbm2(1164)=dbm1(1325)
dbm2(1165)=dbm1(1326)
dbm2(1166)=dbm1(1327)
dbm2(1167)=dbm1(1328)
dbm2(1168)=dbm1(1329)
dbm2(1169)=dbm1(1330)
dbm2(1170)=dbm1(1331)
dbm2(1171)=dbm1(1332)
dbm2(1172)=dbm1(1333)
dbm2(1173)=dbm1(1334)
dbm2(1174)=dbm1(1335)
dbm2(1175)=dbm1(1336)
dbm2(1176)=dbm1(1337)
dbm2(1177)=dbm1(1338)
dbm2(1178)=dbm1(1339)
dbm2(1179)=dbm1(1340)
dbm2(1180)=dbm1(1341)
dbm2(1181)=dbm1(1342)
dbm2(1182)=dbm1(1343)
dbm2(1183)=dbm1(1344)
dbm2(1184)=dbm1(1345)
dbm2(1185)=dbm1(1346)
dbm2(1186)=dbm1(1347)
dbm2(1187)=dbm1(1348)
dbm2(1188)=dbm1(1349)
dbm2(1189)=dbm1(1350)
dbm2(1190)=dbm1(1351)
dbm2(1191)=dbm1(1352)
dbm2(1192)=dbm1(1353)
dbm2(1193)=dbm1(1354)
dbm2(1194)=dbm1(1355)
dbm2(1195)=dbm1(1356)
dbm2(1196)=dbm1(1357)
dbm2(1197)=dbm1(1358)
dbm2(1198)=dbm1(1359)
dbm2(1199)=dbm1(1360)
dbm2(1200)=dbm1(1361)
dbm2(1201)=dbm1(1362)
dbm2(1202)=dbm1(1363)
dbm2(1203)=dbm1(1364)
dbm2(1204)=dbm1(1365)
dbm2(1205)=dbm1(1366)
dbm2(1206)=dbm1(1367)
dbm2(1207)=dbm1(1368)
dbm2(1208)=dbm1(1369)
dbm2(1209)=dbm1(1371)
dbm2(1210)=dbm1(1372)
dbm2(1211)=dbm1(1373)
dbm2(1212)=dbm1(1374)
dbm2(1213)=dbm1(1375)
dbm2(1214)=dbm1(1376)
dbm2(1215)=dbm1(1377)
dbm2(1216)=dbm1(1378)
dbm2(1217)=dbm1(1379)
dbm2(1218)=dbm1(1380)
dbm2(1219)=dbm1(1381)
dbm2(1220)=dbm1(1382)
dbm2(1221)=dbm1(1383)
dbm2(1222)=dbm1(1385)
dbm2(1223)=dbm1(1387)
dbm2(1224)=dbm1(1388)
dbm2(1225)=dbm1(1389)
dbm2(1226)=dbm1(1390)
dbm2(1227)=dbm1(1391)
dbm2(1228)=dbm1(1392)
dbm2(1229)=dbm1(1393)
dbm2(1230)=dbm1(1394)
dbm2(1231)=dbm1(1395)
dbm2(1232)=dbm1(1396)
dbm2(1233)=dbm1(1397)
dbm2(1234)=dbm1(1399)
dbm2(1235)=dbm1(1400)
dbm2(1236)=dbm1(1402)
dbm2(1237)=dbm1(1404)
dbm2(1238)=dbm1(1405)
dbm2(1239)=dbm1(1406)
dbm2(1240)=dbm1(1407)
dbm2(1241)=dbm1(1408)
dbm2(1242)=dbm1(1409)
dbm2(1243)=dbm1(1410)
dbm2(1244)=dbm1(1411)
dbm2(1245)=dbm1(1412)
dbm2(1246)=dbm1(1413)
dbm2(1247)=dbm1(1415)
dbm2(1248)=dbm1(1416)
dbm2(1249)=dbm1(1418)
dbm2(1250)=dbm1(1419)
dbm2(1251)=dbm1(1421)
dbm2(1252)=dbm1(1422)
dbm2(1253)=dbm1(1424)
dbm2(1254)=dbm1(1425)
dbm2(1255)=dbm1(1426)
dbm2(1256)=dbm1(1427)
dbm2(1257)=dbm1(1428)
dbm2(1258)=dbm1(1429)
dbm2(1259)=dbm1(1430)
dbm2(1260)=dbm1(1431)
dbm2(1261)=dbm1(1432)
dbm2(1262)=dbm1(1434)
dbm2(1263)=dbm1(1435)
dbm2(1264)=dbm1(1437)
dbm2(1265)=dbm1(1438)
dbm2(1266)=dbm1(1440)
dbm2(1267)=dbm1(1441)
dbm2(1268)=dbm1(1442)
dbm2(1269)=dbm1(1444)
dbm2(1270)=dbm1(1445)
dbm2(1271)=dbm1(1446)
dbm2(1272)=dbm1(1447)
dbm2(1273)=dbm1(1448)
dbm2(1274)=dbm1(1449)
dbm2(1275)=dbm1(1450)
dbm2(1276)=dbm1(1451)
dbm2(1277)=dbm1(1453)
dbm2(1278)=dbm1(1454)
dbm2(1279)=dbm1(1456)
dbm2(1280)=dbm1(1457)
dbm2(1281)=dbm1(1458)
dbm2(1282)=dbm1(1460)
dbm2(1283)=dbm1(1461)
dbm2(1284)=dbm1(1462)
dbm2(1285)=dbm1(1463)
dbm2(1286)=dbm1(1464)
dbm2(1287)=dbm1(1465)
dbm2(1288)=dbm1(1467)
dbm2(1289)=dbm1(1468)
dbm2(1290)=dbm1(1470)
dbm2(1291)=dbm1(1471)
dbm2(1292)=dbm1(1472)
dbm2(1293)=dbm1(1473)
dbm2(1294)=dbm1(1474)
dbm2(1295)=dbm1(1476)
dbm2(1296)=dbm1(1477)
dbm2(1297)=dbm1(1478)
dbm2(1298)=dbm1(1479)
dbm2(1299)=dbm1(1480)
dbm2(1300)=dbm1(1481)
dbm2(1301)=dbm1(1507)
dbm2(1302)=dbm1(1508)
dbm2(1303)=dbm1(1509)
dbm2(1304)=dbm1(1510)
dbm2(1305)=dbm1(1511)
dbm2(1306)=dbm1(1512)
dbm2(1307)=dbm1(1513)
dbm2(1308)=dbm1(1514)
dbm2(1309)=dbm1(1515)
dbm2(1310)=dbm1(1516)
dbm2(1311)=dbm1(1517)
dbm2(1312)=dbm1(1518)
dbm2(1313)=dbm1(1519)
dbm2(1314)=dbm1(1520)
dbm2(1315)=dbm1(1521)
dbm2(1316)=dbm1(1522)
dbm2(1317)=dbm1(1523)
dbm2(1318)=dbm1(1524)
dbm2(1319)=dbm1(1525)
dbm2(1320)=dbm1(1526)
dbm2(1321)=dbm1(1527)
dbm2(1322)=dbm1(1528)
dbm2(1323)=dbm1(1529)
dbm2(1324)=dbm1(1530)
dbm2(1325)=dbm1(1531)
dbm2(1326)=dbm1(1532)
dbm2(1327)=dbm1(1533)
dbm2(1328)=dbm1(1534)
dbm2(1329)=dbm1(1535)
dbm2(1330)=dbm1(1536)
dbm2(1331)=dbm1(1537)
dbm2(1332)=dbm1(1538)
dbm2(1333)=dbm1(1539)
dbm2(1334)=dbm1(1540)
dbm2(1335)=dbm1(1541)
dbm2(1336)=dbm1(1542)
dbm2(1337)=dbm1(1543)
dbm2(1338)=dbm1(1544)
dbm2(1339)=dbm1(1545)
dbm2(1340)=dbm1(1546)
dbm2(1341)=dbm1(1547)
dbm2(1342)=dbm1(1548)
dbm2(1343)=dbm1(1549)
dbm2(1344)=dbm1(1550)
dbm2(1345)=dbm1(1551)
dbm2(1346)=dbm1(1552)
dbm2(1347)=dbm1(1553)
dbm2(1348)=dbm1(1554)
dbm2(1349)=dbm1(1555)
dbm2(1350)=dbm1(1556)
dbm2(1351)=dbm1(1557)
dbm2(1352)=dbm1(1558)
dbm2(1353)=dbm1(1559)
dbm2(1354)=dbm1(1560)
dbm2(1355)=dbm1(1561)
dbm2(1356)=dbm1(1562)
dbm2(1357)=dbm1(1563)
dbm2(1358)=dbm1(1564)
dbm2(1359)=dbm1(1565)
dbm2(1360)=dbm1(1566)
dbm2(1361)=dbm1(1567)
dbm2(1362)=dbm1(1568)
dbm2(1363)=dbm1(1569)
dbm2(1364)=dbm1(1570)
dbm2(1365)=dbm1(1571)
dbm2(1366)=dbm1(1572)
dbm2(1367)=dbm1(1573)
dbm2(1368)=dbm1(1574)
dbm2(1369)=dbm1(1575)
dbm2(1370)=dbm1(1576)
dbm2(1371)=dbm1(1577)
dbm2(1372)=dbm1(1578)
dbm2(1373)=dbm1(1579)
dbm2(1374)=dbm1(1580)
dbm2(1375)=dbm1(1581)
dbm2(1376)=dbm1(1582)
dbm2(1377)=dbm1(1583)
dbm2(1378)=dbm1(1584)
dbm2(1379)=dbm1(1585)
dbm2(1380)=dbm1(1586)
dbm2(1381)=dbm1(1587)
dbm2(1382)=dbm1(1588)
dbm2(1383)=dbm1(1589)
dbm2(1384)=dbm1(1590)
dbm2(1385)=dbm1(1591)
dbm2(1386)=dbm1(1592)
dbm2(1387)=dbm1(1593)
dbm2(1388)=dbm1(1594)
dbm2(1389)=dbm1(1595)
dbm2(1390)=dbm1(1596)
dbm2(1391)=dbm1(1597)
dbm2(1392)=dbm1(1598)
dbm2(1393)=dbm1(1599)
dbm2(1394)=dbm1(1600)
dbm2(1395)=dbm1(1601)
dbm2(1396)=dbm1(1602)
dbm2(1397)=dbm1(1603)
dbm2(1398)=dbm1(1604)
dbm2(1399)=dbm1(1605)
dbm2(1400)=dbm1(1606)
dbm2(1401)=dbm1(1607)
dbm2(1402)=dbm1(1608)
dbm2(1403)=dbm1(1609)
dbm2(1404)=dbm1(1610)
dbm2(1405)=dbm1(1611)
dbm2(1406)=dbm1(1612)
dbm2(1407)=dbm1(1613)
dbm2(1408)=dbm1(1614)
dbm2(1409)=dbm1(1615)
dbm2(1410)=dbm1(1616)
dbm2(1411)=dbm1(1617)
dbm2(1412)=dbm1(1618)
dbm2(1413)=dbm1(1619)
dbm2(1414)=dbm1(1620)
dbm2(1415)=dbm1(1621)
dbm2(1416)=dbm1(1622)
dbm2(1417)=dbm1(1623)
dbm2(1418)=dbm1(1624)
dbm2(1419)=dbm1(1625)
dbm2(1420)=dbm1(1626)
dbm2(1421)=dbm1(1627)
dbm2(1422)=dbm1(1628)
dbm2(1423)=dbm1(1629)
dbm2(1424)=dbm1(1630)
dbm2(1425)=dbm1(1631)
dbm2(1426)=dbm1(1632)
dbm2(1427)=dbm1(1633)
dbm2(1428)=dbm1(1634)
dbm2(1429)=dbm1(1635)
dbm2(1430)=dbm1(1636)
dbm2(1431)=dbm1(1637)
dbm2(1432)=dbm1(1638)
dbm2(1433)=dbm1(1639)
dbm2(1434)=dbm1(1640)
dbm2(1435)=dbm1(1641)
dbm2(1436)=dbm1(1642)
dbm2(1437)=dbm1(1643)
dbm2(1438)=dbm1(1644)
dbm2(1439)=dbm1(1645)
dbm2(1440)=dbm1(1646)
dbm2(1441)=dbm1(1647)
dbm2(1442)=dbm1(1648)
dbm2(1443)=dbm1(1649)
dbm2(1444)=dbm1(1650)
dbm2(1445)=dbm1(1651)
dbm2(1446)=dbm1(1652)
dbm2(1447)=dbm1(1653)
dbm2(1448)=dbm1(1654)
dbm2(1449)=dbm1(1655)
dbm2(1450)=dbm1(1656)
dbm2(1451)=dbm1(1657)
dbm2(1452)=dbm1(1658)
dbm2(1453)=dbm1(1659)
dbm2(1454)=dbm1(1660)
dbm2(1455)=dbm1(1661)
dbm2(1456)=dbm1(1662)
dbm2(1457)=dbm1(1663)
dbm2(1458)=dbm1(1664)
dbm2(1459)=dbm1(1665)
dbm2(1460)=dbm1(1666)
dbm2(1461)=dbm1(1667)
dbm2(1462)=dbm1(1668)
dbm2(1463)=dbm1(1669)
dbm2(1464)=dbm1(1670)
dbm2(1465)=dbm1(1671)
dbm2(1466)=dbm1(1672)
dbm2(1467)=dbm1(1673)
dbm2(1468)=dbm1(1674)
dbm2(1469)=dbm1(1675)
dbm2(1470)=dbm1(1676)
dbm2(1471)=dbm1(1677)
dbm2(1472)=dbm1(1678)
dbm2(1473)=dbm1(1679)
dbm2(1474)=dbm1(1680)
dbm2(1475)=dbm1(1681)
dbm2(1476)=dbm1(1682)
dbm2(1477)=dbm1(1683)
dbm2(1478)=dbm1(1684)
dbm2(1479)=dbm1(1685)
dbm2(1480)=dbm1(1686)
dbm2(1481)=dbm1(1687)
dbm2(1482)=dbm1(1688)
dbm2(1483)=dbm1(1689)
dbm2(1484)=dbm1(1690)
dbm2(1485)=dbm1(1691)
dbm2(1486)=dbm1(1692)
dbm2(1487)=dbm1(1693)
dbm2(1488)=dbm1(1694)
dbm2(1489)=dbm1(1695)
dbm2(1490)=dbm1(1696)
dbm2(1491)=dbm1(1697)
dbm2(1492)=dbm1(1698)
dbm2(1493)=dbm1(1699)
dbm2(1494)=dbm1(1700)
dbm2(1495)=dbm1(1701)
dbm2(1496)=dbm1(1702)
dbm2(1497)=dbm1(1703)
dbm2(1498)=dbm1(1704)
dbm2(1499)=dbm1(1705)
dbm2(1500)=dbm1(1706)
dbm2(1501)=dbm1(1707)
dbm2(1502)=dbm1(1708)
dbm2(1503)=dbm1(1709)
dbm2(1504)=dbm1(1710)
dbm2(1505)=dbm1(1711)
dbm2(1506)=dbm1(1712)
dbm2(1507)=dbm1(1713)
dbm2(1508)=dbm1(1714)
dbm2(1509)=dbm1(1715)
dbm2(1510)=dbm1(1716)
dbm2(1511)=dbm1(1717)
dbm2(1512)=dbm1(1718)
dbm2(1513)=dbm1(1719)
dbm2(1514)=dbm1(1720)
dbm2(1515)=dbm1(1721)
dbm2(1516)=dbm1(1722)
dbm2(1517)=dbm1(1723)
dbm2(1518)=dbm1(1724)
dbm2(1519)=dbm1(1725)
dbm2(1520)=dbm1(1726)
dbm2(1521)=dbm1(1727)
dbm2(1522)=dbm1(1728)
dbm2(1523)=dbm1(1729)
dbm2(1524)=dbm1(1730)
dbm2(1525)=dbm1(1731)
dbm2(1526)=dbm1(1732)
dbm2(1527)=dbm1(1733)
dbm2(1528)=dbm1(1734)
dbm2(1529)=dbm1(1735)
dbm2(1530)=dbm1(1736)
dbm2(1531)=dbm1(1737)
dbm2(1532)=dbm1(1738)
dbm2(1533)=dbm1(1739)
dbm2(1534)=dbm1(1740)
dbm2(1535)=dbm1(1741)
dbm2(1536)=dbm1(1742)
dbm2(1537)=dbm1(1743)
dbm2(1538)=dbm1(1744)
dbm2(1539)=dbm1(1745)
dbm2(1540)=dbm1(1746)
dbm2(1541)=dbm1(1747)
dbm2(1542)=dbm1(1748)
dbm2(1543)=dbm1(1749)
dbm2(1544)=dbm1(1750)
dbm2(1545)=dbm1(1751)
dbm2(1546)=dbm1(1752)
dbm2(1547)=dbm1(1753)
dbm2(1548)=dbm1(1754)
dbm2(1549)=dbm1(1755)
dbm2(1550)=dbm1(1756)
dbm2(1551)=dbm1(1757)
dbm2(1552)=dbm1(1758)
dbm2(1553)=dbm1(1759)
dbm2(1554)=dbm1(1760)
dbm2(1555)=dbm1(1761)
dbm2(1556)=dbm1(1762)
dbm2(1557)=dbm1(1763)
dbm2(1558)=dbm1(1764)
dbm2(1559)=dbm1(1765)
dbm2(1560)=dbm1(1766)
dbm2(1561)=dbm1(1767)
dbm2(1562)=dbm1(1768)
dbm2(1563)=dbm1(1769)
dbm2(1564)=dbm1(1770)
dbm2(1565)=dbm1(1771)
dbm2(1566)=dbm1(1772)
dbm2(1567)=dbm1(1773)
dbm2(1568)=dbm1(1774)
dbm2(1569)=dbm1(1775)
dbm2(1570)=dbm1(1776)
dbm2(1571)=dbm1(1777)
dbm2(1572)=dbm1(1778)
dbm2(1573)=dbm1(1779)
dbm2(1574)=dbm1(1780)
dbm2(1575)=dbm1(1781)
dbm2(1576)=dbm1(1782)
dbm2(1577)=dbm1(1783)
dbm2(1578)=dbm1(1784)
dbm2(1579)=dbm1(1785)
dbm2(1580)=dbm1(1786)
dbm2(1581)=dbm1(1787)
dbm2(1582)=dbm1(1788)
dbm2(1583)=dbm1(1789)
dbm2(1584)=dbm1(1790)
dbm2(1585)=dbm1(1791)
dbm2(1586)=dbm1(1792)
dbm2(1587)=dbm1(1793)
dbm2(1588)=dbm1(1794)
dbm2(1589)=dbm1(1795)
dbm2(1590)=dbm1(1796)
dbm2(1591)=dbm1(1797)
dbm2(1592)=dbm1(1798)
dbm2(1593)=dbm1(1799)
dbm2(1594)=dbm1(1800)
dbm2(1595)=dbm1(1801)
dbm2(1596)=dbm1(1802)
dbm2(1597)=dbm1(1803)
dbm2(1598)=dbm1(1804)
dbm2(1599)=dbm1(1805)
dbm2(1600)=dbm1(1806)
dbm2(1601)=dbm1(1807)
dbm2(1602)=dbm1(1808)
dbm2(1603)=dbm1(1809)
dbm2(1604)=dbm1(1810)
dbm2(1605)=dbm1(1811)
dbm2(1606)=dbm1(1812)
dbm2(1607)=dbm1(1813)
dbm2(1608)=dbm1(1814)
dbm2(1609)=dbm1(1815)
dbm2(1610)=dbm1(1816)
dbm2(1611)=dbm1(1817)
dbm2(1612)=dbm1(1818)
dbm2(1613)=dbm1(1819)
dbm2(1614)=dbm1(1820)
dbm2(1615)=dbm1(1821)
dbm2(1616)=dbm1(1822)
dbm2(1617)=dbm1(1823)
dbm2(1618)=dbm1(1824)
dbm2(1619)=dbm1(1825)
dbm2(1620)=dbm1(1826)
dbm2(1621)=dbm1(1827)
dbm2(1622)=dbm1(1828)
dbm2(1623)=dbm1(1829)
dbm2(1624)=dbm1(1830)
dbm2(1625)=dbm1(1831)
dbm2(1626)=dbm1(1832)
dbm2(1627)=dbm1(1833)
dbm2(1628)=dbm1(1834)
dbm2(1629)=dbm1(1835)
dbm2(1630)=dbm1(1836)
dbm2(1631)=dbm1(1837)
dbm2(1632)=dbm1(1838)
dbm2(1633)=dbm1(1839)
dbm2(1634)=dbm1(1840)
dbm2(1635)=dbm1(1841)
dbm2(1636)=dbm1(1842)
dbm2(1637)=dbm1(1843)
dbm2(1638)=dbm1(1844)
dbm2(1639)=dbm1(1845)
dbm2(1640)=dbm1(1846)
dbm2(1641)=dbm1(1847)
dbm2(1642)=dbm1(1848)
dbm2(1643)=dbm1(1849)
dbm2(1644)=dbm1(1850)
dbm2(1645)=dbm1(1851)
dbm2(1646)=dbm1(1852)
dbm2(1647)=dbm1(1853)
dbm2(1648)=dbm1(1854)
dbm2(1649)=dbm1(1855)
dbm2(1650)=dbm1(1856)
dbm2(1651)=dbm1(1857)
dbm2(1652)=dbm1(1858)
dbm2(1653)=dbm1(1859)
dbm2(1654)=dbm1(1860)
dbm2(1655)=dbm1(1861)
dbm2(1656)=dbm1(1862)
dbm2(1657)=dbm1(1863)
dbm2(1658)=dbm1(1864)
dbm2(1659)=dbm1(1865)
dbm2(1660)=dbm1(1866)
dbm2(1661)=dbm1(1867)
dbm2(1662)=dbm1(1868)
dbm2(1663)=dbm1(1869)
dbm2(1664)=dbm1(1870)
dbm2(1665)=dbm1(1871)
dbm2(1666)=dbm1(1872)
dbm2(1667)=dbm1(1873)
dbm2(1668)=dbm1(1874)
dbm2(1669)=dbm1(1875)
dbm2(1670)=dbm1(1876)
dbm2(1671)=dbm1(1877)
dbm2(1672)=dbm1(1878)
dbm2(1673)=dbm1(1879)
dbm2(1674)=dbm1(1880)
dbm2(1675)=dbm1(1881)
dbm2(1676)=dbm1(1882)
dbm2(1677)=dbm1(1883)
dbm2(1678)=dbm1(1884)
dbm2(1679)=dbm1(1885)
dbm2(1680)=dbm1(1886)
dbm2(1681)=dbm1(1887)
dbm2(1682)=dbm1(1888)
dbm2(1683)=dbm1(1889)
dbm2(1684)=dbm1(1890)
dbm2(1685)=dbm1(1891)
dbm2(1686)=dbm1(1892)
dbm2(1687)=dbm1(1893)
dbm2(1688)=dbm1(1894)
dbm2(1689)=dbm1(1895)
dbm2(1690)=dbm1(1896)
dbm2(1691)=dbm1(1897)
dbm2(1692)=dbm1(1898)
dbm2(1693)=dbm1(1899)
dbm2(1694)=dbm1(1900)
dbm2(1695)=dbm1(1901)
dbm2(1696)=dbm1(1902)
dbm2(1697)=dbm1(1903)
dbm2(1698)=dbm1(1904)
dbm2(1699)=dbm1(1905)
dbm2(1700)=dbm1(1906)
dbm2(1701)=dbm1(1907)
dbm2(1702)=dbm1(1908)
dbm2(1703)=dbm1(1909)
dbm2(1704)=dbm1(1910)
dbm2(1705)=dbm1(1911)
dbm2(1706)=dbm1(1912)
dbm2(1707)=dbm1(1913)
dbm2(1708)=dbm1(1914)
dbm2(1709)=dbm1(1915)
dbm2(1710)=dbm1(1916)
dbm2(1711)=dbm1(1917)
dbm2(1712)=dbm1(1918)
dbm2(1713)=dbm1(1919)
dbm2(1714)=dbm1(1920)
dbm2(1715)=dbm1(1921)
dbm2(1716)=dbm1(1922)
dbm2(1717)=dbm1(1923)
dbm2(1718)=dbm1(1924)
dbm2(1719)=dbm1(1925)
dbm2(1720)=dbm1(1926)
dbm2(1721)=dbm1(1927)
dbm2(1722)=dbm1(1928)
dbm2(1723)=dbm1(1929)
dbm2(1724)=dbm1(1930)
dbm2(1725)=dbm1(1931)
dbm2(1726)=dbm1(1932)
dbm2(1727)=dbm1(1933)
dbm2(1728)=dbm1(1934)
dbm2(1729)=dbm1(1935)
dbm2(1730)=dbm1(1936)
dbm2(1731)=dbm1(1937)
dbm2(1732)=dbm1(1938)
dbm2(1733)=dbm1(1939)
dbm2(1734)=dbm1(1940)
dbm2(1735)=dbm1(1941)
dbm2(1736)=dbm1(1942)
dbm2(1737)=dbm1(1943)
dbm2(1738)=dbm1(1944)
dbm2(1739)=dbm1(1945)
dbm2(1740)=dbm1(1946)
dbm2(1741)=dbm1(1947)
dbm2(1742)=dbm1(1948)
dbm2(1743)=dbm1(1949)
dbm2(1744)=dbm1(1950)
dbm2(1745)=dbm1(1951)
dbm2(1746)=dbm1(1952)
dbm2(1747)=dbm1(1953)
dbm2(1748)=dbm1(1954)
dbm2(1749)=dbm1(1955)
dbm2(1750)=dbm1(1956)
dbm2(1751)=dbm1(1957)
dbm2(1752)=dbm1(1958)
dbm2(1753)=dbm1(1959)
dbm2(1754)=dbm1(1960)
dbm2(1755)=dbm1(1961)
dbm2(1756)=dbm1(1962)
dbm2(1757)=dbm1(1963)
dbm2(1758)=dbm1(1964)
dbm2(1759)=dbm1(1965)
dbm2(1760)=dbm1(1966)
dbm2(1761)=dbm1(1967)
dbm2(1762)=dbm1(1968)
dbm2(1763)=dbm1(1969)
dbm2(1764)=dbm1(1970)
dbm2(1765)=dbm1(1971)
dbm2(1766)=dbm1(1972)
dbm2(1767)=dbm1(1973)
dbm2(1768)=dbm1(1974)
dbm2(1769)=dbm1(1975)
dbm2(1770)=dbm1(1976)
dbm2(1771)=dbm1(1977)
dbm2(1772)=dbm1(1978)
dbm2(1773)=dbm1(1979)
dbm2(1774)=dbm1(1980)
dbm2(1775)=dbm1(1981)
dbm2(1776)=dbm1(1982)
dbm2(1777)=dbm1(1983)
dbm2(1778)=dbm1(1984)
dbm2(1779)=dbm1(1985)
dbm2(1780)=dbm1(1986)
dbm2(1781)=dbm1(1987)
dbm2(1782)=dbm1(1988)
dbm2(1783)=dbm1(1989)
dbm2(1784)=dbm1(1990)
dbm2(1785)=dbm1(1991)
dbm2(1786)=dbm1(1992)
dbm2(1787)=dbm1(1993)
dbm2(1788)=dbm1(1994)
dbm2(1789)=dbm1(1995)
dbm2(1790)=dbm1(1996)
dbm2(1791)=dbm1(1997)
dbm2(1792)=dbm1(1998)
dbm2(1793)=dbm1(1999)
dbm2(1794)=dbm1(2000)
dbm2(1795)=dbm1(2001)
dbm2(1796)=dbm1(2002)
dbm2(1797)=dbm1(2003)
dbm2(1798)=dbm1(2004)
dbm2(1799)=dbm1(2005)
dbm2(1800)=dbm1(2006)
dbm2(1801)=dbm1(2007)
dbm2(1802)=dbm1(2008)
dbm2(1803)=dbm1(2009)
dbm2(1804)=dbm1(2010)
dbm2(1805)=dbm1(2011)
dbm2(1806)=dbm1(2012)
dbm2(1807)=dbm1(2013)
dbm2(1808)=dbm1(2014)
dbm2(1809)=dbm1(2015)
dbm2(1810)=dbm1(2016)
dbm2(1811)=dbm1(2017)
dbm2(1812)=dbm1(2018)
dbm2(1813)=dbm1(2019)
dbm2(1814)=dbm1(2020)
dbm2(1815)=dbm1(2021)
dbm2(1816)=dbm1(2022)
dbm2(1817)=dbm1(2023)
dbm2(1818)=dbm1(2024)
dbm2(1819)=dbm1(2025)
dbm2(1820)=dbm1(2026)
dbm2(1821)=dbm1(2027)
dbm2(1822)=dbm1(2028)
dbm2(1823)=dbm1(2029)
dbm2(1824)=dbm1(2030)
dbm2(1825)=dbm1(2031)
dbm2(1826)=dbm1(2032)
dbm2(1827)=dbm1(2033)
dbm2(1828)=dbm1(2034)
dbm2(1829)=dbm1(2035)
dbm2(1830)=dbm1(2036)
dbm2(1831)=dbm1(2037)
dbm2(1832)=dbm1(2038)
dbm2(1833)=dbm1(2039)
dbm2(1834)=dbm1(2040)
dbm2(1835)=dbm1(2041)
dbm2(1836)=dbm1(2042)
dbm2(1837)=dbm1(2043)
dbm2(1838)=dbm1(2044)
dbm2(1839)=dbm1(2045)
dbm2(1840)=dbm1(2046)
dbm2(1841)=dbm1(2047)
dbm2(1842)=dbm1(2048)
dbm2(1843)=dbm1(2049)
dbm2(1844)=dbm1(2050)
dbm2(1845)=dbm1(2051)
dbm2(1846)=dbm1(2052)
dbm2(1847)=dbm1(2053)
dbm2(1848)=dbm1(2054)
dbm2(1849)=dbm1(2055)
dbm2(1850)=dbm1(2056)
dbm2(1851)=dbm1(2057)
dbm2(1852)=dbm1(2058)
dbm2(1853)=dbm1(2059)
dbm2(1854)=dbm1(2060)
dbm2(1855)=dbm1(2061)
dbm2(1856)=dbm1(2062)
dbm2(1857)=dbm1(2063)
dbm2(1858)=dbm1(2064)
dbm2(1859)=dbm1(2065)
dbm2(1860)=dbm1(2066)
dbm2(1861)=dbm1(2067)
dbm2(1862)=dbm1(2068)
dbm2(1863)=dbm1(2069)
dbm2(1864)=dbm1(2070)
dbm2(1865)=dbm1(2071)
dbm2(1866)=dbm1(2072)
dbm2(1867)=dbm1(2073)
dbm2(1868)=dbm1(2074)
dbm2(1869)=dbm1(2075)
dbm2(1870)=dbm1(2076)
dbm2(1871)=dbm1(2077)
dbm2(1872)=dbm1(2078)
dbm2(1873)=dbm1(2079)
dbm2(1874)=dbm1(2080)
dbm2(1875)=dbm1(2081)
dbm2(1876)=dbm1(2082)
dbm2(1877)=dbm1(2083)
dbm2(1878)=dbm1(2084)
dbm2(1879)=dbm1(2085)
dbm2(1880)=dbm1(2086)
dbm2(1881)=dbm1(2087)
dbm2(1882)=dbm1(2088)
dbm2(1883)=dbm1(2089)
dbm2(1884)=dbm1(2090)
dbm2(1885)=dbm1(2091)
dbm2(1886)=dbm1(2092)
dbm2(1887)=dbm1(2093)
dbm2(1888)=dbm1(2094)
dbm2(1889)=dbm1(2095)
dbm2(1890)=dbm1(2096)
dbm2(1891)=dbm1(2097)
dbm2(1892)=dbm1(2098)
dbm2(1893)=dbm1(2099)
dbm2(1894)=dbm1(2100)
dbm2(1895)=dbm1(2101)
dbm2(1896)=dbm1(2102)
dbm2(1897)=dbm1(2103)
dbm2(1898)=dbm1(2104)
dbm2(1899)=dbm1(2105)
dbm2(1900)=dbm1(2106)
dbm2(1901)=dbm1(2107)
dbm2(1902)=dbm1(2108)
dbm2(1903)=dbm1(2109)
dbm2(1904)=dbm1(2110)
dbm2(1905)=dbm1(2111)
dbm2(1906)=dbm1(2112)
dbm2(1907)=dbm1(2113)
dbm2(1908)=dbm1(2114)
dbm2(1909)=dbm1(2115)
dbm2(1910)=dbm1(2116)
dbm2(1911)=dbm1(2117)
dbm2(1912)=dbm1(2118)
dbm2(1913)=dbm1(2119)
dbm2(1914)=dbm1(2120)
dbm2(1915)=dbm1(2121)
dbm2(1916)=dbm1(2122)
dbm2(1917)=dbm1(2123)
dbm2(1918)=dbm1(2125)
dbm2(1919)=dbm1(2126)
dbm2(1920)=dbm1(2127)
dbm2(1921)=dbm1(2128)
dbm2(1922)=dbm1(2129)
dbm2(1923)=dbm1(2130)
dbm2(1924)=dbm1(2131)
dbm2(1925)=dbm1(2132)
dbm2(1926)=dbm1(2133)
dbm2(1927)=dbm1(2134)
dbm2(1928)=dbm1(2135)
dbm2(1929)=dbm1(2136)
dbm2(1930)=dbm1(2137)
dbm2(1931)=dbm1(2138)
dbm2(1932)=dbm1(2140)
dbm2(1933)=dbm1(2142)
dbm2(1934)=dbm1(2143)
dbm2(1935)=dbm1(2144)
dbm2(1936)=dbm1(2145)
dbm2(1937)=dbm1(2146)
dbm2(1938)=dbm1(2147)
dbm2(1939)=dbm1(2148)
dbm2(1940)=dbm1(2149)
dbm2(1941)=dbm1(2150)
dbm2(1942)=dbm1(2151)
dbm2(1943)=dbm1(2152)
dbm2(1944)=dbm1(2153)
dbm2(1945)=dbm1(2154)
dbm2(1946)=dbm1(2156)
dbm2(1947)=dbm1(2157)
dbm2(1948)=dbm1(2159)
dbm2(1949)=dbm1(2161)
dbm2(1950)=dbm1(2162)
dbm2(1951)=dbm1(2163)
dbm2(1952)=dbm1(2164)
dbm2(1953)=dbm1(2165)
dbm2(1954)=dbm1(2166)
dbm2(1955)=dbm1(2167)
dbm2(1956)=dbm1(2168)
dbm2(1957)=dbm1(2169)
dbm2(1958)=dbm1(2170)
dbm2(1959)=dbm1(2171)
dbm2(1960)=dbm1(2173)
dbm2(1961)=dbm1(2174)
dbm2(1962)=dbm1(2176)
dbm2(1963)=dbm1(2177)
dbm2(1964)=dbm1(2179)
dbm2(1965)=dbm1(2181)
dbm2(1966)=dbm1(2182)
dbm2(1967)=dbm1(2183)
dbm2(1968)=dbm1(2184)
dbm2(1969)=dbm1(2185)
dbm2(1970)=dbm1(2186)
dbm2(1971)=dbm1(2187)
dbm2(1972)=dbm1(2188)
dbm2(1973)=dbm1(2189)
dbm2(1974)=dbm1(2190)
dbm2(1975)=dbm1(2191)
dbm2(1976)=dbm1(2193)
dbm2(1977)=dbm1(2194)
dbm2(1978)=dbm1(2196)
dbm2(1979)=dbm1(2197)
dbm2(1980)=dbm1(2199)
dbm2(1981)=dbm1(2200)
dbm2(1982)=dbm1(2201)
dbm2(1983)=dbm1(2203)
dbm2(1984)=dbm1(2204)
dbm2(1985)=dbm1(2206)
dbm2(1986)=dbm1(2207)
dbm2(1987)=dbm1(2208)
dbm2(1988)=dbm1(2209)
dbm2(1989)=dbm1(2210)
dbm2(1990)=dbm1(2211)
dbm2(1991)=dbm1(2212)
dbm2(1992)=dbm1(2213)
dbm2(1993)=dbm1(2214)
dbm2(1994)=dbm1(2216)
dbm2(1995)=dbm1(2217)
dbm2(1996)=dbm1(2219)
dbm2(1997)=dbm1(2220)
dbm2(1998)=dbm1(2222)
dbm2(1999)=dbm1(2223)
dbm2(2000)=dbm1(2224)
dbm2(2001)=dbm1(2226)
dbm2(2002)=dbm1(2227)
dbm2(2003)=dbm1(2228)
dbm2(2004)=dbm1(2229)
dbm2(2005)=dbm1(2230)
dbm2(2006)=dbm1(2231)
dbm2(2007)=dbm1(2232)
dbm2(2008)=dbm1(2233)
dbm2(2009)=dbm1(2235)
dbm2(2010)=dbm1(2236)
dbm2(2011)=dbm1(2238)
dbm2(2012)=dbm1(2239)
dbm2(2013)=dbm1(2240)
dbm2(2014)=dbm1(2242)
dbm2(2015)=dbm1(2243)
dbm2(2016)=dbm1(2244)
dbm2(2017)=dbm1(2245)
dbm2(2018)=dbm1(2246)
dbm2(2019)=dbm1(2247)
dbm2(2020)=dbm1(2249)
dbm2(2021)=dbm1(2250)
dbm2(2022)=dbm1(2252)
dbm2(2023)=dbm1(2253)
dbm2(2024)=dbm1(2254)
dbm2(2025)=dbm1(2255)
dbm2(2026)=dbm1(2256)
dbm2(2027)=dbm1(2258)
dbm2(2028)=dbm1(2259)
dbm2(2029)=dbm1(2260)
dbm2(2030)=dbm1(2261)
dbm2(2031)=dbm1(2262)
dbm2(2032)=dbm1(2263)
dbm2(2033)=dbm1(2291)
dbm2(2034)=dbm1(2292)
dbm2(2035)=dbm1(2293)
dbm2(2036)=dbm1(2294)
dbm2(2037)=dbm1(2295)
dbm2(2038)=dbm1(2296)
dbm2(2039)=dbm1(2297)
dbm2(2040)=dbm1(2298)
dbm2(2041)=dbm1(2299)
dbm2(2042)=dbm1(2300)
dbm2(2043)=dbm1(2301)
dbm2(2044)=dbm1(2302)
dbm2(2045)=dbm1(2303)
dbm2(2046)=dbm1(2304)
dbm2(2047)=dbm1(2305)
dbm2(2048)=dbm1(2306)
dbm2(2049)=dbm1(2307)
dbm2(2050)=dbm1(2308)
dbm2(2051)=dbm1(2309)
dbm2(2052)=dbm1(2310)
dbm2(2053)=dbm1(2311)
dbm2(2054)=dbm1(2312)
dbm2(2055)=dbm1(2313)
dbm2(2056)=dbm1(2314)
dbm2(2057)=dbm1(2315)
dbm2(2058)=dbm1(2316)
dbm2(2059)=dbm1(2317)
dbm2(2060)=dbm1(2318)
dbm2(2061)=dbm1(2319)
dbm2(2062)=dbm1(2320)
dbm2(2063)=dbm1(2321)
dbm2(2064)=dbm1(2322)
dbm2(2065)=dbm1(2323)
dbm2(2066)=dbm1(2324)
dbm2(2067)=dbm1(2325)
dbm2(2068)=dbm1(2326)
dbm2(2069)=dbm1(2327)
dbm2(2070)=dbm1(2328)
dbm2(2071)=dbm1(2329)
dbm2(2072)=dbm1(2330)
dbm2(2073)=dbm1(2331)
dbm2(2074)=dbm1(2332)
dbm2(2075)=dbm1(2333)
dbm2(2076)=dbm1(2334)
dbm2(2077)=dbm1(2335)
dbm2(2078)=dbm1(2336)
dbm2(2079)=dbm1(2337)
dbm2(2080)=dbm1(2338)
dbm2(2081)=dbm1(2339)
dbm2(2082)=dbm1(2340)
dbm2(2083)=dbm1(2341)
dbm2(2084)=dbm1(2342)
dbm2(2085)=dbm1(2343)
dbm2(2086)=dbm1(2344)
dbm2(2087)=dbm1(2345)
dbm2(2088)=dbm1(2346)
dbm2(2089)=dbm1(2347)
dbm2(2090)=dbm1(2348)
dbm2(2091)=dbm1(2349)
dbm2(2092)=dbm1(2350)
dbm2(2093)=dbm1(2351)
dbm2(2094)=dbm1(2352)
dbm2(2095)=dbm1(2353)
dbm2(2096)=dbm1(2354)
dbm2(2097)=dbm1(2355)
dbm2(2098)=dbm1(2356)
dbm2(2099)=dbm1(2357)
dbm2(2100)=dbm1(2358)
dbm2(2101)=dbm1(2359)
dbm2(2102)=dbm1(2360)
dbm2(2103)=dbm1(2361)
dbm2(2104)=dbm1(2362)
dbm2(2105)=dbm1(2363)
dbm2(2106)=dbm1(2364)
dbm2(2107)=dbm1(2365)
dbm2(2108)=dbm1(2366)
dbm2(2109)=dbm1(2367)
dbm2(2110)=dbm1(2368)
dbm2(2111)=dbm1(2369)
dbm2(2112)=dbm1(2370)
dbm2(2113)=dbm1(2371)
dbm2(2114)=dbm1(2372)
dbm2(2115)=dbm1(2373)
dbm2(2116)=dbm1(2374)
dbm2(2117)=dbm1(2375)
dbm2(2118)=dbm1(2376)
dbm2(2119)=dbm1(2377)
dbm2(2120)=dbm1(2378)
dbm2(2121)=dbm1(2379)
dbm2(2122)=dbm1(2380)
dbm2(2123)=dbm1(2381)
dbm2(2124)=dbm1(2382)
dbm2(2125)=dbm1(2383)
dbm2(2126)=dbm1(2384)
dbm2(2127)=dbm1(2385)
dbm2(2128)=dbm1(2386)
dbm2(2129)=dbm1(2387)
dbm2(2130)=dbm1(2388)
dbm2(2131)=dbm1(2389)
dbm2(2132)=dbm1(2390)
dbm2(2133)=dbm1(2391)
dbm2(2134)=dbm1(2392)
dbm2(2135)=dbm1(2393)
dbm2(2136)=dbm1(2394)
dbm2(2137)=dbm1(2395)
dbm2(2138)=dbm1(2396)
dbm2(2139)=dbm1(2397)
dbm2(2140)=dbm1(2398)
dbm2(2141)=dbm1(2399)
dbm2(2142)=dbm1(2400)
dbm2(2143)=dbm1(2401)
dbm2(2144)=dbm1(2402)
dbm2(2145)=dbm1(2403)
dbm2(2146)=dbm1(2404)
dbm2(2147)=dbm1(2405)
dbm2(2148)=dbm1(2406)
dbm2(2149)=dbm1(2407)
dbm2(2150)=dbm1(2408)
dbm2(2151)=dbm1(2409)
dbm2(2152)=dbm1(2410)
dbm2(2153)=dbm1(2411)
dbm2(2154)=dbm1(2412)
dbm2(2155)=dbm1(2413)
dbm2(2156)=dbm1(2414)
dbm2(2157)=dbm1(2415)
dbm2(2158)=dbm1(2416)
dbm2(2159)=dbm1(2417)
dbm2(2160)=dbm1(2418)
dbm2(2161)=dbm1(2419)
dbm2(2162)=dbm1(2420)
dbm2(2163)=dbm1(2421)
dbm2(2164)=dbm1(2422)
dbm2(2165)=dbm1(2423)
dbm2(2166)=dbm1(2424)
dbm2(2167)=dbm1(2425)
dbm2(2168)=dbm1(2426)
dbm2(2169)=dbm1(2427)
dbm2(2170)=dbm1(2428)
dbm2(2171)=dbm1(2429)
dbm2(2172)=dbm1(2430)
dbm2(2173)=dbm1(2431)
dbm2(2174)=dbm1(2432)
dbm2(2175)=dbm1(2433)
dbm2(2176)=dbm1(2434)
dbm2(2177)=dbm1(2435)
dbm2(2178)=dbm1(2436)
dbm2(2179)=dbm1(2437)
dbm2(2180)=dbm1(2438)
dbm2(2181)=dbm1(2439)
dbm2(2182)=dbm1(2440)
dbm2(2183)=dbm1(2441)
dbm2(2184)=dbm1(2442)
dbm2(2185)=dbm1(2443)
dbm2(2186)=dbm1(2444)
dbm2(2187)=dbm1(2445)
dbm2(2188)=dbm1(2446)
dbm2(2189)=dbm1(2447)
dbm2(2190)=dbm1(2448)
dbm2(2191)=dbm1(2449)
dbm2(2192)=dbm1(2450)
dbm2(2193)=dbm1(2451)
dbm2(2194)=dbm1(2452)
dbm2(2195)=dbm1(2453)
dbm2(2196)=dbm1(2454)
dbm2(2197)=dbm1(2455)
dbm2(2198)=dbm1(2456)
dbm2(2199)=dbm1(2457)
dbm2(2200)=dbm1(2458)
dbm2(2201)=dbm1(2459)
dbm2(2202)=dbm1(2460)
dbm2(2203)=dbm1(2461)
dbm2(2204)=dbm1(2462)
dbm2(2205)=dbm1(2463)
dbm2(2206)=dbm1(2464)
dbm2(2207)=dbm1(2465)
dbm2(2208)=dbm1(2466)
dbm2(2209)=dbm1(2467)
dbm2(2210)=dbm1(2468)
dbm2(2211)=dbm1(2469)
dbm2(2212)=dbm1(2470)
dbm2(2213)=dbm1(2471)
dbm2(2214)=dbm1(2472)
dbm2(2215)=dbm1(2473)
dbm2(2216)=dbm1(2474)
dbm2(2217)=dbm1(2475)
dbm2(2218)=dbm1(2476)
dbm2(2219)=dbm1(2477)
dbm2(2220)=dbm1(2478)
dbm2(2221)=dbm1(2479)
dbm2(2222)=dbm1(2480)
dbm2(2223)=dbm1(2481)
dbm2(2224)=dbm1(2482)
dbm2(2225)=dbm1(2483)
dbm2(2226)=dbm1(2484)
dbm2(2227)=dbm1(2485)
dbm2(2228)=dbm1(2486)
dbm2(2229)=dbm1(2487)
dbm2(2230)=dbm1(2488)
dbm2(2231)=dbm1(2489)
dbm2(2232)=dbm1(2490)
dbm2(2233)=dbm1(2491)
dbm2(2234)=dbm1(2492)
dbm2(2235)=dbm1(2493)
dbm2(2236)=dbm1(2494)
dbm2(2237)=dbm1(2495)
dbm2(2238)=dbm1(2496)
dbm2(2239)=dbm1(2497)
dbm2(2240)=dbm1(2498)
dbm2(2241)=dbm1(2499)
dbm2(2242)=dbm1(2500)
dbm2(2243)=dbm1(2501)
dbm2(2244)=dbm1(2502)
dbm2(2245)=dbm1(2503)
dbm2(2246)=dbm1(2504)
dbm2(2247)=dbm1(2505)
dbm2(2248)=dbm1(2506)
dbm2(2249)=dbm1(2507)
dbm2(2250)=dbm1(2508)
dbm2(2251)=dbm1(2509)
dbm2(2252)=dbm1(2510)
dbm2(2253)=dbm1(2511)
dbm2(2254)=dbm1(2512)
dbm2(2255)=dbm1(2513)
dbm2(2256)=dbm1(2514)
dbm2(2257)=dbm1(2515)
dbm2(2258)=dbm1(2516)
dbm2(2259)=dbm1(2517)
dbm2(2260)=dbm1(2518)
dbm2(2261)=dbm1(2519)
dbm2(2262)=dbm1(2520)
dbm2(2263)=dbm1(2521)
dbm2(2264)=dbm1(2522)
dbm2(2265)=dbm1(2523)
dbm2(2266)=dbm1(2524)
dbm2(2267)=dbm1(2525)
dbm2(2268)=dbm1(2526)
dbm2(2269)=dbm1(2527)
dbm2(2270)=dbm1(2528)
dbm2(2271)=dbm1(2529)
dbm2(2272)=dbm1(2530)
dbm2(2273)=dbm1(2531)
dbm2(2274)=dbm1(2532)
dbm2(2275)=dbm1(2533)
dbm2(2276)=dbm1(2534)
dbm2(2277)=dbm1(2535)
dbm2(2278)=dbm1(2536)
dbm2(2279)=dbm1(2537)
dbm2(2280)=dbm1(2538)
dbm2(2281)=dbm1(2539)
dbm2(2282)=dbm1(2540)
dbm2(2283)=dbm1(2541)
dbm2(2284)=dbm1(2542)
dbm2(2285)=dbm1(2543)
dbm2(2286)=dbm1(2544)
dbm2(2287)=dbm1(2545)
dbm2(2288)=dbm1(2546)
dbm2(2289)=dbm1(2547)
dbm2(2290)=dbm1(2548)
dbm2(2291)=dbm1(2549)
dbm2(2292)=dbm1(2550)
dbm2(2293)=dbm1(2551)
dbm2(2294)=dbm1(2552)
dbm2(2295)=dbm1(2553)
dbm2(2296)=dbm1(2554)
dbm2(2297)=dbm1(2555)
dbm2(2298)=dbm1(2556)
dbm2(2299)=dbm1(2557)
dbm2(2300)=dbm1(2558)
dbm2(2301)=dbm1(2559)
dbm2(2302)=dbm1(2560)
dbm2(2303)=dbm1(2561)
dbm2(2304)=dbm1(2562)
dbm2(2305)=dbm1(2563)
dbm2(2306)=dbm1(2564)
dbm2(2307)=dbm1(2565)
dbm2(2308)=dbm1(2566)
dbm2(2309)=dbm1(2567)
dbm2(2310)=dbm1(2568)
dbm2(2311)=dbm1(2569)
dbm2(2312)=dbm1(2570)
dbm2(2313)=dbm1(2571)
dbm2(2314)=dbm1(2572)
dbm2(2315)=dbm1(2573)
dbm2(2316)=dbm1(2574)
dbm2(2317)=dbm1(2575)
dbm2(2318)=dbm1(2576)
dbm2(2319)=dbm1(2577)
dbm2(2320)=dbm1(2578)
dbm2(2321)=dbm1(2579)
dbm2(2322)=dbm1(2580)
dbm2(2323)=dbm1(2581)
dbm2(2324)=dbm1(2582)
dbm2(2325)=dbm1(2583)
dbm2(2326)=dbm1(2584)
dbm2(2327)=dbm1(2585)
dbm2(2328)=dbm1(2586)
dbm2(2329)=dbm1(2587)
dbm2(2330)=dbm1(2588)
dbm2(2331)=dbm1(2589)
dbm2(2332)=dbm1(2590)
dbm2(2333)=dbm1(2591)
dbm2(2334)=dbm1(2592)
dbm2(2335)=dbm1(2593)
dbm2(2336)=dbm1(2594)
dbm2(2337)=dbm1(2595)
dbm2(2338)=dbm1(2596)
dbm2(2339)=dbm1(2597)
dbm2(2340)=dbm1(2598)
dbm2(2341)=dbm1(2599)
dbm2(2342)=dbm1(2600)
dbm2(2343)=dbm1(2601)
dbm2(2344)=dbm1(2602)
dbm2(2345)=dbm1(2603)
dbm2(2346)=dbm1(2604)
dbm2(2347)=dbm1(2605)
dbm2(2348)=dbm1(2606)
dbm2(2349)=dbm1(2607)
dbm2(2350)=dbm1(2608)
dbm2(2351)=dbm1(2609)
dbm2(2352)=dbm1(2610)
dbm2(2353)=dbm1(2611)
dbm2(2354)=dbm1(2612)
dbm2(2355)=dbm1(2613)
dbm2(2356)=dbm1(2614)
dbm2(2357)=dbm1(2615)
dbm2(2358)=dbm1(2616)
dbm2(2359)=dbm1(2617)
dbm2(2360)=dbm1(2618)
dbm2(2361)=dbm1(2619)
dbm2(2362)=dbm1(2620)
dbm2(2363)=dbm1(2621)
dbm2(2364)=dbm1(2622)
dbm2(2365)=dbm1(2623)
dbm2(2366)=dbm1(2624)
dbm2(2367)=dbm1(2625)
dbm2(2368)=dbm1(2626)
dbm2(2369)=dbm1(2627)
dbm2(2370)=dbm1(2628)
dbm2(2371)=dbm1(2629)
dbm2(2372)=dbm1(2630)
dbm2(2373)=dbm1(2631)
dbm2(2374)=dbm1(2632)
dbm2(2375)=dbm1(2633)
dbm2(2376)=dbm1(2634)
dbm2(2377)=dbm1(2635)
dbm2(2378)=dbm1(2636)
dbm2(2379)=dbm1(2637)
dbm2(2380)=dbm1(2638)
dbm2(2381)=dbm1(2639)
dbm2(2382)=dbm1(2640)
dbm2(2383)=dbm1(2641)
dbm2(2384)=dbm1(2642)
dbm2(2385)=dbm1(2643)
dbm2(2386)=dbm1(2644)
dbm2(2387)=dbm1(2645)
dbm2(2388)=dbm1(2646)
dbm2(2389)=dbm1(2647)
dbm2(2390)=dbm1(2648)
dbm2(2391)=dbm1(2649)
dbm2(2392)=dbm1(2650)
dbm2(2393)=dbm1(2651)
dbm2(2394)=dbm1(2652)
dbm2(2395)=dbm1(2653)
dbm2(2396)=dbm1(2654)
dbm2(2397)=dbm1(2655)
dbm2(2398)=dbm1(2656)
dbm2(2399)=dbm1(2657)
dbm2(2400)=dbm1(2658)
dbm2(2401)=dbm1(2659)
dbm2(2402)=dbm1(2660)
dbm2(2403)=dbm1(2661)
dbm2(2404)=dbm1(2662)
dbm2(2405)=dbm1(2663)
dbm2(2406)=dbm1(2664)
dbm2(2407)=dbm1(2665)
dbm2(2408)=dbm1(2666)
dbm2(2409)=dbm1(2667)
dbm2(2410)=dbm1(2668)
dbm2(2411)=dbm1(2669)
dbm2(2412)=dbm1(2670)
dbm2(2413)=dbm1(2671)
dbm2(2414)=dbm1(2672)
dbm2(2415)=dbm1(2673)
dbm2(2416)=dbm1(2674)
dbm2(2417)=dbm1(2675)
dbm2(2418)=dbm1(2676)
dbm2(2419)=dbm1(2677)
dbm2(2420)=dbm1(2678)
dbm2(2421)=dbm1(2679)
dbm2(2422)=dbm1(2680)
dbm2(2423)=dbm1(2681)
dbm2(2424)=dbm1(2682)
dbm2(2425)=dbm1(2683)
dbm2(2426)=dbm1(2684)
dbm2(2427)=dbm1(2685)
dbm2(2428)=dbm1(2686)
dbm2(2429)=dbm1(2687)
dbm2(2430)=dbm1(2688)
dbm2(2431)=dbm1(2689)
dbm2(2432)=dbm1(2690)
dbm2(2433)=dbm1(2691)
dbm2(2434)=dbm1(2692)
dbm2(2435)=dbm1(2693)
dbm2(2436)=dbm1(2694)
dbm2(2437)=dbm1(2695)
dbm2(2438)=dbm1(2696)
dbm2(2439)=dbm1(2697)
dbm2(2440)=dbm1(2698)
dbm2(2441)=dbm1(2699)
dbm2(2442)=dbm1(2700)
dbm2(2443)=dbm1(2701)
dbm2(2444)=dbm1(2702)
dbm2(2445)=dbm1(2703)
dbm2(2446)=dbm1(2704)
dbm2(2447)=dbm1(2705)
dbm2(2448)=dbm1(2706)
dbm2(2449)=dbm1(2707)
dbm2(2450)=dbm1(2708)
dbm2(2451)=dbm1(2709)
dbm2(2452)=dbm1(2710)
dbm2(2453)=dbm1(2711)
dbm2(2454)=dbm1(2712)
dbm2(2455)=dbm1(2713)
dbm2(2456)=dbm1(2714)
dbm2(2457)=dbm1(2715)
dbm2(2458)=dbm1(2716)
dbm2(2459)=dbm1(2717)
dbm2(2460)=dbm1(2718)
dbm2(2461)=dbm1(2719)
dbm2(2462)=dbm1(2720)
dbm2(2463)=dbm1(2721)
dbm2(2464)=dbm1(2722)
dbm2(2465)=dbm1(2723)
dbm2(2466)=dbm1(2724)
dbm2(2467)=dbm1(2725)
dbm2(2468)=dbm1(2726)
dbm2(2469)=dbm1(2727)
dbm2(2470)=dbm1(2728)
dbm2(2471)=dbm1(2729)
dbm2(2472)=dbm1(2730)
dbm2(2473)=dbm1(2731)
dbm2(2474)=dbm1(2732)
dbm2(2475)=dbm1(2733)
dbm2(2476)=dbm1(2734)
dbm2(2477)=dbm1(2735)
dbm2(2478)=dbm1(2736)
dbm2(2479)=dbm1(2737)
dbm2(2480)=dbm1(2738)
dbm2(2481)=dbm1(2739)
dbm2(2482)=dbm1(2740)
dbm2(2483)=dbm1(2741)
dbm2(2484)=dbm1(2742)
dbm2(2485)=dbm1(2743)
dbm2(2486)=dbm1(2744)
dbm2(2487)=dbm1(2745)
dbm2(2488)=dbm1(2746)
dbm2(2489)=dbm1(2747)
dbm2(2490)=dbm1(2748)
dbm2(2491)=dbm1(2749)
dbm2(2492)=dbm1(2750)
dbm2(2493)=dbm1(2751)
dbm2(2494)=dbm1(2752)
dbm2(2495)=dbm1(2753)
dbm2(2496)=dbm1(2754)
dbm2(2497)=dbm1(2755)
dbm2(2498)=dbm1(2756)
dbm2(2499)=dbm1(2757)
dbm2(2500)=dbm1(2758)
dbm2(2501)=dbm1(2759)
dbm2(2502)=dbm1(2760)
dbm2(2503)=dbm1(2761)
dbm2(2504)=dbm1(2762)
dbm2(2505)=dbm1(2763)
dbm2(2506)=dbm1(2764)
dbm2(2507)=dbm1(2765)
dbm2(2508)=dbm1(2766)
dbm2(2509)=dbm1(2767)
dbm2(2510)=dbm1(2768)
dbm2(2511)=dbm1(2769)
dbm2(2512)=dbm1(2770)
dbm2(2513)=dbm1(2771)
dbm2(2514)=dbm1(2772)
dbm2(2515)=dbm1(2773)
dbm2(2516)=dbm1(2774)
dbm2(2517)=dbm1(2775)
dbm2(2518)=dbm1(2776)
dbm2(2519)=dbm1(2777)
dbm2(2520)=dbm1(2778)
dbm2(2521)=dbm1(2779)
dbm2(2522)=dbm1(2780)
dbm2(2523)=dbm1(2781)
dbm2(2524)=dbm1(2782)
dbm2(2525)=dbm1(2783)
dbm2(2526)=dbm1(2784)
dbm2(2527)=dbm1(2785)
dbm2(2528)=dbm1(2786)
dbm2(2529)=dbm1(2787)
dbm2(2530)=dbm1(2788)
dbm2(2531)=dbm1(2789)
dbm2(2532)=dbm1(2790)
dbm2(2533)=dbm1(2791)
dbm2(2534)=dbm1(2792)
dbm2(2535)=dbm1(2793)
dbm2(2536)=dbm1(2794)
dbm2(2537)=dbm1(2795)
dbm2(2538)=dbm1(2796)
dbm2(2539)=dbm1(2797)
dbm2(2540)=dbm1(2798)
dbm2(2541)=dbm1(2799)
dbm2(2542)=dbm1(2800)
dbm2(2543)=dbm1(2801)
dbm2(2544)=dbm1(2802)
dbm2(2545)=dbm1(2803)
dbm2(2546)=dbm1(2804)
dbm2(2547)=dbm1(2805)
dbm2(2548)=dbm1(2806)
dbm2(2549)=dbm1(2807)
dbm2(2550)=dbm1(2808)
dbm2(2551)=dbm1(2809)
dbm2(2552)=dbm1(2810)
dbm2(2553)=dbm1(2811)
dbm2(2554)=dbm1(2812)
dbm2(2555)=dbm1(2813)
dbm2(2556)=dbm1(2814)
dbm2(2557)=dbm1(2815)
dbm2(2558)=dbm1(2816)
dbm2(2559)=dbm1(2817)
dbm2(2560)=dbm1(2818)
dbm2(2561)=dbm1(2819)
dbm2(2562)=dbm1(2820)
dbm2(2563)=dbm1(2821)
dbm2(2564)=dbm1(2822)
dbm2(2565)=dbm1(2823)
dbm2(2566)=dbm1(2824)
dbm2(2567)=dbm1(2825)
dbm2(2568)=dbm1(2826)
dbm2(2569)=dbm1(2827)
dbm2(2570)=dbm1(2828)
dbm2(2571)=dbm1(2829)
dbm2(2572)=dbm1(2830)
dbm2(2573)=dbm1(2831)
dbm2(2574)=dbm1(2832)
dbm2(2575)=dbm1(2833)
dbm2(2576)=dbm1(2834)
dbm2(2577)=dbm1(2835)
dbm2(2578)=dbm1(2836)
dbm2(2579)=dbm1(2837)
dbm2(2580)=dbm1(2838)
dbm2(2581)=dbm1(2839)
dbm2(2582)=dbm1(2840)
dbm2(2583)=dbm1(2841)
dbm2(2584)=dbm1(2842)
dbm2(2585)=dbm1(2843)
dbm2(2586)=dbm1(2844)
dbm2(2587)=dbm1(2845)
dbm2(2588)=dbm1(2846)
dbm2(2589)=dbm1(2847)
dbm2(2590)=dbm1(2848)
dbm2(2591)=dbm1(2849)
dbm2(2592)=dbm1(2850)
dbm2(2593)=dbm1(2851)
dbm2(2594)=dbm1(2852)
dbm2(2595)=dbm1(2853)
dbm2(2596)=dbm1(2854)
dbm2(2597)=dbm1(2855)
dbm2(2598)=dbm1(2856)
dbm2(2599)=dbm1(2857)
dbm2(2600)=dbm1(2858)
dbm2(2601)=dbm1(2859)
dbm2(2602)=dbm1(2860)
dbm2(2603)=dbm1(2861)
dbm2(2604)=dbm1(2862)
dbm2(2605)=dbm1(2863)
dbm2(2606)=dbm1(2864)
dbm2(2607)=dbm1(2865)
dbm2(2608)=dbm1(2866)
dbm2(2609)=dbm1(2867)
dbm2(2610)=dbm1(2868)
dbm2(2611)=dbm1(2869)
dbm2(2612)=dbm1(2870)
dbm2(2613)=dbm1(2871)
dbm2(2614)=dbm1(2872)
dbm2(2615)=dbm1(2873)
dbm2(2616)=dbm1(2874)
dbm2(2617)=dbm1(2875)
dbm2(2618)=dbm1(2876)
dbm2(2619)=dbm1(2877)
dbm2(2620)=dbm1(2878)
dbm2(2621)=dbm1(2879)
dbm2(2622)=dbm1(2880)
dbm2(2623)=dbm1(2881)
dbm2(2624)=dbm1(2882)
dbm2(2625)=dbm1(2883)
dbm2(2626)=dbm1(2884)
dbm2(2627)=dbm1(2885)
dbm2(2628)=dbm1(2886)
dbm2(2629)=dbm1(2887)
dbm2(2630)=dbm1(2888)
dbm2(2631)=dbm1(2889)
dbm2(2632)=dbm1(2890)
dbm2(2633)=dbm1(2891)
dbm2(2634)=dbm1(2892)
dbm2(2635)=dbm1(2893)
dbm2(2636)=dbm1(2894)
dbm2(2637)=dbm1(2895)
dbm2(2638)=dbm1(2896)
dbm2(2639)=dbm1(2897)
dbm2(2640)=dbm1(2898)
dbm2(2641)=dbm1(2899)
dbm2(2642)=dbm1(2900)
dbm2(2643)=dbm1(2901)
dbm2(2644)=dbm1(2902)
dbm2(2645)=dbm1(2903)
dbm2(2646)=dbm1(2904)
dbm2(2647)=dbm1(2905)
dbm2(2648)=dbm1(2906)
dbm2(2649)=dbm1(2907)
dbm2(2650)=dbm1(2908)
dbm2(2651)=dbm1(2909)
dbm2(2652)=dbm1(2910)
dbm2(2653)=dbm1(2911)
dbm2(2654)=dbm1(2912)
dbm2(2655)=dbm1(2913)
dbm2(2656)=dbm1(2914)
dbm2(2657)=dbm1(2915)
dbm2(2658)=dbm1(2916)
dbm2(2659)=dbm1(2917)
dbm2(2660)=dbm1(2918)
dbm2(2661)=dbm1(2919)
dbm2(2662)=dbm1(2920)
dbm2(2663)=dbm1(2921)
dbm2(2664)=dbm1(2922)
dbm2(2665)=dbm1(2923)
dbm2(2666)=dbm1(2924)
dbm2(2667)=dbm1(2925)
dbm2(2668)=dbm1(2926)
dbm2(2669)=dbm1(2927)
dbm2(2670)=dbm1(2928)
dbm2(2671)=dbm1(2929)
dbm2(2672)=dbm1(2930)
dbm2(2673)=dbm1(2931)
dbm2(2674)=dbm1(2932)
dbm2(2675)=dbm1(2933)
dbm2(2676)=dbm1(2934)
dbm2(2677)=dbm1(2935)
dbm2(2678)=dbm1(2936)
dbm2(2679)=dbm1(2937)
dbm2(2680)=dbm1(2938)
dbm2(2681)=dbm1(2939)
dbm2(2682)=dbm1(2940)
dbm2(2683)=dbm1(2941)
dbm2(2684)=dbm1(2942)
dbm2(2685)=dbm1(2943)
dbm2(2686)=dbm1(2944)
dbm2(2687)=dbm1(2945)
dbm2(2688)=dbm1(2946)
dbm2(2689)=dbm1(2947)
dbm2(2690)=dbm1(2948)
dbm2(2691)=dbm1(2949)
dbm2(2692)=dbm1(2950)
dbm2(2693)=dbm1(2951)
dbm2(2694)=dbm1(2952)
dbm2(2695)=dbm1(2953)
dbm2(2696)=dbm1(2954)
dbm2(2697)=dbm1(2955)
dbm2(2698)=dbm1(2956)
dbm2(2699)=dbm1(2957)
dbm2(2700)=dbm1(2958)
dbm2(2701)=dbm1(2959)
dbm2(2702)=dbm1(2960)
dbm2(2703)=dbm1(2961)
dbm2(2704)=dbm1(2962)
dbm2(2705)=dbm1(2963)
dbm2(2706)=dbm1(2964)
dbm2(2707)=dbm1(2965)
dbm2(2708)=dbm1(2966)
dbm2(2709)=dbm1(2967)
dbm2(2710)=dbm1(2968)
dbm2(2711)=dbm1(2969)
dbm2(2712)=dbm1(2970)
dbm2(2713)=dbm1(2971)
dbm2(2714)=dbm1(2972)
dbm2(2715)=dbm1(2973)
dbm2(2716)=dbm1(2974)
dbm2(2717)=dbm1(2975)
dbm2(2718)=dbm1(2976)
dbm2(2719)=dbm1(2977)
dbm2(2720)=dbm1(2978)
dbm2(2721)=dbm1(2979)
dbm2(2722)=dbm1(2980)
dbm2(2723)=dbm1(2981)
dbm2(2724)=dbm1(2982)
dbm2(2725)=dbm1(2983)
dbm2(2726)=dbm1(2984)
dbm2(2727)=dbm1(2985)
dbm2(2728)=dbm1(2986)
dbm2(2729)=dbm1(2987)
dbm2(2730)=dbm1(2988)
dbm2(2731)=dbm1(2989)
dbm2(2732)=dbm1(2990)
dbm2(2733)=dbm1(2991)
dbm2(2734)=dbm1(2992)
dbm2(2735)=dbm1(2993)
dbm2(2736)=dbm1(2994)
dbm2(2737)=dbm1(2995)
dbm2(2738)=dbm1(2996)
dbm2(2739)=dbm1(2997)
dbm2(2740)=dbm1(2998)
dbm2(2741)=dbm1(2999)
dbm2(2742)=dbm1(3000)
dbm2(2743)=dbm1(3001)
dbm2(2744)=dbm1(3002)
dbm2(2745)=dbm1(3003)
dbm2(2746)=dbm1(3004)
dbm2(2747)=dbm1(3005)
dbm2(2748)=dbm1(3006)
dbm2(2749)=dbm1(3007)
dbm2(2750)=dbm1(3008)
dbm2(2751)=dbm1(3009)
dbm2(2752)=dbm1(3010)
dbm2(2753)=dbm1(3011)
dbm2(2754)=dbm1(3012)
dbm2(2755)=dbm1(3013)
dbm2(2756)=dbm1(3014)
dbm2(2757)=dbm1(3015)
dbm2(2758)=dbm1(3016)
dbm2(2759)=dbm1(3017)
dbm2(2760)=dbm1(3018)
dbm2(2761)=dbm1(3019)
dbm2(2762)=dbm1(3020)
dbm2(2763)=dbm1(3021)
dbm2(2764)=dbm1(3022)
dbm2(2765)=dbm1(3023)
dbm2(2766)=dbm1(3024)
dbm2(2767)=dbm1(3025)
dbm2(2768)=dbm1(3026)
dbm2(2769)=dbm1(3027)
dbm2(2770)=dbm1(3028)
dbm2(2771)=dbm1(3029)
dbm2(2772)=dbm1(3030)
dbm2(2773)=dbm1(3031)
dbm2(2774)=dbm1(3032)
dbm2(2775)=dbm1(3033)
dbm2(2776)=dbm1(3034)
dbm2(2777)=dbm1(3035)
dbm2(2778)=dbm1(3036)
dbm2(2779)=dbm1(3037)
dbm2(2780)=dbm1(3038)
dbm2(2781)=dbm1(3039)
dbm2(2782)=dbm1(3040)
dbm2(2783)=dbm1(3041)
dbm2(2784)=dbm1(3042)
dbm2(2785)=dbm1(3043)
dbm2(2786)=dbm1(3044)
dbm2(2787)=dbm1(3045)
dbm2(2788)=dbm1(3046)
dbm2(2789)=dbm1(3047)
dbm2(2790)=dbm1(3048)
dbm2(2791)=dbm1(3049)
dbm2(2792)=dbm1(3050)
dbm2(2793)=dbm1(3051)
dbm2(2794)=dbm1(3052)
dbm2(2795)=dbm1(3053)
dbm2(2796)=dbm1(3054)
dbm2(2797)=dbm1(3055)
dbm2(2798)=dbm1(3056)
dbm2(2799)=dbm1(3057)
dbm2(2800)=dbm1(3058)
dbm2(2801)=dbm1(3059)
dbm2(2802)=dbm1(3060)
dbm2(2803)=dbm1(3061)
dbm2(2804)=dbm1(3062)
dbm2(2805)=dbm1(3063)
dbm2(2806)=dbm1(3064)
dbm2(2807)=dbm1(3065)
dbm2(2808)=dbm1(3066)
dbm2(2809)=dbm1(3067)
dbm2(2810)=dbm1(3068)
dbm2(2811)=dbm1(3069)
dbm2(2812)=dbm1(3070)
dbm2(2813)=dbm1(3071)
dbm2(2814)=dbm1(3072)
dbm2(2815)=dbm1(3073)
dbm2(2816)=dbm1(3074)
dbm2(2817)=dbm1(3075)
dbm2(2818)=dbm1(3076)
dbm2(2819)=dbm1(3077)
dbm2(2820)=dbm1(3078)
dbm2(2821)=dbm1(3079)
dbm2(2822)=dbm1(3080)
dbm2(2823)=dbm1(3081)
dbm2(2824)=dbm1(3082)
dbm2(2825)=dbm1(3083)
dbm2(2826)=dbm1(3084)
dbm2(2827)=dbm1(3085)
dbm2(2828)=dbm1(3086)
dbm2(2829)=dbm1(3087)
dbm2(2830)=dbm1(3088)
dbm2(2831)=dbm1(3089)
dbm2(2832)=dbm1(3090)
dbm2(2833)=dbm1(3091)
dbm2(2834)=dbm1(3092)
dbm2(2835)=dbm1(3093)
dbm2(2836)=dbm1(3094)
dbm2(2837)=dbm1(3095)
dbm2(2838)=dbm1(3096)
dbm2(2839)=dbm1(3097)
dbm2(2840)=dbm1(3098)
dbm2(2841)=dbm1(3099)
dbm2(2842)=dbm1(3100)
dbm2(2843)=dbm1(3101)
dbm2(2844)=dbm1(3102)
dbm2(2845)=dbm1(3103)
dbm2(2846)=dbm1(3104)
dbm2(2847)=dbm1(3105)
dbm2(2848)=dbm1(3106)
dbm2(2849)=dbm1(3107)
dbm2(2850)=dbm1(3108)
dbm2(2851)=dbm1(3109)
dbm2(2852)=dbm1(3110)
dbm2(2853)=dbm1(3111)
dbm2(2854)=dbm1(3112)
dbm2(2855)=dbm1(3113)
dbm2(2856)=dbm1(3114)
dbm2(2857)=dbm1(3115)
dbm2(2858)=dbm1(3116)
dbm2(2859)=dbm1(3117)
dbm2(2860)=dbm1(3118)
dbm2(2861)=dbm1(3119)
dbm2(2862)=dbm1(3120)
dbm2(2863)=dbm1(3121)
dbm2(2864)=dbm1(3122)
dbm2(2865)=dbm1(3123)
dbm2(2866)=dbm1(3124)
dbm2(2867)=dbm1(3125)
dbm2(2868)=dbm1(3126)
dbm2(2869)=dbm1(3127)
dbm2(2870)=dbm1(3128)
dbm2(2871)=dbm1(3129)
dbm2(2872)=dbm1(3130)
dbm2(2873)=dbm1(3131)
dbm2(2874)=dbm1(3132)
dbm2(2875)=dbm1(3133)
dbm2(2876)=dbm1(3134)
dbm2(2877)=dbm1(3135)
dbm2(2878)=dbm1(3136)
dbm2(2879)=dbm1(3137)
dbm2(2880)=dbm1(3138)
dbm2(2881)=dbm1(3139)
dbm2(2882)=dbm1(3140)
dbm2(2883)=dbm1(3141)
dbm2(2884)=dbm1(3142)
dbm2(2885)=dbm1(3143)
dbm2(2886)=dbm1(3144)
dbm2(2887)=dbm1(3145)
dbm2(2888)=dbm1(3146)
dbm2(2889)=dbm1(3147)
dbm2(2890)=dbm1(3148)
dbm2(2891)=dbm1(3149)
dbm2(2892)=dbm1(3150)
dbm2(2893)=dbm1(3151)
dbm2(2894)=dbm1(3152)
dbm2(2895)=dbm1(3153)
dbm2(2896)=dbm1(3154)
dbm2(2897)=dbm1(3155)
dbm2(2898)=dbm1(3156)
dbm2(2899)=dbm1(3157)
dbm2(2900)=dbm1(3158)
dbm2(2901)=dbm1(3159)
dbm2(2902)=dbm1(3160)
dbm2(2903)=dbm1(3161)
dbm2(2904)=dbm1(3162)
dbm2(2905)=dbm1(3163)
dbm2(2906)=dbm1(3164)
dbm2(2907)=dbm1(3165)
dbm2(2908)=dbm1(3166)
dbm2(2909)=dbm1(3167)
dbm2(2910)=dbm1(3168)
dbm2(2911)=dbm1(3169)
dbm2(2912)=dbm1(3170)
dbm2(2913)=dbm1(3171)
dbm2(2914)=dbm1(3172)
dbm2(2915)=dbm1(3173)
dbm2(2916)=dbm1(3174)
dbm2(2917)=dbm1(3175)
dbm2(2918)=dbm1(3176)
dbm2(2919)=dbm1(3177)
dbm2(2920)=dbm1(3178)
dbm2(2921)=dbm1(3179)
dbm2(2922)=dbm1(3180)
dbm2(2923)=dbm1(3181)
dbm2(2924)=dbm1(3182)
dbm2(2925)=dbm1(3183)
dbm2(2926)=dbm1(3184)
dbm2(2927)=dbm1(3185)
dbm2(2928)=dbm1(3186)
dbm2(2929)=dbm1(3187)
dbm2(2930)=dbm1(3188)
dbm2(2931)=dbm1(3189)
dbm2(2932)=dbm1(3190)
dbm2(2933)=dbm1(3191)
dbm2(2934)=dbm1(3193)
dbm2(2935)=dbm1(3194)
dbm2(2936)=dbm1(3195)
dbm2(2937)=dbm1(3196)
dbm2(2938)=dbm1(3197)
dbm2(2939)=dbm1(3198)
dbm2(2940)=dbm1(3199)
dbm2(2941)=dbm1(3200)
dbm2(2942)=dbm1(3201)
dbm2(2943)=dbm1(3202)
dbm2(2944)=dbm1(3203)
dbm2(2945)=dbm1(3204)
dbm2(2946)=dbm1(3205)
dbm2(2947)=dbm1(3206)
dbm2(2948)=dbm1(3207)
dbm2(2949)=dbm1(3208)
dbm2(2950)=dbm1(3210)
dbm2(2951)=dbm1(3212)
dbm2(2952)=dbm1(3213)
dbm2(2953)=dbm1(3214)
dbm2(2954)=dbm1(3215)
dbm2(2955)=dbm1(3216)
dbm2(2956)=dbm1(3217)
dbm2(2957)=dbm1(3218)
dbm2(2958)=dbm1(3219)
dbm2(2959)=dbm1(3220)
dbm2(2960)=dbm1(3221)
dbm2(2961)=dbm1(3222)
dbm2(2962)=dbm1(3223)
dbm2(2963)=dbm1(3224)
dbm2(2964)=dbm1(3225)
dbm2(2965)=dbm1(3227)
dbm2(2966)=dbm1(3228)
dbm2(2967)=dbm1(3230)
dbm2(2968)=dbm1(3232)
dbm2(2969)=dbm1(3233)
dbm2(2970)=dbm1(3234)
dbm2(2971)=dbm1(3235)
dbm2(2972)=dbm1(3236)
dbm2(2973)=dbm1(3237)
dbm2(2974)=dbm1(3238)
dbm2(2975)=dbm1(3239)
dbm2(2976)=dbm1(3240)
dbm2(2977)=dbm1(3241)
dbm2(2978)=dbm1(3242)
dbm2(2979)=dbm1(3243)
dbm2(2980)=dbm1(3244)
dbm2(2981)=dbm1(3246)
dbm2(2982)=dbm1(3247)
dbm2(2983)=dbm1(3249)
dbm2(2984)=dbm1(3250)
dbm2(2985)=dbm1(3252)
dbm2(2986)=dbm1(3254)
dbm2(2987)=dbm1(3255)
dbm2(2988)=dbm1(3256)
dbm2(2989)=dbm1(3257)
dbm2(2990)=dbm1(3258)
dbm2(2991)=dbm1(3259)
dbm2(2992)=dbm1(3260)
dbm2(2993)=dbm1(3261)
dbm2(2994)=dbm1(3262)
dbm2(2995)=dbm1(3263)
dbm2(2996)=dbm1(3264)
dbm2(2997)=dbm1(3265)
dbm2(2998)=dbm1(3267)
dbm2(2999)=dbm1(3268)
dbm2(3000)=dbm1(3270)
dbm2(3001)=dbm1(3271)
dbm2(3002)=dbm1(3273)
dbm2(3003)=dbm1(3274)
dbm2(3004)=dbm1(3276)
dbm2(3005)=dbm1(3277)
dbm2(3006)=dbm1(3279)
dbm2(3007)=dbm1(3280)
dbm2(3008)=dbm1(3281)
dbm2(3009)=dbm1(3282)
dbm2(3010)=dbm1(3283)
dbm2(3011)=dbm1(3284)
dbm2(3012)=dbm1(3285)
dbm2(3013)=dbm1(3286)
dbm2(3014)=dbm1(3287)
dbm2(3015)=dbm1(3288)
dbm2(3016)=dbm1(3289)
dbm2(3017)=dbm1(3291)
dbm2(3018)=dbm1(3292)
dbm2(3019)=dbm1(3294)
dbm2(3020)=dbm1(3295)
dbm2(3021)=dbm1(3297)
dbm2(3022)=dbm1(3298)
dbm2(3023)=dbm1(3299)
dbm2(3024)=dbm1(3301)
dbm2(3025)=dbm1(3302)
dbm2(3026)=dbm1(3303)
dbm2(3027)=dbm1(3305)
dbm2(3028)=dbm1(3306)
dbm2(3029)=dbm1(3307)
dbm2(3030)=dbm1(3308)
dbm2(3031)=dbm1(3309)
dbm2(3032)=dbm1(3310)
dbm2(3033)=dbm1(3311)
dbm2(3034)=dbm1(3312)
dbm2(3035)=dbm1(3313)
dbm2(3036)=dbm1(3315)
dbm2(3037)=dbm1(3316)
dbm2(3038)=dbm1(3318)
dbm2(3039)=dbm1(3319)
dbm2(3040)=dbm1(3321)
dbm2(3041)=dbm1(3322)
dbm2(3042)=dbm1(3323)
dbm2(3043)=dbm1(3325)
dbm2(3044)=dbm1(3326)
dbm2(3045)=dbm1(3327)
dbm2(3046)=dbm1(3328)
dbm2(3047)=dbm1(3329)
dbm2(3048)=dbm1(3330)
dbm2(3049)=dbm1(3331)
dbm2(3050)=dbm1(3332)
dbm2(3051)=dbm1(3334)
dbm2(3052)=dbm1(3335)
dbm2(3053)=dbm1(3337)
dbm2(3054)=dbm1(3338)
dbm2(3055)=dbm1(3339)
dbm2(3056)=dbm1(3341)
dbm2(3057)=dbm1(3342)
dbm2(3058)=dbm1(3343)
dbm2(3059)=dbm1(3344)
dbm2(3060)=dbm1(3345)
dbm2(3061)=dbm1(3346)
dbm2(3062)=dbm1(3348)
dbm2(3063)=dbm1(3349)
dbm2(3064)=dbm1(3351)
dbm2(3065)=dbm1(3352)
dbm2(3066)=dbm1(3353)
dbm2(3067)=dbm1(3354)
dbm2(3068)=dbm1(3355)
dbm2(3069)=dbm1(3357)
dbm2(3070)=dbm1(3358)
dbm2(3071)=dbm1(3359)
dbm2(3072)=dbm1(3360)
dbm2(3073)=dbm1(3361)
dbm2(3074)=dbm1(3362)
dbm2(3075)=dbm1(3363)

dVdR(i)=dVdR(i) + dot_product(coeff,dbm2)

end do

end subroutine EvU13O10dVdR

!===========================================================================
! *def u23_dvdx(xyz,u23,s23,du23dx)
!  main subtoutine of evaluating derivative of u23 w.r.t. the xyz coordinates
!  xyz is the coordinate
!  u23 is u23
!  s23 is u23 devided by the hyperbolic function
!  du23dx is the gradient of u23
!===========================================================================
subroutine u23_dvdx(xyz,u23,s23,du23dx)
implicit none
integer, parameter:: dp=kind(0.d0)   
integer :: i,j,k
real(dp) :: du23dx(4,3),xyz(4,3)
real(dp) :: u23,s23
real(dp) :: coord(12), r(6)
real(dp) :: dudx(12)
real(dp) :: dvdx(12), dvdr(6), drdx(6,12)
real(dp) :: dtdx(12)
real(dp) :: volume

! Initialize dVdX(12)
      do i=1,12
        dVdX(i)=0.0d0
      enddo

! Initialize dRdX(6,12)
      do i=1,6
        do j=1,12
          dRdX(i,j)=0.0d0
        enddo
      enddo

! Initialize dVdR(6)
      do i=1,6
        dVdR(i)=0.0d0
      enddo

! Assign xyz to coord
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        coord(k)=xyz(i,j)
      enddo
      enddo

! Compute distances r
r(1)=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r(2)=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r(3)=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r(4)=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r(5)=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r(6)=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

! Call EvdVdR to evaluate dVdR(6)
      Call EvU23O10dVdR(coord,r,dvdr)

! Call EvdRdX to evaluate dRdX(6,12)
      Call EvdRdX(coord,r,drdx)

! Calculate dVdX by using chain rule: dV/dXi=(dV/dRj)*(dRj/dXi), j=1 to 6
      do i=1,12
        do j=1,6
          dVdX(i)=dVdX(i) + dVdR(j)*dRdX(j,i)
        enddo
      enddo

      do i=1,12
          dVdX(i)=dVdX(i)/627.509474
      enddo

  call P_volume(xyz,volume)

! Now compute numerical dtdx
  call Num_dtdx(xyz,dtdx)

! Chain rule again, because U=tanh*V
! dUdX=dtanhdX*V+tanh*dVdX
do i=1,12
  dudx(i)=0
enddo

do i=1,12
  dudx(i)=dtdx(i)*s23+volume*dvdx(i)
enddo


! Assign dvdx back to du23dvdx
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        du23dx(i,j)=dudx(k)
      enddo
      enddo

      return

end subroutine u23_dvdx 

!===========================================================================
! *def EvU23O10dVdR(coord,r,dvdr)
!  subtoutine of evaluating derivative of u23 w.r.t. the 6 nuclear distances
!  coord is the xyz coordinate 
!  r is the nuclear distances
!  dvdr is the derivative of u23 w.r.t. the 6 nuclear distances
!===========================================================================
subroutine EvU23O10dVdR(coord,r,dvdr)
integer, parameter:: ddp=kind(0.d0)
integer :: i,j
real(ddp) :: coord(12),r(6),dvdr(6)
real(ddp) :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=1300
real(ddp) :: coeff(ncoeff)
real(ddp),dimension(0:32) :: rm
real(ddp),dimension(0:32) :: dm
real(ddp),dimension(0:1506) :: p
real(ddp),dimension(0:1506) :: dp
real(ddp) :: x(6),dmsdr(6,6)
real(ddp) :: dbm1(1506), dbm2(1300)

! Assign coefficients first
a1=1.323343
a2=1.09
b1=1.823343
b2=1.633190
ra1=1.113545
ra2=1.001178
rb1=1.113545
rb2=1.001178
coeff(1)=-0.367835424500D+03
coeff(2)=-0.345009613875D+02
coeff(3)=-0.462704697963D+02
coeff(4)=-0.257814027132D+03
coeff(5)=0.946252414190D+03
coeff(6)=-0.575616726211D+03
coeff(7)=0.107541261550D+04
coeff(8)=-0.342927934077D+04
coeff(9)=-0.116299721266D+04
coeff(10)=0.211962361903D+04
coeff(11)=0.153034426652D+04
coeff(12)=0.103589153519D+04
coeff(13)=0.828106949415D+03
coeff(14)=0.419251628265D+03
coeff(15)=-0.836537222863D+03
coeff(16)=-0.431897961889D+04
coeff(17)=-0.140540899389D+02
coeff(18)=0.449100064823D+04
coeff(19)=0.278519709531D+04
coeff(20)=-0.658597887101D+04
coeff(21)=0.699050520516D+03
coeff(22)=-0.164807813975D+04
coeff(23)=-0.140738486070D+04
coeff(24)=-0.346478740365D+04
coeff(25)=0.235735603792D+04
coeff(26)=0.411965512626D+04
coeff(27)=0.820611235955D+03
coeff(28)=-0.385908245676D+04
coeff(29)=-0.473016769361D+04
coeff(30)=0.381219312953D+04
coeff(31)=0.433328754209D+04
coeff(32)=0.372882884263D+04
coeff(33)=0.261013202025D+04
coeff(34)=0.393899344143D+04
coeff(35)=-0.113569983057D+05
coeff(36)=-0.282463707363D+04
coeff(37)=-0.160715147111D+04
coeff(38)=0.286536038561D+04
coeff(39)=-0.404423909654D+04
coeff(40)=-0.649715515940D+04
coeff(41)=0.939321624777D+04
coeff(42)=0.502323356360D+04
coeff(43)=-0.128203546195D+04
coeff(44)=-0.376099802131D+03
coeff(45)=0.474984074250D+04
coeff(46)=0.440350761738D+04
coeff(47)=-0.468418243432D+04
coeff(48)=-0.341079999403D+04
coeff(49)=0.238471554165D+04
coeff(50)=0.752383966347D+04
coeff(51)=0.192045488199D+04
coeff(52)=0.108311816199D+04
coeff(53)=-0.143904374334D+04
coeff(54)=0.121553301268D+04
coeff(55)=-0.107680056311D+05
coeff(56)=0.125593839622D+05
coeff(57)=-0.251996129077D+04
coeff(58)=-0.183904970884D+04
coeff(59)=0.238388495782D+04
coeff(60)=-0.738807873666D+03
coeff(61)=-0.516012112821D+04
coeff(62)=0.462101285280D+04
coeff(63)=-0.321450777912D+04
coeff(64)=0.173968689564D+04
coeff(65)=-0.337210232279D+04
coeff(66)=0.877094977033D+04
coeff(67)=-0.554325748354D+04
coeff(68)=0.161724600127D+04
coeff(69)=-0.580968391425D+04
coeff(70)=0.665541231310D+04
coeff(71)=0.108263227956D+05
coeff(72)=0.454197796856D+04
coeff(73)=0.679734620048D+04
coeff(74)=-0.218868002192D+04
coeff(75)=-0.454009571933D+04
coeff(76)=-0.544550570280D+04
coeff(77)=0.222219940610D+04
coeff(78)=-0.393839910795D+04
coeff(79)=-0.845854847286D+04
coeff(80)=0.226751123643D+04
coeff(81)=-0.162675144178D+04
coeff(82)=-0.202476041765D+04
coeff(83)=0.664614290680D+04
coeff(84)=0.266540417775D+04
coeff(85)=0.275858273058D+04
coeff(86)=-0.916330727797D+03
coeff(87)=-0.104846769522D+04
coeff(88)=0.412981963827D+04
coeff(89)=-0.297139851012D+04
coeff(90)=0.748726341796D+04
coeff(91)=0.579021241779D+03
coeff(92)=0.782645432997D+04
coeff(93)=-0.109465976363D+05
coeff(94)=0.928671941038D+04
coeff(95)=-0.466667596663D+04
coeff(96)=0.133132901304D+04
coeff(97)=0.881666806763D+02
coeff(98)=-0.224008647080D+04
coeff(99)=-0.399205407155D+02
coeff(100)=0.667502239116D+04
coeff(101)=-0.542295731393D+04
coeff(102)=-0.118221709938D+05
coeff(103)=-0.890289643931D+03
coeff(104)=-0.327451117343D+02
coeff(105)=-0.109995874857D+04
coeff(106)=0.107986170238D+04
coeff(107)=0.100324745229D+04
coeff(108)=-0.138647513710D+04
coeff(109)=0.329635913623D+04
coeff(110)=0.670671118252D+04
coeff(111)=-0.551561957094D+02
coeff(112)=-0.282348630248D+04
coeff(113)=-0.130763286775D+04
coeff(114)=-0.647714742330D+01
coeff(115)=-0.141965234378D+05
coeff(116)=-0.701759518863D+04
coeff(117)=-0.195777320933D+04
coeff(118)=0.958385393117D+03
coeff(119)=-0.264326312752D+04
coeff(120)=-0.950233173026D+04
coeff(121)=0.705855727994D+04
coeff(122)=-0.623652655890D+04
coeff(123)=0.135484091136D+04
coeff(124)=0.744886705077D+03
coeff(125)=-0.118100284881D+05
coeff(126)=0.161166991979D+05
coeff(127)=-0.773330616301D+04
coeff(128)=0.589700169323D+04
coeff(129)=0.657938208285D+04
coeff(130)=0.518063867482D+03
coeff(131)=-0.781054689301D+03
coeff(132)=0.312678743874D+04
coeff(133)=0.124006202857D+04
coeff(134)=-0.638036122473D+04
coeff(135)=0.472961572395D+04
coeff(136)=-0.421710158595D+03
coeff(137)=0.105528682416D+05
coeff(138)=0.246269767643D+04
coeff(139)=-0.756736764690D+04
coeff(140)=0.273504342772D+04
coeff(141)=0.496370553796D+04
coeff(142)=-0.462502519609D+04
coeff(143)=0.283032404220D+04
coeff(144)=0.125532910623D+04
coeff(145)=-0.360581880056D+04
coeff(146)=-0.947072598409D+04
coeff(147)=-0.541756927948D+04
coeff(148)=0.711898208422D+04
coeff(149)=-0.232696841455D+04
coeff(150)=0.836533032633D+04
coeff(151)=0.111967998615D+02
coeff(152)=0.496805631746D+04
coeff(153)=-0.636048476102D+04
coeff(154)=0.172055547944D+04
coeff(155)=-0.105714625705D+04
coeff(156)=0.671542301917D+03
coeff(157)=-0.330882405524D+04
coeff(158)=-0.166076720293D+04
coeff(159)=0.628670636052D+04
coeff(160)=0.265934908583D+04
coeff(161)=0.482177566657D+03
coeff(162)=-0.274839328001D+04
coeff(163)=-0.834748817821D+03
coeff(164)=0.486398135384D+04
coeff(165)=-0.608697288597D+04
coeff(166)=0.758405515895D+04
coeff(167)=-0.198625652855D+03
coeff(168)=-0.242723305190D+04
coeff(169)=-0.432100503043D+04
coeff(170)=-0.293232197620D+03
coeff(171)=-0.254320839597D+04
coeff(172)=-0.330411138137D+04
coeff(173)=-0.361724748470D+04
coeff(174)=-0.210659871609D+04
coeff(175)=0.546680635466D+04
coeff(176)=0.165147545386D+05
coeff(177)=0.127390508919D+04
coeff(178)=-0.821804096790D+04
coeff(179)=0.114930447041D+04
coeff(180)=0.687544315269D+04
coeff(181)=0.515778555850D+04
coeff(182)=-0.147965639565D+04
coeff(183)=0.540850040950D+04
coeff(184)=-0.445180889629D+04
coeff(185)=0.114204652896D+05
coeff(186)=-0.198814661644D+04
coeff(187)=0.212638783175D+04
coeff(188)=-0.124833465296D+03
coeff(189)=-0.370467653948D+04
coeff(190)=0.132835848653D+05
coeff(191)=0.862919856597D+04
coeff(192)=-0.209573836340D+05
coeff(193)=-0.115366604924D+05
coeff(194)=0.789948817648D+04
coeff(195)=-0.119538246304D+05
coeff(196)=-0.633754337685D+04
coeff(197)=-0.103650234002D+04
coeff(198)=-0.113243816680D+05
coeff(199)=-0.581364132495D+04
coeff(200)=0.811017192348D+04
coeff(201)=-0.283817069037D+04
coeff(202)=0.118476164078D+04
coeff(203)=-0.429015815782D+04
coeff(204)=0.224479331357D+04
coeff(205)=0.829405231639D+04
coeff(206)=0.544538500283D+04
coeff(207)=-0.602321263279D+04
coeff(208)=-0.327825276556D+03
coeff(209)=0.619062316534D+04
coeff(210)=-0.110928274616D+04
coeff(211)=0.381511970295D+04
coeff(212)=-0.373447290333D+04
coeff(213)=0.136684856964D+04
coeff(214)=0.511821653262D+04
coeff(215)=0.196396104857D+04
coeff(216)=0.175029099358D+04
coeff(217)=-0.680500538919D+03
coeff(218)=0.119240062097D+04
coeff(219)=0.370284816179D+03
coeff(220)=-0.344105978136D+04
coeff(221)=-0.129537270071D+04
coeff(222)=-0.514585232221D+04
coeff(223)=-0.119763438152D+04
coeff(224)=0.541369500945D+03
coeff(225)=-0.605262300083D+04
coeff(226)=-0.517393918206D+04
coeff(227)=-0.418279136769D+04
coeff(228)=-0.957397967350D+04
coeff(229)=-0.147217548360D+04
coeff(230)=0.514400991805D+04
coeff(231)=0.183286032713D+04
coeff(232)=-0.480363257132D+04
coeff(233)=0.691814088532D+04
coeff(234)=0.735656185985D+04
coeff(235)=0.819633023420D+04
coeff(236)=0.116081368060D+04
coeff(237)=-0.248855218267D+04
coeff(238)=-0.293599100953D+04
coeff(239)=-0.468310318496D+04
coeff(240)=-0.479079059019D+04
coeff(241)=-0.451225353179D+04
coeff(242)=-0.427150911597D+04
coeff(243)=0.685535798577D+04
coeff(244)=0.329917085958D+04
coeff(245)=0.493208722835D+04
coeff(246)=0.970475894071D+04
coeff(247)=0.259670944404D+04
coeff(248)=-0.524725514137D+04
coeff(249)=0.745834398105D+04
coeff(250)=0.874175129647D+04
coeff(251)=0.382426709208D+04
coeff(252)=-0.153137318074D+05
coeff(253)=-0.820907617520D+03
coeff(254)=-0.148148997881D+05
coeff(255)=-0.241152271829D+03
coeff(256)=0.222434529291D+04
coeff(257)=-0.156676187969D+04
coeff(258)=0.104900284587D+04
coeff(259)=0.698199779897D+04
coeff(260)=0.833356477690D+04
coeff(261)=0.596002972241D+04
coeff(262)=0.238128092367D+03
coeff(263)=-0.554530813740D+04
coeff(264)=-0.101384830110D+05
coeff(265)=0.208990929710D+04
coeff(266)=0.823852728551D+04
coeff(267)=0.923174784554D+04
coeff(268)=-0.184892841970D+04
coeff(269)=-0.960772441060D+03
coeff(270)=0.488615859707D+04
coeff(271)=-0.821390053318D+04
coeff(272)=0.574111416236D+03
coeff(273)=-0.538818853967D+04
coeff(274)=0.313439609455D+04
coeff(275)=0.449145965581D+04
coeff(276)=-0.109110539789D+05
coeff(277)=0.574261189575D+04
coeff(278)=0.133781890686D+04
coeff(279)=-0.136209055325D+05
coeff(280)=0.663939514979D+03
coeff(281)=0.287741720427D+04
coeff(282)=0.101688960572D+05
coeff(283)=-0.245413008752D+04
coeff(284)=-0.165452745584D+05
coeff(285)=-0.130515874728D+05
coeff(286)=0.619885116893D+03
coeff(287)=0.166661326240D+05
coeff(288)=-0.344461980992D+04
coeff(289)=0.102382964426D+04
coeff(290)=0.758112279170D+03
coeff(291)=-0.364243631154D+03
coeff(292)=0.361064519426D+04
coeff(293)=0.178616186551D+04
coeff(294)=0.216835505324D+04
coeff(295)=-0.516785165085D+03
coeff(296)=-0.671538332502D+04
coeff(297)=-0.848837848950D+04
coeff(298)=-0.158205059491D+05
coeff(299)=0.600652260191D+04
coeff(300)=-0.394262345527D+04
coeff(301)=-0.909525607161D+04
coeff(302)=0.575601276321D+04
coeff(303)=-0.105544231512D+04
coeff(304)=0.158632720587D+04
coeff(305)=0.563758494400D+04
coeff(306)=-0.492046521687D+04
coeff(307)=0.569056906056D+04
coeff(308)=-0.558058858412D+03
coeff(309)=-0.425493940543D+04
coeff(310)=-0.554434391982D+04
coeff(311)=-0.420562435377D+04
coeff(312)=-0.829196907842D+04
coeff(313)=-0.572137267381D+04
coeff(314)=-0.542389050975D+04
coeff(315)=0.837947499692D+03
coeff(316)=-0.816688275621D+04
coeff(317)=0.449166904450D+04
coeff(318)=-0.475521259861D+04
coeff(319)=-0.145842099714D+04
coeff(320)=-0.280639093768D+04
coeff(321)=0.611797468167D+04
coeff(322)=0.710963682096D+04
coeff(323)=-0.710028178902D+03
coeff(324)=0.627003601409D+03
coeff(325)=-0.276771841393D+04
coeff(326)=0.111113056094D+05
coeff(327)=0.125748167898D+04
coeff(328)=0.812477534759D+04
coeff(329)=-0.278821875158D+04
coeff(330)=0.128238137520D+02
coeff(331)=0.695569776319D+04
coeff(332)=-0.903654083864D+04
coeff(333)=-0.234316450571D+04
coeff(334)=-0.304758806709D+04
coeff(335)=0.672220753490D+04
coeff(336)=-0.706504758965D+04
coeff(337)=0.933877044741D+03
coeff(338)=0.637958521306D+04
coeff(339)=-0.264019578911D+04
coeff(340)=-0.127483329516D+04
coeff(341)=-0.332416724586D+04
coeff(342)=0.400445981066D+04
coeff(343)=0.311945252255D+04
coeff(344)=0.185538870080D+04
coeff(345)=0.474663810152D+04
coeff(346)=-0.820906711510D+03
coeff(347)=-0.872526964795D+04
coeff(348)=-0.684440987309D+04
coeff(349)=-0.125666574203D+05
coeff(350)=-0.903019367902D+03
coeff(351)=0.168013636752D+04
coeff(352)=0.375116831426D+04
coeff(353)=-0.470572854324D+04
coeff(354)=-0.211079073771D+04
coeff(355)=-0.130331398223D+05
coeff(356)=-0.137031186316D+04
coeff(357)=-0.888945974074D+03
coeff(358)=-0.569649851052D+04
coeff(359)=-0.293749508683D+04
coeff(360)=-0.844962712287D+04
coeff(361)=-0.179016889516D+05
coeff(362)=0.657108338380D+04
coeff(363)=0.294129092958D+04
coeff(364)=0.116517106569D+05
coeff(365)=0.269049406131D+04
coeff(366)=-0.159199737050D+05
coeff(367)=0.814964679961D+04
coeff(368)=0.101226260412D+04
coeff(369)=0.163640872706D+04
coeff(370)=-0.282683677188D+04
coeff(371)=0.634808529872D+04
coeff(372)=0.191083430734D+05
coeff(373)=-0.140543560454D+05
coeff(374)=-0.544054690430D+04
coeff(375)=0.578952585067D+04
coeff(376)=0.265417258672D+04
coeff(377)=0.281039764703D+04
coeff(378)=0.653880486083D+04
coeff(379)=0.985511700271D+02
coeff(380)=0.492157315788D+04
coeff(381)=-0.591533993286D+04
coeff(382)=0.395872365552D+04
coeff(383)=-0.128063916003D+05
coeff(384)=0.657223409307D+04
coeff(385)=0.747524974438D+04
coeff(386)=0.453505681501D+04
coeff(387)=0.111885724721D+05
coeff(388)=0.114156662380D+05
coeff(389)=0.779309558600D+04
coeff(390)=0.937273860471D+04
coeff(391)=-0.413375916246D+04
coeff(392)=0.397122713575D+03
coeff(393)=-0.222709331754D+04
coeff(394)=-0.777378798393D+04
coeff(395)=-0.930902728262D+04
coeff(396)=0.621625884086D+04
coeff(397)=-0.385260410311D+04
coeff(398)=0.152499117066D+04
coeff(399)=0.378683038537D+04
coeff(400)=0.142163336058D+03
coeff(401)=0.134184880805D+04
coeff(402)=-0.181216570851D+04
coeff(403)=0.290896365230D+03
coeff(404)=0.349297061233D+04
coeff(405)=0.391978656658D+04
coeff(406)=-0.745562412047D+04
coeff(407)=0.869356301685D+04
coeff(408)=0.725261491508D+04
coeff(409)=0.323196510043D+04
coeff(410)=0.202161376258D+04
coeff(411)=-0.441281603719D+04
coeff(412)=0.266887406498D+03
coeff(413)=0.525075007362D+04
coeff(414)=0.635584664345D+03
coeff(415)=-0.454321323793D+04
coeff(416)=0.173163910771D+04
coeff(417)=-0.136732328301D+04
coeff(418)=0.680856479237D+04
coeff(419)=0.150469616240D+04
coeff(420)=-0.734626291745D+04
coeff(421)=0.633626063676D+02
coeff(422)=0.123080593503D+05
coeff(423)=0.408449266416D+04
coeff(424)=0.130458468293D+05
coeff(425)=0.497257155672D+04
coeff(426)=0.736869360840D+04
coeff(427)=0.819683587443D+04
coeff(428)=0.168981906427D+03
coeff(429)=-0.482134204815D+03
coeff(430)=-0.816929457230D+03
coeff(431)=-0.263058064907D+04
coeff(432)=-0.191969760870D+04
coeff(433)=-0.234000057365D+04
coeff(434)=0.363326843531D+04
coeff(435)=0.409517884491D+04
coeff(436)=0.605630597188D+04
coeff(437)=0.133432212271D+05
coeff(438)=-0.627021853583D+04
coeff(439)=0.451199752065D+04
coeff(440)=0.712859545407D+03
coeff(441)=-0.847755181538D+04
coeff(442)=-0.370698880273D+04
coeff(443)=-0.134172023007D+05
coeff(444)=0.920861843822D+03
coeff(445)=-0.103206698690D+05
coeff(446)=0.296530307164D+04
coeff(447)=0.352097991202D+04
coeff(448)=0.444742736105D+04
coeff(449)=0.621001219513D+04
coeff(450)=-0.688169934970D+04
coeff(451)=0.804008375333D+04
coeff(452)=-0.812444607721D+04
coeff(453)=-0.275636174869D+04
coeff(454)=0.851818922709D+03
coeff(455)=-0.231453210240D+04
coeff(456)=-0.156198576697D+05
coeff(457)=0.935357790037D+04
coeff(458)=0.657355424533D+02
coeff(459)=-0.123934508791D+05
coeff(460)=0.673560382218D+04
coeff(461)=0.925741028087D+04
coeff(462)=0.182802028879D+04
coeff(463)=0.133378484726D+05
coeff(464)=-0.111192675107D+05
coeff(465)=-0.264188981328D+04
coeff(466)=-0.995853980731D+03
coeff(467)=-0.417452903272D+02
coeff(468)=0.475770751289D+04
coeff(469)=-0.850577659984D+04
coeff(470)=-0.310113378973D+04
coeff(471)=0.243007093487D+04
coeff(472)=0.431432276029D+04
coeff(473)=0.323183339715D+04
coeff(474)=-0.654850179019D+04
coeff(475)=0.634615831637D+04
coeff(476)=-0.901231753930D+03
coeff(477)=-0.133286182037D+04
coeff(478)=-0.127429960990D+05
coeff(479)=-0.126798418101D+05
coeff(480)=-0.116960336330D+04
coeff(481)=-0.903232745740D+03
coeff(482)=-0.297478095451D+04
coeff(483)=0.541817039536D+04
coeff(484)=0.110834152036D+04
coeff(485)=0.416521520534D+04
coeff(486)=-0.477962130508D+04
coeff(487)=-0.308720868256D+04
coeff(488)=-0.801942203929D+04
coeff(489)=-0.250355643852D+04
coeff(490)=-0.105574135264D+04
coeff(491)=0.978731469572D+04
coeff(492)=-0.457555565805D+03
coeff(493)=0.200014871749D+05
coeff(494)=-0.759689512963D+04
coeff(495)=-0.235893307581D+04
coeff(496)=0.144213920814D+03
coeff(497)=0.213972388428D+04
coeff(498)=0.849710786793D+04
coeff(499)=0.658314276450D+04
coeff(500)=-0.213100159235D+04
coeff(501)=-0.147897313249D+04
coeff(502)=-0.171573447271D+04
coeff(503)=0.156830868977D+04
coeff(504)=-0.224884814468D+04
coeff(505)=0.536070951672D+04
coeff(506)=-0.332821028082D+04
coeff(507)=-0.289530756480D+04
coeff(508)=-0.162320739564D+05
coeff(509)=-0.176672227420D+05
coeff(510)=-0.493190523032D+04
coeff(511)=0.137990168448D+04
coeff(512)=0.640079240320D+04
coeff(513)=0.263014402317D+04
coeff(514)=-0.127314627629D+05
coeff(515)=0.184381083911D+05
coeff(516)=-0.301459374592D+04
coeff(517)=-0.685810278012D+04
coeff(518)=0.599784691095D+04
coeff(519)=0.871177317929D+04
coeff(520)=-0.461015933184D+04
coeff(521)=0.521227667926D+04
coeff(522)=-0.269792856992D+04
coeff(523)=0.205685105673D+04
coeff(524)=-0.583817668847D+04
coeff(525)=0.947736034247D+04
coeff(526)=-0.296286719670D+04
coeff(527)=0.164460401455D+04
coeff(528)=0.104412676407D+05
coeff(529)=0.261680184936D+04
coeff(530)=-0.380463578039D+04
coeff(531)=-0.185178694868D+04
coeff(532)=0.507936347459D+04
coeff(533)=-0.917977526583D+04
coeff(534)=0.343450328971D+04
coeff(535)=0.697460434880D+04
coeff(536)=-0.156539336339D+04
coeff(537)=0.699619423890D+04
coeff(538)=-0.129506195421D+05
coeff(539)=0.398588730481D+04
coeff(540)=-0.509733403894D+04
coeff(541)=-0.298378014765D+03
coeff(542)=0.234748053703D+05
coeff(543)=-0.316957449929D+03
coeff(544)=0.583000382799D+04
coeff(545)=-0.320985994134D+04
coeff(546)=0.674098846439D+04
coeff(547)=-0.108741604552D+05
coeff(548)=0.241159950016D+03
coeff(549)=-0.363858048101D+03
coeff(550)=-0.100071122815D+03
coeff(551)=0.485940323869D+04
coeff(552)=0.112875858652D+05
coeff(553)=0.271926177349D+04
coeff(554)=0.620749428298D+04
coeff(555)=0.357495604871D+04
coeff(556)=0.144167969557D+04
coeff(557)=0.586967733519D+04
coeff(558)=-0.657094384556D+04
coeff(559)=0.327179675620D+04
coeff(560)=0.610108249216D+04
coeff(561)=-0.878632716035D+04
coeff(562)=0.106488219799D+04
coeff(563)=-0.114492497862D+05
coeff(564)=0.524292961444D+04
coeff(565)=0.453341997630D+04
coeff(566)=0.111983245824D+04
coeff(567)=-0.443198995846D+04
coeff(568)=0.925464045683D+04
coeff(569)=0.627696468426D+04
coeff(570)=0.479579923375D+04
coeff(571)=0.463424193401D+04
coeff(572)=0.993508858888D+03
coeff(573)=-0.388113858691D+04
coeff(574)=0.789908571267D+04
coeff(575)=-0.725551022886D+04
coeff(576)=0.678834224896D+04
coeff(577)=0.383350107885D+04
coeff(578)=0.527525919252D+04
coeff(579)=-0.464631886677D+04
coeff(580)=0.115370085875D+04
coeff(581)=-0.502110385377D+04
coeff(582)=0.349332131632D+03
coeff(583)=0.469407874476D+03
coeff(584)=-0.352578249546D+04
coeff(585)=-0.492553639245D+04
coeff(586)=-0.179002723487D+04
coeff(587)=0.627323422434D+04
coeff(588)=-0.475505154165D+04
coeff(589)=-0.541938909213D+04
coeff(590)=0.437616307647D+04
coeff(591)=-0.141108971604D+05
coeff(592)=0.113144064254D+04
coeff(593)=-0.102623903869D+04
coeff(594)=0.581199263033D+04
coeff(595)=0.926932540912D+03
coeff(596)=0.167164073483D+05
coeff(597)=-0.167106483543D+05
coeff(598)=-0.364806741747D+04
coeff(599)=0.551533774565D+04
coeff(600)=-0.659483337233D+04
coeff(601)=-0.107187933879D+04
coeff(602)=-0.598697170366D+04
coeff(603)=-0.312240395865D+04
coeff(604)=0.297210625089D+04
coeff(605)=0.414108065286D+04
coeff(606)=0.648554246687D+04
coeff(607)=-0.121347167843D+05
coeff(608)=-0.425864049762D+04
coeff(609)=0.409596664449D+03
coeff(610)=-0.573976263020D+04
coeff(611)=0.863696053220D+03
coeff(612)=-0.848834024145D+04
coeff(613)=-0.625955837146D+04
coeff(614)=0.170520511275D+04
coeff(615)=-0.138836174181D+04
coeff(616)=0.766118995455D+04
coeff(617)=0.269000964290D+04
coeff(618)=0.143885833650D+04
coeff(619)=-0.124458132842D+04
coeff(620)=0.519932714339D+04
coeff(621)=-0.740490591296D+04
coeff(622)=0.949146758138D+04
coeff(623)=0.119870696397D+05
coeff(624)=0.787031467003D+04
coeff(625)=-0.667464870154D+04
coeff(626)=0.596048766627D+04
coeff(627)=0.709373189691D+04
coeff(628)=0.160501605672D+05
coeff(629)=0.660600527316D+04
coeff(630)=-0.146018997184D+04
coeff(631)=0.314768899710D+04
coeff(632)=0.128946203595D+05
coeff(633)=0.118381931472D+05
coeff(634)=0.135735409664D+05
coeff(635)=0.427621323191D+04
coeff(636)=-0.728940030723D+04
coeff(637)=-0.869197135014D+02
coeff(638)=-0.176931358601D+04
coeff(639)=0.931380052006D+02
coeff(640)=-0.106341863140D+05
coeff(641)=0.383442938583D+04
coeff(642)=0.716033212891D+04
coeff(643)=0.531524793798D+03
coeff(644)=0.775180073296D+04
coeff(645)=-0.185489269909D+04
coeff(646)=-0.806587057432D+04
coeff(647)=0.297695674882D+04
coeff(648)=0.522343989758D+04
coeff(649)=-0.206634568723D+03
coeff(650)=0.249600653311D+04
coeff(651)=0.792202665631D+04
coeff(652)=-0.459787669277D+03
coeff(653)=0.341985079166D+04
coeff(654)=-0.153887886976D+05
coeff(655)=0.676085270033D+04
coeff(656)=0.116115508819D+05
coeff(657)=-0.765133389221D+04
coeff(658)=-0.724026521902D+03
coeff(659)=0.215130525089D+04
coeff(660)=0.608061221627D+04
coeff(661)=-0.974494924016D+04
coeff(662)=-0.684931329002D+04
coeff(663)=0.795515782485D+04
coeff(664)=-0.103558078659D+05
coeff(665)=0.436235465165D+04
coeff(666)=0.360169189176D+04
coeff(667)=0.420572503350D+04
coeff(668)=-0.898313497748D+04
coeff(669)=-0.960432590416D+04
coeff(670)=-0.981281012985D+04
coeff(671)=-0.906281546956D+04
coeff(672)=-0.651279481427D+04
coeff(673)=-0.788149409944D+03
coeff(674)=0.961823671620D+04
coeff(675)=-0.701975029046D+04
coeff(676)=-0.156184513199D+05
coeff(677)=-0.971310148666D+04
coeff(678)=-0.120118451352D+05
coeff(679)=-0.874900865831D+04
coeff(680)=0.451369109109D+04
coeff(681)=0.898867912443D+04
coeff(682)=-0.443397881454D+04
coeff(683)=-0.109708401867D+05
coeff(684)=-0.323315966979D+04
coeff(685)=0.237403133566D+05
coeff(686)=-0.911933866624D+04
coeff(687)=0.509891715930D+04
coeff(688)=0.431657965422D+04
coeff(689)=0.619602815759D+03
coeff(690)=-0.100934403330D+05
coeff(691)=0.151162717496D+05
coeff(692)=-0.767383019975D+04
coeff(693)=0.678193747197D+04
coeff(694)=-0.381735571055D+04
coeff(695)=0.515970018592D+04
coeff(696)=0.109283411153D+05
coeff(697)=0.671175030491D+04
coeff(698)=-0.281469178974D+04
coeff(699)=-0.417381151905D+04
coeff(700)=-0.466716264247D+02
coeff(701)=-0.103875216613D+05
coeff(702)=0.524794123093D+03
coeff(703)=0.354103893798D+04
coeff(704)=0.259576231120D+04
coeff(705)=0.953426051756D+04
coeff(706)=-0.461331192982D+04
coeff(707)=-0.602230459504D+04
coeff(708)=-0.318147619661D+04
coeff(709)=-0.471430098404D+04
coeff(710)=-0.216930874740D+04
coeff(711)=0.231753869439D+03
coeff(712)=0.835911235804D+03
coeff(713)=0.287252311920D+04
coeff(714)=-0.191957787092D+04
coeff(715)=-0.113617250504D+04
coeff(716)=-0.626312003731D+04
coeff(717)=-0.115337988887D+04
coeff(718)=0.289476076067D+04
coeff(719)=-0.755789367041D+04
coeff(720)=-0.413975020837D+03
coeff(721)=0.178322700581D+05
coeff(722)=-0.682747128739D+04
coeff(723)=-0.157863058895D+05
coeff(724)=-0.175224916363D+04
coeff(725)=0.197058767975D+04
coeff(726)=-0.143865666655D+04
coeff(727)=-0.301295354062D+04
coeff(728)=-0.396853680315D+03
coeff(729)=-0.132726103880D+04
coeff(730)=0.153539265292D+05
coeff(731)=-0.848378575358D+04
coeff(732)=0.455176998948D+04
coeff(733)=-0.518345501711D+04
coeff(734)=-0.395934645745D+04
coeff(735)=0.174914128190D+05
coeff(736)=0.622387891248D+04
coeff(737)=-0.133847321500D+05
coeff(738)=0.211764634838D+04
coeff(739)=-0.143739288346D+04
coeff(740)=-0.125246608714D+05
coeff(741)=0.238030539303D+04
coeff(742)=-0.183226672167D+05
coeff(743)=-0.449974430893D+04
coeff(744)=0.446547333355D+04
coeff(745)=0.638607350495D+04
coeff(746)=-0.110467638159D+05
coeff(747)=-0.555642925150D+03
coeff(748)=-0.131023500648D+04
coeff(749)=-0.141994162192D+04
coeff(750)=-0.825152430118D+04
coeff(751)=-0.309413676548D+03
coeff(752)=0.132679303825D+05
coeff(753)=-0.121140269305D+05
coeff(754)=0.720521968580D+04
coeff(755)=-0.645173090181D+04
coeff(756)=0.889237099871D+04
coeff(757)=-0.249246090505D+03
coeff(758)=0.129689467045D+04
coeff(759)=-0.598146243060D+04
coeff(760)=0.599863012790D+04
coeff(761)=0.296683743234D+04
coeff(762)=0.247124814152D+04
coeff(763)=0.993289384528D+04
coeff(764)=-0.801617082545D+04
coeff(765)=0.338736983072D+04
coeff(766)=-0.514051619351D+04
coeff(767)=-0.526262880696D+04
coeff(768)=-0.473144024685D+04
coeff(769)=-0.678884392864D+04
coeff(770)=0.350831061286D+04
coeff(771)=-0.123007029797D+05
coeff(772)=0.352220832604D+04
coeff(773)=0.833033374770D+04
coeff(774)=-0.102370632487D+04
coeff(775)=0.863744735377D+04
coeff(776)=0.142192096924D+04
coeff(777)=0.110226817656D+04
coeff(778)=0.483211530216D+04
coeff(779)=-0.364609182950D+04
coeff(780)=0.149663860470D+04
coeff(781)=0.378272399804D+03
coeff(782)=0.572071564209D+04
coeff(783)=-0.301729355826D+04
coeff(784)=0.528987460814D+04
coeff(785)=0.664668251747D+03
coeff(786)=-0.396868806365D+04
coeff(787)=-0.635436857902D+04
coeff(788)=0.140417789719D+05
coeff(789)=-0.787096748982D+04
coeff(790)=-0.209470345377D+04
coeff(791)=0.618676774083D+04
coeff(792)=-0.488461348983D+04
coeff(793)=-0.361034095986D+04
coeff(794)=0.218452796522D+04
coeff(795)=-0.545725264101D+04
coeff(796)=0.443694310943D+04
coeff(797)=0.453693400057D+04
coeff(798)=-0.228810099190D+03
coeff(799)=0.119154190423D+04
coeff(800)=-0.669132826805D+04
coeff(801)=0.346082728014D+04
coeff(802)=0.622812314045D+04
coeff(803)=-0.765606572028D+04
coeff(804)=0.859990755517D+04
coeff(805)=0.549954225316D+04
coeff(806)=0.803219123319D+03
coeff(807)=-0.102799253882D+05
coeff(808)=0.282498907013D+04
coeff(809)=0.660475019749D+04
coeff(810)=0.150201979252D+05
coeff(811)=-0.117285145454D+05
coeff(812)=-0.309889420852D+04
coeff(813)=-0.136638342139D+05
coeff(814)=-0.627999913607D+04
coeff(815)=0.821797056314D+04
coeff(816)=-0.108309472165D+05
coeff(817)=-0.244942749190D+04
coeff(818)=0.215474528212D+05
coeff(819)=-0.587215304502D+03
coeff(820)=0.982835463127D+04
coeff(821)=-0.106120576748D+03
coeff(822)=0.867792633902D+04
coeff(823)=-0.730899901822D+01
coeff(824)=-0.660445363188D+04
coeff(825)=-0.398118233620D+04
coeff(826)=-0.446363574614D+02
coeff(827)=-0.340603891022D+04
coeff(828)=-0.202715014096D+04
coeff(829)=0.829038008528D+04
coeff(830)=-0.280297125641D+04
coeff(831)=-0.112042304333D+04
coeff(832)=-0.113681583766D+05
coeff(833)=0.294903116377D+04
coeff(834)=0.708697861443D+04
coeff(835)=0.149159891240D+04
coeff(836)=0.608787114365D+04
coeff(837)=-0.609023789676D+04
coeff(838)=0.640859211313D+04
coeff(839)=-0.689913392934D+04
coeff(840)=-0.391622509158D+04
coeff(841)=0.151774630273D+04
coeff(842)=-0.608140199329D+04
coeff(843)=0.961899246462D+04
coeff(844)=-0.104319044347D+05
coeff(845)=0.111462494692D+05
coeff(846)=-0.500598854689D+04
coeff(847)=-0.792469784431D+04
coeff(848)=-0.125442923405D+05
coeff(849)=0.661552589549D+04
coeff(850)=0.531966122545D+04
coeff(851)=0.903654597155D+04
coeff(852)=0.470632805721D+04
coeff(853)=-0.910361951596D+04
coeff(854)=-0.998505322029D+03
coeff(855)=-0.302553169622D+04
coeff(856)=-0.703909146614D+04
coeff(857)=0.780405948270D+04
coeff(858)=-0.658069877486D+04
coeff(859)=-0.756769242658D+04
coeff(860)=0.403433699024D+04
coeff(861)=0.626629125258D+04
coeff(862)=0.928415214728D+02
coeff(863)=-0.726328577263D+04
coeff(864)=-0.244818590500D+04
coeff(865)=0.174737755043D+05
coeff(866)=-0.240034076315D+04
coeff(867)=-0.433492914106D+04
coeff(868)=-0.597695024070D+04
coeff(869)=0.300156744094D+04
coeff(870)=-0.742248787374D+04
coeff(871)=-0.457481634085D+04
coeff(872)=-0.640563237891D+04
coeff(873)=0.960610843969D+03
coeff(874)=0.526798367365D+04
coeff(875)=-0.218841434546D+04
coeff(876)=0.154927346225D+05
coeff(877)=-0.106234703402D+05
coeff(878)=-0.130876465497D+05
coeff(879)=0.609693051817D+04
coeff(880)=0.157141121715D+04
coeff(881)=0.823445565864D+03
coeff(882)=-0.636504306523D+04
coeff(883)=0.514383388637D+04
coeff(884)=0.193175901855D+04
coeff(885)=-0.410543897317D+04
coeff(886)=0.304825012836D+04
coeff(887)=0.753145768085D+04
coeff(888)=0.596377160517D+04
coeff(889)=-0.616544837130D+04
coeff(890)=0.114958168461D+05
coeff(891)=0.365567851033D+04
coeff(892)=0.808734325444D+04
coeff(893)=-0.270202453051D+04
coeff(894)=0.887421671804D+04
coeff(895)=-0.842759271027D+04
coeff(896)=0.124689602525D+05
coeff(897)=0.622190450481D+03
coeff(898)=-0.145460078587D+05
coeff(899)=0.159973555461D+05
coeff(900)=0.599363728251D+04
coeff(901)=0.875888259826D+04
coeff(902)=-0.107026908704D+05
coeff(903)=0.151895895743D+05
coeff(904)=-0.645866657311D+04
coeff(905)=-0.653482811066D+04
coeff(906)=-0.327037998462D+04
coeff(907)=0.334200366735D+04
coeff(908)=-0.135897640204D+05
coeff(909)=-0.166811726803D+05
coeff(910)=0.222351153830D+05
coeff(911)=0.876859990081D+04
coeff(912)=-0.190178534295D+05
coeff(913)=0.122455480758D+05
coeff(914)=-0.108722324822D+05
coeff(915)=-0.392624293854D+04
coeff(916)=-0.836957168474D+04
coeff(917)=0.264074451034D+04
coeff(918)=-0.170874301392D+04
coeff(919)=-0.106582084486D+03
coeff(920)=-0.563015853103D+04
coeff(921)=-0.783595422440D+04
coeff(922)=0.145949507640D+05
coeff(923)=-0.457531687561D+04
coeff(924)=-0.151216900827D+05
coeff(925)=-0.451338958315D+04
coeff(926)=-0.119495110419D+05
coeff(927)=-0.586886831164D+04
coeff(928)=-0.111948229455D+05
coeff(929)=0.631794091383D+04
coeff(930)=-0.329924254116D+04
coeff(931)=-0.153095901072D+04
coeff(932)=-0.737866256058D+04
coeff(933)=0.193672079479D+04
coeff(934)=-0.312812140961D+03
coeff(935)=0.306045495291D+04
coeff(936)=0.128348581258D+04
coeff(937)=0.895679541613D+04
coeff(938)=-0.571914858414D+04
coeff(939)=0.305016141784D+04
coeff(940)=0.152312280251D+04
coeff(941)=0.246864514472D+04
coeff(942)=0.518490654753D+04
coeff(943)=-0.876881629319D+04
coeff(944)=0.807231776153D+03
coeff(945)=0.958463724926D+04
coeff(946)=-0.851010550573D+04
coeff(947)=0.110523383480D+05
coeff(948)=0.179337787236D+04
coeff(949)=0.109342348293D+05
coeff(950)=0.234133553125D+04
coeff(951)=0.161723769881D+05
coeff(952)=-0.418236278659D+04
coeff(953)=0.109122345998D+05
coeff(954)=-0.546652118579D+03
coeff(955)=-0.143941230192D+05
coeff(956)=-0.219797398282D+04
coeff(957)=-0.110946434836D+05
coeff(958)=-0.198385093952D+04
coeff(959)=0.380855415849D+04
coeff(960)=0.111590286576D+04
coeff(961)=0.886484671456D+03
coeff(962)=0.412462833020D+04
coeff(963)=-0.119981969952D+05
coeff(964)=0.223210314877D+04
coeff(965)=0.590463790687D+02
coeff(966)=0.216166052940D+05
coeff(967)=-0.160809208417D+05
coeff(968)=0.766341907126D+04
coeff(969)=-0.650017150465D+04
coeff(970)=-0.761017854370D+04
coeff(971)=0.550679341401D+04
coeff(972)=-0.256332288928D+04
coeff(973)=0.201249504513D+04
coeff(974)=-0.303705929638D+04
coeff(975)=-0.229244843579D+04
coeff(976)=-0.152272423153D+05
coeff(977)=0.982793014843D+04
coeff(978)=-0.646493575849D+04
coeff(979)=0.831640847167D+03
coeff(980)=0.184018549327D+04
coeff(981)=-0.383223722938D+04
coeff(982)=0.103215710337D+05
coeff(983)=-0.748191927143D+04
coeff(984)=0.112240605237D+04
coeff(985)=-0.380604915567D+04
coeff(986)=0.342555098708D+03
coeff(987)=0.216471333873D+04
coeff(988)=0.134412554573D+04
coeff(989)=0.200754235195D+04
coeff(990)=-0.593224663924D+04
coeff(991)=0.190765592764D+04
coeff(992)=0.939079430635D+04
coeff(993)=-0.439065192563D+04
coeff(994)=-0.142929707860D+05
coeff(995)=-0.332222469198D+03
coeff(996)=0.510433842291D+04
coeff(997)=0.206245253731D+04
coeff(998)=-0.431081292855D+04
coeff(999)=0.415579592387D+04
coeff(1000)=0.180807463184D+04
coeff(1001)=-0.331403111143D+04
coeff(1002)=0.553285679982D+04
coeff(1003)=-0.104607307260D+04
coeff(1004)=0.306172002928D+04
coeff(1005)=0.207678880089D+04
coeff(1006)=-0.180851916949D+04
coeff(1007)=0.667153466510D+03
coeff(1008)=-0.120395924416D+05
coeff(1009)=0.326015745094D+04
coeff(1010)=0.152860086117D+04
coeff(1011)=-0.724261421978D+04
coeff(1012)=0.131351980628D+05
coeff(1013)=0.526645619693D+03
coeff(1014)=-0.184903026323D+04
coeff(1015)=0.413563271614D+04
coeff(1016)=0.246690024307D+04
coeff(1017)=-0.419934885200D+04
coeff(1018)=0.568202689379D+03
coeff(1019)=-0.399709302924D+03
coeff(1020)=-0.166938571478D+04
coeff(1021)=0.483452007822D+04
coeff(1022)=0.210855967931D+04
coeff(1023)=0.118469662141D+04
coeff(1024)=0.153800383458D+05
coeff(1025)=-0.515133783794D+04
coeff(1026)=0.744125416632D+04
coeff(1027)=0.897883899397D+03
coeff(1028)=0.511700887211D+04
coeff(1029)=0.104434297318D+05
coeff(1030)=-0.711287075458D+04
coeff(1031)=-0.515738331964D+04
coeff(1032)=-0.289646215747D+04
coeff(1033)=-0.615835822210D+04
coeff(1034)=0.991863746891D+04
coeff(1035)=-0.458943746774D+04
coeff(1036)=-0.365340269661D+04
coeff(1037)=0.114808921955D+05
coeff(1038)=-0.390456621146D+04
coeff(1039)=-0.304474999015D+04
coeff(1040)=0.989693347304D+03
coeff(1041)=0.250686128187D+03
coeff(1042)=-0.477497291279D+04
coeff(1043)=-0.101947712569D+05
coeff(1044)=-0.198719409967D+04
coeff(1045)=0.871143961720D+04
coeff(1046)=-0.620547420282D+04
coeff(1047)=-0.619261251286D+03
coeff(1048)=0.297802600116D+04
coeff(1049)=-0.163394249340D+05
coeff(1050)=0.723551647487D+04
coeff(1051)=-0.221467539168D+04
coeff(1052)=-0.133291612495D+05
coeff(1053)=0.103307251810D+03
coeff(1054)=0.609501076248D+04
coeff(1055)=-0.118555402861D+05
coeff(1056)=-0.200723682953D+05
coeff(1057)=0.502980459713D+04
coeff(1058)=-0.369779159423D+04
coeff(1059)=-0.256401647756D+03
coeff(1060)=0.198720705793D+04
coeff(1061)=-0.767333243543D+04
coeff(1062)=-0.176031539671D+04
coeff(1063)=-0.105431670030D+05
coeff(1064)=-0.276734200789D+04
coeff(1065)=-0.160822597107D+05
coeff(1066)=0.404326370631D+04
coeff(1067)=-0.547164108737D+04
coeff(1068)=-0.238877134414D+04
coeff(1069)=-0.516877763229D+04
coeff(1070)=-0.245942142727D+04
coeff(1071)=0.942593223117D+04
coeff(1072)=-0.412957772290D+04
coeff(1073)=0.976740504130D+04
coeff(1074)=0.680565205848D+04
coeff(1075)=-0.246706159290D+04
coeff(1076)=0.639111966099D+04
coeff(1077)=-0.320017599445D+04
coeff(1078)=0.291452882756D+04
coeff(1079)=0.122086126468D+05
coeff(1080)=-0.139600132006D+05
coeff(1081)=-0.573997723125D+03
coeff(1082)=0.162833499998D+05
coeff(1083)=-0.331916417761D+04
coeff(1084)=0.141490020834D+05
coeff(1085)=-0.222899211058D+04
coeff(1086)=-0.500776744843D+03
coeff(1087)=-0.119446081191D+05
coeff(1088)=0.107778789563D+05
coeff(1089)=-0.201987316516D+04
coeff(1090)=-0.519985098394D+04
coeff(1091)=-0.104897599003D+04
coeff(1092)=0.651974429738D+04
coeff(1093)=0.382404950375D+04
coeff(1094)=-0.640166727399D+04
coeff(1095)=0.214596122299D+04
coeff(1096)=0.113178847611D+05
coeff(1097)=-0.129328917801D+05
coeff(1098)=0.163294478204D+05
coeff(1099)=0.906101257744D+04
coeff(1100)=0.257920958264D+04
coeff(1101)=-0.741013527203D+04
coeff(1102)=0.149307807837D+05
coeff(1103)=0.178577274228D+05
coeff(1104)=-0.107865124120D+04
coeff(1105)=-0.261569734753D+04
coeff(1106)=0.256857721181D+04
coeff(1107)=0.473746367844D+04
coeff(1108)=0.375023045824D+04
coeff(1109)=0.197046937372D+05
coeff(1110)=0.618443160167D+04
coeff(1111)=0.521618260495D+04
coeff(1112)=-0.370342687520D+04
coeff(1113)=-0.508472110831D+03
coeff(1114)=-0.433657118271D+04
coeff(1115)=-0.120340102130D+05
coeff(1116)=-0.538114634742D+04
coeff(1117)=-0.230745887492D+05
coeff(1118)=0.281043972690D+04
coeff(1119)=0.633233979747D+04
coeff(1120)=-0.241083905305D+04
coeff(1121)=-0.668913938057D+04
coeff(1122)=0.323993270899D+04
coeff(1123)=0.123264398543D+04
coeff(1124)=-0.935263616247D+04
coeff(1125)=0.188016727290D+04
coeff(1126)=0.874789085330D+04
coeff(1127)=0.974682670867D+04
coeff(1128)=-0.102477332870D+04
coeff(1129)=0.238337250164D+04
coeff(1130)=0.138154924590D+05
coeff(1131)=-0.956196175249D+04
coeff(1132)=0.383554392058D+03
coeff(1133)=-0.892474878851D+04
coeff(1134)=-0.170138486404D+05
coeff(1135)=0.101345570473D+05
coeff(1136)=-0.973135005910D+04
coeff(1137)=0.125501901799D+04
coeff(1138)=0.496387007153D+04
coeff(1139)=-0.240442713865D+03
coeff(1140)=-0.565203290848D+04
coeff(1141)=0.868866018014D+03
coeff(1142)=0.793774826517D+04
coeff(1143)=-0.192395278181D+05
coeff(1144)=-0.561628155543D+04
coeff(1145)=-0.977214787122D+04
coeff(1146)=0.117337772976D+05
coeff(1147)=-0.638685491342D+04
coeff(1148)=-0.745674470961D+04
coeff(1149)=0.840741371960D+03
coeff(1150)=-0.459159664820D+04
coeff(1151)=-0.230676338093D+04
coeff(1152)=-0.151941459869D+04
coeff(1153)=-0.110701195688D+05
coeff(1154)=-0.255880516380D+04
coeff(1155)=0.666739586311D+04
coeff(1156)=0.147950722068D+05
coeff(1157)=0.144354611955D+05
coeff(1158)=-0.698706756443D+04
coeff(1159)=0.220593391505D+04
coeff(1160)=-0.974957827449D+04
coeff(1161)=0.121261245698D+05
coeff(1162)=0.105373139105D+05
coeff(1163)=-0.854854503945D+04
coeff(1164)=0.193929639108D+04
coeff(1165)=-0.436740704211D+04
coeff(1166)=-0.165605913195D+04
coeff(1167)=-0.110557648462D+03
coeff(1168)=0.192199798763D+04
coeff(1169)=-0.420258792953D+04
coeff(1170)=0.301788129934D+04
coeff(1171)=-0.232708947986D+04
coeff(1172)=0.106206300298D+05
coeff(1173)=-0.174585930162D+04
coeff(1174)=-0.460092543600D+04
coeff(1175)=-0.652113327614D+04
coeff(1176)=0.347959010668D+04
coeff(1177)=-0.135215683270D+03
coeff(1178)=-0.238609175971D+04
coeff(1179)=0.783200838063D+04
coeff(1180)=-0.296562332309D+04
coeff(1181)=-0.328537488519D+04
coeff(1182)=0.798943932482D+03
coeff(1183)=-0.184958966715D+04
coeff(1184)=0.279254896870D+03
coeff(1185)=0.280243985721D+04
coeff(1186)=-0.139458365598D+03
coeff(1187)=0.279769424037D+04
coeff(1188)=-0.322694424801D+04
coeff(1189)=0.106093261359D+05
coeff(1190)=-0.238656886078D+04
coeff(1191)=-0.366793710079D+04
coeff(1192)=-0.570311820870D+04
coeff(1193)=0.548713240308D+04
coeff(1194)=0.805685815275D+03
coeff(1195)=0.192260154154D+04
coeff(1196)=-0.457746300623D+03
coeff(1197)=0.964359279775D+03
coeff(1198)=0.277470005233D+04
coeff(1199)=-0.461434226329D+04
coeff(1200)=0.636347656147D+04
coeff(1201)=0.881088004759D+02
coeff(1202)=-0.583184353708D+04
coeff(1203)=0.274187407323D+04
coeff(1204)=0.411058452762D+04
coeff(1205)=0.122693782758D+04
coeff(1206)=0.343717398908D+02
coeff(1207)=0.664500630329D+02
coeff(1208)=0.758613003656D+03
coeff(1209)=0.156827825106D+04
coeff(1210)=-0.659040833365D+04
coeff(1211)=0.322116349922D+04
coeff(1212)=-0.344388648273D+04
coeff(1213)=-0.302348694474D+04
coeff(1214)=0.841440746410D+04
coeff(1215)=0.179307724515D+04
coeff(1216)=-0.813107794171D+03
coeff(1217)=-0.739726053024D+04
coeff(1218)=-0.321535299670D+04
coeff(1219)=0.486537780487D+04
coeff(1220)=-0.303849457366D+04
coeff(1221)=-0.239163770271D+04
coeff(1222)=0.717666977877D+04
coeff(1223)=0.556750537194D+02
coeff(1224)=0.220346394014D+04
coeff(1225)=0.783390445115D+04
coeff(1226)=-0.930410994636D+04
coeff(1227)=0.212252690927D+04
coeff(1228)=-0.577771601494D+04
coeff(1229)=0.641874574677D+04
coeff(1230)=0.257291567001D+04
coeff(1231)=-0.117429768634D+04
coeff(1232)=0.332384306895D+04
coeff(1233)=0.543405554945D+04
coeff(1234)=-0.466007324056D+04
coeff(1235)=-0.701997893616D+04
coeff(1236)=0.733285296748D+04
coeff(1237)=-0.291298766024D+04
coeff(1238)=-0.456580744569D+03
coeff(1239)=0.144518615142D+04
coeff(1240)=-0.516148353645D+04
coeff(1241)=0.775303605000D+04
coeff(1242)=-0.379272587804D+04
coeff(1243)=-0.179142917876D+04
coeff(1244)=0.161889625574D+04
coeff(1245)=-0.316720201679D+04
coeff(1246)=0.369911359740D+03
coeff(1247)=0.376038859230D+04
coeff(1248)=0.630993948202D+03
coeff(1249)=-0.350217671305D+04
coeff(1250)=-0.198397116591D+04
coeff(1251)=0.141845071963D+04
coeff(1252)=0.204890416389D+04
coeff(1253)=-0.323409074000D+04
coeff(1254)=0.598729081206D+04
coeff(1255)=-0.185445665808D+04
coeff(1256)=-0.630189380355D+04
coeff(1257)=0.368804489827D+04
coeff(1258)=0.279116885743D+04
coeff(1259)=-0.269376979853D+04
coeff(1260)=0.281521415304D+03
coeff(1261)=-0.248719675490D+04
coeff(1262)=-0.678096550427D+04
coeff(1263)=0.246303901850D+04
coeff(1264)=0.109093683165D+04
coeff(1265)=0.240827842518D+04
coeff(1266)=-0.467038045590D+03
coeff(1267)=0.190652819034D+04
coeff(1268)=-0.119253418319D+04
coeff(1269)=0.297504970699D+04
coeff(1270)=-0.372180817227D+04
coeff(1271)=0.725833288768D+04
coeff(1272)=-0.155064916395D+04
coeff(1273)=-0.139361901313D+04
coeff(1274)=-0.856926532827D+03
coeff(1275)=-0.133633074298D+04
coeff(1276)=0.452716521479D+03
coeff(1277)=0.293372004239D+04
coeff(1278)=-0.491761024527D+04
coeff(1279)=0.285776563194D+04
coeff(1280)=-0.274642512431D+04
coeff(1281)=-0.621678198877D+03
coeff(1282)=-0.229000176391D+03
coeff(1283)=0.959662025967D+03
coeff(1284)=-0.272501955635D+04
coeff(1285)=-0.205673067271D+03
coeff(1286)=0.715401340326D+03
coeff(1287)=0.437786789597D+04
coeff(1288)=-0.416087909185D+04
coeff(1289)=0.321283843292D+04
coeff(1290)=-0.530917677250D+03
coeff(1291)=-0.107625457168D+04
coeff(1292)=0.127583028738D+04
coeff(1293)=0.945732637439D+03
coeff(1294)=-0.208122586535D+04
coeff(1295)=0.150056915542D+04
coeff(1296)=-0.104445028506D+05
coeff(1297)=0.646843188624D+04
coeff(1298)=-0.441894669672D+04
coeff(1299)=0.238816376192D+03
coeff(1300)=-0.362769384165D+03

! Calculate MEG
x(1)=exp(-((r(1)-ra1))/a1 -((r(1)-rb1)**2)/b1)
x(2)=exp(-((r(2)-ra1))/a1 -((r(2)-rb1)**2)/b1)
x(3)=exp(-((r(3)-ra2))/a2 -((r(3)-rb2)**2)/b2)
x(4)=exp(-((r(4)-ra1))/a1 -((r(4)-rb1)**2)/b1)
x(5)=exp(-((r(5)-ra2))/a2 -((r(5)-rb2)**2)/b2)
x(6)=exp(-((r(6)-ra2))/a2 -((r(6)-rb2)**2)/b2)

! Calculate dMEG/dr(6,6) for giving R(6)
! dmsdr(i,j)=0  i!=j
! Initialize dmsdr
      do i=1,6
        do j=1,6
          dmsdr(i,j)=0.0d0
        enddo
      enddo
dmsdr(1,1)=(-2.0d0*(r(1)-rb1)/b1-1/a1)*Exp(-(r(1)-ra1)/a1-((r(1)-rb1)**2.0d0)/b1)
dmsdr(2,2)=(-2.0d0*(r(2)-rb1)/b1-1/a1)*Exp(-(r(2)-ra1)/a1-((r(2)-rb1)**2.0d0)/b1)
dmsdr(3,3)=(-2.0d0*(r(3)-rb2)/b2-1/a2)*Exp(-(r(3)-ra2)/a2-((r(3)-rb2)**2.0d0)/b2)
dmsdr(4,4)=(-2.0d0*(r(4)-rb1)/b1-1/a1)*Exp(-(r(4)-ra1)/a1-((r(4)-rb1)**2.0d0)/b1)
dmsdr(5,5)=(-2.0d0*(r(5)-rb2)/b2-1/a2)*Exp(-(r(5)-ra2)/a2-((r(5)-rb2)**2.0d0)/b2)
dmsdr(6,6)=(-2.0d0*(r(6)-rb2)/b2-1/a2)*Exp(-(r(6)-ra2)/a2-((r(6)-rb2)**2.0d0)/b2)

rm(0) = 1.0D0
rm(1) = x(4)
rm(2) = x(2)
rm(3) = x(1)
rm(4) = x(6)
rm(5) = x(5)
rm(6) = x(3)
rm(7) = rm(1)*rm(2)
rm(8) = rm(1)*rm(3)
rm(9) = rm(2)*rm(3)
rm(10) = rm(3)*rm(4)
rm(11) = rm(2)*rm(5)
rm(12) = rm(1)*rm(6)
rm(13) = rm(4)*rm(5)
rm(14) = rm(4)*rm(6)
rm(15) = rm(5)*rm(6)
rm(16) = rm(1)*rm(9)
rm(17) = rm(1)*rm(10)
rm(18) = rm(2)*rm(10)
rm(19) = rm(1)*rm(11)
rm(20) = rm(3)*rm(11)
rm(21) = rm(2)*rm(12)
rm(22) = rm(3)*rm(12)
rm(23) = rm(2)*rm(13)
rm(24) = rm(3)*rm(13)
rm(25) = rm(1)*rm(14)
rm(26) = rm(3)*rm(14)
rm(27) = rm(1)*rm(15)
rm(28) = rm(2)*rm(15)
rm(29) = rm(4)*rm(15)
rm(30) = rm(2)*rm(24)
rm(31) = rm(1)*rm(26)
rm(32) = rm(1)*rm(28)

p(0) = rm(0)
p(1) = rm(1) + rm(2) + rm(3)
p(2) = rm(4) + rm(5) + rm(6)
p(3) = rm(7) + rm(8) + rm(9)
p(4) = rm(10) + rm(11) + rm(12)
p(5) = p(1)*p(2) - p(4)
p(6) = rm(13) + rm(14) + rm(15)
p(7) = p(1)*p(1) - p(3) - p(3)
p(8) = p(2)*p(2) - p(6) - p(6)
p(9) = rm(16)
p(10) = rm(17) + rm(18) + rm(19) + rm(20) + rm(21) + rm(22)
p(11) = p(2)*p(3) - p(10)
p(12) = rm(23) + rm(24) + rm(25) + rm(26) + rm(27) + rm(28)
p(13) = p(1)*p(6) - p(12)
p(14) = rm(29)
p(15) = p(1)*p(3) - p(9) - p(9) - p(9)
p(16) = p(1)*p(4) - p(10)
p(17) = p(2)*p(7) - p(16)
p(18) = p(2)*p(4) - p(12)
p(19) = p(1)*p(8) - p(18)
p(20) = p(2)*p(6) - p(14) - p(14) - p(14)
p(21) = p(1)*p(7) - p(15)
p(22) = p(2)*p(8) - p(20)
p(23) = p(9)*p(2)
p(24) = rm(30) + rm(31) + rm(32)
p(25) = p(3)*p(6) - p(24)
p(26) = p(14)*p(1)
p(27) = p(9)*p(1)
p(28) = p(3)*p(4) - p(23)
p(29) = p(1)*p(10) - p(23) - p(28) - p(23)
p(30) = p(1)*p(11) - p(23)
p(31) = p(1)*p(12) - p(25) - p(24) - p(24)
p(32) = p(1)*p(13) - p(25)
p(33) = p(4)*p(5) - p(25) - p(31)
p(34) = p(2)*p(11) - p(25)
p(35) = p(4)*p(6) - p(26)
p(36) = p(2)*p(12) - p(26) - p(35) - p(26)
p(37) = p(2)*p(13) - p(26)
p(38) = p(14)*p(2)
p(39) = p(3)*p(3) - p(27) - p(27)
p(40) = p(3)*p(7) - p(27)
p(41) = p(1)*p(16) - p(28)
p(42) = p(2)*p(21) - p(41)
p(43) = p(1)*p(18) - p(33)
p(44) = p(7)*p(8) - p(43)
p(45) = p(6)*p(6) - p(38) - p(38)
p(46) = p(2)*p(18) - p(35)
p(47) = p(1)*p(22) - p(46)
p(48) = p(6)*p(8) - p(38)
p(49) = p(1)*p(21) - p(40)
p(50) = p(2)*p(22) - p(48)
p(51) = p(9)*p(6)
p(52) = p(14)*p(3)
p(53) = p(9)*p(4)
p(54) = p(9)*p(5)
p(55) = p(1)*p(24) - p(51)
p(56) = p(3)*p(12) - p(51) - p(55) - p(51)
p(57) = p(3)*p(13) - p(51)
p(58) = p(14)*p(7)
p(59) = p(9)*p(8)
p(60) = p(2)*p(24) - p(52)
p(61) = p(4)*p(13) - p(58)
p(62) = p(6)*p(11) - p(52)
p(63) = p(14)*p(4)
p(64) = p(14)*p(5)
p(65) = p(9)*p(3)
p(66) = p(9)*p(7)
p(67) = p(3)*p(16) - p(53)
p(68) = p(4)*p(15) - p(54) - p(67)
p(69) = p(2)*p(39) - p(68)
p(70) = p(1)*p(29) - p(54) - p(68)
p(71) = p(7)*p(11) - p(53)
p(72) = p(1)*p(31) - p(56) - p(55)
p(73) = p(1)*p(32) - p(57)
p(74) = p(3)*p(18) - p(59)
p(75) = p(2)*p(29) - p(57) - p(55)
p(76) = p(1)*p(34) - p(59)
p(77) = p(6)*p(16) - p(58)
p(78) = p(1)*p(36) - p(62) - p(60)
p(79) = p(2)*p(32) - p(58)
p(80) = p(6)*p(12) - p(64) - p(63) - p(63)
p(81) = p(1)*p(45) - p(80)
p(82) = p(14)*p(6)
p(83) = p(2)*p(33) - p(61) - p(60)
p(84) = p(2)*p(34) - p(62)
p(85) = p(6)*p(18) - p(63)
p(86) = p(2)*p(36) - p(64) - p(80)
p(87) = p(8)*p(13) - p(63)
p(88) = p(14)*p(8)
p(89) = p(1)*p(39) - p(65)
p(90) = p(3)*p(21) - p(66)
p(91) = p(1)*p(41) - p(67)
p(92) = p(2)*p(49) - p(91)
p(93) = p(1)*p(43) - p(74)
p(94) = p(8)*p(21) - p(93)
p(95) = p(1)*p(46) - p(83)
p(96) = p(7)*p(22) - p(95)
p(97) = p(2)*p(45) - p(82)
p(98) = p(2)*p(46) - p(85)
p(99) = p(1)*p(50) - p(98)
p(100) = p(6)*p(22) - p(88)
p(101) = p(1)*p(49) - p(90)
p(102) = p(2)*p(50) - p(100)
p(103) = p(9)*p(14)
p(104) = p(9)*p(12)
p(105) = p(9)*p(13)
p(106) = p(14)*p(15)
p(107) = p(9)*p(20)
p(108) = p(14)*p(10)
p(109) = p(14)*p(11)
p(110) = p(9)*p(16)
p(111) = p(9)*p(10)
p(112) = p(9)*p(11)
p(113) = p(9)*p(17)
p(114) = p(3)*p(24) - p(104)
p(115) = p(7)*p(24) - p(105)
p(116) = p(3)*p(31) - p(104) - p(115)
p(117) = p(6)*p(39) - p(114)
p(118) = p(3)*p(32) - p(105)
p(119) = p(14)*p(21)
p(120) = p(9)*p(18)
p(121) = p(9)*p(19)
p(122) = p(4)*p(24) - p(103) - p(103) - p(103)
p(123) = p(1)*p(60) - p(107) - p(122)
p(124) = p(13)*p(16) - p(119)
p(125) = p(2)*p(56) - p(106) - p(124)
p(126) = p(4)*p(32) - p(119)
p(127) = p(1)*p(62) - p(107) - p(125)
p(128) = p(14)*p(16)
p(129) = p(14)*p(17)
p(130) = p(6)*p(24) - p(108)
p(131) = p(3)*p(45) - p(130)
p(132) = p(14)*p(12)
p(133) = p(14)*p(13)
p(134) = p(9)*p(22)
p(135) = p(8)*p(24) - p(109)
p(136) = p(13)*p(18) - p(128)
p(137) = p(6)*p(34) - p(109)
p(138) = p(14)*p(18)
p(139) = p(14)*p(19)
p(140) = p(9)*p(9)
p(141) = p(9)*p(15)
p(142) = p(9)*p(21)
p(143) = p(3)*p(41) - p(110)
p(144) = p(4)*p(39) - p(112)
p(145) = p(1)*p(68) - p(111) - p(144)
p(146) = p(1)*p(69) - p(112)
p(147) = p(1)*p(70) - p(113) - p(145)
p(148) = p(11)*p(21) - p(110)
p(149) = p(1)*p(72) - p(116) - p(115)
p(150) = p(1)*p(73) - p(118)
p(151) = p(3)*p(43) - p(120)
p(152) = p(15)*p(18) - p(121) - p(151)
p(153) = p(2)*p(69) - p(117)
p(154) = p(1)*p(75) - p(121) - p(152)
p(155) = p(7)*p(34) - p(120)
p(156) = p(6)*p(41) - p(119)
p(157) = p(1)*p(78) - p(125) - p(123)
p(158) = p(2)*p(73) - p(119)
p(159) = p(1)*p(80) - p(131) - p(130) - p(130)
p(160) = p(1)*p(81) - p(131)
p(161) = p(14)*p(14)
p(162) = p(3)*p(46) - p(134)
p(163) = p(2)*p(75) - p(126) - p(123)
p(164) = p(1)*p(84) - p(134)
p(165) = p(6)*p(43) - p(128)
p(166) = p(1)*p(86) - p(137) - p(135)
p(167) = p(8)*p(32) - p(128)
p(168) = p(4)*p(45) - p(133)
p(169) = p(2)*p(80) - p(132) - p(168)
p(170) = p(2)*p(81) - p(133)
p(171) = p(14)*p(20)
p(172) = p(2)*p(83) - p(136) - p(135)
p(173) = p(2)*p(84) - p(137)
p(174) = p(6)*p(46) - p(138)
p(175) = p(2)*p(86) - p(139) - p(169)
p(176) = p(13)*p(22) - p(138)
p(177) = p(14)*p(22)
p(178) = p(3)*p(39) - p(141)
p(179) = p(1)*p(89) - p(141) - p(178) - p(178)
p(180) = p(3)*p(49) - p(142)
p(181) = p(1)*p(91) - p(143)
p(182) = p(2)*p(101) - p(181)
p(183) = p(1)*p(93) - p(151)
p(184) = p(8)*p(49) - p(183)
p(185) = p(1)*p(95) - p(162)
p(186) = p(21)*p(22) - p(185)
p(187) = p(6)*p(45) - p(171)
p(188) = p(1)*p(98) - p(172)
p(189) = p(7)*p(50) - p(188)
p(190) = p(2)*p(97) - p(171) - p(187) - p(187)
p(191) = p(2)*p(98) - p(174)
p(192) = p(1)*p(102) - p(191)
p(193) = p(6)*p(50) - p(177)
p(194) = p(1)*p(101) - p(180)
p(195) = p(2)*p(102) - p(193)
p(196) = p(9)*p(26)
p(197) = p(9)*p(38)
p(198) = p(9)*p(24)
p(199) = p(9)*p(31)
p(200) = p(9)*p(25)
p(201) = p(9)*p(32)
p(202) = p(14)*p(39)
p(203) = p(14)*p(40)
p(204) = p(9)*p(35)
p(205) = p(9)*p(36)
p(206) = p(9)*p(37)
p(207) = p(14)*p(28)
p(208) = p(14)*p(29)
p(209) = p(14)*p(30)
p(210) = p(9)*p(45)
p(211) = p(14)*p(24)
p(212) = p(14)*p(25)
p(213) = p(9)*p(48)
p(214) = p(14)*p(33)
p(215) = p(14)*p(34)
p(216) = p(9)*p(41)
p(217) = p(9)*p(28)
p(218) = p(9)*p(23)
p(219) = p(9)*p(29)
p(220) = p(9)*p(30)
p(221) = p(9)*p(42)
p(222) = p(1)*p(114) - p(198)
p(223) = p(21)*p(24) - p(201)
p(224) = p(3)*p(72) - p(199) - p(223)
p(225) = p(1)*p(116) - p(199) - p(224)
p(226) = p(13)*p(39) - p(198)
p(227) = p(3)*p(73) - p(201)
p(228) = p(14)*p(49)
p(229) = p(9)*p(43)
p(230) = p(9)*p(33)
p(231) = p(9)*p(34)
p(232) = p(9)*p(44)
p(233) = p(16)*p(24) - p(196)
p(234) = p(2)*p(114) - p(202)
p(235) = p(1)*p(123) - p(205) - p(234)
p(236) = p(13)*p(41) - p(228)
p(237) = p(2)*p(116) - p(203) - p(236)
p(238) = p(16)*p(32) - p(228)
p(239) = p(6)*p(69) - p(202)
p(240) = p(4)*p(73) - p(228)
p(241) = p(11)*p(32) - p(196)
p(242) = p(14)*p(41)
p(243) = p(14)*p(42)
p(244) = p(1)*p(130) - p(210)
p(245) = p(4)*p(62) - p(213) - p(209)
p(246) = p(3)*p(81) - p(210)
p(247) = p(14)*p(31)
p(248) = p(14)*p(32)
p(249) = p(14)*p(26)
p(250) = p(9)*p(46)
p(251) = p(9)*p(47)
p(252) = p(18)*p(24) - p(197)
p(253) = p(1)*p(135) - p(213) - p(252)
p(254) = p(13)*p(43) - p(242)
p(255) = p(2)*p(125) - p(209) - p(245)
p(256) = p(18)*p(32) - p(242)
p(257) = p(13)*p(34) - p(197)
p(258) = p(14)*p(43)
p(259) = p(14)*p(44)
p(260) = p(2)*p(130) - p(211)
p(261) = p(4)*p(81) - p(248)
p(262) = p(11)*p(45) - p(211)
p(263) = p(14)*p(35)
p(264) = p(14)*p(36)
p(265) = p(14)*p(37)
p(266) = p(9)*p(50)
p(267) = p(22)*p(24) - p(215)
p(268) = p(13)*p(46) - p(258)
p(269) = p(6)*p(84) - p(215)
p(270) = p(14)*p(46)
p(271) = p(14)*p(47)
p(272) = p(9)*p(27)
p(273) = p(9)*p(39)
p(274) = p(9)*p(40)
p(275) = p(9)*p(49)
p(276) = p(3)*p(91) - p(216)
p(277) = p(16)*p(39) - p(218)
p(278) = p(1)*p(144) - p(217) - p(277)
p(279) = p(2)*p(178) - p(278)
p(280) = p(1)*p(145) - p(219) - p(278)
p(281) = p(7)*p(69) - p(218)
p(282) = p(1)*p(147) - p(221) - p(280)
p(283) = p(11)*p(49) - p(216)
p(284) = p(1)*p(149) - p(224) - p(223)
p(285) = p(1)*p(150) - p(227)
p(286) = p(3)*p(93) - p(229)
p(287) = p(18)*p(39) - p(231)
p(288) = p(1)*p(152) - p(230) - p(287)
p(289) = p(1)*p(153) - p(231)
p(290) = p(1)*p(154) - p(232) - p(288)
p(291) = p(21)*p(34) - p(229)
p(292) = p(6)*p(91) - p(228)
p(293) = p(1)*p(157) - p(237) - p(235)
p(294) = p(2)*p(150) - p(228)
p(295) = p(1)*p(159) - p(245) - p(244)
p(296) = p(1)*p(160) - p(246)
p(297) = p(3)*p(95) - p(250)
p(298) = p(2)*p(152) - p(238) - p(234)
p(299) = p(2)*p(153) - p(239)
p(300) = p(1)*p(163) - p(251) - p(298)
p(301) = p(7)*p(84) - p(250)
p(302) = p(6)*p(93) - p(242)
p(303) = p(1)*p(166) - p(255) - p(253)
p(304) = p(8)*p(73) - p(242)
p(305) = p(16)*p(45) - p(248)
p(306) = p(1)*p(169) - p(262) - p(260)
p(307) = p(2)*p(160) - p(248)
p(308) = p(14)*p(38)
p(309) = p(6)*p(80) - p(264) - p(263)
p(310) = p(1)*p(187) - p(309)
p(311) = p(14)*p(45)
p(312) = p(3)*p(98) - p(266)
p(313) = p(2)*p(163) - p(256) - p(253)
p(314) = p(1)*p(173) - p(266)
p(315) = p(6)*p(95) - p(258)
p(316) = p(1)*p(175) - p(269) - p(267)
p(317) = p(22)*p(32) - p(258)
p(318) = p(18)*p(45) - p(249)
p(319) = p(2)*p(169) - p(264) - p(309)
p(320) = p(8)*p(81) - p(249)
p(321) = p(14)*p(48)
p(322) = p(2)*p(172) - p(268) - p(267)
p(323) = p(2)*p(173) - p(269)
p(324) = p(6)*p(98) - p(270)
p(325) = p(2)*p(175) - p(271) - p(319)
p(326) = p(13)*p(50) - p(270)
p(327) = p(14)*p(50)
p(328) = p(1)*p(178) - p(273)
p(329) = p(21)*p(39) - p(272)
p(330) = p(3)*p(101) - p(275)
p(331) = p(1)*p(181) - p(276)
p(332) = p(2)*p(194) - p(331)
p(333) = p(1)*p(183) - p(286)
p(334) = p(8)*p(101) - p(333)
p(335) = p(1)*p(185) - p(297)
p(336) = p(22)*p(49) - p(335)
p(337) = p(1)*p(188) - p(312)
p(338) = p(21)*p(50) - p(337)
p(339) = p(2)*p(187) - p(311)
p(340) = p(1)*p(191) - p(322)
p(341) = p(7)*p(102) - p(340)
p(342) = p(22)*p(45) - p(308)
p(343) = p(2)*p(191) - p(324)
p(344) = p(1)*p(195) - p(343)
p(345) = p(6)*p(102) - p(327)
p(346) = p(1)*p(194) - p(330)
p(347) = p(2)*p(195) - p(345)
p(348) = p(9)*p(52)
p(349) = p(9)*p(58)
p(350) = p(9)*p(63)
p(351) = p(9)*p(64)
p(352) = p(9)*p(82)
p(353) = p(9)*p(88)
p(354) = p(9)*p(55)
p(355) = p(9)*p(72)
p(356) = p(9)*p(51)
p(357) = p(9)*p(56)
p(358) = p(9)*p(57)
p(359) = p(9)*p(73)
p(360) = p(14)*p(89)
p(361) = p(14)*p(90)
p(362) = p(9)*p(77)
p(363) = p(9)*p(60)
p(364) = p(9)*p(78)
p(365) = p(9)*p(61)
p(366) = p(9)*p(62)
p(367) = p(9)*p(79)
p(368) = p(14)*p(67)
p(369) = p(14)*p(68)
p(370) = p(14)*p(69)
p(371) = p(14)*p(70)
p(372) = p(14)*p(71)
p(373) = p(9)*p(80)
p(374) = p(9)*p(81)
p(375) = p(14)*p(55)
p(376) = p(14)*p(56)
p(377) = p(14)*p(57)
p(378) = p(14)*p(52)
p(379) = p(9)*p(85)
p(380) = p(9)*p(86)
p(381) = p(9)*p(87)
p(382) = p(14)*p(74)
p(383) = p(14)*p(75)
p(384) = p(14)*p(76)
p(385) = p(9)*p(97)
p(386) = p(14)*p(60)
p(387) = p(14)*p(61)
p(388) = p(14)*p(62)
p(389) = p(9)*p(100)
p(390) = p(14)*p(83)
p(391) = p(14)*p(84)
p(392) = p(9)*p(91)
p(393) = p(9)*p(67)
p(394) = p(9)*p(53)
p(395) = p(9)*p(68)
p(396) = p(9)*p(54)
p(397) = p(9)*p(69)
p(398) = p(9)*p(70)
p(399) = p(9)*p(71)
p(400) = p(9)*p(92)
p(401) = p(3)*p(114) - p(354)
p(402) = p(7)*p(114) - p(356)
p(403) = p(24)*p(49) - p(359)
p(404) = p(3)*p(149) - p(355) - p(403)
p(405) = p(1)*p(224) - p(355) - p(404)
p(406) = p(6)*p(178) - p(401)
p(407) = p(32)*p(39) - p(356)
p(408) = p(3)*p(150) - p(359)
p(409) = p(14)*p(101)
p(410) = p(9)*p(93)
p(411) = p(9)*p(74)
p(412) = p(9)*p(59)
p(413) = p(9)*p(75)
p(414) = p(9)*p(76)
p(415) = p(9)*p(94)
p(416) = p(24)*p(41) - p(349)
p(417) = p(4)*p(114) - p(348)
p(418) = p(1)*p(234) - p(363) - p(417)
p(419) = p(1)*p(235) - p(364) - p(418)
p(420) = p(13)*p(91) - p(409)
p(421) = p(2)*p(224) - p(361) - p(420)
p(422) = p(32)*p(41) - p(409)
p(423) = p(1)*p(237) - p(364) - p(421)
p(424) = p(16)*p(73) - p(409)
p(425) = p(13)*p(69) - p(348)
p(426) = p(4)*p(150) - p(409)
p(427) = p(11)*p(73) - p(349)
p(428) = p(14)*p(91)
p(429) = p(14)*p(92)
p(430) = p(3)*p(130) - p(373)
p(431) = p(7)*p(130) - p(374)
p(432) = p(3)*p(159) - p(373) - p(431)
p(433) = p(39)*p(45) - p(430)
p(434) = p(3)*p(160) - p(374)
p(435) = p(14)*p(72)
p(436) = p(14)*p(73)
p(437) = p(14)*p(58)
p(438) = p(9)*p(95)
p(439) = p(9)*p(83)
p(440) = p(9)*p(84)
p(441) = p(9)*p(96)
p(442) = p(24)*p(43) - p(350)
p(443) = p(8)*p(114) - p(370)
p(444) = p(1)*p(253) - p(380) - p(443)
p(445) = p(13)*p(93) - p(428)
p(446) = p(2)*p(237) - p(372) - p(432)
p(447) = p(32)*p(43) - p(428)
p(448) = p(6)*p(153) - p(370)
p(449) = p(18)*p(73) - p(428)
p(450) = p(32)*p(34) - p(350)
p(451) = p(14)*p(93)
p(452) = p(14)*p(94)
p(453) = p(4)*p(130) - p(352)
p(454) = p(1)*p(260) - p(385) - p(453)
p(455) = p(16)*p(81) - p(436)
p(456) = p(2)*p(245) - p(376) - p(455)
p(457) = p(4)*p(160) - p(436)
p(458) = p(11)*p(81) - p(352)
p(459) = p(14)*p(77)
p(460) = p(14)*p(78)
p(461) = p(14)*p(79)
p(462) = p(14)*p(63)
p(463) = p(14)*p(64)
p(464) = p(6)*p(130) - p(386)
p(465) = p(3)*p(187) - p(464)
p(466) = p(14)*p(80)
p(467) = p(14)*p(81)
p(468) = p(9)*p(98)
p(469) = p(9)*p(99)
p(470) = p(24)*p(46) - p(353)
p(471) = p(1)*p(267) - p(389) - p(470)
p(472) = p(13)*p(95) - p(451)
p(473) = p(2)*p(255) - p(384) - p(456)
p(474) = p(32)*p(46) - p(451)
p(475) = p(13)*p(84) - p(353)
p(476) = p(14)*p(95)
p(477) = p(14)*p(96)
p(478) = p(8)*p(130) - p(378)
p(479) = p(18)*p(81) - p(437)
p(480) = p(34)*p(45) - p(378)
p(481) = p(14)*p(85)
p(482) = p(14)*p(86)
p(483) = p(14)*p(87)
p(484) = p(9)*p(102)
p(485) = p(24)*p(50) - p(391)
p(486) = p(13)*p(98) - p(476)
p(487) = p(6)*p(173) - p(391)
p(488) = p(14)*p(98)
p(489) = p(14)*p(99)
p(490) = p(9)*p(65)
p(491) = p(9)*p(66)
p(492) = p(9)*p(89)
p(493) = p(9)*p(90)
p(494) = p(9)*p(101)
p(495) = p(3)*p(181) - p(392)
p(496) = p(39)*p(41) - p(394)
p(497) = p(4)*p(178) - p(397)
p(498) = p(1)*p(278) - p(395) - p(497)
p(499) = p(1)*p(279) - p(397)
p(500) = p(1)*p(280) - p(398) - p(498)
p(501) = p(21)*p(69) - p(394)
p(502) = p(1)*p(282) - p(400) - p(500)
p(503) = p(11)*p(101) - p(392)
p(504) = p(1)*p(284) - p(404) - p(403)
p(505) = p(1)*p(285) - p(408)
p(506) = p(3)*p(183) - p(410)
p(507) = p(39)*p(43) - p(412)
p(508) = p(1)*p(287) - p(411) - p(507)
p(509) = p(2)*p(279) - p(406)
p(510) = p(1)*p(288) - p(413) - p(508)
p(511) = p(7)*p(153) - p(412)
p(512) = p(1)*p(290) - p(415) - p(510)
p(513) = p(34)*p(49) - p(410)
p(514) = p(6)*p(181) - p(409)
p(515) = p(1)*p(293) - p(421) - p(419)
p(516) = p(2)*p(285) - p(409)
p(517) = p(1)*p(295) - p(432) - p(431)
p(518) = p(1)*p(296) - p(434)
p(519) = p(3)*p(185) - p(438)
p(520) = p(39)*p(46) - p(440)
p(521) = p(1)*p(298) - p(439) - p(520)
p(522) = p(1)*p(299) - p(440)
p(523) = p(1)*p(300) - p(441) - p(521)
p(524) = p(21)*p(84) - p(438)
p(525) = p(6)*p(183) - p(428)
p(526) = p(1)*p(303) - p(446) - p(444)
p(527) = p(8)*p(150) - p(428)
p(528) = p(41)*p(45) - p(436)
p(529) = p(1)*p(306) - p(456) - p(454)
p(530) = p(2)*p(296) - p(436)
p(531) = p(4)*p(169) - p(388) - p(478)
p(532) = p(1)*p(310) - p(465)
p(533) = p(14)*p(82)
p(534) = p(3)*p(188) - p(468)
p(535) = p(2)*p(298) - p(447) - p(443)
p(536) = p(2)*p(299) - p(448)
p(537) = p(1)*p(313) - p(469) - p(535)
p(538) = p(7)*p(173) - p(468)
p(539) = p(6)*p(185) - p(451)
p(540) = p(1)*p(316) - p(473) - p(471)
p(541) = p(22)*p(73) - p(451)
p(542) = p(43)*p(45) - p(437)
p(543) = p(1)*p(319) - p(480) - p(478)
p(544) = p(8)*p(160) - p(437)
p(545) = p(14)*p(88)
p(546) = p(4)*p(187) - p(467)
p(547) = p(2)*p(309) - p(466) - p(546)
p(548) = p(2)*p(310) - p(467)
p(549) = p(14)*p(97)
p(550) = p(3)*p(191) - p(484)
p(551) = p(2)*p(313) - p(474) - p(471)
p(552) = p(1)*p(323) - p(484)
p(553) = p(6)*p(188) - p(476)
p(554) = p(1)*p(325) - p(487) - p(485)
p(555) = p(32)*p(50) - p(476)
p(556) = p(45)*p(46) - p(462)
p(557) = p(2)*p(319) - p(482) - p(547)
p(558) = p(22)*p(81) - p(462)
p(559) = p(14)*p(100)
p(560) = p(2)*p(322) - p(486) - p(485)
p(561) = p(2)*p(323) - p(487)
p(562) = p(6)*p(191) - p(488)
p(563) = p(2)*p(325) - p(489) - p(557)
p(564) = p(13)*p(102) - p(488)
p(565) = p(14)*p(102)
p(566) = p(3)*p(178) - p(492)
p(567) = p(7)*p(178) - p(490)
p(568) = p(39)*p(49) - p(491)
p(569) = p(3)*p(194) - p(494)
p(570) = p(1)*p(331) - p(495)
p(571) = p(2)*p(346) - p(570)
p(572) = p(1)*p(333) - p(506)
p(573) = p(8)*p(194) - p(572)
p(574) = p(1)*p(335) - p(519)
p(575) = p(22)*p(101) - p(574)
p(576) = p(1)*p(337) - p(534)
p(577) = p(49)*p(50) - p(576)
p(578) = p(6)*p(187) - p(549)
p(579) = p(1)*p(340) - p(550)
p(580) = p(21)*p(102) - p(579)
p(581) = p(8)*p(187) - p(533)
p(582) = p(1)*p(343) - p(560)
p(583) = p(7)*p(195) - p(582)
p(584) = p(45)*p(50) - p(545)
p(585) = p(2)*p(343) - p(562)
p(586) = p(1)*p(347) - p(585)
p(587) = p(6)*p(195) - p(565)
p(588) = p(1)*p(346) - p(569)
p(589) = p(2)*p(347) - p(587)
p(590) = p(9)*p(103)
p(591) = p(9)*p(106)
p(592) = p(9)*p(119)
p(593) = p(9)*p(128)
p(594) = p(9)*p(108)
p(595) = p(9)*p(109)
p(596) = p(9)*p(129)
p(597) = p(9)*p(132)
p(598) = p(9)*p(133)
p(599) = p(9)*p(161)
p(600) = p(9)*p(138)
p(601) = p(9)*p(139)
p(602) = p(9)*p(171)
p(603) = p(9)*p(177)
p(604) = p(9)*p(114)
p(605) = p(9)*p(115)
p(606) = p(9)*p(149)
p(607) = p(9)*p(104)
p(608) = p(9)*p(116)
p(609) = p(9)*p(105)
p(610) = p(9)*p(117)
p(611) = p(9)*p(118)
p(612) = p(9)*p(150)
p(613) = p(14)*p(178)
p(614) = p(14)*p(179)
p(615) = p(14)*p(180)
p(616) = p(9)*p(156)
p(617) = p(9)*p(122)
p(618) = p(9)*p(123)
p(619) = p(9)*p(157)
p(620) = p(9)*p(124)
p(621) = p(9)*p(107)
p(622) = p(9)*p(125)
p(623) = p(9)*p(126)
p(624) = p(9)*p(127)
p(625) = p(9)*p(158)
p(626) = p(14)*p(143)
p(627) = p(14)*p(144)
p(628) = p(14)*p(145)
p(629) = p(14)*p(146)
p(630) = p(14)*p(147)
p(631) = p(14)*p(148)
p(632) = p(9)*p(130)
p(633) = p(9)*p(159)
p(634) = p(9)*p(131)
p(635) = p(9)*p(160)
p(636) = p(14)*p(114)
p(637) = p(14)*p(115)
p(638) = p(14)*p(116)
p(639) = p(14)*p(117)
p(640) = p(14)*p(118)
p(641) = p(14)*p(106)
p(642) = p(9)*p(165)
p(643) = p(9)*p(135)
p(644) = p(9)*p(166)
p(645) = p(9)*p(136)
p(646) = p(9)*p(137)
p(647) = p(9)*p(167)
p(648) = p(14)*p(151)
p(649) = p(14)*p(152)
p(650) = p(14)*p(153)
p(651) = p(14)*p(154)
p(652) = p(14)*p(155)
p(653) = p(9)*p(168)
p(654) = p(9)*p(169)
p(655) = p(9)*p(170)
p(656) = p(14)*p(122)
p(657) = p(14)*p(123)
p(658) = p(14)*p(124)
p(659) = p(14)*p(125)
p(660) = p(14)*p(126)
p(661) = p(14)*p(127)
p(662) = p(14)*p(108)
p(663) = p(14)*p(109)
p(664) = p(9)*p(187)
p(665) = p(14)*p(130)
p(666) = p(14)*p(131)
p(667) = p(9)*p(174)
p(668) = p(9)*p(175)
p(669) = p(9)*p(176)
p(670) = p(14)*p(162)
p(671) = p(14)*p(163)
p(672) = p(14)*p(164)
p(673) = p(9)*p(190)
p(674) = p(14)*p(135)
p(675) = p(14)*p(136)
p(676) = p(14)*p(137)
p(677) = p(9)*p(193)
p(678) = p(14)*p(172)
p(679) = p(14)*p(173)
p(680) = p(9)*p(181)
p(681) = p(9)*p(143)
p(682) = p(9)*p(110)
p(683) = p(9)*p(144)
p(684) = p(9)*p(111)
p(685) = p(9)*p(112)
p(686) = p(9)*p(145)
p(687) = p(9)*p(113)
p(688) = p(9)*p(146)
p(689) = p(9)*p(147)
p(690) = p(9)*p(148)
p(691) = p(9)*p(182)
p(692) = p(1)*p(401) - p(604)
p(693) = p(21)*p(114) - p(609)
p(694) = p(24)*p(101) - p(612)
p(695) = p(3)*p(284) - p(606) - p(694)
p(696) = p(1)*p(404) - p(606) - p(695)
p(697) = p(1)*p(405) - p(608) - p(696)
p(698) = p(13)*p(178) - p(604)
p(699) = p(39)*p(73) - p(609)
p(700) = p(3)*p(285) - p(612)
p(701) = p(14)*p(194)
p(702) = p(9)*p(183)
p(703) = p(9)*p(151)
p(704) = p(9)*p(120)
p(705) = p(9)*p(152)
p(706) = p(9)*p(121)
p(707) = p(9)*p(153)
p(708) = p(9)*p(154)
p(709) = p(9)*p(155)
p(710) = p(9)*p(184)
p(711) = p(24)*p(91) - p(592)
p(712) = p(1)*p(416) - p(616) - p(711)
p(713) = p(2)*p(401) - p(613)
p(714) = p(1)*p(418) - p(618) - p(713)
p(715) = p(1)*p(419) - p(619) - p(714)
p(716) = p(13)*p(181) - p(701)
p(717) = p(2)*p(404) - p(615) - p(716)
p(718) = p(32)*p(91) - p(701)
p(719) = p(1)*p(421) - p(619) - p(717)
p(720) = p(41)*p(73) - p(701)
p(721) = p(6)*p(279) - p(613)
p(722) = p(16)*p(150) - p(701)
p(723) = p(1)*p(425) - p(624) - p(721)
p(724) = p(4)*p(285) - p(701)
p(725) = p(11)*p(150) - p(592)
p(726) = p(14)*p(181)
p(727) = p(14)*p(182)
p(728) = p(1)*p(430) - p(632)
p(729) = p(21)*p(130) - p(635)
p(730) = p(3)*p(295) - p(633) - p(729)
p(731) = p(1)*p(432) - p(633) - p(730)
p(732) = p(39)*p(81) - p(632)
p(733) = p(3)*p(296) - p(635)
p(734) = p(14)*p(149)
p(735) = p(14)*p(150)
p(736) = p(14)*p(119)
p(737) = p(9)*p(185)
p(738) = p(9)*p(162)
p(739) = p(9)*p(134)
p(740) = p(9)*p(163)
p(741) = p(9)*p(164)
p(742) = p(9)*p(186)
p(743) = p(24)*p(93) - p(593)
p(744) = p(18)*p(114) - p(595)
p(745) = p(1)*p(443) - p(643) - p(744)
p(746) = p(1)*p(444) - p(644) - p(745)
p(747) = p(13)*p(183) - p(726)
p(748) = p(2)*p(421) - p(631) - p(730)
p(749) = p(32)*p(93) - p(726)
p(750) = p(1)*p(446) - p(644) - p(748)
p(751) = p(43)*p(73) - p(726)
p(752) = p(13)*p(153) - p(595)
p(753) = p(18)*p(150) - p(726)
p(754) = p(34)*p(73) - p(593)
p(755) = p(14)*p(183)
p(756) = p(14)*p(184)
p(757) = p(16)*p(130) - p(598)
p(758) = p(2)*p(430) - p(636)
p(759) = p(1)*p(454) - p(654) - p(758)
p(760) = p(41)*p(81) - p(735)
p(761) = p(2)*p(432) - p(638) - p(760)
p(762) = p(16)*p(160) - p(735)
p(763) = p(45)*p(69) - p(636)
p(764) = p(4)*p(296) - p(735)
p(765) = p(11)*p(160) - p(598)
p(766) = p(14)*p(156)
p(767) = p(14)*p(157)
p(768) = p(14)*p(158)
p(769) = p(14)*p(128)
p(770) = p(14)*p(129)
p(771) = p(1)*p(464) - p(664)
p(772) = p(4)*p(262) - p(673) - p(661)
p(773) = p(3)*p(310) - p(664)
p(774) = p(14)*p(159)
p(775) = p(14)*p(160)
p(776) = p(14)*p(132)
p(777) = p(14)*p(133)
p(778) = p(9)*p(188)
p(779) = p(9)*p(172)
p(780) = p(9)*p(173)
p(781) = p(9)*p(189)
p(782) = p(24)*p(95) - p(600)
p(783) = p(22)*p(114) - p(650)
p(784) = p(1)*p(471) - p(668) - p(783)
p(785) = p(13)*p(185) - p(755)
p(786) = p(2)*p(446) - p(652) - p(761)
p(787) = p(32)*p(95) - p(755)
p(788) = p(6)*p(299) - p(650)
p(789) = p(46)*p(73) - p(755)
p(790) = p(32)*p(84) - p(600)
p(791) = p(14)*p(185)
p(792) = p(14)*p(186)
p(793) = p(2)*p(453) - p(656) - p(771)
p(794) = p(1)*p(478) - p(673) - p(793)
p(795) = p(43)*p(81) - p(736)
p(796) = p(2)*p(456) - p(659) - p(772)
p(797) = p(18)*p(160) - p(736)
p(798) = p(1)*p(480) - p(673) - p(796)
p(799) = p(14)*p(165)
p(800) = p(14)*p(166)
p(801) = p(14)*p(167)
p(802) = p(14)*p(138)
p(803) = p(14)*p(139)
p(804) = p(2)*p(464) - p(665)
p(805) = p(4)*p(310) - p(775)
p(806) = p(11)*p(187) - p(665)
p(807) = p(14)*p(168)
p(808) = p(14)*p(169)
p(809) = p(14)*p(170)
p(810) = p(9)*p(191)
p(811) = p(9)*p(192)
p(812) = p(24)*p(98) - p(603)
p(813) = p(1)*p(485) - p(677) - p(812)
p(814) = p(13)*p(188) - p(791)
p(815) = p(2)*p(473) - p(672) - p(796)
p(816) = p(32)*p(98) - p(791)
p(817) = p(13)*p(173) - p(603)
p(818) = p(14)*p(188)
p(819) = p(14)*p(189)
p(820) = p(22)*p(130) - p(663)
p(821) = p(46)*p(81) - p(769)
p(822) = p(45)*p(84) - p(663)
p(823) = p(14)*p(174)
p(824) = p(14)*p(175)
p(825) = p(14)*p(176)
p(826) = p(9)*p(195)
p(827) = p(24)*p(102) - p(679)
p(828) = p(13)*p(191) - p(818)
p(829) = p(6)*p(323) - p(679)
p(830) = p(14)*p(191)
p(831) = p(14)*p(192)
p(832) = p(9)*p(140)
p(833) = p(9)*p(141)
p(834) = p(9)*p(178)
p(835) = p(9)*p(142)
p(836) = p(9)*p(179)
p(837) = p(9)*p(180)
p(838) = p(9)*p(194)
p(839) = p(3)*p(331) - p(680)
p(840) = p(39)*p(91) - p(682)
p(841) = p(16)*p(178) - p(685)
p(842) = p(1)*p(497) - p(683) - p(841)
p(843) = p(2)*p(566) - p(842)
p(844) = p(1)*p(498) - p(686) - p(842)
p(845) = p(7)*p(279) - p(685)
p(846) = p(1)*p(500) - p(689) - p(844)
p(847) = p(49)*p(69) - p(682)
p(848) = p(1)*p(502) - p(691) - p(846)
p(849) = p(11)*p(194) - p(680)
p(850) = p(1)*p(504) - p(695) - p(694)
p(851) = p(1)*p(505) - p(700)
p(852) = p(3)*p(333) - p(702)
p(853) = p(39)*p(93) - p(704)
p(854) = p(18)*p(178) - p(707)
p(855) = p(1)*p(508) - p(705) - p(854)
p(856) = p(1)*p(509) - p(707)
p(857) = p(1)*p(510) - p(708) - p(855)
p(858) = p(21)*p(153) - p(704)
p(859) = p(1)*p(512) - p(710) - p(857)
p(860) = p(34)*p(101) - p(702)
p(861) = p(6)*p(331) - p(701)
p(862) = p(1)*p(515) - p(717) - p(715)
p(863) = p(2)*p(505) - p(701)
p(864) = p(1)*p(517) - p(730) - p(729)
p(865) = p(1)*p(518) - p(733)
p(866) = p(3)*p(335) - p(737)
p(867) = p(39)*p(95) - p(739)
p(868) = p(1)*p(520) - p(738) - p(867)
p(869) = p(2)*p(509) - p(721)
p(870) = p(1)*p(521) - p(740) - p(868)
p(871) = p(7)*p(299) - p(739)
p(872) = p(1)*p(523) - p(742) - p(870)
p(873) = p(49)*p(84) - p(737)
p(874) = p(6)*p(333) - p(726)
p(875) = p(1)*p(526) - p(748) - p(746)
p(876) = p(8)*p(285) - p(726)
p(877) = p(45)*p(91) - p(735)
p(878) = p(1)*p(529) - p(761) - p(759)
p(879) = p(2)*p(518) - p(735)
p(880) = p(1)*p(531) - p(772) - p(771)
p(881) = p(1)*p(532) - p(773)
p(882) = p(14)*p(161)
p(883) = p(3)*p(337) - p(778)
p(884) = p(39)*p(98) - p(780)
p(885) = p(1)*p(535) - p(779) - p(884)
p(886) = p(1)*p(536) - p(780)
p(887) = p(1)*p(537) - p(781) - p(885)
p(888) = p(21)*p(173) - p(778)
p(889) = p(6)*p(335) - p(755)
p(890) = p(1)*p(540) - p(786) - p(784)
p(891) = p(22)*p(150) - p(755)
p(892) = p(45)*p(93) - p(736)
p(893) = p(1)*p(543) - p(796) - p(794)
p(894) = p(8)*p(296) - p(736)
p(895) = p(16)*p(187) - p(775)
p(896) = p(1)*p(547) - p(806) - p(804)
p(897) = p(2)*p(532) - p(775)
p(898) = p(14)*p(171)
p(899) = p(6)*p(309) - p(808) - p(807)
p(900) = p(1)*p(578) - p(899)
p(901) = p(14)*p(187)
p(902) = p(3)*p(340) - p(810)
p(903) = p(2)*p(535) - p(787) - p(783)
p(904) = p(2)*p(536) - p(788)
p(905) = p(1)*p(551) - p(811) - p(903)
p(906) = p(7)*p(323) - p(810)
p(907) = p(6)*p(337) - p(791)
p(908) = p(1)*p(554) - p(815) - p(813)
p(909) = p(50)*p(73) - p(791)
p(910) = p(45)*p(95) - p(769)
p(911) = p(1)*p(557) - p(822) - p(820)
p(912) = p(22)*p(160) - p(769)
p(913) = p(14)*p(177)
p(914) = p(18)*p(187) - p(777)
p(915) = p(2)*p(547) - p(808) - p(899)
p(916) = p(8)*p(310) - p(777)
p(917) = p(14)*p(190)
p(918) = p(3)*p(343) - p(826)
p(919) = p(2)*p(551) - p(816) - p(813)
p(920) = p(1)*p(561) - p(826)
p(921) = p(6)*p(340) - p(818)
p(922) = p(1)*p(563) - p(829) - p(827)
p(923) = p(32)*p(102) - p(818)
p(924) = p(45)*p(98) - p(802)
p(925) = p(2)*p(557) - p(824) - p(915)
p(926) = p(50)*p(81) - p(802)
p(927) = p(14)*p(193)
p(928) = p(2)*p(560) - p(828) - p(827)
p(929) = p(2)*p(561) - p(829)
p(930) = p(6)*p(343) - p(830)
p(931) = p(2)*p(563) - p(831) - p(925)
p(932) = p(13)*p(195) - p(830)
p(933) = p(14)*p(195)
p(934) = p(1)*p(566) - p(834)
p(935) = p(1)*p(567) - p(836) - p(934)
p(936) = p(39)*p(101) - p(835)
p(937) = p(3)*p(346) - p(838)
p(938) = p(1)*p(570) - p(839)
p(939) = p(2)*p(588) - p(938)
p(940) = p(1)*p(572) - p(852)
p(941) = p(8)*p(346) - p(940)
p(942) = p(1)*p(574) - p(866)
p(943) = p(22)*p(194) - p(942)
p(944) = p(1)*p(576) - p(883)
p(945) = p(50)*p(101) - p(944)
p(946) = p(1)*p(579) - p(902)
p(947) = p(49)*p(102) - p(946)
p(948) = p(2)*p(578) - p(901)
p(949) = p(1)*p(582) - p(918)
p(950) = p(21)*p(195) - p(949)
p(951) = p(2)*p(581) - p(917) - p(948)
p(952) = p(1)*p(585) - p(928)
p(953) = p(7)*p(347) - p(952)
p(954) = p(45)*p(102) - p(913)
p(955) = p(2)*p(585) - p(930)
p(956) = p(1)*p(589) - p(955)
p(957) = p(6)*p(347) - p(933)
p(958) = p(1)*p(588) - p(937)
p(959) = p(2)*p(589) - p(957)
p(960) = p(9)*p(196)
p(961) = p(9)*p(202)
p(962) = p(9)*p(203)
p(963) = p(9)*p(228)
p(964) = p(9)*p(242)
p(965) = p(9)*p(207)
p(966) = p(9)*p(197)
p(967) = p(9)*p(208)
p(968) = p(9)*p(209)
p(969) = p(9)*p(243)
p(970) = p(9)*p(211)
p(971) = p(9)*p(247)
p(972) = p(9)*p(212)
p(973) = p(9)*p(248)
p(974) = p(9)*p(249)
p(975) = p(9)*p(258)
p(976) = p(9)*p(214)
p(977) = p(9)*p(215)
p(978) = p(9)*p(259)
p(979) = p(9)*p(263)
p(980) = p(9)*p(264)
p(981) = p(9)*p(265)
p(982) = p(9)*p(308)
p(983) = p(9)*p(311)
p(984) = p(9)*p(270)
p(985) = p(9)*p(271)
p(986) = p(9)*p(321)
p(987) = p(9)*p(327)
p(988) = p(9)*p(222)
p(989) = p(9)*p(223)
p(990) = p(9)*p(284)
p(991) = p(9)*p(198)
p(992) = p(9)*p(199)
p(993) = p(9)*p(224)
p(994) = p(9)*p(200)
p(995) = p(9)*p(225)
p(996) = p(9)*p(201)
p(997) = p(9)*p(226)
p(998) = p(9)*p(227)
p(999) = p(9)*p(285)
p(1000) = p(14)*p(328)
p(1001) = p(14)*p(329)
p(1002) = p(14)*p(330)
p(1003) = p(9)*p(292)
p(1004) = p(9)*p(233)
p(1005) = p(9)*p(234)
p(1006) = p(9)*p(235)
p(1007) = p(9)*p(293)
p(1008) = p(9)*p(236)
p(1009) = p(9)*p(204)
p(1010) = p(9)*p(205)
p(1011) = p(9)*p(237)
p(1012) = p(9)*p(238)
p(1013) = p(9)*p(206)
p(1014) = p(9)*p(239)
p(1015) = p(9)*p(240)
p(1016) = p(9)*p(241)
p(1017) = p(9)*p(294)
p(1018) = p(14)*p(276)
p(1019) = p(14)*p(277)
p(1020) = p(14)*p(278)
p(1021) = p(14)*p(279)
p(1022) = p(14)*p(280)
p(1023) = p(14)*p(281)
p(1024) = p(14)*p(282)
p(1025) = p(14)*p(283)
p(1026) = p(9)*p(244)
p(1027) = p(9)*p(295)
p(1028) = p(9)*p(210)
p(1029) = p(9)*p(245)
p(1030) = p(9)*p(246)
p(1031) = p(9)*p(296)
p(1032) = p(14)*p(222)
p(1033) = p(14)*p(223)
p(1034) = p(14)*p(224)
p(1035) = p(14)*p(225)
p(1036) = p(14)*p(226)
p(1037) = p(14)*p(227)
p(1038) = p(14)*p(202)
p(1039) = p(14)*p(203)
p(1040) = p(9)*p(302)
p(1041) = p(9)*p(252)
p(1042) = p(9)*p(253)
p(1043) = p(9)*p(303)
p(1044) = p(9)*p(254)
p(1045) = p(9)*p(213)
p(1046) = p(9)*p(255)
p(1047) = p(9)*p(256)
p(1048) = p(9)*p(257)
p(1049) = p(9)*p(304)
p(1050) = p(14)*p(286)
p(1051) = p(14)*p(287)
p(1052) = p(14)*p(288)
p(1053) = p(14)*p(289)
p(1054) = p(14)*p(290)
p(1055) = p(14)*p(291)
p(1056) = p(9)*p(305)
p(1057) = p(9)*p(260)
p(1058) = p(9)*p(306)
p(1059) = p(9)*p(261)
p(1060) = p(9)*p(262)
p(1061) = p(9)*p(307)
p(1062) = p(14)*p(233)
p(1063) = p(14)*p(234)
p(1064) = p(14)*p(235)
p(1065) = p(14)*p(236)
p(1066) = p(14)*p(237)
p(1067) = p(14)*p(238)
p(1068) = p(14)*p(239)
p(1069) = p(14)*p(240)
p(1070) = p(14)*p(241)
p(1071) = p(14)*p(207)
p(1072) = p(14)*p(208)
p(1073) = p(14)*p(209)
p(1074) = p(9)*p(309)
p(1075) = p(9)*p(310)
p(1076) = p(14)*p(244)
p(1077) = p(14)*p(245)
p(1078) = p(14)*p(246)
p(1079) = p(14)*p(211)
p(1080) = p(14)*p(212)
p(1081) = p(9)*p(315)
p(1082) = p(9)*p(267)
p(1083) = p(9)*p(316)
p(1084) = p(9)*p(268)
p(1085) = p(9)*p(269)
p(1086) = p(9)*p(317)
p(1087) = p(14)*p(297)
p(1088) = p(14)*p(298)
p(1089) = p(14)*p(299)
p(1090) = p(14)*p(300)
p(1091) = p(14)*p(301)
p(1092) = p(9)*p(318)
p(1093) = p(9)*p(319)
p(1094) = p(9)*p(320)
p(1095) = p(14)*p(252)
p(1096) = p(14)*p(253)
p(1097) = p(14)*p(254)
p(1098) = p(14)*p(255)
p(1099) = p(14)*p(256)
p(1100) = p(14)*p(257)
p(1101) = p(14)*p(214)
p(1102) = p(14)*p(215)
p(1103) = p(9)*p(339)
p(1104) = p(14)*p(260)
p(1105) = p(14)*p(261)
p(1106) = p(14)*p(262)
p(1107) = p(9)*p(324)
p(1108) = p(9)*p(325)
p(1109) = p(9)*p(326)
p(1110) = p(14)*p(312)
p(1111) = p(14)*p(313)
p(1112) = p(14)*p(314)
p(1113) = p(9)*p(342)
p(1114) = p(14)*p(267)
p(1115) = p(14)*p(268)
p(1116) = p(14)*p(269)
p(1117) = p(9)*p(345)
p(1118) = p(14)*p(322)
p(1119) = p(14)*p(323)
p(1120) = p(9)*p(331)
p(1121) = p(9)*p(276)
p(1122) = p(9)*p(216)
p(1123) = p(9)*p(277)
p(1124) = p(9)*p(217)
p(1125) = p(9)*p(218)
p(1126) = p(9)*p(278)
p(1127) = p(9)*p(219)
p(1128) = p(9)*p(220)
p(1129) = p(9)*p(279)
p(1130) = p(9)*p(280)
p(1131) = p(9)*p(221)
p(1132) = p(9)*p(281)
p(1133) = p(9)*p(282)
p(1134) = p(9)*p(283)
p(1135) = p(9)*p(332)
p(1136) = p(3)*p(401) - p(988)
p(1137) = p(7)*p(401) - p(991)
p(1138) = p(49)*p(114) - p(996)
p(1139) = p(24)*p(194) - p(999)
p(1140) = p(3)*p(504) - p(990) - p(1139)
p(1141) = p(1)*p(695) - p(990) - p(1140)
p(1142) = p(1)*p(696) - p(993) - p(1141)
p(1143) = p(6)*p(566) - p(1136)
p(1144) = p(32)*p(178) - p(991)
p(1145) = p(39)*p(150) - p(996)
p(1146) = p(3)*p(505) - p(999)
p(1147) = p(14)*p(346)
p(1148) = p(9)*p(333)
p(1149) = p(9)*p(286)
p(1150) = p(9)*p(229)
p(1151) = p(9)*p(287)
p(1152) = p(9)*p(230)
p(1153) = p(9)*p(231)
p(1154) = p(9)*p(288)
p(1155) = p(9)*p(232)
p(1156) = p(9)*p(289)
p(1157) = p(9)*p(290)
p(1158) = p(9)*p(291)
p(1159) = p(9)*p(334)
p(1160) = p(24)*p(181) - p(963)
p(1161) = p(41)*p(114) - p(960)
p(1162) = p(4)*p(401) - p(961)
p(1163) = p(1)*p(713) - p(1005) - p(1162)
p(1164) = p(1)*p(714) - p(1006) - p(1163)
p(1165) = p(1)*p(715) - p(1007) - p(1164)
p(1166) = p(13)*p(331) - p(1147)
p(1167) = p(2)*p(695) - p(1002) - p(1166)
p(1168) = p(32)*p(181) - p(1147)
p(1169) = p(1)*p(717) - p(1007) - p(1167)
p(1170) = p(73)*p(91) - p(1147)
p(1171) = p(1)*p(719) - p(1011) - p(1169)
p(1172) = p(41)*p(150) - p(1147)
p(1173) = p(13)*p(279) - p(961)
p(1174) = p(16)*p(285) - p(1147)
p(1175) = p(69)*p(73) - p(960)
p(1176) = p(4)*p(505) - p(1147)
p(1177) = p(11)*p(285) - p(963)
p(1178) = p(14)*p(331)
p(1179) = p(14)*p(332)
p(1180) = p(3)*p(430) - p(1026)
p(1181) = p(7)*p(430) - p(1028)
p(1182) = p(49)*p(130) - p(1031)
p(1183) = p(3)*p(517) - p(1027) - p(1182)
p(1184) = p(1)*p(730) - p(1027) - p(1183)
p(1185) = p(45)*p(178) - p(1180)
p(1186) = p(39)*p(160) - p(1028)
p(1187) = p(3)*p(518) - p(1031)
p(1188) = p(14)*p(284)
p(1189) = p(14)*p(285)
p(1190) = p(14)*p(228)
p(1191) = p(9)*p(335)
p(1192) = p(9)*p(297)
p(1193) = p(9)*p(250)
p(1194) = p(9)*p(298)
p(1195) = p(9)*p(251)
p(1196) = p(9)*p(299)
p(1197) = p(9)*p(300)
p(1198) = p(9)*p(301)
p(1199) = p(9)*p(336)
p(1200) = p(24)*p(183) - p(964)
p(1201) = p(43)*p(114) - p(966)
p(1202) = p(8)*p(401) - p(1021)
p(1203) = p(1)*p(745) - p(1042) - p(1202)
p(1204) = p(1)*p(746) - p(1043) - p(1203)
p(1205) = p(13)*p(333) - p(1178)
p(1206) = p(2)*p(717) - p(1025) - p(1183)
p(1207) = p(32)*p(183) - p(1178)
p(1208) = p(1)*p(748) - p(1043) - p(1206)
p(1209) = p(73)*p(93) - p(1178)
p(1210) = p(6)*p(509) - p(1021)
p(1211) = p(43)*p(150) - p(1178)
p(1212) = p(32)*p(153) - p(966)
p(1213) = p(18)*p(285) - p(1178)
p(1214) = p(34)*p(150) - p(964)
p(1215) = p(14)*p(333)
p(1216) = p(14)*p(334)
p(1217) = p(41)*p(130) - p(973)
p(1218) = p(4)*p(430) - p(970)
p(1219) = p(1)*p(758) - p(1057) - p(1218)
p(1220) = p(1)*p(759) - p(1058) - p(1219)
p(1221) = p(81)*p(91) - p(1189)
p(1222) = p(2)*p(730) - p(1034) - p(1221)
p(1223) = p(41)*p(160) - p(1189)
p(1224) = p(1)*p(761) - p(1058) - p(1222)
p(1225) = p(16)*p(296) - p(1189)
p(1226) = p(69)*p(81) - p(970)
p(1227) = p(4)*p(518) - p(1189)
p(1228) = p(11)*p(296) - p(973)
p(1229) = p(14)*p(292)
p(1230) = p(14)*p(293)
p(1231) = p(14)*p(294)
p(1232) = p(14)*p(242)
p(1233) = p(14)*p(243)
p(1234) = p(3)*p(464) - p(1074)
p(1235) = p(7)*p(464) - p(1075)
p(1236) = p(3)*p(531) - p(1074) - p(1235)
p(1237) = p(39)*p(187) - p(1234)
p(1238) = p(3)*p(532) - p(1075)
p(1239) = p(14)*p(295)
p(1240) = p(14)*p(296)
p(1241) = p(14)*p(247)
p(1242) = p(14)*p(248)
p(1243) = p(14)*p(249)
p(1244) = p(9)*p(337)
p(1245) = p(9)*p(312)
p(1246) = p(9)*p(266)
p(1247) = p(9)*p(313)
p(1248) = p(9)*p(314)
p(1249) = p(9)*p(338)
p(1250) = p(24)*p(185) - p(975)
p(1251) = p(46)*p(114) - p(977)
p(1252) = p(1)*p(783) - p(1082) - p(1251)
p(1253) = p(1)*p(784) - p(1083) - p(1252)
p(1254) = p(13)*p(335) - p(1215)
p(1255) = p(2)*p(748) - p(1055) - p(1222)
p(1256) = p(32)*p(185) - p(1215)
p(1257) = p(1)*p(786) - p(1083) - p(1255)
p(1258) = p(73)*p(95) - p(1215)
p(1259) = p(13)*p(299) - p(977)
p(1260) = p(46)*p(150) - p(1215)
p(1261) = p(73)*p(84) - p(975)
p(1262) = p(14)*p(335)
p(1263) = p(14)*p(336)
p(1264) = p(43)*p(130) - p(974)
p(1265) = p(8)*p(430) - p(1038)
p(1266) = p(1)*p(794) - p(1093) - p(1265)
p(1267) = p(81)*p(93) - p(1190)
p(1268) = p(2)*p(761) - p(1066) - p(1236)
p(1269) = p(43)*p(160) - p(1190)
p(1270) = p(45)*p(153) - p(1038)
p(1271) = p(18)*p(296) - p(1190)
p(1272) = p(34)*p(160) - p(974)
p(1273) = p(14)*p(302)
p(1274) = p(14)*p(303)
p(1275) = p(14)*p(304)
p(1276) = p(14)*p(258)
p(1277) = p(14)*p(259)
p(1278) = p(4)*p(464) - p(983)
p(1279) = p(1)*p(804) - p(1103) - p(1278)
p(1280) = p(16)*p(310) - p(1240)
p(1281) = p(2)*p(772) - p(1077) - p(1280)
p(1282) = p(4)*p(532) - p(1240)
p(1283) = p(11)*p(310) - p(983)
p(1284) = p(14)*p(305)
p(1285) = p(14)*p(306)
p(1286) = p(14)*p(307)
p(1287) = p(14)*p(263)
p(1288) = p(14)*p(264)
p(1289) = p(14)*p(265)
p(1290) = p(6)*p(464) - p(1104)
p(1291) = p(3)*p(578) - p(1290)
p(1292) = p(14)*p(309)
p(1293) = p(14)*p(310)
p(1294) = p(9)*p(340)
p(1295) = p(9)*p(322)
p(1296) = p(9)*p(323)
p(1297) = p(9)*p(341)
p(1298) = p(24)*p(188) - p(984)
p(1299) = p(50)*p(114) - p(1089)
p(1300) = p(1)*p(813) - p(1108) - p(1299)
p(1301) = p(13)*p(337) - p(1262)
p(1302) = p(2)*p(786) - p(1091) - p(1268)
p(1303) = p(32)*p(188) - p(1262)
p(1304) = p(6)*p(536) - p(1089)
p(1305) = p(73)*p(98) - p(1262)
p(1306) = p(32)*p(173) - p(984)
p(1307) = p(14)*p(337)
p(1308) = p(14)*p(338)
p(1309) = p(46)*p(130) - p(982)
p(1310) = p(1)*p(820) - p(1113) - p(1309)
p(1311) = p(81)*p(95) - p(1232)
p(1312) = p(2)*p(796) - p(1098) - p(1281)
p(1313) = p(46)*p(160) - p(1232)
p(1314) = p(81)*p(84) - p(982)
p(1315) = p(14)*p(315)
p(1316) = p(14)*p(316)
p(1317) = p(14)*p(317)
p(1318) = p(14)*p(270)
p(1319) = p(14)*p(271)
p(1320) = p(8)*p(464) - p(1079)
p(1321) = p(18)*p(310) - p(1242)
p(1322) = p(34)*p(187) - p(1079)
p(1323) = p(14)*p(318)
p(1324) = p(14)*p(319)
p(1325) = p(14)*p(320)
p(1326) = p(9)*p(343)
p(1327) = p(9)*p(344)
p(1328) = p(24)*p(191) - p(987)
p(1329) = p(1)*p(827) - p(1117) - p(1328)
p(1330) = p(13)*p(340) - p(1307)
p(1331) = p(2)*p(815) - p(1112) - p(1312)
p(1332) = p(32)*p(191) - p(1307)
p(1333) = p(13)*p(323) - p(987)
p(1334) = p(14)*p(340)
p(1335) = p(14)*p(341)
p(1336) = p(50)*p(130) - p(1102)
p(1337) = p(81)*p(98) - p(1276)
p(1338) = p(45)*p(173) - p(1102)
p(1339) = p(14)*p(324)
p(1340) = p(14)*p(325)
p(1341) = p(14)*p(326)
p(1342) = p(9)*p(347)
p(1343) = p(24)*p(195) - p(1119)
p(1344) = p(13)*p(343) - p(1334)
p(1345) = p(6)*p(561) - p(1119)
p(1346) = p(14)*p(343)
p(1347) = p(14)*p(344)
p(1348) = p(9)*p(272)
p(1349) = p(9)*p(273)
p(1350) = p(9)*p(274)
p(1351) = p(9)*p(328)
p(1352) = p(9)*p(275)
p(1353) = p(9)*p(329)
p(1354) = p(9)*p(330)
p(1355) = p(9)*p(346)
p(1356) = p(3)*p(570) - p(1120)
p(1357) = p(39)*p(181) - p(1122)
p(1358) = p(41)*p(178) - p(1125)
p(1359) = p(4)*p(566) - p(1129)
p(1360) = p(1)*p(842) - p(1126) - p(1359)
p(1361) = p(1)*p(843) - p(1129)
p(1362) = p(1)*p(844) - p(1130) - p(1360)
p(1363) = p(21)*p(279) - p(1125)
p(1364) = p(1)*p(846) - p(1133) - p(1362)
p(1365) = p(69)*p(101) - p(1122)
p(1366) = p(1)*p(848) - p(1135) - p(1364)
p(1367) = p(11)*p(346) - p(1120)
p(1368) = p(1)*p(850) - p(1140) - p(1139)
p(1369) = p(1)*p(851) - p(1146)
p(1370) = p(3)*p(572) - p(1148)
p(1371) = p(39)*p(183) - p(1150)
p(1372) = p(43)*p(178) - p(1153)
p(1373) = p(1)*p(854) - p(1151) - p(1372)
p(1374) = p(2)*p(843) - p(1143)
p(1375) = p(1)*p(855) - p(1154) - p(1373)
p(1376) = p(7)*p(509) - p(1153)
p(1377) = p(1)*p(857) - p(1157) - p(1375)
p(1378) = p(49)*p(153) - p(1150)
p(1379) = p(1)*p(859) - p(1159) - p(1377)
p(1380) = p(34)*p(194) - p(1148)
p(1381) = p(6)*p(570) - p(1147)
p(1382) = p(1)*p(862) - p(1167) - p(1165)
p(1383) = p(2)*p(851) - p(1147)
p(1384) = p(1)*p(864) - p(1183) - p(1182)
p(1385) = p(1)*p(865) - p(1187)
p(1386) = p(3)*p(574) - p(1191)
p(1387) = p(39)*p(185) - p(1193)
p(1388) = p(46)*p(178) - p(1196)
p(1389) = p(1)*p(868) - p(1194) - p(1388)
p(1390) = p(1)*p(869) - p(1196)
p(1391) = p(1)*p(870) - p(1197) - p(1389)
p(1392) = p(21)*p(299) - p(1193)
p(1393) = p(1)*p(872) - p(1199) - p(1391)
p(1394) = p(84)*p(101) - p(1191)
p(1395) = p(6)*p(572) - p(1178)
p(1396) = p(1)*p(875) - p(1206) - p(1204)
p(1397) = p(8)*p(505) - p(1178)
p(1398) = p(45)*p(181) - p(1189)
p(1399) = p(1)*p(878) - p(1222) - p(1220)
p(1400) = p(2)*p(865) - p(1189)
p(1401) = p(1)*p(880) - p(1236) - p(1235)
p(1402) = p(1)*p(881) - p(1238)
p(1403) = p(3)*p(576) - p(1244)
p(1404) = p(39)*p(188) - p(1246)
p(1405) = p(1)*p(884) - p(1245) - p(1404)
p(1406) = p(2)*p(869) - p(1210)
p(1407) = p(1)*p(885) - p(1247) - p(1405)
p(1408) = p(7)*p(536) - p(1246)
p(1409) = p(1)*p(887) - p(1249) - p(1407)
p(1410) = p(49)*p(173) - p(1244)
p(1411) = p(6)*p(574) - p(1215)
p(1412) = p(1)*p(890) - p(1255) - p(1253)
p(1413) = p(22)*p(285) - p(1215)
p(1414) = p(45)*p(183) - p(1190)
p(1415) = p(1)*p(893) - p(1268) - p(1266)
p(1416) = p(8)*p(518) - p(1190)
p(1417) = p(41)*p(187) - p(1240)
p(1418) = p(1)*p(896) - p(1281) - p(1279)
p(1419) = p(2)*p(881) - p(1240)
p(1420) = p(14)*p(308)
p(1421) = p(4)*p(547) - p(1106) - p(1320)
p(1422) = p(1)*p(900) - p(1291)
p(1423) = p(14)*p(311)
p(1424) = p(3)*p(579) - p(1294)
p(1425) = p(39)*p(191) - p(1296)
p(1426) = p(1)*p(903) - p(1295) - p(1425)
p(1427) = p(1)*p(904) - p(1296)
p(1428) = p(1)*p(905) - p(1297) - p(1426)
p(1429) = p(21)*p(323) - p(1294)
p(1430) = p(6)*p(576) - p(1262)
p(1431) = p(1)*p(908) - p(1302) - p(1300)
p(1432) = p(50)*p(150) - p(1262)
p(1433) = p(45)*p(185) - p(1232)
p(1434) = p(1)*p(911) - p(1312) - p(1310)
p(1435) = p(22)*p(296) - p(1232)
p(1436) = p(43)*p(187) - p(1242)
p(1437) = p(1)*p(915) - p(1322) - p(1320)
p(1438) = p(8)*p(532) - p(1242)
p(1439) = p(14)*p(321)
p(1440) = p(4)*p(578) - p(1293)
p(1441) = p(2)*p(899) - p(1292) - p(1440)
p(1442) = p(2)*p(900) - p(1293)
p(1443) = p(14)*p(339)
p(1444) = p(3)*p(582) - p(1326)
p(1445) = p(2)*p(903) - p(1303) - p(1299)
p(1446) = p(2)*p(904) - p(1304)
p(1447) = p(1)*p(919) - p(1327) - p(1445)
p(1448) = p(7)*p(561) - p(1326)
p(1449) = p(6)*p(579) - p(1307)
p(1450) = p(1)*p(922) - p(1331) - p(1329)
p(1451) = p(73)*p(102) - p(1307)
p(1452) = p(45)*p(188) - p(1276)
p(1453) = p(1)*p(925) - p(1338) - p(1336)
p(1454) = p(50)*p(160) - p(1276)
p(1455) = p(14)*p(327)
p(1456) = p(46)*p(187) - p(1243)
p(1457) = p(2)*p(915) - p(1324) - p(1441)
p(1458) = p(22)*p(310) - p(1243)
p(1459) = p(14)*p(342)
p(1460) = p(3)*p(585) - p(1342)
p(1461) = p(2)*p(919) - p(1332) - p(1329)
p(1462) = p(1)*p(929) - p(1342)
p(1463) = p(6)*p(582) - p(1334)
p(1464) = p(1)*p(931) - p(1345) - p(1343)
p(1465) = p(32)*p(195) - p(1334)
p(1466) = p(45)*p(191) - p(1318)
p(1467) = p(2)*p(925) - p(1340) - p(1457)
p(1468) = p(81)*p(102) - p(1318)
p(1469) = p(14)*p(345)
p(1470) = p(2)*p(928) - p(1344) - p(1343)
p(1471) = p(2)*p(929) - p(1345)
p(1472) = p(6)*p(585) - p(1346)
p(1473) = p(2)*p(931) - p(1347) - p(1467)
p(1474) = p(13)*p(347) - p(1346)
p(1475) = p(14)*p(347)
p(1476) = p(3)*p(566) - p(1351)
p(1477) = p(7)*p(566) - p(1349)
p(1478) = p(49)*p(178) - p(1348)
p(1479) = p(39)*p(194) - p(1352)
p(1480) = p(3)*p(588) - p(1355)
p(1481) = p(1)*p(938) - p(1356)
p(1482) = p(2)*p(958) - p(1481)
p(1483) = p(1)*p(940) - p(1370)
p(1484) = p(8)*p(588) - p(1483)
p(1485) = p(1)*p(942) - p(1386)
p(1486) = p(22)*p(346) - p(1485)
p(1487) = p(1)*p(944) - p(1403)
p(1488) = p(50)*p(194) - p(1487)
p(1489) = p(1)*p(946) - p(1424)
p(1490) = p(101)*p(102) - p(1489)
p(1491) = p(6)*p(578) - p(1443)
p(1492) = p(1)*p(949) - p(1444)
p(1493) = p(49)*p(195) - p(1492)
p(1494) = p(8)*p(578) - p(1423)
p(1495) = p(1)*p(952) - p(1460)
p(1496) = p(21)*p(347) - p(1495)
p(1497) = p(50)*p(187) - p(1420)
p(1498) = p(1)*p(955) - p(1470)
p(1499) = p(7)*p(589) - p(1498)
p(1500) = p(45)*p(195) - p(1455)
p(1501) = p(2)*p(955) - p(1472)
p(1502) = p(1)*p(959) - p(1501)
p(1503) = p(6)*p(589) - p(1475)
p(1504) = p(1)*p(958) - p(1480)
p(1505) = p(2)*p(959) - p(1503)


do i=1,6
dm(0) = 0.0D0
dm(1) = dmsdr(i,4)
dm(2) = dmsdr(i,2)
dm(3) = dmsdr(i,1)
dm(4) = dmsdr(i,6)
dm(5) = dmsdr(i,5)
dm(6) = dmsdr(i,3)
dm(7) = dm(1)*rm(2) + rm(1)*dm(2)
dm(8) = dm(1)*rm(3) + rm(1)*dm(3)
dm(9) = dm(2)*rm(3) + rm(2)*dm(3)
dm(10) = dm(3)*rm(4) + rm(3)*dm(4)
dm(11) = dm(2)*rm(5) + rm(2)*dm(5)
dm(12) = dm(1)*rm(6) + rm(1)*dm(6)
dm(13) = dm(4)*rm(5) + rm(4)*dm(5)
dm(14) = dm(4)*rm(6) + rm(4)*dm(6)
dm(15) = dm(5)*rm(6) + rm(5)*dm(6)
dm(16) = dm(1)*rm(9) + rm(1)*dm(9)
dm(17) = dm(1)*rm(10) + rm(1)*dm(10)
dm(18) = dm(2)*rm(10) + rm(2)*dm(10)
dm(19) = dm(1)*rm(11) + rm(1)*dm(11)
dm(20) = dm(3)*rm(11) + rm(3)*dm(11)
dm(21) = dm(2)*rm(12) + rm(2)*dm(12)
dm(22) = dm(3)*rm(12) + rm(3)*dm(12)
dm(23) = dm(2)*rm(13) + rm(2)*dm(13)
dm(24) = dm(3)*rm(13) + rm(3)*dm(13)
dm(25) = dm(1)*rm(14) + rm(1)*dm(14)
dm(26) = dm(3)*rm(14) + rm(3)*dm(14)
dm(27) = dm(1)*rm(15) + rm(1)*dm(15)
dm(28) = dm(2)*rm(15) + rm(2)*dm(15)
dm(29) = dm(4)*rm(15) + rm(4)*dm(15)
dm(30) = dm(2)*rm(24) + rm(2)*dm(24)
dm(31) = dm(1)*rm(26) + rm(1)*dm(26)
dm(32) = dm(1)*rm(28) + rm(1)*dm(28)

dp(0) = dm(0)
dp(1) = dm(1) + dm(2) + dm(3)
dp(2) = dm(4) + dm(5) + dm(6)
dp(3) = dm(7) + dm(8) + dm(9)
dp(4) = dm(10) + dm(11) + dm(12)
dp(5) = dp(1)*p(2) + p(1)*dp(2) - dp(4)
dp(6) = dm(13) + dm(14) + dm(15)
dp(7) = dp(1)*p(1) + p(1)*dp(1) - dp(3) - dp(3)
dp(8) = dp(2)*p(2) + p(2)*dp(2) - dp(6) - dp(6)
dp(9) = dm(16)
dp(10) = dm(17) + dm(18) + dm(19) + dm(20) + dm(21) + dm(22)
dp(11) = dp(2)*p(3) + p(2)*dp(3) - dp(10)
dp(12) = dm(23) + dm(24) + dm(25) + dm(26) + dm(27) + dm(28)
dp(13) = dp(1)*p(6) + p(1)*dp(6) - dp(12)
dp(14) = dm(29)
dp(15) = dp(1)*p(3) + p(1)*dp(3) - dp(9) - dp(9) - dp(9)
dp(16) = dp(1)*p(4) + p(1)*dp(4) - dp(10)
dp(17) = dp(2)*p(7) + p(2)*dp(7) - dp(16)
dp(18) = dp(2)*p(4) + p(2)*dp(4) - dp(12)
dp(19) = dp(1)*p(8) + p(1)*dp(8) - dp(18)
dp(20) = dp(2)*p(6) + p(2)*dp(6) - dp(14) - dp(14) - dp(14)
dp(21) = dp(1)*p(7) + p(1)*dp(7) - dp(15)
dp(22) = dp(2)*p(8) + p(2)*dp(8) - dp(20)
dp(23) = dp(9)*p(2) + p(9)*dp(2)
dp(24) = dm(30) + dm(31) + dm(32)
dp(25) = dp(3)*p(6) + p(3)*dp(6) - dp(24)
dp(26) = dp(14)*p(1) + p(14)*dp(1)
dp(27) = dp(9)*p(1) + p(9)*dp(1)
dp(28) = dp(3)*p(4) + p(3)*dp(4) - dp(23)
dp(29) = dp(1)*p(10) + p(1)*dp(10) - dp(23) - dp(28) - dp(23)
dp(30) = dp(1)*p(11) + p(1)*dp(11) - dp(23)
dp(31) = dp(1)*p(12) + p(1)*dp(12) - dp(25) - dp(24) - dp(24)
dp(32) = dp(1)*p(13) + p(1)*dp(13) - dp(25)
dp(33) = dp(4)*p(5) + p(4)*dp(5) - dp(25) - dp(31)
dp(34) = dp(2)*p(11) + p(2)*dp(11) - dp(25)
dp(35) = dp(4)*p(6) + p(4)*dp(6) - dp(26)
dp(36) = dp(2)*p(12) + p(2)*dp(12) - dp(26) - dp(35) - dp(26)
dp(37) = dp(2)*p(13) + p(2)*dp(13) - dp(26)
dp(38) = dp(14)*p(2) + p(14)*dp(2)
dp(39) = dp(3)*p(3) + p(3)*dp(3) - dp(27) - dp(27)
dp(40) = dp(3)*p(7) + p(3)*dp(7) - dp(27)
dp(41) = dp(1)*p(16) + p(1)*dp(16) - dp(28)
dp(42) = dp(2)*p(21) + p(2)*dp(21) - dp(41)
dp(43) = dp(1)*p(18) + p(1)*dp(18) - dp(33)
dp(44) = dp(7)*p(8) + p(7)*dp(8) - dp(43)
dp(45) = dp(6)*p(6) + p(6)*dp(6) - dp(38) - dp(38)
dp(46) = dp(2)*p(18) + p(2)*dp(18) - dp(35)
dp(47) = dp(1)*p(22) + p(1)*dp(22) - dp(46)
dp(48) = dp(6)*p(8) + p(6)*dp(8) - dp(38)
dp(49) = dp(1)*p(21) + p(1)*dp(21) - dp(40)
dp(50) = dp(2)*p(22) + p(2)*dp(22) - dp(48)
dp(51) = dp(9)*p(6) + p(9)*dp(6)
dp(52) = dp(14)*p(3) + p(14)*dp(3)
dp(53) = dp(9)*p(4) + p(9)*dp(4)
dp(54) = dp(9)*p(5) + p(9)*dp(5)
dp(55) = dp(1)*p(24) + p(1)*dp(24) - dp(51)
dp(56) = dp(3)*p(12) + p(3)*dp(12) - dp(51) - dp(55) - dp(51)
dp(57) = dp(3)*p(13) + p(3)*dp(13) - dp(51)
dp(58) = dp(14)*p(7) + p(14)*dp(7)
dp(59) = dp(9)*p(8) + p(9)*dp(8)
dp(60) = dp(2)*p(24) + p(2)*dp(24) - dp(52)
dp(61) = dp(4)*p(13) + p(4)*dp(13) - dp(58)
dp(62) = dp(6)*p(11) + p(6)*dp(11) - dp(52)
dp(63) = dp(14)*p(4) + p(14)*dp(4)
dp(64) = dp(14)*p(5) + p(14)*dp(5)
dp(65) = dp(9)*p(3) + p(9)*dp(3)
dp(66) = dp(9)*p(7) + p(9)*dp(7)
dp(67) = dp(3)*p(16) + p(3)*dp(16) - dp(53)
dp(68) = dp(4)*p(15) + p(4)*dp(15) - dp(54) - dp(67)
dp(69) = dp(2)*p(39) + p(2)*dp(39) - dp(68)
dp(70) = dp(1)*p(29) + p(1)*dp(29) - dp(54) - dp(68)
dp(71) = dp(7)*p(11) + p(7)*dp(11) - dp(53)
dp(72) = dp(1)*p(31) + p(1)*dp(31) - dp(56) - dp(55)
dp(73) = dp(1)*p(32) + p(1)*dp(32) - dp(57)
dp(74) = dp(3)*p(18) + p(3)*dp(18) - dp(59)
dp(75) = dp(2)*p(29) + p(2)*dp(29) - dp(57) - dp(55)
dp(76) = dp(1)*p(34) + p(1)*dp(34) - dp(59)
dp(77) = dp(6)*p(16) + p(6)*dp(16) - dp(58)
dp(78) = dp(1)*p(36) + p(1)*dp(36) - dp(62) - dp(60)
dp(79) = dp(2)*p(32) + p(2)*dp(32) - dp(58)
dp(80) = dp(6)*p(12) + p(6)*dp(12) - dp(64) - dp(63) - dp(63)
dp(81) = dp(1)*p(45) + p(1)*dp(45) - dp(80)
dp(82) = dp(14)*p(6) + p(14)*dp(6)
dp(83) = dp(2)*p(33) + p(2)*dp(33) - dp(61) - dp(60)
dp(84) = dp(2)*p(34) + p(2)*dp(34) - dp(62)
dp(85) = dp(6)*p(18) + p(6)*dp(18) - dp(63)
dp(86) = dp(2)*p(36) + p(2)*dp(36) - dp(64) - dp(80)
dp(87) = dp(8)*p(13) + p(8)*dp(13) - dp(63)
dp(88) = dp(14)*p(8) + p(14)*dp(8)
dp(89) = dp(1)*p(39) + p(1)*dp(39) - dp(65)
dp(90) = dp(3)*p(21) + p(3)*dp(21) - dp(66)
dp(91) = dp(1)*p(41) + p(1)*dp(41) - dp(67)
dp(92) = dp(2)*p(49) + p(2)*dp(49) - dp(91)
dp(93) = dp(1)*p(43) + p(1)*dp(43) - dp(74)
dp(94) = dp(8)*p(21) + p(8)*dp(21) - dp(93)
dp(95) = dp(1)*p(46) + p(1)*dp(46) - dp(83)
dp(96) = dp(7)*p(22) + p(7)*dp(22) - dp(95)
dp(97) = dp(2)*p(45) + p(2)*dp(45) - dp(82)
dp(98) = dp(2)*p(46) + p(2)*dp(46) - dp(85)
dp(99) = dp(1)*p(50) + p(1)*dp(50) - dp(98)
dp(100) = dp(6)*p(22) + p(6)*dp(22) - dp(88)
dp(101) = dp(1)*p(49) + p(1)*dp(49) - dp(90)
dp(102) = dp(2)*p(50) + p(2)*dp(50) - dp(100)
dp(103) = dp(9)*p(14) + p(9)*dp(14)
dp(104) = dp(9)*p(12) + p(9)*dp(12)
dp(105) = dp(9)*p(13) + p(9)*dp(13)
dp(106) = dp(14)*p(15) + p(14)*dp(15)
dp(107) = dp(9)*p(20) + p(9)*dp(20)
dp(108) = dp(14)*p(10) + p(14)*dp(10)
dp(109) = dp(14)*p(11) + p(14)*dp(11)
dp(110) = dp(9)*p(16) + p(9)*dp(16)
dp(111) = dp(9)*p(10) + p(9)*dp(10)
dp(112) = dp(9)*p(11) + p(9)*dp(11)
dp(113) = dp(9)*p(17) + p(9)*dp(17)
dp(114) = dp(3)*p(24) + p(3)*dp(24) - dp(104)
dp(115) = dp(7)*p(24) + p(7)*dp(24) - dp(105)
dp(116) = dp(3)*p(31) + p(3)*dp(31) - dp(104) - dp(115)
dp(117) = dp(6)*p(39) + p(6)*dp(39) - dp(114)
dp(118) = dp(3)*p(32) + p(3)*dp(32) - dp(105)
dp(119) = dp(14)*p(21) + p(14)*dp(21)
dp(120) = dp(9)*p(18) + p(9)*dp(18)
dp(121) = dp(9)*p(19) + p(9)*dp(19)
dp(122) = dp(4)*p(24) + p(4)*dp(24) - dp(103) - dp(103) - dp(103)
dp(123) = dp(1)*p(60) + p(1)*dp(60) - dp(107) - dp(122)
dp(124) = dp(13)*p(16) + p(13)*dp(16) - dp(119)
dp(125) = dp(2)*p(56) + p(2)*dp(56) - dp(106) - dp(124)
dp(126) = dp(4)*p(32) + p(4)*dp(32) - dp(119)
dp(127) = dp(1)*p(62) + p(1)*dp(62) - dp(107) - dp(125)
dp(128) = dp(14)*p(16) + p(14)*dp(16)
dp(129) = dp(14)*p(17) + p(14)*dp(17)
dp(130) = dp(6)*p(24) + p(6)*dp(24) - dp(108)
dp(131) = dp(3)*p(45) + p(3)*dp(45) - dp(130)
dp(132) = dp(14)*p(12) + p(14)*dp(12)
dp(133) = dp(14)*p(13) + p(14)*dp(13)
dp(134) = dp(9)*p(22) + p(9)*dp(22)
dp(135) = dp(8)*p(24) + p(8)*dp(24) - dp(109)
dp(136) = dp(13)*p(18) + p(13)*dp(18) - dp(128)
dp(137) = dp(6)*p(34) + p(6)*dp(34) - dp(109)
dp(138) = dp(14)*p(18) + p(14)*dp(18)
dp(139) = dp(14)*p(19) + p(14)*dp(19)
dp(140) = dp(9)*p(9) + p(9)*dp(9)
dp(141) = dp(9)*p(15) + p(9)*dp(15)
dp(142) = dp(9)*p(21) + p(9)*dp(21)
dp(143) = dp(3)*p(41) + p(3)*dp(41) - dp(110)
dp(144) = dp(4)*p(39) + p(4)*dp(39) - dp(112)
dp(145) = dp(1)*p(68) + p(1)*dp(68) - dp(111) - dp(144)
dp(146) = dp(1)*p(69) + p(1)*dp(69) - dp(112)
dp(147) = dp(1)*p(70) + p(1)*dp(70) - dp(113) - dp(145)
dp(148) = dp(11)*p(21) + p(11)*dp(21) - dp(110)
dp(149) = dp(1)*p(72) + p(1)*dp(72) - dp(116) - dp(115)
dp(150) = dp(1)*p(73) + p(1)*dp(73) - dp(118)
dp(151) = dp(3)*p(43) + p(3)*dp(43) - dp(120)
dp(152) = dp(15)*p(18) + p(15)*dp(18) - dp(121) - dp(151)
dp(153) = dp(2)*p(69) + p(2)*dp(69) - dp(117)
dp(154) = dp(1)*p(75) + p(1)*dp(75) - dp(121) - dp(152)
dp(155) = dp(7)*p(34) + p(7)*dp(34) - dp(120)
dp(156) = dp(6)*p(41) + p(6)*dp(41) - dp(119)
dp(157) = dp(1)*p(78) + p(1)*dp(78) - dp(125) - dp(123)
dp(158) = dp(2)*p(73) + p(2)*dp(73) - dp(119)
dp(159) = dp(1)*p(80) + p(1)*dp(80) - dp(131) - dp(130) - dp(130)
dp(160) = dp(1)*p(81) + p(1)*dp(81) - dp(131)
dp(161) = dp(14)*p(14) + p(14)*dp(14)
dp(162) = dp(3)*p(46) + p(3)*dp(46) - dp(134)
dp(163) = dp(2)*p(75) + p(2)*dp(75) - dp(126) - dp(123)
dp(164) = dp(1)*p(84) + p(1)*dp(84) - dp(134)
dp(165) = dp(6)*p(43) + p(6)*dp(43) - dp(128)
dp(166) = dp(1)*p(86) + p(1)*dp(86) - dp(137) - dp(135)
dp(167) = dp(8)*p(32) + p(8)*dp(32) - dp(128)
dp(168) = dp(4)*p(45) + p(4)*dp(45) - dp(133)
dp(169) = dp(2)*p(80) + p(2)*dp(80) - dp(132) - dp(168)
dp(170) = dp(2)*p(81) + p(2)*dp(81) - dp(133)
dp(171) = dp(14)*p(20) + p(14)*dp(20)
dp(172) = dp(2)*p(83) + p(2)*dp(83) - dp(136) - dp(135)
dp(173) = dp(2)*p(84) + p(2)*dp(84) - dp(137)
dp(174) = dp(6)*p(46) + p(6)*dp(46) - dp(138)
dp(175) = dp(2)*p(86) + p(2)*dp(86) - dp(139) - dp(169)
dp(176) = dp(13)*p(22) + p(13)*dp(22) - dp(138)
dp(177) = dp(14)*p(22) + p(14)*dp(22)
dp(178) = dp(3)*p(39) + p(3)*dp(39) - dp(141)
dp(179) = dp(1)*p(89) + p(1)*dp(89) - dp(141) - dp(178) - dp(178)
dp(180) = dp(3)*p(49) + p(3)*dp(49) - dp(142)
dp(181) = dp(1)*p(91) + p(1)*dp(91) - dp(143)
dp(182) = dp(2)*p(101) + p(2)*dp(101) - dp(181)
dp(183) = dp(1)*p(93) + p(1)*dp(93) - dp(151)
dp(184) = dp(8)*p(49) + p(8)*dp(49) - dp(183)
dp(185) = dp(1)*p(95) + p(1)*dp(95) - dp(162)
dp(186) = dp(21)*p(22) + p(21)*dp(22) - dp(185)
dp(187) = dp(6)*p(45) + p(6)*dp(45) - dp(171)
dp(188) = dp(1)*p(98) + p(1)*dp(98) - dp(172)
dp(189) = dp(7)*p(50) + p(7)*dp(50) - dp(188)
dp(190) = dp(2)*p(97) + p(2)*dp(97) - dp(171) - dp(187) - dp(187)
dp(191) = dp(2)*p(98) + p(2)*dp(98) - dp(174)
dp(192) = dp(1)*p(102) + p(1)*dp(102) - dp(191)
dp(193) = dp(6)*p(50) + p(6)*dp(50) - dp(177)
dp(194) = dp(1)*p(101) + p(1)*dp(101) - dp(180)
dp(195) = dp(2)*p(102) + p(2)*dp(102) - dp(193)
dp(196) = dp(9)*p(26) + p(9)*dp(26)
dp(197) = dp(9)*p(38) + p(9)*dp(38)
dp(198) = dp(9)*p(24) + p(9)*dp(24)
dp(199) = dp(9)*p(31) + p(9)*dp(31)
dp(200) = dp(9)*p(25) + p(9)*dp(25)
dp(201) = dp(9)*p(32) + p(9)*dp(32)
dp(202) = dp(14)*p(39) + p(14)*dp(39)
dp(203) = dp(14)*p(40) + p(14)*dp(40)
dp(204) = dp(9)*p(35) + p(9)*dp(35)
dp(205) = dp(9)*p(36) + p(9)*dp(36)
dp(206) = dp(9)*p(37) + p(9)*dp(37)
dp(207) = dp(14)*p(28) + p(14)*dp(28)
dp(208) = dp(14)*p(29) + p(14)*dp(29)
dp(209) = dp(14)*p(30) + p(14)*dp(30)
dp(210) = dp(9)*p(45) + p(9)*dp(45)
dp(211) = dp(14)*p(24) + p(14)*dp(24)
dp(212) = dp(14)*p(25) + p(14)*dp(25)
dp(213) = dp(9)*p(48) + p(9)*dp(48)
dp(214) = dp(14)*p(33) + p(14)*dp(33)
dp(215) = dp(14)*p(34) + p(14)*dp(34)
dp(216) = dp(9)*p(41) + p(9)*dp(41)
dp(217) = dp(9)*p(28) + p(9)*dp(28)
dp(218) = dp(9)*p(23) + p(9)*dp(23)
dp(219) = dp(9)*p(29) + p(9)*dp(29)
dp(220) = dp(9)*p(30) + p(9)*dp(30)
dp(221) = dp(9)*p(42) + p(9)*dp(42)
dp(222) = dp(1)*p(114) + p(1)*dp(114) - dp(198)
dp(223) = dp(21)*p(24) + p(21)*dp(24) - dp(201)
dp(224) = dp(3)*p(72) + p(3)*dp(72) - dp(199) - dp(223)
dp(225) = dp(1)*p(116) + p(1)*dp(116) - dp(199) - dp(224)
dp(226) = dp(13)*p(39) + p(13)*dp(39) - dp(198)
dp(227) = dp(3)*p(73) + p(3)*dp(73) - dp(201)
dp(228) = dp(14)*p(49) + p(14)*dp(49)
dp(229) = dp(9)*p(43) + p(9)*dp(43)
dp(230) = dp(9)*p(33) + p(9)*dp(33)
dp(231) = dp(9)*p(34) + p(9)*dp(34)
dp(232) = dp(9)*p(44) + p(9)*dp(44)
dp(233) = dp(16)*p(24) + p(16)*dp(24) - dp(196)
dp(234) = dp(2)*p(114) + p(2)*dp(114) - dp(202)
dp(235) = dp(1)*p(123) + p(1)*dp(123) - dp(205) - dp(234)
dp(236) = dp(13)*p(41) + p(13)*dp(41) - dp(228)
dp(237) = dp(2)*p(116) + p(2)*dp(116) - dp(203) - dp(236)
dp(238) = dp(16)*p(32) + p(16)*dp(32) - dp(228)
dp(239) = dp(6)*p(69) + p(6)*dp(69) - dp(202)
dp(240) = dp(4)*p(73) + p(4)*dp(73) - dp(228)
dp(241) = dp(11)*p(32) + p(11)*dp(32) - dp(196)
dp(242) = dp(14)*p(41) + p(14)*dp(41)
dp(243) = dp(14)*p(42) + p(14)*dp(42)
dp(244) = dp(1)*p(130) + p(1)*dp(130) - dp(210)
dp(245) = dp(4)*p(62) + p(4)*dp(62) - dp(213) - dp(209)
dp(246) = dp(3)*p(81) + p(3)*dp(81) - dp(210)
dp(247) = dp(14)*p(31) + p(14)*dp(31)
dp(248) = dp(14)*p(32) + p(14)*dp(32)
dp(249) = dp(14)*p(26) + p(14)*dp(26)
dp(250) = dp(9)*p(46) + p(9)*dp(46)
dp(251) = dp(9)*p(47) + p(9)*dp(47)
dp(252) = dp(18)*p(24) + p(18)*dp(24) - dp(197)
dp(253) = dp(1)*p(135) + p(1)*dp(135) - dp(213) - dp(252)
dp(254) = dp(13)*p(43) + p(13)*dp(43) - dp(242)
dp(255) = dp(2)*p(125) + p(2)*dp(125) - dp(209) - dp(245)
dp(256) = dp(18)*p(32) + p(18)*dp(32) - dp(242)
dp(257) = dp(13)*p(34) + p(13)*dp(34) - dp(197)
dp(258) = dp(14)*p(43) + p(14)*dp(43)
dp(259) = dp(14)*p(44) + p(14)*dp(44)
dp(260) = dp(2)*p(130) + p(2)*dp(130) - dp(211)
dp(261) = dp(4)*p(81) + p(4)*dp(81) - dp(248)
dp(262) = dp(11)*p(45) + p(11)*dp(45) - dp(211)
dp(263) = dp(14)*p(35) + p(14)*dp(35)
dp(264) = dp(14)*p(36) + p(14)*dp(36)
dp(265) = dp(14)*p(37) + p(14)*dp(37)
dp(266) = dp(9)*p(50) + p(9)*dp(50)
dp(267) = dp(22)*p(24) + p(22)*dp(24) - dp(215)
dp(268) = dp(13)*p(46) + p(13)*dp(46) - dp(258)
dp(269) = dp(6)*p(84) + p(6)*dp(84) - dp(215)
dp(270) = dp(14)*p(46) + p(14)*dp(46)
dp(271) = dp(14)*p(47) + p(14)*dp(47)
dp(272) = dp(9)*p(27) + p(9)*dp(27)
dp(273) = dp(9)*p(39) + p(9)*dp(39)
dp(274) = dp(9)*p(40) + p(9)*dp(40)
dp(275) = dp(9)*p(49) + p(9)*dp(49)
dp(276) = dp(3)*p(91) + p(3)*dp(91) - dp(216)
dp(277) = dp(16)*p(39) + p(16)*dp(39) - dp(218)
dp(278) = dp(1)*p(144) + p(1)*dp(144) - dp(217) - dp(277)
dp(279) = dp(2)*p(178) + p(2)*dp(178) - dp(278)
dp(280) = dp(1)*p(145) + p(1)*dp(145) - dp(219) - dp(278)
dp(281) = dp(7)*p(69) + p(7)*dp(69) - dp(218)
dp(282) = dp(1)*p(147) + p(1)*dp(147) - dp(221) - dp(280)
dp(283) = dp(11)*p(49) + p(11)*dp(49) - dp(216)
dp(284) = dp(1)*p(149) + p(1)*dp(149) - dp(224) - dp(223)
dp(285) = dp(1)*p(150) + p(1)*dp(150) - dp(227)
dp(286) = dp(3)*p(93) + p(3)*dp(93) - dp(229)
dp(287) = dp(18)*p(39) + p(18)*dp(39) - dp(231)
dp(288) = dp(1)*p(152) + p(1)*dp(152) - dp(230) - dp(287)
dp(289) = dp(1)*p(153) + p(1)*dp(153) - dp(231)
dp(290) = dp(1)*p(154) + p(1)*dp(154) - dp(232) - dp(288)
dp(291) = dp(21)*p(34) + p(21)*dp(34) - dp(229)
dp(292) = dp(6)*p(91) + p(6)*dp(91) - dp(228)
dp(293) = dp(1)*p(157) + p(1)*dp(157) - dp(237) - dp(235)
dp(294) = dp(2)*p(150) + p(2)*dp(150) - dp(228)
dp(295) = dp(1)*p(159) + p(1)*dp(159) - dp(245) - dp(244)
dp(296) = dp(1)*p(160) + p(1)*dp(160) - dp(246)
dp(297) = dp(3)*p(95) + p(3)*dp(95) - dp(250)
dp(298) = dp(2)*p(152) + p(2)*dp(152) - dp(238) - dp(234)
dp(299) = dp(2)*p(153) + p(2)*dp(153) - dp(239)
dp(300) = dp(1)*p(163) + p(1)*dp(163) - dp(251) - dp(298)
dp(301) = dp(7)*p(84) + p(7)*dp(84) - dp(250)
dp(302) = dp(6)*p(93) + p(6)*dp(93) - dp(242)
dp(303) = dp(1)*p(166) + p(1)*dp(166) - dp(255) - dp(253)
dp(304) = dp(8)*p(73) + p(8)*dp(73) - dp(242)
dp(305) = dp(16)*p(45) + p(16)*dp(45) - dp(248)
dp(306) = dp(1)*p(169) + p(1)*dp(169) - dp(262) - dp(260)
dp(307) = dp(2)*p(160) + p(2)*dp(160) - dp(248)
dp(308) = dp(14)*p(38) + p(14)*dp(38)
dp(309) = dp(6)*p(80) + p(6)*dp(80) - dp(264) - dp(263)
dp(310) = dp(1)*p(187) + p(1)*dp(187) - dp(309)
dp(311) = dp(14)*p(45) + p(14)*dp(45)
dp(312) = dp(3)*p(98) + p(3)*dp(98) - dp(266)
dp(313) = dp(2)*p(163) + p(2)*dp(163) - dp(256) - dp(253)
dp(314) = dp(1)*p(173) + p(1)*dp(173) - dp(266)
dp(315) = dp(6)*p(95) + p(6)*dp(95) - dp(258)
dp(316) = dp(1)*p(175) + p(1)*dp(175) - dp(269) - dp(267)
dp(317) = dp(22)*p(32) + p(22)*dp(32) - dp(258)
dp(318) = dp(18)*p(45) + p(18)*dp(45) - dp(249)
dp(319) = dp(2)*p(169) + p(2)*dp(169) - dp(264) - dp(309)
dp(320) = dp(8)*p(81) + p(8)*dp(81) - dp(249)
dp(321) = dp(14)*p(48) + p(14)*dp(48)
dp(322) = dp(2)*p(172) + p(2)*dp(172) - dp(268) - dp(267)
dp(323) = dp(2)*p(173) + p(2)*dp(173) - dp(269)
dp(324) = dp(6)*p(98) + p(6)*dp(98) - dp(270)
dp(325) = dp(2)*p(175) + p(2)*dp(175) - dp(271) - dp(319)
dp(326) = dp(13)*p(50) + p(13)*dp(50) - dp(270)
dp(327) = dp(14)*p(50) + p(14)*dp(50)
dp(328) = dp(1)*p(178) + p(1)*dp(178) - dp(273)
dp(329) = dp(21)*p(39) + p(21)*dp(39) - dp(272)
dp(330) = dp(3)*p(101) + p(3)*dp(101) - dp(275)
dp(331) = dp(1)*p(181) + p(1)*dp(181) - dp(276)
dp(332) = dp(2)*p(194) + p(2)*dp(194) - dp(331)
dp(333) = dp(1)*p(183) + p(1)*dp(183) - dp(286)
dp(334) = dp(8)*p(101) + p(8)*dp(101) - dp(333)
dp(335) = dp(1)*p(185) + p(1)*dp(185) - dp(297)
dp(336) = dp(22)*p(49) + p(22)*dp(49) - dp(335)
dp(337) = dp(1)*p(188) + p(1)*dp(188) - dp(312)
dp(338) = dp(21)*p(50) + p(21)*dp(50) - dp(337)
dp(339) = dp(2)*p(187) + p(2)*dp(187) - dp(311)
dp(340) = dp(1)*p(191) + p(1)*dp(191) - dp(322)
dp(341) = dp(7)*p(102) + p(7)*dp(102) - dp(340)
dp(342) = dp(22)*p(45) + p(22)*dp(45) - dp(308)
dp(343) = dp(2)*p(191) + p(2)*dp(191) - dp(324)
dp(344) = dp(1)*p(195) + p(1)*dp(195) - dp(343)
dp(345) = dp(6)*p(102) + p(6)*dp(102) - dp(327)
dp(346) = dp(1)*p(194) + p(1)*dp(194) - dp(330)
dp(347) = dp(2)*p(195) + p(2)*dp(195) - dp(345)
dp(348) = dp(9)*p(52) + p(9)*dp(52)
dp(349) = dp(9)*p(58) + p(9)*dp(58)
dp(350) = dp(9)*p(63) + p(9)*dp(63)
dp(351) = dp(9)*p(64) + p(9)*dp(64)
dp(352) = dp(9)*p(82) + p(9)*dp(82)
dp(353) = dp(9)*p(88) + p(9)*dp(88)
dp(354) = dp(9)*p(55) + p(9)*dp(55)
dp(355) = dp(9)*p(72) + p(9)*dp(72)
dp(356) = dp(9)*p(51) + p(9)*dp(51)
dp(357) = dp(9)*p(56) + p(9)*dp(56)
dp(358) = dp(9)*p(57) + p(9)*dp(57)
dp(359) = dp(9)*p(73) + p(9)*dp(73)
dp(360) = dp(14)*p(89) + p(14)*dp(89)
dp(361) = dp(14)*p(90) + p(14)*dp(90)
dp(362) = dp(9)*p(77) + p(9)*dp(77)
dp(363) = dp(9)*p(60) + p(9)*dp(60)
dp(364) = dp(9)*p(78) + p(9)*dp(78)
dp(365) = dp(9)*p(61) + p(9)*dp(61)
dp(366) = dp(9)*p(62) + p(9)*dp(62)
dp(367) = dp(9)*p(79) + p(9)*dp(79)
dp(368) = dp(14)*p(67) + p(14)*dp(67)
dp(369) = dp(14)*p(68) + p(14)*dp(68)
dp(370) = dp(14)*p(69) + p(14)*dp(69)
dp(371) = dp(14)*p(70) + p(14)*dp(70)
dp(372) = dp(14)*p(71) + p(14)*dp(71)
dp(373) = dp(9)*p(80) + p(9)*dp(80)
dp(374) = dp(9)*p(81) + p(9)*dp(81)
dp(375) = dp(14)*p(55) + p(14)*dp(55)
dp(376) = dp(14)*p(56) + p(14)*dp(56)
dp(377) = dp(14)*p(57) + p(14)*dp(57)
dp(378) = dp(14)*p(52) + p(14)*dp(52)
dp(379) = dp(9)*p(85) + p(9)*dp(85)
dp(380) = dp(9)*p(86) + p(9)*dp(86)
dp(381) = dp(9)*p(87) + p(9)*dp(87)
dp(382) = dp(14)*p(74) + p(14)*dp(74)
dp(383) = dp(14)*p(75) + p(14)*dp(75)
dp(384) = dp(14)*p(76) + p(14)*dp(76)
dp(385) = dp(9)*p(97) + p(9)*dp(97)
dp(386) = dp(14)*p(60) + p(14)*dp(60)
dp(387) = dp(14)*p(61) + p(14)*dp(61)
dp(388) = dp(14)*p(62) + p(14)*dp(62)
dp(389) = dp(9)*p(100) + p(9)*dp(100)
dp(390) = dp(14)*p(83) + p(14)*dp(83)
dp(391) = dp(14)*p(84) + p(14)*dp(84)
dp(392) = dp(9)*p(91) + p(9)*dp(91)
dp(393) = dp(9)*p(67) + p(9)*dp(67)
dp(394) = dp(9)*p(53) + p(9)*dp(53)
dp(395) = dp(9)*p(68) + p(9)*dp(68)
dp(396) = dp(9)*p(54) + p(9)*dp(54)
dp(397) = dp(9)*p(69) + p(9)*dp(69)
dp(398) = dp(9)*p(70) + p(9)*dp(70)
dp(399) = dp(9)*p(71) + p(9)*dp(71)
dp(400) = dp(9)*p(92) + p(9)*dp(92)
dp(401) = dp(3)*p(114) + p(3)*dp(114) - dp(354)
dp(402) = dp(7)*p(114) + p(7)*dp(114) - dp(356)
dp(403) = dp(24)*p(49) + p(24)*dp(49) - dp(359)
dp(404) = dp(3)*p(149) + p(3)*dp(149) - dp(355) - dp(403)
dp(405) = dp(1)*p(224) + p(1)*dp(224) - dp(355) - dp(404)
dp(406) = dp(6)*p(178) + p(6)*dp(178) - dp(401)
dp(407) = dp(32)*p(39) + p(32)*dp(39) - dp(356)
dp(408) = dp(3)*p(150) + p(3)*dp(150) - dp(359)
dp(409) = dp(14)*p(101) + p(14)*dp(101)
dp(410) = dp(9)*p(93) + p(9)*dp(93)
dp(411) = dp(9)*p(74) + p(9)*dp(74)
dp(412) = dp(9)*p(59) + p(9)*dp(59)
dp(413) = dp(9)*p(75) + p(9)*dp(75)
dp(414) = dp(9)*p(76) + p(9)*dp(76)
dp(415) = dp(9)*p(94) + p(9)*dp(94)
dp(416) = dp(24)*p(41) + p(24)*dp(41) - dp(349)
dp(417) = dp(4)*p(114) + p(4)*dp(114) - dp(348)
dp(418) = dp(1)*p(234) + p(1)*dp(234) - dp(363) - dp(417)
dp(419) = dp(1)*p(235) + p(1)*dp(235) - dp(364) - dp(418)
dp(420) = dp(13)*p(91) + p(13)*dp(91) - dp(409)
dp(421) = dp(2)*p(224) + p(2)*dp(224) - dp(361) - dp(420)
dp(422) = dp(32)*p(41) + p(32)*dp(41) - dp(409)
dp(423) = dp(1)*p(237) + p(1)*dp(237) - dp(364) - dp(421)
dp(424) = dp(16)*p(73) + p(16)*dp(73) - dp(409)
dp(425) = dp(13)*p(69) + p(13)*dp(69) - dp(348)
dp(426) = dp(4)*p(150) + p(4)*dp(150) - dp(409)
dp(427) = dp(11)*p(73) + p(11)*dp(73) - dp(349)
dp(428) = dp(14)*p(91) + p(14)*dp(91)
dp(429) = dp(14)*p(92) + p(14)*dp(92)
dp(430) = dp(3)*p(130) + p(3)*dp(130) - dp(373)
dp(431) = dp(7)*p(130) + p(7)*dp(130) - dp(374)
dp(432) = dp(3)*p(159) + p(3)*dp(159) - dp(373) - dp(431)
dp(433) = dp(39)*p(45) + p(39)*dp(45) - dp(430)
dp(434) = dp(3)*p(160) + p(3)*dp(160) - dp(374)
dp(435) = dp(14)*p(72) + p(14)*dp(72)
dp(436) = dp(14)*p(73) + p(14)*dp(73)
dp(437) = dp(14)*p(58) + p(14)*dp(58)
dp(438) = dp(9)*p(95) + p(9)*dp(95)
dp(439) = dp(9)*p(83) + p(9)*dp(83)
dp(440) = dp(9)*p(84) + p(9)*dp(84)
dp(441) = dp(9)*p(96) + p(9)*dp(96)
dp(442) = dp(24)*p(43) + p(24)*dp(43) - dp(350)
dp(443) = dp(8)*p(114) + p(8)*dp(114) - dp(370)
dp(444) = dp(1)*p(253) + p(1)*dp(253) - dp(380) - dp(443)
dp(445) = dp(13)*p(93) + p(13)*dp(93) - dp(428)
dp(446) = dp(2)*p(237) + p(2)*dp(237) - dp(372) - dp(432)
dp(447) = dp(32)*p(43) + p(32)*dp(43) - dp(428)
dp(448) = dp(6)*p(153) + p(6)*dp(153) - dp(370)
dp(449) = dp(18)*p(73) + p(18)*dp(73) - dp(428)
dp(450) = dp(32)*p(34) + p(32)*dp(34) - dp(350)
dp(451) = dp(14)*p(93) + p(14)*dp(93)
dp(452) = dp(14)*p(94) + p(14)*dp(94)
dp(453) = dp(4)*p(130) + p(4)*dp(130) - dp(352)
dp(454) = dp(1)*p(260) + p(1)*dp(260) - dp(385) - dp(453)
dp(455) = dp(16)*p(81) + p(16)*dp(81) - dp(436)
dp(456) = dp(2)*p(245) + p(2)*dp(245) - dp(376) - dp(455)
dp(457) = dp(4)*p(160) + p(4)*dp(160) - dp(436)
dp(458) = dp(11)*p(81) + p(11)*dp(81) - dp(352)
dp(459) = dp(14)*p(77) + p(14)*dp(77)
dp(460) = dp(14)*p(78) + p(14)*dp(78)
dp(461) = dp(14)*p(79) + p(14)*dp(79)
dp(462) = dp(14)*p(63) + p(14)*dp(63)
dp(463) = dp(14)*p(64) + p(14)*dp(64)
dp(464) = dp(6)*p(130) + p(6)*dp(130) - dp(386)
dp(465) = dp(3)*p(187) + p(3)*dp(187) - dp(464)
dp(466) = dp(14)*p(80) + p(14)*dp(80)
dp(467) = dp(14)*p(81) + p(14)*dp(81)
dp(468) = dp(9)*p(98) + p(9)*dp(98)
dp(469) = dp(9)*p(99) + p(9)*dp(99)
dp(470) = dp(24)*p(46) + p(24)*dp(46) - dp(353)
dp(471) = dp(1)*p(267) + p(1)*dp(267) - dp(389) - dp(470)
dp(472) = dp(13)*p(95) + p(13)*dp(95) - dp(451)
dp(473) = dp(2)*p(255) + p(2)*dp(255) - dp(384) - dp(456)
dp(474) = dp(32)*p(46) + p(32)*dp(46) - dp(451)
dp(475) = dp(13)*p(84) + p(13)*dp(84) - dp(353)
dp(476) = dp(14)*p(95) + p(14)*dp(95)
dp(477) = dp(14)*p(96) + p(14)*dp(96)
dp(478) = dp(8)*p(130) + p(8)*dp(130) - dp(378)
dp(479) = dp(18)*p(81) + p(18)*dp(81) - dp(437)
dp(480) = dp(34)*p(45) + p(34)*dp(45) - dp(378)
dp(481) = dp(14)*p(85) + p(14)*dp(85)
dp(482) = dp(14)*p(86) + p(14)*dp(86)
dp(483) = dp(14)*p(87) + p(14)*dp(87)
dp(484) = dp(9)*p(102) + p(9)*dp(102)
dp(485) = dp(24)*p(50) + p(24)*dp(50) - dp(391)
dp(486) = dp(13)*p(98) + p(13)*dp(98) - dp(476)
dp(487) = dp(6)*p(173) + p(6)*dp(173) - dp(391)
dp(488) = dp(14)*p(98) + p(14)*dp(98)
dp(489) = dp(14)*p(99) + p(14)*dp(99)
dp(490) = dp(9)*p(65) + p(9)*dp(65)
dp(491) = dp(9)*p(66) + p(9)*dp(66)
dp(492) = dp(9)*p(89) + p(9)*dp(89)
dp(493) = dp(9)*p(90) + p(9)*dp(90)
dp(494) = dp(9)*p(101) + p(9)*dp(101)
dp(495) = dp(3)*p(181) + p(3)*dp(181) - dp(392)
dp(496) = dp(39)*p(41) + p(39)*dp(41) - dp(394)
dp(497) = dp(4)*p(178) + p(4)*dp(178) - dp(397)
dp(498) = dp(1)*p(278) + p(1)*dp(278) - dp(395) - dp(497)
dp(499) = dp(1)*p(279) + p(1)*dp(279) - dp(397)
dp(500) = dp(1)*p(280) + p(1)*dp(280) - dp(398) - dp(498)
dp(501) = dp(21)*p(69) + p(21)*dp(69) - dp(394)
dp(502) = dp(1)*p(282) + p(1)*dp(282) - dp(400) - dp(500)
dp(503) = dp(11)*p(101) + p(11)*dp(101) - dp(392)
dp(504) = dp(1)*p(284) + p(1)*dp(284) - dp(404) - dp(403)
dp(505) = dp(1)*p(285) + p(1)*dp(285) - dp(408)
dp(506) = dp(3)*p(183) + p(3)*dp(183) - dp(410)
dp(507) = dp(39)*p(43) + p(39)*dp(43) - dp(412)
dp(508) = dp(1)*p(287) + p(1)*dp(287) - dp(411) - dp(507)
dp(509) = dp(2)*p(279) + p(2)*dp(279) - dp(406)
dp(510) = dp(1)*p(288) + p(1)*dp(288) - dp(413) - dp(508)
dp(511) = dp(7)*p(153) + p(7)*dp(153) - dp(412)
dp(512) = dp(1)*p(290) + p(1)*dp(290) - dp(415) - dp(510)
dp(513) = dp(34)*p(49) + p(34)*dp(49) - dp(410)
dp(514) = dp(6)*p(181) + p(6)*dp(181) - dp(409)
dp(515) = dp(1)*p(293) + p(1)*dp(293) - dp(421) - dp(419)
dp(516) = dp(2)*p(285) + p(2)*dp(285) - dp(409)
dp(517) = dp(1)*p(295) + p(1)*dp(295) - dp(432) - dp(431)
dp(518) = dp(1)*p(296) + p(1)*dp(296) - dp(434)
dp(519) = dp(3)*p(185) + p(3)*dp(185) - dp(438)
dp(520) = dp(39)*p(46) + p(39)*dp(46) - dp(440)
dp(521) = dp(1)*p(298) + p(1)*dp(298) - dp(439) - dp(520)
dp(522) = dp(1)*p(299) + p(1)*dp(299) - dp(440)
dp(523) = dp(1)*p(300) + p(1)*dp(300) - dp(441) - dp(521)
dp(524) = dp(21)*p(84) + p(21)*dp(84) - dp(438)
dp(525) = dp(6)*p(183) + p(6)*dp(183) - dp(428)
dp(526) = dp(1)*p(303) + p(1)*dp(303) - dp(446) - dp(444)
dp(527) = dp(8)*p(150) + p(8)*dp(150) - dp(428)
dp(528) = dp(41)*p(45) + p(41)*dp(45) - dp(436)
dp(529) = dp(1)*p(306) + p(1)*dp(306) - dp(456) - dp(454)
dp(530) = dp(2)*p(296) + p(2)*dp(296) - dp(436)
dp(531) = dp(4)*p(169) + p(4)*dp(169) - dp(388) - dp(478)
dp(532) = dp(1)*p(310) + p(1)*dp(310) - dp(465)
dp(533) = dp(14)*p(82) + p(14)*dp(82)
dp(534) = dp(3)*p(188) + p(3)*dp(188) - dp(468)
dp(535) = dp(2)*p(298) + p(2)*dp(298) - dp(447) - dp(443)
dp(536) = dp(2)*p(299) + p(2)*dp(299) - dp(448)
dp(537) = dp(1)*p(313) + p(1)*dp(313) - dp(469) - dp(535)
dp(538) = dp(7)*p(173) + p(7)*dp(173) - dp(468)
dp(539) = dp(6)*p(185) + p(6)*dp(185) - dp(451)
dp(540) = dp(1)*p(316) + p(1)*dp(316) - dp(473) - dp(471)
dp(541) = dp(22)*p(73) + p(22)*dp(73) - dp(451)
dp(542) = dp(43)*p(45) + p(43)*dp(45) - dp(437)
dp(543) = dp(1)*p(319) + p(1)*dp(319) - dp(480) - dp(478)
dp(544) = dp(8)*p(160) + p(8)*dp(160) - dp(437)
dp(545) = dp(14)*p(88) + p(14)*dp(88)
dp(546) = dp(4)*p(187) + p(4)*dp(187) - dp(467)
dp(547) = dp(2)*p(309) + p(2)*dp(309) - dp(466) - dp(546)
dp(548) = dp(2)*p(310) + p(2)*dp(310) - dp(467)
dp(549) = dp(14)*p(97) + p(14)*dp(97)
dp(550) = dp(3)*p(191) + p(3)*dp(191) - dp(484)
dp(551) = dp(2)*p(313) + p(2)*dp(313) - dp(474) - dp(471)
dp(552) = dp(1)*p(323) + p(1)*dp(323) - dp(484)
dp(553) = dp(6)*p(188) + p(6)*dp(188) - dp(476)
dp(554) = dp(1)*p(325) + p(1)*dp(325) - dp(487) - dp(485)
dp(555) = dp(32)*p(50) + p(32)*dp(50) - dp(476)
dp(556) = dp(45)*p(46) + p(45)*dp(46) - dp(462)
dp(557) = dp(2)*p(319) + p(2)*dp(319) - dp(482) - dp(547)
dp(558) = dp(22)*p(81) + p(22)*dp(81) - dp(462)
dp(559) = dp(14)*p(100) + p(14)*dp(100)
dp(560) = dp(2)*p(322) + p(2)*dp(322) - dp(486) - dp(485)
dp(561) = dp(2)*p(323) + p(2)*dp(323) - dp(487)
dp(562) = dp(6)*p(191) + p(6)*dp(191) - dp(488)
dp(563) = dp(2)*p(325) + p(2)*dp(325) - dp(489) - dp(557)
dp(564) = dp(13)*p(102) + p(13)*dp(102) - dp(488)
dp(565) = dp(14)*p(102) + p(14)*dp(102)
dp(566) = dp(3)*p(178) + p(3)*dp(178) - dp(492)
dp(567) = dp(7)*p(178) + p(7)*dp(178) - dp(490)
dp(568) = dp(39)*p(49) + p(39)*dp(49) - dp(491)
dp(569) = dp(3)*p(194) + p(3)*dp(194) - dp(494)
dp(570) = dp(1)*p(331) + p(1)*dp(331) - dp(495)
dp(571) = dp(2)*p(346) + p(2)*dp(346) - dp(570)
dp(572) = dp(1)*p(333) + p(1)*dp(333) - dp(506)
dp(573) = dp(8)*p(194) + p(8)*dp(194) - dp(572)
dp(574) = dp(1)*p(335) + p(1)*dp(335) - dp(519)
dp(575) = dp(22)*p(101) + p(22)*dp(101) - dp(574)
dp(576) = dp(1)*p(337) + p(1)*dp(337) - dp(534)
dp(577) = dp(49)*p(50) + p(49)*dp(50) - dp(576)
dp(578) = dp(6)*p(187) + p(6)*dp(187) - dp(549)
dp(579) = dp(1)*p(340) + p(1)*dp(340) - dp(550)
dp(580) = dp(21)*p(102) + p(21)*dp(102) - dp(579)
dp(581) = dp(8)*p(187) + p(8)*dp(187) - dp(533)
dp(582) = dp(1)*p(343) + p(1)*dp(343) - dp(560)
dp(583) = dp(7)*p(195) + p(7)*dp(195) - dp(582)
dp(584) = dp(45)*p(50) + p(45)*dp(50) - dp(545)
dp(585) = dp(2)*p(343) + p(2)*dp(343) - dp(562)
dp(586) = dp(1)*p(347) + p(1)*dp(347) - dp(585)
dp(587) = dp(6)*p(195) + p(6)*dp(195) - dp(565)
dp(588) = dp(1)*p(346) + p(1)*dp(346) - dp(569)
dp(589) = dp(2)*p(347) + p(2)*dp(347) - dp(587)
dp(590) = dp(9)*p(103) + p(9)*dp(103)
dp(591) = dp(9)*p(106) + p(9)*dp(106)
dp(592) = dp(9)*p(119) + p(9)*dp(119)
dp(593) = dp(9)*p(128) + p(9)*dp(128)
dp(594) = dp(9)*p(108) + p(9)*dp(108)
dp(595) = dp(9)*p(109) + p(9)*dp(109)
dp(596) = dp(9)*p(129) + p(9)*dp(129)
dp(597) = dp(9)*p(132) + p(9)*dp(132)
dp(598) = dp(9)*p(133) + p(9)*dp(133)
dp(599) = dp(9)*p(161) + p(9)*dp(161)
dp(600) = dp(9)*p(138) + p(9)*dp(138)
dp(601) = dp(9)*p(139) + p(9)*dp(139)
dp(602) = dp(9)*p(171) + p(9)*dp(171)
dp(603) = dp(9)*p(177) + p(9)*dp(177)
dp(604) = dp(9)*p(114) + p(9)*dp(114)
dp(605) = dp(9)*p(115) + p(9)*dp(115)
dp(606) = dp(9)*p(149) + p(9)*dp(149)
dp(607) = dp(9)*p(104) + p(9)*dp(104)
dp(608) = dp(9)*p(116) + p(9)*dp(116)
dp(609) = dp(9)*p(105) + p(9)*dp(105)
dp(610) = dp(9)*p(117) + p(9)*dp(117)
dp(611) = dp(9)*p(118) + p(9)*dp(118)
dp(612) = dp(9)*p(150) + p(9)*dp(150)
dp(613) = dp(14)*p(178) + p(14)*dp(178)
dp(614) = dp(14)*p(179) + p(14)*dp(179)
dp(615) = dp(14)*p(180) + p(14)*dp(180)
dp(616) = dp(9)*p(156) + p(9)*dp(156)
dp(617) = dp(9)*p(122) + p(9)*dp(122)
dp(618) = dp(9)*p(123) + p(9)*dp(123)
dp(619) = dp(9)*p(157) + p(9)*dp(157)
dp(620) = dp(9)*p(124) + p(9)*dp(124)
dp(621) = dp(9)*p(107) + p(9)*dp(107)
dp(622) = dp(9)*p(125) + p(9)*dp(125)
dp(623) = dp(9)*p(126) + p(9)*dp(126)
dp(624) = dp(9)*p(127) + p(9)*dp(127)
dp(625) = dp(9)*p(158) + p(9)*dp(158)
dp(626) = dp(14)*p(143) + p(14)*dp(143)
dp(627) = dp(14)*p(144) + p(14)*dp(144)
dp(628) = dp(14)*p(145) + p(14)*dp(145)
dp(629) = dp(14)*p(146) + p(14)*dp(146)
dp(630) = dp(14)*p(147) + p(14)*dp(147)
dp(631) = dp(14)*p(148) + p(14)*dp(148)
dp(632) = dp(9)*p(130) + p(9)*dp(130)
dp(633) = dp(9)*p(159) + p(9)*dp(159)
dp(634) = dp(9)*p(131) + p(9)*dp(131)
dp(635) = dp(9)*p(160) + p(9)*dp(160)
dp(636) = dp(14)*p(114) + p(14)*dp(114)
dp(637) = dp(14)*p(115) + p(14)*dp(115)
dp(638) = dp(14)*p(116) + p(14)*dp(116)
dp(639) = dp(14)*p(117) + p(14)*dp(117)
dp(640) = dp(14)*p(118) + p(14)*dp(118)
dp(641) = dp(14)*p(106) + p(14)*dp(106)
dp(642) = dp(9)*p(165) + p(9)*dp(165)
dp(643) = dp(9)*p(135) + p(9)*dp(135)
dp(644) = dp(9)*p(166) + p(9)*dp(166)
dp(645) = dp(9)*p(136) + p(9)*dp(136)
dp(646) = dp(9)*p(137) + p(9)*dp(137)
dp(647) = dp(9)*p(167) + p(9)*dp(167)
dp(648) = dp(14)*p(151) + p(14)*dp(151)
dp(649) = dp(14)*p(152) + p(14)*dp(152)
dp(650) = dp(14)*p(153) + p(14)*dp(153)
dp(651) = dp(14)*p(154) + p(14)*dp(154)
dp(652) = dp(14)*p(155) + p(14)*dp(155)
dp(653) = dp(9)*p(168) + p(9)*dp(168)
dp(654) = dp(9)*p(169) + p(9)*dp(169)
dp(655) = dp(9)*p(170) + p(9)*dp(170)
dp(656) = dp(14)*p(122) + p(14)*dp(122)
dp(657) = dp(14)*p(123) + p(14)*dp(123)
dp(658) = dp(14)*p(124) + p(14)*dp(124)
dp(659) = dp(14)*p(125) + p(14)*dp(125)
dp(660) = dp(14)*p(126) + p(14)*dp(126)
dp(661) = dp(14)*p(127) + p(14)*dp(127)
dp(662) = dp(14)*p(108) + p(14)*dp(108)
dp(663) = dp(14)*p(109) + p(14)*dp(109)
dp(664) = dp(9)*p(187) + p(9)*dp(187)
dp(665) = dp(14)*p(130) + p(14)*dp(130)
dp(666) = dp(14)*p(131) + p(14)*dp(131)
dp(667) = dp(9)*p(174) + p(9)*dp(174)
dp(668) = dp(9)*p(175) + p(9)*dp(175)
dp(669) = dp(9)*p(176) + p(9)*dp(176)
dp(670) = dp(14)*p(162) + p(14)*dp(162)
dp(671) = dp(14)*p(163) + p(14)*dp(163)
dp(672) = dp(14)*p(164) + p(14)*dp(164)
dp(673) = dp(9)*p(190) + p(9)*dp(190)
dp(674) = dp(14)*p(135) + p(14)*dp(135)
dp(675) = dp(14)*p(136) + p(14)*dp(136)
dp(676) = dp(14)*p(137) + p(14)*dp(137)
dp(677) = dp(9)*p(193) + p(9)*dp(193)
dp(678) = dp(14)*p(172) + p(14)*dp(172)
dp(679) = dp(14)*p(173) + p(14)*dp(173)
dp(680) = dp(9)*p(181) + p(9)*dp(181)
dp(681) = dp(9)*p(143) + p(9)*dp(143)
dp(682) = dp(9)*p(110) + p(9)*dp(110)
dp(683) = dp(9)*p(144) + p(9)*dp(144)
dp(684) = dp(9)*p(111) + p(9)*dp(111)
dp(685) = dp(9)*p(112) + p(9)*dp(112)
dp(686) = dp(9)*p(145) + p(9)*dp(145)
dp(687) = dp(9)*p(113) + p(9)*dp(113)
dp(688) = dp(9)*p(146) + p(9)*dp(146)
dp(689) = dp(9)*p(147) + p(9)*dp(147)
dp(690) = dp(9)*p(148) + p(9)*dp(148)
dp(691) = dp(9)*p(182) + p(9)*dp(182)
dp(692) = dp(1)*p(401) + p(1)*dp(401) - dp(604)
dp(693) = dp(21)*p(114) + p(21)*dp(114) - dp(609)
dp(694) = dp(24)*p(101) + p(24)*dp(101) - dp(612)
dp(695) = dp(3)*p(284) + p(3)*dp(284) - dp(606) - dp(694)
dp(696) = dp(1)*p(404) + p(1)*dp(404) - dp(606) - dp(695)
dp(697) = dp(1)*p(405) + p(1)*dp(405) - dp(608) - dp(696)
dp(698) = dp(13)*p(178) + p(13)*dp(178) - dp(604)
dp(699) = dp(39)*p(73) + p(39)*dp(73) - dp(609)
dp(700) = dp(3)*p(285) + p(3)*dp(285) - dp(612)
dp(701) = dp(14)*p(194) + p(14)*dp(194)
dp(702) = dp(9)*p(183) + p(9)*dp(183)
dp(703) = dp(9)*p(151) + p(9)*dp(151)
dp(704) = dp(9)*p(120) + p(9)*dp(120)
dp(705) = dp(9)*p(152) + p(9)*dp(152)
dp(706) = dp(9)*p(121) + p(9)*dp(121)
dp(707) = dp(9)*p(153) + p(9)*dp(153)
dp(708) = dp(9)*p(154) + p(9)*dp(154)
dp(709) = dp(9)*p(155) + p(9)*dp(155)
dp(710) = dp(9)*p(184) + p(9)*dp(184)
dp(711) = dp(24)*p(91) + p(24)*dp(91) - dp(592)
dp(712) = dp(1)*p(416) + p(1)*dp(416) - dp(616) - dp(711)
dp(713) = dp(2)*p(401) + p(2)*dp(401) - dp(613)
dp(714) = dp(1)*p(418) + p(1)*dp(418) - dp(618) - dp(713)
dp(715) = dp(1)*p(419) + p(1)*dp(419) - dp(619) - dp(714)
dp(716) = dp(13)*p(181) + p(13)*dp(181) - dp(701)
dp(717) = dp(2)*p(404) + p(2)*dp(404) - dp(615) - dp(716)
dp(718) = dp(32)*p(91) + p(32)*dp(91) - dp(701)
dp(719) = dp(1)*p(421) + p(1)*dp(421) - dp(619) - dp(717)
dp(720) = dp(41)*p(73) + p(41)*dp(73) - dp(701)
dp(721) = dp(6)*p(279) + p(6)*dp(279) - dp(613)
dp(722) = dp(16)*p(150) + p(16)*dp(150) - dp(701)
dp(723) = dp(1)*p(425) + p(1)*dp(425) - dp(624) - dp(721)
dp(724) = dp(4)*p(285) + p(4)*dp(285) - dp(701)
dp(725) = dp(11)*p(150) + p(11)*dp(150) - dp(592)
dp(726) = dp(14)*p(181) + p(14)*dp(181)
dp(727) = dp(14)*p(182) + p(14)*dp(182)
dp(728) = dp(1)*p(430) + p(1)*dp(430) - dp(632)
dp(729) = dp(21)*p(130) + p(21)*dp(130) - dp(635)
dp(730) = dp(3)*p(295) + p(3)*dp(295) - dp(633) - dp(729)
dp(731) = dp(1)*p(432) + p(1)*dp(432) - dp(633) - dp(730)
dp(732) = dp(39)*p(81) + p(39)*dp(81) - dp(632)
dp(733) = dp(3)*p(296) + p(3)*dp(296) - dp(635)
dp(734) = dp(14)*p(149) + p(14)*dp(149)
dp(735) = dp(14)*p(150) + p(14)*dp(150)
dp(736) = dp(14)*p(119) + p(14)*dp(119)
dp(737) = dp(9)*p(185) + p(9)*dp(185)
dp(738) = dp(9)*p(162) + p(9)*dp(162)
dp(739) = dp(9)*p(134) + p(9)*dp(134)
dp(740) = dp(9)*p(163) + p(9)*dp(163)
dp(741) = dp(9)*p(164) + p(9)*dp(164)
dp(742) = dp(9)*p(186) + p(9)*dp(186)
dp(743) = dp(24)*p(93) + p(24)*dp(93) - dp(593)
dp(744) = dp(18)*p(114) + p(18)*dp(114) - dp(595)
dp(745) = dp(1)*p(443) + p(1)*dp(443) - dp(643) - dp(744)
dp(746) = dp(1)*p(444) + p(1)*dp(444) - dp(644) - dp(745)
dp(747) = dp(13)*p(183) + p(13)*dp(183) - dp(726)
dp(748) = dp(2)*p(421) + p(2)*dp(421) - dp(631) - dp(730)
dp(749) = dp(32)*p(93) + p(32)*dp(93) - dp(726)
dp(750) = dp(1)*p(446) + p(1)*dp(446) - dp(644) - dp(748)
dp(751) = dp(43)*p(73) + p(43)*dp(73) - dp(726)
dp(752) = dp(13)*p(153) + p(13)*dp(153) - dp(595)
dp(753) = dp(18)*p(150) + p(18)*dp(150) - dp(726)
dp(754) = dp(34)*p(73) + p(34)*dp(73) - dp(593)
dp(755) = dp(14)*p(183) + p(14)*dp(183)
dp(756) = dp(14)*p(184) + p(14)*dp(184)
dp(757) = dp(16)*p(130) + p(16)*dp(130) - dp(598)
dp(758) = dp(2)*p(430) + p(2)*dp(430) - dp(636)
dp(759) = dp(1)*p(454) + p(1)*dp(454) - dp(654) - dp(758)
dp(760) = dp(41)*p(81) + p(41)*dp(81) - dp(735)
dp(761) = dp(2)*p(432) + p(2)*dp(432) - dp(638) - dp(760)
dp(762) = dp(16)*p(160) + p(16)*dp(160) - dp(735)
dp(763) = dp(45)*p(69) + p(45)*dp(69) - dp(636)
dp(764) = dp(4)*p(296) + p(4)*dp(296) - dp(735)
dp(765) = dp(11)*p(160) + p(11)*dp(160) - dp(598)
dp(766) = dp(14)*p(156) + p(14)*dp(156)
dp(767) = dp(14)*p(157) + p(14)*dp(157)
dp(768) = dp(14)*p(158) + p(14)*dp(158)
dp(769) = dp(14)*p(128) + p(14)*dp(128)
dp(770) = dp(14)*p(129) + p(14)*dp(129)
dp(771) = dp(1)*p(464) + p(1)*dp(464) - dp(664)
dp(772) = dp(4)*p(262) + p(4)*dp(262) - dp(673) - dp(661)
dp(773) = dp(3)*p(310) + p(3)*dp(310) - dp(664)
dp(774) = dp(14)*p(159) + p(14)*dp(159)
dp(775) = dp(14)*p(160) + p(14)*dp(160)
dp(776) = dp(14)*p(132) + p(14)*dp(132)
dp(777) = dp(14)*p(133) + p(14)*dp(133)
dp(778) = dp(9)*p(188) + p(9)*dp(188)
dp(779) = dp(9)*p(172) + p(9)*dp(172)
dp(780) = dp(9)*p(173) + p(9)*dp(173)
dp(781) = dp(9)*p(189) + p(9)*dp(189)
dp(782) = dp(24)*p(95) + p(24)*dp(95) - dp(600)
dp(783) = dp(22)*p(114) + p(22)*dp(114) - dp(650)
dp(784) = dp(1)*p(471) + p(1)*dp(471) - dp(668) - dp(783)
dp(785) = dp(13)*p(185) + p(13)*dp(185) - dp(755)
dp(786) = dp(2)*p(446) + p(2)*dp(446) - dp(652) - dp(761)
dp(787) = dp(32)*p(95) + p(32)*dp(95) - dp(755)
dp(788) = dp(6)*p(299) + p(6)*dp(299) - dp(650)
dp(789) = dp(46)*p(73) + p(46)*dp(73) - dp(755)
dp(790) = dp(32)*p(84) + p(32)*dp(84) - dp(600)
dp(791) = dp(14)*p(185) + p(14)*dp(185)
dp(792) = dp(14)*p(186) + p(14)*dp(186)
dp(793) = dp(2)*p(453) + p(2)*dp(453) - dp(656) - dp(771)
dp(794) = dp(1)*p(478) + p(1)*dp(478) - dp(673) - dp(793)
dp(795) = dp(43)*p(81) + p(43)*dp(81) - dp(736)
dp(796) = dp(2)*p(456) + p(2)*dp(456) - dp(659) - dp(772)
dp(797) = dp(18)*p(160) + p(18)*dp(160) - dp(736)
dp(798) = dp(1)*p(480) + p(1)*dp(480) - dp(673) - dp(796)
dp(799) = dp(14)*p(165) + p(14)*dp(165)
dp(800) = dp(14)*p(166) + p(14)*dp(166)
dp(801) = dp(14)*p(167) + p(14)*dp(167)
dp(802) = dp(14)*p(138) + p(14)*dp(138)
dp(803) = dp(14)*p(139) + p(14)*dp(139)
dp(804) = dp(2)*p(464) + p(2)*dp(464) - dp(665)
dp(805) = dp(4)*p(310) + p(4)*dp(310) - dp(775)
dp(806) = dp(11)*p(187) + p(11)*dp(187) - dp(665)
dp(807) = dp(14)*p(168) + p(14)*dp(168)
dp(808) = dp(14)*p(169) + p(14)*dp(169)
dp(809) = dp(14)*p(170) + p(14)*dp(170)
dp(810) = dp(9)*p(191) + p(9)*dp(191)
dp(811) = dp(9)*p(192) + p(9)*dp(192)
dp(812) = dp(24)*p(98) + p(24)*dp(98) - dp(603)
dp(813) = dp(1)*p(485) + p(1)*dp(485) - dp(677) - dp(812)
dp(814) = dp(13)*p(188) + p(13)*dp(188) - dp(791)
dp(815) = dp(2)*p(473) + p(2)*dp(473) - dp(672) - dp(796)
dp(816) = dp(32)*p(98) + p(32)*dp(98) - dp(791)
dp(817) = dp(13)*p(173) + p(13)*dp(173) - dp(603)
dp(818) = dp(14)*p(188) + p(14)*dp(188)
dp(819) = dp(14)*p(189) + p(14)*dp(189)
dp(820) = dp(22)*p(130) + p(22)*dp(130) - dp(663)
dp(821) = dp(46)*p(81) + p(46)*dp(81) - dp(769)
dp(822) = dp(45)*p(84) + p(45)*dp(84) - dp(663)
dp(823) = dp(14)*p(174) + p(14)*dp(174)
dp(824) = dp(14)*p(175) + p(14)*dp(175)
dp(825) = dp(14)*p(176) + p(14)*dp(176)
dp(826) = dp(9)*p(195) + p(9)*dp(195)
dp(827) = dp(24)*p(102) + p(24)*dp(102) - dp(679)
dp(828) = dp(13)*p(191) + p(13)*dp(191) - dp(818)
dp(829) = dp(6)*p(323) + p(6)*dp(323) - dp(679)
dp(830) = dp(14)*p(191) + p(14)*dp(191)
dp(831) = dp(14)*p(192) + p(14)*dp(192)
dp(832) = dp(9)*p(140) + p(9)*dp(140)
dp(833) = dp(9)*p(141) + p(9)*dp(141)
dp(834) = dp(9)*p(178) + p(9)*dp(178)
dp(835) = dp(9)*p(142) + p(9)*dp(142)
dp(836) = dp(9)*p(179) + p(9)*dp(179)
dp(837) = dp(9)*p(180) + p(9)*dp(180)
dp(838) = dp(9)*p(194) + p(9)*dp(194)
dp(839) = dp(3)*p(331) + p(3)*dp(331) - dp(680)
dp(840) = dp(39)*p(91) + p(39)*dp(91) - dp(682)
dp(841) = dp(16)*p(178) + p(16)*dp(178) - dp(685)
dp(842) = dp(1)*p(497) + p(1)*dp(497) - dp(683) - dp(841)
dp(843) = dp(2)*p(566) + p(2)*dp(566) - dp(842)
dp(844) = dp(1)*p(498) + p(1)*dp(498) - dp(686) - dp(842)
dp(845) = dp(7)*p(279) + p(7)*dp(279) - dp(685)
dp(846) = dp(1)*p(500) + p(1)*dp(500) - dp(689) - dp(844)
dp(847) = dp(49)*p(69) + p(49)*dp(69) - dp(682)
dp(848) = dp(1)*p(502) + p(1)*dp(502) - dp(691) - dp(846)
dp(849) = dp(11)*p(194) + p(11)*dp(194) - dp(680)
dp(850) = dp(1)*p(504) + p(1)*dp(504) - dp(695) - dp(694)
dp(851) = dp(1)*p(505) + p(1)*dp(505) - dp(700)
dp(852) = dp(3)*p(333) + p(3)*dp(333) - dp(702)
dp(853) = dp(39)*p(93) + p(39)*dp(93) - dp(704)
dp(854) = dp(18)*p(178) + p(18)*dp(178) - dp(707)
dp(855) = dp(1)*p(508) + p(1)*dp(508) - dp(705) - dp(854)
dp(856) = dp(1)*p(509) + p(1)*dp(509) - dp(707)
dp(857) = dp(1)*p(510) + p(1)*dp(510) - dp(708) - dp(855)
dp(858) = dp(21)*p(153) + p(21)*dp(153) - dp(704)
dp(859) = dp(1)*p(512) + p(1)*dp(512) - dp(710) - dp(857)
dp(860) = dp(34)*p(101) + p(34)*dp(101) - dp(702)
dp(861) = dp(6)*p(331) + p(6)*dp(331) - dp(701)
dp(862) = dp(1)*p(515) + p(1)*dp(515) - dp(717) - dp(715)
dp(863) = dp(2)*p(505) + p(2)*dp(505) - dp(701)
dp(864) = dp(1)*p(517) + p(1)*dp(517) - dp(730) - dp(729)
dp(865) = dp(1)*p(518) + p(1)*dp(518) - dp(733)
dp(866) = dp(3)*p(335) + p(3)*dp(335) - dp(737)
dp(867) = dp(39)*p(95) + p(39)*dp(95) - dp(739)
dp(868) = dp(1)*p(520) + p(1)*dp(520) - dp(738) - dp(867)
dp(869) = dp(2)*p(509) + p(2)*dp(509) - dp(721)
dp(870) = dp(1)*p(521) + p(1)*dp(521) - dp(740) - dp(868)
dp(871) = dp(7)*p(299) + p(7)*dp(299) - dp(739)
dp(872) = dp(1)*p(523) + p(1)*dp(523) - dp(742) - dp(870)
dp(873) = dp(49)*p(84) + p(49)*dp(84) - dp(737)
dp(874) = dp(6)*p(333) + p(6)*dp(333) - dp(726)
dp(875) = dp(1)*p(526) + p(1)*dp(526) - dp(748) - dp(746)
dp(876) = dp(8)*p(285) + p(8)*dp(285) - dp(726)
dp(877) = dp(45)*p(91) + p(45)*dp(91) - dp(735)
dp(878) = dp(1)*p(529) + p(1)*dp(529) - dp(761) - dp(759)
dp(879) = dp(2)*p(518) + p(2)*dp(518) - dp(735)
dp(880) = dp(1)*p(531) + p(1)*dp(531) - dp(772) - dp(771)
dp(881) = dp(1)*p(532) + p(1)*dp(532) - dp(773)
dp(882) = dp(14)*p(161) + p(14)*dp(161)
dp(883) = dp(3)*p(337) + p(3)*dp(337) - dp(778)
dp(884) = dp(39)*p(98) + p(39)*dp(98) - dp(780)
dp(885) = dp(1)*p(535) + p(1)*dp(535) - dp(779) - dp(884)
dp(886) = dp(1)*p(536) + p(1)*dp(536) - dp(780)
dp(887) = dp(1)*p(537) + p(1)*dp(537) - dp(781) - dp(885)
dp(888) = dp(21)*p(173) + p(21)*dp(173) - dp(778)
dp(889) = dp(6)*p(335) + p(6)*dp(335) - dp(755)
dp(890) = dp(1)*p(540) + p(1)*dp(540) - dp(786) - dp(784)
dp(891) = dp(22)*p(150) + p(22)*dp(150) - dp(755)
dp(892) = dp(45)*p(93) + p(45)*dp(93) - dp(736)
dp(893) = dp(1)*p(543) + p(1)*dp(543) - dp(796) - dp(794)
dp(894) = dp(8)*p(296) + p(8)*dp(296) - dp(736)
dp(895) = dp(16)*p(187) + p(16)*dp(187) - dp(775)
dp(896) = dp(1)*p(547) + p(1)*dp(547) - dp(806) - dp(804)
dp(897) = dp(2)*p(532) + p(2)*dp(532) - dp(775)
dp(898) = dp(14)*p(171) + p(14)*dp(171)
dp(899) = dp(6)*p(309) + p(6)*dp(309) - dp(808) - dp(807)
dp(900) = dp(1)*p(578) + p(1)*dp(578) - dp(899)
dp(901) = dp(14)*p(187) + p(14)*dp(187)
dp(902) = dp(3)*p(340) + p(3)*dp(340) - dp(810)
dp(903) = dp(2)*p(535) + p(2)*dp(535) - dp(787) - dp(783)
dp(904) = dp(2)*p(536) + p(2)*dp(536) - dp(788)
dp(905) = dp(1)*p(551) + p(1)*dp(551) - dp(811) - dp(903)
dp(906) = dp(7)*p(323) + p(7)*dp(323) - dp(810)
dp(907) = dp(6)*p(337) + p(6)*dp(337) - dp(791)
dp(908) = dp(1)*p(554) + p(1)*dp(554) - dp(815) - dp(813)
dp(909) = dp(50)*p(73) + p(50)*dp(73) - dp(791)
dp(910) = dp(45)*p(95) + p(45)*dp(95) - dp(769)
dp(911) = dp(1)*p(557) + p(1)*dp(557) - dp(822) - dp(820)
dp(912) = dp(22)*p(160) + p(22)*dp(160) - dp(769)
dp(913) = dp(14)*p(177) + p(14)*dp(177)
dp(914) = dp(18)*p(187) + p(18)*dp(187) - dp(777)
dp(915) = dp(2)*p(547) + p(2)*dp(547) - dp(808) - dp(899)
dp(916) = dp(8)*p(310) + p(8)*dp(310) - dp(777)
dp(917) = dp(14)*p(190) + p(14)*dp(190)
dp(918) = dp(3)*p(343) + p(3)*dp(343) - dp(826)
dp(919) = dp(2)*p(551) + p(2)*dp(551) - dp(816) - dp(813)
dp(920) = dp(1)*p(561) + p(1)*dp(561) - dp(826)
dp(921) = dp(6)*p(340) + p(6)*dp(340) - dp(818)
dp(922) = dp(1)*p(563) + p(1)*dp(563) - dp(829) - dp(827)
dp(923) = dp(32)*p(102) + p(32)*dp(102) - dp(818)
dp(924) = dp(45)*p(98) + p(45)*dp(98) - dp(802)
dp(925) = dp(2)*p(557) + p(2)*dp(557) - dp(824) - dp(915)
dp(926) = dp(50)*p(81) + p(50)*dp(81) - dp(802)
dp(927) = dp(14)*p(193) + p(14)*dp(193)
dp(928) = dp(2)*p(560) + p(2)*dp(560) - dp(828) - dp(827)
dp(929) = dp(2)*p(561) + p(2)*dp(561) - dp(829)
dp(930) = dp(6)*p(343) + p(6)*dp(343) - dp(830)
dp(931) = dp(2)*p(563) + p(2)*dp(563) - dp(831) - dp(925)
dp(932) = dp(13)*p(195) + p(13)*dp(195) - dp(830)
dp(933) = dp(14)*p(195) + p(14)*dp(195)
dp(934) = dp(1)*p(566) + p(1)*dp(566) - dp(834)
dp(935) = dp(1)*p(567) + p(1)*dp(567) - dp(836) - dp(934)
dp(936) = dp(39)*p(101) + p(39)*dp(101) - dp(835)
dp(937) = dp(3)*p(346) + p(3)*dp(346) - dp(838)
dp(938) = dp(1)*p(570) + p(1)*dp(570) - dp(839)
dp(939) = dp(2)*p(588) + p(2)*dp(588) - dp(938)
dp(940) = dp(1)*p(572) + p(1)*dp(572) - dp(852)
dp(941) = dp(8)*p(346) + p(8)*dp(346) - dp(940)
dp(942) = dp(1)*p(574) + p(1)*dp(574) - dp(866)
dp(943) = dp(22)*p(194) + p(22)*dp(194) - dp(942)
dp(944) = dp(1)*p(576) + p(1)*dp(576) - dp(883)
dp(945) = dp(50)*p(101) + p(50)*dp(101) - dp(944)
dp(946) = dp(1)*p(579) + p(1)*dp(579) - dp(902)
dp(947) = dp(49)*p(102) + p(49)*dp(102) - dp(946)
dp(948) = dp(2)*p(578) + p(2)*dp(578) - dp(901)
dp(949) = dp(1)*p(582) + p(1)*dp(582) - dp(918)
dp(950) = dp(21)*p(195) + p(21)*dp(195) - dp(949)
dp(951) = dp(2)*p(581) + p(2)*dp(581) - dp(917) - dp(948)
dp(952) = dp(1)*p(585) + p(1)*dp(585) - dp(928)
dp(953) = dp(7)*p(347) + p(7)*dp(347) - dp(952)
dp(954) = dp(45)*p(102) + p(45)*dp(102) - dp(913)
dp(955) = dp(2)*p(585) + p(2)*dp(585) - dp(930)
dp(956) = dp(1)*p(589) + p(1)*dp(589) - dp(955)
dp(957) = dp(6)*p(347) + p(6)*dp(347) - dp(933)
dp(958) = dp(1)*p(588) + p(1)*dp(588) - dp(937)
dp(959) = dp(2)*p(589) + p(2)*dp(589) - dp(957)
dp(960) = dp(9)*p(196) + p(9)*dp(196)
dp(961) = dp(9)*p(202) + p(9)*dp(202)
dp(962) = dp(9)*p(203) + p(9)*dp(203)
dp(963) = dp(9)*p(228) + p(9)*dp(228)
dp(964) = dp(9)*p(242) + p(9)*dp(242)
dp(965) = dp(9)*p(207) + p(9)*dp(207)
dp(966) = dp(9)*p(197) + p(9)*dp(197)
dp(967) = dp(9)*p(208) + p(9)*dp(208)
dp(968) = dp(9)*p(209) + p(9)*dp(209)
dp(969) = dp(9)*p(243) + p(9)*dp(243)
dp(970) = dp(9)*p(211) + p(9)*dp(211)
dp(971) = dp(9)*p(247) + p(9)*dp(247)
dp(972) = dp(9)*p(212) + p(9)*dp(212)
dp(973) = dp(9)*p(248) + p(9)*dp(248)
dp(974) = dp(9)*p(249) + p(9)*dp(249)
dp(975) = dp(9)*p(258) + p(9)*dp(258)
dp(976) = dp(9)*p(214) + p(9)*dp(214)
dp(977) = dp(9)*p(215) + p(9)*dp(215)
dp(978) = dp(9)*p(259) + p(9)*dp(259)
dp(979) = dp(9)*p(263) + p(9)*dp(263)
dp(980) = dp(9)*p(264) + p(9)*dp(264)
dp(981) = dp(9)*p(265) + p(9)*dp(265)
dp(982) = dp(9)*p(308) + p(9)*dp(308)
dp(983) = dp(9)*p(311) + p(9)*dp(311)
dp(984) = dp(9)*p(270) + p(9)*dp(270)
dp(985) = dp(9)*p(271) + p(9)*dp(271)
dp(986) = dp(9)*p(321) + p(9)*dp(321)
dp(987) = dp(9)*p(327) + p(9)*dp(327)
dp(988) = dp(9)*p(222) + p(9)*dp(222)
dp(989) = dp(9)*p(223) + p(9)*dp(223)
dp(990) = dp(9)*p(284) + p(9)*dp(284)
dp(991) = dp(9)*p(198) + p(9)*dp(198)
dp(992) = dp(9)*p(199) + p(9)*dp(199)
dp(993) = dp(9)*p(224) + p(9)*dp(224)
dp(994) = dp(9)*p(200) + p(9)*dp(200)
dp(995) = dp(9)*p(225) + p(9)*dp(225)
dp(996) = dp(9)*p(201) + p(9)*dp(201)
dp(997) = dp(9)*p(226) + p(9)*dp(226)
dp(998) = dp(9)*p(227) + p(9)*dp(227)
dp(999) = dp(9)*p(285) + p(9)*dp(285)
dp(1000) = dp(14)*p(328) + p(14)*dp(328)
dp(1001) = dp(14)*p(329) + p(14)*dp(329)
dp(1002) = dp(14)*p(330) + p(14)*dp(330)
dp(1003) = dp(9)*p(292) + p(9)*dp(292)
dp(1004) = dp(9)*p(233) + p(9)*dp(233)
dp(1005) = dp(9)*p(234) + p(9)*dp(234)
dp(1006) = dp(9)*p(235) + p(9)*dp(235)
dp(1007) = dp(9)*p(293) + p(9)*dp(293)
dp(1008) = dp(9)*p(236) + p(9)*dp(236)
dp(1009) = dp(9)*p(204) + p(9)*dp(204)
dp(1010) = dp(9)*p(205) + p(9)*dp(205)
dp(1011) = dp(9)*p(237) + p(9)*dp(237)
dp(1012) = dp(9)*p(238) + p(9)*dp(238)
dp(1013) = dp(9)*p(206) + p(9)*dp(206)
dp(1014) = dp(9)*p(239) + p(9)*dp(239)
dp(1015) = dp(9)*p(240) + p(9)*dp(240)
dp(1016) = dp(9)*p(241) + p(9)*dp(241)
dp(1017) = dp(9)*p(294) + p(9)*dp(294)
dp(1018) = dp(14)*p(276) + p(14)*dp(276)
dp(1019) = dp(14)*p(277) + p(14)*dp(277)
dp(1020) = dp(14)*p(278) + p(14)*dp(278)
dp(1021) = dp(14)*p(279) + p(14)*dp(279)
dp(1022) = dp(14)*p(280) + p(14)*dp(280)
dp(1023) = dp(14)*p(281) + p(14)*dp(281)
dp(1024) = dp(14)*p(282) + p(14)*dp(282)
dp(1025) = dp(14)*p(283) + p(14)*dp(283)
dp(1026) = dp(9)*p(244) + p(9)*dp(244)
dp(1027) = dp(9)*p(295) + p(9)*dp(295)
dp(1028) = dp(9)*p(210) + p(9)*dp(210)
dp(1029) = dp(9)*p(245) + p(9)*dp(245)
dp(1030) = dp(9)*p(246) + p(9)*dp(246)
dp(1031) = dp(9)*p(296) + p(9)*dp(296)
dp(1032) = dp(14)*p(222) + p(14)*dp(222)
dp(1033) = dp(14)*p(223) + p(14)*dp(223)
dp(1034) = dp(14)*p(224) + p(14)*dp(224)
dp(1035) = dp(14)*p(225) + p(14)*dp(225)
dp(1036) = dp(14)*p(226) + p(14)*dp(226)
dp(1037) = dp(14)*p(227) + p(14)*dp(227)
dp(1038) = dp(14)*p(202) + p(14)*dp(202)
dp(1039) = dp(14)*p(203) + p(14)*dp(203)
dp(1040) = dp(9)*p(302) + p(9)*dp(302)
dp(1041) = dp(9)*p(252) + p(9)*dp(252)
dp(1042) = dp(9)*p(253) + p(9)*dp(253)
dp(1043) = dp(9)*p(303) + p(9)*dp(303)
dp(1044) = dp(9)*p(254) + p(9)*dp(254)
dp(1045) = dp(9)*p(213) + p(9)*dp(213)
dp(1046) = dp(9)*p(255) + p(9)*dp(255)
dp(1047) = dp(9)*p(256) + p(9)*dp(256)
dp(1048) = dp(9)*p(257) + p(9)*dp(257)
dp(1049) = dp(9)*p(304) + p(9)*dp(304)
dp(1050) = dp(14)*p(286) + p(14)*dp(286)
dp(1051) = dp(14)*p(287) + p(14)*dp(287)
dp(1052) = dp(14)*p(288) + p(14)*dp(288)
dp(1053) = dp(14)*p(289) + p(14)*dp(289)
dp(1054) = dp(14)*p(290) + p(14)*dp(290)
dp(1055) = dp(14)*p(291) + p(14)*dp(291)
dp(1056) = dp(9)*p(305) + p(9)*dp(305)
dp(1057) = dp(9)*p(260) + p(9)*dp(260)
dp(1058) = dp(9)*p(306) + p(9)*dp(306)
dp(1059) = dp(9)*p(261) + p(9)*dp(261)
dp(1060) = dp(9)*p(262) + p(9)*dp(262)
dp(1061) = dp(9)*p(307) + p(9)*dp(307)
dp(1062) = dp(14)*p(233) + p(14)*dp(233)
dp(1063) = dp(14)*p(234) + p(14)*dp(234)
dp(1064) = dp(14)*p(235) + p(14)*dp(235)
dp(1065) = dp(14)*p(236) + p(14)*dp(236)
dp(1066) = dp(14)*p(237) + p(14)*dp(237)
dp(1067) = dp(14)*p(238) + p(14)*dp(238)
dp(1068) = dp(14)*p(239) + p(14)*dp(239)
dp(1069) = dp(14)*p(240) + p(14)*dp(240)
dp(1070) = dp(14)*p(241) + p(14)*dp(241)
dp(1071) = dp(14)*p(207) + p(14)*dp(207)
dp(1072) = dp(14)*p(208) + p(14)*dp(208)
dp(1073) = dp(14)*p(209) + p(14)*dp(209)
dp(1074) = dp(9)*p(309) + p(9)*dp(309)
dp(1075) = dp(9)*p(310) + p(9)*dp(310)
dp(1076) = dp(14)*p(244) + p(14)*dp(244)
dp(1077) = dp(14)*p(245) + p(14)*dp(245)
dp(1078) = dp(14)*p(246) + p(14)*dp(246)
dp(1079) = dp(14)*p(211) + p(14)*dp(211)
dp(1080) = dp(14)*p(212) + p(14)*dp(212)
dp(1081) = dp(9)*p(315) + p(9)*dp(315)
dp(1082) = dp(9)*p(267) + p(9)*dp(267)
dp(1083) = dp(9)*p(316) + p(9)*dp(316)
dp(1084) = dp(9)*p(268) + p(9)*dp(268)
dp(1085) = dp(9)*p(269) + p(9)*dp(269)
dp(1086) = dp(9)*p(317) + p(9)*dp(317)
dp(1087) = dp(14)*p(297) + p(14)*dp(297)
dp(1088) = dp(14)*p(298) + p(14)*dp(298)
dp(1089) = dp(14)*p(299) + p(14)*dp(299)
dp(1090) = dp(14)*p(300) + p(14)*dp(300)
dp(1091) = dp(14)*p(301) + p(14)*dp(301)
dp(1092) = dp(9)*p(318) + p(9)*dp(318)
dp(1093) = dp(9)*p(319) + p(9)*dp(319)
dp(1094) = dp(9)*p(320) + p(9)*dp(320)
dp(1095) = dp(14)*p(252) + p(14)*dp(252)
dp(1096) = dp(14)*p(253) + p(14)*dp(253)
dp(1097) = dp(14)*p(254) + p(14)*dp(254)
dp(1098) = dp(14)*p(255) + p(14)*dp(255)
dp(1099) = dp(14)*p(256) + p(14)*dp(256)
dp(1100) = dp(14)*p(257) + p(14)*dp(257)
dp(1101) = dp(14)*p(214) + p(14)*dp(214)
dp(1102) = dp(14)*p(215) + p(14)*dp(215)
dp(1103) = dp(9)*p(339) + p(9)*dp(339)
dp(1104) = dp(14)*p(260) + p(14)*dp(260)
dp(1105) = dp(14)*p(261) + p(14)*dp(261)
dp(1106) = dp(14)*p(262) + p(14)*dp(262)
dp(1107) = dp(9)*p(324) + p(9)*dp(324)
dp(1108) = dp(9)*p(325) + p(9)*dp(325)
dp(1109) = dp(9)*p(326) + p(9)*dp(326)
dp(1110) = dp(14)*p(312) + p(14)*dp(312)
dp(1111) = dp(14)*p(313) + p(14)*dp(313)
dp(1112) = dp(14)*p(314) + p(14)*dp(314)
dp(1113) = dp(9)*p(342) + p(9)*dp(342)
dp(1114) = dp(14)*p(267) + p(14)*dp(267)
dp(1115) = dp(14)*p(268) + p(14)*dp(268)
dp(1116) = dp(14)*p(269) + p(14)*dp(269)
dp(1117) = dp(9)*p(345) + p(9)*dp(345)
dp(1118) = dp(14)*p(322) + p(14)*dp(322)
dp(1119) = dp(14)*p(323) + p(14)*dp(323)
dp(1120) = dp(9)*p(331) + p(9)*dp(331)
dp(1121) = dp(9)*p(276) + p(9)*dp(276)
dp(1122) = dp(9)*p(216) + p(9)*dp(216)
dp(1123) = dp(9)*p(277) + p(9)*dp(277)
dp(1124) = dp(9)*p(217) + p(9)*dp(217)
dp(1125) = dp(9)*p(218) + p(9)*dp(218)
dp(1126) = dp(9)*p(278) + p(9)*dp(278)
dp(1127) = dp(9)*p(219) + p(9)*dp(219)
dp(1128) = dp(9)*p(220) + p(9)*dp(220)
dp(1129) = dp(9)*p(279) + p(9)*dp(279)
dp(1130) = dp(9)*p(280) + p(9)*dp(280)
dp(1131) = dp(9)*p(221) + p(9)*dp(221)
dp(1132) = dp(9)*p(281) + p(9)*dp(281)
dp(1133) = dp(9)*p(282) + p(9)*dp(282)
dp(1134) = dp(9)*p(283) + p(9)*dp(283)
dp(1135) = dp(9)*p(332) + p(9)*dp(332)
dp(1136) = dp(3)*p(401) + p(3)*dp(401) - dp(988)
dp(1137) = dp(7)*p(401) + p(7)*dp(401) - dp(991)
dp(1138) = dp(49)*p(114) + p(49)*dp(114) - dp(996)
dp(1139) = dp(24)*p(194) + p(24)*dp(194) - dp(999)
dp(1140) = dp(3)*p(504) + p(3)*dp(504) - dp(990) - dp(1139)
dp(1141) = dp(1)*p(695) + p(1)*dp(695) - dp(990) - dp(1140)
dp(1142) = dp(1)*p(696) + p(1)*dp(696) - dp(993) - dp(1141)
dp(1143) = dp(6)*p(566) + p(6)*dp(566) - dp(1136)
dp(1144) = dp(32)*p(178) + p(32)*dp(178) - dp(991)
dp(1145) = dp(39)*p(150) + p(39)*dp(150) - dp(996)
dp(1146) = dp(3)*p(505) + p(3)*dp(505) - dp(999)
dp(1147) = dp(14)*p(346) + p(14)*dp(346)
dp(1148) = dp(9)*p(333) + p(9)*dp(333)
dp(1149) = dp(9)*p(286) + p(9)*dp(286)
dp(1150) = dp(9)*p(229) + p(9)*dp(229)
dp(1151) = dp(9)*p(287) + p(9)*dp(287)
dp(1152) = dp(9)*p(230) + p(9)*dp(230)
dp(1153) = dp(9)*p(231) + p(9)*dp(231)
dp(1154) = dp(9)*p(288) + p(9)*dp(288)
dp(1155) = dp(9)*p(232) + p(9)*dp(232)
dp(1156) = dp(9)*p(289) + p(9)*dp(289)
dp(1157) = dp(9)*p(290) + p(9)*dp(290)
dp(1158) = dp(9)*p(291) + p(9)*dp(291)
dp(1159) = dp(9)*p(334) + p(9)*dp(334)
dp(1160) = dp(24)*p(181) + p(24)*dp(181) - dp(963)
dp(1161) = dp(41)*p(114) + p(41)*dp(114) - dp(960)
dp(1162) = dp(4)*p(401) + p(4)*dp(401) - dp(961)
dp(1163) = dp(1)*p(713) + p(1)*dp(713) - dp(1005) - dp(1162)
dp(1164) = dp(1)*p(714) + p(1)*dp(714) - dp(1006) - dp(1163)
dp(1165) = dp(1)*p(715) + p(1)*dp(715) - dp(1007) - dp(1164)
dp(1166) = dp(13)*p(331) + p(13)*dp(331) - dp(1147)
dp(1167) = dp(2)*p(695) + p(2)*dp(695) - dp(1002) - dp(1166)
dp(1168) = dp(32)*p(181) + p(32)*dp(181) - dp(1147)
dp(1169) = dp(1)*p(717) + p(1)*dp(717) - dp(1007) - dp(1167)
dp(1170) = dp(73)*p(91) + p(73)*dp(91) - dp(1147)
dp(1171) = dp(1)*p(719) + p(1)*dp(719) - dp(1011) - dp(1169)
dp(1172) = dp(41)*p(150) + p(41)*dp(150) - dp(1147)
dp(1173) = dp(13)*p(279) + p(13)*dp(279) - dp(961)
dp(1174) = dp(16)*p(285) + p(16)*dp(285) - dp(1147)
dp(1175) = dp(69)*p(73) + p(69)*dp(73) - dp(960)
dp(1176) = dp(4)*p(505) + p(4)*dp(505) - dp(1147)
dp(1177) = dp(11)*p(285) + p(11)*dp(285) - dp(963)
dp(1178) = dp(14)*p(331) + p(14)*dp(331)
dp(1179) = dp(14)*p(332) + p(14)*dp(332)
dp(1180) = dp(3)*p(430) + p(3)*dp(430) - dp(1026)
dp(1181) = dp(7)*p(430) + p(7)*dp(430) - dp(1028)
dp(1182) = dp(49)*p(130) + p(49)*dp(130) - dp(1031)
dp(1183) = dp(3)*p(517) + p(3)*dp(517) - dp(1027) - dp(1182)
dp(1184) = dp(1)*p(730) + p(1)*dp(730) - dp(1027) - dp(1183)
dp(1185) = dp(45)*p(178) + p(45)*dp(178) - dp(1180)
dp(1186) = dp(39)*p(160) + p(39)*dp(160) - dp(1028)
dp(1187) = dp(3)*p(518) + p(3)*dp(518) - dp(1031)
dp(1188) = dp(14)*p(284) + p(14)*dp(284)
dp(1189) = dp(14)*p(285) + p(14)*dp(285)
dp(1190) = dp(14)*p(228) + p(14)*dp(228)
dp(1191) = dp(9)*p(335) + p(9)*dp(335)
dp(1192) = dp(9)*p(297) + p(9)*dp(297)
dp(1193) = dp(9)*p(250) + p(9)*dp(250)
dp(1194) = dp(9)*p(298) + p(9)*dp(298)
dp(1195) = dp(9)*p(251) + p(9)*dp(251)
dp(1196) = dp(9)*p(299) + p(9)*dp(299)
dp(1197) = dp(9)*p(300) + p(9)*dp(300)
dp(1198) = dp(9)*p(301) + p(9)*dp(301)
dp(1199) = dp(9)*p(336) + p(9)*dp(336)
dp(1200) = dp(24)*p(183) + p(24)*dp(183) - dp(964)
dp(1201) = dp(43)*p(114) + p(43)*dp(114) - dp(966)
dp(1202) = dp(8)*p(401) + p(8)*dp(401) - dp(1021)
dp(1203) = dp(1)*p(745) + p(1)*dp(745) - dp(1042) - dp(1202)
dp(1204) = dp(1)*p(746) + p(1)*dp(746) - dp(1043) - dp(1203)
dp(1205) = dp(13)*p(333) + p(13)*dp(333) - dp(1178)
dp(1206) = dp(2)*p(717) + p(2)*dp(717) - dp(1025) - dp(1183)
dp(1207) = dp(32)*p(183) + p(32)*dp(183) - dp(1178)
dp(1208) = dp(1)*p(748) + p(1)*dp(748) - dp(1043) - dp(1206)
dp(1209) = dp(73)*p(93) + p(73)*dp(93) - dp(1178)
dp(1210) = dp(6)*p(509) + p(6)*dp(509) - dp(1021)
dp(1211) = dp(43)*p(150) + p(43)*dp(150) - dp(1178)
dp(1212) = dp(32)*p(153) + p(32)*dp(153) - dp(966)
dp(1213) = dp(18)*p(285) + p(18)*dp(285) - dp(1178)
dp(1214) = dp(34)*p(150) + p(34)*dp(150) - dp(964)
dp(1215) = dp(14)*p(333) + p(14)*dp(333)
dp(1216) = dp(14)*p(334) + p(14)*dp(334)
dp(1217) = dp(41)*p(130) + p(41)*dp(130) - dp(973)
dp(1218) = dp(4)*p(430) + p(4)*dp(430) - dp(970)
dp(1219) = dp(1)*p(758) + p(1)*dp(758) - dp(1057) - dp(1218)
dp(1220) = dp(1)*p(759) + p(1)*dp(759) - dp(1058) - dp(1219)
dp(1221) = dp(81)*p(91) + p(81)*dp(91) - dp(1189)
dp(1222) = dp(2)*p(730) + p(2)*dp(730) - dp(1034) - dp(1221)
dp(1223) = dp(41)*p(160) + p(41)*dp(160) - dp(1189)
dp(1224) = dp(1)*p(761) + p(1)*dp(761) - dp(1058) - dp(1222)
dp(1225) = dp(16)*p(296) + p(16)*dp(296) - dp(1189)
dp(1226) = dp(69)*p(81) + p(69)*dp(81) - dp(970)
dp(1227) = dp(4)*p(518) + p(4)*dp(518) - dp(1189)
dp(1228) = dp(11)*p(296) + p(11)*dp(296) - dp(973)
dp(1229) = dp(14)*p(292) + p(14)*dp(292)
dp(1230) = dp(14)*p(293) + p(14)*dp(293)
dp(1231) = dp(14)*p(294) + p(14)*dp(294)
dp(1232) = dp(14)*p(242) + p(14)*dp(242)
dp(1233) = dp(14)*p(243) + p(14)*dp(243)
dp(1234) = dp(3)*p(464) + p(3)*dp(464) - dp(1074)
dp(1235) = dp(7)*p(464) + p(7)*dp(464) - dp(1075)
dp(1236) = dp(3)*p(531) + p(3)*dp(531) - dp(1074) - dp(1235)
dp(1237) = dp(39)*p(187) + p(39)*dp(187) - dp(1234)
dp(1238) = dp(3)*p(532) + p(3)*dp(532) - dp(1075)
dp(1239) = dp(14)*p(295) + p(14)*dp(295)
dp(1240) = dp(14)*p(296) + p(14)*dp(296)
dp(1241) = dp(14)*p(247) + p(14)*dp(247)
dp(1242) = dp(14)*p(248) + p(14)*dp(248)
dp(1243) = dp(14)*p(249) + p(14)*dp(249)
dp(1244) = dp(9)*p(337) + p(9)*dp(337)
dp(1245) = dp(9)*p(312) + p(9)*dp(312)
dp(1246) = dp(9)*p(266) + p(9)*dp(266)
dp(1247) = dp(9)*p(313) + p(9)*dp(313)
dp(1248) = dp(9)*p(314) + p(9)*dp(314)
dp(1249) = dp(9)*p(338) + p(9)*dp(338)
dp(1250) = dp(24)*p(185) + p(24)*dp(185) - dp(975)
dp(1251) = dp(46)*p(114) + p(46)*dp(114) - dp(977)
dp(1252) = dp(1)*p(783) + p(1)*dp(783) - dp(1082) - dp(1251)
dp(1253) = dp(1)*p(784) + p(1)*dp(784) - dp(1083) - dp(1252)
dp(1254) = dp(13)*p(335) + p(13)*dp(335) - dp(1215)
dp(1255) = dp(2)*p(748) + p(2)*dp(748) - dp(1055) - dp(1222)
dp(1256) = dp(32)*p(185) + p(32)*dp(185) - dp(1215)
dp(1257) = dp(1)*p(786) + p(1)*dp(786) - dp(1083) - dp(1255)
dp(1258) = dp(73)*p(95) + p(73)*dp(95) - dp(1215)
dp(1259) = dp(13)*p(299) + p(13)*dp(299) - dp(977)
dp(1260) = dp(46)*p(150) + p(46)*dp(150) - dp(1215)
dp(1261) = dp(73)*p(84) + p(73)*dp(84) - dp(975)
dp(1262) = dp(14)*p(335) + p(14)*dp(335)
dp(1263) = dp(14)*p(336) + p(14)*dp(336)
dp(1264) = dp(43)*p(130) + p(43)*dp(130) - dp(974)
dp(1265) = dp(8)*p(430) + p(8)*dp(430) - dp(1038)
dp(1266) = dp(1)*p(794) + p(1)*dp(794) - dp(1093) - dp(1265)
dp(1267) = dp(81)*p(93) + p(81)*dp(93) - dp(1190)
dp(1268) = dp(2)*p(761) + p(2)*dp(761) - dp(1066) - dp(1236)
dp(1269) = dp(43)*p(160) + p(43)*dp(160) - dp(1190)
dp(1270) = dp(45)*p(153) + p(45)*dp(153) - dp(1038)
dp(1271) = dp(18)*p(296) + p(18)*dp(296) - dp(1190)
dp(1272) = dp(34)*p(160) + p(34)*dp(160) - dp(974)
dp(1273) = dp(14)*p(302) + p(14)*dp(302)
dp(1274) = dp(14)*p(303) + p(14)*dp(303)
dp(1275) = dp(14)*p(304) + p(14)*dp(304)
dp(1276) = dp(14)*p(258) + p(14)*dp(258)
dp(1277) = dp(14)*p(259) + p(14)*dp(259)
dp(1278) = dp(4)*p(464) + p(4)*dp(464) - dp(983)
dp(1279) = dp(1)*p(804) + p(1)*dp(804) - dp(1103) - dp(1278)
dp(1280) = dp(16)*p(310) + p(16)*dp(310) - dp(1240)
dp(1281) = dp(2)*p(772) + p(2)*dp(772) - dp(1077) - dp(1280)
dp(1282) = dp(4)*p(532) + p(4)*dp(532) - dp(1240)
dp(1283) = dp(11)*p(310) + p(11)*dp(310) - dp(983)
dp(1284) = dp(14)*p(305) + p(14)*dp(305)
dp(1285) = dp(14)*p(306) + p(14)*dp(306)
dp(1286) = dp(14)*p(307) + p(14)*dp(307)
dp(1287) = dp(14)*p(263) + p(14)*dp(263)
dp(1288) = dp(14)*p(264) + p(14)*dp(264)
dp(1289) = dp(14)*p(265) + p(14)*dp(265)
dp(1290) = dp(6)*p(464) + p(6)*dp(464) - dp(1104)
dp(1291) = dp(3)*p(578) + p(3)*dp(578) - dp(1290)
dp(1292) = dp(14)*p(309) + p(14)*dp(309)
dp(1293) = dp(14)*p(310) + p(14)*dp(310)
dp(1294) = dp(9)*p(340) + p(9)*dp(340)
dp(1295) = dp(9)*p(322) + p(9)*dp(322)
dp(1296) = dp(9)*p(323) + p(9)*dp(323)
dp(1297) = dp(9)*p(341) + p(9)*dp(341)
dp(1298) = dp(24)*p(188) + p(24)*dp(188) - dp(984)
dp(1299) = dp(50)*p(114) + p(50)*dp(114) - dp(1089)
dp(1300) = dp(1)*p(813) + p(1)*dp(813) - dp(1108) - dp(1299)
dp(1301) = dp(13)*p(337) + p(13)*dp(337) - dp(1262)
dp(1302) = dp(2)*p(786) + p(2)*dp(786) - dp(1091) - dp(1268)
dp(1303) = dp(32)*p(188) + p(32)*dp(188) - dp(1262)
dp(1304) = dp(6)*p(536) + p(6)*dp(536) - dp(1089)
dp(1305) = dp(73)*p(98) + p(73)*dp(98) - dp(1262)
dp(1306) = dp(32)*p(173) + p(32)*dp(173) - dp(984)
dp(1307) = dp(14)*p(337) + p(14)*dp(337)
dp(1308) = dp(14)*p(338) + p(14)*dp(338)
dp(1309) = dp(46)*p(130) + p(46)*dp(130) - dp(982)
dp(1310) = dp(1)*p(820) + p(1)*dp(820) - dp(1113) - dp(1309)
dp(1311) = dp(81)*p(95) + p(81)*dp(95) - dp(1232)
dp(1312) = dp(2)*p(796) + p(2)*dp(796) - dp(1098) - dp(1281)
dp(1313) = dp(46)*p(160) + p(46)*dp(160) - dp(1232)
dp(1314) = dp(81)*p(84) + p(81)*dp(84) - dp(982)
dp(1315) = dp(14)*p(315) + p(14)*dp(315)
dp(1316) = dp(14)*p(316) + p(14)*dp(316)
dp(1317) = dp(14)*p(317) + p(14)*dp(317)
dp(1318) = dp(14)*p(270) + p(14)*dp(270)
dp(1319) = dp(14)*p(271) + p(14)*dp(271)
dp(1320) = dp(8)*p(464) + p(8)*dp(464) - dp(1079)
dp(1321) = dp(18)*p(310) + p(18)*dp(310) - dp(1242)
dp(1322) = dp(34)*p(187) + p(34)*dp(187) - dp(1079)
dp(1323) = dp(14)*p(318) + p(14)*dp(318)
dp(1324) = dp(14)*p(319) + p(14)*dp(319)
dp(1325) = dp(14)*p(320) + p(14)*dp(320)
dp(1326) = dp(9)*p(343) + p(9)*dp(343)
dp(1327) = dp(9)*p(344) + p(9)*dp(344)
dp(1328) = dp(24)*p(191) + p(24)*dp(191) - dp(987)
dp(1329) = dp(1)*p(827) + p(1)*dp(827) - dp(1117) - dp(1328)
dp(1330) = dp(13)*p(340) + p(13)*dp(340) - dp(1307)
dp(1331) = dp(2)*p(815) + p(2)*dp(815) - dp(1112) - dp(1312)
dp(1332) = dp(32)*p(191) + p(32)*dp(191) - dp(1307)
dp(1333) = dp(13)*p(323) + p(13)*dp(323) - dp(987)
dp(1334) = dp(14)*p(340) + p(14)*dp(340)
dp(1335) = dp(14)*p(341) + p(14)*dp(341)
dp(1336) = dp(50)*p(130) + p(50)*dp(130) - dp(1102)
dp(1337) = dp(81)*p(98) + p(81)*dp(98) - dp(1276)
dp(1338) = dp(45)*p(173) + p(45)*dp(173) - dp(1102)
dp(1339) = dp(14)*p(324) + p(14)*dp(324)
dp(1340) = dp(14)*p(325) + p(14)*dp(325)
dp(1341) = dp(14)*p(326) + p(14)*dp(326)
dp(1342) = dp(9)*p(347) + p(9)*dp(347)
dp(1343) = dp(24)*p(195) + p(24)*dp(195) - dp(1119)
dp(1344) = dp(13)*p(343) + p(13)*dp(343) - dp(1334)
dp(1345) = dp(6)*p(561) + p(6)*dp(561) - dp(1119)
dp(1346) = dp(14)*p(343) + p(14)*dp(343)
dp(1347) = dp(14)*p(344) + p(14)*dp(344)
dp(1348) = dp(9)*p(272) + p(9)*dp(272)
dp(1349) = dp(9)*p(273) + p(9)*dp(273)
dp(1350) = dp(9)*p(274) + p(9)*dp(274)
dp(1351) = dp(9)*p(328) + p(9)*dp(328)
dp(1352) = dp(9)*p(275) + p(9)*dp(275)
dp(1353) = dp(9)*p(329) + p(9)*dp(329)
dp(1354) = dp(9)*p(330) + p(9)*dp(330)
dp(1355) = dp(9)*p(346) + p(9)*dp(346)
dp(1356) = dp(3)*p(570) + p(3)*dp(570) - dp(1120)
dp(1357) = dp(39)*p(181) + p(39)*dp(181) - dp(1122)
dp(1358) = dp(41)*p(178) + p(41)*dp(178) - dp(1125)
dp(1359) = dp(4)*p(566) + p(4)*dp(566) - dp(1129)
dp(1360) = dp(1)*p(842) + p(1)*dp(842) - dp(1126) - dp(1359)
dp(1361) = dp(1)*p(843) + p(1)*dp(843) - dp(1129)
dp(1362) = dp(1)*p(844) + p(1)*dp(844) - dp(1130) - dp(1360)
dp(1363) = dp(21)*p(279) + p(21)*dp(279) - dp(1125)
dp(1364) = dp(1)*p(846) + p(1)*dp(846) - dp(1133) - dp(1362)
dp(1365) = dp(69)*p(101) + p(69)*dp(101) - dp(1122)
dp(1366) = dp(1)*p(848) + p(1)*dp(848) - dp(1135) - dp(1364)
dp(1367) = dp(11)*p(346) + p(11)*dp(346) - dp(1120)
dp(1368) = dp(1)*p(850) + p(1)*dp(850) - dp(1140) - dp(1139)
dp(1369) = dp(1)*p(851) + p(1)*dp(851) - dp(1146)
dp(1370) = dp(3)*p(572) + p(3)*dp(572) - dp(1148)
dp(1371) = dp(39)*p(183) + p(39)*dp(183) - dp(1150)
dp(1372) = dp(43)*p(178) + p(43)*dp(178) - dp(1153)
dp(1373) = dp(1)*p(854) + p(1)*dp(854) - dp(1151) - dp(1372)
dp(1374) = dp(2)*p(843) + p(2)*dp(843) - dp(1143)
dp(1375) = dp(1)*p(855) + p(1)*dp(855) - dp(1154) - dp(1373)
dp(1376) = dp(7)*p(509) + p(7)*dp(509) - dp(1153)
dp(1377) = dp(1)*p(857) + p(1)*dp(857) - dp(1157) - dp(1375)
dp(1378) = dp(49)*p(153) + p(49)*dp(153) - dp(1150)
dp(1379) = dp(1)*p(859) + p(1)*dp(859) - dp(1159) - dp(1377)
dp(1380) = dp(34)*p(194) + p(34)*dp(194) - dp(1148)
dp(1381) = dp(6)*p(570) + p(6)*dp(570) - dp(1147)
dp(1382) = dp(1)*p(862) + p(1)*dp(862) - dp(1167) - dp(1165)
dp(1383) = dp(2)*p(851) + p(2)*dp(851) - dp(1147)
dp(1384) = dp(1)*p(864) + p(1)*dp(864) - dp(1183) - dp(1182)
dp(1385) = dp(1)*p(865) + p(1)*dp(865) - dp(1187)
dp(1386) = dp(3)*p(574) + p(3)*dp(574) - dp(1191)
dp(1387) = dp(39)*p(185) + p(39)*dp(185) - dp(1193)
dp(1388) = dp(46)*p(178) + p(46)*dp(178) - dp(1196)
dp(1389) = dp(1)*p(868) + p(1)*dp(868) - dp(1194) - dp(1388)
dp(1390) = dp(1)*p(869) + p(1)*dp(869) - dp(1196)
dp(1391) = dp(1)*p(870) + p(1)*dp(870) - dp(1197) - dp(1389)
dp(1392) = dp(21)*p(299) + p(21)*dp(299) - dp(1193)
dp(1393) = dp(1)*p(872) + p(1)*dp(872) - dp(1199) - dp(1391)
dp(1394) = dp(84)*p(101) + p(84)*dp(101) - dp(1191)
dp(1395) = dp(6)*p(572) + p(6)*dp(572) - dp(1178)
dp(1396) = dp(1)*p(875) + p(1)*dp(875) - dp(1206) - dp(1204)
dp(1397) = dp(8)*p(505) + p(8)*dp(505) - dp(1178)
dp(1398) = dp(45)*p(181) + p(45)*dp(181) - dp(1189)
dp(1399) = dp(1)*p(878) + p(1)*dp(878) - dp(1222) - dp(1220)
dp(1400) = dp(2)*p(865) + p(2)*dp(865) - dp(1189)
dp(1401) = dp(1)*p(880) + p(1)*dp(880) - dp(1236) - dp(1235)
dp(1402) = dp(1)*p(881) + p(1)*dp(881) - dp(1238)
dp(1403) = dp(3)*p(576) + p(3)*dp(576) - dp(1244)
dp(1404) = dp(39)*p(188) + p(39)*dp(188) - dp(1246)
dp(1405) = dp(1)*p(884) + p(1)*dp(884) - dp(1245) - dp(1404)
dp(1406) = dp(2)*p(869) + p(2)*dp(869) - dp(1210)
dp(1407) = dp(1)*p(885) + p(1)*dp(885) - dp(1247) - dp(1405)
dp(1408) = dp(7)*p(536) + p(7)*dp(536) - dp(1246)
dp(1409) = dp(1)*p(887) + p(1)*dp(887) - dp(1249) - dp(1407)
dp(1410) = dp(49)*p(173) + p(49)*dp(173) - dp(1244)
dp(1411) = dp(6)*p(574) + p(6)*dp(574) - dp(1215)
dp(1412) = dp(1)*p(890) + p(1)*dp(890) - dp(1255) - dp(1253)
dp(1413) = dp(22)*p(285) + p(22)*dp(285) - dp(1215)
dp(1414) = dp(45)*p(183) + p(45)*dp(183) - dp(1190)
dp(1415) = dp(1)*p(893) + p(1)*dp(893) - dp(1268) - dp(1266)
dp(1416) = dp(8)*p(518) + p(8)*dp(518) - dp(1190)
dp(1417) = dp(41)*p(187) + p(41)*dp(187) - dp(1240)
dp(1418) = dp(1)*p(896) + p(1)*dp(896) - dp(1281) - dp(1279)
dp(1419) = dp(2)*p(881) + p(2)*dp(881) - dp(1240)
dp(1420) = dp(14)*p(308) + p(14)*dp(308)
dp(1421) = dp(4)*p(547) + p(4)*dp(547) - dp(1106) - dp(1320)
dp(1422) = dp(1)*p(900) + p(1)*dp(900) - dp(1291)
dp(1423) = dp(14)*p(311) + p(14)*dp(311)
dp(1424) = dp(3)*p(579) + p(3)*dp(579) - dp(1294)
dp(1425) = dp(39)*p(191) + p(39)*dp(191) - dp(1296)
dp(1426) = dp(1)*p(903) + p(1)*dp(903) - dp(1295) - dp(1425)
dp(1427) = dp(1)*p(904) + p(1)*dp(904) - dp(1296)
dp(1428) = dp(1)*p(905) + p(1)*dp(905) - dp(1297) - dp(1426)
dp(1429) = dp(21)*p(323) + p(21)*dp(323) - dp(1294)
dp(1430) = dp(6)*p(576) + p(6)*dp(576) - dp(1262)
dp(1431) = dp(1)*p(908) + p(1)*dp(908) - dp(1302) - dp(1300)
dp(1432) = dp(50)*p(150) + p(50)*dp(150) - dp(1262)
dp(1433) = dp(45)*p(185) + p(45)*dp(185) - dp(1232)
dp(1434) = dp(1)*p(911) + p(1)*dp(911) - dp(1312) - dp(1310)
dp(1435) = dp(22)*p(296) + p(22)*dp(296) - dp(1232)
dp(1436) = dp(43)*p(187) + p(43)*dp(187) - dp(1242)
dp(1437) = dp(1)*p(915) + p(1)*dp(915) - dp(1322) - dp(1320)
dp(1438) = dp(8)*p(532) + p(8)*dp(532) - dp(1242)
dp(1439) = dp(14)*p(321) + p(14)*dp(321)
dp(1440) = dp(4)*p(578) + p(4)*dp(578) - dp(1293)
dp(1441) = dp(2)*p(899) + p(2)*dp(899) - dp(1292) - dp(1440)
dp(1442) = dp(2)*p(900) + p(2)*dp(900) - dp(1293)
dp(1443) = dp(14)*p(339) + p(14)*dp(339)
dp(1444) = dp(3)*p(582) + p(3)*dp(582) - dp(1326)
dp(1445) = dp(2)*p(903) + p(2)*dp(903) - dp(1303) - dp(1299)
dp(1446) = dp(2)*p(904) + p(2)*dp(904) - dp(1304)
dp(1447) = dp(1)*p(919) + p(1)*dp(919) - dp(1327) - dp(1445)
dp(1448) = dp(7)*p(561) + p(7)*dp(561) - dp(1326)
dp(1449) = dp(6)*p(579) + p(6)*dp(579) - dp(1307)
dp(1450) = dp(1)*p(922) + p(1)*dp(922) - dp(1331) - dp(1329)
dp(1451) = dp(73)*p(102) + p(73)*dp(102) - dp(1307)
dp(1452) = dp(45)*p(188) + p(45)*dp(188) - dp(1276)
dp(1453) = dp(1)*p(925) + p(1)*dp(925) - dp(1338) - dp(1336)
dp(1454) = dp(50)*p(160) + p(50)*dp(160) - dp(1276)
dp(1455) = dp(14)*p(327) + p(14)*dp(327)
dp(1456) = dp(46)*p(187) + p(46)*dp(187) - dp(1243)
dp(1457) = dp(2)*p(915) + p(2)*dp(915) - dp(1324) - dp(1441)
dp(1458) = dp(22)*p(310) + p(22)*dp(310) - dp(1243)
dp(1459) = dp(14)*p(342) + p(14)*dp(342)
dp(1460) = dp(3)*p(585) + p(3)*dp(585) - dp(1342)
dp(1461) = dp(2)*p(919) + p(2)*dp(919) - dp(1332) - dp(1329)
dp(1462) = dp(1)*p(929) + p(1)*dp(929) - dp(1342)
dp(1463) = dp(6)*p(582) + p(6)*dp(582) - dp(1334)
dp(1464) = dp(1)*p(931) + p(1)*dp(931) - dp(1345) - dp(1343)
dp(1465) = dp(32)*p(195) + p(32)*dp(195) - dp(1334)
dp(1466) = dp(45)*p(191) + p(45)*dp(191) - dp(1318)
dp(1467) = dp(2)*p(925) + p(2)*dp(925) - dp(1340) - dp(1457)
dp(1468) = dp(81)*p(102) + p(81)*dp(102) - dp(1318)
dp(1469) = dp(14)*p(345) + p(14)*dp(345)
dp(1470) = dp(2)*p(928) + p(2)*dp(928) - dp(1344) - dp(1343)
dp(1471) = dp(2)*p(929) + p(2)*dp(929) - dp(1345)
dp(1472) = dp(6)*p(585) + p(6)*dp(585) - dp(1346)
dp(1473) = dp(2)*p(931) + p(2)*dp(931) - dp(1347) - dp(1467)
dp(1474) = dp(13)*p(347) + p(13)*dp(347) - dp(1346)
dp(1475) = dp(14)*p(347) + p(14)*dp(347)
dp(1476) = dp(3)*p(566) + p(3)*dp(566) - dp(1351)
dp(1477) = dp(7)*p(566) + p(7)*dp(566) - dp(1349)
dp(1478) = dp(49)*p(178) + p(49)*dp(178) - dp(1348)
dp(1479) = dp(39)*p(194) + p(39)*dp(194) - dp(1352)
dp(1480) = dp(3)*p(588) + p(3)*dp(588) - dp(1355)
dp(1481) = dp(1)*p(938) + p(1)*dp(938) - dp(1356)
dp(1482) = dp(2)*p(958) + p(2)*dp(958) - dp(1481)
dp(1483) = dp(1)*p(940) + p(1)*dp(940) - dp(1370)
dp(1484) = dp(8)*p(588) + p(8)*dp(588) - dp(1483)
dp(1485) = dp(1)*p(942) + p(1)*dp(942) - dp(1386)
dp(1486) = dp(22)*p(346) + p(22)*dp(346) - dp(1485)
dp(1487) = dp(1)*p(944) + p(1)*dp(944) - dp(1403)
dp(1488) = dp(50)*p(194) + p(50)*dp(194) - dp(1487)
dp(1489) = dp(1)*p(946) + p(1)*dp(946) - dp(1424)
dp(1490) = dp(101)*p(102) + p(101)*dp(102) - dp(1489)
dp(1491) = dp(6)*p(578) + p(6)*dp(578) - dp(1443)
dp(1492) = dp(1)*p(949) + p(1)*dp(949) - dp(1444)
dp(1493) = dp(49)*p(195) + p(49)*dp(195) - dp(1492)
dp(1494) = dp(8)*p(578) + p(8)*dp(578) - dp(1423)
dp(1495) = dp(1)*p(952) + p(1)*dp(952) - dp(1460)
dp(1496) = dp(21)*p(347) + p(21)*dp(347) - dp(1495)
dp(1497) = dp(50)*p(187) + p(50)*dp(187) - dp(1420)
dp(1498) = dp(1)*p(955) + p(1)*dp(955) - dp(1470)
dp(1499) = dp(7)*p(589) + p(7)*dp(589) - dp(1498)
dp(1500) = dp(45)*p(195) + p(45)*dp(195) - dp(1455)
dp(1501) = dp(2)*p(955) + p(2)*dp(955) - dp(1472)
dp(1502) = dp(1)*p(959) + p(1)*dp(959) - dp(1501)
dp(1503) = dp(6)*p(589) + p(6)*dp(589) - dp(1475)
dp(1504) = dp(1)*p(958) + p(1)*dp(958) - dp(1480)
dp(1505) = dp(2)*p(959) + p(2)*dp(959) - dp(1503)

do j=1,1506
  dbm1(j)=dp(j-1)
enddo

dbm2(1)=dbm1(4)
dbm2(2)=dbm1(10)
dbm2(3)=dbm1(11)
dbm2(4)=dbm1(12)
dbm2(5)=dbm1(13)
dbm2(6)=dbm1(15)
dbm2(7)=dbm1(16)
dbm2(8)=dbm1(24)
dbm2(9)=dbm1(25)
dbm2(10)=dbm1(26)
dbm2(11)=dbm1(27)
dbm2(12)=dbm1(28)
dbm2(13)=dbm1(29)
dbm2(14)=dbm1(30)
dbm2(15)=dbm1(31)
dbm2(16)=dbm1(32)
dbm2(17)=dbm1(34)
dbm2(18)=dbm1(35)
dbm2(19)=dbm1(36)
dbm2(20)=dbm1(37)
dbm2(21)=dbm1(39)
dbm2(22)=dbm1(40)
dbm2(23)=dbm1(41)
dbm2(24)=dbm1(52)
dbm2(25)=dbm1(53)
dbm2(26)=dbm1(54)
dbm2(27)=dbm1(55)
dbm2(28)=dbm1(56)
dbm2(29)=dbm1(57)
dbm2(30)=dbm1(58)
dbm2(31)=dbm1(59)
dbm2(32)=dbm1(60)
dbm2(33)=dbm1(61)
dbm2(34)=dbm1(62)
dbm2(35)=dbm1(63)
dbm2(36)=dbm1(64)
dbm2(37)=dbm1(65)
dbm2(38)=dbm1(66)
dbm2(39)=dbm1(67)
dbm2(40)=dbm1(68)
dbm2(41)=dbm1(69)
dbm2(42)=dbm1(70)
dbm2(43)=dbm1(71)
dbm2(44)=dbm1(72)
dbm2(45)=dbm1(73)
dbm2(46)=dbm1(75)
dbm2(47)=dbm1(76)
dbm2(48)=dbm1(77)
dbm2(49)=dbm1(78)
dbm2(50)=dbm1(79)
dbm2(51)=dbm1(81)
dbm2(52)=dbm1(83)
dbm2(53)=dbm1(84)
dbm2(54)=dbm1(85)
dbm2(55)=dbm1(86)
dbm2(56)=dbm1(87)
dbm2(57)=dbm1(89)
dbm2(58)=dbm1(90)
dbm2(59)=dbm1(91)
dbm2(60)=dbm1(104)
dbm2(61)=dbm1(105)
dbm2(62)=dbm1(106)
dbm2(63)=dbm1(107)
dbm2(64)=dbm1(108)
dbm2(65)=dbm1(109)
dbm2(66)=dbm1(110)
dbm2(67)=dbm1(111)
dbm2(68)=dbm1(112)
dbm2(69)=dbm1(113)
dbm2(70)=dbm1(114)
dbm2(71)=dbm1(115)
dbm2(72)=dbm1(116)
dbm2(73)=dbm1(117)
dbm2(74)=dbm1(118)
dbm2(75)=dbm1(119)
dbm2(76)=dbm1(120)
dbm2(77)=dbm1(121)
dbm2(78)=dbm1(122)
dbm2(79)=dbm1(123)
dbm2(80)=dbm1(124)
dbm2(81)=dbm1(125)
dbm2(82)=dbm1(126)
dbm2(83)=dbm1(127)
dbm2(84)=dbm1(128)
dbm2(85)=dbm1(129)
dbm2(86)=dbm1(130)
dbm2(87)=dbm1(131)
dbm2(88)=dbm1(132)
dbm2(89)=dbm1(133)
dbm2(90)=dbm1(134)
dbm2(91)=dbm1(135)
dbm2(92)=dbm1(136)
dbm2(93)=dbm1(137)
dbm2(94)=dbm1(138)
dbm2(95)=dbm1(139)
dbm2(96)=dbm1(140)
dbm2(97)=dbm1(141)
dbm2(98)=dbm1(142)
dbm2(99)=dbm1(143)
dbm2(100)=dbm1(144)
dbm2(101)=dbm1(145)
dbm2(102)=dbm1(146)
dbm2(103)=dbm1(147)
dbm2(104)=dbm1(148)
dbm2(105)=dbm1(149)
dbm2(106)=dbm1(150)
dbm2(107)=dbm1(152)
dbm2(108)=dbm1(153)
dbm2(109)=dbm1(154)
dbm2(110)=dbm1(155)
dbm2(111)=dbm1(156)
dbm2(112)=dbm1(157)
dbm2(113)=dbm1(158)
dbm2(114)=dbm1(160)
dbm2(115)=dbm1(162)
dbm2(116)=dbm1(163)
dbm2(117)=dbm1(164)
dbm2(118)=dbm1(165)
dbm2(119)=dbm1(166)
dbm2(120)=dbm1(167)
dbm2(121)=dbm1(169)
dbm2(122)=dbm1(170)
dbm2(123)=dbm1(172)
dbm2(124)=dbm1(173)
dbm2(125)=dbm1(174)
dbm2(126)=dbm1(175)
dbm2(127)=dbm1(176)
dbm2(128)=dbm1(178)
dbm2(129)=dbm1(179)
dbm2(130)=dbm1(180)
dbm2(131)=dbm1(181)
dbm2(132)=dbm1(197)
dbm2(133)=dbm1(198)
dbm2(134)=dbm1(199)
dbm2(135)=dbm1(200)
dbm2(136)=dbm1(201)
dbm2(137)=dbm1(202)
dbm2(138)=dbm1(203)
dbm2(139)=dbm1(204)
dbm2(140)=dbm1(205)
dbm2(141)=dbm1(206)
dbm2(142)=dbm1(207)
dbm2(143)=dbm1(208)
dbm2(144)=dbm1(209)
dbm2(145)=dbm1(210)
dbm2(146)=dbm1(211)
dbm2(147)=dbm1(212)
dbm2(148)=dbm1(213)
dbm2(149)=dbm1(214)
dbm2(150)=dbm1(215)
dbm2(151)=dbm1(216)
dbm2(152)=dbm1(217)
dbm2(153)=dbm1(218)
dbm2(154)=dbm1(219)
dbm2(155)=dbm1(220)
dbm2(156)=dbm1(221)
dbm2(157)=dbm1(222)
dbm2(158)=dbm1(223)
dbm2(159)=dbm1(224)
dbm2(160)=dbm1(225)
dbm2(161)=dbm1(226)
dbm2(162)=dbm1(227)
dbm2(163)=dbm1(228)
dbm2(164)=dbm1(229)
dbm2(165)=dbm1(230)
dbm2(166)=dbm1(231)
dbm2(167)=dbm1(232)
dbm2(168)=dbm1(233)
dbm2(169)=dbm1(234)
dbm2(170)=dbm1(235)
dbm2(171)=dbm1(236)
dbm2(172)=dbm1(237)
dbm2(173)=dbm1(238)
dbm2(174)=dbm1(239)
dbm2(175)=dbm1(240)
dbm2(176)=dbm1(241)
dbm2(177)=dbm1(242)
dbm2(178)=dbm1(243)
dbm2(179)=dbm1(244)
dbm2(180)=dbm1(245)
dbm2(181)=dbm1(246)
dbm2(182)=dbm1(247)
dbm2(183)=dbm1(248)
dbm2(184)=dbm1(249)
dbm2(185)=dbm1(250)
dbm2(186)=dbm1(251)
dbm2(187)=dbm1(252)
dbm2(188)=dbm1(253)
dbm2(189)=dbm1(254)
dbm2(190)=dbm1(255)
dbm2(191)=dbm1(256)
dbm2(192)=dbm1(257)
dbm2(193)=dbm1(258)
dbm2(194)=dbm1(259)
dbm2(195)=dbm1(260)
dbm2(196)=dbm1(261)
dbm2(197)=dbm1(262)
dbm2(198)=dbm1(263)
dbm2(199)=dbm1(264)
dbm2(200)=dbm1(265)
dbm2(201)=dbm1(266)
dbm2(202)=dbm1(267)
dbm2(203)=dbm1(268)
dbm2(204)=dbm1(269)
dbm2(205)=dbm1(270)
dbm2(206)=dbm1(271)
dbm2(207)=dbm1(272)
dbm2(208)=dbm1(273)
dbm2(209)=dbm1(274)
dbm2(210)=dbm1(275)
dbm2(211)=dbm1(276)
dbm2(212)=dbm1(277)
dbm2(213)=dbm1(278)
dbm2(214)=dbm1(279)
dbm2(215)=dbm1(280)
dbm2(216)=dbm1(281)
dbm2(217)=dbm1(282)
dbm2(218)=dbm1(283)
dbm2(219)=dbm1(284)
dbm2(220)=dbm1(285)
dbm2(221)=dbm1(287)
dbm2(222)=dbm1(288)
dbm2(223)=dbm1(289)
dbm2(224)=dbm1(290)
dbm2(225)=dbm1(291)
dbm2(226)=dbm1(292)
dbm2(227)=dbm1(293)
dbm2(228)=dbm1(294)
dbm2(229)=dbm1(296)
dbm2(230)=dbm1(298)
dbm2(231)=dbm1(299)
dbm2(232)=dbm1(300)
dbm2(233)=dbm1(301)
dbm2(234)=dbm1(302)
dbm2(235)=dbm1(303)
dbm2(236)=dbm1(304)
dbm2(237)=dbm1(306)
dbm2(238)=dbm1(307)
dbm2(239)=dbm1(309)
dbm2(240)=dbm1(310)
dbm2(241)=dbm1(312)
dbm2(242)=dbm1(313)
dbm2(243)=dbm1(314)
dbm2(244)=dbm1(315)
dbm2(245)=dbm1(316)
dbm2(246)=dbm1(317)
dbm2(247)=dbm1(319)
dbm2(248)=dbm1(320)
dbm2(249)=dbm1(322)
dbm2(250)=dbm1(323)
dbm2(251)=dbm1(324)
dbm2(252)=dbm1(325)
dbm2(253)=dbm1(326)
dbm2(254)=dbm1(328)
dbm2(255)=dbm1(329)
dbm2(256)=dbm1(330)
dbm2(257)=dbm1(331)
dbm2(258)=dbm1(349)
dbm2(259)=dbm1(350)
dbm2(260)=dbm1(351)
dbm2(261)=dbm1(352)
dbm2(262)=dbm1(353)
dbm2(263)=dbm1(354)
dbm2(264)=dbm1(355)
dbm2(265)=dbm1(356)
dbm2(266)=dbm1(357)
dbm2(267)=dbm1(358)
dbm2(268)=dbm1(359)
dbm2(269)=dbm1(360)
dbm2(270)=dbm1(361)
dbm2(271)=dbm1(362)
dbm2(272)=dbm1(363)
dbm2(273)=dbm1(364)
dbm2(274)=dbm1(365)
dbm2(275)=dbm1(366)
dbm2(276)=dbm1(367)
dbm2(277)=dbm1(368)
dbm2(278)=dbm1(369)
dbm2(279)=dbm1(370)
dbm2(280)=dbm1(371)
dbm2(281)=dbm1(372)
dbm2(282)=dbm1(373)
dbm2(283)=dbm1(374)
dbm2(284)=dbm1(375)
dbm2(285)=dbm1(376)
dbm2(286)=dbm1(377)
dbm2(287)=dbm1(378)
dbm2(288)=dbm1(379)
dbm2(289)=dbm1(380)
dbm2(290)=dbm1(381)
dbm2(291)=dbm1(382)
dbm2(292)=dbm1(383)
dbm2(293)=dbm1(384)
dbm2(294)=dbm1(385)
dbm2(295)=dbm1(386)
dbm2(296)=dbm1(387)
dbm2(297)=dbm1(388)
dbm2(298)=dbm1(389)
dbm2(299)=dbm1(390)
dbm2(300)=dbm1(391)
dbm2(301)=dbm1(392)
dbm2(302)=dbm1(393)
dbm2(303)=dbm1(394)
dbm2(304)=dbm1(395)
dbm2(305)=dbm1(396)
dbm2(306)=dbm1(397)
dbm2(307)=dbm1(398)
dbm2(308)=dbm1(399)
dbm2(309)=dbm1(400)
dbm2(310)=dbm1(401)
dbm2(311)=dbm1(402)
dbm2(312)=dbm1(403)
dbm2(313)=dbm1(404)
dbm2(314)=dbm1(405)
dbm2(315)=dbm1(406)
dbm2(316)=dbm1(407)
dbm2(317)=dbm1(408)
dbm2(318)=dbm1(409)
dbm2(319)=dbm1(410)
dbm2(320)=dbm1(411)
dbm2(321)=dbm1(412)
dbm2(322)=dbm1(413)
dbm2(323)=dbm1(414)
dbm2(324)=dbm1(415)
dbm2(325)=dbm1(416)
dbm2(326)=dbm1(417)
dbm2(327)=dbm1(418)
dbm2(328)=dbm1(419)
dbm2(329)=dbm1(420)
dbm2(330)=dbm1(421)
dbm2(331)=dbm1(422)
dbm2(332)=dbm1(423)
dbm2(333)=dbm1(424)
dbm2(334)=dbm1(425)
dbm2(335)=dbm1(426)
dbm2(336)=dbm1(427)
dbm2(337)=dbm1(428)
dbm2(338)=dbm1(429)
dbm2(339)=dbm1(430)
dbm2(340)=dbm1(431)
dbm2(341)=dbm1(432)
dbm2(342)=dbm1(433)
dbm2(343)=dbm1(434)
dbm2(344)=dbm1(435)
dbm2(345)=dbm1(436)
dbm2(346)=dbm1(437)
dbm2(347)=dbm1(438)
dbm2(348)=dbm1(439)
dbm2(349)=dbm1(440)
dbm2(350)=dbm1(441)
dbm2(351)=dbm1(442)
dbm2(352)=dbm1(443)
dbm2(353)=dbm1(444)
dbm2(354)=dbm1(445)
dbm2(355)=dbm1(446)
dbm2(356)=dbm1(447)
dbm2(357)=dbm1(448)
dbm2(358)=dbm1(449)
dbm2(359)=dbm1(450)
dbm2(360)=dbm1(451)
dbm2(361)=dbm1(452)
dbm2(362)=dbm1(453)
dbm2(363)=dbm1(454)
dbm2(364)=dbm1(455)
dbm2(365)=dbm1(456)
dbm2(366)=dbm1(457)
dbm2(367)=dbm1(458)
dbm2(368)=dbm1(459)
dbm2(369)=dbm1(460)
dbm2(370)=dbm1(461)
dbm2(371)=dbm1(462)
dbm2(372)=dbm1(463)
dbm2(373)=dbm1(464)
dbm2(374)=dbm1(465)
dbm2(375)=dbm1(466)
dbm2(376)=dbm1(467)
dbm2(377)=dbm1(468)
dbm2(378)=dbm1(469)
dbm2(379)=dbm1(470)
dbm2(380)=dbm1(471)
dbm2(381)=dbm1(472)
dbm2(382)=dbm1(473)
dbm2(383)=dbm1(474)
dbm2(384)=dbm1(475)
dbm2(385)=dbm1(476)
dbm2(386)=dbm1(477)
dbm2(387)=dbm1(478)
dbm2(388)=dbm1(479)
dbm2(389)=dbm1(480)
dbm2(390)=dbm1(481)
dbm2(391)=dbm1(482)
dbm2(392)=dbm1(483)
dbm2(393)=dbm1(484)
dbm2(394)=dbm1(485)
dbm2(395)=dbm1(486)
dbm2(396)=dbm1(487)
dbm2(397)=dbm1(488)
dbm2(398)=dbm1(489)
dbm2(399)=dbm1(490)
dbm2(400)=dbm1(491)
dbm2(401)=dbm1(492)
dbm2(402)=dbm1(493)
dbm2(403)=dbm1(494)
dbm2(404)=dbm1(495)
dbm2(405)=dbm1(496)
dbm2(406)=dbm1(497)
dbm2(407)=dbm1(498)
dbm2(408)=dbm1(499)
dbm2(409)=dbm1(500)
dbm2(410)=dbm1(501)
dbm2(411)=dbm1(502)
dbm2(412)=dbm1(503)
dbm2(413)=dbm1(504)
dbm2(414)=dbm1(505)
dbm2(415)=dbm1(507)
dbm2(416)=dbm1(508)
dbm2(417)=dbm1(509)
dbm2(418)=dbm1(510)
dbm2(419)=dbm1(511)
dbm2(420)=dbm1(512)
dbm2(421)=dbm1(513)
dbm2(422)=dbm1(514)
dbm2(423)=dbm1(515)
dbm2(424)=dbm1(516)
dbm2(425)=dbm1(518)
dbm2(426)=dbm1(520)
dbm2(427)=dbm1(521)
dbm2(428)=dbm1(522)
dbm2(429)=dbm1(523)
dbm2(430)=dbm1(524)
dbm2(431)=dbm1(525)
dbm2(432)=dbm1(526)
dbm2(433)=dbm1(527)
dbm2(434)=dbm1(529)
dbm2(435)=dbm1(530)
dbm2(436)=dbm1(532)
dbm2(437)=dbm1(534)
dbm2(438)=dbm1(535)
dbm2(439)=dbm1(536)
dbm2(440)=dbm1(537)
dbm2(441)=dbm1(538)
dbm2(442)=dbm1(539)
dbm2(443)=dbm1(540)
dbm2(444)=dbm1(541)
dbm2(445)=dbm1(543)
dbm2(446)=dbm1(544)
dbm2(447)=dbm1(546)
dbm2(448)=dbm1(547)
dbm2(449)=dbm1(548)
dbm2(450)=dbm1(550)
dbm2(451)=dbm1(551)
dbm2(452)=dbm1(552)
dbm2(453)=dbm1(553)
dbm2(454)=dbm1(554)
dbm2(455)=dbm1(555)
dbm2(456)=dbm1(557)
dbm2(457)=dbm1(558)
dbm2(458)=dbm1(560)
dbm2(459)=dbm1(561)
dbm2(460)=dbm1(562)
dbm2(461)=dbm1(563)
dbm2(462)=dbm1(564)
dbm2(463)=dbm1(566)
dbm2(464)=dbm1(567)
dbm2(465)=dbm1(568)
dbm2(466)=dbm1(569)
dbm2(467)=dbm1(570)
dbm2(468)=dbm1(591)
dbm2(469)=dbm1(592)
dbm2(470)=dbm1(593)
dbm2(471)=dbm1(594)
dbm2(472)=dbm1(595)
dbm2(473)=dbm1(596)
dbm2(474)=dbm1(597)
dbm2(475)=dbm1(598)
dbm2(476)=dbm1(599)
dbm2(477)=dbm1(600)
dbm2(478)=dbm1(601)
dbm2(479)=dbm1(602)
dbm2(480)=dbm1(603)
dbm2(481)=dbm1(604)
dbm2(482)=dbm1(605)
dbm2(483)=dbm1(606)
dbm2(484)=dbm1(607)
dbm2(485)=dbm1(608)
dbm2(486)=dbm1(609)
dbm2(487)=dbm1(610)
dbm2(488)=dbm1(611)
dbm2(489)=dbm1(612)
dbm2(490)=dbm1(613)
dbm2(491)=dbm1(614)
dbm2(492)=dbm1(615)
dbm2(493)=dbm1(616)
dbm2(494)=dbm1(617)
dbm2(495)=dbm1(618)
dbm2(496)=dbm1(619)
dbm2(497)=dbm1(620)
dbm2(498)=dbm1(621)
dbm2(499)=dbm1(622)
dbm2(500)=dbm1(623)
dbm2(501)=dbm1(624)
dbm2(502)=dbm1(625)
dbm2(503)=dbm1(626)
dbm2(504)=dbm1(627)
dbm2(505)=dbm1(628)
dbm2(506)=dbm1(629)
dbm2(507)=dbm1(630)
dbm2(508)=dbm1(631)
dbm2(509)=dbm1(632)
dbm2(510)=dbm1(633)
dbm2(511)=dbm1(634)
dbm2(512)=dbm1(635)
dbm2(513)=dbm1(636)
dbm2(514)=dbm1(637)
dbm2(515)=dbm1(638)
dbm2(516)=dbm1(639)
dbm2(517)=dbm1(640)
dbm2(518)=dbm1(641)
dbm2(519)=dbm1(642)
dbm2(520)=dbm1(643)
dbm2(521)=dbm1(644)
dbm2(522)=dbm1(645)
dbm2(523)=dbm1(646)
dbm2(524)=dbm1(647)
dbm2(525)=dbm1(648)
dbm2(526)=dbm1(649)
dbm2(527)=dbm1(650)
dbm2(528)=dbm1(651)
dbm2(529)=dbm1(652)
dbm2(530)=dbm1(653)
dbm2(531)=dbm1(654)
dbm2(532)=dbm1(655)
dbm2(533)=dbm1(656)
dbm2(534)=dbm1(657)
dbm2(535)=dbm1(658)
dbm2(536)=dbm1(659)
dbm2(537)=dbm1(660)
dbm2(538)=dbm1(661)
dbm2(539)=dbm1(662)
dbm2(540)=dbm1(663)
dbm2(541)=dbm1(664)
dbm2(542)=dbm1(665)
dbm2(543)=dbm1(666)
dbm2(544)=dbm1(667)
dbm2(545)=dbm1(668)
dbm2(546)=dbm1(669)
dbm2(547)=dbm1(670)
dbm2(548)=dbm1(671)
dbm2(549)=dbm1(672)
dbm2(550)=dbm1(673)
dbm2(551)=dbm1(674)
dbm2(552)=dbm1(675)
dbm2(553)=dbm1(676)
dbm2(554)=dbm1(677)
dbm2(555)=dbm1(678)
dbm2(556)=dbm1(679)
dbm2(557)=dbm1(680)
dbm2(558)=dbm1(681)
dbm2(559)=dbm1(682)
dbm2(560)=dbm1(683)
dbm2(561)=dbm1(684)
dbm2(562)=dbm1(685)
dbm2(563)=dbm1(686)
dbm2(564)=dbm1(687)
dbm2(565)=dbm1(688)
dbm2(566)=dbm1(689)
dbm2(567)=dbm1(690)
dbm2(568)=dbm1(691)
dbm2(569)=dbm1(692)
dbm2(570)=dbm1(693)
dbm2(571)=dbm1(694)
dbm2(572)=dbm1(695)
dbm2(573)=dbm1(696)
dbm2(574)=dbm1(697)
dbm2(575)=dbm1(698)
dbm2(576)=dbm1(699)
dbm2(577)=dbm1(700)
dbm2(578)=dbm1(701)
dbm2(579)=dbm1(702)
dbm2(580)=dbm1(703)
dbm2(581)=dbm1(704)
dbm2(582)=dbm1(705)
dbm2(583)=dbm1(706)
dbm2(584)=dbm1(707)
dbm2(585)=dbm1(708)
dbm2(586)=dbm1(709)
dbm2(587)=dbm1(710)
dbm2(588)=dbm1(711)
dbm2(589)=dbm1(712)
dbm2(590)=dbm1(713)
dbm2(591)=dbm1(714)
dbm2(592)=dbm1(715)
dbm2(593)=dbm1(716)
dbm2(594)=dbm1(717)
dbm2(595)=dbm1(718)
dbm2(596)=dbm1(719)
dbm2(597)=dbm1(720)
dbm2(598)=dbm1(721)
dbm2(599)=dbm1(722)
dbm2(600)=dbm1(723)
dbm2(601)=dbm1(724)
dbm2(602)=dbm1(725)
dbm2(603)=dbm1(726)
dbm2(604)=dbm1(727)
dbm2(605)=dbm1(728)
dbm2(606)=dbm1(729)
dbm2(607)=dbm1(730)
dbm2(608)=dbm1(731)
dbm2(609)=dbm1(732)
dbm2(610)=dbm1(733)
dbm2(611)=dbm1(734)
dbm2(612)=dbm1(735)
dbm2(613)=dbm1(736)
dbm2(614)=dbm1(737)
dbm2(615)=dbm1(738)
dbm2(616)=dbm1(739)
dbm2(617)=dbm1(740)
dbm2(618)=dbm1(741)
dbm2(619)=dbm1(742)
dbm2(620)=dbm1(743)
dbm2(621)=dbm1(744)
dbm2(622)=dbm1(745)
dbm2(623)=dbm1(746)
dbm2(624)=dbm1(747)
dbm2(625)=dbm1(748)
dbm2(626)=dbm1(749)
dbm2(627)=dbm1(750)
dbm2(628)=dbm1(751)
dbm2(629)=dbm1(752)
dbm2(630)=dbm1(753)
dbm2(631)=dbm1(754)
dbm2(632)=dbm1(755)
dbm2(633)=dbm1(756)
dbm2(634)=dbm1(757)
dbm2(635)=dbm1(758)
dbm2(636)=dbm1(759)
dbm2(637)=dbm1(760)
dbm2(638)=dbm1(761)
dbm2(639)=dbm1(762)
dbm2(640)=dbm1(763)
dbm2(641)=dbm1(764)
dbm2(642)=dbm1(765)
dbm2(643)=dbm1(766)
dbm2(644)=dbm1(767)
dbm2(645)=dbm1(768)
dbm2(646)=dbm1(769)
dbm2(647)=dbm1(770)
dbm2(648)=dbm1(771)
dbm2(649)=dbm1(772)
dbm2(650)=dbm1(773)
dbm2(651)=dbm1(774)
dbm2(652)=dbm1(775)
dbm2(653)=dbm1(776)
dbm2(654)=dbm1(777)
dbm2(655)=dbm1(778)
dbm2(656)=dbm1(779)
dbm2(657)=dbm1(780)
dbm2(658)=dbm1(781)
dbm2(659)=dbm1(782)
dbm2(660)=dbm1(783)
dbm2(661)=dbm1(784)
dbm2(662)=dbm1(785)
dbm2(663)=dbm1(786)
dbm2(664)=dbm1(787)
dbm2(665)=dbm1(788)
dbm2(666)=dbm1(789)
dbm2(667)=dbm1(790)
dbm2(668)=dbm1(791)
dbm2(669)=dbm1(792)
dbm2(670)=dbm1(793)
dbm2(671)=dbm1(794)
dbm2(672)=dbm1(795)
dbm2(673)=dbm1(796)
dbm2(674)=dbm1(797)
dbm2(675)=dbm1(798)
dbm2(676)=dbm1(799)
dbm2(677)=dbm1(800)
dbm2(678)=dbm1(801)
dbm2(679)=dbm1(802)
dbm2(680)=dbm1(803)
dbm2(681)=dbm1(804)
dbm2(682)=dbm1(805)
dbm2(683)=dbm1(806)
dbm2(684)=dbm1(807)
dbm2(685)=dbm1(808)
dbm2(686)=dbm1(809)
dbm2(687)=dbm1(810)
dbm2(688)=dbm1(811)
dbm2(689)=dbm1(812)
dbm2(690)=dbm1(813)
dbm2(691)=dbm1(814)
dbm2(692)=dbm1(815)
dbm2(693)=dbm1(816)
dbm2(694)=dbm1(817)
dbm2(695)=dbm1(818)
dbm2(696)=dbm1(819)
dbm2(697)=dbm1(820)
dbm2(698)=dbm1(821)
dbm2(699)=dbm1(822)
dbm2(700)=dbm1(823)
dbm2(701)=dbm1(824)
dbm2(702)=dbm1(825)
dbm2(703)=dbm1(826)
dbm2(704)=dbm1(827)
dbm2(705)=dbm1(828)
dbm2(706)=dbm1(829)
dbm2(707)=dbm1(830)
dbm2(708)=dbm1(831)
dbm2(709)=dbm1(832)
dbm2(710)=dbm1(833)
dbm2(711)=dbm1(834)
dbm2(712)=dbm1(835)
dbm2(713)=dbm1(836)
dbm2(714)=dbm1(837)
dbm2(715)=dbm1(838)
dbm2(716)=dbm1(839)
dbm2(717)=dbm1(840)
dbm2(718)=dbm1(841)
dbm2(719)=dbm1(842)
dbm2(720)=dbm1(843)
dbm2(721)=dbm1(844)
dbm2(722)=dbm1(845)
dbm2(723)=dbm1(846)
dbm2(724)=dbm1(847)
dbm2(725)=dbm1(848)
dbm2(726)=dbm1(849)
dbm2(727)=dbm1(850)
dbm2(728)=dbm1(851)
dbm2(729)=dbm1(853)
dbm2(730)=dbm1(854)
dbm2(731)=dbm1(855)
dbm2(732)=dbm1(856)
dbm2(733)=dbm1(857)
dbm2(734)=dbm1(858)
dbm2(735)=dbm1(859)
dbm2(736)=dbm1(860)
dbm2(737)=dbm1(861)
dbm2(738)=dbm1(862)
dbm2(739)=dbm1(863)
dbm2(740)=dbm1(865)
dbm2(741)=dbm1(867)
dbm2(742)=dbm1(868)
dbm2(743)=dbm1(869)
dbm2(744)=dbm1(870)
dbm2(745)=dbm1(871)
dbm2(746)=dbm1(872)
dbm2(747)=dbm1(873)
dbm2(748)=dbm1(874)
dbm2(749)=dbm1(875)
dbm2(750)=dbm1(876)
dbm2(751)=dbm1(878)
dbm2(752)=dbm1(879)
dbm2(753)=dbm1(881)
dbm2(754)=dbm1(883)
dbm2(755)=dbm1(884)
dbm2(756)=dbm1(885)
dbm2(757)=dbm1(886)
dbm2(758)=dbm1(887)
dbm2(759)=dbm1(888)
dbm2(760)=dbm1(889)
dbm2(761)=dbm1(890)
dbm2(762)=dbm1(891)
dbm2(763)=dbm1(893)
dbm2(764)=dbm1(894)
dbm2(765)=dbm1(896)
dbm2(766)=dbm1(897)
dbm2(767)=dbm1(899)
dbm2(768)=dbm1(900)
dbm2(769)=dbm1(902)
dbm2(770)=dbm1(903)
dbm2(771)=dbm1(904)
dbm2(772)=dbm1(905)
dbm2(773)=dbm1(906)
dbm2(774)=dbm1(907)
dbm2(775)=dbm1(908)
dbm2(776)=dbm1(909)
dbm2(777)=dbm1(911)
dbm2(778)=dbm1(912)
dbm2(779)=dbm1(914)
dbm2(780)=dbm1(915)
dbm2(781)=dbm1(916)
dbm2(782)=dbm1(918)
dbm2(783)=dbm1(919)
dbm2(784)=dbm1(920)
dbm2(785)=dbm1(921)
dbm2(786)=dbm1(922)
dbm2(787)=dbm1(923)
dbm2(788)=dbm1(925)
dbm2(789)=dbm1(926)
dbm2(790)=dbm1(928)
dbm2(791)=dbm1(929)
dbm2(792)=dbm1(930)
dbm2(793)=dbm1(931)
dbm2(794)=dbm1(932)
dbm2(795)=dbm1(934)
dbm2(796)=dbm1(935)
dbm2(797)=dbm1(936)
dbm2(798)=dbm1(937)
dbm2(799)=dbm1(938)
dbm2(800)=dbm1(961)
dbm2(801)=dbm1(962)
dbm2(802)=dbm1(963)
dbm2(803)=dbm1(964)
dbm2(804)=dbm1(965)
dbm2(805)=dbm1(966)
dbm2(806)=dbm1(967)
dbm2(807)=dbm1(968)
dbm2(808)=dbm1(969)
dbm2(809)=dbm1(970)
dbm2(810)=dbm1(971)
dbm2(811)=dbm1(972)
dbm2(812)=dbm1(973)
dbm2(813)=dbm1(974)
dbm2(814)=dbm1(975)
dbm2(815)=dbm1(976)
dbm2(816)=dbm1(977)
dbm2(817)=dbm1(978)
dbm2(818)=dbm1(979)
dbm2(819)=dbm1(980)
dbm2(820)=dbm1(981)
dbm2(821)=dbm1(982)
dbm2(822)=dbm1(983)
dbm2(823)=dbm1(984)
dbm2(824)=dbm1(985)
dbm2(825)=dbm1(986)
dbm2(826)=dbm1(987)
dbm2(827)=dbm1(988)
dbm2(828)=dbm1(989)
dbm2(829)=dbm1(990)
dbm2(830)=dbm1(991)
dbm2(831)=dbm1(992)
dbm2(832)=dbm1(993)
dbm2(833)=dbm1(994)
dbm2(834)=dbm1(995)
dbm2(835)=dbm1(996)
dbm2(836)=dbm1(997)
dbm2(837)=dbm1(998)
dbm2(838)=dbm1(999)
dbm2(839)=dbm1(1000)
dbm2(840)=dbm1(1001)
dbm2(841)=dbm1(1002)
dbm2(842)=dbm1(1003)
dbm2(843)=dbm1(1004)
dbm2(844)=dbm1(1005)
dbm2(845)=dbm1(1006)
dbm2(846)=dbm1(1007)
dbm2(847)=dbm1(1008)
dbm2(848)=dbm1(1009)
dbm2(849)=dbm1(1010)
dbm2(850)=dbm1(1011)
dbm2(851)=dbm1(1012)
dbm2(852)=dbm1(1013)
dbm2(853)=dbm1(1014)
dbm2(854)=dbm1(1015)
dbm2(855)=dbm1(1016)
dbm2(856)=dbm1(1017)
dbm2(857)=dbm1(1018)
dbm2(858)=dbm1(1019)
dbm2(859)=dbm1(1020)
dbm2(860)=dbm1(1021)
dbm2(861)=dbm1(1022)
dbm2(862)=dbm1(1023)
dbm2(863)=dbm1(1024)
dbm2(864)=dbm1(1025)
dbm2(865)=dbm1(1026)
dbm2(866)=dbm1(1027)
dbm2(867)=dbm1(1028)
dbm2(868)=dbm1(1029)
dbm2(869)=dbm1(1030)
dbm2(870)=dbm1(1031)
dbm2(871)=dbm1(1032)
dbm2(872)=dbm1(1033)
dbm2(873)=dbm1(1034)
dbm2(874)=dbm1(1035)
dbm2(875)=dbm1(1036)
dbm2(876)=dbm1(1037)
dbm2(877)=dbm1(1038)
dbm2(878)=dbm1(1039)
dbm2(879)=dbm1(1040)
dbm2(880)=dbm1(1041)
dbm2(881)=dbm1(1042)
dbm2(882)=dbm1(1043)
dbm2(883)=dbm1(1044)
dbm2(884)=dbm1(1045)
dbm2(885)=dbm1(1046)
dbm2(886)=dbm1(1047)
dbm2(887)=dbm1(1048)
dbm2(888)=dbm1(1049)
dbm2(889)=dbm1(1050)
dbm2(890)=dbm1(1051)
dbm2(891)=dbm1(1052)
dbm2(892)=dbm1(1053)
dbm2(893)=dbm1(1054)
dbm2(894)=dbm1(1055)
dbm2(895)=dbm1(1056)
dbm2(896)=dbm1(1057)
dbm2(897)=dbm1(1058)
dbm2(898)=dbm1(1059)
dbm2(899)=dbm1(1060)
dbm2(900)=dbm1(1061)
dbm2(901)=dbm1(1062)
dbm2(902)=dbm1(1063)
dbm2(903)=dbm1(1064)
dbm2(904)=dbm1(1065)
dbm2(905)=dbm1(1066)
dbm2(906)=dbm1(1067)
dbm2(907)=dbm1(1068)
dbm2(908)=dbm1(1069)
dbm2(909)=dbm1(1070)
dbm2(910)=dbm1(1071)
dbm2(911)=dbm1(1072)
dbm2(912)=dbm1(1073)
dbm2(913)=dbm1(1074)
dbm2(914)=dbm1(1075)
dbm2(915)=dbm1(1076)
dbm2(916)=dbm1(1077)
dbm2(917)=dbm1(1078)
dbm2(918)=dbm1(1079)
dbm2(919)=dbm1(1080)
dbm2(920)=dbm1(1081)
dbm2(921)=dbm1(1082)
dbm2(922)=dbm1(1083)
dbm2(923)=dbm1(1084)
dbm2(924)=dbm1(1085)
dbm2(925)=dbm1(1086)
dbm2(926)=dbm1(1087)
dbm2(927)=dbm1(1088)
dbm2(928)=dbm1(1089)
dbm2(929)=dbm1(1090)
dbm2(930)=dbm1(1091)
dbm2(931)=dbm1(1092)
dbm2(932)=dbm1(1093)
dbm2(933)=dbm1(1094)
dbm2(934)=dbm1(1095)
dbm2(935)=dbm1(1096)
dbm2(936)=dbm1(1097)
dbm2(937)=dbm1(1098)
dbm2(938)=dbm1(1099)
dbm2(939)=dbm1(1100)
dbm2(940)=dbm1(1101)
dbm2(941)=dbm1(1102)
dbm2(942)=dbm1(1103)
dbm2(943)=dbm1(1104)
dbm2(944)=dbm1(1105)
dbm2(945)=dbm1(1106)
dbm2(946)=dbm1(1107)
dbm2(947)=dbm1(1108)
dbm2(948)=dbm1(1109)
dbm2(949)=dbm1(1110)
dbm2(950)=dbm1(1111)
dbm2(951)=dbm1(1112)
dbm2(952)=dbm1(1113)
dbm2(953)=dbm1(1114)
dbm2(954)=dbm1(1115)
dbm2(955)=dbm1(1116)
dbm2(956)=dbm1(1117)
dbm2(957)=dbm1(1118)
dbm2(958)=dbm1(1119)
dbm2(959)=dbm1(1120)
dbm2(960)=dbm1(1121)
dbm2(961)=dbm1(1122)
dbm2(962)=dbm1(1123)
dbm2(963)=dbm1(1124)
dbm2(964)=dbm1(1125)
dbm2(965)=dbm1(1126)
dbm2(966)=dbm1(1127)
dbm2(967)=dbm1(1128)
dbm2(968)=dbm1(1129)
dbm2(969)=dbm1(1130)
dbm2(970)=dbm1(1131)
dbm2(971)=dbm1(1132)
dbm2(972)=dbm1(1133)
dbm2(973)=dbm1(1134)
dbm2(974)=dbm1(1135)
dbm2(975)=dbm1(1136)
dbm2(976)=dbm1(1137)
dbm2(977)=dbm1(1138)
dbm2(978)=dbm1(1139)
dbm2(979)=dbm1(1140)
dbm2(980)=dbm1(1141)
dbm2(981)=dbm1(1142)
dbm2(982)=dbm1(1143)
dbm2(983)=dbm1(1144)
dbm2(984)=dbm1(1145)
dbm2(985)=dbm1(1146)
dbm2(986)=dbm1(1147)
dbm2(987)=dbm1(1148)
dbm2(988)=dbm1(1149)
dbm2(989)=dbm1(1150)
dbm2(990)=dbm1(1151)
dbm2(991)=dbm1(1152)
dbm2(992)=dbm1(1153)
dbm2(993)=dbm1(1154)
dbm2(994)=dbm1(1155)
dbm2(995)=dbm1(1156)
dbm2(996)=dbm1(1157)
dbm2(997)=dbm1(1158)
dbm2(998)=dbm1(1159)
dbm2(999)=dbm1(1160)
dbm2(1000)=dbm1(1161)
dbm2(1001)=dbm1(1162)
dbm2(1002)=dbm1(1163)
dbm2(1003)=dbm1(1164)
dbm2(1004)=dbm1(1165)
dbm2(1005)=dbm1(1166)
dbm2(1006)=dbm1(1167)
dbm2(1007)=dbm1(1168)
dbm2(1008)=dbm1(1169)
dbm2(1009)=dbm1(1170)
dbm2(1010)=dbm1(1171)
dbm2(1011)=dbm1(1172)
dbm2(1012)=dbm1(1173)
dbm2(1013)=dbm1(1174)
dbm2(1014)=dbm1(1175)
dbm2(1015)=dbm1(1176)
dbm2(1016)=dbm1(1177)
dbm2(1017)=dbm1(1178)
dbm2(1018)=dbm1(1179)
dbm2(1019)=dbm1(1180)
dbm2(1020)=dbm1(1181)
dbm2(1021)=dbm1(1182)
dbm2(1022)=dbm1(1183)
dbm2(1023)=dbm1(1184)
dbm2(1024)=dbm1(1185)
dbm2(1025)=dbm1(1186)
dbm2(1026)=dbm1(1187)
dbm2(1027)=dbm1(1188)
dbm2(1028)=dbm1(1189)
dbm2(1029)=dbm1(1190)
dbm2(1030)=dbm1(1191)
dbm2(1031)=dbm1(1192)
dbm2(1032)=dbm1(1193)
dbm2(1033)=dbm1(1194)
dbm2(1034)=dbm1(1195)
dbm2(1035)=dbm1(1196)
dbm2(1036)=dbm1(1197)
dbm2(1037)=dbm1(1198)
dbm2(1038)=dbm1(1199)
dbm2(1039)=dbm1(1200)
dbm2(1040)=dbm1(1201)
dbm2(1041)=dbm1(1202)
dbm2(1042)=dbm1(1203)
dbm2(1043)=dbm1(1204)
dbm2(1044)=dbm1(1205)
dbm2(1045)=dbm1(1206)
dbm2(1046)=dbm1(1207)
dbm2(1047)=dbm1(1208)
dbm2(1048)=dbm1(1209)
dbm2(1049)=dbm1(1210)
dbm2(1050)=dbm1(1211)
dbm2(1051)=dbm1(1212)
dbm2(1052)=dbm1(1213)
dbm2(1053)=dbm1(1214)
dbm2(1054)=dbm1(1215)
dbm2(1055)=dbm1(1216)
dbm2(1056)=dbm1(1217)
dbm2(1057)=dbm1(1218)
dbm2(1058)=dbm1(1219)
dbm2(1059)=dbm1(1220)
dbm2(1060)=dbm1(1221)
dbm2(1061)=dbm1(1222)
dbm2(1062)=dbm1(1223)
dbm2(1063)=dbm1(1224)
dbm2(1064)=dbm1(1225)
dbm2(1065)=dbm1(1226)
dbm2(1066)=dbm1(1227)
dbm2(1067)=dbm1(1228)
dbm2(1068)=dbm1(1229)
dbm2(1069)=dbm1(1230)
dbm2(1070)=dbm1(1231)
dbm2(1071)=dbm1(1232)
dbm2(1072)=dbm1(1233)
dbm2(1073)=dbm1(1234)
dbm2(1074)=dbm1(1235)
dbm2(1075)=dbm1(1236)
dbm2(1076)=dbm1(1237)
dbm2(1077)=dbm1(1238)
dbm2(1078)=dbm1(1239)
dbm2(1079)=dbm1(1240)
dbm2(1080)=dbm1(1241)
dbm2(1081)=dbm1(1242)
dbm2(1082)=dbm1(1243)
dbm2(1083)=dbm1(1244)
dbm2(1084)=dbm1(1245)
dbm2(1085)=dbm1(1246)
dbm2(1086)=dbm1(1247)
dbm2(1087)=dbm1(1248)
dbm2(1088)=dbm1(1249)
dbm2(1089)=dbm1(1250)
dbm2(1090)=dbm1(1251)
dbm2(1091)=dbm1(1252)
dbm2(1092)=dbm1(1253)
dbm2(1093)=dbm1(1254)
dbm2(1094)=dbm1(1255)
dbm2(1095)=dbm1(1256)
dbm2(1096)=dbm1(1257)
dbm2(1097)=dbm1(1258)
dbm2(1098)=dbm1(1259)
dbm2(1099)=dbm1(1260)
dbm2(1100)=dbm1(1261)
dbm2(1101)=dbm1(1262)
dbm2(1102)=dbm1(1263)
dbm2(1103)=dbm1(1264)
dbm2(1104)=dbm1(1265)
dbm2(1105)=dbm1(1266)
dbm2(1106)=dbm1(1267)
dbm2(1107)=dbm1(1268)
dbm2(1108)=dbm1(1269)
dbm2(1109)=dbm1(1270)
dbm2(1110)=dbm1(1271)
dbm2(1111)=dbm1(1272)
dbm2(1112)=dbm1(1273)
dbm2(1113)=dbm1(1274)
dbm2(1114)=dbm1(1275)
dbm2(1115)=dbm1(1276)
dbm2(1116)=dbm1(1277)
dbm2(1117)=dbm1(1278)
dbm2(1118)=dbm1(1279)
dbm2(1119)=dbm1(1280)
dbm2(1120)=dbm1(1281)
dbm2(1121)=dbm1(1282)
dbm2(1122)=dbm1(1283)
dbm2(1123)=dbm1(1284)
dbm2(1124)=dbm1(1285)
dbm2(1125)=dbm1(1286)
dbm2(1126)=dbm1(1287)
dbm2(1127)=dbm1(1288)
dbm2(1128)=dbm1(1289)
dbm2(1129)=dbm1(1290)
dbm2(1130)=dbm1(1291)
dbm2(1131)=dbm1(1292)
dbm2(1132)=dbm1(1293)
dbm2(1133)=dbm1(1294)
dbm2(1134)=dbm1(1295)
dbm2(1135)=dbm1(1296)
dbm2(1136)=dbm1(1297)
dbm2(1137)=dbm1(1298)
dbm2(1138)=dbm1(1299)
dbm2(1139)=dbm1(1300)
dbm2(1140)=dbm1(1301)
dbm2(1141)=dbm1(1302)
dbm2(1142)=dbm1(1303)
dbm2(1143)=dbm1(1304)
dbm2(1144)=dbm1(1305)
dbm2(1145)=dbm1(1306)
dbm2(1146)=dbm1(1307)
dbm2(1147)=dbm1(1308)
dbm2(1148)=dbm1(1309)
dbm2(1149)=dbm1(1310)
dbm2(1150)=dbm1(1311)
dbm2(1151)=dbm1(1312)
dbm2(1152)=dbm1(1313)
dbm2(1153)=dbm1(1314)
dbm2(1154)=dbm1(1315)
dbm2(1155)=dbm1(1316)
dbm2(1156)=dbm1(1317)
dbm2(1157)=dbm1(1318)
dbm2(1158)=dbm1(1319)
dbm2(1159)=dbm1(1320)
dbm2(1160)=dbm1(1321)
dbm2(1161)=dbm1(1322)
dbm2(1162)=dbm1(1323)
dbm2(1163)=dbm1(1324)
dbm2(1164)=dbm1(1325)
dbm2(1165)=dbm1(1326)
dbm2(1166)=dbm1(1327)
dbm2(1167)=dbm1(1328)
dbm2(1168)=dbm1(1329)
dbm2(1169)=dbm1(1330)
dbm2(1170)=dbm1(1331)
dbm2(1171)=dbm1(1332)
dbm2(1172)=dbm1(1333)
dbm2(1173)=dbm1(1334)
dbm2(1174)=dbm1(1335)
dbm2(1175)=dbm1(1336)
dbm2(1176)=dbm1(1337)
dbm2(1177)=dbm1(1338)
dbm2(1178)=dbm1(1339)
dbm2(1179)=dbm1(1340)
dbm2(1180)=dbm1(1341)
dbm2(1181)=dbm1(1342)
dbm2(1182)=dbm1(1343)
dbm2(1183)=dbm1(1344)
dbm2(1184)=dbm1(1345)
dbm2(1185)=dbm1(1346)
dbm2(1186)=dbm1(1347)
dbm2(1187)=dbm1(1348)
dbm2(1188)=dbm1(1349)
dbm2(1189)=dbm1(1350)
dbm2(1190)=dbm1(1351)
dbm2(1191)=dbm1(1352)
dbm2(1192)=dbm1(1353)
dbm2(1193)=dbm1(1354)
dbm2(1194)=dbm1(1355)
dbm2(1195)=dbm1(1356)
dbm2(1196)=dbm1(1357)
dbm2(1197)=dbm1(1358)
dbm2(1198)=dbm1(1359)
dbm2(1199)=dbm1(1360)
dbm2(1200)=dbm1(1361)
dbm2(1201)=dbm1(1362)
dbm2(1202)=dbm1(1363)
dbm2(1203)=dbm1(1364)
dbm2(1204)=dbm1(1365)
dbm2(1205)=dbm1(1366)
dbm2(1206)=dbm1(1367)
dbm2(1207)=dbm1(1368)
dbm2(1208)=dbm1(1369)
dbm2(1209)=dbm1(1371)
dbm2(1210)=dbm1(1372)
dbm2(1211)=dbm1(1373)
dbm2(1212)=dbm1(1374)
dbm2(1213)=dbm1(1375)
dbm2(1214)=dbm1(1376)
dbm2(1215)=dbm1(1377)
dbm2(1216)=dbm1(1378)
dbm2(1217)=dbm1(1379)
dbm2(1218)=dbm1(1380)
dbm2(1219)=dbm1(1381)
dbm2(1220)=dbm1(1382)
dbm2(1221)=dbm1(1383)
dbm2(1222)=dbm1(1385)
dbm2(1223)=dbm1(1387)
dbm2(1224)=dbm1(1388)
dbm2(1225)=dbm1(1389)
dbm2(1226)=dbm1(1390)
dbm2(1227)=dbm1(1391)
dbm2(1228)=dbm1(1392)
dbm2(1229)=dbm1(1393)
dbm2(1230)=dbm1(1394)
dbm2(1231)=dbm1(1395)
dbm2(1232)=dbm1(1396)
dbm2(1233)=dbm1(1397)
dbm2(1234)=dbm1(1399)
dbm2(1235)=dbm1(1400)
dbm2(1236)=dbm1(1402)
dbm2(1237)=dbm1(1404)
dbm2(1238)=dbm1(1405)
dbm2(1239)=dbm1(1406)
dbm2(1240)=dbm1(1407)
dbm2(1241)=dbm1(1408)
dbm2(1242)=dbm1(1409)
dbm2(1243)=dbm1(1410)
dbm2(1244)=dbm1(1411)
dbm2(1245)=dbm1(1412)
dbm2(1246)=dbm1(1413)
dbm2(1247)=dbm1(1415)
dbm2(1248)=dbm1(1416)
dbm2(1249)=dbm1(1418)
dbm2(1250)=dbm1(1419)
dbm2(1251)=dbm1(1421)
dbm2(1252)=dbm1(1422)
dbm2(1253)=dbm1(1424)
dbm2(1254)=dbm1(1425)
dbm2(1255)=dbm1(1426)
dbm2(1256)=dbm1(1427)
dbm2(1257)=dbm1(1428)
dbm2(1258)=dbm1(1429)
dbm2(1259)=dbm1(1430)
dbm2(1260)=dbm1(1431)
dbm2(1261)=dbm1(1432)
dbm2(1262)=dbm1(1434)
dbm2(1263)=dbm1(1435)
dbm2(1264)=dbm1(1437)
dbm2(1265)=dbm1(1438)
dbm2(1266)=dbm1(1440)
dbm2(1267)=dbm1(1441)
dbm2(1268)=dbm1(1442)
dbm2(1269)=dbm1(1444)
dbm2(1270)=dbm1(1445)
dbm2(1271)=dbm1(1446)
dbm2(1272)=dbm1(1447)
dbm2(1273)=dbm1(1448)
dbm2(1274)=dbm1(1449)
dbm2(1275)=dbm1(1450)
dbm2(1276)=dbm1(1451)
dbm2(1277)=dbm1(1453)
dbm2(1278)=dbm1(1454)
dbm2(1279)=dbm1(1456)
dbm2(1280)=dbm1(1457)
dbm2(1281)=dbm1(1458)
dbm2(1282)=dbm1(1460)
dbm2(1283)=dbm1(1461)
dbm2(1284)=dbm1(1462)
dbm2(1285)=dbm1(1463)
dbm2(1286)=dbm1(1464)
dbm2(1287)=dbm1(1465)
dbm2(1288)=dbm1(1467)
dbm2(1289)=dbm1(1468)
dbm2(1290)=dbm1(1470)
dbm2(1291)=dbm1(1471)
dbm2(1292)=dbm1(1472)
dbm2(1293)=dbm1(1473)
dbm2(1294)=dbm1(1474)
dbm2(1295)=dbm1(1476)
dbm2(1296)=dbm1(1477)
dbm2(1297)=dbm1(1478)
dbm2(1298)=dbm1(1479)
dbm2(1299)=dbm1(1480)
dbm2(1300)=dbm1(1481)

dVdR(i)=dVdR(i) + dot_product(coeff,dbm2)

end do

end subroutine EvU23O10dVdR
!===========================================================================
! *def u12_order10
! subtoutine of computing u12
!===========================================================================
subroutine u12_order10
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp) :: xyz(4,3)
real(dp) :: v, r12, r13, r14, r23, r24, r34
real(dp) :: s
real(dp) :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=1300
real(dp) :: coeff(ncoeff), q(ncoeff)
real(dp) :: x(6)
integer ::  ncall
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer :: i, j
real(dp) :: rrel(3,3),cross(3)
real(dp) :: volume

if(ncall.eq.1) then
a1=0.610
a2=0.800
b1=1.300
b2=1.300
ra1=0.802
ra2=0.702
rb1=0.802
rb2=0.702
coeff(1)=-0.200894146381D+03
coeff(2)=0.928472731432D+03
coeff(3)=-0.261022613087D+03
coeff(4)=-0.539380483332D+03
coeff(5)=0.101994742299D+04
coeff(6)=0.647330515917D+03
coeff(7)=0.675393110624D+03
coeff(8)=0.899938890523D+03
coeff(9)=-0.235323238714D+04
coeff(10)=-0.102800493753D+04
coeff(11)=0.933981557311D+02
coeff(12)=-0.171628272374D+04
coeff(13)=0.201570736612D+03
coeff(14)=-0.426825593199D+02
coeff(15)=0.673922176155D+03
coeff(16)=-0.483927233291D+03
coeff(17)=0.867741279535D+03
coeff(18)=0.142255818055D+04
coeff(19)=-0.427373733305D+04
coeff(20)=-0.784412615099D+03
coeff(21)=-0.155530579244D+04
coeff(22)=-0.155883450796D+04
coeff(23)=-0.654639670041D+03
coeff(24)=0.181115660004D+04
coeff(25)=-0.244496279907D+04
coeff(26)=-0.165264596406D+04
coeff(27)=0.737035495930D+02
coeff(28)=0.141620626401D+04
coeff(29)=0.269991251659D+04
coeff(30)=-0.676458531477D+03
coeff(31)=-0.169154008032D+04
coeff(32)=-0.944404764593D+03
coeff(33)=0.340855836109D+04
coeff(34)=0.226704552355D+03
coeff(35)=0.246754054573D+04
coeff(36)=0.148288275501D+03
coeff(37)=0.233408841386D+04
coeff(38)=0.318853150581D+03
coeff(39)=0.238613613333D+04
coeff(40)=-0.876239994343D+03
coeff(41)=0.112504579845D+04
coeff(42)=0.506300363576D+03
coeff(43)=0.638915481375D+03
coeff(44)=-0.623598752472D+03
coeff(45)=0.449543177665D+03
coeff(46)=0.589381722221D+03
coeff(47)=-0.200968926514D+04
coeff(48)=-0.186832614839D+04
coeff(49)=-0.182206810242D+04
coeff(50)=0.212950142336D+04
coeff(51)=0.186957763652D+04
coeff(52)=0.989689773718D+03
coeff(53)=-0.104938016545D+04
coeff(54)=-0.623003253753D+03
coeff(55)=0.862418465912D+04
coeff(56)=-0.214380543403D+04
coeff(57)=0.235459109660D+04
coeff(58)=0.736192412255D+03
coeff(59)=0.325410137455D+03
coeff(60)=-0.205469280833D+03
coeff(61)=0.130184460755D+03
coeff(62)=-0.108472886090D+04
coeff(63)=0.399737276425D+03
coeff(64)=-0.167176651975D+03
coeff(65)=-0.141072947461D+04
coeff(66)=0.444092859290D+03
coeff(67)=0.123658740994D+04
coeff(68)=-0.226202667988D+03
coeff(69)=0.352138440560D+02
coeff(70)=-0.483844332811D+03
coeff(71)=0.264395043830D+04
coeff(72)=-0.183085592243D+04
coeff(73)=-0.107346947910D+04
coeff(74)=-0.734823867609D+03
coeff(75)=0.117033290131D+04
coeff(76)=-0.169258470030D+04
coeff(77)=-0.119438319169D+04
coeff(78)=0.788790342290D+02
coeff(79)=-0.133808310276D+04
coeff(80)=-0.142161213940D+04
coeff(81)=-0.993662708023D+03
coeff(82)=0.734629534896D+03
coeff(83)=-0.161178090757D+04
coeff(84)=-0.888189147482D+03
coeff(85)=-0.218372314811D+04
coeff(86)=0.343491747509D+04
coeff(87)=-0.236749781172D+03
coeff(88)=0.325758505660D+03
coeff(89)=-0.875218917296D+03
coeff(90)=-0.120235160216D+04
coeff(91)=0.683181195726D+03
coeff(92)=0.523282563065D+02
coeff(93)=0.129950054147D+03
coeff(94)=-0.134769267040D+04
coeff(95)=0.231629678179D+04
coeff(96)=-0.415227960771D+04
coeff(97)=-0.179195717636D+03
coeff(98)=0.765656978143D+03
coeff(99)=-0.965862571783D+03
coeff(100)=0.135201263341D+04
coeff(101)=-0.510678236717D+03
coeff(102)=-0.152720550294D+04
coeff(103)=0.133169086131D+03
coeff(104)=-0.220644036625D+03
coeff(105)=-0.163708057120D+03
coeff(106)=-0.703466042263D+03
coeff(107)=-0.162495506707D+04
coeff(108)=0.711166708732D+03
coeff(109)=-0.747350308009D+03
coeff(110)=0.190408564193D+04
coeff(111)=0.592613801639D+03
coeff(112)=0.521056675424D+04
coeff(113)=-0.280208388985D+04
coeff(114)=0.224666365548D+04
coeff(115)=0.725205768488D+03
coeff(116)=0.646228531058D+03
coeff(117)=-0.103535653021D+03
coeff(118)=-0.123554448405D+03
coeff(119)=-0.303225918119D+04
coeff(120)=0.805717049353D+03
coeff(121)=-0.355539982112D+03
coeff(122)=0.945988029053D+03
coeff(123)=-0.655811716051D+03
coeff(124)=0.583955330275D+03
coeff(125)=0.611619773338D+03
coeff(126)=-0.468866760034D+04
coeff(127)=0.157548388637D+03
coeff(128)=-0.729793420938D+03
coeff(129)=-0.145369942131D+04
coeff(130)=0.126799369428D+04
coeff(131)=-0.667404937619D+03
coeff(132)=0.914719526679D+03
coeff(133)=0.322150888689D+02
coeff(134)=-0.538667256852D+03
coeff(135)=-0.206725654102D+03
coeff(136)=-0.149140806794D+04
coeff(137)=-0.151596074474D+04
coeff(138)=0.141871203235D+03
coeff(139)=0.143592027921D+03
coeff(140)=0.143880424018D+04
coeff(141)=0.170738383011D+03
coeff(142)=0.343896560635D+03
coeff(143)=0.638029490904D+03
coeff(144)=-0.112636907440D+03
coeff(145)=0.214222335835D+04
coeff(146)=-0.523222254632D+03
coeff(147)=-0.140579762449D+03
coeff(148)=-0.351804416562D+03
coeff(149)=-0.888396679338D+03
coeff(150)=0.434711683128D+03
coeff(151)=0.867984744801D+03
coeff(152)=0.331629185490D+03
coeff(153)=0.521630120106D+03
coeff(154)=-0.542106566074D+03
coeff(155)=0.795716722331D+03
coeff(156)=0.710700065662D+03
coeff(157)=-0.152090367813D+04
coeff(158)=0.413206522727D+03
coeff(159)=-0.520312375797D+03
coeff(160)=-0.110915179801D+03
coeff(161)=0.204693037890D+03
coeff(162)=0.373821957710D+03
coeff(163)=0.133096503606D+04
coeff(164)=0.196348207409D+04
coeff(165)=0.288120718428D+03
coeff(166)=0.356128093066D+03
coeff(167)=0.331838064532D+03
coeff(168)=0.146026855084D+04
coeff(169)=-0.195553151498D+03
coeff(170)=-0.117756698159D+04
coeff(171)=-0.171550596151D+03
coeff(172)=-0.130759930834D+04
coeff(173)=-0.445191810623D+03
coeff(174)=-0.173737607500D+04
coeff(175)=0.687299557843D+03
coeff(176)=0.706403699237D+03
coeff(177)=-0.744779260936D+03
coeff(178)=-0.646267841442D+03
coeff(179)=-0.811729433516D+02
coeff(180)=-0.193988487167D+04
coeff(181)=0.633356171360D+03
coeff(182)=-0.867221391780D+03
coeff(183)=-0.125897020563D+04
coeff(184)=0.404060352679D+03
coeff(185)=-0.280889332955D+04
coeff(186)=0.240531094170D+03
coeff(187)=-0.216103417754D+01
coeff(188)=0.601430406874D+03
coeff(189)=-0.207029470562D+04
coeff(190)=-0.543435091641D+03
coeff(191)=-0.243916614522D+04
coeff(192)=0.109598685360D+04
coeff(193)=0.645162644262D+03
coeff(194)=0.655697219345D+03
coeff(195)=-0.604903522140D+03
coeff(196)=-0.107827290966D+04
coeff(197)=-0.191719074532D+03
coeff(198)=-0.302041540440D+03
coeff(199)=-0.355405487572D+03
coeff(200)=0.615023569188D+03
coeff(201)=0.124155541609D+04
coeff(202)=0.258695630266D+03
coeff(203)=-0.401480283518D+03
coeff(204)=0.100072827158D+04
coeff(205)=-0.102976277956D+04
coeff(206)=0.150123586377D+03
coeff(207)=-0.637287736160D+03
coeff(208)=-0.574794457263D+03
coeff(209)=-0.119233079810D+04
coeff(210)=-0.109843119367D+04
coeff(211)=-0.102908495195D+04
coeff(212)=-0.662602554218D+02
coeff(213)=0.489870096551D+03
coeff(214)=-0.751847472718D+03
coeff(215)=-0.776415366781D+03
coeff(216)=-0.527277102837D+03
coeff(217)=-0.403547176807D+02
coeff(218)=-0.739812735654D+01
coeff(219)=0.262775980485D+03
coeff(220)=0.392616254888D+02
coeff(221)=-0.812206587196D+03
coeff(222)=0.122742838024D+04
coeff(223)=-0.764260201236D+03
coeff(224)=-0.228614614402D+02
coeff(225)=0.518227998892D+03
coeff(226)=0.193496684816D+04
coeff(227)=-0.273066697453D+03
coeff(228)=-0.725380271752D+03
coeff(229)=0.369109280008D+03
coeff(230)=-0.554039003548D+02
coeff(231)=-0.450553396709D+03
coeff(232)=0.226272404697D+03
coeff(233)=0.765893207415D+03
coeff(234)=-0.110874166091D+04
coeff(235)=-0.215106244001D+04
coeff(236)=0.816626550797D+03
coeff(237)=-0.791092175399D+03
coeff(238)=0.388689940385D+03
coeff(239)=0.699668741983D+03
coeff(240)=-0.164500177961D+04
coeff(241)=-0.187038976801D+04
coeff(242)=0.600680605931D+03
coeff(243)=0.163282488546D+04
coeff(244)=0.299455560877D+04
coeff(245)=0.565737355058D+03
coeff(246)=-0.178734131557D+04
coeff(247)=-0.365618527190D+04
coeff(248)=0.147887247668D+04
coeff(249)=-0.235421917192D+03
coeff(250)=-0.813363491057D+03
coeff(251)=-0.969058058779D+03
coeff(252)=-0.215789755499D+04
coeff(253)=0.233340738590D+04
coeff(254)=-0.683119251331D+03
coeff(255)=0.578809963854D+02
coeff(256)=-0.915318425202D+03
coeff(257)=0.901603106348D+03
coeff(258)=0.828814047928D+03
coeff(259)=0.615349573822D+03
coeff(260)=0.527217977439D+03
coeff(261)=0.967204713220D+03
coeff(262)=0.238558368661D+03
coeff(263)=-0.479955026989D+03
coeff(264)=-0.403638464503D+03
coeff(265)=-0.102797104104D+04
coeff(266)=-0.827124260843D+03
coeff(267)=0.115387476210D+03
coeff(268)=-0.515873671862D+03
coeff(269)=-0.293002428734D+03
coeff(270)=0.508032401944D+03
coeff(271)=-0.103047877262D+04
coeff(272)=0.890738386657D+03
coeff(273)=0.828553053434D+03
coeff(274)=-0.375657226633D+03
coeff(275)=0.186456532104D+04
coeff(276)=0.984441155653D+03
coeff(277)=-0.877572070872D+03
coeff(278)=0.951474515645D+03
coeff(279)=-0.103451129958D+04
coeff(280)=0.282312445083D+03
coeff(281)=0.466225814165D+02
coeff(282)=0.154658379689D+04
coeff(283)=0.122789252503D+04
coeff(284)=0.260060004925D+03
coeff(285)=0.205501411752D+04
coeff(286)=0.110306699892D+04
coeff(287)=-0.917039012542D+03
coeff(288)=0.576611060955D+03
coeff(289)=0.415142439150D+03
coeff(290)=-0.141661903790D+04
coeff(291)=0.104650805634D+04
coeff(292)=-0.565009767167D+02
coeff(293)=0.706000553461D+03
coeff(294)=0.110389054648D+04
coeff(295)=-0.580662972080D+03
coeff(296)=0.164126287102D+04
coeff(297)=-0.117362142784D+03
coeff(298)=-0.149586544576D+04
coeff(299)=0.444078089560D+03
coeff(300)=0.714031242900D+03
coeff(301)=0.127569062970D+04
coeff(302)=-0.815004933142D+03
coeff(303)=-0.200408526616D+03
coeff(304)=-0.109337385682D+04
coeff(305)=-0.375115315414D+03
coeff(306)=-0.163747780822D+03
coeff(307)=-0.610925607666D+03
coeff(308)=0.159945093873D+04
coeff(309)=0.732222462123D+03
coeff(310)=0.734388032520D+03
coeff(311)=-0.997506333285D+03
coeff(312)=-0.287706522688D+03
coeff(313)=0.178956389419D+04
coeff(314)=0.711334699833D+03
coeff(315)=0.702902388406D+03
coeff(316)=0.265059183422D+03
coeff(317)=0.294521896644D+03
coeff(318)=-0.917895768273D+03
coeff(319)=0.173697825919D+04
coeff(320)=-0.538653526579D+02
coeff(321)=-0.118945502938D+03
coeff(322)=-0.497197798650D+03
coeff(323)=0.115094733875D+03
coeff(324)=0.807005375994D+03
coeff(325)=-0.109067760430D+04
coeff(326)=-0.997367502542D+02
coeff(327)=0.386789371762D+03
coeff(328)=-0.710112939863D+03
coeff(329)=0.594666995642D+03
coeff(330)=-0.212142037563D+03
coeff(331)=0.496222702748D+02
coeff(332)=0.150769143441D+03
coeff(333)=0.119752869441D+04
coeff(334)=-0.773015119993D+03
coeff(335)=0.104793566531D+03
coeff(336)=-0.303063663809D+03
coeff(337)=-0.123111783563D+04
coeff(338)=0.104113220732D+04
coeff(339)=-0.128125941323D+03
coeff(340)=-0.663444508785D+03
coeff(341)=0.144018364126D+04
coeff(342)=0.402697329492D+03
coeff(343)=0.480736871685D+03
coeff(344)=-0.831966474099D+03
coeff(345)=-0.152861557954D+04
coeff(346)=-0.166818456682D+02
coeff(347)=-0.114133211609D+04
coeff(348)=0.272323226168D+03
coeff(349)=0.277106837249D+03
coeff(350)=-0.360293667318D+03
coeff(351)=0.902991974160D+03
coeff(352)=0.187459087864D+04
coeff(353)=-0.485188980960D+03
coeff(354)=-0.332195464323D+03
coeff(355)=0.383494096550D+03
coeff(356)=-0.238605308839D+04
coeff(357)=0.459660093879D+03
coeff(358)=0.133821259985D+04
coeff(359)=0.134343031413D+04
coeff(360)=-0.127270918304D+04
coeff(361)=0.728823250699D+03
coeff(362)=-0.181795228771D+04
coeff(363)=-0.531202184175D+03
coeff(364)=-0.611068123602D+02
coeff(365)=0.111597848440D+04
coeff(366)=0.592271799368D+03
coeff(367)=-0.272340952710D+03
coeff(368)=0.291732395295D+03
coeff(369)=0.997992961141D+03
coeff(370)=0.752427070673D+03
coeff(371)=0.116879825892D+04
coeff(372)=-0.226492037080D+04
coeff(373)=0.706536608610D+02
coeff(374)=0.117211504329D+04
coeff(375)=0.164871862358D+04
coeff(376)=0.150177348742D+04
coeff(377)=0.122552880984D+04
coeff(378)=0.833942293197D+01
coeff(379)=-0.144562096311D+04
coeff(380)=0.108610304952D+04
coeff(381)=0.115845888720D+04
coeff(382)=-0.303114876624D+03
coeff(383)=-0.616893081338D+03
coeff(384)=0.229094909894D+03
coeff(385)=0.120724798904D+04
coeff(386)=0.114325641937D+04
coeff(387)=-0.601175020694D+03
coeff(388)=-0.106757585888D+04
coeff(389)=-0.130709707153D+04
coeff(390)=0.161883051938D+02
coeff(391)=-0.747973705109D+03
coeff(392)=0.175601704664D+04
coeff(393)=0.133209253299D+04
coeff(394)=-0.860039085035D+03
coeff(395)=-0.696526399350D+03
coeff(396)=0.558222931165D+03
coeff(397)=-0.964634988739D+03
coeff(398)=-0.108401713961D+04
coeff(399)=0.150355363499D+04
coeff(400)=0.195005627260D+04
coeff(401)=-0.119425105391D+03
coeff(402)=-0.963239464768D+03
coeff(403)=0.183169576071D+04
coeff(404)=0.861432942401D+03
coeff(405)=-0.823569238545D+03
coeff(406)=-0.156794901970D+04
coeff(407)=0.133151445989D+04
coeff(408)=0.674267200599D+03
coeff(409)=-0.645159550604D+03
coeff(410)=0.161649331238D+04
coeff(411)=0.732457298162D+03
coeff(412)=0.160560300437D+03
coeff(413)=-0.292196795980D+03
coeff(414)=-0.398542514294D+02
coeff(415)=0.202041467094D+04
coeff(416)=0.571721782022D+03
coeff(417)=-0.285549174304D+03
coeff(418)=0.307213237068D+02
coeff(419)=0.174105889266D+03
coeff(420)=-0.291680491586D+03
coeff(421)=-0.268685548781D+04
coeff(422)=-0.126505882604D+04
coeff(423)=0.385265788397D+03
coeff(424)=0.115016053641D+04
coeff(425)=-0.163453738874D+04
coeff(426)=0.936310771331D+01
coeff(427)=-0.175710163164D+03
coeff(428)=-0.170833884633D+04
coeff(429)=-0.153747078616D+03
coeff(430)=-0.511240193455D+03
coeff(431)=0.358595282594D+03
coeff(432)=-0.406710982140D+04
coeff(433)=0.141516174424D+04
coeff(434)=0.118306285456D+02
coeff(435)=0.366870943342D+03
coeff(436)=-0.590909258567D+03
coeff(437)=0.144739072535D+04
coeff(438)=0.328343416805D+03
coeff(439)=0.291703134667D+02
coeff(440)=0.333237942269D+03
coeff(441)=0.489995424913D+03
coeff(442)=-0.109229818515D+04
coeff(443)=0.319085386891D+04
coeff(444)=-0.272190042873D+03
coeff(445)=-0.424021824758D+03
coeff(446)=-0.170416402581D+04
coeff(447)=-0.110358676803D+04
coeff(448)=0.165430956456D+03
coeff(449)=-0.386023778652D+03
coeff(450)=-0.531851966314D+03
coeff(451)=-0.890017961352D+03
coeff(452)=-0.123308947404D+04
coeff(453)=0.254669121177D+03
coeff(454)=0.485681682270D+04
coeff(455)=-0.569509725803D+03
coeff(456)=0.711128996725D+03
coeff(457)=0.930661054566D+03
coeff(458)=0.141812201479D+04
coeff(459)=-0.492867496407D+02
coeff(460)=-0.104363883497D+04
coeff(461)=0.273694374072D+04
coeff(462)=0.220304340529D+03
coeff(463)=0.183300412904D+03
coeff(464)=0.664334681884D+03
coeff(465)=0.353027487652D+03
coeff(466)=-0.267288987777D+03
coeff(467)=-0.482241049121D+03
coeff(468)=-0.110217883940D+03
coeff(469)=0.322264660023D+03
coeff(470)=-0.217399469820D+03
coeff(471)=0.242575232684D+02
coeff(472)=-0.109734034213D+04
coeff(473)=-0.398257155806D+03
coeff(474)=-0.574697619066D+03
coeff(475)=-0.729462388898D+03
coeff(476)=-0.736623294831D+03
coeff(477)=0.334236097737D+02
coeff(478)=-0.126937524889D+04
coeff(479)=-0.119193622826D+04
coeff(480)=-0.870017491909D+03
coeff(481)=-0.681954378432D+03
coeff(482)=0.106167566292D+04
coeff(483)=0.118520710382D+04
coeff(484)=-0.919777051739D+03
coeff(485)=0.101054492837D+04
coeff(486)=0.931885890355D+03
coeff(487)=0.199666062577D+03
coeff(488)=0.191948779147D+03
coeff(489)=0.440614337404D+03
coeff(490)=0.155589770254D+04
coeff(491)=0.488336383020D+03
coeff(492)=-0.476504103188D+03
coeff(493)=-0.696991352970D+03
coeff(494)=-0.106328689179D+04
coeff(495)=0.497922173936D+03
coeff(496)=-0.476666575521D+03
coeff(497)=-0.129787759381D+04
coeff(498)=0.571043074522D+03
coeff(499)=-0.334729399903D+03
coeff(500)=-0.223179165024D+03
coeff(501)=0.445892726440D+03
coeff(502)=0.744070320545D+03
coeff(503)=-0.561246657194D+03
coeff(504)=-0.911481598494D+03
coeff(505)=-0.215654857975D+03
coeff(506)=-0.196356529005D+04
coeff(507)=-0.886295470882D+03
coeff(508)=0.432484657908D+03
coeff(509)=0.118116483286D+04
coeff(510)=0.288121004311D+03
coeff(511)=-0.273300884384D+03
coeff(512)=-0.259969247987D+03
coeff(513)=-0.657470456907D+00
coeff(514)=-0.693063655201D+02
coeff(515)=0.137726966176D+04
coeff(516)=0.468654649420D+03
coeff(517)=-0.166913198066D+04
coeff(518)=-0.759182341524D+03
coeff(519)=0.202100237163D+02
coeff(520)=-0.113903757439D+04
coeff(521)=-0.710507800932D+03
coeff(522)=-0.202666148906D+04
coeff(523)=0.533055799192D+03
coeff(524)=-0.196039560712D+03
coeff(525)=-0.409029103950D+03
coeff(526)=-0.123649522596D+04
coeff(527)=-0.130243677819D+04
coeff(528)=-0.434542640413D+03
coeff(529)=0.274808896035D+03
coeff(530)=0.407656503257D+03
coeff(531)=-0.688392082085D+03
coeff(532)=-0.459875362577D+03
coeff(533)=0.432578162687D+03
coeff(534)=0.105931172699D+04
coeff(535)=0.155156224614D+04
coeff(536)=-0.221859508320D+03
coeff(537)=-0.141118210955D+04
coeff(538)=-0.853604283865D+03
coeff(539)=-0.124270468709D+04
coeff(540)=0.133070209273D+04
coeff(541)=-0.452590412670D+03
coeff(542)=-0.183509101733D+03
coeff(543)=0.806928303707D+03
coeff(544)=-0.247773330985D+04
coeff(545)=-0.477528152949D+03
coeff(546)=-0.551733973202D+03
coeff(547)=0.181617550170D+04
coeff(548)=-0.216829867454D+04
coeff(549)=0.678477870304D+03
coeff(550)=0.910554148127D+03
coeff(551)=0.564110892140D+03
coeff(552)=0.482630352812D+03
coeff(553)=-0.109430716115D+04
coeff(554)=-0.247453220057D+04
coeff(555)=0.182404973935D+04
coeff(556)=0.402958113233D+03
coeff(557)=0.122165175710D+04
coeff(558)=0.542396357702D+03
coeff(559)=-0.404555791448D+02
coeff(560)=-0.898418720803D+03
coeff(561)=-0.104026267941D+03
coeff(562)=0.191178910774D+04
coeff(563)=0.481413517018D+03
coeff(564)=-0.176500661637D+04
coeff(565)=-0.477412927288D+03
coeff(566)=-0.106917580421D+04
coeff(567)=-0.272967117890D+03
coeff(568)=-0.415654891599D+02
coeff(569)=0.323727531398D+03
coeff(570)=-0.104436979345D+04
coeff(571)=0.918317346619D+03
coeff(572)=-0.667033790216D+03
coeff(573)=-0.435970202404D+03
coeff(574)=-0.365538033567D+03
coeff(575)=0.741866656973D+02
coeff(576)=-0.213753280023D+04
coeff(577)=0.113644402142D+04
coeff(578)=-0.631013525921D+03
coeff(579)=-0.276658416152D+04
coeff(580)=-0.151784147768D+03
coeff(581)=0.486728796989D+03
coeff(582)=-0.310236882771D+03
coeff(583)=0.381767791178D+03
coeff(584)=-0.648471686890D+03
coeff(585)=0.685319873424D+03
coeff(586)=-0.152892098213D+04
coeff(587)=-0.987710749220D+03
coeff(588)=0.110412207467D+03
coeff(589)=-0.104120923245D+04
coeff(590)=0.601688981211D+03
coeff(591)=0.102907866531D+04
coeff(592)=0.268539591175D+03
coeff(593)=0.142527339436D+04
coeff(594)=0.572688649201D+03
coeff(595)=-0.836020173812D+03
coeff(596)=0.379828586053D+03
coeff(597)=0.373927219535D+03
coeff(598)=0.188017519963D+03
coeff(599)=0.545351816777D+03
coeff(600)=-0.146526773826D+04
coeff(601)=-0.812996330025D+03
coeff(602)=0.167358059918D+04
coeff(603)=0.127391808967D+04
coeff(604)=0.784290292303D+03
coeff(605)=-0.450889307660D+03
coeff(606)=-0.378586398769D+03
coeff(607)=0.152213100397D+04
coeff(608)=0.328446964726D+03
coeff(609)=-0.582238241183D+03
coeff(610)=-0.887236493377D+03
coeff(611)=0.431346099543D+03
coeff(612)=-0.224972197644D+03
coeff(613)=0.962309764252D+03
coeff(614)=-0.215408013477D+03
coeff(615)=-0.409144714174D+03
coeff(616)=-0.559821532716D+03
coeff(617)=-0.834111975926D+03
coeff(618)=0.687762150873D+03
coeff(619)=0.895125541943D+03
coeff(620)=-0.473542723079D+03
coeff(621)=0.839273770121D+02
coeff(622)=-0.130814952490D+02
coeff(623)=-0.462232124995D+03
coeff(624)=-0.178183063527D+02
coeff(625)=0.167686010390D+04
coeff(626)=0.708275026070D+03
coeff(627)=0.126898063922D+02
coeff(628)=0.587628117112D+02
coeff(629)=0.557430997762D+03
coeff(630)=-0.324110747332D+03
coeff(631)=0.166302415601D+03
coeff(632)=-0.105443223548D+04
coeff(633)=0.448554201091D+03
coeff(634)=-0.174447193055D+03
coeff(635)=-0.468055913492D+03
coeff(636)=-0.417988013706D+03
coeff(637)=0.846787981095D+03
coeff(638)=-0.500475884073D+03
coeff(639)=-0.436114643411D+03
coeff(640)=0.239415826614D+03
coeff(641)=0.908231766583D+03
coeff(642)=0.264613351005D+03
coeff(643)=0.288614833348D+03
coeff(644)=-0.210219607069D+03
coeff(645)=0.106580587714D+04
coeff(646)=0.526409114921D+03
coeff(647)=-0.736249843968D+03
coeff(648)=-0.339503480133D+02
coeff(649)=0.603429707038D+03
coeff(650)=0.187694149708D+04
coeff(651)=0.222377875231D+03
coeff(652)=0.196411778222D+04
coeff(653)=-0.728277545729D+03
coeff(654)=-0.668068870282D+02
coeff(655)=0.743759509460D+03
coeff(656)=0.170009589387D+03
coeff(657)=-0.310818251140D+03
coeff(658)=-0.100649360668D+04
coeff(659)=0.102643247869D+04
coeff(660)=-0.864078998130D+03
coeff(661)=0.114877393754D+04
coeff(662)=0.196738365345D+04
coeff(663)=-0.134501632890D+04
coeff(664)=-0.945872693227D+03
coeff(665)=0.870835003943D+03
coeff(666)=0.121064432358D+04
coeff(667)=0.541968625873D+03
coeff(668)=0.426737159319D+03
coeff(669)=-0.542892752873D+03
coeff(670)=-0.378199814788D+03
coeff(671)=-0.145496008416D+04
coeff(672)=0.134261933989D+04
coeff(673)=0.434064280967D+01
coeff(674)=0.670163862814D+03
coeff(675)=-0.146073263958D+04
coeff(676)=0.428079867303D+03
coeff(677)=0.820360259395D+03
coeff(678)=-0.130304812150D+03
coeff(679)=-0.141022391841D+04
coeff(680)=-0.214232126866D+04
coeff(681)=-0.648977147044D+03
coeff(682)=0.136332703641D+04
coeff(683)=-0.611149219657D+03
coeff(684)=0.114886551935D+04
coeff(685)=0.112448354779D+04
coeff(686)=0.644345401165D+03
coeff(687)=-0.112295175333D+04
coeff(688)=0.553130893649D+03
coeff(689)=-0.142538496464D+04
coeff(690)=-0.627398969291D+03
coeff(691)=0.291344570937D+04
coeff(692)=-0.101294963862D+04
coeff(693)=0.154649595301D+04
coeff(694)=-0.123781545726D+04
coeff(695)=0.744022296821D+03
coeff(696)=-0.426493526811D+03
coeff(697)=-0.103445729493D+04
coeff(698)=-0.126418582284D+04
coeff(699)=-0.260144118684D+04
coeff(700)=-0.378539491072D+03
coeff(701)=-0.863605731108D+03
coeff(702)=-0.341205593745D+02
coeff(703)=-0.988450757460D+03
coeff(704)=0.401754972535D+03
coeff(705)=-0.114611135973D+03
coeff(706)=0.894044562449D+02
coeff(707)=-0.184638575268D+03
coeff(708)=-0.171528081185D+04
coeff(709)=0.840898735986D+03
coeff(710)=0.212964728845D+03
coeff(711)=-0.668835959301D+03
coeff(712)=0.433808563325D+03
coeff(713)=-0.740048058179D+03
coeff(714)=0.111829357105D+04
coeff(715)=-0.845759237421D+03
coeff(716)=-0.269656040685D+03
coeff(717)=0.442905594231D+03
coeff(718)=0.107572603149D+04
coeff(719)=-0.204645607594D+03
coeff(720)=-0.150026658364D+03
coeff(721)=-0.479738416366D+03
coeff(722)=-0.274090404112D+03
coeff(723)=0.176186699440D+04
coeff(724)=-0.123779714692D+04
coeff(725)=-0.131561864553D+04
coeff(726)=-0.121536957722D+03
coeff(727)=0.300689377521D+03
coeff(728)=0.105636961740D+03
coeff(729)=-0.952261995960D+03
coeff(730)=-0.966399108389D+03
coeff(731)=0.172820582872D+03
coeff(732)=-0.304698537128D+03
coeff(733)=-0.750588809057D+02
coeff(734)=0.164182917388D+04
coeff(735)=0.109080442129D+03
coeff(736)=0.154651572753D+04
coeff(737)=0.274782124870D+03
coeff(738)=-0.691976219372D+03
coeff(739)=0.559085800680D+03
coeff(740)=-0.738793649917D+03
coeff(741)=0.292917139543D+03
coeff(742)=-0.704812518161D+03
coeff(743)=0.448091620052D+03
coeff(744)=0.970506326084D+03
coeff(745)=0.194909985978D+03
coeff(746)=0.344731938163D+03
coeff(747)=-0.891954733555D+03
coeff(748)=0.289533176313D+03
coeff(749)=0.295280117292D+04
coeff(750)=-0.154317195008D+04
coeff(751)=0.930647584777D+03
coeff(752)=0.616158921579D+03
coeff(753)=0.242901914479D+03
coeff(754)=0.109287602336D+04
coeff(755)=-0.117455456733D+04
coeff(756)=0.109479856879D+04
coeff(757)=0.435321565221D+02
coeff(758)=-0.765361169878D+03
coeff(759)=0.109550292652D+03
coeff(760)=0.516290781232D+03
coeff(761)=-0.986592856670D+03
coeff(762)=0.987111834618D+03
coeff(763)=0.407065058944D+03
coeff(764)=-0.929354962849D+03
coeff(765)=-0.150055100616D+03
coeff(766)=-0.193921317597D+03
coeff(767)=0.730790282071D+03
coeff(768)=0.185521562506D+03
coeff(769)=0.393793989874D+03
coeff(770)=0.113364330708D+04
coeff(771)=0.103436177101D+04
coeff(772)=-0.733166267615D+03
coeff(773)=-0.895624248230D+03
coeff(774)=-0.629343594325D+03
coeff(775)=-0.130433353431D+04
coeff(776)=0.189706657317D+04
coeff(777)=-0.744178065026D+01
coeff(778)=-0.112378085191D+04
coeff(779)=-0.262027558607D+04
coeff(780)=0.270152357694D+04
coeff(781)=-0.255961426306D+03
coeff(782)=0.535559236440D+03
coeff(783)=-0.110709583167D+04
coeff(784)=-0.727169903046D+03
coeff(785)=-0.542142984231D+03
coeff(786)=-0.300722465910D+04
coeff(787)=0.329659537836D+03
coeff(788)=0.306592425402D+04
coeff(789)=-0.181658378789D+04
coeff(790)=0.431101830140D+03
coeff(791)=0.184943014079D+04
coeff(792)=0.125325656321D+04
coeff(793)=-0.149159516141D+04
coeff(794)=-0.146226487596D+04
coeff(795)=-0.212958297191D+03
coeff(796)=-0.359525903700D+03
coeff(797)=-0.253306322066D+03
coeff(798)=0.376958206517D+03
coeff(799)=0.100826406175D+03
coeff(800)=-0.509354356069D+03
coeff(801)=0.413470114876D+03
coeff(802)=-0.147625039436D+04
coeff(803)=0.578218903654D+03
coeff(804)=0.123801548392D+04
coeff(805)=0.112017428241D+04
coeff(806)=-0.554416247172D+03
coeff(807)=0.125895805095D+03
coeff(808)=0.535252336290D+03
coeff(809)=0.194289420478D+04
coeff(810)=-0.430942336608D+03
coeff(811)=0.120361347074D+04
coeff(812)=-0.116989894998D+04
coeff(813)=0.419112280870D+03
coeff(814)=-0.383876588539D+03
coeff(815)=0.649702036500D+03
coeff(816)=-0.941825304474D+03
coeff(817)=-0.297407533918D+02
coeff(818)=0.396621719189D+03
coeff(819)=-0.989297931327D+03
coeff(820)=-0.410794247322D+03
coeff(821)=0.508105281671D+03
coeff(822)=0.353380429993D+03
coeff(823)=0.193378235932D+03
coeff(824)=-0.847791152991D+03
coeff(825)=-0.417126272731D+03
coeff(826)=0.831868573031D+03
coeff(827)=-0.834561615517D+03
coeff(828)=0.256644195574D+03
coeff(829)=-0.131377219611D+04
coeff(830)=0.922209289912D+03
coeff(831)=0.116868294384D+04
coeff(832)=-0.681251615742D+03
coeff(833)=0.300537101820D+03
coeff(834)=0.191472355117D+03
coeff(835)=-0.316254731343D+02
coeff(836)=-0.897474883408D+03
coeff(837)=-0.646668001780D+03
coeff(838)=0.135048083407D+03
coeff(839)=0.415265009386D+03
coeff(840)=0.147812024152D+04
coeff(841)=-0.850768388694D+03
coeff(842)=0.866536792776D+03
coeff(843)=-0.974047404355D+03
coeff(844)=-0.126920527447D+04
coeff(845)=0.510945270788D+03
coeff(846)=-0.836347745877D+03
coeff(847)=0.162153350741D+04
coeff(848)=-0.535856305100D+03
coeff(849)=-0.867597775544D+02
coeff(850)=0.493535335014D+03
coeff(851)=-0.914292514554D+03
coeff(852)=-0.552243299454D+03
coeff(853)=-0.669639264347D+03
coeff(854)=0.656054444885D+03
coeff(855)=0.459281503320D+03
coeff(856)=-0.441517232875D+03
coeff(857)=-0.533490984669D+02
coeff(858)=0.149978211478D+03
coeff(859)=0.283067645823D+03
coeff(860)=-0.577097962282D+03
coeff(861)=0.436733143571D+03
coeff(862)=-0.456568155511D+03
coeff(863)=-0.102325802422D+04
coeff(864)=-0.807124513642D+03
coeff(865)=-0.251889274997D+03
coeff(866)=0.104195725379D+04
coeff(867)=0.425583468559D+03
coeff(868)=-0.278010818210D+03
coeff(869)=-0.419557591074D+03
coeff(870)=-0.950101664245D+03
coeff(871)=0.596168662854D+03
coeff(872)=0.538761559833D+03
coeff(873)=0.285298124915D+03
coeff(874)=-0.921581859458D+03
coeff(875)=0.619750776384D+03
coeff(876)=0.233365897608D+03
coeff(877)=-0.349144526689D+03
coeff(878)=0.173874291384D+03
coeff(879)=-0.905139731843D+03
coeff(880)=0.984698261413D+03
coeff(881)=0.123544538472D+04
coeff(882)=0.588244664573D+03
coeff(883)=0.269810835856D+02
coeff(884)=0.589598491891D+03
coeff(885)=0.664628668244D+03
coeff(886)=0.111127362414D+04
coeff(887)=-0.427896717813D+03
coeff(888)=-0.509821824566D+03
coeff(889)=-0.588038480143D+03
coeff(890)=-0.895584625021D+03
coeff(891)=0.937991991686D+03
coeff(892)=0.456730737708D+03
coeff(893)=0.406042473672D+03
coeff(894)=0.268200847684D+01
coeff(895)=-0.105193135930D+04
coeff(896)=-0.462713768808D+03
coeff(897)=0.518553215758D+03
coeff(898)=-0.523878120002D+03
coeff(899)=0.741321358233D+01
coeff(900)=0.118545736520D+03
coeff(901)=0.313186593724D+03
coeff(902)=0.341306746876D+03
coeff(903)=0.301472657276D+03
coeff(904)=-0.509263340806D+03
coeff(905)=0.210617629652D+03
coeff(906)=-0.116757072865D+04
coeff(907)=0.844523242843D+03
coeff(908)=0.136887715141D+04
coeff(909)=0.151091819768D+04
coeff(910)=0.769035680854D+03
coeff(911)=-0.609231319106D+03
coeff(912)=0.428391326365D+03
coeff(913)=0.108495754402D+04
coeff(914)=-0.992397172542D+01
coeff(915)=0.320532413721D+03
coeff(916)=-0.965252707472D+03
coeff(917)=-0.133460325470D+04
coeff(918)=0.115332681735D+04
coeff(919)=0.798677044685D+03
coeff(920)=0.110563483324D+04
coeff(921)=-0.350999716219D+03
coeff(922)=-0.272095808884D+03
coeff(923)=-0.787216187265D+03
coeff(924)=0.888971923311D+03
coeff(925)=0.141212883279D+04
coeff(926)=-0.141307057590D+04
coeff(927)=0.170771867825D+04
coeff(928)=0.313898175314D+03
coeff(929)=0.783562797437D+03
coeff(930)=-0.230233505270D+03
coeff(931)=0.263757825840D+03
coeff(932)=0.978477854417D+03
coeff(933)=0.483935175169D+03
coeff(934)=-0.226052178385D+03
coeff(935)=0.124649523057D+03
coeff(936)=0.614714008960D+03
coeff(937)=-0.396288191176D+03
coeff(938)=0.288683355363D+03
coeff(939)=0.592023663627D+03
coeff(940)=0.646648053027D+03
coeff(941)=0.750286657756D+03
coeff(942)=0.862602468382D+03
coeff(943)=0.288465014161D+03
coeff(944)=-0.114823695008D+03
coeff(945)=-0.166530771141D+04
coeff(946)=-0.128628503740D+04
coeff(947)=-0.631364517004D+03
coeff(948)=-0.481200474454D+03
coeff(949)=-0.746334216306D+03
coeff(950)=-0.312561013916D+03
coeff(951)=-0.146331241713D+04
coeff(952)=-0.219175753853D+02
coeff(953)=0.838621498478D+03
coeff(954)=0.544228907647D+03
coeff(955)=-0.333445099150D+03
coeff(956)=-0.105703402694D+04
coeff(957)=-0.695161929374D+03
coeff(958)=-0.240420733159D+02
coeff(959)=-0.174614042031D+03
coeff(960)=-0.105312649922D+03
coeff(961)=0.726268386366D+02
coeff(962)=-0.209751596255D+04
coeff(963)=0.162145609797D+04
coeff(964)=0.105957094267D+03
coeff(965)=0.106712202802D+04
coeff(966)=-0.631681863635D+03
coeff(967)=0.244027872165D+03
coeff(968)=-0.182569961830D+04
coeff(969)=-0.144021304825D+03
coeff(970)=-0.153325159420D+03
coeff(971)=0.915108033500D+03
coeff(972)=0.163592541694D+04
coeff(973)=0.547237873098D+03
coeff(974)=-0.107889158241D+04
coeff(975)=-0.190874349056D+03
coeff(976)=-0.943154000711D+03
coeff(977)=0.108986084341D+04
coeff(978)=-0.909283426483D+03
coeff(979)=0.275805966187D+03
coeff(980)=-0.606560009780D+02
coeff(981)=-0.259437627884D+03
coeff(982)=0.126843746440D+04
coeff(983)=-0.150937073318D+04
coeff(984)=0.114970709985D+04
coeff(985)=-0.567841275548D+03
coeff(986)=0.283610498527D+03
coeff(987)=0.712600606168D+03
coeff(988)=-0.267241782967D+03
coeff(989)=-0.568876307555D+02
coeff(990)=-0.133841484146D+04
coeff(991)=0.779739870483D+03
coeff(992)=0.365330487117D+03
coeff(993)=0.644485000970D+03
coeff(994)=-0.907971924625D+03
coeff(995)=0.283845057404D+03
coeff(996)=0.379116282709D+03
coeff(997)=0.968717451648D+03
coeff(998)=-0.564669735966D+03
coeff(999)=-0.438316231280D+03
coeff(1000)=-0.441003623068D+03
coeff(1001)=0.104346932162D+04
coeff(1002)=0.242710040410D+03
coeff(1003)=-0.104245635726D+04
coeff(1004)=0.117372700406D+04
coeff(1005)=-0.127126896861D+04
coeff(1006)=0.398674536588D+03
coeff(1007)=0.317116052587D+03
coeff(1008)=0.290513286965D+03
coeff(1009)=-0.135199446280D+04
coeff(1010)=0.717047179818D+02
coeff(1011)=-0.531200038958D+03
coeff(1012)=0.142433608264D+04
coeff(1013)=0.181286590005D+04
coeff(1014)=0.230541931196D+03
coeff(1015)=-0.105814438252D+04
coeff(1016)=-0.892070269124D+03
coeff(1017)=-0.940084485187D+02
coeff(1018)=0.110887189048D+03
coeff(1019)=0.209181254458D+03
coeff(1020)=-0.117439367693D+04
coeff(1021)=-0.908979742124D+03
coeff(1022)=0.151173939206D+04
coeff(1023)=-0.151317328772D+04
coeff(1024)=0.270384790163D+03
coeff(1025)=-0.113452961665D+04
coeff(1026)=0.124590581641D+04
coeff(1027)=0.207842337795D+03
coeff(1028)=0.246388161822D+02
coeff(1029)=-0.410661157288D+03
coeff(1030)=-0.118419605886D+02
coeff(1031)=0.689300649881D+03
coeff(1032)=0.575483274122D+03
coeff(1033)=-0.126388671499D+03
coeff(1034)=0.339145717082D+03
coeff(1035)=-0.438558634051D+03
coeff(1036)=0.124210493012D+04
coeff(1037)=0.541024017371D+02
coeff(1038)=-0.644489381247D+03
coeff(1039)=0.131581867264D+04
coeff(1040)=0.424746394752D+03
coeff(1041)=-0.111942931285D+04
coeff(1042)=-0.546812128074D+03
coeff(1043)=0.801962980907D+03
coeff(1044)=-0.759658903832D+03
coeff(1045)=-0.664527658943D+03
coeff(1046)=0.178952253219D+04
coeff(1047)=0.286838615321D+03
coeff(1048)=0.780361277773D+02
coeff(1049)=-0.154958191573D+04
coeff(1050)=-0.501284912702D+03
coeff(1051)=0.559405974860D+03
coeff(1052)=0.121692752118D+03
coeff(1053)=-0.283331576202D+03
coeff(1054)=0.325632871445D+03
coeff(1055)=-0.444219038648D+03
coeff(1056)=-0.125105143005D+02
coeff(1057)=-0.847470734714D+03
coeff(1058)=0.325438027632D+03
coeff(1059)=-0.473193223727D+03
coeff(1060)=-0.103719158409D+04
coeff(1061)=0.337821948790D+03
coeff(1062)=0.807314292183D+03
coeff(1063)=-0.715443226262D+03
coeff(1064)=0.129158332028D+04
coeff(1065)=0.175222198573D+03
coeff(1066)=0.350106317093D+03
coeff(1067)=-0.294734374839D+03
coeff(1068)=0.302847495379D+03
coeff(1069)=-0.349660543765D+03
coeff(1070)=0.433820572813D+03
coeff(1071)=0.445387557326D+03
coeff(1072)=-0.803645112521D+03
coeff(1073)=0.394408809956D+03
coeff(1074)=0.185651190567D+03
coeff(1075)=-0.177178524836D+04
coeff(1076)=0.412473610315D+03
coeff(1077)=-0.904887210113D+02
coeff(1078)=-0.845677886274D+03
coeff(1079)=0.810108811542D+03
coeff(1080)=-0.117257739071D+04
coeff(1081)=-0.134381550699D+04
coeff(1082)=-0.536600287806D+02
coeff(1083)=0.524784614299D+03
coeff(1084)=0.485832262128D+03
coeff(1085)=-0.488484583757D+03
coeff(1086)=-0.535956693347D+03
coeff(1087)=-0.447454648568D+03
coeff(1088)=-0.208988832778D+03
coeff(1089)=-0.119205806902D+04
coeff(1090)=0.299071382229D+03
coeff(1091)=-0.826175014462D+02
coeff(1092)=0.704611174873D+03
coeff(1093)=-0.168745994395D+03
coeff(1094)=0.726146481992D+02
coeff(1095)=-0.937229777391D+03
coeff(1096)=0.187608490239D+03
coeff(1097)=0.628854160940D+03
coeff(1098)=-0.996549696591D+03
coeff(1099)=-0.769460200506D+03
coeff(1100)=-0.387591332540D+03
coeff(1101)=0.144559985045D+04
coeff(1102)=-0.962130351985D+03
coeff(1103)=0.147885370940D+04
coeff(1104)=0.542123762827D+03
coeff(1105)=0.136512849210D+04
coeff(1106)=-0.127121185597D+04
coeff(1107)=0.951619656941D+03
coeff(1108)=-0.371643607022D+03
coeff(1109)=-0.456641561680D+03
coeff(1110)=-0.121468619499D+04
coeff(1111)=-0.300755262823D+03
coeff(1112)=-0.132172061196D+04
coeff(1113)=0.289032071780D+03
coeff(1114)=0.409659948437D+03
coeff(1115)=-0.131533943633D+04
coeff(1116)=-0.114890707449D+04
coeff(1117)=-0.483607821818D+03
coeff(1118)=-0.969074521943D+03
coeff(1119)=-0.490316124651D+03
coeff(1120)=-0.278424768500D+02
coeff(1121)=-0.441796229452D+03
coeff(1122)=0.963001362157D+03
coeff(1123)=0.372714116713D+01
coeff(1124)=0.558427058664D+03
coeff(1125)=-0.200309700913D+03
coeff(1126)=0.320466607449D+03
coeff(1127)=-0.796530331730D+03
coeff(1128)=-0.889629179493D+03
coeff(1129)=0.844592404607D+03
coeff(1130)=-0.586157398311D+03
coeff(1131)=-0.633823333257D+03
coeff(1132)=0.925710277074D+03
coeff(1133)=-0.231464596247D+03
coeff(1134)=-0.160575737949D+04
coeff(1135)=0.828672727513D+03
coeff(1136)=0.126360360184D+04
coeff(1137)=0.124676491569D+04
coeff(1138)=0.256946706711D+03
coeff(1139)=-0.742766733518D+03
coeff(1140)=-0.177149572871D+03
coeff(1141)=-0.627985310668D+03
coeff(1142)=0.335384927885D+03
coeff(1143)=-0.314165256213D+03
coeff(1144)=-0.155670589465D+04
coeff(1145)=-0.653887151793D+02
coeff(1146)=-0.501360088256D+03
coeff(1147)=0.441606424868D+03
coeff(1148)=-0.374882165177D+03
coeff(1149)=0.114083440242D+04
coeff(1150)=0.102037535577D+04
coeff(1151)=0.142390086141D+04
coeff(1152)=-0.709152731754D+03
coeff(1153)=0.168050750114D+04
coeff(1154)=-0.105928043189D+03
coeff(1155)=-0.996800114645D+02
coeff(1156)=-0.127459424296D+04
coeff(1157)=0.146559608982D+04
coeff(1158)=-0.180938100587D+04
coeff(1159)=0.597186255202D+03
coeff(1160)=-0.227020303104D+03
coeff(1161)=-0.701337143255D+03
coeff(1162)=0.117995683968D+04
coeff(1163)=0.893153497507D+03
coeff(1164)=-0.291529227035D+03
coeff(1165)=0.678616699636D+03
coeff(1166)=0.380228973581D+02
coeff(1167)=-0.145148604850D+03
coeff(1168)=0.815232464044D+02
coeff(1169)=-0.148195594398D+04
coeff(1170)=0.416662065914D+03
coeff(1171)=0.435571572134D+03
coeff(1172)=0.880405755598D+03
coeff(1173)=-0.507439168033D+03
coeff(1174)=0.463036208145D+03
coeff(1175)=0.110218263571D+03
coeff(1176)=-0.781111130955D+03
coeff(1177)=-0.152657829837D+03
coeff(1178)=0.106839181130D+04
coeff(1179)=0.163844137363D+04
coeff(1180)=0.135167796165D+04
coeff(1181)=0.758970760256D+03
coeff(1182)=0.619902088128D+03
coeff(1183)=0.471877252647D+03
coeff(1184)=0.449786906441D+03
coeff(1185)=-0.133512222720D+03
coeff(1186)=0.311931994116D+03
coeff(1187)=-0.105493416415D+04
coeff(1188)=0.103606954190D+04
coeff(1189)=-0.113000780247D+04
coeff(1190)=-0.375958582914D+03
coeff(1191)=0.536817987879D+03
coeff(1192)=0.150607505009D+04
coeff(1193)=-0.914627203058D+03
coeff(1194)=0.416025771015D+02
coeff(1195)=0.639094667177D+02
coeff(1196)=-0.584212183834D+02
coeff(1197)=-0.282348206881D+03
coeff(1198)=0.606235547326D+02
coeff(1199)=-0.551425162549D+03
coeff(1200)=0.541762739601D+03
coeff(1201)=0.325471388543D+02
coeff(1202)=-0.263213114026D+03
coeff(1203)=-0.633153770283D+03
coeff(1204)=0.387692613807D+03
coeff(1205)=0.501913327067D+03
coeff(1206)=-0.378536513970D+01
coeff(1207)=-0.667770587051D+02
coeff(1208)=-0.244261122061D+02
coeff(1209)=0.803909394313D+02
coeff(1210)=0.745867465159D+03
coeff(1211)=-0.845307917248D+03
coeff(1212)=0.852775965474D+03
coeff(1213)=-0.135620800399D+04
coeff(1214)=-0.535358098165D+02
coeff(1215)=0.766486281763D+03
coeff(1216)=-0.784622172043D+03
coeff(1217)=0.233665366596D+03
coeff(1218)=-0.657701455618D+02
coeff(1219)=-0.217221749827D+03
coeff(1220)=0.129436038672D+03
coeff(1221)=-0.293177233847D+03
coeff(1222)=0.195416693238D+03
coeff(1223)=0.365715914566D+02
coeff(1224)=-0.923172236915D+03
coeff(1225)=0.617395036136D+03
coeff(1226)=-0.119746249449D+04
coeff(1227)=-0.186009536405D+03
coeff(1228)=0.389860247659D+03
coeff(1229)=-0.619628810350D+03
coeff(1230)=-0.118423070495D+03
coeff(1231)=0.699562468520D+03
coeff(1232)=-0.312325217530D+03
coeff(1233)=0.349285941651D+03
coeff(1234)=-0.442442176062D+02
coeff(1235)=-0.553886207381D+02
coeff(1236)=0.575169687895D+03
coeff(1237)=0.305173949266D+03
coeff(1238)=0.722502112196D+03
coeff(1239)=0.917323942304D+02
coeff(1240)=-0.186387786804D+04
coeff(1241)=-0.141887166381D+03
coeff(1242)=0.139514722556D+04
coeff(1243)=0.871767009671D+03
coeff(1244)=-0.205526936836D+04
coeff(1245)=-0.573516056701D+03
coeff(1246)=-0.426932340609D+03
coeff(1247)=-0.578242229134D+02
coeff(1248)=-0.321381543341D+03
coeff(1249)=-0.720931460948D+03
coeff(1250)=-0.289711189197D+02
coeff(1251)=0.113669858936D+04
coeff(1252)=0.723705696869D+02
coeff(1253)=0.356817667452D+03
coeff(1254)=-0.641658374459D+03
coeff(1255)=-0.460350140182D+03
coeff(1256)=0.493760009139D+03
coeff(1257)=0.546456358422D+03
coeff(1258)=0.165422563510D+03
coeff(1259)=0.137588500168D+04
coeff(1260)=0.131809130624D+04
coeff(1261)=-0.928585535879D+03
coeff(1262)=0.303851764638D+03
coeff(1263)=0.181682484961D+02
coeff(1264)=-0.155382345206D+04
coeff(1265)=0.142866845562D+04
coeff(1266)=-0.141687897913D+04
coeff(1267)=-0.550195297286D+03
coeff(1268)=0.240699158743D+03
coeff(1269)=-0.113353921503D+03
coeff(1270)=0.686216585031D+03
coeff(1271)=-0.500241061804D+03
coeff(1272)=0.724222184257D+02
coeff(1273)=-0.628372397936D+03
coeff(1274)=-0.416807260988D+03
coeff(1275)=-0.853832216207D+03
coeff(1276)=0.424448323442D+03
coeff(1277)=0.579508347763D+02
coeff(1278)=0.595664081595D+03
coeff(1279)=0.593123002258D+03
coeff(1280)=-0.602844904630D+02
coeff(1281)=-0.104733576711D+04
coeff(1282)=-0.578334529939D+03
coeff(1283)=0.244615208757D+03
coeff(1284)=0.821744176743D+03
coeff(1285)=0.348023130542D+03
coeff(1286)=0.864866481770D+03
coeff(1287)=-0.577697850745D+03
coeff(1288)=-0.182769814570D+04
coeff(1289)=0.507666885877D+03
coeff(1290)=-0.151104842348D+03
coeff(1291)=-0.108945986646D+04
coeff(1292)=-0.421915481270D+03
coeff(1293)=0.529996658902D+03
coeff(1294)=0.780772142695D+03
coeff(1295)=-0.675774310053D+02
coeff(1296)=-0.153225343325D+03
coeff(1297)=0.140252305046D+03
coeff(1298)=0.116561264677D+03
coeff(1299)=-0.932888244019D+02
coeff(1300)=-0.547157825248D+01
  return
end if

entry u12_dc_10(xyz,v,s)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)

do i=1,3
do j=1,3
rrel(i,j)=xyz(i,j)-xyz(4,j)
enddo
enddo

!compute r2xr3
cross(1)=rrel(2,2)*rrel(3,3)-rrel(2,3)*rrel(3,2)
cross(2)=-rrel(2,1)*rrel(3,3)+rrel(2,3)*rrel(3,1)
cross(3)=rrel(2,1)*rrel(3,2)-rrel(2,2)*rrel(3,1)
!compute p*r1*(r2xr3)
volume=0.00000
do i=1,3
volume=volume+rrel(1,i)*cross(i)
enddo
volume=tanh(15.4*volume)

call ev_pol_con_10(x,q)
v=dot_product(coeff,q)*volume/627.509474
s=dot_product(coeff,q)/627.509474

end  subroutine

!===========================================================================
! *def u13_order12
! subtoutine of computing u13
!===========================================================================
subroutine u13_order12
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp) :: xyz(4,3)
real(dp) :: v, r12, r13, r14, r23, r24, r34
real(dp) :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=3075
real(dp) :: coeff(ncoeff), q(ncoeff)
real(dp) :: x(6)
integer ::  ncall
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2

if(ncall.eq.1) then
a1=1.2561573
a2=1.1028022
b1=1.75
b2=1.61
ra1=1.196078
ra2=0.9333333
rb1=1.196078
rb2=0.9333333
coeff(1)=-0.107805854956D+03
coeff(2)=0.556492382441D+04
coeff(3)=0.193434802916D+03
coeff(4)=-0.100413112592D+03
coeff(5)=-0.962913723109D+03
coeff(6)=0.258159989406D+03
coeff(7)=-0.594309510988D+02
coeff(8)=0.520242005173D+04
coeff(9)=-0.325756660001D+04
coeff(10)=0.155028183157D+04
coeff(11)=-0.260255253474D+04
coeff(12)=-0.769852905947D+04
coeff(13)=-0.422487957664D+04
coeff(14)=0.996340850356D+03
coeff(15)=-0.201351147418D+03
coeff(16)=-0.440848464034D+03
coeff(17)=0.378709216760D+04
coeff(18)=-0.201015182836D+04
coeff(19)=0.413325479981D+04
coeff(20)=0.977420605649D+03
coeff(21)=0.261126119687D+02
coeff(22)=0.424137062336D+04
coeff(23)=-0.130257967411D+04
coeff(24)=0.362748811845D+04
coeff(25)=0.910702300766D+04
coeff(26)=-0.853818568342D+04
coeff(27)=-0.402964477375D+04
coeff(28)=0.435320240770D+03
coeff(29)=0.146757593582D+04
coeff(30)=-0.261012671677D+04
coeff(31)=0.331796204693D+04
coeff(32)=-0.683541059650D+04
coeff(33)=0.174397680298D+04
coeff(34)=-0.146831465382D+04
coeff(35)=-0.531866907147D+04
coeff(36)=-0.315352428484D+04
coeff(37)=0.293780615249D+04
coeff(38)=0.543009742828D+04
coeff(39)=-0.109281730359D+04
coeff(40)=0.680089026261D+04
coeff(41)=0.586134744782D+04
coeff(42)=-0.192649448148D+04
coeff(43)=-0.196009392878D+04
coeff(44)=-0.221733872858D+04
coeff(45)=0.261076354681D+04
coeff(46)=0.350958320486D+04
coeff(47)=-0.725492063236D+04
coeff(48)=0.437475668292D+04
coeff(49)=-0.575222465416D+04
coeff(50)=0.523544545563D+04
coeff(51)=-0.677504733647D+04
coeff(52)=-0.595479172762D+04
coeff(53)=-0.978467077710D+04
coeff(54)=0.367036537086D+04
coeff(55)=-0.106628088225D+04
coeff(56)=-0.169786011672D+04
coeff(57)=0.731840470613D+04
coeff(58)=-0.217881043512D+04
coeff(59)=0.249813535804D+04
coeff(60)=-0.121358158841D+05
coeff(61)=0.587315627322D+03
coeff(62)=-0.112588193422D+05
coeff(63)=0.366443741831D+04
coeff(64)=-0.174656345670D+04
coeff(65)=-0.489330596378D+04
coeff(66)=-0.744617023483D+04
coeff(67)=0.260258849965D+04
coeff(68)=0.766626640423D+04
coeff(69)=-0.715425120454D+04
coeff(70)=0.178477512694D+04
coeff(71)=-0.523539693397D+04
coeff(72)=0.695288613120D+04
coeff(73)=-0.413313261058D+04
coeff(74)=0.768778023273D+04
coeff(75)=-0.648298681147D+04
coeff(76)=-0.971855162615D+03
coeff(77)=-0.261786514289D+04
coeff(78)=0.766713806339D+04
coeff(79)=-0.496827607375D+04
coeff(80)=0.478604062448D+04
coeff(81)=0.538645545516D+04
coeff(82)=-0.817662901213D+04
coeff(83)=0.568475291513D+04
coeff(84)=0.975304501171D+04
coeff(85)=-0.287070485044D+04
coeff(86)=0.195871548191D+04
coeff(87)=-0.282356694067D+04
coeff(88)=0.501334661052D+04
coeff(89)=0.103320853448D+05
coeff(90)=-0.613378598659D+04
coeff(91)=0.439531914634D+04
coeff(92)=-0.356568484445D+04
coeff(93)=-0.133234136221D+04
coeff(94)=0.201388847908D+04
coeff(95)=0.991982988395D+03
coeff(96)=-0.292683504915D+04
coeff(97)=0.333046176871D+04
coeff(98)=0.468142884976D+04
coeff(99)=0.296566086861D+04
coeff(100)=-0.913038723143D+04
coeff(101)=0.103228824928D+04
coeff(102)=-0.331738823198D+04
coeff(103)=0.582301539087D+03
coeff(104)=0.576779172857D+03
coeff(105)=0.517189552576D+04
coeff(106)=0.721242624197D+02
coeff(107)=0.276821240742D+04
coeff(108)=-0.485599635311D+04
coeff(109)=-0.124864038024D+03
coeff(110)=0.117483397565D+04
coeff(111)=0.249086866111D+03
coeff(112)=-0.316445055337D+03
coeff(113)=-0.299283762368D+04
coeff(114)=0.223301711435D+04
coeff(115)=0.131389743485D+04
coeff(116)=-0.382916965685D+04
coeff(117)=0.550752651717D+04
coeff(118)=-0.659518085813D+04
coeff(119)=0.472053660298D+04
coeff(120)=-0.780079561157D+04
coeff(121)=0.184901878700D+03
coeff(122)=0.264061490802D+04
coeff(123)=-0.129771772344D+04
coeff(124)=0.105086475123D+05
coeff(125)=0.132440226246D+03
coeff(126)=-0.330626294776D+04
coeff(127)=0.677809914060D+04
coeff(128)=-0.396533195638D+04
coeff(129)=-0.547422907894D+04
coeff(130)=-0.166675842740D+04
coeff(131)=0.100087439390D+04
coeff(132)=-0.167226435639D+05
coeff(133)=-0.344019824375D+04
coeff(134)=-0.290157920744D+03
coeff(135)=0.100531700656D+04
coeff(136)=-0.521312336969D+04
coeff(137)=0.779485039709D+04
coeff(138)=-0.272535477840D+04
coeff(139)=0.821518252484D+04
coeff(140)=0.125933647946D+05
coeff(141)=-0.563502583800D+03
coeff(142)=-0.546720959422D+03
coeff(143)=-0.119038392140D+05
coeff(144)=0.112696215519D+05
coeff(145)=-0.144432012129D+05
coeff(146)=0.839533759209D+03
coeff(147)=0.258853456422D+02
coeff(148)=0.136945760667D+05
coeff(149)=0.541250034911D+04
coeff(150)=-0.562836964249D+04
coeff(151)=-0.321945570780D+04
coeff(152)=0.180738230713D+04
coeff(153)=-0.514050452496D+04
coeff(154)=0.107969569798D+05
coeff(155)=-0.210734409400D+04
coeff(156)=0.791755929595D+03
coeff(157)=0.166708780882D+04
coeff(158)=0.313890348080D+02
coeff(159)=-0.496716993434D+04
coeff(160)=0.216253779217D+04
coeff(161)=0.325394759772D+04
coeff(162)=-0.102853700128D+05
coeff(163)=0.348283350173D+04
coeff(164)=-0.379526359077D+04
coeff(165)=0.448745720285D+04
coeff(166)=-0.392602418969D+04
coeff(167)=-0.929493835355D+04
coeff(168)=0.225866744943D+04
coeff(169)=-0.954364170265D+03
coeff(170)=-0.575931156204D+04
coeff(171)=0.841350791990D+03
coeff(172)=-0.660858556247D+04
coeff(173)=0.269801897969D+04
coeff(174)=-0.131429167515D+04
coeff(175)=0.435242013662D+04
coeff(176)=0.275914570173D+04
coeff(177)=-0.923975646121D+01
coeff(178)=0.146399645948D+05
coeff(179)=-0.524220145689D+04
coeff(180)=-0.264457809985D+04
coeff(181)=0.699675921073D+04
coeff(182)=0.337148753180D+04
coeff(183)=-0.105292055752D+05
coeff(184)=0.161021708980D+04
coeff(185)=0.831309307447D+03
coeff(186)=0.701965349716D+04
coeff(187)=0.295372599711D+04
coeff(188)=0.124195215561D+05
coeff(189)=-0.515151743495D+04
coeff(190)=-0.289975739423D+04
coeff(191)=-0.710478933714D+04
coeff(192)=-0.797019624127D+02
coeff(193)=-0.414712873298D+04
coeff(194)=-0.820076294612D+04
coeff(195)=0.399666969667D+04
coeff(196)=-0.146222798800D+04
coeff(197)=-0.340348128991D+04
coeff(198)=-0.965861882237D+02
coeff(199)=-0.311666311991D+04
coeff(200)=0.323083089884D+04
coeff(201)=-0.946283217606D+03
coeff(202)=-0.348855312076D+04
coeff(203)=0.228631194179D+04
coeff(204)=-0.449106214490D+03
coeff(205)=-0.672498152636D+02
coeff(206)=0.253664423823D+04
coeff(207)=-0.374873654012D+04
coeff(208)=-0.397050214251D+03
coeff(209)=-0.766757897677D+04
coeff(210)=0.405457142830D+03
coeff(211)=0.203412222135D+04
coeff(212)=0.118293756545D+04
coeff(213)=-0.312493852811D+04
coeff(214)=-0.476936785652D+04
coeff(215)=-0.202922841783D+04
coeff(216)=0.243379999089D+04
coeff(217)=0.352151429125D+04
coeff(218)=0.149933738827D+04
coeff(219)=-0.172113294992D+04
coeff(220)=-0.437766778899D+04
coeff(221)=-0.237423991738D+04
coeff(222)=0.254413392378D+04
coeff(223)=-0.359665480377D+03
coeff(224)=-0.666707777676D+04
coeff(225)=-0.325575259134D+04
coeff(226)=0.644735567268D+04
coeff(227)=0.257998835105D+04
coeff(228)=0.164090389148D+04
coeff(229)=-0.555772777942D+03
coeff(230)=-0.308398832034D+04
coeff(231)=-0.165965884572D+04
coeff(232)=0.981963769796D+04
coeff(233)=0.794025475702D+04
coeff(234)=-0.701902520620D+04
coeff(235)=0.351894951252D+03
coeff(236)=-0.563868506925D+04
coeff(237)=0.427832149093D+04
coeff(238)=0.261366412479D+04
coeff(239)=0.309089987127D+04
coeff(240)=0.286323876161D+04
coeff(241)=-0.235481870178D+04
coeff(242)=0.379177554378D+04
coeff(243)=0.478908110983D+04
coeff(244)=-0.210026897121D+04
coeff(245)=-0.132597157745D+04
coeff(246)=0.179569895337D+04
coeff(247)=0.318070051192D+04
coeff(248)=0.960507416584D+03
coeff(249)=0.505244563123D+04
coeff(250)=-0.292026768029D+04
coeff(251)=0.813322320306D+03
coeff(252)=0.113311573124D+04
coeff(253)=-0.445276250691D+04
coeff(254)=-0.674745426631D+04
coeff(255)=0.372414183187D+04
coeff(256)=-0.186437831344D+04
coeff(257)=-0.177550110678D+04
coeff(258)=-0.122210466174D+02
coeff(259)=-0.984844784985D+04
coeff(260)=-0.616804558993D+04
coeff(261)=0.695621482343D+04
coeff(262)=0.576544790814D+04
coeff(263)=0.645405082836D+04
coeff(264)=0.100733478455D+04
coeff(265)=0.143829905745D+04
coeff(266)=0.348235974021D+04
coeff(267)=-0.117882300464D+05
coeff(268)=-0.102152222171D+04
coeff(269)=0.102475729190D+05
coeff(270)=0.882153670199D+04
coeff(271)=-0.351910199592D+04
coeff(272)=-0.506938301030D+04
coeff(273)=0.535548505717D+04
coeff(274)=0.511995029741D+04
coeff(275)=0.471341436755D+04
coeff(276)=0.770532761675D+04
coeff(277)=0.238028739459D+04
coeff(278)=0.100768265568D+05
coeff(279)=0.198123464986D+04
coeff(280)=-0.130303589030D+05
coeff(281)=-0.439389428901D+04
coeff(282)=0.583340712936D+04
coeff(283)=0.530462094504D+04
coeff(284)=0.591193033623D+04
coeff(285)=-0.664089718831D+02
coeff(286)=-0.754163070070D+03
coeff(287)=0.333210856124D+04
coeff(288)=0.896174144519D+04
coeff(289)=0.115564069815D+05
coeff(290)=-0.960649263013D+03
coeff(291)=-0.591770325940D+04
coeff(292)=-0.694604725701D+04
coeff(293)=0.121627742272D+05
coeff(294)=-0.916591763387D+04
coeff(295)=0.599696004709D+03
coeff(296)=0.227961221925D+03
coeff(297)=-0.292648197440D+04
coeff(298)=0.469027185398D+04
coeff(299)=-0.863043022874D+04
coeff(300)=-0.579821316090D+04
coeff(301)=0.467595931517D+04
coeff(302)=0.386823495530D+04
coeff(303)=-0.452421697838D+04
coeff(304)=-0.711305318831D+03
coeff(305)=-0.450977699982D+04
coeff(306)=0.348599483089D+04
coeff(307)=-0.181466327006D+04
coeff(308)=0.324741922533D+04
coeff(309)=0.288574515362D+04
coeff(310)=-0.213282353252D+04
coeff(311)=-0.271454163222D+03
coeff(312)=0.241509371352D+04
coeff(313)=-0.129138476648D+04
coeff(314)=-0.327397001064D+04
coeff(315)=0.531138261779D+04
coeff(316)=-0.790705447704D+03
coeff(317)=0.215900010953D+04
coeff(318)=0.570444159462D+04
coeff(319)=-0.414041509505D+04
coeff(320)=-0.114860777723D+04
coeff(321)=0.690979866908D+04
coeff(322)=-0.203431603414D+04
coeff(323)=-0.123271047050D+05
coeff(324)=-0.801806086687D+03
coeff(325)=0.489991455472D+03
coeff(326)=-0.351812235040D+04
coeff(327)=-0.163488568805D+04
coeff(328)=-0.910142455268D+04
coeff(329)=-0.129455680708D+04
coeff(330)=0.385328367788D+04
coeff(331)=0.765097579182D+04
coeff(332)=0.318703046143D+04
coeff(333)=-0.159691866552D+04
coeff(334)=-0.342318706833D+04
coeff(335)=-0.999156540271D+04
coeff(336)=-0.522582186078D+04
coeff(337)=-0.199055986189D+04
coeff(338)=0.121903589022D+05
coeff(339)=0.457912364223D+03
coeff(340)=0.137753195774D+05
coeff(341)=-0.700796775791D+04
coeff(342)=-0.701841366467D+04
coeff(343)=0.737949598184D+04
coeff(344)=-0.374647692312D+04
coeff(345)=-0.180334840352D+04
coeff(346)=0.360256820856D+04
coeff(347)=-0.783058169425D+04
coeff(348)=0.549773392774D+04
coeff(349)=-0.972418203440D+04
coeff(350)=-0.879545397553D+04
coeff(351)=-0.338374445537D+04
coeff(352)=0.229807703766D+04
coeff(353)=0.129959425011D+04
coeff(354)=-0.292215696401D+04
coeff(355)=-0.786165993311D+04
coeff(356)=0.115130446144D+04
coeff(357)=-0.116271331660D+05
coeff(358)=0.507452785639D+04
coeff(359)=0.127791748619D+04
coeff(360)=0.113702500233D+04
coeff(361)=-0.121989538830D+04
coeff(362)=-0.409418849432D+04
coeff(363)=0.968009147926D+04
coeff(364)=-0.122620800929D+05
coeff(365)=-0.903257026874D+03
coeff(366)=0.771901317997D+03
coeff(367)=0.574072893465D+03
coeff(368)=-0.139571855118D+05
coeff(369)=-0.344066641805D+04
coeff(370)=0.149182817186D+04
coeff(371)=-0.474614973867D+04
coeff(372)=-0.440039636746D+04
coeff(373)=-0.203646165285D+04
coeff(374)=0.747973620420D+03
coeff(375)=-0.327331501414D+04
coeff(376)=-0.209504004354D+04
coeff(377)=0.864480648907D+04
coeff(378)=-0.160472325012D+04
coeff(379)=0.105222826678D+04
coeff(380)=0.324813977483D+04
coeff(381)=0.464161870216D+04
coeff(382)=0.192848890211D+04
coeff(383)=-0.571469394719D+03
coeff(384)=-0.362772377121D+04
coeff(385)=-0.633677399664D+04
coeff(386)=-0.533008920058D+04
coeff(387)=0.981406766016D+04
coeff(388)=0.734106707511D+04
coeff(389)=-0.220061794127D+04
coeff(390)=-0.119671448630D+04
coeff(391)=-0.177695877775D+04
coeff(392)=-0.319871416200D+04
coeff(393)=-0.162364660011D+03
coeff(394)=-0.519750323948D+04
coeff(395)=-0.172512792168D+04
coeff(396)=0.214456128923D+04
coeff(397)=0.900967485080D+04
coeff(398)=0.400882405844D+04
coeff(399)=0.213192890114D+04
coeff(400)=-0.587264688785D+04
coeff(401)=-0.326016787812D+04
coeff(402)=-0.259141339343D+04
coeff(403)=-0.449341783756D+04
coeff(404)=-0.610712178359D+02
coeff(405)=0.875833445985D+04
coeff(406)=0.116402371133D+04
coeff(407)=-0.160069009170D+04
coeff(408)=0.558411293881D+04
coeff(409)=-0.575078710957D+03
coeff(410)=0.250287793957D+03
coeff(411)=-0.157164971378D+02
coeff(412)=0.202194599765D+03
coeff(413)=-0.320951472298D+04
coeff(414)=0.489483718387D+04
coeff(415)=-0.157739291733D+04
coeff(416)=0.613466773446D+04
coeff(417)=-0.234782693399D+04
coeff(418)=-0.530677341710D+04
coeff(419)=-0.498874098015D+03
coeff(420)=-0.181942160903D+04
coeff(421)=-0.191920757991D+04
coeff(422)=-0.665426251681D+02
coeff(423)=-0.604431496801D+04
coeff(424)=-0.665096985136D+04
coeff(425)=0.473795854187D+04
coeff(426)=-0.392906895120D+04
coeff(427)=0.437113451267D+04
coeff(428)=0.745940117335D+04
coeff(429)=0.792554652285D+04
coeff(430)=0.471288257195D+03
coeff(431)=-0.440098288441D+04
coeff(432)=-0.856634152336D+02
coeff(433)=0.725112328074D+04
coeff(434)=0.184831707351D+04
coeff(435)=-0.898652549900D+03
coeff(436)=0.383895349557D+04
coeff(437)=-0.110342747033D+04
coeff(438)=0.420443701235D+04
coeff(439)=-0.984151260238D+04
coeff(440)=0.856329236011D+04
coeff(441)=0.544692751367D+03
coeff(442)=0.311983721722D+04
coeff(443)=-0.184961214930D+04
coeff(444)=0.374485423080D+04
coeff(445)=-0.116267342075D+05
coeff(446)=0.265267582132D+04
coeff(447)=0.373097069174D+04
coeff(448)=0.397343152033D+04
coeff(449)=-0.100574646960D+05
coeff(450)=-0.204500008905D+04
coeff(451)=-0.425640679841D+03
coeff(452)=-0.548362520617D+04
coeff(453)=0.465831258726D+04
coeff(454)=0.545468242256D+04
coeff(455)=0.153248782112D+04
coeff(456)=0.235964497787D+04
coeff(457)=0.967375565360D+03
coeff(458)=0.197886863766D+04
coeff(459)=-0.530386942561D+04
coeff(460)=-0.173772734583D+04
coeff(461)=0.350003096482D+03
coeff(462)=-0.334266697500D+04
coeff(463)=-0.760316096488D+02
coeff(464)=0.529192262132D+04
coeff(465)=0.560821550643D+04
coeff(466)=-0.163156133726D+04
coeff(467)=-0.211635427550D+04
coeff(468)=0.164393994416D+04
coeff(469)=-0.409116092794D+02
coeff(470)=-0.267291854454D+04
coeff(471)=-0.106587009389D+05
coeff(472)=0.622037982506D+04
coeff(473)=0.611403934736D+04
coeff(474)=0.159869686383D+04
coeff(475)=-0.116140335506D+05
coeff(476)=0.212918975708D+04
coeff(477)=0.156532694031D+03
coeff(478)=-0.699138630514D+03
coeff(479)=0.863460004145D+04
coeff(480)=0.225206485971D+04
coeff(481)=0.698803832451D+04
coeff(482)=0.141778956036D+04
coeff(483)=0.438010987475D+04
coeff(484)=0.318946851180D+04
coeff(485)=-0.320031186342D+04
coeff(486)=-0.246710244270D+04
coeff(487)=-0.807859559922D+04
coeff(488)=-0.238849278722D+04
coeff(489)=0.610917892595D+03
coeff(490)=0.782557519575D+04
coeff(491)=0.100209162980D+05
coeff(492)=0.282928641797D+04
coeff(493)=-0.113934042294D+05
coeff(494)=-0.556504143900D+04
coeff(495)=-0.404507944021D+04
coeff(496)=0.461636114906D+04
coeff(497)=0.551662065859D+04
coeff(498)=-0.190520348737D+04
coeff(499)=0.865922780710D+04
coeff(500)=0.456199010537D+04
coeff(501)=-0.524985380104D+04
coeff(502)=-0.529585468721D+04
coeff(503)=-0.848017804513D+04
coeff(504)=0.939332354249D+04
coeff(505)=0.933827322920D+04
coeff(506)=-0.470234562466D+04
coeff(507)=-0.403219744627D+04
coeff(508)=-0.975418841331D+04
coeff(509)=0.391443720377D+04
coeff(510)=0.683891168876D+03
coeff(511)=-0.251925677467D+04
coeff(512)=0.271139386401D+04
coeff(513)=0.367990253568D+04
coeff(514)=0.261012304015D+04
coeff(515)=0.153399778525D+03
coeff(516)=0.624230121787D+04
coeff(517)=-0.299046272458D+04
coeff(518)=0.125718754283D+04
coeff(519)=0.202129576680D+04
coeff(520)=-0.539619736835D+04
coeff(521)=0.152567208369D+04
coeff(522)=0.357142673430D+04
coeff(523)=-0.468052963933D+04
coeff(524)=0.406508401236D+04
coeff(525)=-0.110018018041D+01
coeff(526)=0.332935869890D+04
coeff(527)=0.336274078550D+04
coeff(528)=-0.521348222923D+04
coeff(529)=-0.138695069878D+04
coeff(530)=0.638975976646D+03
coeff(531)=-0.404384090170D+04
coeff(532)=0.150296784799D+04
coeff(533)=0.670600329633D+03
coeff(534)=0.910930394153D+03
coeff(535)=-0.784253707668D+04
coeff(536)=0.488983002390D+04
coeff(537)=-0.159993247503D+04
coeff(538)=-0.278515763215D+04
coeff(539)=-0.866203054620D+04
coeff(540)=0.357600624734D+04
coeff(541)=-0.755702075213D+04
coeff(542)=-0.108413177078D+04
coeff(543)=-0.253083124687D+03
coeff(544)=0.219159479112D+04
coeff(545)=0.135251232539D+04
coeff(546)=-0.268012532444D+04
coeff(547)=-0.478268653843D+04
coeff(548)=-0.281799705061D+03
coeff(549)=0.102025071098D+05
coeff(550)=0.254214645348D+04
coeff(551)=-0.731442052441D+04
coeff(552)=0.783932223551D+03
coeff(553)=-0.136201177337D+05
coeff(554)=0.773489867592D+04
coeff(555)=-0.661783943376D+04
coeff(556)=-0.152735567075D+04
coeff(557)=0.611968390277D+04
coeff(558)=0.893275817796D+03
coeff(559)=-0.134480171440D+04
coeff(560)=0.377210133454D+04
coeff(561)=0.553462891570D+04
coeff(562)=-0.480440462234D+04
coeff(563)=-0.699698714521D+04
coeff(564)=0.148017261796D+03
coeff(565)=-0.153964047013D+04
coeff(566)=-0.119606149687D+04
coeff(567)=0.677557687114D+04
coeff(568)=0.142452566985D+04
coeff(569)=-0.448453370198D+04
coeff(570)=-0.207455300192D+04
coeff(571)=0.800007744289D+04
coeff(572)=-0.311671062455D+03
coeff(573)=-0.100384946859D+04
coeff(574)=-0.432089884328D+04
coeff(575)=0.175387422469D+04
coeff(576)=0.531676532667D+04
coeff(577)=0.763661013983D+04
coeff(578)=-0.131055186640D+04
coeff(579)=-0.466758384958D+04
coeff(580)=-0.547918958318D+04
coeff(581)=0.645436043507D+04
coeff(582)=0.211647606214D+04
coeff(583)=0.283431692119D+04
coeff(584)=0.519250396573D+04
coeff(585)=0.886753353460D+03
coeff(586)=-0.105524171860D+05
coeff(587)=0.747317968705D+04
coeff(588)=0.284552041924D+04
coeff(589)=0.354329951765D+04
coeff(590)=-0.627365506391D+04
coeff(591)=-0.945053184258D+04
coeff(592)=-0.763908026006D+04
coeff(593)=0.912846768227D+04
coeff(594)=0.105563819637D+03
coeff(595)=0.239533569783D+03
coeff(596)=0.262395471362D+04
coeff(597)=-0.437738896210D+04
coeff(598)=0.106513369321D+05
coeff(599)=-0.245214257736D+03
coeff(600)=-0.207571727837D+04
coeff(601)=-0.629888508187D+04
coeff(602)=-0.441282731647D+04
coeff(603)=0.822347438984D+03
coeff(604)=0.267818768554D+04
coeff(605)=0.990183052076D+04
coeff(606)=0.173166610351D+05
coeff(607)=-0.472538913425D+04
coeff(608)=-0.688736832733D+04
coeff(609)=-0.282078776769D+04
coeff(610)=0.534853091597D+04
coeff(611)=-0.305820595119D+04
coeff(612)=-0.854243292263D+03
coeff(613)=0.104567205830D+05
coeff(614)=-0.257068050809D+04
coeff(615)=-0.212411210139D+04
coeff(616)=0.112611153158D+04
coeff(617)=-0.391523592919D+04
coeff(618)=0.242677687643D+04
coeff(619)=-0.159227059124D+04
coeff(620)=-0.615667957168D+04
coeff(621)=-0.662707627440D+04
coeff(622)=0.230504207607D+04
coeff(623)=0.315828497100D+04
coeff(624)=-0.648342940824D+04
coeff(625)=0.455575051858D+04
coeff(626)=0.145800972592D+04
coeff(627)=-0.279073472566D+04
coeff(628)=-0.292106498590D+04
coeff(629)=-0.472449254079D+04
coeff(630)=-0.102034796647D+04
coeff(631)=-0.339067930629D+04
coeff(632)=0.760783393282D+04
coeff(633)=-0.275743270627D+04
coeff(634)=-0.413015468570D+04
coeff(635)=-0.119612153698D+04
coeff(636)=0.886214723028D+04
coeff(637)=-0.437684947292D+04
coeff(638)=0.187681134406D+04
coeff(639)=-0.472771509100D+04
coeff(640)=0.214248446677D+04
coeff(641)=0.815398485566D+04
coeff(642)=0.307425085589D+04
coeff(643)=-0.166281061783D+04
coeff(644)=0.139126833380D+04
coeff(645)=0.730464094838D+03
coeff(646)=-0.673353923747D+04
coeff(647)=0.104491571244D+04
coeff(648)=-0.108357187980D+05
coeff(649)=-0.510571460248D+03
coeff(650)=-0.534143200711D+04
coeff(651)=-0.108357081465D+04
coeff(652)=0.692212078297D+04
coeff(653)=-0.328220902393D+04
coeff(654)=0.230385940559D+04
coeff(655)=-0.203034544268D+04
coeff(656)=-0.427649579650D+04
coeff(657)=-0.655505542391D+04
coeff(658)=-0.379140850200D+04
coeff(659)=0.283382428517D+04
coeff(660)=-0.467855577607D+04
coeff(661)=0.673033282704D+03
coeff(662)=0.639584583428D+04
coeff(663)=0.261943328455D+03
coeff(664)=0.215367771242D+04
coeff(665)=-0.459257213891D+04
coeff(666)=0.211517171096D+04
coeff(667)=0.302961365907D+04
coeff(668)=-0.323697443737D+03
coeff(669)=-0.409099545936D+04
coeff(670)=-0.816815742392D+03
coeff(671)=-0.192509302122D+04
coeff(672)=-0.858450465108D+04
coeff(673)=0.169993445359D+04
coeff(674)=-0.916532651536D+03
coeff(675)=0.687985768945D+04
coeff(676)=-0.887053192969D+04
coeff(677)=0.922143477453D+04
coeff(678)=0.113346023223D+05
coeff(679)=0.542708644029D+04
coeff(680)=-0.234260762613D+04
coeff(681)=-0.803920809982D+03
coeff(682)=0.396230878084D+04
coeff(683)=-0.436640904035D+04
coeff(684)=0.338577070966D+04
coeff(685)=-0.322043040794D+04
coeff(686)=0.200884056984D+04
coeff(687)=0.114711936775D+05
coeff(688)=0.172791070005D+04
coeff(689)=0.651789089802D+04
coeff(690)=-0.554811797954D+04
coeff(691)=0.316514891251D+04
coeff(692)=0.789658531588D+04
coeff(693)=0.733802195729D+04
coeff(694)=-0.770707753049D+03
coeff(695)=0.433639764932D+03
coeff(696)=-0.168373909630D+03
coeff(697)=0.116829516701D+04
coeff(698)=0.313305901413D+04
coeff(699)=-0.801825373574D+03
coeff(700)=0.670020516718D+04
coeff(701)=0.227108756524D+04
coeff(702)=-0.113724011779D+05
coeff(703)=-0.780354273824D+04
coeff(704)=0.207182621052D+04
coeff(705)=-0.446804197556D+03
coeff(706)=0.210597653540D+04
coeff(707)=-0.645281606368D+03
coeff(708)=0.151954866521D+04
coeff(709)=0.375192155267D+04
coeff(710)=-0.376696718032D+04
coeff(711)=0.621107320156D+04
coeff(712)=0.713558263832D+04
coeff(713)=-0.542053892297D+04
coeff(714)=-0.286557476753D+03
coeff(715)=-0.405688419539D+04
coeff(716)=0.109150355593D+04
coeff(717)=-0.237822982002D+04
coeff(718)=0.127856192923D+03
coeff(719)=0.254292276025D+04
coeff(720)=0.133724998223D+04
coeff(721)=-0.964615895757D+03
coeff(722)=-0.779681481279D+03
coeff(723)=-0.175014395815D+04
coeff(724)=-0.638673256553D+03
coeff(725)=-0.986311947402D+03
coeff(726)=-0.321348229949D+04
coeff(727)=0.503250518154D+03
coeff(728)=0.223899596957D+04
coeff(729)=0.270962426893D+04
coeff(730)=-0.599913094017D+03
coeff(731)=0.388467525003D+03
coeff(732)=-0.430442435991D+04
coeff(733)=0.265865777736D+04
coeff(734)=-0.350140710179D+04
coeff(735)=-0.303616816438D+03
coeff(736)=0.627955800705D+04
coeff(737)=0.238277609241D+03
coeff(738)=0.386155101263D+04
coeff(739)=0.270566325905D+04
coeff(740)=-0.827232549869D+04
coeff(741)=-0.322387901436D+03
coeff(742)=-0.459453233231D+03
coeff(743)=0.352631411905D+04
coeff(744)=-0.182137505393D+03
coeff(745)=0.515275625786D+04
coeff(746)=0.462595796809D+04
coeff(747)=-0.474029571434D+04
coeff(748)=-0.625981468858D+04
coeff(749)=-0.689469259164D+04
coeff(750)=-0.177505388688D+04
coeff(751)=0.914100397641D+04
coeff(752)=0.339735369683D+04
coeff(753)=-0.456349691849D+04
coeff(754)=-0.851353435472D+03
coeff(755)=0.394858777645D+04
coeff(756)=0.182362243362D+03
coeff(757)=0.520534498947D+04
coeff(758)=-0.814113513412D+04
coeff(759)=-0.177274196694D+04
coeff(760)=0.273466581089D+03
coeff(761)=0.331524171601D+04
coeff(762)=0.498306349233D+04
coeff(763)=-0.102467419378D+04
coeff(764)=0.581236108173D+03
coeff(765)=-0.301582879741D+04
coeff(766)=0.104433701401D+04
coeff(767)=-0.585108969635D+03
coeff(768)=-0.774725294492D+04
coeff(769)=-0.439514058041D+04
coeff(770)=-0.286196877509D+04
coeff(771)=-0.736656800865D+04
coeff(772)=0.303923566470D+03
coeff(773)=-0.967334193548D+04
coeff(774)=0.119872192234D+05
coeff(775)=0.173476435546D+04
coeff(776)=-0.224595771568D+04
coeff(777)=-0.872095484711D+04
coeff(778)=-0.103295574816D+04
coeff(779)=-0.951361262185D+03
coeff(780)=0.414803607935D+04
coeff(781)=0.197556282325D+04
coeff(782)=0.260107706235D+04
coeff(783)=0.125644165342D+04
coeff(784)=0.285487750641D+04
coeff(785)=-0.362810717590D+04
coeff(786)=0.546731302099D+03
coeff(787)=0.364639391725D+04
coeff(788)=0.173359459555D+03
coeff(789)=-0.791823016511D+02
coeff(790)=-0.180980735297D+04
coeff(791)=0.158664351829D+04
coeff(792)=-0.204144619172D+04
coeff(793)=-0.198433519839D+04
coeff(794)=-0.268904902638D+03
coeff(795)=0.668962212935D+04
coeff(796)=-0.232741398902D+04
coeff(797)=0.133824453394D+04
coeff(798)=0.227419148864D+04
coeff(799)=0.113705660248D+04
coeff(800)=-0.285638279647D+04
coeff(801)=-0.271572700567D+04
coeff(802)=0.238263817700D+04
coeff(803)=0.483200022816D+04
coeff(804)=-0.390782147567D+04
coeff(805)=-0.564439100772D+04
coeff(806)=0.645792646487D+04
coeff(807)=-0.259389919566D+04
coeff(808)=0.700631661681D+04
coeff(809)=0.220994865750D+04
coeff(810)=-0.537442958082D+04
coeff(811)=-0.129979626604D+05
coeff(812)=-0.349970251249D+04
coeff(813)=0.339827952231D+04
coeff(814)=-0.194465903189D+04
coeff(815)=-0.638621742840D+04
coeff(816)=0.820916050164D+04
coeff(817)=0.335629054765D+04
coeff(818)=-0.776108218461D+03
coeff(819)=-0.499923682955D+04
coeff(820)=-0.141990075613D+05
coeff(821)=-0.310810620384D+04
coeff(822)=-0.143493030106D+04
coeff(823)=-0.291660790881D+04
coeff(824)=0.109888867219D+04
coeff(825)=0.844309762256D+03
coeff(826)=0.184038096918D+04
coeff(827)=0.620179186042D+04
coeff(828)=0.570414585449D+04
coeff(829)=0.405416954774D+04
coeff(830)=-0.100723168224D+04
coeff(831)=-0.591705937405D+04
coeff(832)=0.362080911736D+04
coeff(833)=-0.163232241542D+04
coeff(834)=-0.103398441723D+05
coeff(835)=0.413099558467D+04
coeff(836)=-0.161823062999D+04
coeff(837)=-0.136344764328D+04
coeff(838)=0.585076949419D+04
coeff(839)=0.418139619576D+04
coeff(840)=0.668582948439D+04
coeff(841)=-0.102467876213D+04
coeff(842)=-0.763186118001D+04
coeff(843)=-0.463073970559D+04
coeff(844)=-0.151312889228D+04
coeff(845)=-0.131297688392D+04
coeff(846)=-0.542729000058D+04
coeff(847)=0.171881722410D+04
coeff(848)=0.679325315864D+04
coeff(849)=0.287596183545D+04
coeff(850)=0.601254798876D+04
coeff(851)=0.946792162420D+04
coeff(852)=-0.527084295119D+04
coeff(853)=-0.697650993545D+04
coeff(854)=0.119161529759D+04
coeff(855)=-0.615992308679D+04
coeff(856)=-0.812116840196D+04
coeff(857)=-0.722193171437D+04
coeff(858)=0.849588654398D+03
coeff(859)=-0.389067307675D+04
coeff(860)=0.719343194410D+03
coeff(861)=0.845879971174D+04
coeff(862)=-0.811212668024D+04
coeff(863)=-0.695743771305D+04
coeff(864)=-0.663663098450D+03
coeff(865)=0.354025562287D+04
coeff(866)=-0.505270416032D+04
coeff(867)=-0.134289107801D+04
coeff(868)=0.694948065547D+03
coeff(869)=0.953454377426D+03
coeff(870)=-0.503712086928D+04
coeff(871)=0.686364244264D+03
coeff(872)=0.257988482847D+04
coeff(873)=0.468833841491D+04
coeff(874)=0.993377617354D+03
coeff(875)=0.960835446959D+03
coeff(876)=-0.123666286125D+04
coeff(877)=0.239246827615D+04
coeff(878)=0.628522933075D+04
coeff(879)=0.195777237207D+04
coeff(880)=0.857307578668D+03
coeff(881)=-0.308623946052D+04
coeff(882)=0.599894922656D+04
coeff(883)=-0.170304014359D+04
coeff(884)=-0.714891238772D+04
coeff(885)=0.306433129920D+04
coeff(886)=-0.423059057304D+04
coeff(887)=-0.296393868940D+04
coeff(888)=-0.467675576038D+04
coeff(889)=-0.290207681496D+04
coeff(890)=0.561385396360D+04
coeff(891)=-0.652281958112D+04
coeff(892)=-0.133520102046D+03
coeff(893)=0.336682815023D+04
coeff(894)=-0.211437664795D+04
coeff(895)=-0.216017879049D+04
coeff(896)=-0.293113306321D+04
coeff(897)=-0.430061788331D+04
coeff(898)=0.355113999210D+04
coeff(899)=-0.444807746914D+04
coeff(900)=0.508560788558D+03
coeff(901)=0.611734691241D+04
coeff(902)=0.978799007953D+04
coeff(903)=-0.477271724306D+04
coeff(904)=-0.928816048485D+04
coeff(905)=0.439487279608D+04
coeff(906)=-0.259825249501D+04
coeff(907)=0.194300039477D+04
coeff(908)=0.397795783662D+04
coeff(909)=0.410024697680D+04
coeff(910)=-0.785823150597D+04
coeff(911)=0.131895795318D+05
coeff(912)=-0.871344386398D+04
coeff(913)=-0.155359558512D+05
coeff(914)=-0.405364932364D+04
coeff(915)=-0.872198047599D+03
coeff(916)=-0.628412810614D+04
coeff(917)=0.639538242471D+04
coeff(918)=-0.188431119343D+04
coeff(919)=-0.314626515048D+04
coeff(920)=-0.118459551514D+05
coeff(921)=-0.942870170220D+03
coeff(922)=-0.136593815375D+04
coeff(923)=-0.143262477808D+04
coeff(924)=0.985165785666D+03
coeff(925)=0.591940521630D+04
coeff(926)=0.515469406697D+04
coeff(927)=-0.567035428856D+04
coeff(928)=0.629580501037D+04
coeff(929)=0.108136279483D+04
coeff(930)=0.115454596502D+04
coeff(931)=-0.128499612405D+04
coeff(932)=-0.472463274368D+04
coeff(933)=0.320097720171D+04
coeff(934)=0.589585656992D+02
coeff(935)=0.372765603380D+04
coeff(936)=-0.326366481930D+04
coeff(937)=-0.726052419592D+03
coeff(938)=0.193761112928D+04
coeff(939)=0.275078722414D+04
coeff(940)=-0.248814762629D+04
coeff(941)=0.610412998208D+04
coeff(942)=-0.785162231229D+04
coeff(943)=-0.359550133602D+04
coeff(944)=-0.129301532079D+04
coeff(945)=-0.327241264019D+03
coeff(946)=0.724059311508D+04
coeff(947)=0.857415157827D+03
coeff(948)=-0.285823757913D+04
coeff(949)=0.252918019976D+04
coeff(950)=0.353090516922D+04
coeff(951)=0.171286713381D+04
coeff(952)=-0.346185113900D+03
coeff(953)=-0.420025734514D+04
coeff(954)=0.450040313832D+04
coeff(955)=-0.129962582372D+05
coeff(956)=0.112250767623D+05
coeff(957)=0.125378382543D+04
coeff(958)=0.148093899477D+04
coeff(959)=-0.108395556867D+04
coeff(960)=-0.141652531122D+04
coeff(961)=-0.369146332484D+04
coeff(962)=0.345680814425D+04
coeff(963)=0.387083894453D+04
coeff(964)=0.553563508621D+04
coeff(965)=-0.187485442933D+04
coeff(966)=-0.785632700125D+03
coeff(967)=-0.709803936818D+03
coeff(968)=-0.271782276909D+04
coeff(969)=0.594281396160D+03
coeff(970)=-0.279938675517D+04
coeff(971)=-0.265089500396D+04
coeff(972)=-0.298288184259D+04
coeff(973)=0.597177379528D+04
coeff(974)=0.159829333914D+04
coeff(975)=-0.387533997721D+04
coeff(976)=-0.491413466208D+04
coeff(977)=-0.817290321090D+03
coeff(978)=0.696344202568D+04
coeff(979)=-0.319404240382D+04
coeff(980)=0.245035838056D+04
coeff(981)=-0.447870473814D+04
coeff(982)=-0.476427973556D+04
coeff(983)=-0.970513929402D+02
coeff(984)=-0.309243143103D+04
coeff(985)=0.381237819542D+04
coeff(986)=-0.838855083624D+04
coeff(987)=0.127769660656D+02
coeff(988)=-0.405290008935D+04
coeff(989)=0.101047744347D+04
coeff(990)=-0.361362418130D+04
coeff(991)=0.298309067739D+04
coeff(992)=0.266529060876D+04
coeff(993)=0.302966848025D+04
coeff(994)=0.429631879942D+04
coeff(995)=0.383081380201D+04
coeff(996)=0.165139030789D+04
coeff(997)=-0.697347366153D+04
coeff(998)=0.364721933224D+04
coeff(999)=0.697011835779D+04
coeff(1000)=0.924248637202D+04
coeff(1001)=-0.660225750744D+04
coeff(1002)=-0.364110101131D+04
coeff(1003)=-0.207703732406D+04
coeff(1004)=-0.198309975358D+04
coeff(1005)=0.607059133251D+04
coeff(1006)=0.254465207033D+03
coeff(1007)=-0.985212037975D+03
coeff(1008)=-0.534330503852D+04
coeff(1009)=-0.387417140410D+04
coeff(1010)=0.109547169012D+05
coeff(1011)=0.289106240127D+03
coeff(1012)=0.467549616506D+04
coeff(1013)=0.546202906067D+04
coeff(1014)=-0.225671109132D+04
coeff(1015)=-0.134290804142D+04
coeff(1016)=0.269280898779D+04
coeff(1017)=0.260642166533D+04
coeff(1018)=-0.403278026650D+04
coeff(1019)=0.425644374383D+04
coeff(1020)=0.277441319745D+04
coeff(1021)=0.613281895815D+04
coeff(1022)=0.127349197300D+03
coeff(1023)=-0.756197138398D+04
coeff(1024)=-0.109675806004D+05
coeff(1025)=-0.732358640903D+04
coeff(1026)=0.707745297692D+04
coeff(1027)=0.568723294198D+03
coeff(1028)=-0.176239755148D+04
coeff(1029)=0.108278234716D+05
coeff(1030)=0.248410825274D+04
coeff(1031)=-0.446895630368D+04
coeff(1032)=0.176582284223D+04
coeff(1033)=0.346779381690D+03
coeff(1034)=0.373093400615D+04
coeff(1035)=0.539892518356D+04
coeff(1036)=-0.341344325676D+04
coeff(1037)=0.323776807156D+04
coeff(1038)=0.733346286354D+04
coeff(1039)=-0.642818105198D+04
coeff(1040)=-0.642888934322D+04
coeff(1041)=-0.251064482772D+04
coeff(1042)=0.605855409561D+04
coeff(1043)=-0.200431312927D+04
coeff(1044)=0.158018908739D+04
coeff(1045)=0.450136216492D+04
coeff(1046)=0.744567317717D+04
coeff(1047)=0.966428568588D+03
coeff(1048)=-0.478531000974D+04
coeff(1049)=-0.185934122253D+03
coeff(1050)=-0.220900136301D+04
coeff(1051)=0.246055898846D+04
coeff(1052)=0.401617037831D+04
coeff(1053)=-0.197538300053D+04
coeff(1054)=0.781390645928D+04
coeff(1055)=-0.558500200852D+04
coeff(1056)=-0.103038190318D+04
coeff(1057)=-0.422217948696D+04
coeff(1058)=0.253461923577D+04
coeff(1059)=-0.512618338912D+04
coeff(1060)=0.525414129663D+04
coeff(1061)=0.452189161244D+04
coeff(1062)=-0.253867745052D+04
coeff(1063)=0.116601861568D+04
coeff(1064)=0.211559916667D+03
coeff(1065)=0.159161970068D+04
coeff(1066)=0.983032601258D+04
coeff(1067)=0.460601351901D+04
coeff(1068)=0.416883535006D+04
coeff(1069)=0.646292313724D+03
coeff(1070)=0.839966637022D+03
coeff(1071)=-0.239236615647D+04
coeff(1072)=0.310530526333D+04
coeff(1073)=-0.329626564421D+04
coeff(1074)=-0.118564045253D+04
coeff(1075)=-0.199255897523D+04
coeff(1076)=0.615511785304D+04
coeff(1077)=0.457417831074D+03
coeff(1078)=0.100544174073D+05
coeff(1079)=-0.822138052118D+04
coeff(1080)=-0.323552538419D+04
coeff(1081)=0.743489991296D+04
coeff(1082)=-0.147897354584D+04
coeff(1083)=0.360667585945D+04
coeff(1084)=-0.304670502497D+04
coeff(1085)=-0.604646339331D+03
coeff(1086)=-0.105602443262D+04
coeff(1087)=0.141064306858D+05
coeff(1088)=0.169976572749D+04
coeff(1089)=-0.495222616753D+02
coeff(1090)=-0.655251243636D+03
coeff(1091)=-0.884615614665D+02
coeff(1092)=0.110976910442D+05
coeff(1093)=-0.746516512225D+04
coeff(1094)=0.533495155304D+03
coeff(1095)=-0.940366623712D+04
coeff(1096)=0.576150289482D+04
coeff(1097)=0.300882332264D+03
coeff(1098)=0.178976644449D+04
coeff(1099)=-0.978570889587D+04
coeff(1100)=-0.406427750381D+04
coeff(1101)=0.108764867609D+04
coeff(1102)=0.293047410406D+04
coeff(1103)=-0.365509535161D+04
coeff(1104)=0.208595945524D+03
coeff(1105)=-0.988931538584D+04
coeff(1106)=0.105340589852D+05
coeff(1107)=0.842748381769D+04
coeff(1108)=-0.974671423844D+02
coeff(1109)=0.367510001612D+04
coeff(1110)=0.530820222586D+04
coeff(1111)=0.360738351786D+04
coeff(1112)=0.211943615724D+04
coeff(1113)=0.166571993513D+04
coeff(1114)=0.460763360004D+04
coeff(1115)=0.541111749390D+04
coeff(1116)=-0.176004580223D+03
coeff(1117)=0.329340692865D+04
coeff(1118)=-0.661615612991D+03
coeff(1119)=0.706175656535D+03
coeff(1120)=-0.217473365145D+04
coeff(1121)=-0.924204405434D+04
coeff(1122)=0.261039440190D+04
coeff(1123)=0.221152529456D+04
coeff(1124)=0.518340126064D+04
coeff(1125)=0.793490096849D+04
coeff(1126)=-0.253684469761D+04
coeff(1127)=0.816077796629D+02
coeff(1128)=0.136120679431D+05
coeff(1129)=-0.177093715903D+04
coeff(1130)=0.275752903209D+03
coeff(1131)=0.519097658231D+04
coeff(1132)=0.232637408374D+04
coeff(1133)=0.305098646123D+04
coeff(1134)=-0.246439513793D+04
coeff(1135)=0.381075749083D+03
coeff(1136)=-0.132782248250D+04
coeff(1137)=0.208360149216D+04
coeff(1138)=-0.330059152522D+03
coeff(1139)=-0.114777104025D+04
coeff(1140)=-0.192177247123D+04
coeff(1141)=0.115875335722D+04
coeff(1142)=0.651198552723D+04
coeff(1143)=-0.354390261294D+04
coeff(1144)=-0.112783130936D+02
coeff(1145)=0.768178900378D+04
coeff(1146)=0.299767725149D+04
coeff(1147)=-0.535183126171D+03
coeff(1148)=-0.318406973261D+04
coeff(1149)=-0.865547103704D+04
coeff(1150)=-0.865365723557D+04
coeff(1151)=0.392643585907D+04
coeff(1152)=0.343087301238D+04
coeff(1153)=-0.421470935186D+03
coeff(1154)=-0.714124724622D+04
coeff(1155)=0.109394424051D+05
coeff(1156)=0.344427761421D+04
coeff(1157)=0.404718269546D+04
coeff(1158)=0.212663076996D+03
coeff(1159)=-0.462369788634D+04
coeff(1160)=-0.116676304036D+04
coeff(1161)=-0.327064077662D+04
coeff(1162)=0.109184565318D+05
coeff(1163)=0.189507373653D+04
coeff(1164)=-0.323949128813D+04
coeff(1165)=-0.188770793407D+04
coeff(1166)=0.774644363785D+04
coeff(1167)=0.656174678904D+04
coeff(1168)=-0.340858382807D+04
coeff(1169)=0.172559395339D+04
coeff(1170)=0.405575701012D+04
coeff(1171)=-0.611609620118D+04
coeff(1172)=0.141957131970D+04
coeff(1173)=0.265498732774D+04
coeff(1174)=-0.264801852964D+04
coeff(1175)=-0.128273555390D+05
coeff(1176)=0.195230907709D+04
coeff(1177)=0.861711704726D+03
coeff(1178)=-0.131032450196D+04
coeff(1179)=-0.295056491945D+04
coeff(1180)=-0.112547743132D+05
coeff(1181)=-0.794948562521D+04
coeff(1182)=0.152391973531D+04
coeff(1183)=0.895009352775D+02
coeff(1184)=-0.423600077380D+04
coeff(1185)=-0.981526927104D+04
coeff(1186)=-0.212713329495D+03
coeff(1187)=0.221237254413D+04
coeff(1188)=-0.243654139798D+04
coeff(1189)=0.916092315980D+04
coeff(1190)=0.565231087690D+04
coeff(1191)=0.636255698004D+04
coeff(1192)=-0.731107570418D+04
coeff(1193)=-0.433999150105D+04
coeff(1194)=0.629288629511D+04
coeff(1195)=-0.210396749445D+03
coeff(1196)=-0.898770983255D+04
coeff(1197)=-0.506113377716D+04
coeff(1198)=0.344491388982D+04
coeff(1199)=-0.257555136355D+04
coeff(1200)=-0.248826070222D+04
coeff(1201)=-0.589091069218D+04
coeff(1202)=-0.423689772725D+04
coeff(1203)=0.477125966426D+04
coeff(1204)=-0.156018858550D+03
coeff(1205)=-0.180196597355D+04
coeff(1206)=-0.931143229892D+03
coeff(1207)=0.225728069618D+04
coeff(1208)=-0.667484896620D+04
coeff(1209)=-0.430020131177D+03
coeff(1210)=-0.238873994655D+02
coeff(1211)=-0.400358528528D+04
coeff(1212)=0.141943424192D+04
coeff(1213)=0.456145966758D+04
coeff(1214)=-0.839768028371D+04
coeff(1215)=0.543589724276D+02
coeff(1216)=0.383260856742D+04
coeff(1217)=0.256507491551D+03
coeff(1218)=0.458477517473D+04
coeff(1219)=0.150835538952D+04
coeff(1220)=0.586627995689D+04
coeff(1221)=0.583853063866D+04
coeff(1222)=-0.397768016164D+04
coeff(1223)=0.455885915761D+04
coeff(1224)=-0.482716175588D+04
coeff(1225)=-0.764602811983D+04
coeff(1226)=0.464475557522D+04
coeff(1227)=0.750453330854D+04
coeff(1228)=-0.559336548227D+04
coeff(1229)=0.275782710016D+04
coeff(1230)=0.525966134472D+03
coeff(1231)=0.138190778596D+04
coeff(1232)=0.205381186736D+04
coeff(1233)=0.417049666136D+03
coeff(1234)=0.226173544889D+04
coeff(1235)=-0.186153044393D+04
coeff(1236)=0.234621096954D+04
coeff(1237)=0.348428675936D+04
coeff(1238)=-0.106907069000D+04
coeff(1239)=0.281544004787D+04
coeff(1240)=-0.987780407164D+04
coeff(1241)=0.471772574237D+04
coeff(1242)=-0.546514209700D+04
coeff(1243)=-0.300318389923D+04
coeff(1244)=-0.475755291575D+04
coeff(1245)=-0.622507694571D+03
coeff(1246)=-0.492447936827D+04
coeff(1247)=0.154589496381D+04
coeff(1248)=-0.586556312158D+04
coeff(1249)=-0.382546525645D+04
coeff(1250)=-0.439047911745D+04
coeff(1251)=-0.490063977063D+04
coeff(1252)=0.511215847277D+04
coeff(1253)=0.321805992646D+04
coeff(1254)=-0.437211573566D+03
coeff(1255)=0.209249843807D+04
coeff(1256)=0.582466316839D+03
coeff(1257)=-0.128228709780D+05
coeff(1258)=-0.324305834593D+04
coeff(1259)=0.390387160887D+04
coeff(1260)=0.412759623882D+04
coeff(1261)=-0.165009616996D+04
coeff(1262)=0.467340766290D+04
coeff(1263)=0.156808725331D+03
coeff(1264)=0.242657770631D+04
coeff(1265)=-0.159156218320D+04
coeff(1266)=-0.530906095380D+03
coeff(1267)=-0.269533565694D+04
coeff(1268)=0.632500144195D+04
coeff(1269)=-0.534386103031D+04
coeff(1270)=-0.725948657040D+04
coeff(1271)=0.566338801128D+04
coeff(1272)=-0.163138329881D+04
coeff(1273)=-0.166799858183D+04
coeff(1274)=0.357800886489D+03
coeff(1275)=-0.531036932056D+04
coeff(1276)=-0.450558189171D+04
coeff(1277)=-0.196465679536D+04
coeff(1278)=-0.219245461228D+04
coeff(1279)=-0.252145215376D+04
coeff(1280)=-0.869354844635D+03
coeff(1281)=0.413912773439D+04
coeff(1282)=0.585679964401D+04
coeff(1283)=0.233464530165D+04
coeff(1284)=0.545800447565D+04
coeff(1285)=-0.766160819489D+04
coeff(1286)=-0.570055515318D+04
coeff(1287)=-0.208725811325D+04
coeff(1288)=0.146561555773D+03
coeff(1289)=-0.487786815956D+04
coeff(1290)=-0.843526946567D+03
coeff(1291)=0.458564299547D+04
coeff(1292)=0.352300550187D+04
coeff(1293)=0.694459550864D+03
coeff(1294)=0.383356983851D+04
coeff(1295)=0.313316650052D+04
coeff(1296)=-0.503137949219D+04
coeff(1297)=-0.921335598421D+04
coeff(1298)=-0.114107178778D+04
coeff(1299)=0.403338142248D+04
coeff(1300)=0.235934276817D+04
coeff(1301)=-0.784127533284D+04
coeff(1302)=-0.116911898770D+04
coeff(1303)=-0.213013426449D+04
coeff(1304)=0.224120655229D+04
coeff(1305)=0.936810723073D+04
coeff(1306)=0.361889013459D+04
coeff(1307)=0.176023057069D+04
coeff(1308)=-0.102318418027D+04
coeff(1309)=-0.883822983261D+04
coeff(1310)=0.426737802393D+04
coeff(1311)=0.481921279463D+04
coeff(1312)=-0.546775052124D+04
coeff(1313)=0.382206447718D+04
coeff(1314)=0.407439168733D+04
coeff(1315)=-0.707971959337D+04
coeff(1316)=0.281218286642D+04
coeff(1317)=0.679188096630D+04
coeff(1318)=-0.269767556933D+04
coeff(1319)=0.254984424795D+04
coeff(1320)=0.363271527487D+04
coeff(1321)=0.339733191659D+04
coeff(1322)=0.528348772227D+04
coeff(1323)=-0.169260314240D+04
coeff(1324)=0.361983485828D+04
coeff(1325)=0.831050693951D+04
coeff(1326)=0.977922646829D+03
coeff(1327)=0.297909850154D+04
coeff(1328)=-0.485620462071D+04
coeff(1329)=-0.210204540006D+03
coeff(1330)=0.551219010957D+04
coeff(1331)=-0.551965179499D+04
coeff(1332)=0.740812510128D+04
coeff(1333)=-0.984669227592D+03
coeff(1334)=0.463690043442D+04
coeff(1335)=0.518687295662D+04
coeff(1336)=0.305610209008D+04
coeff(1337)=0.443167430076D+03
coeff(1338)=0.181377044601D+03
coeff(1339)=0.368990526919D+04
coeff(1340)=-0.278420758149D+04
coeff(1341)=-0.678785614207D+03
coeff(1342)=-0.290287416908D+04
coeff(1343)=-0.119036678366D+05
coeff(1344)=0.531723205695D+04
coeff(1345)=-0.103098678147D+05
coeff(1346)=-0.702106798656D+04
coeff(1347)=0.233657681636D+04
coeff(1348)=-0.411586036219D+04
coeff(1349)=-0.283549697528D+04
coeff(1350)=-0.270075359772D+04
coeff(1351)=0.466887349689D+04
coeff(1352)=0.344447894447D+04
coeff(1353)=0.187925138023D+04
coeff(1354)=0.533714411434D+04
coeff(1355)=-0.336076832799D+04
coeff(1356)=-0.314968702868D+04
coeff(1357)=-0.379339825855D+03
coeff(1358)=0.454273213898D+04
coeff(1359)=-0.296719531599D+04
coeff(1360)=-0.442195528356D+02
coeff(1361)=-0.595966400356D+04
coeff(1362)=-0.318615329685D+04
coeff(1363)=0.498563586090D+04
coeff(1364)=0.279295672742D+04
coeff(1365)=0.740213663526D+04
coeff(1366)=0.571620697418D+03
coeff(1367)=0.782333068791D+04
coeff(1368)=-0.372943609937D+04
coeff(1369)=-0.688290211194D+03
coeff(1370)=-0.188544682825D+04
coeff(1371)=0.306999235866D+04
coeff(1372)=0.302137654847D+04
coeff(1373)=-0.292958454255D+04
coeff(1374)=-0.172662893655D+04
coeff(1375)=0.510168846340D+04
coeff(1376)=-0.183783057904D+03
coeff(1377)=-0.120842419084D+05
coeff(1378)=-0.753601816822D+04
coeff(1379)=0.687722731659D+04
coeff(1380)=0.230256919123D+04
coeff(1381)=-0.164668336854D+04
coeff(1382)=0.303494134586D+04
coeff(1383)=0.292228349447D+04
coeff(1384)=0.507944660565D+04
coeff(1385)=-0.199630688820D+03
coeff(1386)=0.649820222412D+04
coeff(1387)=0.524455069908D+04
coeff(1388)=-0.533200360842D+04
coeff(1389)=0.459046588904D+04
coeff(1390)=-0.713704906691D+03
coeff(1391)=0.378330808793D+04
coeff(1392)=-0.518264963547D+03
coeff(1393)=0.335525533754D+04
coeff(1394)=-0.410072513157D+04
coeff(1395)=-0.104131908120D+05
coeff(1396)=-0.333742691874D+04
coeff(1397)=-0.395008045890D+04
coeff(1398)=0.787999005324D+04
coeff(1399)=0.585545077961D+02
coeff(1400)=-0.695975440622D+04
coeff(1401)=0.626402540067D+04
coeff(1402)=0.460453315766D+04
coeff(1403)=-0.390858819994D+03
coeff(1404)=-0.751201455893D+04
coeff(1405)=-0.450430526715D+04
coeff(1406)=0.628397742941D+04
coeff(1407)=0.234164117487D+04
coeff(1408)=0.262549325244D+04
coeff(1409)=0.311113942247D+03
coeff(1410)=-0.113154746189D+04
coeff(1411)=0.925370486919D+03
coeff(1412)=0.149502452278D+04
coeff(1413)=0.173386502391D+04
coeff(1414)=0.590692933917D+04
coeff(1415)=-0.116606250099D+04
coeff(1416)=-0.550953964165D+04
coeff(1417)=0.220329123421D+03
coeff(1418)=-0.505561925372D+04
coeff(1419)=0.275323385159D+04
coeff(1420)=0.156241365460D+05
coeff(1421)=-0.135360262873D+04
coeff(1422)=0.231843406649D+04
coeff(1423)=0.531235244924D+04
coeff(1424)=0.366028320867D+03
coeff(1425)=-0.534549698496D+04
coeff(1426)=-0.421016067155D+04
coeff(1427)=0.675715476947D+04
coeff(1428)=0.244701151387D+04
coeff(1429)=-0.699863619734D+03
coeff(1430)=-0.662021531466D+04
coeff(1431)=-0.423437075982D+04
coeff(1432)=-0.609196061512D+04
coeff(1433)=-0.508040160573D+04
coeff(1434)=-0.140825037794D+04
coeff(1435)=-0.477791251521D+04
coeff(1436)=-0.146729593426D+04
coeff(1437)=-0.377657040749D+03
coeff(1438)=-0.969568066210D+04
coeff(1439)=-0.775041461211D+04
coeff(1440)=0.611938916632D+04
coeff(1441)=0.106999887055D+04
coeff(1442)=0.183651787972D+04
coeff(1443)=0.654726391872D+04
coeff(1444)=0.470488873924D+03
coeff(1445)=-0.125710719286D+03
coeff(1446)=-0.382616572092D+04
coeff(1447)=0.300550131581D+04
coeff(1448)=0.551586974341D+04
coeff(1449)=0.219209121218D+04
coeff(1450)=0.277556131025D+02
coeff(1451)=0.366035481580D+03
coeff(1452)=0.238199438603D+04
coeff(1453)=0.256386272660D+02
coeff(1454)=-0.111128723796D+04
coeff(1455)=0.945591296366D+04
coeff(1456)=0.428343119295D+04
coeff(1457)=-0.715749201403D+04
coeff(1458)=0.219881486224D+03
coeff(1459)=0.329403435941D+04
coeff(1460)=-0.335175584888D+04
coeff(1461)=-0.206749038461D+04
coeff(1462)=0.594368258058D+04
coeff(1463)=0.479645793478D+04
coeff(1464)=0.943403035490D+03
coeff(1465)=0.489266533576D+04
coeff(1466)=-0.550297036336D+04
coeff(1467)=0.139514175182D+05
coeff(1468)=0.130690854316D+05
coeff(1469)=0.427670758765D+04
coeff(1470)=-0.184065504274D+04
coeff(1471)=-0.768783102734D+04
coeff(1472)=-0.136765206640D+03
coeff(1473)=0.713259028189D+04
coeff(1474)=0.252477859678D+04
coeff(1475)=0.163311417597D+04
coeff(1476)=0.684129449224D+02
coeff(1477)=-0.438019039036D+03
coeff(1478)=-0.178028566715D+04
coeff(1479)=0.287142621647D+04
coeff(1480)=-0.226725295163D+03
coeff(1481)=0.848749881608D+03
coeff(1482)=-0.417812383943D+04
coeff(1483)=-0.806853548701D+04
coeff(1484)=-0.407726801349D+04
coeff(1485)=0.112181370697D+05
coeff(1486)=-0.320935351340D+04
coeff(1487)=-0.155965050488D+04
coeff(1488)=-0.644472789303D+04
coeff(1489)=-0.217792325494D+04
coeff(1490)=0.465044851326D+04
coeff(1491)=-0.630475426034D+04
coeff(1492)=0.524463315398D+04
coeff(1493)=0.302339369999D+04
coeff(1494)=0.701262213087D+03
coeff(1495)=0.722463144284D+03
coeff(1496)=-0.159391551611D+05
coeff(1497)=0.528171054053D+04
coeff(1498)=0.429648782102D+04
coeff(1499)=-0.112167040384D+04
coeff(1500)=-0.601891043981D+04
coeff(1501)=0.690627377718D+04
coeff(1502)=0.521845216308D+04
coeff(1503)=0.797141341696D+04
coeff(1504)=0.670572856553D+03
coeff(1505)=0.292387973823D+04
coeff(1506)=0.611109566485D+04
coeff(1507)=0.699609734731D+04
coeff(1508)=-0.284570556417D+04
coeff(1509)=0.366782428600D+04
coeff(1510)=-0.123553820451D+05
coeff(1511)=-0.303130691183D+04
coeff(1512)=-0.302720207328D+02
coeff(1513)=0.322505441853D+04
coeff(1514)=0.767977499963D+04
coeff(1515)=-0.388793103185D+04
coeff(1516)=0.130724004556D+05
coeff(1517)=0.184281358200D+04
coeff(1518)=-0.118985930295D+05
coeff(1519)=0.103207791676D+04
coeff(1520)=0.421526960269D+04
coeff(1521)=0.428363234678D+03
coeff(1522)=0.793872535379D+03
coeff(1523)=-0.942139962288D+03
coeff(1524)=-0.235458594883D+04
coeff(1525)=-0.339789062608D+03
coeff(1526)=0.143003045302D+03
coeff(1527)=-0.968015286084D+03
coeff(1528)=-0.364974982411D+04
coeff(1529)=0.273931994536D+04
coeff(1530)=-0.123383061777D+05
coeff(1531)=0.284866328148D+04
coeff(1532)=-0.355152655117D+04
coeff(1533)=0.670663101301D+04
coeff(1534)=0.136169664399D+04
coeff(1535)=-0.835054280224D+04
coeff(1536)=-0.530326793691D+04
coeff(1537)=-0.449672032164D+03
coeff(1538)=0.110043536270D+04
coeff(1539)=0.419141796830D+04
coeff(1540)=-0.889780741300D+04
coeff(1541)=0.869695333094D+03
coeff(1542)=0.777393945670D+03
coeff(1543)=-0.122037820540D+04
coeff(1544)=-0.311483346138D+04
coeff(1545)=0.739994438765D+03
coeff(1546)=0.445316417310D+04
coeff(1547)=-0.240169875367D+04
coeff(1548)=0.193240266352D+04
coeff(1549)=-0.606941584521D+03
coeff(1550)=-0.801626493389D+04
coeff(1551)=-0.270530088196D+03
coeff(1552)=0.411030442928D+04
coeff(1553)=-0.100996618996D+04
coeff(1554)=0.318461804694D+04
coeff(1555)=-0.714451405522D+03
coeff(1556)=-0.205570347754D+04
coeff(1557)=0.104345483798D+04
coeff(1558)=0.149320462593D+04
coeff(1559)=0.521871811426D+04
coeff(1560)=-0.635336097696D+03
coeff(1561)=-0.149633259987D+04
coeff(1562)=0.318831390032D+04
coeff(1563)=0.116470081787D+04
coeff(1564)=-0.230190707084D+04
coeff(1565)=-0.338415442202D+04
coeff(1566)=0.657975469670D+04
coeff(1567)=0.712584464679D+04
coeff(1568)=0.228044642078D+03
coeff(1569)=0.970202935416D+04
coeff(1570)=0.527969981399D+04
coeff(1571)=0.257156470396D+04
coeff(1572)=-0.371358354093D+04
coeff(1573)=0.332542075050D+04
coeff(1574)=0.121132362788D+04
coeff(1575)=0.658482490341D+03
coeff(1576)=-0.374397764429D+04
coeff(1577)=-0.288429352250D+04
coeff(1578)=0.236517177188D+04
coeff(1579)=-0.461144478341D+04
coeff(1580)=-0.153509118242D+04
coeff(1581)=0.101261547429D+05
coeff(1582)=0.454198326761D+04
coeff(1583)=-0.426866870342D+04
coeff(1584)=0.213411341608D+04
coeff(1585)=0.187018015332D+03
coeff(1586)=-0.371822605060D+03
coeff(1587)=-0.605227363135D+04
coeff(1588)=-0.408317691867D+04
coeff(1589)=0.280140543217D+03
coeff(1590)=0.447382982200D+04
coeff(1591)=0.569430871271D+03
coeff(1592)=-0.259589836920D+04
coeff(1593)=-0.621778774035D+04
coeff(1594)=-0.450629417588D+04
coeff(1595)=0.985587430356D+03
coeff(1596)=0.185060469762D+03
coeff(1597)=0.351964808419D+04
coeff(1598)=0.345903811390D+03
coeff(1599)=0.235860627607D+04
coeff(1600)=-0.253552910003D+03
coeff(1601)=-0.788717024814D+04
coeff(1602)=-0.691309822501D+04
coeff(1603)=-0.318314342459D+04
coeff(1604)=-0.418446012478D+04
coeff(1605)=0.102291716971D+05
coeff(1606)=0.200975714494D+04
coeff(1607)=0.606346251947D+04
coeff(1608)=-0.854156027551D+04
coeff(1609)=-0.525235085901D+04
coeff(1610)=-0.103039248422D+05
coeff(1611)=-0.509684774468D+04
coeff(1612)=-0.105208174343D+04
coeff(1613)=0.553359591855D+03
coeff(1614)=0.338537856236D+04
coeff(1615)=-0.538031956833D+04
coeff(1616)=0.120821684210D+05
coeff(1617)=-0.306174639622D+04
coeff(1618)=-0.112452291615D+05
coeff(1619)=-0.657494670118D+04
coeff(1620)=-0.213609691626D+04
coeff(1621)=0.536131325365D+04
coeff(1622)=0.943078873534D+04
coeff(1623)=-0.350960160301D+04
coeff(1624)=0.653965611527D+04
coeff(1625)=0.812187058081D+04
coeff(1626)=0.653205334425D+04
coeff(1627)=-0.500232972370D+04
coeff(1628)=-0.848337045113D+04
coeff(1629)=-0.343989799573D+04
coeff(1630)=-0.570779722167D+04
coeff(1631)=-0.599290777050D+03
coeff(1632)=0.429350419747D+04
coeff(1633)=0.801771655278D+04
coeff(1634)=0.337474338964D+04
coeff(1635)=0.872224467648D+04
coeff(1636)=0.231541978133D+03
coeff(1637)=-0.671325549194D+04
coeff(1638)=0.480158384864D+04
coeff(1639)=-0.311991790124D+04
coeff(1640)=-0.632497019801D+03
coeff(1641)=0.621509003858D+04
coeff(1642)=0.257461753461D+04
coeff(1643)=-0.286828619827D+04
coeff(1644)=-0.856516978220D+04
coeff(1645)=0.693610602227D+04
coeff(1646)=0.140370963126D+03
coeff(1647)=-0.474766289367D+04
coeff(1648)=0.206485138236D+04
coeff(1649)=-0.418483736458D+04
coeff(1650)=-0.971373006524D+04
coeff(1651)=-0.721735693203D+04
coeff(1652)=0.335885974718D+04
coeff(1653)=-0.855508772034D+03
coeff(1654)=-0.440108430676D+04
coeff(1655)=0.106973624993D+04
coeff(1656)=0.292132495382D+04
coeff(1657)=0.161820448551D+04
coeff(1658)=0.223155608665D+03
coeff(1659)=-0.397693675999D+04
coeff(1660)=0.161155324026D+04
coeff(1661)=-0.905598703660D+04
coeff(1662)=0.341690572518D+03
coeff(1663)=-0.314966451844D+04
coeff(1664)=-0.253096823982D+04
coeff(1665)=-0.492657260739D+04
coeff(1666)=0.288900703328D+04
coeff(1667)=-0.794952444929D+03
coeff(1668)=-0.430407948577D+04
coeff(1669)=-0.341041591888D+04
coeff(1670)=-0.486153516824D+03
coeff(1671)=0.794803275646D+04
coeff(1672)=0.837344240377D+04
coeff(1673)=-0.109262989834D+04
coeff(1674)=0.325248417449D+04
coeff(1675)=0.380918523667D+04
coeff(1676)=0.168925025653D+05
coeff(1677)=0.329868733097D+04
coeff(1678)=0.560655400588D+04
coeff(1679)=0.639891518410D+03
coeff(1680)=-0.345799339857D+04
coeff(1681)=-0.687650682928D+03
coeff(1682)=0.261745688329D+04
coeff(1683)=0.489602167636D+04
coeff(1684)=0.319062813778D+04
coeff(1685)=0.317849354393D+04
coeff(1686)=-0.401873611424D+03
coeff(1687)=-0.402328718411D+04
coeff(1688)=-0.693247946716D+04
coeff(1689)=-0.545402847496D+04
coeff(1690)=0.718528687551D+03
coeff(1691)=-0.112029207632D+05
coeff(1692)=-0.109015208131D+05
coeff(1693)=0.991560457174D+04
coeff(1694)=0.341394176351D+04
coeff(1695)=0.613311087212D+04
coeff(1696)=-0.223759583481D+03
coeff(1697)=-0.383539647630D+04
coeff(1698)=-0.129793739096D+05
coeff(1699)=-0.940870950472D+04
coeff(1700)=0.318893194122D+04
coeff(1701)=0.376025745971D+04
coeff(1702)=-0.383734942043D+04
coeff(1703)=-0.246331579106D+04
coeff(1704)=0.165405630091D+04
coeff(1705)=0.384465924275D+04
coeff(1706)=-0.766593724604D+04
coeff(1707)=0.167199559851D+04
coeff(1708)=0.374844803590D+03
coeff(1709)=-0.905257077178D+04
coeff(1710)=0.208379952716D+04
coeff(1711)=0.735501289726D+04
coeff(1712)=-0.247591696260D+04
coeff(1713)=-0.236168643409D+04
coeff(1714)=0.371540339217D+04
coeff(1715)=-0.918202771482D+04
coeff(1716)=-0.188253934617D+04
coeff(1717)=-0.148749663114D+03
coeff(1718)=0.280901604410D+04
coeff(1719)=0.324748898501D+04
coeff(1720)=0.930892779089D+03
coeff(1721)=0.484862127602D+04
coeff(1722)=0.163508843623D+03
coeff(1723)=-0.447380827022D+04
coeff(1724)=-0.611561655777D+03
coeff(1725)=-0.254911948352D+04
coeff(1726)=0.764883156988D+04
coeff(1727)=0.856108315755D+04
coeff(1728)=-0.164667759135D+04
coeff(1729)=-0.173093682703D+03
coeff(1730)=-0.123695998904D+04
coeff(1731)=0.812058911781D+04
coeff(1732)=-0.473337774896D+03
coeff(1733)=-0.528917287816D+04
coeff(1734)=-0.797257215589D+04
coeff(1735)=-0.406433246747D+04
coeff(1736)=0.269025337172D+04
coeff(1737)=0.238040197178D+04
coeff(1738)=0.108365908013D+02
coeff(1739)=-0.706243548244D+03
coeff(1740)=0.201303879121D+04
coeff(1741)=-0.337234916572D+04
coeff(1742)=-0.610303758865D+04
coeff(1743)=-0.708809695804D+04
coeff(1744)=0.540021904386D+04
coeff(1745)=0.124287958636D+04
coeff(1746)=0.222783360766D+04
coeff(1747)=-0.257443630691D+04
coeff(1748)=-0.907481165316D+04
coeff(1749)=0.155006106858D+05
coeff(1750)=-0.624628775735D+04
coeff(1751)=-0.146460219163D+04
coeff(1752)=0.630466966640D+04
coeff(1753)=0.200027542669D+04
coeff(1754)=-0.774251241289D+04
coeff(1755)=-0.348853156051D+04
coeff(1756)=-0.458391353177D+04
coeff(1757)=-0.436477888362D+04
coeff(1758)=-0.115484029061D+03
coeff(1759)=0.270594670226D+04
coeff(1760)=0.254333324066D+04
coeff(1761)=-0.426559054681D+04
coeff(1762)=0.669590056693D+04
coeff(1763)=0.318810401033D+04
coeff(1764)=-0.196055288835D+04
coeff(1765)=0.637530011142D+04
coeff(1766)=0.725087908250D+04
coeff(1767)=0.271095141008D+04
coeff(1768)=-0.118785380183D+04
coeff(1769)=-0.636995965469D+04
coeff(1770)=-0.636076919813D+04
coeff(1771)=-0.439027522118D+04
coeff(1772)=-0.189635799701D+05
coeff(1773)=-0.152673827290D+05
coeff(1774)=-0.443187280787D+04
coeff(1775)=-0.537419941669D+04
coeff(1776)=0.515500027176D+04
coeff(1777)=0.444490396141D+04
coeff(1778)=0.126398252305D+03
coeff(1779)=0.322961442852D+04
coeff(1780)=0.307133746842D+04
coeff(1781)=-0.116310520645D+05
coeff(1782)=0.306408151415D+04
coeff(1783)=-0.225462560057D+04
coeff(1784)=-0.241845040713D+04
coeff(1785)=0.241807565854D+04
coeff(1786)=0.508232517514D+04
coeff(1787)=0.320398472435D+04
coeff(1788)=-0.372131363431D+04
coeff(1789)=-0.275236121532D+04
coeff(1790)=0.240804867905D+04
coeff(1791)=0.153854262615D+04
coeff(1792)=-0.451416732623D+03
coeff(1793)=0.127834773885D+05
coeff(1794)=-0.392258487939D+04
coeff(1795)=-0.452900157406D+03
coeff(1796)=-0.800060463197D+04
coeff(1797)=-0.466995188313D+04
coeff(1798)=0.271278955824D+03
coeff(1799)=-0.894142032090D+03
coeff(1800)=0.785845972382D+04
coeff(1801)=-0.363827758374D+04
coeff(1802)=-0.602618055019D+04
coeff(1803)=-0.126343751442D+04
coeff(1804)=-0.178891752771D+04
coeff(1805)=0.928561671641D+04
coeff(1806)=0.575617854295D+04
coeff(1807)=0.358339688611D+04
coeff(1808)=-0.514654214671D+04
coeff(1809)=0.134613352866D+05
coeff(1810)=0.430114109498D+04
coeff(1811)=0.144446711558D+05
coeff(1812)=-0.560080238022D+04
coeff(1813)=0.235005894601D+04
coeff(1814)=-0.619549983633D+04
coeff(1815)=-0.147405112041D+04
coeff(1816)=-0.570290044303D+04
coeff(1817)=0.522808345600D+04
coeff(1818)=0.970294967992D+04
coeff(1819)=-0.646528856004D+04
coeff(1820)=-0.287166248982D+02
coeff(1821)=0.361507273846D+04
coeff(1822)=0.308013940368D+03
coeff(1823)=0.634774141386D+04
coeff(1824)=-0.521983383043D+04
coeff(1825)=-0.568135736711D+04
coeff(1826)=-0.589269538587D+04
coeff(1827)=0.117707689479D+04
coeff(1828)=-0.640155846250D+04
coeff(1829)=0.244403201341D+03
coeff(1830)=-0.269247404070D+04
coeff(1831)=0.828556249539D+04
coeff(1832)=-0.683428223228D+04
coeff(1833)=-0.563643161495D+04
coeff(1834)=-0.490553473028D+03
coeff(1835)=0.888095556096D+04
coeff(1836)=0.658712474065D+04
coeff(1837)=-0.275575008034D+04
coeff(1838)=-0.388584606800D+04
coeff(1839)=-0.722312080478D+03
coeff(1840)=-0.818092049254D+04
coeff(1841)=-0.374255735816D+04
coeff(1842)=-0.413184957878D+04
coeff(1843)=0.976047972766D+04
coeff(1844)=0.302060159758D+02
coeff(1845)=-0.511856801217D+04
coeff(1846)=-0.572834441665D+04
coeff(1847)=-0.298847823232D+04
coeff(1848)=-0.767537539097D+04
coeff(1849)=0.335631320310D+04
coeff(1850)=0.848293136108D+04
coeff(1851)=0.605899211479D+04
coeff(1852)=0.689367040569D+03
coeff(1853)=0.117417722293D+04
coeff(1854)=-0.451957223727D+04
coeff(1855)=-0.148093441166D+04
coeff(1856)=-0.113234139681D+01
coeff(1857)=-0.517151166207D+04
coeff(1858)=-0.597332284059D+04
coeff(1859)=-0.632128310387D+04
coeff(1860)=-0.249399765153D+04
coeff(1861)=0.180848030316D+04
coeff(1862)=0.666061450116D+04
coeff(1863)=-0.162819118731D+04
coeff(1864)=0.623621793039D+03
coeff(1865)=-0.155991267938D+04
coeff(1866)=0.329421557497D+04
coeff(1867)=-0.129933638415D+04
coeff(1868)=-0.398245487150D+04
coeff(1869)=-0.843494388399D+04
coeff(1870)=-0.298920255608D+04
coeff(1871)=0.200324979770D+04
coeff(1872)=-0.714748381148D+04
coeff(1873)=0.226043787221D+03
coeff(1874)=0.211334177439D+04
coeff(1875)=-0.218773903112D+04
coeff(1876)=-0.421570564546D+04
coeff(1877)=0.266501155839D+04
coeff(1878)=0.108513133789D+05
coeff(1879)=-0.576171491254D+04
coeff(1880)=-0.860704305134D+04
coeff(1881)=0.507327703481D+04
coeff(1882)=0.314385150595D+04
coeff(1883)=-0.156659911785D+04
coeff(1884)=-0.447162020515D+04
coeff(1885)=0.564771109634D+04
coeff(1886)=0.447297524030D+04
coeff(1887)=-0.794243712678D+04
coeff(1888)=-0.232554322156D+03
coeff(1889)=-0.754610133600D+04
coeff(1890)=0.121539135213D+04
coeff(1891)=0.297154280761D+04
coeff(1892)=0.381914468496D+04
coeff(1893)=-0.570222890432D+04
coeff(1894)=-0.673177652479D+04
coeff(1895)=0.232226798736D+04
coeff(1896)=0.246445102993D+04
coeff(1897)=-0.568700736086D+04
coeff(1898)=-0.542123834641D+04
coeff(1899)=-0.402236364873D+04
coeff(1900)=0.332208946418D+04
coeff(1901)=0.830228436796D+04
coeff(1902)=-0.774213624481D+04
coeff(1903)=0.902945704718D+04
coeff(1904)=0.776959741250D+02
coeff(1905)=0.299885335938D+04
coeff(1906)=-0.990172364618D+03
coeff(1907)=0.417380587178D+04
coeff(1908)=-0.225704328190D+04
coeff(1909)=0.718021228549D+04
coeff(1910)=0.549565069132D+04
coeff(1911)=-0.364150243043D+04
coeff(1912)=0.113614128607D+05
coeff(1913)=0.865838645260D+03
coeff(1914)=-0.429423332172D+04
coeff(1915)=0.589664150177D+04
coeff(1916)=-0.347627655341D+03
coeff(1917)=0.288396419694D+04
coeff(1918)=-0.336492360115D+04
coeff(1919)=0.469947421650D+04
coeff(1920)=-0.230039019749D+03
coeff(1921)=0.137360492692D+04
coeff(1922)=0.693149293090D+04
coeff(1923)=-0.247732490504D+04
coeff(1924)=-0.194864931661D+04
coeff(1925)=-0.734181098581D+04
coeff(1926)=0.673258793574D+04
coeff(1927)=-0.706758837524D+03
coeff(1928)=-0.744536094771D+04
coeff(1929)=-0.290004913825D+04
coeff(1930)=-0.506125877936D+04
coeff(1931)=-0.354547996961D+04
coeff(1932)=-0.125968743856D+04
coeff(1933)=-0.191882486553D+04
coeff(1934)=0.270057117423D+04
coeff(1935)=-0.664133188041D+04
coeff(1936)=-0.399406758942D+04
coeff(1937)=0.296924289670D+04
coeff(1938)=0.454408091152D+04
coeff(1939)=0.213163516701D+04
coeff(1940)=-0.352443108409D+04
coeff(1941)=-0.413728216785D+04
coeff(1942)=-0.871257674002D+02
coeff(1943)=0.568541051948D+04
coeff(1944)=0.507244407248D+03
coeff(1945)=-0.335035953784D+04
coeff(1946)=0.639063105821D+04
coeff(1947)=0.690303564791D+04
coeff(1948)=0.343613007887D+04
coeff(1949)=-0.433968285381D+04
coeff(1950)=0.139512929427D+04
coeff(1951)=-0.327039782357D+04
coeff(1952)=0.198532582192D+04
coeff(1953)=0.202311703749D+04
coeff(1954)=-0.131290943355D+04
coeff(1955)=0.166730105750D+04
coeff(1956)=0.636890925273D+04
coeff(1957)=-0.119595017075D+04
coeff(1958)=-0.197379643102D+03
coeff(1959)=0.207431417683D+04
coeff(1960)=-0.734628098016D+04
coeff(1961)=-0.597246574410D+04
coeff(1962)=-0.416054670780D+04
coeff(1963)=0.100685081831D+04
coeff(1964)=-0.633320251778D+03
coeff(1965)=-0.850597430086D+04
coeff(1966)=-0.191450709783D+02
coeff(1967)=0.195996141026D+04
coeff(1968)=-0.731445856875D+04
coeff(1969)=0.276960719772D+04
coeff(1970)=-0.535478354970D+04
coeff(1971)=0.498788541945D+04
coeff(1972)=0.243044052830D+04
coeff(1973)=-0.124315946859D+04
coeff(1974)=-0.558831332912D+04
coeff(1975)=0.223134652313D+04
coeff(1976)=-0.780653689808D+04
coeff(1977)=-0.395476920373D+04
coeff(1978)=-0.373679531200D+03
coeff(1979)=-0.466268046957D+04
coeff(1980)=-0.779086445424D+04
coeff(1981)=0.109224481968D+05
coeff(1982)=0.131693193146D+04
coeff(1983)=0.115456766355D+05
coeff(1984)=0.292322958265D+04
coeff(1985)=-0.153568026718D+04
coeff(1986)=0.290103594845D+04
coeff(1987)=0.245924888088D+04
coeff(1988)=-0.806994447147D+03
coeff(1989)=0.835387468550D+04
coeff(1990)=0.827034015786D+04
coeff(1991)=-0.548234429694D+04
coeff(1992)=0.562678815449D+04
coeff(1993)=0.966366658574D+04
coeff(1994)=0.483261942469D+04
coeff(1995)=0.104370349227D+05
coeff(1996)=0.105152622258D+03
coeff(1997)=-0.569904795279D+04
coeff(1998)=-0.243721845999D+04
coeff(1999)=-0.558952360352D+04
coeff(2000)=-0.585578134878D+03
coeff(2001)=-0.419488204307D+04
coeff(2002)=-0.100730261381D+04
coeff(2003)=0.416951718689D+04
coeff(2004)=0.686451204024D+04
coeff(2005)=0.205881247203D+04
coeff(2006)=-0.480651230042D+04
coeff(2007)=-0.158119018335D+04
coeff(2008)=-0.662285250249D+04
coeff(2009)=0.660974620055D+04
coeff(2010)=0.223879682417D+04
coeff(2011)=0.448512011568D+04
coeff(2012)=-0.559361713826D+04
coeff(2013)=-0.541862809368D+04
coeff(2014)=0.650166035899D+04
coeff(2015)=0.133530849914D+04
coeff(2016)=-0.561880246410D+04
coeff(2017)=0.587415444283D+04
coeff(2018)=0.311890337339D+04
coeff(2019)=0.264334084876D+04
coeff(2020)=0.281862290927D+04
coeff(2021)=0.341148368487D+04
coeff(2022)=0.108794519643D+04
coeff(2023)=-0.492203085223D+04
coeff(2024)=0.155422283171D+04
coeff(2025)=0.227037634423D+04
coeff(2026)=0.131775728710D+03
coeff(2027)=-0.106065769624D+05
coeff(2028)=0.988094066793D+04
coeff(2029)=-0.275286445585D+04
coeff(2030)=-0.152995057709D+04
coeff(2031)=-0.408752237129D+04
coeff(2032)=-0.192373308637D+04
coeff(2033)=-0.356683374375D+03
coeff(2034)=0.698159909077D+03
coeff(2035)=0.107786292059D+05
coeff(2036)=0.293555865778D+03
coeff(2037)=-0.753344987900D+02
coeff(2038)=-0.104925335615D+05
coeff(2039)=0.314836744994D+04
coeff(2040)=0.111368466293D+05
coeff(2041)=0.707241411974D+04
coeff(2042)=-0.142003210581D+04
coeff(2043)=0.744895281616D+04
coeff(2044)=-0.953936060590D+04
coeff(2045)=-0.107678838419D+04
coeff(2046)=-0.551263134527D+03
coeff(2047)=-0.910052165316D+03
coeff(2048)=0.142144418596D+05
coeff(2049)=-0.797756059826D+04
coeff(2050)=-0.536521428573D+04
coeff(2051)=0.107161020251D+04
coeff(2052)=-0.368677599844D+04
coeff(2053)=-0.158325902067D+04
coeff(2054)=0.177702462627D+05
coeff(2055)=-0.357542719472D+04
coeff(2056)=-0.501465725727D+04
coeff(2057)=0.133017452359D+04
coeff(2058)=0.199911297887D+04
coeff(2059)=-0.514077055629D+04
coeff(2060)=-0.130357181669D+04
coeff(2061)=-0.115521942022D+04
coeff(2062)=-0.124778871548D+05
coeff(2063)=0.533433198246D+04
coeff(2064)=-0.306814928984D+04
coeff(2065)=0.806319178153D+04
coeff(2066)=-0.724193718699D+03
coeff(2067)=0.428988976516D+04
coeff(2068)=0.481340914223D+04
coeff(2069)=0.551208944845D+04
coeff(2070)=-0.251596729494D+04
coeff(2071)=-0.495771517530D+02
coeff(2072)=-0.648095102539D+04
coeff(2073)=-0.530961427605D+04
coeff(2074)=-0.165609003282D+04
coeff(2075)=0.416731771235D+04
coeff(2076)=0.418213110798D+04
coeff(2077)=0.436148562562D+04
coeff(2078)=0.918418603709D+04
coeff(2079)=-0.770440783218D+04
coeff(2080)=0.920806985176D+04
coeff(2081)=0.181040984405D+04
coeff(2082)=0.406829539863D+04
coeff(2083)=-0.529988404774D+04
coeff(2084)=-0.931368036866D+04
coeff(2085)=-0.969596495353D+04
coeff(2086)=0.416989353906D+04
coeff(2087)=0.733716009425D+04
coeff(2088)=0.110406162728D+04
coeff(2089)=0.823785315729D+04
coeff(2090)=0.528193560091D+04
coeff(2091)=-0.564446816977D+04
coeff(2092)=-0.343285059154D+03
coeff(2093)=0.276760496186D+04
coeff(2094)=-0.112949588919D+03
coeff(2095)=-0.195609113495D+04
coeff(2096)=0.148583381887D+04
coeff(2097)=-0.722918613184D+04
coeff(2098)=-0.356479774628D+04
coeff(2099)=-0.148723675905D+05
coeff(2100)=0.107261115348D+04
coeff(2101)=0.235747700238D+04
coeff(2102)=-0.359746266299D+04
coeff(2103)=0.377143148977D+03
coeff(2104)=-0.560683265922D+04
coeff(2105)=0.250584250175D+04
coeff(2106)=0.663320523308D+03
coeff(2107)=-0.248820882533D+04
coeff(2108)=0.448947573462D+04
coeff(2109)=0.296918988664D+04
coeff(2110)=-0.152325335140D+04
coeff(2111)=0.732554220551D+04
coeff(2112)=-0.255007913249D+04
coeff(2113)=-0.304014893503D+04
coeff(2114)=-0.154464209027D+04
coeff(2115)=0.532802285071D+04
coeff(2116)=0.547382526354D+03
coeff(2117)=0.359649899627D+03
coeff(2118)=-0.156310782776D+04
coeff(2119)=-0.491527898076D+04
coeff(2120)=0.181363402621D+04
coeff(2121)=-0.108103704225D+04
coeff(2122)=-0.428183037236D+04
coeff(2123)=0.164188843115D+05
coeff(2124)=0.144998140401D+04
coeff(2125)=-0.170797625190D+04
coeff(2126)=-0.464834897729D+04
coeff(2127)=-0.410622125672D+04
coeff(2128)=-0.342313273872D+04
coeff(2129)=0.325509036002D+03
coeff(2130)=0.104883519368D+04
coeff(2131)=0.396160710477D+04
coeff(2132)=0.976925223794D+04
coeff(2133)=0.669523027421D+04
coeff(2134)=0.628538590789D+04
coeff(2135)=-0.945040543625D+04
coeff(2136)=-0.467670014301D+04
coeff(2137)=-0.218689899997D+04
coeff(2138)=-0.561280041133D+04
coeff(2139)=0.604504282977D+04
coeff(2140)=-0.511878537301D+04
coeff(2141)=0.477304674079D+04
coeff(2142)=0.362058806551D+04
coeff(2143)=-0.332777196450D+04
coeff(2144)=0.607013805273D+03
coeff(2145)=-0.193279622789D+05
coeff(2146)=0.261627308591D+04
coeff(2147)=-0.671971586429D+04
coeff(2148)=0.180392109562D+04
coeff(2149)=0.351719587313D+04
coeff(2150)=0.536701032515D+04
coeff(2151)=-0.100036605938D+05
coeff(2152)=-0.728094259394D+04
coeff(2153)=-0.763355728014D+04
coeff(2154)=0.658033564846D+04
coeff(2155)=-0.100127686738D+04
coeff(2156)=-0.224715449670D+04
coeff(2157)=0.152819215930D+04
coeff(2158)=-0.326392863279D+04
coeff(2159)=-0.396205063596D+03
coeff(2160)=0.146842445620D+04
coeff(2161)=0.292960289445D+04
coeff(2162)=0.804677659705D+04
coeff(2163)=-0.411226063207D+04
coeff(2164)=0.175745331172D+03
coeff(2165)=0.306227080142D+04
coeff(2166)=-0.371321388605D+04
coeff(2167)=-0.813771894122D+04
coeff(2168)=0.390794372240D+04
coeff(2169)=0.348877565211D+04
coeff(2170)=0.522625174031D+04
coeff(2171)=-0.146988270491D+04
coeff(2172)=-0.105899874681D+05
coeff(2173)=0.595418186057D+03
coeff(2174)=-0.587286455688D+04
coeff(2175)=0.588106963212D+04
coeff(2176)=0.807874661403D+03
coeff(2177)=0.104580247464D+05
coeff(2178)=0.133058177035D+03
coeff(2179)=-0.191002285971D+04
coeff(2180)=0.259838966950D+04
coeff(2181)=0.661586368113D+04
coeff(2182)=-0.248312525277D+04
coeff(2183)=-0.113677905545D+04
coeff(2184)=-0.526602454110D+04
coeff(2185)=0.132174530382D+05
coeff(2186)=-0.329444459769D+04
coeff(2187)=-0.942812329602D+04
coeff(2188)=-0.515286707083D+04
coeff(2189)=0.112668736710D+05
coeff(2190)=-0.341833928177D+04
coeff(2191)=0.337406727501D+04
coeff(2192)=0.313827965688D+04
coeff(2193)=-0.550095052695D+04
coeff(2194)=-0.965028833402D+04
coeff(2195)=0.761863465026D+04
coeff(2196)=0.115033977012D+04
coeff(2197)=0.506107031518D+04
coeff(2198)=0.706904209963D+04
coeff(2199)=0.309541799716D+03
coeff(2200)=0.343916068150D+04
coeff(2201)=0.317044730433D+03
coeff(2202)=0.760192782512D+04
coeff(2203)=-0.460745523594D+04
coeff(2204)=0.398437599002D+04
coeff(2205)=-0.451013668109D+04
coeff(2206)=-0.324211087933D+04
coeff(2207)=-0.848014048688D+04
coeff(2208)=-0.439344992346D+04
coeff(2209)=-0.461040779488D+04
coeff(2210)=0.985123889023D+03
coeff(2211)=-0.125517150387D+04
coeff(2212)=-0.947015935007D+04
coeff(2213)=-0.813621066922D+03
coeff(2214)=0.230999385403D+04
coeff(2215)=0.738326209975D+04
coeff(2216)=-0.281203992644D+04
coeff(2217)=0.113065282578D+05
coeff(2218)=0.495184346413D+04
coeff(2219)=-0.291845665395D+04
coeff(2220)=0.601259450000D+04
coeff(2221)=-0.849511448248D+04
coeff(2222)=-0.303811988444D+04
coeff(2223)=-0.596949761512D+04
coeff(2224)=0.103011696784D+05
coeff(2225)=-0.117609106989D+04
coeff(2226)=-0.103186567835D+04
coeff(2227)=-0.266700309665D+04
coeff(2228)=-0.348962161080D+04
coeff(2229)=-0.574919137098D+04
coeff(2230)=0.185110264149D+04
coeff(2231)=-0.699625831850D+04
coeff(2232)=-0.753001122072D+04
coeff(2233)=0.274324972908D+04
coeff(2234)=-0.341024511191D+04
coeff(2235)=-0.650756183322D+04
coeff(2236)=0.595668625393D+04
coeff(2237)=-0.570252149192D+04
coeff(2238)=-0.481109811723D+03
coeff(2239)=0.233895851173D+04
coeff(2240)=0.509424503111D+04
coeff(2241)=0.774886091306D+04
coeff(2242)=0.914215000839D+04
coeff(2243)=-0.121342918166D+05
coeff(2244)=-0.810178657058D+04
coeff(2245)=0.383877049091D+04
coeff(2246)=0.133003773772D+05
coeff(2247)=0.521052038881D+04
coeff(2248)=-0.811141747024D+04
coeff(2249)=-0.133097765512D+05
coeff(2250)=-0.834081242501D+04
coeff(2251)=-0.835654516584D+04
coeff(2252)=0.398139858102D+04
coeff(2253)=-0.786940976740D+04
coeff(2254)=0.134785063045D+04
coeff(2255)=0.104362135306D+04
coeff(2256)=-0.340839565231D+03
coeff(2257)=-0.936086692576D+04
coeff(2258)=-0.127245241920D+05
coeff(2259)=0.350760469225D+04
coeff(2260)=-0.180251065015D+04
coeff(2261)=-0.469069344422D+04
coeff(2262)=-0.204801397528D+04
coeff(2263)=-0.896919266194D+03
coeff(2264)=0.411365635735D+03
coeff(2265)=0.643927008948D+04
coeff(2266)=0.620320108187D+03
coeff(2267)=0.507024901314D+04
coeff(2268)=-0.614321149636D+04
coeff(2269)=0.103645022878D+04
coeff(2270)=0.315228320166D+03
coeff(2271)=-0.157078815148D+04
coeff(2272)=0.893913734052D+04
coeff(2273)=-0.906390162451D+04
coeff(2274)=0.634449107059D+04
coeff(2275)=0.335274880756D+04
coeff(2276)=0.158295722800D+04
coeff(2277)=-0.100141689686D+04
coeff(2278)=0.230472700700D+04
coeff(2279)=0.250655154002D+04
coeff(2280)=-0.223922923859D+03
coeff(2281)=-0.696387646186D+04
coeff(2282)=0.803703350940D+03
coeff(2283)=0.625359700918D+04
coeff(2284)=-0.189491576741D+04
coeff(2285)=0.201917643793D+04
coeff(2286)=-0.678730814307D+04
coeff(2287)=0.635097455805D+04
coeff(2288)=0.380833996453D+04
coeff(2289)=-0.104253274549D+04
coeff(2290)=0.116490192781D+04
coeff(2291)=0.212778160737D+04
coeff(2292)=-0.691391988406D+04
coeff(2293)=-0.163734054179D+04
coeff(2294)=-0.138529568387D+04
coeff(2295)=0.740916763170D+04
coeff(2296)=-0.717295981100D+04
coeff(2297)=-0.313917991527D+04
coeff(2298)=-0.310760078268D+04
coeff(2299)=-0.542971875012D+04
coeff(2300)=0.354822753631D+04
coeff(2301)=0.467387992303D+04
coeff(2302)=0.927941776070D+03
coeff(2303)=0.404931472446D+04
coeff(2304)=0.513605548983D+04
coeff(2305)=-0.231924543268D+04
coeff(2306)=0.311345093977D+04
coeff(2307)=0.399633855262D+04
coeff(2308)=0.124268986328D+05
coeff(2309)=0.232497117105D+04
coeff(2310)=0.795891055137D+04
coeff(2311)=-0.163208336066D+04
coeff(2312)=0.252713672297D+04
coeff(2313)=0.296483761797D+04
coeff(2314)=-0.501303657264D+03
coeff(2315)=-0.706341412925D+02
coeff(2316)=-0.193438948628D+03
coeff(2317)=-0.211923826699D+03
coeff(2318)=-0.344891149458D+04
coeff(2319)=-0.207425065557D+04
coeff(2320)=0.181247316354D+04
coeff(2321)=-0.430885972228D+04
coeff(2322)=0.143698732250D+05
coeff(2323)=0.414380730928D+04
coeff(2324)=-0.716529174836D+03
coeff(2325)=-0.723640067444D+04
coeff(2326)=-0.382429868054D+04
coeff(2327)=0.623071719025D+04
coeff(2328)=0.248526150778D+04
coeff(2329)=-0.421625820199D+02
coeff(2330)=-0.874555773519D+04
coeff(2331)=-0.414741223656D+04
coeff(2332)=-0.478371120556D+04
coeff(2333)=0.914968210324D+04
coeff(2334)=0.262772720421D+04
coeff(2335)=0.164927581525D+04
coeff(2336)=0.255923689781D+04
coeff(2337)=-0.173537267429D+04
coeff(2338)=0.236963942264D+02
coeff(2339)=-0.901878158578D+03
coeff(2340)=-0.737189373990D+04
coeff(2341)=0.701829807766D+04
coeff(2342)=0.453233172963D+04
coeff(2343)=-0.105355225302D+03
coeff(2344)=-0.498464955832D+03
coeff(2345)=0.105751749839D+05
coeff(2346)=0.757769039757D+04
coeff(2347)=0.777753686001D+03
coeff(2348)=-0.492703752413D+03
coeff(2349)=0.737215624842D+04
coeff(2350)=-0.313458407326D+03
coeff(2351)=0.151570058092D+04
coeff(2352)=0.534020119754D+03
coeff(2353)=-0.484357180561D+04
coeff(2354)=0.181449890574D+04
coeff(2355)=0.864220009426D+04
coeff(2356)=-0.891785287831D+03
coeff(2357)=0.448831833828D+04
coeff(2358)=-0.565794714606D+04
coeff(2359)=-0.352478359296D+04
coeff(2360)=-0.128788186635D+05
coeff(2361)=-0.219092158892D+04
coeff(2362)=0.887636514835D+03
coeff(2363)=0.207767570367D+03
coeff(2364)=0.124827995498D+04
coeff(2365)=-0.807783812599D+04
coeff(2366)=0.293095722207D+04
coeff(2367)=0.274656830737D+04
coeff(2368)=0.934350578684D+04
coeff(2369)=-0.430042419061D+04
coeff(2370)=0.199556046040D+04
coeff(2371)=0.191308793761D+04
coeff(2372)=0.981611104688D+03
coeff(2373)=-0.290448803248D+04
coeff(2374)=-0.396048265390D+04
coeff(2375)=-0.704800510818D+04
coeff(2376)=0.155254726006D+04
coeff(2377)=0.140170413484D+04
coeff(2378)=-0.719901510703D+04
coeff(2379)=-0.556346037866D+04
coeff(2380)=0.249259725073D+04
coeff(2381)=-0.327529196707D+04
coeff(2382)=0.510303162681D+04
coeff(2383)=-0.655723043046D+04
coeff(2384)=0.258726135904D+04
coeff(2385)=0.199471322125D+04
coeff(2386)=0.197958728300D+04
coeff(2387)=0.161040962752D+05
coeff(2388)=-0.315409733391D+04
coeff(2389)=0.104641704337D+05
coeff(2390)=0.100093529853D+04
coeff(2391)=-0.609166604810D+04
coeff(2392)=-0.458107242594D+04
coeff(2393)=0.641410521700D+04
coeff(2394)=0.646253330630D+04
coeff(2395)=-0.353354897436D+04
coeff(2396)=-0.771019154551D+04
coeff(2397)=-0.101843684372D+05
coeff(2398)=0.817300218509D+04
coeff(2399)=-0.403472988797D+04
coeff(2400)=-0.120468074788D+05
coeff(2401)=0.615553556400D+04
coeff(2402)=-0.627328472753D+02
coeff(2403)=0.110901919330D+03
coeff(2404)=-0.499509434103D+04
coeff(2405)=-0.326981001935D+03
coeff(2406)=-0.140061637723D+04
coeff(2407)=0.520097024469D+04
coeff(2408)=-0.207545820775D+04
coeff(2409)=-0.205063361221D+04
coeff(2410)=0.263109109727D+04
coeff(2411)=-0.135647389838D+05
coeff(2412)=-0.425392927152D+04
coeff(2413)=-0.682309507236D+04
coeff(2414)=0.146411794661D+03
coeff(2415)=0.815722427399D+04
coeff(2416)=-0.627148119679D+04
coeff(2417)=-0.167044649105D+04
coeff(2418)=-0.317594691882D+04
coeff(2419)=0.363588082780D+03
coeff(2420)=-0.410974466262D+04
coeff(2421)=0.378158415269D+04
coeff(2422)=0.303968744512D+04
coeff(2423)=0.716505886668D+04
coeff(2424)=-0.631911049503D+04
coeff(2425)=0.443563061956D+04
coeff(2426)=0.747111142787D+04
coeff(2427)=-0.116622075493D+02
coeff(2428)=-0.236469276634D+04
coeff(2429)=0.709290358837D+04
coeff(2430)=-0.276695529004D+03
coeff(2431)=-0.489287028476D+04
coeff(2432)=-0.282086163289D+04
coeff(2433)=0.989963136751D+03
coeff(2434)=0.807214757730D+04
coeff(2435)=0.222937441697D+04
coeff(2436)=0.421650313600D+03
coeff(2437)=0.664157669983D+04
coeff(2438)=-0.145775896697D+03
coeff(2439)=-0.382273705463D+04
coeff(2440)=-0.524559955657D+04
coeff(2441)=-0.169617473983D+04
coeff(2442)=-0.115149673322D+05
coeff(2443)=-0.808865657474D+03
coeff(2444)=-0.520425700519D+04
coeff(2445)=0.415098542525D+04
coeff(2446)=-0.207692356661D+04
coeff(2447)=-0.658988373807D+04
coeff(2448)=0.782225031757D+04
coeff(2449)=-0.858493488885D+04
coeff(2450)=0.623728389068D+04
coeff(2451)=0.695554630454D+04
coeff(2452)=-0.313856078833D+04
coeff(2453)=0.979229826615D+03
coeff(2454)=0.505715840216D+04
coeff(2455)=-0.517546882346D+04
coeff(2456)=-0.616944196297D+03
coeff(2457)=-0.207779058938D+04
coeff(2458)=0.329022601175D+04
coeff(2459)=0.101129278635D+05
coeff(2460)=-0.319419751300D+03
coeff(2461)=0.112451655003D+05
coeff(2462)=-0.806874765046D+04
coeff(2463)=-0.325783332425D+04
coeff(2464)=-0.131109728884D+05
coeff(2465)=-0.747300769436D+04
coeff(2466)=-0.557219683383D+02
coeff(2467)=-0.120050731600D+04
coeff(2468)=0.624156967015D+04
coeff(2469)=-0.200970152921D+04
coeff(2470)=-0.231897145605D+04
coeff(2471)=0.799080764311D+02
coeff(2472)=0.477525598399D+04
coeff(2473)=-0.613978344127D+04
coeff(2474)=0.986628867534D+03
coeff(2475)=0.587959830950D+02
coeff(2476)=0.127352047031D+05
coeff(2477)=-0.154604148617D+04
coeff(2478)=0.879692901533D+03
coeff(2479)=-0.628663134357D+04
coeff(2480)=0.167200190336D+04
coeff(2481)=-0.800025881395D+04
coeff(2482)=0.751201003045D+04
coeff(2483)=-0.603711392068D+04
coeff(2484)=0.350345027056D+04
coeff(2485)=-0.266263268914D+04
coeff(2486)=0.405698373005D+04
coeff(2487)=0.690742945896D+04
coeff(2488)=0.242648950729D+04
coeff(2489)=-0.111973373228D+05
coeff(2490)=0.569718759609D+04
coeff(2491)=0.656727473807D+04
coeff(2492)=0.599783608820D+03
coeff(2493)=0.219484158302D+04
coeff(2494)=-0.689561175900D+03
coeff(2495)=-0.250272111788D+04
coeff(2496)=0.439018591439D+03
coeff(2497)=0.343743196314D+03
coeff(2498)=-0.198822299239D+04
coeff(2499)=0.521341713563D+04
coeff(2500)=-0.208282464355D+04
coeff(2501)=-0.267137551800D+04
coeff(2502)=0.148792149546D+05
coeff(2503)=-0.481027047932D+04
coeff(2504)=0.822473255488D+04
coeff(2505)=-0.461851763955D+04
coeff(2506)=-0.679881822993D+04
coeff(2507)=0.498564133518D+04
coeff(2508)=-0.125890765704D+04
coeff(2509)=-0.498423823829D+04
coeff(2510)=0.117128036202D+05
coeff(2511)=0.321161910005D+04
coeff(2512)=-0.778869381721D+04
coeff(2513)=-0.137945866764D+04
coeff(2514)=-0.462843642188D+04
coeff(2515)=0.256889766561D+04
coeff(2516)=-0.142222938727D+04
coeff(2517)=0.127324670544D+04
coeff(2518)=0.264972986640D+04
coeff(2519)=-0.503023755967D+04
coeff(2520)=0.104496978865D+05
coeff(2521)=-0.462109463780D+04
coeff(2522)=0.878418606682D+04
coeff(2523)=-0.976886478575D+04
coeff(2524)=0.663019265476D+04
coeff(2525)=-0.250624773592D+04
coeff(2526)=-0.119925534370D+04
coeff(2527)=0.196570061701D+04
coeff(2528)=0.141354206133D+04
coeff(2529)=-0.309204034696D+03
coeff(2530)=-0.597983161195D+03
coeff(2531)=0.426640000426D+04
coeff(2532)=-0.379617017928D+04
coeff(2533)=0.924786642898D+04
coeff(2534)=-0.506795579163D+04
coeff(2535)=-0.152301638841D+04
coeff(2536)=0.321721707576D+04
coeff(2537)=-0.295246799109D+04
coeff(2538)=-0.129823280862D+05
coeff(2539)=0.175456286899D+04
coeff(2540)=0.495383748823D+04
coeff(2541)=-0.155178111018D+04
coeff(2542)=-0.468773983439D+04
coeff(2543)=-0.946230214582D+04
coeff(2544)=-0.671887203876D+04
coeff(2545)=0.550226476533D+04
coeff(2546)=-0.227400040603D+03
coeff(2547)=0.236657886290D+04
coeff(2548)=-0.261343567276D+04
coeff(2549)=-0.487951378398D+04
coeff(2550)=0.198473394455D+04
coeff(2551)=0.188803416218D+04
coeff(2552)=-0.451267159500D+04
coeff(2553)=0.226150364986D+04
coeff(2554)=-0.311825623798D+04
coeff(2555)=-0.490315471020D+04
coeff(2556)=0.990583322562D+04
coeff(2557)=0.352359684934D+04
coeff(2558)=0.367165501706D+04
coeff(2559)=0.120446323638D+05
coeff(2560)=0.242233780973D+04
coeff(2561)=-0.371363808878D+04
coeff(2562)=-0.683580653626D+04
coeff(2563)=0.513484312670D+03
coeff(2564)=-0.159519055166D+04
coeff(2565)=-0.864573460377D+03
coeff(2566)=0.431124039201D+04
coeff(2567)=-0.754990096551D+04
coeff(2568)=0.307302557544D+03
coeff(2569)=0.790368199050D+04
coeff(2570)=0.638636287987D+04
coeff(2571)=-0.639570301612D+04
coeff(2572)=0.199407498973D+04
coeff(2573)=-0.328245161860D+04
coeff(2574)=0.928282637138D+04
coeff(2575)=0.360223888355D+04
coeff(2576)=-0.571864435631D+03
coeff(2577)=-0.345246443949D+04
coeff(2578)=0.133745112348D+03
coeff(2579)=0.782770559830D+03
coeff(2580)=-0.104033828782D+04
coeff(2581)=0.382414268427D+04
coeff(2582)=-0.320520965865D+04
coeff(2583)=0.628743985891D+04
coeff(2584)=-0.606865351245D+03
coeff(2585)=-0.160459229083D+05
coeff(2586)=-0.402139273924D+04
coeff(2587)=0.170412019742D+04
coeff(2588)=-0.167548891146D+04
coeff(2589)=0.295211220060D+04
coeff(2590)=-0.711172160825D+04
coeff(2591)=0.252377787916D+04
coeff(2592)=0.505377384533D+04
coeff(2593)=-0.911031629160D+04
coeff(2594)=0.109581598294D+04
coeff(2595)=0.104820020831D+05
coeff(2596)=-0.257568736504D+04
coeff(2597)=-0.756833784511D+03
coeff(2598)=0.574990561412D+03
coeff(2599)=0.271301519857D+04
coeff(2600)=-0.193146591011D+04
coeff(2601)=0.723223457048D+04
coeff(2602)=-0.345654251659D+04
coeff(2603)=0.759767127819D+03
coeff(2604)=0.837448752940D+03
coeff(2605)=0.492441758664D+04
coeff(2606)=0.100069971485D+05
coeff(2607)=-0.375455621478D+04
coeff(2608)=-0.750225277064D+04
coeff(2609)=0.178603244244D+04
coeff(2610)=-0.492309935307D+04
coeff(2611)=0.279168511371D+04
coeff(2612)=-0.688066025204D+04
coeff(2613)=0.742001944952D+04
coeff(2614)=0.459637291676D+04
coeff(2615)=0.968838214708D+03
coeff(2616)=0.638323949600D+03
coeff(2617)=0.659087948854D+04
coeff(2618)=0.166869022533D+04
coeff(2619)=-0.237205806744D+03
coeff(2620)=-0.303226361894D+04
coeff(2621)=-0.718738284558D+04
coeff(2622)=0.999672364676D+04
coeff(2623)=0.324978876538D+04
coeff(2624)=0.376783539084D+04
coeff(2625)=0.274966091146D+04
coeff(2626)=0.112509765305D+04
coeff(2627)=-0.285581749030D+04
coeff(2628)=-0.506735802031D+04
coeff(2629)=0.154072800410D+05
coeff(2630)=-0.197260755468D+04
coeff(2631)=-0.387460936216D+04
coeff(2632)=0.257930179094D+04
coeff(2633)=-0.286559519074D+04
coeff(2634)=0.307009710200D+04
coeff(2635)=0.145907296089D+05
coeff(2636)=0.368016010200D+04
coeff(2637)=0.851306187212D+04
coeff(2638)=0.383596243019D+04
coeff(2639)=0.335571301258D+04
coeff(2640)=-0.223807280433D+04
coeff(2641)=0.820693565714D+04
coeff(2642)=-0.978192595115D+04
coeff(2643)=0.177740908136D+04
coeff(2644)=0.364348123161D+04
coeff(2645)=-0.622777902863D+04
coeff(2646)=-0.174413608379D+02
coeff(2647)=-0.627888887497D+04
coeff(2648)=-0.161204086537D+04
coeff(2649)=0.337318230276D+04
coeff(2650)=0.842699604150D+04
coeff(2651)=-0.481287330029D+04
coeff(2652)=-0.288567938052D+04
coeff(2653)=0.287459381221D+04
coeff(2654)=-0.545307981804D+04
coeff(2655)=0.533177310497D+04
coeff(2656)=0.288552072913D+04
coeff(2657)=0.324048872373D+04
coeff(2658)=-0.618065803667D+04
coeff(2659)=-0.414528668943D+04
coeff(2660)=-0.891201224749D+04
coeff(2661)=-0.508870106120D+04
coeff(2662)=0.448811049614D+04
coeff(2663)=-0.833612157964D+04
coeff(2664)=-0.414921383575D+04
coeff(2665)=-0.108451538018D+04
coeff(2666)=0.136030580369D+05
coeff(2667)=-0.187702781527D+04
coeff(2668)=-0.243940151661D+03
coeff(2669)=-0.261537642105D+03
coeff(2670)=0.803618487712D+03
coeff(2671)=0.187568625225D+04
coeff(2672)=0.577380133217D+04
coeff(2673)=0.763631490622D+04
coeff(2674)=0.351016601970D+04
coeff(2675)=-0.668131094128D+02
coeff(2676)=0.227762286439D+03
coeff(2677)=-0.445359022695D+04
coeff(2678)=-0.115107819924D+05
coeff(2679)=-0.923815178370D+04
coeff(2680)=0.735622994303D+04
coeff(2681)=-0.508321525370D+04
coeff(2682)=-0.934995282353D+04
coeff(2683)=-0.791426230952D+04
coeff(2684)=-0.173929056258D+03
coeff(2685)=-0.764423379662D+04
coeff(2686)=-0.753920544556D+04
coeff(2687)=0.356092413232D+04
coeff(2688)=-0.135367775317D+04
coeff(2689)=-0.555105846941D+04
coeff(2690)=-0.316432100033D+04
coeff(2691)=-0.228289120448D+04
coeff(2692)=-0.381311543058D+04
coeff(2693)=0.430375898313D+04
coeff(2694)=-0.102948948085D+05
coeff(2695)=0.608406458518D+04
coeff(2696)=-0.696117398084D+04
coeff(2697)=-0.547126869688D+04
coeff(2698)=-0.563662784213D+04
coeff(2699)=0.256541357478D+04
coeff(2700)=0.108050695092D+04
coeff(2701)=0.261770526778D+04
coeff(2702)=-0.420317856353D+04
coeff(2703)=-0.301551082629D+04
coeff(2704)=-0.444491775495D+04
coeff(2705)=0.913026471741D+04
coeff(2706)=-0.325945058377D+04
coeff(2707)=-0.716011462109D+03
coeff(2708)=0.565818861289D+04
coeff(2709)=0.202666687986D+04
coeff(2710)=0.297621062994D+04
coeff(2711)=0.808021673352D+03
coeff(2712)=-0.127380045931D+05
coeff(2713)=-0.189952161154D+04
coeff(2714)=-0.768530038679D+04
coeff(2715)=-0.160276455890D+05
coeff(2716)=0.114701648895D+05
coeff(2717)=-0.454394960926D+04
coeff(2718)=-0.385410333397D+02
coeff(2719)=0.701674363953D+03
coeff(2720)=-0.209914183063D+04
coeff(2721)=0.329754342226D+04
coeff(2722)=0.362869465454D+04
coeff(2723)=-0.170697701533D+03
coeff(2724)=-0.237314007930D+04
coeff(2725)=0.491775410378D+04
coeff(2726)=0.112693741834D+05
coeff(2727)=-0.871215356689D+04
coeff(2728)=0.222973285776D+04
coeff(2729)=-0.443864073155D+04
coeff(2730)=-0.584647592529D+03
coeff(2731)=-0.333042110539D+04
coeff(2732)=-0.402029941073D+04
coeff(2733)=-0.539392898597D+04
coeff(2734)=-0.444996976150D+04
coeff(2735)=0.691903698832D+04
coeff(2736)=-0.581004642397D+04
coeff(2737)=0.102718780733D+05
coeff(2738)=0.949871269600D+04
coeff(2739)=0.109169904602D+05
coeff(2740)=0.239252710793D+03
coeff(2741)=-0.644196332485D+04
coeff(2742)=-0.287649529625D+04
coeff(2743)=0.219463490962D+04
coeff(2744)=0.164060768269D+04
coeff(2745)=-0.945266709634D+03
coeff(2746)=0.139518405889D+04
coeff(2747)=-0.133576308163D+04
coeff(2748)=-0.535188205426D+04
coeff(2749)=0.151721608734D+04
coeff(2750)=-0.349231124670D+04
coeff(2751)=0.965918151096D+04
coeff(2752)=-0.665173856572D+04
coeff(2753)=-0.266838362559D+04
coeff(2754)=-0.146491347052D+04
coeff(2755)=0.867508821750D+04
coeff(2756)=0.571493788708D+04
coeff(2757)=0.112153594458D+03
coeff(2758)=-0.519622748190D+04
coeff(2759)=0.248818314494D+04
coeff(2760)=0.772235918654D+04
coeff(2761)=-0.307399912432D+04
coeff(2762)=-0.341714111403D+04
coeff(2763)=-0.995092920642D+03
coeff(2764)=-0.112124565330D+05
coeff(2765)=-0.266252999684D+04
coeff(2766)=-0.490960925594D+04
coeff(2767)=-0.456425905617D+04
coeff(2768)=0.330844850505D+04
coeff(2769)=-0.525761915365D+04
coeff(2770)=0.800597895015D+04
coeff(2771)=0.142203879216D+05
coeff(2772)=0.540166716463D+04
coeff(2773)=0.516479358394D+04
coeff(2774)=0.119055487806D+04
coeff(2775)=-0.797399528936D+04
coeff(2776)=0.628249398030D+04
coeff(2777)=-0.140046606304D+04
coeff(2778)=0.550279383587D+03
coeff(2779)=0.429856963489D+04
coeff(2780)=0.503163310760D+03
coeff(2781)=0.507795504335D+03
coeff(2782)=0.125723951697D+05
coeff(2783)=-0.164417110812D+04
coeff(2784)=0.214439803000D+04
coeff(2785)=0.354058695452D+04
coeff(2786)=-0.131263472069D+05
coeff(2787)=0.339882660509D+02
coeff(2788)=0.891799314064D+04
coeff(2789)=0.663898538719D+04
coeff(2790)=0.490650406623D+04
coeff(2791)=0.200187481962D+04
coeff(2792)=0.536017295486D+04
coeff(2793)=-0.136506225999D+05
coeff(2794)=-0.610368236337D+04
coeff(2795)=0.336225869636D+04
coeff(2796)=0.529861519499D+04
coeff(2797)=-0.658122658218D+03
coeff(2798)=0.209921987438D+04
coeff(2799)=-0.753359747150D+04
coeff(2800)=-0.387251140826D+04
coeff(2801)=0.855342972594D+04
coeff(2802)=0.851721580001D+04
coeff(2803)=-0.719994047079D+04
coeff(2804)=0.140443149571D+04
coeff(2805)=0.562297992148D+04
coeff(2806)=-0.660937910290D+04
coeff(2807)=0.133424195634D+04
coeff(2808)=-0.971543314436D+04
coeff(2809)=0.956050284063D+04
coeff(2810)=0.490471125012D+04
coeff(2811)=-0.485463704025D+04
coeff(2812)=0.421138920669D+04
coeff(2813)=-0.462965062708D+04
coeff(2814)=0.600258838324D+04
coeff(2815)=-0.406595435891D+04
coeff(2816)=-0.217741238126D+04
coeff(2817)=-0.183614035364D+04
coeff(2818)=-0.405603422703D+03
coeff(2819)=-0.446893466336D+04
coeff(2820)=-0.573749539439D+04
coeff(2821)=0.162143576870D+05
coeff(2822)=-0.118447853539D+05
coeff(2823)=-0.281977576893D+04
coeff(2824)=-0.416107216562D+04
coeff(2825)=-0.232851081544D+04
coeff(2826)=0.140882821321D+05
coeff(2827)=0.110036998024D+05
coeff(2828)=0.104605892911D+05
coeff(2829)=-0.868082446451D+03
coeff(2830)=0.610732273849D+04
coeff(2831)=0.122776167787D+04
coeff(2832)=-0.515641825097D+04
coeff(2833)=-0.101210593212D+05
coeff(2834)=0.269910041475D+04
coeff(2835)=0.365401431508D+04
coeff(2836)=0.118811737689D+05
coeff(2837)=0.144490081507D+04
coeff(2838)=-0.173793700578D+04
coeff(2839)=-0.259060232757D+04
coeff(2840)=0.410972358445D+04
coeff(2841)=0.152225209608D+04
coeff(2842)=0.965136001993D+04
coeff(2843)=-0.804047261909D+03
coeff(2844)=0.169842531271D+05
coeff(2845)=-0.564044884227D+04
coeff(2846)=-0.758686477506D+03
coeff(2847)=-0.111619823657D+05
coeff(2848)=-0.808083945966D+03
coeff(2849)=-0.353651277464D+04
coeff(2850)=0.212582205013D+04
coeff(2851)=-0.694769900709D+04
coeff(2852)=-0.402825890148D+04
coeff(2853)=-0.157656855567D+04
coeff(2854)=-0.174082614868D+04
coeff(2855)=0.185597784172D+03
coeff(2856)=0.641186983495D+02
coeff(2857)=0.216031048655D+04
coeff(2858)=-0.367228462704D+03
coeff(2859)=-0.698181454172D+04
coeff(2860)=-0.402038509637D+04
coeff(2861)=0.635764948831D+04
coeff(2862)=-0.688457769229D+03
coeff(2863)=-0.679397320802D+04
coeff(2864)=0.209543441304D+03
coeff(2865)=-0.307630296893D+03
coeff(2866)=0.401549176962D+04
coeff(2867)=-0.265274199886D+03
coeff(2868)=-0.241443517942D+04
coeff(2869)=-0.103386428489D+05
coeff(2870)=-0.134599690878D+05
coeff(2871)=-0.653411817968D+04
coeff(2872)=0.339814098490D+04
coeff(2873)=0.705217858584D+04
coeff(2874)=0.153167175235D+04
coeff(2875)=-0.293538669243D+04
coeff(2876)=0.408185257938D+04
coeff(2877)=-0.894429590079D+03
coeff(2878)=-0.100797186151D+04
coeff(2879)=0.560515967928D+04
coeff(2880)=-0.525909343972D+04
coeff(2881)=-0.895929012913D+04
coeff(2882)=-0.856069044334D+04
coeff(2883)=-0.293035628958D+04
coeff(2884)=0.248884299494D+04
coeff(2885)=0.341109844477D+04
coeff(2886)=0.212396213004D+04
coeff(2887)=0.976765598821D+03
coeff(2888)=0.346947131732D+04
coeff(2889)=0.114503391872D+05
coeff(2890)=-0.169189934634D+04
coeff(2891)=-0.858902039533D+04
coeff(2892)=-0.335294684210D+04
coeff(2893)=-0.117799960145D+04
coeff(2894)=-0.271260684734D+03
coeff(2895)=0.786478223036D+03
coeff(2896)=0.479671366223D+04
coeff(2897)=0.278828916986D+04
coeff(2898)=0.554239081521D+04
coeff(2899)=0.904609376648D+03
coeff(2900)=0.360698096619D+04
coeff(2901)=-0.261644226231D+04
coeff(2902)=0.634817938550D+04
coeff(2903)=-0.196181541688D+03
coeff(2904)=0.558937011003D+04
coeff(2905)=0.365339376217D+04
coeff(2906)=0.622025730897D+03
coeff(2907)=0.320227311492D+03
coeff(2908)=0.521773791672D+04
coeff(2909)=-0.781787211592D+03
coeff(2910)=0.137017511505D+04
coeff(2911)=0.407419149972D+04
coeff(2912)=0.492142805140D+03
coeff(2913)=-0.141603824872D+05
coeff(2914)=0.567352888517D+04
coeff(2915)=0.904582668641D+04
coeff(2916)=-0.875921929400D+04
coeff(2917)=0.519463858314D+04
coeff(2918)=-0.259851210980D+04
coeff(2919)=0.760161688226D+03
coeff(2920)=-0.615740488490D+03
coeff(2921)=-0.165198838366D+04
coeff(2922)=0.319520210741D+03
coeff(2923)=-0.671372444928D+04
coeff(2924)=-0.951461813318D+03
coeff(2925)=0.398158783263D+04
coeff(2926)=-0.273952308082D+04
coeff(2927)=0.187619569205D+04
coeff(2928)=-0.552698532582D+04
coeff(2929)=-0.282386746269D+04
coeff(2930)=0.352227965219D+04
coeff(2931)=-0.215314689783D+04
coeff(2932)=-0.668980016924D+03
coeff(2933)=-0.215399438657D+03
coeff(2934)=0.156123034911D+04
coeff(2935)=-0.124977369837D+04
coeff(2936)=-0.311119566607D+04
coeff(2937)=0.539309974568D+04
coeff(2938)=-0.293072114342D+04
coeff(2939)=0.148041366150D+04
coeff(2940)=0.119507107659D+04
coeff(2941)=0.711930911678D+04
coeff(2942)=-0.575074553917D+04
coeff(2943)=-0.688530721090D+04
coeff(2944)=0.500513277248D+04
coeff(2945)=0.511188461417D+04
coeff(2946)=-0.135618493659D+04
coeff(2947)=-0.334033829885D+03
coeff(2948)=0.139442208575D+04
coeff(2949)=0.555721103232D+03
coeff(2950)=0.260178112016D+03
coeff(2951)=0.164092685134D+04
coeff(2952)=-0.563870085159D+02
coeff(2953)=0.349904970964D+04
coeff(2954)=0.313703324047D+04
coeff(2955)=-0.776736182639D+04
coeff(2956)=-0.327553802687D+03
coeff(2957)=0.648654139233D+04
coeff(2958)=-0.224501728600D+04
coeff(2959)=-0.541744827237D+04
coeff(2960)=-0.478905496321D+04
coeff(2961)=0.290255696183D+04
coeff(2962)=0.171640978749D+03
coeff(2963)=-0.165120895138D+04
coeff(2964)=0.274101293418D+04
coeff(2965)=-0.148192868409D+04
coeff(2966)=0.803775032658D+03
coeff(2967)=-0.115969022929D+05
coeff(2968)=-0.249750136414D+04
coeff(2969)=-0.178200452910D+04
coeff(2970)=0.425646031483D+04
coeff(2971)=-0.381033070607D+04
coeff(2972)=-0.974944559301D+02
coeff(2973)=0.587185570789D+04
coeff(2974)=-0.376082135851D+04
coeff(2975)=-0.429379648625D+03
coeff(2976)=0.717323949686D+04
coeff(2977)=-0.280699408010D+04
coeff(2978)=-0.286149402814D+03
coeff(2979)=0.404096927440D+04
coeff(2980)=-0.474667006459D+04
coeff(2981)=0.468015498679D+04
coeff(2982)=0.568989336897D+04
coeff(2983)=-0.334606451561D+04
coeff(2984)=0.923638150573D+04
coeff(2985)=0.645182659636D+04
coeff(2986)=0.310818942925D+04
coeff(2987)=0.233225538644D+04
coeff(2988)=-0.373516077781D+04
coeff(2989)=0.680561640275D+04
coeff(2990)=-0.315928262192D+04
coeff(2991)=-0.206064991676D+04
coeff(2992)=0.116747610721D+04
coeff(2993)=-0.169237032766D+04
coeff(2994)=-0.557232777659D+04
coeff(2995)=0.553266174159D+04
coeff(2996)=-0.586432181138D+04
coeff(2997)=0.393015616244D+04
coeff(2998)=0.984850954799D+03
coeff(2999)=-0.513111310098D+04
coeff(3000)=0.848852282333D+04
coeff(3001)=0.238143827175D+04
coeff(3002)=-0.374150590352D+04
coeff(3003)=-0.114972070316D+05
coeff(3004)=0.332955397006D+03
coeff(3005)=-0.601535653135D+04
coeff(3006)=-0.451113575253D+04
coeff(3007)=0.349269451060D+04
coeff(3008)=0.334516836208D+04
coeff(3009)=-0.705225115955D+03
coeff(3010)=0.191360399606D+04
coeff(3011)=-0.294002664057D+03
coeff(3012)=0.890265984020D+03
coeff(3013)=0.717274912531D+04
coeff(3014)=-0.513586611859D+04
coeff(3015)=0.689004316081D+04
coeff(3016)=-0.471759653228D+04
coeff(3017)=-0.300734700638D+04
coeff(3018)=-0.464754555446D+03
coeff(3019)=0.374631318959D+04
coeff(3020)=0.568258200909D+04
coeff(3021)=0.196184886603D+04
coeff(3022)=0.169034480368D+04
coeff(3023)=0.101126263644D+05
coeff(3024)=-0.722990168962D+02
coeff(3025)=0.383736836088D+04
coeff(3026)=-0.162998096490D+04
coeff(3027)=0.438989498772D+04
coeff(3028)=-0.971059709897D+04
coeff(3029)=-0.471394037434D+04
coeff(3030)=0.149653042499D+04
coeff(3031)=0.551860275077D+02
coeff(3032)=-0.753375029871D+04
coeff(3033)=0.305885206786D+04
coeff(3034)=-0.417631895861D+04
coeff(3035)=-0.655247282959D+03
coeff(3036)=-0.119762346100D+04
coeff(3037)=-0.252495367013D+04
coeff(3038)=-0.913836146132D+04
coeff(3039)=0.605832508668D+03
coeff(3040)=-0.296858490377D+04
coeff(3041)=0.223016906343D+03
coeff(3042)=-0.361004368083D+04
coeff(3043)=0.174381660202D+04
coeff(3044)=0.104224913650D+05
coeff(3045)=-0.125711995122D+04
coeff(3046)=-0.808755196175D+04
coeff(3047)=0.163530505692D+04
coeff(3048)=0.252412587721D+04
coeff(3049)=0.131389254890D+04
coeff(3050)=0.335992537393D+04
coeff(3051)=0.316123378276D+04
coeff(3052)=-0.152763655114D+04
coeff(3053)=0.420416018704D+04
coeff(3054)=0.582183718574D+04
coeff(3055)=0.420677872160D+04
coeff(3056)=0.166689028613D+04
coeff(3057)=-0.551745218857D+04
coeff(3058)=0.329283838135D+03
coeff(3059)=-0.335764278340D+03
coeff(3060)=-0.714636418404D+03
coeff(3061)=-0.177059458798D+04
coeff(3062)=-0.506911661227D+04
coeff(3063)=-0.382964491144D+03
coeff(3064)=-0.606620534751D+04
coeff(3065)=0.237362202899D+04
coeff(3066)=-0.200395081191D+04
coeff(3067)=-0.100401603384D+04
coeff(3068)=-0.100545548774D+04
coeff(3069)=0.380515603086D+04
coeff(3070)=0.762575524766D+03
coeff(3071)=-0.341312861784D+04
coeff(3072)=0.342299577716D+04
coeff(3073)=0.550185197452D+03
coeff(3074)=0.974246061586D+03
coeff(3075)=0.340299040498D+03
  return
end if

entry u13_dc_12(xyz,v)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)

call ev_pol_con_12(x,q)
v=dot_product(coeff,q)/627.509474

end  subroutine


!===========================================================================
! *def u23_order10
! subtoutine of computing u23
!===========================================================================
subroutine u23_order10
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp) :: xyz(4,3)
real(dp) :: v, r12, r13, r14, r23, r24, r34
real(dp) :: s
real(dp) :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=1300
real(dp) :: coeff(ncoeff), q(ncoeff)
real(dp) :: x(6)
integer ::  ncall
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer :: i, j
real(dp) :: rrel(3,3),cross(3)
real(dp) :: volume

if(ncall.eq.1) then
a1=1.323343
a2=1.09
b1=1.823343
b2=1.633190
ra1=1.113545
ra2=1.001178
rb1=1.113545
rb2=1.001178
coeff(1)=-0.367835424500D+03
coeff(2)=-0.345009613875D+02
coeff(3)=-0.462704697963D+02
coeff(4)=-0.257814027132D+03
coeff(5)=0.946252414190D+03
coeff(6)=-0.575616726211D+03
coeff(7)=0.107541261550D+04
coeff(8)=-0.342927934077D+04
coeff(9)=-0.116299721266D+04
coeff(10)=0.211962361903D+04
coeff(11)=0.153034426652D+04
coeff(12)=0.103589153519D+04
coeff(13)=0.828106949415D+03
coeff(14)=0.419251628265D+03
coeff(15)=-0.836537222863D+03
coeff(16)=-0.431897961889D+04
coeff(17)=-0.140540899389D+02
coeff(18)=0.449100064823D+04
coeff(19)=0.278519709531D+04
coeff(20)=-0.658597887101D+04
coeff(21)=0.699050520516D+03
coeff(22)=-0.164807813975D+04
coeff(23)=-0.140738486070D+04
coeff(24)=-0.346478740365D+04
coeff(25)=0.235735603792D+04
coeff(26)=0.411965512626D+04
coeff(27)=0.820611235955D+03
coeff(28)=-0.385908245676D+04
coeff(29)=-0.473016769361D+04
coeff(30)=0.381219312953D+04
coeff(31)=0.433328754209D+04
coeff(32)=0.372882884263D+04
coeff(33)=0.261013202025D+04
coeff(34)=0.393899344143D+04
coeff(35)=-0.113569983057D+05
coeff(36)=-0.282463707363D+04
coeff(37)=-0.160715147111D+04
coeff(38)=0.286536038561D+04
coeff(39)=-0.404423909654D+04
coeff(40)=-0.649715515940D+04
coeff(41)=0.939321624777D+04
coeff(42)=0.502323356360D+04
coeff(43)=-0.128203546195D+04
coeff(44)=-0.376099802131D+03
coeff(45)=0.474984074250D+04
coeff(46)=0.440350761738D+04
coeff(47)=-0.468418243432D+04
coeff(48)=-0.341079999403D+04
coeff(49)=0.238471554165D+04
coeff(50)=0.752383966347D+04
coeff(51)=0.192045488199D+04
coeff(52)=0.108311816199D+04
coeff(53)=-0.143904374334D+04
coeff(54)=0.121553301268D+04
coeff(55)=-0.107680056311D+05
coeff(56)=0.125593839622D+05
coeff(57)=-0.251996129077D+04
coeff(58)=-0.183904970884D+04
coeff(59)=0.238388495782D+04
coeff(60)=-0.738807873666D+03
coeff(61)=-0.516012112821D+04
coeff(62)=0.462101285280D+04
coeff(63)=-0.321450777912D+04
coeff(64)=0.173968689564D+04
coeff(65)=-0.337210232279D+04
coeff(66)=0.877094977033D+04
coeff(67)=-0.554325748354D+04
coeff(68)=0.161724600127D+04
coeff(69)=-0.580968391425D+04
coeff(70)=0.665541231310D+04
coeff(71)=0.108263227956D+05
coeff(72)=0.454197796856D+04
coeff(73)=0.679734620048D+04
coeff(74)=-0.218868002192D+04
coeff(75)=-0.454009571933D+04
coeff(76)=-0.544550570280D+04
coeff(77)=0.222219940610D+04
coeff(78)=-0.393839910795D+04
coeff(79)=-0.845854847286D+04
coeff(80)=0.226751123643D+04
coeff(81)=-0.162675144178D+04
coeff(82)=-0.202476041765D+04
coeff(83)=0.664614290680D+04
coeff(84)=0.266540417775D+04
coeff(85)=0.275858273058D+04
coeff(86)=-0.916330727797D+03
coeff(87)=-0.104846769522D+04
coeff(88)=0.412981963827D+04
coeff(89)=-0.297139851012D+04
coeff(90)=0.748726341796D+04
coeff(91)=0.579021241779D+03
coeff(92)=0.782645432997D+04
coeff(93)=-0.109465976363D+05
coeff(94)=0.928671941038D+04
coeff(95)=-0.466667596663D+04
coeff(96)=0.133132901304D+04
coeff(97)=0.881666806763D+02
coeff(98)=-0.224008647080D+04
coeff(99)=-0.399205407155D+02
coeff(100)=0.667502239116D+04
coeff(101)=-0.542295731393D+04
coeff(102)=-0.118221709938D+05
coeff(103)=-0.890289643931D+03
coeff(104)=-0.327451117343D+02
coeff(105)=-0.109995874857D+04
coeff(106)=0.107986170238D+04
coeff(107)=0.100324745229D+04
coeff(108)=-0.138647513710D+04
coeff(109)=0.329635913623D+04
coeff(110)=0.670671118252D+04
coeff(111)=-0.551561957094D+02
coeff(112)=-0.282348630248D+04
coeff(113)=-0.130763286775D+04
coeff(114)=-0.647714742330D+01
coeff(115)=-0.141965234378D+05
coeff(116)=-0.701759518863D+04
coeff(117)=-0.195777320933D+04
coeff(118)=0.958385393117D+03
coeff(119)=-0.264326312752D+04
coeff(120)=-0.950233173026D+04
coeff(121)=0.705855727994D+04
coeff(122)=-0.623652655890D+04
coeff(123)=0.135484091136D+04
coeff(124)=0.744886705077D+03
coeff(125)=-0.118100284881D+05
coeff(126)=0.161166991979D+05
coeff(127)=-0.773330616301D+04
coeff(128)=0.589700169323D+04
coeff(129)=0.657938208285D+04
coeff(130)=0.518063867482D+03
coeff(131)=-0.781054689301D+03
coeff(132)=0.312678743874D+04
coeff(133)=0.124006202857D+04
coeff(134)=-0.638036122473D+04
coeff(135)=0.472961572395D+04
coeff(136)=-0.421710158595D+03
coeff(137)=0.105528682416D+05
coeff(138)=0.246269767643D+04
coeff(139)=-0.756736764690D+04
coeff(140)=0.273504342772D+04
coeff(141)=0.496370553796D+04
coeff(142)=-0.462502519609D+04
coeff(143)=0.283032404220D+04
coeff(144)=0.125532910623D+04
coeff(145)=-0.360581880056D+04
coeff(146)=-0.947072598409D+04
coeff(147)=-0.541756927948D+04
coeff(148)=0.711898208422D+04
coeff(149)=-0.232696841455D+04
coeff(150)=0.836533032633D+04
coeff(151)=0.111967998615D+02
coeff(152)=0.496805631746D+04
coeff(153)=-0.636048476102D+04
coeff(154)=0.172055547944D+04
coeff(155)=-0.105714625705D+04
coeff(156)=0.671542301917D+03
coeff(157)=-0.330882405524D+04
coeff(158)=-0.166076720293D+04
coeff(159)=0.628670636052D+04
coeff(160)=0.265934908583D+04
coeff(161)=0.482177566657D+03
coeff(162)=-0.274839328001D+04
coeff(163)=-0.834748817821D+03
coeff(164)=0.486398135384D+04
coeff(165)=-0.608697288597D+04
coeff(166)=0.758405515895D+04
coeff(167)=-0.198625652855D+03
coeff(168)=-0.242723305190D+04
coeff(169)=-0.432100503043D+04
coeff(170)=-0.293232197620D+03
coeff(171)=-0.254320839597D+04
coeff(172)=-0.330411138137D+04
coeff(173)=-0.361724748470D+04
coeff(174)=-0.210659871609D+04
coeff(175)=0.546680635466D+04
coeff(176)=0.165147545386D+05
coeff(177)=0.127390508919D+04
coeff(178)=-0.821804096790D+04
coeff(179)=0.114930447041D+04
coeff(180)=0.687544315269D+04
coeff(181)=0.515778555850D+04
coeff(182)=-0.147965639565D+04
coeff(183)=0.540850040950D+04
coeff(184)=-0.445180889629D+04
coeff(185)=0.114204652896D+05
coeff(186)=-0.198814661644D+04
coeff(187)=0.212638783175D+04
coeff(188)=-0.124833465296D+03
coeff(189)=-0.370467653948D+04
coeff(190)=0.132835848653D+05
coeff(191)=0.862919856597D+04
coeff(192)=-0.209573836340D+05
coeff(193)=-0.115366604924D+05
coeff(194)=0.789948817648D+04
coeff(195)=-0.119538246304D+05
coeff(196)=-0.633754337685D+04
coeff(197)=-0.103650234002D+04
coeff(198)=-0.113243816680D+05
coeff(199)=-0.581364132495D+04
coeff(200)=0.811017192348D+04
coeff(201)=-0.283817069037D+04
coeff(202)=0.118476164078D+04
coeff(203)=-0.429015815782D+04
coeff(204)=0.224479331357D+04
coeff(205)=0.829405231639D+04
coeff(206)=0.544538500283D+04
coeff(207)=-0.602321263279D+04
coeff(208)=-0.327825276556D+03
coeff(209)=0.619062316534D+04
coeff(210)=-0.110928274616D+04
coeff(211)=0.381511970295D+04
coeff(212)=-0.373447290333D+04
coeff(213)=0.136684856964D+04
coeff(214)=0.511821653262D+04
coeff(215)=0.196396104857D+04
coeff(216)=0.175029099358D+04
coeff(217)=-0.680500538919D+03
coeff(218)=0.119240062097D+04
coeff(219)=0.370284816179D+03
coeff(220)=-0.344105978136D+04
coeff(221)=-0.129537270071D+04
coeff(222)=-0.514585232221D+04
coeff(223)=-0.119763438152D+04
coeff(224)=0.541369500945D+03
coeff(225)=-0.605262300083D+04
coeff(226)=-0.517393918206D+04
coeff(227)=-0.418279136769D+04
coeff(228)=-0.957397967350D+04
coeff(229)=-0.147217548360D+04
coeff(230)=0.514400991805D+04
coeff(231)=0.183286032713D+04
coeff(232)=-0.480363257132D+04
coeff(233)=0.691814088532D+04
coeff(234)=0.735656185985D+04
coeff(235)=0.819633023420D+04
coeff(236)=0.116081368060D+04
coeff(237)=-0.248855218267D+04
coeff(238)=-0.293599100953D+04
coeff(239)=-0.468310318496D+04
coeff(240)=-0.479079059019D+04
coeff(241)=-0.451225353179D+04
coeff(242)=-0.427150911597D+04
coeff(243)=0.685535798577D+04
coeff(244)=0.329917085958D+04
coeff(245)=0.493208722835D+04
coeff(246)=0.970475894071D+04
coeff(247)=0.259670944404D+04
coeff(248)=-0.524725514137D+04
coeff(249)=0.745834398105D+04
coeff(250)=0.874175129647D+04
coeff(251)=0.382426709208D+04
coeff(252)=-0.153137318074D+05
coeff(253)=-0.820907617520D+03
coeff(254)=-0.148148997881D+05
coeff(255)=-0.241152271829D+03
coeff(256)=0.222434529291D+04
coeff(257)=-0.156676187969D+04
coeff(258)=0.104900284587D+04
coeff(259)=0.698199779897D+04
coeff(260)=0.833356477690D+04
coeff(261)=0.596002972241D+04
coeff(262)=0.238128092367D+03
coeff(263)=-0.554530813740D+04
coeff(264)=-0.101384830110D+05
coeff(265)=0.208990929710D+04
coeff(266)=0.823852728551D+04
coeff(267)=0.923174784554D+04
coeff(268)=-0.184892841970D+04
coeff(269)=-0.960772441060D+03
coeff(270)=0.488615859707D+04
coeff(271)=-0.821390053318D+04
coeff(272)=0.574111416236D+03
coeff(273)=-0.538818853967D+04
coeff(274)=0.313439609455D+04
coeff(275)=0.449145965581D+04
coeff(276)=-0.109110539789D+05
coeff(277)=0.574261189575D+04
coeff(278)=0.133781890686D+04
coeff(279)=-0.136209055325D+05
coeff(280)=0.663939514979D+03
coeff(281)=0.287741720427D+04
coeff(282)=0.101688960572D+05
coeff(283)=-0.245413008752D+04
coeff(284)=-0.165452745584D+05
coeff(285)=-0.130515874728D+05
coeff(286)=0.619885116893D+03
coeff(287)=0.166661326240D+05
coeff(288)=-0.344461980992D+04
coeff(289)=0.102382964426D+04
coeff(290)=0.758112279170D+03
coeff(291)=-0.364243631154D+03
coeff(292)=0.361064519426D+04
coeff(293)=0.178616186551D+04
coeff(294)=0.216835505324D+04
coeff(295)=-0.516785165085D+03
coeff(296)=-0.671538332502D+04
coeff(297)=-0.848837848950D+04
coeff(298)=-0.158205059491D+05
coeff(299)=0.600652260191D+04
coeff(300)=-0.394262345527D+04
coeff(301)=-0.909525607161D+04
coeff(302)=0.575601276321D+04
coeff(303)=-0.105544231512D+04
coeff(304)=0.158632720587D+04
coeff(305)=0.563758494400D+04
coeff(306)=-0.492046521687D+04
coeff(307)=0.569056906056D+04
coeff(308)=-0.558058858412D+03
coeff(309)=-0.425493940543D+04
coeff(310)=-0.554434391982D+04
coeff(311)=-0.420562435377D+04
coeff(312)=-0.829196907842D+04
coeff(313)=-0.572137267381D+04
coeff(314)=-0.542389050975D+04
coeff(315)=0.837947499692D+03
coeff(316)=-0.816688275621D+04
coeff(317)=0.449166904450D+04
coeff(318)=-0.475521259861D+04
coeff(319)=-0.145842099714D+04
coeff(320)=-0.280639093768D+04
coeff(321)=0.611797468167D+04
coeff(322)=0.710963682096D+04
coeff(323)=-0.710028178902D+03
coeff(324)=0.627003601409D+03
coeff(325)=-0.276771841393D+04
coeff(326)=0.111113056094D+05
coeff(327)=0.125748167898D+04
coeff(328)=0.812477534759D+04
coeff(329)=-0.278821875158D+04
coeff(330)=0.128238137520D+02
coeff(331)=0.695569776319D+04
coeff(332)=-0.903654083864D+04
coeff(333)=-0.234316450571D+04
coeff(334)=-0.304758806709D+04
coeff(335)=0.672220753490D+04
coeff(336)=-0.706504758965D+04
coeff(337)=0.933877044741D+03
coeff(338)=0.637958521306D+04
coeff(339)=-0.264019578911D+04
coeff(340)=-0.127483329516D+04
coeff(341)=-0.332416724586D+04
coeff(342)=0.400445981066D+04
coeff(343)=0.311945252255D+04
coeff(344)=0.185538870080D+04
coeff(345)=0.474663810152D+04
coeff(346)=-0.820906711510D+03
coeff(347)=-0.872526964795D+04
coeff(348)=-0.684440987309D+04
coeff(349)=-0.125666574203D+05
coeff(350)=-0.903019367902D+03
coeff(351)=0.168013636752D+04
coeff(352)=0.375116831426D+04
coeff(353)=-0.470572854324D+04
coeff(354)=-0.211079073771D+04
coeff(355)=-0.130331398223D+05
coeff(356)=-0.137031186316D+04
coeff(357)=-0.888945974074D+03
coeff(358)=-0.569649851052D+04
coeff(359)=-0.293749508683D+04
coeff(360)=-0.844962712287D+04
coeff(361)=-0.179016889516D+05
coeff(362)=0.657108338380D+04
coeff(363)=0.294129092958D+04
coeff(364)=0.116517106569D+05
coeff(365)=0.269049406131D+04
coeff(366)=-0.159199737050D+05
coeff(367)=0.814964679961D+04
coeff(368)=0.101226260412D+04
coeff(369)=0.163640872706D+04
coeff(370)=-0.282683677188D+04
coeff(371)=0.634808529872D+04
coeff(372)=0.191083430734D+05
coeff(373)=-0.140543560454D+05
coeff(374)=-0.544054690430D+04
coeff(375)=0.578952585067D+04
coeff(376)=0.265417258672D+04
coeff(377)=0.281039764703D+04
coeff(378)=0.653880486083D+04
coeff(379)=0.985511700271D+02
coeff(380)=0.492157315788D+04
coeff(381)=-0.591533993286D+04
coeff(382)=0.395872365552D+04
coeff(383)=-0.128063916003D+05
coeff(384)=0.657223409307D+04
coeff(385)=0.747524974438D+04
coeff(386)=0.453505681501D+04
coeff(387)=0.111885724721D+05
coeff(388)=0.114156662380D+05
coeff(389)=0.779309558600D+04
coeff(390)=0.937273860471D+04
coeff(391)=-0.413375916246D+04
coeff(392)=0.397122713575D+03
coeff(393)=-0.222709331754D+04
coeff(394)=-0.777378798393D+04
coeff(395)=-0.930902728262D+04
coeff(396)=0.621625884086D+04
coeff(397)=-0.385260410311D+04
coeff(398)=0.152499117066D+04
coeff(399)=0.378683038537D+04
coeff(400)=0.142163336058D+03
coeff(401)=0.134184880805D+04
coeff(402)=-0.181216570851D+04
coeff(403)=0.290896365230D+03
coeff(404)=0.349297061233D+04
coeff(405)=0.391978656658D+04
coeff(406)=-0.745562412047D+04
coeff(407)=0.869356301685D+04
coeff(408)=0.725261491508D+04
coeff(409)=0.323196510043D+04
coeff(410)=0.202161376258D+04
coeff(411)=-0.441281603719D+04
coeff(412)=0.266887406498D+03
coeff(413)=0.525075007362D+04
coeff(414)=0.635584664345D+03
coeff(415)=-0.454321323793D+04
coeff(416)=0.173163910771D+04
coeff(417)=-0.136732328301D+04
coeff(418)=0.680856479237D+04
coeff(419)=0.150469616240D+04
coeff(420)=-0.734626291745D+04
coeff(421)=0.633626063676D+02
coeff(422)=0.123080593503D+05
coeff(423)=0.408449266416D+04
coeff(424)=0.130458468293D+05
coeff(425)=0.497257155672D+04
coeff(426)=0.736869360840D+04
coeff(427)=0.819683587443D+04
coeff(428)=0.168981906427D+03
coeff(429)=-0.482134204815D+03
coeff(430)=-0.816929457230D+03
coeff(431)=-0.263058064907D+04
coeff(432)=-0.191969760870D+04
coeff(433)=-0.234000057365D+04
coeff(434)=0.363326843531D+04
coeff(435)=0.409517884491D+04
coeff(436)=0.605630597188D+04
coeff(437)=0.133432212271D+05
coeff(438)=-0.627021853583D+04
coeff(439)=0.451199752065D+04
coeff(440)=0.712859545407D+03
coeff(441)=-0.847755181538D+04
coeff(442)=-0.370698880273D+04
coeff(443)=-0.134172023007D+05
coeff(444)=0.920861843822D+03
coeff(445)=-0.103206698690D+05
coeff(446)=0.296530307164D+04
coeff(447)=0.352097991202D+04
coeff(448)=0.444742736105D+04
coeff(449)=0.621001219513D+04
coeff(450)=-0.688169934970D+04
coeff(451)=0.804008375333D+04
coeff(452)=-0.812444607721D+04
coeff(453)=-0.275636174869D+04
coeff(454)=0.851818922709D+03
coeff(455)=-0.231453210240D+04
coeff(456)=-0.156198576697D+05
coeff(457)=0.935357790037D+04
coeff(458)=0.657355424533D+02
coeff(459)=-0.123934508791D+05
coeff(460)=0.673560382218D+04
coeff(461)=0.925741028087D+04
coeff(462)=0.182802028879D+04
coeff(463)=0.133378484726D+05
coeff(464)=-0.111192675107D+05
coeff(465)=-0.264188981328D+04
coeff(466)=-0.995853980731D+03
coeff(467)=-0.417452903272D+02
coeff(468)=0.475770751289D+04
coeff(469)=-0.850577659984D+04
coeff(470)=-0.310113378973D+04
coeff(471)=0.243007093487D+04
coeff(472)=0.431432276029D+04
coeff(473)=0.323183339715D+04
coeff(474)=-0.654850179019D+04
coeff(475)=0.634615831637D+04
coeff(476)=-0.901231753930D+03
coeff(477)=-0.133286182037D+04
coeff(478)=-0.127429960990D+05
coeff(479)=-0.126798418101D+05
coeff(480)=-0.116960336330D+04
coeff(481)=-0.903232745740D+03
coeff(482)=-0.297478095451D+04
coeff(483)=0.541817039536D+04
coeff(484)=0.110834152036D+04
coeff(485)=0.416521520534D+04
coeff(486)=-0.477962130508D+04
coeff(487)=-0.308720868256D+04
coeff(488)=-0.801942203929D+04
coeff(489)=-0.250355643852D+04
coeff(490)=-0.105574135264D+04
coeff(491)=0.978731469572D+04
coeff(492)=-0.457555565805D+03
coeff(493)=0.200014871749D+05
coeff(494)=-0.759689512963D+04
coeff(495)=-0.235893307581D+04
coeff(496)=0.144213920814D+03
coeff(497)=0.213972388428D+04
coeff(498)=0.849710786793D+04
coeff(499)=0.658314276450D+04
coeff(500)=-0.213100159235D+04
coeff(501)=-0.147897313249D+04
coeff(502)=-0.171573447271D+04
coeff(503)=0.156830868977D+04
coeff(504)=-0.224884814468D+04
coeff(505)=0.536070951672D+04
coeff(506)=-0.332821028082D+04
coeff(507)=-0.289530756480D+04
coeff(508)=-0.162320739564D+05
coeff(509)=-0.176672227420D+05
coeff(510)=-0.493190523032D+04
coeff(511)=0.137990168448D+04
coeff(512)=0.640079240320D+04
coeff(513)=0.263014402317D+04
coeff(514)=-0.127314627629D+05
coeff(515)=0.184381083911D+05
coeff(516)=-0.301459374592D+04
coeff(517)=-0.685810278012D+04
coeff(518)=0.599784691095D+04
coeff(519)=0.871177317929D+04
coeff(520)=-0.461015933184D+04
coeff(521)=0.521227667926D+04
coeff(522)=-0.269792856992D+04
coeff(523)=0.205685105673D+04
coeff(524)=-0.583817668847D+04
coeff(525)=0.947736034247D+04
coeff(526)=-0.296286719670D+04
coeff(527)=0.164460401455D+04
coeff(528)=0.104412676407D+05
coeff(529)=0.261680184936D+04
coeff(530)=-0.380463578039D+04
coeff(531)=-0.185178694868D+04
coeff(532)=0.507936347459D+04
coeff(533)=-0.917977526583D+04
coeff(534)=0.343450328971D+04
coeff(535)=0.697460434880D+04
coeff(536)=-0.156539336339D+04
coeff(537)=0.699619423890D+04
coeff(538)=-0.129506195421D+05
coeff(539)=0.398588730481D+04
coeff(540)=-0.509733403894D+04
coeff(541)=-0.298378014765D+03
coeff(542)=0.234748053703D+05
coeff(543)=-0.316957449929D+03
coeff(544)=0.583000382799D+04
coeff(545)=-0.320985994134D+04
coeff(546)=0.674098846439D+04
coeff(547)=-0.108741604552D+05
coeff(548)=0.241159950016D+03
coeff(549)=-0.363858048101D+03
coeff(550)=-0.100071122815D+03
coeff(551)=0.485940323869D+04
coeff(552)=0.112875858652D+05
coeff(553)=0.271926177349D+04
coeff(554)=0.620749428298D+04
coeff(555)=0.357495604871D+04
coeff(556)=0.144167969557D+04
coeff(557)=0.586967733519D+04
coeff(558)=-0.657094384556D+04
coeff(559)=0.327179675620D+04
coeff(560)=0.610108249216D+04
coeff(561)=-0.878632716035D+04
coeff(562)=0.106488219799D+04
coeff(563)=-0.114492497862D+05
coeff(564)=0.524292961444D+04
coeff(565)=0.453341997630D+04
coeff(566)=0.111983245824D+04
coeff(567)=-0.443198995846D+04
coeff(568)=0.925464045683D+04
coeff(569)=0.627696468426D+04
coeff(570)=0.479579923375D+04
coeff(571)=0.463424193401D+04
coeff(572)=0.993508858888D+03
coeff(573)=-0.388113858691D+04
coeff(574)=0.789908571267D+04
coeff(575)=-0.725551022886D+04
coeff(576)=0.678834224896D+04
coeff(577)=0.383350107885D+04
coeff(578)=0.527525919252D+04
coeff(579)=-0.464631886677D+04
coeff(580)=0.115370085875D+04
coeff(581)=-0.502110385377D+04
coeff(582)=0.349332131632D+03
coeff(583)=0.469407874476D+03
coeff(584)=-0.352578249546D+04
coeff(585)=-0.492553639245D+04
coeff(586)=-0.179002723487D+04
coeff(587)=0.627323422434D+04
coeff(588)=-0.475505154165D+04
coeff(589)=-0.541938909213D+04
coeff(590)=0.437616307647D+04
coeff(591)=-0.141108971604D+05
coeff(592)=0.113144064254D+04
coeff(593)=-0.102623903869D+04
coeff(594)=0.581199263033D+04
coeff(595)=0.926932540912D+03
coeff(596)=0.167164073483D+05
coeff(597)=-0.167106483543D+05
coeff(598)=-0.364806741747D+04
coeff(599)=0.551533774565D+04
coeff(600)=-0.659483337233D+04
coeff(601)=-0.107187933879D+04
coeff(602)=-0.598697170366D+04
coeff(603)=-0.312240395865D+04
coeff(604)=0.297210625089D+04
coeff(605)=0.414108065286D+04
coeff(606)=0.648554246687D+04
coeff(607)=-0.121347167843D+05
coeff(608)=-0.425864049762D+04
coeff(609)=0.409596664449D+03
coeff(610)=-0.573976263020D+04
coeff(611)=0.863696053220D+03
coeff(612)=-0.848834024145D+04
coeff(613)=-0.625955837146D+04
coeff(614)=0.170520511275D+04
coeff(615)=-0.138836174181D+04
coeff(616)=0.766118995455D+04
coeff(617)=0.269000964290D+04
coeff(618)=0.143885833650D+04
coeff(619)=-0.124458132842D+04
coeff(620)=0.519932714339D+04
coeff(621)=-0.740490591296D+04
coeff(622)=0.949146758138D+04
coeff(623)=0.119870696397D+05
coeff(624)=0.787031467003D+04
coeff(625)=-0.667464870154D+04
coeff(626)=0.596048766627D+04
coeff(627)=0.709373189691D+04
coeff(628)=0.160501605672D+05
coeff(629)=0.660600527316D+04
coeff(630)=-0.146018997184D+04
coeff(631)=0.314768899710D+04
coeff(632)=0.128946203595D+05
coeff(633)=0.118381931472D+05
coeff(634)=0.135735409664D+05
coeff(635)=0.427621323191D+04
coeff(636)=-0.728940030723D+04
coeff(637)=-0.869197135014D+02
coeff(638)=-0.176931358601D+04
coeff(639)=0.931380052006D+02
coeff(640)=-0.106341863140D+05
coeff(641)=0.383442938583D+04
coeff(642)=0.716033212891D+04
coeff(643)=0.531524793798D+03
coeff(644)=0.775180073296D+04
coeff(645)=-0.185489269909D+04
coeff(646)=-0.806587057432D+04
coeff(647)=0.297695674882D+04
coeff(648)=0.522343989758D+04
coeff(649)=-0.206634568723D+03
coeff(650)=0.249600653311D+04
coeff(651)=0.792202665631D+04
coeff(652)=-0.459787669277D+03
coeff(653)=0.341985079166D+04
coeff(654)=-0.153887886976D+05
coeff(655)=0.676085270033D+04
coeff(656)=0.116115508819D+05
coeff(657)=-0.765133389221D+04
coeff(658)=-0.724026521902D+03
coeff(659)=0.215130525089D+04
coeff(660)=0.608061221627D+04
coeff(661)=-0.974494924016D+04
coeff(662)=-0.684931329002D+04
coeff(663)=0.795515782485D+04
coeff(664)=-0.103558078659D+05
coeff(665)=0.436235465165D+04
coeff(666)=0.360169189176D+04
coeff(667)=0.420572503350D+04
coeff(668)=-0.898313497748D+04
coeff(669)=-0.960432590416D+04
coeff(670)=-0.981281012985D+04
coeff(671)=-0.906281546956D+04
coeff(672)=-0.651279481427D+04
coeff(673)=-0.788149409944D+03
coeff(674)=0.961823671620D+04
coeff(675)=-0.701975029046D+04
coeff(676)=-0.156184513199D+05
coeff(677)=-0.971310148666D+04
coeff(678)=-0.120118451352D+05
coeff(679)=-0.874900865831D+04
coeff(680)=0.451369109109D+04
coeff(681)=0.898867912443D+04
coeff(682)=-0.443397881454D+04
coeff(683)=-0.109708401867D+05
coeff(684)=-0.323315966979D+04
coeff(685)=0.237403133566D+05
coeff(686)=-0.911933866624D+04
coeff(687)=0.509891715930D+04
coeff(688)=0.431657965422D+04
coeff(689)=0.619602815759D+03
coeff(690)=-0.100934403330D+05
coeff(691)=0.151162717496D+05
coeff(692)=-0.767383019975D+04
coeff(693)=0.678193747197D+04
coeff(694)=-0.381735571055D+04
coeff(695)=0.515970018592D+04
coeff(696)=0.109283411153D+05
coeff(697)=0.671175030491D+04
coeff(698)=-0.281469178974D+04
coeff(699)=-0.417381151905D+04
coeff(700)=-0.466716264247D+02
coeff(701)=-0.103875216613D+05
coeff(702)=0.524794123093D+03
coeff(703)=0.354103893798D+04
coeff(704)=0.259576231120D+04
coeff(705)=0.953426051756D+04
coeff(706)=-0.461331192982D+04
coeff(707)=-0.602230459504D+04
coeff(708)=-0.318147619661D+04
coeff(709)=-0.471430098404D+04
coeff(710)=-0.216930874740D+04
coeff(711)=0.231753869439D+03
coeff(712)=0.835911235804D+03
coeff(713)=0.287252311920D+04
coeff(714)=-0.191957787092D+04
coeff(715)=-0.113617250504D+04
coeff(716)=-0.626312003731D+04
coeff(717)=-0.115337988887D+04
coeff(718)=0.289476076067D+04
coeff(719)=-0.755789367041D+04
coeff(720)=-0.413975020837D+03
coeff(721)=0.178322700581D+05
coeff(722)=-0.682747128739D+04
coeff(723)=-0.157863058895D+05
coeff(724)=-0.175224916363D+04
coeff(725)=0.197058767975D+04
coeff(726)=-0.143865666655D+04
coeff(727)=-0.301295354062D+04
coeff(728)=-0.396853680315D+03
coeff(729)=-0.132726103880D+04
coeff(730)=0.153539265292D+05
coeff(731)=-0.848378575358D+04
coeff(732)=0.455176998948D+04
coeff(733)=-0.518345501711D+04
coeff(734)=-0.395934645745D+04
coeff(735)=0.174914128190D+05
coeff(736)=0.622387891248D+04
coeff(737)=-0.133847321500D+05
coeff(738)=0.211764634838D+04
coeff(739)=-0.143739288346D+04
coeff(740)=-0.125246608714D+05
coeff(741)=0.238030539303D+04
coeff(742)=-0.183226672167D+05
coeff(743)=-0.449974430893D+04
coeff(744)=0.446547333355D+04
coeff(745)=0.638607350495D+04
coeff(746)=-0.110467638159D+05
coeff(747)=-0.555642925150D+03
coeff(748)=-0.131023500648D+04
coeff(749)=-0.141994162192D+04
coeff(750)=-0.825152430118D+04
coeff(751)=-0.309413676548D+03
coeff(752)=0.132679303825D+05
coeff(753)=-0.121140269305D+05
coeff(754)=0.720521968580D+04
coeff(755)=-0.645173090181D+04
coeff(756)=0.889237099871D+04
coeff(757)=-0.249246090505D+03
coeff(758)=0.129689467045D+04
coeff(759)=-0.598146243060D+04
coeff(760)=0.599863012790D+04
coeff(761)=0.296683743234D+04
coeff(762)=0.247124814152D+04
coeff(763)=0.993289384528D+04
coeff(764)=-0.801617082545D+04
coeff(765)=0.338736983072D+04
coeff(766)=-0.514051619351D+04
coeff(767)=-0.526262880696D+04
coeff(768)=-0.473144024685D+04
coeff(769)=-0.678884392864D+04
coeff(770)=0.350831061286D+04
coeff(771)=-0.123007029797D+05
coeff(772)=0.352220832604D+04
coeff(773)=0.833033374770D+04
coeff(774)=-0.102370632487D+04
coeff(775)=0.863744735377D+04
coeff(776)=0.142192096924D+04
coeff(777)=0.110226817656D+04
coeff(778)=0.483211530216D+04
coeff(779)=-0.364609182950D+04
coeff(780)=0.149663860470D+04
coeff(781)=0.378272399804D+03
coeff(782)=0.572071564209D+04
coeff(783)=-0.301729355826D+04
coeff(784)=0.528987460814D+04
coeff(785)=0.664668251747D+03
coeff(786)=-0.396868806365D+04
coeff(787)=-0.635436857902D+04
coeff(788)=0.140417789719D+05
coeff(789)=-0.787096748982D+04
coeff(790)=-0.209470345377D+04
coeff(791)=0.618676774083D+04
coeff(792)=-0.488461348983D+04
coeff(793)=-0.361034095986D+04
coeff(794)=0.218452796522D+04
coeff(795)=-0.545725264101D+04
coeff(796)=0.443694310943D+04
coeff(797)=0.453693400057D+04
coeff(798)=-0.228810099190D+03
coeff(799)=0.119154190423D+04
coeff(800)=-0.669132826805D+04
coeff(801)=0.346082728014D+04
coeff(802)=0.622812314045D+04
coeff(803)=-0.765606572028D+04
coeff(804)=0.859990755517D+04
coeff(805)=0.549954225316D+04
coeff(806)=0.803219123319D+03
coeff(807)=-0.102799253882D+05
coeff(808)=0.282498907013D+04
coeff(809)=0.660475019749D+04
coeff(810)=0.150201979252D+05
coeff(811)=-0.117285145454D+05
coeff(812)=-0.309889420852D+04
coeff(813)=-0.136638342139D+05
coeff(814)=-0.627999913607D+04
coeff(815)=0.821797056314D+04
coeff(816)=-0.108309472165D+05
coeff(817)=-0.244942749190D+04
coeff(818)=0.215474528212D+05
coeff(819)=-0.587215304502D+03
coeff(820)=0.982835463127D+04
coeff(821)=-0.106120576748D+03
coeff(822)=0.867792633902D+04
coeff(823)=-0.730899901822D+01
coeff(824)=-0.660445363188D+04
coeff(825)=-0.398118233620D+04
coeff(826)=-0.446363574614D+02
coeff(827)=-0.340603891022D+04
coeff(828)=-0.202715014096D+04
coeff(829)=0.829038008528D+04
coeff(830)=-0.280297125641D+04
coeff(831)=-0.112042304333D+04
coeff(832)=-0.113681583766D+05
coeff(833)=0.294903116377D+04
coeff(834)=0.708697861443D+04
coeff(835)=0.149159891240D+04
coeff(836)=0.608787114365D+04
coeff(837)=-0.609023789676D+04
coeff(838)=0.640859211313D+04
coeff(839)=-0.689913392934D+04
coeff(840)=-0.391622509158D+04
coeff(841)=0.151774630273D+04
coeff(842)=-0.608140199329D+04
coeff(843)=0.961899246462D+04
coeff(844)=-0.104319044347D+05
coeff(845)=0.111462494692D+05
coeff(846)=-0.500598854689D+04
coeff(847)=-0.792469784431D+04
coeff(848)=-0.125442923405D+05
coeff(849)=0.661552589549D+04
coeff(850)=0.531966122545D+04
coeff(851)=0.903654597155D+04
coeff(852)=0.470632805721D+04
coeff(853)=-0.910361951596D+04
coeff(854)=-0.998505322029D+03
coeff(855)=-0.302553169622D+04
coeff(856)=-0.703909146614D+04
coeff(857)=0.780405948270D+04
coeff(858)=-0.658069877486D+04
coeff(859)=-0.756769242658D+04
coeff(860)=0.403433699024D+04
coeff(861)=0.626629125258D+04
coeff(862)=0.928415214728D+02
coeff(863)=-0.726328577263D+04
coeff(864)=-0.244818590500D+04
coeff(865)=0.174737755043D+05
coeff(866)=-0.240034076315D+04
coeff(867)=-0.433492914106D+04
coeff(868)=-0.597695024070D+04
coeff(869)=0.300156744094D+04
coeff(870)=-0.742248787374D+04
coeff(871)=-0.457481634085D+04
coeff(872)=-0.640563237891D+04
coeff(873)=0.960610843969D+03
coeff(874)=0.526798367365D+04
coeff(875)=-0.218841434546D+04
coeff(876)=0.154927346225D+05
coeff(877)=-0.106234703402D+05
coeff(878)=-0.130876465497D+05
coeff(879)=0.609693051817D+04
coeff(880)=0.157141121715D+04
coeff(881)=0.823445565864D+03
coeff(882)=-0.636504306523D+04
coeff(883)=0.514383388637D+04
coeff(884)=0.193175901855D+04
coeff(885)=-0.410543897317D+04
coeff(886)=0.304825012836D+04
coeff(887)=0.753145768085D+04
coeff(888)=0.596377160517D+04
coeff(889)=-0.616544837130D+04
coeff(890)=0.114958168461D+05
coeff(891)=0.365567851033D+04
coeff(892)=0.808734325444D+04
coeff(893)=-0.270202453051D+04
coeff(894)=0.887421671804D+04
coeff(895)=-0.842759271027D+04
coeff(896)=0.124689602525D+05
coeff(897)=0.622190450481D+03
coeff(898)=-0.145460078587D+05
coeff(899)=0.159973555461D+05
coeff(900)=0.599363728251D+04
coeff(901)=0.875888259826D+04
coeff(902)=-0.107026908704D+05
coeff(903)=0.151895895743D+05
coeff(904)=-0.645866657311D+04
coeff(905)=-0.653482811066D+04
coeff(906)=-0.327037998462D+04
coeff(907)=0.334200366735D+04
coeff(908)=-0.135897640204D+05
coeff(909)=-0.166811726803D+05
coeff(910)=0.222351153830D+05
coeff(911)=0.876859990081D+04
coeff(912)=-0.190178534295D+05
coeff(913)=0.122455480758D+05
coeff(914)=-0.108722324822D+05
coeff(915)=-0.392624293854D+04
coeff(916)=-0.836957168474D+04
coeff(917)=0.264074451034D+04
coeff(918)=-0.170874301392D+04
coeff(919)=-0.106582084486D+03
coeff(920)=-0.563015853103D+04
coeff(921)=-0.783595422440D+04
coeff(922)=0.145949507640D+05
coeff(923)=-0.457531687561D+04
coeff(924)=-0.151216900827D+05
coeff(925)=-0.451338958315D+04
coeff(926)=-0.119495110419D+05
coeff(927)=-0.586886831164D+04
coeff(928)=-0.111948229455D+05
coeff(929)=0.631794091383D+04
coeff(930)=-0.329924254116D+04
coeff(931)=-0.153095901072D+04
coeff(932)=-0.737866256058D+04
coeff(933)=0.193672079479D+04
coeff(934)=-0.312812140961D+03
coeff(935)=0.306045495291D+04
coeff(936)=0.128348581258D+04
coeff(937)=0.895679541613D+04
coeff(938)=-0.571914858414D+04
coeff(939)=0.305016141784D+04
coeff(940)=0.152312280251D+04
coeff(941)=0.246864514472D+04
coeff(942)=0.518490654753D+04
coeff(943)=-0.876881629319D+04
coeff(944)=0.807231776153D+03
coeff(945)=0.958463724926D+04
coeff(946)=-0.851010550573D+04
coeff(947)=0.110523383480D+05
coeff(948)=0.179337787236D+04
coeff(949)=0.109342348293D+05
coeff(950)=0.234133553125D+04
coeff(951)=0.161723769881D+05
coeff(952)=-0.418236278659D+04
coeff(953)=0.109122345998D+05
coeff(954)=-0.546652118579D+03
coeff(955)=-0.143941230192D+05
coeff(956)=-0.219797398282D+04
coeff(957)=-0.110946434836D+05
coeff(958)=-0.198385093952D+04
coeff(959)=0.380855415849D+04
coeff(960)=0.111590286576D+04
coeff(961)=0.886484671456D+03
coeff(962)=0.412462833020D+04
coeff(963)=-0.119981969952D+05
coeff(964)=0.223210314877D+04
coeff(965)=0.590463790687D+02
coeff(966)=0.216166052940D+05
coeff(967)=-0.160809208417D+05
coeff(968)=0.766341907126D+04
coeff(969)=-0.650017150465D+04
coeff(970)=-0.761017854370D+04
coeff(971)=0.550679341401D+04
coeff(972)=-0.256332288928D+04
coeff(973)=0.201249504513D+04
coeff(974)=-0.303705929638D+04
coeff(975)=-0.229244843579D+04
coeff(976)=-0.152272423153D+05
coeff(977)=0.982793014843D+04
coeff(978)=-0.646493575849D+04
coeff(979)=0.831640847167D+03
coeff(980)=0.184018549327D+04
coeff(981)=-0.383223722938D+04
coeff(982)=0.103215710337D+05
coeff(983)=-0.748191927143D+04
coeff(984)=0.112240605237D+04
coeff(985)=-0.380604915567D+04
coeff(986)=0.342555098708D+03
coeff(987)=0.216471333873D+04
coeff(988)=0.134412554573D+04
coeff(989)=0.200754235195D+04
coeff(990)=-0.593224663924D+04
coeff(991)=0.190765592764D+04
coeff(992)=0.939079430635D+04
coeff(993)=-0.439065192563D+04
coeff(994)=-0.142929707860D+05
coeff(995)=-0.332222469198D+03
coeff(996)=0.510433842291D+04
coeff(997)=0.206245253731D+04
coeff(998)=-0.431081292855D+04
coeff(999)=0.415579592387D+04
coeff(1000)=0.180807463184D+04
coeff(1001)=-0.331403111143D+04
coeff(1002)=0.553285679982D+04
coeff(1003)=-0.104607307260D+04
coeff(1004)=0.306172002928D+04
coeff(1005)=0.207678880089D+04
coeff(1006)=-0.180851916949D+04
coeff(1007)=0.667153466510D+03
coeff(1008)=-0.120395924416D+05
coeff(1009)=0.326015745094D+04
coeff(1010)=0.152860086117D+04
coeff(1011)=-0.724261421978D+04
coeff(1012)=0.131351980628D+05
coeff(1013)=0.526645619693D+03
coeff(1014)=-0.184903026323D+04
coeff(1015)=0.413563271614D+04
coeff(1016)=0.246690024307D+04
coeff(1017)=-0.419934885200D+04
coeff(1018)=0.568202689379D+03
coeff(1019)=-0.399709302924D+03
coeff(1020)=-0.166938571478D+04
coeff(1021)=0.483452007822D+04
coeff(1022)=0.210855967931D+04
coeff(1023)=0.118469662141D+04
coeff(1024)=0.153800383458D+05
coeff(1025)=-0.515133783794D+04
coeff(1026)=0.744125416632D+04
coeff(1027)=0.897883899397D+03
coeff(1028)=0.511700887211D+04
coeff(1029)=0.104434297318D+05
coeff(1030)=-0.711287075458D+04
coeff(1031)=-0.515738331964D+04
coeff(1032)=-0.289646215747D+04
coeff(1033)=-0.615835822210D+04
coeff(1034)=0.991863746891D+04
coeff(1035)=-0.458943746774D+04
coeff(1036)=-0.365340269661D+04
coeff(1037)=0.114808921955D+05
coeff(1038)=-0.390456621146D+04
coeff(1039)=-0.304474999015D+04
coeff(1040)=0.989693347304D+03
coeff(1041)=0.250686128187D+03
coeff(1042)=-0.477497291279D+04
coeff(1043)=-0.101947712569D+05
coeff(1044)=-0.198719409967D+04
coeff(1045)=0.871143961720D+04
coeff(1046)=-0.620547420282D+04
coeff(1047)=-0.619261251286D+03
coeff(1048)=0.297802600116D+04
coeff(1049)=-0.163394249340D+05
coeff(1050)=0.723551647487D+04
coeff(1051)=-0.221467539168D+04
coeff(1052)=-0.133291612495D+05
coeff(1053)=0.103307251810D+03
coeff(1054)=0.609501076248D+04
coeff(1055)=-0.118555402861D+05
coeff(1056)=-0.200723682953D+05
coeff(1057)=0.502980459713D+04
coeff(1058)=-0.369779159423D+04
coeff(1059)=-0.256401647756D+03
coeff(1060)=0.198720705793D+04
coeff(1061)=-0.767333243543D+04
coeff(1062)=-0.176031539671D+04
coeff(1063)=-0.105431670030D+05
coeff(1064)=-0.276734200789D+04
coeff(1065)=-0.160822597107D+05
coeff(1066)=0.404326370631D+04
coeff(1067)=-0.547164108737D+04
coeff(1068)=-0.238877134414D+04
coeff(1069)=-0.516877763229D+04
coeff(1070)=-0.245942142727D+04
coeff(1071)=0.942593223117D+04
coeff(1072)=-0.412957772290D+04
coeff(1073)=0.976740504130D+04
coeff(1074)=0.680565205848D+04
coeff(1075)=-0.246706159290D+04
coeff(1076)=0.639111966099D+04
coeff(1077)=-0.320017599445D+04
coeff(1078)=0.291452882756D+04
coeff(1079)=0.122086126468D+05
coeff(1080)=-0.139600132006D+05
coeff(1081)=-0.573997723125D+03
coeff(1082)=0.162833499998D+05
coeff(1083)=-0.331916417761D+04
coeff(1084)=0.141490020834D+05
coeff(1085)=-0.222899211058D+04
coeff(1086)=-0.500776744843D+03
coeff(1087)=-0.119446081191D+05
coeff(1088)=0.107778789563D+05
coeff(1089)=-0.201987316516D+04
coeff(1090)=-0.519985098394D+04
coeff(1091)=-0.104897599003D+04
coeff(1092)=0.651974429738D+04
coeff(1093)=0.382404950375D+04
coeff(1094)=-0.640166727399D+04
coeff(1095)=0.214596122299D+04
coeff(1096)=0.113178847611D+05
coeff(1097)=-0.129328917801D+05
coeff(1098)=0.163294478204D+05
coeff(1099)=0.906101257744D+04
coeff(1100)=0.257920958264D+04
coeff(1101)=-0.741013527203D+04
coeff(1102)=0.149307807837D+05
coeff(1103)=0.178577274228D+05
coeff(1104)=-0.107865124120D+04
coeff(1105)=-0.261569734753D+04
coeff(1106)=0.256857721181D+04
coeff(1107)=0.473746367844D+04
coeff(1108)=0.375023045824D+04
coeff(1109)=0.197046937372D+05
coeff(1110)=0.618443160167D+04
coeff(1111)=0.521618260495D+04
coeff(1112)=-0.370342687520D+04
coeff(1113)=-0.508472110831D+03
coeff(1114)=-0.433657118271D+04
coeff(1115)=-0.120340102130D+05
coeff(1116)=-0.538114634742D+04
coeff(1117)=-0.230745887492D+05
coeff(1118)=0.281043972690D+04
coeff(1119)=0.633233979747D+04
coeff(1120)=-0.241083905305D+04
coeff(1121)=-0.668913938057D+04
coeff(1122)=0.323993270899D+04
coeff(1123)=0.123264398543D+04
coeff(1124)=-0.935263616247D+04
coeff(1125)=0.188016727290D+04
coeff(1126)=0.874789085330D+04
coeff(1127)=0.974682670867D+04
coeff(1128)=-0.102477332870D+04
coeff(1129)=0.238337250164D+04
coeff(1130)=0.138154924590D+05
coeff(1131)=-0.956196175249D+04
coeff(1132)=0.383554392058D+03
coeff(1133)=-0.892474878851D+04
coeff(1134)=-0.170138486404D+05
coeff(1135)=0.101345570473D+05
coeff(1136)=-0.973135005910D+04
coeff(1137)=0.125501901799D+04
coeff(1138)=0.496387007153D+04
coeff(1139)=-0.240442713865D+03
coeff(1140)=-0.565203290848D+04
coeff(1141)=0.868866018014D+03
coeff(1142)=0.793774826517D+04
coeff(1143)=-0.192395278181D+05
coeff(1144)=-0.561628155543D+04
coeff(1145)=-0.977214787122D+04
coeff(1146)=0.117337772976D+05
coeff(1147)=-0.638685491342D+04
coeff(1148)=-0.745674470961D+04
coeff(1149)=0.840741371960D+03
coeff(1150)=-0.459159664820D+04
coeff(1151)=-0.230676338093D+04
coeff(1152)=-0.151941459869D+04
coeff(1153)=-0.110701195688D+05
coeff(1154)=-0.255880516380D+04
coeff(1155)=0.666739586311D+04
coeff(1156)=0.147950722068D+05
coeff(1157)=0.144354611955D+05
coeff(1158)=-0.698706756443D+04
coeff(1159)=0.220593391505D+04
coeff(1160)=-0.974957827449D+04
coeff(1161)=0.121261245698D+05
coeff(1162)=0.105373139105D+05
coeff(1163)=-0.854854503945D+04
coeff(1164)=0.193929639108D+04
coeff(1165)=-0.436740704211D+04
coeff(1166)=-0.165605913195D+04
coeff(1167)=-0.110557648462D+03
coeff(1168)=0.192199798763D+04
coeff(1169)=-0.420258792953D+04
coeff(1170)=0.301788129934D+04
coeff(1171)=-0.232708947986D+04
coeff(1172)=0.106206300298D+05
coeff(1173)=-0.174585930162D+04
coeff(1174)=-0.460092543600D+04
coeff(1175)=-0.652113327614D+04
coeff(1176)=0.347959010668D+04
coeff(1177)=-0.135215683270D+03
coeff(1178)=-0.238609175971D+04
coeff(1179)=0.783200838063D+04
coeff(1180)=-0.296562332309D+04
coeff(1181)=-0.328537488519D+04
coeff(1182)=0.798943932482D+03
coeff(1183)=-0.184958966715D+04
coeff(1184)=0.279254896870D+03
coeff(1185)=0.280243985721D+04
coeff(1186)=-0.139458365598D+03
coeff(1187)=0.279769424037D+04
coeff(1188)=-0.322694424801D+04
coeff(1189)=0.106093261359D+05
coeff(1190)=-0.238656886078D+04
coeff(1191)=-0.366793710079D+04
coeff(1192)=-0.570311820870D+04
coeff(1193)=0.548713240308D+04
coeff(1194)=0.805685815275D+03
coeff(1195)=0.192260154154D+04
coeff(1196)=-0.457746300623D+03
coeff(1197)=0.964359279775D+03
coeff(1198)=0.277470005233D+04
coeff(1199)=-0.461434226329D+04
coeff(1200)=0.636347656147D+04
coeff(1201)=0.881088004759D+02
coeff(1202)=-0.583184353708D+04
coeff(1203)=0.274187407323D+04
coeff(1204)=0.411058452762D+04
coeff(1205)=0.122693782758D+04
coeff(1206)=0.343717398908D+02
coeff(1207)=0.664500630329D+02
coeff(1208)=0.758613003656D+03
coeff(1209)=0.156827825106D+04
coeff(1210)=-0.659040833365D+04
coeff(1211)=0.322116349922D+04
coeff(1212)=-0.344388648273D+04
coeff(1213)=-0.302348694474D+04
coeff(1214)=0.841440746410D+04
coeff(1215)=0.179307724515D+04
coeff(1216)=-0.813107794171D+03
coeff(1217)=-0.739726053024D+04
coeff(1218)=-0.321535299670D+04
coeff(1219)=0.486537780487D+04
coeff(1220)=-0.303849457366D+04
coeff(1221)=-0.239163770271D+04
coeff(1222)=0.717666977877D+04
coeff(1223)=0.556750537194D+02
coeff(1224)=0.220346394014D+04
coeff(1225)=0.783390445115D+04
coeff(1226)=-0.930410994636D+04
coeff(1227)=0.212252690927D+04
coeff(1228)=-0.577771601494D+04
coeff(1229)=0.641874574677D+04
coeff(1230)=0.257291567001D+04
coeff(1231)=-0.117429768634D+04
coeff(1232)=0.332384306895D+04
coeff(1233)=0.543405554945D+04
coeff(1234)=-0.466007324056D+04
coeff(1235)=-0.701997893616D+04
coeff(1236)=0.733285296748D+04
coeff(1237)=-0.291298766024D+04
coeff(1238)=-0.456580744569D+03
coeff(1239)=0.144518615142D+04
coeff(1240)=-0.516148353645D+04
coeff(1241)=0.775303605000D+04
coeff(1242)=-0.379272587804D+04
coeff(1243)=-0.179142917876D+04
coeff(1244)=0.161889625574D+04
coeff(1245)=-0.316720201679D+04
coeff(1246)=0.369911359740D+03
coeff(1247)=0.376038859230D+04
coeff(1248)=0.630993948202D+03
coeff(1249)=-0.350217671305D+04
coeff(1250)=-0.198397116591D+04
coeff(1251)=0.141845071963D+04
coeff(1252)=0.204890416389D+04
coeff(1253)=-0.323409074000D+04
coeff(1254)=0.598729081206D+04
coeff(1255)=-0.185445665808D+04
coeff(1256)=-0.630189380355D+04
coeff(1257)=0.368804489827D+04
coeff(1258)=0.279116885743D+04
coeff(1259)=-0.269376979853D+04
coeff(1260)=0.281521415304D+03
coeff(1261)=-0.248719675490D+04
coeff(1262)=-0.678096550427D+04
coeff(1263)=0.246303901850D+04
coeff(1264)=0.109093683165D+04
coeff(1265)=0.240827842518D+04
coeff(1266)=-0.467038045590D+03
coeff(1267)=0.190652819034D+04
coeff(1268)=-0.119253418319D+04
coeff(1269)=0.297504970699D+04
coeff(1270)=-0.372180817227D+04
coeff(1271)=0.725833288768D+04
coeff(1272)=-0.155064916395D+04
coeff(1273)=-0.139361901313D+04
coeff(1274)=-0.856926532827D+03
coeff(1275)=-0.133633074298D+04
coeff(1276)=0.452716521479D+03
coeff(1277)=0.293372004239D+04
coeff(1278)=-0.491761024527D+04
coeff(1279)=0.285776563194D+04
coeff(1280)=-0.274642512431D+04
coeff(1281)=-0.621678198877D+03
coeff(1282)=-0.229000176391D+03
coeff(1283)=0.959662025967D+03
coeff(1284)=-0.272501955635D+04
coeff(1285)=-0.205673067271D+03
coeff(1286)=0.715401340326D+03
coeff(1287)=0.437786789597D+04
coeff(1288)=-0.416087909185D+04
coeff(1289)=0.321283843292D+04
coeff(1290)=-0.530917677250D+03
coeff(1291)=-0.107625457168D+04
coeff(1292)=0.127583028738D+04
coeff(1293)=0.945732637439D+03
coeff(1294)=-0.208122586535D+04
coeff(1295)=0.150056915542D+04
coeff(1296)=-0.104445028506D+05
coeff(1297)=0.646843188624D+04
coeff(1298)=-0.441894669672D+04
coeff(1299)=0.238816376192D+03
coeff(1300)=-0.362769384165D+03
  return
end if

entry u23_dc_10(xyz,v,s)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

!x(1)=exp(-((r12-ra1))/a1)
!x(2)=exp(-((r13-ra1))/a1)
!x(3)=exp(-((r14-ra2))/a2)
!x(4)=exp(-((r23-ra1))/a1)
!x(5)=exp(-((r24-ra2))/a2)
!x(6)=exp(-((r34-ra2))/a2)
 
x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)

do i=1,3
do j=1,3
rrel(i,j)=xyz(i,j)-xyz(4,j)
enddo
enddo

!compute r2xr3
cross(1)=rrel(2,2)*rrel(3,3)-rrel(2,3)*rrel(3,2)
cross(2)=-rrel(2,1)*rrel(3,3)+rrel(2,3)*rrel(3,1)
cross(3)=rrel(2,1)*rrel(3,2)-rrel(2,2)*rrel(3,1)
!compute p*r1*(r2xr3)
volume=0.00000
do i=1,3
volume=volume+rrel(1,i)*cross(i)
enddo
volume=tanh(15.4*volume)

call ev_pol_con_10(x,q)
v=dot_product(coeff,q)*volume/627.509474
s=dot_product(coeff,q)/627.509474

end  subroutine

!===========================================================================
! *def ev_pol_con_12(x,q)
! subtoutine of evaluating polynomials at 12th order
!===========================================================================
subroutine ev_pol_con_12(x,q)
implicit none
integer :: j
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp),dimension(1:6),intent(in)::x
real(dp),dimension(1:3075),intent(out)::q
real(dp),dimension(0:32) :: rm
real(dp),dimension(0:3392) :: p
real(dp) :: bm1(3394), bm2(3076)

rm(0) = 1.0D0
rm(1) = x(4)
rm(2) = x(2)
rm(3) = x(1)
rm(4) = x(6)
rm(5) = x(5)
rm(6) = x(3)
rm(7) = rm(1)*rm(2)
rm(8) = rm(1)*rm(3)
rm(9) = rm(2)*rm(3)
rm(10) = rm(3)*rm(4)
rm(11) = rm(2)*rm(5)
rm(12) = rm(1)*rm(6)
rm(13) = rm(4)*rm(5)
rm(14) = rm(4)*rm(6)
rm(15) = rm(5)*rm(6)
rm(16) = rm(1)*rm(9)
rm(17) = rm(1)*rm(10)
rm(18) = rm(2)*rm(10)
rm(19) = rm(1)*rm(11)
rm(20) = rm(3)*rm(11)
rm(21) = rm(2)*rm(12)
rm(22) = rm(3)*rm(12)
rm(23) = rm(2)*rm(13)
rm(24) = rm(3)*rm(13)
rm(25) = rm(1)*rm(14)
rm(26) = rm(3)*rm(14)
rm(27) = rm(1)*rm(15)
rm(28) = rm(2)*rm(15)
rm(29) = rm(4)*rm(15)
rm(30) = rm(2)*rm(24)
rm(31) = rm(1)*rm(26)
rm(32) = rm(1)*rm(28)

p(0) = rm(0)
p(1) = rm(1) + rm(2) + rm(3)
p(2) = rm(4) + rm(5) + rm(6)
p(3) = rm(7) + rm(8) + rm(9)
p(4) = rm(10) + rm(11) + rm(12)
p(5) = p(1)*p(2) - p(4)
p(6) = rm(13) + rm(14) + rm(15)
p(7) = p(1)*p(1) - p(3) - p(3)
p(8) = p(2)*p(2) - p(6) - p(6)
p(9) = rm(16)
p(10) = rm(17) + rm(18) + rm(19) + rm(20) + rm(21) + rm(22)
p(11) = p(2)*p(3) - p(10)
p(12) = rm(23) + rm(24) + rm(25) + rm(26) + rm(27) + rm(28)
p(13) = p(1)*p(6) - p(12)
p(14) = rm(29)
p(15) = p(1)*p(3) - p(9) - p(9) - p(9)
p(16) = p(1)*p(4) - p(10)
p(17) = p(2)*p(7) - p(16)
p(18) = p(2)*p(4) - p(12)
p(19) = p(1)*p(8) - p(18)
p(20) = p(2)*p(6) - p(14) - p(14) - p(14)
p(21) = p(1)*p(7) - p(15)
p(22) = p(2)*p(8) - p(20)
p(23) = p(9)*p(2)
p(24) = rm(30) + rm(31) + rm(32)
p(25) = p(3)*p(6) - p(24)
p(26) = p(14)*p(1)
p(27) = p(9)*p(1)
p(28) = p(3)*p(4) - p(23)
p(29) = p(1)*p(10) - p(23) - p(28) - p(23)
p(30) = p(1)*p(11) - p(23)
p(31) = p(1)*p(12) - p(25) - p(24) - p(24)
p(32) = p(1)*p(13) - p(25)
p(33) = p(4)*p(5) - p(25) - p(31)
p(34) = p(2)*p(11) - p(25)
p(35) = p(4)*p(6) - p(26)
p(36) = p(2)*p(12) - p(26) - p(35) - p(26)
p(37) = p(2)*p(13) - p(26)
p(38) = p(14)*p(2)
p(39) = p(3)*p(3) - p(27) - p(27)
p(40) = p(3)*p(7) - p(27)
p(41) = p(1)*p(16) - p(28)
p(42) = p(2)*p(21) - p(41)
p(43) = p(1)*p(18) - p(33)
p(44) = p(7)*p(8) - p(43)
p(45) = p(6)*p(6) - p(38) - p(38)
p(46) = p(2)*p(18) - p(35)
p(47) = p(1)*p(22) - p(46)
p(48) = p(6)*p(8) - p(38)
p(49) = p(1)*p(21) - p(40)
p(50) = p(2)*p(22) - p(48)
p(51) = p(9)*p(6)
p(52) = p(14)*p(3)
p(53) = p(9)*p(4)
p(54) = p(9)*p(5)
p(55) = p(1)*p(24) - p(51)
p(56) = p(3)*p(12) - p(51) - p(55) - p(51)
p(57) = p(3)*p(13) - p(51)
p(58) = p(14)*p(7)
p(59) = p(9)*p(8)
p(60) = p(2)*p(24) - p(52)
p(61) = p(4)*p(13) - p(58)
p(62) = p(6)*p(11) - p(52)
p(63) = p(14)*p(4)
p(64) = p(14)*p(5)
p(65) = p(9)*p(3)
p(66) = p(9)*p(7)
p(67) = p(3)*p(16) - p(53)
p(68) = p(4)*p(15) - p(54) - p(67)
p(69) = p(2)*p(39) - p(68)
p(70) = p(1)*p(29) - p(54) - p(68)
p(71) = p(7)*p(11) - p(53)
p(72) = p(1)*p(31) - p(56) - p(55)
p(73) = p(1)*p(32) - p(57)
p(74) = p(3)*p(18) - p(59)
p(75) = p(2)*p(29) - p(57) - p(55)
p(76) = p(1)*p(34) - p(59)
p(77) = p(6)*p(16) - p(58)
p(78) = p(1)*p(36) - p(62) - p(60)
p(79) = p(2)*p(32) - p(58)
p(80) = p(6)*p(12) - p(64) - p(63) - p(63)
p(81) = p(1)*p(45) - p(80)
p(82) = p(14)*p(6)
p(83) = p(2)*p(33) - p(61) - p(60)
p(84) = p(2)*p(34) - p(62)
p(85) = p(6)*p(18) - p(63)
p(86) = p(2)*p(36) - p(64) - p(80)
p(87) = p(8)*p(13) - p(63)
p(88) = p(14)*p(8)
p(89) = p(1)*p(39) - p(65)
p(90) = p(3)*p(21) - p(66)
p(91) = p(1)*p(41) - p(67)
p(92) = p(2)*p(49) - p(91)
p(93) = p(1)*p(43) - p(74)
p(94) = p(8)*p(21) - p(93)
p(95) = p(1)*p(46) - p(83)
p(96) = p(7)*p(22) - p(95)
p(97) = p(2)*p(45) - p(82)
p(98) = p(2)*p(46) - p(85)
p(99) = p(1)*p(50) - p(98)
p(100) = p(6)*p(22) - p(88)
p(101) = p(1)*p(49) - p(90)
p(102) = p(2)*p(50) - p(100)
p(103) = p(9)*p(14)
p(104) = p(9)*p(12)
p(105) = p(9)*p(13)
p(106) = p(14)*p(15)
p(107) = p(9)*p(20)
p(108) = p(14)*p(10)
p(109) = p(14)*p(11)
p(110) = p(9)*p(16)
p(111) = p(9)*p(10)
p(112) = p(9)*p(11)
p(113) = p(9)*p(17)
p(114) = p(3)*p(24) - p(104)
p(115) = p(7)*p(24) - p(105)
p(116) = p(3)*p(31) - p(104) - p(115)
p(117) = p(6)*p(39) - p(114)
p(118) = p(3)*p(32) - p(105)
p(119) = p(14)*p(21)
p(120) = p(9)*p(18)
p(121) = p(9)*p(19)
p(122) = p(4)*p(24) - p(103) - p(103) - p(103)
p(123) = p(1)*p(60) - p(107) - p(122)
p(124) = p(13)*p(16) - p(119)
p(125) = p(2)*p(56) - p(106) - p(124)
p(126) = p(4)*p(32) - p(119)
p(127) = p(1)*p(62) - p(107) - p(125)
p(128) = p(14)*p(16)
p(129) = p(14)*p(17)
p(130) = p(6)*p(24) - p(108)
p(131) = p(3)*p(45) - p(130)
p(132) = p(14)*p(12)
p(133) = p(14)*p(13)
p(134) = p(9)*p(22)
p(135) = p(8)*p(24) - p(109)
p(136) = p(13)*p(18) - p(128)
p(137) = p(6)*p(34) - p(109)
p(138) = p(14)*p(18)
p(139) = p(14)*p(19)
p(140) = p(9)*p(9)
p(141) = p(9)*p(15)
p(142) = p(9)*p(21)
p(143) = p(3)*p(41) - p(110)
p(144) = p(4)*p(39) - p(112)
p(145) = p(1)*p(68) - p(111) - p(144)
p(146) = p(1)*p(69) - p(112)
p(147) = p(1)*p(70) - p(113) - p(145)
p(148) = p(11)*p(21) - p(110)
p(149) = p(1)*p(72) - p(116) - p(115)
p(150) = p(1)*p(73) - p(118)
p(151) = p(3)*p(43) - p(120)
p(152) = p(15)*p(18) - p(121) - p(151)
p(153) = p(2)*p(69) - p(117)
p(154) = p(1)*p(75) - p(121) - p(152)
p(155) = p(7)*p(34) - p(120)
p(156) = p(6)*p(41) - p(119)
p(157) = p(1)*p(78) - p(125) - p(123)
p(158) = p(2)*p(73) - p(119)
p(159) = p(1)*p(80) - p(131) - p(130) - p(130)
p(160) = p(1)*p(81) - p(131)
p(161) = p(14)*p(14)
p(162) = p(3)*p(46) - p(134)
p(163) = p(2)*p(75) - p(126) - p(123)
p(164) = p(1)*p(84) - p(134)
p(165) = p(6)*p(43) - p(128)
p(166) = p(1)*p(86) - p(137) - p(135)
p(167) = p(8)*p(32) - p(128)
p(168) = p(4)*p(45) - p(133)
p(169) = p(2)*p(80) - p(132) - p(168)
p(170) = p(2)*p(81) - p(133)
p(171) = p(14)*p(20)
p(172) = p(2)*p(83) - p(136) - p(135)
p(173) = p(2)*p(84) - p(137)
p(174) = p(6)*p(46) - p(138)
p(175) = p(2)*p(86) - p(139) - p(169)
p(176) = p(13)*p(22) - p(138)
p(177) = p(14)*p(22)
p(178) = p(3)*p(39) - p(141)
p(179) = p(1)*p(89) - p(141) - p(178) - p(178)
p(180) = p(3)*p(49) - p(142)
p(181) = p(1)*p(91) - p(143)
p(182) = p(2)*p(101) - p(181)
p(183) = p(1)*p(93) - p(151)
p(184) = p(8)*p(49) - p(183)
p(185) = p(1)*p(95) - p(162)
p(186) = p(21)*p(22) - p(185)
p(187) = p(6)*p(45) - p(171)
p(188) = p(1)*p(98) - p(172)
p(189) = p(7)*p(50) - p(188)
p(190) = p(2)*p(97) - p(171) - p(187) - p(187)
p(191) = p(2)*p(98) - p(174)
p(192) = p(1)*p(102) - p(191)
p(193) = p(6)*p(50) - p(177)
p(194) = p(1)*p(101) - p(180)
p(195) = p(2)*p(102) - p(193)
p(196) = p(9)*p(26)
p(197) = p(9)*p(38)
p(198) = p(9)*p(24)
p(199) = p(9)*p(31)
p(200) = p(9)*p(25)
p(201) = p(9)*p(32)
p(202) = p(14)*p(39)
p(203) = p(14)*p(40)
p(204) = p(9)*p(35)
p(205) = p(9)*p(36)
p(206) = p(9)*p(37)
p(207) = p(14)*p(28)
p(208) = p(14)*p(29)
p(209) = p(14)*p(30)
p(210) = p(9)*p(45)
p(211) = p(14)*p(24)
p(212) = p(14)*p(25)
p(213) = p(9)*p(48)
p(214) = p(14)*p(33)
p(215) = p(14)*p(34)
p(216) = p(9)*p(41)
p(217) = p(9)*p(28)
p(218) = p(9)*p(23)
p(219) = p(9)*p(29)
p(220) = p(9)*p(30)
p(221) = p(9)*p(42)
p(222) = p(1)*p(114) - p(198)
p(223) = p(21)*p(24) - p(201)
p(224) = p(3)*p(72) - p(199) - p(223)
p(225) = p(1)*p(116) - p(199) - p(224)
p(226) = p(13)*p(39) - p(198)
p(227) = p(3)*p(73) - p(201)
p(228) = p(14)*p(49)
p(229) = p(9)*p(43)
p(230) = p(9)*p(33)
p(231) = p(9)*p(34)
p(232) = p(9)*p(44)
p(233) = p(16)*p(24) - p(196)
p(234) = p(2)*p(114) - p(202)
p(235) = p(1)*p(123) - p(205) - p(234)
p(236) = p(13)*p(41) - p(228)
p(237) = p(2)*p(116) - p(203) - p(236)
p(238) = p(16)*p(32) - p(228)
p(239) = p(6)*p(69) - p(202)
p(240) = p(4)*p(73) - p(228)
p(241) = p(11)*p(32) - p(196)
p(242) = p(14)*p(41)
p(243) = p(14)*p(42)
p(244) = p(1)*p(130) - p(210)
p(245) = p(4)*p(62) - p(213) - p(209)
p(246) = p(3)*p(81) - p(210)
p(247) = p(14)*p(31)
p(248) = p(14)*p(32)
p(249) = p(14)*p(26)
p(250) = p(9)*p(46)
p(251) = p(9)*p(47)
p(252) = p(18)*p(24) - p(197)
p(253) = p(1)*p(135) - p(213) - p(252)
p(254) = p(13)*p(43) - p(242)
p(255) = p(2)*p(125) - p(209) - p(245)
p(256) = p(18)*p(32) - p(242)
p(257) = p(13)*p(34) - p(197)
p(258) = p(14)*p(43)
p(259) = p(14)*p(44)
p(260) = p(2)*p(130) - p(211)
p(261) = p(4)*p(81) - p(248)
p(262) = p(11)*p(45) - p(211)
p(263) = p(14)*p(35)
p(264) = p(14)*p(36)
p(265) = p(14)*p(37)
p(266) = p(9)*p(50)
p(267) = p(22)*p(24) - p(215)
p(268) = p(13)*p(46) - p(258)
p(269) = p(6)*p(84) - p(215)
p(270) = p(14)*p(46)
p(271) = p(14)*p(47)
p(272) = p(9)*p(27)
p(273) = p(9)*p(39)
p(274) = p(9)*p(40)
p(275) = p(9)*p(49)
p(276) = p(3)*p(91) - p(216)
p(277) = p(16)*p(39) - p(218)
p(278) = p(1)*p(144) - p(217) - p(277)
p(279) = p(2)*p(178) - p(278)
p(280) = p(1)*p(145) - p(219) - p(278)
p(281) = p(7)*p(69) - p(218)
p(282) = p(1)*p(147) - p(221) - p(280)
p(283) = p(11)*p(49) - p(216)
p(284) = p(1)*p(149) - p(224) - p(223)
p(285) = p(1)*p(150) - p(227)
p(286) = p(3)*p(93) - p(229)
p(287) = p(18)*p(39) - p(231)
p(288) = p(1)*p(152) - p(230) - p(287)
p(289) = p(1)*p(153) - p(231)
p(290) = p(1)*p(154) - p(232) - p(288)
p(291) = p(21)*p(34) - p(229)
p(292) = p(6)*p(91) - p(228)
p(293) = p(1)*p(157) - p(237) - p(235)
p(294) = p(2)*p(150) - p(228)
p(295) = p(1)*p(159) - p(245) - p(244)
p(296) = p(1)*p(160) - p(246)
p(297) = p(3)*p(95) - p(250)
p(298) = p(2)*p(152) - p(238) - p(234)
p(299) = p(2)*p(153) - p(239)
p(300) = p(1)*p(163) - p(251) - p(298)
p(301) = p(7)*p(84) - p(250)
p(302) = p(6)*p(93) - p(242)
p(303) = p(1)*p(166) - p(255) - p(253)
p(304) = p(8)*p(73) - p(242)
p(305) = p(16)*p(45) - p(248)
p(306) = p(1)*p(169) - p(262) - p(260)
p(307) = p(2)*p(160) - p(248)
p(308) = p(14)*p(38)
p(309) = p(6)*p(80) - p(264) - p(263)
p(310) = p(1)*p(187) - p(309)
p(311) = p(14)*p(45)
p(312) = p(3)*p(98) - p(266)
p(313) = p(2)*p(163) - p(256) - p(253)
p(314) = p(1)*p(173) - p(266)
p(315) = p(6)*p(95) - p(258)
p(316) = p(1)*p(175) - p(269) - p(267)
p(317) = p(22)*p(32) - p(258)
p(318) = p(18)*p(45) - p(249)
p(319) = p(2)*p(169) - p(264) - p(309)
p(320) = p(8)*p(81) - p(249)
p(321) = p(14)*p(48)
p(322) = p(2)*p(172) - p(268) - p(267)
p(323) = p(2)*p(173) - p(269)
p(324) = p(6)*p(98) - p(270)
p(325) = p(2)*p(175) - p(271) - p(319)
p(326) = p(13)*p(50) - p(270)
p(327) = p(14)*p(50)
p(328) = p(1)*p(178) - p(273)
p(329) = p(21)*p(39) - p(272)
p(330) = p(3)*p(101) - p(275)
p(331) = p(1)*p(181) - p(276)
p(332) = p(2)*p(194) - p(331)
p(333) = p(1)*p(183) - p(286)
p(334) = p(8)*p(101) - p(333)
p(335) = p(1)*p(185) - p(297)
p(336) = p(22)*p(49) - p(335)
p(337) = p(1)*p(188) - p(312)
p(338) = p(21)*p(50) - p(337)
p(339) = p(2)*p(187) - p(311)
p(340) = p(1)*p(191) - p(322)
p(341) = p(7)*p(102) - p(340)
p(342) = p(22)*p(45) - p(308)
p(343) = p(2)*p(191) - p(324)
p(344) = p(1)*p(195) - p(343)
p(345) = p(6)*p(102) - p(327)
p(346) = p(1)*p(194) - p(330)
p(347) = p(2)*p(195) - p(345)
p(348) = p(9)*p(52)
p(349) = p(9)*p(58)
p(350) = p(9)*p(63)
p(351) = p(9)*p(64)
p(352) = p(9)*p(82)
p(353) = p(9)*p(88)
p(354) = p(9)*p(55)
p(355) = p(9)*p(72)
p(356) = p(9)*p(51)
p(357) = p(9)*p(56)
p(358) = p(9)*p(57)
p(359) = p(9)*p(73)
p(360) = p(14)*p(89)
p(361) = p(14)*p(90)
p(362) = p(9)*p(77)
p(363) = p(9)*p(60)
p(364) = p(9)*p(78)
p(365) = p(9)*p(61)
p(366) = p(9)*p(62)
p(367) = p(9)*p(79)
p(368) = p(14)*p(67)
p(369) = p(14)*p(68)
p(370) = p(14)*p(69)
p(371) = p(14)*p(70)
p(372) = p(14)*p(71)
p(373) = p(9)*p(80)
p(374) = p(9)*p(81)
p(375) = p(14)*p(55)
p(376) = p(14)*p(56)
p(377) = p(14)*p(57)
p(378) = p(14)*p(52)
p(379) = p(9)*p(85)
p(380) = p(9)*p(86)
p(381) = p(9)*p(87)
p(382) = p(14)*p(74)
p(383) = p(14)*p(75)
p(384) = p(14)*p(76)
p(385) = p(9)*p(97)
p(386) = p(14)*p(60)
p(387) = p(14)*p(61)
p(388) = p(14)*p(62)
p(389) = p(9)*p(100)
p(390) = p(14)*p(83)
p(391) = p(14)*p(84)
p(392) = p(9)*p(91)
p(393) = p(9)*p(67)
p(394) = p(9)*p(53)
p(395) = p(9)*p(68)
p(396) = p(9)*p(54)
p(397) = p(9)*p(69)
p(398) = p(9)*p(70)
p(399) = p(9)*p(71)
p(400) = p(9)*p(92)
p(401) = p(3)*p(114) - p(354)
p(402) = p(7)*p(114) - p(356)
p(403) = p(24)*p(49) - p(359)
p(404) = p(3)*p(149) - p(355) - p(403)
p(405) = p(1)*p(224) - p(355) - p(404)
p(406) = p(6)*p(178) - p(401)
p(407) = p(32)*p(39) - p(356)
p(408) = p(3)*p(150) - p(359)
p(409) = p(14)*p(101)
p(410) = p(9)*p(93)
p(411) = p(9)*p(74)
p(412) = p(9)*p(59)
p(413) = p(9)*p(75)
p(414) = p(9)*p(76)
p(415) = p(9)*p(94)
p(416) = p(24)*p(41) - p(349)
p(417) = p(4)*p(114) - p(348)
p(418) = p(1)*p(234) - p(363) - p(417)
p(419) = p(1)*p(235) - p(364) - p(418)
p(420) = p(13)*p(91) - p(409)
p(421) = p(2)*p(224) - p(361) - p(420)
p(422) = p(32)*p(41) - p(409)
p(423) = p(1)*p(237) - p(364) - p(421)
p(424) = p(16)*p(73) - p(409)
p(425) = p(13)*p(69) - p(348)
p(426) = p(4)*p(150) - p(409)
p(427) = p(11)*p(73) - p(349)
p(428) = p(14)*p(91)
p(429) = p(14)*p(92)
p(430) = p(3)*p(130) - p(373)
p(431) = p(7)*p(130) - p(374)
p(432) = p(3)*p(159) - p(373) - p(431)
p(433) = p(39)*p(45) - p(430)
p(434) = p(3)*p(160) - p(374)
p(435) = p(14)*p(72)
p(436) = p(14)*p(73)
p(437) = p(14)*p(58)
p(438) = p(9)*p(95)
p(439) = p(9)*p(83)
p(440) = p(9)*p(84)
p(441) = p(9)*p(96)
p(442) = p(24)*p(43) - p(350)
p(443) = p(8)*p(114) - p(370)
p(444) = p(1)*p(253) - p(380) - p(443)
p(445) = p(13)*p(93) - p(428)
p(446) = p(2)*p(237) - p(372) - p(432)
p(447) = p(32)*p(43) - p(428)
p(448) = p(6)*p(153) - p(370)
p(449) = p(18)*p(73) - p(428)
p(450) = p(32)*p(34) - p(350)
p(451) = p(14)*p(93)
p(452) = p(14)*p(94)
p(453) = p(4)*p(130) - p(352)
p(454) = p(1)*p(260) - p(385) - p(453)
p(455) = p(16)*p(81) - p(436)
p(456) = p(2)*p(245) - p(376) - p(455)
p(457) = p(4)*p(160) - p(436)
p(458) = p(11)*p(81) - p(352)
p(459) = p(14)*p(77)
p(460) = p(14)*p(78)
p(461) = p(14)*p(79)
p(462) = p(14)*p(63)
p(463) = p(14)*p(64)
p(464) = p(6)*p(130) - p(386)
p(465) = p(3)*p(187) - p(464)
p(466) = p(14)*p(80)
p(467) = p(14)*p(81)
p(468) = p(9)*p(98)
p(469) = p(9)*p(99)
p(470) = p(24)*p(46) - p(353)
p(471) = p(1)*p(267) - p(389) - p(470)
p(472) = p(13)*p(95) - p(451)
p(473) = p(2)*p(255) - p(384) - p(456)
p(474) = p(32)*p(46) - p(451)
p(475) = p(13)*p(84) - p(353)
p(476) = p(14)*p(95)
p(477) = p(14)*p(96)
p(478) = p(8)*p(130) - p(378)
p(479) = p(18)*p(81) - p(437)
p(480) = p(34)*p(45) - p(378)
p(481) = p(14)*p(85)
p(482) = p(14)*p(86)
p(483) = p(14)*p(87)
p(484) = p(9)*p(102)
p(485) = p(24)*p(50) - p(391)
p(486) = p(13)*p(98) - p(476)
p(487) = p(6)*p(173) - p(391)
p(488) = p(14)*p(98)
p(489) = p(14)*p(99)
p(490) = p(9)*p(65)
p(491) = p(9)*p(66)
p(492) = p(9)*p(89)
p(493) = p(9)*p(90)
p(494) = p(9)*p(101)
p(495) = p(3)*p(181) - p(392)
p(496) = p(39)*p(41) - p(394)
p(497) = p(4)*p(178) - p(397)
p(498) = p(1)*p(278) - p(395) - p(497)
p(499) = p(1)*p(279) - p(397)
p(500) = p(1)*p(280) - p(398) - p(498)
p(501) = p(21)*p(69) - p(394)
p(502) = p(1)*p(282) - p(400) - p(500)
p(503) = p(11)*p(101) - p(392)
p(504) = p(1)*p(284) - p(404) - p(403)
p(505) = p(1)*p(285) - p(408)
p(506) = p(3)*p(183) - p(410)
p(507) = p(39)*p(43) - p(412)
p(508) = p(1)*p(287) - p(411) - p(507)
p(509) = p(2)*p(279) - p(406)
p(510) = p(1)*p(288) - p(413) - p(508)
p(511) = p(7)*p(153) - p(412)
p(512) = p(1)*p(290) - p(415) - p(510)
p(513) = p(34)*p(49) - p(410)
p(514) = p(6)*p(181) - p(409)
p(515) = p(1)*p(293) - p(421) - p(419)
p(516) = p(2)*p(285) - p(409)
p(517) = p(1)*p(295) - p(432) - p(431)
p(518) = p(1)*p(296) - p(434)
p(519) = p(3)*p(185) - p(438)
p(520) = p(39)*p(46) - p(440)
p(521) = p(1)*p(298) - p(439) - p(520)
p(522) = p(1)*p(299) - p(440)
p(523) = p(1)*p(300) - p(441) - p(521)
p(524) = p(21)*p(84) - p(438)
p(525) = p(6)*p(183) - p(428)
p(526) = p(1)*p(303) - p(446) - p(444)
p(527) = p(8)*p(150) - p(428)
p(528) = p(41)*p(45) - p(436)
p(529) = p(1)*p(306) - p(456) - p(454)
p(530) = p(2)*p(296) - p(436)
p(531) = p(4)*p(169) - p(388) - p(478)
p(532) = p(1)*p(310) - p(465)
p(533) = p(14)*p(82)
p(534) = p(3)*p(188) - p(468)
p(535) = p(2)*p(298) - p(447) - p(443)
p(536) = p(2)*p(299) - p(448)
p(537) = p(1)*p(313) - p(469) - p(535)
p(538) = p(7)*p(173) - p(468)
p(539) = p(6)*p(185) - p(451)
p(540) = p(1)*p(316) - p(473) - p(471)
p(541) = p(22)*p(73) - p(451)
p(542) = p(43)*p(45) - p(437)
p(543) = p(1)*p(319) - p(480) - p(478)
p(544) = p(8)*p(160) - p(437)
p(545) = p(14)*p(88)
p(546) = p(4)*p(187) - p(467)
p(547) = p(2)*p(309) - p(466) - p(546)
p(548) = p(2)*p(310) - p(467)
p(549) = p(14)*p(97)
p(550) = p(3)*p(191) - p(484)
p(551) = p(2)*p(313) - p(474) - p(471)
p(552) = p(1)*p(323) - p(484)
p(553) = p(6)*p(188) - p(476)
p(554) = p(1)*p(325) - p(487) - p(485)
p(555) = p(32)*p(50) - p(476)
p(556) = p(45)*p(46) - p(462)
p(557) = p(2)*p(319) - p(482) - p(547)
p(558) = p(22)*p(81) - p(462)
p(559) = p(14)*p(100)
p(560) = p(2)*p(322) - p(486) - p(485)
p(561) = p(2)*p(323) - p(487)
p(562) = p(6)*p(191) - p(488)
p(563) = p(2)*p(325) - p(489) - p(557)
p(564) = p(13)*p(102) - p(488)
p(565) = p(14)*p(102)
p(566) = p(3)*p(178) - p(492)
p(567) = p(7)*p(178) - p(490)
p(568) = p(39)*p(49) - p(491)
p(569) = p(3)*p(194) - p(494)
p(570) = p(1)*p(331) - p(495)
p(571) = p(2)*p(346) - p(570)
p(572) = p(1)*p(333) - p(506)
p(573) = p(8)*p(194) - p(572)
p(574) = p(1)*p(335) - p(519)
p(575) = p(22)*p(101) - p(574)
p(576) = p(1)*p(337) - p(534)
p(577) = p(49)*p(50) - p(576)
p(578) = p(6)*p(187) - p(549)
p(579) = p(1)*p(340) - p(550)
p(580) = p(21)*p(102) - p(579)
p(581) = p(8)*p(187) - p(533)
p(582) = p(1)*p(343) - p(560)
p(583) = p(7)*p(195) - p(582)
p(584) = p(45)*p(50) - p(545)
p(585) = p(2)*p(343) - p(562)
p(586) = p(1)*p(347) - p(585)
p(587) = p(6)*p(195) - p(565)
p(588) = p(1)*p(346) - p(569)
p(589) = p(2)*p(347) - p(587)
p(590) = p(9)*p(103)
p(591) = p(9)*p(106)
p(592) = p(9)*p(119)
p(593) = p(9)*p(128)
p(594) = p(9)*p(108)
p(595) = p(9)*p(109)
p(596) = p(9)*p(129)
p(597) = p(9)*p(132)
p(598) = p(9)*p(133)
p(599) = p(9)*p(161)
p(600) = p(9)*p(138)
p(601) = p(9)*p(139)
p(602) = p(9)*p(171)
p(603) = p(9)*p(177)
p(604) = p(9)*p(114)
p(605) = p(9)*p(115)
p(606) = p(9)*p(149)
p(607) = p(9)*p(104)
p(608) = p(9)*p(116)
p(609) = p(9)*p(105)
p(610) = p(9)*p(117)
p(611) = p(9)*p(118)
p(612) = p(9)*p(150)
p(613) = p(14)*p(178)
p(614) = p(14)*p(179)
p(615) = p(14)*p(180)
p(616) = p(9)*p(156)
p(617) = p(9)*p(122)
p(618) = p(9)*p(123)
p(619) = p(9)*p(157)
p(620) = p(9)*p(124)
p(621) = p(9)*p(107)
p(622) = p(9)*p(125)
p(623) = p(9)*p(126)
p(624) = p(9)*p(127)
p(625) = p(9)*p(158)
p(626) = p(14)*p(143)
p(627) = p(14)*p(144)
p(628) = p(14)*p(145)
p(629) = p(14)*p(146)
p(630) = p(14)*p(147)
p(631) = p(14)*p(148)
p(632) = p(9)*p(130)
p(633) = p(9)*p(159)
p(634) = p(9)*p(131)
p(635) = p(9)*p(160)
p(636) = p(14)*p(114)
p(637) = p(14)*p(115)
p(638) = p(14)*p(116)
p(639) = p(14)*p(117)
p(640) = p(14)*p(118)
p(641) = p(14)*p(106)
p(642) = p(9)*p(165)
p(643) = p(9)*p(135)
p(644) = p(9)*p(166)
p(645) = p(9)*p(136)
p(646) = p(9)*p(137)
p(647) = p(9)*p(167)
p(648) = p(14)*p(151)
p(649) = p(14)*p(152)
p(650) = p(14)*p(153)
p(651) = p(14)*p(154)
p(652) = p(14)*p(155)
p(653) = p(9)*p(168)
p(654) = p(9)*p(169)
p(655) = p(9)*p(170)
p(656) = p(14)*p(122)
p(657) = p(14)*p(123)
p(658) = p(14)*p(124)
p(659) = p(14)*p(125)
p(660) = p(14)*p(126)
p(661) = p(14)*p(127)
p(662) = p(14)*p(108)
p(663) = p(14)*p(109)
p(664) = p(9)*p(187)
p(665) = p(14)*p(130)
p(666) = p(14)*p(131)
p(667) = p(9)*p(174)
p(668) = p(9)*p(175)
p(669) = p(9)*p(176)
p(670) = p(14)*p(162)
p(671) = p(14)*p(163)
p(672) = p(14)*p(164)
p(673) = p(9)*p(190)
p(674) = p(14)*p(135)
p(675) = p(14)*p(136)
p(676) = p(14)*p(137)
p(677) = p(9)*p(193)
p(678) = p(14)*p(172)
p(679) = p(14)*p(173)
p(680) = p(9)*p(181)
p(681) = p(9)*p(143)
p(682) = p(9)*p(110)
p(683) = p(9)*p(144)
p(684) = p(9)*p(111)
p(685) = p(9)*p(112)
p(686) = p(9)*p(145)
p(687) = p(9)*p(113)
p(688) = p(9)*p(146)
p(689) = p(9)*p(147)
p(690) = p(9)*p(148)
p(691) = p(9)*p(182)
p(692) = p(1)*p(401) - p(604)
p(693) = p(21)*p(114) - p(609)
p(694) = p(24)*p(101) - p(612)
p(695) = p(3)*p(284) - p(606) - p(694)
p(696) = p(1)*p(404) - p(606) - p(695)
p(697) = p(1)*p(405) - p(608) - p(696)
p(698) = p(13)*p(178) - p(604)
p(699) = p(39)*p(73) - p(609)
p(700) = p(3)*p(285) - p(612)
p(701) = p(14)*p(194)
p(702) = p(9)*p(183)
p(703) = p(9)*p(151)
p(704) = p(9)*p(120)
p(705) = p(9)*p(152)
p(706) = p(9)*p(121)
p(707) = p(9)*p(153)
p(708) = p(9)*p(154)
p(709) = p(9)*p(155)
p(710) = p(9)*p(184)
p(711) = p(24)*p(91) - p(592)
p(712) = p(1)*p(416) - p(616) - p(711)
p(713) = p(2)*p(401) - p(613)
p(714) = p(1)*p(418) - p(618) - p(713)
p(715) = p(1)*p(419) - p(619) - p(714)
p(716) = p(13)*p(181) - p(701)
p(717) = p(2)*p(404) - p(615) - p(716)
p(718) = p(32)*p(91) - p(701)
p(719) = p(1)*p(421) - p(619) - p(717)
p(720) = p(41)*p(73) - p(701)
p(721) = p(6)*p(279) - p(613)
p(722) = p(16)*p(150) - p(701)
p(723) = p(1)*p(425) - p(624) - p(721)
p(724) = p(4)*p(285) - p(701)
p(725) = p(11)*p(150) - p(592)
p(726) = p(14)*p(181)
p(727) = p(14)*p(182)
p(728) = p(1)*p(430) - p(632)
p(729) = p(21)*p(130) - p(635)
p(730) = p(3)*p(295) - p(633) - p(729)
p(731) = p(1)*p(432) - p(633) - p(730)
p(732) = p(39)*p(81) - p(632)
p(733) = p(3)*p(296) - p(635)
p(734) = p(14)*p(149)
p(735) = p(14)*p(150)
p(736) = p(14)*p(119)
p(737) = p(9)*p(185)
p(738) = p(9)*p(162)
p(739) = p(9)*p(134)
p(740) = p(9)*p(163)
p(741) = p(9)*p(164)
p(742) = p(9)*p(186)
p(743) = p(24)*p(93) - p(593)
p(744) = p(18)*p(114) - p(595)
p(745) = p(1)*p(443) - p(643) - p(744)
p(746) = p(1)*p(444) - p(644) - p(745)
p(747) = p(13)*p(183) - p(726)
p(748) = p(2)*p(421) - p(631) - p(730)
p(749) = p(32)*p(93) - p(726)
p(750) = p(1)*p(446) - p(644) - p(748)
p(751) = p(43)*p(73) - p(726)
p(752) = p(13)*p(153) - p(595)
p(753) = p(18)*p(150) - p(726)
p(754) = p(34)*p(73) - p(593)
p(755) = p(14)*p(183)
p(756) = p(14)*p(184)
p(757) = p(16)*p(130) - p(598)
p(758) = p(2)*p(430) - p(636)
p(759) = p(1)*p(454) - p(654) - p(758)
p(760) = p(41)*p(81) - p(735)
p(761) = p(2)*p(432) - p(638) - p(760)
p(762) = p(16)*p(160) - p(735)
p(763) = p(45)*p(69) - p(636)
p(764) = p(4)*p(296) - p(735)
p(765) = p(11)*p(160) - p(598)
p(766) = p(14)*p(156)
p(767) = p(14)*p(157)
p(768) = p(14)*p(158)
p(769) = p(14)*p(128)
p(770) = p(14)*p(129)
p(771) = p(1)*p(464) - p(664)
p(772) = p(4)*p(262) - p(673) - p(661)
p(773) = p(3)*p(310) - p(664)
p(774) = p(14)*p(159)
p(775) = p(14)*p(160)
p(776) = p(14)*p(132)
p(777) = p(14)*p(133)
p(778) = p(9)*p(188)
p(779) = p(9)*p(172)
p(780) = p(9)*p(173)
p(781) = p(9)*p(189)
p(782) = p(24)*p(95) - p(600)
p(783) = p(22)*p(114) - p(650)
p(784) = p(1)*p(471) - p(668) - p(783)
p(785) = p(13)*p(185) - p(755)
p(786) = p(2)*p(446) - p(652) - p(761)
p(787) = p(32)*p(95) - p(755)
p(788) = p(6)*p(299) - p(650)
p(789) = p(46)*p(73) - p(755)
p(790) = p(32)*p(84) - p(600)
p(791) = p(14)*p(185)
p(792) = p(14)*p(186)
p(793) = p(2)*p(453) - p(656) - p(771)
p(794) = p(1)*p(478) - p(673) - p(793)
p(795) = p(43)*p(81) - p(736)
p(796) = p(2)*p(456) - p(659) - p(772)
p(797) = p(18)*p(160) - p(736)
p(798) = p(1)*p(480) - p(673) - p(796)
p(799) = p(14)*p(165)
p(800) = p(14)*p(166)
p(801) = p(14)*p(167)
p(802) = p(14)*p(138)
p(803) = p(14)*p(139)
p(804) = p(2)*p(464) - p(665)
p(805) = p(4)*p(310) - p(775)
p(806) = p(11)*p(187) - p(665)
p(807) = p(14)*p(168)
p(808) = p(14)*p(169)
p(809) = p(14)*p(170)
p(810) = p(9)*p(191)
p(811) = p(9)*p(192)
p(812) = p(24)*p(98) - p(603)
p(813) = p(1)*p(485) - p(677) - p(812)
p(814) = p(13)*p(188) - p(791)
p(815) = p(2)*p(473) - p(672) - p(796)
p(816) = p(32)*p(98) - p(791)
p(817) = p(13)*p(173) - p(603)
p(818) = p(14)*p(188)
p(819) = p(14)*p(189)
p(820) = p(22)*p(130) - p(663)
p(821) = p(46)*p(81) - p(769)
p(822) = p(45)*p(84) - p(663)
p(823) = p(14)*p(174)
p(824) = p(14)*p(175)
p(825) = p(14)*p(176)
p(826) = p(9)*p(195)
p(827) = p(24)*p(102) - p(679)
p(828) = p(13)*p(191) - p(818)
p(829) = p(6)*p(323) - p(679)
p(830) = p(14)*p(191)
p(831) = p(14)*p(192)
p(832) = p(9)*p(140)
p(833) = p(9)*p(141)
p(834) = p(9)*p(178)
p(835) = p(9)*p(142)
p(836) = p(9)*p(179)
p(837) = p(9)*p(180)
p(838) = p(9)*p(194)
p(839) = p(3)*p(331) - p(680)
p(840) = p(39)*p(91) - p(682)
p(841) = p(16)*p(178) - p(685)
p(842) = p(1)*p(497) - p(683) - p(841)
p(843) = p(2)*p(566) - p(842)
p(844) = p(1)*p(498) - p(686) - p(842)
p(845) = p(7)*p(279) - p(685)
p(846) = p(1)*p(500) - p(689) - p(844)
p(847) = p(49)*p(69) - p(682)
p(848) = p(1)*p(502) - p(691) - p(846)
p(849) = p(11)*p(194) - p(680)
p(850) = p(1)*p(504) - p(695) - p(694)
p(851) = p(1)*p(505) - p(700)
p(852) = p(3)*p(333) - p(702)
p(853) = p(39)*p(93) - p(704)
p(854) = p(18)*p(178) - p(707)
p(855) = p(1)*p(508) - p(705) - p(854)
p(856) = p(1)*p(509) - p(707)
p(857) = p(1)*p(510) - p(708) - p(855)
p(858) = p(21)*p(153) - p(704)
p(859) = p(1)*p(512) - p(710) - p(857)
p(860) = p(34)*p(101) - p(702)
p(861) = p(6)*p(331) - p(701)
p(862) = p(1)*p(515) - p(717) - p(715)
p(863) = p(2)*p(505) - p(701)
p(864) = p(1)*p(517) - p(730) - p(729)
p(865) = p(1)*p(518) - p(733)
p(866) = p(3)*p(335) - p(737)
p(867) = p(39)*p(95) - p(739)
p(868) = p(1)*p(520) - p(738) - p(867)
p(869) = p(2)*p(509) - p(721)
p(870) = p(1)*p(521) - p(740) - p(868)
p(871) = p(7)*p(299) - p(739)
p(872) = p(1)*p(523) - p(742) - p(870)
p(873) = p(49)*p(84) - p(737)
p(874) = p(6)*p(333) - p(726)
p(875) = p(1)*p(526) - p(748) - p(746)
p(876) = p(8)*p(285) - p(726)
p(877) = p(45)*p(91) - p(735)
p(878) = p(1)*p(529) - p(761) - p(759)
p(879) = p(2)*p(518) - p(735)
p(880) = p(1)*p(531) - p(772) - p(771)
p(881) = p(1)*p(532) - p(773)
p(882) = p(14)*p(161)
p(883) = p(3)*p(337) - p(778)
p(884) = p(39)*p(98) - p(780)
p(885) = p(1)*p(535) - p(779) - p(884)
p(886) = p(1)*p(536) - p(780)
p(887) = p(1)*p(537) - p(781) - p(885)
p(888) = p(21)*p(173) - p(778)
p(889) = p(6)*p(335) - p(755)
p(890) = p(1)*p(540) - p(786) - p(784)
p(891) = p(22)*p(150) - p(755)
p(892) = p(45)*p(93) - p(736)
p(893) = p(1)*p(543) - p(796) - p(794)
p(894) = p(8)*p(296) - p(736)
p(895) = p(16)*p(187) - p(775)
p(896) = p(1)*p(547) - p(806) - p(804)
p(897) = p(2)*p(532) - p(775)
p(898) = p(14)*p(171)
p(899) = p(6)*p(309) - p(808) - p(807)
p(900) = p(1)*p(578) - p(899)
p(901) = p(14)*p(187)
p(902) = p(3)*p(340) - p(810)
p(903) = p(2)*p(535) - p(787) - p(783)
p(904) = p(2)*p(536) - p(788)
p(905) = p(1)*p(551) - p(811) - p(903)
p(906) = p(7)*p(323) - p(810)
p(907) = p(6)*p(337) - p(791)
p(908) = p(1)*p(554) - p(815) - p(813)
p(909) = p(50)*p(73) - p(791)
p(910) = p(45)*p(95) - p(769)
p(911) = p(1)*p(557) - p(822) - p(820)
p(912) = p(22)*p(160) - p(769)
p(913) = p(14)*p(177)
p(914) = p(18)*p(187) - p(777)
p(915) = p(2)*p(547) - p(808) - p(899)
p(916) = p(8)*p(310) - p(777)
p(917) = p(14)*p(190)
p(918) = p(3)*p(343) - p(826)
p(919) = p(2)*p(551) - p(816) - p(813)
p(920) = p(1)*p(561) - p(826)
p(921) = p(6)*p(340) - p(818)
p(922) = p(1)*p(563) - p(829) - p(827)
p(923) = p(32)*p(102) - p(818)
p(924) = p(45)*p(98) - p(802)
p(925) = p(2)*p(557) - p(824) - p(915)
p(926) = p(50)*p(81) - p(802)
p(927) = p(14)*p(193)
p(928) = p(2)*p(560) - p(828) - p(827)
p(929) = p(2)*p(561) - p(829)
p(930) = p(6)*p(343) - p(830)
p(931) = p(2)*p(563) - p(831) - p(925)
p(932) = p(13)*p(195) - p(830)
p(933) = p(14)*p(195)
p(934) = p(1)*p(566) - p(834)
p(935) = p(1)*p(567) - p(836) - p(934)
p(936) = p(39)*p(101) - p(835)
p(937) = p(3)*p(346) - p(838)
p(938) = p(1)*p(570) - p(839)
p(939) = p(2)*p(588) - p(938)
p(940) = p(1)*p(572) - p(852)
p(941) = p(8)*p(346) - p(940)
p(942) = p(1)*p(574) - p(866)
p(943) = p(22)*p(194) - p(942)
p(944) = p(1)*p(576) - p(883)
p(945) = p(50)*p(101) - p(944)
p(946) = p(1)*p(579) - p(902)
p(947) = p(49)*p(102) - p(946)
p(948) = p(2)*p(578) - p(901)
p(949) = p(1)*p(582) - p(918)
p(950) = p(21)*p(195) - p(949)
p(951) = p(2)*p(581) - p(917) - p(948)
p(952) = p(1)*p(585) - p(928)
p(953) = p(7)*p(347) - p(952)
p(954) = p(45)*p(102) - p(913)
p(955) = p(2)*p(585) - p(930)
p(956) = p(1)*p(589) - p(955)
p(957) = p(6)*p(347) - p(933)
p(958) = p(1)*p(588) - p(937)
p(959) = p(2)*p(589) - p(957)
p(960) = p(9)*p(196)
p(961) = p(9)*p(202)
p(962) = p(9)*p(203)
p(963) = p(9)*p(228)
p(964) = p(9)*p(242)
p(965) = p(9)*p(207)
p(966) = p(9)*p(197)
p(967) = p(9)*p(208)
p(968) = p(9)*p(209)
p(969) = p(9)*p(243)
p(970) = p(9)*p(211)
p(971) = p(9)*p(247)
p(972) = p(9)*p(212)
p(973) = p(9)*p(248)
p(974) = p(9)*p(249)
p(975) = p(9)*p(258)
p(976) = p(9)*p(214)
p(977) = p(9)*p(215)
p(978) = p(9)*p(259)
p(979) = p(9)*p(263)
p(980) = p(9)*p(264)
p(981) = p(9)*p(265)
p(982) = p(9)*p(308)
p(983) = p(9)*p(311)
p(984) = p(9)*p(270)
p(985) = p(9)*p(271)
p(986) = p(9)*p(321)
p(987) = p(9)*p(327)
p(988) = p(9)*p(222)
p(989) = p(9)*p(223)
p(990) = p(9)*p(284)
p(991) = p(9)*p(198)
p(992) = p(9)*p(199)
p(993) = p(9)*p(224)
p(994) = p(9)*p(200)
p(995) = p(9)*p(225)
p(996) = p(9)*p(201)
p(997) = p(9)*p(226)
p(998) = p(9)*p(227)
p(999) = p(9)*p(285)
p(1000) = p(14)*p(328)
p(1001) = p(14)*p(329)
p(1002) = p(14)*p(330)
p(1003) = p(9)*p(292)
p(1004) = p(9)*p(233)
p(1005) = p(9)*p(234)
p(1006) = p(9)*p(235)
p(1007) = p(9)*p(293)
p(1008) = p(9)*p(236)
p(1009) = p(9)*p(204)
p(1010) = p(9)*p(205)
p(1011) = p(9)*p(237)
p(1012) = p(9)*p(238)
p(1013) = p(9)*p(206)
p(1014) = p(9)*p(239)
p(1015) = p(9)*p(240)
p(1016) = p(9)*p(241)
p(1017) = p(9)*p(294)
p(1018) = p(14)*p(276)
p(1019) = p(14)*p(277)
p(1020) = p(14)*p(278)
p(1021) = p(14)*p(279)
p(1022) = p(14)*p(280)
p(1023) = p(14)*p(281)
p(1024) = p(14)*p(282)
p(1025) = p(14)*p(283)
p(1026) = p(9)*p(244)
p(1027) = p(9)*p(295)
p(1028) = p(9)*p(210)
p(1029) = p(9)*p(245)
p(1030) = p(9)*p(246)
p(1031) = p(9)*p(296)
p(1032) = p(14)*p(222)
p(1033) = p(14)*p(223)
p(1034) = p(14)*p(224)
p(1035) = p(14)*p(225)
p(1036) = p(14)*p(226)
p(1037) = p(14)*p(227)
p(1038) = p(14)*p(202)
p(1039) = p(14)*p(203)
p(1040) = p(9)*p(302)
p(1041) = p(9)*p(252)
p(1042) = p(9)*p(253)
p(1043) = p(9)*p(303)
p(1044) = p(9)*p(254)
p(1045) = p(9)*p(213)
p(1046) = p(9)*p(255)
p(1047) = p(9)*p(256)
p(1048) = p(9)*p(257)
p(1049) = p(9)*p(304)
p(1050) = p(14)*p(286)
p(1051) = p(14)*p(287)
p(1052) = p(14)*p(288)
p(1053) = p(14)*p(289)
p(1054) = p(14)*p(290)
p(1055) = p(14)*p(291)
p(1056) = p(9)*p(305)
p(1057) = p(9)*p(260)
p(1058) = p(9)*p(306)
p(1059) = p(9)*p(261)
p(1060) = p(9)*p(262)
p(1061) = p(9)*p(307)
p(1062) = p(14)*p(233)
p(1063) = p(14)*p(234)
p(1064) = p(14)*p(235)
p(1065) = p(14)*p(236)
p(1066) = p(14)*p(237)
p(1067) = p(14)*p(238)
p(1068) = p(14)*p(239)
p(1069) = p(14)*p(240)
p(1070) = p(14)*p(241)
p(1071) = p(14)*p(207)
p(1072) = p(14)*p(208)
p(1073) = p(14)*p(209)
p(1074) = p(9)*p(309)
p(1075) = p(9)*p(310)
p(1076) = p(14)*p(244)
p(1077) = p(14)*p(245)
p(1078) = p(14)*p(246)
p(1079) = p(14)*p(211)
p(1080) = p(14)*p(212)
p(1081) = p(9)*p(315)
p(1082) = p(9)*p(267)
p(1083) = p(9)*p(316)
p(1084) = p(9)*p(268)
p(1085) = p(9)*p(269)
p(1086) = p(9)*p(317)
p(1087) = p(14)*p(297)
p(1088) = p(14)*p(298)
p(1089) = p(14)*p(299)
p(1090) = p(14)*p(300)
p(1091) = p(14)*p(301)
p(1092) = p(9)*p(318)
p(1093) = p(9)*p(319)
p(1094) = p(9)*p(320)
p(1095) = p(14)*p(252)
p(1096) = p(14)*p(253)
p(1097) = p(14)*p(254)
p(1098) = p(14)*p(255)
p(1099) = p(14)*p(256)
p(1100) = p(14)*p(257)
p(1101) = p(14)*p(214)
p(1102) = p(14)*p(215)
p(1103) = p(9)*p(339)
p(1104) = p(14)*p(260)
p(1105) = p(14)*p(261)
p(1106) = p(14)*p(262)
p(1107) = p(9)*p(324)
p(1108) = p(9)*p(325)
p(1109) = p(9)*p(326)
p(1110) = p(14)*p(312)
p(1111) = p(14)*p(313)
p(1112) = p(14)*p(314)
p(1113) = p(9)*p(342)
p(1114) = p(14)*p(267)
p(1115) = p(14)*p(268)
p(1116) = p(14)*p(269)
p(1117) = p(9)*p(345)
p(1118) = p(14)*p(322)
p(1119) = p(14)*p(323)
p(1120) = p(9)*p(331)
p(1121) = p(9)*p(276)
p(1122) = p(9)*p(216)
p(1123) = p(9)*p(277)
p(1124) = p(9)*p(217)
p(1125) = p(9)*p(218)
p(1126) = p(9)*p(278)
p(1127) = p(9)*p(219)
p(1128) = p(9)*p(220)
p(1129) = p(9)*p(279)
p(1130) = p(9)*p(280)
p(1131) = p(9)*p(221)
p(1132) = p(9)*p(281)
p(1133) = p(9)*p(282)
p(1134) = p(9)*p(283)
p(1135) = p(9)*p(332)
p(1136) = p(3)*p(401) - p(988)
p(1137) = p(7)*p(401) - p(991)
p(1138) = p(49)*p(114) - p(996)
p(1139) = p(24)*p(194) - p(999)
p(1140) = p(3)*p(504) - p(990) - p(1139)
p(1141) = p(1)*p(695) - p(990) - p(1140)
p(1142) = p(1)*p(696) - p(993) - p(1141)
p(1143) = p(6)*p(566) - p(1136)
p(1144) = p(32)*p(178) - p(991)
p(1145) = p(39)*p(150) - p(996)
p(1146) = p(3)*p(505) - p(999)
p(1147) = p(14)*p(346)
p(1148) = p(9)*p(333)
p(1149) = p(9)*p(286)
p(1150) = p(9)*p(229)
p(1151) = p(9)*p(287)
p(1152) = p(9)*p(230)
p(1153) = p(9)*p(231)
p(1154) = p(9)*p(288)
p(1155) = p(9)*p(232)
p(1156) = p(9)*p(289)
p(1157) = p(9)*p(290)
p(1158) = p(9)*p(291)
p(1159) = p(9)*p(334)
p(1160) = p(24)*p(181) - p(963)
p(1161) = p(41)*p(114) - p(960)
p(1162) = p(4)*p(401) - p(961)
p(1163) = p(1)*p(713) - p(1005) - p(1162)
p(1164) = p(1)*p(714) - p(1006) - p(1163)
p(1165) = p(1)*p(715) - p(1007) - p(1164)
p(1166) = p(13)*p(331) - p(1147)
p(1167) = p(2)*p(695) - p(1002) - p(1166)
p(1168) = p(32)*p(181) - p(1147)
p(1169) = p(1)*p(717) - p(1007) - p(1167)
p(1170) = p(73)*p(91) - p(1147)
p(1171) = p(1)*p(719) - p(1011) - p(1169)
p(1172) = p(41)*p(150) - p(1147)
p(1173) = p(13)*p(279) - p(961)
p(1174) = p(16)*p(285) - p(1147)
p(1175) = p(69)*p(73) - p(960)
p(1176) = p(4)*p(505) - p(1147)
p(1177) = p(11)*p(285) - p(963)
p(1178) = p(14)*p(331)
p(1179) = p(14)*p(332)
p(1180) = p(3)*p(430) - p(1026)
p(1181) = p(7)*p(430) - p(1028)
p(1182) = p(49)*p(130) - p(1031)
p(1183) = p(3)*p(517) - p(1027) - p(1182)
p(1184) = p(1)*p(730) - p(1027) - p(1183)
p(1185) = p(45)*p(178) - p(1180)
p(1186) = p(39)*p(160) - p(1028)
p(1187) = p(3)*p(518) - p(1031)
p(1188) = p(14)*p(284)
p(1189) = p(14)*p(285)
p(1190) = p(14)*p(228)
p(1191) = p(9)*p(335)
p(1192) = p(9)*p(297)
p(1193) = p(9)*p(250)
p(1194) = p(9)*p(298)
p(1195) = p(9)*p(251)
p(1196) = p(9)*p(299)
p(1197) = p(9)*p(300)
p(1198) = p(9)*p(301)
p(1199) = p(9)*p(336)
p(1200) = p(24)*p(183) - p(964)
p(1201) = p(43)*p(114) - p(966)
p(1202) = p(8)*p(401) - p(1021)
p(1203) = p(1)*p(745) - p(1042) - p(1202)
p(1204) = p(1)*p(746) - p(1043) - p(1203)
p(1205) = p(13)*p(333) - p(1178)
p(1206) = p(2)*p(717) - p(1025) - p(1183)
p(1207) = p(32)*p(183) - p(1178)
p(1208) = p(1)*p(748) - p(1043) - p(1206)
p(1209) = p(73)*p(93) - p(1178)
p(1210) = p(6)*p(509) - p(1021)
p(1211) = p(43)*p(150) - p(1178)
p(1212) = p(32)*p(153) - p(966)
p(1213) = p(18)*p(285) - p(1178)
p(1214) = p(34)*p(150) - p(964)
p(1215) = p(14)*p(333)
p(1216) = p(14)*p(334)
p(1217) = p(41)*p(130) - p(973)
p(1218) = p(4)*p(430) - p(970)
p(1219) = p(1)*p(758) - p(1057) - p(1218)
p(1220) = p(1)*p(759) - p(1058) - p(1219)
p(1221) = p(81)*p(91) - p(1189)
p(1222) = p(2)*p(730) - p(1034) - p(1221)
p(1223) = p(41)*p(160) - p(1189)
p(1224) = p(1)*p(761) - p(1058) - p(1222)
p(1225) = p(16)*p(296) - p(1189)
p(1226) = p(69)*p(81) - p(970)
p(1227) = p(4)*p(518) - p(1189)
p(1228) = p(11)*p(296) - p(973)
p(1229) = p(14)*p(292)
p(1230) = p(14)*p(293)
p(1231) = p(14)*p(294)
p(1232) = p(14)*p(242)
p(1233) = p(14)*p(243)
p(1234) = p(3)*p(464) - p(1074)
p(1235) = p(7)*p(464) - p(1075)
p(1236) = p(3)*p(531) - p(1074) - p(1235)
p(1237) = p(39)*p(187) - p(1234)
p(1238) = p(3)*p(532) - p(1075)
p(1239) = p(14)*p(295)
p(1240) = p(14)*p(296)
p(1241) = p(14)*p(247)
p(1242) = p(14)*p(248)
p(1243) = p(14)*p(249)
p(1244) = p(9)*p(337)
p(1245) = p(9)*p(312)
p(1246) = p(9)*p(266)
p(1247) = p(9)*p(313)
p(1248) = p(9)*p(314)
p(1249) = p(9)*p(338)
p(1250) = p(24)*p(185) - p(975)
p(1251) = p(46)*p(114) - p(977)
p(1252) = p(1)*p(783) - p(1082) - p(1251)
p(1253) = p(1)*p(784) - p(1083) - p(1252)
p(1254) = p(13)*p(335) - p(1215)
p(1255) = p(2)*p(748) - p(1055) - p(1222)
p(1256) = p(32)*p(185) - p(1215)
p(1257) = p(1)*p(786) - p(1083) - p(1255)
p(1258) = p(73)*p(95) - p(1215)
p(1259) = p(13)*p(299) - p(977)
p(1260) = p(46)*p(150) - p(1215)
p(1261) = p(73)*p(84) - p(975)
p(1262) = p(14)*p(335)
p(1263) = p(14)*p(336)
p(1264) = p(43)*p(130) - p(974)
p(1265) = p(8)*p(430) - p(1038)
p(1266) = p(1)*p(794) - p(1093) - p(1265)
p(1267) = p(81)*p(93) - p(1190)
p(1268) = p(2)*p(761) - p(1066) - p(1236)
p(1269) = p(43)*p(160) - p(1190)
p(1270) = p(45)*p(153) - p(1038)
p(1271) = p(18)*p(296) - p(1190)
p(1272) = p(34)*p(160) - p(974)
p(1273) = p(14)*p(302)
p(1274) = p(14)*p(303)
p(1275) = p(14)*p(304)
p(1276) = p(14)*p(258)
p(1277) = p(14)*p(259)
p(1278) = p(4)*p(464) - p(983)
p(1279) = p(1)*p(804) - p(1103) - p(1278)
p(1280) = p(16)*p(310) - p(1240)
p(1281) = p(2)*p(772) - p(1077) - p(1280)
p(1282) = p(4)*p(532) - p(1240)
p(1283) = p(11)*p(310) - p(983)
p(1284) = p(14)*p(305)
p(1285) = p(14)*p(306)
p(1286) = p(14)*p(307)
p(1287) = p(14)*p(263)
p(1288) = p(14)*p(264)
p(1289) = p(14)*p(265)
p(1290) = p(6)*p(464) - p(1104)
p(1291) = p(3)*p(578) - p(1290)
p(1292) = p(14)*p(309)
p(1293) = p(14)*p(310)
p(1294) = p(9)*p(340)
p(1295) = p(9)*p(322)
p(1296) = p(9)*p(323)
p(1297) = p(9)*p(341)
p(1298) = p(24)*p(188) - p(984)
p(1299) = p(50)*p(114) - p(1089)
p(1300) = p(1)*p(813) - p(1108) - p(1299)
p(1301) = p(13)*p(337) - p(1262)
p(1302) = p(2)*p(786) - p(1091) - p(1268)
p(1303) = p(32)*p(188) - p(1262)
p(1304) = p(6)*p(536) - p(1089)
p(1305) = p(73)*p(98) - p(1262)
p(1306) = p(32)*p(173) - p(984)
p(1307) = p(14)*p(337)
p(1308) = p(14)*p(338)
p(1309) = p(46)*p(130) - p(982)
p(1310) = p(1)*p(820) - p(1113) - p(1309)
p(1311) = p(81)*p(95) - p(1232)
p(1312) = p(2)*p(796) - p(1098) - p(1281)
p(1313) = p(46)*p(160) - p(1232)
p(1314) = p(81)*p(84) - p(982)
p(1315) = p(14)*p(315)
p(1316) = p(14)*p(316)
p(1317) = p(14)*p(317)
p(1318) = p(14)*p(270)
p(1319) = p(14)*p(271)
p(1320) = p(8)*p(464) - p(1079)
p(1321) = p(18)*p(310) - p(1242)
p(1322) = p(34)*p(187) - p(1079)
p(1323) = p(14)*p(318)
p(1324) = p(14)*p(319)
p(1325) = p(14)*p(320)
p(1326) = p(9)*p(343)
p(1327) = p(9)*p(344)
p(1328) = p(24)*p(191) - p(987)
p(1329) = p(1)*p(827) - p(1117) - p(1328)
p(1330) = p(13)*p(340) - p(1307)
p(1331) = p(2)*p(815) - p(1112) - p(1312)
p(1332) = p(32)*p(191) - p(1307)
p(1333) = p(13)*p(323) - p(987)
p(1334) = p(14)*p(340)
p(1335) = p(14)*p(341)
p(1336) = p(50)*p(130) - p(1102)
p(1337) = p(81)*p(98) - p(1276)
p(1338) = p(45)*p(173) - p(1102)
p(1339) = p(14)*p(324)
p(1340) = p(14)*p(325)
p(1341) = p(14)*p(326)
p(1342) = p(9)*p(347)
p(1343) = p(24)*p(195) - p(1119)
p(1344) = p(13)*p(343) - p(1334)
p(1345) = p(6)*p(561) - p(1119)
p(1346) = p(14)*p(343)
p(1347) = p(14)*p(344)
p(1348) = p(9)*p(272)
p(1349) = p(9)*p(273)
p(1350) = p(9)*p(274)
p(1351) = p(9)*p(328)
p(1352) = p(9)*p(275)
p(1353) = p(9)*p(329)
p(1354) = p(9)*p(330)
p(1355) = p(9)*p(346)
p(1356) = p(3)*p(570) - p(1120)
p(1357) = p(39)*p(181) - p(1122)
p(1358) = p(41)*p(178) - p(1125)
p(1359) = p(4)*p(566) - p(1129)
p(1360) = p(1)*p(842) - p(1126) - p(1359)
p(1361) = p(1)*p(843) - p(1129)
p(1362) = p(1)*p(844) - p(1130) - p(1360)
p(1363) = p(21)*p(279) - p(1125)
p(1364) = p(1)*p(846) - p(1133) - p(1362)
p(1365) = p(69)*p(101) - p(1122)
p(1366) = p(1)*p(848) - p(1135) - p(1364)
p(1367) = p(11)*p(346) - p(1120)
p(1368) = p(1)*p(850) - p(1140) - p(1139)
p(1369) = p(1)*p(851) - p(1146)
p(1370) = p(3)*p(572) - p(1148)
p(1371) = p(39)*p(183) - p(1150)
p(1372) = p(43)*p(178) - p(1153)
p(1373) = p(1)*p(854) - p(1151) - p(1372)
p(1374) = p(2)*p(843) - p(1143)
p(1375) = p(1)*p(855) - p(1154) - p(1373)
p(1376) = p(7)*p(509) - p(1153)
p(1377) = p(1)*p(857) - p(1157) - p(1375)
p(1378) = p(49)*p(153) - p(1150)
p(1379) = p(1)*p(859) - p(1159) - p(1377)
p(1380) = p(34)*p(194) - p(1148)
p(1381) = p(6)*p(570) - p(1147)
p(1382) = p(1)*p(862) - p(1167) - p(1165)
p(1383) = p(2)*p(851) - p(1147)
p(1384) = p(1)*p(864) - p(1183) - p(1182)
p(1385) = p(1)*p(865) - p(1187)
p(1386) = p(3)*p(574) - p(1191)
p(1387) = p(39)*p(185) - p(1193)
p(1388) = p(46)*p(178) - p(1196)
p(1389) = p(1)*p(868) - p(1194) - p(1388)
p(1390) = p(1)*p(869) - p(1196)
p(1391) = p(1)*p(870) - p(1197) - p(1389)
p(1392) = p(21)*p(299) - p(1193)
p(1393) = p(1)*p(872) - p(1199) - p(1391)
p(1394) = p(84)*p(101) - p(1191)
p(1395) = p(6)*p(572) - p(1178)
p(1396) = p(1)*p(875) - p(1206) - p(1204)
p(1397) = p(8)*p(505) - p(1178)
p(1398) = p(45)*p(181) - p(1189)
p(1399) = p(1)*p(878) - p(1222) - p(1220)
p(1400) = p(2)*p(865) - p(1189)
p(1401) = p(1)*p(880) - p(1236) - p(1235)
p(1402) = p(1)*p(881) - p(1238)
p(1403) = p(3)*p(576) - p(1244)
p(1404) = p(39)*p(188) - p(1246)
p(1405) = p(1)*p(884) - p(1245) - p(1404)
p(1406) = p(2)*p(869) - p(1210)
p(1407) = p(1)*p(885) - p(1247) - p(1405)
p(1408) = p(7)*p(536) - p(1246)
p(1409) = p(1)*p(887) - p(1249) - p(1407)
p(1410) = p(49)*p(173) - p(1244)
p(1411) = p(6)*p(574) - p(1215)
p(1412) = p(1)*p(890) - p(1255) - p(1253)
p(1413) = p(22)*p(285) - p(1215)
p(1414) = p(45)*p(183) - p(1190)
p(1415) = p(1)*p(893) - p(1268) - p(1266)
p(1416) = p(8)*p(518) - p(1190)
p(1417) = p(41)*p(187) - p(1240)
p(1418) = p(1)*p(896) - p(1281) - p(1279)
p(1419) = p(2)*p(881) - p(1240)
p(1420) = p(14)*p(308)
p(1421) = p(4)*p(547) - p(1106) - p(1320)
p(1422) = p(1)*p(900) - p(1291)
p(1423) = p(14)*p(311)
p(1424) = p(3)*p(579) - p(1294)
p(1425) = p(39)*p(191) - p(1296)
p(1426) = p(1)*p(903) - p(1295) - p(1425)
p(1427) = p(1)*p(904) - p(1296)
p(1428) = p(1)*p(905) - p(1297) - p(1426)
p(1429) = p(21)*p(323) - p(1294)
p(1430) = p(6)*p(576) - p(1262)
p(1431) = p(1)*p(908) - p(1302) - p(1300)
p(1432) = p(50)*p(150) - p(1262)
p(1433) = p(45)*p(185) - p(1232)
p(1434) = p(1)*p(911) - p(1312) - p(1310)
p(1435) = p(22)*p(296) - p(1232)
p(1436) = p(43)*p(187) - p(1242)
p(1437) = p(1)*p(915) - p(1322) - p(1320)
p(1438) = p(8)*p(532) - p(1242)
p(1439) = p(14)*p(321)
p(1440) = p(4)*p(578) - p(1293)
p(1441) = p(2)*p(899) - p(1292) - p(1440)
p(1442) = p(2)*p(900) - p(1293)
p(1443) = p(14)*p(339)
p(1444) = p(3)*p(582) - p(1326)
p(1445) = p(2)*p(903) - p(1303) - p(1299)
p(1446) = p(2)*p(904) - p(1304)
p(1447) = p(1)*p(919) - p(1327) - p(1445)
p(1448) = p(7)*p(561) - p(1326)
p(1449) = p(6)*p(579) - p(1307)
p(1450) = p(1)*p(922) - p(1331) - p(1329)
p(1451) = p(73)*p(102) - p(1307)
p(1452) = p(45)*p(188) - p(1276)
p(1453) = p(1)*p(925) - p(1338) - p(1336)
p(1454) = p(50)*p(160) - p(1276)
p(1455) = p(14)*p(327)
p(1456) = p(46)*p(187) - p(1243)
p(1457) = p(2)*p(915) - p(1324) - p(1441)
p(1458) = p(22)*p(310) - p(1243)
p(1459) = p(14)*p(342)
p(1460) = p(3)*p(585) - p(1342)
p(1461) = p(2)*p(919) - p(1332) - p(1329)
p(1462) = p(1)*p(929) - p(1342)
p(1463) = p(6)*p(582) - p(1334)
p(1464) = p(1)*p(931) - p(1345) - p(1343)
p(1465) = p(32)*p(195) - p(1334)
p(1466) = p(45)*p(191) - p(1318)
p(1467) = p(2)*p(925) - p(1340) - p(1457)
p(1468) = p(81)*p(102) - p(1318)
p(1469) = p(14)*p(345)
p(1470) = p(2)*p(928) - p(1344) - p(1343)
p(1471) = p(2)*p(929) - p(1345)
p(1472) = p(6)*p(585) - p(1346)
p(1473) = p(2)*p(931) - p(1347) - p(1467)
p(1474) = p(13)*p(347) - p(1346)
p(1475) = p(14)*p(347)
p(1476) = p(3)*p(566) - p(1351)
p(1477) = p(7)*p(566) - p(1349)
p(1478) = p(49)*p(178) - p(1348)
p(1479) = p(39)*p(194) - p(1352)
p(1480) = p(3)*p(588) - p(1355)
p(1481) = p(1)*p(938) - p(1356)
p(1482) = p(2)*p(958) - p(1481)
p(1483) = p(1)*p(940) - p(1370)
p(1484) = p(8)*p(588) - p(1483)
p(1485) = p(1)*p(942) - p(1386)
p(1486) = p(22)*p(346) - p(1485)
p(1487) = p(1)*p(944) - p(1403)
p(1488) = p(50)*p(194) - p(1487)
p(1489) = p(1)*p(946) - p(1424)
p(1490) = p(101)*p(102) - p(1489)
p(1491) = p(6)*p(578) - p(1443)
p(1492) = p(1)*p(949) - p(1444)
p(1493) = p(49)*p(195) - p(1492)
p(1494) = p(8)*p(578) - p(1423)
p(1495) = p(1)*p(952) - p(1460)
p(1496) = p(21)*p(347) - p(1495)
p(1497) = p(50)*p(187) - p(1420)
p(1498) = p(1)*p(955) - p(1470)
p(1499) = p(7)*p(589) - p(1498)
p(1500) = p(45)*p(195) - p(1455)
p(1501) = p(2)*p(955) - p(1472)
p(1502) = p(1)*p(959) - p(1501)
p(1503) = p(6)*p(589) - p(1475)
p(1504) = p(1)*p(958) - p(1480)
p(1505) = p(2)*p(959) - p(1503)
p(1506) = p(9)*p(348)
p(1507) = p(9)*p(349)
p(1508) = p(9)*p(360)
p(1509) = p(9)*p(361)
p(1510) = p(9)*p(409)
p(1511) = p(9)*p(428)
p(1512) = p(9)*p(368)
p(1513) = p(9)*p(350)
p(1514) = p(9)*p(369)
p(1515) = p(9)*p(351)
p(1516) = p(9)*p(370)
p(1517) = p(9)*p(371)
p(1518) = p(9)*p(372)
p(1519) = p(9)*p(429)
p(1520) = p(9)*p(375)
p(1521) = p(9)*p(435)
p(1522) = p(9)*p(352)
p(1523) = p(9)*p(376)
p(1524) = p(9)*p(377)
p(1525) = p(9)*p(436)
p(1526) = p(9)*p(378)
p(1527) = p(9)*p(437)
p(1528) = p(9)*p(451)
p(1529) = p(9)*p(382)
p(1530) = p(9)*p(353)
p(1531) = p(9)*p(383)
p(1532) = p(9)*p(384)
p(1533) = p(9)*p(452)
p(1534) = p(9)*p(459)
p(1535) = p(9)*p(386)
p(1536) = p(9)*p(460)
p(1537) = p(9)*p(387)
p(1538) = p(9)*p(388)
p(1539) = p(9)*p(461)
p(1540) = p(9)*p(462)
p(1541) = p(9)*p(463)
p(1542) = p(9)*p(466)
p(1543) = p(9)*p(467)
p(1544) = p(9)*p(533)
p(1545) = p(9)*p(476)
p(1546) = p(9)*p(390)
p(1547) = p(9)*p(391)
p(1548) = p(9)*p(477)
p(1549) = p(9)*p(481)
p(1550) = p(9)*p(482)
p(1551) = p(9)*p(483)
p(1552) = p(9)*p(545)
p(1553) = p(9)*p(549)
p(1554) = p(9)*p(488)
p(1555) = p(9)*p(489)
p(1556) = p(9)*p(559)
p(1557) = p(9)*p(565)
p(1558) = p(9)*p(401)
p(1559) = p(9)*p(402)
p(1560) = p(9)*p(403)
p(1561) = p(9)*p(504)
p(1562) = p(9)*p(354)
p(1563) = p(9)*p(355)
p(1564) = p(9)*p(404)
p(1565) = p(9)*p(356)
p(1566) = p(9)*p(357)
p(1567) = p(9)*p(405)
p(1568) = p(9)*p(358)
p(1569) = p(9)*p(406)
p(1570) = p(9)*p(359)
p(1571) = p(9)*p(407)
p(1572) = p(9)*p(408)
p(1573) = p(9)*p(505)
p(1574) = p(14)*p(566)
p(1575) = p(14)*p(567)
p(1576) = p(14)*p(568)
p(1577) = p(14)*p(569)
p(1578) = p(9)*p(514)
p(1579) = p(9)*p(416)
p(1580) = p(9)*p(417)
p(1581) = p(9)*p(418)
p(1582) = p(9)*p(419)
p(1583) = p(9)*p(515)
p(1584) = p(9)*p(420)
p(1585) = p(9)*p(362)
p(1586) = p(9)*p(363)
p(1587) = p(9)*p(364)
p(1588) = p(9)*p(421)
p(1589) = p(9)*p(422)
p(1590) = p(9)*p(365)
p(1591) = p(9)*p(366)
p(1592) = p(9)*p(423)
p(1593) = p(9)*p(424)
p(1594) = p(9)*p(367)
p(1595) = p(9)*p(425)
p(1596) = p(9)*p(426)
p(1597) = p(9)*p(427)
p(1598) = p(9)*p(516)
p(1599) = p(14)*p(495)
p(1600) = p(14)*p(496)
p(1601) = p(14)*p(497)
p(1602) = p(14)*p(498)
p(1603) = p(14)*p(499)
p(1604) = p(14)*p(500)
p(1605) = p(14)*p(501)
p(1606) = p(14)*p(502)
p(1607) = p(14)*p(503)
p(1608) = p(9)*p(430)
p(1609) = p(9)*p(431)
p(1610) = p(9)*p(517)
p(1611) = p(9)*p(373)
p(1612) = p(9)*p(432)
p(1613) = p(9)*p(374)
p(1614) = p(9)*p(433)
p(1615) = p(9)*p(434)
p(1616) = p(9)*p(518)
p(1617) = p(14)*p(401)
p(1618) = p(14)*p(402)
p(1619) = p(14)*p(403)
p(1620) = p(14)*p(404)
p(1621) = p(14)*p(405)
p(1622) = p(14)*p(406)
p(1623) = p(14)*p(407)
p(1624) = p(14)*p(408)
p(1625) = p(14)*p(360)
p(1626) = p(14)*p(361)
p(1627) = p(9)*p(525)
p(1628) = p(9)*p(442)
p(1629) = p(9)*p(443)
p(1630) = p(9)*p(444)
p(1631) = p(9)*p(526)
p(1632) = p(9)*p(445)
p(1633) = p(9)*p(379)
p(1634) = p(9)*p(380)
p(1635) = p(9)*p(446)
p(1636) = p(9)*p(447)
p(1637) = p(9)*p(381)
p(1638) = p(9)*p(448)
p(1639) = p(9)*p(449)
p(1640) = p(9)*p(450)
p(1641) = p(9)*p(527)
p(1642) = p(14)*p(506)
p(1643) = p(14)*p(507)
p(1644) = p(14)*p(508)
p(1645) = p(14)*p(509)
p(1646) = p(14)*p(510)
p(1647) = p(14)*p(511)
p(1648) = p(14)*p(512)
p(1649) = p(14)*p(513)
p(1650) = p(9)*p(528)
p(1651) = p(9)*p(453)
p(1652) = p(9)*p(454)
p(1653) = p(9)*p(529)
p(1654) = p(9)*p(455)
p(1655) = p(9)*p(385)
p(1656) = p(9)*p(456)
p(1657) = p(9)*p(457)
p(1658) = p(9)*p(458)
p(1659) = p(9)*p(530)
p(1660) = p(14)*p(416)
p(1661) = p(14)*p(417)
p(1662) = p(14)*p(418)
p(1663) = p(14)*p(419)
p(1664) = p(14)*p(420)
p(1665) = p(14)*p(421)
p(1666) = p(14)*p(422)
p(1667) = p(14)*p(423)
p(1668) = p(14)*p(424)
p(1669) = p(14)*p(425)
p(1670) = p(14)*p(426)
p(1671) = p(14)*p(427)
p(1672) = p(14)*p(368)
p(1673) = p(14)*p(369)
p(1674) = p(14)*p(370)
p(1675) = p(14)*p(371)
p(1676) = p(14)*p(372)
p(1677) = p(9)*p(464)
p(1678) = p(9)*p(531)
p(1679) = p(9)*p(465)
p(1680) = p(9)*p(532)
p(1681) = p(14)*p(430)
p(1682) = p(14)*p(431)
p(1683) = p(14)*p(432)
p(1684) = p(14)*p(433)
p(1685) = p(14)*p(434)
p(1686) = p(14)*p(375)
p(1687) = p(14)*p(376)
p(1688) = p(14)*p(377)
p(1689) = p(14)*p(378)
p(1690) = p(9)*p(539)
p(1691) = p(9)*p(470)
p(1692) = p(9)*p(471)
p(1693) = p(9)*p(540)
p(1694) = p(9)*p(472)
p(1695) = p(9)*p(389)
p(1696) = p(9)*p(473)
p(1697) = p(9)*p(474)
p(1698) = p(9)*p(475)
p(1699) = p(9)*p(541)
p(1700) = p(14)*p(519)
p(1701) = p(14)*p(520)
p(1702) = p(14)*p(521)
p(1703) = p(14)*p(522)
p(1704) = p(14)*p(523)
p(1705) = p(14)*p(524)
p(1706) = p(9)*p(542)
p(1707) = p(9)*p(478)
p(1708) = p(9)*p(543)
p(1709) = p(9)*p(479)
p(1710) = p(9)*p(480)
p(1711) = p(9)*p(544)
p(1712) = p(14)*p(442)
p(1713) = p(14)*p(443)
p(1714) = p(14)*p(444)
p(1715) = p(14)*p(445)
p(1716) = p(14)*p(446)
p(1717) = p(14)*p(447)
p(1718) = p(14)*p(448)
p(1719) = p(14)*p(449)
p(1720) = p(14)*p(450)
p(1721) = p(14)*p(382)
p(1722) = p(14)*p(383)
p(1723) = p(14)*p(384)
p(1724) = p(9)*p(546)
p(1725) = p(9)*p(547)
p(1726) = p(9)*p(548)
p(1727) = p(14)*p(453)
p(1728) = p(14)*p(454)
p(1729) = p(14)*p(455)
p(1730) = p(14)*p(456)
p(1731) = p(14)*p(457)
p(1732) = p(14)*p(458)
p(1733) = p(14)*p(386)
p(1734) = p(14)*p(387)
p(1735) = p(14)*p(388)
p(1736) = p(9)*p(578)
p(1737) = p(14)*p(464)
p(1738) = p(14)*p(465)
p(1739) = p(9)*p(553)
p(1740) = p(9)*p(485)
p(1741) = p(9)*p(554)
p(1742) = p(9)*p(486)
p(1743) = p(9)*p(487)
p(1744) = p(9)*p(555)
p(1745) = p(14)*p(534)
p(1746) = p(14)*p(535)
p(1747) = p(14)*p(536)
p(1748) = p(14)*p(537)
p(1749) = p(14)*p(538)
p(1750) = p(9)*p(556)
p(1751) = p(9)*p(557)
p(1752) = p(9)*p(558)
p(1753) = p(14)*p(470)
p(1754) = p(14)*p(471)
p(1755) = p(14)*p(472)
p(1756) = p(14)*p(473)
p(1757) = p(14)*p(474)
p(1758) = p(14)*p(475)
p(1759) = p(14)*p(390)
p(1760) = p(14)*p(391)
p(1761) = p(9)*p(581)
p(1762) = p(14)*p(478)
p(1763) = p(14)*p(479)
p(1764) = p(14)*p(480)
p(1765) = p(9)*p(562)
p(1766) = p(9)*p(563)
p(1767) = p(9)*p(564)
p(1768) = p(14)*p(550)
p(1769) = p(14)*p(551)
p(1770) = p(14)*p(552)
p(1771) = p(9)*p(584)
p(1772) = p(14)*p(485)
p(1773) = p(14)*p(486)
p(1774) = p(14)*p(487)
p(1775) = p(9)*p(587)
p(1776) = p(14)*p(560)
p(1777) = p(14)*p(561)
p(1778) = p(9)*p(570)
p(1779) = p(9)*p(495)
p(1780) = p(9)*p(392)
p(1781) = p(9)*p(496)
p(1782) = p(9)*p(393)
p(1783) = p(9)*p(394)
p(1784) = p(9)*p(497)
p(1785) = p(9)*p(395)
p(1786) = p(9)*p(396)
p(1787) = p(9)*p(397)
p(1788) = p(9)*p(498)
p(1789) = p(9)*p(398)
p(1790) = p(9)*p(399)
p(1791) = p(9)*p(499)
p(1792) = p(9)*p(500)
p(1793) = p(9)*p(400)
p(1794) = p(9)*p(501)
p(1795) = p(9)*p(502)
p(1796) = p(9)*p(503)
p(1797) = p(9)*p(571)
p(1798) = p(1)*p(1136) - p(1558)
p(1799) = p(21)*p(401) - p(1565)
p(1800) = p(101)*p(114) - p(1570)
p(1801) = p(24)*p(346) - p(1573)
p(1802) = p(3)*p(850) - p(1561) - p(1801)
p(1803) = p(1)*p(1140) - p(1561) - p(1802)
p(1804) = p(1)*p(1141) - p(1564) - p(1803)
p(1805) = p(1)*p(1142) - p(1567) - p(1804)
p(1806) = p(13)*p(566) - p(1558)
p(1807) = p(73)*p(178) - p(1565)
p(1808) = p(39)*p(285) - p(1570)
p(1809) = p(3)*p(851) - p(1573)
p(1810) = p(14)*p(588)
p(1811) = p(9)*p(572)
p(1812) = p(9)*p(506)
p(1813) = p(9)*p(410)
p(1814) = p(9)*p(507)
p(1815) = p(9)*p(411)
p(1816) = p(9)*p(412)
p(1817) = p(9)*p(508)
p(1818) = p(9)*p(413)
p(1819) = p(9)*p(414)
p(1820) = p(9)*p(509)
p(1821) = p(9)*p(510)
p(1822) = p(9)*p(415)
p(1823) = p(9)*p(511)
p(1824) = p(9)*p(512)
p(1825) = p(9)*p(513)
p(1826) = p(9)*p(573)
p(1827) = p(24)*p(331) - p(1510)
p(1828) = p(91)*p(114) - p(1507)
p(1829) = p(16)*p(401) - p(1506)
p(1830) = p(2)*p(1136) - p(1574)
p(1831) = p(1)*p(1163) - p(1581) - p(1830)
p(1832) = p(1)*p(1164) - p(1582) - p(1831)
p(1833) = p(1)*p(1165) - p(1583) - p(1832)
p(1834) = p(13)*p(570) - p(1810)
p(1835) = p(2)*p(1140) - p(1577) - p(1834)
p(1836) = p(32)*p(331) - p(1810)
p(1837) = p(1)*p(1167) - p(1583) - p(1835)
p(1838) = p(73)*p(181) - p(1810)
p(1839) = p(1)*p(1169) - p(1588) - p(1837)
p(1840) = p(91)*p(150) - p(1810)
p(1841) = p(6)*p(843) - p(1574)
p(1842) = p(41)*p(285) - p(1810)
p(1843) = p(32)*p(279) - p(1506)
p(1844) = p(16)*p(505) - p(1810)
p(1845) = p(69)*p(150) - p(1507)
p(1846) = p(4)*p(851) - p(1810)
p(1847) = p(11)*p(505) - p(1510)
p(1848) = p(14)*p(570)
p(1849) = p(14)*p(571)
p(1850) = p(1)*p(1180) - p(1608)
p(1851) = p(21)*p(430) - p(1613)
p(1852) = p(101)*p(130) - p(1616)
p(1853) = p(3)*p(864) - p(1610) - p(1852)
p(1854) = p(1)*p(1183) - p(1610) - p(1853)
p(1855) = p(1)*p(1184) - p(1612) - p(1854)
p(1856) = p(81)*p(178) - p(1608)
p(1857) = p(39)*p(296) - p(1613)
p(1858) = p(3)*p(865) - p(1616)
p(1859) = p(14)*p(504)
p(1860) = p(14)*p(505)
p(1861) = p(14)*p(409)
p(1862) = p(9)*p(574)
p(1863) = p(9)*p(519)
p(1864) = p(9)*p(438)
p(1865) = p(9)*p(520)
p(1866) = p(9)*p(439)
p(1867) = p(9)*p(440)
p(1868) = p(9)*p(521)
p(1869) = p(9)*p(441)
p(1870) = p(9)*p(522)
p(1871) = p(9)*p(523)
p(1872) = p(9)*p(524)
p(1873) = p(9)*p(575)
p(1874) = p(24)*p(333) - p(1511)
p(1875) = p(93)*p(114) - p(1513)
p(1876) = p(18)*p(401) - p(1516)
p(1877) = p(1)*p(1202) - p(1629) - p(1876)
p(1878) = p(1)*p(1203) - p(1630) - p(1877)
p(1879) = p(1)*p(1204) - p(1631) - p(1878)
p(1880) = p(13)*p(572) - p(1848)
p(1881) = p(2)*p(1167) - p(1607) - p(1853)
p(1882) = p(32)*p(333) - p(1848)
p(1883) = p(1)*p(1206) - p(1631) - p(1881)
p(1884) = p(73)*p(183) - p(1848)
p(1885) = p(1)*p(1208) - p(1635) - p(1883)
p(1886) = p(93)*p(150) - p(1848)
p(1887) = p(13)*p(509) - p(1516)
p(1888) = p(43)*p(285) - p(1848)
p(1889) = p(73)*p(153) - p(1513)
p(1890) = p(18)*p(505) - p(1848)
p(1891) = p(34)*p(285) - p(1511)
p(1892) = p(14)*p(572)
p(1893) = p(14)*p(573)
p(1894) = p(91)*p(130) - p(1525)
p(1895) = p(16)*p(430) - p(1522)
p(1896) = p(2)*p(1180) - p(1617)
p(1897) = p(1)*p(1219) - p(1652) - p(1896)
p(1898) = p(1)*p(1220) - p(1653) - p(1897)
p(1899) = p(81)*p(181) - p(1860)
p(1900) = p(2)*p(1183) - p(1620) - p(1899)
p(1901) = p(91)*p(160) - p(1860)
p(1902) = p(1)*p(1222) - p(1653) - p(1900)
p(1903) = p(41)*p(296) - p(1860)
p(1904) = p(45)*p(279) - p(1617)
p(1905) = p(16)*p(518) - p(1860)
p(1906) = p(69)*p(160) - p(1522)
p(1907) = p(4)*p(865) - p(1860)
p(1908) = p(11)*p(518) - p(1525)
p(1909) = p(14)*p(514)
p(1910) = p(14)*p(515)
p(1911) = p(14)*p(516)
p(1912) = p(14)*p(428)
p(1913) = p(14)*p(429)
p(1914) = p(1)*p(1234) - p(1677)
p(1915) = p(21)*p(464) - p(1680)
p(1916) = p(3)*p(880) - p(1678) - p(1915)
p(1917) = p(1)*p(1236) - p(1678) - p(1916)
p(1918) = p(39)*p(310) - p(1677)
p(1919) = p(3)*p(881) - p(1680)
p(1920) = p(14)*p(517)
p(1921) = p(14)*p(518)
p(1922) = p(14)*p(435)
p(1923) = p(14)*p(436)
p(1924) = p(14)*p(437)
p(1925) = p(9)*p(576)
p(1926) = p(9)*p(534)
p(1927) = p(9)*p(468)
p(1928) = p(9)*p(535)
p(1929) = p(9)*p(469)
p(1930) = p(9)*p(536)
p(1931) = p(9)*p(537)
p(1932) = p(9)*p(538)
p(1933) = p(9)*p(577)
p(1934) = p(24)*p(335) - p(1528)
p(1935) = p(95)*p(114) - p(1530)
p(1936) = p(22)*p(401) - p(1645)
p(1937) = p(1)*p(1252) - p(1692) - p(1936)
p(1938) = p(1)*p(1253) - p(1693) - p(1937)
p(1939) = p(13)*p(574) - p(1892)
p(1940) = p(2)*p(1206) - p(1649) - p(1900)
p(1941) = p(32)*p(335) - p(1892)
p(1942) = p(1)*p(1255) - p(1693) - p(1940)
p(1943) = p(73)*p(185) - p(1892)
p(1944) = p(6)*p(869) - p(1645)
p(1945) = p(95)*p(150) - p(1892)
p(1946) = p(32)*p(299) - p(1530)
p(1947) = p(46)*p(285) - p(1892)
p(1948) = p(84)*p(150) - p(1528)
p(1949) = p(14)*p(574)
p(1950) = p(14)*p(575)
p(1951) = p(93)*p(130) - p(1527)
p(1952) = p(18)*p(430) - p(1526)
p(1953) = p(1)*p(1265) - p(1707) - p(1952)
p(1954) = p(1)*p(1266) - p(1708) - p(1953)
p(1955) = p(81)*p(183) - p(1861)
p(1956) = p(2)*p(1222) - p(1665) - p(1916)
p(1957) = p(93)*p(160) - p(1861)
p(1958) = p(1)*p(1268) - p(1708) - p(1956)
p(1959) = p(43)*p(296) - p(1861)
p(1960) = p(81)*p(153) - p(1526)
p(1961) = p(18)*p(518) - p(1861)
p(1962) = p(34)*p(296) - p(1527)
p(1963) = p(14)*p(525)
p(1964) = p(14)*p(526)
p(1965) = p(14)*p(527)
p(1966) = p(14)*p(451)
p(1967) = p(14)*p(452)
p(1968) = p(16)*p(464) - p(1543)
p(1969) = p(2)*p(1234) - p(1681)
p(1970) = p(1)*p(1279) - p(1725) - p(1969)
p(1971) = p(41)*p(310) - p(1921)
p(1972) = p(2)*p(1236) - p(1683) - p(1971)
p(1973) = p(16)*p(532) - p(1921)
p(1974) = p(69)*p(187) - p(1681)
p(1975) = p(4)*p(881) - p(1921)
p(1976) = p(11)*p(532) - p(1543)
p(1977) = p(14)*p(528)
p(1978) = p(14)*p(529)
p(1979) = p(14)*p(530)
p(1980) = p(14)*p(459)
p(1981) = p(14)*p(460)
p(1982) = p(14)*p(461)
p(1983) = p(14)*p(462)
p(1984) = p(14)*p(463)
p(1985) = p(1)*p(1290) - p(1736)
p(1986) = p(4)*p(806) - p(1761) - p(1732)
p(1987) = p(3)*p(900) - p(1736)
p(1988) = p(14)*p(531)
p(1989) = p(14)*p(532)
p(1990) = p(14)*p(466)
p(1991) = p(14)*p(467)
p(1992) = p(9)*p(579)
p(1993) = p(9)*p(550)
p(1994) = p(9)*p(484)
p(1995) = p(9)*p(551)
p(1996) = p(9)*p(552)
p(1997) = p(9)*p(580)
p(1998) = p(24)*p(337) - p(1545)
p(1999) = p(98)*p(114) - p(1547)
p(2000) = p(1)*p(1299) - p(1740) - p(1999)
p(2001) = p(1)*p(1300) - p(1741) - p(2000)
p(2002) = p(13)*p(576) - p(1949)
p(2003) = p(2)*p(1255) - p(1705) - p(1956)
p(2004) = p(32)*p(337) - p(1949)
p(2005) = p(1)*p(1302) - p(1741) - p(2003)
p(2006) = p(73)*p(188) - p(1949)
p(2007) = p(13)*p(536) - p(1547)
p(2008) = p(98)*p(150) - p(1949)
p(2009) = p(73)*p(173) - p(1545)
p(2010) = p(14)*p(576)
p(2011) = p(14)*p(577)
p(2012) = p(95)*p(130) - p(1540)
p(2013) = p(22)*p(430) - p(1674)
p(2014) = p(1)*p(1310) - p(1751) - p(2013)
p(2015) = p(81)*p(185) - p(1912)
p(2016) = p(2)*p(1268) - p(1716) - p(1972)
p(2017) = p(95)*p(160) - p(1912)
p(2018) = p(45)*p(299) - p(1674)
p(2019) = p(46)*p(296) - p(1912)
p(2020) = p(84)*p(160) - p(1540)
p(2021) = p(14)*p(539)
p(2022) = p(14)*p(540)
p(2023) = p(14)*p(541)
p(2024) = p(14)*p(476)
p(2025) = p(14)*p(477)
p(2026) = p(18)*p(464) - p(1544)
p(2027) = p(1)*p(1320) - p(1761) - p(2026)
p(2028) = p(43)*p(310) - p(1923)
p(2029) = p(2)*p(1281) - p(1730) - p(1986)
p(2030) = p(18)*p(532) - p(1923)
p(2031) = p(34)*p(310) - p(1544)
p(2032) = p(14)*p(542)
p(2033) = p(14)*p(543)
p(2034) = p(14)*p(544)
p(2035) = p(14)*p(481)
p(2036) = p(14)*p(482)
p(2037) = p(14)*p(483)
p(2038) = p(2)*p(1290) - p(1737)
p(2039) = p(4)*p(900) - p(1989)
p(2040) = p(11)*p(578) - p(1737)
p(2041) = p(14)*p(546)
p(2042) = p(14)*p(547)
p(2043) = p(14)*p(548)
p(2044) = p(9)*p(582)
p(2045) = p(9)*p(560)
p(2046) = p(9)*p(561)
p(2047) = p(9)*p(583)
p(2048) = p(24)*p(340) - p(1554)
p(2049) = p(102)*p(114) - p(1747)
p(2050) = p(1)*p(1329) - p(1766) - p(2049)
p(2051) = p(13)*p(579) - p(2010)
p(2052) = p(2)*p(1302) - p(1749) - p(2016)
p(2053) = p(32)*p(340) - p(2010)
p(2054) = p(6)*p(904) - p(1747)
p(2055) = p(73)*p(191) - p(2010)
p(2056) = p(32)*p(323) - p(1554)
p(2057) = p(14)*p(579)
p(2058) = p(14)*p(580)
p(2059) = p(98)*p(130) - p(1552)
p(2060) = p(1)*p(1336) - p(1771) - p(2059)
p(2061) = p(81)*p(188) - p(1966)
p(2062) = p(2)*p(1312) - p(1756) - p(2029)
p(2063) = p(98)*p(160) - p(1966)
p(2064) = p(81)*p(173) - p(1552)
p(2065) = p(14)*p(553)
p(2066) = p(14)*p(554)
p(2067) = p(14)*p(555)
p(2068) = p(14)*p(488)
p(2069) = p(14)*p(489)
p(2070) = p(22)*p(464) - p(1689)
p(2071) = p(46)*p(310) - p(1924)
p(2072) = p(84)*p(187) - p(1689)
p(2073) = p(14)*p(556)
p(2074) = p(14)*p(557)
p(2075) = p(14)*p(558)
p(2076) = p(9)*p(585)
p(2077) = p(9)*p(586)
p(2078) = p(24)*p(343) - p(1557)
p(2079) = p(1)*p(1343) - p(1775) - p(2078)
p(2080) = p(13)*p(582) - p(2057)
p(2081) = p(2)*p(1331) - p(1770) - p(2062)
p(2082) = p(32)*p(343) - p(2057)
p(2083) = p(13)*p(561) - p(1557)
p(2084) = p(14)*p(582)
p(2085) = p(14)*p(583)
p(2086) = p(102)*p(130) - p(1760)
p(2087) = p(81)*p(191) - p(2024)
p(2088) = p(45)*p(323) - p(1760)
p(2089) = p(14)*p(562)
p(2090) = p(14)*p(563)
p(2091) = p(14)*p(564)
p(2092) = p(9)*p(589)
p(2093) = p(24)*p(347) - p(1777)
p(2094) = p(13)*p(585) - p(2084)
p(2095) = p(6)*p(929) - p(1777)
p(2096) = p(14)*p(585)
p(2097) = p(14)*p(586)
p(2098) = p(9)*p(490)
p(2099) = p(9)*p(491)
p(2100) = p(9)*p(492)
p(2101) = p(9)*p(566)
p(2102) = p(9)*p(493)
p(2103) = p(9)*p(567)
p(2104) = p(9)*p(494)
p(2105) = p(9)*p(568)
p(2106) = p(9)*p(569)
p(2107) = p(9)*p(588)
p(2108) = p(3)*p(938) - p(1778)
p(2109) = p(39)*p(331) - p(1780)
p(2110) = p(91)*p(178) - p(1783)
p(2111) = p(16)*p(566) - p(1787)
p(2112) = p(1)*p(1359) - p(1784) - p(2111)
p(2113) = p(2)*p(1476) - p(2112)
p(2114) = p(1)*p(1360) - p(1788) - p(2112)
p(2115) = p(7)*p(843) - p(1787)
p(2116) = p(1)*p(1362) - p(1792) - p(2114)
p(2117) = p(49)*p(279) - p(1783)
p(2118) = p(1)*p(1364) - p(1795) - p(2116)
p(2119) = p(69)*p(194) - p(1780)
p(2120) = p(1)*p(1366) - p(1797) - p(2118)
p(2121) = p(11)*p(588) - p(1778)
p(2122) = p(1)*p(1368) - p(1802) - p(1801)
p(2123) = p(1)*p(1369) - p(1809)
p(2124) = p(3)*p(940) - p(1811)
p(2125) = p(39)*p(333) - p(1813)
p(2126) = p(93)*p(178) - p(1816)
p(2127) = p(18)*p(566) - p(1820)
p(2128) = p(1)*p(1373) - p(1817) - p(2127)
p(2129) = p(1)*p(1374) - p(1820)
p(2130) = p(1)*p(1375) - p(1821) - p(2128)
p(2131) = p(21)*p(509) - p(1816)
p(2132) = p(1)*p(1377) - p(1824) - p(2130)
p(2133) = p(101)*p(153) - p(1813)
p(2134) = p(1)*p(1379) - p(1826) - p(2132)
p(2135) = p(34)*p(346) - p(1811)
p(2136) = p(6)*p(938) - p(1810)
p(2137) = p(1)*p(1382) - p(1835) - p(1833)
p(2138) = p(2)*p(1369) - p(1810)
p(2139) = p(1)*p(1384) - p(1853) - p(1852)
p(2140) = p(1)*p(1385) - p(1858)
p(2141) = p(3)*p(942) - p(1862)
p(2142) = p(39)*p(335) - p(1864)
p(2143) = p(95)*p(178) - p(1867)
p(2144) = p(1)*p(1388) - p(1865) - p(2143)
p(2145) = p(2)*p(1374) - p(1841)
p(2146) = p(1)*p(1389) - p(1868) - p(2144)
p(2147) = p(7)*p(869) - p(1867)
p(2148) = p(1)*p(1391) - p(1871) - p(2146)
p(2149) = p(49)*p(299) - p(1864)
p(2150) = p(1)*p(1393) - p(1873) - p(2148)
p(2151) = p(84)*p(194) - p(1862)
p(2152) = p(6)*p(940) - p(1848)
p(2153) = p(1)*p(1396) - p(1881) - p(1879)
p(2154) = p(8)*p(851) - p(1848)
p(2155) = p(45)*p(331) - p(1860)
p(2156) = p(1)*p(1399) - p(1900) - p(1898)
p(2157) = p(2)*p(1385) - p(1860)
p(2158) = p(1)*p(1401) - p(1916) - p(1915)
p(2159) = p(1)*p(1402) - p(1919)
p(2160) = p(3)*p(944) - p(1925)
p(2161) = p(39)*p(337) - p(1927)
p(2162) = p(98)*p(178) - p(1930)
p(2163) = p(1)*p(1405) - p(1928) - p(2162)
p(2164) = p(1)*p(1406) - p(1930)
p(2165) = p(1)*p(1407) - p(1931) - p(2163)
p(2166) = p(21)*p(536) - p(1927)
p(2167) = p(1)*p(1409) - p(1933) - p(2165)
p(2168) = p(101)*p(173) - p(1925)
p(2169) = p(6)*p(942) - p(1892)
p(2170) = p(1)*p(1412) - p(1940) - p(1938)
p(2171) = p(22)*p(505) - p(1892)
p(2172) = p(45)*p(333) - p(1861)
p(2173) = p(1)*p(1415) - p(1956) - p(1954)
p(2174) = p(8)*p(865) - p(1861)
p(2175) = p(91)*p(187) - p(1921)
p(2176) = p(1)*p(1418) - p(1972) - p(1970)
p(2177) = p(2)*p(1402) - p(1921)
p(2178) = p(1)*p(1421) - p(1986) - p(1985)
p(2179) = p(1)*p(1422) - p(1987)
p(2180) = p(14)*p(533)
p(2181) = p(3)*p(946) - p(1992)
p(2182) = p(39)*p(340) - p(1994)
p(2183) = p(1)*p(1425) - p(1993) - p(2182)
p(2184) = p(2)*p(1406) - p(1944)
p(2185) = p(1)*p(1426) - p(1995) - p(2183)
p(2186) = p(7)*p(904) - p(1994)
p(2187) = p(1)*p(1428) - p(1997) - p(2185)
p(2188) = p(49)*p(323) - p(1992)
p(2189) = p(6)*p(944) - p(1949)
p(2190) = p(1)*p(1431) - p(2003) - p(2001)
p(2191) = p(50)*p(285) - p(1949)
p(2192) = p(45)*p(335) - p(1912)
p(2193) = p(1)*p(1434) - p(2016) - p(2014)
p(2194) = p(22)*p(518) - p(1912)
p(2195) = p(93)*p(187) - p(1923)
p(2196) = p(1)*p(1437) - p(2029) - p(2027)
p(2197) = p(8)*p(881) - p(1923)
p(2198) = p(14)*p(545)
p(2199) = p(16)*p(578) - p(1989)
p(2200) = p(1)*p(1441) - p(2040) - p(2038)
p(2201) = p(2)*p(1422) - p(1989)
p(2202) = p(14)*p(549)
p(2203) = p(6)*p(899) - p(2042) - p(2041)
p(2204) = p(1)*p(1491) - p(2203)
p(2205) = p(14)*p(578)
p(2206) = p(3)*p(949) - p(2044)
p(2207) = p(39)*p(343) - p(2046)
p(2208) = p(1)*p(1445) - p(2045) - p(2207)
p(2209) = p(1)*p(1446) - p(2046)
p(2210) = p(1)*p(1447) - p(2047) - p(2208)
p(2211) = p(21)*p(561) - p(2044)
p(2212) = p(6)*p(946) - p(2010)
p(2213) = p(1)*p(1450) - p(2052) - p(2050)
p(2214) = p(102)*p(150) - p(2010)
p(2215) = p(45)*p(337) - p(1966)
p(2216) = p(1)*p(1453) - p(2062) - p(2060)
p(2217) = p(50)*p(296) - p(1966)
p(2218) = p(95)*p(187) - p(1924)
p(2219) = p(1)*p(1457) - p(2072) - p(2070)
p(2220) = p(22)*p(532) - p(1924)
p(2221) = p(14)*p(559)
p(2222) = p(18)*p(578) - p(1991)
p(2223) = p(2)*p(1441) - p(2042) - p(2203)
p(2224) = p(8)*p(900) - p(1991)
p(2225) = p(14)*p(581)
p(2226) = p(3)*p(952) - p(2076)
p(2227) = p(2)*p(1445) - p(2053) - p(2049)
p(2228) = p(2)*p(1446) - p(2054)
p(2229) = p(1)*p(1461) - p(2077) - p(2227)
p(2230) = p(7)*p(929) - p(2076)
p(2231) = p(6)*p(949) - p(2057)
p(2232) = p(1)*p(1464) - p(2081) - p(2079)
p(2233) = p(73)*p(195) - p(2057)
p(2234) = p(45)*p(340) - p(2024)
p(2235) = p(1)*p(1467) - p(2088) - p(2086)
p(2236) = p(102)*p(160) - p(2024)
p(2237) = p(14)*p(565)
p(2238) = p(98)*p(187) - p(1983)
p(2239) = p(2)*p(1457) - p(2074) - p(2223)
p(2240) = p(50)*p(310) - p(1983)
p(2241) = p(14)*p(584)
p(2242) = p(3)*p(955) - p(2092)
p(2243) = p(2)*p(1461) - p(2082) - p(2079)
p(2244) = p(1)*p(1471) - p(2092)
p(2245) = p(6)*p(952) - p(2084)
p(2246) = p(1)*p(1473) - p(2095) - p(2093)
p(2247) = p(32)*p(347) - p(2084)
p(2248) = p(45)*p(343) - p(2068)
p(2249) = p(2)*p(1467) - p(2090) - p(2239)
p(2250) = p(81)*p(195) - p(2068)
p(2251) = p(14)*p(587)
p(2252) = p(2)*p(1470) - p(2094) - p(2093)
p(2253) = p(2)*p(1471) - p(2095)
p(2254) = p(6)*p(955) - p(2096)
p(2255) = p(2)*p(1473) - p(2097) - p(2249)
p(2256) = p(13)*p(589) - p(2096)
p(2257) = p(14)*p(589)
p(2258) = p(1)*p(1476) - p(2101)
p(2259) = p(21)*p(566) - p(2098)
p(2260) = p(101)*p(178) - p(2099)
p(2261) = p(39)*p(346) - p(2104)
p(2262) = p(3)*p(958) - p(2107)
p(2263) = p(1)*p(1481) - p(2108)
p(2264) = p(2)*p(1504) - p(2263)
p(2265) = p(1)*p(1483) - p(2124)
p(2266) = p(8)*p(958) - p(2265)
p(2267) = p(1)*p(1485) - p(2141)
p(2268) = p(22)*p(588) - p(2267)
p(2269) = p(1)*p(1487) - p(2160)
p(2270) = p(50)*p(346) - p(2269)
p(2271) = p(1)*p(1489) - p(2181)
p(2272) = p(102)*p(194) - p(2271)
p(2273) = p(1)*p(1492) - p(2206)
p(2274) = p(101)*p(195) - p(2273)
p(2275) = p(2)*p(1491) - p(2205)
p(2276) = p(1)*p(1495) - p(2226)
p(2277) = p(49)*p(347) - p(2276)
p(2278) = p(22)*p(578) - p(2180)
p(2279) = p(1)*p(1498) - p(2242)
p(2280) = p(21)*p(589) - p(2279)
p(2281) = p(102)*p(187) - p(2198)
p(2282) = p(1)*p(1501) - p(2252)
p(2283) = p(7)*p(959) - p(2282)
p(2284) = p(45)*p(347) - p(2237)
p(2285) = p(2)*p(1501) - p(2254)
p(2286) = p(1)*p(1505) - p(2285)
p(2287) = p(6)*p(959) - p(2257)
p(2288) = p(1)*p(1504) - p(2262)
p(2289) = p(2)*p(1505) - p(2287)
p(2290) = p(9)*p(590)
p(2291) = p(9)*p(591)
p(2292) = p(9)*p(613)
p(2293) = p(9)*p(592)
p(2294) = p(9)*p(614)
p(2295) = p(9)*p(615)
p(2296) = p(9)*p(701)
p(2297) = p(9)*p(726)
p(2298) = p(9)*p(626)
p(2299) = p(9)*p(593)
p(2300) = p(9)*p(627)
p(2301) = p(9)*p(594)
p(2302) = p(9)*p(595)
p(2303) = p(9)*p(628)
p(2304) = p(9)*p(596)
p(2305) = p(9)*p(629)
p(2306) = p(9)*p(630)
p(2307) = p(9)*p(631)
p(2308) = p(9)*p(727)
p(2309) = p(9)*p(636)
p(2310) = p(9)*p(637)
p(2311) = p(9)*p(734)
p(2312) = p(9)*p(597)
p(2313) = p(9)*p(638)
p(2314) = p(9)*p(598)
p(2315) = p(9)*p(639)
p(2316) = p(9)*p(640)
p(2317) = p(9)*p(735)
p(2318) = p(9)*p(599)
p(2319) = p(9)*p(641)
p(2320) = p(9)*p(736)
p(2321) = p(9)*p(755)
p(2322) = p(9)*p(648)
p(2323) = p(9)*p(600)
p(2324) = p(9)*p(649)
p(2325) = p(9)*p(601)
p(2326) = p(9)*p(650)
p(2327) = p(9)*p(651)
p(2328) = p(9)*p(652)
p(2329) = p(9)*p(756)
p(2330) = p(9)*p(766)
p(2331) = p(9)*p(656)
p(2332) = p(9)*p(657)
p(2333) = p(9)*p(767)
p(2334) = p(9)*p(658)
p(2335) = p(9)*p(602)
p(2336) = p(9)*p(659)
p(2337) = p(9)*p(660)
p(2338) = p(9)*p(661)
p(2339) = p(9)*p(768)
p(2340) = p(9)*p(769)
p(2341) = p(9)*p(662)
p(2342) = p(9)*p(663)
p(2343) = p(9)*p(770)
p(2344) = p(9)*p(665)
p(2345) = p(9)*p(774)
p(2346) = p(9)*p(666)
p(2347) = p(9)*p(775)
p(2348) = p(9)*p(776)
p(2349) = p(9)*p(777)
p(2350) = p(9)*p(882)
p(2351) = p(9)*p(791)
p(2352) = p(9)*p(670)
p(2353) = p(9)*p(603)
p(2354) = p(9)*p(671)
p(2355) = p(9)*p(672)
p(2356) = p(9)*p(792)
p(2357) = p(9)*p(799)
p(2358) = p(9)*p(674)
p(2359) = p(9)*p(800)
p(2360) = p(9)*p(675)
p(2361) = p(9)*p(676)
p(2362) = p(9)*p(801)
p(2363) = p(9)*p(802)
p(2364) = p(9)*p(803)
p(2365) = p(9)*p(807)
p(2366) = p(9)*p(808)
p(2367) = p(9)*p(809)
p(2368) = p(9)*p(898)
p(2369) = p(9)*p(901)
p(2370) = p(9)*p(818)
p(2371) = p(9)*p(678)
p(2372) = p(9)*p(679)
p(2373) = p(9)*p(819)
p(2374) = p(9)*p(823)
p(2375) = p(9)*p(824)
p(2376) = p(9)*p(825)
p(2377) = p(9)*p(913)
p(2378) = p(9)*p(917)
p(2379) = p(9)*p(830)
p(2380) = p(9)*p(831)
p(2381) = p(9)*p(927)
p(2382) = p(9)*p(933)
p(2383) = p(9)*p(692)
p(2384) = p(9)*p(693)
p(2385) = p(9)*p(694)
p(2386) = p(9)*p(850)
p(2387) = p(9)*p(604)
p(2388) = p(9)*p(605)
p(2389) = p(9)*p(606)
p(2390) = p(9)*p(695)
p(2391) = p(9)*p(607)
p(2392) = p(9)*p(608)
p(2393) = p(9)*p(696)
p(2394) = p(9)*p(609)
p(2395) = p(9)*p(610)
p(2396) = p(9)*p(697)
p(2397) = p(9)*p(611)
p(2398) = p(9)*p(698)
p(2399) = p(9)*p(612)
p(2400) = p(9)*p(699)
p(2401) = p(9)*p(700)
p(2402) = p(9)*p(851)
p(2403) = p(14)*p(934)
p(2404) = p(14)*p(935)
p(2405) = p(14)*p(936)
p(2406) = p(14)*p(937)
p(2407) = p(9)*p(861)
p(2408) = p(9)*p(711)
p(2409) = p(9)*p(712)
p(2410) = p(9)*p(713)
p(2411) = p(9)*p(714)
p(2412) = p(9)*p(715)
p(2413) = p(9)*p(862)
p(2414) = p(9)*p(716)
p(2415) = p(9)*p(616)
p(2416) = p(9)*p(617)
p(2417) = p(9)*p(618)
p(2418) = p(9)*p(619)
p(2419) = p(9)*p(717)
p(2420) = p(9)*p(718)
p(2421) = p(9)*p(620)
p(2422) = p(9)*p(621)
p(2423) = p(9)*p(622)
p(2424) = p(9)*p(719)
p(2425) = p(9)*p(720)
p(2426) = p(9)*p(623)
p(2427) = p(9)*p(624)
p(2428) = p(9)*p(721)
p(2429) = p(9)*p(722)
p(2430) = p(9)*p(625)
p(2431) = p(9)*p(723)
p(2432) = p(9)*p(724)
p(2433) = p(9)*p(725)
p(2434) = p(9)*p(863)
p(2435) = p(14)*p(839)
p(2436) = p(14)*p(840)
p(2437) = p(14)*p(841)
p(2438) = p(14)*p(842)
p(2439) = p(14)*p(843)
p(2440) = p(14)*p(844)
p(2441) = p(14)*p(845)
p(2442) = p(14)*p(846)
p(2443) = p(14)*p(847)
p(2444) = p(14)*p(848)
p(2445) = p(14)*p(849)
p(2446) = p(9)*p(728)
p(2447) = p(9)*p(729)
p(2448) = p(9)*p(864)
p(2449) = p(9)*p(632)
p(2450) = p(9)*p(633)
p(2451) = p(9)*p(730)
p(2452) = p(9)*p(634)
p(2453) = p(9)*p(731)
p(2454) = p(9)*p(635)
p(2455) = p(9)*p(732)
p(2456) = p(9)*p(733)
p(2457) = p(9)*p(865)
p(2458) = p(14)*p(692)
p(2459) = p(14)*p(693)
p(2460) = p(14)*p(694)
p(2461) = p(14)*p(695)
p(2462) = p(14)*p(696)
p(2463) = p(14)*p(697)
p(2464) = p(14)*p(698)
p(2465) = p(14)*p(699)
p(2466) = p(14)*p(700)
p(2467) = p(14)*p(613)
p(2468) = p(14)*p(614)
p(2469) = p(14)*p(615)
p(2470) = p(9)*p(874)
p(2471) = p(9)*p(743)
p(2472) = p(9)*p(744)
p(2473) = p(9)*p(745)
p(2474) = p(9)*p(746)
p(2475) = p(9)*p(875)
p(2476) = p(9)*p(747)
p(2477) = p(9)*p(642)
p(2478) = p(9)*p(643)
p(2479) = p(9)*p(644)
p(2480) = p(9)*p(748)
p(2481) = p(9)*p(749)
p(2482) = p(9)*p(645)
p(2483) = p(9)*p(646)
p(2484) = p(9)*p(750)
p(2485) = p(9)*p(751)
p(2486) = p(9)*p(647)
p(2487) = p(9)*p(752)
p(2488) = p(9)*p(753)
p(2489) = p(9)*p(754)
p(2490) = p(9)*p(876)
p(2491) = p(14)*p(852)
p(2492) = p(14)*p(853)
p(2493) = p(14)*p(854)
p(2494) = p(14)*p(855)
p(2495) = p(14)*p(856)
p(2496) = p(14)*p(857)
p(2497) = p(14)*p(858)
p(2498) = p(14)*p(859)
p(2499) = p(14)*p(860)
p(2500) = p(9)*p(877)
p(2501) = p(9)*p(757)
p(2502) = p(9)*p(758)
p(2503) = p(9)*p(759)
p(2504) = p(9)*p(878)
p(2505) = p(9)*p(760)
p(2506) = p(9)*p(653)
p(2507) = p(9)*p(654)
p(2508) = p(9)*p(761)
p(2509) = p(9)*p(762)
p(2510) = p(9)*p(655)
p(2511) = p(9)*p(763)
p(2512) = p(9)*p(764)
p(2513) = p(9)*p(765)
p(2514) = p(9)*p(879)
p(2515) = p(14)*p(711)
p(2516) = p(14)*p(712)
p(2517) = p(14)*p(713)
p(2518) = p(14)*p(714)
p(2519) = p(14)*p(715)
p(2520) = p(14)*p(716)
p(2521) = p(14)*p(717)
p(2522) = p(14)*p(718)
p(2523) = p(14)*p(719)
p(2524) = p(14)*p(720)
p(2525) = p(14)*p(721)
p(2526) = p(14)*p(722)
p(2527) = p(14)*p(723)
p(2528) = p(14)*p(724)
p(2529) = p(14)*p(725)
p(2530) = p(14)*p(626)
p(2531) = p(14)*p(627)
p(2532) = p(14)*p(628)
p(2533) = p(14)*p(629)
p(2534) = p(14)*p(630)
p(2535) = p(14)*p(631)
p(2536) = p(9)*p(771)
p(2537) = p(9)*p(880)
p(2538) = p(9)*p(664)
p(2539) = p(9)*p(772)
p(2540) = p(9)*p(773)
p(2541) = p(9)*p(881)
p(2542) = p(14)*p(728)
p(2543) = p(14)*p(729)
p(2544) = p(14)*p(730)
p(2545) = p(14)*p(731)
p(2546) = p(14)*p(732)
p(2547) = p(14)*p(733)
p(2548) = p(14)*p(636)
p(2549) = p(14)*p(637)
p(2550) = p(14)*p(638)
p(2551) = p(14)*p(639)
p(2552) = p(14)*p(640)
p(2553) = p(14)*p(641)
p(2554) = p(9)*p(889)
p(2555) = p(9)*p(782)
p(2556) = p(9)*p(783)
p(2557) = p(9)*p(784)
p(2558) = p(9)*p(890)
p(2559) = p(9)*p(785)
p(2560) = p(9)*p(667)
p(2561) = p(9)*p(668)
p(2562) = p(9)*p(786)
p(2563) = p(9)*p(787)
p(2564) = p(9)*p(669)
p(2565) = p(9)*p(788)
p(2566) = p(9)*p(789)
p(2567) = p(9)*p(790)
p(2568) = p(9)*p(891)
p(2569) = p(14)*p(866)
p(2570) = p(14)*p(867)
p(2571) = p(14)*p(868)
p(2572) = p(14)*p(869)
p(2573) = p(14)*p(870)
p(2574) = p(14)*p(871)
p(2575) = p(14)*p(872)
p(2576) = p(14)*p(873)
p(2577) = p(9)*p(892)
p(2578) = p(9)*p(793)
p(2579) = p(9)*p(794)
p(2580) = p(9)*p(893)
p(2581) = p(9)*p(795)
p(2582) = p(9)*p(673)
p(2583) = p(9)*p(796)
p(2584) = p(9)*p(797)
p(2585) = p(9)*p(798)
p(2586) = p(9)*p(894)
p(2587) = p(14)*p(743)
p(2588) = p(14)*p(744)
p(2589) = p(14)*p(745)
p(2590) = p(14)*p(746)
p(2591) = p(14)*p(747)
p(2592) = p(14)*p(748)
p(2593) = p(14)*p(749)
p(2594) = p(14)*p(750)
p(2595) = p(14)*p(751)
p(2596) = p(14)*p(752)
p(2597) = p(14)*p(753)
p(2598) = p(14)*p(754)
p(2599) = p(14)*p(648)
p(2600) = p(14)*p(649)
p(2601) = p(14)*p(650)
p(2602) = p(14)*p(651)
p(2603) = p(14)*p(652)
p(2604) = p(9)*p(895)
p(2605) = p(9)*p(804)
p(2606) = p(9)*p(896)
p(2607) = p(9)*p(805)
p(2608) = p(9)*p(806)
p(2609) = p(9)*p(897)
p(2610) = p(14)*p(757)
p(2611) = p(14)*p(758)
p(2612) = p(14)*p(759)
p(2613) = p(14)*p(760)
p(2614) = p(14)*p(761)
p(2615) = p(14)*p(762)
p(2616) = p(14)*p(763)
p(2617) = p(14)*p(764)
p(2618) = p(14)*p(765)
p(2619) = p(14)*p(656)
p(2620) = p(14)*p(657)
p(2621) = p(14)*p(658)
p(2622) = p(14)*p(659)
p(2623) = p(14)*p(660)
p(2624) = p(14)*p(661)
p(2625) = p(14)*p(662)
p(2626) = p(14)*p(663)
p(2627) = p(9)*p(899)
p(2628) = p(9)*p(900)
p(2629) = p(14)*p(771)
p(2630) = p(14)*p(772)
p(2631) = p(14)*p(773)
p(2632) = p(14)*p(665)
p(2633) = p(14)*p(666)
p(2634) = p(9)*p(907)
p(2635) = p(9)*p(812)
p(2636) = p(9)*p(813)
p(2637) = p(9)*p(908)
p(2638) = p(9)*p(814)
p(2639) = p(9)*p(677)
p(2640) = p(9)*p(815)
p(2641) = p(9)*p(816)
p(2642) = p(9)*p(817)
p(2643) = p(9)*p(909)
p(2644) = p(14)*p(883)
p(2645) = p(14)*p(884)
p(2646) = p(14)*p(885)
p(2647) = p(14)*p(886)
p(2648) = p(14)*p(887)
p(2649) = p(14)*p(888)
p(2650) = p(9)*p(910)
p(2651) = p(9)*p(820)
p(2652) = p(9)*p(911)
p(2653) = p(9)*p(821)
p(2654) = p(9)*p(822)
p(2655) = p(9)*p(912)
p(2656) = p(14)*p(782)
p(2657) = p(14)*p(783)
p(2658) = p(14)*p(784)
p(2659) = p(14)*p(785)
p(2660) = p(14)*p(786)
p(2661) = p(14)*p(787)
p(2662) = p(14)*p(788)
p(2663) = p(14)*p(789)
p(2664) = p(14)*p(790)
p(2665) = p(14)*p(670)
p(2666) = p(14)*p(671)
p(2667) = p(14)*p(672)
p(2668) = p(9)*p(914)
p(2669) = p(9)*p(915)
p(2670) = p(9)*p(916)
p(2671) = p(14)*p(793)
p(2672) = p(14)*p(794)
p(2673) = p(14)*p(795)
p(2674) = p(14)*p(796)
p(2675) = p(14)*p(797)
p(2676) = p(14)*p(798)
p(2677) = p(14)*p(674)
p(2678) = p(14)*p(675)
p(2679) = p(14)*p(676)
p(2680) = p(9)*p(948)
p(2681) = p(14)*p(804)
p(2682) = p(14)*p(805)
p(2683) = p(14)*p(806)
p(2684) = p(9)*p(921)
p(2685) = p(9)*p(827)
p(2686) = p(9)*p(922)
p(2687) = p(9)*p(828)
p(2688) = p(9)*p(829)
p(2689) = p(9)*p(923)
p(2690) = p(14)*p(902)
p(2691) = p(14)*p(903)
p(2692) = p(14)*p(904)
p(2693) = p(14)*p(905)
p(2694) = p(14)*p(906)
p(2695) = p(9)*p(924)
p(2696) = p(9)*p(925)
p(2697) = p(9)*p(926)
p(2698) = p(14)*p(812)
p(2699) = p(14)*p(813)
p(2700) = p(14)*p(814)
p(2701) = p(14)*p(815)
p(2702) = p(14)*p(816)
p(2703) = p(14)*p(817)
p(2704) = p(14)*p(678)
p(2705) = p(14)*p(679)
p(2706) = p(9)*p(951)
p(2707) = p(14)*p(820)
p(2708) = p(14)*p(821)
p(2709) = p(14)*p(822)
p(2710) = p(9)*p(930)
p(2711) = p(9)*p(931)
p(2712) = p(9)*p(932)
p(2713) = p(14)*p(918)
p(2714) = p(14)*p(919)
p(2715) = p(14)*p(920)
p(2716) = p(9)*p(954)
p(2717) = p(14)*p(827)
p(2718) = p(14)*p(828)
p(2719) = p(14)*p(829)
p(2720) = p(9)*p(957)
p(2721) = p(14)*p(928)
p(2722) = p(14)*p(929)
p(2723) = p(9)*p(938)
p(2724) = p(9)*p(839)
p(2725) = p(9)*p(680)
p(2726) = p(9)*p(840)
p(2727) = p(9)*p(681)
p(2728) = p(9)*p(682)
p(2729) = p(9)*p(841)
p(2730) = p(9)*p(683)
p(2731) = p(9)*p(684)
p(2732) = p(9)*p(685)
p(2733) = p(9)*p(842)
p(2734) = p(9)*p(686)
p(2735) = p(9)*p(687)
p(2736) = p(9)*p(688)
p(2737) = p(9)*p(843)
p(2738) = p(9)*p(844)
p(2739) = p(9)*p(689)
p(2740) = p(9)*p(690)
p(2741) = p(9)*p(845)
p(2742) = p(9)*p(846)
p(2743) = p(9)*p(691)
p(2744) = p(9)*p(847)
p(2745) = p(9)*p(848)
p(2746) = p(9)*p(849)
p(2747) = p(9)*p(939)
p(2748) = p(3)*p(1136) - p(2383)
p(2749) = p(7)*p(1136) - p(2387)
p(2750) = p(49)*p(401) - p(2394)
p(2751) = p(114)*p(194) - p(2399)
p(2752) = p(24)*p(588) - p(2402)
p(2753) = p(3)*p(1368) - p(2386) - p(2752)
p(2754) = p(1)*p(1802) - p(2386) - p(2753)
p(2755) = p(1)*p(1803) - p(2390) - p(2754)
p(2756) = p(1)*p(1804) - p(2393) - p(2755)
p(2757) = p(6)*p(1476) - p(2748)
p(2758) = p(32)*p(566) - p(2387)
p(2759) = p(150)*p(178) - p(2394)
p(2760) = p(39)*p(505) - p(2399)
p(2761) = p(3)*p(1369) - p(2402)
p(2762) = p(14)*p(958)
p(2763) = p(9)*p(940)
p(2764) = p(9)*p(852)
p(2765) = p(9)*p(702)
p(2766) = p(9)*p(853)
p(2767) = p(9)*p(703)
p(2768) = p(9)*p(704)
p(2769) = p(9)*p(854)
p(2770) = p(9)*p(705)
p(2771) = p(9)*p(706)
p(2772) = p(9)*p(707)
p(2773) = p(9)*p(855)
p(2774) = p(9)*p(708)
p(2775) = p(9)*p(709)
p(2776) = p(9)*p(856)
p(2777) = p(9)*p(857)
p(2778) = p(9)*p(710)
p(2779) = p(9)*p(858)
p(2780) = p(9)*p(859)
p(2781) = p(9)*p(860)
p(2782) = p(9)*p(941)
p(2783) = p(24)*p(570) - p(2296)
p(2784) = p(114)*p(181) - p(2293)
p(2785) = p(1)*p(1828) - p(2408) - p(2784)
p(2786) = p(4)*p(1136) - p(2292)
p(2787) = p(1)*p(1830) - p(2410) - p(2786)
p(2788) = p(1)*p(1831) - p(2411) - p(2787)
p(2789) = p(1)*p(1832) - p(2412) - p(2788)
p(2790) = p(1)*p(1833) - p(2413) - p(2789)
p(2791) = p(13)*p(938) - p(2762)
p(2792) = p(2)*p(1802) - p(2406) - p(2791)
p(2793) = p(32)*p(570) - p(2762)
p(2794) = p(1)*p(1835) - p(2413) - p(2792)
p(2795) = p(73)*p(331) - p(2762)
p(2796) = p(1)*p(1837) - p(2419) - p(2794)
p(2797) = p(150)*p(181) - p(2762)
p(2798) = p(1)*p(1839) - p(2424) - p(2796)
p(2799) = p(91)*p(285) - p(2762)
p(2800) = p(13)*p(843) - p(2292)
p(2801) = p(41)*p(505) - p(2762)
p(2802) = p(1)*p(1843) - p(2431) - p(2800)
p(2803) = p(16)*p(851) - p(2762)
p(2804) = p(69)*p(285) - p(2293)
p(2805) = p(4)*p(1369) - p(2762)
p(2806) = p(11)*p(851) - p(2296)
p(2807) = p(14)*p(938)
p(2808) = p(14)*p(939)
p(2809) = p(3)*p(1180) - p(2446)
p(2810) = p(7)*p(1180) - p(2449)
p(2811) = p(49)*p(430) - p(2454)
p(2812) = p(130)*p(194) - p(2457)
p(2813) = p(3)*p(1384) - p(2448) - p(2812)
p(2814) = p(1)*p(1853) - p(2448) - p(2813)
p(2815) = p(1)*p(1854) - p(2451) - p(2814)
p(2816) = p(45)*p(566) - p(2809)
p(2817) = p(160)*p(178) - p(2449)
p(2818) = p(39)*p(518) - p(2454)
p(2819) = p(3)*p(1385) - p(2457)
p(2820) = p(14)*p(850)
p(2821) = p(14)*p(851)
p(2822) = p(14)*p(701)
p(2823) = p(9)*p(942)
p(2824) = p(9)*p(866)
p(2825) = p(9)*p(737)
p(2826) = p(9)*p(867)
p(2827) = p(9)*p(738)
p(2828) = p(9)*p(739)
p(2829) = p(9)*p(868)
p(2830) = p(9)*p(740)
p(2831) = p(9)*p(741)
p(2832) = p(9)*p(869)
p(2833) = p(9)*p(870)
p(2834) = p(9)*p(742)
p(2835) = p(9)*p(871)
p(2836) = p(9)*p(872)
p(2837) = p(9)*p(873)
p(2838) = p(9)*p(943)
p(2839) = p(24)*p(572) - p(2297)
p(2840) = p(114)*p(183) - p(2299)
p(2841) = p(43)*p(401) - p(2302)
p(2842) = p(8)*p(1136) - p(2439)
p(2843) = p(1)*p(1877) - p(2473) - p(2842)
p(2844) = p(1)*p(1878) - p(2474) - p(2843)
p(2845) = p(1)*p(1879) - p(2475) - p(2844)
p(2846) = p(13)*p(940) - p(2807)
p(2847) = p(2)*p(1835) - p(2445) - p(2813)
p(2848) = p(32)*p(572) - p(2807)
p(2849) = p(1)*p(1881) - p(2475) - p(2847)
p(2850) = p(73)*p(333) - p(2807)
p(2851) = p(1)*p(1883) - p(2480) - p(2849)
p(2852) = p(150)*p(183) - p(2807)
p(2853) = p(6)*p(1374) - p(2439)
p(2854) = p(93)*p(285) - p(2807)
p(2855) = p(32)*p(509) - p(2302)
p(2856) = p(43)*p(505) - p(2807)
p(2857) = p(150)*p(153) - p(2299)
p(2858) = p(18)*p(851) - p(2807)
p(2859) = p(34)*p(505) - p(2297)
p(2860) = p(14)*p(940)
p(2861) = p(14)*p(941)
p(2862) = p(130)*p(181) - p(2317)
p(2863) = p(41)*p(430) - p(2314)
p(2864) = p(4)*p(1180) - p(2309)
p(2865) = p(1)*p(1896) - p(2502) - p(2864)
p(2866) = p(1)*p(1897) - p(2503) - p(2865)
p(2867) = p(1)*p(1898) - p(2504) - p(2866)
p(2868) = p(81)*p(331) - p(2821)
p(2869) = p(2)*p(1853) - p(2461) - p(2868)
p(2870) = p(160)*p(181) - p(2821)
p(2871) = p(1)*p(1900) - p(2504) - p(2869)
p(2872) = p(91)*p(296) - p(2821)
p(2873) = p(1)*p(1902) - p(2508) - p(2871)
p(2874) = p(41)*p(518) - p(2821)
p(2875) = p(81)*p(279) - p(2309)
p(2876) = p(16)*p(865) - p(2821)
p(2877) = p(69)*p(296) - p(2314)
p(2878) = p(4)*p(1385) - p(2821)
p(2879) = p(11)*p(865) - p(2317)
p(2880) = p(14)*p(861)
p(2881) = p(14)*p(862)
p(2882) = p(14)*p(863)
p(2883) = p(14)*p(726)
p(2884) = p(14)*p(727)
p(2885) = p(3)*p(1234) - p(2536)
p(2886) = p(7)*p(1234) - p(2538)
p(2887) = p(49)*p(464) - p(2541)
p(2888) = p(3)*p(1401) - p(2537) - p(2887)
p(2889) = p(1)*p(1916) - p(2537) - p(2888)
p(2890) = p(178)*p(187) - p(2885)
p(2891) = p(39)*p(532) - p(2538)
p(2892) = p(3)*p(1402) - p(2541)
p(2893) = p(14)*p(864)
p(2894) = p(14)*p(865)
p(2895) = p(14)*p(734)
p(2896) = p(14)*p(735)
p(2897) = p(14)*p(736)
p(2898) = p(9)*p(944)
p(2899) = p(9)*p(883)
p(2900) = p(9)*p(778)
p(2901) = p(9)*p(884)
p(2902) = p(9)*p(779)
p(2903) = p(9)*p(780)
p(2904) = p(9)*p(885)
p(2905) = p(9)*p(781)
p(2906) = p(9)*p(886)
p(2907) = p(9)*p(887)
p(2908) = p(9)*p(888)
p(2909) = p(9)*p(945)
p(2910) = p(24)*p(574) - p(2321)
p(2911) = p(114)*p(185) - p(2323)
p(2912) = p(46)*p(401) - p(2326)
p(2913) = p(1)*p(1936) - p(2556) - p(2912)
p(2914) = p(1)*p(1937) - p(2557) - p(2913)
p(2915) = p(1)*p(1938) - p(2558) - p(2914)
p(2916) = p(13)*p(942) - p(2860)
p(2917) = p(2)*p(1881) - p(2499) - p(2869)
p(2918) = p(32)*p(574) - p(2860)
p(2919) = p(1)*p(1940) - p(2558) - p(2917)
p(2920) = p(73)*p(335) - p(2860)
p(2921) = p(1)*p(1942) - p(2562) - p(2919)
p(2922) = p(150)*p(185) - p(2860)
p(2923) = p(13)*p(869) - p(2326)
p(2924) = p(95)*p(285) - p(2860)
p(2925) = p(73)*p(299) - p(2323)
p(2926) = p(46)*p(505) - p(2860)
p(2927) = p(84)*p(285) - p(2321)
p(2928) = p(14)*p(942)
p(2929) = p(14)*p(943)
p(2930) = p(130)*p(183) - p(2320)
p(2931) = p(1)*p(1951) - p(2577) - p(2930)
p(2932) = p(8)*p(1180) - p(2467)
p(2933) = p(1)*p(1953) - p(2579) - p(2932)
p(2934) = p(1)*p(1954) - p(2580) - p(2933)
p(2935) = p(81)*p(333) - p(2822)
p(2936) = p(2)*p(1900) - p(2521) - p(2888)
p(2937) = p(160)*p(183) - p(2822)
p(2938) = p(1)*p(1956) - p(2580) - p(2936)
p(2939) = p(93)*p(296) - p(2822)
p(2940) = p(45)*p(509) - p(2467)
p(2941) = p(43)*p(518) - p(2822)
p(2942) = p(1)*p(1960) - p(2585) - p(2940)
p(2943) = p(18)*p(865) - p(2822)
p(2944) = p(34)*p(518) - p(2320)
p(2945) = p(14)*p(874)
p(2946) = p(14)*p(875)
p(2947) = p(14)*p(876)
p(2948) = p(14)*p(755)
p(2949) = p(14)*p(756)
p(2950) = p(41)*p(464) - p(2347)
p(2951) = p(4)*p(1234) - p(2344)
p(2952) = p(1)*p(1969) - p(2605) - p(2951)
p(2953) = p(1)*p(1970) - p(2606) - p(2952)
p(2954) = p(91)*p(310) - p(2894)
p(2955) = p(2)*p(1916) - p(2544) - p(2954)
p(2956) = p(41)*p(532) - p(2894)
p(2957) = p(1)*p(1972) - p(2606) - p(2955)
p(2958) = p(16)*p(881) - p(2894)
p(2959) = p(69)*p(310) - p(2344)
p(2960) = p(4)*p(1402) - p(2894)
p(2961) = p(11)*p(881) - p(2347)
p(2962) = p(14)*p(877)
p(2963) = p(14)*p(878)
p(2964) = p(14)*p(879)
p(2965) = p(14)*p(766)
p(2966) = p(14)*p(767)
p(2967) = p(14)*p(768)
p(2968) = p(14)*p(769)
p(2969) = p(14)*p(770)
p(2970) = p(3)*p(1290) - p(2627)
p(2971) = p(7)*p(1290) - p(2628)
p(2972) = p(3)*p(1421) - p(2627) - p(2971)
p(2973) = p(39)*p(578) - p(2970)
p(2974) = p(3)*p(1422) - p(2628)
p(2975) = p(14)*p(880)
p(2976) = p(14)*p(881)
p(2977) = p(14)*p(774)
p(2978) = p(14)*p(775)
p(2979) = p(14)*p(776)
p(2980) = p(14)*p(777)
p(2981) = p(9)*p(946)
p(2982) = p(9)*p(902)
p(2983) = p(9)*p(810)
p(2984) = p(9)*p(903)
p(2985) = p(9)*p(811)
p(2986) = p(9)*p(904)
p(2987) = p(9)*p(905)
p(2988) = p(9)*p(906)
p(2989) = p(9)*p(947)
p(2990) = p(24)*p(576) - p(2351)
p(2991) = p(114)*p(188) - p(2353)
p(2992) = p(50)*p(401) - p(2572)
p(2993) = p(1)*p(2000) - p(2636) - p(2992)
p(2994) = p(1)*p(2001) - p(2637) - p(2993)
p(2995) = p(13)*p(944) - p(2928)
p(2996) = p(2)*p(1940) - p(2576) - p(2936)
p(2997) = p(32)*p(576) - p(2928)
p(2998) = p(1)*p(2003) - p(2637) - p(2996)
p(2999) = p(73)*p(337) - p(2928)
p(3000) = p(6)*p(1406) - p(2572)
p(3001) = p(150)*p(188) - p(2928)
p(3002) = p(32)*p(536) - p(2353)
p(3003) = p(98)*p(285) - p(2928)
p(3004) = p(150)*p(173) - p(2351)
p(3005) = p(14)*p(944)
p(3006) = p(14)*p(945)
p(3007) = p(130)*p(185) - p(2340)
p(3008) = p(46)*p(430) - p(2342)
p(3009) = p(1)*p(2013) - p(2651) - p(3008)
p(3010) = p(1)*p(2014) - p(2652) - p(3009)
p(3011) = p(81)*p(335) - p(2883)
p(3012) = p(2)*p(1956) - p(2592) - p(2955)
p(3013) = p(160)*p(185) - p(2883)
p(3014) = p(1)*p(2016) - p(2652) - p(3012)
p(3015) = p(95)*p(296) - p(2883)
p(3016) = p(81)*p(299) - p(2342)
p(3017) = p(46)*p(518) - p(2883)
p(3018) = p(84)*p(296) - p(2340)
p(3019) = p(14)*p(889)
p(3020) = p(14)*p(890)
p(3021) = p(14)*p(891)
p(3022) = p(14)*p(791)
p(3023) = p(14)*p(792)
p(3024) = p(43)*p(464) - p(2349)
p(3025) = p(8)*p(1234) - p(2548)
p(3026) = p(1)*p(2027) - p(2669) - p(3025)
p(3027) = p(93)*p(310) - p(2896)
p(3028) = p(2)*p(1972) - p(2614) - p(2972)
p(3029) = p(43)*p(532) - p(2896)
p(3030) = p(153)*p(187) - p(2548)
p(3031) = p(18)*p(881) - p(2896)
p(3032) = p(34)*p(532) - p(2349)
p(3033) = p(14)*p(892)
p(3034) = p(14)*p(893)
p(3035) = p(14)*p(894)
p(3036) = p(14)*p(799)
p(3037) = p(14)*p(800)
p(3038) = p(14)*p(801)
p(3039) = p(14)*p(802)
p(3040) = p(14)*p(803)
p(3041) = p(4)*p(1290) - p(2369)
p(3042) = p(1)*p(2038) - p(2680) - p(3041)
p(3043) = p(16)*p(900) - p(2976)
p(3044) = p(2)*p(1986) - p(2630) - p(3043)
p(3045) = p(4)*p(1422) - p(2976)
p(3046) = p(11)*p(900) - p(2369)
p(3047) = p(14)*p(895)
p(3048) = p(14)*p(896)
p(3049) = p(14)*p(897)
p(3050) = p(14)*p(807)
p(3051) = p(14)*p(808)
p(3052) = p(14)*p(809)
p(3053) = p(6)*p(1290) - p(2681)
p(3054) = p(3)*p(1491) - p(3053)
p(3055) = p(14)*p(899)
p(3056) = p(14)*p(900)
p(3057) = p(9)*p(949)
p(3058) = p(9)*p(918)
p(3059) = p(9)*p(826)
p(3060) = p(9)*p(919)
p(3061) = p(9)*p(920)
p(3062) = p(9)*p(950)
p(3063) = p(24)*p(579) - p(2370)
p(3064) = p(114)*p(191) - p(2372)
p(3065) = p(1)*p(2049) - p(2685) - p(3064)
p(3066) = p(1)*p(2050) - p(2686) - p(3065)
p(3067) = p(13)*p(946) - p(3005)
p(3068) = p(2)*p(2003) - p(2649) - p(3012)
p(3069) = p(32)*p(579) - p(3005)
p(3070) = p(1)*p(2052) - p(2686) - p(3068)
p(3071) = p(73)*p(340) - p(3005)
p(3072) = p(13)*p(904) - p(2372)
p(3073) = p(150)*p(191) - p(3005)
p(3074) = p(73)*p(323) - p(2370)
p(3075) = p(14)*p(946)
p(3076) = p(14)*p(947)
p(3077) = p(130)*p(188) - p(2363)
p(3078) = p(50)*p(430) - p(2601)
p(3079) = p(1)*p(2060) - p(2696) - p(3078)
p(3080) = p(81)*p(337) - p(2948)
p(3081) = p(2)*p(2016) - p(2660) - p(3028)
p(3082) = p(160)*p(188) - p(2948)
p(3083) = p(45)*p(536) - p(2601)
p(3084) = p(98)*p(296) - p(2948)
p(3085) = p(160)*p(173) - p(2363)
p(3086) = p(14)*p(907)
p(3087) = p(14)*p(908)
p(3088) = p(14)*p(909)
p(3089) = p(14)*p(818)
p(3090) = p(14)*p(819)
p(3091) = p(2)*p(2026) - p(2671) - p(3041)
p(3092) = p(1)*p(2070) - p(2706) - p(3091)
p(3093) = p(95)*p(310) - p(2897)
p(3094) = p(2)*p(2029) - p(2674) - p(3044)
p(3095) = p(46)*p(532) - p(2897)
p(3096) = p(1)*p(2072) - p(2706) - p(3094)
p(3097) = p(14)*p(910)
p(3098) = p(14)*p(911)
p(3099) = p(14)*p(912)
p(3100) = p(14)*p(823)
p(3101) = p(14)*p(824)
p(3102) = p(14)*p(825)
p(3103) = p(8)*p(1290) - p(2632)
p(3104) = p(18)*p(900) - p(2978)
p(3105) = p(34)*p(578) - p(2632)
p(3106) = p(14)*p(914)
p(3107) = p(14)*p(915)
p(3108) = p(14)*p(916)
p(3109) = p(9)*p(952)
p(3110) = p(9)*p(928)
p(3111) = p(9)*p(929)
p(3112) = p(9)*p(953)
p(3113) = p(24)*p(582) - p(2379)
p(3114) = p(114)*p(195) - p(2692)
p(3115) = p(1)*p(2079) - p(2711) - p(3114)
p(3116) = p(13)*p(949) - p(3075)
p(3117) = p(2)*p(2052) - p(2694) - p(3081)
p(3118) = p(32)*p(582) - p(3075)
p(3119) = p(6)*p(1446) - p(2692)
p(3120) = p(73)*p(343) - p(3075)
p(3121) = p(32)*p(561) - p(2379)
p(3122) = p(14)*p(949)
p(3123) = p(14)*p(950)
p(3124) = p(130)*p(191) - p(2377)
p(3125) = p(1)*p(2086) - p(2716) - p(3124)
p(3126) = p(81)*p(340) - p(3022)
p(3127) = p(2)*p(2062) - p(2701) - p(3094)
p(3128) = p(160)*p(191) - p(3022)
p(3129) = p(81)*p(323) - p(2377)
p(3130) = p(14)*p(921)
p(3131) = p(14)*p(922)
p(3132) = p(14)*p(923)
p(3133) = p(14)*p(830)
p(3134) = p(14)*p(831)
p(3135) = p(50)*p(464) - p(2626)
p(3136) = p(98)*p(310) - p(2968)
p(3137) = p(173)*p(187) - p(2626)
p(3138) = p(14)*p(924)
p(3139) = p(14)*p(925)
p(3140) = p(14)*p(926)
p(3141) = p(9)*p(955)
p(3142) = p(9)*p(956)
p(3143) = p(24)*p(585) - p(2382)
p(3144) = p(1)*p(2093) - p(2720) - p(3143)
p(3145) = p(13)*p(952) - p(3122)
p(3146) = p(2)*p(2081) - p(2715) - p(3127)
p(3147) = p(32)*p(585) - p(3122)
p(3148) = p(13)*p(929) - p(2382)
p(3149) = p(14)*p(952)
p(3150) = p(14)*p(953)
p(3151) = p(130)*p(195) - p(2705)
p(3152) = p(81)*p(343) - p(3089)
p(3153) = p(45)*p(561) - p(2705)
p(3154) = p(14)*p(930)
p(3155) = p(14)*p(931)
p(3156) = p(14)*p(932)
p(3157) = p(9)*p(959)
p(3158) = p(24)*p(589) - p(2722)
p(3159) = p(13)*p(955) - p(3149)
p(3160) = p(6)*p(1471) - p(2722)
p(3161) = p(14)*p(955)
p(3162) = p(14)*p(956)
p(3163) = p(9)*p(832)
p(3164) = p(9)*p(833)
p(3165) = p(9)*p(834)
p(3166) = p(9)*p(835)
p(3167) = p(9)*p(836)
p(3168) = p(9)*p(934)
p(3169) = p(9)*p(837)
p(3170) = p(9)*p(935)
p(3171) = p(9)*p(838)
p(3172) = p(9)*p(936)
p(3173) = p(9)*p(937)
p(3174) = p(9)*p(958)
p(3175) = p(3)*p(1481) - p(2723)
p(3176) = p(39)*p(570) - p(2725)
p(3177) = p(178)*p(181) - p(2728)
p(3178) = p(41)*p(566) - p(2732)
p(3179) = p(4)*p(1476) - p(2737)
p(3180) = p(1)*p(2112) - p(2733) - p(3179)
p(3181) = p(1)*p(2113) - p(2737)
p(3182) = p(1)*p(2114) - p(2738) - p(3180)
p(3183) = p(21)*p(843) - p(2732)
p(3184) = p(1)*p(2116) - p(2742) - p(3182)
p(3185) = p(101)*p(279) - p(2728)
p(3186) = p(1)*p(2118) - p(2745) - p(3184)
p(3187) = p(69)*p(346) - p(2725)
p(3188) = p(1)*p(2120) - p(2747) - p(3186)
p(3189) = p(11)*p(958) - p(2723)
p(3190) = p(1)*p(2122) - p(2753) - p(2752)
p(3191) = p(1)*p(2123) - p(2761)
p(3192) = p(3)*p(1483) - p(2763)
p(3193) = p(39)*p(572) - p(2765)
p(3194) = p(178)*p(183) - p(2768)
p(3195) = p(43)*p(566) - p(2772)
p(3196) = p(1)*p(2127) - p(2769) - p(3195)
p(3197) = p(2)*p(2113) - p(2757)
p(3198) = p(1)*p(2128) - p(2773) - p(3196)
p(3199) = p(7)*p(1374) - p(2772)
p(3200) = p(1)*p(2130) - p(2777) - p(3198)
p(3201) = p(49)*p(509) - p(2768)
p(3202) = p(1)*p(2132) - p(2780) - p(3200)
p(3203) = p(153)*p(194) - p(2765)
p(3204) = p(1)*p(2134) - p(2782) - p(3202)
p(3205) = p(34)*p(588) - p(2763)
p(3206) = p(6)*p(1481) - p(2762)
p(3207) = p(1)*p(2137) - p(2792) - p(2790)
p(3208) = p(2)*p(2123) - p(2762)
p(3209) = p(1)*p(2139) - p(2813) - p(2812)
p(3210) = p(1)*p(2140) - p(2819)
p(3211) = p(3)*p(1485) - p(2823)
p(3212) = p(39)*p(574) - p(2825)
p(3213) = p(178)*p(185) - p(2828)
p(3214) = p(46)*p(566) - p(2832)
p(3215) = p(1)*p(2144) - p(2829) - p(3214)
p(3216) = p(1)*p(2145) - p(2832)
p(3217) = p(1)*p(2146) - p(2833) - p(3215)
p(3218) = p(21)*p(869) - p(2828)
p(3219) = p(1)*p(2148) - p(2836) - p(3217)
p(3220) = p(101)*p(299) - p(2825)
p(3221) = p(1)*p(2150) - p(2838) - p(3219)
p(3222) = p(84)*p(346) - p(2823)
p(3223) = p(6)*p(1483) - p(2807)
p(3224) = p(1)*p(2153) - p(2847) - p(2845)
p(3225) = p(8)*p(1369) - p(2807)
p(3226) = p(45)*p(570) - p(2821)
p(3227) = p(1)*p(2156) - p(2869) - p(2867)
p(3228) = p(2)*p(2140) - p(2821)
p(3229) = p(1)*p(2158) - p(2888) - p(2887)
p(3230) = p(1)*p(2159) - p(2892)
p(3231) = p(3)*p(1487) - p(2898)
p(3232) = p(39)*p(576) - p(2900)
p(3233) = p(178)*p(188) - p(2903)
p(3234) = p(1)*p(2162) - p(2901) - p(3233)
p(3235) = p(2)*p(2145) - p(2853)
p(3236) = p(1)*p(2163) - p(2904) - p(3234)
p(3237) = p(7)*p(1406) - p(2903)
p(3238) = p(1)*p(2165) - p(2907) - p(3236)
p(3239) = p(49)*p(536) - p(2900)
p(3240) = p(1)*p(2167) - p(2909) - p(3238)
p(3241) = p(173)*p(194) - p(2898)
p(3242) = p(6)*p(1485) - p(2860)
p(3243) = p(1)*p(2170) - p(2917) - p(2915)
p(3244) = p(22)*p(851) - p(2860)
p(3245) = p(45)*p(572) - p(2822)
p(3246) = p(1)*p(2173) - p(2936) - p(2934)
p(3247) = p(8)*p(1385) - p(2822)
p(3248) = p(181)*p(187) - p(2894)
p(3249) = p(1)*p(2176) - p(2955) - p(2953)
p(3250) = p(2)*p(2159) - p(2894)
p(3251) = p(1)*p(2178) - p(2972) - p(2971)
p(3252) = p(1)*p(2179) - p(2974)
p(3253) = p(14)*p(882)
p(3254) = p(3)*p(1489) - p(2981)
p(3255) = p(39)*p(579) - p(2983)
p(3256) = p(178)*p(191) - p(2986)
p(3257) = p(1)*p(2183) - p(2984) - p(3256)
p(3258) = p(1)*p(2184) - p(2986)
p(3259) = p(1)*p(2185) - p(2987) - p(3257)
p(3260) = p(21)*p(904) - p(2983)
p(3261) = p(1)*p(2187) - p(2989) - p(3259)
p(3262) = p(101)*p(323) - p(2981)
p(3263) = p(6)*p(1487) - p(2928)
p(3264) = p(1)*p(2190) - p(2996) - p(2994)
p(3265) = p(50)*p(505) - p(2928)
p(3266) = p(45)*p(574) - p(2883)
p(3267) = p(1)*p(2193) - p(3012) - p(3010)
p(3268) = p(22)*p(865) - p(2883)
p(3269) = p(183)*p(187) - p(2896)
p(3270) = p(1)*p(2196) - p(3028) - p(3026)
p(3271) = p(8)*p(1402) - p(2896)
p(3272) = p(41)*p(578) - p(2976)
p(3273) = p(1)*p(2200) - p(3044) - p(3042)
p(3274) = p(2)*p(2179) - p(2976)
p(3275) = p(14)*p(898)
p(3276) = p(4)*p(1441) - p(2683) - p(3103)
p(3277) = p(1)*p(2204) - p(3054)
p(3278) = p(14)*p(901)
p(3279) = p(3)*p(1492) - p(3057)
p(3280) = p(39)*p(582) - p(3059)
p(3281) = p(1)*p(2207) - p(3058) - p(3280)
p(3282) = p(2)*p(2184) - p(3000)
p(3283) = p(1)*p(2208) - p(3060) - p(3281)
p(3284) = p(7)*p(1446) - p(3059)
p(3285) = p(1)*p(2210) - p(3062) - p(3283)
p(3286) = p(49)*p(561) - p(3057)
p(3287) = p(6)*p(1489) - p(3005)
p(3288) = p(1)*p(2213) - p(3068) - p(3066)
p(3289) = p(102)*p(285) - p(3005)
p(3290) = p(45)*p(576) - p(2948)
p(3291) = p(1)*p(2216) - p(3081) - p(3079)
p(3292) = p(50)*p(518) - p(2948)
p(3293) = p(185)*p(187) - p(2897)
p(3294) = p(1)*p(2219) - p(3094) - p(3092)
p(3295) = p(22)*p(881) - p(2897)
p(3296) = p(14)*p(913)
p(3297) = p(43)*p(578) - p(2978)
p(3298) = p(1)*p(2223) - p(3105) - p(3103)
p(3299) = p(8)*p(1422) - p(2978)
p(3300) = p(14)*p(917)
p(3301) = p(4)*p(1491) - p(3056)
p(3302) = p(2)*p(2203) - p(3055) - p(3301)
p(3303) = p(2)*p(2204) - p(3056)
p(3304) = p(14)*p(948)
p(3305) = p(3)*p(1495) - p(3109)
p(3306) = p(39)*p(585) - p(3111)
p(3307) = p(1)*p(2227) - p(3110) - p(3306)
p(3308) = p(1)*p(2228) - p(3111)
p(3309) = p(1)*p(2229) - p(3112) - p(3307)
p(3310) = p(21)*p(929) - p(3109)
p(3311) = p(6)*p(1492) - p(3075)
p(3312) = p(1)*p(2232) - p(3117) - p(3115)
p(3313) = p(150)*p(195) - p(3075)
p(3314) = p(45)*p(579) - p(3022)
p(3315) = p(1)*p(2235) - p(3127) - p(3125)
p(3316) = p(102)*p(296) - p(3022)
p(3317) = p(187)*p(188) - p(2968)
p(3318) = p(1)*p(2239) - p(3137) - p(3135)
p(3319) = p(50)*p(532) - p(2968)
p(3320) = p(14)*p(927)
p(3321) = p(46)*p(578) - p(2980)
p(3322) = p(2)*p(2223) - p(3107) - p(3302)
p(3323) = p(22)*p(900) - p(2980)
p(3324) = p(14)*p(951)
p(3325) = p(3)*p(1498) - p(3141)
p(3326) = p(2)*p(2227) - p(3118) - p(3114)
p(3327) = p(2)*p(2228) - p(3119)
p(3328) = p(1)*p(2243) - p(3142) - p(3326)
p(3329) = p(7)*p(1471) - p(3141)
p(3330) = p(6)*p(1495) - p(3122)
p(3331) = p(1)*p(2246) - p(3146) - p(3144)
p(3332) = p(73)*p(347) - p(3122)
p(3333) = p(45)*p(582) - p(3089)
p(3334) = p(1)*p(2249) - p(3153) - p(3151)
p(3335) = p(160)*p(195) - p(3089)
p(3336) = p(14)*p(933)
p(3337) = p(187)*p(191) - p(3039)
p(3338) = p(2)*p(2239) - p(3139) - p(3322)
p(3339) = p(102)*p(310) - p(3039)
p(3340) = p(14)*p(954)
p(3341) = p(3)*p(1501) - p(3157)
p(3342) = p(2)*p(2243) - p(3147) - p(3144)
p(3343) = p(1)*p(2253) - p(3157)
p(3344) = p(6)*p(1498) - p(3149)
p(3345) = p(1)*p(2255) - p(3160) - p(3158)
p(3346) = p(32)*p(589) - p(3149)
p(3347) = p(45)*p(585) - p(3133)
p(3348) = p(2)*p(2249) - p(3155) - p(3338)
p(3349) = p(81)*p(347) - p(3133)
p(3350) = p(14)*p(957)
p(3351) = p(2)*p(2252) - p(3159) - p(3158)
p(3352) = p(2)*p(2253) - p(3160)
p(3353) = p(6)*p(1501) - p(3161)
p(3354) = p(2)*p(2255) - p(3162) - p(3348)
p(3355) = p(13)*p(959) - p(3161)
p(3356) = p(14)*p(959)
p(3357) = p(3)*p(1476) - p(3168)
p(3358) = p(7)*p(1476) - p(3165)
p(3359) = p(1)*p(2259) - p(3170) - p(3358)
p(3360) = p(178)*p(194) - p(3166)
p(3361) = p(39)*p(588) - p(3171)
p(3362) = p(3)*p(1504) - p(3174)
p(3363) = p(1)*p(2263) - p(3175)
p(3364) = p(2)*p(2288) - p(3363)
p(3365) = p(1)*p(2265) - p(3192)
p(3366) = p(8)*p(1504) - p(3365)
p(3367) = p(1)*p(2267) - p(3211)
p(3368) = p(22)*p(958) - p(3367)
p(3369) = p(1)*p(2269) - p(3231)
p(3370) = p(50)*p(588) - p(3369)
p(3371) = p(1)*p(2271) - p(3254)
p(3372) = p(102)*p(346) - p(3371)
p(3373) = p(1)*p(2273) - p(3279)
p(3374) = p(194)*p(195) - p(3373)
p(3375) = p(6)*p(1491) - p(3304)
p(3376) = p(1)*p(2276) - p(3305)
p(3377) = p(101)*p(347) - p(3376)
p(3378) = p(8)*p(1491) - p(3278)
p(3379) = p(1)*p(2279) - p(3325)
p(3380) = p(49)*p(589) - p(3379)
p(3381) = p(2)*p(2278) - p(3324) - p(3378)
p(3382) = p(1)*p(2282) - p(3341)
p(3383) = p(21)*p(959) - p(3382)
p(3384) = p(187)*p(195) - p(3296)
p(3385) = p(1)*p(2285) - p(3351)
p(3386) = p(7)*p(1505) - p(3385)
p(3387) = p(45)*p(589) - p(3336)
p(3388) = p(2)*p(2285) - p(3353)
p(3389) = p(1)*p(2289) - p(3388)
p(3390) = p(6)*p(1505) - p(3356)
p(3391) = p(1)*p(2288) - p(3362)
p(3392) = p(2)*p(2289) - p(3390)

do j=1,3393
  bm1(j)=p(j-1)
enddo
 
bm2(1)=bm1(4)
bm2(2)=bm1(10)
bm2(3)=bm1(11)
bm2(4)=bm1(12)
bm2(5)=bm1(13)
bm2(6)=bm1(15)
bm2(7)=bm1(16)
bm2(8)=bm1(24)
bm2(9)=bm1(25)
bm2(10)=bm1(26)
bm2(11)=bm1(27)
bm2(12)=bm1(28)
bm2(13)=bm1(29)
bm2(14)=bm1(30)
bm2(15)=bm1(31)
bm2(16)=bm1(32)
bm2(17)=bm1(34)
bm2(18)=bm1(35)
bm2(19)=bm1(36)
bm2(20)=bm1(37)
bm2(21)=bm1(39)
bm2(22)=bm1(40)
bm2(23)=bm1(41)
bm2(24)=bm1(52)
bm2(25)=bm1(53)
bm2(26)=bm1(54)
bm2(27)=bm1(55)
bm2(28)=bm1(56)
bm2(29)=bm1(57)
bm2(30)=bm1(58)
bm2(31)=bm1(59)
bm2(32)=bm1(60)
bm2(33)=bm1(61)
bm2(34)=bm1(62)
bm2(35)=bm1(63)
bm2(36)=bm1(64)
bm2(37)=bm1(65)
bm2(38)=bm1(66)
bm2(39)=bm1(67)
bm2(40)=bm1(68)
bm2(41)=bm1(69)
bm2(42)=bm1(70)
bm2(43)=bm1(71)
bm2(44)=bm1(72)
bm2(45)=bm1(73)
bm2(46)=bm1(75)
bm2(47)=bm1(76)
bm2(48)=bm1(77)
bm2(49)=bm1(78)
bm2(50)=bm1(79)
bm2(51)=bm1(81)
bm2(52)=bm1(83)
bm2(53)=bm1(84)
bm2(54)=bm1(85)
bm2(55)=bm1(86)
bm2(56)=bm1(87)
bm2(57)=bm1(89)
bm2(58)=bm1(90)
bm2(59)=bm1(91)
bm2(60)=bm1(104)
bm2(61)=bm1(105)
bm2(62)=bm1(106)
bm2(63)=bm1(107)
bm2(64)=bm1(108)
bm2(65)=bm1(109)
bm2(66)=bm1(110)
bm2(67)=bm1(111)
bm2(68)=bm1(112)
bm2(69)=bm1(113)
bm2(70)=bm1(114)
bm2(71)=bm1(115)
bm2(72)=bm1(116)
bm2(73)=bm1(117)
bm2(74)=bm1(118)
bm2(75)=bm1(119)
bm2(76)=bm1(120)
bm2(77)=bm1(121)
bm2(78)=bm1(122)
bm2(79)=bm1(123)
bm2(80)=bm1(124)
bm2(81)=bm1(125)
bm2(82)=bm1(126)
bm2(83)=bm1(127)
bm2(84)=bm1(128)
bm2(85)=bm1(129)
bm2(86)=bm1(130)
bm2(87)=bm1(131)
bm2(88)=bm1(132)
bm2(89)=bm1(133)
bm2(90)=bm1(134)
bm2(91)=bm1(135)
bm2(92)=bm1(136)
bm2(93)=bm1(137)
bm2(94)=bm1(138)
bm2(95)=bm1(139)
bm2(96)=bm1(140)
bm2(97)=bm1(141)
bm2(98)=bm1(142)
bm2(99)=bm1(143)
bm2(100)=bm1(144)
bm2(101)=bm1(145)
bm2(102)=bm1(146)
bm2(103)=bm1(147)
bm2(104)=bm1(148)
bm2(105)=bm1(149)
bm2(106)=bm1(150)
bm2(107)=bm1(152)
bm2(108)=bm1(153)
bm2(109)=bm1(154)
bm2(110)=bm1(155)
bm2(111)=bm1(156)
bm2(112)=bm1(157)
bm2(113)=bm1(158)
bm2(114)=bm1(160)
bm2(115)=bm1(162)
bm2(116)=bm1(163)
bm2(117)=bm1(164)
bm2(118)=bm1(165)
bm2(119)=bm1(166)
bm2(120)=bm1(167)
bm2(121)=bm1(169)
bm2(122)=bm1(170)
bm2(123)=bm1(172)
bm2(124)=bm1(173)
bm2(125)=bm1(174)
bm2(126)=bm1(175)
bm2(127)=bm1(176)
bm2(128)=bm1(178)
bm2(129)=bm1(179)
bm2(130)=bm1(180)
bm2(131)=bm1(181)
bm2(132)=bm1(197)
bm2(133)=bm1(198)
bm2(134)=bm1(199)
bm2(135)=bm1(200)
bm2(136)=bm1(201)
bm2(137)=bm1(202)
bm2(138)=bm1(203)
bm2(139)=bm1(204)
bm2(140)=bm1(205)
bm2(141)=bm1(206)
bm2(142)=bm1(207)
bm2(143)=bm1(208)
bm2(144)=bm1(209)
bm2(145)=bm1(210)
bm2(146)=bm1(211)
bm2(147)=bm1(212)
bm2(148)=bm1(213)
bm2(149)=bm1(214)
bm2(150)=bm1(215)
bm2(151)=bm1(216)
bm2(152)=bm1(217)
bm2(153)=bm1(218)
bm2(154)=bm1(219)
bm2(155)=bm1(220)
bm2(156)=bm1(221)
bm2(157)=bm1(222)
bm2(158)=bm1(223)
bm2(159)=bm1(224)
bm2(160)=bm1(225)
bm2(161)=bm1(226)
bm2(162)=bm1(227)
bm2(163)=bm1(228)
bm2(164)=bm1(229)
bm2(165)=bm1(230)
bm2(166)=bm1(231)
bm2(167)=bm1(232)
bm2(168)=bm1(233)
bm2(169)=bm1(234)
bm2(170)=bm1(235)
bm2(171)=bm1(236)
bm2(172)=bm1(237)
bm2(173)=bm1(238)
bm2(174)=bm1(239)
bm2(175)=bm1(240)
bm2(176)=bm1(241)
bm2(177)=bm1(242)
bm2(178)=bm1(243)
bm2(179)=bm1(244)
bm2(180)=bm1(245)
bm2(181)=bm1(246)
bm2(182)=bm1(247)
bm2(183)=bm1(248)
bm2(184)=bm1(249)
bm2(185)=bm1(250)
bm2(186)=bm1(251)
bm2(187)=bm1(252)
bm2(188)=bm1(253)
bm2(189)=bm1(254)
bm2(190)=bm1(255)
bm2(191)=bm1(256)
bm2(192)=bm1(257)
bm2(193)=bm1(258)
bm2(194)=bm1(259)
bm2(195)=bm1(260)
bm2(196)=bm1(261)
bm2(197)=bm1(262)
bm2(198)=bm1(263)
bm2(199)=bm1(264)
bm2(200)=bm1(265)
bm2(201)=bm1(266)
bm2(202)=bm1(267)
bm2(203)=bm1(268)
bm2(204)=bm1(269)
bm2(205)=bm1(270)
bm2(206)=bm1(271)
bm2(207)=bm1(272)
bm2(208)=bm1(273)
bm2(209)=bm1(274)
bm2(210)=bm1(275)
bm2(211)=bm1(276)
bm2(212)=bm1(277)
bm2(213)=bm1(278)
bm2(214)=bm1(279)
bm2(215)=bm1(280)
bm2(216)=bm1(281)
bm2(217)=bm1(282)
bm2(218)=bm1(283)
bm2(219)=bm1(284)
bm2(220)=bm1(285)
bm2(221)=bm1(287)
bm2(222)=bm1(288)
bm2(223)=bm1(289)
bm2(224)=bm1(290)
bm2(225)=bm1(291)
bm2(226)=bm1(292)
bm2(227)=bm1(293)
bm2(228)=bm1(294)
bm2(229)=bm1(296)
bm2(230)=bm1(298)
bm2(231)=bm1(299)
bm2(232)=bm1(300)
bm2(233)=bm1(301)
bm2(234)=bm1(302)
bm2(235)=bm1(303)
bm2(236)=bm1(304)
bm2(237)=bm1(306)
bm2(238)=bm1(307)
bm2(239)=bm1(309)
bm2(240)=bm1(310)
bm2(241)=bm1(312)
bm2(242)=bm1(313)
bm2(243)=bm1(314)
bm2(244)=bm1(315)
bm2(245)=bm1(316)
bm2(246)=bm1(317)
bm2(247)=bm1(319)
bm2(248)=bm1(320)
bm2(249)=bm1(322)
bm2(250)=bm1(323)
bm2(251)=bm1(324)
bm2(252)=bm1(325)
bm2(253)=bm1(326)
bm2(254)=bm1(328)
bm2(255)=bm1(329)
bm2(256)=bm1(330)
bm2(257)=bm1(331)
bm2(258)=bm1(349)
bm2(259)=bm1(350)
bm2(260)=bm1(351)
bm2(261)=bm1(352)
bm2(262)=bm1(353)
bm2(263)=bm1(354)
bm2(264)=bm1(355)
bm2(265)=bm1(356)
bm2(266)=bm1(357)
bm2(267)=bm1(358)
bm2(268)=bm1(359)
bm2(269)=bm1(360)
bm2(270)=bm1(361)
bm2(271)=bm1(362)
bm2(272)=bm1(363)
bm2(273)=bm1(364)
bm2(274)=bm1(365)
bm2(275)=bm1(366)
bm2(276)=bm1(367)
bm2(277)=bm1(368)
bm2(278)=bm1(369)
bm2(279)=bm1(370)
bm2(280)=bm1(371)
bm2(281)=bm1(372)
bm2(282)=bm1(373)
bm2(283)=bm1(374)
bm2(284)=bm1(375)
bm2(285)=bm1(376)
bm2(286)=bm1(377)
bm2(287)=bm1(378)
bm2(288)=bm1(379)
bm2(289)=bm1(380)
bm2(290)=bm1(381)
bm2(291)=bm1(382)
bm2(292)=bm1(383)
bm2(293)=bm1(384)
bm2(294)=bm1(385)
bm2(295)=bm1(386)
bm2(296)=bm1(387)
bm2(297)=bm1(388)
bm2(298)=bm1(389)
bm2(299)=bm1(390)
bm2(300)=bm1(391)
bm2(301)=bm1(392)
bm2(302)=bm1(393)
bm2(303)=bm1(394)
bm2(304)=bm1(395)
bm2(305)=bm1(396)
bm2(306)=bm1(397)
bm2(307)=bm1(398)
bm2(308)=bm1(399)
bm2(309)=bm1(400)
bm2(310)=bm1(401)
bm2(311)=bm1(402)
bm2(312)=bm1(403)
bm2(313)=bm1(404)
bm2(314)=bm1(405)
bm2(315)=bm1(406)
bm2(316)=bm1(407)
bm2(317)=bm1(408)
bm2(318)=bm1(409)
bm2(319)=bm1(410)
bm2(320)=bm1(411)
bm2(321)=bm1(412)
bm2(322)=bm1(413)
bm2(323)=bm1(414)
bm2(324)=bm1(415)
bm2(325)=bm1(416)
bm2(326)=bm1(417)
bm2(327)=bm1(418)
bm2(328)=bm1(419)
bm2(329)=bm1(420)
bm2(330)=bm1(421)
bm2(331)=bm1(422)
bm2(332)=bm1(423)
bm2(333)=bm1(424)
bm2(334)=bm1(425)
bm2(335)=bm1(426)
bm2(336)=bm1(427)
bm2(337)=bm1(428)
bm2(338)=bm1(429)
bm2(339)=bm1(430)
bm2(340)=bm1(431)
bm2(341)=bm1(432)
bm2(342)=bm1(433)
bm2(343)=bm1(434)
bm2(344)=bm1(435)
bm2(345)=bm1(436)
bm2(346)=bm1(437)
bm2(347)=bm1(438)
bm2(348)=bm1(439)
bm2(349)=bm1(440)
bm2(350)=bm1(441)
bm2(351)=bm1(442)
bm2(352)=bm1(443)
bm2(353)=bm1(444)
bm2(354)=bm1(445)
bm2(355)=bm1(446)
bm2(356)=bm1(447)
bm2(357)=bm1(448)
bm2(358)=bm1(449)
bm2(359)=bm1(450)
bm2(360)=bm1(451)
bm2(361)=bm1(452)
bm2(362)=bm1(453)
bm2(363)=bm1(454)
bm2(364)=bm1(455)
bm2(365)=bm1(456)
bm2(366)=bm1(457)
bm2(367)=bm1(458)
bm2(368)=bm1(459)
bm2(369)=bm1(460)
bm2(370)=bm1(461)
bm2(371)=bm1(462)
bm2(372)=bm1(463)
bm2(373)=bm1(464)
bm2(374)=bm1(465)
bm2(375)=bm1(466)
bm2(376)=bm1(467)
bm2(377)=bm1(468)
bm2(378)=bm1(469)
bm2(379)=bm1(470)
bm2(380)=bm1(471)
bm2(381)=bm1(472)
bm2(382)=bm1(473)
bm2(383)=bm1(474)
bm2(384)=bm1(475)
bm2(385)=bm1(476)
bm2(386)=bm1(477)
bm2(387)=bm1(478)
bm2(388)=bm1(479)
bm2(389)=bm1(480)
bm2(390)=bm1(481)
bm2(391)=bm1(482)
bm2(392)=bm1(483)
bm2(393)=bm1(484)
bm2(394)=bm1(485)
bm2(395)=bm1(486)
bm2(396)=bm1(487)
bm2(397)=bm1(488)
bm2(398)=bm1(489)
bm2(399)=bm1(490)
bm2(400)=bm1(491)
bm2(401)=bm1(492)
bm2(402)=bm1(493)
bm2(403)=bm1(494)
bm2(404)=bm1(495)
bm2(405)=bm1(496)
bm2(406)=bm1(497)
bm2(407)=bm1(498)
bm2(408)=bm1(499)
bm2(409)=bm1(500)
bm2(410)=bm1(501)
bm2(411)=bm1(502)
bm2(412)=bm1(503)
bm2(413)=bm1(504)
bm2(414)=bm1(505)
bm2(415)=bm1(507)
bm2(416)=bm1(508)
bm2(417)=bm1(509)
bm2(418)=bm1(510)
bm2(419)=bm1(511)
bm2(420)=bm1(512)
bm2(421)=bm1(513)
bm2(422)=bm1(514)
bm2(423)=bm1(515)
bm2(424)=bm1(516)
bm2(425)=bm1(518)
bm2(426)=bm1(520)
bm2(427)=bm1(521)
bm2(428)=bm1(522)
bm2(429)=bm1(523)
bm2(430)=bm1(524)
bm2(431)=bm1(525)
bm2(432)=bm1(526)
bm2(433)=bm1(527)
bm2(434)=bm1(529)
bm2(435)=bm1(530)
bm2(436)=bm1(532)
bm2(437)=bm1(534)
bm2(438)=bm1(535)
bm2(439)=bm1(536)
bm2(440)=bm1(537)
bm2(441)=bm1(538)
bm2(442)=bm1(539)
bm2(443)=bm1(540)
bm2(444)=bm1(541)
bm2(445)=bm1(543)
bm2(446)=bm1(544)
bm2(447)=bm1(546)
bm2(448)=bm1(547)
bm2(449)=bm1(548)
bm2(450)=bm1(550)
bm2(451)=bm1(551)
bm2(452)=bm1(552)
bm2(453)=bm1(553)
bm2(454)=bm1(554)
bm2(455)=bm1(555)
bm2(456)=bm1(557)
bm2(457)=bm1(558)
bm2(458)=bm1(560)
bm2(459)=bm1(561)
bm2(460)=bm1(562)
bm2(461)=bm1(563)
bm2(462)=bm1(564)
bm2(463)=bm1(566)
bm2(464)=bm1(567)
bm2(465)=bm1(568)
bm2(466)=bm1(569)
bm2(467)=bm1(570)
bm2(468)=bm1(591)
bm2(469)=bm1(592)
bm2(470)=bm1(593)
bm2(471)=bm1(594)
bm2(472)=bm1(595)
bm2(473)=bm1(596)
bm2(474)=bm1(597)
bm2(475)=bm1(598)
bm2(476)=bm1(599)
bm2(477)=bm1(600)
bm2(478)=bm1(601)
bm2(479)=bm1(602)
bm2(480)=bm1(603)
bm2(481)=bm1(604)
bm2(482)=bm1(605)
bm2(483)=bm1(606)
bm2(484)=bm1(607)
bm2(485)=bm1(608)
bm2(486)=bm1(609)
bm2(487)=bm1(610)
bm2(488)=bm1(611)
bm2(489)=bm1(612)
bm2(490)=bm1(613)
bm2(491)=bm1(614)
bm2(492)=bm1(615)
bm2(493)=bm1(616)
bm2(494)=bm1(617)
bm2(495)=bm1(618)
bm2(496)=bm1(619)
bm2(497)=bm1(620)
bm2(498)=bm1(621)
bm2(499)=bm1(622)
bm2(500)=bm1(623)
bm2(501)=bm1(624)
bm2(502)=bm1(625)
bm2(503)=bm1(626)
bm2(504)=bm1(627)
bm2(505)=bm1(628)
bm2(506)=bm1(629)
bm2(507)=bm1(630)
bm2(508)=bm1(631)
bm2(509)=bm1(632)
bm2(510)=bm1(633)
bm2(511)=bm1(634)
bm2(512)=bm1(635)
bm2(513)=bm1(636)
bm2(514)=bm1(637)
bm2(515)=bm1(638)
bm2(516)=bm1(639)
bm2(517)=bm1(640)
bm2(518)=bm1(641)
bm2(519)=bm1(642)
bm2(520)=bm1(643)
bm2(521)=bm1(644)
bm2(522)=bm1(645)
bm2(523)=bm1(646)
bm2(524)=bm1(647)
bm2(525)=bm1(648)
bm2(526)=bm1(649)
bm2(527)=bm1(650)
bm2(528)=bm1(651)
bm2(529)=bm1(652)
bm2(530)=bm1(653)
bm2(531)=bm1(654)
bm2(532)=bm1(655)
bm2(533)=bm1(656)
bm2(534)=bm1(657)
bm2(535)=bm1(658)
bm2(536)=bm1(659)
bm2(537)=bm1(660)
bm2(538)=bm1(661)
bm2(539)=bm1(662)
bm2(540)=bm1(663)
bm2(541)=bm1(664)
bm2(542)=bm1(665)
bm2(543)=bm1(666)
bm2(544)=bm1(667)
bm2(545)=bm1(668)
bm2(546)=bm1(669)
bm2(547)=bm1(670)
bm2(548)=bm1(671)
bm2(549)=bm1(672)
bm2(550)=bm1(673)
bm2(551)=bm1(674)
bm2(552)=bm1(675)
bm2(553)=bm1(676)
bm2(554)=bm1(677)
bm2(555)=bm1(678)
bm2(556)=bm1(679)
bm2(557)=bm1(680)
bm2(558)=bm1(681)
bm2(559)=bm1(682)
bm2(560)=bm1(683)
bm2(561)=bm1(684)
bm2(562)=bm1(685)
bm2(563)=bm1(686)
bm2(564)=bm1(687)
bm2(565)=bm1(688)
bm2(566)=bm1(689)
bm2(567)=bm1(690)
bm2(568)=bm1(691)
bm2(569)=bm1(692)
bm2(570)=bm1(693)
bm2(571)=bm1(694)
bm2(572)=bm1(695)
bm2(573)=bm1(696)
bm2(574)=bm1(697)
bm2(575)=bm1(698)
bm2(576)=bm1(699)
bm2(577)=bm1(700)
bm2(578)=bm1(701)
bm2(579)=bm1(702)
bm2(580)=bm1(703)
bm2(581)=bm1(704)
bm2(582)=bm1(705)
bm2(583)=bm1(706)
bm2(584)=bm1(707)
bm2(585)=bm1(708)
bm2(586)=bm1(709)
bm2(587)=bm1(710)
bm2(588)=bm1(711)
bm2(589)=bm1(712)
bm2(590)=bm1(713)
bm2(591)=bm1(714)
bm2(592)=bm1(715)
bm2(593)=bm1(716)
bm2(594)=bm1(717)
bm2(595)=bm1(718)
bm2(596)=bm1(719)
bm2(597)=bm1(720)
bm2(598)=bm1(721)
bm2(599)=bm1(722)
bm2(600)=bm1(723)
bm2(601)=bm1(724)
bm2(602)=bm1(725)
bm2(603)=bm1(726)
bm2(604)=bm1(727)
bm2(605)=bm1(728)
bm2(606)=bm1(729)
bm2(607)=bm1(730)
bm2(608)=bm1(731)
bm2(609)=bm1(732)
bm2(610)=bm1(733)
bm2(611)=bm1(734)
bm2(612)=bm1(735)
bm2(613)=bm1(736)
bm2(614)=bm1(737)
bm2(615)=bm1(738)
bm2(616)=bm1(739)
bm2(617)=bm1(740)
bm2(618)=bm1(741)
bm2(619)=bm1(742)
bm2(620)=bm1(743)
bm2(621)=bm1(744)
bm2(622)=bm1(745)
bm2(623)=bm1(746)
bm2(624)=bm1(747)
bm2(625)=bm1(748)
bm2(626)=bm1(749)
bm2(627)=bm1(750)
bm2(628)=bm1(751)
bm2(629)=bm1(752)
bm2(630)=bm1(753)
bm2(631)=bm1(754)
bm2(632)=bm1(755)
bm2(633)=bm1(756)
bm2(634)=bm1(757)
bm2(635)=bm1(758)
bm2(636)=bm1(759)
bm2(637)=bm1(760)
bm2(638)=bm1(761)
bm2(639)=bm1(762)
bm2(640)=bm1(763)
bm2(641)=bm1(764)
bm2(642)=bm1(765)
bm2(643)=bm1(766)
bm2(644)=bm1(767)
bm2(645)=bm1(768)
bm2(646)=bm1(769)
bm2(647)=bm1(770)
bm2(648)=bm1(771)
bm2(649)=bm1(772)
bm2(650)=bm1(773)
bm2(651)=bm1(774)
bm2(652)=bm1(775)
bm2(653)=bm1(776)
bm2(654)=bm1(777)
bm2(655)=bm1(778)
bm2(656)=bm1(779)
bm2(657)=bm1(780)
bm2(658)=bm1(781)
bm2(659)=bm1(782)
bm2(660)=bm1(783)
bm2(661)=bm1(784)
bm2(662)=bm1(785)
bm2(663)=bm1(786)
bm2(664)=bm1(787)
bm2(665)=bm1(788)
bm2(666)=bm1(789)
bm2(667)=bm1(790)
bm2(668)=bm1(791)
bm2(669)=bm1(792)
bm2(670)=bm1(793)
bm2(671)=bm1(794)
bm2(672)=bm1(795)
bm2(673)=bm1(796)
bm2(674)=bm1(797)
bm2(675)=bm1(798)
bm2(676)=bm1(799)
bm2(677)=bm1(800)
bm2(678)=bm1(801)
bm2(679)=bm1(802)
bm2(680)=bm1(803)
bm2(681)=bm1(804)
bm2(682)=bm1(805)
bm2(683)=bm1(806)
bm2(684)=bm1(807)
bm2(685)=bm1(808)
bm2(686)=bm1(809)
bm2(687)=bm1(810)
bm2(688)=bm1(811)
bm2(689)=bm1(812)
bm2(690)=bm1(813)
bm2(691)=bm1(814)
bm2(692)=bm1(815)
bm2(693)=bm1(816)
bm2(694)=bm1(817)
bm2(695)=bm1(818)
bm2(696)=bm1(819)
bm2(697)=bm1(820)
bm2(698)=bm1(821)
bm2(699)=bm1(822)
bm2(700)=bm1(823)
bm2(701)=bm1(824)
bm2(702)=bm1(825)
bm2(703)=bm1(826)
bm2(704)=bm1(827)
bm2(705)=bm1(828)
bm2(706)=bm1(829)
bm2(707)=bm1(830)
bm2(708)=bm1(831)
bm2(709)=bm1(832)
bm2(710)=bm1(833)
bm2(711)=bm1(834)
bm2(712)=bm1(835)
bm2(713)=bm1(836)
bm2(714)=bm1(837)
bm2(715)=bm1(838)
bm2(716)=bm1(839)
bm2(717)=bm1(840)
bm2(718)=bm1(841)
bm2(719)=bm1(842)
bm2(720)=bm1(843)
bm2(721)=bm1(844)
bm2(722)=bm1(845)
bm2(723)=bm1(846)
bm2(724)=bm1(847)
bm2(725)=bm1(848)
bm2(726)=bm1(849)
bm2(727)=bm1(850)
bm2(728)=bm1(851)
bm2(729)=bm1(853)
bm2(730)=bm1(854)
bm2(731)=bm1(855)
bm2(732)=bm1(856)
bm2(733)=bm1(857)
bm2(734)=bm1(858)
bm2(735)=bm1(859)
bm2(736)=bm1(860)
bm2(737)=bm1(861)
bm2(738)=bm1(862)
bm2(739)=bm1(863)
bm2(740)=bm1(865)
bm2(741)=bm1(867)
bm2(742)=bm1(868)
bm2(743)=bm1(869)
bm2(744)=bm1(870)
bm2(745)=bm1(871)
bm2(746)=bm1(872)
bm2(747)=bm1(873)
bm2(748)=bm1(874)
bm2(749)=bm1(875)
bm2(750)=bm1(876)
bm2(751)=bm1(878)
bm2(752)=bm1(879)
bm2(753)=bm1(881)
bm2(754)=bm1(883)
bm2(755)=bm1(884)
bm2(756)=bm1(885)
bm2(757)=bm1(886)
bm2(758)=bm1(887)
bm2(759)=bm1(888)
bm2(760)=bm1(889)
bm2(761)=bm1(890)
bm2(762)=bm1(891)
bm2(763)=bm1(893)
bm2(764)=bm1(894)
bm2(765)=bm1(896)
bm2(766)=bm1(897)
bm2(767)=bm1(899)
bm2(768)=bm1(900)
bm2(769)=bm1(902)
bm2(770)=bm1(903)
bm2(771)=bm1(904)
bm2(772)=bm1(905)
bm2(773)=bm1(906)
bm2(774)=bm1(907)
bm2(775)=bm1(908)
bm2(776)=bm1(909)
bm2(777)=bm1(911)
bm2(778)=bm1(912)
bm2(779)=bm1(914)
bm2(780)=bm1(915)
bm2(781)=bm1(916)
bm2(782)=bm1(918)
bm2(783)=bm1(919)
bm2(784)=bm1(920)
bm2(785)=bm1(921)
bm2(786)=bm1(922)
bm2(787)=bm1(923)
bm2(788)=bm1(925)
bm2(789)=bm1(926)
bm2(790)=bm1(928)
bm2(791)=bm1(929)
bm2(792)=bm1(930)
bm2(793)=bm1(931)
bm2(794)=bm1(932)
bm2(795)=bm1(934)
bm2(796)=bm1(935)
bm2(797)=bm1(936)
bm2(798)=bm1(937)
bm2(799)=bm1(938)
bm2(800)=bm1(961)
bm2(801)=bm1(962)
bm2(802)=bm1(963)
bm2(803)=bm1(964)
bm2(804)=bm1(965)
bm2(805)=bm1(966)
bm2(806)=bm1(967)
bm2(807)=bm1(968)
bm2(808)=bm1(969)
bm2(809)=bm1(970)
bm2(810)=bm1(971)
bm2(811)=bm1(972)
bm2(812)=bm1(973)
bm2(813)=bm1(974)
bm2(814)=bm1(975)
bm2(815)=bm1(976)
bm2(816)=bm1(977)
bm2(817)=bm1(978)
bm2(818)=bm1(979)
bm2(819)=bm1(980)
bm2(820)=bm1(981)
bm2(821)=bm1(982)
bm2(822)=bm1(983)
bm2(823)=bm1(984)
bm2(824)=bm1(985)
bm2(825)=bm1(986)
bm2(826)=bm1(987)
bm2(827)=bm1(988)
bm2(828)=bm1(989)
bm2(829)=bm1(990)
bm2(830)=bm1(991)
bm2(831)=bm1(992)
bm2(832)=bm1(993)
bm2(833)=bm1(994)
bm2(834)=bm1(995)
bm2(835)=bm1(996)
bm2(836)=bm1(997)
bm2(837)=bm1(998)
bm2(838)=bm1(999)
bm2(839)=bm1(1000)
bm2(840)=bm1(1001)
bm2(841)=bm1(1002)
bm2(842)=bm1(1003)
bm2(843)=bm1(1004)
bm2(844)=bm1(1005)
bm2(845)=bm1(1006)
bm2(846)=bm1(1007)
bm2(847)=bm1(1008)
bm2(848)=bm1(1009)
bm2(849)=bm1(1010)
bm2(850)=bm1(1011)
bm2(851)=bm1(1012)
bm2(852)=bm1(1013)
bm2(853)=bm1(1014)
bm2(854)=bm1(1015)
bm2(855)=bm1(1016)
bm2(856)=bm1(1017)
bm2(857)=bm1(1018)
bm2(858)=bm1(1019)
bm2(859)=bm1(1020)
bm2(860)=bm1(1021)
bm2(861)=bm1(1022)
bm2(862)=bm1(1023)
bm2(863)=bm1(1024)
bm2(864)=bm1(1025)
bm2(865)=bm1(1026)
bm2(866)=bm1(1027)
bm2(867)=bm1(1028)
bm2(868)=bm1(1029)
bm2(869)=bm1(1030)
bm2(870)=bm1(1031)
bm2(871)=bm1(1032)
bm2(872)=bm1(1033)
bm2(873)=bm1(1034)
bm2(874)=bm1(1035)
bm2(875)=bm1(1036)
bm2(876)=bm1(1037)
bm2(877)=bm1(1038)
bm2(878)=bm1(1039)
bm2(879)=bm1(1040)
bm2(880)=bm1(1041)
bm2(881)=bm1(1042)
bm2(882)=bm1(1043)
bm2(883)=bm1(1044)
bm2(884)=bm1(1045)
bm2(885)=bm1(1046)
bm2(886)=bm1(1047)
bm2(887)=bm1(1048)
bm2(888)=bm1(1049)
bm2(889)=bm1(1050)
bm2(890)=bm1(1051)
bm2(891)=bm1(1052)
bm2(892)=bm1(1053)
bm2(893)=bm1(1054)
bm2(894)=bm1(1055)
bm2(895)=bm1(1056)
bm2(896)=bm1(1057)
bm2(897)=bm1(1058)
bm2(898)=bm1(1059)
bm2(899)=bm1(1060)
bm2(900)=bm1(1061)
bm2(901)=bm1(1062)
bm2(902)=bm1(1063)
bm2(903)=bm1(1064)
bm2(904)=bm1(1065)
bm2(905)=bm1(1066)
bm2(906)=bm1(1067)
bm2(907)=bm1(1068)
bm2(908)=bm1(1069)
bm2(909)=bm1(1070)
bm2(910)=bm1(1071)
bm2(911)=bm1(1072)
bm2(912)=bm1(1073)
bm2(913)=bm1(1074)
bm2(914)=bm1(1075)
bm2(915)=bm1(1076)
bm2(916)=bm1(1077)
bm2(917)=bm1(1078)
bm2(918)=bm1(1079)
bm2(919)=bm1(1080)
bm2(920)=bm1(1081)
bm2(921)=bm1(1082)
bm2(922)=bm1(1083)
bm2(923)=bm1(1084)
bm2(924)=bm1(1085)
bm2(925)=bm1(1086)
bm2(926)=bm1(1087)
bm2(927)=bm1(1088)
bm2(928)=bm1(1089)
bm2(929)=bm1(1090)
bm2(930)=bm1(1091)
bm2(931)=bm1(1092)
bm2(932)=bm1(1093)
bm2(933)=bm1(1094)
bm2(934)=bm1(1095)
bm2(935)=bm1(1096)
bm2(936)=bm1(1097)
bm2(937)=bm1(1098)
bm2(938)=bm1(1099)
bm2(939)=bm1(1100)
bm2(940)=bm1(1101)
bm2(941)=bm1(1102)
bm2(942)=bm1(1103)
bm2(943)=bm1(1104)
bm2(944)=bm1(1105)
bm2(945)=bm1(1106)
bm2(946)=bm1(1107)
bm2(947)=bm1(1108)
bm2(948)=bm1(1109)
bm2(949)=bm1(1110)
bm2(950)=bm1(1111)
bm2(951)=bm1(1112)
bm2(952)=bm1(1113)
bm2(953)=bm1(1114)
bm2(954)=bm1(1115)
bm2(955)=bm1(1116)
bm2(956)=bm1(1117)
bm2(957)=bm1(1118)
bm2(958)=bm1(1119)
bm2(959)=bm1(1120)
bm2(960)=bm1(1121)
bm2(961)=bm1(1122)
bm2(962)=bm1(1123)
bm2(963)=bm1(1124)
bm2(964)=bm1(1125)
bm2(965)=bm1(1126)
bm2(966)=bm1(1127)
bm2(967)=bm1(1128)
bm2(968)=bm1(1129)
bm2(969)=bm1(1130)
bm2(970)=bm1(1131)
bm2(971)=bm1(1132)
bm2(972)=bm1(1133)
bm2(973)=bm1(1134)
bm2(974)=bm1(1135)
bm2(975)=bm1(1136)
bm2(976)=bm1(1137)
bm2(977)=bm1(1138)
bm2(978)=bm1(1139)
bm2(979)=bm1(1140)
bm2(980)=bm1(1141)
bm2(981)=bm1(1142)
bm2(982)=bm1(1143)
bm2(983)=bm1(1144)
bm2(984)=bm1(1145)
bm2(985)=bm1(1146)
bm2(986)=bm1(1147)
bm2(987)=bm1(1148)
bm2(988)=bm1(1149)
bm2(989)=bm1(1150)
bm2(990)=bm1(1151)
bm2(991)=bm1(1152)
bm2(992)=bm1(1153)
bm2(993)=bm1(1154)
bm2(994)=bm1(1155)
bm2(995)=bm1(1156)
bm2(996)=bm1(1157)
bm2(997)=bm1(1158)
bm2(998)=bm1(1159)
bm2(999)=bm1(1160)
bm2(1000)=bm1(1161)
bm2(1001)=bm1(1162)
bm2(1002)=bm1(1163)
bm2(1003)=bm1(1164)
bm2(1004)=bm1(1165)
bm2(1005)=bm1(1166)
bm2(1006)=bm1(1167)
bm2(1007)=bm1(1168)
bm2(1008)=bm1(1169)
bm2(1009)=bm1(1170)
bm2(1010)=bm1(1171)
bm2(1011)=bm1(1172)
bm2(1012)=bm1(1173)
bm2(1013)=bm1(1174)
bm2(1014)=bm1(1175)
bm2(1015)=bm1(1176)
bm2(1016)=bm1(1177)
bm2(1017)=bm1(1178)
bm2(1018)=bm1(1179)
bm2(1019)=bm1(1180)
bm2(1020)=bm1(1181)
bm2(1021)=bm1(1182)
bm2(1022)=bm1(1183)
bm2(1023)=bm1(1184)
bm2(1024)=bm1(1185)
bm2(1025)=bm1(1186)
bm2(1026)=bm1(1187)
bm2(1027)=bm1(1188)
bm2(1028)=bm1(1189)
bm2(1029)=bm1(1190)
bm2(1030)=bm1(1191)
bm2(1031)=bm1(1192)
bm2(1032)=bm1(1193)
bm2(1033)=bm1(1194)
bm2(1034)=bm1(1195)
bm2(1035)=bm1(1196)
bm2(1036)=bm1(1197)
bm2(1037)=bm1(1198)
bm2(1038)=bm1(1199)
bm2(1039)=bm1(1200)
bm2(1040)=bm1(1201)
bm2(1041)=bm1(1202)
bm2(1042)=bm1(1203)
bm2(1043)=bm1(1204)
bm2(1044)=bm1(1205)
bm2(1045)=bm1(1206)
bm2(1046)=bm1(1207)
bm2(1047)=bm1(1208)
bm2(1048)=bm1(1209)
bm2(1049)=bm1(1210)
bm2(1050)=bm1(1211)
bm2(1051)=bm1(1212)
bm2(1052)=bm1(1213)
bm2(1053)=bm1(1214)
bm2(1054)=bm1(1215)
bm2(1055)=bm1(1216)
bm2(1056)=bm1(1217)
bm2(1057)=bm1(1218)
bm2(1058)=bm1(1219)
bm2(1059)=bm1(1220)
bm2(1060)=bm1(1221)
bm2(1061)=bm1(1222)
bm2(1062)=bm1(1223)
bm2(1063)=bm1(1224)
bm2(1064)=bm1(1225)
bm2(1065)=bm1(1226)
bm2(1066)=bm1(1227)
bm2(1067)=bm1(1228)
bm2(1068)=bm1(1229)
bm2(1069)=bm1(1230)
bm2(1070)=bm1(1231)
bm2(1071)=bm1(1232)
bm2(1072)=bm1(1233)
bm2(1073)=bm1(1234)
bm2(1074)=bm1(1235)
bm2(1075)=bm1(1236)
bm2(1076)=bm1(1237)
bm2(1077)=bm1(1238)
bm2(1078)=bm1(1239)
bm2(1079)=bm1(1240)
bm2(1080)=bm1(1241)
bm2(1081)=bm1(1242)
bm2(1082)=bm1(1243)
bm2(1083)=bm1(1244)
bm2(1084)=bm1(1245)
bm2(1085)=bm1(1246)
bm2(1086)=bm1(1247)
bm2(1087)=bm1(1248)
bm2(1088)=bm1(1249)
bm2(1089)=bm1(1250)
bm2(1090)=bm1(1251)
bm2(1091)=bm1(1252)
bm2(1092)=bm1(1253)
bm2(1093)=bm1(1254)
bm2(1094)=bm1(1255)
bm2(1095)=bm1(1256)
bm2(1096)=bm1(1257)
bm2(1097)=bm1(1258)
bm2(1098)=bm1(1259)
bm2(1099)=bm1(1260)
bm2(1100)=bm1(1261)
bm2(1101)=bm1(1262)
bm2(1102)=bm1(1263)
bm2(1103)=bm1(1264)
bm2(1104)=bm1(1265)
bm2(1105)=bm1(1266)
bm2(1106)=bm1(1267)
bm2(1107)=bm1(1268)
bm2(1108)=bm1(1269)
bm2(1109)=bm1(1270)
bm2(1110)=bm1(1271)
bm2(1111)=bm1(1272)
bm2(1112)=bm1(1273)
bm2(1113)=bm1(1274)
bm2(1114)=bm1(1275)
bm2(1115)=bm1(1276)
bm2(1116)=bm1(1277)
bm2(1117)=bm1(1278)
bm2(1118)=bm1(1279)
bm2(1119)=bm1(1280)
bm2(1120)=bm1(1281)
bm2(1121)=bm1(1282)
bm2(1122)=bm1(1283)
bm2(1123)=bm1(1284)
bm2(1124)=bm1(1285)
bm2(1125)=bm1(1286)
bm2(1126)=bm1(1287)
bm2(1127)=bm1(1288)
bm2(1128)=bm1(1289)
bm2(1129)=bm1(1290)
bm2(1130)=bm1(1291)
bm2(1131)=bm1(1292)
bm2(1132)=bm1(1293)
bm2(1133)=bm1(1294)
bm2(1134)=bm1(1295)
bm2(1135)=bm1(1296)
bm2(1136)=bm1(1297)
bm2(1137)=bm1(1298)
bm2(1138)=bm1(1299)
bm2(1139)=bm1(1300)
bm2(1140)=bm1(1301)
bm2(1141)=bm1(1302)
bm2(1142)=bm1(1303)
bm2(1143)=bm1(1304)
bm2(1144)=bm1(1305)
bm2(1145)=bm1(1306)
bm2(1146)=bm1(1307)
bm2(1147)=bm1(1308)
bm2(1148)=bm1(1309)
bm2(1149)=bm1(1310)
bm2(1150)=bm1(1311)
bm2(1151)=bm1(1312)
bm2(1152)=bm1(1313)
bm2(1153)=bm1(1314)
bm2(1154)=bm1(1315)
bm2(1155)=bm1(1316)
bm2(1156)=bm1(1317)
bm2(1157)=bm1(1318)
bm2(1158)=bm1(1319)
bm2(1159)=bm1(1320)
bm2(1160)=bm1(1321)
bm2(1161)=bm1(1322)
bm2(1162)=bm1(1323)
bm2(1163)=bm1(1324)
bm2(1164)=bm1(1325)
bm2(1165)=bm1(1326)
bm2(1166)=bm1(1327)
bm2(1167)=bm1(1328)
bm2(1168)=bm1(1329)
bm2(1169)=bm1(1330)
bm2(1170)=bm1(1331)
bm2(1171)=bm1(1332)
bm2(1172)=bm1(1333)
bm2(1173)=bm1(1334)
bm2(1174)=bm1(1335)
bm2(1175)=bm1(1336)
bm2(1176)=bm1(1337)
bm2(1177)=bm1(1338)
bm2(1178)=bm1(1339)
bm2(1179)=bm1(1340)
bm2(1180)=bm1(1341)
bm2(1181)=bm1(1342)
bm2(1182)=bm1(1343)
bm2(1183)=bm1(1344)
bm2(1184)=bm1(1345)
bm2(1185)=bm1(1346)
bm2(1186)=bm1(1347)
bm2(1187)=bm1(1348)
bm2(1188)=bm1(1349)
bm2(1189)=bm1(1350)
bm2(1190)=bm1(1351)
bm2(1191)=bm1(1352)
bm2(1192)=bm1(1353)
bm2(1193)=bm1(1354)
bm2(1194)=bm1(1355)
bm2(1195)=bm1(1356)
bm2(1196)=bm1(1357)
bm2(1197)=bm1(1358)
bm2(1198)=bm1(1359)
bm2(1199)=bm1(1360)
bm2(1200)=bm1(1361)
bm2(1201)=bm1(1362)
bm2(1202)=bm1(1363)
bm2(1203)=bm1(1364)
bm2(1204)=bm1(1365)
bm2(1205)=bm1(1366)
bm2(1206)=bm1(1367)
bm2(1207)=bm1(1368)
bm2(1208)=bm1(1369)
bm2(1209)=bm1(1371)
bm2(1210)=bm1(1372)
bm2(1211)=bm1(1373)
bm2(1212)=bm1(1374)
bm2(1213)=bm1(1375)
bm2(1214)=bm1(1376)
bm2(1215)=bm1(1377)
bm2(1216)=bm1(1378)
bm2(1217)=bm1(1379)
bm2(1218)=bm1(1380)
bm2(1219)=bm1(1381)
bm2(1220)=bm1(1382)
bm2(1221)=bm1(1383)
bm2(1222)=bm1(1385)
bm2(1223)=bm1(1387)
bm2(1224)=bm1(1388)
bm2(1225)=bm1(1389)
bm2(1226)=bm1(1390)
bm2(1227)=bm1(1391)
bm2(1228)=bm1(1392)
bm2(1229)=bm1(1393)
bm2(1230)=bm1(1394)
bm2(1231)=bm1(1395)
bm2(1232)=bm1(1396)
bm2(1233)=bm1(1397)
bm2(1234)=bm1(1399)
bm2(1235)=bm1(1400)
bm2(1236)=bm1(1402)
bm2(1237)=bm1(1404)
bm2(1238)=bm1(1405)
bm2(1239)=bm1(1406)
bm2(1240)=bm1(1407)
bm2(1241)=bm1(1408)
bm2(1242)=bm1(1409)
bm2(1243)=bm1(1410)
bm2(1244)=bm1(1411)
bm2(1245)=bm1(1412)
bm2(1246)=bm1(1413)
bm2(1247)=bm1(1415)
bm2(1248)=bm1(1416)
bm2(1249)=bm1(1418)
bm2(1250)=bm1(1419)
bm2(1251)=bm1(1421)
bm2(1252)=bm1(1422)
bm2(1253)=bm1(1424)
bm2(1254)=bm1(1425)
bm2(1255)=bm1(1426)
bm2(1256)=bm1(1427)
bm2(1257)=bm1(1428)
bm2(1258)=bm1(1429)
bm2(1259)=bm1(1430)
bm2(1260)=bm1(1431)
bm2(1261)=bm1(1432)
bm2(1262)=bm1(1434)
bm2(1263)=bm1(1435)
bm2(1264)=bm1(1437)
bm2(1265)=bm1(1438)
bm2(1266)=bm1(1440)
bm2(1267)=bm1(1441)
bm2(1268)=bm1(1442)
bm2(1269)=bm1(1444)
bm2(1270)=bm1(1445)
bm2(1271)=bm1(1446)
bm2(1272)=bm1(1447)
bm2(1273)=bm1(1448)
bm2(1274)=bm1(1449)
bm2(1275)=bm1(1450)
bm2(1276)=bm1(1451)
bm2(1277)=bm1(1453)
bm2(1278)=bm1(1454)
bm2(1279)=bm1(1456)
bm2(1280)=bm1(1457)
bm2(1281)=bm1(1458)
bm2(1282)=bm1(1460)
bm2(1283)=bm1(1461)
bm2(1284)=bm1(1462)
bm2(1285)=bm1(1463)
bm2(1286)=bm1(1464)
bm2(1287)=bm1(1465)
bm2(1288)=bm1(1467)
bm2(1289)=bm1(1468)
bm2(1290)=bm1(1470)
bm2(1291)=bm1(1471)
bm2(1292)=bm1(1472)
bm2(1293)=bm1(1473)
bm2(1294)=bm1(1474)
bm2(1295)=bm1(1476)
bm2(1296)=bm1(1477)
bm2(1297)=bm1(1478)
bm2(1298)=bm1(1479)
bm2(1299)=bm1(1480)
bm2(1300)=bm1(1481)
bm2(1301)=bm1(1507)
bm2(1302)=bm1(1508)
bm2(1303)=bm1(1509)
bm2(1304)=bm1(1510)
bm2(1305)=bm1(1511)
bm2(1306)=bm1(1512)
bm2(1307)=bm1(1513)
bm2(1308)=bm1(1514)
bm2(1309)=bm1(1515)
bm2(1310)=bm1(1516)
bm2(1311)=bm1(1517)
bm2(1312)=bm1(1518)
bm2(1313)=bm1(1519)
bm2(1314)=bm1(1520)
bm2(1315)=bm1(1521)
bm2(1316)=bm1(1522)
bm2(1317)=bm1(1523)
bm2(1318)=bm1(1524)
bm2(1319)=bm1(1525)
bm2(1320)=bm1(1526)
bm2(1321)=bm1(1527)
bm2(1322)=bm1(1528)
bm2(1323)=bm1(1529)
bm2(1324)=bm1(1530)
bm2(1325)=bm1(1531)
bm2(1326)=bm1(1532)
bm2(1327)=bm1(1533)
bm2(1328)=bm1(1534)
bm2(1329)=bm1(1535)
bm2(1330)=bm1(1536)
bm2(1331)=bm1(1537)
bm2(1332)=bm1(1538)
bm2(1333)=bm1(1539)
bm2(1334)=bm1(1540)
bm2(1335)=bm1(1541)
bm2(1336)=bm1(1542)
bm2(1337)=bm1(1543)
bm2(1338)=bm1(1544)
bm2(1339)=bm1(1545)
bm2(1340)=bm1(1546)
bm2(1341)=bm1(1547)
bm2(1342)=bm1(1548)
bm2(1343)=bm1(1549)
bm2(1344)=bm1(1550)
bm2(1345)=bm1(1551)
bm2(1346)=bm1(1552)
bm2(1347)=bm1(1553)
bm2(1348)=bm1(1554)
bm2(1349)=bm1(1555)
bm2(1350)=bm1(1556)
bm2(1351)=bm1(1557)
bm2(1352)=bm1(1558)
bm2(1353)=bm1(1559)
bm2(1354)=bm1(1560)
bm2(1355)=bm1(1561)
bm2(1356)=bm1(1562)
bm2(1357)=bm1(1563)
bm2(1358)=bm1(1564)
bm2(1359)=bm1(1565)
bm2(1360)=bm1(1566)
bm2(1361)=bm1(1567)
bm2(1362)=bm1(1568)
bm2(1363)=bm1(1569)
bm2(1364)=bm1(1570)
bm2(1365)=bm1(1571)
bm2(1366)=bm1(1572)
bm2(1367)=bm1(1573)
bm2(1368)=bm1(1574)
bm2(1369)=bm1(1575)
bm2(1370)=bm1(1576)
bm2(1371)=bm1(1577)
bm2(1372)=bm1(1578)
bm2(1373)=bm1(1579)
bm2(1374)=bm1(1580)
bm2(1375)=bm1(1581)
bm2(1376)=bm1(1582)
bm2(1377)=bm1(1583)
bm2(1378)=bm1(1584)
bm2(1379)=bm1(1585)
bm2(1380)=bm1(1586)
bm2(1381)=bm1(1587)
bm2(1382)=bm1(1588)
bm2(1383)=bm1(1589)
bm2(1384)=bm1(1590)
bm2(1385)=bm1(1591)
bm2(1386)=bm1(1592)
bm2(1387)=bm1(1593)
bm2(1388)=bm1(1594)
bm2(1389)=bm1(1595)
bm2(1390)=bm1(1596)
bm2(1391)=bm1(1597)
bm2(1392)=bm1(1598)
bm2(1393)=bm1(1599)
bm2(1394)=bm1(1600)
bm2(1395)=bm1(1601)
bm2(1396)=bm1(1602)
bm2(1397)=bm1(1603)
bm2(1398)=bm1(1604)
bm2(1399)=bm1(1605)
bm2(1400)=bm1(1606)
bm2(1401)=bm1(1607)
bm2(1402)=bm1(1608)
bm2(1403)=bm1(1609)
bm2(1404)=bm1(1610)
bm2(1405)=bm1(1611)
bm2(1406)=bm1(1612)
bm2(1407)=bm1(1613)
bm2(1408)=bm1(1614)
bm2(1409)=bm1(1615)
bm2(1410)=bm1(1616)
bm2(1411)=bm1(1617)
bm2(1412)=bm1(1618)
bm2(1413)=bm1(1619)
bm2(1414)=bm1(1620)
bm2(1415)=bm1(1621)
bm2(1416)=bm1(1622)
bm2(1417)=bm1(1623)
bm2(1418)=bm1(1624)
bm2(1419)=bm1(1625)
bm2(1420)=bm1(1626)
bm2(1421)=bm1(1627)
bm2(1422)=bm1(1628)
bm2(1423)=bm1(1629)
bm2(1424)=bm1(1630)
bm2(1425)=bm1(1631)
bm2(1426)=bm1(1632)
bm2(1427)=bm1(1633)
bm2(1428)=bm1(1634)
bm2(1429)=bm1(1635)
bm2(1430)=bm1(1636)
bm2(1431)=bm1(1637)
bm2(1432)=bm1(1638)
bm2(1433)=bm1(1639)
bm2(1434)=bm1(1640)
bm2(1435)=bm1(1641)
bm2(1436)=bm1(1642)
bm2(1437)=bm1(1643)
bm2(1438)=bm1(1644)
bm2(1439)=bm1(1645)
bm2(1440)=bm1(1646)
bm2(1441)=bm1(1647)
bm2(1442)=bm1(1648)
bm2(1443)=bm1(1649)
bm2(1444)=bm1(1650)
bm2(1445)=bm1(1651)
bm2(1446)=bm1(1652)
bm2(1447)=bm1(1653)
bm2(1448)=bm1(1654)
bm2(1449)=bm1(1655)
bm2(1450)=bm1(1656)
bm2(1451)=bm1(1657)
bm2(1452)=bm1(1658)
bm2(1453)=bm1(1659)
bm2(1454)=bm1(1660)
bm2(1455)=bm1(1661)
bm2(1456)=bm1(1662)
bm2(1457)=bm1(1663)
bm2(1458)=bm1(1664)
bm2(1459)=bm1(1665)
bm2(1460)=bm1(1666)
bm2(1461)=bm1(1667)
bm2(1462)=bm1(1668)
bm2(1463)=bm1(1669)
bm2(1464)=bm1(1670)
bm2(1465)=bm1(1671)
bm2(1466)=bm1(1672)
bm2(1467)=bm1(1673)
bm2(1468)=bm1(1674)
bm2(1469)=bm1(1675)
bm2(1470)=bm1(1676)
bm2(1471)=bm1(1677)
bm2(1472)=bm1(1678)
bm2(1473)=bm1(1679)
bm2(1474)=bm1(1680)
bm2(1475)=bm1(1681)
bm2(1476)=bm1(1682)
bm2(1477)=bm1(1683)
bm2(1478)=bm1(1684)
bm2(1479)=bm1(1685)
bm2(1480)=bm1(1686)
bm2(1481)=bm1(1687)
bm2(1482)=bm1(1688)
bm2(1483)=bm1(1689)
bm2(1484)=bm1(1690)
bm2(1485)=bm1(1691)
bm2(1486)=bm1(1692)
bm2(1487)=bm1(1693)
bm2(1488)=bm1(1694)
bm2(1489)=bm1(1695)
bm2(1490)=bm1(1696)
bm2(1491)=bm1(1697)
bm2(1492)=bm1(1698)
bm2(1493)=bm1(1699)
bm2(1494)=bm1(1700)
bm2(1495)=bm1(1701)
bm2(1496)=bm1(1702)
bm2(1497)=bm1(1703)
bm2(1498)=bm1(1704)
bm2(1499)=bm1(1705)
bm2(1500)=bm1(1706)
bm2(1501)=bm1(1707)
bm2(1502)=bm1(1708)
bm2(1503)=bm1(1709)
bm2(1504)=bm1(1710)
bm2(1505)=bm1(1711)
bm2(1506)=bm1(1712)
bm2(1507)=bm1(1713)
bm2(1508)=bm1(1714)
bm2(1509)=bm1(1715)
bm2(1510)=bm1(1716)
bm2(1511)=bm1(1717)
bm2(1512)=bm1(1718)
bm2(1513)=bm1(1719)
bm2(1514)=bm1(1720)
bm2(1515)=bm1(1721)
bm2(1516)=bm1(1722)
bm2(1517)=bm1(1723)
bm2(1518)=bm1(1724)
bm2(1519)=bm1(1725)
bm2(1520)=bm1(1726)
bm2(1521)=bm1(1727)
bm2(1522)=bm1(1728)
bm2(1523)=bm1(1729)
bm2(1524)=bm1(1730)
bm2(1525)=bm1(1731)
bm2(1526)=bm1(1732)
bm2(1527)=bm1(1733)
bm2(1528)=bm1(1734)
bm2(1529)=bm1(1735)
bm2(1530)=bm1(1736)
bm2(1531)=bm1(1737)
bm2(1532)=bm1(1738)
bm2(1533)=bm1(1739)
bm2(1534)=bm1(1740)
bm2(1535)=bm1(1741)
bm2(1536)=bm1(1742)
bm2(1537)=bm1(1743)
bm2(1538)=bm1(1744)
bm2(1539)=bm1(1745)
bm2(1540)=bm1(1746)
bm2(1541)=bm1(1747)
bm2(1542)=bm1(1748)
bm2(1543)=bm1(1749)
bm2(1544)=bm1(1750)
bm2(1545)=bm1(1751)
bm2(1546)=bm1(1752)
bm2(1547)=bm1(1753)
bm2(1548)=bm1(1754)
bm2(1549)=bm1(1755)
bm2(1550)=bm1(1756)
bm2(1551)=bm1(1757)
bm2(1552)=bm1(1758)
bm2(1553)=bm1(1759)
bm2(1554)=bm1(1760)
bm2(1555)=bm1(1761)
bm2(1556)=bm1(1762)
bm2(1557)=bm1(1763)
bm2(1558)=bm1(1764)
bm2(1559)=bm1(1765)
bm2(1560)=bm1(1766)
bm2(1561)=bm1(1767)
bm2(1562)=bm1(1768)
bm2(1563)=bm1(1769)
bm2(1564)=bm1(1770)
bm2(1565)=bm1(1771)
bm2(1566)=bm1(1772)
bm2(1567)=bm1(1773)
bm2(1568)=bm1(1774)
bm2(1569)=bm1(1775)
bm2(1570)=bm1(1776)
bm2(1571)=bm1(1777)
bm2(1572)=bm1(1778)
bm2(1573)=bm1(1779)
bm2(1574)=bm1(1780)
bm2(1575)=bm1(1781)
bm2(1576)=bm1(1782)
bm2(1577)=bm1(1783)
bm2(1578)=bm1(1784)
bm2(1579)=bm1(1785)
bm2(1580)=bm1(1786)
bm2(1581)=bm1(1787)
bm2(1582)=bm1(1788)
bm2(1583)=bm1(1789)
bm2(1584)=bm1(1790)
bm2(1585)=bm1(1791)
bm2(1586)=bm1(1792)
bm2(1587)=bm1(1793)
bm2(1588)=bm1(1794)
bm2(1589)=bm1(1795)
bm2(1590)=bm1(1796)
bm2(1591)=bm1(1797)
bm2(1592)=bm1(1798)
bm2(1593)=bm1(1799)
bm2(1594)=bm1(1800)
bm2(1595)=bm1(1801)
bm2(1596)=bm1(1802)
bm2(1597)=bm1(1803)
bm2(1598)=bm1(1804)
bm2(1599)=bm1(1805)
bm2(1600)=bm1(1806)
bm2(1601)=bm1(1807)
bm2(1602)=bm1(1808)
bm2(1603)=bm1(1809)
bm2(1604)=bm1(1810)
bm2(1605)=bm1(1811)
bm2(1606)=bm1(1812)
bm2(1607)=bm1(1813)
bm2(1608)=bm1(1814)
bm2(1609)=bm1(1815)
bm2(1610)=bm1(1816)
bm2(1611)=bm1(1817)
bm2(1612)=bm1(1818)
bm2(1613)=bm1(1819)
bm2(1614)=bm1(1820)
bm2(1615)=bm1(1821)
bm2(1616)=bm1(1822)
bm2(1617)=bm1(1823)
bm2(1618)=bm1(1824)
bm2(1619)=bm1(1825)
bm2(1620)=bm1(1826)
bm2(1621)=bm1(1827)
bm2(1622)=bm1(1828)
bm2(1623)=bm1(1829)
bm2(1624)=bm1(1830)
bm2(1625)=bm1(1831)
bm2(1626)=bm1(1832)
bm2(1627)=bm1(1833)
bm2(1628)=bm1(1834)
bm2(1629)=bm1(1835)
bm2(1630)=bm1(1836)
bm2(1631)=bm1(1837)
bm2(1632)=bm1(1838)
bm2(1633)=bm1(1839)
bm2(1634)=bm1(1840)
bm2(1635)=bm1(1841)
bm2(1636)=bm1(1842)
bm2(1637)=bm1(1843)
bm2(1638)=bm1(1844)
bm2(1639)=bm1(1845)
bm2(1640)=bm1(1846)
bm2(1641)=bm1(1847)
bm2(1642)=bm1(1848)
bm2(1643)=bm1(1849)
bm2(1644)=bm1(1850)
bm2(1645)=bm1(1851)
bm2(1646)=bm1(1852)
bm2(1647)=bm1(1853)
bm2(1648)=bm1(1854)
bm2(1649)=bm1(1855)
bm2(1650)=bm1(1856)
bm2(1651)=bm1(1857)
bm2(1652)=bm1(1858)
bm2(1653)=bm1(1859)
bm2(1654)=bm1(1860)
bm2(1655)=bm1(1861)
bm2(1656)=bm1(1862)
bm2(1657)=bm1(1863)
bm2(1658)=bm1(1864)
bm2(1659)=bm1(1865)
bm2(1660)=bm1(1866)
bm2(1661)=bm1(1867)
bm2(1662)=bm1(1868)
bm2(1663)=bm1(1869)
bm2(1664)=bm1(1870)
bm2(1665)=bm1(1871)
bm2(1666)=bm1(1872)
bm2(1667)=bm1(1873)
bm2(1668)=bm1(1874)
bm2(1669)=bm1(1875)
bm2(1670)=bm1(1876)
bm2(1671)=bm1(1877)
bm2(1672)=bm1(1878)
bm2(1673)=bm1(1879)
bm2(1674)=bm1(1880)
bm2(1675)=bm1(1881)
bm2(1676)=bm1(1882)
bm2(1677)=bm1(1883)
bm2(1678)=bm1(1884)
bm2(1679)=bm1(1885)
bm2(1680)=bm1(1886)
bm2(1681)=bm1(1887)
bm2(1682)=bm1(1888)
bm2(1683)=bm1(1889)
bm2(1684)=bm1(1890)
bm2(1685)=bm1(1891)
bm2(1686)=bm1(1892)
bm2(1687)=bm1(1893)
bm2(1688)=bm1(1894)
bm2(1689)=bm1(1895)
bm2(1690)=bm1(1896)
bm2(1691)=bm1(1897)
bm2(1692)=bm1(1898)
bm2(1693)=bm1(1899)
bm2(1694)=bm1(1900)
bm2(1695)=bm1(1901)
bm2(1696)=bm1(1902)
bm2(1697)=bm1(1903)
bm2(1698)=bm1(1904)
bm2(1699)=bm1(1905)
bm2(1700)=bm1(1906)
bm2(1701)=bm1(1907)
bm2(1702)=bm1(1908)
bm2(1703)=bm1(1909)
bm2(1704)=bm1(1910)
bm2(1705)=bm1(1911)
bm2(1706)=bm1(1912)
bm2(1707)=bm1(1913)
bm2(1708)=bm1(1914)
bm2(1709)=bm1(1915)
bm2(1710)=bm1(1916)
bm2(1711)=bm1(1917)
bm2(1712)=bm1(1918)
bm2(1713)=bm1(1919)
bm2(1714)=bm1(1920)
bm2(1715)=bm1(1921)
bm2(1716)=bm1(1922)
bm2(1717)=bm1(1923)
bm2(1718)=bm1(1924)
bm2(1719)=bm1(1925)
bm2(1720)=bm1(1926)
bm2(1721)=bm1(1927)
bm2(1722)=bm1(1928)
bm2(1723)=bm1(1929)
bm2(1724)=bm1(1930)
bm2(1725)=bm1(1931)
bm2(1726)=bm1(1932)
bm2(1727)=bm1(1933)
bm2(1728)=bm1(1934)
bm2(1729)=bm1(1935)
bm2(1730)=bm1(1936)
bm2(1731)=bm1(1937)
bm2(1732)=bm1(1938)
bm2(1733)=bm1(1939)
bm2(1734)=bm1(1940)
bm2(1735)=bm1(1941)
bm2(1736)=bm1(1942)
bm2(1737)=bm1(1943)
bm2(1738)=bm1(1944)
bm2(1739)=bm1(1945)
bm2(1740)=bm1(1946)
bm2(1741)=bm1(1947)
bm2(1742)=bm1(1948)
bm2(1743)=bm1(1949)
bm2(1744)=bm1(1950)
bm2(1745)=bm1(1951)
bm2(1746)=bm1(1952)
bm2(1747)=bm1(1953)
bm2(1748)=bm1(1954)
bm2(1749)=bm1(1955)
bm2(1750)=bm1(1956)
bm2(1751)=bm1(1957)
bm2(1752)=bm1(1958)
bm2(1753)=bm1(1959)
bm2(1754)=bm1(1960)
bm2(1755)=bm1(1961)
bm2(1756)=bm1(1962)
bm2(1757)=bm1(1963)
bm2(1758)=bm1(1964)
bm2(1759)=bm1(1965)
bm2(1760)=bm1(1966)
bm2(1761)=bm1(1967)
bm2(1762)=bm1(1968)
bm2(1763)=bm1(1969)
bm2(1764)=bm1(1970)
bm2(1765)=bm1(1971)
bm2(1766)=bm1(1972)
bm2(1767)=bm1(1973)
bm2(1768)=bm1(1974)
bm2(1769)=bm1(1975)
bm2(1770)=bm1(1976)
bm2(1771)=bm1(1977)
bm2(1772)=bm1(1978)
bm2(1773)=bm1(1979)
bm2(1774)=bm1(1980)
bm2(1775)=bm1(1981)
bm2(1776)=bm1(1982)
bm2(1777)=bm1(1983)
bm2(1778)=bm1(1984)
bm2(1779)=bm1(1985)
bm2(1780)=bm1(1986)
bm2(1781)=bm1(1987)
bm2(1782)=bm1(1988)
bm2(1783)=bm1(1989)
bm2(1784)=bm1(1990)
bm2(1785)=bm1(1991)
bm2(1786)=bm1(1992)
bm2(1787)=bm1(1993)
bm2(1788)=bm1(1994)
bm2(1789)=bm1(1995)
bm2(1790)=bm1(1996)
bm2(1791)=bm1(1997)
bm2(1792)=bm1(1998)
bm2(1793)=bm1(1999)
bm2(1794)=bm1(2000)
bm2(1795)=bm1(2001)
bm2(1796)=bm1(2002)
bm2(1797)=bm1(2003)
bm2(1798)=bm1(2004)
bm2(1799)=bm1(2005)
bm2(1800)=bm1(2006)
bm2(1801)=bm1(2007)
bm2(1802)=bm1(2008)
bm2(1803)=bm1(2009)
bm2(1804)=bm1(2010)
bm2(1805)=bm1(2011)
bm2(1806)=bm1(2012)
bm2(1807)=bm1(2013)
bm2(1808)=bm1(2014)
bm2(1809)=bm1(2015)
bm2(1810)=bm1(2016)
bm2(1811)=bm1(2017)
bm2(1812)=bm1(2018)
bm2(1813)=bm1(2019)
bm2(1814)=bm1(2020)
bm2(1815)=bm1(2021)
bm2(1816)=bm1(2022)
bm2(1817)=bm1(2023)
bm2(1818)=bm1(2024)
bm2(1819)=bm1(2025)
bm2(1820)=bm1(2026)
bm2(1821)=bm1(2027)
bm2(1822)=bm1(2028)
bm2(1823)=bm1(2029)
bm2(1824)=bm1(2030)
bm2(1825)=bm1(2031)
bm2(1826)=bm1(2032)
bm2(1827)=bm1(2033)
bm2(1828)=bm1(2034)
bm2(1829)=bm1(2035)
bm2(1830)=bm1(2036)
bm2(1831)=bm1(2037)
bm2(1832)=bm1(2038)
bm2(1833)=bm1(2039)
bm2(1834)=bm1(2040)
bm2(1835)=bm1(2041)
bm2(1836)=bm1(2042)
bm2(1837)=bm1(2043)
bm2(1838)=bm1(2044)
bm2(1839)=bm1(2045)
bm2(1840)=bm1(2046)
bm2(1841)=bm1(2047)
bm2(1842)=bm1(2048)
bm2(1843)=bm1(2049)
bm2(1844)=bm1(2050)
bm2(1845)=bm1(2051)
bm2(1846)=bm1(2052)
bm2(1847)=bm1(2053)
bm2(1848)=bm1(2054)
bm2(1849)=bm1(2055)
bm2(1850)=bm1(2056)
bm2(1851)=bm1(2057)
bm2(1852)=bm1(2058)
bm2(1853)=bm1(2059)
bm2(1854)=bm1(2060)
bm2(1855)=bm1(2061)
bm2(1856)=bm1(2062)
bm2(1857)=bm1(2063)
bm2(1858)=bm1(2064)
bm2(1859)=bm1(2065)
bm2(1860)=bm1(2066)
bm2(1861)=bm1(2067)
bm2(1862)=bm1(2068)
bm2(1863)=bm1(2069)
bm2(1864)=bm1(2070)
bm2(1865)=bm1(2071)
bm2(1866)=bm1(2072)
bm2(1867)=bm1(2073)
bm2(1868)=bm1(2074)
bm2(1869)=bm1(2075)
bm2(1870)=bm1(2076)
bm2(1871)=bm1(2077)
bm2(1872)=bm1(2078)
bm2(1873)=bm1(2079)
bm2(1874)=bm1(2080)
bm2(1875)=bm1(2081)
bm2(1876)=bm1(2082)
bm2(1877)=bm1(2083)
bm2(1878)=bm1(2084)
bm2(1879)=bm1(2085)
bm2(1880)=bm1(2086)
bm2(1881)=bm1(2087)
bm2(1882)=bm1(2088)
bm2(1883)=bm1(2089)
bm2(1884)=bm1(2090)
bm2(1885)=bm1(2091)
bm2(1886)=bm1(2092)
bm2(1887)=bm1(2093)
bm2(1888)=bm1(2094)
bm2(1889)=bm1(2095)
bm2(1890)=bm1(2096)
bm2(1891)=bm1(2097)
bm2(1892)=bm1(2098)
bm2(1893)=bm1(2099)
bm2(1894)=bm1(2100)
bm2(1895)=bm1(2101)
bm2(1896)=bm1(2102)
bm2(1897)=bm1(2103)
bm2(1898)=bm1(2104)
bm2(1899)=bm1(2105)
bm2(1900)=bm1(2106)
bm2(1901)=bm1(2107)
bm2(1902)=bm1(2108)
bm2(1903)=bm1(2109)
bm2(1904)=bm1(2110)
bm2(1905)=bm1(2111)
bm2(1906)=bm1(2112)
bm2(1907)=bm1(2113)
bm2(1908)=bm1(2114)
bm2(1909)=bm1(2115)
bm2(1910)=bm1(2116)
bm2(1911)=bm1(2117)
bm2(1912)=bm1(2118)
bm2(1913)=bm1(2119)
bm2(1914)=bm1(2120)
bm2(1915)=bm1(2121)
bm2(1916)=bm1(2122)
bm2(1917)=bm1(2123)
bm2(1918)=bm1(2125)
bm2(1919)=bm1(2126)
bm2(1920)=bm1(2127)
bm2(1921)=bm1(2128)
bm2(1922)=bm1(2129)
bm2(1923)=bm1(2130)
bm2(1924)=bm1(2131)
bm2(1925)=bm1(2132)
bm2(1926)=bm1(2133)
bm2(1927)=bm1(2134)
bm2(1928)=bm1(2135)
bm2(1929)=bm1(2136)
bm2(1930)=bm1(2137)
bm2(1931)=bm1(2138)
bm2(1932)=bm1(2140)
bm2(1933)=bm1(2142)
bm2(1934)=bm1(2143)
bm2(1935)=bm1(2144)
bm2(1936)=bm1(2145)
bm2(1937)=bm1(2146)
bm2(1938)=bm1(2147)
bm2(1939)=bm1(2148)
bm2(1940)=bm1(2149)
bm2(1941)=bm1(2150)
bm2(1942)=bm1(2151)
bm2(1943)=bm1(2152)
bm2(1944)=bm1(2153)
bm2(1945)=bm1(2154)
bm2(1946)=bm1(2156)
bm2(1947)=bm1(2157)
bm2(1948)=bm1(2159)
bm2(1949)=bm1(2161)
bm2(1950)=bm1(2162)
bm2(1951)=bm1(2163)
bm2(1952)=bm1(2164)
bm2(1953)=bm1(2165)
bm2(1954)=bm1(2166)
bm2(1955)=bm1(2167)
bm2(1956)=bm1(2168)
bm2(1957)=bm1(2169)
bm2(1958)=bm1(2170)
bm2(1959)=bm1(2171)
bm2(1960)=bm1(2173)
bm2(1961)=bm1(2174)
bm2(1962)=bm1(2176)
bm2(1963)=bm1(2177)
bm2(1964)=bm1(2179)
bm2(1965)=bm1(2181)
bm2(1966)=bm1(2182)
bm2(1967)=bm1(2183)
bm2(1968)=bm1(2184)
bm2(1969)=bm1(2185)
bm2(1970)=bm1(2186)
bm2(1971)=bm1(2187)
bm2(1972)=bm1(2188)
bm2(1973)=bm1(2189)
bm2(1974)=bm1(2190)
bm2(1975)=bm1(2191)
bm2(1976)=bm1(2193)
bm2(1977)=bm1(2194)
bm2(1978)=bm1(2196)
bm2(1979)=bm1(2197)
bm2(1980)=bm1(2199)
bm2(1981)=bm1(2200)
bm2(1982)=bm1(2201)
bm2(1983)=bm1(2203)
bm2(1984)=bm1(2204)
bm2(1985)=bm1(2206)
bm2(1986)=bm1(2207)
bm2(1987)=bm1(2208)
bm2(1988)=bm1(2209)
bm2(1989)=bm1(2210)
bm2(1990)=bm1(2211)
bm2(1991)=bm1(2212)
bm2(1992)=bm1(2213)
bm2(1993)=bm1(2214)
bm2(1994)=bm1(2216)
bm2(1995)=bm1(2217)
bm2(1996)=bm1(2219)
bm2(1997)=bm1(2220)
bm2(1998)=bm1(2222)
bm2(1999)=bm1(2223)
bm2(2000)=bm1(2224)
bm2(2001)=bm1(2226)
bm2(2002)=bm1(2227)
bm2(2003)=bm1(2228)
bm2(2004)=bm1(2229)
bm2(2005)=bm1(2230)
bm2(2006)=bm1(2231)
bm2(2007)=bm1(2232)
bm2(2008)=bm1(2233)
bm2(2009)=bm1(2235)
bm2(2010)=bm1(2236)
bm2(2011)=bm1(2238)
bm2(2012)=bm1(2239)
bm2(2013)=bm1(2240)
bm2(2014)=bm1(2242)
bm2(2015)=bm1(2243)
bm2(2016)=bm1(2244)
bm2(2017)=bm1(2245)
bm2(2018)=bm1(2246)
bm2(2019)=bm1(2247)
bm2(2020)=bm1(2249)
bm2(2021)=bm1(2250)
bm2(2022)=bm1(2252)
bm2(2023)=bm1(2253)
bm2(2024)=bm1(2254)
bm2(2025)=bm1(2255)
bm2(2026)=bm1(2256)
bm2(2027)=bm1(2258)
bm2(2028)=bm1(2259)
bm2(2029)=bm1(2260)
bm2(2030)=bm1(2261)
bm2(2031)=bm1(2262)
bm2(2032)=bm1(2263)
bm2(2033)=bm1(2291)
bm2(2034)=bm1(2292)
bm2(2035)=bm1(2293)
bm2(2036)=bm1(2294)
bm2(2037)=bm1(2295)
bm2(2038)=bm1(2296)
bm2(2039)=bm1(2297)
bm2(2040)=bm1(2298)
bm2(2041)=bm1(2299)
bm2(2042)=bm1(2300)
bm2(2043)=bm1(2301)
bm2(2044)=bm1(2302)
bm2(2045)=bm1(2303)
bm2(2046)=bm1(2304)
bm2(2047)=bm1(2305)
bm2(2048)=bm1(2306)
bm2(2049)=bm1(2307)
bm2(2050)=bm1(2308)
bm2(2051)=bm1(2309)
bm2(2052)=bm1(2310)
bm2(2053)=bm1(2311)
bm2(2054)=bm1(2312)
bm2(2055)=bm1(2313)
bm2(2056)=bm1(2314)
bm2(2057)=bm1(2315)
bm2(2058)=bm1(2316)
bm2(2059)=bm1(2317)
bm2(2060)=bm1(2318)
bm2(2061)=bm1(2319)
bm2(2062)=bm1(2320)
bm2(2063)=bm1(2321)
bm2(2064)=bm1(2322)
bm2(2065)=bm1(2323)
bm2(2066)=bm1(2324)
bm2(2067)=bm1(2325)
bm2(2068)=bm1(2326)
bm2(2069)=bm1(2327)
bm2(2070)=bm1(2328)
bm2(2071)=bm1(2329)
bm2(2072)=bm1(2330)
bm2(2073)=bm1(2331)
bm2(2074)=bm1(2332)
bm2(2075)=bm1(2333)
bm2(2076)=bm1(2334)
bm2(2077)=bm1(2335)
bm2(2078)=bm1(2336)
bm2(2079)=bm1(2337)
bm2(2080)=bm1(2338)
bm2(2081)=bm1(2339)
bm2(2082)=bm1(2340)
bm2(2083)=bm1(2341)
bm2(2084)=bm1(2342)
bm2(2085)=bm1(2343)
bm2(2086)=bm1(2344)
bm2(2087)=bm1(2345)
bm2(2088)=bm1(2346)
bm2(2089)=bm1(2347)
bm2(2090)=bm1(2348)
bm2(2091)=bm1(2349)
bm2(2092)=bm1(2350)
bm2(2093)=bm1(2351)
bm2(2094)=bm1(2352)
bm2(2095)=bm1(2353)
bm2(2096)=bm1(2354)
bm2(2097)=bm1(2355)
bm2(2098)=bm1(2356)
bm2(2099)=bm1(2357)
bm2(2100)=bm1(2358)
bm2(2101)=bm1(2359)
bm2(2102)=bm1(2360)
bm2(2103)=bm1(2361)
bm2(2104)=bm1(2362)
bm2(2105)=bm1(2363)
bm2(2106)=bm1(2364)
bm2(2107)=bm1(2365)
bm2(2108)=bm1(2366)
bm2(2109)=bm1(2367)
bm2(2110)=bm1(2368)
bm2(2111)=bm1(2369)
bm2(2112)=bm1(2370)
bm2(2113)=bm1(2371)
bm2(2114)=bm1(2372)
bm2(2115)=bm1(2373)
bm2(2116)=bm1(2374)
bm2(2117)=bm1(2375)
bm2(2118)=bm1(2376)
bm2(2119)=bm1(2377)
bm2(2120)=bm1(2378)
bm2(2121)=bm1(2379)
bm2(2122)=bm1(2380)
bm2(2123)=bm1(2381)
bm2(2124)=bm1(2382)
bm2(2125)=bm1(2383)
bm2(2126)=bm1(2384)
bm2(2127)=bm1(2385)
bm2(2128)=bm1(2386)
bm2(2129)=bm1(2387)
bm2(2130)=bm1(2388)
bm2(2131)=bm1(2389)
bm2(2132)=bm1(2390)
bm2(2133)=bm1(2391)
bm2(2134)=bm1(2392)
bm2(2135)=bm1(2393)
bm2(2136)=bm1(2394)
bm2(2137)=bm1(2395)
bm2(2138)=bm1(2396)
bm2(2139)=bm1(2397)
bm2(2140)=bm1(2398)
bm2(2141)=bm1(2399)
bm2(2142)=bm1(2400)
bm2(2143)=bm1(2401)
bm2(2144)=bm1(2402)
bm2(2145)=bm1(2403)
bm2(2146)=bm1(2404)
bm2(2147)=bm1(2405)
bm2(2148)=bm1(2406)
bm2(2149)=bm1(2407)
bm2(2150)=bm1(2408)
bm2(2151)=bm1(2409)
bm2(2152)=bm1(2410)
bm2(2153)=bm1(2411)
bm2(2154)=bm1(2412)
bm2(2155)=bm1(2413)
bm2(2156)=bm1(2414)
bm2(2157)=bm1(2415)
bm2(2158)=bm1(2416)
bm2(2159)=bm1(2417)
bm2(2160)=bm1(2418)
bm2(2161)=bm1(2419)
bm2(2162)=bm1(2420)
bm2(2163)=bm1(2421)
bm2(2164)=bm1(2422)
bm2(2165)=bm1(2423)
bm2(2166)=bm1(2424)
bm2(2167)=bm1(2425)
bm2(2168)=bm1(2426)
bm2(2169)=bm1(2427)
bm2(2170)=bm1(2428)
bm2(2171)=bm1(2429)
bm2(2172)=bm1(2430)
bm2(2173)=bm1(2431)
bm2(2174)=bm1(2432)
bm2(2175)=bm1(2433)
bm2(2176)=bm1(2434)
bm2(2177)=bm1(2435)
bm2(2178)=bm1(2436)
bm2(2179)=bm1(2437)
bm2(2180)=bm1(2438)
bm2(2181)=bm1(2439)
bm2(2182)=bm1(2440)
bm2(2183)=bm1(2441)
bm2(2184)=bm1(2442)
bm2(2185)=bm1(2443)
bm2(2186)=bm1(2444)
bm2(2187)=bm1(2445)
bm2(2188)=bm1(2446)
bm2(2189)=bm1(2447)
bm2(2190)=bm1(2448)
bm2(2191)=bm1(2449)
bm2(2192)=bm1(2450)
bm2(2193)=bm1(2451)
bm2(2194)=bm1(2452)
bm2(2195)=bm1(2453)
bm2(2196)=bm1(2454)
bm2(2197)=bm1(2455)
bm2(2198)=bm1(2456)
bm2(2199)=bm1(2457)
bm2(2200)=bm1(2458)
bm2(2201)=bm1(2459)
bm2(2202)=bm1(2460)
bm2(2203)=bm1(2461)
bm2(2204)=bm1(2462)
bm2(2205)=bm1(2463)
bm2(2206)=bm1(2464)
bm2(2207)=bm1(2465)
bm2(2208)=bm1(2466)
bm2(2209)=bm1(2467)
bm2(2210)=bm1(2468)
bm2(2211)=bm1(2469)
bm2(2212)=bm1(2470)
bm2(2213)=bm1(2471)
bm2(2214)=bm1(2472)
bm2(2215)=bm1(2473)
bm2(2216)=bm1(2474)
bm2(2217)=bm1(2475)
bm2(2218)=bm1(2476)
bm2(2219)=bm1(2477)
bm2(2220)=bm1(2478)
bm2(2221)=bm1(2479)
bm2(2222)=bm1(2480)
bm2(2223)=bm1(2481)
bm2(2224)=bm1(2482)
bm2(2225)=bm1(2483)
bm2(2226)=bm1(2484)
bm2(2227)=bm1(2485)
bm2(2228)=bm1(2486)
bm2(2229)=bm1(2487)
bm2(2230)=bm1(2488)
bm2(2231)=bm1(2489)
bm2(2232)=bm1(2490)
bm2(2233)=bm1(2491)
bm2(2234)=bm1(2492)
bm2(2235)=bm1(2493)
bm2(2236)=bm1(2494)
bm2(2237)=bm1(2495)
bm2(2238)=bm1(2496)
bm2(2239)=bm1(2497)
bm2(2240)=bm1(2498)
bm2(2241)=bm1(2499)
bm2(2242)=bm1(2500)
bm2(2243)=bm1(2501)
bm2(2244)=bm1(2502)
bm2(2245)=bm1(2503)
bm2(2246)=bm1(2504)
bm2(2247)=bm1(2505)
bm2(2248)=bm1(2506)
bm2(2249)=bm1(2507)
bm2(2250)=bm1(2508)
bm2(2251)=bm1(2509)
bm2(2252)=bm1(2510)
bm2(2253)=bm1(2511)
bm2(2254)=bm1(2512)
bm2(2255)=bm1(2513)
bm2(2256)=bm1(2514)
bm2(2257)=bm1(2515)
bm2(2258)=bm1(2516)
bm2(2259)=bm1(2517)
bm2(2260)=bm1(2518)
bm2(2261)=bm1(2519)
bm2(2262)=bm1(2520)
bm2(2263)=bm1(2521)
bm2(2264)=bm1(2522)
bm2(2265)=bm1(2523)
bm2(2266)=bm1(2524)
bm2(2267)=bm1(2525)
bm2(2268)=bm1(2526)
bm2(2269)=bm1(2527)
bm2(2270)=bm1(2528)
bm2(2271)=bm1(2529)
bm2(2272)=bm1(2530)
bm2(2273)=bm1(2531)
bm2(2274)=bm1(2532)
bm2(2275)=bm1(2533)
bm2(2276)=bm1(2534)
bm2(2277)=bm1(2535)
bm2(2278)=bm1(2536)
bm2(2279)=bm1(2537)
bm2(2280)=bm1(2538)
bm2(2281)=bm1(2539)
bm2(2282)=bm1(2540)
bm2(2283)=bm1(2541)
bm2(2284)=bm1(2542)
bm2(2285)=bm1(2543)
bm2(2286)=bm1(2544)
bm2(2287)=bm1(2545)
bm2(2288)=bm1(2546)
bm2(2289)=bm1(2547)
bm2(2290)=bm1(2548)
bm2(2291)=bm1(2549)
bm2(2292)=bm1(2550)
bm2(2293)=bm1(2551)
bm2(2294)=bm1(2552)
bm2(2295)=bm1(2553)
bm2(2296)=bm1(2554)
bm2(2297)=bm1(2555)
bm2(2298)=bm1(2556)
bm2(2299)=bm1(2557)
bm2(2300)=bm1(2558)
bm2(2301)=bm1(2559)
bm2(2302)=bm1(2560)
bm2(2303)=bm1(2561)
bm2(2304)=bm1(2562)
bm2(2305)=bm1(2563)
bm2(2306)=bm1(2564)
bm2(2307)=bm1(2565)
bm2(2308)=bm1(2566)
bm2(2309)=bm1(2567)
bm2(2310)=bm1(2568)
bm2(2311)=bm1(2569)
bm2(2312)=bm1(2570)
bm2(2313)=bm1(2571)
bm2(2314)=bm1(2572)
bm2(2315)=bm1(2573)
bm2(2316)=bm1(2574)
bm2(2317)=bm1(2575)
bm2(2318)=bm1(2576)
bm2(2319)=bm1(2577)
bm2(2320)=bm1(2578)
bm2(2321)=bm1(2579)
bm2(2322)=bm1(2580)
bm2(2323)=bm1(2581)
bm2(2324)=bm1(2582)
bm2(2325)=bm1(2583)
bm2(2326)=bm1(2584)
bm2(2327)=bm1(2585)
bm2(2328)=bm1(2586)
bm2(2329)=bm1(2587)
bm2(2330)=bm1(2588)
bm2(2331)=bm1(2589)
bm2(2332)=bm1(2590)
bm2(2333)=bm1(2591)
bm2(2334)=bm1(2592)
bm2(2335)=bm1(2593)
bm2(2336)=bm1(2594)
bm2(2337)=bm1(2595)
bm2(2338)=bm1(2596)
bm2(2339)=bm1(2597)
bm2(2340)=bm1(2598)
bm2(2341)=bm1(2599)
bm2(2342)=bm1(2600)
bm2(2343)=bm1(2601)
bm2(2344)=bm1(2602)
bm2(2345)=bm1(2603)
bm2(2346)=bm1(2604)
bm2(2347)=bm1(2605)
bm2(2348)=bm1(2606)
bm2(2349)=bm1(2607)
bm2(2350)=bm1(2608)
bm2(2351)=bm1(2609)
bm2(2352)=bm1(2610)
bm2(2353)=bm1(2611)
bm2(2354)=bm1(2612)
bm2(2355)=bm1(2613)
bm2(2356)=bm1(2614)
bm2(2357)=bm1(2615)
bm2(2358)=bm1(2616)
bm2(2359)=bm1(2617)
bm2(2360)=bm1(2618)
bm2(2361)=bm1(2619)
bm2(2362)=bm1(2620)
bm2(2363)=bm1(2621)
bm2(2364)=bm1(2622)
bm2(2365)=bm1(2623)
bm2(2366)=bm1(2624)
bm2(2367)=bm1(2625)
bm2(2368)=bm1(2626)
bm2(2369)=bm1(2627)
bm2(2370)=bm1(2628)
bm2(2371)=bm1(2629)
bm2(2372)=bm1(2630)
bm2(2373)=bm1(2631)
bm2(2374)=bm1(2632)
bm2(2375)=bm1(2633)
bm2(2376)=bm1(2634)
bm2(2377)=bm1(2635)
bm2(2378)=bm1(2636)
bm2(2379)=bm1(2637)
bm2(2380)=bm1(2638)
bm2(2381)=bm1(2639)
bm2(2382)=bm1(2640)
bm2(2383)=bm1(2641)
bm2(2384)=bm1(2642)
bm2(2385)=bm1(2643)
bm2(2386)=bm1(2644)
bm2(2387)=bm1(2645)
bm2(2388)=bm1(2646)
bm2(2389)=bm1(2647)
bm2(2390)=bm1(2648)
bm2(2391)=bm1(2649)
bm2(2392)=bm1(2650)
bm2(2393)=bm1(2651)
bm2(2394)=bm1(2652)
bm2(2395)=bm1(2653)
bm2(2396)=bm1(2654)
bm2(2397)=bm1(2655)
bm2(2398)=bm1(2656)
bm2(2399)=bm1(2657)
bm2(2400)=bm1(2658)
bm2(2401)=bm1(2659)
bm2(2402)=bm1(2660)
bm2(2403)=bm1(2661)
bm2(2404)=bm1(2662)
bm2(2405)=bm1(2663)
bm2(2406)=bm1(2664)
bm2(2407)=bm1(2665)
bm2(2408)=bm1(2666)
bm2(2409)=bm1(2667)
bm2(2410)=bm1(2668)
bm2(2411)=bm1(2669)
bm2(2412)=bm1(2670)
bm2(2413)=bm1(2671)
bm2(2414)=bm1(2672)
bm2(2415)=bm1(2673)
bm2(2416)=bm1(2674)
bm2(2417)=bm1(2675)
bm2(2418)=bm1(2676)
bm2(2419)=bm1(2677)
bm2(2420)=bm1(2678)
bm2(2421)=bm1(2679)
bm2(2422)=bm1(2680)
bm2(2423)=bm1(2681)
bm2(2424)=bm1(2682)
bm2(2425)=bm1(2683)
bm2(2426)=bm1(2684)
bm2(2427)=bm1(2685)
bm2(2428)=bm1(2686)
bm2(2429)=bm1(2687)
bm2(2430)=bm1(2688)
bm2(2431)=bm1(2689)
bm2(2432)=bm1(2690)
bm2(2433)=bm1(2691)
bm2(2434)=bm1(2692)
bm2(2435)=bm1(2693)
bm2(2436)=bm1(2694)
bm2(2437)=bm1(2695)
bm2(2438)=bm1(2696)
bm2(2439)=bm1(2697)
bm2(2440)=bm1(2698)
bm2(2441)=bm1(2699)
bm2(2442)=bm1(2700)
bm2(2443)=bm1(2701)
bm2(2444)=bm1(2702)
bm2(2445)=bm1(2703)
bm2(2446)=bm1(2704)
bm2(2447)=bm1(2705)
bm2(2448)=bm1(2706)
bm2(2449)=bm1(2707)
bm2(2450)=bm1(2708)
bm2(2451)=bm1(2709)
bm2(2452)=bm1(2710)
bm2(2453)=bm1(2711)
bm2(2454)=bm1(2712)
bm2(2455)=bm1(2713)
bm2(2456)=bm1(2714)
bm2(2457)=bm1(2715)
bm2(2458)=bm1(2716)
bm2(2459)=bm1(2717)
bm2(2460)=bm1(2718)
bm2(2461)=bm1(2719)
bm2(2462)=bm1(2720)
bm2(2463)=bm1(2721)
bm2(2464)=bm1(2722)
bm2(2465)=bm1(2723)
bm2(2466)=bm1(2724)
bm2(2467)=bm1(2725)
bm2(2468)=bm1(2726)
bm2(2469)=bm1(2727)
bm2(2470)=bm1(2728)
bm2(2471)=bm1(2729)
bm2(2472)=bm1(2730)
bm2(2473)=bm1(2731)
bm2(2474)=bm1(2732)
bm2(2475)=bm1(2733)
bm2(2476)=bm1(2734)
bm2(2477)=bm1(2735)
bm2(2478)=bm1(2736)
bm2(2479)=bm1(2737)
bm2(2480)=bm1(2738)
bm2(2481)=bm1(2739)
bm2(2482)=bm1(2740)
bm2(2483)=bm1(2741)
bm2(2484)=bm1(2742)
bm2(2485)=bm1(2743)
bm2(2486)=bm1(2744)
bm2(2487)=bm1(2745)
bm2(2488)=bm1(2746)
bm2(2489)=bm1(2747)
bm2(2490)=bm1(2748)
bm2(2491)=bm1(2749)
bm2(2492)=bm1(2750)
bm2(2493)=bm1(2751)
bm2(2494)=bm1(2752)
bm2(2495)=bm1(2753)
bm2(2496)=bm1(2754)
bm2(2497)=bm1(2755)
bm2(2498)=bm1(2756)
bm2(2499)=bm1(2757)
bm2(2500)=bm1(2758)
bm2(2501)=bm1(2759)
bm2(2502)=bm1(2760)
bm2(2503)=bm1(2761)
bm2(2504)=bm1(2762)
bm2(2505)=bm1(2763)
bm2(2506)=bm1(2764)
bm2(2507)=bm1(2765)
bm2(2508)=bm1(2766)
bm2(2509)=bm1(2767)
bm2(2510)=bm1(2768)
bm2(2511)=bm1(2769)
bm2(2512)=bm1(2770)
bm2(2513)=bm1(2771)
bm2(2514)=bm1(2772)
bm2(2515)=bm1(2773)
bm2(2516)=bm1(2774)
bm2(2517)=bm1(2775)
bm2(2518)=bm1(2776)
bm2(2519)=bm1(2777)
bm2(2520)=bm1(2778)
bm2(2521)=bm1(2779)
bm2(2522)=bm1(2780)
bm2(2523)=bm1(2781)
bm2(2524)=bm1(2782)
bm2(2525)=bm1(2783)
bm2(2526)=bm1(2784)
bm2(2527)=bm1(2785)
bm2(2528)=bm1(2786)
bm2(2529)=bm1(2787)
bm2(2530)=bm1(2788)
bm2(2531)=bm1(2789)
bm2(2532)=bm1(2790)
bm2(2533)=bm1(2791)
bm2(2534)=bm1(2792)
bm2(2535)=bm1(2793)
bm2(2536)=bm1(2794)
bm2(2537)=bm1(2795)
bm2(2538)=bm1(2796)
bm2(2539)=bm1(2797)
bm2(2540)=bm1(2798)
bm2(2541)=bm1(2799)
bm2(2542)=bm1(2800)
bm2(2543)=bm1(2801)
bm2(2544)=bm1(2802)
bm2(2545)=bm1(2803)
bm2(2546)=bm1(2804)
bm2(2547)=bm1(2805)
bm2(2548)=bm1(2806)
bm2(2549)=bm1(2807)
bm2(2550)=bm1(2808)
bm2(2551)=bm1(2809)
bm2(2552)=bm1(2810)
bm2(2553)=bm1(2811)
bm2(2554)=bm1(2812)
bm2(2555)=bm1(2813)
bm2(2556)=bm1(2814)
bm2(2557)=bm1(2815)
bm2(2558)=bm1(2816)
bm2(2559)=bm1(2817)
bm2(2560)=bm1(2818)
bm2(2561)=bm1(2819)
bm2(2562)=bm1(2820)
bm2(2563)=bm1(2821)
bm2(2564)=bm1(2822)
bm2(2565)=bm1(2823)
bm2(2566)=bm1(2824)
bm2(2567)=bm1(2825)
bm2(2568)=bm1(2826)
bm2(2569)=bm1(2827)
bm2(2570)=bm1(2828)
bm2(2571)=bm1(2829)
bm2(2572)=bm1(2830)
bm2(2573)=bm1(2831)
bm2(2574)=bm1(2832)
bm2(2575)=bm1(2833)
bm2(2576)=bm1(2834)
bm2(2577)=bm1(2835)
bm2(2578)=bm1(2836)
bm2(2579)=bm1(2837)
bm2(2580)=bm1(2838)
bm2(2581)=bm1(2839)
bm2(2582)=bm1(2840)
bm2(2583)=bm1(2841)
bm2(2584)=bm1(2842)
bm2(2585)=bm1(2843)
bm2(2586)=bm1(2844)
bm2(2587)=bm1(2845)
bm2(2588)=bm1(2846)
bm2(2589)=bm1(2847)
bm2(2590)=bm1(2848)
bm2(2591)=bm1(2849)
bm2(2592)=bm1(2850)
bm2(2593)=bm1(2851)
bm2(2594)=bm1(2852)
bm2(2595)=bm1(2853)
bm2(2596)=bm1(2854)
bm2(2597)=bm1(2855)
bm2(2598)=bm1(2856)
bm2(2599)=bm1(2857)
bm2(2600)=bm1(2858)
bm2(2601)=bm1(2859)
bm2(2602)=bm1(2860)
bm2(2603)=bm1(2861)
bm2(2604)=bm1(2862)
bm2(2605)=bm1(2863)
bm2(2606)=bm1(2864)
bm2(2607)=bm1(2865)
bm2(2608)=bm1(2866)
bm2(2609)=bm1(2867)
bm2(2610)=bm1(2868)
bm2(2611)=bm1(2869)
bm2(2612)=bm1(2870)
bm2(2613)=bm1(2871)
bm2(2614)=bm1(2872)
bm2(2615)=bm1(2873)
bm2(2616)=bm1(2874)
bm2(2617)=bm1(2875)
bm2(2618)=bm1(2876)
bm2(2619)=bm1(2877)
bm2(2620)=bm1(2878)
bm2(2621)=bm1(2879)
bm2(2622)=bm1(2880)
bm2(2623)=bm1(2881)
bm2(2624)=bm1(2882)
bm2(2625)=bm1(2883)
bm2(2626)=bm1(2884)
bm2(2627)=bm1(2885)
bm2(2628)=bm1(2886)
bm2(2629)=bm1(2887)
bm2(2630)=bm1(2888)
bm2(2631)=bm1(2889)
bm2(2632)=bm1(2890)
bm2(2633)=bm1(2891)
bm2(2634)=bm1(2892)
bm2(2635)=bm1(2893)
bm2(2636)=bm1(2894)
bm2(2637)=bm1(2895)
bm2(2638)=bm1(2896)
bm2(2639)=bm1(2897)
bm2(2640)=bm1(2898)
bm2(2641)=bm1(2899)
bm2(2642)=bm1(2900)
bm2(2643)=bm1(2901)
bm2(2644)=bm1(2902)
bm2(2645)=bm1(2903)
bm2(2646)=bm1(2904)
bm2(2647)=bm1(2905)
bm2(2648)=bm1(2906)
bm2(2649)=bm1(2907)
bm2(2650)=bm1(2908)
bm2(2651)=bm1(2909)
bm2(2652)=bm1(2910)
bm2(2653)=bm1(2911)
bm2(2654)=bm1(2912)
bm2(2655)=bm1(2913)
bm2(2656)=bm1(2914)
bm2(2657)=bm1(2915)
bm2(2658)=bm1(2916)
bm2(2659)=bm1(2917)
bm2(2660)=bm1(2918)
bm2(2661)=bm1(2919)
bm2(2662)=bm1(2920)
bm2(2663)=bm1(2921)
bm2(2664)=bm1(2922)
bm2(2665)=bm1(2923)
bm2(2666)=bm1(2924)
bm2(2667)=bm1(2925)
bm2(2668)=bm1(2926)
bm2(2669)=bm1(2927)
bm2(2670)=bm1(2928)
bm2(2671)=bm1(2929)
bm2(2672)=bm1(2930)
bm2(2673)=bm1(2931)
bm2(2674)=bm1(2932)
bm2(2675)=bm1(2933)
bm2(2676)=bm1(2934)
bm2(2677)=bm1(2935)
bm2(2678)=bm1(2936)
bm2(2679)=bm1(2937)
bm2(2680)=bm1(2938)
bm2(2681)=bm1(2939)
bm2(2682)=bm1(2940)
bm2(2683)=bm1(2941)
bm2(2684)=bm1(2942)
bm2(2685)=bm1(2943)
bm2(2686)=bm1(2944)
bm2(2687)=bm1(2945)
bm2(2688)=bm1(2946)
bm2(2689)=bm1(2947)
bm2(2690)=bm1(2948)
bm2(2691)=bm1(2949)
bm2(2692)=bm1(2950)
bm2(2693)=bm1(2951)
bm2(2694)=bm1(2952)
bm2(2695)=bm1(2953)
bm2(2696)=bm1(2954)
bm2(2697)=bm1(2955)
bm2(2698)=bm1(2956)
bm2(2699)=bm1(2957)
bm2(2700)=bm1(2958)
bm2(2701)=bm1(2959)
bm2(2702)=bm1(2960)
bm2(2703)=bm1(2961)
bm2(2704)=bm1(2962)
bm2(2705)=bm1(2963)
bm2(2706)=bm1(2964)
bm2(2707)=bm1(2965)
bm2(2708)=bm1(2966)
bm2(2709)=bm1(2967)
bm2(2710)=bm1(2968)
bm2(2711)=bm1(2969)
bm2(2712)=bm1(2970)
bm2(2713)=bm1(2971)
bm2(2714)=bm1(2972)
bm2(2715)=bm1(2973)
bm2(2716)=bm1(2974)
bm2(2717)=bm1(2975)
bm2(2718)=bm1(2976)
bm2(2719)=bm1(2977)
bm2(2720)=bm1(2978)
bm2(2721)=bm1(2979)
bm2(2722)=bm1(2980)
bm2(2723)=bm1(2981)
bm2(2724)=bm1(2982)
bm2(2725)=bm1(2983)
bm2(2726)=bm1(2984)
bm2(2727)=bm1(2985)
bm2(2728)=bm1(2986)
bm2(2729)=bm1(2987)
bm2(2730)=bm1(2988)
bm2(2731)=bm1(2989)
bm2(2732)=bm1(2990)
bm2(2733)=bm1(2991)
bm2(2734)=bm1(2992)
bm2(2735)=bm1(2993)
bm2(2736)=bm1(2994)
bm2(2737)=bm1(2995)
bm2(2738)=bm1(2996)
bm2(2739)=bm1(2997)
bm2(2740)=bm1(2998)
bm2(2741)=bm1(2999)
bm2(2742)=bm1(3000)
bm2(2743)=bm1(3001)
bm2(2744)=bm1(3002)
bm2(2745)=bm1(3003)
bm2(2746)=bm1(3004)
bm2(2747)=bm1(3005)
bm2(2748)=bm1(3006)
bm2(2749)=bm1(3007)
bm2(2750)=bm1(3008)
bm2(2751)=bm1(3009)
bm2(2752)=bm1(3010)
bm2(2753)=bm1(3011)
bm2(2754)=bm1(3012)
bm2(2755)=bm1(3013)
bm2(2756)=bm1(3014)
bm2(2757)=bm1(3015)
bm2(2758)=bm1(3016)
bm2(2759)=bm1(3017)
bm2(2760)=bm1(3018)
bm2(2761)=bm1(3019)
bm2(2762)=bm1(3020)
bm2(2763)=bm1(3021)
bm2(2764)=bm1(3022)
bm2(2765)=bm1(3023)
bm2(2766)=bm1(3024)
bm2(2767)=bm1(3025)
bm2(2768)=bm1(3026)
bm2(2769)=bm1(3027)
bm2(2770)=bm1(3028)
bm2(2771)=bm1(3029)
bm2(2772)=bm1(3030)
bm2(2773)=bm1(3031)
bm2(2774)=bm1(3032)
bm2(2775)=bm1(3033)
bm2(2776)=bm1(3034)
bm2(2777)=bm1(3035)
bm2(2778)=bm1(3036)
bm2(2779)=bm1(3037)
bm2(2780)=bm1(3038)
bm2(2781)=bm1(3039)
bm2(2782)=bm1(3040)
bm2(2783)=bm1(3041)
bm2(2784)=bm1(3042)
bm2(2785)=bm1(3043)
bm2(2786)=bm1(3044)
bm2(2787)=bm1(3045)
bm2(2788)=bm1(3046)
bm2(2789)=bm1(3047)
bm2(2790)=bm1(3048)
bm2(2791)=bm1(3049)
bm2(2792)=bm1(3050)
bm2(2793)=bm1(3051)
bm2(2794)=bm1(3052)
bm2(2795)=bm1(3053)
bm2(2796)=bm1(3054)
bm2(2797)=bm1(3055)
bm2(2798)=bm1(3056)
bm2(2799)=bm1(3057)
bm2(2800)=bm1(3058)
bm2(2801)=bm1(3059)
bm2(2802)=bm1(3060)
bm2(2803)=bm1(3061)
bm2(2804)=bm1(3062)
bm2(2805)=bm1(3063)
bm2(2806)=bm1(3064)
bm2(2807)=bm1(3065)
bm2(2808)=bm1(3066)
bm2(2809)=bm1(3067)
bm2(2810)=bm1(3068)
bm2(2811)=bm1(3069)
bm2(2812)=bm1(3070)
bm2(2813)=bm1(3071)
bm2(2814)=bm1(3072)
bm2(2815)=bm1(3073)
bm2(2816)=bm1(3074)
bm2(2817)=bm1(3075)
bm2(2818)=bm1(3076)
bm2(2819)=bm1(3077)
bm2(2820)=bm1(3078)
bm2(2821)=bm1(3079)
bm2(2822)=bm1(3080)
bm2(2823)=bm1(3081)
bm2(2824)=bm1(3082)
bm2(2825)=bm1(3083)
bm2(2826)=bm1(3084)
bm2(2827)=bm1(3085)
bm2(2828)=bm1(3086)
bm2(2829)=bm1(3087)
bm2(2830)=bm1(3088)
bm2(2831)=bm1(3089)
bm2(2832)=bm1(3090)
bm2(2833)=bm1(3091)
bm2(2834)=bm1(3092)
bm2(2835)=bm1(3093)
bm2(2836)=bm1(3094)
bm2(2837)=bm1(3095)
bm2(2838)=bm1(3096)
bm2(2839)=bm1(3097)
bm2(2840)=bm1(3098)
bm2(2841)=bm1(3099)
bm2(2842)=bm1(3100)
bm2(2843)=bm1(3101)
bm2(2844)=bm1(3102)
bm2(2845)=bm1(3103)
bm2(2846)=bm1(3104)
bm2(2847)=bm1(3105)
bm2(2848)=bm1(3106)
bm2(2849)=bm1(3107)
bm2(2850)=bm1(3108)
bm2(2851)=bm1(3109)
bm2(2852)=bm1(3110)
bm2(2853)=bm1(3111)
bm2(2854)=bm1(3112)
bm2(2855)=bm1(3113)
bm2(2856)=bm1(3114)
bm2(2857)=bm1(3115)
bm2(2858)=bm1(3116)
bm2(2859)=bm1(3117)
bm2(2860)=bm1(3118)
bm2(2861)=bm1(3119)
bm2(2862)=bm1(3120)
bm2(2863)=bm1(3121)
bm2(2864)=bm1(3122)
bm2(2865)=bm1(3123)
bm2(2866)=bm1(3124)
bm2(2867)=bm1(3125)
bm2(2868)=bm1(3126)
bm2(2869)=bm1(3127)
bm2(2870)=bm1(3128)
bm2(2871)=bm1(3129)
bm2(2872)=bm1(3130)
bm2(2873)=bm1(3131)
bm2(2874)=bm1(3132)
bm2(2875)=bm1(3133)
bm2(2876)=bm1(3134)
bm2(2877)=bm1(3135)
bm2(2878)=bm1(3136)
bm2(2879)=bm1(3137)
bm2(2880)=bm1(3138)
bm2(2881)=bm1(3139)
bm2(2882)=bm1(3140)
bm2(2883)=bm1(3141)
bm2(2884)=bm1(3142)
bm2(2885)=bm1(3143)
bm2(2886)=bm1(3144)
bm2(2887)=bm1(3145)
bm2(2888)=bm1(3146)
bm2(2889)=bm1(3147)
bm2(2890)=bm1(3148)
bm2(2891)=bm1(3149)
bm2(2892)=bm1(3150)
bm2(2893)=bm1(3151)
bm2(2894)=bm1(3152)
bm2(2895)=bm1(3153)
bm2(2896)=bm1(3154)
bm2(2897)=bm1(3155)
bm2(2898)=bm1(3156)
bm2(2899)=bm1(3157)
bm2(2900)=bm1(3158)
bm2(2901)=bm1(3159)
bm2(2902)=bm1(3160)
bm2(2903)=bm1(3161)
bm2(2904)=bm1(3162)
bm2(2905)=bm1(3163)
bm2(2906)=bm1(3164)
bm2(2907)=bm1(3165)
bm2(2908)=bm1(3166)
bm2(2909)=bm1(3167)
bm2(2910)=bm1(3168)
bm2(2911)=bm1(3169)
bm2(2912)=bm1(3170)
bm2(2913)=bm1(3171)
bm2(2914)=bm1(3172)
bm2(2915)=bm1(3173)
bm2(2916)=bm1(3174)
bm2(2917)=bm1(3175)
bm2(2918)=bm1(3176)
bm2(2919)=bm1(3177)
bm2(2920)=bm1(3178)
bm2(2921)=bm1(3179)
bm2(2922)=bm1(3180)
bm2(2923)=bm1(3181)
bm2(2924)=bm1(3182)
bm2(2925)=bm1(3183)
bm2(2926)=bm1(3184)
bm2(2927)=bm1(3185)
bm2(2928)=bm1(3186)
bm2(2929)=bm1(3187)
bm2(2930)=bm1(3188)
bm2(2931)=bm1(3189)
bm2(2932)=bm1(3190)
bm2(2933)=bm1(3191)
bm2(2934)=bm1(3193)
bm2(2935)=bm1(3194)
bm2(2936)=bm1(3195)
bm2(2937)=bm1(3196)
bm2(2938)=bm1(3197)
bm2(2939)=bm1(3198)
bm2(2940)=bm1(3199)
bm2(2941)=bm1(3200)
bm2(2942)=bm1(3201)
bm2(2943)=bm1(3202)
bm2(2944)=bm1(3203)
bm2(2945)=bm1(3204)
bm2(2946)=bm1(3205)
bm2(2947)=bm1(3206)
bm2(2948)=bm1(3207)
bm2(2949)=bm1(3208)
bm2(2950)=bm1(3210)
bm2(2951)=bm1(3212)
bm2(2952)=bm1(3213)
bm2(2953)=bm1(3214)
bm2(2954)=bm1(3215)
bm2(2955)=bm1(3216)
bm2(2956)=bm1(3217)
bm2(2957)=bm1(3218)
bm2(2958)=bm1(3219)
bm2(2959)=bm1(3220)
bm2(2960)=bm1(3221)
bm2(2961)=bm1(3222)
bm2(2962)=bm1(3223)
bm2(2963)=bm1(3224)
bm2(2964)=bm1(3225)
bm2(2965)=bm1(3227)
bm2(2966)=bm1(3228)
bm2(2967)=bm1(3230)
bm2(2968)=bm1(3232)
bm2(2969)=bm1(3233)
bm2(2970)=bm1(3234)
bm2(2971)=bm1(3235)
bm2(2972)=bm1(3236)
bm2(2973)=bm1(3237)
bm2(2974)=bm1(3238)
bm2(2975)=bm1(3239)
bm2(2976)=bm1(3240)
bm2(2977)=bm1(3241)
bm2(2978)=bm1(3242)
bm2(2979)=bm1(3243)
bm2(2980)=bm1(3244)
bm2(2981)=bm1(3246)
bm2(2982)=bm1(3247)
bm2(2983)=bm1(3249)
bm2(2984)=bm1(3250)
bm2(2985)=bm1(3252)
bm2(2986)=bm1(3254)
bm2(2987)=bm1(3255)
bm2(2988)=bm1(3256)
bm2(2989)=bm1(3257)
bm2(2990)=bm1(3258)
bm2(2991)=bm1(3259)
bm2(2992)=bm1(3260)
bm2(2993)=bm1(3261)
bm2(2994)=bm1(3262)
bm2(2995)=bm1(3263)
bm2(2996)=bm1(3264)
bm2(2997)=bm1(3265)
bm2(2998)=bm1(3267)
bm2(2999)=bm1(3268)
bm2(3000)=bm1(3270)
bm2(3001)=bm1(3271)
bm2(3002)=bm1(3273)
bm2(3003)=bm1(3274)
bm2(3004)=bm1(3276)
bm2(3005)=bm1(3277)
bm2(3006)=bm1(3279)
bm2(3007)=bm1(3280)
bm2(3008)=bm1(3281)
bm2(3009)=bm1(3282)
bm2(3010)=bm1(3283)
bm2(3011)=bm1(3284)
bm2(3012)=bm1(3285)
bm2(3013)=bm1(3286)
bm2(3014)=bm1(3287)
bm2(3015)=bm1(3288)
bm2(3016)=bm1(3289)
bm2(3017)=bm1(3291)
bm2(3018)=bm1(3292)
bm2(3019)=bm1(3294)
bm2(3020)=bm1(3295)
bm2(3021)=bm1(3297)
bm2(3022)=bm1(3298)
bm2(3023)=bm1(3299)
bm2(3024)=bm1(3301)
bm2(3025)=bm1(3302)
bm2(3026)=bm1(3303)
bm2(3027)=bm1(3305)
bm2(3028)=bm1(3306)
bm2(3029)=bm1(3307)
bm2(3030)=bm1(3308)
bm2(3031)=bm1(3309)
bm2(3032)=bm1(3310)
bm2(3033)=bm1(3311)
bm2(3034)=bm1(3312)
bm2(3035)=bm1(3313)
bm2(3036)=bm1(3315)
bm2(3037)=bm1(3316)
bm2(3038)=bm1(3318)
bm2(3039)=bm1(3319)
bm2(3040)=bm1(3321)
bm2(3041)=bm1(3322)
bm2(3042)=bm1(3323)
bm2(3043)=bm1(3325)
bm2(3044)=bm1(3326)
bm2(3045)=bm1(3327)
bm2(3046)=bm1(3328)
bm2(3047)=bm1(3329)
bm2(3048)=bm1(3330)
bm2(3049)=bm1(3331)
bm2(3050)=bm1(3332)
bm2(3051)=bm1(3334)
bm2(3052)=bm1(3335)
bm2(3053)=bm1(3337)
bm2(3054)=bm1(3338)
bm2(3055)=bm1(3339)
bm2(3056)=bm1(3341)
bm2(3057)=bm1(3342)
bm2(3058)=bm1(3343)
bm2(3059)=bm1(3344)
bm2(3060)=bm1(3345)
bm2(3061)=bm1(3346)
bm2(3062)=bm1(3348)
bm2(3063)=bm1(3349)
bm2(3064)=bm1(3351)
bm2(3065)=bm1(3352)
bm2(3066)=bm1(3353)
bm2(3067)=bm1(3354)
bm2(3068)=bm1(3355)
bm2(3069)=bm1(3357)
bm2(3070)=bm1(3358)
bm2(3071)=bm1(3359)
bm2(3072)=bm1(3360)
bm2(3073)=bm1(3361)
bm2(3074)=bm1(3362)
bm2(3075)=bm1(3363)

do j=1,3075
  q(j)=bm2(j)
enddo

end subroutine ev_pol_con_12

!===========================================================================
! *def ev_pol_con_10(x,q)
! subtoutine of evaluating polynomials at 10th order
!===========================================================================
subroutine ev_pol_con_10(x,q)
implicit none
integer :: i, j
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp),dimension(1:6),intent(in)::x
real(dp),dimension(1:1300),intent(out)::q
real(dp),dimension(0:32) :: rm
real(dp),dimension(0:1506) :: p
real(dp) :: bm1(1506), bm2(1300)

rm(0) = 1.0D0
rm(1) = x(4)
rm(2) = x(2)
rm(3) = x(1)
rm(4) = x(6)
rm(5) = x(5)
rm(6) = x(3)
rm(7) = rm(1)*rm(2)
rm(8) = rm(1)*rm(3)
rm(9) = rm(2)*rm(3)
rm(10) = rm(3)*rm(4)
rm(11) = rm(2)*rm(5)
rm(12) = rm(1)*rm(6)
rm(13) = rm(4)*rm(5)
rm(14) = rm(4)*rm(6)
rm(15) = rm(5)*rm(6)
rm(16) = rm(1)*rm(9)
rm(17) = rm(1)*rm(10)
rm(18) = rm(2)*rm(10)
rm(19) = rm(1)*rm(11)
rm(20) = rm(3)*rm(11)
rm(21) = rm(2)*rm(12)
rm(22) = rm(3)*rm(12)
rm(23) = rm(2)*rm(13)
rm(24) = rm(3)*rm(13)
rm(25) = rm(1)*rm(14)
rm(26) = rm(3)*rm(14)
rm(27) = rm(1)*rm(15)
rm(28) = rm(2)*rm(15)
rm(29) = rm(4)*rm(15)
rm(30) = rm(2)*rm(24)
rm(31) = rm(1)*rm(26)
rm(32) = rm(1)*rm(28)

p(0) = rm(0)
p(1) = rm(1) + rm(2) + rm(3)
p(2) = rm(4) + rm(5) + rm(6)
p(3) = rm(7) + rm(8) + rm(9)
p(4) = rm(10) + rm(11) + rm(12)
p(5) = p(1)*p(2) - p(4)
p(6) = rm(13) + rm(14) + rm(15)
p(7) = p(1)*p(1) - p(3) - p(3)
p(8) = p(2)*p(2) - p(6) - p(6)
p(9) = rm(16)
p(10) = rm(17) + rm(18) + rm(19) + rm(20) + rm(21) + rm(22)
p(11) = p(2)*p(3) - p(10)
p(12) = rm(23) + rm(24) + rm(25) + rm(26) + rm(27) + rm(28)
p(13) = p(1)*p(6) - p(12)
p(14) = rm(29)
p(15) = p(1)*p(3) - p(9) - p(9) - p(9)
p(16) = p(1)*p(4) - p(10)
p(17) = p(2)*p(7) - p(16)
p(18) = p(2)*p(4) - p(12)
p(19) = p(1)*p(8) - p(18)
p(20) = p(2)*p(6) - p(14) - p(14) - p(14)
p(21) = p(1)*p(7) - p(15)
p(22) = p(2)*p(8) - p(20)
p(23) = p(9)*p(2)
p(24) = rm(30) + rm(31) + rm(32)
p(25) = p(3)*p(6) - p(24)
p(26) = p(14)*p(1)
p(27) = p(9)*p(1)
p(28) = p(3)*p(4) - p(23)
p(29) = p(1)*p(10) - p(23) - p(28) - p(23)
p(30) = p(1)*p(11) - p(23)
p(31) = p(1)*p(12) - p(25) - p(24) - p(24)
p(32) = p(1)*p(13) - p(25)
p(33) = p(4)*p(5) - p(25) - p(31)
p(34) = p(2)*p(11) - p(25)
p(35) = p(4)*p(6) - p(26)
p(36) = p(2)*p(12) - p(26) - p(35) - p(26)
p(37) = p(2)*p(13) - p(26)
p(38) = p(14)*p(2)
p(39) = p(3)*p(3) - p(27) - p(27)
p(40) = p(3)*p(7) - p(27)
p(41) = p(1)*p(16) - p(28)
p(42) = p(2)*p(21) - p(41)
p(43) = p(1)*p(18) - p(33)
p(44) = p(7)*p(8) - p(43)
p(45) = p(6)*p(6) - p(38) - p(38)
p(46) = p(2)*p(18) - p(35)
p(47) = p(1)*p(22) - p(46)
p(48) = p(6)*p(8) - p(38)
p(49) = p(1)*p(21) - p(40)
p(50) = p(2)*p(22) - p(48)
p(51) = p(9)*p(6)
p(52) = p(14)*p(3)
p(53) = p(9)*p(4)
p(54) = p(9)*p(5)
p(55) = p(1)*p(24) - p(51)
p(56) = p(3)*p(12) - p(51) - p(55) - p(51)
p(57) = p(3)*p(13) - p(51)
p(58) = p(14)*p(7)
p(59) = p(9)*p(8)
p(60) = p(2)*p(24) - p(52)
p(61) = p(4)*p(13) - p(58)
p(62) = p(6)*p(11) - p(52)
p(63) = p(14)*p(4)
p(64) = p(14)*p(5)
p(65) = p(9)*p(3)
p(66) = p(9)*p(7)
p(67) = p(3)*p(16) - p(53)
p(68) = p(4)*p(15) - p(54) - p(67)
p(69) = p(2)*p(39) - p(68)
p(70) = p(1)*p(29) - p(54) - p(68)
p(71) = p(7)*p(11) - p(53)
p(72) = p(1)*p(31) - p(56) - p(55)
p(73) = p(1)*p(32) - p(57)
p(74) = p(3)*p(18) - p(59)
p(75) = p(2)*p(29) - p(57) - p(55)
p(76) = p(1)*p(34) - p(59)
p(77) = p(6)*p(16) - p(58)
p(78) = p(1)*p(36) - p(62) - p(60)
p(79) = p(2)*p(32) - p(58)
p(80) = p(6)*p(12) - p(64) - p(63) - p(63)
p(81) = p(1)*p(45) - p(80)
p(82) = p(14)*p(6)
p(83) = p(2)*p(33) - p(61) - p(60)
p(84) = p(2)*p(34) - p(62)
p(85) = p(6)*p(18) - p(63)
p(86) = p(2)*p(36) - p(64) - p(80)
p(87) = p(8)*p(13) - p(63)
p(88) = p(14)*p(8)
p(89) = p(1)*p(39) - p(65)
p(90) = p(3)*p(21) - p(66)
p(91) = p(1)*p(41) - p(67)
p(92) = p(2)*p(49) - p(91)
p(93) = p(1)*p(43) - p(74)
p(94) = p(8)*p(21) - p(93)
p(95) = p(1)*p(46) - p(83)
p(96) = p(7)*p(22) - p(95)
p(97) = p(2)*p(45) - p(82)
p(98) = p(2)*p(46) - p(85)
p(99) = p(1)*p(50) - p(98)
p(100) = p(6)*p(22) - p(88)
p(101) = p(1)*p(49) - p(90)
p(102) = p(2)*p(50) - p(100)
p(103) = p(9)*p(14)
p(104) = p(9)*p(12)
p(105) = p(9)*p(13)
p(106) = p(14)*p(15)
p(107) = p(9)*p(20)
p(108) = p(14)*p(10)
p(109) = p(14)*p(11)
p(110) = p(9)*p(16)
p(111) = p(9)*p(10)
p(112) = p(9)*p(11)
p(113) = p(9)*p(17)
p(114) = p(3)*p(24) - p(104)
p(115) = p(7)*p(24) - p(105)
p(116) = p(3)*p(31) - p(104) - p(115)
p(117) = p(6)*p(39) - p(114)
p(118) = p(3)*p(32) - p(105)
p(119) = p(14)*p(21)
p(120) = p(9)*p(18)
p(121) = p(9)*p(19)
p(122) = p(4)*p(24) - p(103) - p(103) - p(103)
p(123) = p(1)*p(60) - p(107) - p(122)
p(124) = p(13)*p(16) - p(119)
p(125) = p(2)*p(56) - p(106) - p(124)
p(126) = p(4)*p(32) - p(119)
p(127) = p(1)*p(62) - p(107) - p(125)
p(128) = p(14)*p(16)
p(129) = p(14)*p(17)
p(130) = p(6)*p(24) - p(108)
p(131) = p(3)*p(45) - p(130)
p(132) = p(14)*p(12)
p(133) = p(14)*p(13)
p(134) = p(9)*p(22)
p(135) = p(8)*p(24) - p(109)
p(136) = p(13)*p(18) - p(128)
p(137) = p(6)*p(34) - p(109)
p(138) = p(14)*p(18)
p(139) = p(14)*p(19)
p(140) = p(9)*p(9)
p(141) = p(9)*p(15)
p(142) = p(9)*p(21)
p(143) = p(3)*p(41) - p(110)
p(144) = p(4)*p(39) - p(112)
p(145) = p(1)*p(68) - p(111) - p(144)
p(146) = p(1)*p(69) - p(112)
p(147) = p(1)*p(70) - p(113) - p(145)
p(148) = p(11)*p(21) - p(110)
p(149) = p(1)*p(72) - p(116) - p(115)
p(150) = p(1)*p(73) - p(118)
p(151) = p(3)*p(43) - p(120)
p(152) = p(15)*p(18) - p(121) - p(151)
p(153) = p(2)*p(69) - p(117)
p(154) = p(1)*p(75) - p(121) - p(152)
p(155) = p(7)*p(34) - p(120)
p(156) = p(6)*p(41) - p(119)
p(157) = p(1)*p(78) - p(125) - p(123)
p(158) = p(2)*p(73) - p(119)
p(159) = p(1)*p(80) - p(131) - p(130) - p(130)
p(160) = p(1)*p(81) - p(131)
p(161) = p(14)*p(14)
p(162) = p(3)*p(46) - p(134)
p(163) = p(2)*p(75) - p(126) - p(123)
p(164) = p(1)*p(84) - p(134)
p(165) = p(6)*p(43) - p(128)
p(166) = p(1)*p(86) - p(137) - p(135)
p(167) = p(8)*p(32) - p(128)
p(168) = p(4)*p(45) - p(133)
p(169) = p(2)*p(80) - p(132) - p(168)
p(170) = p(2)*p(81) - p(133)
p(171) = p(14)*p(20)
p(172) = p(2)*p(83) - p(136) - p(135)
p(173) = p(2)*p(84) - p(137)
p(174) = p(6)*p(46) - p(138)
p(175) = p(2)*p(86) - p(139) - p(169)
p(176) = p(13)*p(22) - p(138)
p(177) = p(14)*p(22)
p(178) = p(3)*p(39) - p(141)
p(179) = p(1)*p(89) - p(141) - p(178) - p(178)
p(180) = p(3)*p(49) - p(142)
p(181) = p(1)*p(91) - p(143)
p(182) = p(2)*p(101) - p(181)
p(183) = p(1)*p(93) - p(151)
p(184) = p(8)*p(49) - p(183)
p(185) = p(1)*p(95) - p(162)
p(186) = p(21)*p(22) - p(185)
p(187) = p(6)*p(45) - p(171)
p(188) = p(1)*p(98) - p(172)
p(189) = p(7)*p(50) - p(188)
p(190) = p(2)*p(97) - p(171) - p(187) - p(187)
p(191) = p(2)*p(98) - p(174)
p(192) = p(1)*p(102) - p(191)
p(193) = p(6)*p(50) - p(177)
p(194) = p(1)*p(101) - p(180)
p(195) = p(2)*p(102) - p(193)
p(196) = p(9)*p(26)
p(197) = p(9)*p(38)
p(198) = p(9)*p(24)
p(199) = p(9)*p(31)
p(200) = p(9)*p(25)
p(201) = p(9)*p(32)
p(202) = p(14)*p(39)
p(203) = p(14)*p(40)
p(204) = p(9)*p(35)
p(205) = p(9)*p(36)
p(206) = p(9)*p(37)
p(207) = p(14)*p(28)
p(208) = p(14)*p(29)
p(209) = p(14)*p(30)
p(210) = p(9)*p(45)
p(211) = p(14)*p(24)
p(212) = p(14)*p(25)
p(213) = p(9)*p(48)
p(214) = p(14)*p(33)
p(215) = p(14)*p(34)
p(216) = p(9)*p(41)
p(217) = p(9)*p(28)
p(218) = p(9)*p(23)
p(219) = p(9)*p(29)
p(220) = p(9)*p(30)
p(221) = p(9)*p(42)
p(222) = p(1)*p(114) - p(198)
p(223) = p(21)*p(24) - p(201)
p(224) = p(3)*p(72) - p(199) - p(223)
p(225) = p(1)*p(116) - p(199) - p(224)
p(226) = p(13)*p(39) - p(198)
p(227) = p(3)*p(73) - p(201)
p(228) = p(14)*p(49)
p(229) = p(9)*p(43)
p(230) = p(9)*p(33)
p(231) = p(9)*p(34)
p(232) = p(9)*p(44)
p(233) = p(16)*p(24) - p(196)
p(234) = p(2)*p(114) - p(202)
p(235) = p(1)*p(123) - p(205) - p(234)
p(236) = p(13)*p(41) - p(228)
p(237) = p(2)*p(116) - p(203) - p(236)
p(238) = p(16)*p(32) - p(228)
p(239) = p(6)*p(69) - p(202)
p(240) = p(4)*p(73) - p(228)
p(241) = p(11)*p(32) - p(196)
p(242) = p(14)*p(41)
p(243) = p(14)*p(42)
p(244) = p(1)*p(130) - p(210)
p(245) = p(4)*p(62) - p(213) - p(209)
p(246) = p(3)*p(81) - p(210)
p(247) = p(14)*p(31)
p(248) = p(14)*p(32)
p(249) = p(14)*p(26)
p(250) = p(9)*p(46)
p(251) = p(9)*p(47)
p(252) = p(18)*p(24) - p(197)
p(253) = p(1)*p(135) - p(213) - p(252)
p(254) = p(13)*p(43) - p(242)
p(255) = p(2)*p(125) - p(209) - p(245)
p(256) = p(18)*p(32) - p(242)
p(257) = p(13)*p(34) - p(197)
p(258) = p(14)*p(43)
p(259) = p(14)*p(44)
p(260) = p(2)*p(130) - p(211)
p(261) = p(4)*p(81) - p(248)
p(262) = p(11)*p(45) - p(211)
p(263) = p(14)*p(35)
p(264) = p(14)*p(36)
p(265) = p(14)*p(37)
p(266) = p(9)*p(50)
p(267) = p(22)*p(24) - p(215)
p(268) = p(13)*p(46) - p(258)
p(269) = p(6)*p(84) - p(215)
p(270) = p(14)*p(46)
p(271) = p(14)*p(47)
p(272) = p(9)*p(27)
p(273) = p(9)*p(39)
p(274) = p(9)*p(40)
p(275) = p(9)*p(49)
p(276) = p(3)*p(91) - p(216)
p(277) = p(16)*p(39) - p(218)
p(278) = p(1)*p(144) - p(217) - p(277)
p(279) = p(2)*p(178) - p(278)
p(280) = p(1)*p(145) - p(219) - p(278)
p(281) = p(7)*p(69) - p(218)
p(282) = p(1)*p(147) - p(221) - p(280)
p(283) = p(11)*p(49) - p(216)
p(284) = p(1)*p(149) - p(224) - p(223)
p(285) = p(1)*p(150) - p(227)
p(286) = p(3)*p(93) - p(229)
p(287) = p(18)*p(39) - p(231)
p(288) = p(1)*p(152) - p(230) - p(287)
p(289) = p(1)*p(153) - p(231)
p(290) = p(1)*p(154) - p(232) - p(288)
p(291) = p(21)*p(34) - p(229)
p(292) = p(6)*p(91) - p(228)
p(293) = p(1)*p(157) - p(237) - p(235)
p(294) = p(2)*p(150) - p(228)
p(295) = p(1)*p(159) - p(245) - p(244)
p(296) = p(1)*p(160) - p(246)
p(297) = p(3)*p(95) - p(250)
p(298) = p(2)*p(152) - p(238) - p(234)
p(299) = p(2)*p(153) - p(239)
p(300) = p(1)*p(163) - p(251) - p(298)
p(301) = p(7)*p(84) - p(250)
p(302) = p(6)*p(93) - p(242)
p(303) = p(1)*p(166) - p(255) - p(253)
p(304) = p(8)*p(73) - p(242)
p(305) = p(16)*p(45) - p(248)
p(306) = p(1)*p(169) - p(262) - p(260)
p(307) = p(2)*p(160) - p(248)
p(308) = p(14)*p(38)
p(309) = p(6)*p(80) - p(264) - p(263)
p(310) = p(1)*p(187) - p(309)
p(311) = p(14)*p(45)
p(312) = p(3)*p(98) - p(266)
p(313) = p(2)*p(163) - p(256) - p(253)
p(314) = p(1)*p(173) - p(266)
p(315) = p(6)*p(95) - p(258)
p(316) = p(1)*p(175) - p(269) - p(267)
p(317) = p(22)*p(32) - p(258)
p(318) = p(18)*p(45) - p(249)
p(319) = p(2)*p(169) - p(264) - p(309)
p(320) = p(8)*p(81) - p(249)
p(321) = p(14)*p(48)
p(322) = p(2)*p(172) - p(268) - p(267)
p(323) = p(2)*p(173) - p(269)
p(324) = p(6)*p(98) - p(270)
p(325) = p(2)*p(175) - p(271) - p(319)
p(326) = p(13)*p(50) - p(270)
p(327) = p(14)*p(50)
p(328) = p(1)*p(178) - p(273)
p(329) = p(21)*p(39) - p(272)
p(330) = p(3)*p(101) - p(275)
p(331) = p(1)*p(181) - p(276)
p(332) = p(2)*p(194) - p(331)
p(333) = p(1)*p(183) - p(286)
p(334) = p(8)*p(101) - p(333)
p(335) = p(1)*p(185) - p(297)
p(336) = p(22)*p(49) - p(335)
p(337) = p(1)*p(188) - p(312)
p(338) = p(21)*p(50) - p(337)
p(339) = p(2)*p(187) - p(311)
p(340) = p(1)*p(191) - p(322)
p(341) = p(7)*p(102) - p(340)
p(342) = p(22)*p(45) - p(308)
p(343) = p(2)*p(191) - p(324)
p(344) = p(1)*p(195) - p(343)
p(345) = p(6)*p(102) - p(327)
p(346) = p(1)*p(194) - p(330)
p(347) = p(2)*p(195) - p(345)
p(348) = p(9)*p(52)
p(349) = p(9)*p(58)
p(350) = p(9)*p(63)
p(351) = p(9)*p(64)
p(352) = p(9)*p(82)
p(353) = p(9)*p(88)
p(354) = p(9)*p(55)
p(355) = p(9)*p(72)
p(356) = p(9)*p(51)
p(357) = p(9)*p(56)
p(358) = p(9)*p(57)
p(359) = p(9)*p(73)
p(360) = p(14)*p(89)
p(361) = p(14)*p(90)
p(362) = p(9)*p(77)
p(363) = p(9)*p(60)
p(364) = p(9)*p(78)
p(365) = p(9)*p(61)
p(366) = p(9)*p(62)
p(367) = p(9)*p(79)
p(368) = p(14)*p(67)
p(369) = p(14)*p(68)
p(370) = p(14)*p(69)
p(371) = p(14)*p(70)
p(372) = p(14)*p(71)
p(373) = p(9)*p(80)
p(374) = p(9)*p(81)
p(375) = p(14)*p(55)
p(376) = p(14)*p(56)
p(377) = p(14)*p(57)
p(378) = p(14)*p(52)
p(379) = p(9)*p(85)
p(380) = p(9)*p(86)
p(381) = p(9)*p(87)
p(382) = p(14)*p(74)
p(383) = p(14)*p(75)
p(384) = p(14)*p(76)
p(385) = p(9)*p(97)
p(386) = p(14)*p(60)
p(387) = p(14)*p(61)
p(388) = p(14)*p(62)
p(389) = p(9)*p(100)
p(390) = p(14)*p(83)
p(391) = p(14)*p(84)
p(392) = p(9)*p(91)
p(393) = p(9)*p(67)
p(394) = p(9)*p(53)
p(395) = p(9)*p(68)
p(396) = p(9)*p(54)
p(397) = p(9)*p(69)
p(398) = p(9)*p(70)
p(399) = p(9)*p(71)
p(400) = p(9)*p(92)
p(401) = p(3)*p(114) - p(354)
p(402) = p(7)*p(114) - p(356)
p(403) = p(24)*p(49) - p(359)
p(404) = p(3)*p(149) - p(355) - p(403)
p(405) = p(1)*p(224) - p(355) - p(404)
p(406) = p(6)*p(178) - p(401)
p(407) = p(32)*p(39) - p(356)
p(408) = p(3)*p(150) - p(359)
p(409) = p(14)*p(101)
p(410) = p(9)*p(93)
p(411) = p(9)*p(74)
p(412) = p(9)*p(59)
p(413) = p(9)*p(75)
p(414) = p(9)*p(76)
p(415) = p(9)*p(94)
p(416) = p(24)*p(41) - p(349)
p(417) = p(4)*p(114) - p(348)
p(418) = p(1)*p(234) - p(363) - p(417)
p(419) = p(1)*p(235) - p(364) - p(418)
p(420) = p(13)*p(91) - p(409)
p(421) = p(2)*p(224) - p(361) - p(420)
p(422) = p(32)*p(41) - p(409)
p(423) = p(1)*p(237) - p(364) - p(421)
p(424) = p(16)*p(73) - p(409)
p(425) = p(13)*p(69) - p(348)
p(426) = p(4)*p(150) - p(409)
p(427) = p(11)*p(73) - p(349)
p(428) = p(14)*p(91)
p(429) = p(14)*p(92)
p(430) = p(3)*p(130) - p(373)
p(431) = p(7)*p(130) - p(374)
p(432) = p(3)*p(159) - p(373) - p(431)
p(433) = p(39)*p(45) - p(430)
p(434) = p(3)*p(160) - p(374)
p(435) = p(14)*p(72)
p(436) = p(14)*p(73)
p(437) = p(14)*p(58)
p(438) = p(9)*p(95)
p(439) = p(9)*p(83)
p(440) = p(9)*p(84)
p(441) = p(9)*p(96)
p(442) = p(24)*p(43) - p(350)
p(443) = p(8)*p(114) - p(370)
p(444) = p(1)*p(253) - p(380) - p(443)
p(445) = p(13)*p(93) - p(428)
p(446) = p(2)*p(237) - p(372) - p(432)
p(447) = p(32)*p(43) - p(428)
p(448) = p(6)*p(153) - p(370)
p(449) = p(18)*p(73) - p(428)
p(450) = p(32)*p(34) - p(350)
p(451) = p(14)*p(93)
p(452) = p(14)*p(94)
p(453) = p(4)*p(130) - p(352)
p(454) = p(1)*p(260) - p(385) - p(453)
p(455) = p(16)*p(81) - p(436)
p(456) = p(2)*p(245) - p(376) - p(455)
p(457) = p(4)*p(160) - p(436)
p(458) = p(11)*p(81) - p(352)
p(459) = p(14)*p(77)
p(460) = p(14)*p(78)
p(461) = p(14)*p(79)
p(462) = p(14)*p(63)
p(463) = p(14)*p(64)
p(464) = p(6)*p(130) - p(386)
p(465) = p(3)*p(187) - p(464)
p(466) = p(14)*p(80)
p(467) = p(14)*p(81)
p(468) = p(9)*p(98)
p(469) = p(9)*p(99)
p(470) = p(24)*p(46) - p(353)
p(471) = p(1)*p(267) - p(389) - p(470)
p(472) = p(13)*p(95) - p(451)
p(473) = p(2)*p(255) - p(384) - p(456)
p(474) = p(32)*p(46) - p(451)
p(475) = p(13)*p(84) - p(353)
p(476) = p(14)*p(95)
p(477) = p(14)*p(96)
p(478) = p(8)*p(130) - p(378)
p(479) = p(18)*p(81) - p(437)
p(480) = p(34)*p(45) - p(378)
p(481) = p(14)*p(85)
p(482) = p(14)*p(86)
p(483) = p(14)*p(87)
p(484) = p(9)*p(102)
p(485) = p(24)*p(50) - p(391)
p(486) = p(13)*p(98) - p(476)
p(487) = p(6)*p(173) - p(391)
p(488) = p(14)*p(98)
p(489) = p(14)*p(99)
p(490) = p(9)*p(65)
p(491) = p(9)*p(66)
p(492) = p(9)*p(89)
p(493) = p(9)*p(90)
p(494) = p(9)*p(101)
p(495) = p(3)*p(181) - p(392)
p(496) = p(39)*p(41) - p(394)
p(497) = p(4)*p(178) - p(397)
p(498) = p(1)*p(278) - p(395) - p(497)
p(499) = p(1)*p(279) - p(397)
p(500) = p(1)*p(280) - p(398) - p(498)
p(501) = p(21)*p(69) - p(394)
p(502) = p(1)*p(282) - p(400) - p(500)
p(503) = p(11)*p(101) - p(392)
p(504) = p(1)*p(284) - p(404) - p(403)
p(505) = p(1)*p(285) - p(408)
p(506) = p(3)*p(183) - p(410)
p(507) = p(39)*p(43) - p(412)
p(508) = p(1)*p(287) - p(411) - p(507)
p(509) = p(2)*p(279) - p(406)
p(510) = p(1)*p(288) - p(413) - p(508)
p(511) = p(7)*p(153) - p(412)
p(512) = p(1)*p(290) - p(415) - p(510)
p(513) = p(34)*p(49) - p(410)
p(514) = p(6)*p(181) - p(409)
p(515) = p(1)*p(293) - p(421) - p(419)
p(516) = p(2)*p(285) - p(409)
p(517) = p(1)*p(295) - p(432) - p(431)
p(518) = p(1)*p(296) - p(434)
p(519) = p(3)*p(185) - p(438)
p(520) = p(39)*p(46) - p(440)
p(521) = p(1)*p(298) - p(439) - p(520)
p(522) = p(1)*p(299) - p(440)
p(523) = p(1)*p(300) - p(441) - p(521)
p(524) = p(21)*p(84) - p(438)
p(525) = p(6)*p(183) - p(428)
p(526) = p(1)*p(303) - p(446) - p(444)
p(527) = p(8)*p(150) - p(428)
p(528) = p(41)*p(45) - p(436)
p(529) = p(1)*p(306) - p(456) - p(454)
p(530) = p(2)*p(296) - p(436)
p(531) = p(4)*p(169) - p(388) - p(478)
p(532) = p(1)*p(310) - p(465)
p(533) = p(14)*p(82)
p(534) = p(3)*p(188) - p(468)
p(535) = p(2)*p(298) - p(447) - p(443)
p(536) = p(2)*p(299) - p(448)
p(537) = p(1)*p(313) - p(469) - p(535)
p(538) = p(7)*p(173) - p(468)
p(539) = p(6)*p(185) - p(451)
p(540) = p(1)*p(316) - p(473) - p(471)
p(541) = p(22)*p(73) - p(451)
p(542) = p(43)*p(45) - p(437)
p(543) = p(1)*p(319) - p(480) - p(478)
p(544) = p(8)*p(160) - p(437)
p(545) = p(14)*p(88)
p(546) = p(4)*p(187) - p(467)
p(547) = p(2)*p(309) - p(466) - p(546)
p(548) = p(2)*p(310) - p(467)
p(549) = p(14)*p(97)
p(550) = p(3)*p(191) - p(484)
p(551) = p(2)*p(313) - p(474) - p(471)
p(552) = p(1)*p(323) - p(484)
p(553) = p(6)*p(188) - p(476)
p(554) = p(1)*p(325) - p(487) - p(485)
p(555) = p(32)*p(50) - p(476)
p(556) = p(45)*p(46) - p(462)
p(557) = p(2)*p(319) - p(482) - p(547)
p(558) = p(22)*p(81) - p(462)
p(559) = p(14)*p(100)
p(560) = p(2)*p(322) - p(486) - p(485)
p(561) = p(2)*p(323) - p(487)
p(562) = p(6)*p(191) - p(488)
p(563) = p(2)*p(325) - p(489) - p(557)
p(564) = p(13)*p(102) - p(488)
p(565) = p(14)*p(102)
p(566) = p(3)*p(178) - p(492)
p(567) = p(7)*p(178) - p(490)
p(568) = p(39)*p(49) - p(491)
p(569) = p(3)*p(194) - p(494)
p(570) = p(1)*p(331) - p(495)
p(571) = p(2)*p(346) - p(570)
p(572) = p(1)*p(333) - p(506)
p(573) = p(8)*p(194) - p(572)
p(574) = p(1)*p(335) - p(519)
p(575) = p(22)*p(101) - p(574)
p(576) = p(1)*p(337) - p(534)
p(577) = p(49)*p(50) - p(576)
p(578) = p(6)*p(187) - p(549)
p(579) = p(1)*p(340) - p(550)
p(580) = p(21)*p(102) - p(579)
p(581) = p(8)*p(187) - p(533)
p(582) = p(1)*p(343) - p(560)
p(583) = p(7)*p(195) - p(582)
p(584) = p(45)*p(50) - p(545)
p(585) = p(2)*p(343) - p(562)
p(586) = p(1)*p(347) - p(585)
p(587) = p(6)*p(195) - p(565)
p(588) = p(1)*p(346) - p(569)
p(589) = p(2)*p(347) - p(587)
p(590) = p(9)*p(103)
p(591) = p(9)*p(106)
p(592) = p(9)*p(119)
p(593) = p(9)*p(128)
p(594) = p(9)*p(108)
p(595) = p(9)*p(109)
p(596) = p(9)*p(129)
p(597) = p(9)*p(132)
p(598) = p(9)*p(133)
p(599) = p(9)*p(161)
p(600) = p(9)*p(138)
p(601) = p(9)*p(139)
p(602) = p(9)*p(171)
p(603) = p(9)*p(177)
p(604) = p(9)*p(114)
p(605) = p(9)*p(115)
p(606) = p(9)*p(149)
p(607) = p(9)*p(104)
p(608) = p(9)*p(116)
p(609) = p(9)*p(105)
p(610) = p(9)*p(117)
p(611) = p(9)*p(118)
p(612) = p(9)*p(150)
p(613) = p(14)*p(178)
p(614) = p(14)*p(179)
p(615) = p(14)*p(180)
p(616) = p(9)*p(156)
p(617) = p(9)*p(122)
p(618) = p(9)*p(123)
p(619) = p(9)*p(157)
p(620) = p(9)*p(124)
p(621) = p(9)*p(107)
p(622) = p(9)*p(125)
p(623) = p(9)*p(126)
p(624) = p(9)*p(127)
p(625) = p(9)*p(158)
p(626) = p(14)*p(143)
p(627) = p(14)*p(144)
p(628) = p(14)*p(145)
p(629) = p(14)*p(146)
p(630) = p(14)*p(147)
p(631) = p(14)*p(148)
p(632) = p(9)*p(130)
p(633) = p(9)*p(159)
p(634) = p(9)*p(131)
p(635) = p(9)*p(160)
p(636) = p(14)*p(114)
p(637) = p(14)*p(115)
p(638) = p(14)*p(116)
p(639) = p(14)*p(117)
p(640) = p(14)*p(118)
p(641) = p(14)*p(106)
p(642) = p(9)*p(165)
p(643) = p(9)*p(135)
p(644) = p(9)*p(166)
p(645) = p(9)*p(136)
p(646) = p(9)*p(137)
p(647) = p(9)*p(167)
p(648) = p(14)*p(151)
p(649) = p(14)*p(152)
p(650) = p(14)*p(153)
p(651) = p(14)*p(154)
p(652) = p(14)*p(155)
p(653) = p(9)*p(168)
p(654) = p(9)*p(169)
p(655) = p(9)*p(170)
p(656) = p(14)*p(122)
p(657) = p(14)*p(123)
p(658) = p(14)*p(124)
p(659) = p(14)*p(125)
p(660) = p(14)*p(126)
p(661) = p(14)*p(127)
p(662) = p(14)*p(108)
p(663) = p(14)*p(109)
p(664) = p(9)*p(187)
p(665) = p(14)*p(130)
p(666) = p(14)*p(131)
p(667) = p(9)*p(174)
p(668) = p(9)*p(175)
p(669) = p(9)*p(176)
p(670) = p(14)*p(162)
p(671) = p(14)*p(163)
p(672) = p(14)*p(164)
p(673) = p(9)*p(190)
p(674) = p(14)*p(135)
p(675) = p(14)*p(136)
p(676) = p(14)*p(137)
p(677) = p(9)*p(193)
p(678) = p(14)*p(172)
p(679) = p(14)*p(173)
p(680) = p(9)*p(181)
p(681) = p(9)*p(143)
p(682) = p(9)*p(110)
p(683) = p(9)*p(144)
p(684) = p(9)*p(111)
p(685) = p(9)*p(112)
p(686) = p(9)*p(145)
p(687) = p(9)*p(113)
p(688) = p(9)*p(146)
p(689) = p(9)*p(147)
p(690) = p(9)*p(148)
p(691) = p(9)*p(182)
p(692) = p(1)*p(401) - p(604)
p(693) = p(21)*p(114) - p(609)
p(694) = p(24)*p(101) - p(612)
p(695) = p(3)*p(284) - p(606) - p(694)
p(696) = p(1)*p(404) - p(606) - p(695)
p(697) = p(1)*p(405) - p(608) - p(696)
p(698) = p(13)*p(178) - p(604)
p(699) = p(39)*p(73) - p(609)
p(700) = p(3)*p(285) - p(612)
p(701) = p(14)*p(194)
p(702) = p(9)*p(183)
p(703) = p(9)*p(151)
p(704) = p(9)*p(120)
p(705) = p(9)*p(152)
p(706) = p(9)*p(121)
p(707) = p(9)*p(153)
p(708) = p(9)*p(154)
p(709) = p(9)*p(155)
p(710) = p(9)*p(184)
p(711) = p(24)*p(91) - p(592)
p(712) = p(1)*p(416) - p(616) - p(711)
p(713) = p(2)*p(401) - p(613)
p(714) = p(1)*p(418) - p(618) - p(713)
p(715) = p(1)*p(419) - p(619) - p(714)
p(716) = p(13)*p(181) - p(701)
p(717) = p(2)*p(404) - p(615) - p(716)
p(718) = p(32)*p(91) - p(701)
p(719) = p(1)*p(421) - p(619) - p(717)
p(720) = p(41)*p(73) - p(701)
p(721) = p(6)*p(279) - p(613)
p(722) = p(16)*p(150) - p(701)
p(723) = p(1)*p(425) - p(624) - p(721)
p(724) = p(4)*p(285) - p(701)
p(725) = p(11)*p(150) - p(592)
p(726) = p(14)*p(181)
p(727) = p(14)*p(182)
p(728) = p(1)*p(430) - p(632)
p(729) = p(21)*p(130) - p(635)
p(730) = p(3)*p(295) - p(633) - p(729)
p(731) = p(1)*p(432) - p(633) - p(730)
p(732) = p(39)*p(81) - p(632)
p(733) = p(3)*p(296) - p(635)
p(734) = p(14)*p(149)
p(735) = p(14)*p(150)
p(736) = p(14)*p(119)
p(737) = p(9)*p(185)
p(738) = p(9)*p(162)
p(739) = p(9)*p(134)
p(740) = p(9)*p(163)
p(741) = p(9)*p(164)
p(742) = p(9)*p(186)
p(743) = p(24)*p(93) - p(593)
p(744) = p(18)*p(114) - p(595)
p(745) = p(1)*p(443) - p(643) - p(744)
p(746) = p(1)*p(444) - p(644) - p(745)
p(747) = p(13)*p(183) - p(726)
p(748) = p(2)*p(421) - p(631) - p(730)
p(749) = p(32)*p(93) - p(726)
p(750) = p(1)*p(446) - p(644) - p(748)
p(751) = p(43)*p(73) - p(726)
p(752) = p(13)*p(153) - p(595)
p(753) = p(18)*p(150) - p(726)
p(754) = p(34)*p(73) - p(593)
p(755) = p(14)*p(183)
p(756) = p(14)*p(184)
p(757) = p(16)*p(130) - p(598)
p(758) = p(2)*p(430) - p(636)
p(759) = p(1)*p(454) - p(654) - p(758)
p(760) = p(41)*p(81) - p(735)
p(761) = p(2)*p(432) - p(638) - p(760)
p(762) = p(16)*p(160) - p(735)
p(763) = p(45)*p(69) - p(636)
p(764) = p(4)*p(296) - p(735)
p(765) = p(11)*p(160) - p(598)
p(766) = p(14)*p(156)
p(767) = p(14)*p(157)
p(768) = p(14)*p(158)
p(769) = p(14)*p(128)
p(770) = p(14)*p(129)
p(771) = p(1)*p(464) - p(664)
p(772) = p(4)*p(262) - p(673) - p(661)
p(773) = p(3)*p(310) - p(664)
p(774) = p(14)*p(159)
p(775) = p(14)*p(160)
p(776) = p(14)*p(132)
p(777) = p(14)*p(133)
p(778) = p(9)*p(188)
p(779) = p(9)*p(172)
p(780) = p(9)*p(173)
p(781) = p(9)*p(189)
p(782) = p(24)*p(95) - p(600)
p(783) = p(22)*p(114) - p(650)
p(784) = p(1)*p(471) - p(668) - p(783)
p(785) = p(13)*p(185) - p(755)
p(786) = p(2)*p(446) - p(652) - p(761)
p(787) = p(32)*p(95) - p(755)
p(788) = p(6)*p(299) - p(650)
p(789) = p(46)*p(73) - p(755)
p(790) = p(32)*p(84) - p(600)
p(791) = p(14)*p(185)
p(792) = p(14)*p(186)
p(793) = p(2)*p(453) - p(656) - p(771)
p(794) = p(1)*p(478) - p(673) - p(793)
p(795) = p(43)*p(81) - p(736)
p(796) = p(2)*p(456) - p(659) - p(772)
p(797) = p(18)*p(160) - p(736)
p(798) = p(1)*p(480) - p(673) - p(796)
p(799) = p(14)*p(165)
p(800) = p(14)*p(166)
p(801) = p(14)*p(167)
p(802) = p(14)*p(138)
p(803) = p(14)*p(139)
p(804) = p(2)*p(464) - p(665)
p(805) = p(4)*p(310) - p(775)
p(806) = p(11)*p(187) - p(665)
p(807) = p(14)*p(168)
p(808) = p(14)*p(169)
p(809) = p(14)*p(170)
p(810) = p(9)*p(191)
p(811) = p(9)*p(192)
p(812) = p(24)*p(98) - p(603)
p(813) = p(1)*p(485) - p(677) - p(812)
p(814) = p(13)*p(188) - p(791)
p(815) = p(2)*p(473) - p(672) - p(796)
p(816) = p(32)*p(98) - p(791)
p(817) = p(13)*p(173) - p(603)
p(818) = p(14)*p(188)
p(819) = p(14)*p(189)
p(820) = p(22)*p(130) - p(663)
p(821) = p(46)*p(81) - p(769)
p(822) = p(45)*p(84) - p(663)
p(823) = p(14)*p(174)
p(824) = p(14)*p(175)
p(825) = p(14)*p(176)
p(826) = p(9)*p(195)
p(827) = p(24)*p(102) - p(679)
p(828) = p(13)*p(191) - p(818)
p(829) = p(6)*p(323) - p(679)
p(830) = p(14)*p(191)
p(831) = p(14)*p(192)
p(832) = p(9)*p(140)
p(833) = p(9)*p(141)
p(834) = p(9)*p(178)
p(835) = p(9)*p(142)
p(836) = p(9)*p(179)
p(837) = p(9)*p(180)
p(838) = p(9)*p(194)
p(839) = p(3)*p(331) - p(680)
p(840) = p(39)*p(91) - p(682)
p(841) = p(16)*p(178) - p(685)
p(842) = p(1)*p(497) - p(683) - p(841)
p(843) = p(2)*p(566) - p(842)
p(844) = p(1)*p(498) - p(686) - p(842)
p(845) = p(7)*p(279) - p(685)
p(846) = p(1)*p(500) - p(689) - p(844)
p(847) = p(49)*p(69) - p(682)
p(848) = p(1)*p(502) - p(691) - p(846)
p(849) = p(11)*p(194) - p(680)
p(850) = p(1)*p(504) - p(695) - p(694)
p(851) = p(1)*p(505) - p(700)
p(852) = p(3)*p(333) - p(702)
p(853) = p(39)*p(93) - p(704)
p(854) = p(18)*p(178) - p(707)
p(855) = p(1)*p(508) - p(705) - p(854)
p(856) = p(1)*p(509) - p(707)
p(857) = p(1)*p(510) - p(708) - p(855)
p(858) = p(21)*p(153) - p(704)
p(859) = p(1)*p(512) - p(710) - p(857)
p(860) = p(34)*p(101) - p(702)
p(861) = p(6)*p(331) - p(701)
p(862) = p(1)*p(515) - p(717) - p(715)
p(863) = p(2)*p(505) - p(701)
p(864) = p(1)*p(517) - p(730) - p(729)
p(865) = p(1)*p(518) - p(733)
p(866) = p(3)*p(335) - p(737)
p(867) = p(39)*p(95) - p(739)
p(868) = p(1)*p(520) - p(738) - p(867)
p(869) = p(2)*p(509) - p(721)
p(870) = p(1)*p(521) - p(740) - p(868)
p(871) = p(7)*p(299) - p(739)
p(872) = p(1)*p(523) - p(742) - p(870)
p(873) = p(49)*p(84) - p(737)
p(874) = p(6)*p(333) - p(726)
p(875) = p(1)*p(526) - p(748) - p(746)
p(876) = p(8)*p(285) - p(726)
p(877) = p(45)*p(91) - p(735)
p(878) = p(1)*p(529) - p(761) - p(759)
p(879) = p(2)*p(518) - p(735)
p(880) = p(1)*p(531) - p(772) - p(771)
p(881) = p(1)*p(532) - p(773)
p(882) = p(14)*p(161)
p(883) = p(3)*p(337) - p(778)
p(884) = p(39)*p(98) - p(780)
p(885) = p(1)*p(535) - p(779) - p(884)
p(886) = p(1)*p(536) - p(780)
p(887) = p(1)*p(537) - p(781) - p(885)
p(888) = p(21)*p(173) - p(778)
p(889) = p(6)*p(335) - p(755)
p(890) = p(1)*p(540) - p(786) - p(784)
p(891) = p(22)*p(150) - p(755)
p(892) = p(45)*p(93) - p(736)
p(893) = p(1)*p(543) - p(796) - p(794)
p(894) = p(8)*p(296) - p(736)
p(895) = p(16)*p(187) - p(775)
p(896) = p(1)*p(547) - p(806) - p(804)
p(897) = p(2)*p(532) - p(775)
p(898) = p(14)*p(171)
p(899) = p(6)*p(309) - p(808) - p(807)
p(900) = p(1)*p(578) - p(899)
p(901) = p(14)*p(187)
p(902) = p(3)*p(340) - p(810)
p(903) = p(2)*p(535) - p(787) - p(783)
p(904) = p(2)*p(536) - p(788)
p(905) = p(1)*p(551) - p(811) - p(903)
p(906) = p(7)*p(323) - p(810)
p(907) = p(6)*p(337) - p(791)
p(908) = p(1)*p(554) - p(815) - p(813)
p(909) = p(50)*p(73) - p(791)
p(910) = p(45)*p(95) - p(769)
p(911) = p(1)*p(557) - p(822) - p(820)
p(912) = p(22)*p(160) - p(769)
p(913) = p(14)*p(177)
p(914) = p(18)*p(187) - p(777)
p(915) = p(2)*p(547) - p(808) - p(899)
p(916) = p(8)*p(310) - p(777)
p(917) = p(14)*p(190)
p(918) = p(3)*p(343) - p(826)
p(919) = p(2)*p(551) - p(816) - p(813)
p(920) = p(1)*p(561) - p(826)
p(921) = p(6)*p(340) - p(818)
p(922) = p(1)*p(563) - p(829) - p(827)
p(923) = p(32)*p(102) - p(818)
p(924) = p(45)*p(98) - p(802)
p(925) = p(2)*p(557) - p(824) - p(915)
p(926) = p(50)*p(81) - p(802)
p(927) = p(14)*p(193)
p(928) = p(2)*p(560) - p(828) - p(827)
p(929) = p(2)*p(561) - p(829)
p(930) = p(6)*p(343) - p(830)
p(931) = p(2)*p(563) - p(831) - p(925)
p(932) = p(13)*p(195) - p(830)
p(933) = p(14)*p(195)
p(934) = p(1)*p(566) - p(834)
p(935) = p(1)*p(567) - p(836) - p(934)
p(936) = p(39)*p(101) - p(835)
p(937) = p(3)*p(346) - p(838)
p(938) = p(1)*p(570) - p(839)
p(939) = p(2)*p(588) - p(938)
p(940) = p(1)*p(572) - p(852)
p(941) = p(8)*p(346) - p(940)
p(942) = p(1)*p(574) - p(866)
p(943) = p(22)*p(194) - p(942)
p(944) = p(1)*p(576) - p(883)
p(945) = p(50)*p(101) - p(944)
p(946) = p(1)*p(579) - p(902)
p(947) = p(49)*p(102) - p(946)
p(948) = p(2)*p(578) - p(901)
p(949) = p(1)*p(582) - p(918)
p(950) = p(21)*p(195) - p(949)
p(951) = p(2)*p(581) - p(917) - p(948)
p(952) = p(1)*p(585) - p(928)
p(953) = p(7)*p(347) - p(952)
p(954) = p(45)*p(102) - p(913)
p(955) = p(2)*p(585) - p(930)
p(956) = p(1)*p(589) - p(955)
p(957) = p(6)*p(347) - p(933)
p(958) = p(1)*p(588) - p(937)
p(959) = p(2)*p(589) - p(957)
p(960) = p(9)*p(196)
p(961) = p(9)*p(202)
p(962) = p(9)*p(203)
p(963) = p(9)*p(228)
p(964) = p(9)*p(242)
p(965) = p(9)*p(207)
p(966) = p(9)*p(197)
p(967) = p(9)*p(208)
p(968) = p(9)*p(209)
p(969) = p(9)*p(243)
p(970) = p(9)*p(211)
p(971) = p(9)*p(247)
p(972) = p(9)*p(212)
p(973) = p(9)*p(248)
p(974) = p(9)*p(249)
p(975) = p(9)*p(258)
p(976) = p(9)*p(214)
p(977) = p(9)*p(215)
p(978) = p(9)*p(259)
p(979) = p(9)*p(263)
p(980) = p(9)*p(264)
p(981) = p(9)*p(265)
p(982) = p(9)*p(308)
p(983) = p(9)*p(311)
p(984) = p(9)*p(270)
p(985) = p(9)*p(271)
p(986) = p(9)*p(321)
p(987) = p(9)*p(327)
p(988) = p(9)*p(222)
p(989) = p(9)*p(223)
p(990) = p(9)*p(284)
p(991) = p(9)*p(198)
p(992) = p(9)*p(199)
p(993) = p(9)*p(224)
p(994) = p(9)*p(200)
p(995) = p(9)*p(225)
p(996) = p(9)*p(201)
p(997) = p(9)*p(226)
p(998) = p(9)*p(227)
p(999) = p(9)*p(285)
p(1000) = p(14)*p(328)
p(1001) = p(14)*p(329)
p(1002) = p(14)*p(330)
p(1003) = p(9)*p(292)
p(1004) = p(9)*p(233)
p(1005) = p(9)*p(234)
p(1006) = p(9)*p(235)
p(1007) = p(9)*p(293)
p(1008) = p(9)*p(236)
p(1009) = p(9)*p(204)
p(1010) = p(9)*p(205)
p(1011) = p(9)*p(237)
p(1012) = p(9)*p(238)
p(1013) = p(9)*p(206)
p(1014) = p(9)*p(239)
p(1015) = p(9)*p(240)
p(1016) = p(9)*p(241)
p(1017) = p(9)*p(294)
p(1018) = p(14)*p(276)
p(1019) = p(14)*p(277)
p(1020) = p(14)*p(278)
p(1021) = p(14)*p(279)
p(1022) = p(14)*p(280)
p(1023) = p(14)*p(281)
p(1024) = p(14)*p(282)
p(1025) = p(14)*p(283)
p(1026) = p(9)*p(244)
p(1027) = p(9)*p(295)
p(1028) = p(9)*p(210)
p(1029) = p(9)*p(245)
p(1030) = p(9)*p(246)
p(1031) = p(9)*p(296)
p(1032) = p(14)*p(222)
p(1033) = p(14)*p(223)
p(1034) = p(14)*p(224)
p(1035) = p(14)*p(225)
p(1036) = p(14)*p(226)
p(1037) = p(14)*p(227)
p(1038) = p(14)*p(202)
p(1039) = p(14)*p(203)
p(1040) = p(9)*p(302)
p(1041) = p(9)*p(252)
p(1042) = p(9)*p(253)
p(1043) = p(9)*p(303)
p(1044) = p(9)*p(254)
p(1045) = p(9)*p(213)
p(1046) = p(9)*p(255)
p(1047) = p(9)*p(256)
p(1048) = p(9)*p(257)
p(1049) = p(9)*p(304)
p(1050) = p(14)*p(286)
p(1051) = p(14)*p(287)
p(1052) = p(14)*p(288)
p(1053) = p(14)*p(289)
p(1054) = p(14)*p(290)
p(1055) = p(14)*p(291)
p(1056) = p(9)*p(305)
p(1057) = p(9)*p(260)
p(1058) = p(9)*p(306)
p(1059) = p(9)*p(261)
p(1060) = p(9)*p(262)
p(1061) = p(9)*p(307)
p(1062) = p(14)*p(233)
p(1063) = p(14)*p(234)
p(1064) = p(14)*p(235)
p(1065) = p(14)*p(236)
p(1066) = p(14)*p(237)
p(1067) = p(14)*p(238)
p(1068) = p(14)*p(239)
p(1069) = p(14)*p(240)
p(1070) = p(14)*p(241)
p(1071) = p(14)*p(207)
p(1072) = p(14)*p(208)
p(1073) = p(14)*p(209)
p(1074) = p(9)*p(309)
p(1075) = p(9)*p(310)
p(1076) = p(14)*p(244)
p(1077) = p(14)*p(245)
p(1078) = p(14)*p(246)
p(1079) = p(14)*p(211)
p(1080) = p(14)*p(212)
p(1081) = p(9)*p(315)
p(1082) = p(9)*p(267)
p(1083) = p(9)*p(316)
p(1084) = p(9)*p(268)
p(1085) = p(9)*p(269)
p(1086) = p(9)*p(317)
p(1087) = p(14)*p(297)
p(1088) = p(14)*p(298)
p(1089) = p(14)*p(299)
p(1090) = p(14)*p(300)
p(1091) = p(14)*p(301)
p(1092) = p(9)*p(318)
p(1093) = p(9)*p(319)
p(1094) = p(9)*p(320)
p(1095) = p(14)*p(252)
p(1096) = p(14)*p(253)
p(1097) = p(14)*p(254)
p(1098) = p(14)*p(255)
p(1099) = p(14)*p(256)
p(1100) = p(14)*p(257)
p(1101) = p(14)*p(214)
p(1102) = p(14)*p(215)
p(1103) = p(9)*p(339)
p(1104) = p(14)*p(260)
p(1105) = p(14)*p(261)
p(1106) = p(14)*p(262)
p(1107) = p(9)*p(324)
p(1108) = p(9)*p(325)
p(1109) = p(9)*p(326)
p(1110) = p(14)*p(312)
p(1111) = p(14)*p(313)
p(1112) = p(14)*p(314)
p(1113) = p(9)*p(342)
p(1114) = p(14)*p(267)
p(1115) = p(14)*p(268)
p(1116) = p(14)*p(269)
p(1117) = p(9)*p(345)
p(1118) = p(14)*p(322)
p(1119) = p(14)*p(323)
p(1120) = p(9)*p(331)
p(1121) = p(9)*p(276)
p(1122) = p(9)*p(216)
p(1123) = p(9)*p(277)
p(1124) = p(9)*p(217)
p(1125) = p(9)*p(218)
p(1126) = p(9)*p(278)
p(1127) = p(9)*p(219)
p(1128) = p(9)*p(220)
p(1129) = p(9)*p(279)
p(1130) = p(9)*p(280)
p(1131) = p(9)*p(221)
p(1132) = p(9)*p(281)
p(1133) = p(9)*p(282)
p(1134) = p(9)*p(283)
p(1135) = p(9)*p(332)
p(1136) = p(3)*p(401) - p(988)
p(1137) = p(7)*p(401) - p(991)
p(1138) = p(49)*p(114) - p(996)
p(1139) = p(24)*p(194) - p(999)
p(1140) = p(3)*p(504) - p(990) - p(1139)
p(1141) = p(1)*p(695) - p(990) - p(1140)
p(1142) = p(1)*p(696) - p(993) - p(1141)
p(1143) = p(6)*p(566) - p(1136)
p(1144) = p(32)*p(178) - p(991)
p(1145) = p(39)*p(150) - p(996)
p(1146) = p(3)*p(505) - p(999)
p(1147) = p(14)*p(346)
p(1148) = p(9)*p(333)
p(1149) = p(9)*p(286)
p(1150) = p(9)*p(229)
p(1151) = p(9)*p(287)
p(1152) = p(9)*p(230)
p(1153) = p(9)*p(231)
p(1154) = p(9)*p(288)
p(1155) = p(9)*p(232)
p(1156) = p(9)*p(289)
p(1157) = p(9)*p(290)
p(1158) = p(9)*p(291)
p(1159) = p(9)*p(334)
p(1160) = p(24)*p(181) - p(963)
p(1161) = p(41)*p(114) - p(960)
p(1162) = p(4)*p(401) - p(961)
p(1163) = p(1)*p(713) - p(1005) - p(1162)
p(1164) = p(1)*p(714) - p(1006) - p(1163)
p(1165) = p(1)*p(715) - p(1007) - p(1164)
p(1166) = p(13)*p(331) - p(1147)
p(1167) = p(2)*p(695) - p(1002) - p(1166)
p(1168) = p(32)*p(181) - p(1147)
p(1169) = p(1)*p(717) - p(1007) - p(1167)
p(1170) = p(73)*p(91) - p(1147)
p(1171) = p(1)*p(719) - p(1011) - p(1169)
p(1172) = p(41)*p(150) - p(1147)
p(1173) = p(13)*p(279) - p(961)
p(1174) = p(16)*p(285) - p(1147)
p(1175) = p(69)*p(73) - p(960)
p(1176) = p(4)*p(505) - p(1147)
p(1177) = p(11)*p(285) - p(963)
p(1178) = p(14)*p(331)
p(1179) = p(14)*p(332)
p(1180) = p(3)*p(430) - p(1026)
p(1181) = p(7)*p(430) - p(1028)
p(1182) = p(49)*p(130) - p(1031)
p(1183) = p(3)*p(517) - p(1027) - p(1182)
p(1184) = p(1)*p(730) - p(1027) - p(1183)
p(1185) = p(45)*p(178) - p(1180)
p(1186) = p(39)*p(160) - p(1028)
p(1187) = p(3)*p(518) - p(1031)
p(1188) = p(14)*p(284)
p(1189) = p(14)*p(285)
p(1190) = p(14)*p(228)
p(1191) = p(9)*p(335)
p(1192) = p(9)*p(297)
p(1193) = p(9)*p(250)
p(1194) = p(9)*p(298)
p(1195) = p(9)*p(251)
p(1196) = p(9)*p(299)
p(1197) = p(9)*p(300)
p(1198) = p(9)*p(301)
p(1199) = p(9)*p(336)
p(1200) = p(24)*p(183) - p(964)
p(1201) = p(43)*p(114) - p(966)
p(1202) = p(8)*p(401) - p(1021)
p(1203) = p(1)*p(745) - p(1042) - p(1202)
p(1204) = p(1)*p(746) - p(1043) - p(1203)
p(1205) = p(13)*p(333) - p(1178)
p(1206) = p(2)*p(717) - p(1025) - p(1183)
p(1207) = p(32)*p(183) - p(1178)
p(1208) = p(1)*p(748) - p(1043) - p(1206)
p(1209) = p(73)*p(93) - p(1178)
p(1210) = p(6)*p(509) - p(1021)
p(1211) = p(43)*p(150) - p(1178)
p(1212) = p(32)*p(153) - p(966)
p(1213) = p(18)*p(285) - p(1178)
p(1214) = p(34)*p(150) - p(964)
p(1215) = p(14)*p(333)
p(1216) = p(14)*p(334)
p(1217) = p(41)*p(130) - p(973)
p(1218) = p(4)*p(430) - p(970)
p(1219) = p(1)*p(758) - p(1057) - p(1218)
p(1220) = p(1)*p(759) - p(1058) - p(1219)
p(1221) = p(81)*p(91) - p(1189)
p(1222) = p(2)*p(730) - p(1034) - p(1221)
p(1223) = p(41)*p(160) - p(1189)
p(1224) = p(1)*p(761) - p(1058) - p(1222)
p(1225) = p(16)*p(296) - p(1189)
p(1226) = p(69)*p(81) - p(970)
p(1227) = p(4)*p(518) - p(1189)
p(1228) = p(11)*p(296) - p(973)
p(1229) = p(14)*p(292)
p(1230) = p(14)*p(293)
p(1231) = p(14)*p(294)
p(1232) = p(14)*p(242)
p(1233) = p(14)*p(243)
p(1234) = p(3)*p(464) - p(1074)
p(1235) = p(7)*p(464) - p(1075)
p(1236) = p(3)*p(531) - p(1074) - p(1235)
p(1237) = p(39)*p(187) - p(1234)
p(1238) = p(3)*p(532) - p(1075)
p(1239) = p(14)*p(295)
p(1240) = p(14)*p(296)
p(1241) = p(14)*p(247)
p(1242) = p(14)*p(248)
p(1243) = p(14)*p(249)
p(1244) = p(9)*p(337)
p(1245) = p(9)*p(312)
p(1246) = p(9)*p(266)
p(1247) = p(9)*p(313)
p(1248) = p(9)*p(314)
p(1249) = p(9)*p(338)
p(1250) = p(24)*p(185) - p(975)
p(1251) = p(46)*p(114) - p(977)
p(1252) = p(1)*p(783) - p(1082) - p(1251)
p(1253) = p(1)*p(784) - p(1083) - p(1252)
p(1254) = p(13)*p(335) - p(1215)
p(1255) = p(2)*p(748) - p(1055) - p(1222)
p(1256) = p(32)*p(185) - p(1215)
p(1257) = p(1)*p(786) - p(1083) - p(1255)
p(1258) = p(73)*p(95) - p(1215)
p(1259) = p(13)*p(299) - p(977)
p(1260) = p(46)*p(150) - p(1215)
p(1261) = p(73)*p(84) - p(975)
p(1262) = p(14)*p(335)
p(1263) = p(14)*p(336)
p(1264) = p(43)*p(130) - p(974)
p(1265) = p(8)*p(430) - p(1038)
p(1266) = p(1)*p(794) - p(1093) - p(1265)
p(1267) = p(81)*p(93) - p(1190)
p(1268) = p(2)*p(761) - p(1066) - p(1236)
p(1269) = p(43)*p(160) - p(1190)
p(1270) = p(45)*p(153) - p(1038)
p(1271) = p(18)*p(296) - p(1190)
p(1272) = p(34)*p(160) - p(974)
p(1273) = p(14)*p(302)
p(1274) = p(14)*p(303)
p(1275) = p(14)*p(304)
p(1276) = p(14)*p(258)
p(1277) = p(14)*p(259)
p(1278) = p(4)*p(464) - p(983)
p(1279) = p(1)*p(804) - p(1103) - p(1278)
p(1280) = p(16)*p(310) - p(1240)
p(1281) = p(2)*p(772) - p(1077) - p(1280)
p(1282) = p(4)*p(532) - p(1240)
p(1283) = p(11)*p(310) - p(983)
p(1284) = p(14)*p(305)
p(1285) = p(14)*p(306)
p(1286) = p(14)*p(307)
p(1287) = p(14)*p(263)
p(1288) = p(14)*p(264)
p(1289) = p(14)*p(265)
p(1290) = p(6)*p(464) - p(1104)
p(1291) = p(3)*p(578) - p(1290)
p(1292) = p(14)*p(309)
p(1293) = p(14)*p(310)
p(1294) = p(9)*p(340)
p(1295) = p(9)*p(322)
p(1296) = p(9)*p(323)
p(1297) = p(9)*p(341)
p(1298) = p(24)*p(188) - p(984)
p(1299) = p(50)*p(114) - p(1089)
p(1300) = p(1)*p(813) - p(1108) - p(1299)
p(1301) = p(13)*p(337) - p(1262)
p(1302) = p(2)*p(786) - p(1091) - p(1268)
p(1303) = p(32)*p(188) - p(1262)
p(1304) = p(6)*p(536) - p(1089)
p(1305) = p(73)*p(98) - p(1262)
p(1306) = p(32)*p(173) - p(984)
p(1307) = p(14)*p(337)
p(1308) = p(14)*p(338)
p(1309) = p(46)*p(130) - p(982)
p(1310) = p(1)*p(820) - p(1113) - p(1309)
p(1311) = p(81)*p(95) - p(1232)
p(1312) = p(2)*p(796) - p(1098) - p(1281)
p(1313) = p(46)*p(160) - p(1232)
p(1314) = p(81)*p(84) - p(982)
p(1315) = p(14)*p(315)
p(1316) = p(14)*p(316)
p(1317) = p(14)*p(317)
p(1318) = p(14)*p(270)
p(1319) = p(14)*p(271)
p(1320) = p(8)*p(464) - p(1079)
p(1321) = p(18)*p(310) - p(1242)
p(1322) = p(34)*p(187) - p(1079)
p(1323) = p(14)*p(318)
p(1324) = p(14)*p(319)
p(1325) = p(14)*p(320)
p(1326) = p(9)*p(343)
p(1327) = p(9)*p(344)
p(1328) = p(24)*p(191) - p(987)
p(1329) = p(1)*p(827) - p(1117) - p(1328)
p(1330) = p(13)*p(340) - p(1307)
p(1331) = p(2)*p(815) - p(1112) - p(1312)
p(1332) = p(32)*p(191) - p(1307)
p(1333) = p(13)*p(323) - p(987)
p(1334) = p(14)*p(340)
p(1335) = p(14)*p(341)
p(1336) = p(50)*p(130) - p(1102)
p(1337) = p(81)*p(98) - p(1276)
p(1338) = p(45)*p(173) - p(1102)
p(1339) = p(14)*p(324)
p(1340) = p(14)*p(325)
p(1341) = p(14)*p(326)
p(1342) = p(9)*p(347)
p(1343) = p(24)*p(195) - p(1119)
p(1344) = p(13)*p(343) - p(1334)
p(1345) = p(6)*p(561) - p(1119)
p(1346) = p(14)*p(343)
p(1347) = p(14)*p(344)
p(1348) = p(9)*p(272)
p(1349) = p(9)*p(273)
p(1350) = p(9)*p(274)
p(1351) = p(9)*p(328)
p(1352) = p(9)*p(275)
p(1353) = p(9)*p(329)
p(1354) = p(9)*p(330)
p(1355) = p(9)*p(346)
p(1356) = p(3)*p(570) - p(1120)
p(1357) = p(39)*p(181) - p(1122)
p(1358) = p(41)*p(178) - p(1125)
p(1359) = p(4)*p(566) - p(1129)
p(1360) = p(1)*p(842) - p(1126) - p(1359)
p(1361) = p(1)*p(843) - p(1129)
p(1362) = p(1)*p(844) - p(1130) - p(1360)
p(1363) = p(21)*p(279) - p(1125)
p(1364) = p(1)*p(846) - p(1133) - p(1362)
p(1365) = p(69)*p(101) - p(1122)
p(1366) = p(1)*p(848) - p(1135) - p(1364)
p(1367) = p(11)*p(346) - p(1120)
p(1368) = p(1)*p(850) - p(1140) - p(1139)
p(1369) = p(1)*p(851) - p(1146)
p(1370) = p(3)*p(572) - p(1148)
p(1371) = p(39)*p(183) - p(1150)
p(1372) = p(43)*p(178) - p(1153)
p(1373) = p(1)*p(854) - p(1151) - p(1372)
p(1374) = p(2)*p(843) - p(1143)
p(1375) = p(1)*p(855) - p(1154) - p(1373)
p(1376) = p(7)*p(509) - p(1153)
p(1377) = p(1)*p(857) - p(1157) - p(1375)
p(1378) = p(49)*p(153) - p(1150)
p(1379) = p(1)*p(859) - p(1159) - p(1377)
p(1380) = p(34)*p(194) - p(1148)
p(1381) = p(6)*p(570) - p(1147)
p(1382) = p(1)*p(862) - p(1167) - p(1165)
p(1383) = p(2)*p(851) - p(1147)
p(1384) = p(1)*p(864) - p(1183) - p(1182)
p(1385) = p(1)*p(865) - p(1187)
p(1386) = p(3)*p(574) - p(1191)
p(1387) = p(39)*p(185) - p(1193)
p(1388) = p(46)*p(178) - p(1196)
p(1389) = p(1)*p(868) - p(1194) - p(1388)
p(1390) = p(1)*p(869) - p(1196)
p(1391) = p(1)*p(870) - p(1197) - p(1389)
p(1392) = p(21)*p(299) - p(1193)
p(1393) = p(1)*p(872) - p(1199) - p(1391)
p(1394) = p(84)*p(101) - p(1191)
p(1395) = p(6)*p(572) - p(1178)
p(1396) = p(1)*p(875) - p(1206) - p(1204)
p(1397) = p(8)*p(505) - p(1178)
p(1398) = p(45)*p(181) - p(1189)
p(1399) = p(1)*p(878) - p(1222) - p(1220)
p(1400) = p(2)*p(865) - p(1189)
p(1401) = p(1)*p(880) - p(1236) - p(1235)
p(1402) = p(1)*p(881) - p(1238)
p(1403) = p(3)*p(576) - p(1244)
p(1404) = p(39)*p(188) - p(1246)
p(1405) = p(1)*p(884) - p(1245) - p(1404)
p(1406) = p(2)*p(869) - p(1210)
p(1407) = p(1)*p(885) - p(1247) - p(1405)
p(1408) = p(7)*p(536) - p(1246)
p(1409) = p(1)*p(887) - p(1249) - p(1407)
p(1410) = p(49)*p(173) - p(1244)
p(1411) = p(6)*p(574) - p(1215)
p(1412) = p(1)*p(890) - p(1255) - p(1253)
p(1413) = p(22)*p(285) - p(1215)
p(1414) = p(45)*p(183) - p(1190)
p(1415) = p(1)*p(893) - p(1268) - p(1266)
p(1416) = p(8)*p(518) - p(1190)
p(1417) = p(41)*p(187) - p(1240)
p(1418) = p(1)*p(896) - p(1281) - p(1279)
p(1419) = p(2)*p(881) - p(1240)
p(1420) = p(14)*p(308)
p(1421) = p(4)*p(547) - p(1106) - p(1320)
p(1422) = p(1)*p(900) - p(1291)
p(1423) = p(14)*p(311)
p(1424) = p(3)*p(579) - p(1294)
p(1425) = p(39)*p(191) - p(1296)
p(1426) = p(1)*p(903) - p(1295) - p(1425)
p(1427) = p(1)*p(904) - p(1296)
p(1428) = p(1)*p(905) - p(1297) - p(1426)
p(1429) = p(21)*p(323) - p(1294)
p(1430) = p(6)*p(576) - p(1262)
p(1431) = p(1)*p(908) - p(1302) - p(1300)
p(1432) = p(50)*p(150) - p(1262)
p(1433) = p(45)*p(185) - p(1232)
p(1434) = p(1)*p(911) - p(1312) - p(1310)
p(1435) = p(22)*p(296) - p(1232)
p(1436) = p(43)*p(187) - p(1242)
p(1437) = p(1)*p(915) - p(1322) - p(1320)
p(1438) = p(8)*p(532) - p(1242)
p(1439) = p(14)*p(321)
p(1440) = p(4)*p(578) - p(1293)
p(1441) = p(2)*p(899) - p(1292) - p(1440)
p(1442) = p(2)*p(900) - p(1293)
p(1443) = p(14)*p(339)
p(1444) = p(3)*p(582) - p(1326)
p(1445) = p(2)*p(903) - p(1303) - p(1299)
p(1446) = p(2)*p(904) - p(1304)
p(1447) = p(1)*p(919) - p(1327) - p(1445)
p(1448) = p(7)*p(561) - p(1326)
p(1449) = p(6)*p(579) - p(1307)
p(1450) = p(1)*p(922) - p(1331) - p(1329)
p(1451) = p(73)*p(102) - p(1307)
p(1452) = p(45)*p(188) - p(1276)
p(1453) = p(1)*p(925) - p(1338) - p(1336)
p(1454) = p(50)*p(160) - p(1276)
p(1455) = p(14)*p(327)
p(1456) = p(46)*p(187) - p(1243)
p(1457) = p(2)*p(915) - p(1324) - p(1441)
p(1458) = p(22)*p(310) - p(1243)
p(1459) = p(14)*p(342)
p(1460) = p(3)*p(585) - p(1342)
p(1461) = p(2)*p(919) - p(1332) - p(1329)
p(1462) = p(1)*p(929) - p(1342)
p(1463) = p(6)*p(582) - p(1334)
p(1464) = p(1)*p(931) - p(1345) - p(1343)
p(1465) = p(32)*p(195) - p(1334)
p(1466) = p(45)*p(191) - p(1318)
p(1467) = p(2)*p(925) - p(1340) - p(1457)
p(1468) = p(81)*p(102) - p(1318)
p(1469) = p(14)*p(345)
p(1470) = p(2)*p(928) - p(1344) - p(1343)
p(1471) = p(2)*p(929) - p(1345)
p(1472) = p(6)*p(585) - p(1346)
p(1473) = p(2)*p(931) - p(1347) - p(1467)
p(1474) = p(13)*p(347) - p(1346)
p(1475) = p(14)*p(347)
p(1476) = p(3)*p(566) - p(1351)
p(1477) = p(7)*p(566) - p(1349)
p(1478) = p(49)*p(178) - p(1348)
p(1479) = p(39)*p(194) - p(1352)
p(1480) = p(3)*p(588) - p(1355)
p(1481) = p(1)*p(938) - p(1356)
p(1482) = p(2)*p(958) - p(1481)
p(1483) = p(1)*p(940) - p(1370)
p(1484) = p(8)*p(588) - p(1483)
p(1485) = p(1)*p(942) - p(1386)
p(1486) = p(22)*p(346) - p(1485)
p(1487) = p(1)*p(944) - p(1403)
p(1488) = p(50)*p(194) - p(1487)
p(1489) = p(1)*p(946) - p(1424)
p(1490) = p(101)*p(102) - p(1489)
p(1491) = p(6)*p(578) - p(1443)
p(1492) = p(1)*p(949) - p(1444)
p(1493) = p(49)*p(195) - p(1492)
p(1494) = p(8)*p(578) - p(1423)
p(1495) = p(1)*p(952) - p(1460)
p(1496) = p(21)*p(347) - p(1495)
p(1497) = p(50)*p(187) - p(1420)
p(1498) = p(1)*p(955) - p(1470)
p(1499) = p(7)*p(589) - p(1498)
p(1500) = p(45)*p(195) - p(1455)
p(1501) = p(2)*p(955) - p(1472)
p(1502) = p(1)*p(959) - p(1501)
p(1503) = p(6)*p(589) - p(1475)
p(1504) = p(1)*p(958) - p(1480)
p(1505) = p(2)*p(959) - p(1503)

do j=1,1506
  bm1(j)=p(j-1)
enddo

bm2(1)=bm1(4)
bm2(2)=bm1(10)
bm2(3)=bm1(11)
bm2(4)=bm1(12)
bm2(5)=bm1(13)
bm2(6)=bm1(15)
bm2(7)=bm1(16)
bm2(8)=bm1(24)
bm2(9)=bm1(25)
bm2(10)=bm1(26)
bm2(11)=bm1(27)
bm2(12)=bm1(28)
bm2(13)=bm1(29)
bm2(14)=bm1(30)
bm2(15)=bm1(31)
bm2(16)=bm1(32)
bm2(17)=bm1(34)
bm2(18)=bm1(35)
bm2(19)=bm1(36)
bm2(20)=bm1(37)
bm2(21)=bm1(39)
bm2(22)=bm1(40)
bm2(23)=bm1(41)
bm2(24)=bm1(52)
bm2(25)=bm1(53)
bm2(26)=bm1(54)
bm2(27)=bm1(55)
bm2(28)=bm1(56)
bm2(29)=bm1(57)
bm2(30)=bm1(58)
bm2(31)=bm1(59)
bm2(32)=bm1(60)
bm2(33)=bm1(61)
bm2(34)=bm1(62)
bm2(35)=bm1(63)
bm2(36)=bm1(64)
bm2(37)=bm1(65)
bm2(38)=bm1(66)
bm2(39)=bm1(67)
bm2(40)=bm1(68)
bm2(41)=bm1(69)
bm2(42)=bm1(70)
bm2(43)=bm1(71)
bm2(44)=bm1(72)
bm2(45)=bm1(73)
bm2(46)=bm1(75)
bm2(47)=bm1(76)
bm2(48)=bm1(77)
bm2(49)=bm1(78)
bm2(50)=bm1(79)
bm2(51)=bm1(81)
bm2(52)=bm1(83)
bm2(53)=bm1(84)
bm2(54)=bm1(85)
bm2(55)=bm1(86)
bm2(56)=bm1(87)
bm2(57)=bm1(89)
bm2(58)=bm1(90)
bm2(59)=bm1(91)
bm2(60)=bm1(104)
bm2(61)=bm1(105)
bm2(62)=bm1(106)
bm2(63)=bm1(107)
bm2(64)=bm1(108)
bm2(65)=bm1(109)
bm2(66)=bm1(110)
bm2(67)=bm1(111)
bm2(68)=bm1(112)
bm2(69)=bm1(113)
bm2(70)=bm1(114)
bm2(71)=bm1(115)
bm2(72)=bm1(116)
bm2(73)=bm1(117)
bm2(74)=bm1(118)
bm2(75)=bm1(119)
bm2(76)=bm1(120)
bm2(77)=bm1(121)
bm2(78)=bm1(122)
bm2(79)=bm1(123)
bm2(80)=bm1(124)
bm2(81)=bm1(125)
bm2(82)=bm1(126)
bm2(83)=bm1(127)
bm2(84)=bm1(128)
bm2(85)=bm1(129)
bm2(86)=bm1(130)
bm2(87)=bm1(131)
bm2(88)=bm1(132)
bm2(89)=bm1(133)
bm2(90)=bm1(134)
bm2(91)=bm1(135)
bm2(92)=bm1(136)
bm2(93)=bm1(137)
bm2(94)=bm1(138)
bm2(95)=bm1(139)
bm2(96)=bm1(140)
bm2(97)=bm1(141)
bm2(98)=bm1(142)
bm2(99)=bm1(143)
bm2(100)=bm1(144)
bm2(101)=bm1(145)
bm2(102)=bm1(146)
bm2(103)=bm1(147)
bm2(104)=bm1(148)
bm2(105)=bm1(149)
bm2(106)=bm1(150)
bm2(107)=bm1(152)
bm2(108)=bm1(153)
bm2(109)=bm1(154)
bm2(110)=bm1(155)
bm2(111)=bm1(156)
bm2(112)=bm1(157)
bm2(113)=bm1(158)
bm2(114)=bm1(160)
bm2(115)=bm1(162)
bm2(116)=bm1(163)
bm2(117)=bm1(164)
bm2(118)=bm1(165)
bm2(119)=bm1(166)
bm2(120)=bm1(167)
bm2(121)=bm1(169)
bm2(122)=bm1(170)
bm2(123)=bm1(172)
bm2(124)=bm1(173)
bm2(125)=bm1(174)
bm2(126)=bm1(175)
bm2(127)=bm1(176)
bm2(128)=bm1(178)
bm2(129)=bm1(179)
bm2(130)=bm1(180)
bm2(131)=bm1(181)
bm2(132)=bm1(197)
bm2(133)=bm1(198)
bm2(134)=bm1(199)
bm2(135)=bm1(200)
bm2(136)=bm1(201)
bm2(137)=bm1(202)
bm2(138)=bm1(203)
bm2(139)=bm1(204)
bm2(140)=bm1(205)
bm2(141)=bm1(206)
bm2(142)=bm1(207)
bm2(143)=bm1(208)
bm2(144)=bm1(209)
bm2(145)=bm1(210)
bm2(146)=bm1(211)
bm2(147)=bm1(212)
bm2(148)=bm1(213)
bm2(149)=bm1(214)
bm2(150)=bm1(215)
bm2(151)=bm1(216)
bm2(152)=bm1(217)
bm2(153)=bm1(218)
bm2(154)=bm1(219)
bm2(155)=bm1(220)
bm2(156)=bm1(221)
bm2(157)=bm1(222)
bm2(158)=bm1(223)
bm2(159)=bm1(224)
bm2(160)=bm1(225)
bm2(161)=bm1(226)
bm2(162)=bm1(227)
bm2(163)=bm1(228)
bm2(164)=bm1(229)
bm2(165)=bm1(230)
bm2(166)=bm1(231)
bm2(167)=bm1(232)
bm2(168)=bm1(233)
bm2(169)=bm1(234)
bm2(170)=bm1(235)
bm2(171)=bm1(236)
bm2(172)=bm1(237)
bm2(173)=bm1(238)
bm2(174)=bm1(239)
bm2(175)=bm1(240)
bm2(176)=bm1(241)
bm2(177)=bm1(242)
bm2(178)=bm1(243)
bm2(179)=bm1(244)
bm2(180)=bm1(245)
bm2(181)=bm1(246)
bm2(182)=bm1(247)
bm2(183)=bm1(248)
bm2(184)=bm1(249)
bm2(185)=bm1(250)
bm2(186)=bm1(251)
bm2(187)=bm1(252)
bm2(188)=bm1(253)
bm2(189)=bm1(254)
bm2(190)=bm1(255)
bm2(191)=bm1(256)
bm2(192)=bm1(257)
bm2(193)=bm1(258)
bm2(194)=bm1(259)
bm2(195)=bm1(260)
bm2(196)=bm1(261)
bm2(197)=bm1(262)
bm2(198)=bm1(263)
bm2(199)=bm1(264)
bm2(200)=bm1(265)
bm2(201)=bm1(266)
bm2(202)=bm1(267)
bm2(203)=bm1(268)
bm2(204)=bm1(269)
bm2(205)=bm1(270)
bm2(206)=bm1(271)
bm2(207)=bm1(272)
bm2(208)=bm1(273)
bm2(209)=bm1(274)
bm2(210)=bm1(275)
bm2(211)=bm1(276)
bm2(212)=bm1(277)
bm2(213)=bm1(278)
bm2(214)=bm1(279)
bm2(215)=bm1(280)
bm2(216)=bm1(281)
bm2(217)=bm1(282)
bm2(218)=bm1(283)
bm2(219)=bm1(284)
bm2(220)=bm1(285)
bm2(221)=bm1(287)
bm2(222)=bm1(288)
bm2(223)=bm1(289)
bm2(224)=bm1(290)
bm2(225)=bm1(291)
bm2(226)=bm1(292)
bm2(227)=bm1(293)
bm2(228)=bm1(294)
bm2(229)=bm1(296)
bm2(230)=bm1(298)
bm2(231)=bm1(299)
bm2(232)=bm1(300)
bm2(233)=bm1(301)
bm2(234)=bm1(302)
bm2(235)=bm1(303)
bm2(236)=bm1(304)
bm2(237)=bm1(306)
bm2(238)=bm1(307)
bm2(239)=bm1(309)
bm2(240)=bm1(310)
bm2(241)=bm1(312)
bm2(242)=bm1(313)
bm2(243)=bm1(314)
bm2(244)=bm1(315)
bm2(245)=bm1(316)
bm2(246)=bm1(317)
bm2(247)=bm1(319)
bm2(248)=bm1(320)
bm2(249)=bm1(322)
bm2(250)=bm1(323)
bm2(251)=bm1(324)
bm2(252)=bm1(325)
bm2(253)=bm1(326)
bm2(254)=bm1(328)
bm2(255)=bm1(329)
bm2(256)=bm1(330)
bm2(257)=bm1(331)
bm2(258)=bm1(349)
bm2(259)=bm1(350)
bm2(260)=bm1(351)
bm2(261)=bm1(352)
bm2(262)=bm1(353)
bm2(263)=bm1(354)
bm2(264)=bm1(355)
bm2(265)=bm1(356)
bm2(266)=bm1(357)
bm2(267)=bm1(358)
bm2(268)=bm1(359)
bm2(269)=bm1(360)
bm2(270)=bm1(361)
bm2(271)=bm1(362)
bm2(272)=bm1(363)
bm2(273)=bm1(364)
bm2(274)=bm1(365)
bm2(275)=bm1(366)
bm2(276)=bm1(367)
bm2(277)=bm1(368)
bm2(278)=bm1(369)
bm2(279)=bm1(370)
bm2(280)=bm1(371)
bm2(281)=bm1(372)
bm2(282)=bm1(373)
bm2(283)=bm1(374)
bm2(284)=bm1(375)
bm2(285)=bm1(376)
bm2(286)=bm1(377)
bm2(287)=bm1(378)
bm2(288)=bm1(379)
bm2(289)=bm1(380)
bm2(290)=bm1(381)
bm2(291)=bm1(382)
bm2(292)=bm1(383)
bm2(293)=bm1(384)
bm2(294)=bm1(385)
bm2(295)=bm1(386)
bm2(296)=bm1(387)
bm2(297)=bm1(388)
bm2(298)=bm1(389)
bm2(299)=bm1(390)
bm2(300)=bm1(391)
bm2(301)=bm1(392)
bm2(302)=bm1(393)
bm2(303)=bm1(394)
bm2(304)=bm1(395)
bm2(305)=bm1(396)
bm2(306)=bm1(397)
bm2(307)=bm1(398)
bm2(308)=bm1(399)
bm2(309)=bm1(400)
bm2(310)=bm1(401)
bm2(311)=bm1(402)
bm2(312)=bm1(403)
bm2(313)=bm1(404)
bm2(314)=bm1(405)
bm2(315)=bm1(406)
bm2(316)=bm1(407)
bm2(317)=bm1(408)
bm2(318)=bm1(409)
bm2(319)=bm1(410)
bm2(320)=bm1(411)
bm2(321)=bm1(412)
bm2(322)=bm1(413)
bm2(323)=bm1(414)
bm2(324)=bm1(415)
bm2(325)=bm1(416)
bm2(326)=bm1(417)
bm2(327)=bm1(418)
bm2(328)=bm1(419)
bm2(329)=bm1(420)
bm2(330)=bm1(421)
bm2(331)=bm1(422)
bm2(332)=bm1(423)
bm2(333)=bm1(424)
bm2(334)=bm1(425)
bm2(335)=bm1(426)
bm2(336)=bm1(427)
bm2(337)=bm1(428)
bm2(338)=bm1(429)
bm2(339)=bm1(430)
bm2(340)=bm1(431)
bm2(341)=bm1(432)
bm2(342)=bm1(433)
bm2(343)=bm1(434)
bm2(344)=bm1(435)
bm2(345)=bm1(436)
bm2(346)=bm1(437)
bm2(347)=bm1(438)
bm2(348)=bm1(439)
bm2(349)=bm1(440)
bm2(350)=bm1(441)
bm2(351)=bm1(442)
bm2(352)=bm1(443)
bm2(353)=bm1(444)
bm2(354)=bm1(445)
bm2(355)=bm1(446)
bm2(356)=bm1(447)
bm2(357)=bm1(448)
bm2(358)=bm1(449)
bm2(359)=bm1(450)
bm2(360)=bm1(451)
bm2(361)=bm1(452)
bm2(362)=bm1(453)
bm2(363)=bm1(454)
bm2(364)=bm1(455)
bm2(365)=bm1(456)
bm2(366)=bm1(457)
bm2(367)=bm1(458)
bm2(368)=bm1(459)
bm2(369)=bm1(460)
bm2(370)=bm1(461)
bm2(371)=bm1(462)
bm2(372)=bm1(463)
bm2(373)=bm1(464)
bm2(374)=bm1(465)
bm2(375)=bm1(466)
bm2(376)=bm1(467)
bm2(377)=bm1(468)
bm2(378)=bm1(469)
bm2(379)=bm1(470)
bm2(380)=bm1(471)
bm2(381)=bm1(472)
bm2(382)=bm1(473)
bm2(383)=bm1(474)
bm2(384)=bm1(475)
bm2(385)=bm1(476)
bm2(386)=bm1(477)
bm2(387)=bm1(478)
bm2(388)=bm1(479)
bm2(389)=bm1(480)
bm2(390)=bm1(481)
bm2(391)=bm1(482)
bm2(392)=bm1(483)
bm2(393)=bm1(484)
bm2(394)=bm1(485)
bm2(395)=bm1(486)
bm2(396)=bm1(487)
bm2(397)=bm1(488)
bm2(398)=bm1(489)
bm2(399)=bm1(490)
bm2(400)=bm1(491)
bm2(401)=bm1(492)
bm2(402)=bm1(493)
bm2(403)=bm1(494)
bm2(404)=bm1(495)
bm2(405)=bm1(496)
bm2(406)=bm1(497)
bm2(407)=bm1(498)
bm2(408)=bm1(499)
bm2(409)=bm1(500)
bm2(410)=bm1(501)
bm2(411)=bm1(502)
bm2(412)=bm1(503)
bm2(413)=bm1(504)
bm2(414)=bm1(505)
bm2(415)=bm1(507)
bm2(416)=bm1(508)
bm2(417)=bm1(509)
bm2(418)=bm1(510)
bm2(419)=bm1(511)
bm2(420)=bm1(512)
bm2(421)=bm1(513)
bm2(422)=bm1(514)
bm2(423)=bm1(515)
bm2(424)=bm1(516)
bm2(425)=bm1(518)
bm2(426)=bm1(520)
bm2(427)=bm1(521)
bm2(428)=bm1(522)
bm2(429)=bm1(523)
bm2(430)=bm1(524)
bm2(431)=bm1(525)
bm2(432)=bm1(526)
bm2(433)=bm1(527)
bm2(434)=bm1(529)
bm2(435)=bm1(530)
bm2(436)=bm1(532)
bm2(437)=bm1(534)
bm2(438)=bm1(535)
bm2(439)=bm1(536)
bm2(440)=bm1(537)
bm2(441)=bm1(538)
bm2(442)=bm1(539)
bm2(443)=bm1(540)
bm2(444)=bm1(541)
bm2(445)=bm1(543)
bm2(446)=bm1(544)
bm2(447)=bm1(546)
bm2(448)=bm1(547)
bm2(449)=bm1(548)
bm2(450)=bm1(550)
bm2(451)=bm1(551)
bm2(452)=bm1(552)
bm2(453)=bm1(553)
bm2(454)=bm1(554)
bm2(455)=bm1(555)
bm2(456)=bm1(557)
bm2(457)=bm1(558)
bm2(458)=bm1(560)
bm2(459)=bm1(561)
bm2(460)=bm1(562)
bm2(461)=bm1(563)
bm2(462)=bm1(564)
bm2(463)=bm1(566)
bm2(464)=bm1(567)
bm2(465)=bm1(568)
bm2(466)=bm1(569)
bm2(467)=bm1(570)
bm2(468)=bm1(591)
bm2(469)=bm1(592)
bm2(470)=bm1(593)
bm2(471)=bm1(594)
bm2(472)=bm1(595)
bm2(473)=bm1(596)
bm2(474)=bm1(597)
bm2(475)=bm1(598)
bm2(476)=bm1(599)
bm2(477)=bm1(600)
bm2(478)=bm1(601)
bm2(479)=bm1(602)
bm2(480)=bm1(603)
bm2(481)=bm1(604)
bm2(482)=bm1(605)
bm2(483)=bm1(606)
bm2(484)=bm1(607)
bm2(485)=bm1(608)
bm2(486)=bm1(609)
bm2(487)=bm1(610)
bm2(488)=bm1(611)
bm2(489)=bm1(612)
bm2(490)=bm1(613)
bm2(491)=bm1(614)
bm2(492)=bm1(615)
bm2(493)=bm1(616)
bm2(494)=bm1(617)
bm2(495)=bm1(618)
bm2(496)=bm1(619)
bm2(497)=bm1(620)
bm2(498)=bm1(621)
bm2(499)=bm1(622)
bm2(500)=bm1(623)
bm2(501)=bm1(624)
bm2(502)=bm1(625)
bm2(503)=bm1(626)
bm2(504)=bm1(627)
bm2(505)=bm1(628)
bm2(506)=bm1(629)
bm2(507)=bm1(630)
bm2(508)=bm1(631)
bm2(509)=bm1(632)
bm2(510)=bm1(633)
bm2(511)=bm1(634)
bm2(512)=bm1(635)
bm2(513)=bm1(636)
bm2(514)=bm1(637)
bm2(515)=bm1(638)
bm2(516)=bm1(639)
bm2(517)=bm1(640)
bm2(518)=bm1(641)
bm2(519)=bm1(642)
bm2(520)=bm1(643)
bm2(521)=bm1(644)
bm2(522)=bm1(645)
bm2(523)=bm1(646)
bm2(524)=bm1(647)
bm2(525)=bm1(648)
bm2(526)=bm1(649)
bm2(527)=bm1(650)
bm2(528)=bm1(651)
bm2(529)=bm1(652)
bm2(530)=bm1(653)
bm2(531)=bm1(654)
bm2(532)=bm1(655)
bm2(533)=bm1(656)
bm2(534)=bm1(657)
bm2(535)=bm1(658)
bm2(536)=bm1(659)
bm2(537)=bm1(660)
bm2(538)=bm1(661)
bm2(539)=bm1(662)
bm2(540)=bm1(663)
bm2(541)=bm1(664)
bm2(542)=bm1(665)
bm2(543)=bm1(666)
bm2(544)=bm1(667)
bm2(545)=bm1(668)
bm2(546)=bm1(669)
bm2(547)=bm1(670)
bm2(548)=bm1(671)
bm2(549)=bm1(672)
bm2(550)=bm1(673)
bm2(551)=bm1(674)
bm2(552)=bm1(675)
bm2(553)=bm1(676)
bm2(554)=bm1(677)
bm2(555)=bm1(678)
bm2(556)=bm1(679)
bm2(557)=bm1(680)
bm2(558)=bm1(681)
bm2(559)=bm1(682)
bm2(560)=bm1(683)
bm2(561)=bm1(684)
bm2(562)=bm1(685)
bm2(563)=bm1(686)
bm2(564)=bm1(687)
bm2(565)=bm1(688)
bm2(566)=bm1(689)
bm2(567)=bm1(690)
bm2(568)=bm1(691)
bm2(569)=bm1(692)
bm2(570)=bm1(693)
bm2(571)=bm1(694)
bm2(572)=bm1(695)
bm2(573)=bm1(696)
bm2(574)=bm1(697)
bm2(575)=bm1(698)
bm2(576)=bm1(699)
bm2(577)=bm1(700)
bm2(578)=bm1(701)
bm2(579)=bm1(702)
bm2(580)=bm1(703)
bm2(581)=bm1(704)
bm2(582)=bm1(705)
bm2(583)=bm1(706)
bm2(584)=bm1(707)
bm2(585)=bm1(708)
bm2(586)=bm1(709)
bm2(587)=bm1(710)
bm2(588)=bm1(711)
bm2(589)=bm1(712)
bm2(590)=bm1(713)
bm2(591)=bm1(714)
bm2(592)=bm1(715)
bm2(593)=bm1(716)
bm2(594)=bm1(717)
bm2(595)=bm1(718)
bm2(596)=bm1(719)
bm2(597)=bm1(720)
bm2(598)=bm1(721)
bm2(599)=bm1(722)
bm2(600)=bm1(723)
bm2(601)=bm1(724)
bm2(602)=bm1(725)
bm2(603)=bm1(726)
bm2(604)=bm1(727)
bm2(605)=bm1(728)
bm2(606)=bm1(729)
bm2(607)=bm1(730)
bm2(608)=bm1(731)
bm2(609)=bm1(732)
bm2(610)=bm1(733)
bm2(611)=bm1(734)
bm2(612)=bm1(735)
bm2(613)=bm1(736)
bm2(614)=bm1(737)
bm2(615)=bm1(738)
bm2(616)=bm1(739)
bm2(617)=bm1(740)
bm2(618)=bm1(741)
bm2(619)=bm1(742)
bm2(620)=bm1(743)
bm2(621)=bm1(744)
bm2(622)=bm1(745)
bm2(623)=bm1(746)
bm2(624)=bm1(747)
bm2(625)=bm1(748)
bm2(626)=bm1(749)
bm2(627)=bm1(750)
bm2(628)=bm1(751)
bm2(629)=bm1(752)
bm2(630)=bm1(753)
bm2(631)=bm1(754)
bm2(632)=bm1(755)
bm2(633)=bm1(756)
bm2(634)=bm1(757)
bm2(635)=bm1(758)
bm2(636)=bm1(759)
bm2(637)=bm1(760)
bm2(638)=bm1(761)
bm2(639)=bm1(762)
bm2(640)=bm1(763)
bm2(641)=bm1(764)
bm2(642)=bm1(765)
bm2(643)=bm1(766)
bm2(644)=bm1(767)
bm2(645)=bm1(768)
bm2(646)=bm1(769)
bm2(647)=bm1(770)
bm2(648)=bm1(771)
bm2(649)=bm1(772)
bm2(650)=bm1(773)
bm2(651)=bm1(774)
bm2(652)=bm1(775)
bm2(653)=bm1(776)
bm2(654)=bm1(777)
bm2(655)=bm1(778)
bm2(656)=bm1(779)
bm2(657)=bm1(780)
bm2(658)=bm1(781)
bm2(659)=bm1(782)
bm2(660)=bm1(783)
bm2(661)=bm1(784)
bm2(662)=bm1(785)
bm2(663)=bm1(786)
bm2(664)=bm1(787)
bm2(665)=bm1(788)
bm2(666)=bm1(789)
bm2(667)=bm1(790)
bm2(668)=bm1(791)
bm2(669)=bm1(792)
bm2(670)=bm1(793)
bm2(671)=bm1(794)
bm2(672)=bm1(795)
bm2(673)=bm1(796)
bm2(674)=bm1(797)
bm2(675)=bm1(798)
bm2(676)=bm1(799)
bm2(677)=bm1(800)
bm2(678)=bm1(801)
bm2(679)=bm1(802)
bm2(680)=bm1(803)
bm2(681)=bm1(804)
bm2(682)=bm1(805)
bm2(683)=bm1(806)
bm2(684)=bm1(807)
bm2(685)=bm1(808)
bm2(686)=bm1(809)
bm2(687)=bm1(810)
bm2(688)=bm1(811)
bm2(689)=bm1(812)
bm2(690)=bm1(813)
bm2(691)=bm1(814)
bm2(692)=bm1(815)
bm2(693)=bm1(816)
bm2(694)=bm1(817)
bm2(695)=bm1(818)
bm2(696)=bm1(819)
bm2(697)=bm1(820)
bm2(698)=bm1(821)
bm2(699)=bm1(822)
bm2(700)=bm1(823)
bm2(701)=bm1(824)
bm2(702)=bm1(825)
bm2(703)=bm1(826)
bm2(704)=bm1(827)
bm2(705)=bm1(828)
bm2(706)=bm1(829)
bm2(707)=bm1(830)
bm2(708)=bm1(831)
bm2(709)=bm1(832)
bm2(710)=bm1(833)
bm2(711)=bm1(834)
bm2(712)=bm1(835)
bm2(713)=bm1(836)
bm2(714)=bm1(837)
bm2(715)=bm1(838)
bm2(716)=bm1(839)
bm2(717)=bm1(840)
bm2(718)=bm1(841)
bm2(719)=bm1(842)
bm2(720)=bm1(843)
bm2(721)=bm1(844)
bm2(722)=bm1(845)
bm2(723)=bm1(846)
bm2(724)=bm1(847)
bm2(725)=bm1(848)
bm2(726)=bm1(849)
bm2(727)=bm1(850)
bm2(728)=bm1(851)
bm2(729)=bm1(853)
bm2(730)=bm1(854)
bm2(731)=bm1(855)
bm2(732)=bm1(856)
bm2(733)=bm1(857)
bm2(734)=bm1(858)
bm2(735)=bm1(859)
bm2(736)=bm1(860)
bm2(737)=bm1(861)
bm2(738)=bm1(862)
bm2(739)=bm1(863)
bm2(740)=bm1(865)
bm2(741)=bm1(867)
bm2(742)=bm1(868)
bm2(743)=bm1(869)
bm2(744)=bm1(870)
bm2(745)=bm1(871)
bm2(746)=bm1(872)
bm2(747)=bm1(873)
bm2(748)=bm1(874)
bm2(749)=bm1(875)
bm2(750)=bm1(876)
bm2(751)=bm1(878)
bm2(752)=bm1(879)
bm2(753)=bm1(881)
bm2(754)=bm1(883)
bm2(755)=bm1(884)
bm2(756)=bm1(885)
bm2(757)=bm1(886)
bm2(758)=bm1(887)
bm2(759)=bm1(888)
bm2(760)=bm1(889)
bm2(761)=bm1(890)
bm2(762)=bm1(891)
bm2(763)=bm1(893)
bm2(764)=bm1(894)
bm2(765)=bm1(896)
bm2(766)=bm1(897)
bm2(767)=bm1(899)
bm2(768)=bm1(900)
bm2(769)=bm1(902)
bm2(770)=bm1(903)
bm2(771)=bm1(904)
bm2(772)=bm1(905)
bm2(773)=bm1(906)
bm2(774)=bm1(907)
bm2(775)=bm1(908)
bm2(776)=bm1(909)
bm2(777)=bm1(911)
bm2(778)=bm1(912)
bm2(779)=bm1(914)
bm2(780)=bm1(915)
bm2(781)=bm1(916)
bm2(782)=bm1(918)
bm2(783)=bm1(919)
bm2(784)=bm1(920)
bm2(785)=bm1(921)
bm2(786)=bm1(922)
bm2(787)=bm1(923)
bm2(788)=bm1(925)
bm2(789)=bm1(926)
bm2(790)=bm1(928)
bm2(791)=bm1(929)
bm2(792)=bm1(930)
bm2(793)=bm1(931)
bm2(794)=bm1(932)
bm2(795)=bm1(934)
bm2(796)=bm1(935)
bm2(797)=bm1(936)
bm2(798)=bm1(937)
bm2(799)=bm1(938)
bm2(800)=bm1(961)
bm2(801)=bm1(962)
bm2(802)=bm1(963)
bm2(803)=bm1(964)
bm2(804)=bm1(965)
bm2(805)=bm1(966)
bm2(806)=bm1(967)
bm2(807)=bm1(968)
bm2(808)=bm1(969)
bm2(809)=bm1(970)
bm2(810)=bm1(971)
bm2(811)=bm1(972)
bm2(812)=bm1(973)
bm2(813)=bm1(974)
bm2(814)=bm1(975)
bm2(815)=bm1(976)
bm2(816)=bm1(977)
bm2(817)=bm1(978)
bm2(818)=bm1(979)
bm2(819)=bm1(980)
bm2(820)=bm1(981)
bm2(821)=bm1(982)
bm2(822)=bm1(983)
bm2(823)=bm1(984)
bm2(824)=bm1(985)
bm2(825)=bm1(986)
bm2(826)=bm1(987)
bm2(827)=bm1(988)
bm2(828)=bm1(989)
bm2(829)=bm1(990)
bm2(830)=bm1(991)
bm2(831)=bm1(992)
bm2(832)=bm1(993)
bm2(833)=bm1(994)
bm2(834)=bm1(995)
bm2(835)=bm1(996)
bm2(836)=bm1(997)
bm2(837)=bm1(998)
bm2(838)=bm1(999)
bm2(839)=bm1(1000)
bm2(840)=bm1(1001)
bm2(841)=bm1(1002)
bm2(842)=bm1(1003)
bm2(843)=bm1(1004)
bm2(844)=bm1(1005)
bm2(845)=bm1(1006)
bm2(846)=bm1(1007)
bm2(847)=bm1(1008)
bm2(848)=bm1(1009)
bm2(849)=bm1(1010)
bm2(850)=bm1(1011)
bm2(851)=bm1(1012)
bm2(852)=bm1(1013)
bm2(853)=bm1(1014)
bm2(854)=bm1(1015)
bm2(855)=bm1(1016)
bm2(856)=bm1(1017)
bm2(857)=bm1(1018)
bm2(858)=bm1(1019)
bm2(859)=bm1(1020)
bm2(860)=bm1(1021)
bm2(861)=bm1(1022)
bm2(862)=bm1(1023)
bm2(863)=bm1(1024)
bm2(864)=bm1(1025)
bm2(865)=bm1(1026)
bm2(866)=bm1(1027)
bm2(867)=bm1(1028)
bm2(868)=bm1(1029)
bm2(869)=bm1(1030)
bm2(870)=bm1(1031)
bm2(871)=bm1(1032)
bm2(872)=bm1(1033)
bm2(873)=bm1(1034)
bm2(874)=bm1(1035)
bm2(875)=bm1(1036)
bm2(876)=bm1(1037)
bm2(877)=bm1(1038)
bm2(878)=bm1(1039)
bm2(879)=bm1(1040)
bm2(880)=bm1(1041)
bm2(881)=bm1(1042)
bm2(882)=bm1(1043)
bm2(883)=bm1(1044)
bm2(884)=bm1(1045)
bm2(885)=bm1(1046)
bm2(886)=bm1(1047)
bm2(887)=bm1(1048)
bm2(888)=bm1(1049)
bm2(889)=bm1(1050)
bm2(890)=bm1(1051)
bm2(891)=bm1(1052)
bm2(892)=bm1(1053)
bm2(893)=bm1(1054)
bm2(894)=bm1(1055)
bm2(895)=bm1(1056)
bm2(896)=bm1(1057)
bm2(897)=bm1(1058)
bm2(898)=bm1(1059)
bm2(899)=bm1(1060)
bm2(900)=bm1(1061)
bm2(901)=bm1(1062)
bm2(902)=bm1(1063)
bm2(903)=bm1(1064)
bm2(904)=bm1(1065)
bm2(905)=bm1(1066)
bm2(906)=bm1(1067)
bm2(907)=bm1(1068)
bm2(908)=bm1(1069)
bm2(909)=bm1(1070)
bm2(910)=bm1(1071)
bm2(911)=bm1(1072)
bm2(912)=bm1(1073)
bm2(913)=bm1(1074)
bm2(914)=bm1(1075)
bm2(915)=bm1(1076)
bm2(916)=bm1(1077)
bm2(917)=bm1(1078)
bm2(918)=bm1(1079)
bm2(919)=bm1(1080)
bm2(920)=bm1(1081)
bm2(921)=bm1(1082)
bm2(922)=bm1(1083)
bm2(923)=bm1(1084)
bm2(924)=bm1(1085)
bm2(925)=bm1(1086)
bm2(926)=bm1(1087)
bm2(927)=bm1(1088)
bm2(928)=bm1(1089)
bm2(929)=bm1(1090)
bm2(930)=bm1(1091)
bm2(931)=bm1(1092)
bm2(932)=bm1(1093)
bm2(933)=bm1(1094)
bm2(934)=bm1(1095)
bm2(935)=bm1(1096)
bm2(936)=bm1(1097)
bm2(937)=bm1(1098)
bm2(938)=bm1(1099)
bm2(939)=bm1(1100)
bm2(940)=bm1(1101)
bm2(941)=bm1(1102)
bm2(942)=bm1(1103)
bm2(943)=bm1(1104)
bm2(944)=bm1(1105)
bm2(945)=bm1(1106)
bm2(946)=bm1(1107)
bm2(947)=bm1(1108)
bm2(948)=bm1(1109)
bm2(949)=bm1(1110)
bm2(950)=bm1(1111)
bm2(951)=bm1(1112)
bm2(952)=bm1(1113)
bm2(953)=bm1(1114)
bm2(954)=bm1(1115)
bm2(955)=bm1(1116)
bm2(956)=bm1(1117)
bm2(957)=bm1(1118)
bm2(958)=bm1(1119)
bm2(959)=bm1(1120)
bm2(960)=bm1(1121)
bm2(961)=bm1(1122)
bm2(962)=bm1(1123)
bm2(963)=bm1(1124)
bm2(964)=bm1(1125)
bm2(965)=bm1(1126)
bm2(966)=bm1(1127)
bm2(967)=bm1(1128)
bm2(968)=bm1(1129)
bm2(969)=bm1(1130)
bm2(970)=bm1(1131)
bm2(971)=bm1(1132)
bm2(972)=bm1(1133)
bm2(973)=bm1(1134)
bm2(974)=bm1(1135)
bm2(975)=bm1(1136)
bm2(976)=bm1(1137)
bm2(977)=bm1(1138)
bm2(978)=bm1(1139)
bm2(979)=bm1(1140)
bm2(980)=bm1(1141)
bm2(981)=bm1(1142)
bm2(982)=bm1(1143)
bm2(983)=bm1(1144)
bm2(984)=bm1(1145)
bm2(985)=bm1(1146)
bm2(986)=bm1(1147)
bm2(987)=bm1(1148)
bm2(988)=bm1(1149)
bm2(989)=bm1(1150)
bm2(990)=bm1(1151)
bm2(991)=bm1(1152)
bm2(992)=bm1(1153)
bm2(993)=bm1(1154)
bm2(994)=bm1(1155)
bm2(995)=bm1(1156)
bm2(996)=bm1(1157)
bm2(997)=bm1(1158)
bm2(998)=bm1(1159)
bm2(999)=bm1(1160)
bm2(1000)=bm1(1161)
bm2(1001)=bm1(1162)
bm2(1002)=bm1(1163)
bm2(1003)=bm1(1164)
bm2(1004)=bm1(1165)
bm2(1005)=bm1(1166)
bm2(1006)=bm1(1167)
bm2(1007)=bm1(1168)
bm2(1008)=bm1(1169)
bm2(1009)=bm1(1170)
bm2(1010)=bm1(1171)
bm2(1011)=bm1(1172)
bm2(1012)=bm1(1173)
bm2(1013)=bm1(1174)
bm2(1014)=bm1(1175)
bm2(1015)=bm1(1176)
bm2(1016)=bm1(1177)
bm2(1017)=bm1(1178)
bm2(1018)=bm1(1179)
bm2(1019)=bm1(1180)
bm2(1020)=bm1(1181)
bm2(1021)=bm1(1182)
bm2(1022)=bm1(1183)
bm2(1023)=bm1(1184)
bm2(1024)=bm1(1185)
bm2(1025)=bm1(1186)
bm2(1026)=bm1(1187)
bm2(1027)=bm1(1188)
bm2(1028)=bm1(1189)
bm2(1029)=bm1(1190)
bm2(1030)=bm1(1191)
bm2(1031)=bm1(1192)
bm2(1032)=bm1(1193)
bm2(1033)=bm1(1194)
bm2(1034)=bm1(1195)
bm2(1035)=bm1(1196)
bm2(1036)=bm1(1197)
bm2(1037)=bm1(1198)
bm2(1038)=bm1(1199)
bm2(1039)=bm1(1200)
bm2(1040)=bm1(1201)
bm2(1041)=bm1(1202)
bm2(1042)=bm1(1203)
bm2(1043)=bm1(1204)
bm2(1044)=bm1(1205)
bm2(1045)=bm1(1206)
bm2(1046)=bm1(1207)
bm2(1047)=bm1(1208)
bm2(1048)=bm1(1209)
bm2(1049)=bm1(1210)
bm2(1050)=bm1(1211)
bm2(1051)=bm1(1212)
bm2(1052)=bm1(1213)
bm2(1053)=bm1(1214)
bm2(1054)=bm1(1215)
bm2(1055)=bm1(1216)
bm2(1056)=bm1(1217)
bm2(1057)=bm1(1218)
bm2(1058)=bm1(1219)
bm2(1059)=bm1(1220)
bm2(1060)=bm1(1221)
bm2(1061)=bm1(1222)
bm2(1062)=bm1(1223)
bm2(1063)=bm1(1224)
bm2(1064)=bm1(1225)
bm2(1065)=bm1(1226)
bm2(1066)=bm1(1227)
bm2(1067)=bm1(1228)
bm2(1068)=bm1(1229)
bm2(1069)=bm1(1230)
bm2(1070)=bm1(1231)
bm2(1071)=bm1(1232)
bm2(1072)=bm1(1233)
bm2(1073)=bm1(1234)
bm2(1074)=bm1(1235)
bm2(1075)=bm1(1236)
bm2(1076)=bm1(1237)
bm2(1077)=bm1(1238)
bm2(1078)=bm1(1239)
bm2(1079)=bm1(1240)
bm2(1080)=bm1(1241)
bm2(1081)=bm1(1242)
bm2(1082)=bm1(1243)
bm2(1083)=bm1(1244)
bm2(1084)=bm1(1245)
bm2(1085)=bm1(1246)
bm2(1086)=bm1(1247)
bm2(1087)=bm1(1248)
bm2(1088)=bm1(1249)
bm2(1089)=bm1(1250)
bm2(1090)=bm1(1251)
bm2(1091)=bm1(1252)
bm2(1092)=bm1(1253)
bm2(1093)=bm1(1254)
bm2(1094)=bm1(1255)
bm2(1095)=bm1(1256)
bm2(1096)=bm1(1257)
bm2(1097)=bm1(1258)
bm2(1098)=bm1(1259)
bm2(1099)=bm1(1260)
bm2(1100)=bm1(1261)
bm2(1101)=bm1(1262)
bm2(1102)=bm1(1263)
bm2(1103)=bm1(1264)
bm2(1104)=bm1(1265)
bm2(1105)=bm1(1266)
bm2(1106)=bm1(1267)
bm2(1107)=bm1(1268)
bm2(1108)=bm1(1269)
bm2(1109)=bm1(1270)
bm2(1110)=bm1(1271)
bm2(1111)=bm1(1272)
bm2(1112)=bm1(1273)
bm2(1113)=bm1(1274)
bm2(1114)=bm1(1275)
bm2(1115)=bm1(1276)
bm2(1116)=bm1(1277)
bm2(1117)=bm1(1278)
bm2(1118)=bm1(1279)
bm2(1119)=bm1(1280)
bm2(1120)=bm1(1281)
bm2(1121)=bm1(1282)
bm2(1122)=bm1(1283)
bm2(1123)=bm1(1284)
bm2(1124)=bm1(1285)
bm2(1125)=bm1(1286)
bm2(1126)=bm1(1287)
bm2(1127)=bm1(1288)
bm2(1128)=bm1(1289)
bm2(1129)=bm1(1290)
bm2(1130)=bm1(1291)
bm2(1131)=bm1(1292)
bm2(1132)=bm1(1293)
bm2(1133)=bm1(1294)
bm2(1134)=bm1(1295)
bm2(1135)=bm1(1296)
bm2(1136)=bm1(1297)
bm2(1137)=bm1(1298)
bm2(1138)=bm1(1299)
bm2(1139)=bm1(1300)
bm2(1140)=bm1(1301)
bm2(1141)=bm1(1302)
bm2(1142)=bm1(1303)
bm2(1143)=bm1(1304)
bm2(1144)=bm1(1305)
bm2(1145)=bm1(1306)
bm2(1146)=bm1(1307)
bm2(1147)=bm1(1308)
bm2(1148)=bm1(1309)
bm2(1149)=bm1(1310)
bm2(1150)=bm1(1311)
bm2(1151)=bm1(1312)
bm2(1152)=bm1(1313)
bm2(1153)=bm1(1314)
bm2(1154)=bm1(1315)
bm2(1155)=bm1(1316)
bm2(1156)=bm1(1317)
bm2(1157)=bm1(1318)
bm2(1158)=bm1(1319)
bm2(1159)=bm1(1320)
bm2(1160)=bm1(1321)
bm2(1161)=bm1(1322)
bm2(1162)=bm1(1323)
bm2(1163)=bm1(1324)
bm2(1164)=bm1(1325)
bm2(1165)=bm1(1326)
bm2(1166)=bm1(1327)
bm2(1167)=bm1(1328)
bm2(1168)=bm1(1329)
bm2(1169)=bm1(1330)
bm2(1170)=bm1(1331)
bm2(1171)=bm1(1332)
bm2(1172)=bm1(1333)
bm2(1173)=bm1(1334)
bm2(1174)=bm1(1335)
bm2(1175)=bm1(1336)
bm2(1176)=bm1(1337)
bm2(1177)=bm1(1338)
bm2(1178)=bm1(1339)
bm2(1179)=bm1(1340)
bm2(1180)=bm1(1341)
bm2(1181)=bm1(1342)
bm2(1182)=bm1(1343)
bm2(1183)=bm1(1344)
bm2(1184)=bm1(1345)
bm2(1185)=bm1(1346)
bm2(1186)=bm1(1347)
bm2(1187)=bm1(1348)
bm2(1188)=bm1(1349)
bm2(1189)=bm1(1350)
bm2(1190)=bm1(1351)
bm2(1191)=bm1(1352)
bm2(1192)=bm1(1353)
bm2(1193)=bm1(1354)
bm2(1194)=bm1(1355)
bm2(1195)=bm1(1356)
bm2(1196)=bm1(1357)
bm2(1197)=bm1(1358)
bm2(1198)=bm1(1359)
bm2(1199)=bm1(1360)
bm2(1200)=bm1(1361)
bm2(1201)=bm1(1362)
bm2(1202)=bm1(1363)
bm2(1203)=bm1(1364)
bm2(1204)=bm1(1365)
bm2(1205)=bm1(1366)
bm2(1206)=bm1(1367)
bm2(1207)=bm1(1368)
bm2(1208)=bm1(1369)
bm2(1209)=bm1(1371)
bm2(1210)=bm1(1372)
bm2(1211)=bm1(1373)
bm2(1212)=bm1(1374)
bm2(1213)=bm1(1375)
bm2(1214)=bm1(1376)
bm2(1215)=bm1(1377)
bm2(1216)=bm1(1378)
bm2(1217)=bm1(1379)
bm2(1218)=bm1(1380)
bm2(1219)=bm1(1381)
bm2(1220)=bm1(1382)
bm2(1221)=bm1(1383)
bm2(1222)=bm1(1385)
bm2(1223)=bm1(1387)
bm2(1224)=bm1(1388)
bm2(1225)=bm1(1389)
bm2(1226)=bm1(1390)
bm2(1227)=bm1(1391)
bm2(1228)=bm1(1392)
bm2(1229)=bm1(1393)
bm2(1230)=bm1(1394)
bm2(1231)=bm1(1395)
bm2(1232)=bm1(1396)
bm2(1233)=bm1(1397)
bm2(1234)=bm1(1399)
bm2(1235)=bm1(1400)
bm2(1236)=bm1(1402)
bm2(1237)=bm1(1404)
bm2(1238)=bm1(1405)
bm2(1239)=bm1(1406)
bm2(1240)=bm1(1407)
bm2(1241)=bm1(1408)
bm2(1242)=bm1(1409)
bm2(1243)=bm1(1410)
bm2(1244)=bm1(1411)
bm2(1245)=bm1(1412)
bm2(1246)=bm1(1413)
bm2(1247)=bm1(1415)
bm2(1248)=bm1(1416)
bm2(1249)=bm1(1418)
bm2(1250)=bm1(1419)
bm2(1251)=bm1(1421)
bm2(1252)=bm1(1422)
bm2(1253)=bm1(1424)
bm2(1254)=bm1(1425)
bm2(1255)=bm1(1426)
bm2(1256)=bm1(1427)
bm2(1257)=bm1(1428)
bm2(1258)=bm1(1429)
bm2(1259)=bm1(1430)
bm2(1260)=bm1(1431)
bm2(1261)=bm1(1432)
bm2(1262)=bm1(1434)
bm2(1263)=bm1(1435)
bm2(1264)=bm1(1437)
bm2(1265)=bm1(1438)
bm2(1266)=bm1(1440)
bm2(1267)=bm1(1441)
bm2(1268)=bm1(1442)
bm2(1269)=bm1(1444)
bm2(1270)=bm1(1445)
bm2(1271)=bm1(1446)
bm2(1272)=bm1(1447)
bm2(1273)=bm1(1448)
bm2(1274)=bm1(1449)
bm2(1275)=bm1(1450)
bm2(1276)=bm1(1451)
bm2(1277)=bm1(1453)
bm2(1278)=bm1(1454)
bm2(1279)=bm1(1456)
bm2(1280)=bm1(1457)
bm2(1281)=bm1(1458)
bm2(1282)=bm1(1460)
bm2(1283)=bm1(1461)
bm2(1284)=bm1(1462)
bm2(1285)=bm1(1463)
bm2(1286)=bm1(1464)
bm2(1287)=bm1(1465)
bm2(1288)=bm1(1467)
bm2(1289)=bm1(1468)
bm2(1290)=bm1(1470)
bm2(1291)=bm1(1471)
bm2(1292)=bm1(1472)
bm2(1293)=bm1(1473)
bm2(1294)=bm1(1474)
bm2(1295)=bm1(1476)
bm2(1296)=bm1(1477)
bm2(1297)=bm1(1478)
bm2(1298)=bm1(1479)
bm2(1299)=bm1(1480)
bm2(1300)=bm1(1481)

do j=1,1300
  q(j)=bm2(j)
enddo

end subroutine ev_pol_con_10
!===========================================================================
! *def u11_prepot_8
! subtoutine of computing u11 and gradient of u11
!===========================================================================
subroutine u11_prepot_8
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp) :: xyz(4,3)
real(dp) :: v, r12, r13, r14, r23, r24, r34
real(dp) :: a1, a2, b1, b2, oh, hh, ra1, ra2, rb1, rb2, grad_hh, grad_oh
integer, parameter :: ncoeff=546
real(dp) :: coeff(ncoeff), p(ncoeff)
real(dp) :: dpdx1(ncoeff), dpdx2(ncoeff), dpdx3(ncoeff) ,dpdx4(ncoeff), dpdx5(ncoeff) ,dpdx6(ncoeff)
real(dp) :: x(6), re_hh, re_oh, ra_hh, ra_oh, rb_hh, rb_oh
real(dp) :: dx1, dx2, dx3, dx4, dx5, dx6, dr1, dr2, dr3, dr4, dr5, dr6, dxyz(4,3)
integer ::  ncall
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2

if(ncall.eq.1) then

a1 = 8.0000000000000004E-01
a2 = 1.2000000000000000E+00
b1 = 1.0000000000000000E+00
b2 = 1.5000000000000000E+00
ra1 = 1.2000000000000000E+00
ra2 = 1.5000000000000000E+00
rb1 = 1.2000000000000000E+00
rb2 = 1.5000000000000000E+00
coeff(   1)=-7.646796671713911e+01
coeff(   2)= 7.7876317743884882E-01
coeff(   3)= 1.1794040543902753E+00
coeff(   4)=-9.9262043560615387E-01
coeff(   5)= 7.3849271289436862E+00
coeff(   6)=-4.6924898339223180E+00
coeff(   7)= 3.5859245772359882E-01
coeff(   8)= 1.4607400965298865E+00
coeff(   9)= 2.0645068421926487E+00
coeff(  10)= 1.7673791990484615E+00
coeff(  11)=-8.5725950176296628E-01
coeff(  12)=-3.3118498797192615E+00
coeff(  13)=-4.2710722465321709E+00
coeff(  14)= 3.7067369789810862E+00
coeff(  15)= 1.4933201830433001E+01
coeff(  16)=-1.6777983005494383E+00
coeff(  17)=-1.9801918184108036E+00
coeff(  18)=-5.4155210042106834E+00
coeff(  19)=-1.4173211594042272E+01
coeff(  20)= 3.8648454355370623E+00
coeff(  21)= 1.0087835988323341E+01
coeff(  22)=-4.8635344866169259E+00
coeff(  23)=-4.1470221111056560E-01
coeff(  24)=-1.1618552387909981E+01
coeff(  25)= 3.9673172144122071E+00
coeff(  26)= 1.0055459592728800E+01
coeff(  27)=-3.3281076750465979E+00
coeff(  28)= 5.1478488754645835E-01
coeff(  29)=-8.7017439756100212E-01
coeff(  30)=-8.2546834005169301E-01
coeff(  31)=-2.2747986805382698E+00
coeff(  32)=-2.7726795980628887E+00
coeff(  33)= 5.0089463382025601E+00
coeff(  34)= 1.1731137407529850E+01
coeff(  35)=-8.5126838478299049E+00
coeff(  36)= 3.0195435714756620E+00
coeff(  37)=-6.5376846650176308E+00
coeff(  38)= 1.1654556143413575E+00
coeff(  39)= 3.5088366403160758E+00
coeff(  40)=-3.9077754833104237E+01
coeff(  41)=-5.0548258872587635E+00
coeff(  42)= 3.9492299120195691E+00
coeff(  43)= 2.4813707317304292E+00
coeff(  44)= 3.1195068141615860E+00
coeff(  45)= 6.9303576156200197E+00
coeff(  46)=-1.3930908138507782E+01
coeff(  47)= 1.9713100927107039E+00
coeff(  48)= 4.0940951042857261E+00
coeff(  49)=-5.8420820734681298E+00
coeff(  50)= 1.8279530492278704E+00
coeff(  51)=-7.4316544574244303E-01
coeff(  52)= 2.9867073493431047E+01
coeff(  53)= 4.5525662231504942E+00
coeff(  54)= 3.7746419054920297E+00
coeff(  55)=-8.1513398808223396E+00
coeff(  56)= 5.9974793465990439E+00
coeff(  57)=-8.9796088126629297E+00
coeff(  58)= 6.8799889721016996E+00
coeff(  59)=-6.5229489584762179E-01
coeff(  60)= 1.9875602353473138E+01
coeff(  61)= 3.1875836495753873E+00
coeff(  62)=-9.7146249198526533E+00
coeff(  63)=-7.3375121728996744E+00
coeff(  64)=-1.7490679705742216E+00
coeff(  65)=-4.7428616039413116E+00
coeff(  66)= 7.4179982248624770E+00
coeff(  67)= 2.3982203066027874E+00
coeff(  68)= 4.9728537609330310E+00
coeff(  69)= 4.7605803678815146E+00
coeff(  70)=-2.5213197624313608E+00
coeff(  71)=-1.0780808767301369E+01
coeff(  72)= 7.4616947290754441E+00
coeff(  73)=-1.1191757772688939E+00
coeff(  74)=-6.2212481047878248E+00
coeff(  75)=-3.8625853205490168E-01
coeff(  76)= 9.4778981969106333E+00
coeff(  77)= 5.5893285319008292E+00
coeff(  78)=-1.7604665918466655E+00
coeff(  79)=-1.8066929342184370E+01
coeff(  80)=-4.4165812619390250E+00
coeff(  81)= 9.5362788459258834E+00
coeff(  82)=-7.6158249722126503E-01
coeff(  83)= 6.7189434363019256E+00
coeff(  84)=-2.8382230822992125E+00
coeff(  85)=-9.6516495250128986E-01
coeff(  86)= 1.0927147938077065E+01
coeff(  87)=-4.1897185669541415E+00
coeff(  88)=-4.8171787866259681E+00
coeff(  89)= 8.0561081574993432E-01
coeff(  90)= 2.3793862582344869E+00
coeff(  91)= 3.7351467920676107E+01
coeff(  92)= 1.4731683579267752E+01
coeff(  93)= 6.2559824067578580E+00
coeff(  94)=-6.3677169148211341E+00
coeff(  95)= 6.0199557712761758E+00
coeff(  96)=-5.5903915893213325E+00
coeff(  97)=-2.4879752174649763E+00
coeff(  98)=-5.8304220613066038E+00
coeff(  99)= 7.6753632863976518E-01
coeff( 100)= 2.9075709234165230E+00
coeff( 101)= 1.6556729395302071E+01
coeff( 102)= 6.4405268926810217E+00
coeff( 103)=-3.6100507039206717E+00
coeff( 104)=-3.5307655383311980E+00
coeff( 105)=-8.5766602125652245E+00
coeff( 106)= 3.5254298302696778E+00
coeff( 107)= 3.9306169130909963E-01
coeff( 108)= 2.0907540069669492E+00
coeff( 109)=-4.4607217322813479E+00
coeff( 110)= 4.6996102176355881E-01
coeff( 111)= 2.8527200847989360E+00
coeff( 112)=-4.6020447664945090E+00
coeff( 113)=-1.5648478327955786E+00
coeff( 114)=-2.2039076154778683E+00
coeff( 115)= 7.4610349294295535E+00
coeff( 116)=-9.3083293996950833E-01
coeff( 117)= 1.8779043578917929E+00
coeff( 118)= 7.1200611336737198E+00
coeff( 119)= 8.0820182064413126E-01
coeff( 120)= 6.8993823001087495E+00
coeff( 121)=-4.2004003554222344E+01
coeff( 122)=-1.5899228447851129E+01
coeff( 123)= 2.6126184650531306E+00
coeff( 124)=-5.1210463987460288E+00
coeff( 125)=-9.8785783819390165E+00
coeff( 126)= 1.3483337308185991E+01
coeff( 127)=-3.7524701084777909E+00
coeff( 128)= 3.9020905694305581E+00
coeff( 129)=-2.7394798543854995E+00
coeff( 130)= 5.0276723809242685E-01
coeff( 131)=-1.3787009015122454E+01
coeff( 132)=-4.4886123915308183E+00
coeff( 133)= 1.2793999833037428E+00
coeff( 134)=-3.2077147497486278E+00
coeff( 135)= 5.3800582888746193E+00
coeff( 136)= 1.4519969000808164E+00
coeff( 137)= 2.6165863586326017E+00
coeff( 138)= 4.1187350360894666E+00
coeff( 139)=-1.0435605347701113E+01
coeff( 140)= 3.7323501128054897E+00
coeff( 141)=-2.3684732456651045E+00
coeff( 142)=-3.4789373800425971E+00
coeff( 143)=-4.0941376072835292E+00
coeff( 144)= 5.4140219214855225E+00
coeff( 145)= 1.1258327512094816E+01
coeff( 146)=-2.9602300690068040E+00
coeff( 147)= 2.8333421768287033E+00
coeff( 148)=-3.8613097952341882E+00
coeff( 149)=-1.6597325765941853E+00
coeff( 150)=-3.2922926622753210E+00
coeff( 151)=-1.7493051382273999E+00
coeff( 152)=-3.1372062180053342E+00
coeff( 153)= 1.5825625898597855E+00
coeff( 154)= 2.6545430109186480E-01
coeff( 155)=-6.5603649375763196E+00
coeff( 156)= 6.8646942172870551E-01
coeff( 157)= 6.9842138816057240E+00
coeff( 158)= 1.6780619157977789E-01
coeff( 159)=-1.2875637142352645E+01
coeff( 160)=-8.3782757280653009E+00
coeff( 161)=-3.7270413195605130E+00
coeff( 162)=-4.0406000164111209E+00
coeff( 163)= 1.3879428252515750E+01
coeff( 164)= 8.3303955076651270E+00
coeff( 165)=-2.2601872008733670E+00
coeff( 166)=-2.5567787439729930E+00
coeff( 167)=-8.9384005152887127E+00
coeff( 168)= 6.9080335669656812E-01
coeff( 169)=-3.0549911456064862E+00
coeff( 170)= 6.4657453646137997E+00
coeff( 171)=-3.3269603616862553E+00
coeff( 172)=-5.1960286548806520E+00
coeff( 173)=-4.4491049919986836E-01
coeff( 174)= 1.9365049340590410E+00
coeff( 175)= 5.3489316172970742E+00
coeff( 176)=-6.0750318748367551E+00
coeff( 177)=-4.6758546712791080E+00
coeff( 178)=-2.5965464508369678E+00
coeff( 179)= 8.7989288293501158E+00
coeff( 180)= 2.3909340454155061E+00
coeff( 181)=-9.6650231933087341E+00
coeff( 182)= 1.4364138875414287E-01
coeff( 183)= 7.6813064539931597E-01
coeff( 184)=-3.5298653082913565E+00
coeff( 185)= 7.6211829402516207E+00
coeff( 186)= 4.9749948967296413E+00
coeff( 187)= 1.7559766550519371E+00
coeff( 188)=-1.7228206650670121E+00
coeff( 189)=-9.6942416063253471E+00
coeff( 190)=-1.3115682095782846E+01
coeff( 191)=-1.2809267414048096E+01
coeff( 192)=-5.0726180160487222E-01
coeff( 193)=-9.4481668735382107E-01
coeff( 194)=-3.2741035986426907E+00
coeff( 195)= 5.9027956517890132E+00
coeff( 196)= 1.7416851562568687E+00
coeff( 197)=-4.7127891291886677E+00
coeff( 198)= 8.8832995552776186E+00
coeff( 199)=-2.4998488150012328E+00
coeff( 200)= 1.8253213937789421E+00
coeff( 201)=-1.3728599476993952E+00
coeff( 202)=-2.1652410779786462E+00
coeff( 203)=-5.6141437366207683E+00
coeff( 204)=-2.8390976787496744E+00
coeff( 205)= 7.2949555712110226E-01
coeff( 206)=-1.2591651952170917E+00
coeff( 207)= 1.0119579969012118E+01
coeff( 208)=-6.6300031109047204E+00
coeff( 209)= 5.8860235656639164E+00
coeff( 210)= 5.6412544841438370E+00
coeff( 211)=-5.8683183971901309E+00
coeff( 212)=-2.7609246403416168E+00
coeff( 213)= 5.8183421518863145E+00
coeff( 214)=-2.2671092477278132E-01
coeff( 215)=-4.6261813893620891E+00
coeff( 216)= 1.0547119583374050E+01
coeff( 217)=-1.1151533996889360E+00
coeff( 218)=-1.9263891475367472E+00
coeff( 219)= 2.3150923926264246E+00
coeff( 220)=-1.0784009889477499E+00
coeff( 221)= 5.1949987973148541E+00
coeff( 222)= 9.7929203045815916E-01
coeff( 223)= 2.8913056302651476E+00
coeff( 224)=-9.3119066489916147E+00
coeff( 225)=-4.1836761618622669E+00
coeff( 226)= 1.0000182484864670E+00
coeff( 227)= 1.3020094471018144E+00
coeff( 228)=-4.7902365928830646E+00
coeff( 229)=-1.7651170968425023E+00
coeff( 230)=-8.2778175265012410E-02
coeff( 231)= 1.4437770618954013E+00
coeff( 232)=-2.9932856718150567E+00
coeff( 233)=-3.7518699510720199E+00
coeff( 234)=-1.5001115183149946E-01
coeff( 235)=-3.3039583983116561E-01
coeff( 236)= 5.6343743088055154E-01
coeff( 237)=-3.4113436567816664E+00
coeff( 238)= 1.8442772486685792E+00
coeff( 239)=-2.4617619479035562E+00
coeff( 240)= 1.8180973413378807E+00
coeff( 241)=-1.4652763366946138E+00
coeff( 242)= 2.3247382857749539E+00
coeff( 243)=-2.0490381961875310E+00
coeff( 244)= 1.6517371050916543E+00
coeff( 245)=-1.8945861189848290E+00
coeff( 246)= 1.4197521768987475E+01
coeff( 247)= 1.0227860521699098E+01
coeff( 248)= 2.4714338915708520E+00
coeff( 249)=-2.1252172487641428E+00
coeff( 250)= 2.1502321962333446E+00
coeff( 251)= 8.7335421547071430E+00
coeff( 252)= 1.5587236448527040E+00
coeff( 253)=-4.3329612143098934E+00
coeff( 254)=-8.4483618534259808E+00
coeff( 255)= 5.5910215373494943E+00
coeff( 256)=-7.9671161922111722E-01
coeff( 257)=-2.5812353254654710E+00
coeff( 258)=-2.2727620925938159E+00
coeff( 259)= 1.9362952165946561E+00
coeff( 260)= 8.1825528568747319E-01
coeff( 261)= 1.2228188206418691E+00
coeff( 262)=-2.9203038218499495E+00
coeff( 263)= 3.3057282976074855E+00
coeff( 264)= 2.1631341519303975E+00
coeff( 265)=-2.8617504421872192E-01
coeff( 266)= 3.8487014422909693E+00
coeff( 267)=-6.4720424315747294E-01
coeff( 268)= 6.2533445444193232E+00
coeff( 269)=-5.2525624939286555E+00
coeff( 270)= 1.9171150093015745E+00
coeff( 271)= 3.0023685088967529E+00
coeff( 272)= 3.0454765566965363E+00
coeff( 273)=-5.5783610059733730E+00
coeff( 274)=-5.3286248122447697E+00
coeff( 275)=-2.5519667379108211E+00
coeff( 276)=-4.6415699811140145E+00
coeff( 277)=-1.3843350960524257E+00
coeff( 278)= 1.2568194672154629E+00
coeff( 279)= 1.3206427999331514E+00
coeff( 280)= 1.1934049224050081E+00
coeff( 281)=-1.0528189029097546E-01
coeff( 282)= 1.5874027809315721E+00
coeff( 283)= 3.1846094732095009E-01
coeff( 284)= 2.5469272388509774E+00
coeff( 285)= 1.8088992332444480E+00
coeff( 286)= 8.3321787701683547E-01
coeff( 287)=-1.1788743194924431E+00
coeff( 288)=-2.6327898257520181E+00
coeff( 289)= 5.4034463120082590E+00
coeff( 290)=-7.8078332425914398E-01
coeff( 291)= 1.1600812038406143E+00
coeff( 292)= 3.1694663153761027E-01
coeff( 293)= 1.1052814983280606E+00
coeff( 294)=-5.2638030174444805E-01
coeff( 295)= 1.6112908962346402E+00
coeff( 296)=-9.1613313636539018E-01
coeff( 297)= 1.6400620562702519E+00
coeff( 298)= 1.7404543600098692E+00
coeff( 299)=-2.5912929889599823E-01
coeff( 300)=-3.0463560955139450E+00
coeff( 301)=-4.1511859691495884E-01
coeff( 302)= 5.0982279243415620E+00
coeff( 303)= 4.0446843322366028E+00
coeff( 304)= 5.7512013416151353E-01
coeff( 305)= 4.9305793226006474E+00
coeff( 306)=-4.6449325767067497E+00
coeff( 307)=-7.3199780620913435E+00
coeff( 308)= 2.0281422673377021E+00
coeff( 309)= 6.5555374018164092E-01
coeff( 310)= 5.5643266170266648E-01
coeff( 311)= 4.4940361786563612E+00
coeff( 312)=-2.9978362187623131E-01
coeff( 313)= 2.4434093071592811E-01
coeff( 314)= 1.3573658647185716E+00
coeff( 315)=-4.9443281461643549E+00
coeff( 316)= 7.7544014328702282E+00
coeff( 317)=-4.4297233602378165E+00
coeff( 318)= 2.6870553794686347E+00
coeff( 319)= 4.8494184741507396E-01
coeff( 320)= 1.7075168189168237E+00
coeff( 321)= 7.6187247498409916E-01
coeff( 322)=-1.3214538504323659E-01
coeff( 323)=-2.8302458897015237E+00
coeff( 324)= 9.1366555092535284E-01
coeff( 325)=-1.7413043385112468E+00
coeff( 326)=-2.1109533575279466E-01
coeff( 327)= 3.4689833355880766E+00
coeff( 328)= 2.7703551125447947E+00
coeff( 329)=-2.2690003664258169E+00
coeff( 330)=-2.5407868767486774E+00
coeff( 331)=-6.9414248929038502E-01
coeff( 332)= 2.5449476434499037E+00
coeff( 333)= 6.5811234464232693E-01
coeff( 334)=-2.1674302158088494E+00
coeff( 335)= 4.8298398294233182E+00
coeff( 336)= 2.8798403608334948E+00
coeff( 337)= 3.6986736936699360E+00
coeff( 338)=-1.8367260304337094E+00
coeff( 339)=-1.0231352822643182E-01
coeff( 340)=-1.4940488769621256E+00
coeff( 341)= 7.6006997046550484E-01
coeff( 342)=-1.3986425157315274E+00
coeff( 343)= 1.4732728942045498E-01
coeff( 344)=-4.8326418262866868E+00
coeff( 345)=-1.2919968515825375E+00
coeff( 346)=-8.9855993706162429E-02
coeff( 347)=-1.2383202696921563E+00
coeff( 348)= 1.4417897589335271E+00
coeff( 349)= 1.2054049204167190E+00
coeff( 350)= 1.9538252223384247E+00
coeff( 351)=-2.0155583802662499E-01
coeff( 352)=-1.8576879948008638E+00
coeff( 353)= 1.9939679798866389E-01
coeff( 354)= 1.0871534389480102E+00
coeff( 355)=-7.5293801205590663E-02
coeff( 356)= 4.3658129753519848E-01
coeff( 357)= 5.8755218192487535E-01
coeff( 358)= 9.1789720387516904E-01
coeff( 359)= 2.9834512880357429E+00
coeff( 360)=-1.9594723938006470E-01
coeff( 361)= 2.3400744644045992E+00
coeff( 362)=-4.7607498461875725E-01
coeff( 363)= 3.7103324074770203E+00
coeff( 364)=-7.1534060782063380E-01
coeff( 365)=-7.2391386096434951E-01
coeff( 366)=-1.1917307970240372E+00
coeff( 367)=-1.3722103653425142E+00
coeff( 368)=-7.6221564984241941E-01
coeff( 369)= 3.7531155275591246E+00
coeff( 370)=-4.6602472484944375E+00
coeff( 371)= 1.5024832311641464E-01
coeff( 372)= 2.0560421244715910E+00
coeff( 373)=-2.5605603756813235E-02
coeff( 374)=-7.0348560828721179E-02
coeff( 375)=-6.0585407991282669E-01
coeff( 376)= 3.5980111925846825E-01
coeff( 377)= 7.5703478900365062E-01
coeff( 378)= 2.5572330655714390E+00
coeff( 379)=-1.1449557563352037E+00
coeff( 380)=-1.0857477052327318E+00
coeff( 381)= 4.6123721093490794E-01
coeff( 382)=-4.6406347796997949E+00
coeff( 383)= 3.1486098953421643E-01
coeff( 384)= 2.1540642868104474E+00
coeff( 385)=-1.6424310828637037E+00
coeff( 386)=-2.2363305042164208E-01
coeff( 387)= 3.7691580716333042E+00
coeff( 388)= 1.9162893444514921E+00
coeff( 389)=-1.1525120253203878E+00
coeff( 390)=-2.9294178394215553E-01
coeff( 391)=-2.5865123590999359E+00
coeff( 392)= 6.8057864571834881E-02
coeff( 393)= 2.7773499281900831E+00
coeff( 394)=-5.0133361639494076E+00
coeff( 395)=-2.3735965259794267E-01
coeff( 396)= 2.0710942748848891E+00
coeff( 397)=-1.4253117772768555E+00
coeff( 398)=-6.5268145844462655E-01
coeff( 399)= 9.2320634454141770E-01
coeff( 400)=-2.2246688280929250E-01
coeff( 401)=-1.6208686366234168E+00
coeff( 402)=-1.4589000715481282E+00
coeff( 403)= 6.1274800262494045E-02
coeff( 404)= 1.5291790381915495E+00
coeff( 405)= 2.0080955456981886E-01
coeff( 406)=-2.1795522507916282E-01
coeff( 407)= 1.6658190875595975E+00
coeff( 408)= 6.9717833580073985E-01
coeff( 409)= 6.4352370933934844E-01
coeff( 410)=-1.1582361141975122E+00
coeff( 411)=-2.4855861552799365E-02
coeff( 412)=-2.5620433712715447E+00
coeff( 413)=-3.4437972546924889E-01
coeff( 414)= 5.7942511016929765E-01
coeff( 415)=-7.0238480687048999E-01
coeff( 416)=-7.9294174185022237E-01
coeff( 417)= 1.8260587936317600E+00
coeff( 418)= 7.6596972647940076E-01
coeff( 419)= 2.3115856759785381E-02
coeff( 420)= 1.1018986179417907E+00
coeff( 421)= 1.0509869050945277E+00
coeff( 422)=-2.8946317254779602E-01
coeff( 423)=-7.6535162966433223E-01
coeff( 424)= 2.0870284003082786E+00
coeff( 425)=-2.7577810945351229E+00
coeff( 426)= 6.3906852178622423E-02
coeff( 427)= 1.6035587275102672E-01
coeff( 428)= 3.5482737345720867E+00
coeff( 429)= 5.2025109760330601E-01
coeff( 430)= 6.3836313115553989E-02
coeff( 431)=-3.6832940601541519E-01
coeff( 432)=-4.8662031078343054E-01
coeff( 433)= 1.5068829622325488E+00
coeff( 434)= 1.8790818617288012E+00
coeff( 435)= 2.8887845679870477E-01
coeff( 436)=-5.2474962089972055E-01
coeff( 437)=-1.1412775770410516E-01
coeff( 438)= 1.1911790626406193E+00
coeff( 439)= 1.5074719859899188E+00
coeff( 440)= 2.1471159713521146E+00
coeff( 441)=-1.1137536609288261E+00
coeff( 442)=-2.7634568016488652E+00
coeff( 443)= 3.9752255844840256E-01
coeff( 444)= 2.4228023992482967E-01
coeff( 445)= 3.2446781863463961E-02
coeff( 446)=-9.7156401947406240E-03
coeff( 447)= 5.8942103761346432E-01
coeff( 448)=-6.4866021223308268E-01
coeff( 449)=-5.1357109161102088E-01
coeff( 450)=-1.1930374078838737E+00
coeff( 451)= 5.5288317735004167E-01
coeff( 452)=-1.0753124024563721E+00
coeff( 453)= 1.1838813013919236E-01
coeff( 454)=-1.4971533331259462E-01
coeff( 455)= 5.0219089540227080E-01
coeff( 456)=-2.7888176422034254E+00
coeff( 457)=-1.0853318309111692E+00
coeff( 458)=-2.0173243217542915E+00
coeff( 459)= 3.7055753660241364E-01
coeff( 460)= 3.8541145542111066E-01
coeff( 461)=-5.8934676288190124E-01
coeff( 462)=-8.3063863800221149E-01
coeff( 463)=-2.9448292524078008E+00
coeff( 464)= 1.0604387007197955E+00
coeff( 465)= 1.0247269356760331E+00
coeff( 466)= 1.9784113415905924E+00
coeff( 467)=-2.4914540204333759E+00
coeff( 468)=-8.6221815908529439E-02
coeff( 469)= 1.8187218898338242E+00
coeff( 470)= 1.5094782992958111E+00
coeff( 471)= 8.8883485594534917E-01
coeff( 472)= 3.4157011727848219E-01
coeff( 473)=-8.6083273227199142E-01
coeff( 474)= 6.5725035473876525E-01
coeff( 475)=-3.8662141520677089E+00
coeff( 476)= 4.0318691328739414E-03
coeff( 477)= 1.0852780030075413E+00
coeff( 478)=-1.1703347646957267E+00
coeff( 479)= 3.9995354148396223E-02
coeff( 480)=-2.5201468278233414E+00
coeff( 481)=-4.1643525700321776E-01
coeff( 482)=-8.9590768149195465E-01
coeff( 483)= 1.0139347353317918E+00
coeff( 484)=-6.3487685964092022E-01
coeff( 485)=-3.4235478864743152E-01
coeff( 486)=-7.5076270032017789E-01
coeff( 487)= 8.5893694234028539E-01
coeff( 488)= 1.5026761022471009E+00
coeff( 489)= 5.2187886604005806E-01
coeff( 490)=-1.0281468209206828E+00
coeff( 491)= 1.6229214389426507E+00
coeff( 492)= 3.2041967201846328E-01
coeff( 493)=-2.8088960298726262E-01
coeff( 494)= 1.2756205119937012E+00
coeff( 495)= 1.3515596132281463E-01
coeff( 496)=-7.7722727275176473E-01
coeff( 497)=-7.4107557289311943E-01
coeff( 498)= 2.9965532371321635E-01
coeff( 499)=-1.3727569485387697E-01
coeff( 500)=-5.3209896389364353E-01
coeff( 501)=-9.6019207382858196E-01
coeff( 502)=-2.9414058722115138E-01
coeff( 503)= 7.3009376654877245E-01
coeff( 504)= 8.7436963653200284E-01
coeff( 505)=-1.7147879750538586E-01
coeff( 506)=-1.1417773572048569E-01
coeff( 507)= 5.1494333471955067E-01
coeff( 508)=-1.2447353415133720E+00
coeff( 509)= 3.3877278323586857E-01
coeff( 510)=-2.7525353021279153E-01
coeff( 511)=-5.2804401619323782E-01
coeff( 512)= 4.6324405430049126E-01
coeff( 513)=-8.3374942400442714E-02
coeff( 514)=-5.1877759378297073E-01
coeff( 515)=-2.5365469784251748E-01
coeff( 516)= 7.5111041556451458E-02
coeff( 517)= 2.1853731019223233E-01
coeff( 518)= 2.9026757103752071E-01
coeff( 519)=-1.1908878969504679E+00
coeff( 520)=-2.9115976217173062E-01
coeff( 521)= 2.1181448559603705E-01
coeff( 522)= 4.8161292738104260E-02
coeff( 523)=-2.3254529266092747E-01
coeff( 524)= 5.3715430496634098E-01
coeff( 525)=-3.8014612443896967E-01
coeff( 526)= 2.2187197188594807E-01
coeff( 527)=-4.6333313670876308E-01
coeff( 528)=-5.9727901811108139E-02
coeff( 529)= 8.8468844788676212E-02
coeff( 530)= 2.9304220623294852E-01
coeff( 531)= 8.6833853447909545E-02
coeff( 532)=-1.7013486867583769E+00
coeff( 533)= 6.8495592605266151E-02
coeff( 534)=-1.3020626651613874E+00
coeff( 535)= 2.5776875935500926E-01
coeff( 536)=-1.6687806579856557E+00
coeff( 537)= 5.4674794167953167E-01
coeff( 538)= 1.7418371550962481E+00
coeff( 539)= 6.3154191109838076E-01
coeff( 540)= 2.7059371539380317E-01
coeff( 541)=-1.1906474432004064E+00
coeff( 542)=-3.7028343813137332E-01
coeff( 543)= 3.3961923724707760E-01
coeff( 544)=-7.7494619194758485E-01
coeff( 545)=-1.0943981026761402E-02
coeff( 546)= 1.2391525461474276E-01
  return
end if

entry u11_pot_8(xyz,v, dxyz)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)
call ev_pol_con_8(x,p)
v=dot_product(coeff,p) +(oh(r14)+oh(r24)+oh(r34)+hh(r12)+hh(r13)+hh(r23))

call grad_ev_pol_con_8(x,dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6)

dx1 = dot_product(coeff,dpdx1)
dx2 = dot_product(coeff,dpdx2)
dx3 = dot_product(coeff,dpdx3)
dx4 = dot_product(coeff,dpdx4)
dx5 = dot_product(coeff,dpdx5)
dx6 = dot_product(coeff,dpdx6)

dr1=-x(1)*(1.0d0/(a1)+2.0d0*(r12-rb1)/(b1))*dx1 + grad_hh(r12)
dr2=-x(2)*(1.0d0/(a1)+2.0d0*(r13-rb1)/(b1))*dx2 + grad_hh(r13)
dr3=-x(3)*(1.0d0/(a2)+2.0d0*(r14-rb2)/(b2))*dx3 + grad_oh(r14)
dr4=-x(4)*(1.0d0/(a1)+2.0d0*(r23-rb1)/(b1))*dx4 + grad_hh(r23)
dr5=-x(5)*(1.0d0/(a2)+2.0d0*(r24-rb2)/(b2))*dx5 + grad_oh(r24)
dr6=-x(6)*(1.0d0/(a2)+2.0d0*(r34-rb2)/(b2))*dx6 + grad_oh(r34)

dxyz(1,1)=dr1/r12*(xyz(1,1)-xyz(2,1))+dr2/r13*(xyz(1,1)-xyz(3,1))+dr3/r14*(xyz(1,1)-xyz(4,1))
dxyz(1,2)=dr1/r12*(xyz(1,2)-xyz(2,2))+dr2/r13*(xyz(1,2)-xyz(3,2))+dr3/r14*(xyz(1,2)-xyz(4,2))
dxyz(1,3)=dr1/r12*(xyz(1,3)-xyz(2,3))+dr2/r13*(xyz(1,3)-xyz(3,3))+dr3/r14*(xyz(1,3)-xyz(4,3))

dxyz(2,1)=-dr1/r12*(xyz(1,1)-xyz(2,1))+dr4/r23*(xyz(2,1)-xyz(3,1))+dr5/r24*(xyz(2,1)-xyz(4,1))
dxyz(2,2)=-dr1/r12*(xyz(1,2)-xyz(2,2))+dr4/r23*(xyz(2,2)-xyz(3,2))+dr5/r24*(xyz(2,2)-xyz(4,2))
dxyz(2,3)=-dr1/r12*(xyz(1,3)-xyz(2,3))+dr4/r23*(xyz(2,3)-xyz(3,3))+dr5/r24*(xyz(2,3)-xyz(4,3))

dxyz(3,1)=-dr2/r13*(xyz(1,1)-xyz(3,1))-dr4/r23*(xyz(2,1)-xyz(3,1))+dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(3,2)=-dr2/r13*(xyz(1,2)-xyz(3,2))-dr4/r23*(xyz(2,2)-xyz(3,2))+dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(3,3)=-dr2/r13*(xyz(1,3)-xyz(3,3))-dr4/r23*(xyz(2,3)-xyz(3,3))+dr6/r34*(xyz(3,3)-xyz(4,3))

dxyz(4,1)=-dr3/r14*(xyz(1,1)-xyz(4,1))-dr5/r24*(xyz(2,1)-xyz(4,1))-dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(4,2)=-dr3/r14*(xyz(1,2)-xyz(4,2))-dr5/r24*(xyz(2,2)-xyz(4,2))-dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(4,3)=-dr3/r14*(xyz(1,3)-xyz(4,3))-dr5/r24*(xyz(2,3)-xyz(4,3))-dr6/r34*(xyz(3,3)-xyz(4,3))

return

end  subroutine

!===========================================================================
! *def u22_prepot_8
! subtoutine of computing u22 and gradient of u22
!===========================================================================
subroutine u22_prepot_8
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp) :: xyz(4,3)
real(dp) :: v, r12, r13, r14, r23, r24, r34
real(dp) :: a1, a2, b1, b2, oh, hh, ra1, ra2, rb1, rb2, grad_hh, grad_oh
integer, parameter :: ncoeff=546
real(dp) :: coeff(ncoeff), p(ncoeff)
real(dp) :: dpdx1(ncoeff), dpdx2(ncoeff), dpdx3(ncoeff) ,dpdx4(ncoeff), dpdx5(ncoeff) ,dpdx6(ncoeff)
real(dp) :: x(6), re_hh, re_oh, ra_hh, ra_oh, rb_hh, rb_oh
integer ::  ncall
real(dp) :: dx1, dx2, dx3, dx4, dx5, dx6, dr1, dr2, dr3, dr4, dr5, dr6, dxyz(4,3)
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2

if(ncall.eq.1) then

a1 = 8.0000000000000004E-01
a2 = 1.2000000000000000E+00
b1 = 1.0000000000000000E+00
b2 = 1.5000000000000000E+00
ra1 = 1.2000000000000000E+00
ra2 = 1.5000000000000000E+00
rb1 = 1.2000000000000000E+00
rb2 = 1.5000000000000000E+00
coeff(   1)=-7.646796671713911e+01
coeff(   2)=-2.7502154550057121E+00
coeff(   3)= 6.9431829616266716E-01
coeff(   4)= 1.0367753562327259E-01
coeff(   5)= 2.4559149954612622E+01
coeff(   6)= 1.9510225980061695E+00
coeff(   7)=-3.9792941702953746E+00
coeff(   8)=-6.4468409945039706E-01
coeff(   9)=-2.0219982499309412E+00
coeff(  10)= 2.7560774470610929E+00
coeff(  11)= 1.0177373878409732E+01
coeff(  12)=-4.1697803501886348E+00
coeff(  13)=-9.1493076096472808E-02
coeff(  14)= 2.4900890965256328E-01
coeff(  15)= 2.0418720990507615E+00
coeff(  16)= 6.3984356625937688E-01
coeff(  17)= 4.0237693433863102E+00
coeff(  18)= 1.4599393928229316E+00
coeff(  19)=-3.9431088124088319E+01
coeff(  20)=-3.0797318824342397E+00
coeff(  21)=-3.1009910022915825E+00
coeff(  22)= 3.1229498908356490E-01
coeff(  23)=-6.5037470955087162E-02
coeff(  24)=-5.6668343258158211E+00
coeff(  25)=-7.8976196704897959E+00
coeff(  26)= 1.1384114244945872E+01
coeff(  27)= 2.2225559997049036E+00
coeff(  28)=-1.4778727299439454E+00
coeff(  29)= 4.2661720305421618E+00
coeff(  30)=-6.4484959353297064E+00
coeff(  31)=-2.1715229589260463E+01
coeff(  32)=-1.8413065160271465E+01
coeff(  33)= 1.0232230555038559E+01
coeff(  34)= 7.0117247233279283E+00
coeff(  35)=-3.6106075792038697E+00
coeff(  36)=-1.7325801605856246E+00
coeff(  37)= 3.5297508084557899E-01
coeff(  38)=-2.5266301578521855E+00
coeff(  39)=-2.6396610975397183E+00
coeff(  40)= 5.0283083905357646E+00
coeff(  41)= 3.3718940325582576E+00
coeff(  42)=-3.4732146286353909E+00
coeff(  43)= 2.8207146475680842E+00
coeff(  44)=-6.4792916897043797E+00
coeff(  45)=-1.0243416738269224E+00
coeff(  46)=-9.7466106122115335E+00
coeff(  47)= 2.0036250777444988E+00
coeff(  48)= 2.0789229304474031E+00
coeff(  49)=-5.5097996683472976E+00
coeff(  50)=-1.8662007351619485E+00
coeff(  51)=-2.1419894737944767E+00
coeff(  52)= 4.5863880317168586E+01
coeff(  53)= 2.9455711514513425E+01
coeff(  54)= 7.3019533159050987E-01
coeff(  55)=-1.6741765868245775E+00
coeff(  56)= 1.0734140215447077E+01
coeff(  57)= 4.2167022808008454E+00
coeff(  58)= 4.4264397862883245E+00
coeff(  59)= 1.7581495439925381E+00
coeff(  60)=-3.4274361112123675E-01
coeff(  61)= 1.3521080144211851E+01
coeff(  62)= 9.6384615527812088E+00
coeff(  63)=-1.2934959694085178E+01
coeff(  64)=-5.2678530961069443E+00
coeff(  65)= 3.6010654298052138E+00
coeff(  66)= 1.1417968962908249E+01
coeff(  67)= 1.0429400641362896E+00
coeff(  68)=-3.5378443810234850E+00
coeff(  69)= 5.8419932086660715E+00
coeff(  70)= 5.7316229251599085E+00
coeff(  71)=-7.6663902350226980E+00
coeff(  72)=-7.2151724091694158E-01
coeff(  73)= 4.7511436578519484E-01
coeff(  74)=-5.8267860248089640E+00
coeff(  75)= 6.4926287444551152E+00
coeff(  76)= 2.7280834693649055E+01
coeff(  77)= 1.5321782251353401E+01
coeff(  78)=-1.2210673841397089E+01
coeff(  79)=-1.3774866195298060E+01
coeff(  80)=-5.1359348417486164E+00
coeff(  81)= 4.8916628826083572E+00
coeff(  82)= 2.0190044690452460E+00
coeff(  83)=-1.5165259818282708E+00
coeff(  84)=-1.0211795034920851E+01
coeff(  85)= 2.4892942168584060E+00
coeff(  86)=-2.6981703288168655E+00
coeff(  87)= 2.4955863936004006E+00
coeff(  88)= 2.5668464185439186E+00
coeff(  89)= 4.0402220399973041E+00
coeff(  90)=-1.6171379865600191E+00
coeff(  91)=-9.6042596538362961E+00
coeff(  92)= 9.7245451153536777E-01
coeff(  93)=-5.8875160768946309E+00
coeff(  94)=-5.4534591289110148E+00
coeff(  95)= 9.5110963613618456E+00
coeff(  96)= 1.8890425358154115E+00
coeff(  97)=-5.1485135485621720E+00
coeff(  98)= 2.5893421847451554E+00
coeff(  99)= 3.4350988085856544E+00
coeff( 100)=-7.7281385745801368E-01
coeff( 101)= 4.4917461734825279E+00
coeff( 102)= 3.1379175653889160E+00
coeff( 103)=-3.6835538451654721E+00
coeff( 104)=-2.0079106426499118E+00
coeff( 105)=-2.3189814359907781E+00
coeff( 106)= 4.6220437842121704E+00
coeff( 107)=-2.5324791965310078E+00
coeff( 108)= 3.7082435819454060E+00
coeff( 109)= 2.8154644189277667E+00
coeff( 110)= 4.5505695306225225E-01
coeff( 111)=-1.5477446745778654E+00
coeff( 112)=-2.0653831332850978E+00
coeff( 113)= 2.0189279905596198E+00
coeff( 114)= 4.9460550737293705E+00
coeff( 115)= 2.2659099623653800E+00
coeff( 116)= 1.6840075146546840E+00
coeff( 117)= 1.4615963765336576E-01
coeff( 118)=-6.0963227154817523E-01
coeff( 119)= 1.9261108992852827E-02
coeff( 120)= 2.4316254246988184E-01
coeff( 121)=-5.5486775782534195E+01
coeff( 122)=-1.9613883991639533E+01
coeff( 123)=-1.5003879878965634E+01
coeff( 124)= 9.4162340673119460E-01
coeff( 125)= 4.8634690668032565E+00
coeff( 126)=-4.7631323349539905E+00
coeff( 127)=-1.2320198525204582E+01
coeff( 128)=-8.6215766767313573E-01
coeff( 129)=-5.7545529662969939E+00
coeff( 130)=-2.4930415351219950E+00
coeff( 131)= 4.7782105235954946E+00
coeff( 132)=-7.2195256266963304E+00
coeff( 133)=-9.3432137233592769E+00
coeff( 134)= 6.9177021911175398E+00
coeff( 135)=-2.2682749377872051E+00
coeff( 136)= 8.9862203265929583E+00
coeff( 137)= 4.4967307737697917E+00
coeff( 138)=-2.9730942317945290E+00
coeff( 139)=-3.8406730843943944E+00
coeff( 140)=-2.6901737321978242E-01
coeff( 141)=-4.0851009850898229E-01
coeff( 142)=-9.8778405689549356E+00
coeff( 143)=-7.3568675100195922E+00
coeff( 144)=-7.2082108606269042E+00
coeff( 145)= 7.4396023559245110E+00
coeff( 146)= 2.1358735147061827E+00
coeff( 147)=-1.7838018492938952E+00
coeff( 148)=-8.5773122324006987E+00
coeff( 149)=-1.3935623222366900E-01
coeff( 150)=-7.2203439156470817E-02
coeff( 151)= 3.1689827300800273E+00
coeff( 152)=-3.2498125433717000E+00
coeff( 153)=-9.4833228746206211E-01
coeff( 154)= 1.9550672819540464E+00
coeff( 155)=-1.1104291352053466E+00
coeff( 156)= 3.2293145263062356E-01
coeff( 157)= 2.9323305368172345E+00
coeff( 158)=-1.5728268573449442E+00
coeff( 159)=-1.5470215446775175E+01
coeff( 160)=-2.0130521271061205E+01
coeff( 161)=-4.6526149257778497E+00
coeff( 162)= 9.0254232848185545E+00
coeff( 163)= 8.6499282046804034E+00
coeff( 164)= 1.0170187348212657E+01
coeff( 165)=-5.0144083888358812E+00
coeff( 166)= 1.8866631281856858E+00
coeff( 167)=-3.2098351008160741E+00
coeff( 168)=-1.0647787940633091E+00
coeff( 169)= 1.9094348570676365E+00
coeff( 170)= 1.7044851339553748E+00
coeff( 171)= 4.1941659905806672E-01
coeff( 172)=-5.9072465259707370E+00
coeff( 173)=-1.9744526152389543E+00
coeff( 174)= 5.0082278282011128E+00
coeff( 175)= 2.0257633375610382E+00
coeff( 176)= 1.8335591043970825E+00
coeff( 177)=-2.3128380645145348E+00
coeff( 178)=-1.7675080056461197E+00
coeff( 179)= 2.7918213217392771E+00
coeff( 180)=-2.6475815710901900E+00
coeff( 181)=-5.4062438558654051E+00
coeff( 182)=-6.8276245297024629E-01
coeff( 183)= 2.6826687336734207E+00
coeff( 184)= 1.3771207191057426E+00
coeff( 185)= 1.3579075920241150E+00
coeff( 186)=-1.4506472750442725E+00
coeff( 187)= 2.8075476494003659E-01
coeff( 188)=-1.5425720481106606E+00
coeff( 189)=-8.4240036208304814E-01
coeff( 190)= 5.7158884946036688E+00
coeff( 191)=-1.5386527139868358E+00
coeff( 192)= 3.6917122827534428E+00
coeff( 193)=-1.3032296592405221E+00
coeff( 194)= 3.5271352553951663E+00
coeff( 195)= 6.2118024746959510E-01
coeff( 196)=-4.1736138102379803E+00
coeff( 197)= 4.0089764965990515E-01
coeff( 198)= 5.2066162989466647E-01
coeff( 199)= 3.5235955835096648E+00
coeff( 200)=-6.3775079790605309E+00
coeff( 201)= 3.5607043991707799E-01
coeff( 202)= 1.3528256658510198E+00
coeff( 203)= 4.5071025895820904E-02
coeff( 204)=-1.0199309774733838E+00
coeff( 205)=-7.9352113268857432E+00
coeff( 206)= 6.0494788254432397E+00
coeff( 207)= 3.4764699860124955E+00
coeff( 208)= 7.2752802875478584E-01
coeff( 209)=-2.0188449175462853E+00
coeff( 210)= 1.6029474108727668E+00
coeff( 211)=-1.0155617667574344E+00
coeff( 212)= 3.6573722255560619E+00
coeff( 213)=-3.3206077674452583E+00
coeff( 214)= 1.2517500432674067E+00
coeff( 215)= 8.4401004147904468E-01
coeff( 216)=-1.1927682870719720E+00
coeff( 217)=-3.6950386611637243E-01
coeff( 218)= 1.0212225908965520E+00
coeff( 219)=-1.0306433047823582E-01
coeff( 220)=-8.1284195316091390E-01
coeff( 221)= 1.4407093403661870E+00
coeff( 222)=-9.1945778306732173E-01
coeff( 223)=-1.8704562997446543E+00
coeff( 224)= 1.5418404318850856E-01
coeff( 225)=-1.9854925832635604E+00
coeff( 226)=-6.2087598617258477E-01
coeff( 227)= 2.0730609155379480E+00
coeff( 228)= 2.2354715809929129E+00
coeff( 229)=-8.5603676113197735E-01
coeff( 230)=-2.2155484771585430E+00
coeff( 231)=-1.6169469397373043E-01
coeff( 232)=-1.2435952659153997E+00
coeff( 233)= 7.7466344177749624E-01
coeff( 234)=-2.6664416120906370E-01
coeff( 235)= 1.1114470508384238E+00
coeff( 236)=-2.6130787485122686E-01
coeff( 237)=-2.1820371483444345E-02
coeff( 238)=-6.9118668803481054E-01
coeff( 239)=-2.0180014047091004E+00
coeff( 240)= 6.1975411207578635E-01
coeff( 241)=-2.4659358956008215E+00
coeff( 242)=-1.2342841850784860E+00
coeff( 243)= 1.9992789771158666E+00
coeff( 244)= 1.8923703757160224E+00
coeff( 245)= 1.0953461647048499E+00
coeff( 246)= 1.7846061058365311E+01
coeff( 247)= 1.2532686143083174E+00
coeff( 248)= 6.2876467453656142E+00
coeff( 249)= 4.0577712208222723E+00
coeff( 250)=-4.4430261017554074E-01
coeff( 251)=-2.3100945569260150E+00
coeff( 252)= 1.1235779150685283E+00
coeff( 253)= 1.1415740134108191E+01
coeff( 254)= 5.0251215105534390E+00
coeff( 255)= 2.8269190855468715E+00
coeff( 256)=-1.5405351785712416E+00
coeff( 257)= 4.7591835184820832E+00
coeff( 258)= 1.3537777007731706E+00
coeff( 259)=-2.1405254574641637E+00
coeff( 260)= 2.5239734178863271E-02
coeff( 261)= 3.4069430142973305E+00
coeff( 262)=-5.3750655331443498E-01
coeff( 263)=-1.3767444298591394E+00
coeff( 264)=-4.4731712104225441E-01
coeff( 265)=-4.5168253582982905E+00
coeff( 266)=-1.2603627572640967E+00
coeff( 267)= 2.2200479828376931E+00
coeff( 268)=-1.7647185180571927E+00
coeff( 269)=-2.1776876348783678E+00
coeff( 270)= 3.3841362862404409E+00
coeff( 271)= 4.6634774948091700E+00
coeff( 272)= 4.9052623584558361E+00
coeff( 273)=-4.5972285280175953E+00
coeff( 274)= 1.7370017222227183E+00
coeff( 275)=-3.0567272749206493E+00
coeff( 276)=-2.0245186358614928E+00
coeff( 277)= 2.6043080800238921E-01
coeff( 278)= 3.4040574655062779E+00
coeff( 279)=-2.3958683511046547E-01
coeff( 280)= 2.6733841605381681E+00
coeff( 281)=-2.7729772938229902E-01
coeff( 282)= 3.5800281686165802E+00
coeff( 283)=-2.0900608730894671E-01
coeff( 284)=-4.5797115636087926E-01
coeff( 285)= 1.3321669957070563E+00
coeff( 286)=-3.6492703446048974E-01
coeff( 287)= 2.1225267938082419E+00
coeff( 288)=-1.0039446710933724E+00
coeff( 289)= 7.3158202405926964E-01
coeff( 290)= 4.4138717841245212E-01
coeff( 291)= 2.1573327397025412E+00
coeff( 292)=-1.4113966366144376E-01
coeff( 293)=-9.3790187000861858E-01
coeff( 294)=-1.6267291208197612E+00
coeff( 295)= 5.9127690904256169E-01
coeff( 296)=-2.0311089186678420E-02
coeff( 297)=-3.7546735047960944E-01
coeff( 298)= 8.2195983391062455E-01
coeff( 299)=-2.9524184137972453E-01
coeff( 300)=-2.3621721785101732E-02
coeff( 301)=-9.5069580999225001E-01
coeff( 302)= 5.9379925048374096E+00
coeff( 303)= 6.4005733000604517E+00
coeff( 304)=-2.7082255143664091E-01
coeff( 305)=-4.3788809395458834E+00
coeff( 306)=-1.6689040428129411E+00
coeff( 307)=-4.7228512046006346E+00
coeff( 308)=-3.6914552179609532E+00
coeff( 309)= 1.9656601498132851E+00
coeff( 310)=-3.4820396342953569E-01
coeff( 311)= 1.2255058610945668E+00
coeff( 312)= 3.8115324548203128E-01
coeff( 313)=-1.1628457646600598E+00
coeff( 314)=-2.5328425926302849E+00
coeff( 315)= 3.4674634752346041E+00
coeff( 316)=-3.8297904155624085E-01
coeff( 317)= 1.3097021495315941E-02
coeff( 318)=-5.8363502629333042E-01
coeff( 319)= 1.2518527765414824E+00
coeff( 320)= 2.2926313558564324E+00
coeff( 321)= 1.2046542121465336E+00
coeff( 322)=-1.8662328923021447E+00
coeff( 323)=-3.1072595329350663E+00
coeff( 324)= 9.7293960221090559E-01
coeff( 325)= 1.0449380870017458E-01
coeff( 326)=-1.8846006411206275E+00
coeff( 327)=-4.0865416538513560E-01
coeff( 328)= 4.9107654887430879E-01
coeff( 329)=-1.8983312988980803E+00
coeff( 330)=-1.9859569438590137E+00
coeff( 331)= 1.7801795817554928E+00
coeff( 332)= 3.0317774161160922E+00
coeff( 333)= 1.0585134391879054E+00
coeff( 334)=-5.7848445293225870E-01
coeff( 335)= 8.3942679580676471E-01
coeff( 336)=-3.4198788071298419E-01
coeff( 337)=-7.0465854873622336E-01
coeff( 338)=-1.3330623848746299E+00
coeff( 339)=-1.4705835187178690E-01
coeff( 340)=-4.5320115964702268E-01
coeff( 341)= 1.8243917167861493E-01
coeff( 342)=-5.1525312503248977E-01
coeff( 343)=-5.6190068909233615E-01
coeff( 344)= 8.6420683785456698E-01
coeff( 345)= 4.0140896726395967E-02
coeff( 346)= 1.3371873729637198E+00
coeff( 347)=-1.8619546545321605E-01
coeff( 348)= 1.7229999922578809E-01
coeff( 349)= 3.1224442783619732E-01
coeff( 350)=-9.3324050520616419E-01
coeff( 351)= 1.8778710349764233E-01
coeff( 352)= 1.3141935606476959E-01
coeff( 353)= 3.4784032326948189E-01
coeff( 354)= 1.8381368776665738E-01
coeff( 355)=-1.0676256746145447E+00
coeff( 356)=-8.3865115781146846E-02
coeff( 357)= 4.3619304924538016E-01
coeff( 358)=-1.1890973208814262E+00
coeff( 359)=-2.2988947677592678E-01
coeff( 360)=-1.7786280259522091E+00
coeff( 361)= 3.0220796885569530E+00
coeff( 362)=-1.5003854650367887E+00
coeff( 363)=-1.0619906398363868E+00
coeff( 364)= 1.0305557750285466E-01
coeff( 365)= 4.7611755838713632E-01
coeff( 366)=-3.2487252263296251E-01
coeff( 367)= 1.5074408297552790E+00
coeff( 368)=-1.3328474367398721E-01
coeff( 369)=-2.6464869845202488E-01
coeff( 370)= 4.4044812798588628E-01
coeff( 371)=-1.0311025815385040E+00
coeff( 372)= 8.2818249798881161E-01
coeff( 373)= 5.3485149607650673E-01
coeff( 374)= 5.0346579511022682E-01
coeff( 375)=-5.0462017437063467E-02
coeff( 376)=-2.6909880447812695E-01
coeff( 377)=-1.0984579858486537E-01
coeff( 378)= 7.7505530024159803E+00
coeff( 379)=-4.4097621226088801E+00
coeff( 380)=-4.1997028950321214E-01
coeff( 381)=-4.8227881059969024E-01
coeff( 382)=-8.6263779366781412E-01
coeff( 383)=-8.8293789570217007E-01
coeff( 384)= 1.1679841011434891E+00
coeff( 385)= 9.6202164809922830E-01
coeff( 386)=-1.4381510708700715E+00
coeff( 387)= 1.4193364188980828E-01
coeff( 388)=-1.3574495378231037E+00
coeff( 389)= 9.3820657307126343E-01
coeff( 390)=-6.3516220847543525E-01
coeff( 391)= 4.0414234671096394E-01
coeff( 392)= 5.4086835438540815E-01
coeff( 393)=-5.5108739928192119E-01
coeff( 394)=-1.5985714476012849E-01
coeff( 395)= 7.2719543450442536E-01
coeff( 396)= 4.6138165299096440E-01
coeff( 397)=-3.4192089699803196E-01
coeff( 398)= 6.7209829916076783E-02
coeff( 399)=-3.1036690759257335E-01
coeff( 400)=-4.5021594742869214E-01
coeff( 401)=-1.7902348252368152E+00
coeff( 402)= 1.6034936299954534E+00
coeff( 403)= 1.4281912013423819E+00
coeff( 404)= 1.4771413707790917E-01
coeff( 405)=-2.2296060618326854E-01
coeff( 406)=-6.1004043709268652E-01
coeff( 407)= 1.0393565913335334E+00
coeff( 408)=-1.4532827809490223E-02
coeff( 409)=-8.3766779203299002E-02
coeff( 410)=-2.0601576446689154E-01
coeff( 411)=-3.6293451247375225E-01
coeff( 412)=-2.9357169480585382E-01
coeff( 413)= 1.4599124159602634E+00
coeff( 414)=-5.0981035808786712E-01
coeff( 415)=-4.9747940909221899E-01
coeff( 416)=-1.0393931082408084E-01
coeff( 417)= 1.4681839169392891E-01
coeff( 418)=-1.0506110113215004E+00
coeff( 419)= 3.4641794190381958E-01
coeff( 420)=-5.8818393366893962E-01
coeff( 421)=-1.5251411205573862E-01
coeff( 422)= 1.2767017187633583E+00
coeff( 423)=-1.2444139214811237E-01
coeff( 424)= 5.2587912733918851E-01
coeff( 425)= 5.1428309189768695E-01
coeff( 426)= 2.9548613493627823E-01
coeff( 427)=-2.7764431047129290E-01
coeff( 428)=-5.0645781431531722E-01
coeff( 429)=-3.6333831789367332E-02
coeff( 430)=-1.8557045404879502E-01
coeff( 431)=-2.8273722081035224E-01
coeff( 432)=-1.3442122606100051E-01
coeff( 433)= 6.3459688232895806E-01
coeff( 434)=-4.1250715407548028E-01
coeff( 435)= 9.2086308896760921E-02
coeff( 436)= 2.7418458993686073E-01
coeff( 437)=-4.5698460523138901E-01
coeff( 438)=-9.7178760550843873E-02
coeff( 439)= 4.3522164022499499E-01
coeff( 440)=-5.7046398640034235E-01
coeff( 441)= 3.6496214135019539E-01
coeff( 442)= 8.1205536780454557E-01
coeff( 443)= 1.8544214671681949E-01
coeff( 444)= 2.6377499854018427E-01
coeff( 445)=-1.2023050430967615E-01
coeff( 446)=-3.8340784313555576E-01
coeff( 447)=-8.4831674129954196E-02
coeff( 448)= 3.1102926350244292E-01
coeff( 449)= 3.4329239178196351E-02
coeff( 450)= 6.6239005177141555E-02
coeff( 451)= 8.7728848692189421E-01
coeff( 452)= 2.9864332522511217E-01
coeff( 453)=-7.4950762018871120E-01
coeff( 454)=-4.7104520347315149E-01
coeff( 455)=-2.4090901864319969E-01
coeff( 456)=-3.1540183858209083E+00
coeff( 457)=-1.5506936911592635E+00
coeff( 458)= 1.5200567925662589E-01
coeff( 459)=-1.2634576597689979E+00
coeff( 460)=-2.1383962206714013E-01
coeff( 461)= 9.3174447983933040E-02
coeff( 462)= 3.7062151038043450E-01
coeff( 463)=-2.2278255049600140E-01
coeff( 464)=-1.8076468082546757E+00
coeff( 465)=-2.3203731213217256E+00
coeff( 466)=-4.0420505553045716E-01
coeff( 467)= 2.9215410046481260E-01
coeff( 468)= 5.7425734420163144E-01
coeff( 469)=-1.5229428845979558E+00
coeff( 470)=-3.7703039013967682E-01
coeff( 471)=-1.8023182995841897E-01
coeff( 472)= 1.6820759674471658E-01
coeff( 473)= 4.4838592993526111E-01
coeff( 474)= 2.8962183126556629E-01
coeff( 475)= 1.4234533741383926E+00
coeff( 476)= 5.0560749133940930E-01
coeff( 477)=-7.5606329498494751E-01
coeff( 478)= 1.1346694396971221E-01
coeff( 479)= 1.0265904450057970E+00
coeff( 480)= 3.4816229607880028E-01
coeff( 481)=-3.3766740558914021E-01
coeff( 482)= 1.0031749558155332E+00
coeff( 483)= 4.3302019290757499E-01
coeff( 484)=-8.4468895463984695E-01
coeff( 485)=-4.2596936067116908E-01
coeff( 486)=-2.3570803849456650E+00
coeff( 487)= 5.5521085696906480E-01
coeff( 488)= 4.8564090103836666E-01
coeff( 489)=-4.2628863928955157E-01
coeff( 490)= 1.2372422430097705E+00
coeff( 491)= 2.0714529509678034E-01
coeff( 492)=-8.7274303545715282E-01
coeff( 493)=-4.6518962844061390E-01
coeff( 494)= 9.0088357470542080E-01
coeff( 495)= 6.2908904347424341E-02
coeff( 496)=-6.4904605984239161E-01
coeff( 497)=-7.2103773276520544E-01
coeff( 498)=-9.7851586898227483E-02
coeff( 499)=-1.2375137839871787E-01
coeff( 500)=-2.7523333904754871E-01
coeff( 501)=-2.4904811807989938E-02
coeff( 502)= 1.0204980595529873E+00
coeff( 503)=-5.4353467014821200E-01
coeff( 504)=-5.8298732760416205E-01
coeff( 505)= 4.1953620652665030E-02
coeff( 506)= 7.3187516936841701E-01
coeff( 507)=-3.8656761623117603E-01
coeff( 508)=-3.0732211746014304E-01
coeff( 509)=-7.4886640892541378E-01
coeff( 510)= 2.9001900880087755E-01
coeff( 511)=-6.0572896757403805E-01
coeff( 512)= 3.6209991984036946E-02
coeff( 513)= 5.3541634427518814E-01
coeff( 514)=-4.6454067351083136E-02
coeff( 515)=-2.5930041654223701E-01
coeff( 516)= 3.1595887459719818E-01
coeff( 517)=-3.3138647995130138E-01
coeff( 518)= 3.0256678306418072E-01
coeff( 519)=-2.3144515906086538E-01
coeff( 520)=-2.4675692864849894E-01
coeff( 521)=-2.4449830460238736E-01
coeff( 522)= 1.4084434452835659E-01
coeff( 523)= 1.5308225624862731E-01
coeff( 524)= 4.5724504555314271E-01
coeff( 525)= 1.0791258500801802E-01
coeff( 526)=-1.0104129787049030E-02
coeff( 527)=-1.6600752723886453E-01
coeff( 528)=-1.9639703338349412E-01
coeff( 529)= 1.1348076214318861E-01
coeff( 530)=-3.8961970663378021E-01
coeff( 531)= 3.8688369073795559E-01
coeff( 532)= 6.3482672896089332E-02
coeff( 533)=-1.3978743950286421E+00
coeff( 534)=-5.1910964689129113E-01
coeff( 535)= 2.5959135814056572E-01
coeff( 536)= 1.0190019447452237E+00
coeff( 537)=-1.1365817971187073E-01
coeff( 538)= 6.5602750389910602E-01
coeff( 539)= 9.2857378981881378E-01
coeff( 540)=-2.8696357964551922E-01
coeff( 541)= 4.9546250591471730E-01
coeff( 542)=-3.5457257363465022E-01
coeff( 543)= 5.6127307870510387E-02
coeff( 544)=-1.8209179716641116E-01
coeff( 545)=-6.9918524820912029E-02
coeff( 546)= 2.7869725578937232E-01
  return
end if

entry u22_pot_8(xyz,v, dxyz)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

re_hh = 0.741
re_oh = 0.970

ra_hh=ra1
ra_oh=ra2
rb_hh=rb1
rb_oh=rb2


x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)
call ev_pol_con_8(x,p)
v=dot_product(coeff,p)  +(oh(r14)+oh(r24)+oh(r34)+hh(r12)+hh(r13)+hh(r23))

call grad_ev_pol_con_8(x,dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6)

dx1 = dot_product(coeff,dpdx1)
dx2 = dot_product(coeff,dpdx2)
dx3 = dot_product(coeff,dpdx3)
dx4 = dot_product(coeff,dpdx4)
dx5 = dot_product(coeff,dpdx5)
dx6 = dot_product(coeff,dpdx6)

dr1=-x(1)*(1.0d0/(a1)+2.0d0*(r12-rb1)/(b1))*dx1 + grad_hh(r12)
dr2=-x(2)*(1.0d0/(a1)+2.0d0*(r13-rb1)/(b1))*dx2 + grad_hh(r13)
dr3=-x(3)*(1.0d0/(a2)+2.0d0*(r14-rb2)/(b2))*dx3 + grad_oh(r14)
dr4=-x(4)*(1.0d0/(a1)+2.0d0*(r23-rb1)/(b1))*dx4 + grad_hh(r23)
dr5=-x(5)*(1.0d0/(a2)+2.0d0*(r24-rb2)/(b2))*dx5 + grad_oh(r24)
dr6=-x(6)*(1.0d0/(a2)+2.0d0*(r34-rb2)/(b2))*dx6 + grad_oh(r34)

dxyz(1,1)=dr1/r12*(xyz(1,1)-xyz(2,1))+dr2/r13*(xyz(1,1)-xyz(3,1))+dr3/r14*(xyz(1,1)-xyz(4,1))
dxyz(1,2)=dr1/r12*(xyz(1,2)-xyz(2,2))+dr2/r13*(xyz(1,2)-xyz(3,2))+dr3/r14*(xyz(1,2)-xyz(4,2))
dxyz(1,3)=dr1/r12*(xyz(1,3)-xyz(2,3))+dr2/r13*(xyz(1,3)-xyz(3,3))+dr3/r14*(xyz(1,3)-xyz(4,3))

dxyz(2,1)=-dr1/r12*(xyz(1,1)-xyz(2,1))+dr4/r23*(xyz(2,1)-xyz(3,1))+dr5/r24*(xyz(2,1)-xyz(4,1))
dxyz(2,2)=-dr1/r12*(xyz(1,2)-xyz(2,2))+dr4/r23*(xyz(2,2)-xyz(3,2))+dr5/r24*(xyz(2,2)-xyz(4,2))
dxyz(2,3)=-dr1/r12*(xyz(1,3)-xyz(2,3))+dr4/r23*(xyz(2,3)-xyz(3,3))+dr5/r24*(xyz(2,3)-xyz(4,3))

dxyz(3,1)=-dr2/r13*(xyz(1,1)-xyz(3,1))-dr4/r23*(xyz(2,1)-xyz(3,1))+dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(3,2)=-dr2/r13*(xyz(1,2)-xyz(3,2))-dr4/r23*(xyz(2,2)-xyz(3,2))+dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(3,3)=-dr2/r13*(xyz(1,3)-xyz(3,3))-dr4/r23*(xyz(2,3)-xyz(3,3))+dr6/r34*(xyz(3,3)-xyz(4,3))

dxyz(4,1)=-dr3/r14*(xyz(1,1)-xyz(4,1))-dr5/r24*(xyz(2,1)-xyz(4,1))-dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(4,2)=-dr3/r14*(xyz(1,2)-xyz(4,2))-dr5/r24*(xyz(2,2)-xyz(4,2))-dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(4,3)=-dr3/r14*(xyz(1,3)-xyz(4,3))-dr5/r24*(xyz(2,3)-xyz(4,3))-dr6/r34*(xyz(3,3)-xyz(4,3))

return
end  subroutine

!===========================================================================
! *def u33_prepot_8
! subtoutine of computing u33 and gradient of u33
!===========================================================================
subroutine u33_prepot_8
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp) :: xyz(4,3)
real(dp) :: v, r12, r13, r14, r23, r24, r34
real(dp) :: a1, a2, b1, b2, ohstar_alt, hh, ra1, ra2, rb1, rb2, grad_hh, grad_ohstar_alt
integer, parameter :: ncoeff=546
real(dp) :: coeff(ncoeff), p(ncoeff)
real(dp) :: dpdx1(ncoeff), dpdx2(ncoeff), dpdx3(ncoeff) ,dpdx4(ncoeff), dpdx5(ncoeff) ,dpdx6(ncoeff)
real(dp) :: x(6), re_hh, re_oh, ra_hh, ra_oh, rb_hh, rb_oh
real(dp) :: dx1, dx2, dx3, dx4, dx5, dx6, dr1, dr2, dr3, dr4, dr5, dr6, dxyz(4,3)
integer ::  ncall
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2

if(ncall.eq.1) then

a1 = 8.0000000000000004E-01
a2 = 1.2000000000000000E+00
b1 = 1.0000000000000000E+00
b2 = 1.5000000000000000E+00
ra1 = 1.2000000000000000E+00
ra2 = 1.3999999999999999E+00
rb1 = 1.2000000000000000E+00
rb2 = 1.3999999999999999E+00
coeff(   1)=-7.6468015323493532E+01
coeff(   2)=-7.7785871473122921E+00
coeff(   3)= 2.9041254262044678E+00
coeff(   4)=-1.9927572893413348E+00
coeff(   5)= 1.4538649380449556E+01
coeff(   6)=-5.1433021212821703E+00
coeff(   7)=-6.5744486079638085E+00
coeff(   8)= 2.2034706258437176E+00
coeff(   9)=-5.2416588228291250E-01
coeff(  10)= 8.2968035725286384E+00
coeff(  11)= 2.1671410744528615E+01
coeff(  12)=-1.3635009991271957E+01
coeff(  13)=-7.0967549440132895E+00
coeff(  14)= 3.8487361549117298E+00
coeff(  15)=-2.7285568944879319E+01
coeff(  16)=-1.5628423119210950E+01
coeff(  17)= 4.9300018092043416E+00
coeff(  18)=-6.5478835494701793E-02
coeff(  19)=-3.0982447108505831E+01
coeff(  20)= 1.2490199733030671E+00
coeff(  21)= 9.3285528922366758E+00
coeff(  22)= 2.0147809386002134E+01
coeff(  23)= 2.8725830045108633E+00
coeff(  24)=-1.0346142708294272E+01
coeff(  25)= 1.6658546120948444E+01
coeff(  26)= 8.0424271340503584E+00
coeff(  27)=-4.5272734855769556E+00
coeff(  28)=-5.0179150614889565E+00
coeff(  29)= 6.7340979726396410E-01
coeff(  30)=-1.0723380475123141E+01
coeff(  31)=-2.6443535339118437E+01
coeff(  32)=-3.1101772478221044E+01
coeff(  33)= 4.1600901054279205E+01
coeff(  34)= 7.0481409410542906E+00
coeff(  35)=-1.1737123339451731E+01
coeff(  36)= 1.3832134665167157E+01
coeff(  37)=-4.9236864515856954E-01
coeff(  38)= 5.0057864974567767E+00
coeff(  39)=-1.2456958037698154E+01
coeff(  40)= 4.3226755335640753E+01
coeff(  41)= 1.2199637663343585E+01
coeff(  42)= 1.6683762282730896E+01
coeff(  43)=-2.8631820144317167E+01
coeff(  44)= 4.1695108346446306E+00
coeff(  45)= 1.5353871613761694E+01
coeff(  46)= 2.3098158905691914E+01
coeff(  47)= 1.5955201597340949E+01
coeff(  48)= 6.1550895862658361E+00
coeff(  49)=-3.7834740851047517E+00
coeff(  50)= 8.8547581261149482E+00
coeff(  51)=-1.8661396319720343E+00
coeff(  52)= 3.6124139175111196E+00
coeff(  53)= 6.8133338403696584E+01
coeff(  54)= 1.2618209074408677E+01
coeff(  55)=-1.8332394710636812E+01
coeff(  56)=-1.0754217104084578E+01
coeff(  57)= 9.7913966995834976E+00
coeff(  58)=-2.6982571542405978E+01
coeff(  59)=-3.4538896421044916E+00
coeff(  60)=-1.2013063020200130E+01
coeff(  61)=-1.2472802418406841E+01
coeff(  62)=-3.0477823437968915E+01
coeff(  63)=-2.4822657268765383E+01
coeff(  64)=-5.8606938917463012E+00
coeff(  65)= 3.8576978563502298E+00
coeff(  66)= 1.3379059656607705E+01
coeff(  67)= 5.1601059040620596E+00
coeff(  68)=-4.1889364807404057E+00
coeff(  69)= 2.1052811846913883E+01
coeff(  70)=-1.3881806221730784E+01
coeff(  71)=-7.9810877827239191E-01
coeff(  72)= 9.6889032908464368E-01
coeff(  73)= 3.0595542896685729E-02
coeff(  74)=-7.0573827280523005E-01
coeff(  75)= 7.5437980168847751E-01
coeff(  76)= 1.3256562266777649E+01
coeff(  77)= 2.5426210517610290E+01
coeff(  78)=-7.2348857965643134E+01
coeff(  79)= 4.8504642426865918E-01
coeff(  80)=-1.2362805020410114E+01
coeff(  81)= 9.2396185634282979E+00
coeff(  82)=-1.3242932497741929E+01
coeff(  83)=-6.3969230537909927E-01
coeff(  84)= 2.4414837193397030E+01
coeff(  85)= 6.5511347396798207E+00
coeff(  86)=-1.2568010952698659E+01
coeff(  87)= 8.9685758978146755E+00
coeff(  88)=-8.2907727611317288E+00
coeff(  89)=-4.5228068633018896E+00
coeff(  90)= 1.1884588291116680E+01
coeff(  91)=-4.2189423298995230E+01
coeff(  92)=-5.4556957597689824E+00
coeff(  93)=-3.3745450659364735E+01
coeff(  94)= 9.4977014463225302E+00
coeff(  95)=-7.5360015259223285E+00
coeff(  96)=-1.5875910510097070E+01
coeff(  97)= 3.5054381825230138E+01
coeff(  98)=-2.1381708278165892E+00
coeff(  99)=-9.7920773186297385E+00
coeff( 100)=-3.4887402379865001E+01
coeff( 101)=-3.0086553520679114E+01
coeff( 102)=-4.4664857373689211E+00
coeff( 103)=-1.1351367107690564E+01
coeff( 104)=-1.1937791374898502E+01
coeff( 105)=-1.6196374807997202E+01
coeff( 106)= 9.0406429303504936E+00
coeff( 107)=-4.9629281066048367E+00
coeff( 108)= 1.2227649900678060E+01
coeff( 109)= 4.1483774980024046E+00
coeff( 110)= 9.9547558396061646E+00
coeff( 111)=-9.6353063692147032E+00
coeff( 112)= 3.5563250284790895E+00
coeff( 113)=-1.1777568520429018E+01
coeff( 114)=-1.0695727706386942E+01
coeff( 115)=-4.0572129082656314E+00
coeff( 116)=-1.6405969488193262E+00
coeff( 117)=-9.0592771574196505E+00
coeff( 118)= 3.8973113675542610E+00
coeff( 119)=-3.3781266963875329E+00
coeff( 120)= 1.1869590200573110E+00
coeff( 121)= 5.6024816926541703E+01
coeff( 122)=-2.1350687182206919E+01
coeff( 123)=-5.2005801625130992E+01
coeff( 124)=-1.6824194400298140E+01
coeff( 125)= 1.4227360597792018E+01
coeff( 126)=-9.2705670678361756E+00
coeff( 127)= 1.7789161929928046E+01
coeff( 128)=-1.0430388606857107E+01
coeff( 129)= 3.3191126706851133E+00
coeff( 130)=-9.6877360862033002E-01
coeff( 131)= 4.3821935613911201E+01
coeff( 132)=-1.3372852452100727E+00
coeff( 133)= 3.4247948203759897E+01
coeff( 134)=-4.4098518320213853E-01
coeff( 135)= 1.0774796858430665E+01
coeff( 136)= 3.1307453329613551E+01
coeff( 137)= 1.5252644440901408E+01
coeff( 138)=-3.4561275399755131E+00
coeff( 139)=-9.2308222357864604E+00
coeff( 140)=-8.3994073691589466E-01
coeff( 141)=-1.2813998292379201E+00
coeff( 142)=-1.5351805643920209E+00
coeff( 143)= 7.1988802043463540E+00
coeff( 144)= 1.1336061431361252E+01
coeff( 145)= 9.5667697498304847E+00
coeff( 146)=-2.4912724597065172E-01
coeff( 147)=-1.0499072306192841E-01
coeff( 148)=-1.5117967842343283E+01
coeff( 149)= 2.3805470257240082E+00
coeff( 150)=-2.9592248991850965E+00
coeff( 151)= 4.0497930688359967E+00
coeff( 152)=-1.0517158080502101E+01
coeff( 153)= 6.8445354228874811E+00
coeff( 154)=-8.5816302676241762E-01
coeff( 155)= 1.8364976699482205E+00
coeff( 156)= 1.7294329797605159E+00
coeff( 157)= 2.8809067769377159E+00
coeff( 158)= 7.1328145092181749E+00
coeff( 159)=-7.3638361165970103E+00
coeff( 160)= 8.3586757077111020E+00
coeff( 161)=-1.4590057490714891E+01
coeff( 162)= 6.2669994729684021E+01
coeff( 163)= 1.1574357968889540E+01
coeff( 164)=-1.2252994637118878E+01
coeff( 165)=-1.0504276511823054E+01
coeff( 166)= 1.5590917322781079E+01
coeff( 167)=-3.0593386084945320E+00
coeff( 168)= 5.7132482698621487E+00
coeff( 169)=-1.4765632206995147E+00
coeff( 170)=-2.8200349396342204E+00
coeff( 171)=-2.1535507648409249E+00
coeff( 172)= 3.3251012571433467E+00
coeff( 173)=-1.2944625082222180E+01
coeff( 174)= 1.0177339275235566E+01
coeff( 175)= 7.8063104536305283E-01
coeff( 176)= 1.8745350544497974E+00
coeff( 177)=-8.3566961866248821E+00
coeff( 178)= 1.3087260438281301E-01
coeff( 179)=-3.1757384177084629E+00
coeff( 180)= 9.4688660755311416E-01
coeff( 181)= 3.8792886920252095E-01
coeff( 182)=-3.9498153427461777E+00
coeff( 183)=-1.2107967599733978E+00
coeff( 184)=-1.8336821626970197E+00
coeff( 185)= 5.8446123062225999E+00
coeff( 186)= 2.7006966283993727E+00
coeff( 187)= 1.9341324616870383E+00
coeff( 188)= 5.8394292211569301E+00
coeff( 189)=-1.5298037898809701E+01
coeff( 190)= 2.1200644827465904E+01
coeff( 191)= 4.8979842989470770E+00
coeff( 192)= 2.0416969567522468E+01
coeff( 193)=-1.6534672334970075E+01
coeff( 194)= 3.4128918662599772E+00
coeff( 195)= 4.2998900721696014E+00
coeff( 196)= 6.9659892307364069E-01
coeff( 197)= 1.3275174794076031E+01
coeff( 198)=-1.4411986074454591E+01
coeff( 199)=-9.3634535372553329E-01
coeff( 200)=-1.1454301714398474E+00
coeff( 201)= 1.0314768123246552E+01
coeff( 202)= 2.2621438078606371E+01
coeff( 203)= 1.2843139039769310E+01
coeff( 204)= 7.5568267701118295E+00
coeff( 205)= 1.0613120067258174E+01
coeff( 206)=-7.0394709972349396E+00
coeff( 207)=-8.3731301672361180E-01
coeff( 208)= 9.6287133979231196E+00
coeff( 209)= 6.4711929355187072E+00
coeff( 210)= 1.7958153020662660E+01
coeff( 211)=-4.5942659781248665E+00
coeff( 212)=-5.6847822945860589E+00
coeff( 213)=-6.5634806329865318E+00
coeff( 214)= 3.9863918965041316E+00
coeff( 215)=-5.8843611770981568E+00
coeff( 216)=-2.5348082566759436E+00
coeff( 217)=-3.1945720064249352E+00
coeff( 218)= 6.5867645443208636E+00
coeff( 219)=-2.6053428360830799E+00
coeff( 220)= 2.6323950141475816E+00
coeff( 221)= 3.2969657590793071E+00
coeff( 222)= 6.5927322822420384E+00
coeff( 223)= 8.6937288903035128E+00
coeff( 224)= 3.1351072649026097E+00
coeff( 225)=-9.7652145608884255E-01
coeff( 226)= 2.7148512494032588E+00
coeff( 227)=-7.7924426532907309E-01
coeff( 228)= 2.8266895471289160E+00
coeff( 229)=-1.5417562672076015E+00
coeff( 230)= 6.2744152630435490E+00
coeff( 231)=-3.9923917637087927E+00
coeff( 232)=-5.5580361762797130E+00
coeff( 233)=-5.7560341501979746E+00
coeff( 234)= 4.9285619065261793E-01
coeff( 235)=-3.9246058628705667E+00
coeff( 236)=-8.4291931825272082E-01
coeff( 237)= 2.6603733053297010E+00
coeff( 238)=-2.6479404562092757E-01
coeff( 239)= 2.2732720395087762E+00
coeff( 240)= 1.3417608777331476E+00
coeff( 241)=-1.7101664045530085E+00
coeff( 242)= 8.0173680299345573E+00
coeff( 243)=-1.4334471367061217E+00
coeff( 244)= 3.9161038122732061E+00
coeff( 245)= 1.3546674365603073E+00
coeff( 246)=-1.6420314385669762E+01
coeff( 247)= 6.3902475520747748E+00
coeff( 248)= 1.6054731045617043E+01
coeff( 249)= 1.5488991326035556E+01
coeff( 250)= 9.9271864269654770E+00
coeff( 251)=-7.1831321633055616E+00
coeff( 252)= 7.6474520233854748E-01
coeff( 253)= 9.7211272860462064E+00
coeff( 254)= 1.6763099167215987E+01
coeff( 255)=-1.8920848823860496E+01
coeff( 256)=-2.6075457529618085E+00
coeff( 257)= 8.1061082698422506E+00
coeff( 258)= 3.8917136416090503E-01
coeff( 259)=-3.8019839458420144E+01
coeff( 260)= 2.2447871886831168E+00
coeff( 261)=-9.2349824610253020E+00
coeff( 262)=-1.8015652497699371E+01
coeff( 263)=-5.6811633620048747E+00
coeff( 264)=-1.6659255842318443E+00
coeff( 265)=-8.2693871214882719E+00
coeff( 266)=-8.5072486809186252E+00
coeff( 267)= 4.1545387595729109E+00
coeff( 268)= 2.4169468733563653E+00
coeff( 269)=-4.5498543474709354E+00
coeff( 270)=-8.6995617273570662E+00
coeff( 271)= 1.0597352490873801E+00
coeff( 272)=-5.6078701828058097E+00
coeff( 273)= 7.4106128811501835E+00
coeff( 274)= 4.8652613102291449E+00
coeff( 275)=-1.3208009257309197E+01
coeff( 276)=-3.1460151599042550E+00
coeff( 277)= 1.5057419341616958E+00
coeff( 278)= 1.0596065530211487E+01
coeff( 279)= 7.4923446493342283E-01
coeff( 280)= 2.4565945076120008E+00
coeff( 281)= 7.1352266671692310E-01
coeff( 282)= 2.3882441204216778E+00
coeff( 283)= 4.9497877329465991E+00
coeff( 284)=-5.2865252967524921E+00
coeff( 285)= 2.6645485704220060E+00
coeff( 286)=-3.4897935464357634E+00
coeff( 287)=-6.4156773457569152E+00
coeff( 288)=-8.3272408801672493E-01
coeff( 289)= 7.4431077866657058E-01
coeff( 290)=-2.2809776949865723E+00
coeff( 291)= 4.0842262351894538E+00
coeff( 292)=-6.3443244707627784E+00
coeff( 293)=-1.6408116194312978E+00
coeff( 294)=-1.1813749546631742E-01
coeff( 295)= 2.7816712526480858E+00
coeff( 296)=-1.0389354637403914E+00
coeff( 297)=-1.4682555256432577E+00
coeff( 298)=-5.2414798829584264E-01
coeff( 299)=-6.4484198586025809E-01
coeff( 300)=-3.4397406742799368E+00
coeff( 301)=-5.0528639760129721E+00
coeff( 302)=-1.3622778171890400E+00
coeff( 303)=-8.8240064948552135E+00
coeff( 304)= 5.4982147821226413E+00
coeff( 305)=-2.4267059186256297E+01
coeff( 306)=-1.6077983717752588E+01
coeff( 307)= 1.2414594912188148E+01
coeff( 308)=-2.8526297274620780E+00
coeff( 309)= 3.2549376322225840E+00
coeff( 310)=-4.4775506931029723E+00
coeff( 311)= 1.6613808557455167E+00
coeff( 312)=-1.2063718347175105E+00
coeff( 313)= 1.2945831656951228E+00
coeff( 314)=-3.0375945528865649E+00
coeff( 315)= 3.8535747751244553E+00
coeff( 316)= 2.4440236985851533E+00
coeff( 317)=-1.6957839395378418E+00
coeff( 318)= 2.4334366827790128E+00
coeff( 319)=-1.8991672808581548E+00
coeff( 320)=-1.7657715595448562E+00
coeff( 321)= 2.7476985416786954E+00
coeff( 322)= 1.1690217322226590E+00
coeff( 323)=-4.8547883135392944E-01
coeff( 324)=-1.3506989097806239E+00
coeff( 325)= 2.7739192175771983E+00
coeff( 326)=-2.8446302416239999E+00
coeff( 327)=-9.4141081968519880E-01
coeff( 328)= 2.8427781309412681E+00
coeff( 329)=-1.3059742696666379E+00
coeff( 330)= 3.4550309866104767E+00
coeff( 331)= 7.9144366958521795E-01
coeff( 332)=-2.1355869815761932E+00
coeff( 333)= 4.8135975758927441E-01
coeff( 334)=-3.1948605827931629E+00
coeff( 335)= 1.9287997209801664E+00
coeff( 336)=-8.6997860752143330E-02
coeff( 337)= 4.8969089014904839E+00
coeff( 338)=-2.0303721310381277E+00
coeff( 339)=-1.5001903795896082E-02
coeff( 340)=-7.6063638233352959E-01
coeff( 341)=-3.9977115445032316E+00
coeff( 342)=-1.7073524047644095E-01
coeff( 343)= 8.1042826906369236E-01
coeff( 344)= 3.4485375846756940E+00
coeff( 345)=-5.1450163070503185E-01
coeff( 346)= 1.1495034314328409E+00
coeff( 347)= 1.0816113788326640E+00
coeff( 348)= 5.1535542088462682E-01
coeff( 349)=-6.8490269998755926E-01
coeff( 350)= 1.6273090258083258E+00
coeff( 351)=-3.6044673692027895E-01
coeff( 352)=-1.1025822120220103E+00
coeff( 353)= 1.1596106819111558E+00
coeff( 354)=-1.7175748485953792E+00
coeff( 355)=-3.0711145131593603E-01
coeff( 356)=-1.7870128651239578E+00
coeff( 357)= 5.8347920769078838E+00
coeff( 358)=-3.7102641311655158E+00
coeff( 359)=-2.0045067463630262E+00
coeff( 360)=-3.5054074427592754E+00
coeff( 361)= 3.2174218074616712E+00
coeff( 362)=-1.6204636523697953E+00
coeff( 363)=-9.3964327509739896E+00
coeff( 364)= 3.2786792002760392E+00
coeff( 365)= 6.7239901136269626E+00
coeff( 366)=-4.7750673341186047E+00
coeff( 367)= 1.8262794313457682E+00
coeff( 368)=-6.2325950974609068E-01
coeff( 369)=-3.8231731730594158E+00
coeff( 370)= 1.6181259148522518E+00
coeff( 371)= 4.8423723319843308E+00
coeff( 372)=-5.6315873588585017E+00
coeff( 373)= 1.2890714639568157E+00
coeff( 374)=-1.4471726975393762E+00
coeff( 375)=-3.0743812621439566E+00
coeff( 376)=-2.2285372489088782E+00
coeff( 377)=-1.4764798624828954E+00
coeff( 378)=-3.8087126712104777E+00
coeff( 379)= 1.7178172609397184E+00
coeff( 380)= 1.1389090605224288E+00
coeff( 381)=-1.4205231980746078E+00
coeff( 382)= 1.1406577347811797E+00
coeff( 383)=-1.3737846955620121E+00
coeff( 384)=-1.4758154629136782E+00
coeff( 385)=-2.2815085957881815E+00
coeff( 386)=-6.7490533756386224E+00
coeff( 387)=-1.8352179265553450E+00
coeff( 388)= 1.1467647194573352E+00
coeff( 389)= 1.4223329091084647E+00
coeff( 390)=-1.8455426028439013E+00
coeff( 391)= 3.7092723576289828E+00
coeff( 392)= 2.8665190527773632E+00
coeff( 393)=-2.2247166772545979E-01
coeff( 394)= 1.3940177920068841E-01
coeff( 395)=-2.0115470572603775E-01
coeff( 396)=-2.6880782977013964E+00
coeff( 397)=-1.1585656293720397E+00
coeff( 398)= 1.3394182739532952E+00
coeff( 399)=-3.5604494388409669E-01
coeff( 400)=-6.1704124863575915E-01
coeff( 401)=-1.4530689648529849E+00
coeff( 402)=-2.2000884845160602E+00
coeff( 403)=-6.0870569434698947E+00
coeff( 404)=-1.0115538716286094E+00
coeff( 405)=-1.0538610806480184E+00
coeff( 406)=-5.5120829929049844E+00
coeff( 407)= 3.9664954621257604E+00
coeff( 408)= 4.6468644466593723E-01
coeff( 409)=-5.9849457783188065E-01
coeff( 410)=-9.5476521712891005E-01
coeff( 411)=-9.6221272785335343E-01
coeff( 412)= 1.8141691054970466E+00
coeff( 413)= 2.4949674315510042E+00
coeff( 414)= 1.3927540336418201E-01
coeff( 415)=-3.5665769620658998E+00
coeff( 416)= 1.1028316127944719E+00
coeff( 417)= 2.4457392508109996E-01
coeff( 418)= 3.5364943583207453E+00
coeff( 419)= 1.1435391015128700E-02
coeff( 420)=-4.8331025430762142E-01
coeff( 421)=-2.8237906872740776E-01
coeff( 422)= 2.7083520522696287E+00
coeff( 423)= 1.2293194768724405E+00
coeff( 424)=-2.9490144214082709E-01
coeff( 425)= 5.7608847538584296E-01
coeff( 426)=-7.9183365392273197E-01
coeff( 427)=-1.3620815119298040E+00
coeff( 428)=-2.6835182920619660E-01
coeff( 429)=-8.5305917024831290E-01
coeff( 430)= 5.0546842377379608E-01
coeff( 431)= 5.3835101687814690E-01
coeff( 432)=-5.1558672300137243E-01
coeff( 433)=-7.1617331918728777E-01
coeff( 434)= 9.7019636226407080E-01
coeff( 435)= 4.0017685026907135E-01
coeff( 436)=-1.2987157958432782E+00
coeff( 437)= 1.0788516996965705E+00
coeff( 438)=-7.1483563300949438E-01
coeff( 439)= 3.7013323520633856E-01
coeff( 440)=-2.5459189038667240E+00
coeff( 441)= 3.9410152128973175E-01
coeff( 442)= 2.9112779500373329E+00
coeff( 443)=-1.7133458833710611E+00
coeff( 444)= 9.0841754477818268E-02
coeff( 445)= 7.5279073613423717E-02
coeff( 446)=-2.8053371207171052E-01
coeff( 447)= 4.6914327394656863E-01
coeff( 448)= 2.7966341856928673E-01
coeff( 449)= 4.1966145093262730E-01
coeff( 450)=-7.8311604446740346E-01
coeff( 451)= 9.1877578785165359E-01
coeff( 452)=-1.5234095403954540E+00
coeff( 453)=-3.0477671500380216E-01
coeff( 454)=-1.7479460881251674E+00
coeff( 455)=-3.1876970767181073E-01
coeff( 456)= 4.9517652390664084E+00
coeff( 457)=-1.7080374701809049E+00
coeff( 458)=-1.8828369461557644E+00
coeff( 459)=-2.4131718038054619E+00
coeff( 460)=-1.6497371220875012E+00
coeff( 461)=-2.2953648565050662E+00
coeff( 462)= 1.0122402339973691E+00
coeff( 463)= 1.5445958083617630E+00
coeff( 464)=-3.3275206726250426E+00
coeff( 465)= 7.3280126240857713E-01
coeff( 466)=-3.8371144931651329E+00
coeff( 467)= 1.5896309093919934E+00
coeff( 468)= 2.3220997850443053E+00
coeff( 469)=-1.8595308854320587E+00
coeff( 470)= 1.3524745557602175E-01
coeff( 471)= 1.0156679283571934E+01
coeff( 472)=-4.3482754964439568E-01
coeff( 473)= 1.2063856061846348E+00
coeff( 474)= 2.9172716033586390E+00
coeff( 475)=-1.9755015490356171E+00
coeff( 476)= 2.5911148095660357E+00
coeff( 477)= 2.7012725746692574E+00
coeff( 478)= 2.0225183360884182E+00
coeff( 479)= 1.0318111247739745E+00
coeff( 480)= 2.1568846932248715E+00
coeff( 481)=-1.2286720215463736E+00
coeff( 482)=-4.5511067568549302E-01
coeff( 483)= 3.0339855833699816E+00
coeff( 484)= 4.4118091849922747E+00
coeff( 485)=-5.6386424839144156E-01
coeff( 486)=-2.0787463252603663E-01
coeff( 487)= 1.0395649823709165E+00
coeff( 488)= 1.0613244179034557E-01
coeff( 489)=-3.7717143858636057E+00
coeff( 490)= 1.5691076912205115E+00
coeff( 491)= 4.2178727119246534E-01
coeff( 492)=-1.5539991709714214E+00
coeff( 493)=-2.1040978905359058E+00
coeff( 494)=-4.8869045045256010E-01
coeff( 495)=-1.2628799937425612E+00
coeff( 496)= 4.9090806564891981E-01
coeff( 497)=-1.4628216491170801E-01
coeff( 498)= 2.9696984728613457E-02
coeff( 499)= 1.3085793620211266E+00
coeff( 500)= 2.8813286343148364E-01
coeff( 501)= 9.4864505400099131E-01
coeff( 502)=-8.0888788392997429E-02
coeff( 503)= 3.8794465513599841E-01
coeff( 504)= 1.1556072453231392E+00
coeff( 505)= 1.4861729965379664E-01
coeff( 506)= 5.8704596939716880E-01
coeff( 507)=-2.0864129646083547E+00
coeff( 508)=-5.9615183109247863E-02
coeff( 509)= 5.2219681734116896E-02
coeff( 510)=-3.6791967915834883E-01
coeff( 511)=-1.2725394707449531E+00
coeff( 512)=-5.2468343752451674E-01
coeff( 513)=-1.2470837743666729E+00
coeff( 514)= 7.2183693550240902E-01
coeff( 515)=-4.3394807313141920E-01
coeff( 516)= 3.8653255059720459E-01
coeff( 517)= 1.0557791415136821E+00
coeff( 518)=-3.4337122714221557E-01
coeff( 519)= 1.7321534661984275E-01
coeff( 520)= 6.4972833487745540E-01
coeff( 521)= 2.0917228952935285E-01
coeff( 522)= 2.0685406077219191E+00
coeff( 523)= 8.7987632664633730E-01
coeff( 524)= 1.2912734182183921E-01
coeff( 525)=-1.2436655331699129E-01
coeff( 526)=-9.8332541878633512E-03
coeff( 527)= 8.7865855279560312E-01
coeff( 528)=-2.5725909018359205E-01
coeff( 529)= 6.4832007990277413E-02
coeff( 530)= 7.4561364190202617E-01
coeff( 531)= 1.0021076233248494E+00
coeff( 532)=-1.2826478909402255E+00
coeff( 533)= 2.2233793360719871E+00
coeff( 534)= 1.6219750500429315E+00
coeff( 535)=-8.5371018013869082E-01
coeff( 536)= 3.1067067068582048E+00
coeff( 537)= 4.4543568131268865E+00
coeff( 538)=-1.5231202538022268E+00
coeff( 539)=-1.4440276181270184E+00
coeff( 540)= 4.8279983543147525E-01
coeff( 541)= 1.5289496511782750E+00
coeff( 542)=-1.0932617206558288E+00
coeff( 543)= 5.0608466028924159E-02
coeff( 544)=-3.9106709900631764E-01
coeff( 545)= 1.2745737472810612E-01
coeff( 546)=-2.3636874831335580E-01
  return
end if

!entry u33_pot_8(xyz,v)
entry u33_pot_8(xyz,v, dxyz)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

re_hh = 0.741
re_oh = 0.970

ra_hh=ra1
ra_oh=ra2
rb_hh=rb1
rb_oh=rb2

x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)

call ev_pol_con_8(x,p)
v=dot_product(coeff,p) + (ohstar_alt(r14)+ohstar_alt(r24)+ohstar_alt(r34)+hh(r12)+hh(r13)+hh(r23))


call grad_ev_pol_con_8(x,dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6)

dx1 = dot_product(coeff,dpdx1)
dx2 = dot_product(coeff,dpdx2)
dx3 = dot_product(coeff,dpdx3)
dx4 = dot_product(coeff,dpdx4)
dx5 = dot_product(coeff,dpdx5)
dx6 = dot_product(coeff,dpdx6)

dr1=-x(1)*(1.0d0/(a1)+2.0d0*(r12-rb1)/(b1))*dx1 + grad_hh(r12)
dr2=-x(2)*(1.0d0/(a1)+2.0d0*(r13-rb1)/(b1))*dx2 + grad_hh(r13)
dr3=-x(3)*(1.0d0/(a2)+2.0d0*(r14-rb2)/(b2))*dx3 + grad_ohstar_alt(r14)
dr4=-x(4)*(1.0d0/(a1)+2.0d0*(r23-rb1)/(b1))*dx4 + grad_hh(r23)
dr5=-x(5)*(1.0d0/(a2)+2.0d0*(r24-rb2)/(b2))*dx5 + grad_ohstar_alt(r24)
dr6=-x(6)*(1.0d0/(a2)+2.0d0*(r34-rb2)/(b2))*dx6 + grad_ohstar_alt(r34)

dxyz(1,1)=dr1/r12*(xyz(1,1)-xyz(2,1))+dr2/r13*(xyz(1,1)-xyz(3,1))+dr3/r14*(xyz(1,1)-xyz(4,1))
dxyz(1,2)=dr1/r12*(xyz(1,2)-xyz(2,2))+dr2/r13*(xyz(1,2)-xyz(3,2))+dr3/r14*(xyz(1,2)-xyz(4,2))
dxyz(1,3)=dr1/r12*(xyz(1,3)-xyz(2,3))+dr2/r13*(xyz(1,3)-xyz(3,3))+dr3/r14*(xyz(1,3)-xyz(4,3))

dxyz(2,1)=-dr1/r12*(xyz(1,1)-xyz(2,1))+dr4/r23*(xyz(2,1)-xyz(3,1))+dr5/r24*(xyz(2,1)-xyz(4,1))
dxyz(2,2)=-dr1/r12*(xyz(1,2)-xyz(2,2))+dr4/r23*(xyz(2,2)-xyz(3,2))+dr5/r24*(xyz(2,2)-xyz(4,2))
dxyz(2,3)=-dr1/r12*(xyz(1,3)-xyz(2,3))+dr4/r23*(xyz(2,3)-xyz(3,3))+dr5/r24*(xyz(2,3)-xyz(4,3))

dxyz(3,1)=-dr2/r13*(xyz(1,1)-xyz(3,1))-dr4/r23*(xyz(2,1)-xyz(3,1))+dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(3,2)=-dr2/r13*(xyz(1,2)-xyz(3,2))-dr4/r23*(xyz(2,2)-xyz(3,2))+dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(3,3)=-dr2/r13*(xyz(1,3)-xyz(3,3))-dr4/r23*(xyz(2,3)-xyz(3,3))+dr6/r34*(xyz(3,3)-xyz(4,3))

dxyz(4,1)=-dr3/r14*(xyz(1,1)-xyz(4,1))-dr5/r24*(xyz(2,1)-xyz(4,1))-dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(4,2)=-dr3/r14*(xyz(1,2)-xyz(4,2))-dr5/r24*(xyz(2,2)-xyz(4,2))-dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(4,3)=-dr3/r14*(xyz(1,3)-xyz(4,3))-dr5/r24*(xyz(2,3)-xyz(4,3))-dr6/r34*(xyz(3,3)-xyz(4,3))

return
end  subroutine

real*8 function d3bj_hh(r)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp), intent(in) :: r
real(dp) :: s6, s8, C6au, C8au, C6, C8, a1, a2, R0, fR0

s6 = 1.0
s8 = 2.0

C6au = 7.5915999999999997
C8au = 91.76991785999999

C6 = C6au*0.021958709
C8 = C8au*0.0061490647

a1 = 0.5299
a2 = 2.2

R0 = sqrt(C8/C6)
fR0 = a1*R0 + a2

d3bj_hh = -s6*C6/(r**6 + fR0**6) -s8*C8/(r**8 + fR0**8)

end function d3bj_hh

real*8 function grad_d3bj_hh(r)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp), intent(in) :: r
real(dp) :: s6, s8, C6au, C8au, C6, C8, a1, a2, R0, fR0

s6 = 1.0
s8 = 2.0

C6au = 7.5915999999999997
C8au = 91.76991785999999

C6 = C6au*0.021958709
C8 = C8au*0.0061490647

a1 = 0.5299
a2 = 2.2

R0 = sqrt(C8/C6)
fR0 = a1*R0 + a2

grad_d3bj_hh = 6.0_dp*s6*C6*r**5/(r**6 + fR0**6)**2  + 8.0_dp*s8*C8*r**7/(r**8 + fR0**8)**2

end function grad_d3bj_hh

real*8 function d3bj_oh(r)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp), intent(in) :: r
real(dp) :: s6, s8, C6au, C8au, C6, C8, a1, a2, R0, fR0

s6 = 1.0
s8 = 2.0

C6au = 10.84963549802481
C8au = 169.4591809288624

C6 = C6au*0.021958709
C8 = C8au*0.0061490647

a1 = 0.5299
a2 = 2.2

R0 = sqrt(C8/C6)
fR0 = a1*R0 + a2

d3bj_oh = -s6*C6/(r**6 + fR0**6) -s8*C8/(r**8 + fR0**8)

end function d3bj_oh

real*8 function grad_d3bj_oh(r)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp), intent(in) :: r
real(dp) :: s6, s8, C6au, C8au, C6, C8, a1, a2, R0, fR0

s6 = 1.0
s8 = 2.0

C6au = 10.84963549802481
C8au = 169.4591809288624

C6 = C6au*0.021958709
C8 = C8au*0.0061490647

a1 = 0.5299
a2 = 2.2

R0 = sqrt(C8/C6)
fR0 = a1*R0 + a2

grad_d3bj_oh = 6.0_dp*s6*C6*r**5/(r**6 + fR0**6)**2 + 8.0_dp*s8*C8*r**7/(r**8 + fR0**8)**2

end function grad_d3bj_oh

real*8 function hh(r)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp) :: d3bj_hh
real(dp), intent(in) :: r
real(dp) :: vsr, alpha, beta, c(8), p(8)
integer :: i

alpha = 0.53931748805004220
beta = 1.6459680933202987

c(1) = -0.10896821437780629_dp
c(2) = -0.29798048492728535_dp
c(3) = 0.20089464558076681_dp
c(4) = -8.2216773488458517E-002_dp
c(5) = 0.15456941633221136_dp
c(6) = 0.29533240911519809_dp
c(7) = 0.22790629347946670_dp
c(8) = 0.93346028818320326_dp

do i=0, 7
    p(i+1) = exp(-alpha*beta**i*r**2)
end do

vsr = dot_product(c, p)

hh = vsr + d3bj_hh(r)
end function hh

real*8 function grad_hh(r)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp) :: grad_d3bj_hh
real(dp), intent(in) :: r
real(dp) :: vsr, alpha, beta, c(8), p(8)
integer :: i

alpha = 0.53931748805004220
beta = 1.6459680933202987

c(1) = -0.10896821437780629_dp
c(2) = -0.29798048492728535_dp
c(3) = 0.20089464558076681_dp
c(4) = -8.2216773488458517E-002_dp
c(5) = 0.15456941633221136_dp
c(6) = 0.29533240911519809_dp
c(7) = 0.22790629347946670_dp
c(8) = 0.93346028818320326_dp

do i=0, 7
p(i+1) = -2.0_dp*alpha*beta**i*r*exp(-alpha*beta**i*r**2)
end do

vsr = dot_product(c, p)

grad_hh = vsr + grad_d3bj_hh(r)

end function grad_hh

real*8 function oh(r)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp) :: d3bj_oh
real(dp), intent(in) :: r
real(dp) :: vsr, alpha, beta, c(8), p(8)
integer :: i

alpha = 0.51045358916334960_dp
beta = 1.5433662455565513_dp

c(1) = -3.3264921812231049E-002_dp
c(2) = -0.19210399441400144_dp
c(3) = -0.92587310862479089_dp
c(4) = 1.9962515830881151_dp
c(5) = -2.3610928168877425_dp
c(6) = 4.0512644189749327_dp
c(7) = -2.1935688479612669_dp
c(8) = 6.4929855978115452_dp

do i=0, 7
    p(i+1) = exp(-alpha*beta**i*r**2)
end do

vsr = dot_product(c, p)
oh = vsr + d3bj_oh(r)

end function oh 

real*8 function grad_oh(r)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp) :: grad_d3bj_oh
real(dp), intent(in) :: r
real(dp) :: vsr, alpha, beta, c(8), p(8)
integer :: i

alpha = 0.51045358916334960_dp
beta = 1.5433662455565513_dp

c(1) = -3.3264921812231049E-002_dp
c(2) = -0.19210399441400144_dp
c(3) = -0.92587310862479089_dp
c(4) = 1.9962515830881151_dp
c(5) = -2.3610928168877425_dp
c(6) = 4.0512644189749327_dp
c(7) = -2.1935688479612669_dp
c(8) = 6.4929855978115452_dp

do i=0, 7
    p(i+1) = -2.0_dp*alpha*beta**i*r*exp(-alpha*beta**i*r**2)
end do

vsr = dot_product(c, p)
grad_oh = vsr + grad_d3bj_oh(r)

end function grad_oh 

real*8 function ohstar_alt(r)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp), intent(in) :: r
integer :: i
real(dp) :: alr, blr, vsr, vlr, fs, d3bj_oh
real(dp), dimension(3) :: plr, clr
real(dp) :: asr, bsr
real(dp), dimension(6) :: psr, csr

asr = 0.22510201499513907
bsr = 2.2657615066074954

csr(1) = 0.35455513969718444    
csr(2) = -0.52912871421810181     
csr(3) = -6.0697181582250126E-002
csr(4) = 0.47976112924154690     
csr(5) = 2.3310416022954366     
csr(6) = 5.6903894590457540

do i = 1, 6
    psr(i) = exp(-asr*bsr**(i-1)*r**2)
end do

vsr = dot_product(csr, psr)


fs = 0.5_dp*(1.0_dp - tanh(1250.0_dp*(r-1.177_dp)))

ohstar_alt = fs*vsr + (1.0_dp - fs)*d3bj_oh(r)

end function ohstar_alt

real*8 function grad_ohstar_alt(r)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp), intent(in) :: r
integer :: i
real(dp) :: alr, blr, vsr, vlr, fs, gsr, glr, gfs, d3bj_oh, grad_d3bj_oh
real(dp), dimension(3) :: plr, clr, pglr
real(dp) :: asr, bsr
real(dp), dimension(6) :: psr, csr, pgsr


asr = 0.22510201499513907
bsr = 2.2657615066074954

csr(1) = 0.35455513969718444    
csr(2) = -0.52912871421810181     
csr(3) = -6.0697181582250126E-002
csr(4) = 0.47976112924154690     
csr(5) = 2.3310416022954366     
csr(6) = 5.6903894590457540

do i = 1, 6
    psr(i) = exp(-asr*bsr**(i-1)*r**2)
    pgsr(i) = -2.0_dp*asr*bsr**(i-1)*r*exp(-asr*bsr**(i-1)*r**2)
end do

vsr = dot_product(csr, psr)
gsr = dot_product(csr, pgsr)

fs = 0.5_dp*(1.0_dp - tanh(1250.0_dp*(r-1.177_dp)))
gfs = -0.5_dp*1250.0_dp/(cosh(1250.0_dp*(r-1.177_dp)))**2

grad_ohstar_alt = gfs*(vsr - d3bj_oh(r)) + fs*gsr + (1.0_dp - fs)*grad_d3bj_oh(r)

end function grad_ohstar_alt
!===========================================================================
! *def ev_pol_con_8(x,q)
! subtoutine of evaluating polynomials at 8th order
!===========================================================================
subroutine ev_pol_con_8(x,p)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp),dimension(1:6),intent(in)::x
real(dp),dimension(1:546),intent(out)::p
real(dp)::x1,x2,x3,x4,x5,x6

x1=x(1)
x2=x(2)
x3=x(3)
x4=x(4)
x5=x(5)
x6=x(6)

p(1) = 1.d+0
p(2) = x5*x6+x3*x6+x3*x5
p(3) = x4*x6+x2*x6+x4*x5+x1*x5+x2*x3+x1*x3
p(4) = x2*x4+x1*x4+x1*x2
p(5) = x3*x5*x6
p(6) = x2*x5*x6+x1*x5*x6+x3*x4*x6+x1*x3*x6+x3*x4*x5+x2*x3*x5
p(7) = x4*x5*x6+x2*x3*x6+x1*x3*x5
p(8) = x1*x4*x6+x1*x2*x6+x2*x4*x5+x1*x2*x5+x2*x3*x4+x1*x3*x4
p(9) = x1*x2*x4
p(10) = x2*x4*x6+x1*x4*x5+x1*x2*x3
p(11) = x5*x6**2+x3*x6**2+x5**2*x6+x3**2*x6+x3*x5**2+x3**2*x5
p(12) = x4*x6**2+x2*x6**2+x4*x5**2+x1*x5**2+x2*x3**2+x1*x3**2
p(13) = x4**2*x6+x2**2*x6+x4**2*x5+x1**2*x5+x2**2*x3+x1**2*x3
p(14) = x2*x4**2+x1*x4**2+x2**2*x4+x1**2*x4+x1*x2**2+x1**2*x2
p(15) = x3*x4*x5*x6+x2*x3*x5*x6+x1*x3*x5*x6
p(16) = x1*x2*x5*x6+x1*x3*x4*x6+x2*x3*x4*x5
p(17) = x2*x4*x5*x6+x1*x4*x5*x6+x2*x3*x4*x6+x1*x2*x3*x6+x1*x3*x4*&
&x5+x1*x2*x3*x5
p(18) = x1*x2*x4*x6+x1*x2*x4*x5+x1*x2*x3*x4
p(19) = x3*x5*x6**2+x3*x5**2*x6+x3**2*x5*x6
p(20) = x1*x5*x6**2+x1*x3*x6**2+x2*x5**2*x6+x3**2*x4*x6+x2*x3*x5*&
&*2+x3**2*x4*x5
p(21) = x2*x5*x6**2+x3*x4*x6**2+x1*x5**2*x6+x1*x3**2*x6+x3*x4*x5*&
&*2+x2*x3**2*x5
p(22) = x4*x5*x6**2+x2*x3*x6**2+x4*x5**2*x6+x2*x3**2*x6+x1*x3*x5*&
&*2+x1*x3**2*x5
p(23) = x1*x4*x6**2+x1*x2*x6**2+x2*x4*x5**2+x1*x2*x5**2+x2*x3**2*&
&x4+x1*x3**2*x4
p(24) = x2*x4*x6**2+x1*x4*x5**2+x1*x2*x3**2
p(25) = x2**2*x5*x6+x1**2*x5*x6+x3*x4**2*x6+x1**2*x3*x6+x3*x4**2*&
&x5+x2**2*x3*x5
p(26) = x4**2*x5*x6+x2**2*x3*x6+x1**2*x3*x5
p(27) = x1**2*x4*x6+x1**2*x2*x6+x2**2*x4*x5+x1*x2**2*x5+x2*x3*x4*&
&*2+x1*x3*x4**2
p(28) = x1*x4**2*x6+x1*x2**2*x6+x2*x4**2*x5+x1**2*x2*x5+x2**2*x3*&
&x4+x1**2*x3*x4
p(29) = x1*x2*x4**2+x1*x2**2*x4+x1**2*x2*x4
p(30) = x2*x4**2*x6+x2**2*x4*x6+x1*x4**2*x5+x1**2*x4*x5+x1*x2**2*&
&x3+x1**2*x2*x3
p(31) = x5**2*x6**2+x3**2*x6**2+x3**2*x5**2
p(32) = x5*x6**3+x3*x6**3+x5**3*x6+x3**3*x6+x3*x5**3+x3**3*x5
p(33) = x4*x6**3+x2*x6**3+x4*x5**3+x1*x5**3+x2*x3**3+x1*x3**3
p(34) = x4**2*x6**2+x2**2*x6**2+x4**2*x5**2+x1**2*x5**2+x2**2*x3*&
&*2+x1**2*x3**2
p(35) = x2**2*x4**2+x1**2*x4**2+x1**2*x2**2
p(36) = x4**3*x6+x2**3*x6+x4**3*x5+x1**3*x5+x2**3*x3+x1**3*x3
p(37) = x2*x4**3+x1*x4**3+x2**3*x4+x1**3*x4+x1*x2**3+x1**3*x2
p(38) = x2*x3*x4*x5*x6+x1*x3*x4*x5*x6+x1*x2*x3*x5*x6
p(39) = x1*x2*x4*x5*x6+x1*x2*x3*x4*x6+x1*x2*x3*x4*x5
p(40) = x1*x3*x5*x6**2+x2*x3*x5**2*x6+x3**2*x4*x5*x6
p(41) = x3*x4*x5*x6**2+x2*x3*x5*x6**2+x3*x4*x5**2*x6+x1*x3*x5**2*&
&x6+x2*x3**2*x5*x6+x1*x3**2*x5*x6
p(42) = x1*x2*x5*x6**2+x1*x3*x4*x6**2+x1*x2*x5**2*x6+x1*x3**2*x4*&
&x6+x2*x3*x4*x5**2+x2*x3**2*x4*x5
p(43) = x1*x4*x5*x6**2+x1*x2*x3*x6**2+x2*x4*x5**2*x6+x2*x3**2*x4*&
&x6+x1*x2*x3*x5**2+x1*x3**2*x4*x5
p(44) = x2*x4*x5*x6**2+x2*x3*x4*x6**2+x1*x4*x5**2*x6+x1*x2*x3**2*&
&x6+x1*x3*x4*x5**2+x1*x2*x3**2*x5
p(45) = x1*x2*x4*x6**2+x1*x2*x4*x5**2+x1*x2*x3**2*x4
p(46) = x3*x4**2*x5*x6+x2**2*x3*x5*x6+x1**2*x3*x5*x6
p(47) = x1*x2**2*x5*x6+x1**2*x2*x5*x6+x1*x3*x4**2*x6+x1**2*x3*x4*&
&x6+x2*x3*x4**2*x5+x2**2*x3*x4*x5
p(48) = x1**2*x2*x4*x6+x1*x2**2*x4*x5+x1*x2*x3*x4**2
p(49) = x2**2*x4*x5*x6+x1**2*x4*x5*x6+x2*x3*x4**2*x6+x1**2*x2*x3*&
&x6+x1*x3*x4**2*x5+x1*x2**2*x3*x5
p(50) = x2*x4**2*x5*x6+x1*x4**2*x5*x6+x2**2*x3*x4*x6+x1*x2**2*x3*&
&x6+x1**2*x3*x4*x5+x1**2*x2*x3*x5
p(51) = x1*x2*x4**2*x6+x1*x2**2*x4*x6+x1*x2*x4**2*x5+x1**2*x2*x4*&
&x5+x1*x2**2*x3*x4+x1**2*x2*x3*x4
p(52) = x3*x5**2*x6**2+x3**2*x5*x6**2+x3**2*x5**2*x6
p(53) = x3*x5*x6**3+x3*x5**3*x6+x3**3*x5*x6
p(54) = x1*x5*x6**3+x1*x3*x6**3+x2*x5**3*x6+x3**3*x4*x6+x2*x3*x5*&
&*3+x3**3*x4*x5
p(55) = x2*x5**2*x6**2+x1*x5**2*x6**2+x3**2*x4*x6**2+x1*x3**2*x6*&
&*2+x3**2*x4*x5**2+x2*x3**2*x5**2
p(56) = x4*x5**2*x6**2+x2*x3**2*x6**2+x1*x3**2*x5**2
p(57) = x2*x5*x6**3+x3*x4*x6**3+x1*x5**3*x6+x1*x3**3*x6+x3*x4*x5*&
&*3+x2*x3**3*x5
p(58) = x4*x5*x6**3+x2*x3*x6**3+x4*x5**3*x6+x2*x3**3*x6+x1*x3*x5*&
&*3+x1*x3**3*x5
p(59) = x1*x4*x6**3+x1*x2*x6**3+x2*x4*x5**3+x1*x2*x5**3+x2*x3**3*&
&x4+x1*x3**3*x4
p(60) = x2*x4*x6**3+x1*x4*x5**3+x1*x2*x3**3
p(61) = x1**2*x5*x6**2+x1**2*x3*x6**2+x2**2*x5**2*x6+x3**2*x4**2*&
&x6+x2**2*x3*x5**2+x3**2*x4**2*x5
p(62) = x2**2*x5*x6**2+x3*x4**2*x6**2+x1**2*x5**2*x6+x1**2*x3**2*&
&x6+x3*x4**2*x5**2+x2**2*x3**2*x5
p(63) = x4**2*x5*x6**2+x2**2*x3*x6**2+x4**2*x5**2*x6+x2**2*x3**2*&
&x6+x1**2*x3*x5**2+x1**2*x3**2*x5
p(64) = x1**2*x4*x6**2+x1**2*x2*x6**2+x2**2*x4*x5**2+x1*x2**2*x5*&
&*2+x2*x3**2*x4**2+x1*x3**2*x4**2
p(65) = x1*x4**2*x6**2+x1*x2**2*x6**2+x2*x4**2*x5**2+x1**2*x2*x5*&
&*2+x2**2*x3**2*x4+x1**2*x3**2*x4
p(66) = x2*x4**2*x6**2+x2**2*x4*x6**2+x1*x4**2*x5**2+x1**2*x4*x5*&
&*2+x1*x2**2*x3**2+x1**2*x2*x3**2
p(67) = x1**2*x4**2*x6+x1**2*x2**2*x6+x2**2*x4**2*x5+x1**2*x2**2*&
&x5+x2**2*x3*x4**2+x1**2*x3*x4**2
p(68) = x1*x2**2*x4**2+x1**2*x2*x4**2+x1**2*x2**2*x4
p(69) = x2**2*x4**2*x6+x1**2*x4**2*x5+x1**2*x2**2*x3
p(70) = x2**3*x5*x6+x1**3*x5*x6+x3*x4**3*x6+x1**3*x3*x6+x3*x4**3*&
&x5+x2**3*x3*x5
p(71) = x4**3*x5*x6+x2**3*x3*x6+x1**3*x3*x5
p(72) = x1**3*x4*x6+x1**3*x2*x6+x2**3*x4*x5+x1*x2**3*x5+x2*x3*x4*&
&*3+x1*x3*x4**3
p(73) = x1*x4**3*x6+x1*x2**3*x6+x2*x4**3*x5+x1**3*x2*x5+x2**3*x3*&
&x4+x1**3*x3*x4
p(74) = x1*x2*x4**3+x1*x2**3*x4+x1**3*x2*x4
p(75) = x2*x4**3*x6+x2**3*x4*x6+x1*x4**3*x5+x1**3*x4*x5+x1*x2**3*&
&x3+x1**3*x2*x3
p(76) = x5**2*x6**3+x3**2*x6**3+x5**3*x6**2+x3**3*x6**2+x3**2*x5*&
&*3+x3**3*x5**2
p(77) = x5*x6**4+x3*x6**4+x5**4*x6+x3**4*x6+x3*x5**4+x3**4*x5
p(78) = x4*x6**4+x2*x6**4+x4*x5**4+x1*x5**4+x2*x3**4+x1*x3**4
p(79) = x4**2*x6**3+x2**2*x6**3+x4**2*x5**3+x1**2*x5**3+x2**2*x3*&
&*3+x1**2*x3**3
p(80) = x4**3*x6**2+x2**3*x6**2+x4**3*x5**2+x1**3*x5**2+x2**3*x3*&
&*2+x1**3*x3**2
p(81) = x2**2*x4**3+x1**2*x4**3+x2**3*x4**2+x1**3*x4**2+x1**2*x2*&
&*3+x1**3*x2**2
p(82) = x4**4*x6+x2**4*x6+x4**4*x5+x1**4*x5+x2**4*x3+x1**4*x3
p(83) = x2*x4**4+x1*x4**4+x2**4*x4+x1**4*x4+x1*x2**4+x1**4*x2
p(84) = x1*x2*x3*x4*x5*x6
p(85) = x1*x3*x4*x5*x6**2+x1*x2*x3*x5*x6**2+x2*x3*x4*x5**2*x6+x1&
&*x2*x3*x5**2*x6+x2*x3**2*x4*x5*x6+x1*x3**2*x4*x5*x6
p(86) = x2*x3*x4*x5*x6**2+x1*x3*x4*x5**2*x6+x1*x2*x3**2*x5*x6
p(87) = x1*x2*x4*x5*x6**2+x1*x2*x3*x4*x6**2+x1*x2*x4*x5**2*x6+x1&
&*x2*x3**2*x4*x6+x1*x2*x3*x4*x5**2+x1*x2*x3**2*x4*x5
p(88) = x2*x3*x4**2*x5*x6+x1*x3*x4**2*x5*x6+x2**2*x3*x4*x5*x6+x1&
&**2*x3*x4*x5*x6+x1*x2**2*x3*x5*x6+x1**2*x2*x3*x5*x6
p(89) = x1*x2**2*x4*x5*x6+x1**2*x2*x4*x5*x6+x1*x2*x3*x4**2*x6+x1&
&**2*x2*x3*x4*x6+x1*x2*x3*x4**2*x5+x1*x2**2*x3*x4*x5
p(90) = x1*x2*x4**2*x5*x6+x1*x2**2*x3*x4*x6+x1**2*x2*x3*x4*x5
p(91) = x1*x3*x5*x6**3+x2*x3*x5**3*x6+x3**3*x4*x5*x6
p(92) = x2*x3*x5**2*x6**2+x1*x3*x5**2*x6**2+x3**2*x4*x5*x6**2+x1&
&*x3**2*x5*x6**2+x3**2*x4*x5**2*x6+x2*x3**2*x5**2*x6
p(93) = x3*x4*x5**2*x6**2+x2*x3**2*x5*x6**2+x1*x3**2*x5**2*x6
p(94) = x3*x4*x5*x6**3+x2*x3*x5*x6**3+x3*x4*x5**3*x6+x1*x3*x5**3&
&*x6+x2*x3**3*x5*x6+x1*x3**3*x5*x6
p(95) = x1*x2*x5**2*x6**2+x1*x3**2*x4*x6**2+x2*x3**2*x4*x5**2
p(96) = x1*x2*x5*x6**3+x1*x3*x4*x6**3+x1*x2*x5**3*x6+x1*x3**3*x4&
&*x6+x2*x3*x4*x5**3+x2*x3**3*x4*x5
p(97) = x1*x4*x5*x6**3+x1*x2*x3*x6**3+x2*x4*x5**3*x6+x2*x3**3*x4&
&*x6+x1*x2*x3*x5**3+x1*x3**3*x4*x5
p(98) = x2*x4*x5**2*x6**2+x1*x4*x5**2*x6**2+x2*x3**2*x4*x6**2+x1&
&*x2*x3**2*x6**2+x1*x3**2*x4*x5**2+x1*x2*x3**2*x5**2
p(99) = x2*x4*x5*x6**3+x2*x3*x4*x6**3+x1*x4*x5**3*x6+x1*x2*x3**3&
&*x6+x1*x3*x4*x5**3+x1*x2*x3**3*x5
p(100) = x1*x2*x4*x6**3+x1*x2*x4*x5**3+x1*x2*x3**3*x4
p(101) = x1**2*x3*x5*x6**2+x2**2*x3*x5**2*x6+x3**2*x4**2*x5*x6
p(102) = x3*x4**2*x5*x6**2+x2**2*x3*x5*x6**2+x3*x4**2*x5**2*x6+x1&
&**2*x3*x5**2*x6+x2**2*x3**2*x5*x6+x1**2*x3**2*x5*x6
p(103) = x1**2*x2*x5*x6**2+x1**2*x3*x4*x6**2+x1*x2**2*x5**2*x6+x1&
&*x3**2*x4**2*x6+x2**2*x3*x4*x5**2+x2*x3**2*x4**2*x5
p(104) = x1*x2**2*x5*x6**2+x1*x3*x4**2*x6**2+x1**2*x2*x5**2*x6+x1&
&**2*x3**2*x4*x6+x2*x3*x4**2*x5**2+x2**2*x3**2*x4*x5
p(105) = x1**2*x2*x4*x6**2+x1*x2**2*x4*x5**2+x1*x2*x3**2*x4**2
p(106) = x1**2*x4*x5*x6**2+x1**2*x2*x3*x6**2+x2**2*x4*x5**2*x6+x2&
&*x3**2*x4**2*x6+x1*x2**2*x3*x5**2+x1*x3**2*x4**2*x5
p(107) = x1*x4**2*x5*x6**2+x1*x2**2*x3*x6**2+x2*x4**2*x5**2*x6+x2&
&**2*x3**2*x4*x6+x1**2*x2*x3*x5**2+x1**2*x3**2*x4*x5
p(108) = x2**2*x4*x5*x6**2+x2*x3*x4**2*x6**2+x1**2*x4*x5**2*x6+x1&
&**2*x2*x3**2*x6+x1*x3*x4**2*x5**2+x1*x2**2*x3**2*x5
p(109) = x2*x4**2*x5*x6**2+x2**2*x3*x4*x6**2+x1*x4**2*x5**2*x6+x1&
&*x2**2*x3**2*x6+x1**2*x3*x4*x5**2+x1**2*x2*x3**2*x5
p(110) = x1*x2*x4**2*x6**2+x1*x2**2*x4*x6**2+x1*x2*x4**2*x5**2+x1&
&**2*x2*x4*x5**2+x1*x2**2*x3**2*x4+x1**2*x2*x3**2*x4
p(111) = x1**2*x2**2*x5*x6+x1**2*x3*x4**2*x6+x2**2*x3*x4**2*x5
p(112) = x1**2*x2*x4**2*x6+x1**2*x2**2*x4*x6+x1*x2**2*x4**2*x5+x1&
&**2*x2**2*x4*x5+x1*x2**2*x3*x4**2+x1**2*x2*x3*x4**2
p(113) = x2**2*x4**2*x5*x6+x1**2*x4**2*x5*x6+x2**2*x3*x4**2*x6+x1&
&**2*x2**2*x3*x6+x1**2*x3*x4**2*x5+x1**2*x2**2*x3*x5
p(114) = x1*x2**2*x4**2*x6+x1**2*x2*x4**2*x5+x1**2*x2**2*x3*x4
p(115) = x3*x4**3*x5*x6+x2**3*x3*x5*x6+x1**3*x3*x5*x6
p(116) = x1*x2**3*x5*x6+x1**3*x2*x5*x6+x1*x3*x4**3*x6+x1**3*x3*x4&
&*x6+x2*x3*x4**3*x5+x2**3*x3*x4*x5
p(117) = x1**3*x2*x4*x6+x1*x2**3*x4*x5+x1*x2*x3*x4**3
p(118) = x2**3*x4*x5*x6+x1**3*x4*x5*x6+x2*x3*x4**3*x6+x1**3*x2*x3&
&*x6+x1*x3*x4**3*x5+x1*x2**3*x3*x5
p(119) = x2*x4**3*x5*x6+x1*x4**3*x5*x6+x2**3*x3*x4*x6+x1*x2**3*x3&
&*x6+x1**3*x3*x4*x5+x1**3*x2*x3*x5
p(120) = x1*x2*x4**3*x6+x1*x2**3*x4*x6+x1*x2*x4**3*x5+x1**3*x2*x4&
&*x5+x1*x2**3*x3*x4+x1**3*x2*x3*x4
p(121) = x3**2*x5**2*x6**2
p(122) = x3*x5**2*x6**3+x3**2*x5*x6**3+x3*x5**3*x6**2+x3**3*x5*x6&
&**2+x3**2*x5**3*x6+x3**3*x5**2*x6
p(123) = x3*x5*x6**4+x3*x5**4*x6+x3**4*x5*x6
p(124) = x1*x5*x6**4+x1*x3*x6**4+x2*x5**4*x6+x3**4*x4*x6+x2*x3*x5&
&**4+x3**4*x4*x5
p(125) = x1*x5**2*x6**3+x1*x3**2*x6**3+x2*x5**3*x6**2+x3**3*x4*x6&
&**2+x2*x3**2*x5**3+x3**3*x4*x5**2
p(126) = x2*x5**2*x6**3+x3**2*x4*x6**3+x1*x5**3*x6**2+x1*x3**3*x6&
&**2+x3**2*x4*x5**3+x2*x3**3*x5**2
p(127) = x4*x5**2*x6**3+x2*x3**2*x6**3+x4*x5**3*x6**2+x2*x3**3*x6&
&**2+x1*x3**2*x5**3+x1*x3**3*x5**2
p(128) = x2*x5*x6**4+x3*x4*x6**4+x1*x5**4*x6+x1*x3**4*x6+x3*x4*x5&
&**4+x2*x3**4*x5
p(129) = x4*x5*x6**4+x2*x3*x6**4+x4*x5**4*x6+x2*x3**4*x6+x1*x3*x5&
&**4+x1*x3**4*x5
p(130) = x1*x4*x6**4+x1*x2*x6**4+x2*x4*x5**4+x1*x2*x5**4+x2*x3**4&
&*x4+x1*x3**4*x4
p(131) = x2*x4*x6**4+x1*x4*x5**4+x1*x2*x3**4
p(132) = x1**2*x5*x6**3+x1**2*x3*x6**3+x2**2*x5**3*x6+x3**3*x4**2&
&*x6+x2**2*x3*x5**3+x3**3*x4**2*x5
p(133) = x2**2*x5**2*x6**2+x1**2*x5**2*x6**2+x3**2*x4**2*x6**2+x1&
&**2*x3**2*x6**2+x3**2*x4**2*x5**2+x2**2*x3**2*x5**2
p(134) = x4**2*x5**2*x6**2+x2**2*x3**2*x6**2+x1**2*x3**2*x5**2
p(135) = x2**2*x5*x6**3+x3*x4**2*x6**3+x1**2*x5**3*x6+x1**2*x3**3&
&*x6+x3*x4**2*x5**3+x2**2*x3**3*x5
p(136) = x4**2*x5*x6**3+x2**2*x3*x6**3+x4**2*x5**3*x6+x2**2*x3**3&
&*x6+x1**2*x3*x5**3+x1**2*x3**3*x5
p(137) = x1**2*x4*x6**3+x1**2*x2*x6**3+x2**2*x4*x5**3+x1*x2**2*x5&
&**3+x2*x3**3*x4**2+x1*x3**3*x4**2
p(138) = x1*x4**2*x6**3+x1*x2**2*x6**3+x2*x4**2*x5**3+x1**2*x2*x5&
&**3+x2**2*x3**3*x4+x1**2*x3**3*x4
p(139) = x2*x4**2*x6**3+x2**2*x4*x6**3+x1*x4**2*x5**3+x1**2*x4*x5&
&**3+x1*x2**2*x3**3+x1**2*x2*x3**3
p(140) = x1**2*x4**2*x6**2+x1**2*x2**2*x6**2+x2**2*x4**2*x5**2+x1&
&**2*x2**2*x5**2+x2**2*x3**2*x4**2+x1**2*x3**2*x4**2
p(141) = x1**2*x2**2*x4**2
p(142) = x2**2*x4**2*x6**2+x1**2*x4**2*x5**2+x1**2*x2**2*x3**2
p(143) = x1**3*x5*x6**2+x1**3*x3*x6**2+x2**3*x5**2*x6+x3**2*x4**3&
&*x6+x2**3*x3*x5**2+x3**2*x4**3*x5
p(144) = x2**3*x5*x6**2+x3*x4**3*x6**2+x1**3*x5**2*x6+x1**3*x3**2&
&*x6+x3*x4**3*x5**2+x2**3*x3**2*x5
p(145) = x4**3*x5*x6**2+x2**3*x3*x6**2+x4**3*x5**2*x6+x2**3*x3**2&
&*x6+x1**3*x3*x5**2+x1**3*x3**2*x5
p(146) = x1**3*x4*x6**2+x1**3*x2*x6**2+x2**3*x4*x5**2+x1*x2**3*x5&
&**2+x2*x3**2*x4**3+x1*x3**2*x4**3
p(147) = x1*x4**3*x6**2+x1*x2**3*x6**2+x2*x4**3*x5**2+x1**3*x2*x5&
&**2+x2**3*x3**2*x4+x1**3*x3**2*x4
p(148) = x2*x4**3*x6**2+x2**3*x4*x6**2+x1*x4**3*x5**2+x1**3*x4*x5&
&**2+x1*x2**3*x3**2+x1**3*x2*x3**2
p(149) = x1**3*x4**2*x6+x1**3*x2**2*x6+x2**3*x4**2*x5+x1**2*x2**3&
&*x5+x2**2*x3*x4**3+x1**2*x3*x4**3
p(150) = x1**2*x4**3*x6+x1**2*x2**3*x6+x2**2*x4**3*x5+x1**3*x2**2&
&*x5+x2**3*x3*x4**2+x1**3*x3*x4**2
p(151) = x1*x2**2*x4**3+x1**2*x2*x4**3+x1*x2**3*x4**2+x1**3*x2*x4&
&**2+x1**2*x2**3*x4+x1**3*x2**2*x4
p(152) = x2**2*x4**3*x6+x2**3*x4**2*x6+x1**2*x4**3*x5+x1**3*x4**2&
&*x5+x1**2*x2**3*x3+x1**3*x2**2*x3
p(153) = x2**4*x5*x6+x1**4*x5*x6+x3*x4**4*x6+x1**4*x3*x6+x3*x4**4&
&*x5+x2**4*x3*x5
p(154) = x4**4*x5*x6+x2**4*x3*x6+x1**4*x3*x5
p(155) = x1**4*x4*x6+x1**4*x2*x6+x2**4*x4*x5+x1*x2**4*x5+x2*x3*x4&
&**4+x1*x3*x4**4
p(156) = x1*x4**4*x6+x1*x2**4*x6+x2*x4**4*x5+x1**4*x2*x5+x2**4*x3&
&*x4+x1**4*x3*x4
p(157) = x1*x2*x4**4+x1*x2**4*x4+x1**4*x2*x4
p(158) = x2*x4**4*x6+x2**4*x4*x6+x1*x4**4*x5+x1**4*x4*x5+x1*x2**4&
&*x3+x1**4*x2*x3
p(159) = x5**3*x6**3+x3**3*x6**3+x3**3*x5**3
p(160) = x5**2*x6**4+x3**2*x6**4+x5**4*x6**2+x3**4*x6**2+x3**2*x5&
&**4+x3**4*x5**2
p(161) = x5*x6**5+x3*x6**5+x5**5*x6+x3**5*x6+x3*x5**5+x3**5*x5
p(162) = x4*x6**5+x2*x6**5+x4*x5**5+x1*x5**5+x2*x3**5+x1*x3**5
p(163) = x4**2*x6**4+x2**2*x6**4+x4**2*x5**4+x1**2*x5**4+x2**2*x3&
&**4+x1**2*x3**4
p(164) = x4**3*x6**3+x2**3*x6**3+x4**3*x5**3+x1**3*x5**3+x2**3*x3&
&**3+x1**3*x3**3
p(165) = x2**3*x4**3+x1**3*x4**3+x1**3*x2**3
p(166) = x4**4*x6**2+x2**4*x6**2+x4**4*x5**2+x1**4*x5**2+x2**4*x3&
&**2+x1**4*x3**2
p(167) = x2**2*x4**4+x1**2*x4**4+x2**4*x4**2+x1**4*x4**2+x1**2*x2&
&**4+x1**4*x2**2
p(168) = x4**5*x6+x2**5*x6+x4**5*x5+x1**5*x5+x2**5*x3+x1**5*x3
p(169) = x2*x4**5+x1*x4**5+x2**5*x4+x1**5*x4+x1*x2**5+x1**5*x2
p(170) = x1*x2*x3*x4*x5*x6**2+x1*x2*x3*x4*x5**2*x6+x1*x2*x3**2*x4&
&*x5*x6
p(171) = x1*x2*x3*x4**2*x5*x6+x1*x2**2*x3*x4*x5*x6+x1**2*x2*x3*x4&
&*x5*x6
p(172) = x1*x2*x3*x5**2*x6**2+x1*x3**2*x4*x5*x6**2+x2*x3**2*x4*x5&
&**2*x6
p(173) = x1*x3*x4*x5*x6**3+x1*x2*x3*x5*x6**3+x2*x3*x4*x5**3*x6+x1&
&*x2*x3*x5**3*x6+x2*x3**3*x4*x5*x6+x1*x3**3*x4*x5*x6
p(174) = x2*x3*x4*x5**2*x6**2+x1*x3*x4*x5**2*x6**2+x2*x3**2*x4*x5&
&*x6**2+x1*x2*x3**2*x5*x6**2+x1*x3**2*x4*x5**2*x6+x1*x2*x3**2*x5**&
&2*x6
p(175) = x1*x2*x4*x5**2*x6**2+x1*x2*x3**2*x4*x6**2+x1*x2*x3**2*x4&
&*x5**2
p(176) = x2*x3*x4*x5*x6**3+x1*x3*x4*x5**3*x6+x1*x2*x3**3*x5*x6
p(177) = x1*x2*x4*x5*x6**3+x1*x2*x3*x4*x6**3+x1*x2*x4*x5**3*x6+x1&
&*x2*x3**3*x4*x6+x1*x2*x3*x4*x5**3+x1*x2*x3**3*x4*x5
p(178) = x1**2*x3*x4*x5*x6**2+x1**2*x2*x3*x5*x6**2+x2**2*x3*x4*x5&
&**2*x6+x1*x2**2*x3*x5**2*x6+x2*x3**2*x4**2*x5*x6+x1*x3**2*x4**2*x&
&5*x6
p(179) = x1*x3*x4**2*x5*x6**2+x1*x2**2*x3*x5*x6**2+x2*x3*x4**2*x5&
&**2*x6+x1**2*x2*x3*x5**2*x6+x2**2*x3**2*x4*x5*x6+x1**2*x3**2*x4*x&
&5*x6
p(180) = x1**2*x2*x4*x5*x6**2+x1**2*x2*x3*x4*x6**2+x1*x2**2*x4*x5&
&**2*x6+x1*x2*x3**2*x4**2*x6+x1*x2**2*x3*x4*x5**2+x1*x2*x3**2*x4**&
&2*x5
p(181) = x2*x3*x4**2*x5*x6**2+x2**2*x3*x4*x5*x6**2+x1*x3*x4**2*x5&
&**2*x6+x1**2*x3*x4*x5**2*x6+x1*x2**2*x3**2*x5*x6+x1**2*x2*x3**2*x&
&5*x6
p(182) = x1*x2**2*x4*x5*x6**2+x1*x2*x3*x4**2*x6**2+x1**2*x2*x4*x5&
&**2*x6+x1**2*x2*x3**2*x4*x6+x1*x2*x3*x4**2*x5**2+x1*x2**2*x3**2*x&
&4*x5
p(183) = x1*x2*x4**2*x5*x6**2+x1*x2**2*x3*x4*x6**2+x1*x2*x4**2*x5&
&**2*x6+x1*x2**2*x3**2*x4*x6+x1**2*x2*x3*x4*x5**2+x1**2*x2*x3**2*x&
&4*x5
p(184) = x1**2*x2**2*x4*x5*x6+x1**2*x2*x3*x4**2*x6+x1*x2**2*x3*x4&
&**2*x5
p(185) = x2**2*x3*x4**2*x5*x6+x1**2*x3*x4**2*x5*x6+x1**2*x2**2*x3&
&*x5*x6
p(186) = x1*x2**2*x4**2*x5*x6+x1**2*x2*x4**2*x5*x6+x1*x2**2*x3*x4&
&**2*x6+x1**2*x2**2*x3*x4*x6+x1**2*x2*x3*x4**2*x5+x1**2*x2**2*x3*x&
&4*x5
p(187) = x2*x3*x4**3*x5*x6+x1*x3*x4**3*x5*x6+x2**3*x3*x4*x5*x6+x1&
&**3*x3*x4*x5*x6+x1*x2**3*x3*x5*x6+x1**3*x2*x3*x5*x6
p(188) = x1*x2**3*x4*x5*x6+x1**3*x2*x4*x5*x6+x1*x2*x3*x4**3*x6+x1&
&**3*x2*x3*x4*x6+x1*x2*x3*x4**3*x5+x1*x2**3*x3*x4*x5
p(189) = x1*x2*x4**3*x5*x6+x1*x2**3*x3*x4*x6+x1**3*x2*x3*x4*x5
p(190) = x1*x3*x5*x6**4+x2*x3*x5**4*x6+x3**4*x4*x5*x6
p(191) = x1*x3*x5**2*x6**3+x1*x3**2*x5*x6**3+x2*x3*x5**3*x6**2+x3&
&**3*x4*x5*x6**2+x2*x3**2*x5**3*x6+x3**3*x4*x5**2*x6
p(192) = x3**2*x4*x5**2*x6**2+x2*x3**2*x5**2*x6**2+x1*x3**2*x5**2&
&*x6**2
p(193) = x2*x3*x5**2*x6**3+x3**2*x4*x5*x6**3+x1*x3*x5**3*x6**2+x1&
&*x3**3*x5*x6**2+x3**2*x4*x5**3*x6+x2*x3**3*x5**2*x6
p(194) = x3*x4*x5**2*x6**3+x2*x3**2*x5*x6**3+x3*x4*x5**3*x6**2+x2&
&*x3**3*x5*x6**2+x1*x3**2*x5**3*x6+x1*x3**3*x5**2*x6
p(195) = x3*x4*x5*x6**4+x2*x3*x5*x6**4+x3*x4*x5**4*x6+x1*x3*x5**4&
&*x6+x2*x3**4*x5*x6+x1*x3**4*x5*x6
p(196) = x1*x2*x5**2*x6**3+x1*x3**2*x4*x6**3+x1*x2*x5**3*x6**2+x1&
&*x3**3*x4*x6**2+x2*x3**2*x4*x5**3+x2*x3**3*x4*x5**2
p(197) = x1*x2*x5*x6**4+x1*x3*x4*x6**4+x1*x2*x5**4*x6+x1*x3**4*x4&
&*x6+x2*x3*x4*x5**4+x2*x3**4*x4*x5
p(198) = x1*x4*x5*x6**4+x1*x2*x3*x6**4+x2*x4*x5**4*x6+x2*x3**4*x4&
&*x6+x1*x2*x3*x5**4+x1*x3**4*x4*x5
p(199) = x1*x4*x5**2*x6**3+x1*x2*x3**2*x6**3+x2*x4*x5**3*x6**2+x2&
&*x3**3*x4*x6**2+x1*x2*x3**2*x5**3+x1*x3**3*x4*x5**2
p(200) = x2*x4*x5**2*x6**3+x2*x3**2*x4*x6**3+x1*x4*x5**3*x6**2+x1&
&*x2*x3**3*x6**2+x1*x3**2*x4*x5**3+x1*x2*x3**3*x5**2
p(201) = x2*x4*x5*x6**4+x2*x3*x4*x6**4+x1*x4*x5**4*x6+x1*x2*x3**4&
&*x6+x1*x3*x4*x5**4+x1*x2*x3**4*x5
p(202) = x1*x2*x4*x6**4+x1*x2*x4*x5**4+x1*x2*x3**4*x4
p(203) = x1**2*x3*x5*x6**3+x2**2*x3*x5**3*x6+x3**3*x4**2*x5*x6
p(204) = x2**2*x3*x5**2*x6**2+x1**2*x3*x5**2*x6**2+x3**2*x4**2*x5&
&*x6**2+x1**2*x3**2*x5*x6**2+x3**2*x4**2*x5**2*x6+x2**2*x3**2*x5**&
&2*x6
p(205) = x3*x4**2*x5**2*x6**2+x2**2*x3**2*x5*x6**2+x1**2*x3**2*x5&
&**2*x6
p(206) = x3*x4**2*x5*x6**3+x2**2*x3*x5*x6**3+x3*x4**2*x5**3*x6+x1&
&**2*x3*x5**3*x6+x2**2*x3**3*x5*x6+x1**2*x3**3*x5*x6
p(207) = x1**2*x2*x5*x6**3+x1**2*x3*x4*x6**3+x1*x2**2*x5**3*x6+x1&
&*x3**3*x4**2*x6+x2**2*x3*x4*x5**3+x2*x3**3*x4**2*x5
p(208) = x1*x2**2*x5**2*x6**2+x1**2*x2*x5**2*x6**2+x1*x3**2*x4**2&
&*x6**2+x1**2*x3**2*x4*x6**2+x2*x3**2*x4**2*x5**2+x2**2*x3**2*x4*x&
&5**2
p(209) = x1*x2**2*x5*x6**3+x1*x3*x4**2*x6**3+x1**2*x2*x5**3*x6+x1&
&**2*x3**3*x4*x6+x2*x3*x4**2*x5**3+x2**2*x3**3*x4*x5
p(210) = x1**2*x2*x4*x6**3+x1*x2**2*x4*x5**3+x1*x2*x3**3*x4**2
p(211) = x1**2*x4*x5*x6**3+x1**2*x2*x3*x6**3+x2**2*x4*x5**3*x6+x2&
&*x3**3*x4**2*x6+x1*x2**2*x3*x5**3+x1*x3**3*x4**2*x5
p(212) = x1*x4**2*x5*x6**3+x1*x2**2*x3*x6**3+x2*x4**2*x5**3*x6+x2&
&**2*x3**3*x4*x6+x1**2*x2*x3*x5**3+x1**2*x3**3*x4*x5
p(213) = x2**2*x4*x5**2*x6**2+x1**2*x4*x5**2*x6**2+x2*x3**2*x4**2&
&*x6**2+x1**2*x2*x3**2*x6**2+x1*x3**2*x4**2*x5**2+x1*x2**2*x3**2*x&
&5**2
p(214) = x2*x4**2*x5**2*x6**2+x1*x4**2*x5**2*x6**2+x2**2*x3**2*x4&
&*x6**2+x1*x2**2*x3**2*x6**2+x1**2*x3**2*x4*x5**2+x1**2*x2*x3**2*x&
&5**2
p(215) = x2**2*x4*x5*x6**3+x2*x3*x4**2*x6**3+x1**2*x4*x5**3*x6+x1&
&**2*x2*x3**3*x6+x1*x3*x4**2*x5**3+x1*x2**2*x3**3*x5
p(216) = x2*x4**2*x5*x6**3+x2**2*x3*x4*x6**3+x1*x4**2*x5**3*x6+x1&
&*x2**2*x3**3*x6+x1**2*x3*x4*x5**3+x1**2*x2*x3**3*x5
p(217) = x1*x2*x4**2*x6**3+x1*x2**2*x4*x6**3+x1*x2*x4**2*x5**3+x1&
&**2*x2*x4*x5**3+x1*x2**2*x3**3*x4+x1**2*x2*x3**3*x4
p(218) = x1**2*x2**2*x5*x6**2+x1**2*x3*x4**2*x6**2+x1**2*x2**2*x5&
&**2*x6+x1**2*x3**2*x4**2*x6+x2**2*x3*x4**2*x5**2+x2**2*x3**2*x4**&
&2*x5
p(219) = x1**2*x2*x4**2*x6**2+x1**2*x2**2*x4*x6**2+x1*x2**2*x4**2&
&*x5**2+x1**2*x2**2*x4*x5**2+x1*x2**2*x3**2*x4**2+x1**2*x2*x3**2*x&
&4**2
p(220) = x1**2*x4**2*x5*x6**2+x1**2*x2**2*x3*x6**2+x2**2*x4**2*x5&
&**2*x6+x2**2*x3**2*x4**2*x6+x1**2*x2**2*x3*x5**2+x1**2*x3**2*x4**&
&2*x5
p(221) = x1**2*x2**2*x4**2*x6+x1**2*x2**2*x4**2*x5+x1**2*x2**2*x3&
&*x4**2
p(222) = x2**2*x4**2*x5*x6**2+x2**2*x3*x4**2*x6**2+x1**2*x4**2*x5&
&**2*x6+x1**2*x2**2*x3**2*x6+x1**2*x3*x4**2*x5**2+x1**2*x2**2*x3**&
&2*x5
p(223) = x1*x2**2*x4**2*x6**2+x1**2*x2*x4**2*x5**2+x1**2*x2**2*x3&
&**2*x4
p(224) = x1**3*x3*x5*x6**2+x2**3*x3*x5**2*x6+x3**2*x4**3*x5*x6
p(225) = x3*x4**3*x5*x6**2+x2**3*x3*x5*x6**2+x3*x4**3*x5**2*x6+x1&
&**3*x3*x5**2*x6+x2**3*x3**2*x5*x6+x1**3*x3**2*x5*x6
p(226) = x1**3*x2*x5*x6**2+x1**3*x3*x4*x6**2+x1*x2**3*x5**2*x6+x1&
&*x3**2*x4**3*x6+x2**3*x3*x4*x5**2+x2*x3**2*x4**3*x5
p(227) = x1*x2**3*x5*x6**2+x1*x3*x4**3*x6**2+x1**3*x2*x5**2*x6+x1&
&**3*x3**2*x4*x6+x2*x3*x4**3*x5**2+x2**3*x3**2*x4*x5
p(228) = x1**3*x2*x4*x6**2+x1*x2**3*x4*x5**2+x1*x2*x3**2*x4**3
p(229) = x1**3*x4*x5*x6**2+x1**3*x2*x3*x6**2+x2**3*x4*x5**2*x6+x2&
&*x3**2*x4**3*x6+x1*x2**3*x3*x5**2+x1*x3**2*x4**3*x5
p(230) = x1*x4**3*x5*x6**2+x1*x2**3*x3*x6**2+x2*x4**3*x5**2*x6+x2&
&**3*x3**2*x4*x6+x1**3*x2*x3*x5**2+x1**3*x3**2*x4*x5
p(231) = x2**3*x4*x5*x6**2+x2*x3*x4**3*x6**2+x1**3*x4*x5**2*x6+x1&
&**3*x2*x3**2*x6+x1*x3*x4**3*x5**2+x1*x2**3*x3**2*x5
p(232) = x2*x4**3*x5*x6**2+x2**3*x3*x4*x6**2+x1*x4**3*x5**2*x6+x1&
&*x2**3*x3**2*x6+x1**3*x3*x4*x5**2+x1**3*x2*x3**2*x5
p(233) = x1*x2*x4**3*x6**2+x1*x2**3*x4*x6**2+x1*x2*x4**3*x5**2+x1&
&**3*x2*x4*x5**2+x1*x2**3*x3**2*x4+x1**3*x2*x3**2*x4
p(234) = x1**2*x2**3*x5*x6+x1**3*x2**2*x5*x6+x1**2*x3*x4**3*x6+x1&
&**3*x3*x4**2*x6+x2**2*x3*x4**3*x5+x2**3*x3*x4**2*x5
p(235) = x1**3*x2*x4**2*x6+x1**3*x2**2*x4*x6+x1*x2**3*x4**2*x5+x1&
&**2*x2**3*x4*x5+x1*x2**2*x3*x4**3+x1**2*x2*x3*x4**3
p(236) = x1**2*x2*x4**3*x6+x1**2*x2**3*x4*x6+x1*x2**2*x4**3*x5+x1&
&**3*x2**2*x4*x5+x1*x2**3*x3*x4**2+x1**3*x2*x3*x4**2
p(237) = x2**3*x4**2*x5*x6+x1**3*x4**2*x5*x6+x2**2*x3*x4**3*x6+x1&
&**3*x2**2*x3*x6+x1**2*x3*x4**3*x5+x1**2*x2**3*x3*x5
p(238) = x2**2*x4**3*x5*x6+x1**2*x4**3*x5*x6+x2**3*x3*x4**2*x6+x1&
&**2*x2**3*x3*x6+x1**3*x3*x4**2*x5+x1**3*x2**2*x3*x5
p(239) = x1*x2**2*x4**3*x6+x1*x2**3*x4**2*x6+x1**2*x2*x4**3*x5+x1&
&**3*x2*x4**2*x5+x1**2*x2**3*x3*x4+x1**3*x2**2*x3*x4
p(240) = x3*x4**4*x5*x6+x2**4*x3*x5*x6+x1**4*x3*x5*x6
p(241) = x1*x2**4*x5*x6+x1**4*x2*x5*x6+x1*x3*x4**4*x6+x1**4*x3*x4&
&*x6+x2*x3*x4**4*x5+x2**4*x3*x4*x5
p(242) = x1**4*x2*x4*x6+x1*x2**4*x4*x5+x1*x2*x3*x4**4
p(243) = x2**4*x4*x5*x6+x1**4*x4*x5*x6+x2*x3*x4**4*x6+x1**4*x2*x3&
&*x6+x1*x3*x4**4*x5+x1*x2**4*x3*x5
p(244) = x2*x4**4*x5*x6+x1*x4**4*x5*x6+x2**4*x3*x4*x6+x1*x2**4*x3&
&*x6+x1**4*x3*x4*x5+x1**4*x2*x3*x5
p(245) = x1*x2*x4**4*x6+x1*x2**4*x4*x6+x1*x2*x4**4*x5+x1**4*x2*x4&
&*x5+x1*x2**4*x3*x4+x1**4*x2*x3*x4
p(246) = x3**2*x5**2*x6**3+x3**2*x5**3*x6**2+x3**3*x5**2*x6**2
p(247) = x3*x5**3*x6**3+x3**3*x5*x6**3+x3**3*x5**3*x6
p(248) = x3*x5**2*x6**4+x3**2*x5*x6**4+x3*x5**4*x6**2+x3**4*x5*x6&
&**2+x3**2*x5**4*x6+x3**4*x5**2*x6
p(249) = x3*x5*x6**5+x3*x5**5*x6+x3**5*x5*x6
p(250) = x1*x5*x6**5+x1*x3*x6**5+x2*x5**5*x6+x3**5*x4*x6+x2*x3*x5&
&**5+x3**5*x4*x5
p(251) = x1*x5**2*x6**4+x1*x3**2*x6**4+x2*x5**4*x6**2+x3**4*x4*x6&
&**2+x2*x3**2*x5**4+x3**4*x4*x5**2
p(252) = x2*x5**3*x6**3+x1*x5**3*x6**3+x3**3*x4*x6**3+x1*x3**3*x6&
&**3+x3**3*x4*x5**3+x2*x3**3*x5**3
p(253) = x4*x5**3*x6**3+x2*x3**3*x6**3+x1*x3**3*x5**3
p(254) = x2*x5**2*x6**4+x3**2*x4*x6**4+x1*x5**4*x6**2+x1*x3**4*x6&
&**2+x3**2*x4*x5**4+x2*x3**4*x5**2
p(255) = x4*x5**2*x6**4+x2*x3**2*x6**4+x4*x5**4*x6**2+x2*x3**4*x6&
&**2+x1*x3**2*x5**4+x1*x3**4*x5**2
p(256) = x2*x5*x6**5+x3*x4*x6**5+x1*x5**5*x6+x1*x3**5*x6+x3*x4*x5&
&**5+x2*x3**5*x5
p(257) = x4*x5*x6**5+x2*x3*x6**5+x4*x5**5*x6+x2*x3**5*x6+x1*x3*x5&
&**5+x1*x3**5*x5
p(258) = x1*x4*x6**5+x1*x2*x6**5+x2*x4*x5**5+x1*x2*x5**5+x2*x3**5&
&*x4+x1*x3**5*x4
p(259) = x2*x4*x6**5+x1*x4*x5**5+x1*x2*x3**5
p(260) = x1**2*x5*x6**4+x1**2*x3*x6**4+x2**2*x5**4*x6+x3**4*x4**2&
&*x6+x2**2*x3*x5**4+x3**4*x4**2*x5
p(261) = x1**2*x5**2*x6**3+x1**2*x3**2*x6**3+x2**2*x5**3*x6**2+x3&
&**3*x4**2*x6**2+x2**2*x3**2*x5**3+x3**3*x4**2*x5**2
p(262) = x2**2*x5**2*x6**3+x3**2*x4**2*x6**3+x1**2*x5**3*x6**2+x1&
&**2*x3**3*x6**2+x3**2*x4**2*x5**3+x2**2*x3**3*x5**2
p(263) = x4**2*x5**2*x6**3+x2**2*x3**2*x6**3+x4**2*x5**3*x6**2+x2&
&**2*x3**3*x6**2+x1**2*x3**2*x5**3+x1**2*x3**3*x5**2
p(264) = x2**2*x5*x6**4+x3*x4**2*x6**4+x1**2*x5**4*x6+x1**2*x3**4&
&*x6+x3*x4**2*x5**4+x2**2*x3**4*x5
p(265) = x4**2*x5*x6**4+x2**2*x3*x6**4+x4**2*x5**4*x6+x2**2*x3**4&
&*x6+x1**2*x3*x5**4+x1**2*x3**4*x5
p(266) = x1**2*x4*x6**4+x1**2*x2*x6**4+x2**2*x4*x5**4+x1*x2**2*x5&
&**4+x2*x3**4*x4**2+x1*x3**4*x4**2
p(267) = x1*x4**2*x6**4+x1*x2**2*x6**4+x2*x4**2*x5**4+x1**2*x2*x5&
&**4+x2**2*x3**4*x4+x1**2*x3**4*x4
p(268) = x2*x4**2*x6**4+x2**2*x4*x6**4+x1*x4**2*x5**4+x1**2*x4*x5&
&**4+x1*x2**2*x3**4+x1**2*x2*x3**4
p(269) = x1**2*x4**2*x6**3+x1**2*x2**2*x6**3+x2**2*x4**2*x5**3+x1&
&**2*x2**2*x5**3+x2**2*x3**3*x4**2+x1**2*x3**3*x4**2
p(270) = x2**2*x4**2*x6**3+x1**2*x4**2*x5**3+x1**2*x2**2*x3**3
p(271) = x1**3*x5*x6**3+x1**3*x3*x6**3+x2**3*x5**3*x6+x3**3*x4**3&
&*x6+x2**3*x3*x5**3+x3**3*x4**3*x5
p(272) = x2**3*x5**2*x6**2+x1**3*x5**2*x6**2+x3**2*x4**3*x6**2+x1&
&**3*x3**2*x6**2+x3**2*x4**3*x5**2+x2**3*x3**2*x5**2
p(273) = x4**3*x5**2*x6**2+x2**3*x3**2*x6**2+x1**3*x3**2*x5**2
p(274) = x2**3*x5*x6**3+x3*x4**3*x6**3+x1**3*x5**3*x6+x1**3*x3**3&
&*x6+x3*x4**3*x5**3+x2**3*x3**3*x5
p(275) = x4**3*x5*x6**3+x2**3*x3*x6**3+x4**3*x5**3*x6+x2**3*x3**3&
&*x6+x1**3*x3*x5**3+x1**3*x3**3*x5
p(276) = x1**3*x4*x6**3+x1**3*x2*x6**3+x2**3*x4*x5**3+x1*x2**3*x5&
&**3+x2*x3**3*x4**3+x1*x3**3*x4**3
p(277) = x1*x4**3*x6**3+x1*x2**3*x6**3+x2*x4**3*x5**3+x1**3*x2*x5&
&**3+x2**3*x3**3*x4+x1**3*x3**3*x4
p(278) = x2*x4**3*x6**3+x2**3*x4*x6**3+x1*x4**3*x5**3+x1**3*x4*x5&
&**3+x1*x2**3*x3**3+x1**3*x2*x3**3
p(279) = x1**3*x4**2*x6**2+x1**3*x2**2*x6**2+x2**3*x4**2*x5**2+x1&
&**2*x2**3*x5**2+x2**2*x3**2*x4**3+x1**2*x3**2*x4**3
p(280) = x1**2*x4**3*x6**2+x1**2*x2**3*x6**2+x2**2*x4**3*x5**2+x1&
&**3*x2**2*x5**2+x2**3*x3**2*x4**2+x1**3*x3**2*x4**2
p(281) = x1**2*x2**2*x4**3+x1**2*x2**3*x4**2+x1**3*x2**2*x4**2
p(282) = x2**2*x4**3*x6**2+x2**3*x4**2*x6**2+x1**2*x4**3*x5**2+x1&
&**3*x4**2*x5**2+x1**2*x2**3*x3**2+x1**3*x2**2*x3**2
p(283) = x1**3*x4**3*x6+x1**3*x2**3*x6+x2**3*x4**3*x5+x1**3*x2**3&
&*x5+x2**3*x3*x4**3+x1**3*x3*x4**3
p(284) = x1*x2**3*x4**3+x1**3*x2*x4**3+x1**3*x2**3*x4
p(285) = x2**3*x4**3*x6+x1**3*x4**3*x5+x1**3*x2**3*x3
p(286) = x1**4*x5*x6**2+x1**4*x3*x6**2+x2**4*x5**2*x6+x3**2*x4**4&
&*x6+x2**4*x3*x5**2+x3**2*x4**4*x5
p(287) = x2**4*x5*x6**2+x3*x4**4*x6**2+x1**4*x5**2*x6+x1**4*x3**2&
&*x6+x3*x4**4*x5**2+x2**4*x3**2*x5
p(288) = x4**4*x5*x6**2+x2**4*x3*x6**2+x4**4*x5**2*x6+x2**4*x3**2&
&*x6+x1**4*x3*x5**2+x1**4*x3**2*x5
p(289) = x1**4*x4*x6**2+x1**4*x2*x6**2+x2**4*x4*x5**2+x1*x2**4*x5&
&**2+x2*x3**2*x4**4+x1*x3**2*x4**4
p(290) = x1*x4**4*x6**2+x1*x2**4*x6**2+x2*x4**4*x5**2+x1**4*x2*x5&
&**2+x2**4*x3**2*x4+x1**4*x3**2*x4
p(291) = x2*x4**4*x6**2+x2**4*x4*x6**2+x1*x4**4*x5**2+x1**4*x4*x5&
&**2+x1*x2**4*x3**2+x1**4*x2*x3**2
p(292) = x1**4*x4**2*x6+x1**4*x2**2*x6+x2**4*x4**2*x5+x1**2*x2**4&
&*x5+x2**2*x3*x4**4+x1**2*x3*x4**4
p(293) = x1**2*x4**4*x6+x1**2*x2**4*x6+x2**2*x4**4*x5+x1**4*x2**2&
&*x5+x2**4*x3*x4**2+x1**4*x3*x4**2
p(294) = x1*x2**2*x4**4+x1**2*x2*x4**4+x1*x2**4*x4**2+x1**4*x2*x4&
&**2+x1**2*x2**4*x4+x1**4*x2**2*x4
p(295) = x2**2*x4**4*x6+x2**4*x4**2*x6+x1**2*x4**4*x5+x1**4*x4**2&
&*x5+x1**2*x2**4*x3+x1**4*x2**2*x3
p(296) = x2**5*x5*x6+x1**5*x5*x6+x3*x4**5*x6+x1**5*x3*x6+x3*x4**5&
&*x5+x2**5*x3*x5
p(297) = x4**5*x5*x6+x2**5*x3*x6+x1**5*x3*x5
p(298) = x1**5*x4*x6+x1**5*x2*x6+x2**5*x4*x5+x1*x2**5*x5+x2*x3*x4&
&**5+x1*x3*x4**5
p(299) = x1*x4**5*x6+x1*x2**5*x6+x2*x4**5*x5+x1**5*x2*x5+x2**5*x3&
&*x4+x1**5*x3*x4
p(300) = x1*x2*x4**5+x1*x2**5*x4+x1**5*x2*x4
p(301) = x2*x4**5*x6+x2**5*x4*x6+x1*x4**5*x5+x1**5*x4*x5+x1*x2**5&
&*x3+x1**5*x2*x3
p(302) = x5**3*x6**4+x3**3*x6**4+x5**4*x6**3+x3**4*x6**3+x3**3*x5&
&**4+x3**4*x5**3
p(303) = x5**2*x6**5+x3**2*x6**5+x5**5*x6**2+x3**5*x6**2+x3**2*x5&
&**5+x3**5*x5**2
p(304) = x5*x6**6+x3*x6**6+x5**6*x6+x3**6*x6+x3*x5**6+x3**6*x5
p(305) = x4*x6**6+x2*x6**6+x4*x5**6+x1*x5**6+x2*x3**6+x1*x3**6
p(306) = x4**2*x6**5+x2**2*x6**5+x4**2*x5**5+x1**2*x5**5+x2**2*x3&
&**5+x1**2*x3**5
p(307) = x4**3*x6**4+x2**3*x6**4+x4**3*x5**4+x1**3*x5**4+x2**3*x3&
&**4+x1**3*x3**4
p(308) = x4**4*x6**3+x2**4*x6**3+x4**4*x5**3+x1**4*x5**3+x2**4*x3&
&**3+x1**4*x3**3
p(309) = x2**3*x4**4+x1**3*x4**4+x2**4*x4**3+x1**4*x4**3+x1**3*x2&
&**4+x1**4*x2**3
p(310) = x4**5*x6**2+x2**5*x6**2+x4**5*x5**2+x1**5*x5**2+x2**5*x3&
&**2+x1**5*x3**2
p(311) = x2**2*x4**5+x1**2*x4**5+x2**5*x4**2+x1**5*x4**2+x1**2*x2&
&**5+x1**5*x2**2
p(312) = x4**6*x6+x2**6*x6+x4**6*x5+x1**6*x5+x2**6*x3+x1**6*x3
p(313) = x2*x4**6+x1*x4**6+x2**6*x4+x1**6*x4+x1*x2**6+x1**6*x2
p(314) = x1*x2*x3*x4*x5**2*x6**2+x1*x2*x3**2*x4*x5*x6**2+x1*x2*x3&
&**2*x4*x5**2*x6
p(315) = x1*x2*x3*x4*x5*x6**3+x1*x2*x3*x4*x5**3*x6+x1*x2*x3**3*x4&
&*x5*x6
p(316) = x1**2*x2*x3*x4*x5*x6**2+x1*x2**2*x3*x4*x5**2*x6+x1*x2*x3&
&**2*x4**2*x5*x6
p(317) = x1*x2*x3*x4**2*x5*x6**2+x1*x2**2*x3*x4*x5*x6**2+x1*x2*x3&
&*x4**2*x5**2*x6+x1**2*x2*x3*x4*x5**2*x6+x1*x2**2*x3**2*x4*x5*x6+x&
&1**2*x2*x3**2*x4*x5*x6
p(318) = x1*x2**2*x3*x4**2*x5*x6+x1**2*x2*x3*x4**2*x5*x6+x1**2*x2&
&**2*x3*x4*x5*x6
p(319) = x1*x2*x3*x4**3*x5*x6+x1*x2**3*x3*x4*x5*x6+x1**3*x2*x3*x4&
&*x5*x6
p(320) = x1*x2*x3*x5**2*x6**3+x1*x3**2*x4*x5*x6**3+x1*x2*x3*x5**3&
&*x6**2+x1*x3**3*x4*x5*x6**2+x2*x3**2*x4*x5**3*x6+x2*x3**3*x4*x5**&
&2*x6
p(321) = x1*x3*x4*x5*x6**4+x1*x2*x3*x5*x6**4+x2*x3*x4*x5**4*x6+x1&
&*x2*x3*x5**4*x6+x2*x3**4*x4*x5*x6+x1*x3**4*x4*x5*x6
p(322) = x2*x3**2*x4*x5**2*x6**2+x1*x3**2*x4*x5**2*x6**2+x1*x2*x3&
&**2*x5**2*x6**2
p(323) = x1*x3*x4*x5**2*x6**3+x1*x2*x3**2*x5*x6**3+x2*x3*x4*x5**3&
&*x6**2+x2*x3**3*x4*x5*x6**2+x1*x2*x3**2*x5**3*x6+x1*x3**3*x4*x5**&
&2*x6
p(324) = x2*x3*x4*x5**2*x6**3+x2*x3**2*x4*x5*x6**3+x1*x3*x4*x5**3&
&*x6**2+x1*x2*x3**3*x5*x6**2+x1*x3**2*x4*x5**3*x6+x1*x2*x3**3*x5**&
&2*x6
p(325) = x1*x2*x4*x5**2*x6**3+x1*x2*x3**2*x4*x6**3+x1*x2*x4*x5**3&
&*x6**2+x1*x2*x3**3*x4*x6**2+x1*x2*x3**2*x4*x5**3+x1*x2*x3**3*x4*x&
&5**2
p(326) = x2*x3*x4*x5*x6**4+x1*x3*x4*x5**4*x6+x1*x2*x3**4*x5*x6
p(327) = x1*x2*x4*x5*x6**4+x1*x2*x3*x4*x6**4+x1*x2*x4*x5**4*x6+x1&
&*x2*x3**4*x4*x6+x1*x2*x3*x4*x5**4+x1*x2*x3**4*x4*x5
p(328) = x1**2*x3*x4*x5*x6**3+x1**2*x2*x3*x5*x6**3+x2**2*x3*x4*x5&
&**3*x6+x1*x2**2*x3*x5**3*x6+x2*x3**3*x4**2*x5*x6+x1*x3**3*x4**2*x&
&5*x6
p(329) = x1*x2**2*x3*x5**2*x6**2+x1**2*x2*x3*x5**2*x6**2+x1*x3**2&
&*x4**2*x5*x6**2+x1**2*x3**2*x4*x5*x6**2+x2*x3**2*x4**2*x5**2*x6+x&
&2**2*x3**2*x4*x5**2*x6
p(330) = x1*x3*x4**2*x5*x6**3+x1*x2**2*x3*x5*x6**3+x2*x3*x4**2*x5&
&**3*x6+x1**2*x2*x3*x5**3*x6+x2**2*x3**3*x4*x5*x6+x1**2*x3**3*x4*x&
&5*x6
p(331) = x1**2*x2*x4*x5*x6**3+x1**2*x2*x3*x4*x6**3+x1*x2**2*x4*x5&
&**3*x6+x1*x2*x3**3*x4**2*x6+x1*x2**2*x3*x4*x5**3+x1*x2*x3**3*x4**&
&2*x5
p(332) = x2**2*x3*x4*x5**2*x6**2+x1**2*x3*x4*x5**2*x6**2+x2*x3**2&
&*x4**2*x5*x6**2+x1**2*x2*x3**2*x5*x6**2+x1*x3**2*x4**2*x5**2*x6+x&
&1*x2**2*x3**2*x5**2*x6
p(333) = x2*x3*x4**2*x5**2*x6**2+x1*x3*x4**2*x5**2*x6**2+x2**2*x3&
&**2*x4*x5*x6**2+x1*x2**2*x3**2*x5*x6**2+x1**2*x3**2*x4*x5**2*x6+x&
&1**2*x2*x3**2*x5**2*x6
p(334) = x1*x2**2*x4*x5**2*x6**2+x1**2*x2*x4*x5**2*x6**2+x1*x2*x3&
&**2*x4**2*x6**2+x1**2*x2*x3**2*x4*x6**2+x1*x2*x3**2*x4**2*x5**2+x&
&1*x2**2*x3**2*x4*x5**2
p(335) = x1*x2*x4**2*x5**2*x6**2+x1*x2**2*x3**2*x4*x6**2+x1**2*x2&
&*x3**2*x4*x5**2
p(336) = x2*x3*x4**2*x5*x6**3+x2**2*x3*x4*x5*x6**3+x1*x3*x4**2*x5&
&**3*x6+x1**2*x3*x4*x5**3*x6+x1*x2**2*x3**3*x5*x6+x1**2*x2*x3**3*x&
&5*x6
p(337) = x1*x2**2*x4*x5*x6**3+x1*x2*x3*x4**2*x6**3+x1**2*x2*x4*x5&
&**3*x6+x1**2*x2*x3**3*x4*x6+x1*x2*x3*x4**2*x5**3+x1*x2**2*x3**3*x&
&4*x5
p(338) = x1*x2*x4**2*x5*x6**3+x1*x2**2*x3*x4*x6**3+x1*x2*x4**2*x5&
&**3*x6+x1*x2**2*x3**3*x4*x6+x1**2*x2*x3*x4*x5**3+x1**2*x2*x3**3*x&
&4*x5
p(339) = x1**2*x2**2*x4*x5*x6**2+x1**2*x2*x3*x4**2*x6**2+x1**2*x2&
&**2*x4*x5**2*x6+x1**2*x2*x3**2*x4**2*x6+x1*x2**2*x3*x4**2*x5**2+x&
&1*x2**2*x3**2*x4**2*x5
p(340) = x1**2*x3*x4**2*x5*x6**2+x1**2*x2**2*x3*x5*x6**2+x2**2*x3&
&*x4**2*x5**2*x6+x1**2*x2**2*x3*x5**2*x6+x2**2*x3**2*x4**2*x5*x6+x&
&1**2*x3**2*x4**2*x5*x6
p(341) = x1**2*x2*x4**2*x5*x6**2+x1**2*x2**2*x3*x4*x6**2+x1*x2**2&
&*x4**2*x5**2*x6+x1*x2**2*x3**2*x4**2*x6+x1**2*x2**2*x3*x4*x5**2+x&
&1**2*x2*x3**2*x4**2*x5
p(342) = x1**2*x2**2*x4**2*x5*x6+x1**2*x2**2*x3*x4**2*x6+x1**2*x2&
&**2*x3*x4**2*x5
p(343) = x2**2*x3*x4**2*x5*x6**2+x1**2*x3*x4**2*x5**2*x6+x1**2*x2&
&**2*x3**2*x5*x6
p(344) = x1*x2**2*x4**2*x5*x6**2+x1*x2**2*x3*x4**2*x6**2+x1**2*x2&
&*x4**2*x5**2*x6+x1**2*x2**2*x3**2*x4*x6+x1**2*x2*x3*x4**2*x5**2+x&
&1**2*x2**2*x3**2*x4*x5
p(345) = x1**3*x3*x4*x5*x6**2+x1**3*x2*x3*x5*x6**2+x2**3*x3*x4*x5&
&**2*x6+x1*x2**3*x3*x5**2*x6+x2*x3**2*x4**3*x5*x6+x1*x3**2*x4**3*x&
&5*x6
p(346) = x1*x3*x4**3*x5*x6**2+x1*x2**3*x3*x5*x6**2+x2*x3*x4**3*x5&
&**2*x6+x1**3*x2*x3*x5**2*x6+x2**3*x3**2*x4*x5*x6+x1**3*x3**2*x4*x&
&5*x6
p(347) = x1**3*x2*x4*x5*x6**2+x1**3*x2*x3*x4*x6**2+x1*x2**3*x4*x5&
&**2*x6+x1*x2*x3**2*x4**3*x6+x1*x2**3*x3*x4*x5**2+x1*x2*x3**2*x4**&
&3*x5
p(348) = x2*x3*x4**3*x5*x6**2+x2**3*x3*x4*x5*x6**2+x1*x3*x4**3*x5&
&**2*x6+x1**3*x3*x4*x5**2*x6+x1*x2**3*x3**2*x5*x6+x1**3*x2*x3**2*x&
&5*x6
p(349) = x1*x2**3*x4*x5*x6**2+x1*x2*x3*x4**3*x6**2+x1**3*x2*x4*x5&
&**2*x6+x1**3*x2*x3**2*x4*x6+x1*x2*x3*x4**3*x5**2+x1*x2**3*x3**2*x&
&4*x5
p(350) = x1*x2*x4**3*x5*x6**2+x1*x2**3*x3*x4*x6**2+x1*x2*x4**3*x5&
&**2*x6+x1*x2**3*x3**2*x4*x6+x1**3*x2*x3*x4*x5**2+x1**3*x2*x3**2*x&
&4*x5
p(351) = x1**2*x2**3*x4*x5*x6+x1**3*x2**2*x4*x5*x6+x1**2*x2*x3*x4&
&**3*x6+x1**3*x2*x3*x4**2*x6+x1*x2**2*x3*x4**3*x5+x1*x2**3*x3*x4**&
&2*x5
p(352) = x2**2*x3*x4**3*x5*x6+x1**2*x3*x4**3*x5*x6+x2**3*x3*x4**2&
&*x5*x6+x1**3*x3*x4**2*x5*x6+x1**2*x2**3*x3*x5*x6+x1**3*x2**2*x3*x&
&5*x6
p(353) = x1*x2**3*x4**2*x5*x6+x1**3*x2*x4**2*x5*x6+x1*x2**2*x3*x4&
&**3*x6+x1**3*x2**2*x3*x4*x6+x1**2*x2*x3*x4**3*x5+x1**2*x2**3*x3*x&
&4*x5
p(354) = x1*x2**2*x4**3*x5*x6+x1**2*x2*x4**3*x5*x6+x1*x2**3*x3*x4&
&**2*x6+x1**2*x2**3*x3*x4*x6+x1**3*x2*x3*x4**2*x5+x1**3*x2**2*x3*x&
&4*x5
p(355) = x2*x3*x4**4*x5*x6+x1*x3*x4**4*x5*x6+x2**4*x3*x4*x5*x6+x1&
&**4*x3*x4*x5*x6+x1*x2**4*x3*x5*x6+x1**4*x2*x3*x5*x6
p(356) = x1*x2**4*x4*x5*x6+x1**4*x2*x4*x5*x6+x1*x2*x3*x4**4*x6+x1&
&**4*x2*x3*x4*x6+x1*x2*x3*x4**4*x5+x1*x2**4*x3*x4*x5
p(357) = x1*x2*x4**4*x5*x6+x1*x2**4*x3*x4*x6+x1**4*x2*x3*x4*x5
p(358) = x1*x3*x5*x6**5+x2*x3*x5**5*x6+x3**5*x4*x5*x6
p(359) = x1*x3*x5**2*x6**4+x1*x3**2*x5*x6**4+x2*x3*x5**4*x6**2+x3&
&**4*x4*x5*x6**2+x2*x3**2*x5**4*x6+x3**4*x4*x5**2*x6
p(360) = x1*x3**2*x5**2*x6**3+x2*x3**2*x5**3*x6**2+x3**3*x4*x5**2&
&*x6**2
p(361) = x2*x3*x5**3*x6**3+x1*x3*x5**3*x6**3+x3**3*x4*x5*x6**3+x1&
&*x3**3*x5*x6**3+x3**3*x4*x5**3*x6+x2*x3**3*x5**3*x6
p(362) = x3**2*x4*x5**2*x6**3+x2*x3**2*x5**2*x6**3+x3**2*x4*x5**3&
&*x6**2+x1*x3**2*x5**3*x6**2+x2*x3**3*x5**2*x6**2+x1*x3**3*x5**2*x&
&6**2
p(363) = x3*x4*x5**3*x6**3+x2*x3**3*x5*x6**3+x1*x3**3*x5**3*x6
p(364) = x2*x3*x5**2*x6**4+x3**2*x4*x5*x6**4+x1*x3*x5**4*x6**2+x1&
&*x3**4*x5*x6**2+x3**2*x4*x5**4*x6+x2*x3**4*x5**2*x6
p(365) = x3*x4*x5**2*x6**4+x2*x3**2*x5*x6**4+x3*x4*x5**4*x6**2+x2&
&*x3**4*x5*x6**2+x1*x3**2*x5**4*x6+x1*x3**4*x5**2*x6
p(366) = x3*x4*x5*x6**5+x2*x3*x5*x6**5+x3*x4*x5**5*x6+x1*x3*x5**5&
&*x6+x2*x3**5*x5*x6+x1*x3**5*x5*x6
p(367) = x1*x2*x5**3*x6**3+x1*x3**3*x4*x6**3+x2*x3**3*x4*x5**3
p(368) = x1*x2*x5**2*x6**4+x1*x3**2*x4*x6**4+x1*x2*x5**4*x6**2+x1&
&*x3**4*x4*x6**2+x2*x3**2*x4*x5**4+x2*x3**4*x4*x5**2
p(369) = x1*x2*x5*x6**5+x1*x3*x4*x6**5+x1*x2*x5**5*x6+x1*x3**5*x4&
&*x6+x2*x3*x4*x5**5+x2*x3**5*x4*x5
p(370) = x1*x4*x5*x6**5+x1*x2*x3*x6**5+x2*x4*x5**5*x6+x2*x3**5*x4&
&*x6+x1*x2*x3*x5**5+x1*x3**5*x4*x5
p(371) = x1*x4*x5**2*x6**4+x1*x2*x3**2*x6**4+x2*x4*x5**4*x6**2+x2&
&*x3**4*x4*x6**2+x1*x2*x3**2*x5**4+x1*x3**4*x4*x5**2
p(372) = x2*x4*x5**3*x6**3+x1*x4*x5**3*x6**3+x2*x3**3*x4*x6**3+x1&
&*x2*x3**3*x6**3+x1*x3**3*x4*x5**3+x1*x2*x3**3*x5**3
p(373) = x2*x4*x5**2*x6**4+x2*x3**2*x4*x6**4+x1*x4*x5**4*x6**2+x1&
&*x2*x3**4*x6**2+x1*x3**2*x4*x5**4+x1*x2*x3**4*x5**2
p(374) = x2*x4*x5*x6**5+x2*x3*x4*x6**5+x1*x4*x5**5*x6+x1*x2*x3**5&
&*x6+x1*x3*x4*x5**5+x1*x2*x3**5*x5
p(375) = x1*x2*x4*x6**5+x1*x2*x4*x5**5+x1*x2*x3**5*x4
p(376) = x1**2*x3*x5*x6**4+x2**2*x3*x5**4*x6+x3**4*x4**2*x5*x6
p(377) = x1**2*x3*x5**2*x6**3+x1**2*x3**2*x5*x6**3+x2**2*x3*x5**3&
&*x6**2+x3**3*x4**2*x5*x6**2+x2**2*x3**2*x5**3*x6+x3**3*x4**2*x5**&
&2*x6
p(378) = x3**2*x4**2*x5**2*x6**2+x2**2*x3**2*x5**2*x6**2+x1**2*x3&
&**2*x5**2*x6**2
p(379) = x2**2*x3*x5**2*x6**3+x3**2*x4**2*x5*x6**3+x1**2*x3*x5**3&
&*x6**2+x1**2*x3**3*x5*x6**2+x3**2*x4**2*x5**3*x6+x2**2*x3**3*x5**&
&2*x6
p(380) = x3*x4**2*x5**2*x6**3+x2**2*x3**2*x5*x6**3+x3*x4**2*x5**3&
&*x6**2+x2**2*x3**3*x5*x6**2+x1**2*x3**2*x5**3*x6+x1**2*x3**3*x5**&
&2*x6
p(381) = x3*x4**2*x5*x6**4+x2**2*x3*x5*x6**4+x3*x4**2*x5**4*x6+x1&
&**2*x3*x5**4*x6+x2**2*x3**4*x5*x6+x1**2*x3**4*x5*x6
p(382) = x1**2*x2*x5*x6**4+x1**2*x3*x4*x6**4+x1*x2**2*x5**4*x6+x1&
&*x3**4*x4**2*x6+x2**2*x3*x4*x5**4+x2*x3**4*x4**2*x5
p(383) = x1**2*x2*x5**2*x6**3+x1**2*x3**2*x4*x6**3+x1*x2**2*x5**3&
&*x6**2+x1*x3**3*x4**2*x6**2+x2**2*x3**2*x4*x5**3+x2*x3**3*x4**2*x&
&5**2
p(384) = x1*x2**2*x5**2*x6**3+x1*x3**2*x4**2*x6**3+x1**2*x2*x5**3&
&*x6**2+x1**2*x3**3*x4*x6**2+x2*x3**2*x4**2*x5**3+x2**2*x3**3*x4*x&
&5**2
p(385) = x1*x2**2*x5*x6**4+x1*x3*x4**2*x6**4+x1**2*x2*x5**4*x6+x1&
&**2*x3**4*x4*x6+x2*x3*x4**2*x5**4+x2**2*x3**4*x4*x5
p(386) = x1**2*x2*x4*x6**4+x1*x2**2*x4*x5**4+x1*x2*x3**4*x4**2
p(387) = x1**2*x4*x5*x6**4+x1**2*x2*x3*x6**4+x2**2*x4*x5**4*x6+x2&
&*x3**4*x4**2*x6+x1*x2**2*x3*x5**4+x1*x3**4*x4**2*x5
p(388) = x1*x4**2*x5*x6**4+x1*x2**2*x3*x6**4+x2*x4**2*x5**4*x6+x2&
&**2*x3**4*x4*x6+x1**2*x2*x3*x5**4+x1**2*x3**4*x4*x5
p(389) = x1**2*x4*x5**2*x6**3+x1**2*x2*x3**2*x6**3+x2**2*x4*x5**3&
&*x6**2+x2*x3**3*x4**2*x6**2+x1*x2**2*x3**2*x5**3+x1*x3**3*x4**2*x&
&5**2
p(390) = x1*x4**2*x5**2*x6**3+x1*x2**2*x3**2*x6**3+x2*x4**2*x5**3&
&*x6**2+x2**2*x3**3*x4*x6**2+x1**2*x2*x3**2*x5**3+x1**2*x3**3*x4*x&
&5**2
p(391) = x2**2*x4*x5**2*x6**3+x2*x3**2*x4**2*x6**3+x1**2*x4*x5**3&
&*x6**2+x1**2*x2*x3**3*x6**2+x1*x3**2*x4**2*x5**3+x1*x2**2*x3**3*x&
&5**2
p(392) = x2*x4**2*x5**2*x6**3+x2**2*x3**2*x4*x6**3+x1*x4**2*x5**3&
&*x6**2+x1*x2**2*x3**3*x6**2+x1**2*x3**2*x4*x5**3+x1**2*x2*x3**3*x&
&5**2
p(393) = x2**2*x4*x5*x6**4+x2*x3*x4**2*x6**4+x1**2*x4*x5**4*x6+x1&
&**2*x2*x3**4*x6+x1*x3*x4**2*x5**4+x1*x2**2*x3**4*x5
p(394) = x2*x4**2*x5*x6**4+x2**2*x3*x4*x6**4+x1*x4**2*x5**4*x6+x1&
&*x2**2*x3**4*x6+x1**2*x3*x4*x5**4+x1**2*x2*x3**4*x5
p(395) = x1*x2*x4**2*x6**4+x1*x2**2*x4*x6**4+x1*x2*x4**2*x5**4+x1&
&**2*x2*x4*x5**4+x1*x2**2*x3**4*x4+x1**2*x2*x3**4*x4
p(396) = x1**2*x2**2*x5**2*x6**2+x1**2*x3**2*x4**2*x6**2+x2**2*x3&
&**2*x4**2*x5**2
p(397) = x1**2*x2**2*x5*x6**3+x1**2*x3*x4**2*x6**3+x1**2*x2**2*x5&
&**3*x6+x1**2*x3**3*x4**2*x6+x2**2*x3*x4**2*x5**3+x2**2*x3**3*x4**&
&2*x5
p(398) = x1**2*x2*x4**2*x6**3+x1**2*x2**2*x4*x6**3+x1*x2**2*x4**2&
&*x5**3+x1**2*x2**2*x4*x5**3+x1*x2**2*x3**3*x4**2+x1**2*x2*x3**3*x&
&4**2
p(399) = x1**2*x4**2*x5*x6**3+x1**2*x2**2*x3*x6**3+x2**2*x4**2*x5&
&**3*x6+x2**2*x3**3*x4**2*x6+x1**2*x2**2*x3*x5**3+x1**2*x3**3*x4**&
&2*x5
p(400) = x2**2*x4**2*x5**2*x6**2+x1**2*x4**2*x5**2*x6**2+x2**2*x3&
&**2*x4**2*x6**2+x1**2*x2**2*x3**2*x6**2+x1**2*x3**2*x4**2*x5**2+x&
&1**2*x2**2*x3**2*x5**2
p(401) = x1**2*x2**2*x4**2*x6**2+x1**2*x2**2*x4**2*x5**2+x1**2*x2&
&**2*x3**2*x4**2
p(402) = x2**2*x4**2*x5*x6**3+x2**2*x3*x4**2*x6**3+x1**2*x4**2*x5&
&**3*x6+x1**2*x2**2*x3**3*x6+x1**2*x3*x4**2*x5**3+x1**2*x2**2*x3**&
&3*x5
p(403) = x1*x2**2*x4**2*x6**3+x1**2*x2*x4**2*x5**3+x1**2*x2**2*x3&
&**3*x4
p(404) = x1**3*x3*x5*x6**3+x2**3*x3*x5**3*x6+x3**3*x4**3*x5*x6
p(405) = x2**3*x3*x5**2*x6**2+x1**3*x3*x5**2*x6**2+x3**2*x4**3*x5&
&*x6**2+x1**3*x3**2*x5*x6**2+x3**2*x4**3*x5**2*x6+x2**3*x3**2*x5**&
&2*x6
p(406) = x3*x4**3*x5**2*x6**2+x2**3*x3**2*x5*x6**2+x1**3*x3**2*x5&
&**2*x6
p(407) = x3*x4**3*x5*x6**3+x2**3*x3*x5*x6**3+x3*x4**3*x5**3*x6+x1&
&**3*x3*x5**3*x6+x2**3*x3**3*x5*x6+x1**3*x3**3*x5*x6
p(408) = x1**3*x2*x5*x6**3+x1**3*x3*x4*x6**3+x1*x2**3*x5**3*x6+x1&
&*x3**3*x4**3*x6+x2**3*x3*x4*x5**3+x2*x3**3*x4**3*x5
p(409) = x1*x2**3*x5**2*x6**2+x1**3*x2*x5**2*x6**2+x1*x3**2*x4**3&
&*x6**2+x1**3*x3**2*x4*x6**2+x2*x3**2*x4**3*x5**2+x2**3*x3**2*x4*x&
&5**2
p(410) = x1*x2**3*x5*x6**3+x1*x3*x4**3*x6**3+x1**3*x2*x5**3*x6+x1&
&**3*x3**3*x4*x6+x2*x3*x4**3*x5**3+x2**3*x3**3*x4*x5
p(411) = x1**3*x2*x4*x6**3+x1*x2**3*x4*x5**3+x1*x2*x3**3*x4**3
p(412) = x1**3*x4*x5*x6**3+x1**3*x2*x3*x6**3+x2**3*x4*x5**3*x6+x2&
&*x3**3*x4**3*x6+x1*x2**3*x3*x5**3+x1*x3**3*x4**3*x5
p(413) = x1*x4**3*x5*x6**3+x1*x2**3*x3*x6**3+x2*x4**3*x5**3*x6+x2&
&**3*x3**3*x4*x6+x1**3*x2*x3*x5**3+x1**3*x3**3*x4*x5
p(414) = x2**3*x4*x5**2*x6**2+x1**3*x4*x5**2*x6**2+x2*x3**2*x4**3&
&*x6**2+x1**3*x2*x3**2*x6**2+x1*x3**2*x4**3*x5**2+x1*x2**3*x3**2*x&
&5**2
p(415) = x2*x4**3*x5**2*x6**2+x1*x4**3*x5**2*x6**2+x2**3*x3**2*x4&
&*x6**2+x1*x2**3*x3**2*x6**2+x1**3*x3**2*x4*x5**2+x1**3*x2*x3**2*x&
&5**2
p(416) = x2**3*x4*x5*x6**3+x2*x3*x4**3*x6**3+x1**3*x4*x5**3*x6+x1&
&**3*x2*x3**3*x6+x1*x3*x4**3*x5**3+x1*x2**3*x3**3*x5
p(417) = x2*x4**3*x5*x6**3+x2**3*x3*x4*x6**3+x1*x4**3*x5**3*x6+x1&
&*x2**3*x3**3*x6+x1**3*x3*x4*x5**3+x1**3*x2*x3**3*x5
p(418) = x1*x2*x4**3*x6**3+x1*x2**3*x4*x6**3+x1*x2*x4**3*x5**3+x1&
&**3*x2*x4*x5**3+x1*x2**3*x3**3*x4+x1**3*x2*x3**3*x4
p(419) = x1**3*x2**2*x5*x6**2+x1**3*x3*x4**2*x6**2+x1**2*x2**3*x5&
&**2*x6+x1**2*x3**2*x4**3*x6+x2**3*x3*x4**2*x5**2+x2**2*x3**2*x4**&
&3*x5
p(420) = x1**2*x2**3*x5*x6**2+x1**2*x3*x4**3*x6**2+x1**3*x2**2*x5&
&**2*x6+x1**3*x3**2*x4**2*x6+x2**2*x3*x4**3*x5**2+x2**3*x3**2*x4**&
&2*x5
p(421) = x1**3*x2*x4**2*x6**2+x1**3*x2**2*x4*x6**2+x1*x2**3*x4**2&
&*x5**2+x1**2*x2**3*x4*x5**2+x1*x2**2*x3**2*x4**3+x1**2*x2*x3**2*x&
&4**3
p(422) = x1**2*x2*x4**3*x6**2+x1**2*x2**3*x4*x6**2+x1*x2**2*x4**3&
&*x5**2+x1**3*x2**2*x4*x5**2+x1*x2**3*x3**2*x4**2+x1**3*x2*x3**2*x&
&4**2
p(423) = x1**3*x2**2*x4**2*x6+x1**2*x2**3*x4**2*x5+x1**2*x2**2*x3&
&*x4**3
p(424) = x1**3*x4**2*x5*x6**2+x1**3*x2**2*x3*x6**2+x2**3*x4**2*x5&
&**2*x6+x2**2*x3**2*x4**3*x6+x1**2*x2**3*x3*x5**2+x1**2*x3**2*x4**&
&3*x5
p(425) = x1**2*x4**3*x5*x6**2+x1**2*x2**3*x3*x6**2+x2**2*x4**3*x5&
&**2*x6+x2**3*x3**2*x4**2*x6+x1**3*x2**2*x3*x5**2+x1**3*x3**2*x4**&
&2*x5
p(426) = x1**2*x2**2*x4**3*x6+x1**2*x2**3*x4**2*x6+x1**2*x2**2*x4&
&**3*x5+x1**3*x2**2*x4**2*x5+x1**2*x2**3*x3*x4**2+x1**3*x2**2*x3*x&
&4**2
p(427) = x2**3*x4**2*x5*x6**2+x2**2*x3*x4**3*x6**2+x1**3*x4**2*x5&
&**2*x6+x1**3*x2**2*x3**2*x6+x1**2*x3*x4**3*x5**2+x1**2*x2**3*x3**&
&2*x5
p(428) = x2**2*x4**3*x5*x6**2+x2**3*x3*x4**2*x6**2+x1**2*x4**3*x5&
&**2*x6+x1**2*x2**3*x3**2*x6+x1**3*x3*x4**2*x5**2+x1**3*x2**2*x3**&
&2*x5
p(429) = x1*x2**2*x4**3*x6**2+x1*x2**3*x4**2*x6**2+x1**2*x2*x4**3&
&*x5**2+x1**3*x2*x4**2*x5**2+x1**2*x2**3*x3**2*x4+x1**3*x2**2*x3**&
&2*x4
p(430) = x1**3*x2**3*x5*x6+x1**3*x3*x4**3*x6+x2**3*x3*x4**3*x5
p(431) = x1**3*x2*x4**3*x6+x1**3*x2**3*x4*x6+x1*x2**3*x4**3*x5+x1&
&**3*x2**3*x4*x5+x1*x2**3*x3*x4**3+x1**3*x2*x3*x4**3
p(432) = x2**3*x4**3*x5*x6+x1**3*x4**3*x5*x6+x2**3*x3*x4**3*x6+x1&
&**3*x2**3*x3*x6+x1**3*x3*x4**3*x5+x1**3*x2**3*x3*x5
p(433) = x1*x2**3*x4**3*x6+x1**3*x2*x4**3*x5+x1**3*x2**3*x3*x4
p(434) = x1**4*x3*x5*x6**2+x2**4*x3*x5**2*x6+x3**2*x4**4*x5*x6
p(435) = x3*x4**4*x5*x6**2+x2**4*x3*x5*x6**2+x3*x4**4*x5**2*x6+x1&
&**4*x3*x5**2*x6+x2**4*x3**2*x5*x6+x1**4*x3**2*x5*x6
p(436) = x1**4*x2*x5*x6**2+x1**4*x3*x4*x6**2+x1*x2**4*x5**2*x6+x1&
&*x3**2*x4**4*x6+x2**4*x3*x4*x5**2+x2*x3**2*x4**4*x5
p(437) = x1*x2**4*x5*x6**2+x1*x3*x4**4*x6**2+x1**4*x2*x5**2*x6+x1&
&**4*x3**2*x4*x6+x2*x3*x4**4*x5**2+x2**4*x3**2*x4*x5
p(438) = x1**4*x2*x4*x6**2+x1*x2**4*x4*x5**2+x1*x2*x3**2*x4**4
p(439) = x1**4*x4*x5*x6**2+x1**4*x2*x3*x6**2+x2**4*x4*x5**2*x6+x2&
&*x3**2*x4**4*x6+x1*x2**4*x3*x5**2+x1*x3**2*x4**4*x5
p(440) = x1*x4**4*x5*x6**2+x1*x2**4*x3*x6**2+x2*x4**4*x5**2*x6+x2&
&**4*x3**2*x4*x6+x1**4*x2*x3*x5**2+x1**4*x3**2*x4*x5
p(441) = x2**4*x4*x5*x6**2+x2*x3*x4**4*x6**2+x1**4*x4*x5**2*x6+x1&
&**4*x2*x3**2*x6+x1*x3*x4**4*x5**2+x1*x2**4*x3**2*x5
p(442) = x2*x4**4*x5*x6**2+x2**4*x3*x4*x6**2+x1*x4**4*x5**2*x6+x1&
&*x2**4*x3**2*x6+x1**4*x3*x4*x5**2+x1**4*x2*x3**2*x5
p(443) = x1*x2*x4**4*x6**2+x1*x2**4*x4*x6**2+x1*x2*x4**4*x5**2+x1&
&**4*x2*x4*x5**2+x1*x2**4*x3**2*x4+x1**4*x2*x3**2*x4
p(444) = x1**2*x2**4*x5*x6+x1**4*x2**2*x5*x6+x1**2*x3*x4**4*x6+x1&
&**4*x3*x4**2*x6+x2**2*x3*x4**4*x5+x2**4*x3*x4**2*x5
p(445) = x1**4*x2*x4**2*x6+x1**4*x2**2*x4*x6+x1*x2**4*x4**2*x5+x1&
&**2*x2**4*x4*x5+x1*x2**2*x3*x4**4+x1**2*x2*x3*x4**4
p(446) = x1**2*x2*x4**4*x6+x1**2*x2**4*x4*x6+x1*x2**2*x4**4*x5+x1&
&**4*x2**2*x4*x5+x1*x2**4*x3*x4**2+x1**4*x2*x3*x4**2
p(447) = x2**4*x4**2*x5*x6+x1**4*x4**2*x5*x6+x2**2*x3*x4**4*x6+x1&
&**4*x2**2*x3*x6+x1**2*x3*x4**4*x5+x1**2*x2**4*x3*x5
p(448) = x2**2*x4**4*x5*x6+x1**2*x4**4*x5*x6+x2**4*x3*x4**2*x6+x1&
&**2*x2**4*x3*x6+x1**4*x3*x4**2*x5+x1**4*x2**2*x3*x5
p(449) = x1*x2**2*x4**4*x6+x1*x2**4*x4**2*x6+x1**2*x2*x4**4*x5+x1&
&**4*x2*x4**2*x5+x1**2*x2**4*x3*x4+x1**4*x2**2*x3*x4
p(450) = x3*x4**5*x5*x6+x2**5*x3*x5*x6+x1**5*x3*x5*x6
p(451) = x1*x2**5*x5*x6+x1**5*x2*x5*x6+x1*x3*x4**5*x6+x1**5*x3*x4&
&*x6+x2*x3*x4**5*x5+x2**5*x3*x4*x5
p(452) = x1**5*x2*x4*x6+x1*x2**5*x4*x5+x1*x2*x3*x4**5
p(453) = x2**5*x4*x5*x6+x1**5*x4*x5*x6+x2*x3*x4**5*x6+x1**5*x2*x3&
&*x6+x1*x3*x4**5*x5+x1*x2**5*x3*x5
p(454) = x2*x4**5*x5*x6+x1*x4**5*x5*x6+x2**5*x3*x4*x6+x1*x2**5*x3&
&*x6+x1**5*x3*x4*x5+x1**5*x2*x3*x5
p(455) = x1*x2*x4**5*x6+x1*x2**5*x4*x6+x1*x2*x4**5*x5+x1**5*x2*x4&
&*x5+x1*x2**5*x3*x4+x1**5*x2*x3*x4
p(456) = x3**2*x5**3*x6**3+x3**3*x5**2*x6**3+x3**3*x5**3*x6**2
p(457) = x3**2*x5**2*x6**4+x3**2*x5**4*x6**2+x3**4*x5**2*x6**2
p(458) = x3*x5**3*x6**4+x3**3*x5*x6**4+x3*x5**4*x6**3+x3**4*x5*x6&
&**3+x3**3*x5**4*x6+x3**4*x5**3*x6
p(459) = x3*x5**2*x6**5+x3**2*x5*x6**5+x3*x5**5*x6**2+x3**5*x5*x6&
&**2+x3**2*x5**5*x6+x3**5*x5**2*x6
p(460) = x3*x5*x6**6+x3*x5**6*x6+x3**6*x5*x6
p(461) = x1*x5*x6**6+x1*x3*x6**6+x2*x5**6*x6+x3**6*x4*x6+x2*x3*x5&
&**6+x3**6*x4*x5
p(462) = x1*x5**2*x6**5+x1*x3**2*x6**5+x2*x5**5*x6**2+x3**5*x4*x6&
&**2+x2*x3**2*x5**5+x3**5*x4*x5**2
p(463) = x1*x5**3*x6**4+x1*x3**3*x6**4+x2*x5**4*x6**3+x3**4*x4*x6&
&**3+x2*x3**3*x5**4+x3**4*x4*x5**3
p(464) = x2*x5**3*x6**4+x3**3*x4*x6**4+x1*x5**4*x6**3+x1*x3**4*x6&
&**3+x3**3*x4*x5**4+x2*x3**4*x5**3
p(465) = x4*x5**3*x6**4+x2*x3**3*x6**4+x4*x5**4*x6**3+x2*x3**4*x6&
&**3+x1*x3**3*x5**4+x1*x3**4*x5**3
p(466) = x2*x5**2*x6**5+x3**2*x4*x6**5+x1*x5**5*x6**2+x1*x3**5*x6&
&**2+x3**2*x4*x5**5+x2*x3**5*x5**2
p(467) = x4*x5**2*x6**5+x2*x3**2*x6**5+x4*x5**5*x6**2+x2*x3**5*x6&
&**2+x1*x3**2*x5**5+x1*x3**5*x5**2
p(468) = x2*x5*x6**6+x3*x4*x6**6+x1*x5**6*x6+x1*x3**6*x6+x3*x4*x5&
&**6+x2*x3**6*x5
p(469) = x4*x5*x6**6+x2*x3*x6**6+x4*x5**6*x6+x2*x3**6*x6+x1*x3*x5&
&**6+x1*x3**6*x5
p(470) = x1*x4*x6**6+x1*x2*x6**6+x2*x4*x5**6+x1*x2*x5**6+x2*x3**6&
&*x4+x1*x3**6*x4
p(471) = x2*x4*x6**6+x1*x4*x5**6+x1*x2*x3**6
p(472) = x1**2*x5*x6**5+x1**2*x3*x6**5+x2**2*x5**5*x6+x3**5*x4**2&
&*x6+x2**2*x3*x5**5+x3**5*x4**2*x5
p(473) = x1**2*x5**2*x6**4+x1**2*x3**2*x6**4+x2**2*x5**4*x6**2+x3&
&**4*x4**2*x6**2+x2**2*x3**2*x5**4+x3**4*x4**2*x5**2
p(474) = x2**2*x5**3*x6**3+x1**2*x5**3*x6**3+x3**3*x4**2*x6**3+x1&
&**2*x3**3*x6**3+x3**3*x4**2*x5**3+x2**2*x3**3*x5**3
p(475) = x4**2*x5**3*x6**3+x2**2*x3**3*x6**3+x1**2*x3**3*x5**3
p(476) = x2**2*x5**2*x6**4+x3**2*x4**2*x6**4+x1**2*x5**4*x6**2+x1&
&**2*x3**4*x6**2+x3**2*x4**2*x5**4+x2**2*x3**4*x5**2
p(477) = x4**2*x5**2*x6**4+x2**2*x3**2*x6**4+x4**2*x5**4*x6**2+x2&
&**2*x3**4*x6**2+x1**2*x3**2*x5**4+x1**2*x3**4*x5**2
p(478) = x2**2*x5*x6**5+x3*x4**2*x6**5+x1**2*x5**5*x6+x1**2*x3**5&
&*x6+x3*x4**2*x5**5+x2**2*x3**5*x5
p(479) = x4**2*x5*x6**5+x2**2*x3*x6**5+x4**2*x5**5*x6+x2**2*x3**5&
&*x6+x1**2*x3*x5**5+x1**2*x3**5*x5
p(480) = x1**2*x4*x6**5+x1**2*x2*x6**5+x2**2*x4*x5**5+x1*x2**2*x5&
&**5+x2*x3**5*x4**2+x1*x3**5*x4**2
p(481) = x1*x4**2*x6**5+x1*x2**2*x6**5+x2*x4**2*x5**5+x1**2*x2*x5&
&**5+x2**2*x3**5*x4+x1**2*x3**5*x4
p(482) = x2*x4**2*x6**5+x2**2*x4*x6**5+x1*x4**2*x5**5+x1**2*x4*x5&
&**5+x1*x2**2*x3**5+x1**2*x2*x3**5
p(483) = x1**2*x4**2*x6**4+x1**2*x2**2*x6**4+x2**2*x4**2*x5**4+x1&
&**2*x2**2*x5**4+x2**2*x3**4*x4**2+x1**2*x3**4*x4**2
p(484) = x2**2*x4**2*x6**4+x1**2*x4**2*x5**4+x1**2*x2**2*x3**4
p(485) = x1**3*x5*x6**4+x1**3*x3*x6**4+x2**3*x5**4*x6+x3**4*x4**3&
&*x6+x2**3*x3*x5**4+x3**4*x4**3*x5
p(486) = x1**3*x5**2*x6**3+x1**3*x3**2*x6**3+x2**3*x5**3*x6**2+x3&
&**3*x4**3*x6**2+x2**3*x3**2*x5**3+x3**3*x4**3*x5**2
p(487) = x2**3*x5**2*x6**3+x3**2*x4**3*x6**3+x1**3*x5**3*x6**2+x1&
&**3*x3**3*x6**2+x3**2*x4**3*x5**3+x2**3*x3**3*x5**2
p(488) = x4**3*x5**2*x6**3+x2**3*x3**2*x6**3+x4**3*x5**3*x6**2+x2&
&**3*x3**3*x6**2+x1**3*x3**2*x5**3+x1**3*x3**3*x5**2
p(489) = x2**3*x5*x6**4+x3*x4**3*x6**4+x1**3*x5**4*x6+x1**3*x3**4&
&*x6+x3*x4**3*x5**4+x2**3*x3**4*x5
p(490) = x4**3*x5*x6**4+x2**3*x3*x6**4+x4**3*x5**4*x6+x2**3*x3**4&
&*x6+x1**3*x3*x5**4+x1**3*x3**4*x5
p(491) = x1**3*x4*x6**4+x1**3*x2*x6**4+x2**3*x4*x5**4+x1*x2**3*x5&
&**4+x2*x3**4*x4**3+x1*x3**4*x4**3
p(492) = x1*x4**3*x6**4+x1*x2**3*x6**4+x2*x4**3*x5**4+x1**3*x2*x5&
&**4+x2**3*x3**4*x4+x1**3*x3**4*x4
p(493) = x2*x4**3*x6**4+x2**3*x4*x6**4+x1*x4**3*x5**4+x1**3*x4*x5&
&**4+x1*x2**3*x3**4+x1**3*x2*x3**4
p(494) = x1**3*x4**2*x6**3+x1**3*x2**2*x6**3+x2**3*x4**2*x5**3+x1&
&**2*x2**3*x5**3+x2**2*x3**3*x4**3+x1**2*x3**3*x4**3
p(495) = x1**2*x4**3*x6**3+x1**2*x2**3*x6**3+x2**2*x4**3*x5**3+x1&
&**3*x2**2*x5**3+x2**3*x3**3*x4**2+x1**3*x3**3*x4**2
p(496) = x2**2*x4**3*x6**3+x2**3*x4**2*x6**3+x1**2*x4**3*x5**3+x1&
&**3*x4**2*x5**3+x1**2*x2**3*x3**3+x1**3*x2**2*x3**3
p(497) = x1**3*x4**3*x6**2+x1**3*x2**3*x6**2+x2**3*x4**3*x5**2+x1&
&**3*x2**3*x5**2+x2**3*x3**2*x4**3+x1**3*x3**2*x4**3
p(498) = x1**2*x2**3*x4**3+x1**3*x2**2*x4**3+x1**3*x2**3*x4**2
p(499) = x2**3*x4**3*x6**2+x1**3*x4**3*x5**2+x1**3*x2**3*x3**2
p(500) = x1**4*x5*x6**3+x1**4*x3*x6**3+x2**4*x5**3*x6+x3**3*x4**4&
&*x6+x2**4*x3*x5**3+x3**3*x4**4*x5
p(501) = x2**4*x5**2*x6**2+x1**4*x5**2*x6**2+x3**2*x4**4*x6**2+x1&
&**4*x3**2*x6**2+x3**2*x4**4*x5**2+x2**4*x3**2*x5**2
p(502) = x4**4*x5**2*x6**2+x2**4*x3**2*x6**2+x1**4*x3**2*x5**2
p(503) = x2**4*x5*x6**3+x3*x4**4*x6**3+x1**4*x5**3*x6+x1**4*x3**3&
&*x6+x3*x4**4*x5**3+x2**4*x3**3*x5
p(504) = x4**4*x5*x6**3+x2**4*x3*x6**3+x4**4*x5**3*x6+x2**4*x3**3&
&*x6+x1**4*x3*x5**3+x1**4*x3**3*x5
p(505) = x1**4*x4*x6**3+x1**4*x2*x6**3+x2**4*x4*x5**3+x1*x2**4*x5&
&**3+x2*x3**3*x4**4+x1*x3**3*x4**4
p(506) = x1*x4**4*x6**3+x1*x2**4*x6**3+x2*x4**4*x5**3+x1**4*x2*x5&
&**3+x2**4*x3**3*x4+x1**4*x3**3*x4
p(507) = x2*x4**4*x6**3+x2**4*x4*x6**3+x1*x4**4*x5**3+x1**4*x4*x5&
&**3+x1*x2**4*x3**3+x1**4*x2*x3**3
p(508) = x1**4*x4**2*x6**2+x1**4*x2**2*x6**2+x2**4*x4**2*x5**2+x1&
&**2*x2**4*x5**2+x2**2*x3**2*x4**4+x1**2*x3**2*x4**4
p(509) = x1**2*x4**4*x6**2+x1**2*x2**4*x6**2+x2**2*x4**4*x5**2+x1&
&**4*x2**2*x5**2+x2**4*x3**2*x4**2+x1**4*x3**2*x4**2
p(510) = x1**2*x2**2*x4**4+x1**2*x2**4*x4**2+x1**4*x2**2*x4**2
p(511) = x2**2*x4**4*x6**2+x2**4*x4**2*x6**2+x1**2*x4**4*x5**2+x1&
&**4*x4**2*x5**2+x1**2*x2**4*x3**2+x1**4*x2**2*x3**2
p(512) = x1**4*x4**3*x6+x1**4*x2**3*x6+x2**4*x4**3*x5+x1**3*x2**4&
&*x5+x2**3*x3*x4**4+x1**3*x3*x4**4
p(513) = x1**3*x4**4*x6+x1**3*x2**4*x6+x2**3*x4**4*x5+x1**4*x2**3&
&*x5+x2**4*x3*x4**3+x1**4*x3*x4**3
p(514) = x1*x2**3*x4**4+x1**3*x2*x4**4+x1*x2**4*x4**3+x1**4*x2*x4&
&**3+x1**3*x2**4*x4+x1**4*x2**3*x4
p(515) = x2**3*x4**4*x6+x2**4*x4**3*x6+x1**3*x4**4*x5+x1**4*x4**3&
&*x5+x1**3*x2**4*x3+x1**4*x2**3*x3
p(516) = x1**5*x5*x6**2+x1**5*x3*x6**2+x2**5*x5**2*x6+x3**2*x4**5&
&*x6+x2**5*x3*x5**2+x3**2*x4**5*x5
p(517) = x2**5*x5*x6**2+x3*x4**5*x6**2+x1**5*x5**2*x6+x1**5*x3**2&
&*x6+x3*x4**5*x5**2+x2**5*x3**2*x5
p(518) = x4**5*x5*x6**2+x2**5*x3*x6**2+x4**5*x5**2*x6+x2**5*x3**2&
&*x6+x1**5*x3*x5**2+x1**5*x3**2*x5
p(519) = x1**5*x4*x6**2+x1**5*x2*x6**2+x2**5*x4*x5**2+x1*x2**5*x5&
&**2+x2*x3**2*x4**5+x1*x3**2*x4**5
p(520) = x1*x4**5*x6**2+x1*x2**5*x6**2+x2*x4**5*x5**2+x1**5*x2*x5&
&**2+x2**5*x3**2*x4+x1**5*x3**2*x4
p(521) = x2*x4**5*x6**2+x2**5*x4*x6**2+x1*x4**5*x5**2+x1**5*x4*x5&
&**2+x1*x2**5*x3**2+x1**5*x2*x3**2
p(522) = x1**5*x4**2*x6+x1**5*x2**2*x6+x2**5*x4**2*x5+x1**2*x2**5&
&*x5+x2**2*x3*x4**5+x1**2*x3*x4**5
p(523) = x1**2*x4**5*x6+x1**2*x2**5*x6+x2**2*x4**5*x5+x1**5*x2**2&
&*x5+x2**5*x3*x4**2+x1**5*x3*x4**2
p(524) = x1*x2**2*x4**5+x1**2*x2*x4**5+x1*x2**5*x4**2+x1**5*x2*x4&
&**2+x1**2*x2**5*x4+x1**5*x2**2*x4
p(525) = x2**2*x4**5*x6+x2**5*x4**2*x6+x1**2*x4**5*x5+x1**5*x4**2&
&*x5+x1**2*x2**5*x3+x1**5*x2**2*x3
p(526) = x2**6*x5*x6+x1**6*x5*x6+x3*x4**6*x6+x1**6*x3*x6+x3*x4**6&
&*x5+x2**6*x3*x5
p(527) = x4**6*x5*x6+x2**6*x3*x6+x1**6*x3*x5
p(528) = x1**6*x4*x6+x1**6*x2*x6+x2**6*x4*x5+x1*x2**6*x5+x2*x3*x4&
&**6+x1*x3*x4**6
p(529) = x1*x4**6*x6+x1*x2**6*x6+x2*x4**6*x5+x1**6*x2*x5+x2**6*x3&
&*x4+x1**6*x3*x4
p(530) = x1*x2*x4**6+x1*x2**6*x4+x1**6*x2*x4
p(531) = x2*x4**6*x6+x2**6*x4*x6+x1*x4**6*x5+x1**6*x4*x5+x1*x2**6&
&*x3+x1**6*x2*x3
p(532) = x5**4*x6**4+x3**4*x6**4+x3**4*x5**4
p(533) = x5**3*x6**5+x3**3*x6**5+x5**5*x6**3+x3**5*x6**3+x3**3*x5&
&**5+x3**5*x5**3
p(534) = x5**2*x6**6+x3**2*x6**6+x5**6*x6**2+x3**6*x6**2+x3**2*x5&
&**6+x3**6*x5**2
p(535) = x5*x6**7+x3*x6**7+x5**7*x6+x3**7*x6+x3*x5**7+x3**7*x5
p(536) = x4*x6**7+x2*x6**7+x4*x5**7+x1*x5**7+x2*x3**7+x1*x3**7
p(537) = x4**2*x6**6+x2**2*x6**6+x4**2*x5**6+x1**2*x5**6+x2**2*x3&
&**6+x1**2*x3**6
p(538) = x4**3*x6**5+x2**3*x6**5+x4**3*x5**5+x1**3*x5**5+x2**3*x3&
&**5+x1**3*x3**5
p(539) = x4**4*x6**4+x2**4*x6**4+x4**4*x5**4+x1**4*x5**4+x2**4*x3&
&**4+x1**4*x3**4
p(540) = x2**4*x4**4+x1**4*x4**4+x1**4*x2**4
p(541) = x4**5*x6**3+x2**5*x6**3+x4**5*x5**3+x1**5*x5**3+x2**5*x3&
&**3+x1**5*x3**3
p(542) = x2**3*x4**5+x1**3*x4**5+x2**5*x4**3+x1**5*x4**3+x1**3*x2&
&**5+x1**5*x2**3
p(543) = x4**6*x6**2+x2**6*x6**2+x4**6*x5**2+x1**6*x5**2+x2**6*x3&
&**2+x1**6*x3**2
p(544) = x2**2*x4**6+x1**2*x4**6+x2**6*x4**2+x1**6*x4**2+x1**2*x2&
&**6+x1**6*x2**2
p(545) = x4**7*x6+x2**7*x6+x4**7*x5+x1**7*x5+x2**7*x3+x1**7*x3
p(546) = x2*x4**7+x1*x4**7+x2**7*x4+x1**7*x4+x1*x2**7+x1**7*x2
end subroutine ev_pol_con_8

!===========================================================================
! *def grad_ev_pol_con_8(x,dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6)
! subtoutine of evaluating polynomials gradient at 8th order
!===========================================================================
subroutine grad_ev_pol_con_8(x,dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6)
implicit none
integer, parameter:: dp=kind(0.d0)                   ! double precision
real(dp),dimension(1:6),intent(in)::x
real(dp),dimension(1:546),intent(out)::dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6
real(dp)::x1,x2,x3,x4,x5,x6

x1=x(1)
x2=x(2)
x3=x(3)
x4=x(4)
x5=x(5)
x6=x(6)

dpdx1(1) = 0
dpdx1(2) = 0
dpdx1(3) = x5+x3
dpdx1(4) = x4+x2
dpdx1(5) = 0
dpdx1(6) = x5*x6+x3*x6
dpdx1(7) = x3*x5
dpdx1(8) = x4*x6+x2*x6+x2*x5+x3*x4
dpdx1(9) = x2*x4
dpdx1(10) = x4*x5+x2*x3
dpdx1(11) = 0
dpdx1(12) = x5**2+x3**2
dpdx1(13) = 2*x1*x5+2*x1*x3
dpdx1(14) = x4**2+2*x1*x4+x2**2+2*x1*x2
dpdx1(15) = x3*x5*x6
dpdx1(16) = x2*x5*x6+x3*x4*x6
dpdx1(17) = x4*x5*x6+x2*x3*x6+x3*x4*x5+x2*x3*x5
dpdx1(18) = x2*x4*x6+x2*x4*x5+x2*x3*x4
dpdx1(19) = 0
dpdx1(20) = x5*x6**2+x3*x6**2
dpdx1(21) = x5**2*x6+x3**2*x6
dpdx1(22) = x3*x5**2+x3**2*x5
dpdx1(23) = x4*x6**2+x2*x6**2+x2*x5**2+x3**2*x4
dpdx1(24) = x4*x5**2+x2*x3**2
dpdx1(25) = 2*x1*x5*x6+2*x1*x3*x6
dpdx1(26) = 2*x1*x3*x5
dpdx1(27) = 2*x1*x4*x6+2*x1*x2*x6+x2**2*x5+x3*x4**2
dpdx1(28) = x4**2*x6+x2**2*x6+2*x1*x2*x5+2*x1*x3*x4
dpdx1(29) = x2*x4**2+x2**2*x4+2*x1*x2*x4
dpdx1(30) = x4**2*x5+2*x1*x4*x5+x2**2*x3+2*x1*x2*x3
dpdx1(31) = 0
dpdx1(32) = 0
dpdx1(33) = x5**3+x3**3
dpdx1(34) = 2*x1*x5**2+2*x1*x3**2
dpdx1(35) = 2*x1*x4**2+2*x1*x2**2
dpdx1(36) = 3*x1**2*x5+3*x1**2*x3
dpdx1(37) = x4**3+3*x1**2*x4+x2**3+3*x1**2*x2
dpdx1(38) = x3*x4*x5*x6+x2*x3*x5*x6
dpdx1(39) = x2*x4*x5*x6+x2*x3*x4*x6+x2*x3*x4*x5
dpdx1(40) = x3*x5*x6**2
dpdx1(41) = x3*x5**2*x6+x3**2*x5*x6
dpdx1(42) = x2*x5*x6**2+x3*x4*x6**2+x2*x5**2*x6+x3**2*x4*x6
dpdx1(43) = x4*x5*x6**2+x2*x3*x6**2+x2*x3*x5**2+x3**2*x4*x5
dpdx1(44) = x4*x5**2*x6+x2*x3**2*x6+x3*x4*x5**2+x2*x3**2*x5
dpdx1(45) = x2*x4*x6**2+x2*x4*x5**2+x2*x3**2*x4
dpdx1(46) = 2*x1*x3*x5*x6
dpdx1(47) = x2**2*x5*x6+2*x1*x2*x5*x6+x3*x4**2*x6+2*x1*x3*x4*x6
dpdx1(48) = 2*x1*x2*x4*x6+x2**2*x4*x5+x2*x3*x4**2
dpdx1(49) = 2*x1*x4*x5*x6+2*x1*x2*x3*x6+x3*x4**2*x5+x2**2*x3*x5
dpdx1(50) = x4**2*x5*x6+x2**2*x3*x6+2*x1*x3*x4*x5+2*x1*x2*x3*x5
dpdx1(51) = x2*x4**2*x6+x2**2*x4*x6+x2*x4**2*x5+2*x1*x2*x4*x5+x2*&
&*2*x3*x4+2*x1*x2*x3*x4
dpdx1(52) = 0
dpdx1(53) = 0
dpdx1(54) = x5*x6**3+x3*x6**3
dpdx1(55) = x5**2*x6**2+x3**2*x6**2
dpdx1(56) = x3**2*x5**2
dpdx1(57) = x5**3*x6+x3**3*x6
dpdx1(58) = x3*x5**3+x3**3*x5
dpdx1(59) = x4*x6**3+x2*x6**3+x2*x5**3+x3**3*x4
dpdx1(60) = x4*x5**3+x2*x3**3
dpdx1(61) = 2*x1*x5*x6**2+2*x1*x3*x6**2
dpdx1(62) = 2*x1*x5**2*x6+2*x1*x3**2*x6
dpdx1(63) = 2*x1*x3*x5**2+2*x1*x3**2*x5
dpdx1(64) = 2*x1*x4*x6**2+2*x1*x2*x6**2+x2**2*x5**2+x3**2*x4**2
dpdx1(65) = x4**2*x6**2+x2**2*x6**2+2*x1*x2*x5**2+2*x1*x3**2*x4
dpdx1(66) = x4**2*x5**2+2*x1*x4*x5**2+x2**2*x3**2+2*x1*x2*x3**2
dpdx1(67) = 2*x1*x4**2*x6+2*x1*x2**2*x6+2*x1*x2**2*x5+2*x1*x3*x4*&
&*2
dpdx1(68) = x2**2*x4**2+2*x1*x2*x4**2+2*x1*x2**2*x4
dpdx1(69) = 2*x1*x4**2*x5+2*x1*x2**2*x3
dpdx1(70) = 3*x1**2*x5*x6+3*x1**2*x3*x6
dpdx1(71) = 3*x1**2*x3*x5
dpdx1(72) = 3*x1**2*x4*x6+3*x1**2*x2*x6+x2**3*x5+x3*x4**3
dpdx1(73) = x4**3*x6+x2**3*x6+3*x1**2*x2*x5+3*x1**2*x3*x4
dpdx1(74) = x2*x4**3+x2**3*x4+3*x1**2*x2*x4
dpdx1(75) = x4**3*x5+3*x1**2*x4*x5+x2**3*x3+3*x1**2*x2*x3
dpdx1(76) = 0
dpdx1(77) = 0
dpdx1(78) = x5**4+x3**4
dpdx1(79) = 2*x1*x5**3+2*x1*x3**3
dpdx1(80) = 3*x1**2*x5**2+3*x1**2*x3**2
dpdx1(81) = 2*x1*x4**3+3*x1**2*x4**2+2*x1*x2**3+3*x1**2*x2**2
dpdx1(82) = 4*x1**3*x5+4*x1**3*x3
dpdx1(83) = x4**4+4*x1**3*x4+x2**4+4*x1**3*x2
dpdx1(84) = x2*x3*x4*x5*x6
dpdx1(85) = x3*x4*x5*x6**2+x2*x3*x5*x6**2+x2*x3*x5**2*x6+x3**2*x4&
&*x5*x6
dpdx1(86) = x3*x4*x5**2*x6+x2*x3**2*x5*x6
dpdx1(87) = x2*x4*x5*x6**2+x2*x3*x4*x6**2+x2*x4*x5**2*x6+x2*x3**2&
&*x4*x6+x2*x3*x4*x5**2+x2*x3**2*x4*x5
dpdx1(88) = x3*x4**2*x5*x6+2*x1*x3*x4*x5*x6+x2**2*x3*x5*x6+2*x1*x&
&2*x3*x5*x6
dpdx1(89) = x2**2*x4*x5*x6+2*x1*x2*x4*x5*x6+x2*x3*x4**2*x6+2*x1*x&
&2*x3*x4*x6+x2*x3*x4**2*x5+x2**2*x3*x4*x5
dpdx1(90) = x2*x4**2*x5*x6+x2**2*x3*x4*x6+2*x1*x2*x3*x4*x5
dpdx1(91) = x3*x5*x6**3
dpdx1(92) = x3*x5**2*x6**2+x3**2*x5*x6**2
dpdx1(93) = x3**2*x5**2*x6
dpdx1(94) = x3*x5**3*x6+x3**3*x5*x6
dpdx1(95) = x2*x5**2*x6**2+x3**2*x4*x6**2
dpdx1(96) = x2*x5*x6**3+x3*x4*x6**3+x2*x5**3*x6+x3**3*x4*x6
dpdx1(97) = x4*x5*x6**3+x2*x3*x6**3+x2*x3*x5**3+x3**3*x4*x5
dpdx1(98) = x4*x5**2*x6**2+x2*x3**2*x6**2+x3**2*x4*x5**2+x2*x3**2&
&*x5**2
dpdx1(99) = x4*x5**3*x6+x2*x3**3*x6+x3*x4*x5**3+x2*x3**3*x5
dpdx1(100) = x2*x4*x6**3+x2*x4*x5**3+x2*x3**3*x4
dpdx1(101) = 2*x1*x3*x5*x6**2
dpdx1(102) = 2*x1*x3*x5**2*x6+2*x1*x3**2*x5*x6
dpdx1(103) = 2*x1*x2*x5*x6**2+2*x1*x3*x4*x6**2+x2**2*x5**2*x6+x3*&
&*2*x4**2*x6
dpdx1(104) = x2**2*x5*x6**2+x3*x4**2*x6**2+2*x1*x2*x5**2*x6+2*x1*&
&x3**2*x4*x6
dpdx1(105) = 2*x1*x2*x4*x6**2+x2**2*x4*x5**2+x2*x3**2*x4**2
dpdx1(106) = 2*x1*x4*x5*x6**2+2*x1*x2*x3*x6**2+x2**2*x3*x5**2+x3*&
&*2*x4**2*x5
dpdx1(107) = x4**2*x5*x6**2+x2**2*x3*x6**2+2*x1*x2*x3*x5**2+2*x1*&
&x3**2*x4*x5
dpdx1(108) = 2*x1*x4*x5**2*x6+2*x1*x2*x3**2*x6+x3*x4**2*x5**2+x2*&
&*2*x3**2*x5
dpdx1(109) = x4**2*x5**2*x6+x2**2*x3**2*x6+2*x1*x3*x4*x5**2+2*x1*&
&x2*x3**2*x5
dpdx1(110) = x2*x4**2*x6**2+x2**2*x4*x6**2+x2*x4**2*x5**2+2*x1*x2&
&*x4*x5**2+x2**2*x3**2*x4+2*x1*x2*x3**2*x4
dpdx1(111) = 2*x1*x2**2*x5*x6+2*x1*x3*x4**2*x6
dpdx1(112) = 2*x1*x2*x4**2*x6+2*x1*x2**2*x4*x6+x2**2*x4**2*x5+2*x&
&1*x2**2*x4*x5+x2**2*x3*x4**2+2*x1*x2*x3*x4**2
dpdx1(113) = 2*x1*x4**2*x5*x6+2*x1*x2**2*x3*x6+2*x1*x3*x4**2*x5+2&
&*x1*x2**2*x3*x5
dpdx1(114) = x2**2*x4**2*x6+2*x1*x2*x4**2*x5+2*x1*x2**2*x3*x4
dpdx1(115) = 3*x1**2*x3*x5*x6
dpdx1(116) = x2**3*x5*x6+3*x1**2*x2*x5*x6+x3*x4**3*x6+3*x1**2*x3*&
&x4*x6
dpdx1(117) = 3*x1**2*x2*x4*x6+x2**3*x4*x5+x2*x3*x4**3
dpdx1(118) = 3*x1**2*x4*x5*x6+3*x1**2*x2*x3*x6+x3*x4**3*x5+x2**3*&
&x3*x5
dpdx1(119) = x4**3*x5*x6+x2**3*x3*x6+3*x1**2*x3*x4*x5+3*x1**2*x2*&
&x3*x5
dpdx1(120) = x2*x4**3*x6+x2**3*x4*x6+x2*x4**3*x5+3*x1**2*x2*x4*x5&
&+x2**3*x3*x4+3*x1**2*x2*x3*x4
dpdx1(121) = 0
dpdx1(122) = 0
dpdx1(123) = 0
dpdx1(124) = x5*x6**4+x3*x6**4
dpdx1(125) = x5**2*x6**3+x3**2*x6**3
dpdx1(126) = x5**3*x6**2+x3**3*x6**2
dpdx1(127) = x3**2*x5**3+x3**3*x5**2
dpdx1(128) = x5**4*x6+x3**4*x6
dpdx1(129) = x3*x5**4+x3**4*x5
dpdx1(130) = x4*x6**4+x2*x6**4+x2*x5**4+x3**4*x4
dpdx1(131) = x4*x5**4+x2*x3**4
dpdx1(132) = 2*x1*x5*x6**3+2*x1*x3*x6**3
dpdx1(133) = 2*x1*x5**2*x6**2+2*x1*x3**2*x6**2
dpdx1(134) = 2*x1*x3**2*x5**2
dpdx1(135) = 2*x1*x5**3*x6+2*x1*x3**3*x6
dpdx1(136) = 2*x1*x3*x5**3+2*x1*x3**3*x5
dpdx1(137) = 2*x1*x4*x6**3+2*x1*x2*x6**3+x2**2*x5**3+x3**3*x4**2
dpdx1(138) = x4**2*x6**3+x2**2*x6**3+2*x1*x2*x5**3+2*x1*x3**3*x4
dpdx1(139) = x4**2*x5**3+2*x1*x4*x5**3+x2**2*x3**3+2*x1*x2*x3**3
dpdx1(140) = 2*x1*x4**2*x6**2+2*x1*x2**2*x6**2+2*x1*x2**2*x5**2+2&
&*x1*x3**2*x4**2
dpdx1(141) = 2*x1*x2**2*x4**2
dpdx1(142) = 2*x1*x4**2*x5**2+2*x1*x2**2*x3**2
dpdx1(143) = 3*x1**2*x5*x6**2+3*x1**2*x3*x6**2
dpdx1(144) = 3*x1**2*x5**2*x6+3*x1**2*x3**2*x6
dpdx1(145) = 3*x1**2*x3*x5**2+3*x1**2*x3**2*x5
dpdx1(146) = 3*x1**2*x4*x6**2+3*x1**2*x2*x6**2+x2**3*x5**2+x3**2*&
&x4**3
dpdx1(147) = x4**3*x6**2+x2**3*x6**2+3*x1**2*x2*x5**2+3*x1**2*x3*&
&*2*x4
dpdx1(148) = x4**3*x5**2+3*x1**2*x4*x5**2+x2**3*x3**2+3*x1**2*x2*&
&x3**2
dpdx1(149) = 3*x1**2*x4**2*x6+3*x1**2*x2**2*x6+2*x1*x2**3*x5+2*x1&
&*x3*x4**3
dpdx1(150) = 2*x1*x4**3*x6+2*x1*x2**3*x6+3*x1**2*x2**2*x5+3*x1**2&
&*x3*x4**2
dpdx1(151) = x2**2*x4**3+2*x1*x2*x4**3+x2**3*x4**2+3*x1**2*x2*x4*&
&*2+2*x1*x2**3*x4+3*x1**2*x2**2*x4
dpdx1(152) = 2*x1*x4**3*x5+3*x1**2*x4**2*x5+2*x1*x2**3*x3+3*x1**2&
&*x2**2*x3
dpdx1(153) = 4*x1**3*x5*x6+4*x1**3*x3*x6
dpdx1(154) = 4*x1**3*x3*x5
dpdx1(155) = 4*x1**3*x4*x6+4*x1**3*x2*x6+x2**4*x5+x3*x4**4
dpdx1(156) = x4**4*x6+x2**4*x6+4*x1**3*x2*x5+4*x1**3*x3*x4
dpdx1(157) = x2*x4**4+x2**4*x4+4*x1**3*x2*x4
dpdx1(158) = x4**4*x5+4*x1**3*x4*x5+x2**4*x3+4*x1**3*x2*x3
dpdx1(159) = 0
dpdx1(160) = 0
dpdx1(161) = 0
dpdx1(162) = x5**5+x3**5
dpdx1(163) = 2*x1*x5**4+2*x1*x3**4
dpdx1(164) = 3*x1**2*x5**3+3*x1**2*x3**3
dpdx1(165) = 3*x1**2*x4**3+3*x1**2*x2**3
dpdx1(166) = 4*x1**3*x5**2+4*x1**3*x3**2
dpdx1(167) = 2*x1*x4**4+4*x1**3*x4**2+2*x1*x2**4+4*x1**3*x2**2
dpdx1(168) = 5*x1**4*x5+5*x1**4*x3
dpdx1(169) = x4**5+5*x1**4*x4+x2**5+5*x1**4*x2
dpdx1(170) = x2*x3*x4*x5*x6**2+x2*x3*x4*x5**2*x6+x2*x3**2*x4*x5*x&
&6
dpdx1(171) = x2*x3*x4**2*x5*x6+x2**2*x3*x4*x5*x6+2*x1*x2*x3*x4*x5&
&*x6
dpdx1(172) = x2*x3*x5**2*x6**2+x3**2*x4*x5*x6**2
dpdx1(173) = x3*x4*x5*x6**3+x2*x3*x5*x6**3+x2*x3*x5**3*x6+x3**3*x&
&4*x5*x6
dpdx1(174) = x3*x4*x5**2*x6**2+x2*x3**2*x5*x6**2+x3**2*x4*x5**2*x&
&6+x2*x3**2*x5**2*x6
dpdx1(175) = x2*x4*x5**2*x6**2+x2*x3**2*x4*x6**2+x2*x3**2*x4*x5**&
&2
dpdx1(176) = x3*x4*x5**3*x6+x2*x3**3*x5*x6
dpdx1(177) = x2*x4*x5*x6**3+x2*x3*x4*x6**3+x2*x4*x5**3*x6+x2*x3**&
&3*x4*x6+x2*x3*x4*x5**3+x2*x3**3*x4*x5
dpdx1(178) = 2*x1*x3*x4*x5*x6**2+2*x1*x2*x3*x5*x6**2+x2**2*x3*x5*&
&*2*x6+x3**2*x4**2*x5*x6
dpdx1(179) = x3*x4**2*x5*x6**2+x2**2*x3*x5*x6**2+2*x1*x2*x3*x5**2&
&*x6+2*x1*x3**2*x4*x5*x6
dpdx1(180) = 2*x1*x2*x4*x5*x6**2+2*x1*x2*x3*x4*x6**2+x2**2*x4*x5*&
&*2*x6+x2*x3**2*x4**2*x6+x2**2*x3*x4*x5**2+x2*x3**2*x4**2*x5
dpdx1(181) = x3*x4**2*x5**2*x6+2*x1*x3*x4*x5**2*x6+x2**2*x3**2*x5&
&*x6+2*x1*x2*x3**2*x5*x6
dpdx1(182) = x2**2*x4*x5*x6**2+x2*x3*x4**2*x6**2+2*x1*x2*x4*x5**2&
&*x6+2*x1*x2*x3**2*x4*x6+x2*x3*x4**2*x5**2+x2**2*x3**2*x4*x5
dpdx1(183) = x2*x4**2*x5*x6**2+x2**2*x3*x4*x6**2+x2*x4**2*x5**2*x&
&6+x2**2*x3**2*x4*x6+2*x1*x2*x3*x4*x5**2+2*x1*x2*x3**2*x4*x5
dpdx1(184) = 2*x1*x2**2*x4*x5*x6+2*x1*x2*x3*x4**2*x6+x2**2*x3*x4*&
&*2*x5
dpdx1(185) = 2*x1*x3*x4**2*x5*x6+2*x1*x2**2*x3*x5*x6
dpdx1(186) = x2**2*x4**2*x5*x6+2*x1*x2*x4**2*x5*x6+x2**2*x3*x4**2&
&*x6+2*x1*x2**2*x3*x4*x6+2*x1*x2*x3*x4**2*x5+2*x1*x2**2*x3*x4*x5
dpdx1(187) = x3*x4**3*x5*x6+3*x1**2*x3*x4*x5*x6+x2**3*x3*x5*x6+3*&
&x1**2*x2*x3*x5*x6
dpdx1(188) = x2**3*x4*x5*x6+3*x1**2*x2*x4*x5*x6+x2*x3*x4**3*x6+3*&
&x1**2*x2*x3*x4*x6+x2*x3*x4**3*x5+x2**3*x3*x4*x5
dpdx1(189) = x2*x4**3*x5*x6+x2**3*x3*x4*x6+3*x1**2*x2*x3*x4*x5
dpdx1(190) = x3*x5*x6**4
dpdx1(191) = x3*x5**2*x6**3+x3**2*x5*x6**3
dpdx1(192) = x3**2*x5**2*x6**2
dpdx1(193) = x3*x5**3*x6**2+x3**3*x5*x6**2
dpdx1(194) = x3**2*x5**3*x6+x3**3*x5**2*x6
dpdx1(195) = x3*x5**4*x6+x3**4*x5*x6
dpdx1(196) = x2*x5**2*x6**3+x3**2*x4*x6**3+x2*x5**3*x6**2+x3**3*x&
&4*x6**2
dpdx1(197) = x2*x5*x6**4+x3*x4*x6**4+x2*x5**4*x6+x3**4*x4*x6
dpdx1(198) = x4*x5*x6**4+x2*x3*x6**4+x2*x3*x5**4+x3**4*x4*x5
dpdx1(199) = x4*x5**2*x6**3+x2*x3**2*x6**3+x2*x3**2*x5**3+x3**3*x&
&4*x5**2
dpdx1(200) = x4*x5**3*x6**2+x2*x3**3*x6**2+x3**2*x4*x5**3+x2*x3**&
&3*x5**2
dpdx1(201) = x4*x5**4*x6+x2*x3**4*x6+x3*x4*x5**4+x2*x3**4*x5
dpdx1(202) = x2*x4*x6**4+x2*x4*x5**4+x2*x3**4*x4
dpdx1(203) = 2*x1*x3*x5*x6**3
dpdx1(204) = 2*x1*x3*x5**2*x6**2+2*x1*x3**2*x5*x6**2
dpdx1(205) = 2*x1*x3**2*x5**2*x6
dpdx1(206) = 2*x1*x3*x5**3*x6+2*x1*x3**3*x5*x6
dpdx1(207) = 2*x1*x2*x5*x6**3+2*x1*x3*x4*x6**3+x2**2*x5**3*x6+x3*&
&*3*x4**2*x6
dpdx1(208) = x2**2*x5**2*x6**2+2*x1*x2*x5**2*x6**2+x3**2*x4**2*x6&
&**2+2*x1*x3**2*x4*x6**2
dpdx1(209) = x2**2*x5*x6**3+x3*x4**2*x6**3+2*x1*x2*x5**3*x6+2*x1*&
&x3**3*x4*x6
dpdx1(210) = 2*x1*x2*x4*x6**3+x2**2*x4*x5**3+x2*x3**3*x4**2
dpdx1(211) = 2*x1*x4*x5*x6**3+2*x1*x2*x3*x6**3+x2**2*x3*x5**3+x3*&
&*3*x4**2*x5
dpdx1(212) = x4**2*x5*x6**3+x2**2*x3*x6**3+2*x1*x2*x3*x5**3+2*x1*&
&x3**3*x4*x5
dpdx1(213) = 2*x1*x4*x5**2*x6**2+2*x1*x2*x3**2*x6**2+x3**2*x4**2*&
&x5**2+x2**2*x3**2*x5**2
dpdx1(214) = x4**2*x5**2*x6**2+x2**2*x3**2*x6**2+2*x1*x3**2*x4*x5&
&**2+2*x1*x2*x3**2*x5**2
dpdx1(215) = 2*x1*x4*x5**3*x6+2*x1*x2*x3**3*x6+x3*x4**2*x5**3+x2*&
&*2*x3**3*x5
dpdx1(216) = x4**2*x5**3*x6+x2**2*x3**3*x6+2*x1*x3*x4*x5**3+2*x1*&
&x2*x3**3*x5
dpdx1(217) = x2*x4**2*x6**3+x2**2*x4*x6**3+x2*x4**2*x5**3+2*x1*x2&
&*x4*x5**3+x2**2*x3**3*x4+2*x1*x2*x3**3*x4
dpdx1(218) = 2*x1*x2**2*x5*x6**2+2*x1*x3*x4**2*x6**2+2*x1*x2**2*x&
&5**2*x6+2*x1*x3**2*x4**2*x6
dpdx1(219) = 2*x1*x2*x4**2*x6**2+2*x1*x2**2*x4*x6**2+x2**2*x4**2*&
&x5**2+2*x1*x2**2*x4*x5**2+x2**2*x3**2*x4**2+2*x1*x2*x3**2*x4**2
dpdx1(220) = 2*x1*x4**2*x5*x6**2+2*x1*x2**2*x3*x6**2+2*x1*x2**2*x&
&3*x5**2+2*x1*x3**2*x4**2*x5
dpdx1(221) = 2*x1*x2**2*x4**2*x6+2*x1*x2**2*x4**2*x5+2*x1*x2**2*x&
&3*x4**2
dpdx1(222) = 2*x1*x4**2*x5**2*x6+2*x1*x2**2*x3**2*x6+2*x1*x3*x4**&
&2*x5**2+2*x1*x2**2*x3**2*x5
dpdx1(223) = x2**2*x4**2*x6**2+2*x1*x2*x4**2*x5**2+2*x1*x2**2*x3*&
&*2*x4
dpdx1(224) = 3*x1**2*x3*x5*x6**2
dpdx1(225) = 3*x1**2*x3*x5**2*x6+3*x1**2*x3**2*x5*x6
dpdx1(226) = 3*x1**2*x2*x5*x6**2+3*x1**2*x3*x4*x6**2+x2**3*x5**2*&
&x6+x3**2*x4**3*x6
dpdx1(227) = x2**3*x5*x6**2+x3*x4**3*x6**2+3*x1**2*x2*x5**2*x6+3*&
&x1**2*x3**2*x4*x6
dpdx1(228) = 3*x1**2*x2*x4*x6**2+x2**3*x4*x5**2+x2*x3**2*x4**3
dpdx1(229) = 3*x1**2*x4*x5*x6**2+3*x1**2*x2*x3*x6**2+x2**3*x3*x5*&
&*2+x3**2*x4**3*x5
dpdx1(230) = x4**3*x5*x6**2+x2**3*x3*x6**2+3*x1**2*x2*x3*x5**2+3*&
&x1**2*x3**2*x4*x5
dpdx1(231) = 3*x1**2*x4*x5**2*x6+3*x1**2*x2*x3**2*x6+x3*x4**3*x5*&
&*2+x2**3*x3**2*x5
dpdx1(232) = x4**3*x5**2*x6+x2**3*x3**2*x6+3*x1**2*x3*x4*x5**2+3*&
&x1**2*x2*x3**2*x5
dpdx1(233) = x2*x4**3*x6**2+x2**3*x4*x6**2+x2*x4**3*x5**2+3*x1**2&
&*x2*x4*x5**2+x2**3*x3**2*x4+3*x1**2*x2*x3**2*x4
dpdx1(234) = 2*x1*x2**3*x5*x6+3*x1**2*x2**2*x5*x6+2*x1*x3*x4**3*x&
&6+3*x1**2*x3*x4**2*x6
dpdx1(235) = 3*x1**2*x2*x4**2*x6+3*x1**2*x2**2*x4*x6+x2**3*x4**2*&
&x5+2*x1*x2**3*x4*x5+x2**2*x3*x4**3+2*x1*x2*x3*x4**3
dpdx1(236) = 2*x1*x2*x4**3*x6+2*x1*x2**3*x4*x6+x2**2*x4**3*x5+3*x&
&1**2*x2**2*x4*x5+x2**3*x3*x4**2+3*x1**2*x2*x3*x4**2
dpdx1(237) = 3*x1**2*x4**2*x5*x6+3*x1**2*x2**2*x3*x6+2*x1*x3*x4**&
&3*x5+2*x1*x2**3*x3*x5
dpdx1(238) = 2*x1*x4**3*x5*x6+2*x1*x2**3*x3*x6+3*x1**2*x3*x4**2*x&
&5+3*x1**2*x2**2*x3*x5
dpdx1(239) = x2**2*x4**3*x6+x2**3*x4**2*x6+2*x1*x2*x4**3*x5+3*x1*&
&*2*x2*x4**2*x5+2*x1*x2**3*x3*x4+3*x1**2*x2**2*x3*x4
dpdx1(240) = 4*x1**3*x3*x5*x6
dpdx1(241) = x2**4*x5*x6+4*x1**3*x2*x5*x6+x3*x4**4*x6+4*x1**3*x3*&
&x4*x6
dpdx1(242) = 4*x1**3*x2*x4*x6+x2**4*x4*x5+x2*x3*x4**4
dpdx1(243) = 4*x1**3*x4*x5*x6+4*x1**3*x2*x3*x6+x3*x4**4*x5+x2**4*&
&x3*x5
dpdx1(244) = x4**4*x5*x6+x2**4*x3*x6+4*x1**3*x3*x4*x5+4*x1**3*x2*&
&x3*x5
dpdx1(245) = x2*x4**4*x6+x2**4*x4*x6+x2*x4**4*x5+4*x1**3*x2*x4*x5&
&+x2**4*x3*x4+4*x1**3*x2*x3*x4
dpdx1(246) = 0
dpdx1(247) = 0
dpdx1(248) = 0
dpdx1(249) = 0
dpdx1(250) = x5*x6**5+x3*x6**5
dpdx1(251) = x5**2*x6**4+x3**2*x6**4
dpdx1(252) = x5**3*x6**3+x3**3*x6**3
dpdx1(253) = x3**3*x5**3
dpdx1(254) = x5**4*x6**2+x3**4*x6**2
dpdx1(255) = x3**2*x5**4+x3**4*x5**2
dpdx1(256) = x5**5*x6+x3**5*x6
dpdx1(257) = x3*x5**5+x3**5*x5
dpdx1(258) = x4*x6**5+x2*x6**5+x2*x5**5+x3**5*x4
dpdx1(259) = x4*x5**5+x2*x3**5
dpdx1(260) = 2*x1*x5*x6**4+2*x1*x3*x6**4
dpdx1(261) = 2*x1*x5**2*x6**3+2*x1*x3**2*x6**3
dpdx1(262) = 2*x1*x5**3*x6**2+2*x1*x3**3*x6**2
dpdx1(263) = 2*x1*x3**2*x5**3+2*x1*x3**3*x5**2
dpdx1(264) = 2*x1*x5**4*x6+2*x1*x3**4*x6
dpdx1(265) = 2*x1*x3*x5**4+2*x1*x3**4*x5
dpdx1(266) = 2*x1*x4*x6**4+2*x1*x2*x6**4+x2**2*x5**4+x3**4*x4**2
dpdx1(267) = x4**2*x6**4+x2**2*x6**4+2*x1*x2*x5**4+2*x1*x3**4*x4
dpdx1(268) = x4**2*x5**4+2*x1*x4*x5**4+x2**2*x3**4+2*x1*x2*x3**4
dpdx1(269) = 2*x1*x4**2*x6**3+2*x1*x2**2*x6**3+2*x1*x2**2*x5**3+2&
&*x1*x3**3*x4**2
dpdx1(270) = 2*x1*x4**2*x5**3+2*x1*x2**2*x3**3
dpdx1(271) = 3*x1**2*x5*x6**3+3*x1**2*x3*x6**3
dpdx1(272) = 3*x1**2*x5**2*x6**2+3*x1**2*x3**2*x6**2
dpdx1(273) = 3*x1**2*x3**2*x5**2
dpdx1(274) = 3*x1**2*x5**3*x6+3*x1**2*x3**3*x6
dpdx1(275) = 3*x1**2*x3*x5**3+3*x1**2*x3**3*x5
dpdx1(276) = 3*x1**2*x4*x6**3+3*x1**2*x2*x6**3+x2**3*x5**3+x3**3*&
&x4**3
dpdx1(277) = x4**3*x6**3+x2**3*x6**3+3*x1**2*x2*x5**3+3*x1**2*x3*&
&*3*x4
dpdx1(278) = x4**3*x5**3+3*x1**2*x4*x5**3+x2**3*x3**3+3*x1**2*x2*&
&x3**3
dpdx1(279) = 3*x1**2*x4**2*x6**2+3*x1**2*x2**2*x6**2+2*x1*x2**3*x&
&5**2+2*x1*x3**2*x4**3
dpdx1(280) = 2*x1*x4**3*x6**2+2*x1*x2**3*x6**2+3*x1**2*x2**2*x5**&
&2+3*x1**2*x3**2*x4**2
dpdx1(281) = 2*x1*x2**2*x4**3+2*x1*x2**3*x4**2+3*x1**2*x2**2*x4**&
&2
dpdx1(282) = 2*x1*x4**3*x5**2+3*x1**2*x4**2*x5**2+2*x1*x2**3*x3**&
&2+3*x1**2*x2**2*x3**2
dpdx1(283) = 3*x1**2*x4**3*x6+3*x1**2*x2**3*x6+3*x1**2*x2**3*x5+3&
&*x1**2*x3*x4**3
dpdx1(284) = x2**3*x4**3+3*x1**2*x2*x4**3+3*x1**2*x2**3*x4
dpdx1(285) = 3*x1**2*x4**3*x5+3*x1**2*x2**3*x3
dpdx1(286) = 4*x1**3*x5*x6**2+4*x1**3*x3*x6**2
dpdx1(287) = 4*x1**3*x5**2*x6+4*x1**3*x3**2*x6
dpdx1(288) = 4*x1**3*x3*x5**2+4*x1**3*x3**2*x5
dpdx1(289) = 4*x1**3*x4*x6**2+4*x1**3*x2*x6**2+x2**4*x5**2+x3**2*&
&x4**4
dpdx1(290) = x4**4*x6**2+x2**4*x6**2+4*x1**3*x2*x5**2+4*x1**3*x3*&
&*2*x4
dpdx1(291) = x4**4*x5**2+4*x1**3*x4*x5**2+x2**4*x3**2+4*x1**3*x2*&
&x3**2
dpdx1(292) = 4*x1**3*x4**2*x6+4*x1**3*x2**2*x6+2*x1*x2**4*x5+2*x1&
&*x3*x4**4
dpdx1(293) = 2*x1*x4**4*x6+2*x1*x2**4*x6+4*x1**3*x2**2*x5+4*x1**3&
&*x3*x4**2
dpdx1(294) = x2**2*x4**4+2*x1*x2*x4**4+x2**4*x4**2+4*x1**3*x2*x4*&
&*2+2*x1*x2**4*x4+4*x1**3*x2**2*x4
dpdx1(295) = 2*x1*x4**4*x5+4*x1**3*x4**2*x5+2*x1*x2**4*x3+4*x1**3&
&*x2**2*x3
dpdx1(296) = 5*x1**4*x5*x6+5*x1**4*x3*x6
dpdx1(297) = 5*x1**4*x3*x5
dpdx1(298) = 5*x1**4*x4*x6+5*x1**4*x2*x6+x2**5*x5+x3*x4**5
dpdx1(299) = x4**5*x6+x2**5*x6+5*x1**4*x2*x5+5*x1**4*x3*x4
dpdx1(300) = x2*x4**5+x2**5*x4+5*x1**4*x2*x4
dpdx1(301) = x4**5*x5+5*x1**4*x4*x5+x2**5*x3+5*x1**4*x2*x3
dpdx1(302) = 0
dpdx1(303) = 0
dpdx1(304) = 0
dpdx1(305) = x5**6+x3**6
dpdx1(306) = 2*x1*x5**5+2*x1*x3**5
dpdx1(307) = 3*x1**2*x5**4+3*x1**2*x3**4
dpdx1(308) = 4*x1**3*x5**3+4*x1**3*x3**3
dpdx1(309) = 3*x1**2*x4**4+4*x1**3*x4**3+3*x1**2*x2**4+4*x1**3*x2&
&**3
dpdx1(310) = 5*x1**4*x5**2+5*x1**4*x3**2
dpdx1(311) = 2*x1*x4**5+5*x1**4*x4**2+2*x1*x2**5+5*x1**4*x2**2
dpdx1(312) = 6*x1**5*x5+6*x1**5*x3
dpdx1(313) = x4**6+6*x1**5*x4+x2**6+6*x1**5*x2
dpdx1(314) = x2*x3*x4*x5**2*x6**2+x2*x3**2*x4*x5*x6**2+x2*x3**2*x&
&4*x5**2*x6
dpdx1(315) = x2*x3*x4*x5*x6**3+x2*x3*x4*x5**3*x6+x2*x3**3*x4*x5*x&
&6
dpdx1(316) = 2*x1*x2*x3*x4*x5*x6**2+x2**2*x3*x4*x5**2*x6+x2*x3**2&
&*x4**2*x5*x6
dpdx1(317) = x2*x3*x4**2*x5*x6**2+x2**2*x3*x4*x5*x6**2+x2*x3*x4**&
&2*x5**2*x6+2*x1*x2*x3*x4*x5**2*x6+x2**2*x3**2*x4*x5*x6+2*x1*x2*x3&
&**2*x4*x5*x6
dpdx1(318) = x2**2*x3*x4**2*x5*x6+2*x1*x2*x3*x4**2*x5*x6+2*x1*x2*&
&*2*x3*x4*x5*x6
dpdx1(319) = x2*x3*x4**3*x5*x6+x2**3*x3*x4*x5*x6+3*x1**2*x2*x3*x4&
&*x5*x6
dpdx1(320) = x2*x3*x5**2*x6**3+x3**2*x4*x5*x6**3+x2*x3*x5**3*x6**&
&2+x3**3*x4*x5*x6**2
dpdx1(321) = x3*x4*x5*x6**4+x2*x3*x5*x6**4+x2*x3*x5**4*x6+x3**4*x&
&4*x5*x6
dpdx1(322) = x3**2*x4*x5**2*x6**2+x2*x3**2*x5**2*x6**2
dpdx1(323) = x3*x4*x5**2*x6**3+x2*x3**2*x5*x6**3+x2*x3**2*x5**3*x&
&6+x3**3*x4*x5**2*x6
dpdx1(324) = x3*x4*x5**3*x6**2+x2*x3**3*x5*x6**2+x3**2*x4*x5**3*x&
&6+x2*x3**3*x5**2*x6
dpdx1(325) = x2*x4*x5**2*x6**3+x2*x3**2*x4*x6**3+x2*x4*x5**3*x6**&
&2+x2*x3**3*x4*x6**2+x2*x3**2*x4*x5**3+x2*x3**3*x4*x5**2
dpdx1(326) = x3*x4*x5**4*x6+x2*x3**4*x5*x6
dpdx1(327) = x2*x4*x5*x6**4+x2*x3*x4*x6**4+x2*x4*x5**4*x6+x2*x3**&
&4*x4*x6+x2*x3*x4*x5**4+x2*x3**4*x4*x5
dpdx1(328) = 2*x1*x3*x4*x5*x6**3+2*x1*x2*x3*x5*x6**3+x2**2*x3*x5*&
&*3*x6+x3**3*x4**2*x5*x6
dpdx1(329) = x2**2*x3*x5**2*x6**2+2*x1*x2*x3*x5**2*x6**2+x3**2*x4&
&**2*x5*x6**2+2*x1*x3**2*x4*x5*x6**2
dpdx1(330) = x3*x4**2*x5*x6**3+x2**2*x3*x5*x6**3+2*x1*x2*x3*x5**3&
&*x6+2*x1*x3**3*x4*x5*x6
dpdx1(331) = 2*x1*x2*x4*x5*x6**3+2*x1*x2*x3*x4*x6**3+x2**2*x4*x5*&
&*3*x6+x2*x3**3*x4**2*x6+x2**2*x3*x4*x5**3+x2*x3**3*x4**2*x5
dpdx1(332) = 2*x1*x3*x4*x5**2*x6**2+2*x1*x2*x3**2*x5*x6**2+x3**2*&
&x4**2*x5**2*x6+x2**2*x3**2*x5**2*x6
dpdx1(333) = x3*x4**2*x5**2*x6**2+x2**2*x3**2*x5*x6**2+2*x1*x3**2&
&*x4*x5**2*x6+2*x1*x2*x3**2*x5**2*x6
dpdx1(334) = x2**2*x4*x5**2*x6**2+2*x1*x2*x4*x5**2*x6**2+x2*x3**2&
&*x4**2*x6**2+2*x1*x2*x3**2*x4*x6**2+x2*x3**2*x4**2*x5**2+x2**2*x3&
&**2*x4*x5**2
dpdx1(335) = x2*x4**2*x5**2*x6**2+x2**2*x3**2*x4*x6**2+2*x1*x2*x3&
&**2*x4*x5**2
dpdx1(336) = x3*x4**2*x5**3*x6+2*x1*x3*x4*x5**3*x6+x2**2*x3**3*x5&
&*x6+2*x1*x2*x3**3*x5*x6
dpdx1(337) = x2**2*x4*x5*x6**3+x2*x3*x4**2*x6**3+2*x1*x2*x4*x5**3&
&*x6+2*x1*x2*x3**3*x4*x6+x2*x3*x4**2*x5**3+x2**2*x3**3*x4*x5
dpdx1(338) = x2*x4**2*x5*x6**3+x2**2*x3*x4*x6**3+x2*x4**2*x5**3*x&
&6+x2**2*x3**3*x4*x6+2*x1*x2*x3*x4*x5**3+2*x1*x2*x3**3*x4*x5
dpdx1(339) = 2*x1*x2**2*x4*x5*x6**2+2*x1*x2*x3*x4**2*x6**2+2*x1*x&
&2**2*x4*x5**2*x6+2*x1*x2*x3**2*x4**2*x6+x2**2*x3*x4**2*x5**2+x2**&
&2*x3**2*x4**2*x5
dpdx1(340) = 2*x1*x3*x4**2*x5*x6**2+2*x1*x2**2*x3*x5*x6**2+2*x1*x&
&2**2*x3*x5**2*x6+2*x1*x3**2*x4**2*x5*x6
dpdx1(341) = 2*x1*x2*x4**2*x5*x6**2+2*x1*x2**2*x3*x4*x6**2+x2**2*&
&x4**2*x5**2*x6+x2**2*x3**2*x4**2*x6+2*x1*x2**2*x3*x4*x5**2+2*x1*x&
&2*x3**2*x4**2*x5
dpdx1(342) = 2*x1*x2**2*x4**2*x5*x6+2*x1*x2**2*x3*x4**2*x6+2*x1*x&
&2**2*x3*x4**2*x5
dpdx1(343) = 2*x1*x3*x4**2*x5**2*x6+2*x1*x2**2*x3**2*x5*x6
dpdx1(344) = x2**2*x4**2*x5*x6**2+x2**2*x3*x4**2*x6**2+2*x1*x2*x4&
&**2*x5**2*x6+2*x1*x2**2*x3**2*x4*x6+2*x1*x2*x3*x4**2*x5**2+2*x1*x&
&2**2*x3**2*x4*x5
dpdx1(345) = 3*x1**2*x3*x4*x5*x6**2+3*x1**2*x2*x3*x5*x6**2+x2**3*&
&x3*x5**2*x6+x3**2*x4**3*x5*x6
dpdx1(346) = x3*x4**3*x5*x6**2+x2**3*x3*x5*x6**2+3*x1**2*x2*x3*x5&
&**2*x6+3*x1**2*x3**2*x4*x5*x6
dpdx1(347) = 3*x1**2*x2*x4*x5*x6**2+3*x1**2*x2*x3*x4*x6**2+x2**3*&
&x4*x5**2*x6+x2*x3**2*x4**3*x6+x2**3*x3*x4*x5**2+x2*x3**2*x4**3*x5
dpdx1(348) = x3*x4**3*x5**2*x6+3*x1**2*x3*x4*x5**2*x6+x2**3*x3**2&
&*x5*x6+3*x1**2*x2*x3**2*x5*x6
dpdx1(349) = x2**3*x4*x5*x6**2+x2*x3*x4**3*x6**2+3*x1**2*x2*x4*x5&
&**2*x6+3*x1**2*x2*x3**2*x4*x6+x2*x3*x4**3*x5**2+x2**3*x3**2*x4*x5
dpdx1(350) = x2*x4**3*x5*x6**2+x2**3*x3*x4*x6**2+x2*x4**3*x5**2*x&
&6+x2**3*x3**2*x4*x6+3*x1**2*x2*x3*x4*x5**2+3*x1**2*x2*x3**2*x4*x5
dpdx1(351) = 2*x1*x2**3*x4*x5*x6+3*x1**2*x2**2*x4*x5*x6+2*x1*x2*x&
&3*x4**3*x6+3*x1**2*x2*x3*x4**2*x6+x2**2*x3*x4**3*x5+x2**3*x3*x4**&
&2*x5
dpdx1(352) = 2*x1*x3*x4**3*x5*x6+3*x1**2*x3*x4**2*x5*x6+2*x1*x2**&
&3*x3*x5*x6+3*x1**2*x2**2*x3*x5*x6
dpdx1(353) = x2**3*x4**2*x5*x6+3*x1**2*x2*x4**2*x5*x6+x2**2*x3*x4&
&**3*x6+3*x1**2*x2**2*x3*x4*x6+2*x1*x2*x3*x4**3*x5+2*x1*x2**3*x3*x&
&4*x5
dpdx1(354) = x2**2*x4**3*x5*x6+2*x1*x2*x4**3*x5*x6+x2**3*x3*x4**2&
&*x6+2*x1*x2**3*x3*x4*x6+3*x1**2*x2*x3*x4**2*x5+3*x1**2*x2**2*x3*x&
&4*x5
dpdx1(355) = x3*x4**4*x5*x6+4*x1**3*x3*x4*x5*x6+x2**4*x3*x5*x6+4*&
&x1**3*x2*x3*x5*x6
dpdx1(356) = x2**4*x4*x5*x6+4*x1**3*x2*x4*x5*x6+x2*x3*x4**4*x6+4*&
&x1**3*x2*x3*x4*x6+x2*x3*x4**4*x5+x2**4*x3*x4*x5
dpdx1(357) = x2*x4**4*x5*x6+x2**4*x3*x4*x6+4*x1**3*x2*x3*x4*x5
dpdx1(358) = x3*x5*x6**5
dpdx1(359) = x3*x5**2*x6**4+x3**2*x5*x6**4
dpdx1(360) = x3**2*x5**2*x6**3
dpdx1(361) = x3*x5**3*x6**3+x3**3*x5*x6**3
dpdx1(362) = x3**2*x5**3*x6**2+x3**3*x5**2*x6**2
dpdx1(363) = x3**3*x5**3*x6
dpdx1(364) = x3*x5**4*x6**2+x3**4*x5*x6**2
dpdx1(365) = x3**2*x5**4*x6+x3**4*x5**2*x6
dpdx1(366) = x3*x5**5*x6+x3**5*x5*x6
dpdx1(367) = x2*x5**3*x6**3+x3**3*x4*x6**3
dpdx1(368) = x2*x5**2*x6**4+x3**2*x4*x6**4+x2*x5**4*x6**2+x3**4*x&
&4*x6**2
dpdx1(369) = x2*x5*x6**5+x3*x4*x6**5+x2*x5**5*x6+x3**5*x4*x6
dpdx1(370) = x4*x5*x6**5+x2*x3*x6**5+x2*x3*x5**5+x3**5*x4*x5
dpdx1(371) = x4*x5**2*x6**4+x2*x3**2*x6**4+x2*x3**2*x5**4+x3**4*x&
&4*x5**2
dpdx1(372) = x4*x5**3*x6**3+x2*x3**3*x6**3+x3**3*x4*x5**3+x2*x3**&
&3*x5**3
dpdx1(373) = x4*x5**4*x6**2+x2*x3**4*x6**2+x3**2*x4*x5**4+x2*x3**&
&4*x5**2
dpdx1(374) = x4*x5**5*x6+x2*x3**5*x6+x3*x4*x5**5+x2*x3**5*x5
dpdx1(375) = x2*x4*x6**5+x2*x4*x5**5+x2*x3**5*x4
dpdx1(376) = 2*x1*x3*x5*x6**4
dpdx1(377) = 2*x1*x3*x5**2*x6**3+2*x1*x3**2*x5*x6**3
dpdx1(378) = 2*x1*x3**2*x5**2*x6**2
dpdx1(379) = 2*x1*x3*x5**3*x6**2+2*x1*x3**3*x5*x6**2
dpdx1(380) = 2*x1*x3**2*x5**3*x6+2*x1*x3**3*x5**2*x6
dpdx1(381) = 2*x1*x3*x5**4*x6+2*x1*x3**4*x5*x6
dpdx1(382) = 2*x1*x2*x5*x6**4+2*x1*x3*x4*x6**4+x2**2*x5**4*x6+x3*&
&*4*x4**2*x6
dpdx1(383) = 2*x1*x2*x5**2*x6**3+2*x1*x3**2*x4*x6**3+x2**2*x5**3*&
&x6**2+x3**3*x4**2*x6**2
dpdx1(384) = x2**2*x5**2*x6**3+x3**2*x4**2*x6**3+2*x1*x2*x5**3*x6&
&**2+2*x1*x3**3*x4*x6**2
dpdx1(385) = x2**2*x5*x6**4+x3*x4**2*x6**4+2*x1*x2*x5**4*x6+2*x1*&
&x3**4*x4*x6
dpdx1(386) = 2*x1*x2*x4*x6**4+x2**2*x4*x5**4+x2*x3**4*x4**2
dpdx1(387) = 2*x1*x4*x5*x6**4+2*x1*x2*x3*x6**4+x2**2*x3*x5**4+x3*&
&*4*x4**2*x5
dpdx1(388) = x4**2*x5*x6**4+x2**2*x3*x6**4+2*x1*x2*x3*x5**4+2*x1*&
&x3**4*x4*x5
dpdx1(389) = 2*x1*x4*x5**2*x6**3+2*x1*x2*x3**2*x6**3+x2**2*x3**2*&
&x5**3+x3**3*x4**2*x5**2
dpdx1(390) = x4**2*x5**2*x6**3+x2**2*x3**2*x6**3+2*x1*x2*x3**2*x5&
&**3+2*x1*x3**3*x4*x5**2
dpdx1(391) = 2*x1*x4*x5**3*x6**2+2*x1*x2*x3**3*x6**2+x3**2*x4**2*&
&x5**3+x2**2*x3**3*x5**2
dpdx1(392) = x4**2*x5**3*x6**2+x2**2*x3**3*x6**2+2*x1*x3**2*x4*x5&
&**3+2*x1*x2*x3**3*x5**2
dpdx1(393) = 2*x1*x4*x5**4*x6+2*x1*x2*x3**4*x6+x3*x4**2*x5**4+x2*&
&*2*x3**4*x5
dpdx1(394) = x4**2*x5**4*x6+x2**2*x3**4*x6+2*x1*x3*x4*x5**4+2*x1*&
&x2*x3**4*x5
dpdx1(395) = x2*x4**2*x6**4+x2**2*x4*x6**4+x2*x4**2*x5**4+2*x1*x2&
&*x4*x5**4+x2**2*x3**4*x4+2*x1*x2*x3**4*x4
dpdx1(396) = 2*x1*x2**2*x5**2*x6**2+2*x1*x3**2*x4**2*x6**2
dpdx1(397) = 2*x1*x2**2*x5*x6**3+2*x1*x3*x4**2*x6**3+2*x1*x2**2*x&
&5**3*x6+2*x1*x3**3*x4**2*x6
dpdx1(398) = 2*x1*x2*x4**2*x6**3+2*x1*x2**2*x4*x6**3+x2**2*x4**2*&
&x5**3+2*x1*x2**2*x4*x5**3+x2**2*x3**3*x4**2+2*x1*x2*x3**3*x4**2
dpdx1(399) = 2*x1*x4**2*x5*x6**3+2*x1*x2**2*x3*x6**3+2*x1*x2**2*x&
&3*x5**3+2*x1*x3**3*x4**2*x5
dpdx1(400) = 2*x1*x4**2*x5**2*x6**2+2*x1*x2**2*x3**2*x6**2+2*x1*x&
&3**2*x4**2*x5**2+2*x1*x2**2*x3**2*x5**2
dpdx1(401) = 2*x1*x2**2*x4**2*x6**2+2*x1*x2**2*x4**2*x5**2+2*x1*x&
&2**2*x3**2*x4**2
dpdx1(402) = 2*x1*x4**2*x5**3*x6+2*x1*x2**2*x3**3*x6+2*x1*x3*x4**&
&2*x5**3+2*x1*x2**2*x3**3*x5
dpdx1(403) = x2**2*x4**2*x6**3+2*x1*x2*x4**2*x5**3+2*x1*x2**2*x3*&
&*3*x4
dpdx1(404) = 3*x1**2*x3*x5*x6**3
dpdx1(405) = 3*x1**2*x3*x5**2*x6**2+3*x1**2*x3**2*x5*x6**2
dpdx1(406) = 3*x1**2*x3**2*x5**2*x6
dpdx1(407) = 3*x1**2*x3*x5**3*x6+3*x1**2*x3**3*x5*x6
dpdx1(408) = 3*x1**2*x2*x5*x6**3+3*x1**2*x3*x4*x6**3+x2**3*x5**3*&
&x6+x3**3*x4**3*x6
dpdx1(409) = x2**3*x5**2*x6**2+3*x1**2*x2*x5**2*x6**2+x3**2*x4**3&
&*x6**2+3*x1**2*x3**2*x4*x6**2
dpdx1(410) = x2**3*x5*x6**3+x3*x4**3*x6**3+3*x1**2*x2*x5**3*x6+3*&
&x1**2*x3**3*x4*x6
dpdx1(411) = 3*x1**2*x2*x4*x6**3+x2**3*x4*x5**3+x2*x3**3*x4**3
dpdx1(412) = 3*x1**2*x4*x5*x6**3+3*x1**2*x2*x3*x6**3+x2**3*x3*x5*&
&*3+x3**3*x4**3*x5
dpdx1(413) = x4**3*x5*x6**3+x2**3*x3*x6**3+3*x1**2*x2*x3*x5**3+3*&
&x1**2*x3**3*x4*x5
dpdx1(414) = 3*x1**2*x4*x5**2*x6**2+3*x1**2*x2*x3**2*x6**2+x3**2*&
&x4**3*x5**2+x2**3*x3**2*x5**2
dpdx1(415) = x4**3*x5**2*x6**2+x2**3*x3**2*x6**2+3*x1**2*x3**2*x4&
&*x5**2+3*x1**2*x2*x3**2*x5**2
dpdx1(416) = 3*x1**2*x4*x5**3*x6+3*x1**2*x2*x3**3*x6+x3*x4**3*x5*&
&*3+x2**3*x3**3*x5
dpdx1(417) = x4**3*x5**3*x6+x2**3*x3**3*x6+3*x1**2*x3*x4*x5**3+3*&
&x1**2*x2*x3**3*x5
dpdx1(418) = x2*x4**3*x6**3+x2**3*x4*x6**3+x2*x4**3*x5**3+3*x1**2&
&*x2*x4*x5**3+x2**3*x3**3*x4+3*x1**2*x2*x3**3*x4
dpdx1(419) = 3*x1**2*x2**2*x5*x6**2+3*x1**2*x3*x4**2*x6**2+2*x1*x&
&2**3*x5**2*x6+2*x1*x3**2*x4**3*x6
dpdx1(420) = 2*x1*x2**3*x5*x6**2+2*x1*x3*x4**3*x6**2+3*x1**2*x2**&
&2*x5**2*x6+3*x1**2*x3**2*x4**2*x6
dpdx1(421) = 3*x1**2*x2*x4**2*x6**2+3*x1**2*x2**2*x4*x6**2+x2**3*&
&x4**2*x5**2+2*x1*x2**3*x4*x5**2+x2**2*x3**2*x4**3+2*x1*x2*x3**2*x&
&4**3
dpdx1(422) = 2*x1*x2*x4**3*x6**2+2*x1*x2**3*x4*x6**2+x2**2*x4**3*&
&x5**2+3*x1**2*x2**2*x4*x5**2+x2**3*x3**2*x4**2+3*x1**2*x2*x3**2*x&
&4**2
dpdx1(423) = 3*x1**2*x2**2*x4**2*x6+2*x1*x2**3*x4**2*x5+2*x1*x2**&
&2*x3*x4**3
dpdx1(424) = 3*x1**2*x4**2*x5*x6**2+3*x1**2*x2**2*x3*x6**2+2*x1*x&
&2**3*x3*x5**2+2*x1*x3**2*x4**3*x5
dpdx1(425) = 2*x1*x4**3*x5*x6**2+2*x1*x2**3*x3*x6**2+3*x1**2*x2**&
&2*x3*x5**2+3*x1**2*x3**2*x4**2*x5
dpdx1(426) = 2*x1*x2**2*x4**3*x6+2*x1*x2**3*x4**2*x6+2*x1*x2**2*x&
&4**3*x5+3*x1**2*x2**2*x4**2*x5+2*x1*x2**3*x3*x4**2+3*x1**2*x2**2*&
&x3*x4**2
dpdx1(427) = 3*x1**2*x4**2*x5**2*x6+3*x1**2*x2**2*x3**2*x6+2*x1*x&
&3*x4**3*x5**2+2*x1*x2**3*x3**2*x5
dpdx1(428) = 2*x1*x4**3*x5**2*x6+2*x1*x2**3*x3**2*x6+3*x1**2*x3*x&
&4**2*x5**2+3*x1**2*x2**2*x3**2*x5
dpdx1(429) = x2**2*x4**3*x6**2+x2**3*x4**2*x6**2+2*x1*x2*x4**3*x5&
&**2+3*x1**2*x2*x4**2*x5**2+2*x1*x2**3*x3**2*x4+3*x1**2*x2**2*x3**&
&2*x4
dpdx1(430) = 3*x1**2*x2**3*x5*x6+3*x1**2*x3*x4**3*x6
dpdx1(431) = 3*x1**2*x2*x4**3*x6+3*x1**2*x2**3*x4*x6+x2**3*x4**3*&
&x5+3*x1**2*x2**3*x4*x5+x2**3*x3*x4**3+3*x1**2*x2*x3*x4**3
dpdx1(432) = 3*x1**2*x4**3*x5*x6+3*x1**2*x2**3*x3*x6+3*x1**2*x3*x&
&4**3*x5+3*x1**2*x2**3*x3*x5
dpdx1(433) = x2**3*x4**3*x6+3*x1**2*x2*x4**3*x5+3*x1**2*x2**3*x3*&
&x4
dpdx1(434) = 4*x1**3*x3*x5*x6**2
dpdx1(435) = 4*x1**3*x3*x5**2*x6+4*x1**3*x3**2*x5*x6
dpdx1(436) = 4*x1**3*x2*x5*x6**2+4*x1**3*x3*x4*x6**2+x2**4*x5**2*&
&x6+x3**2*x4**4*x6
dpdx1(437) = x2**4*x5*x6**2+x3*x4**4*x6**2+4*x1**3*x2*x5**2*x6+4*&
&x1**3*x3**2*x4*x6
dpdx1(438) = 4*x1**3*x2*x4*x6**2+x2**4*x4*x5**2+x2*x3**2*x4**4
dpdx1(439) = 4*x1**3*x4*x5*x6**2+4*x1**3*x2*x3*x6**2+x2**4*x3*x5*&
&*2+x3**2*x4**4*x5
dpdx1(440) = x4**4*x5*x6**2+x2**4*x3*x6**2+4*x1**3*x2*x3*x5**2+4*&
&x1**3*x3**2*x4*x5
dpdx1(441) = 4*x1**3*x4*x5**2*x6+4*x1**3*x2*x3**2*x6+x3*x4**4*x5*&
&*2+x2**4*x3**2*x5
dpdx1(442) = x4**4*x5**2*x6+x2**4*x3**2*x6+4*x1**3*x3*x4*x5**2+4*&
&x1**3*x2*x3**2*x5
dpdx1(443) = x2*x4**4*x6**2+x2**4*x4*x6**2+x2*x4**4*x5**2+4*x1**3&
&*x2*x4*x5**2+x2**4*x3**2*x4+4*x1**3*x2*x3**2*x4
dpdx1(444) = 2*x1*x2**4*x5*x6+4*x1**3*x2**2*x5*x6+2*x1*x3*x4**4*x&
&6+4*x1**3*x3*x4**2*x6
dpdx1(445) = 4*x1**3*x2*x4**2*x6+4*x1**3*x2**2*x4*x6+x2**4*x4**2*&
&x5+2*x1*x2**4*x4*x5+x2**2*x3*x4**4+2*x1*x2*x3*x4**4
dpdx1(446) = 2*x1*x2*x4**4*x6+2*x1*x2**4*x4*x6+x2**2*x4**4*x5+4*x&
&1**3*x2**2*x4*x5+x2**4*x3*x4**2+4*x1**3*x2*x3*x4**2
dpdx1(447) = 4*x1**3*x4**2*x5*x6+4*x1**3*x2**2*x3*x6+2*x1*x3*x4**&
&4*x5+2*x1*x2**4*x3*x5
dpdx1(448) = 2*x1*x4**4*x5*x6+2*x1*x2**4*x3*x6+4*x1**3*x3*x4**2*x&
&5+4*x1**3*x2**2*x3*x5
dpdx1(449) = x2**2*x4**4*x6+x2**4*x4**2*x6+2*x1*x2*x4**4*x5+4*x1*&
&*3*x2*x4**2*x5+2*x1*x2**4*x3*x4+4*x1**3*x2**2*x3*x4
dpdx1(450) = 5*x1**4*x3*x5*x6
dpdx1(451) = x2**5*x5*x6+5*x1**4*x2*x5*x6+x3*x4**5*x6+5*x1**4*x3*&
&x4*x6
dpdx1(452) = 5*x1**4*x2*x4*x6+x2**5*x4*x5+x2*x3*x4**5
dpdx1(453) = 5*x1**4*x4*x5*x6+5*x1**4*x2*x3*x6+x3*x4**5*x5+x2**5*&
&x3*x5
dpdx1(454) = x4**5*x5*x6+x2**5*x3*x6+5*x1**4*x3*x4*x5+5*x1**4*x2*&
&x3*x5
dpdx1(455) = x2*x4**5*x6+x2**5*x4*x6+x2*x4**5*x5+5*x1**4*x2*x4*x5&
&+x2**5*x3*x4+5*x1**4*x2*x3*x4
dpdx1(456) = 0
dpdx1(457) = 0
dpdx1(458) = 0
dpdx1(459) = 0
dpdx1(460) = 0
dpdx1(461) = x5*x6**6+x3*x6**6
dpdx1(462) = x5**2*x6**5+x3**2*x6**5
dpdx1(463) = x5**3*x6**4+x3**3*x6**4
dpdx1(464) = x5**4*x6**3+x3**4*x6**3
dpdx1(465) = x3**3*x5**4+x3**4*x5**3
dpdx1(466) = x5**5*x6**2+x3**5*x6**2
dpdx1(467) = x3**2*x5**5+x3**5*x5**2
dpdx1(468) = x5**6*x6+x3**6*x6
dpdx1(469) = x3*x5**6+x3**6*x5
dpdx1(470) = x4*x6**6+x2*x6**6+x2*x5**6+x3**6*x4
dpdx1(471) = x4*x5**6+x2*x3**6
dpdx1(472) = 2*x1*x5*x6**5+2*x1*x3*x6**5
dpdx1(473) = 2*x1*x5**2*x6**4+2*x1*x3**2*x6**4
dpdx1(474) = 2*x1*x5**3*x6**3+2*x1*x3**3*x6**3
dpdx1(475) = 2*x1*x3**3*x5**3
dpdx1(476) = 2*x1*x5**4*x6**2+2*x1*x3**4*x6**2
dpdx1(477) = 2*x1*x3**2*x5**4+2*x1*x3**4*x5**2
dpdx1(478) = 2*x1*x5**5*x6+2*x1*x3**5*x6
dpdx1(479) = 2*x1*x3*x5**5+2*x1*x3**5*x5
dpdx1(480) = 2*x1*x4*x6**5+2*x1*x2*x6**5+x2**2*x5**5+x3**5*x4**2
dpdx1(481) = x4**2*x6**5+x2**2*x6**5+2*x1*x2*x5**5+2*x1*x3**5*x4
dpdx1(482) = x4**2*x5**5+2*x1*x4*x5**5+x2**2*x3**5+2*x1*x2*x3**5
dpdx1(483) = 2*x1*x4**2*x6**4+2*x1*x2**2*x6**4+2*x1*x2**2*x5**4+2&
&*x1*x3**4*x4**2
dpdx1(484) = 2*x1*x4**2*x5**4+2*x1*x2**2*x3**4
dpdx1(485) = 3*x1**2*x5*x6**4+3*x1**2*x3*x6**4
dpdx1(486) = 3*x1**2*x5**2*x6**3+3*x1**2*x3**2*x6**3
dpdx1(487) = 3*x1**2*x5**3*x6**2+3*x1**2*x3**3*x6**2
dpdx1(488) = 3*x1**2*x3**2*x5**3+3*x1**2*x3**3*x5**2
dpdx1(489) = 3*x1**2*x5**4*x6+3*x1**2*x3**4*x6
dpdx1(490) = 3*x1**2*x3*x5**4+3*x1**2*x3**4*x5
dpdx1(491) = 3*x1**2*x4*x6**4+3*x1**2*x2*x6**4+x2**3*x5**4+x3**4*&
&x4**3
dpdx1(492) = x4**3*x6**4+x2**3*x6**4+3*x1**2*x2*x5**4+3*x1**2*x3*&
&*4*x4
dpdx1(493) = x4**3*x5**4+3*x1**2*x4*x5**4+x2**3*x3**4+3*x1**2*x2*&
&x3**4
dpdx1(494) = 3*x1**2*x4**2*x6**3+3*x1**2*x2**2*x6**3+2*x1*x2**3*x&
&5**3+2*x1*x3**3*x4**3
dpdx1(495) = 2*x1*x4**3*x6**3+2*x1*x2**3*x6**3+3*x1**2*x2**2*x5**&
&3+3*x1**2*x3**3*x4**2
dpdx1(496) = 2*x1*x4**3*x5**3+3*x1**2*x4**2*x5**3+2*x1*x2**3*x3**&
&3+3*x1**2*x2**2*x3**3
dpdx1(497) = 3*x1**2*x4**3*x6**2+3*x1**2*x2**3*x6**2+3*x1**2*x2**&
&3*x5**2+3*x1**2*x3**2*x4**3
dpdx1(498) = 2*x1*x2**3*x4**3+3*x1**2*x2**2*x4**3+3*x1**2*x2**3*x&
&4**2
dpdx1(499) = 3*x1**2*x4**3*x5**2+3*x1**2*x2**3*x3**2
dpdx1(500) = 4*x1**3*x5*x6**3+4*x1**3*x3*x6**3
dpdx1(501) = 4*x1**3*x5**2*x6**2+4*x1**3*x3**2*x6**2
dpdx1(502) = 4*x1**3*x3**2*x5**2
dpdx1(503) = 4*x1**3*x5**3*x6+4*x1**3*x3**3*x6
dpdx1(504) = 4*x1**3*x3*x5**3+4*x1**3*x3**3*x5
dpdx1(505) = 4*x1**3*x4*x6**3+4*x1**3*x2*x6**3+x2**4*x5**3+x3**3*&
&x4**4
dpdx1(506) = x4**4*x6**3+x2**4*x6**3+4*x1**3*x2*x5**3+4*x1**3*x3*&
&*3*x4
dpdx1(507) = x4**4*x5**3+4*x1**3*x4*x5**3+x2**4*x3**3+4*x1**3*x2*&
&x3**3
dpdx1(508) = 4*x1**3*x4**2*x6**2+4*x1**3*x2**2*x6**2+2*x1*x2**4*x&
&5**2+2*x1*x3**2*x4**4
dpdx1(509) = 2*x1*x4**4*x6**2+2*x1*x2**4*x6**2+4*x1**3*x2**2*x5**&
&2+4*x1**3*x3**2*x4**2
dpdx1(510) = 2*x1*x2**2*x4**4+2*x1*x2**4*x4**2+4*x1**3*x2**2*x4**&
&2
dpdx1(511) = 2*x1*x4**4*x5**2+4*x1**3*x4**2*x5**2+2*x1*x2**4*x3**&
&2+4*x1**3*x2**2*x3**2
dpdx1(512) = 4*x1**3*x4**3*x6+4*x1**3*x2**3*x6+3*x1**2*x2**4*x5+3&
&*x1**2*x3*x4**4
dpdx1(513) = 3*x1**2*x4**4*x6+3*x1**2*x2**4*x6+4*x1**3*x2**3*x5+4&
&*x1**3*x3*x4**3
dpdx1(514) = x2**3*x4**4+3*x1**2*x2*x4**4+x2**4*x4**3+4*x1**3*x2*&
&x4**3+3*x1**2*x2**4*x4+4*x1**3*x2**3*x4
dpdx1(515) = 3*x1**2*x4**4*x5+4*x1**3*x4**3*x5+3*x1**2*x2**4*x3+4&
&*x1**3*x2**3*x3
dpdx1(516) = 5*x1**4*x5*x6**2+5*x1**4*x3*x6**2
dpdx1(517) = 5*x1**4*x5**2*x6+5*x1**4*x3**2*x6
dpdx1(518) = 5*x1**4*x3*x5**2+5*x1**4*x3**2*x5
dpdx1(519) = 5*x1**4*x4*x6**2+5*x1**4*x2*x6**2+x2**5*x5**2+x3**2*&
&x4**5
dpdx1(520) = x4**5*x6**2+x2**5*x6**2+5*x1**4*x2*x5**2+5*x1**4*x3*&
&*2*x4
dpdx1(521) = x4**5*x5**2+5*x1**4*x4*x5**2+x2**5*x3**2+5*x1**4*x2*&
&x3**2
dpdx1(522) = 5*x1**4*x4**2*x6+5*x1**4*x2**2*x6+2*x1*x2**5*x5+2*x1&
&*x3*x4**5
dpdx1(523) = 2*x1*x4**5*x6+2*x1*x2**5*x6+5*x1**4*x2**2*x5+5*x1**4&
&*x3*x4**2
dpdx1(524) = x2**2*x4**5+2*x1*x2*x4**5+x2**5*x4**2+5*x1**4*x2*x4*&
&*2+2*x1*x2**5*x4+5*x1**4*x2**2*x4
dpdx1(525) = 2*x1*x4**5*x5+5*x1**4*x4**2*x5+2*x1*x2**5*x3+5*x1**4&
&*x2**2*x3
dpdx1(526) = 6*x1**5*x5*x6+6*x1**5*x3*x6
dpdx1(527) = 6*x1**5*x3*x5
dpdx1(528) = 6*x1**5*x4*x6+6*x1**5*x2*x6+x2**6*x5+x3*x4**6
dpdx1(529) = x4**6*x6+x2**6*x6+6*x1**5*x2*x5+6*x1**5*x3*x4
dpdx1(530) = x2*x4**6+x2**6*x4+6*x1**5*x2*x4
dpdx1(531) = x4**6*x5+6*x1**5*x4*x5+x2**6*x3+6*x1**5*x2*x3
dpdx1(532) = 0
dpdx1(533) = 0
dpdx1(534) = 0
dpdx1(535) = 0
dpdx1(536) = x5**7+x3**7
dpdx1(537) = 2*x1*x5**6+2*x1*x3**6
dpdx1(538) = 3*x1**2*x5**5+3*x1**2*x3**5
dpdx1(539) = 4*x1**3*x5**4+4*x1**3*x3**4
dpdx1(540) = 4*x1**3*x4**4+4*x1**3*x2**4
dpdx1(541) = 5*x1**4*x5**3+5*x1**4*x3**3
dpdx1(542) = 3*x1**2*x4**5+5*x1**4*x4**3+3*x1**2*x2**5+5*x1**4*x2&
&**3
dpdx1(543) = 6*x1**5*x5**2+6*x1**5*x3**2
dpdx1(544) = 2*x1*x4**6+6*x1**5*x4**2+2*x1*x2**6+6*x1**5*x2**2
dpdx1(545) = 7*x1**6*x5+7*x1**6*x3
dpdx1(546) = x4**7+7*x1**6*x4+x2**7+7*x1**6*x2
dpdx2(1) = 0
dpdx2(2) = 0
dpdx2(3) = x6+x3
dpdx2(4) = x4+x1
dpdx2(5) = 0
dpdx2(6) = x5*x6+x3*x5
dpdx2(7) = x3*x6
dpdx2(8) = x1*x6+x4*x5+x1*x5+x3*x4
dpdx2(9) = x1*x4
dpdx2(10) = x4*x6+x1*x3
dpdx2(11) = 0
dpdx2(12) = x6**2+x3**2
dpdx2(13) = 2*x2*x6+2*x2*x3
dpdx2(14) = x4**2+2*x2*x4+2*x1*x2+x1**2
dpdx2(15) = x3*x5*x6
dpdx2(16) = x1*x5*x6+x3*x4*x5
dpdx2(17) = x4*x5*x6+x3*x4*x6+x1*x3*x6+x1*x3*x5
dpdx2(18) = x1*x4*x6+x1*x4*x5+x1*x3*x4
dpdx2(19) = 0
dpdx2(20) = x5**2*x6+x3*x5**2
dpdx2(21) = x5*x6**2+x3**2*x5
dpdx2(22) = x3*x6**2+x3**2*x6
dpdx2(23) = x1*x6**2+x4*x5**2+x1*x5**2+x3**2*x4
dpdx2(24) = x4*x6**2+x1*x3**2
dpdx2(25) = 2*x2*x5*x6+2*x2*x3*x5
dpdx2(26) = 2*x2*x3*x6
dpdx2(27) = x1**2*x6+2*x2*x4*x5+2*x1*x2*x5+x3*x4**2
dpdx2(28) = 2*x1*x2*x6+x4**2*x5+x1**2*x5+2*x2*x3*x4
dpdx2(29) = x1*x4**2+2*x1*x2*x4+x1**2*x4
dpdx2(30) = x4**2*x6+2*x2*x4*x6+2*x1*x2*x3+x1**2*x3
dpdx2(31) = 0
dpdx2(32) = 0
dpdx2(33) = x6**3+x3**3
dpdx2(34) = 2*x2*x6**2+2*x2*x3**2
dpdx2(35) = 2*x2*x4**2+2*x1**2*x2
dpdx2(36) = 3*x2**2*x6+3*x2**2*x3
dpdx2(37) = x4**3+3*x2**2*x4+3*x1*x2**2+x1**3
dpdx2(38) = x3*x4*x5*x6+x1*x3*x5*x6
dpdx2(39) = x1*x4*x5*x6+x1*x3*x4*x6+x1*x3*x4*x5
dpdx2(40) = x3*x5**2*x6
dpdx2(41) = x3*x5*x6**2+x3**2*x5*x6
dpdx2(42) = x1*x5*x6**2+x1*x5**2*x6+x3*x4*x5**2+x3**2*x4*x5
dpdx2(43) = x1*x3*x6**2+x4*x5**2*x6+x3**2*x4*x6+x1*x3*x5**2
dpdx2(44) = x4*x5*x6**2+x3*x4*x6**2+x1*x3**2*x6+x1*x3**2*x5
dpdx2(45) = x1*x4*x6**2+x1*x4*x5**2+x1*x3**2*x4
dpdx2(46) = 2*x2*x3*x5*x6
dpdx2(47) = 2*x1*x2*x5*x6+x1**2*x5*x6+x3*x4**2*x5+2*x2*x3*x4*x5
dpdx2(48) = x1**2*x4*x6+2*x1*x2*x4*x5+x1*x3*x4**2
dpdx2(49) = 2*x2*x4*x5*x6+x3*x4**2*x6+x1**2*x3*x6+2*x1*x2*x3*x5
dpdx2(50) = x4**2*x5*x6+2*x2*x3*x4*x6+2*x1*x2*x3*x6+x1**2*x3*x5
dpdx2(51) = x1*x4**2*x6+2*x1*x2*x4*x6+x1*x4**2*x5+x1**2*x4*x5+2*x&
&1*x2*x3*x4+x1**2*x3*x4
dpdx2(52) = 0
dpdx2(53) = 0
dpdx2(54) = x5**3*x6+x3*x5**3
dpdx2(55) = x5**2*x6**2+x3**2*x5**2
dpdx2(56) = x3**2*x6**2
dpdx2(57) = x5*x6**3+x3**3*x5
dpdx2(58) = x3*x6**3+x3**3*x6
dpdx2(59) = x1*x6**3+x4*x5**3+x1*x5**3+x3**3*x4
dpdx2(60) = x4*x6**3+x1*x3**3
dpdx2(61) = 2*x2*x5**2*x6+2*x2*x3*x5**2
dpdx2(62) = 2*x2*x5*x6**2+2*x2*x3**2*x5
dpdx2(63) = 2*x2*x3*x6**2+2*x2*x3**2*x6
dpdx2(64) = x1**2*x6**2+2*x2*x4*x5**2+2*x1*x2*x5**2+x3**2*x4**2
dpdx2(65) = 2*x1*x2*x6**2+x4**2*x5**2+x1**2*x5**2+2*x2*x3**2*x4
dpdx2(66) = x4**2*x6**2+2*x2*x4*x6**2+2*x1*x2*x3**2+x1**2*x3**2
dpdx2(67) = 2*x1**2*x2*x6+2*x2*x4**2*x5+2*x1**2*x2*x5+2*x2*x3*x4*&
&*2
dpdx2(68) = 2*x1*x2*x4**2+x1**2*x4**2+2*x1**2*x2*x4
dpdx2(69) = 2*x2*x4**2*x6+2*x1**2*x2*x3
dpdx2(70) = 3*x2**2*x5*x6+3*x2**2*x3*x5
dpdx2(71) = 3*x2**2*x3*x6
dpdx2(72) = x1**3*x6+3*x2**2*x4*x5+3*x1*x2**2*x5+x3*x4**3
dpdx2(73) = 3*x1*x2**2*x6+x4**3*x5+x1**3*x5+3*x2**2*x3*x4
dpdx2(74) = x1*x4**3+3*x1*x2**2*x4+x1**3*x4
dpdx2(75) = x4**3*x6+3*x2**2*x4*x6+3*x1*x2**2*x3+x1**3*x3
dpdx2(76) = 0
dpdx2(77) = 0
dpdx2(78) = x6**4+x3**4
dpdx2(79) = 2*x2*x6**3+2*x2*x3**3
dpdx2(80) = 3*x2**2*x6**2+3*x2**2*x3**2
dpdx2(81) = 2*x2*x4**3+3*x2**2*x4**2+3*x1**2*x2**2+2*x1**3*x2
dpdx2(82) = 4*x2**3*x6+4*x2**3*x3
dpdx2(83) = x4**4+4*x2**3*x4+4*x1*x2**3+x1**4
dpdx2(84) = x1*x3*x4*x5*x6
dpdx2(85) = x1*x3*x5*x6**2+x3*x4*x5**2*x6+x1*x3*x5**2*x6+x3**2*x4&
&*x5*x6
dpdx2(86) = x3*x4*x5*x6**2+x1*x3**2*x5*x6
dpdx2(87) = x1*x4*x5*x6**2+x1*x3*x4*x6**2+x1*x4*x5**2*x6+x1*x3**2&
&*x4*x6+x1*x3*x4*x5**2+x1*x3**2*x4*x5
dpdx2(88) = x3*x4**2*x5*x6+2*x2*x3*x4*x5*x6+2*x1*x2*x3*x5*x6+x1**&
&2*x3*x5*x6
dpdx2(89) = 2*x1*x2*x4*x5*x6+x1**2*x4*x5*x6+x1*x3*x4**2*x6+x1**2*&
&x3*x4*x6+x1*x3*x4**2*x5+2*x1*x2*x3*x4*x5
dpdx2(90) = x1*x4**2*x5*x6+2*x1*x2*x3*x4*x6+x1**2*x3*x4*x5
dpdx2(91) = x3*x5**3*x6
dpdx2(92) = x3*x5**2*x6**2+x3**2*x5**2*x6
dpdx2(93) = x3**2*x5*x6**2
dpdx2(94) = x3*x5*x6**3+x3**3*x5*x6
dpdx2(95) = x1*x5**2*x6**2+x3**2*x4*x5**2
dpdx2(96) = x1*x5*x6**3+x1*x5**3*x6+x3*x4*x5**3+x3**3*x4*x5
dpdx2(97) = x1*x3*x6**3+x4*x5**3*x6+x3**3*x4*x6+x1*x3*x5**3
dpdx2(98) = x4*x5**2*x6**2+x3**2*x4*x6**2+x1*x3**2*x6**2+x1*x3**2&
&*x5**2
dpdx2(99) = x4*x5*x6**3+x3*x4*x6**3+x1*x3**3*x6+x1*x3**3*x5
dpdx2(100) = x1*x4*x6**3+x1*x4*x5**3+x1*x3**3*x4
dpdx2(101) = 2*x2*x3*x5**2*x6
dpdx2(102) = 2*x2*x3*x5*x6**2+2*x2*x3**2*x5*x6
dpdx2(103) = x1**2*x5*x6**2+2*x1*x2*x5**2*x6+2*x2*x3*x4*x5**2+x3*&
&*2*x4**2*x5
dpdx2(104) = 2*x1*x2*x5*x6**2+x1**2*x5**2*x6+x3*x4**2*x5**2+2*x2*&
&x3**2*x4*x5
dpdx2(105) = x1**2*x4*x6**2+2*x1*x2*x4*x5**2+x1*x3**2*x4**2
dpdx2(106) = x1**2*x3*x6**2+2*x2*x4*x5**2*x6+x3**2*x4**2*x6+2*x1*&
&x2*x3*x5**2
dpdx2(107) = 2*x1*x2*x3*x6**2+x4**2*x5**2*x6+2*x2*x3**2*x4*x6+x1*&
&*2*x3*x5**2
dpdx2(108) = 2*x2*x4*x5*x6**2+x3*x4**2*x6**2+x1**2*x3**2*x6+2*x1*&
&x2*x3**2*x5
dpdx2(109) = x4**2*x5*x6**2+2*x2*x3*x4*x6**2+2*x1*x2*x3**2*x6+x1*&
&*2*x3**2*x5
dpdx2(110) = x1*x4**2*x6**2+2*x1*x2*x4*x6**2+x1*x4**2*x5**2+x1**2&
&*x4*x5**2+2*x1*x2*x3**2*x4+x1**2*x3**2*x4
dpdx2(111) = 2*x1**2*x2*x5*x6+2*x2*x3*x4**2*x5
dpdx2(112) = x1**2*x4**2*x6+2*x1**2*x2*x4*x6+2*x1*x2*x4**2*x5+2*x&
&1**2*x2*x4*x5+2*x1*x2*x3*x4**2+x1**2*x3*x4**2
dpdx2(113) = 2*x2*x4**2*x5*x6+2*x2*x3*x4**2*x6+2*x1**2*x2*x3*x6+2&
&*x1**2*x2*x3*x5
dpdx2(114) = 2*x1*x2*x4**2*x6+x1**2*x4**2*x5+2*x1**2*x2*x3*x4
dpdx2(115) = 3*x2**2*x3*x5*x6
dpdx2(116) = 3*x1*x2**2*x5*x6+x1**3*x5*x6+x3*x4**3*x5+3*x2**2*x3*&
&x4*x5
dpdx2(117) = x1**3*x4*x6+3*x1*x2**2*x4*x5+x1*x3*x4**3
dpdx2(118) = 3*x2**2*x4*x5*x6+x3*x4**3*x6+x1**3*x3*x6+3*x1*x2**2*&
&x3*x5
dpdx2(119) = x4**3*x5*x6+3*x2**2*x3*x4*x6+3*x1*x2**2*x3*x6+x1**3*&
&x3*x5
dpdx2(120) = x1*x4**3*x6+3*x1*x2**2*x4*x6+x1*x4**3*x5+x1**3*x4*x5&
&+3*x1*x2**2*x3*x4+x1**3*x3*x4
dpdx2(121) = 0
dpdx2(122) = 0
dpdx2(123) = 0
dpdx2(124) = x5**4*x6+x3*x5**4
dpdx2(125) = x5**3*x6**2+x3**2*x5**3
dpdx2(126) = x5**2*x6**3+x3**3*x5**2
dpdx2(127) = x3**2*x6**3+x3**3*x6**2
dpdx2(128) = x5*x6**4+x3**4*x5
dpdx2(129) = x3*x6**4+x3**4*x6
dpdx2(130) = x1*x6**4+x4*x5**4+x1*x5**4+x3**4*x4
dpdx2(131) = x4*x6**4+x1*x3**4
dpdx2(132) = 2*x2*x5**3*x6+2*x2*x3*x5**3
dpdx2(133) = 2*x2*x5**2*x6**2+2*x2*x3**2*x5**2
dpdx2(134) = 2*x2*x3**2*x6**2
dpdx2(135) = 2*x2*x5*x6**3+2*x2*x3**3*x5
dpdx2(136) = 2*x2*x3*x6**3+2*x2*x3**3*x6
dpdx2(137) = x1**2*x6**3+2*x2*x4*x5**3+2*x1*x2*x5**3+x3**3*x4**2
dpdx2(138) = 2*x1*x2*x6**3+x4**2*x5**3+x1**2*x5**3+2*x2*x3**3*x4
dpdx2(139) = x4**2*x6**3+2*x2*x4*x6**3+2*x1*x2*x3**3+x1**2*x3**3
dpdx2(140) = 2*x1**2*x2*x6**2+2*x2*x4**2*x5**2+2*x1**2*x2*x5**2+2&
&*x2*x3**2*x4**2
dpdx2(141) = 2*x1**2*x2*x4**2
dpdx2(142) = 2*x2*x4**2*x6**2+2*x1**2*x2*x3**2
dpdx2(143) = 3*x2**2*x5**2*x6+3*x2**2*x3*x5**2
dpdx2(144) = 3*x2**2*x5*x6**2+3*x2**2*x3**2*x5
dpdx2(145) = 3*x2**2*x3*x6**2+3*x2**2*x3**2*x6
dpdx2(146) = x1**3*x6**2+3*x2**2*x4*x5**2+3*x1*x2**2*x5**2+x3**2*&
&x4**3
dpdx2(147) = 3*x1*x2**2*x6**2+x4**3*x5**2+x1**3*x5**2+3*x2**2*x3*&
&*2*x4
dpdx2(148) = x4**3*x6**2+3*x2**2*x4*x6**2+3*x1*x2**2*x3**2+x1**3*&
&x3**2
dpdx2(149) = 2*x1**3*x2*x6+3*x2**2*x4**2*x5+3*x1**2*x2**2*x5+2*x2&
&*x3*x4**3
dpdx2(150) = 3*x1**2*x2**2*x6+2*x2*x4**3*x5+2*x1**3*x2*x5+3*x2**2&
&*x3*x4**2
dpdx2(151) = 2*x1*x2*x4**3+x1**2*x4**3+3*x1*x2**2*x4**2+x1**3*x4*&
&*2+3*x1**2*x2**2*x4+2*x1**3*x2*x4
dpdx2(152) = 2*x2*x4**3*x6+3*x2**2*x4**2*x6+3*x1**2*x2**2*x3+2*x1&
&**3*x2*x3
dpdx2(153) = 4*x2**3*x5*x6+4*x2**3*x3*x5
dpdx2(154) = 4*x2**3*x3*x6
dpdx2(155) = x1**4*x6+4*x2**3*x4*x5+4*x1*x2**3*x5+x3*x4**4
dpdx2(156) = 4*x1*x2**3*x6+x4**4*x5+x1**4*x5+4*x2**3*x3*x4
dpdx2(157) = x1*x4**4+4*x1*x2**3*x4+x1**4*x4
dpdx2(158) = x4**4*x6+4*x2**3*x4*x6+4*x1*x2**3*x3+x1**4*x3
dpdx2(159) = 0
dpdx2(160) = 0
dpdx2(161) = 0
dpdx2(162) = x6**5+x3**5
dpdx2(163) = 2*x2*x6**4+2*x2*x3**4
dpdx2(164) = 3*x2**2*x6**3+3*x2**2*x3**3
dpdx2(165) = 3*x2**2*x4**3+3*x1**3*x2**2
dpdx2(166) = 4*x2**3*x6**2+4*x2**3*x3**2
dpdx2(167) = 2*x2*x4**4+4*x2**3*x4**2+4*x1**2*x2**3+2*x1**4*x2
dpdx2(168) = 5*x2**4*x6+5*x2**4*x3
dpdx2(169) = x4**5+5*x2**4*x4+5*x1*x2**4+x1**5
dpdx2(170) = x1*x3*x4*x5*x6**2+x1*x3*x4*x5**2*x6+x1*x3**2*x4*x5*x&
&6
dpdx2(171) = x1*x3*x4**2*x5*x6+2*x1*x2*x3*x4*x5*x6+x1**2*x3*x4*x5&
&*x6
dpdx2(172) = x1*x3*x5**2*x6**2+x3**2*x4*x5**2*x6
dpdx2(173) = x1*x3*x5*x6**3+x3*x4*x5**3*x6+x1*x3*x5**3*x6+x3**3*x&
&4*x5*x6
dpdx2(174) = x3*x4*x5**2*x6**2+x3**2*x4*x5*x6**2+x1*x3**2*x5*x6**&
&2+x1*x3**2*x5**2*x6
dpdx2(175) = x1*x4*x5**2*x6**2+x1*x3**2*x4*x6**2+x1*x3**2*x4*x5**&
&2
dpdx2(176) = x3*x4*x5*x6**3+x1*x3**3*x5*x6
dpdx2(177) = x1*x4*x5*x6**3+x1*x3*x4*x6**3+x1*x4*x5**3*x6+x1*x3**&
&3*x4*x6+x1*x3*x4*x5**3+x1*x3**3*x4*x5
dpdx2(178) = x1**2*x3*x5*x6**2+2*x2*x3*x4*x5**2*x6+2*x1*x2*x3*x5*&
&*2*x6+x3**2*x4**2*x5*x6
dpdx2(179) = 2*x1*x2*x3*x5*x6**2+x3*x4**2*x5**2*x6+x1**2*x3*x5**2&
&*x6+2*x2*x3**2*x4*x5*x6
dpdx2(180) = x1**2*x4*x5*x6**2+x1**2*x3*x4*x6**2+2*x1*x2*x4*x5**2&
&*x6+x1*x3**2*x4**2*x6+2*x1*x2*x3*x4*x5**2+x1*x3**2*x4**2*x5
dpdx2(181) = x3*x4**2*x5*x6**2+2*x2*x3*x4*x5*x6**2+2*x1*x2*x3**2*&
&x5*x6+x1**2*x3**2*x5*x6
dpdx2(182) = 2*x1*x2*x4*x5*x6**2+x1*x3*x4**2*x6**2+x1**2*x4*x5**2&
&*x6+x1**2*x3**2*x4*x6+x1*x3*x4**2*x5**2+2*x1*x2*x3**2*x4*x5
dpdx2(183) = x1*x4**2*x5*x6**2+2*x1*x2*x3*x4*x6**2+x1*x4**2*x5**2&
&*x6+2*x1*x2*x3**2*x4*x6+x1**2*x3*x4*x5**2+x1**2*x3**2*x4*x5
dpdx2(184) = 2*x1**2*x2*x4*x5*x6+x1**2*x3*x4**2*x6+2*x1*x2*x3*x4*&
&*2*x5
dpdx2(185) = 2*x2*x3*x4**2*x5*x6+2*x1**2*x2*x3*x5*x6
dpdx2(186) = 2*x1*x2*x4**2*x5*x6+x1**2*x4**2*x5*x6+2*x1*x2*x3*x4*&
&*2*x6+2*x1**2*x2*x3*x4*x6+x1**2*x3*x4**2*x5+2*x1**2*x2*x3*x4*x5
dpdx2(187) = x3*x4**3*x5*x6+3*x2**2*x3*x4*x5*x6+3*x1*x2**2*x3*x5*&
&x6+x1**3*x3*x5*x6
dpdx2(188) = 3*x1*x2**2*x4*x5*x6+x1**3*x4*x5*x6+x1*x3*x4**3*x6+x1&
&**3*x3*x4*x6+x1*x3*x4**3*x5+3*x1*x2**2*x3*x4*x5
dpdx2(189) = x1*x4**3*x5*x6+3*x1*x2**2*x3*x4*x6+x1**3*x3*x4*x5
dpdx2(190) = x3*x5**4*x6
dpdx2(191) = x3*x5**3*x6**2+x3**2*x5**3*x6
dpdx2(192) = x3**2*x5**2*x6**2
dpdx2(193) = x3*x5**2*x6**3+x3**3*x5**2*x6
dpdx2(194) = x3**2*x5*x6**3+x3**3*x5*x6**2
dpdx2(195) = x3*x5*x6**4+x3**4*x5*x6
dpdx2(196) = x1*x5**2*x6**3+x1*x5**3*x6**2+x3**2*x4*x5**3+x3**3*x&
&4*x5**2
dpdx2(197) = x1*x5*x6**4+x1*x5**4*x6+x3*x4*x5**4+x3**4*x4*x5
dpdx2(198) = x1*x3*x6**4+x4*x5**4*x6+x3**4*x4*x6+x1*x3*x5**4
dpdx2(199) = x1*x3**2*x6**3+x4*x5**3*x6**2+x3**3*x4*x6**2+x1*x3**&
&2*x5**3
dpdx2(200) = x4*x5**2*x6**3+x3**2*x4*x6**3+x1*x3**3*x6**2+x1*x3**&
&3*x5**2
dpdx2(201) = x4*x5*x6**4+x3*x4*x6**4+x1*x3**4*x6+x1*x3**4*x5
dpdx2(202) = x1*x4*x6**4+x1*x4*x5**4+x1*x3**4*x4
dpdx2(203) = 2*x2*x3*x5**3*x6
dpdx2(204) = 2*x2*x3*x5**2*x6**2+2*x2*x3**2*x5**2*x6
dpdx2(205) = 2*x2*x3**2*x5*x6**2
dpdx2(206) = 2*x2*x3*x5*x6**3+2*x2*x3**3*x5*x6
dpdx2(207) = x1**2*x5*x6**3+2*x1*x2*x5**3*x6+2*x2*x3*x4*x5**3+x3*&
&*3*x4**2*x5
dpdx2(208) = 2*x1*x2*x5**2*x6**2+x1**2*x5**2*x6**2+x3**2*x4**2*x5&
&**2+2*x2*x3**2*x4*x5**2
dpdx2(209) = 2*x1*x2*x5*x6**3+x1**2*x5**3*x6+x3*x4**2*x5**3+2*x2*&
&x3**3*x4*x5
dpdx2(210) = x1**2*x4*x6**3+2*x1*x2*x4*x5**3+x1*x3**3*x4**2
dpdx2(211) = x1**2*x3*x6**3+2*x2*x4*x5**3*x6+x3**3*x4**2*x6+2*x1*&
&x2*x3*x5**3
dpdx2(212) = 2*x1*x2*x3*x6**3+x4**2*x5**3*x6+2*x2*x3**3*x4*x6+x1*&
&*2*x3*x5**3
dpdx2(213) = 2*x2*x4*x5**2*x6**2+x3**2*x4**2*x6**2+x1**2*x3**2*x6&
&**2+2*x1*x2*x3**2*x5**2
dpdx2(214) = x4**2*x5**2*x6**2+2*x2*x3**2*x4*x6**2+2*x1*x2*x3**2*&
&x6**2+x1**2*x3**2*x5**2
dpdx2(215) = 2*x2*x4*x5*x6**3+x3*x4**2*x6**3+x1**2*x3**3*x6+2*x1*&
&x2*x3**3*x5
dpdx2(216) = x4**2*x5*x6**3+2*x2*x3*x4*x6**3+2*x1*x2*x3**3*x6+x1*&
&*2*x3**3*x5
dpdx2(217) = x1*x4**2*x6**3+2*x1*x2*x4*x6**3+x1*x4**2*x5**3+x1**2&
&*x4*x5**3+2*x1*x2*x3**3*x4+x1**2*x3**3*x4
dpdx2(218) = 2*x1**2*x2*x5*x6**2+2*x1**2*x2*x5**2*x6+2*x2*x3*x4**&
&2*x5**2+2*x2*x3**2*x4**2*x5
dpdx2(219) = x1**2*x4**2*x6**2+2*x1**2*x2*x4*x6**2+2*x1*x2*x4**2*&
&x5**2+2*x1**2*x2*x4*x5**2+2*x1*x2*x3**2*x4**2+x1**2*x3**2*x4**2
dpdx2(220) = 2*x1**2*x2*x3*x6**2+2*x2*x4**2*x5**2*x6+2*x2*x3**2*x&
&4**2*x6+2*x1**2*x2*x3*x5**2
dpdx2(221) = 2*x1**2*x2*x4**2*x6+2*x1**2*x2*x4**2*x5+2*x1**2*x2*x&
&3*x4**2
dpdx2(222) = 2*x2*x4**2*x5*x6**2+2*x2*x3*x4**2*x6**2+2*x1**2*x2*x&
&3**2*x6+2*x1**2*x2*x3**2*x5
dpdx2(223) = 2*x1*x2*x4**2*x6**2+x1**2*x4**2*x5**2+2*x1**2*x2*x3*&
&*2*x4
dpdx2(224) = 3*x2**2*x3*x5**2*x6
dpdx2(225) = 3*x2**2*x3*x5*x6**2+3*x2**2*x3**2*x5*x6
dpdx2(226) = x1**3*x5*x6**2+3*x1*x2**2*x5**2*x6+3*x2**2*x3*x4*x5*&
&*2+x3**2*x4**3*x5
dpdx2(227) = 3*x1*x2**2*x5*x6**2+x1**3*x5**2*x6+x3*x4**3*x5**2+3*&
&x2**2*x3**2*x4*x5
dpdx2(228) = x1**3*x4*x6**2+3*x1*x2**2*x4*x5**2+x1*x3**2*x4**3
dpdx2(229) = x1**3*x3*x6**2+3*x2**2*x4*x5**2*x6+x3**2*x4**3*x6+3*&
&x1*x2**2*x3*x5**2
dpdx2(230) = 3*x1*x2**2*x3*x6**2+x4**3*x5**2*x6+3*x2**2*x3**2*x4*&
&x6+x1**3*x3*x5**2
dpdx2(231) = 3*x2**2*x4*x5*x6**2+x3*x4**3*x6**2+x1**3*x3**2*x6+3*&
&x1*x2**2*x3**2*x5
dpdx2(232) = x4**3*x5*x6**2+3*x2**2*x3*x4*x6**2+3*x1*x2**2*x3**2*&
&x6+x1**3*x3**2*x5
dpdx2(233) = x1*x4**3*x6**2+3*x1*x2**2*x4*x6**2+x1*x4**3*x5**2+x1&
&**3*x4*x5**2+3*x1*x2**2*x3**2*x4+x1**3*x3**2*x4
dpdx2(234) = 3*x1**2*x2**2*x5*x6+2*x1**3*x2*x5*x6+2*x2*x3*x4**3*x&
&5+3*x2**2*x3*x4**2*x5
dpdx2(235) = x1**3*x4**2*x6+2*x1**3*x2*x4*x6+3*x1*x2**2*x4**2*x5+&
&3*x1**2*x2**2*x4*x5+2*x1*x2*x3*x4**3+x1**2*x3*x4**3
dpdx2(236) = x1**2*x4**3*x6+3*x1**2*x2**2*x4*x6+2*x1*x2*x4**3*x5+&
&2*x1**3*x2*x4*x5+3*x1*x2**2*x3*x4**2+x1**3*x3*x4**2
dpdx2(237) = 3*x2**2*x4**2*x5*x6+2*x2*x3*x4**3*x6+2*x1**3*x2*x3*x&
&6+3*x1**2*x2**2*x3*x5
dpdx2(238) = 2*x2*x4**3*x5*x6+3*x2**2*x3*x4**2*x6+3*x1**2*x2**2*x&
&3*x6+2*x1**3*x2*x3*x5
dpdx2(239) = 2*x1*x2*x4**3*x6+3*x1*x2**2*x4**2*x6+x1**2*x4**3*x5+&
&x1**3*x4**2*x5+3*x1**2*x2**2*x3*x4+2*x1**3*x2*x3*x4
dpdx2(240) = 4*x2**3*x3*x5*x6
dpdx2(241) = 4*x1*x2**3*x5*x6+x1**4*x5*x6+x3*x4**4*x5+4*x2**3*x3*&
&x4*x5
dpdx2(242) = x1**4*x4*x6+4*x1*x2**3*x4*x5+x1*x3*x4**4
dpdx2(243) = 4*x2**3*x4*x5*x6+x3*x4**4*x6+x1**4*x3*x6+4*x1*x2**3*&
&x3*x5
dpdx2(244) = x4**4*x5*x6+4*x2**3*x3*x4*x6+4*x1*x2**3*x3*x6+x1**4*&
&x3*x5
dpdx2(245) = x1*x4**4*x6+4*x1*x2**3*x4*x6+x1*x4**4*x5+x1**4*x4*x5&
&+4*x1*x2**3*x3*x4+x1**4*x3*x4
dpdx2(246) = 0
dpdx2(247) = 0
dpdx2(248) = 0
dpdx2(249) = 0
dpdx2(250) = x5**5*x6+x3*x5**5
dpdx2(251) = x5**4*x6**2+x3**2*x5**4
dpdx2(252) = x5**3*x6**3+x3**3*x5**3
dpdx2(253) = x3**3*x6**3
dpdx2(254) = x5**2*x6**4+x3**4*x5**2
dpdx2(255) = x3**2*x6**4+x3**4*x6**2
dpdx2(256) = x5*x6**5+x3**5*x5
dpdx2(257) = x3*x6**5+x3**5*x6
dpdx2(258) = x1*x6**5+x4*x5**5+x1*x5**5+x3**5*x4
dpdx2(259) = x4*x6**5+x1*x3**5
dpdx2(260) = 2*x2*x5**4*x6+2*x2*x3*x5**4
dpdx2(261) = 2*x2*x5**3*x6**2+2*x2*x3**2*x5**3
dpdx2(262) = 2*x2*x5**2*x6**3+2*x2*x3**3*x5**2
dpdx2(263) = 2*x2*x3**2*x6**3+2*x2*x3**3*x6**2
dpdx2(264) = 2*x2*x5*x6**4+2*x2*x3**4*x5
dpdx2(265) = 2*x2*x3*x6**4+2*x2*x3**4*x6
dpdx2(266) = x1**2*x6**4+2*x2*x4*x5**4+2*x1*x2*x5**4+x3**4*x4**2
dpdx2(267) = 2*x1*x2*x6**4+x4**2*x5**4+x1**2*x5**4+2*x2*x3**4*x4
dpdx2(268) = x4**2*x6**4+2*x2*x4*x6**4+2*x1*x2*x3**4+x1**2*x3**4
dpdx2(269) = 2*x1**2*x2*x6**3+2*x2*x4**2*x5**3+2*x1**2*x2*x5**3+2&
&*x2*x3**3*x4**2
dpdx2(270) = 2*x2*x4**2*x6**3+2*x1**2*x2*x3**3
dpdx2(271) = 3*x2**2*x5**3*x6+3*x2**2*x3*x5**3
dpdx2(272) = 3*x2**2*x5**2*x6**2+3*x2**2*x3**2*x5**2
dpdx2(273) = 3*x2**2*x3**2*x6**2
dpdx2(274) = 3*x2**2*x5*x6**3+3*x2**2*x3**3*x5
dpdx2(275) = 3*x2**2*x3*x6**3+3*x2**2*x3**3*x6
dpdx2(276) = x1**3*x6**3+3*x2**2*x4*x5**3+3*x1*x2**2*x5**3+x3**3*&
&x4**3
dpdx2(277) = 3*x1*x2**2*x6**3+x4**3*x5**3+x1**3*x5**3+3*x2**2*x3*&
&*3*x4
dpdx2(278) = x4**3*x6**3+3*x2**2*x4*x6**3+3*x1*x2**2*x3**3+x1**3*&
&x3**3
dpdx2(279) = 2*x1**3*x2*x6**2+3*x2**2*x4**2*x5**2+3*x1**2*x2**2*x&
&5**2+2*x2*x3**2*x4**3
dpdx2(280) = 3*x1**2*x2**2*x6**2+2*x2*x4**3*x5**2+2*x1**3*x2*x5**&
&2+3*x2**2*x3**2*x4**2
dpdx2(281) = 2*x1**2*x2*x4**3+3*x1**2*x2**2*x4**2+2*x1**3*x2*x4**&
&2
dpdx2(282) = 2*x2*x4**3*x6**2+3*x2**2*x4**2*x6**2+3*x1**2*x2**2*x&
&3**2+2*x1**3*x2*x3**2
dpdx2(283) = 3*x1**3*x2**2*x6+3*x2**2*x4**3*x5+3*x1**3*x2**2*x5+3&
&*x2**2*x3*x4**3
dpdx2(284) = 3*x1*x2**2*x4**3+x1**3*x4**3+3*x1**3*x2**2*x4
dpdx2(285) = 3*x2**2*x4**3*x6+3*x1**3*x2**2*x3
dpdx2(286) = 4*x2**3*x5**2*x6+4*x2**3*x3*x5**2
dpdx2(287) = 4*x2**3*x5*x6**2+4*x2**3*x3**2*x5
dpdx2(288) = 4*x2**3*x3*x6**2+4*x2**3*x3**2*x6
dpdx2(289) = x1**4*x6**2+4*x2**3*x4*x5**2+4*x1*x2**3*x5**2+x3**2*&
&x4**4
dpdx2(290) = 4*x1*x2**3*x6**2+x4**4*x5**2+x1**4*x5**2+4*x2**3*x3*&
&*2*x4
dpdx2(291) = x4**4*x6**2+4*x2**3*x4*x6**2+4*x1*x2**3*x3**2+x1**4*&
&x3**2
dpdx2(292) = 2*x1**4*x2*x6+4*x2**3*x4**2*x5+4*x1**2*x2**3*x5+2*x2&
&*x3*x4**4
dpdx2(293) = 4*x1**2*x2**3*x6+2*x2*x4**4*x5+2*x1**4*x2*x5+4*x2**3&
&*x3*x4**2
dpdx2(294) = 2*x1*x2*x4**4+x1**2*x4**4+4*x1*x2**3*x4**2+x1**4*x4*&
&*2+4*x1**2*x2**3*x4+2*x1**4*x2*x4
dpdx2(295) = 2*x2*x4**4*x6+4*x2**3*x4**2*x6+4*x1**2*x2**3*x3+2*x1&
&**4*x2*x3
dpdx2(296) = 5*x2**4*x5*x6+5*x2**4*x3*x5
dpdx2(297) = 5*x2**4*x3*x6
dpdx2(298) = x1**5*x6+5*x2**4*x4*x5+5*x1*x2**4*x5+x3*x4**5
dpdx2(299) = 5*x1*x2**4*x6+x4**5*x5+x1**5*x5+5*x2**4*x3*x4
dpdx2(300) = x1*x4**5+5*x1*x2**4*x4+x1**5*x4
dpdx2(301) = x4**5*x6+5*x2**4*x4*x6+5*x1*x2**4*x3+x1**5*x3
dpdx2(302) = 0
dpdx2(303) = 0
dpdx2(304) = 0
dpdx2(305) = x6**6+x3**6
dpdx2(306) = 2*x2*x6**5+2*x2*x3**5
dpdx2(307) = 3*x2**2*x6**4+3*x2**2*x3**4
dpdx2(308) = 4*x2**3*x6**3+4*x2**3*x3**3
dpdx2(309) = 3*x2**2*x4**4+4*x2**3*x4**3+4*x1**3*x2**3+3*x1**4*x2&
&**2
dpdx2(310) = 5*x2**4*x6**2+5*x2**4*x3**2
dpdx2(311) = 2*x2*x4**5+5*x2**4*x4**2+5*x1**2*x2**4+2*x1**5*x2
dpdx2(312) = 6*x2**5*x6+6*x2**5*x3
dpdx2(313) = x4**6+6*x2**5*x4+6*x1*x2**5+x1**6
dpdx2(314) = x1*x3*x4*x5**2*x6**2+x1*x3**2*x4*x5*x6**2+x1*x3**2*x&
&4*x5**2*x6
dpdx2(315) = x1*x3*x4*x5*x6**3+x1*x3*x4*x5**3*x6+x1*x3**3*x4*x5*x&
&6
dpdx2(316) = x1**2*x3*x4*x5*x6**2+2*x1*x2*x3*x4*x5**2*x6+x1*x3**2&
&*x4**2*x5*x6
dpdx2(317) = x1*x3*x4**2*x5*x6**2+2*x1*x2*x3*x4*x5*x6**2+x1*x3*x4&
&**2*x5**2*x6+x1**2*x3*x4*x5**2*x6+2*x1*x2*x3**2*x4*x5*x6+x1**2*x3&
&**2*x4*x5*x6
dpdx2(318) = 2*x1*x2*x3*x4**2*x5*x6+x1**2*x3*x4**2*x5*x6+2*x1**2*&
&x2*x3*x4*x5*x6
dpdx2(319) = x1*x3*x4**3*x5*x6+3*x1*x2**2*x3*x4*x5*x6+x1**3*x3*x4&
&*x5*x6
dpdx2(320) = x1*x3*x5**2*x6**3+x1*x3*x5**3*x6**2+x3**2*x4*x5**3*x&
&6+x3**3*x4*x5**2*x6
dpdx2(321) = x1*x3*x5*x6**4+x3*x4*x5**4*x6+x1*x3*x5**4*x6+x3**4*x&
&4*x5*x6
dpdx2(322) = x3**2*x4*x5**2*x6**2+x1*x3**2*x5**2*x6**2
dpdx2(323) = x1*x3**2*x5*x6**3+x3*x4*x5**3*x6**2+x3**3*x4*x5*x6**&
&2+x1*x3**2*x5**3*x6
dpdx2(324) = x3*x4*x5**2*x6**3+x3**2*x4*x5*x6**3+x1*x3**3*x5*x6**&
&2+x1*x3**3*x5**2*x6
dpdx2(325) = x1*x4*x5**2*x6**3+x1*x3**2*x4*x6**3+x1*x4*x5**3*x6**&
&2+x1*x3**3*x4*x6**2+x1*x3**2*x4*x5**3+x1*x3**3*x4*x5**2
dpdx2(326) = x3*x4*x5*x6**4+x1*x3**4*x5*x6
dpdx2(327) = x1*x4*x5*x6**4+x1*x3*x4*x6**4+x1*x4*x5**4*x6+x1*x3**&
&4*x4*x6+x1*x3*x4*x5**4+x1*x3**4*x4*x5
dpdx2(328) = x1**2*x3*x5*x6**3+2*x2*x3*x4*x5**3*x6+2*x1*x2*x3*x5*&
&*3*x6+x3**3*x4**2*x5*x6
dpdx2(329) = 2*x1*x2*x3*x5**2*x6**2+x1**2*x3*x5**2*x6**2+x3**2*x4&
&**2*x5**2*x6+2*x2*x3**2*x4*x5**2*x6
dpdx2(330) = 2*x1*x2*x3*x5*x6**3+x3*x4**2*x5**3*x6+x1**2*x3*x5**3&
&*x6+2*x2*x3**3*x4*x5*x6
dpdx2(331) = x1**2*x4*x5*x6**3+x1**2*x3*x4*x6**3+2*x1*x2*x4*x5**3&
&*x6+x1*x3**3*x4**2*x6+2*x1*x2*x3*x4*x5**3+x1*x3**3*x4**2*x5
dpdx2(332) = 2*x2*x3*x4*x5**2*x6**2+x3**2*x4**2*x5*x6**2+x1**2*x3&
&**2*x5*x6**2+2*x1*x2*x3**2*x5**2*x6
dpdx2(333) = x3*x4**2*x5**2*x6**2+2*x2*x3**2*x4*x5*x6**2+2*x1*x2*&
&x3**2*x5*x6**2+x1**2*x3**2*x5**2*x6
dpdx2(334) = 2*x1*x2*x4*x5**2*x6**2+x1**2*x4*x5**2*x6**2+x1*x3**2&
&*x4**2*x6**2+x1**2*x3**2*x4*x6**2+x1*x3**2*x4**2*x5**2+2*x1*x2*x3&
&**2*x4*x5**2
dpdx2(335) = x1*x4**2*x5**2*x6**2+2*x1*x2*x3**2*x4*x6**2+x1**2*x3&
&**2*x4*x5**2
dpdx2(336) = x3*x4**2*x5*x6**3+2*x2*x3*x4*x5*x6**3+2*x1*x2*x3**3*&
&x5*x6+x1**2*x3**3*x5*x6
dpdx2(337) = 2*x1*x2*x4*x5*x6**3+x1*x3*x4**2*x6**3+x1**2*x4*x5**3&
&*x6+x1**2*x3**3*x4*x6+x1*x3*x4**2*x5**3+2*x1*x2*x3**3*x4*x5
dpdx2(338) = x1*x4**2*x5*x6**3+2*x1*x2*x3*x4*x6**3+x1*x4**2*x5**3&
&*x6+2*x1*x2*x3**3*x4*x6+x1**2*x3*x4*x5**3+x1**2*x3**3*x4*x5
dpdx2(339) = 2*x1**2*x2*x4*x5*x6**2+x1**2*x3*x4**2*x6**2+2*x1**2*&
&x2*x4*x5**2*x6+x1**2*x3**2*x4**2*x6+2*x1*x2*x3*x4**2*x5**2+2*x1*x&
&2*x3**2*x4**2*x5
dpdx2(340) = 2*x1**2*x2*x3*x5*x6**2+2*x2*x3*x4**2*x5**2*x6+2*x1**&
&2*x2*x3*x5**2*x6+2*x2*x3**2*x4**2*x5*x6
dpdx2(341) = x1**2*x4**2*x5*x6**2+2*x1**2*x2*x3*x4*x6**2+2*x1*x2*&
&x4**2*x5**2*x6+2*x1*x2*x3**2*x4**2*x6+2*x1**2*x2*x3*x4*x5**2+x1**&
&2*x3**2*x4**2*x5
dpdx2(342) = 2*x1**2*x2*x4**2*x5*x6+2*x1**2*x2*x3*x4**2*x6+2*x1**&
&2*x2*x3*x4**2*x5
dpdx2(343) = 2*x2*x3*x4**2*x5*x6**2+2*x1**2*x2*x3**2*x5*x6
dpdx2(344) = 2*x1*x2*x4**2*x5*x6**2+2*x1*x2*x3*x4**2*x6**2+x1**2*&
&x4**2*x5**2*x6+2*x1**2*x2*x3**2*x4*x6+x1**2*x3*x4**2*x5**2+2*x1**&
&2*x2*x3**2*x4*x5
dpdx2(345) = x1**3*x3*x5*x6**2+3*x2**2*x3*x4*x5**2*x6+3*x1*x2**2*&
&x3*x5**2*x6+x3**2*x4**3*x5*x6
dpdx2(346) = 3*x1*x2**2*x3*x5*x6**2+x3*x4**3*x5**2*x6+x1**3*x3*x5&
&**2*x6+3*x2**2*x3**2*x4*x5*x6
dpdx2(347) = x1**3*x4*x5*x6**2+x1**3*x3*x4*x6**2+3*x1*x2**2*x4*x5&
&**2*x6+x1*x3**2*x4**3*x6+3*x1*x2**2*x3*x4*x5**2+x1*x3**2*x4**3*x5
dpdx2(348) = x3*x4**3*x5*x6**2+3*x2**2*x3*x4*x5*x6**2+3*x1*x2**2*&
&x3**2*x5*x6+x1**3*x3**2*x5*x6
dpdx2(349) = 3*x1*x2**2*x4*x5*x6**2+x1*x3*x4**3*x6**2+x1**3*x4*x5&
&**2*x6+x1**3*x3**2*x4*x6+x1*x3*x4**3*x5**2+3*x1*x2**2*x3**2*x4*x5
dpdx2(350) = x1*x4**3*x5*x6**2+3*x1*x2**2*x3*x4*x6**2+x1*x4**3*x5&
&**2*x6+3*x1*x2**2*x3**2*x4*x6+x1**3*x3*x4*x5**2+x1**3*x3**2*x4*x5
dpdx2(351) = 3*x1**2*x2**2*x4*x5*x6+2*x1**3*x2*x4*x5*x6+x1**2*x3*&
&x4**3*x6+x1**3*x3*x4**2*x6+2*x1*x2*x3*x4**3*x5+3*x1*x2**2*x3*x4**&
&2*x5
dpdx2(352) = 2*x2*x3*x4**3*x5*x6+3*x2**2*x3*x4**2*x5*x6+3*x1**2*x&
&2**2*x3*x5*x6+2*x1**3*x2*x3*x5*x6
dpdx2(353) = 3*x1*x2**2*x4**2*x5*x6+x1**3*x4**2*x5*x6+2*x1*x2*x3*&
&x4**3*x6+2*x1**3*x2*x3*x4*x6+x1**2*x3*x4**3*x5+3*x1**2*x2**2*x3*x&
&4*x5
dpdx2(354) = 2*x1*x2*x4**3*x5*x6+x1**2*x4**3*x5*x6+3*x1*x2**2*x3*&
&x4**2*x6+3*x1**2*x2**2*x3*x4*x6+x1**3*x3*x4**2*x5+2*x1**3*x2*x3*x&
&4*x5
dpdx2(355) = x3*x4**4*x5*x6+4*x2**3*x3*x4*x5*x6+4*x1*x2**3*x3*x5*&
&x6+x1**4*x3*x5*x6
dpdx2(356) = 4*x1*x2**3*x4*x5*x6+x1**4*x4*x5*x6+x1*x3*x4**4*x6+x1&
&**4*x3*x4*x6+x1*x3*x4**4*x5+4*x1*x2**3*x3*x4*x5
dpdx2(357) = x1*x4**4*x5*x6+4*x1*x2**3*x3*x4*x6+x1**4*x3*x4*x5
dpdx2(358) = x3*x5**5*x6
dpdx2(359) = x3*x5**4*x6**2+x3**2*x5**4*x6
dpdx2(360) = x3**2*x5**3*x6**2
dpdx2(361) = x3*x5**3*x6**3+x3**3*x5**3*x6
dpdx2(362) = x3**2*x5**2*x6**3+x3**3*x5**2*x6**2
dpdx2(363) = x3**3*x5*x6**3
dpdx2(364) = x3*x5**2*x6**4+x3**4*x5**2*x6
dpdx2(365) = x3**2*x5*x6**4+x3**4*x5*x6**2
dpdx2(366) = x3*x5*x6**5+x3**5*x5*x6
dpdx2(367) = x1*x5**3*x6**3+x3**3*x4*x5**3
dpdx2(368) = x1*x5**2*x6**4+x1*x5**4*x6**2+x3**2*x4*x5**4+x3**4*x&
&4*x5**2
dpdx2(369) = x1*x5*x6**5+x1*x5**5*x6+x3*x4*x5**5+x3**5*x4*x5
dpdx2(370) = x1*x3*x6**5+x4*x5**5*x6+x3**5*x4*x6+x1*x3*x5**5
dpdx2(371) = x1*x3**2*x6**4+x4*x5**4*x6**2+x3**4*x4*x6**2+x1*x3**&
&2*x5**4
dpdx2(372) = x4*x5**3*x6**3+x3**3*x4*x6**3+x1*x3**3*x6**3+x1*x3**&
&3*x5**3
dpdx2(373) = x4*x5**2*x6**4+x3**2*x4*x6**4+x1*x3**4*x6**2+x1*x3**&
&4*x5**2
dpdx2(374) = x4*x5*x6**5+x3*x4*x6**5+x1*x3**5*x6+x1*x3**5*x5
dpdx2(375) = x1*x4*x6**5+x1*x4*x5**5+x1*x3**5*x4
dpdx2(376) = 2*x2*x3*x5**4*x6
dpdx2(377) = 2*x2*x3*x5**3*x6**2+2*x2*x3**2*x5**3*x6
dpdx2(378) = 2*x2*x3**2*x5**2*x6**2
dpdx2(379) = 2*x2*x3*x5**2*x6**3+2*x2*x3**3*x5**2*x6
dpdx2(380) = 2*x2*x3**2*x5*x6**3+2*x2*x3**3*x5*x6**2
dpdx2(381) = 2*x2*x3*x5*x6**4+2*x2*x3**4*x5*x6
dpdx2(382) = x1**2*x5*x6**4+2*x1*x2*x5**4*x6+2*x2*x3*x4*x5**4+x3*&
&*4*x4**2*x5
dpdx2(383) = x1**2*x5**2*x6**3+2*x1*x2*x5**3*x6**2+2*x2*x3**2*x4*&
&x5**3+x3**3*x4**2*x5**2
dpdx2(384) = 2*x1*x2*x5**2*x6**3+x1**2*x5**3*x6**2+x3**2*x4**2*x5&
&**3+2*x2*x3**3*x4*x5**2
dpdx2(385) = 2*x1*x2*x5*x6**4+x1**2*x5**4*x6+x3*x4**2*x5**4+2*x2*&
&x3**4*x4*x5
dpdx2(386) = x1**2*x4*x6**4+2*x1*x2*x4*x5**4+x1*x3**4*x4**2
dpdx2(387) = x1**2*x3*x6**4+2*x2*x4*x5**4*x6+x3**4*x4**2*x6+2*x1*&
&x2*x3*x5**4
dpdx2(388) = 2*x1*x2*x3*x6**4+x4**2*x5**4*x6+2*x2*x3**4*x4*x6+x1*&
&*2*x3*x5**4
dpdx2(389) = x1**2*x3**2*x6**3+2*x2*x4*x5**3*x6**2+x3**3*x4**2*x6&
&**2+2*x1*x2*x3**2*x5**3
dpdx2(390) = 2*x1*x2*x3**2*x6**3+x4**2*x5**3*x6**2+2*x2*x3**3*x4*&
&x6**2+x1**2*x3**2*x5**3
dpdx2(391) = 2*x2*x4*x5**2*x6**3+x3**2*x4**2*x6**3+x1**2*x3**3*x6&
&**2+2*x1*x2*x3**3*x5**2
dpdx2(392) = x4**2*x5**2*x6**3+2*x2*x3**2*x4*x6**3+2*x1*x2*x3**3*&
&x6**2+x1**2*x3**3*x5**2
dpdx2(393) = 2*x2*x4*x5*x6**4+x3*x4**2*x6**4+x1**2*x3**4*x6+2*x1*&
&x2*x3**4*x5
dpdx2(394) = x4**2*x5*x6**4+2*x2*x3*x4*x6**4+2*x1*x2*x3**4*x6+x1*&
&*2*x3**4*x5
dpdx2(395) = x1*x4**2*x6**4+2*x1*x2*x4*x6**4+x1*x4**2*x5**4+x1**2&
&*x4*x5**4+2*x1*x2*x3**4*x4+x1**2*x3**4*x4
dpdx2(396) = 2*x1**2*x2*x5**2*x6**2+2*x2*x3**2*x4**2*x5**2
dpdx2(397) = 2*x1**2*x2*x5*x6**3+2*x1**2*x2*x5**3*x6+2*x2*x3*x4**&
&2*x5**3+2*x2*x3**3*x4**2*x5
dpdx2(398) = x1**2*x4**2*x6**3+2*x1**2*x2*x4*x6**3+2*x1*x2*x4**2*&
&x5**3+2*x1**2*x2*x4*x5**3+2*x1*x2*x3**3*x4**2+x1**2*x3**3*x4**2
dpdx2(399) = 2*x1**2*x2*x3*x6**3+2*x2*x4**2*x5**3*x6+2*x2*x3**3*x&
&4**2*x6+2*x1**2*x2*x3*x5**3
dpdx2(400) = 2*x2*x4**2*x5**2*x6**2+2*x2*x3**2*x4**2*x6**2+2*x1**&
&2*x2*x3**2*x6**2+2*x1**2*x2*x3**2*x5**2
dpdx2(401) = 2*x1**2*x2*x4**2*x6**2+2*x1**2*x2*x4**2*x5**2+2*x1**&
&2*x2*x3**2*x4**2
dpdx2(402) = 2*x2*x4**2*x5*x6**3+2*x2*x3*x4**2*x6**3+2*x1**2*x2*x&
&3**3*x6+2*x1**2*x2*x3**3*x5
dpdx2(403) = 2*x1*x2*x4**2*x6**3+x1**2*x4**2*x5**3+2*x1**2*x2*x3*&
&*3*x4
dpdx2(404) = 3*x2**2*x3*x5**3*x6
dpdx2(405) = 3*x2**2*x3*x5**2*x6**2+3*x2**2*x3**2*x5**2*x6
dpdx2(406) = 3*x2**2*x3**2*x5*x6**2
dpdx2(407) = 3*x2**2*x3*x5*x6**3+3*x2**2*x3**3*x5*x6
dpdx2(408) = x1**3*x5*x6**3+3*x1*x2**2*x5**3*x6+3*x2**2*x3*x4*x5*&
&*3+x3**3*x4**3*x5
dpdx2(409) = 3*x1*x2**2*x5**2*x6**2+x1**3*x5**2*x6**2+x3**2*x4**3&
&*x5**2+3*x2**2*x3**2*x4*x5**2
dpdx2(410) = 3*x1*x2**2*x5*x6**3+x1**3*x5**3*x6+x3*x4**3*x5**3+3*&
&x2**2*x3**3*x4*x5
dpdx2(411) = x1**3*x4*x6**3+3*x1*x2**2*x4*x5**3+x1*x3**3*x4**3
dpdx2(412) = x1**3*x3*x6**3+3*x2**2*x4*x5**3*x6+x3**3*x4**3*x6+3*&
&x1*x2**2*x3*x5**3
dpdx2(413) = 3*x1*x2**2*x3*x6**3+x4**3*x5**3*x6+3*x2**2*x3**3*x4*&
&x6+x1**3*x3*x5**3
dpdx2(414) = 3*x2**2*x4*x5**2*x6**2+x3**2*x4**3*x6**2+x1**3*x3**2&
&*x6**2+3*x1*x2**2*x3**2*x5**2
dpdx2(415) = x4**3*x5**2*x6**2+3*x2**2*x3**2*x4*x6**2+3*x1*x2**2*&
&x3**2*x6**2+x1**3*x3**2*x5**2
dpdx2(416) = 3*x2**2*x4*x5*x6**3+x3*x4**3*x6**3+x1**3*x3**3*x6+3*&
&x1*x2**2*x3**3*x5
dpdx2(417) = x4**3*x5*x6**3+3*x2**2*x3*x4*x6**3+3*x1*x2**2*x3**3*&
&x6+x1**3*x3**3*x5
dpdx2(418) = x1*x4**3*x6**3+3*x1*x2**2*x4*x6**3+x1*x4**3*x5**3+x1&
&**3*x4*x5**3+3*x1*x2**2*x3**3*x4+x1**3*x3**3*x4
dpdx2(419) = 2*x1**3*x2*x5*x6**2+3*x1**2*x2**2*x5**2*x6+3*x2**2*x&
&3*x4**2*x5**2+2*x2*x3**2*x4**3*x5
dpdx2(420) = 3*x1**2*x2**2*x5*x6**2+2*x1**3*x2*x5**2*x6+2*x2*x3*x&
&4**3*x5**2+3*x2**2*x3**2*x4**2*x5
dpdx2(421) = x1**3*x4**2*x6**2+2*x1**3*x2*x4*x6**2+3*x1*x2**2*x4*&
&*2*x5**2+3*x1**2*x2**2*x4*x5**2+2*x1*x2*x3**2*x4**3+x1**2*x3**2*x&
&4**3
dpdx2(422) = x1**2*x4**3*x6**2+3*x1**2*x2**2*x4*x6**2+2*x1*x2*x4*&
&*3*x5**2+2*x1**3*x2*x4*x5**2+3*x1*x2**2*x3**2*x4**2+x1**3*x3**2*x&
&4**2
dpdx2(423) = 2*x1**3*x2*x4**2*x6+3*x1**2*x2**2*x4**2*x5+2*x1**2*x&
&2*x3*x4**3
dpdx2(424) = 2*x1**3*x2*x3*x6**2+3*x2**2*x4**2*x5**2*x6+2*x2*x3**&
&2*x4**3*x6+3*x1**2*x2**2*x3*x5**2
dpdx2(425) = 3*x1**2*x2**2*x3*x6**2+2*x2*x4**3*x5**2*x6+3*x2**2*x&
&3**2*x4**2*x6+2*x1**3*x2*x3*x5**2
dpdx2(426) = 2*x1**2*x2*x4**3*x6+3*x1**2*x2**2*x4**2*x6+2*x1**2*x&
&2*x4**3*x5+2*x1**3*x2*x4**2*x5+3*x1**2*x2**2*x3*x4**2+2*x1**3*x2*&
&x3*x4**2
dpdx2(427) = 3*x2**2*x4**2*x5*x6**2+2*x2*x3*x4**3*x6**2+2*x1**3*x&
&2*x3**2*x6+3*x1**2*x2**2*x3**2*x5
dpdx2(428) = 2*x2*x4**3*x5*x6**2+3*x2**2*x3*x4**2*x6**2+3*x1**2*x&
&2**2*x3**2*x6+2*x1**3*x2*x3**2*x5
dpdx2(429) = 2*x1*x2*x4**3*x6**2+3*x1*x2**2*x4**2*x6**2+x1**2*x4*&
&*3*x5**2+x1**3*x4**2*x5**2+3*x1**2*x2**2*x3**2*x4+2*x1**3*x2*x3**&
&2*x4
dpdx2(430) = 3*x1**3*x2**2*x5*x6+3*x2**2*x3*x4**3*x5
dpdx2(431) = x1**3*x4**3*x6+3*x1**3*x2**2*x4*x6+3*x1*x2**2*x4**3*&
&x5+3*x1**3*x2**2*x4*x5+3*x1*x2**2*x3*x4**3+x1**3*x3*x4**3
dpdx2(432) = 3*x2**2*x4**3*x5*x6+3*x2**2*x3*x4**3*x6+3*x1**3*x2**&
&2*x3*x6+3*x1**3*x2**2*x3*x5
dpdx2(433) = 3*x1*x2**2*x4**3*x6+x1**3*x4**3*x5+3*x1**3*x2**2*x3*&
&x4
dpdx2(434) = 4*x2**3*x3*x5**2*x6
dpdx2(435) = 4*x2**3*x3*x5*x6**2+4*x2**3*x3**2*x5*x6
dpdx2(436) = x1**4*x5*x6**2+4*x1*x2**3*x5**2*x6+4*x2**3*x3*x4*x5*&
&*2+x3**2*x4**4*x5
dpdx2(437) = 4*x1*x2**3*x5*x6**2+x1**4*x5**2*x6+x3*x4**4*x5**2+4*&
&x2**3*x3**2*x4*x5
dpdx2(438) = x1**4*x4*x6**2+4*x1*x2**3*x4*x5**2+x1*x3**2*x4**4
dpdx2(439) = x1**4*x3*x6**2+4*x2**3*x4*x5**2*x6+x3**2*x4**4*x6+4*&
&x1*x2**3*x3*x5**2
dpdx2(440) = 4*x1*x2**3*x3*x6**2+x4**4*x5**2*x6+4*x2**3*x3**2*x4*&
&x6+x1**4*x3*x5**2
dpdx2(441) = 4*x2**3*x4*x5*x6**2+x3*x4**4*x6**2+x1**4*x3**2*x6+4*&
&x1*x2**3*x3**2*x5
dpdx2(442) = x4**4*x5*x6**2+4*x2**3*x3*x4*x6**2+4*x1*x2**3*x3**2*&
&x6+x1**4*x3**2*x5
dpdx2(443) = x1*x4**4*x6**2+4*x1*x2**3*x4*x6**2+x1*x4**4*x5**2+x1&
&**4*x4*x5**2+4*x1*x2**3*x3**2*x4+x1**4*x3**2*x4
dpdx2(444) = 4*x1**2*x2**3*x5*x6+2*x1**4*x2*x5*x6+2*x2*x3*x4**4*x&
&5+4*x2**3*x3*x4**2*x5
dpdx2(445) = x1**4*x4**2*x6+2*x1**4*x2*x4*x6+4*x1*x2**3*x4**2*x5+&
&4*x1**2*x2**3*x4*x5+2*x1*x2*x3*x4**4+x1**2*x3*x4**4
dpdx2(446) = x1**2*x4**4*x6+4*x1**2*x2**3*x4*x6+2*x1*x2*x4**4*x5+&
&2*x1**4*x2*x4*x5+4*x1*x2**3*x3*x4**2+x1**4*x3*x4**2
dpdx2(447) = 4*x2**3*x4**2*x5*x6+2*x2*x3*x4**4*x6+2*x1**4*x2*x3*x&
&6+4*x1**2*x2**3*x3*x5
dpdx2(448) = 2*x2*x4**4*x5*x6+4*x2**3*x3*x4**2*x6+4*x1**2*x2**3*x&
&3*x6+2*x1**4*x2*x3*x5
dpdx2(449) = 2*x1*x2*x4**4*x6+4*x1*x2**3*x4**2*x6+x1**2*x4**4*x5+&
&x1**4*x4**2*x5+4*x1**2*x2**3*x3*x4+2*x1**4*x2*x3*x4
dpdx2(450) = 5*x2**4*x3*x5*x6
dpdx2(451) = 5*x1*x2**4*x5*x6+x1**5*x5*x6+x3*x4**5*x5+5*x2**4*x3*&
&x4*x5
dpdx2(452) = x1**5*x4*x6+5*x1*x2**4*x4*x5+x1*x3*x4**5
dpdx2(453) = 5*x2**4*x4*x5*x6+x3*x4**5*x6+x1**5*x3*x6+5*x1*x2**4*&
&x3*x5
dpdx2(454) = x4**5*x5*x6+5*x2**4*x3*x4*x6+5*x1*x2**4*x3*x6+x1**5*&
&x3*x5
dpdx2(455) = x1*x4**5*x6+5*x1*x2**4*x4*x6+x1*x4**5*x5+x1**5*x4*x5&
&+5*x1*x2**4*x3*x4+x1**5*x3*x4
dpdx2(456) = 0
dpdx2(457) = 0
dpdx2(458) = 0
dpdx2(459) = 0
dpdx2(460) = 0
dpdx2(461) = x5**6*x6+x3*x5**6
dpdx2(462) = x5**5*x6**2+x3**2*x5**5
dpdx2(463) = x5**4*x6**3+x3**3*x5**4
dpdx2(464) = x5**3*x6**4+x3**4*x5**3
dpdx2(465) = x3**3*x6**4+x3**4*x6**3
dpdx2(466) = x5**2*x6**5+x3**5*x5**2
dpdx2(467) = x3**2*x6**5+x3**5*x6**2
dpdx2(468) = x5*x6**6+x3**6*x5
dpdx2(469) = x3*x6**6+x3**6*x6
dpdx2(470) = x1*x6**6+x4*x5**6+x1*x5**6+x3**6*x4
dpdx2(471) = x4*x6**6+x1*x3**6
dpdx2(472) = 2*x2*x5**5*x6+2*x2*x3*x5**5
dpdx2(473) = 2*x2*x5**4*x6**2+2*x2*x3**2*x5**4
dpdx2(474) = 2*x2*x5**3*x6**3+2*x2*x3**3*x5**3
dpdx2(475) = 2*x2*x3**3*x6**3
dpdx2(476) = 2*x2*x5**2*x6**4+2*x2*x3**4*x5**2
dpdx2(477) = 2*x2*x3**2*x6**4+2*x2*x3**4*x6**2
dpdx2(478) = 2*x2*x5*x6**5+2*x2*x3**5*x5
dpdx2(479) = 2*x2*x3*x6**5+2*x2*x3**5*x6
dpdx2(480) = x1**2*x6**5+2*x2*x4*x5**5+2*x1*x2*x5**5+x3**5*x4**2
dpdx2(481) = 2*x1*x2*x6**5+x4**2*x5**5+x1**2*x5**5+2*x2*x3**5*x4
dpdx2(482) = x4**2*x6**5+2*x2*x4*x6**5+2*x1*x2*x3**5+x1**2*x3**5
dpdx2(483) = 2*x1**2*x2*x6**4+2*x2*x4**2*x5**4+2*x1**2*x2*x5**4+2&
&*x2*x3**4*x4**2
dpdx2(484) = 2*x2*x4**2*x6**4+2*x1**2*x2*x3**4
dpdx2(485) = 3*x2**2*x5**4*x6+3*x2**2*x3*x5**4
dpdx2(486) = 3*x2**2*x5**3*x6**2+3*x2**2*x3**2*x5**3
dpdx2(487) = 3*x2**2*x5**2*x6**3+3*x2**2*x3**3*x5**2
dpdx2(488) = 3*x2**2*x3**2*x6**3+3*x2**2*x3**3*x6**2
dpdx2(489) = 3*x2**2*x5*x6**4+3*x2**2*x3**4*x5
dpdx2(490) = 3*x2**2*x3*x6**4+3*x2**2*x3**4*x6
dpdx2(491) = x1**3*x6**4+3*x2**2*x4*x5**4+3*x1*x2**2*x5**4+x3**4*&
&x4**3
dpdx2(492) = 3*x1*x2**2*x6**4+x4**3*x5**4+x1**3*x5**4+3*x2**2*x3*&
&*4*x4
dpdx2(493) = x4**3*x6**4+3*x2**2*x4*x6**4+3*x1*x2**2*x3**4+x1**3*&
&x3**4
dpdx2(494) = 2*x1**3*x2*x6**3+3*x2**2*x4**2*x5**3+3*x1**2*x2**2*x&
&5**3+2*x2*x3**3*x4**3
dpdx2(495) = 3*x1**2*x2**2*x6**3+2*x2*x4**3*x5**3+2*x1**3*x2*x5**&
&3+3*x2**2*x3**3*x4**2
dpdx2(496) = 2*x2*x4**3*x6**3+3*x2**2*x4**2*x6**3+3*x1**2*x2**2*x&
&3**3+2*x1**3*x2*x3**3
dpdx2(497) = 3*x1**3*x2**2*x6**2+3*x2**2*x4**3*x5**2+3*x1**3*x2**&
&2*x5**2+3*x2**2*x3**2*x4**3
dpdx2(498) = 3*x1**2*x2**2*x4**3+2*x1**3*x2*x4**3+3*x1**3*x2**2*x&
&4**2
dpdx2(499) = 3*x2**2*x4**3*x6**2+3*x1**3*x2**2*x3**2
dpdx2(500) = 4*x2**3*x5**3*x6+4*x2**3*x3*x5**3
dpdx2(501) = 4*x2**3*x5**2*x6**2+4*x2**3*x3**2*x5**2
dpdx2(502) = 4*x2**3*x3**2*x6**2
dpdx2(503) = 4*x2**3*x5*x6**3+4*x2**3*x3**3*x5
dpdx2(504) = 4*x2**3*x3*x6**3+4*x2**3*x3**3*x6
dpdx2(505) = x1**4*x6**3+4*x2**3*x4*x5**3+4*x1*x2**3*x5**3+x3**3*&
&x4**4
dpdx2(506) = 4*x1*x2**3*x6**3+x4**4*x5**3+x1**4*x5**3+4*x2**3*x3*&
&*3*x4
dpdx2(507) = x4**4*x6**3+4*x2**3*x4*x6**3+4*x1*x2**3*x3**3+x1**4*&
&x3**3
dpdx2(508) = 2*x1**4*x2*x6**2+4*x2**3*x4**2*x5**2+4*x1**2*x2**3*x&
&5**2+2*x2*x3**2*x4**4
dpdx2(509) = 4*x1**2*x2**3*x6**2+2*x2*x4**4*x5**2+2*x1**4*x2*x5**&
&2+4*x2**3*x3**2*x4**2
dpdx2(510) = 2*x1**2*x2*x4**4+4*x1**2*x2**3*x4**2+2*x1**4*x2*x4**&
&2
dpdx2(511) = 2*x2*x4**4*x6**2+4*x2**3*x4**2*x6**2+4*x1**2*x2**3*x&
&3**2+2*x1**4*x2*x3**2
dpdx2(512) = 3*x1**4*x2**2*x6+4*x2**3*x4**3*x5+4*x1**3*x2**3*x5+3&
&*x2**2*x3*x4**4
dpdx2(513) = 4*x1**3*x2**3*x6+3*x2**2*x4**4*x5+3*x1**4*x2**2*x5+4&
&*x2**3*x3*x4**3
dpdx2(514) = 3*x1*x2**2*x4**4+x1**3*x4**4+4*x1*x2**3*x4**3+x1**4*&
&x4**3+4*x1**3*x2**3*x4+3*x1**4*x2**2*x4
dpdx2(515) = 3*x2**2*x4**4*x6+4*x2**3*x4**3*x6+4*x1**3*x2**3*x3+3&
&*x1**4*x2**2*x3
dpdx2(516) = 5*x2**4*x5**2*x6+5*x2**4*x3*x5**2
dpdx2(517) = 5*x2**4*x5*x6**2+5*x2**4*x3**2*x5
dpdx2(518) = 5*x2**4*x3*x6**2+5*x2**4*x3**2*x6
dpdx2(519) = x1**5*x6**2+5*x2**4*x4*x5**2+5*x1*x2**4*x5**2+x3**2*&
&x4**5
dpdx2(520) = 5*x1*x2**4*x6**2+x4**5*x5**2+x1**5*x5**2+5*x2**4*x3*&
&*2*x4
dpdx2(521) = x4**5*x6**2+5*x2**4*x4*x6**2+5*x1*x2**4*x3**2+x1**5*&
&x3**2
dpdx2(522) = 2*x1**5*x2*x6+5*x2**4*x4**2*x5+5*x1**2*x2**4*x5+2*x2&
&*x3*x4**5
dpdx2(523) = 5*x1**2*x2**4*x6+2*x2*x4**5*x5+2*x1**5*x2*x5+5*x2**4&
&*x3*x4**2
dpdx2(524) = 2*x1*x2*x4**5+x1**2*x4**5+5*x1*x2**4*x4**2+x1**5*x4*&
&*2+5*x1**2*x2**4*x4+2*x1**5*x2*x4
dpdx2(525) = 2*x2*x4**5*x6+5*x2**4*x4**2*x6+5*x1**2*x2**4*x3+2*x1&
&**5*x2*x3
dpdx2(526) = 6*x2**5*x5*x6+6*x2**5*x3*x5
dpdx2(527) = 6*x2**5*x3*x6
dpdx2(528) = x1**6*x6+6*x2**5*x4*x5+6*x1*x2**5*x5+x3*x4**6
dpdx2(529) = 6*x1*x2**5*x6+x4**6*x5+x1**6*x5+6*x2**5*x3*x4
dpdx2(530) = x1*x4**6+6*x1*x2**5*x4+x1**6*x4
dpdx2(531) = x4**6*x6+6*x2**5*x4*x6+6*x1*x2**5*x3+x1**6*x3
dpdx2(532) = 0
dpdx2(533) = 0
dpdx2(534) = 0
dpdx2(535) = 0
dpdx2(536) = x6**7+x3**7
dpdx2(537) = 2*x2*x6**6+2*x2*x3**6
dpdx2(538) = 3*x2**2*x6**5+3*x2**2*x3**5
dpdx2(539) = 4*x2**3*x6**4+4*x2**3*x3**4
dpdx2(540) = 4*x2**3*x4**4+4*x1**4*x2**3
dpdx2(541) = 5*x2**4*x6**3+5*x2**4*x3**3
dpdx2(542) = 3*x2**2*x4**5+5*x2**4*x4**3+5*x1**3*x2**4+3*x1**5*x2&
&**2
dpdx2(543) = 6*x2**5*x6**2+6*x2**5*x3**2
dpdx2(544) = 2*x2*x4**6+6*x2**5*x4**2+6*x1**2*x2**5+2*x1**6*x2
dpdx2(545) = 7*x2**6*x6+7*x2**6*x3
dpdx2(546) = x4**7+7*x2**6*x4+7*x1*x2**6+x1**7
dpdx3(1) = 0
dpdx3(2) = x6+x5
dpdx3(3) = x2+x1
dpdx3(4) = 0
dpdx3(5) = x5*x6
dpdx3(6) = x4*x6+x1*x6+x4*x5+x2*x5
dpdx3(7) = x2*x6+x1*x5
dpdx3(8) = x2*x4+x1*x4
dpdx3(9) = 0
dpdx3(10) = x1*x2
dpdx3(11) = x6**2+2*x3*x6+x5**2+2*x3*x5
dpdx3(12) = 2*x2*x3+2*x1*x3
dpdx3(13) = x2**2+x1**2
dpdx3(14) = 0
dpdx3(15) = x4*x5*x6+x2*x5*x6+x1*x5*x6
dpdx3(16) = x1*x4*x6+x2*x4*x5
dpdx3(17) = x2*x4*x6+x1*x2*x6+x1*x4*x5+x1*x2*x5
dpdx3(18) = x1*x2*x4
dpdx3(19) = x5*x6**2+x5**2*x6+2*x3*x5*x6
dpdx3(20) = x1*x6**2+2*x3*x4*x6+x2*x5**2+2*x3*x4*x5
dpdx3(21) = x4*x6**2+2*x1*x3*x6+x4*x5**2+2*x2*x3*x5
dpdx3(22) = x2*x6**2+2*x2*x3*x6+x1*x5**2+2*x1*x3*x5
dpdx3(23) = 2*x2*x3*x4+2*x1*x3*x4
dpdx3(24) = 2*x1*x2*x3
dpdx3(25) = x4**2*x6+x1**2*x6+x4**2*x5+x2**2*x5
dpdx3(26) = x2**2*x6+x1**2*x5
dpdx3(27) = x2*x4**2+x1*x4**2
dpdx3(28) = x2**2*x4+x1**2*x4
dpdx3(29) = 0
dpdx3(30) = x1*x2**2+x1**2*x2
dpdx3(31) = 2*x3*x6**2+2*x3*x5**2
dpdx3(32) = x6**3+3*x3**2*x6+x5**3+3*x3**2*x5
dpdx3(33) = 3*x2*x3**2+3*x1*x3**2
dpdx3(34) = 2*x2**2*x3+2*x1**2*x3
dpdx3(35) = 0
dpdx3(36) = x2**3+x1**3
dpdx3(37) = 0
dpdx3(38) = x2*x4*x5*x6+x1*x4*x5*x6+x1*x2*x5*x6
dpdx3(39) = x1*x2*x4*x6+x1*x2*x4*x5
dpdx3(40) = x1*x5*x6**2+x2*x5**2*x6+2*x3*x4*x5*x6
dpdx3(41) = x4*x5*x6**2+x2*x5*x6**2+x4*x5**2*x6+x1*x5**2*x6+2*x2*&
&x3*x5*x6+2*x1*x3*x5*x6
dpdx3(42) = x1*x4*x6**2+2*x1*x3*x4*x6+x2*x4*x5**2+2*x2*x3*x4*x5
dpdx3(43) = x1*x2*x6**2+2*x2*x3*x4*x6+x1*x2*x5**2+2*x1*x3*x4*x5
dpdx3(44) = x2*x4*x6**2+2*x1*x2*x3*x6+x1*x4*x5**2+2*x1*x2*x3*x5
dpdx3(45) = 2*x1*x2*x3*x4
dpdx3(46) = x4**2*x5*x6+x2**2*x5*x6+x1**2*x5*x6
dpdx3(47) = x1*x4**2*x6+x1**2*x4*x6+x2*x4**2*x5+x2**2*x4*x5
dpdx3(48) = x1*x2*x4**2
dpdx3(49) = x2*x4**2*x6+x1**2*x2*x6+x1*x4**2*x5+x1*x2**2*x5
dpdx3(50) = x2**2*x4*x6+x1*x2**2*x6+x1**2*x4*x5+x1**2*x2*x5
dpdx3(51) = x1*x2**2*x4+x1**2*x2*x4
dpdx3(52) = x5**2*x6**2+2*x3*x5*x6**2+2*x3*x5**2*x6
dpdx3(53) = x5*x6**3+x5**3*x6+3*x3**2*x5*x6
dpdx3(54) = x1*x6**3+3*x3**2*x4*x6+x2*x5**3+3*x3**2*x4*x5
dpdx3(55) = 2*x3*x4*x6**2+2*x1*x3*x6**2+2*x3*x4*x5**2+2*x2*x3*x5*&
&*2
dpdx3(56) = 2*x2*x3*x6**2+2*x1*x3*x5**2
dpdx3(57) = x4*x6**3+3*x1*x3**2*x6+x4*x5**3+3*x2*x3**2*x5
dpdx3(58) = x2*x6**3+3*x2*x3**2*x6+x1*x5**3+3*x1*x3**2*x5
dpdx3(59) = 3*x2*x3**2*x4+3*x1*x3**2*x4
dpdx3(60) = 3*x1*x2*x3**2
dpdx3(61) = x1**2*x6**2+2*x3*x4**2*x6+x2**2*x5**2+2*x3*x4**2*x5
dpdx3(62) = x4**2*x6**2+2*x1**2*x3*x6+x4**2*x5**2+2*x2**2*x3*x5
dpdx3(63) = x2**2*x6**2+2*x2**2*x3*x6+x1**2*x5**2+2*x1**2*x3*x5
dpdx3(64) = 2*x2*x3*x4**2+2*x1*x3*x4**2
dpdx3(65) = 2*x2**2*x3*x4+2*x1**2*x3*x4
dpdx3(66) = 2*x1*x2**2*x3+2*x1**2*x2*x3
dpdx3(67) = x2**2*x4**2+x1**2*x4**2
dpdx3(68) = 0
dpdx3(69) = x1**2*x2**2
dpdx3(70) = x4**3*x6+x1**3*x6+x4**3*x5+x2**3*x5
dpdx3(71) = x2**3*x6+x1**3*x5
dpdx3(72) = x2*x4**3+x1*x4**3
dpdx3(73) = x2**3*x4+x1**3*x4
dpdx3(74) = 0
dpdx3(75) = x1*x2**3+x1**3*x2
dpdx3(76) = 2*x3*x6**3+3*x3**2*x6**2+2*x3*x5**3+3*x3**2*x5**2
dpdx3(77) = x6**4+4*x3**3*x6+x5**4+4*x3**3*x5
dpdx3(78) = 4*x2*x3**3+4*x1*x3**3
dpdx3(79) = 3*x2**2*x3**2+3*x1**2*x3**2
dpdx3(80) = 2*x2**3*x3+2*x1**3*x3
dpdx3(81) = 0
dpdx3(82) = x2**4+x1**4
dpdx3(83) = 0
dpdx3(84) = x1*x2*x4*x5*x6
dpdx3(85) = x1*x4*x5*x6**2+x1*x2*x5*x6**2+x2*x4*x5**2*x6+x1*x2*x5&
&**2*x6+2*x2*x3*x4*x5*x6+2*x1*x3*x4*x5*x6
dpdx3(86) = x2*x4*x5*x6**2+x1*x4*x5**2*x6+2*x1*x2*x3*x5*x6
dpdx3(87) = x1*x2*x4*x6**2+2*x1*x2*x3*x4*x6+x1*x2*x4*x5**2+2*x1*x&
&2*x3*x4*x5
dpdx3(88) = x2*x4**2*x5*x6+x1*x4**2*x5*x6+x2**2*x4*x5*x6+x1**2*x4&
&*x5*x6+x1*x2**2*x5*x6+x1**2*x2*x5*x6
dpdx3(89) = x1*x2*x4**2*x6+x1**2*x2*x4*x6+x1*x2*x4**2*x5+x1*x2**2&
&*x4*x5
dpdx3(90) = x1*x2**2*x4*x6+x1**2*x2*x4*x5
dpdx3(91) = x1*x5*x6**3+x2*x5**3*x6+3*x3**2*x4*x5*x6
dpdx3(92) = x2*x5**2*x6**2+x1*x5**2*x6**2+2*x3*x4*x5*x6**2+2*x1*x&
&3*x5*x6**2+2*x3*x4*x5**2*x6+2*x2*x3*x5**2*x6
dpdx3(93) = x4*x5**2*x6**2+2*x2*x3*x5*x6**2+2*x1*x3*x5**2*x6
dpdx3(94) = x4*x5*x6**3+x2*x5*x6**3+x4*x5**3*x6+x1*x5**3*x6+3*x2*&
&x3**2*x5*x6+3*x1*x3**2*x5*x6
dpdx3(95) = 2*x1*x3*x4*x6**2+2*x2*x3*x4*x5**2
dpdx3(96) = x1*x4*x6**3+3*x1*x3**2*x4*x6+x2*x4*x5**3+3*x2*x3**2*x&
&4*x5
dpdx3(97) = x1*x2*x6**3+3*x2*x3**2*x4*x6+x1*x2*x5**3+3*x1*x3**2*x&
&4*x5
dpdx3(98) = 2*x2*x3*x4*x6**2+2*x1*x2*x3*x6**2+2*x1*x3*x4*x5**2+2*&
&x1*x2*x3*x5**2
dpdx3(99) = x2*x4*x6**3+3*x1*x2*x3**2*x6+x1*x4*x5**3+3*x1*x2*x3**&
&2*x5
dpdx3(100) = 3*x1*x2*x3**2*x4
dpdx3(101) = x1**2*x5*x6**2+x2**2*x5**2*x6+2*x3*x4**2*x5*x6
dpdx3(102) = x4**2*x5*x6**2+x2**2*x5*x6**2+x4**2*x5**2*x6+x1**2*x&
&5**2*x6+2*x2**2*x3*x5*x6+2*x1**2*x3*x5*x6
dpdx3(103) = x1**2*x4*x6**2+2*x1*x3*x4**2*x6+x2**2*x4*x5**2+2*x2*&
&x3*x4**2*x5
dpdx3(104) = x1*x4**2*x6**2+2*x1**2*x3*x4*x6+x2*x4**2*x5**2+2*x2*&
&*2*x3*x4*x5
dpdx3(105) = 2*x1*x2*x3*x4**2
dpdx3(106) = x1**2*x2*x6**2+2*x2*x3*x4**2*x6+x1*x2**2*x5**2+2*x1*&
&x3*x4**2*x5
dpdx3(107) = x1*x2**2*x6**2+2*x2**2*x3*x4*x6+x1**2*x2*x5**2+2*x1*&
&*2*x3*x4*x5
dpdx3(108) = x2*x4**2*x6**2+2*x1**2*x2*x3*x6+x1*x4**2*x5**2+2*x1*&
&x2**2*x3*x5
dpdx3(109) = x2**2*x4*x6**2+2*x1*x2**2*x3*x6+x1**2*x4*x5**2+2*x1*&
&*2*x2*x3*x5
dpdx3(110) = 2*x1*x2**2*x3*x4+2*x1**2*x2*x3*x4
dpdx3(111) = x1**2*x4**2*x6+x2**2*x4**2*x5
dpdx3(112) = x1*x2**2*x4**2+x1**2*x2*x4**2
dpdx3(113) = x2**2*x4**2*x6+x1**2*x2**2*x6+x1**2*x4**2*x5+x1**2*x&
&2**2*x5
dpdx3(114) = x1**2*x2**2*x4
dpdx3(115) = x4**3*x5*x6+x2**3*x5*x6+x1**3*x5*x6
dpdx3(116) = x1*x4**3*x6+x1**3*x4*x6+x2*x4**3*x5+x2**3*x4*x5
dpdx3(117) = x1*x2*x4**3
dpdx3(118) = x2*x4**3*x6+x1**3*x2*x6+x1*x4**3*x5+x1*x2**3*x5
dpdx3(119) = x2**3*x4*x6+x1*x2**3*x6+x1**3*x4*x5+x1**3*x2*x5
dpdx3(120) = x1*x2**3*x4+x1**3*x2*x4
dpdx3(121) = 2*x3*x5**2*x6**2
dpdx3(122) = x5**2*x6**3+2*x3*x5*x6**3+x5**3*x6**2+3*x3**2*x5*x6*&
&*2+2*x3*x5**3*x6+3*x3**2*x5**2*x6
dpdx3(123) = x5*x6**4+x5**4*x6+4*x3**3*x5*x6
dpdx3(124) = x1*x6**4+4*x3**3*x4*x6+x2*x5**4+4*x3**3*x4*x5
dpdx3(125) = 2*x1*x3*x6**3+3*x3**2*x4*x6**2+2*x2*x3*x5**3+3*x3**2&
&*x4*x5**2
dpdx3(126) = 2*x3*x4*x6**3+3*x1*x3**2*x6**2+2*x3*x4*x5**3+3*x2*x3&
&**2*x5**2
dpdx3(127) = 2*x2*x3*x6**3+3*x2*x3**2*x6**2+2*x1*x3*x5**3+3*x1*x3&
&**2*x5**2
dpdx3(128) = x4*x6**4+4*x1*x3**3*x6+x4*x5**4+4*x2*x3**3*x5
dpdx3(129) = x2*x6**4+4*x2*x3**3*x6+x1*x5**4+4*x1*x3**3*x5
dpdx3(130) = 4*x2*x3**3*x4+4*x1*x3**3*x4
dpdx3(131) = 4*x1*x2*x3**3
dpdx3(132) = x1**2*x6**3+3*x3**2*x4**2*x6+x2**2*x5**3+3*x3**2*x4*&
&*2*x5
dpdx3(133) = 2*x3*x4**2*x6**2+2*x1**2*x3*x6**2+2*x3*x4**2*x5**2+2&
&*x2**2*x3*x5**2
dpdx3(134) = 2*x2**2*x3*x6**2+2*x1**2*x3*x5**2
dpdx3(135) = x4**2*x6**3+3*x1**2*x3**2*x6+x4**2*x5**3+3*x2**2*x3*&
&*2*x5
dpdx3(136) = x2**2*x6**3+3*x2**2*x3**2*x6+x1**2*x5**3+3*x1**2*x3*&
&*2*x5
dpdx3(137) = 3*x2*x3**2*x4**2+3*x1*x3**2*x4**2
dpdx3(138) = 3*x2**2*x3**2*x4+3*x1**2*x3**2*x4
dpdx3(139) = 3*x1*x2**2*x3**2+3*x1**2*x2*x3**2
dpdx3(140) = 2*x2**2*x3*x4**2+2*x1**2*x3*x4**2
dpdx3(141) = 0
dpdx3(142) = 2*x1**2*x2**2*x3
dpdx3(143) = x1**3*x6**2+2*x3*x4**3*x6+x2**3*x5**2+2*x3*x4**3*x5
dpdx3(144) = x4**3*x6**2+2*x1**3*x3*x6+x4**3*x5**2+2*x2**3*x3*x5
dpdx3(145) = x2**3*x6**2+2*x2**3*x3*x6+x1**3*x5**2+2*x1**3*x3*x5
dpdx3(146) = 2*x2*x3*x4**3+2*x1*x3*x4**3
dpdx3(147) = 2*x2**3*x3*x4+2*x1**3*x3*x4
dpdx3(148) = 2*x1*x2**3*x3+2*x1**3*x2*x3
dpdx3(149) = x2**2*x4**3+x1**2*x4**3
dpdx3(150) = x2**3*x4**2+x1**3*x4**2
dpdx3(151) = 0
dpdx3(152) = x1**2*x2**3+x1**3*x2**2
dpdx3(153) = x4**4*x6+x1**4*x6+x4**4*x5+x2**4*x5
dpdx3(154) = x2**4*x6+x1**4*x5
dpdx3(155) = x2*x4**4+x1*x4**4
dpdx3(156) = x2**4*x4+x1**4*x4
dpdx3(157) = 0
dpdx3(158) = x1*x2**4+x1**4*x2
dpdx3(159) = 3*x3**2*x6**3+3*x3**2*x5**3
dpdx3(160) = 2*x3*x6**4+4*x3**3*x6**2+2*x3*x5**4+4*x3**3*x5**2
dpdx3(161) = x6**5+5*x3**4*x6+x5**5+5*x3**4*x5
dpdx3(162) = 5*x2*x3**4+5*x1*x3**4
dpdx3(163) = 4*x2**2*x3**3+4*x1**2*x3**3
dpdx3(164) = 3*x2**3*x3**2+3*x1**3*x3**2
dpdx3(165) = 0
dpdx3(166) = 2*x2**4*x3+2*x1**4*x3
dpdx3(167) = 0
dpdx3(168) = x2**5+x1**5
dpdx3(169) = 0
dpdx3(170) = x1*x2*x4*x5*x6**2+x1*x2*x4*x5**2*x6+2*x1*x2*x3*x4*x5&
&*x6
dpdx3(171) = x1*x2*x4**2*x5*x6+x1*x2**2*x4*x5*x6+x1**2*x2*x4*x5*x&
&6
dpdx3(172) = x1*x2*x5**2*x6**2+2*x1*x3*x4*x5*x6**2+2*x2*x3*x4*x5*&
&*2*x6
dpdx3(173) = x1*x4*x5*x6**3+x1*x2*x5*x6**3+x2*x4*x5**3*x6+x1*x2*x&
&5**3*x6+3*x2*x3**2*x4*x5*x6+3*x1*x3**2*x4*x5*x6
dpdx3(174) = x2*x4*x5**2*x6**2+x1*x4*x5**2*x6**2+2*x2*x3*x4*x5*x6&
&**2+2*x1*x2*x3*x5*x6**2+2*x1*x3*x4*x5**2*x6+2*x1*x2*x3*x5**2*x6
dpdx3(175) = 2*x1*x2*x3*x4*x6**2+2*x1*x2*x3*x4*x5**2
dpdx3(176) = x2*x4*x5*x6**3+x1*x4*x5**3*x6+3*x1*x2*x3**2*x5*x6
dpdx3(177) = x1*x2*x4*x6**3+3*x1*x2*x3**2*x4*x6+x1*x2*x4*x5**3+3*&
&x1*x2*x3**2*x4*x5
dpdx3(178) = x1**2*x4*x5*x6**2+x1**2*x2*x5*x6**2+x2**2*x4*x5**2*x&
&6+x1*x2**2*x5**2*x6+2*x2*x3*x4**2*x5*x6+2*x1*x3*x4**2*x5*x6
dpdx3(179) = x1*x4**2*x5*x6**2+x1*x2**2*x5*x6**2+x2*x4**2*x5**2*x&
&6+x1**2*x2*x5**2*x6+2*x2**2*x3*x4*x5*x6+2*x1**2*x3*x4*x5*x6
dpdx3(180) = x1**2*x2*x4*x6**2+2*x1*x2*x3*x4**2*x6+x1*x2**2*x4*x5&
&**2+2*x1*x2*x3*x4**2*x5
dpdx3(181) = x2*x4**2*x5*x6**2+x2**2*x4*x5*x6**2+x1*x4**2*x5**2*x&
&6+x1**2*x4*x5**2*x6+2*x1*x2**2*x3*x5*x6+2*x1**2*x2*x3*x5*x6
dpdx3(182) = x1*x2*x4**2*x6**2+2*x1**2*x2*x3*x4*x6+x1*x2*x4**2*x5&
&**2+2*x1*x2**2*x3*x4*x5
dpdx3(183) = x1*x2**2*x4*x6**2+2*x1*x2**2*x3*x4*x6+x1**2*x2*x4*x5&
&**2+2*x1**2*x2*x3*x4*x5
dpdx3(184) = x1**2*x2*x4**2*x6+x1*x2**2*x4**2*x5
dpdx3(185) = x2**2*x4**2*x5*x6+x1**2*x4**2*x5*x6+x1**2*x2**2*x5*x&
&6
dpdx3(186) = x1*x2**2*x4**2*x6+x1**2*x2**2*x4*x6+x1**2*x2*x4**2*x&
&5+x1**2*x2**2*x4*x5
dpdx3(187) = x2*x4**3*x5*x6+x1*x4**3*x5*x6+x2**3*x4*x5*x6+x1**3*x&
&4*x5*x6+x1*x2**3*x5*x6+x1**3*x2*x5*x6
dpdx3(188) = x1*x2*x4**3*x6+x1**3*x2*x4*x6+x1*x2*x4**3*x5+x1*x2**&
&3*x4*x5
dpdx3(189) = x1*x2**3*x4*x6+x1**3*x2*x4*x5
dpdx3(190) = x1*x5*x6**4+x2*x5**4*x6+4*x3**3*x4*x5*x6
dpdx3(191) = x1*x5**2*x6**3+2*x1*x3*x5*x6**3+x2*x5**3*x6**2+3*x3*&
&*2*x4*x5*x6**2+2*x2*x3*x5**3*x6+3*x3**2*x4*x5**2*x6
dpdx3(192) = 2*x3*x4*x5**2*x6**2+2*x2*x3*x5**2*x6**2+2*x1*x3*x5**&
&2*x6**2
dpdx3(193) = x2*x5**2*x6**3+2*x3*x4*x5*x6**3+x1*x5**3*x6**2+3*x1*&
&x3**2*x5*x6**2+2*x3*x4*x5**3*x6+3*x2*x3**2*x5**2*x6
dpdx3(194) = x4*x5**2*x6**3+2*x2*x3*x5*x6**3+x4*x5**3*x6**2+3*x2*&
&x3**2*x5*x6**2+2*x1*x3*x5**3*x6+3*x1*x3**2*x5**2*x6
dpdx3(195) = x4*x5*x6**4+x2*x5*x6**4+x4*x5**4*x6+x1*x5**4*x6+4*x2&
&*x3**3*x5*x6+4*x1*x3**3*x5*x6
dpdx3(196) = 2*x1*x3*x4*x6**3+3*x1*x3**2*x4*x6**2+2*x2*x3*x4*x5**&
&3+3*x2*x3**2*x4*x5**2
dpdx3(197) = x1*x4*x6**4+4*x1*x3**3*x4*x6+x2*x4*x5**4+4*x2*x3**3*&
&x4*x5
dpdx3(198) = x1*x2*x6**4+4*x2*x3**3*x4*x6+x1*x2*x5**4+4*x1*x3**3*&
&x4*x5
dpdx3(199) = 2*x1*x2*x3*x6**3+3*x2*x3**2*x4*x6**2+2*x1*x2*x3*x5**&
&3+3*x1*x3**2*x4*x5**2
dpdx3(200) = 2*x2*x3*x4*x6**3+3*x1*x2*x3**2*x6**2+2*x1*x3*x4*x5**&
&3+3*x1*x2*x3**2*x5**2
dpdx3(201) = x2*x4*x6**4+4*x1*x2*x3**3*x6+x1*x4*x5**4+4*x1*x2*x3*&
&*3*x5
dpdx3(202) = 4*x1*x2*x3**3*x4
dpdx3(203) = x1**2*x5*x6**3+x2**2*x5**3*x6+3*x3**2*x4**2*x5*x6
dpdx3(204) = x2**2*x5**2*x6**2+x1**2*x5**2*x6**2+2*x3*x4**2*x5*x6&
&**2+2*x1**2*x3*x5*x6**2+2*x3*x4**2*x5**2*x6+2*x2**2*x3*x5**2*x6
dpdx3(205) = x4**2*x5**2*x6**2+2*x2**2*x3*x5*x6**2+2*x1**2*x3*x5*&
&*2*x6
dpdx3(206) = x4**2*x5*x6**3+x2**2*x5*x6**3+x4**2*x5**3*x6+x1**2*x&
&5**3*x6+3*x2**2*x3**2*x5*x6+3*x1**2*x3**2*x5*x6
dpdx3(207) = x1**2*x4*x6**3+3*x1*x3**2*x4**2*x6+x2**2*x4*x5**3+3*&
&x2*x3**2*x4**2*x5
dpdx3(208) = 2*x1*x3*x4**2*x6**2+2*x1**2*x3*x4*x6**2+2*x2*x3*x4**&
&2*x5**2+2*x2**2*x3*x4*x5**2
dpdx3(209) = x1*x4**2*x6**3+3*x1**2*x3**2*x4*x6+x2*x4**2*x5**3+3*&
&x2**2*x3**2*x4*x5
dpdx3(210) = 3*x1*x2*x3**2*x4**2
dpdx3(211) = x1**2*x2*x6**3+3*x2*x3**2*x4**2*x6+x1*x2**2*x5**3+3*&
&x1*x3**2*x4**2*x5
dpdx3(212) = x1*x2**2*x6**3+3*x2**2*x3**2*x4*x6+x1**2*x2*x5**3+3*&
&x1**2*x3**2*x4*x5
dpdx3(213) = 2*x2*x3*x4**2*x6**2+2*x1**2*x2*x3*x6**2+2*x1*x3*x4**&
&2*x5**2+2*x1*x2**2*x3*x5**2
dpdx3(214) = 2*x2**2*x3*x4*x6**2+2*x1*x2**2*x3*x6**2+2*x1**2*x3*x&
&4*x5**2+2*x1**2*x2*x3*x5**2
dpdx3(215) = x2*x4**2*x6**3+3*x1**2*x2*x3**2*x6+x1*x4**2*x5**3+3*&
&x1*x2**2*x3**2*x5
dpdx3(216) = x2**2*x4*x6**3+3*x1*x2**2*x3**2*x6+x1**2*x4*x5**3+3*&
&x1**2*x2*x3**2*x5
dpdx3(217) = 3*x1*x2**2*x3**2*x4+3*x1**2*x2*x3**2*x4
dpdx3(218) = x1**2*x4**2*x6**2+2*x1**2*x3*x4**2*x6+x2**2*x4**2*x5&
&**2+2*x2**2*x3*x4**2*x5
dpdx3(219) = 2*x1*x2**2*x3*x4**2+2*x1**2*x2*x3*x4**2
dpdx3(220) = x1**2*x2**2*x6**2+2*x2**2*x3*x4**2*x6+x1**2*x2**2*x5&
&**2+2*x1**2*x3*x4**2*x5
dpdx3(221) = x1**2*x2**2*x4**2
dpdx3(222) = x2**2*x4**2*x6**2+2*x1**2*x2**2*x3*x6+x1**2*x4**2*x5&
&**2+2*x1**2*x2**2*x3*x5
dpdx3(223) = 2*x1**2*x2**2*x3*x4
dpdx3(224) = x1**3*x5*x6**2+x2**3*x5**2*x6+2*x3*x4**3*x5*x6
dpdx3(225) = x4**3*x5*x6**2+x2**3*x5*x6**2+x4**3*x5**2*x6+x1**3*x&
&5**2*x6+2*x2**3*x3*x5*x6+2*x1**3*x3*x5*x6
dpdx3(226) = x1**3*x4*x6**2+2*x1*x3*x4**3*x6+x2**3*x4*x5**2+2*x2*&
&x3*x4**3*x5
dpdx3(227) = x1*x4**3*x6**2+2*x1**3*x3*x4*x6+x2*x4**3*x5**2+2*x2*&
&*3*x3*x4*x5
dpdx3(228) = 2*x1*x2*x3*x4**3
dpdx3(229) = x1**3*x2*x6**2+2*x2*x3*x4**3*x6+x1*x2**3*x5**2+2*x1*&
&x3*x4**3*x5
dpdx3(230) = x1*x2**3*x6**2+2*x2**3*x3*x4*x6+x1**3*x2*x5**2+2*x1*&
&*3*x3*x4*x5
dpdx3(231) = x2*x4**3*x6**2+2*x1**3*x2*x3*x6+x1*x4**3*x5**2+2*x1*&
&x2**3*x3*x5
dpdx3(232) = x2**3*x4*x6**2+2*x1*x2**3*x3*x6+x1**3*x4*x5**2+2*x1*&
&*3*x2*x3*x5
dpdx3(233) = 2*x1*x2**3*x3*x4+2*x1**3*x2*x3*x4
dpdx3(234) = x1**2*x4**3*x6+x1**3*x4**2*x6+x2**2*x4**3*x5+x2**3*x&
&4**2*x5
dpdx3(235) = x1*x2**2*x4**3+x1**2*x2*x4**3
dpdx3(236) = x1*x2**3*x4**2+x1**3*x2*x4**2
dpdx3(237) = x2**2*x4**3*x6+x1**3*x2**2*x6+x1**2*x4**3*x5+x1**2*x&
&2**3*x5
dpdx3(238) = x2**3*x4**2*x6+x1**2*x2**3*x6+x1**3*x4**2*x5+x1**3*x&
&2**2*x5
dpdx3(239) = x1**2*x2**3*x4+x1**3*x2**2*x4
dpdx3(240) = x4**4*x5*x6+x2**4*x5*x6+x1**4*x5*x6
dpdx3(241) = x1*x4**4*x6+x1**4*x4*x6+x2*x4**4*x5+x2**4*x4*x5
dpdx3(242) = x1*x2*x4**4
dpdx3(243) = x2*x4**4*x6+x1**4*x2*x6+x1*x4**4*x5+x1*x2**4*x5
dpdx3(244) = x2**4*x4*x6+x1*x2**4*x6+x1**4*x4*x5+x1**4*x2*x5
dpdx3(245) = x1*x2**4*x4+x1**4*x2*x4
dpdx3(246) = 2*x3*x5**2*x6**3+2*x3*x5**3*x6**2+3*x3**2*x5**2*x6**&
&2
dpdx3(247) = x5**3*x6**3+3*x3**2*x5*x6**3+3*x3**2*x5**3*x6
dpdx3(248) = x5**2*x6**4+2*x3*x5*x6**4+x5**4*x6**2+4*x3**3*x5*x6*&
&*2+2*x3*x5**4*x6+4*x3**3*x5**2*x6
dpdx3(249) = x5*x6**5+x5**5*x6+5*x3**4*x5*x6
dpdx3(250) = x1*x6**5+5*x3**4*x4*x6+x2*x5**5+5*x3**4*x4*x5
dpdx3(251) = 2*x1*x3*x6**4+4*x3**3*x4*x6**2+2*x2*x3*x5**4+4*x3**3&
&*x4*x5**2
dpdx3(252) = 3*x3**2*x4*x6**3+3*x1*x3**2*x6**3+3*x3**2*x4*x5**3+3&
&*x2*x3**2*x5**3
dpdx3(253) = 3*x2*x3**2*x6**3+3*x1*x3**2*x5**3
dpdx3(254) = 2*x3*x4*x6**4+4*x1*x3**3*x6**2+2*x3*x4*x5**4+4*x2*x3&
&**3*x5**2
dpdx3(255) = 2*x2*x3*x6**4+4*x2*x3**3*x6**2+2*x1*x3*x5**4+4*x1*x3&
&**3*x5**2
dpdx3(256) = x4*x6**5+5*x1*x3**4*x6+x4*x5**5+5*x2*x3**4*x5
dpdx3(257) = x2*x6**5+5*x2*x3**4*x6+x1*x5**5+5*x1*x3**4*x5
dpdx3(258) = 5*x2*x3**4*x4+5*x1*x3**4*x4
dpdx3(259) = 5*x1*x2*x3**4
dpdx3(260) = x1**2*x6**4+4*x3**3*x4**2*x6+x2**2*x5**4+4*x3**3*x4*&
&*2*x5
dpdx3(261) = 2*x1**2*x3*x6**3+3*x3**2*x4**2*x6**2+2*x2**2*x3*x5**&
&3+3*x3**2*x4**2*x5**2
dpdx3(262) = 2*x3*x4**2*x6**3+3*x1**2*x3**2*x6**2+2*x3*x4**2*x5**&
&3+3*x2**2*x3**2*x5**2
dpdx3(263) = 2*x2**2*x3*x6**3+3*x2**2*x3**2*x6**2+2*x1**2*x3*x5**&
&3+3*x1**2*x3**2*x5**2
dpdx3(264) = x4**2*x6**4+4*x1**2*x3**3*x6+x4**2*x5**4+4*x2**2*x3*&
&*3*x5
dpdx3(265) = x2**2*x6**4+4*x2**2*x3**3*x6+x1**2*x5**4+4*x1**2*x3*&
&*3*x5
dpdx3(266) = 4*x2*x3**3*x4**2+4*x1*x3**3*x4**2
dpdx3(267) = 4*x2**2*x3**3*x4+4*x1**2*x3**3*x4
dpdx3(268) = 4*x1*x2**2*x3**3+4*x1**2*x2*x3**3
dpdx3(269) = 3*x2**2*x3**2*x4**2+3*x1**2*x3**2*x4**2
dpdx3(270) = 3*x1**2*x2**2*x3**2
dpdx3(271) = x1**3*x6**3+3*x3**2*x4**3*x6+x2**3*x5**3+3*x3**2*x4*&
&*3*x5
dpdx3(272) = 2*x3*x4**3*x6**2+2*x1**3*x3*x6**2+2*x3*x4**3*x5**2+2&
&*x2**3*x3*x5**2
dpdx3(273) = 2*x2**3*x3*x6**2+2*x1**3*x3*x5**2
dpdx3(274) = x4**3*x6**3+3*x1**3*x3**2*x6+x4**3*x5**3+3*x2**3*x3*&
&*2*x5
dpdx3(275) = x2**3*x6**3+3*x2**3*x3**2*x6+x1**3*x5**3+3*x1**3*x3*&
&*2*x5
dpdx3(276) = 3*x2*x3**2*x4**3+3*x1*x3**2*x4**3
dpdx3(277) = 3*x2**3*x3**2*x4+3*x1**3*x3**2*x4
dpdx3(278) = 3*x1*x2**3*x3**2+3*x1**3*x2*x3**2
dpdx3(279) = 2*x2**2*x3*x4**3+2*x1**2*x3*x4**3
dpdx3(280) = 2*x2**3*x3*x4**2+2*x1**3*x3*x4**2
dpdx3(281) = 0
dpdx3(282) = 2*x1**2*x2**3*x3+2*x1**3*x2**2*x3
dpdx3(283) = x2**3*x4**3+x1**3*x4**3
dpdx3(284) = 0
dpdx3(285) = x1**3*x2**3
dpdx3(286) = x1**4*x6**2+2*x3*x4**4*x6+x2**4*x5**2+2*x3*x4**4*x5
dpdx3(287) = x4**4*x6**2+2*x1**4*x3*x6+x4**4*x5**2+2*x2**4*x3*x5
dpdx3(288) = x2**4*x6**2+2*x2**4*x3*x6+x1**4*x5**2+2*x1**4*x3*x5
dpdx3(289) = 2*x2*x3*x4**4+2*x1*x3*x4**4
dpdx3(290) = 2*x2**4*x3*x4+2*x1**4*x3*x4
dpdx3(291) = 2*x1*x2**4*x3+2*x1**4*x2*x3
dpdx3(292) = x2**2*x4**4+x1**2*x4**4
dpdx3(293) = x2**4*x4**2+x1**4*x4**2
dpdx3(294) = 0
dpdx3(295) = x1**2*x2**4+x1**4*x2**2
dpdx3(296) = x4**5*x6+x1**5*x6+x4**5*x5+x2**5*x5
dpdx3(297) = x2**5*x6+x1**5*x5
dpdx3(298) = x2*x4**5+x1*x4**5
dpdx3(299) = x2**5*x4+x1**5*x4
dpdx3(300) = 0
dpdx3(301) = x1*x2**5+x1**5*x2
dpdx3(302) = 3*x3**2*x6**4+4*x3**3*x6**3+3*x3**2*x5**4+4*x3**3*x5&
&**3
dpdx3(303) = 2*x3*x6**5+5*x3**4*x6**2+2*x3*x5**5+5*x3**4*x5**2
dpdx3(304) = x6**6+6*x3**5*x6+x5**6+6*x3**5*x5
dpdx3(305) = 6*x2*x3**5+6*x1*x3**5
dpdx3(306) = 5*x2**2*x3**4+5*x1**2*x3**4
dpdx3(307) = 4*x2**3*x3**3+4*x1**3*x3**3
dpdx3(308) = 3*x2**4*x3**2+3*x1**4*x3**2
dpdx3(309) = 0
dpdx3(310) = 2*x2**5*x3+2*x1**5*x3
dpdx3(311) = 0
dpdx3(312) = x2**6+x1**6
dpdx3(313) = 0
dpdx3(314) = x1*x2*x4*x5**2*x6**2+2*x1*x2*x3*x4*x5*x6**2+2*x1*x2*&
&x3*x4*x5**2*x6
dpdx3(315) = x1*x2*x4*x5*x6**3+x1*x2*x4*x5**3*x6+3*x1*x2*x3**2*x4&
&*x5*x6
dpdx3(316) = x1**2*x2*x4*x5*x6**2+x1*x2**2*x4*x5**2*x6+2*x1*x2*x3&
&*x4**2*x5*x6
dpdx3(317) = x1*x2*x4**2*x5*x6**2+x1*x2**2*x4*x5*x6**2+x1*x2*x4**&
&2*x5**2*x6+x1**2*x2*x4*x5**2*x6+2*x1*x2**2*x3*x4*x5*x6+2*x1**2*x2&
&*x3*x4*x5*x6
dpdx3(318) = x1*x2**2*x4**2*x5*x6+x1**2*x2*x4**2*x5*x6+x1**2*x2**&
&2*x4*x5*x6
dpdx3(319) = x1*x2*x4**3*x5*x6+x1*x2**3*x4*x5*x6+x1**3*x2*x4*x5*x&
&6
dpdx3(320) = x1*x2*x5**2*x6**3+2*x1*x3*x4*x5*x6**3+x1*x2*x5**3*x6&
&**2+3*x1*x3**2*x4*x5*x6**2+2*x2*x3*x4*x5**3*x6+3*x2*x3**2*x4*x5**&
&2*x6
dpdx3(321) = x1*x4*x5*x6**4+x1*x2*x5*x6**4+x2*x4*x5**4*x6+x1*x2*x&
&5**4*x6+4*x2*x3**3*x4*x5*x6+4*x1*x3**3*x4*x5*x6
dpdx3(322) = 2*x2*x3*x4*x5**2*x6**2+2*x1*x3*x4*x5**2*x6**2+2*x1*x&
&2*x3*x5**2*x6**2
dpdx3(323) = x1*x4*x5**2*x6**3+2*x1*x2*x3*x5*x6**3+x2*x4*x5**3*x6&
&**2+3*x2*x3**2*x4*x5*x6**2+2*x1*x2*x3*x5**3*x6+3*x1*x3**2*x4*x5**&
&2*x6
dpdx3(324) = x2*x4*x5**2*x6**3+2*x2*x3*x4*x5*x6**3+x1*x4*x5**3*x6&
&**2+3*x1*x2*x3**2*x5*x6**2+2*x1*x3*x4*x5**3*x6+3*x1*x2*x3**2*x5**&
&2*x6
dpdx3(325) = 2*x1*x2*x3*x4*x6**3+3*x1*x2*x3**2*x4*x6**2+2*x1*x2*x&
&3*x4*x5**3+3*x1*x2*x3**2*x4*x5**2
dpdx3(326) = x2*x4*x5*x6**4+x1*x4*x5**4*x6+4*x1*x2*x3**3*x5*x6
dpdx3(327) = x1*x2*x4*x6**4+4*x1*x2*x3**3*x4*x6+x1*x2*x4*x5**4+4*&
&x1*x2*x3**3*x4*x5
dpdx3(328) = x1**2*x4*x5*x6**3+x1**2*x2*x5*x6**3+x2**2*x4*x5**3*x&
&6+x1*x2**2*x5**3*x6+3*x2*x3**2*x4**2*x5*x6+3*x1*x3**2*x4**2*x5*x6
dpdx3(329) = x1*x2**2*x5**2*x6**2+x1**2*x2*x5**2*x6**2+2*x1*x3*x4&
&**2*x5*x6**2+2*x1**2*x3*x4*x5*x6**2+2*x2*x3*x4**2*x5**2*x6+2*x2**&
&2*x3*x4*x5**2*x6
dpdx3(330) = x1*x4**2*x5*x6**3+x1*x2**2*x5*x6**3+x2*x4**2*x5**3*x&
&6+x1**2*x2*x5**3*x6+3*x2**2*x3**2*x4*x5*x6+3*x1**2*x3**2*x4*x5*x6
dpdx3(331) = x1**2*x2*x4*x6**3+3*x1*x2*x3**2*x4**2*x6+x1*x2**2*x4&
&*x5**3+3*x1*x2*x3**2*x4**2*x5
dpdx3(332) = x2**2*x4*x5**2*x6**2+x1**2*x4*x5**2*x6**2+2*x2*x3*x4&
&**2*x5*x6**2+2*x1**2*x2*x3*x5*x6**2+2*x1*x3*x4**2*x5**2*x6+2*x1*x&
&2**2*x3*x5**2*x6
dpdx3(333) = x2*x4**2*x5**2*x6**2+x1*x4**2*x5**2*x6**2+2*x2**2*x3&
&*x4*x5*x6**2+2*x1*x2**2*x3*x5*x6**2+2*x1**2*x3*x4*x5**2*x6+2*x1**&
&2*x2*x3*x5**2*x6
dpdx3(334) = 2*x1*x2*x3*x4**2*x6**2+2*x1**2*x2*x3*x4*x6**2+2*x1*x&
&2*x3*x4**2*x5**2+2*x1*x2**2*x3*x4*x5**2
dpdx3(335) = 2*x1*x2**2*x3*x4*x6**2+2*x1**2*x2*x3*x4*x5**2
dpdx3(336) = x2*x4**2*x5*x6**3+x2**2*x4*x5*x6**3+x1*x4**2*x5**3*x&
&6+x1**2*x4*x5**3*x6+3*x1*x2**2*x3**2*x5*x6+3*x1**2*x2*x3**2*x5*x6
dpdx3(337) = x1*x2*x4**2*x6**3+3*x1**2*x2*x3**2*x4*x6+x1*x2*x4**2&
&*x5**3+3*x1*x2**2*x3**2*x4*x5
dpdx3(338) = x1*x2**2*x4*x6**3+3*x1*x2**2*x3**2*x4*x6+x1**2*x2*x4&
&*x5**3+3*x1**2*x2*x3**2*x4*x5
dpdx3(339) = x1**2*x2*x4**2*x6**2+2*x1**2*x2*x3*x4**2*x6+x1*x2**2&
&*x4**2*x5**2+2*x1*x2**2*x3*x4**2*x5
dpdx3(340) = x1**2*x4**2*x5*x6**2+x1**2*x2**2*x5*x6**2+x2**2*x4**&
&2*x5**2*x6+x1**2*x2**2*x5**2*x6+2*x2**2*x3*x4**2*x5*x6+2*x1**2*x3&
&*x4**2*x5*x6
dpdx3(341) = x1**2*x2**2*x4*x6**2+2*x1*x2**2*x3*x4**2*x6+x1**2*x2&
&**2*x4*x5**2+2*x1**2*x2*x3*x4**2*x5
dpdx3(342) = x1**2*x2**2*x4**2*x6+x1**2*x2**2*x4**2*x5
dpdx3(343) = x2**2*x4**2*x5*x6**2+x1**2*x4**2*x5**2*x6+2*x1**2*x2&
&**2*x3*x5*x6
dpdx3(344) = x1*x2**2*x4**2*x6**2+2*x1**2*x2**2*x3*x4*x6+x1**2*x2&
&*x4**2*x5**2+2*x1**2*x2**2*x3*x4*x5
dpdx3(345) = x1**3*x4*x5*x6**2+x1**3*x2*x5*x6**2+x2**3*x4*x5**2*x&
&6+x1*x2**3*x5**2*x6+2*x2*x3*x4**3*x5*x6+2*x1*x3*x4**3*x5*x6
dpdx3(346) = x1*x4**3*x5*x6**2+x1*x2**3*x5*x6**2+x2*x4**3*x5**2*x&
&6+x1**3*x2*x5**2*x6+2*x2**3*x3*x4*x5*x6+2*x1**3*x3*x4*x5*x6
dpdx3(347) = x1**3*x2*x4*x6**2+2*x1*x2*x3*x4**3*x6+x1*x2**3*x4*x5&
&**2+2*x1*x2*x3*x4**3*x5
dpdx3(348) = x2*x4**3*x5*x6**2+x2**3*x4*x5*x6**2+x1*x4**3*x5**2*x&
&6+x1**3*x4*x5**2*x6+2*x1*x2**3*x3*x5*x6+2*x1**3*x2*x3*x5*x6
dpdx3(349) = x1*x2*x4**3*x6**2+2*x1**3*x2*x3*x4*x6+x1*x2*x4**3*x5&
&**2+2*x1*x2**3*x3*x4*x5
dpdx3(350) = x1*x2**3*x4*x6**2+2*x1*x2**3*x3*x4*x6+x1**3*x2*x4*x5&
&**2+2*x1**3*x2*x3*x4*x5
dpdx3(351) = x1**2*x2*x4**3*x6+x1**3*x2*x4**2*x6+x1*x2**2*x4**3*x&
&5+x1*x2**3*x4**2*x5
dpdx3(352) = x2**2*x4**3*x5*x6+x1**2*x4**3*x5*x6+x2**3*x4**2*x5*x&
&6+x1**3*x4**2*x5*x6+x1**2*x2**3*x5*x6+x1**3*x2**2*x5*x6
dpdx3(353) = x1*x2**2*x4**3*x6+x1**3*x2**2*x4*x6+x1**2*x2*x4**3*x&
&5+x1**2*x2**3*x4*x5
dpdx3(354) = x1*x2**3*x4**2*x6+x1**2*x2**3*x4*x6+x1**3*x2*x4**2*x&
&5+x1**3*x2**2*x4*x5
dpdx3(355) = x2*x4**4*x5*x6+x1*x4**4*x5*x6+x2**4*x4*x5*x6+x1**4*x&
&4*x5*x6+x1*x2**4*x5*x6+x1**4*x2*x5*x6
dpdx3(356) = x1*x2*x4**4*x6+x1**4*x2*x4*x6+x1*x2*x4**4*x5+x1*x2**&
&4*x4*x5
dpdx3(357) = x1*x2**4*x4*x6+x1**4*x2*x4*x5
dpdx3(358) = x1*x5*x6**5+x2*x5**5*x6+5*x3**4*x4*x5*x6
dpdx3(359) = x1*x5**2*x6**4+2*x1*x3*x5*x6**4+x2*x5**4*x6**2+4*x3*&
&*3*x4*x5*x6**2+2*x2*x3*x5**4*x6+4*x3**3*x4*x5**2*x6
dpdx3(360) = 2*x1*x3*x5**2*x6**3+2*x2*x3*x5**3*x6**2+3*x3**2*x4*x&
&5**2*x6**2
dpdx3(361) = x2*x5**3*x6**3+x1*x5**3*x6**3+3*x3**2*x4*x5*x6**3+3*&
&x1*x3**2*x5*x6**3+3*x3**2*x4*x5**3*x6+3*x2*x3**2*x5**3*x6
dpdx3(362) = 2*x3*x4*x5**2*x6**3+2*x2*x3*x5**2*x6**3+2*x3*x4*x5**&
&3*x6**2+2*x1*x3*x5**3*x6**2+3*x2*x3**2*x5**2*x6**2+3*x1*x3**2*x5*&
&*2*x6**2
dpdx3(363) = x4*x5**3*x6**3+3*x2*x3**2*x5*x6**3+3*x1*x3**2*x5**3*&
&x6
dpdx3(364) = x2*x5**2*x6**4+2*x3*x4*x5*x6**4+x1*x5**4*x6**2+4*x1*&
&x3**3*x5*x6**2+2*x3*x4*x5**4*x6+4*x2*x3**3*x5**2*x6
dpdx3(365) = x4*x5**2*x6**4+2*x2*x3*x5*x6**4+x4*x5**4*x6**2+4*x2*&
&x3**3*x5*x6**2+2*x1*x3*x5**4*x6+4*x1*x3**3*x5**2*x6
dpdx3(366) = x4*x5*x6**5+x2*x5*x6**5+x4*x5**5*x6+x1*x5**5*x6+5*x2&
&*x3**4*x5*x6+5*x1*x3**4*x5*x6
dpdx3(367) = 3*x1*x3**2*x4*x6**3+3*x2*x3**2*x4*x5**3
dpdx3(368) = 2*x1*x3*x4*x6**4+4*x1*x3**3*x4*x6**2+2*x2*x3*x4*x5**&
&4+4*x2*x3**3*x4*x5**2
dpdx3(369) = x1*x4*x6**5+5*x1*x3**4*x4*x6+x2*x4*x5**5+5*x2*x3**4*&
&x4*x5
dpdx3(370) = x1*x2*x6**5+5*x2*x3**4*x4*x6+x1*x2*x5**5+5*x1*x3**4*&
&x4*x5
dpdx3(371) = 2*x1*x2*x3*x6**4+4*x2*x3**3*x4*x6**2+2*x1*x2*x3*x5**&
&4+4*x1*x3**3*x4*x5**2
dpdx3(372) = 3*x2*x3**2*x4*x6**3+3*x1*x2*x3**2*x6**3+3*x1*x3**2*x&
&4*x5**3+3*x1*x2*x3**2*x5**3
dpdx3(373) = 2*x2*x3*x4*x6**4+4*x1*x2*x3**3*x6**2+2*x1*x3*x4*x5**&
&4+4*x1*x2*x3**3*x5**2
dpdx3(374) = x2*x4*x6**5+5*x1*x2*x3**4*x6+x1*x4*x5**5+5*x1*x2*x3*&
&*4*x5
dpdx3(375) = 5*x1*x2*x3**4*x4
dpdx3(376) = x1**2*x5*x6**4+x2**2*x5**4*x6+4*x3**3*x4**2*x5*x6
dpdx3(377) = x1**2*x5**2*x6**3+2*x1**2*x3*x5*x6**3+x2**2*x5**3*x6&
&**2+3*x3**2*x4**2*x5*x6**2+2*x2**2*x3*x5**3*x6+3*x3**2*x4**2*x5**&
&2*x6
dpdx3(378) = 2*x3*x4**2*x5**2*x6**2+2*x2**2*x3*x5**2*x6**2+2*x1**&
&2*x3*x5**2*x6**2
dpdx3(379) = x2**2*x5**2*x6**3+2*x3*x4**2*x5*x6**3+x1**2*x5**3*x6&
&**2+3*x1**2*x3**2*x5*x6**2+2*x3*x4**2*x5**3*x6+3*x2**2*x3**2*x5**&
&2*x6
dpdx3(380) = x4**2*x5**2*x6**3+2*x2**2*x3*x5*x6**3+x4**2*x5**3*x6&
&**2+3*x2**2*x3**2*x5*x6**2+2*x1**2*x3*x5**3*x6+3*x1**2*x3**2*x5**&
&2*x6
dpdx3(381) = x4**2*x5*x6**4+x2**2*x5*x6**4+x4**2*x5**4*x6+x1**2*x&
&5**4*x6+4*x2**2*x3**3*x5*x6+4*x1**2*x3**3*x5*x6
dpdx3(382) = x1**2*x4*x6**4+4*x1*x3**3*x4**2*x6+x2**2*x4*x5**4+4*&
&x2*x3**3*x4**2*x5
dpdx3(383) = 2*x1**2*x3*x4*x6**3+3*x1*x3**2*x4**2*x6**2+2*x2**2*x&
&3*x4*x5**3+3*x2*x3**2*x4**2*x5**2
dpdx3(384) = 2*x1*x3*x4**2*x6**3+3*x1**2*x3**2*x4*x6**2+2*x2*x3*x&
&4**2*x5**3+3*x2**2*x3**2*x4*x5**2
dpdx3(385) = x1*x4**2*x6**4+4*x1**2*x3**3*x4*x6+x2*x4**2*x5**4+4*&
&x2**2*x3**3*x4*x5
dpdx3(386) = 4*x1*x2*x3**3*x4**2
dpdx3(387) = x1**2*x2*x6**4+4*x2*x3**3*x4**2*x6+x1*x2**2*x5**4+4*&
&x1*x3**3*x4**2*x5
dpdx3(388) = x1*x2**2*x6**4+4*x2**2*x3**3*x4*x6+x1**2*x2*x5**4+4*&
&x1**2*x3**3*x4*x5
dpdx3(389) = 2*x1**2*x2*x3*x6**3+3*x2*x3**2*x4**2*x6**2+2*x1*x2**&
&2*x3*x5**3+3*x1*x3**2*x4**2*x5**2
dpdx3(390) = 2*x1*x2**2*x3*x6**3+3*x2**2*x3**2*x4*x6**2+2*x1**2*x&
&2*x3*x5**3+3*x1**2*x3**2*x4*x5**2
dpdx3(391) = 2*x2*x3*x4**2*x6**3+3*x1**2*x2*x3**2*x6**2+2*x1*x3*x&
&4**2*x5**3+3*x1*x2**2*x3**2*x5**2
dpdx3(392) = 2*x2**2*x3*x4*x6**3+3*x1*x2**2*x3**2*x6**2+2*x1**2*x&
&3*x4*x5**3+3*x1**2*x2*x3**2*x5**2
dpdx3(393) = x2*x4**2*x6**4+4*x1**2*x2*x3**3*x6+x1*x4**2*x5**4+4*&
&x1*x2**2*x3**3*x5
dpdx3(394) = x2**2*x4*x6**4+4*x1*x2**2*x3**3*x6+x1**2*x4*x5**4+4*&
&x1**2*x2*x3**3*x5
dpdx3(395) = 4*x1*x2**2*x3**3*x4+4*x1**2*x2*x3**3*x4
dpdx3(396) = 2*x1**2*x3*x4**2*x6**2+2*x2**2*x3*x4**2*x5**2
dpdx3(397) = x1**2*x4**2*x6**3+3*x1**2*x3**2*x4**2*x6+x2**2*x4**2&
&*x5**3+3*x2**2*x3**2*x4**2*x5
dpdx3(398) = 3*x1*x2**2*x3**2*x4**2+3*x1**2*x2*x3**2*x4**2
dpdx3(399) = x1**2*x2**2*x6**3+3*x2**2*x3**2*x4**2*x6+x1**2*x2**2&
&*x5**3+3*x1**2*x3**2*x4**2*x5
dpdx3(400) = 2*x2**2*x3*x4**2*x6**2+2*x1**2*x2**2*x3*x6**2+2*x1**&
&2*x3*x4**2*x5**2+2*x1**2*x2**2*x3*x5**2
dpdx3(401) = 2*x1**2*x2**2*x3*x4**2
dpdx3(402) = x2**2*x4**2*x6**3+3*x1**2*x2**2*x3**2*x6+x1**2*x4**2&
&*x5**3+3*x1**2*x2**2*x3**2*x5
dpdx3(403) = 3*x1**2*x2**2*x3**2*x4
dpdx3(404) = x1**3*x5*x6**3+x2**3*x5**3*x6+3*x3**2*x4**3*x5*x6
dpdx3(405) = x2**3*x5**2*x6**2+x1**3*x5**2*x6**2+2*x3*x4**3*x5*x6&
&**2+2*x1**3*x3*x5*x6**2+2*x3*x4**3*x5**2*x6+2*x2**3*x3*x5**2*x6
dpdx3(406) = x4**3*x5**2*x6**2+2*x2**3*x3*x5*x6**2+2*x1**3*x3*x5*&
&*2*x6
dpdx3(407) = x4**3*x5*x6**3+x2**3*x5*x6**3+x4**3*x5**3*x6+x1**3*x&
&5**3*x6+3*x2**3*x3**2*x5*x6+3*x1**3*x3**2*x5*x6
dpdx3(408) = x1**3*x4*x6**3+3*x1*x3**2*x4**3*x6+x2**3*x4*x5**3+3*&
&x2*x3**2*x4**3*x5
dpdx3(409) = 2*x1*x3*x4**3*x6**2+2*x1**3*x3*x4*x6**2+2*x2*x3*x4**&
&3*x5**2+2*x2**3*x3*x4*x5**2
dpdx3(410) = x1*x4**3*x6**3+3*x1**3*x3**2*x4*x6+x2*x4**3*x5**3+3*&
&x2**3*x3**2*x4*x5
dpdx3(411) = 3*x1*x2*x3**2*x4**3
dpdx3(412) = x1**3*x2*x6**3+3*x2*x3**2*x4**3*x6+x1*x2**3*x5**3+3*&
&x1*x3**2*x4**3*x5
dpdx3(413) = x1*x2**3*x6**3+3*x2**3*x3**2*x4*x6+x1**3*x2*x5**3+3*&
&x1**3*x3**2*x4*x5
dpdx3(414) = 2*x2*x3*x4**3*x6**2+2*x1**3*x2*x3*x6**2+2*x1*x3*x4**&
&3*x5**2+2*x1*x2**3*x3*x5**2
dpdx3(415) = 2*x2**3*x3*x4*x6**2+2*x1*x2**3*x3*x6**2+2*x1**3*x3*x&
&4*x5**2+2*x1**3*x2*x3*x5**2
dpdx3(416) = x2*x4**3*x6**3+3*x1**3*x2*x3**2*x6+x1*x4**3*x5**3+3*&
&x1*x2**3*x3**2*x5
dpdx3(417) = x2**3*x4*x6**3+3*x1*x2**3*x3**2*x6+x1**3*x4*x5**3+3*&
&x1**3*x2*x3**2*x5
dpdx3(418) = 3*x1*x2**3*x3**2*x4+3*x1**3*x2*x3**2*x4
dpdx3(419) = x1**3*x4**2*x6**2+2*x1**2*x3*x4**3*x6+x2**3*x4**2*x5&
&**2+2*x2**2*x3*x4**3*x5
dpdx3(420) = x1**2*x4**3*x6**2+2*x1**3*x3*x4**2*x6+x2**2*x4**3*x5&
&**2+2*x2**3*x3*x4**2*x5
dpdx3(421) = 2*x1*x2**2*x3*x4**3+2*x1**2*x2*x3*x4**3
dpdx3(422) = 2*x1*x2**3*x3*x4**2+2*x1**3*x2*x3*x4**2
dpdx3(423) = x1**2*x2**2*x4**3
dpdx3(424) = x1**3*x2**2*x6**2+2*x2**2*x3*x4**3*x6+x1**2*x2**3*x5&
&**2+2*x1**2*x3*x4**3*x5
dpdx3(425) = x1**2*x2**3*x6**2+2*x2**3*x3*x4**2*x6+x1**3*x2**2*x5&
&**2+2*x1**3*x3*x4**2*x5
dpdx3(426) = x1**2*x2**3*x4**2+x1**3*x2**2*x4**2
dpdx3(427) = x2**2*x4**3*x6**2+2*x1**3*x2**2*x3*x6+x1**2*x4**3*x5&
&**2+2*x1**2*x2**3*x3*x5
dpdx3(428) = x2**3*x4**2*x6**2+2*x1**2*x2**3*x3*x6+x1**3*x4**2*x5&
&**2+2*x1**3*x2**2*x3*x5
dpdx3(429) = 2*x1**2*x2**3*x3*x4+2*x1**3*x2**2*x3*x4
dpdx3(430) = x1**3*x4**3*x6+x2**3*x4**3*x5
dpdx3(431) = x1*x2**3*x4**3+x1**3*x2*x4**3
dpdx3(432) = x2**3*x4**3*x6+x1**3*x2**3*x6+x1**3*x4**3*x5+x1**3*x&
&2**3*x5
dpdx3(433) = x1**3*x2**3*x4
dpdx3(434) = x1**4*x5*x6**2+x2**4*x5**2*x6+2*x3*x4**4*x5*x6
dpdx3(435) = x4**4*x5*x6**2+x2**4*x5*x6**2+x4**4*x5**2*x6+x1**4*x&
&5**2*x6+2*x2**4*x3*x5*x6+2*x1**4*x3*x5*x6
dpdx3(436) = x1**4*x4*x6**2+2*x1*x3*x4**4*x6+x2**4*x4*x5**2+2*x2*&
&x3*x4**4*x5
dpdx3(437) = x1*x4**4*x6**2+2*x1**4*x3*x4*x6+x2*x4**4*x5**2+2*x2*&
&*4*x3*x4*x5
dpdx3(438) = 2*x1*x2*x3*x4**4
dpdx3(439) = x1**4*x2*x6**2+2*x2*x3*x4**4*x6+x1*x2**4*x5**2+2*x1*&
&x3*x4**4*x5
dpdx3(440) = x1*x2**4*x6**2+2*x2**4*x3*x4*x6+x1**4*x2*x5**2+2*x1*&
&*4*x3*x4*x5
dpdx3(441) = x2*x4**4*x6**2+2*x1**4*x2*x3*x6+x1*x4**4*x5**2+2*x1*&
&x2**4*x3*x5
dpdx3(442) = x2**4*x4*x6**2+2*x1*x2**4*x3*x6+x1**4*x4*x5**2+2*x1*&
&*4*x2*x3*x5
dpdx3(443) = 2*x1*x2**4*x3*x4+2*x1**4*x2*x3*x4
dpdx3(444) = x1**2*x4**4*x6+x1**4*x4**2*x6+x2**2*x4**4*x5+x2**4*x&
&4**2*x5
dpdx3(445) = x1*x2**2*x4**4+x1**2*x2*x4**4
dpdx3(446) = x1*x2**4*x4**2+x1**4*x2*x4**2
dpdx3(447) = x2**2*x4**4*x6+x1**4*x2**2*x6+x1**2*x4**4*x5+x1**2*x&
&2**4*x5
dpdx3(448) = x2**4*x4**2*x6+x1**2*x2**4*x6+x1**4*x4**2*x5+x1**4*x&
&2**2*x5
dpdx3(449) = x1**2*x2**4*x4+x1**4*x2**2*x4
dpdx3(450) = x4**5*x5*x6+x2**5*x5*x6+x1**5*x5*x6
dpdx3(451) = x1*x4**5*x6+x1**5*x4*x6+x2*x4**5*x5+x2**5*x4*x5
dpdx3(452) = x1*x2*x4**5
dpdx3(453) = x2*x4**5*x6+x1**5*x2*x6+x1*x4**5*x5+x1*x2**5*x5
dpdx3(454) = x2**5*x4*x6+x1*x2**5*x6+x1**5*x4*x5+x1**5*x2*x5
dpdx3(455) = x1*x2**5*x4+x1**5*x2*x4
dpdx3(456) = 2*x3*x5**3*x6**3+3*x3**2*x5**2*x6**3+3*x3**2*x5**3*x&
&6**2
dpdx3(457) = 2*x3*x5**2*x6**4+2*x3*x5**4*x6**2+4*x3**3*x5**2*x6**&
&2
dpdx3(458) = x5**3*x6**4+3*x3**2*x5*x6**4+x5**4*x6**3+4*x3**3*x5*&
&x6**3+3*x3**2*x5**4*x6+4*x3**3*x5**3*x6
dpdx3(459) = x5**2*x6**5+2*x3*x5*x6**5+x5**5*x6**2+5*x3**4*x5*x6*&
&*2+2*x3*x5**5*x6+5*x3**4*x5**2*x6
dpdx3(460) = x5*x6**6+x5**6*x6+6*x3**5*x5*x6
dpdx3(461) = x1*x6**6+6*x3**5*x4*x6+x2*x5**6+6*x3**5*x4*x5
dpdx3(462) = 2*x1*x3*x6**5+5*x3**4*x4*x6**2+2*x2*x3*x5**5+5*x3**4&
&*x4*x5**2
dpdx3(463) = 3*x1*x3**2*x6**4+4*x3**3*x4*x6**3+3*x2*x3**2*x5**4+4&
&*x3**3*x4*x5**3
dpdx3(464) = 3*x3**2*x4*x6**4+4*x1*x3**3*x6**3+3*x3**2*x4*x5**4+4&
&*x2*x3**3*x5**3
dpdx3(465) = 3*x2*x3**2*x6**4+4*x2*x3**3*x6**3+3*x1*x3**2*x5**4+4&
&*x1*x3**3*x5**3
dpdx3(466) = 2*x3*x4*x6**5+5*x1*x3**4*x6**2+2*x3*x4*x5**5+5*x2*x3&
&**4*x5**2
dpdx3(467) = 2*x2*x3*x6**5+5*x2*x3**4*x6**2+2*x1*x3*x5**5+5*x1*x3&
&**4*x5**2
dpdx3(468) = x4*x6**6+6*x1*x3**5*x6+x4*x5**6+6*x2*x3**5*x5
dpdx3(469) = x2*x6**6+6*x2*x3**5*x6+x1*x5**6+6*x1*x3**5*x5
dpdx3(470) = 6*x2*x3**5*x4+6*x1*x3**5*x4
dpdx3(471) = 6*x1*x2*x3**5
dpdx3(472) = x1**2*x6**5+5*x3**4*x4**2*x6+x2**2*x5**5+5*x3**4*x4*&
&*2*x5
dpdx3(473) = 2*x1**2*x3*x6**4+4*x3**3*x4**2*x6**2+2*x2**2*x3*x5**&
&4+4*x3**3*x4**2*x5**2
dpdx3(474) = 3*x3**2*x4**2*x6**3+3*x1**2*x3**2*x6**3+3*x3**2*x4**&
&2*x5**3+3*x2**2*x3**2*x5**3
dpdx3(475) = 3*x2**2*x3**2*x6**3+3*x1**2*x3**2*x5**3
dpdx3(476) = 2*x3*x4**2*x6**4+4*x1**2*x3**3*x6**2+2*x3*x4**2*x5**&
&4+4*x2**2*x3**3*x5**2
dpdx3(477) = 2*x2**2*x3*x6**4+4*x2**2*x3**3*x6**2+2*x1**2*x3*x5**&
&4+4*x1**2*x3**3*x5**2
dpdx3(478) = x4**2*x6**5+5*x1**2*x3**4*x6+x4**2*x5**5+5*x2**2*x3*&
&*4*x5
dpdx3(479) = x2**2*x6**5+5*x2**2*x3**4*x6+x1**2*x5**5+5*x1**2*x3*&
&*4*x5
dpdx3(480) = 5*x2*x3**4*x4**2+5*x1*x3**4*x4**2
dpdx3(481) = 5*x2**2*x3**4*x4+5*x1**2*x3**4*x4
dpdx3(482) = 5*x1*x2**2*x3**4+5*x1**2*x2*x3**4
dpdx3(483) = 4*x2**2*x3**3*x4**2+4*x1**2*x3**3*x4**2
dpdx3(484) = 4*x1**2*x2**2*x3**3
dpdx3(485) = x1**3*x6**4+4*x3**3*x4**3*x6+x2**3*x5**4+4*x3**3*x4*&
&*3*x5
dpdx3(486) = 2*x1**3*x3*x6**3+3*x3**2*x4**3*x6**2+2*x2**3*x3*x5**&
&3+3*x3**2*x4**3*x5**2
dpdx3(487) = 2*x3*x4**3*x6**3+3*x1**3*x3**2*x6**2+2*x3*x4**3*x5**&
&3+3*x2**3*x3**2*x5**2
dpdx3(488) = 2*x2**3*x3*x6**3+3*x2**3*x3**2*x6**2+2*x1**3*x3*x5**&
&3+3*x1**3*x3**2*x5**2
dpdx3(489) = x4**3*x6**4+4*x1**3*x3**3*x6+x4**3*x5**4+4*x2**3*x3*&
&*3*x5
dpdx3(490) = x2**3*x6**4+4*x2**3*x3**3*x6+x1**3*x5**4+4*x1**3*x3*&
&*3*x5
dpdx3(491) = 4*x2*x3**3*x4**3+4*x1*x3**3*x4**3
dpdx3(492) = 4*x2**3*x3**3*x4+4*x1**3*x3**3*x4
dpdx3(493) = 4*x1*x2**3*x3**3+4*x1**3*x2*x3**3
dpdx3(494) = 3*x2**2*x3**2*x4**3+3*x1**2*x3**2*x4**3
dpdx3(495) = 3*x2**3*x3**2*x4**2+3*x1**3*x3**2*x4**2
dpdx3(496) = 3*x1**2*x2**3*x3**2+3*x1**3*x2**2*x3**2
dpdx3(497) = 2*x2**3*x3*x4**3+2*x1**3*x3*x4**3
dpdx3(498) = 0
dpdx3(499) = 2*x1**3*x2**3*x3
dpdx3(500) = x1**4*x6**3+3*x3**2*x4**4*x6+x2**4*x5**3+3*x3**2*x4*&
&*4*x5
dpdx3(501) = 2*x3*x4**4*x6**2+2*x1**4*x3*x6**2+2*x3*x4**4*x5**2+2&
&*x2**4*x3*x5**2
dpdx3(502) = 2*x2**4*x3*x6**2+2*x1**4*x3*x5**2
dpdx3(503) = x4**4*x6**3+3*x1**4*x3**2*x6+x4**4*x5**3+3*x2**4*x3*&
&*2*x5
dpdx3(504) = x2**4*x6**3+3*x2**4*x3**2*x6+x1**4*x5**3+3*x1**4*x3*&
&*2*x5
dpdx3(505) = 3*x2*x3**2*x4**4+3*x1*x3**2*x4**4
dpdx3(506) = 3*x2**4*x3**2*x4+3*x1**4*x3**2*x4
dpdx3(507) = 3*x1*x2**4*x3**2+3*x1**4*x2*x3**2
dpdx3(508) = 2*x2**2*x3*x4**4+2*x1**2*x3*x4**4
dpdx3(509) = 2*x2**4*x3*x4**2+2*x1**4*x3*x4**2
dpdx3(510) = 0
dpdx3(511) = 2*x1**2*x2**4*x3+2*x1**4*x2**2*x3
dpdx3(512) = x2**3*x4**4+x1**3*x4**4
dpdx3(513) = x2**4*x4**3+x1**4*x4**3
dpdx3(514) = 0
dpdx3(515) = x1**3*x2**4+x1**4*x2**3
dpdx3(516) = x1**5*x6**2+2*x3*x4**5*x6+x2**5*x5**2+2*x3*x4**5*x5
dpdx3(517) = x4**5*x6**2+2*x1**5*x3*x6+x4**5*x5**2+2*x2**5*x3*x5
dpdx3(518) = x2**5*x6**2+2*x2**5*x3*x6+x1**5*x5**2+2*x1**5*x3*x5
dpdx3(519) = 2*x2*x3*x4**5+2*x1*x3*x4**5
dpdx3(520) = 2*x2**5*x3*x4+2*x1**5*x3*x4
dpdx3(521) = 2*x1*x2**5*x3+2*x1**5*x2*x3
dpdx3(522) = x2**2*x4**5+x1**2*x4**5
dpdx3(523) = x2**5*x4**2+x1**5*x4**2
dpdx3(524) = 0
dpdx3(525) = x1**2*x2**5+x1**5*x2**2
dpdx3(526) = x4**6*x6+x1**6*x6+x4**6*x5+x2**6*x5
dpdx3(527) = x2**6*x6+x1**6*x5
dpdx3(528) = x2*x4**6+x1*x4**6
dpdx3(529) = x2**6*x4+x1**6*x4
dpdx3(530) = 0
dpdx3(531) = x1*x2**6+x1**6*x2
dpdx3(532) = 4*x3**3*x6**4+4*x3**3*x5**4
dpdx3(533) = 3*x3**2*x6**5+5*x3**4*x6**3+3*x3**2*x5**5+5*x3**4*x5&
&**3
dpdx3(534) = 2*x3*x6**6+6*x3**5*x6**2+2*x3*x5**6+6*x3**5*x5**2
dpdx3(535) = x6**7+7*x3**6*x6+x5**7+7*x3**6*x5
dpdx3(536) = 7*x2*x3**6+7*x1*x3**6
dpdx3(537) = 6*x2**2*x3**5+6*x1**2*x3**5
dpdx3(538) = 5*x2**3*x3**4+5*x1**3*x3**4
dpdx3(539) = 4*x2**4*x3**3+4*x1**4*x3**3
dpdx3(540) = 0
dpdx3(541) = 3*x2**5*x3**2+3*x1**5*x3**2
dpdx3(542) = 0
dpdx3(543) = 2*x2**6*x3+2*x1**6*x3
dpdx3(544) = 0
dpdx3(545) = x2**7+x1**7
dpdx3(546) = 0
dpdx4(1) = 0
dpdx4(2) = 0
dpdx4(3) = x6+x5
dpdx4(4) = x2+x1
dpdx4(5) = 0
dpdx4(6) = x3*x6+x3*x5
dpdx4(7) = x5*x6
dpdx4(8) = x1*x6+x2*x5+x2*x3+x1*x3
dpdx4(9) = x1*x2
dpdx4(10) = x2*x6+x1*x5
dpdx4(11) = 0
dpdx4(12) = x6**2+x5**2
dpdx4(13) = 2*x4*x6+2*x4*x5
dpdx4(14) = 2*x2*x4+2*x1*x4+x2**2+x1**2
dpdx4(15) = x3*x5*x6
dpdx4(16) = x1*x3*x6+x2*x3*x5
dpdx4(17) = x2*x5*x6+x1*x5*x6+x2*x3*x6+x1*x3*x5
dpdx4(18) = x1*x2*x6+x1*x2*x5+x1*x2*x3
dpdx4(19) = 0
dpdx4(20) = x3**2*x6+x3**2*x5
dpdx4(21) = x3*x6**2+x3*x5**2
dpdx4(22) = x5*x6**2+x5**2*x6
dpdx4(23) = x1*x6**2+x2*x5**2+x2*x3**2+x1*x3**2
dpdx4(24) = x2*x6**2+x1*x5**2
dpdx4(25) = 2*x3*x4*x6+2*x3*x4*x5
dpdx4(26) = 2*x4*x5*x6
dpdx4(27) = x1**2*x6+x2**2*x5+2*x2*x3*x4+2*x1*x3*x4
dpdx4(28) = 2*x1*x4*x6+2*x2*x4*x5+x2**2*x3+x1**2*x3
dpdx4(29) = 2*x1*x2*x4+x1*x2**2+x1**2*x2
dpdx4(30) = 2*x2*x4*x6+x2**2*x6+2*x1*x4*x5+x1**2*x5
dpdx4(31) = 0
dpdx4(32) = 0
dpdx4(33) = x6**3+x5**3
dpdx4(34) = 2*x4*x6**2+2*x4*x5**2
dpdx4(35) = 2*x2**2*x4+2*x1**2*x4
dpdx4(36) = 3*x4**2*x6+3*x4**2*x5
dpdx4(37) = 3*x2*x4**2+3*x1*x4**2+x2**3+x1**3
dpdx4(38) = x2*x3*x5*x6+x1*x3*x5*x6
dpdx4(39) = x1*x2*x5*x6+x1*x2*x3*x6+x1*x2*x3*x5
dpdx4(40) = x3**2*x5*x6
dpdx4(41) = x3*x5*x6**2+x3*x5**2*x6
dpdx4(42) = x1*x3*x6**2+x1*x3**2*x6+x2*x3*x5**2+x2*x3**2*x5
dpdx4(43) = x1*x5*x6**2+x2*x5**2*x6+x2*x3**2*x6+x1*x3**2*x5
dpdx4(44) = x2*x5*x6**2+x2*x3*x6**2+x1*x5**2*x6+x1*x3*x5**2
dpdx4(45) = x1*x2*x6**2+x1*x2*x5**2+x1*x2*x3**2
dpdx4(46) = 2*x3*x4*x5*x6
dpdx4(47) = 2*x1*x3*x4*x6+x1**2*x3*x6+2*x2*x3*x4*x5+x2**2*x3*x5
dpdx4(48) = x1**2*x2*x6+x1*x2**2*x5+2*x1*x2*x3*x4
dpdx4(49) = x2**2*x5*x6+x1**2*x5*x6+2*x2*x3*x4*x6+2*x1*x3*x4*x5
dpdx4(50) = 2*x2*x4*x5*x6+2*x1*x4*x5*x6+x2**2*x3*x6+x1**2*x3*x5
dpdx4(51) = 2*x1*x2*x4*x6+x1*x2**2*x6+2*x1*x2*x4*x5+x1**2*x2*x5+x&
&1*x2**2*x3+x1**2*x2*x3
dpdx4(52) = 0
dpdx4(53) = 0
dpdx4(54) = x3**3*x6+x3**3*x5
dpdx4(55) = x3**2*x6**2+x3**2*x5**2
dpdx4(56) = x5**2*x6**2
dpdx4(57) = x3*x6**3+x3*x5**3
dpdx4(58) = x5*x6**3+x5**3*x6
dpdx4(59) = x1*x6**3+x2*x5**3+x2*x3**3+x1*x3**3
dpdx4(60) = x2*x6**3+x1*x5**3
dpdx4(61) = 2*x3**2*x4*x6+2*x3**2*x4*x5
dpdx4(62) = 2*x3*x4*x6**2+2*x3*x4*x5**2
dpdx4(63) = 2*x4*x5*x6**2+2*x4*x5**2*x6
dpdx4(64) = x1**2*x6**2+x2**2*x5**2+2*x2*x3**2*x4+2*x1*x3**2*x4
dpdx4(65) = 2*x1*x4*x6**2+2*x2*x4*x5**2+x2**2*x3**2+x1**2*x3**2
dpdx4(66) = 2*x2*x4*x6**2+x2**2*x6**2+2*x1*x4*x5**2+x1**2*x5**2
dpdx4(67) = 2*x1**2*x4*x6+2*x2**2*x4*x5+2*x2**2*x3*x4+2*x1**2*x3*&
&x4
dpdx4(68) = 2*x1*x2**2*x4+2*x1**2*x2*x4+x1**2*x2**2
dpdx4(69) = 2*x2**2*x4*x6+2*x1**2*x4*x5
dpdx4(70) = 3*x3*x4**2*x6+3*x3*x4**2*x5
dpdx4(71) = 3*x4**2*x5*x6
dpdx4(72) = x1**3*x6+x2**3*x5+3*x2*x3*x4**2+3*x1*x3*x4**2
dpdx4(73) = 3*x1*x4**2*x6+3*x2*x4**2*x5+x2**3*x3+x1**3*x3
dpdx4(74) = 3*x1*x2*x4**2+x1*x2**3+x1**3*x2
dpdx4(75) = 3*x2*x4**2*x6+x2**3*x6+3*x1*x4**2*x5+x1**3*x5
dpdx4(76) = 0
dpdx4(77) = 0
dpdx4(78) = x6**4+x5**4
dpdx4(79) = 2*x4*x6**3+2*x4*x5**3
dpdx4(80) = 3*x4**2*x6**2+3*x4**2*x5**2
dpdx4(81) = 3*x2**2*x4**2+3*x1**2*x4**2+2*x2**3*x4+2*x1**3*x4
dpdx4(82) = 4*x4**3*x6+4*x4**3*x5
dpdx4(83) = 4*x2*x4**3+4*x1*x4**3+x2**4+x1**4
dpdx4(84) = x1*x2*x3*x5*x6
dpdx4(85) = x1*x3*x5*x6**2+x2*x3*x5**2*x6+x2*x3**2*x5*x6+x1*x3**2&
&*x5*x6
dpdx4(86) = x2*x3*x5*x6**2+x1*x3*x5**2*x6
dpdx4(87) = x1*x2*x5*x6**2+x1*x2*x3*x6**2+x1*x2*x5**2*x6+x1*x2*x3&
&**2*x6+x1*x2*x3*x5**2+x1*x2*x3**2*x5
dpdx4(88) = 2*x2*x3*x4*x5*x6+2*x1*x3*x4*x5*x6+x2**2*x3*x5*x6+x1**&
&2*x3*x5*x6
dpdx4(89) = x1*x2**2*x5*x6+x1**2*x2*x5*x6+2*x1*x2*x3*x4*x6+x1**2*&
&x2*x3*x6+2*x1*x2*x3*x4*x5+x1*x2**2*x3*x5
dpdx4(90) = 2*x1*x2*x4*x5*x6+x1*x2**2*x3*x6+x1**2*x2*x3*x5
dpdx4(91) = x3**3*x5*x6
dpdx4(92) = x3**2*x5*x6**2+x3**2*x5**2*x6
dpdx4(93) = x3*x5**2*x6**2
dpdx4(94) = x3*x5*x6**3+x3*x5**3*x6
dpdx4(95) = x1*x3**2*x6**2+x2*x3**2*x5**2
dpdx4(96) = x1*x3*x6**3+x1*x3**3*x6+x2*x3*x5**3+x2*x3**3*x5
dpdx4(97) = x1*x5*x6**3+x2*x5**3*x6+x2*x3**3*x6+x1*x3**3*x5
dpdx4(98) = x2*x5**2*x6**2+x1*x5**2*x6**2+x2*x3**2*x6**2+x1*x3**2&
&*x5**2
dpdx4(99) = x2*x5*x6**3+x2*x3*x6**3+x1*x5**3*x6+x1*x3*x5**3
dpdx4(100) = x1*x2*x6**3+x1*x2*x5**3+x1*x2*x3**3
dpdx4(101) = 2*x3**2*x4*x5*x6
dpdx4(102) = 2*x3*x4*x5*x6**2+2*x3*x4*x5**2*x6
dpdx4(103) = x1**2*x3*x6**2+2*x1*x3**2*x4*x6+x2**2*x3*x5**2+2*x2*&
&x3**2*x4*x5
dpdx4(104) = 2*x1*x3*x4*x6**2+x1**2*x3**2*x6+2*x2*x3*x4*x5**2+x2*&
&*2*x3**2*x5
dpdx4(105) = x1**2*x2*x6**2+x1*x2**2*x5**2+2*x1*x2*x3**2*x4
dpdx4(106) = x1**2*x5*x6**2+x2**2*x5**2*x6+2*x2*x3**2*x4*x6+2*x1*&
&x3**2*x4*x5
dpdx4(107) = 2*x1*x4*x5*x6**2+2*x2*x4*x5**2*x6+x2**2*x3**2*x6+x1*&
&*2*x3**2*x5
dpdx4(108) = x2**2*x5*x6**2+2*x2*x3*x4*x6**2+x1**2*x5**2*x6+2*x1*&
&x3*x4*x5**2
dpdx4(109) = 2*x2*x4*x5*x6**2+x2**2*x3*x6**2+2*x1*x4*x5**2*x6+x1*&
&*2*x3*x5**2
dpdx4(110) = 2*x1*x2*x4*x6**2+x1*x2**2*x6**2+2*x1*x2*x4*x5**2+x1*&
&*2*x2*x5**2+x1*x2**2*x3**2+x1**2*x2*x3**2
dpdx4(111) = 2*x1**2*x3*x4*x6+2*x2**2*x3*x4*x5
dpdx4(112) = 2*x1**2*x2*x4*x6+x1**2*x2**2*x6+2*x1*x2**2*x4*x5+x1*&
&*2*x2**2*x5+2*x1*x2**2*x3*x4+2*x1**2*x2*x3*x4
dpdx4(113) = 2*x2**2*x4*x5*x6+2*x1**2*x4*x5*x6+2*x2**2*x3*x4*x6+2&
&*x1**2*x3*x4*x5
dpdx4(114) = 2*x1*x2**2*x4*x6+2*x1**2*x2*x4*x5+x1**2*x2**2*x3
dpdx4(115) = 3*x3*x4**2*x5*x6
dpdx4(116) = 3*x1*x3*x4**2*x6+x1**3*x3*x6+3*x2*x3*x4**2*x5+x2**3*&
&x3*x5
dpdx4(117) = x1**3*x2*x6+x1*x2**3*x5+3*x1*x2*x3*x4**2
dpdx4(118) = x2**3*x5*x6+x1**3*x5*x6+3*x2*x3*x4**2*x6+3*x1*x3*x4*&
&*2*x5
dpdx4(119) = 3*x2*x4**2*x5*x6+3*x1*x4**2*x5*x6+x2**3*x3*x6+x1**3*&
&x3*x5
dpdx4(120) = 3*x1*x2*x4**2*x6+x1*x2**3*x6+3*x1*x2*x4**2*x5+x1**3*&
&x2*x5+x1*x2**3*x3+x1**3*x2*x3
dpdx4(121) = 0
dpdx4(122) = 0
dpdx4(123) = 0
dpdx4(124) = x3**4*x6+x3**4*x5
dpdx4(125) = x3**3*x6**2+x3**3*x5**2
dpdx4(126) = x3**2*x6**3+x3**2*x5**3
dpdx4(127) = x5**2*x6**3+x5**3*x6**2
dpdx4(128) = x3*x6**4+x3*x5**4
dpdx4(129) = x5*x6**4+x5**4*x6
dpdx4(130) = x1*x6**4+x2*x5**4+x2*x3**4+x1*x3**4
dpdx4(131) = x2*x6**4+x1*x5**4
dpdx4(132) = 2*x3**3*x4*x6+2*x3**3*x4*x5
dpdx4(133) = 2*x3**2*x4*x6**2+2*x3**2*x4*x5**2
dpdx4(134) = 2*x4*x5**2*x6**2
dpdx4(135) = 2*x3*x4*x6**3+2*x3*x4*x5**3
dpdx4(136) = 2*x4*x5*x6**3+2*x4*x5**3*x6
dpdx4(137) = x1**2*x6**3+x2**2*x5**3+2*x2*x3**3*x4+2*x1*x3**3*x4
dpdx4(138) = 2*x1*x4*x6**3+2*x2*x4*x5**3+x2**2*x3**3+x1**2*x3**3
dpdx4(139) = 2*x2*x4*x6**3+x2**2*x6**3+2*x1*x4*x5**3+x1**2*x5**3
dpdx4(140) = 2*x1**2*x4*x6**2+2*x2**2*x4*x5**2+2*x2**2*x3**2*x4+2&
&*x1**2*x3**2*x4
dpdx4(141) = 2*x1**2*x2**2*x4
dpdx4(142) = 2*x2**2*x4*x6**2+2*x1**2*x4*x5**2
dpdx4(143) = 3*x3**2*x4**2*x6+3*x3**2*x4**2*x5
dpdx4(144) = 3*x3*x4**2*x6**2+3*x3*x4**2*x5**2
dpdx4(145) = 3*x4**2*x5*x6**2+3*x4**2*x5**2*x6
dpdx4(146) = x1**3*x6**2+x2**3*x5**2+3*x2*x3**2*x4**2+3*x1*x3**2*&
&x4**2
dpdx4(147) = 3*x1*x4**2*x6**2+3*x2*x4**2*x5**2+x2**3*x3**2+x1**3*&
&x3**2
dpdx4(148) = 3*x2*x4**2*x6**2+x2**3*x6**2+3*x1*x4**2*x5**2+x1**3*&
&x5**2
dpdx4(149) = 2*x1**3*x4*x6+2*x2**3*x4*x5+3*x2**2*x3*x4**2+3*x1**2&
&*x3*x4**2
dpdx4(150) = 3*x1**2*x4**2*x6+3*x2**2*x4**2*x5+2*x2**3*x3*x4+2*x1&
&**3*x3*x4
dpdx4(151) = 3*x1*x2**2*x4**2+3*x1**2*x2*x4**2+2*x1*x2**3*x4+2*x1&
&**3*x2*x4+x1**2*x2**3+x1**3*x2**2
dpdx4(152) = 3*x2**2*x4**2*x6+2*x2**3*x4*x6+3*x1**2*x4**2*x5+2*x1&
&**3*x4*x5
dpdx4(153) = 4*x3*x4**3*x6+4*x3*x4**3*x5
dpdx4(154) = 4*x4**3*x5*x6
dpdx4(155) = x1**4*x6+x2**4*x5+4*x2*x3*x4**3+4*x1*x3*x4**3
dpdx4(156) = 4*x1*x4**3*x6+4*x2*x4**3*x5+x2**4*x3+x1**4*x3
dpdx4(157) = 4*x1*x2*x4**3+x1*x2**4+x1**4*x2
dpdx4(158) = 4*x2*x4**3*x6+x2**4*x6+4*x1*x4**3*x5+x1**4*x5
dpdx4(159) = 0
dpdx4(160) = 0
dpdx4(161) = 0
dpdx4(162) = x6**5+x5**5
dpdx4(163) = 2*x4*x6**4+2*x4*x5**4
dpdx4(164) = 3*x4**2*x6**3+3*x4**2*x5**3
dpdx4(165) = 3*x2**3*x4**2+3*x1**3*x4**2
dpdx4(166) = 4*x4**3*x6**2+4*x4**3*x5**2
dpdx4(167) = 4*x2**2*x4**3+4*x1**2*x4**3+2*x2**4*x4+2*x1**4*x4
dpdx4(168) = 5*x4**4*x6+5*x4**4*x5
dpdx4(169) = 5*x2*x4**4+5*x1*x4**4+x2**5+x1**5
dpdx4(170) = x1*x2*x3*x5*x6**2+x1*x2*x3*x5**2*x6+x1*x2*x3**2*x5*x&
&6
dpdx4(171) = 2*x1*x2*x3*x4*x5*x6+x1*x2**2*x3*x5*x6+x1**2*x2*x3*x5&
&*x6
dpdx4(172) = x1*x3**2*x5*x6**2+x2*x3**2*x5**2*x6
dpdx4(173) = x1*x3*x5*x6**3+x2*x3*x5**3*x6+x2*x3**3*x5*x6+x1*x3**&
&3*x5*x6
dpdx4(174) = x2*x3*x5**2*x6**2+x1*x3*x5**2*x6**2+x2*x3**2*x5*x6**&
&2+x1*x3**2*x5**2*x6
dpdx4(175) = x1*x2*x5**2*x6**2+x1*x2*x3**2*x6**2+x1*x2*x3**2*x5**&
&2
dpdx4(176) = x2*x3*x5*x6**3+x1*x3*x5**3*x6
dpdx4(177) = x1*x2*x5*x6**3+x1*x2*x3*x6**3+x1*x2*x5**3*x6+x1*x2*x&
&3**3*x6+x1*x2*x3*x5**3+x1*x2*x3**3*x5
dpdx4(178) = x1**2*x3*x5*x6**2+x2**2*x3*x5**2*x6+2*x2*x3**2*x4*x5&
&*x6+2*x1*x3**2*x4*x5*x6
dpdx4(179) = 2*x1*x3*x4*x5*x6**2+2*x2*x3*x4*x5**2*x6+x2**2*x3**2*&
&x5*x6+x1**2*x3**2*x5*x6
dpdx4(180) = x1**2*x2*x5*x6**2+x1**2*x2*x3*x6**2+x1*x2**2*x5**2*x&
&6+2*x1*x2*x3**2*x4*x6+x1*x2**2*x3*x5**2+2*x1*x2*x3**2*x4*x5
dpdx4(181) = 2*x2*x3*x4*x5*x6**2+x2**2*x3*x5*x6**2+2*x1*x3*x4*x5*&
&*2*x6+x1**2*x3*x5**2*x6
dpdx4(182) = x1*x2**2*x5*x6**2+2*x1*x2*x3*x4*x6**2+x1**2*x2*x5**2&
&*x6+x1**2*x2*x3**2*x6+2*x1*x2*x3*x4*x5**2+x1*x2**2*x3**2*x5
dpdx4(183) = 2*x1*x2*x4*x5*x6**2+x1*x2**2*x3*x6**2+2*x1*x2*x4*x5*&
&*2*x6+x1*x2**2*x3**2*x6+x1**2*x2*x3*x5**2+x1**2*x2*x3**2*x5
dpdx4(184) = x1**2*x2**2*x5*x6+2*x1**2*x2*x3*x4*x6+2*x1*x2**2*x3*&
&x4*x5
dpdx4(185) = 2*x2**2*x3*x4*x5*x6+2*x1**2*x3*x4*x5*x6
dpdx4(186) = 2*x1*x2**2*x4*x5*x6+2*x1**2*x2*x4*x5*x6+2*x1*x2**2*x&
&3*x4*x6+x1**2*x2**2*x3*x6+2*x1**2*x2*x3*x4*x5+x1**2*x2**2*x3*x5
dpdx4(187) = 3*x2*x3*x4**2*x5*x6+3*x1*x3*x4**2*x5*x6+x2**3*x3*x5*&
&x6+x1**3*x3*x5*x6
dpdx4(188) = x1*x2**3*x5*x6+x1**3*x2*x5*x6+3*x1*x2*x3*x4**2*x6+x1&
&**3*x2*x3*x6+3*x1*x2*x3*x4**2*x5+x1*x2**3*x3*x5
dpdx4(189) = 3*x1*x2*x4**2*x5*x6+x1*x2**3*x3*x6+x1**3*x2*x3*x5
dpdx4(190) = x3**4*x5*x6
dpdx4(191) = x3**3*x5*x6**2+x3**3*x5**2*x6
dpdx4(192) = x3**2*x5**2*x6**2
dpdx4(193) = x3**2*x5*x6**3+x3**2*x5**3*x6
dpdx4(194) = x3*x5**2*x6**3+x3*x5**3*x6**2
dpdx4(195) = x3*x5*x6**4+x3*x5**4*x6
dpdx4(196) = x1*x3**2*x6**3+x1*x3**3*x6**2+x2*x3**2*x5**3+x2*x3**&
&3*x5**2
dpdx4(197) = x1*x3*x6**4+x1*x3**4*x6+x2*x3*x5**4+x2*x3**4*x5
dpdx4(198) = x1*x5*x6**4+x2*x5**4*x6+x2*x3**4*x6+x1*x3**4*x5
dpdx4(199) = x1*x5**2*x6**3+x2*x5**3*x6**2+x2*x3**3*x6**2+x1*x3**&
&3*x5**2
dpdx4(200) = x2*x5**2*x6**3+x2*x3**2*x6**3+x1*x5**3*x6**2+x1*x3**&
&2*x5**3
dpdx4(201) = x2*x5*x6**4+x2*x3*x6**4+x1*x5**4*x6+x1*x3*x5**4
dpdx4(202) = x1*x2*x6**4+x1*x2*x5**4+x1*x2*x3**4
dpdx4(203) = 2*x3**3*x4*x5*x6
dpdx4(204) = 2*x3**2*x4*x5*x6**2+2*x3**2*x4*x5**2*x6
dpdx4(205) = 2*x3*x4*x5**2*x6**2
dpdx4(206) = 2*x3*x4*x5*x6**3+2*x3*x4*x5**3*x6
dpdx4(207) = x1**2*x3*x6**3+2*x1*x3**3*x4*x6+x2**2*x3*x5**3+2*x2*&
&x3**3*x4*x5
dpdx4(208) = 2*x1*x3**2*x4*x6**2+x1**2*x3**2*x6**2+2*x2*x3**2*x4*&
&x5**2+x2**2*x3**2*x5**2
dpdx4(209) = 2*x1*x3*x4*x6**3+x1**2*x3**3*x6+2*x2*x3*x4*x5**3+x2*&
&*2*x3**3*x5
dpdx4(210) = x1**2*x2*x6**3+x1*x2**2*x5**3+2*x1*x2*x3**3*x4
dpdx4(211) = x1**2*x5*x6**3+x2**2*x5**3*x6+2*x2*x3**3*x4*x6+2*x1*&
&x3**3*x4*x5
dpdx4(212) = 2*x1*x4*x5*x6**3+2*x2*x4*x5**3*x6+x2**2*x3**3*x6+x1*&
&*2*x3**3*x5
dpdx4(213) = x2**2*x5**2*x6**2+x1**2*x5**2*x6**2+2*x2*x3**2*x4*x6&
&**2+2*x1*x3**2*x4*x5**2
dpdx4(214) = 2*x2*x4*x5**2*x6**2+2*x1*x4*x5**2*x6**2+x2**2*x3**2*&
&x6**2+x1**2*x3**2*x5**2
dpdx4(215) = x2**2*x5*x6**3+2*x2*x3*x4*x6**3+x1**2*x5**3*x6+2*x1*&
&x3*x4*x5**3
dpdx4(216) = 2*x2*x4*x5*x6**3+x2**2*x3*x6**3+2*x1*x4*x5**3*x6+x1*&
&*2*x3*x5**3
dpdx4(217) = 2*x1*x2*x4*x6**3+x1*x2**2*x6**3+2*x1*x2*x4*x5**3+x1*&
&*2*x2*x5**3+x1*x2**2*x3**3+x1**2*x2*x3**3
dpdx4(218) = 2*x1**2*x3*x4*x6**2+2*x1**2*x3**2*x4*x6+2*x2**2*x3*x&
&4*x5**2+2*x2**2*x3**2*x4*x5
dpdx4(219) = 2*x1**2*x2*x4*x6**2+x1**2*x2**2*x6**2+2*x1*x2**2*x4*&
&x5**2+x1**2*x2**2*x5**2+2*x1*x2**2*x3**2*x4+2*x1**2*x2*x3**2*x4
dpdx4(220) = 2*x1**2*x4*x5*x6**2+2*x2**2*x4*x5**2*x6+2*x2**2*x3**&
&2*x4*x6+2*x1**2*x3**2*x4*x5
dpdx4(221) = 2*x1**2*x2**2*x4*x6+2*x1**2*x2**2*x4*x5+2*x1**2*x2**&
&2*x3*x4
dpdx4(222) = 2*x2**2*x4*x5*x6**2+2*x2**2*x3*x4*x6**2+2*x1**2*x4*x&
&5**2*x6+2*x1**2*x3*x4*x5**2
dpdx4(223) = 2*x1*x2**2*x4*x6**2+2*x1**2*x2*x4*x5**2+x1**2*x2**2*&
&x3**2
dpdx4(224) = 3*x3**2*x4**2*x5*x6
dpdx4(225) = 3*x3*x4**2*x5*x6**2+3*x3*x4**2*x5**2*x6
dpdx4(226) = x1**3*x3*x6**2+3*x1*x3**2*x4**2*x6+x2**3*x3*x5**2+3*&
&x2*x3**2*x4**2*x5
dpdx4(227) = 3*x1*x3*x4**2*x6**2+x1**3*x3**2*x6+3*x2*x3*x4**2*x5*&
&*2+x2**3*x3**2*x5
dpdx4(228) = x1**3*x2*x6**2+x1*x2**3*x5**2+3*x1*x2*x3**2*x4**2
dpdx4(229) = x1**3*x5*x6**2+x2**3*x5**2*x6+3*x2*x3**2*x4**2*x6+3*&
&x1*x3**2*x4**2*x5
dpdx4(230) = 3*x1*x4**2*x5*x6**2+3*x2*x4**2*x5**2*x6+x2**3*x3**2*&
&x6+x1**3*x3**2*x5
dpdx4(231) = x2**3*x5*x6**2+3*x2*x3*x4**2*x6**2+x1**3*x5**2*x6+3*&
&x1*x3*x4**2*x5**2
dpdx4(232) = 3*x2*x4**2*x5*x6**2+x2**3*x3*x6**2+3*x1*x4**2*x5**2*&
&x6+x1**3*x3*x5**2
dpdx4(233) = 3*x1*x2*x4**2*x6**2+x1*x2**3*x6**2+3*x1*x2*x4**2*x5*&
&*2+x1**3*x2*x5**2+x1*x2**3*x3**2+x1**3*x2*x3**2
dpdx4(234) = 3*x1**2*x3*x4**2*x6+2*x1**3*x3*x4*x6+3*x2**2*x3*x4**&
&2*x5+2*x2**3*x3*x4*x5
dpdx4(235) = 2*x1**3*x2*x4*x6+x1**3*x2**2*x6+2*x1*x2**3*x4*x5+x1*&
&*2*x2**3*x5+3*x1*x2**2*x3*x4**2+3*x1**2*x2*x3*x4**2
dpdx4(236) = 3*x1**2*x2*x4**2*x6+x1**2*x2**3*x6+3*x1*x2**2*x4**2*&
&x5+x1**3*x2**2*x5+2*x1*x2**3*x3*x4+2*x1**3*x2*x3*x4
dpdx4(237) = 2*x2**3*x4*x5*x6+2*x1**3*x4*x5*x6+3*x2**2*x3*x4**2*x&
&6+3*x1**2*x3*x4**2*x5
dpdx4(238) = 3*x2**2*x4**2*x5*x6+3*x1**2*x4**2*x5*x6+2*x2**3*x3*x&
&4*x6+2*x1**3*x3*x4*x5
dpdx4(239) = 3*x1*x2**2*x4**2*x6+2*x1*x2**3*x4*x6+3*x1**2*x2*x4**&
&2*x5+2*x1**3*x2*x4*x5+x1**2*x2**3*x3+x1**3*x2**2*x3
dpdx4(240) = 4*x3*x4**3*x5*x6
dpdx4(241) = 4*x1*x3*x4**3*x6+x1**4*x3*x6+4*x2*x3*x4**3*x5+x2**4*&
&x3*x5
dpdx4(242) = x1**4*x2*x6+x1*x2**4*x5+4*x1*x2*x3*x4**3
dpdx4(243) = x2**4*x5*x6+x1**4*x5*x6+4*x2*x3*x4**3*x6+4*x1*x3*x4*&
&*3*x5
dpdx4(244) = 4*x2*x4**3*x5*x6+4*x1*x4**3*x5*x6+x2**4*x3*x6+x1**4*&
&x3*x5
dpdx4(245) = 4*x1*x2*x4**3*x6+x1*x2**4*x6+4*x1*x2*x4**3*x5+x1**4*&
&x2*x5+x1*x2**4*x3+x1**4*x2*x3
dpdx4(246) = 0
dpdx4(247) = 0
dpdx4(248) = 0
dpdx4(249) = 0
dpdx4(250) = x3**5*x6+x3**5*x5
dpdx4(251) = x3**4*x6**2+x3**4*x5**2
dpdx4(252) = x3**3*x6**3+x3**3*x5**3
dpdx4(253) = x5**3*x6**3
dpdx4(254) = x3**2*x6**4+x3**2*x5**4
dpdx4(255) = x5**2*x6**4+x5**4*x6**2
dpdx4(256) = x3*x6**5+x3*x5**5
dpdx4(257) = x5*x6**5+x5**5*x6
dpdx4(258) = x1*x6**5+x2*x5**5+x2*x3**5+x1*x3**5
dpdx4(259) = x2*x6**5+x1*x5**5
dpdx4(260) = 2*x3**4*x4*x6+2*x3**4*x4*x5
dpdx4(261) = 2*x3**3*x4*x6**2+2*x3**3*x4*x5**2
dpdx4(262) = 2*x3**2*x4*x6**3+2*x3**2*x4*x5**3
dpdx4(263) = 2*x4*x5**2*x6**3+2*x4*x5**3*x6**2
dpdx4(264) = 2*x3*x4*x6**4+2*x3*x4*x5**4
dpdx4(265) = 2*x4*x5*x6**4+2*x4*x5**4*x6
dpdx4(266) = x1**2*x6**4+x2**2*x5**4+2*x2*x3**4*x4+2*x1*x3**4*x4
dpdx4(267) = 2*x1*x4*x6**4+2*x2*x4*x5**4+x2**2*x3**4+x1**2*x3**4
dpdx4(268) = 2*x2*x4*x6**4+x2**2*x6**4+2*x1*x4*x5**4+x1**2*x5**4
dpdx4(269) = 2*x1**2*x4*x6**3+2*x2**2*x4*x5**3+2*x2**2*x3**3*x4+2&
&*x1**2*x3**3*x4
dpdx4(270) = 2*x2**2*x4*x6**3+2*x1**2*x4*x5**3
dpdx4(271) = 3*x3**3*x4**2*x6+3*x3**3*x4**2*x5
dpdx4(272) = 3*x3**2*x4**2*x6**2+3*x3**2*x4**2*x5**2
dpdx4(273) = 3*x4**2*x5**2*x6**2
dpdx4(274) = 3*x3*x4**2*x6**3+3*x3*x4**2*x5**3
dpdx4(275) = 3*x4**2*x5*x6**3+3*x4**2*x5**3*x6
dpdx4(276) = x1**3*x6**3+x2**3*x5**3+3*x2*x3**3*x4**2+3*x1*x3**3*&
&x4**2
dpdx4(277) = 3*x1*x4**2*x6**3+3*x2*x4**2*x5**3+x2**3*x3**3+x1**3*&
&x3**3
dpdx4(278) = 3*x2*x4**2*x6**3+x2**3*x6**3+3*x1*x4**2*x5**3+x1**3*&
&x5**3
dpdx4(279) = 2*x1**3*x4*x6**2+2*x2**3*x4*x5**2+3*x2**2*x3**2*x4**&
&2+3*x1**2*x3**2*x4**2
dpdx4(280) = 3*x1**2*x4**2*x6**2+3*x2**2*x4**2*x5**2+2*x2**3*x3**&
&2*x4+2*x1**3*x3**2*x4
dpdx4(281) = 3*x1**2*x2**2*x4**2+2*x1**2*x2**3*x4+2*x1**3*x2**2*x&
&4
dpdx4(282) = 3*x2**2*x4**2*x6**2+2*x2**3*x4*x6**2+3*x1**2*x4**2*x&
&5**2+2*x1**3*x4*x5**2
dpdx4(283) = 3*x1**3*x4**2*x6+3*x2**3*x4**2*x5+3*x2**3*x3*x4**2+3&
&*x1**3*x3*x4**2
dpdx4(284) = 3*x1*x2**3*x4**2+3*x1**3*x2*x4**2+x1**3*x2**3
dpdx4(285) = 3*x2**3*x4**2*x6+3*x1**3*x4**2*x5
dpdx4(286) = 4*x3**2*x4**3*x6+4*x3**2*x4**3*x5
dpdx4(287) = 4*x3*x4**3*x6**2+4*x3*x4**3*x5**2
dpdx4(288) = 4*x4**3*x5*x6**2+4*x4**3*x5**2*x6
dpdx4(289) = x1**4*x6**2+x2**4*x5**2+4*x2*x3**2*x4**3+4*x1*x3**2*&
&x4**3
dpdx4(290) = 4*x1*x4**3*x6**2+4*x2*x4**3*x5**2+x2**4*x3**2+x1**4*&
&x3**2
dpdx4(291) = 4*x2*x4**3*x6**2+x2**4*x6**2+4*x1*x4**3*x5**2+x1**4*&
&x5**2
dpdx4(292) = 2*x1**4*x4*x6+2*x2**4*x4*x5+4*x2**2*x3*x4**3+4*x1**2&
&*x3*x4**3
dpdx4(293) = 4*x1**2*x4**3*x6+4*x2**2*x4**3*x5+2*x2**4*x3*x4+2*x1&
&**4*x3*x4
dpdx4(294) = 4*x1*x2**2*x4**3+4*x1**2*x2*x4**3+2*x1*x2**4*x4+2*x1&
&**4*x2*x4+x1**2*x2**4+x1**4*x2**2
dpdx4(295) = 4*x2**2*x4**3*x6+2*x2**4*x4*x6+4*x1**2*x4**3*x5+2*x1&
&**4*x4*x5
dpdx4(296) = 5*x3*x4**4*x6+5*x3*x4**4*x5
dpdx4(297) = 5*x4**4*x5*x6
dpdx4(298) = x1**5*x6+x2**5*x5+5*x2*x3*x4**4+5*x1*x3*x4**4
dpdx4(299) = 5*x1*x4**4*x6+5*x2*x4**4*x5+x2**5*x3+x1**5*x3
dpdx4(300) = 5*x1*x2*x4**4+x1*x2**5+x1**5*x2
dpdx4(301) = 5*x2*x4**4*x6+x2**5*x6+5*x1*x4**4*x5+x1**5*x5
dpdx4(302) = 0
dpdx4(303) = 0
dpdx4(304) = 0
dpdx4(305) = x6**6+x5**6
dpdx4(306) = 2*x4*x6**5+2*x4*x5**5
dpdx4(307) = 3*x4**2*x6**4+3*x4**2*x5**4
dpdx4(308) = 4*x4**3*x6**3+4*x4**3*x5**3
dpdx4(309) = 4*x2**3*x4**3+4*x1**3*x4**3+3*x2**4*x4**2+3*x1**4*x4&
&**2
dpdx4(310) = 5*x4**4*x6**2+5*x4**4*x5**2
dpdx4(311) = 5*x2**2*x4**4+5*x1**2*x4**4+2*x2**5*x4+2*x1**5*x4
dpdx4(312) = 6*x4**5*x6+6*x4**5*x5
dpdx4(313) = 6*x2*x4**5+6*x1*x4**5+x2**6+x1**6
dpdx4(314) = x1*x2*x3*x5**2*x6**2+x1*x2*x3**2*x5*x6**2+x1*x2*x3**&
&2*x5**2*x6
dpdx4(315) = x1*x2*x3*x5*x6**3+x1*x2*x3*x5**3*x6+x1*x2*x3**3*x5*x&
&6
dpdx4(316) = x1**2*x2*x3*x5*x6**2+x1*x2**2*x3*x5**2*x6+2*x1*x2*x3&
&**2*x4*x5*x6
dpdx4(317) = 2*x1*x2*x3*x4*x5*x6**2+x1*x2**2*x3*x5*x6**2+2*x1*x2*&
&x3*x4*x5**2*x6+x1**2*x2*x3*x5**2*x6+x1*x2**2*x3**2*x5*x6+x1**2*x2&
&*x3**2*x5*x6
dpdx4(318) = 2*x1*x2**2*x3*x4*x5*x6+2*x1**2*x2*x3*x4*x5*x6+x1**2*&
&x2**2*x3*x5*x6
dpdx4(319) = 3*x1*x2*x3*x4**2*x5*x6+x1*x2**3*x3*x5*x6+x1**3*x2*x3&
&*x5*x6
dpdx4(320) = x1*x3**2*x5*x6**3+x1*x3**3*x5*x6**2+x2*x3**2*x5**3*x&
&6+x2*x3**3*x5**2*x6
dpdx4(321) = x1*x3*x5*x6**4+x2*x3*x5**4*x6+x2*x3**4*x5*x6+x1*x3**&
&4*x5*x6
dpdx4(322) = x2*x3**2*x5**2*x6**2+x1*x3**2*x5**2*x6**2
dpdx4(323) = x1*x3*x5**2*x6**3+x2*x3*x5**3*x6**2+x2*x3**3*x5*x6**&
&2+x1*x3**3*x5**2*x6
dpdx4(324) = x2*x3*x5**2*x6**3+x2*x3**2*x5*x6**3+x1*x3*x5**3*x6**&
&2+x1*x3**2*x5**3*x6
dpdx4(325) = x1*x2*x5**2*x6**3+x1*x2*x3**2*x6**3+x1*x2*x5**3*x6**&
&2+x1*x2*x3**3*x6**2+x1*x2*x3**2*x5**3+x1*x2*x3**3*x5**2
dpdx4(326) = x2*x3*x5*x6**4+x1*x3*x5**4*x6
dpdx4(327) = x1*x2*x5*x6**4+x1*x2*x3*x6**4+x1*x2*x5**4*x6+x1*x2*x&
&3**4*x6+x1*x2*x3*x5**4+x1*x2*x3**4*x5
dpdx4(328) = x1**2*x3*x5*x6**3+x2**2*x3*x5**3*x6+2*x2*x3**3*x4*x5&
&*x6+2*x1*x3**3*x4*x5*x6
dpdx4(329) = 2*x1*x3**2*x4*x5*x6**2+x1**2*x3**2*x5*x6**2+2*x2*x3*&
&*2*x4*x5**2*x6+x2**2*x3**2*x5**2*x6
dpdx4(330) = 2*x1*x3*x4*x5*x6**3+2*x2*x3*x4*x5**3*x6+x2**2*x3**3*&
&x5*x6+x1**2*x3**3*x5*x6
dpdx4(331) = x1**2*x2*x5*x6**3+x1**2*x2*x3*x6**3+x1*x2**2*x5**3*x&
&6+2*x1*x2*x3**3*x4*x6+x1*x2**2*x3*x5**3+2*x1*x2*x3**3*x4*x5
dpdx4(332) = x2**2*x3*x5**2*x6**2+x1**2*x3*x5**2*x6**2+2*x2*x3**2&
&*x4*x5*x6**2+2*x1*x3**2*x4*x5**2*x6
dpdx4(333) = 2*x2*x3*x4*x5**2*x6**2+2*x1*x3*x4*x5**2*x6**2+x2**2*&
&x3**2*x5*x6**2+x1**2*x3**2*x5**2*x6
dpdx4(334) = x1*x2**2*x5**2*x6**2+x1**2*x2*x5**2*x6**2+2*x1*x2*x3&
&**2*x4*x6**2+x1**2*x2*x3**2*x6**2+2*x1*x2*x3**2*x4*x5**2+x1*x2**2&
&*x3**2*x5**2
dpdx4(335) = 2*x1*x2*x4*x5**2*x6**2+x1*x2**2*x3**2*x6**2+x1**2*x2&
&*x3**2*x5**2
dpdx4(336) = 2*x2*x3*x4*x5*x6**3+x2**2*x3*x5*x6**3+2*x1*x3*x4*x5*&
&*3*x6+x1**2*x3*x5**3*x6
dpdx4(337) = x1*x2**2*x5*x6**3+2*x1*x2*x3*x4*x6**3+x1**2*x2*x5**3&
&*x6+x1**2*x2*x3**3*x6+2*x1*x2*x3*x4*x5**3+x1*x2**2*x3**3*x5
dpdx4(338) = 2*x1*x2*x4*x5*x6**3+x1*x2**2*x3*x6**3+2*x1*x2*x4*x5*&
&*3*x6+x1*x2**2*x3**3*x6+x1**2*x2*x3*x5**3+x1**2*x2*x3**3*x5
dpdx4(339) = x1**2*x2**2*x5*x6**2+2*x1**2*x2*x3*x4*x6**2+x1**2*x2&
&**2*x5**2*x6+2*x1**2*x2*x3**2*x4*x6+2*x1*x2**2*x3*x4*x5**2+2*x1*x&
&2**2*x3**2*x4*x5
dpdx4(340) = 2*x1**2*x3*x4*x5*x6**2+2*x2**2*x3*x4*x5**2*x6+2*x2**&
&2*x3**2*x4*x5*x6+2*x1**2*x3**2*x4*x5*x6
dpdx4(341) = 2*x1**2*x2*x4*x5*x6**2+x1**2*x2**2*x3*x6**2+2*x1*x2*&
&*2*x4*x5**2*x6+2*x1*x2**2*x3**2*x4*x6+x1**2*x2**2*x3*x5**2+2*x1**&
&2*x2*x3**2*x4*x5
dpdx4(342) = 2*x1**2*x2**2*x4*x5*x6+2*x1**2*x2**2*x3*x4*x6+2*x1**&
&2*x2**2*x3*x4*x5
dpdx4(343) = 2*x2**2*x3*x4*x5*x6**2+2*x1**2*x3*x4*x5**2*x6
dpdx4(344) = 2*x1*x2**2*x4*x5*x6**2+2*x1*x2**2*x3*x4*x6**2+2*x1**&
&2*x2*x4*x5**2*x6+x1**2*x2**2*x3**2*x6+2*x1**2*x2*x3*x4*x5**2+x1**&
&2*x2**2*x3**2*x5
dpdx4(345) = x1**3*x3*x5*x6**2+x2**3*x3*x5**2*x6+3*x2*x3**2*x4**2&
&*x5*x6+3*x1*x3**2*x4**2*x5*x6
dpdx4(346) = 3*x1*x3*x4**2*x5*x6**2+3*x2*x3*x4**2*x5**2*x6+x2**3*&
&x3**2*x5*x6+x1**3*x3**2*x5*x6
dpdx4(347) = x1**3*x2*x5*x6**2+x1**3*x2*x3*x6**2+x1*x2**3*x5**2*x&
&6+3*x1*x2*x3**2*x4**2*x6+x1*x2**3*x3*x5**2+3*x1*x2*x3**2*x4**2*x5
dpdx4(348) = 3*x2*x3*x4**2*x5*x6**2+x2**3*x3*x5*x6**2+3*x1*x3*x4*&
&*2*x5**2*x6+x1**3*x3*x5**2*x6
dpdx4(349) = x1*x2**3*x5*x6**2+3*x1*x2*x3*x4**2*x6**2+x1**3*x2*x5&
&**2*x6+x1**3*x2*x3**2*x6+3*x1*x2*x3*x4**2*x5**2+x1*x2**3*x3**2*x5
dpdx4(350) = 3*x1*x2*x4**2*x5*x6**2+x1*x2**3*x3*x6**2+3*x1*x2*x4*&
&*2*x5**2*x6+x1*x2**3*x3**2*x6+x1**3*x2*x3*x5**2+x1**3*x2*x3**2*x5
dpdx4(351) = x1**2*x2**3*x5*x6+x1**3*x2**2*x5*x6+3*x1**2*x2*x3*x4&
&**2*x6+2*x1**3*x2*x3*x4*x6+3*x1*x2**2*x3*x4**2*x5+2*x1*x2**3*x3*x&
&4*x5
dpdx4(352) = 3*x2**2*x3*x4**2*x5*x6+3*x1**2*x3*x4**2*x5*x6+2*x2**&
&3*x3*x4*x5*x6+2*x1**3*x3*x4*x5*x6
dpdx4(353) = 2*x1*x2**3*x4*x5*x6+2*x1**3*x2*x4*x5*x6+3*x1*x2**2*x&
&3*x4**2*x6+x1**3*x2**2*x3*x6+3*x1**2*x2*x3*x4**2*x5+x1**2*x2**3*x&
&3*x5
dpdx4(354) = 3*x1*x2**2*x4**2*x5*x6+3*x1**2*x2*x4**2*x5*x6+2*x1*x&
&2**3*x3*x4*x6+x1**2*x2**3*x3*x6+2*x1**3*x2*x3*x4*x5+x1**3*x2**2*x&
&3*x5
dpdx4(355) = 4*x2*x3*x4**3*x5*x6+4*x1*x3*x4**3*x5*x6+x2**4*x3*x5*&
&x6+x1**4*x3*x5*x6
dpdx4(356) = x1*x2**4*x5*x6+x1**4*x2*x5*x6+4*x1*x2*x3*x4**3*x6+x1&
&**4*x2*x3*x6+4*x1*x2*x3*x4**3*x5+x1*x2**4*x3*x5
dpdx4(357) = 4*x1*x2*x4**3*x5*x6+x1*x2**4*x3*x6+x1**4*x2*x3*x5
dpdx4(358) = x3**5*x5*x6
dpdx4(359) = x3**4*x5*x6**2+x3**4*x5**2*x6
dpdx4(360) = x3**3*x5**2*x6**2
dpdx4(361) = x3**3*x5*x6**3+x3**3*x5**3*x6
dpdx4(362) = x3**2*x5**2*x6**3+x3**2*x5**3*x6**2
dpdx4(363) = x3*x5**3*x6**3
dpdx4(364) = x3**2*x5*x6**4+x3**2*x5**4*x6
dpdx4(365) = x3*x5**2*x6**4+x3*x5**4*x6**2
dpdx4(366) = x3*x5*x6**5+x3*x5**5*x6
dpdx4(367) = x1*x3**3*x6**3+x2*x3**3*x5**3
dpdx4(368) = x1*x3**2*x6**4+x1*x3**4*x6**2+x2*x3**2*x5**4+x2*x3**&
&4*x5**2
dpdx4(369) = x1*x3*x6**5+x1*x3**5*x6+x2*x3*x5**5+x2*x3**5*x5
dpdx4(370) = x1*x5*x6**5+x2*x5**5*x6+x2*x3**5*x6+x1*x3**5*x5
dpdx4(371) = x1*x5**2*x6**4+x2*x5**4*x6**2+x2*x3**4*x6**2+x1*x3**&
&4*x5**2
dpdx4(372) = x2*x5**3*x6**3+x1*x5**3*x6**3+x2*x3**3*x6**3+x1*x3**&
&3*x5**3
dpdx4(373) = x2*x5**2*x6**4+x2*x3**2*x6**4+x1*x5**4*x6**2+x1*x3**&
&2*x5**4
dpdx4(374) = x2*x5*x6**5+x2*x3*x6**5+x1*x5**5*x6+x1*x3*x5**5
dpdx4(375) = x1*x2*x6**5+x1*x2*x5**5+x1*x2*x3**5
dpdx4(376) = 2*x3**4*x4*x5*x6
dpdx4(377) = 2*x3**3*x4*x5*x6**2+2*x3**3*x4*x5**2*x6
dpdx4(378) = 2*x3**2*x4*x5**2*x6**2
dpdx4(379) = 2*x3**2*x4*x5*x6**3+2*x3**2*x4*x5**3*x6
dpdx4(380) = 2*x3*x4*x5**2*x6**3+2*x3*x4*x5**3*x6**2
dpdx4(381) = 2*x3*x4*x5*x6**4+2*x3*x4*x5**4*x6
dpdx4(382) = x1**2*x3*x6**4+2*x1*x3**4*x4*x6+x2**2*x3*x5**4+2*x2*&
&x3**4*x4*x5
dpdx4(383) = x1**2*x3**2*x6**3+2*x1*x3**3*x4*x6**2+x2**2*x3**2*x5&
&**3+2*x2*x3**3*x4*x5**2
dpdx4(384) = 2*x1*x3**2*x4*x6**3+x1**2*x3**3*x6**2+2*x2*x3**2*x4*&
&x5**3+x2**2*x3**3*x5**2
dpdx4(385) = 2*x1*x3*x4*x6**4+x1**2*x3**4*x6+2*x2*x3*x4*x5**4+x2*&
&*2*x3**4*x5
dpdx4(386) = x1**2*x2*x6**4+x1*x2**2*x5**4+2*x1*x2*x3**4*x4
dpdx4(387) = x1**2*x5*x6**4+x2**2*x5**4*x6+2*x2*x3**4*x4*x6+2*x1*&
&x3**4*x4*x5
dpdx4(388) = 2*x1*x4*x5*x6**4+2*x2*x4*x5**4*x6+x2**2*x3**4*x6+x1*&
&*2*x3**4*x5
dpdx4(389) = x1**2*x5**2*x6**3+x2**2*x5**3*x6**2+2*x2*x3**3*x4*x6&
&**2+2*x1*x3**3*x4*x5**2
dpdx4(390) = 2*x1*x4*x5**2*x6**3+2*x2*x4*x5**3*x6**2+x2**2*x3**3*&
&x6**2+x1**2*x3**3*x5**2
dpdx4(391) = x2**2*x5**2*x6**3+2*x2*x3**2*x4*x6**3+x1**2*x5**3*x6&
&**2+2*x1*x3**2*x4*x5**3
dpdx4(392) = 2*x2*x4*x5**2*x6**3+x2**2*x3**2*x6**3+2*x1*x4*x5**3*&
&x6**2+x1**2*x3**2*x5**3
dpdx4(393) = x2**2*x5*x6**4+2*x2*x3*x4*x6**4+x1**2*x5**4*x6+2*x1*&
&x3*x4*x5**4
dpdx4(394) = 2*x2*x4*x5*x6**4+x2**2*x3*x6**4+2*x1*x4*x5**4*x6+x1*&
&*2*x3*x5**4
dpdx4(395) = 2*x1*x2*x4*x6**4+x1*x2**2*x6**4+2*x1*x2*x4*x5**4+x1*&
&*2*x2*x5**4+x1*x2**2*x3**4+x1**2*x2*x3**4
dpdx4(396) = 2*x1**2*x3**2*x4*x6**2+2*x2**2*x3**2*x4*x5**2
dpdx4(397) = 2*x1**2*x3*x4*x6**3+2*x1**2*x3**3*x4*x6+2*x2**2*x3*x&
&4*x5**3+2*x2**2*x3**3*x4*x5
dpdx4(398) = 2*x1**2*x2*x4*x6**3+x1**2*x2**2*x6**3+2*x1*x2**2*x4*&
&x5**3+x1**2*x2**2*x5**3+2*x1*x2**2*x3**3*x4+2*x1**2*x2*x3**3*x4
dpdx4(399) = 2*x1**2*x4*x5*x6**3+2*x2**2*x4*x5**3*x6+2*x2**2*x3**&
&3*x4*x6+2*x1**2*x3**3*x4*x5
dpdx4(400) = 2*x2**2*x4*x5**2*x6**2+2*x1**2*x4*x5**2*x6**2+2*x2**&
&2*x3**2*x4*x6**2+2*x1**2*x3**2*x4*x5**2
dpdx4(401) = 2*x1**2*x2**2*x4*x6**2+2*x1**2*x2**2*x4*x5**2+2*x1**&
&2*x2**2*x3**2*x4
dpdx4(402) = 2*x2**2*x4*x5*x6**3+2*x2**2*x3*x4*x6**3+2*x1**2*x4*x&
&5**3*x6+2*x1**2*x3*x4*x5**3
dpdx4(403) = 2*x1*x2**2*x4*x6**3+2*x1**2*x2*x4*x5**3+x1**2*x2**2*&
&x3**3
dpdx4(404) = 3*x3**3*x4**2*x5*x6
dpdx4(405) = 3*x3**2*x4**2*x5*x6**2+3*x3**2*x4**2*x5**2*x6
dpdx4(406) = 3*x3*x4**2*x5**2*x6**2
dpdx4(407) = 3*x3*x4**2*x5*x6**3+3*x3*x4**2*x5**3*x6
dpdx4(408) = x1**3*x3*x6**3+3*x1*x3**3*x4**2*x6+x2**3*x3*x5**3+3*&
&x2*x3**3*x4**2*x5
dpdx4(409) = 3*x1*x3**2*x4**2*x6**2+x1**3*x3**2*x6**2+3*x2*x3**2*&
&x4**2*x5**2+x2**3*x3**2*x5**2
dpdx4(410) = 3*x1*x3*x4**2*x6**3+x1**3*x3**3*x6+3*x2*x3*x4**2*x5*&
&*3+x2**3*x3**3*x5
dpdx4(411) = x1**3*x2*x6**3+x1*x2**3*x5**3+3*x1*x2*x3**3*x4**2
dpdx4(412) = x1**3*x5*x6**3+x2**3*x5**3*x6+3*x2*x3**3*x4**2*x6+3*&
&x1*x3**3*x4**2*x5
dpdx4(413) = 3*x1*x4**2*x5*x6**3+3*x2*x4**2*x5**3*x6+x2**3*x3**3*&
&x6+x1**3*x3**3*x5
dpdx4(414) = x2**3*x5**2*x6**2+x1**3*x5**2*x6**2+3*x2*x3**2*x4**2&
&*x6**2+3*x1*x3**2*x4**2*x5**2
dpdx4(415) = 3*x2*x4**2*x5**2*x6**2+3*x1*x4**2*x5**2*x6**2+x2**3*&
&x3**2*x6**2+x1**3*x3**2*x5**2
dpdx4(416) = x2**3*x5*x6**3+3*x2*x3*x4**2*x6**3+x1**3*x5**3*x6+3*&
&x1*x3*x4**2*x5**3
dpdx4(417) = 3*x2*x4**2*x5*x6**3+x2**3*x3*x6**3+3*x1*x4**2*x5**3*&
&x6+x1**3*x3*x5**3
dpdx4(418) = 3*x1*x2*x4**2*x6**3+x1*x2**3*x6**3+3*x1*x2*x4**2*x5*&
&*3+x1**3*x2*x5**3+x1*x2**3*x3**3+x1**3*x2*x3**3
dpdx4(419) = 2*x1**3*x3*x4*x6**2+3*x1**2*x3**2*x4**2*x6+2*x2**3*x&
&3*x4*x5**2+3*x2**2*x3**2*x4**2*x5
dpdx4(420) = 3*x1**2*x3*x4**2*x6**2+2*x1**3*x3**2*x4*x6+3*x2**2*x&
&3*x4**2*x5**2+2*x2**3*x3**2*x4*x5
dpdx4(421) = 2*x1**3*x2*x4*x6**2+x1**3*x2**2*x6**2+2*x1*x2**3*x4*&
&x5**2+x1**2*x2**3*x5**2+3*x1*x2**2*x3**2*x4**2+3*x1**2*x2*x3**2*x&
&4**2
dpdx4(422) = 3*x1**2*x2*x4**2*x6**2+x1**2*x2**3*x6**2+3*x1*x2**2*&
&x4**2*x5**2+x1**3*x2**2*x5**2+2*x1*x2**3*x3**2*x4+2*x1**3*x2*x3**&
&2*x4
dpdx4(423) = 2*x1**3*x2**2*x4*x6+2*x1**2*x2**3*x4*x5+3*x1**2*x2**&
&2*x3*x4**2
dpdx4(424) = 2*x1**3*x4*x5*x6**2+2*x2**3*x4*x5**2*x6+3*x2**2*x3**&
&2*x4**2*x6+3*x1**2*x3**2*x4**2*x5
dpdx4(425) = 3*x1**2*x4**2*x5*x6**2+3*x2**2*x4**2*x5**2*x6+2*x2**&
&3*x3**2*x4*x6+2*x1**3*x3**2*x4*x5
dpdx4(426) = 3*x1**2*x2**2*x4**2*x6+2*x1**2*x2**3*x4*x6+3*x1**2*x&
&2**2*x4**2*x5+2*x1**3*x2**2*x4*x5+2*x1**2*x2**3*x3*x4+2*x1**3*x2*&
&*2*x3*x4
dpdx4(427) = 2*x2**3*x4*x5*x6**2+3*x2**2*x3*x4**2*x6**2+2*x1**3*x&
&4*x5**2*x6+3*x1**2*x3*x4**2*x5**2
dpdx4(428) = 3*x2**2*x4**2*x5*x6**2+2*x2**3*x3*x4*x6**2+3*x1**2*x&
&4**2*x5**2*x6+2*x1**3*x3*x4*x5**2
dpdx4(429) = 3*x1*x2**2*x4**2*x6**2+2*x1*x2**3*x4*x6**2+3*x1**2*x&
&2*x4**2*x5**2+2*x1**3*x2*x4*x5**2+x1**2*x2**3*x3**2+x1**3*x2**2*x&
&3**2
dpdx4(430) = 3*x1**3*x3*x4**2*x6+3*x2**3*x3*x4**2*x5
dpdx4(431) = 3*x1**3*x2*x4**2*x6+x1**3*x2**3*x6+3*x1*x2**3*x4**2*&
&x5+x1**3*x2**3*x5+3*x1*x2**3*x3*x4**2+3*x1**3*x2*x3*x4**2
dpdx4(432) = 3*x2**3*x4**2*x5*x6+3*x1**3*x4**2*x5*x6+3*x2**3*x3*x&
&4**2*x6+3*x1**3*x3*x4**2*x5
dpdx4(433) = 3*x1*x2**3*x4**2*x6+3*x1**3*x2*x4**2*x5+x1**3*x2**3*&
&x3
dpdx4(434) = 4*x3**2*x4**3*x5*x6
dpdx4(435) = 4*x3*x4**3*x5*x6**2+4*x3*x4**3*x5**2*x6
dpdx4(436) = x1**4*x3*x6**2+4*x1*x3**2*x4**3*x6+x2**4*x3*x5**2+4*&
&x2*x3**2*x4**3*x5
dpdx4(437) = 4*x1*x3*x4**3*x6**2+x1**4*x3**2*x6+4*x2*x3*x4**3*x5*&
&*2+x2**4*x3**2*x5
dpdx4(438) = x1**4*x2*x6**2+x1*x2**4*x5**2+4*x1*x2*x3**2*x4**3
dpdx4(439) = x1**4*x5*x6**2+x2**4*x5**2*x6+4*x2*x3**2*x4**3*x6+4*&
&x1*x3**2*x4**3*x5
dpdx4(440) = 4*x1*x4**3*x5*x6**2+4*x2*x4**3*x5**2*x6+x2**4*x3**2*&
&x6+x1**4*x3**2*x5
dpdx4(441) = x2**4*x5*x6**2+4*x2*x3*x4**3*x6**2+x1**4*x5**2*x6+4*&
&x1*x3*x4**3*x5**2
dpdx4(442) = 4*x2*x4**3*x5*x6**2+x2**4*x3*x6**2+4*x1*x4**3*x5**2*&
&x6+x1**4*x3*x5**2
dpdx4(443) = 4*x1*x2*x4**3*x6**2+x1*x2**4*x6**2+4*x1*x2*x4**3*x5*&
&*2+x1**4*x2*x5**2+x1*x2**4*x3**2+x1**4*x2*x3**2
dpdx4(444) = 4*x1**2*x3*x4**3*x6+2*x1**4*x3*x4*x6+4*x2**2*x3*x4**&
&3*x5+2*x2**4*x3*x4*x5
dpdx4(445) = 2*x1**4*x2*x4*x6+x1**4*x2**2*x6+2*x1*x2**4*x4*x5+x1*&
&*2*x2**4*x5+4*x1*x2**2*x3*x4**3+4*x1**2*x2*x3*x4**3
dpdx4(446) = 4*x1**2*x2*x4**3*x6+x1**2*x2**4*x6+4*x1*x2**2*x4**3*&
&x5+x1**4*x2**2*x5+2*x1*x2**4*x3*x4+2*x1**4*x2*x3*x4
dpdx4(447) = 2*x2**4*x4*x5*x6+2*x1**4*x4*x5*x6+4*x2**2*x3*x4**3*x&
&6+4*x1**2*x3*x4**3*x5
dpdx4(448) = 4*x2**2*x4**3*x5*x6+4*x1**2*x4**3*x5*x6+2*x2**4*x3*x&
&4*x6+2*x1**4*x3*x4*x5
dpdx4(449) = 4*x1*x2**2*x4**3*x6+2*x1*x2**4*x4*x6+4*x1**2*x2*x4**&
&3*x5+2*x1**4*x2*x4*x5+x1**2*x2**4*x3+x1**4*x2**2*x3
dpdx4(450) = 5*x3*x4**4*x5*x6
dpdx4(451) = 5*x1*x3*x4**4*x6+x1**5*x3*x6+5*x2*x3*x4**4*x5+x2**5*&
&x3*x5
dpdx4(452) = x1**5*x2*x6+x1*x2**5*x5+5*x1*x2*x3*x4**4
dpdx4(453) = x2**5*x5*x6+x1**5*x5*x6+5*x2*x3*x4**4*x6+5*x1*x3*x4*&
&*4*x5
dpdx4(454) = 5*x2*x4**4*x5*x6+5*x1*x4**4*x5*x6+x2**5*x3*x6+x1**5*&
&x3*x5
dpdx4(455) = 5*x1*x2*x4**4*x6+x1*x2**5*x6+5*x1*x2*x4**4*x5+x1**5*&
&x2*x5+x1*x2**5*x3+x1**5*x2*x3
dpdx4(456) = 0
dpdx4(457) = 0
dpdx4(458) = 0
dpdx4(459) = 0
dpdx4(460) = 0
dpdx4(461) = x3**6*x6+x3**6*x5
dpdx4(462) = x3**5*x6**2+x3**5*x5**2
dpdx4(463) = x3**4*x6**3+x3**4*x5**3
dpdx4(464) = x3**3*x6**4+x3**3*x5**4
dpdx4(465) = x5**3*x6**4+x5**4*x6**3
dpdx4(466) = x3**2*x6**5+x3**2*x5**5
dpdx4(467) = x5**2*x6**5+x5**5*x6**2
dpdx4(468) = x3*x6**6+x3*x5**6
dpdx4(469) = x5*x6**6+x5**6*x6
dpdx4(470) = x1*x6**6+x2*x5**6+x2*x3**6+x1*x3**6
dpdx4(471) = x2*x6**6+x1*x5**6
dpdx4(472) = 2*x3**5*x4*x6+2*x3**5*x4*x5
dpdx4(473) = 2*x3**4*x4*x6**2+2*x3**4*x4*x5**2
dpdx4(474) = 2*x3**3*x4*x6**3+2*x3**3*x4*x5**3
dpdx4(475) = 2*x4*x5**3*x6**3
dpdx4(476) = 2*x3**2*x4*x6**4+2*x3**2*x4*x5**4
dpdx4(477) = 2*x4*x5**2*x6**4+2*x4*x5**4*x6**2
dpdx4(478) = 2*x3*x4*x6**5+2*x3*x4*x5**5
dpdx4(479) = 2*x4*x5*x6**5+2*x4*x5**5*x6
dpdx4(480) = x1**2*x6**5+x2**2*x5**5+2*x2*x3**5*x4+2*x1*x3**5*x4
dpdx4(481) = 2*x1*x4*x6**5+2*x2*x4*x5**5+x2**2*x3**5+x1**2*x3**5
dpdx4(482) = 2*x2*x4*x6**5+x2**2*x6**5+2*x1*x4*x5**5+x1**2*x5**5
dpdx4(483) = 2*x1**2*x4*x6**4+2*x2**2*x4*x5**4+2*x2**2*x3**4*x4+2&
&*x1**2*x3**4*x4
dpdx4(484) = 2*x2**2*x4*x6**4+2*x1**2*x4*x5**4
dpdx4(485) = 3*x3**4*x4**2*x6+3*x3**4*x4**2*x5
dpdx4(486) = 3*x3**3*x4**2*x6**2+3*x3**3*x4**2*x5**2
dpdx4(487) = 3*x3**2*x4**2*x6**3+3*x3**2*x4**2*x5**3
dpdx4(488) = 3*x4**2*x5**2*x6**3+3*x4**2*x5**3*x6**2
dpdx4(489) = 3*x3*x4**2*x6**4+3*x3*x4**2*x5**4
dpdx4(490) = 3*x4**2*x5*x6**4+3*x4**2*x5**4*x6
dpdx4(491) = x1**3*x6**4+x2**3*x5**4+3*x2*x3**4*x4**2+3*x1*x3**4*&
&x4**2
dpdx4(492) = 3*x1*x4**2*x6**4+3*x2*x4**2*x5**4+x2**3*x3**4+x1**3*&
&x3**4
dpdx4(493) = 3*x2*x4**2*x6**4+x2**3*x6**4+3*x1*x4**2*x5**4+x1**3*&
&x5**4
dpdx4(494) = 2*x1**3*x4*x6**3+2*x2**3*x4*x5**3+3*x2**2*x3**3*x4**&
&2+3*x1**2*x3**3*x4**2
dpdx4(495) = 3*x1**2*x4**2*x6**3+3*x2**2*x4**2*x5**3+2*x2**3*x3**&
&3*x4+2*x1**3*x3**3*x4
dpdx4(496) = 3*x2**2*x4**2*x6**3+2*x2**3*x4*x6**3+3*x1**2*x4**2*x&
&5**3+2*x1**3*x4*x5**3
dpdx4(497) = 3*x1**3*x4**2*x6**2+3*x2**3*x4**2*x5**2+3*x2**3*x3**&
&2*x4**2+3*x1**3*x3**2*x4**2
dpdx4(498) = 3*x1**2*x2**3*x4**2+3*x1**3*x2**2*x4**2+2*x1**3*x2**&
&3*x4
dpdx4(499) = 3*x2**3*x4**2*x6**2+3*x1**3*x4**2*x5**2
dpdx4(500) = 4*x3**3*x4**3*x6+4*x3**3*x4**3*x5
dpdx4(501) = 4*x3**2*x4**3*x6**2+4*x3**2*x4**3*x5**2
dpdx4(502) = 4*x4**3*x5**2*x6**2
dpdx4(503) = 4*x3*x4**3*x6**3+4*x3*x4**3*x5**3
dpdx4(504) = 4*x4**3*x5*x6**3+4*x4**3*x5**3*x6
dpdx4(505) = x1**4*x6**3+x2**4*x5**3+4*x2*x3**3*x4**3+4*x1*x3**3*&
&x4**3
dpdx4(506) = 4*x1*x4**3*x6**3+4*x2*x4**3*x5**3+x2**4*x3**3+x1**4*&
&x3**3
dpdx4(507) = 4*x2*x4**3*x6**3+x2**4*x6**3+4*x1*x4**3*x5**3+x1**4*&
&x5**3
dpdx4(508) = 2*x1**4*x4*x6**2+2*x2**4*x4*x5**2+4*x2**2*x3**2*x4**&
&3+4*x1**2*x3**2*x4**3
dpdx4(509) = 4*x1**2*x4**3*x6**2+4*x2**2*x4**3*x5**2+2*x2**4*x3**&
&2*x4+2*x1**4*x3**2*x4
dpdx4(510) = 4*x1**2*x2**2*x4**3+2*x1**2*x2**4*x4+2*x1**4*x2**2*x&
&4
dpdx4(511) = 4*x2**2*x4**3*x6**2+2*x2**4*x4*x6**2+4*x1**2*x4**3*x&
&5**2+2*x1**4*x4*x5**2
dpdx4(512) = 3*x1**4*x4**2*x6+3*x2**4*x4**2*x5+4*x2**3*x3*x4**3+4&
&*x1**3*x3*x4**3
dpdx4(513) = 4*x1**3*x4**3*x6+4*x2**3*x4**3*x5+3*x2**4*x3*x4**2+3&
&*x1**4*x3*x4**2
dpdx4(514) = 4*x1*x2**3*x4**3+4*x1**3*x2*x4**3+3*x1*x2**4*x4**2+3&
&*x1**4*x2*x4**2+x1**3*x2**4+x1**4*x2**3
dpdx4(515) = 4*x2**3*x4**3*x6+3*x2**4*x4**2*x6+4*x1**3*x4**3*x5+3&
&*x1**4*x4**2*x5
dpdx4(516) = 5*x3**2*x4**4*x6+5*x3**2*x4**4*x5
dpdx4(517) = 5*x3*x4**4*x6**2+5*x3*x4**4*x5**2
dpdx4(518) = 5*x4**4*x5*x6**2+5*x4**4*x5**2*x6
dpdx4(519) = x1**5*x6**2+x2**5*x5**2+5*x2*x3**2*x4**4+5*x1*x3**2*&
&x4**4
dpdx4(520) = 5*x1*x4**4*x6**2+5*x2*x4**4*x5**2+x2**5*x3**2+x1**5*&
&x3**2
dpdx4(521) = 5*x2*x4**4*x6**2+x2**5*x6**2+5*x1*x4**4*x5**2+x1**5*&
&x5**2
dpdx4(522) = 2*x1**5*x4*x6+2*x2**5*x4*x5+5*x2**2*x3*x4**4+5*x1**2&
&*x3*x4**4
dpdx4(523) = 5*x1**2*x4**4*x6+5*x2**2*x4**4*x5+2*x2**5*x3*x4+2*x1&
&**5*x3*x4
dpdx4(524) = 5*x1*x2**2*x4**4+5*x1**2*x2*x4**4+2*x1*x2**5*x4+2*x1&
&**5*x2*x4+x1**2*x2**5+x1**5*x2**2
dpdx4(525) = 5*x2**2*x4**4*x6+2*x2**5*x4*x6+5*x1**2*x4**4*x5+2*x1&
&**5*x4*x5
dpdx4(526) = 6*x3*x4**5*x6+6*x3*x4**5*x5
dpdx4(527) = 6*x4**5*x5*x6
dpdx4(528) = x1**6*x6+x2**6*x5+6*x2*x3*x4**5+6*x1*x3*x4**5
dpdx4(529) = 6*x1*x4**5*x6+6*x2*x4**5*x5+x2**6*x3+x1**6*x3
dpdx4(530) = 6*x1*x2*x4**5+x1*x2**6+x1**6*x2
dpdx4(531) = 6*x2*x4**5*x6+x2**6*x6+6*x1*x4**5*x5+x1**6*x5
dpdx4(532) = 0
dpdx4(533) = 0
dpdx4(534) = 0
dpdx4(535) = 0
dpdx4(536) = x6**7+x5**7
dpdx4(537) = 2*x4*x6**6+2*x4*x5**6
dpdx4(538) = 3*x4**2*x6**5+3*x4**2*x5**5
dpdx4(539) = 4*x4**3*x6**4+4*x4**3*x5**4
dpdx4(540) = 4*x2**4*x4**3+4*x1**4*x4**3
dpdx4(541) = 5*x4**4*x6**3+5*x4**4*x5**3
dpdx4(542) = 5*x2**3*x4**4+5*x1**3*x4**4+3*x2**5*x4**2+3*x1**5*x4&
&**2
dpdx4(543) = 6*x4**5*x6**2+6*x4**5*x5**2
dpdx4(544) = 6*x2**2*x4**5+6*x1**2*x4**5+2*x2**6*x4+2*x1**6*x4
dpdx4(545) = 7*x4**6*x6+7*x4**6*x5
dpdx4(546) = 7*x2*x4**6+7*x1*x4**6+x2**7+x1**7
dpdx5(1) = 0
dpdx5(2) = x6+x3
dpdx5(3) = x4+x1
dpdx5(4) = 0
dpdx5(5) = x3*x6
dpdx5(6) = x2*x6+x1*x6+x3*x4+x2*x3
dpdx5(7) = x4*x6+x1*x3
dpdx5(8) = x2*x4+x1*x2
dpdx5(9) = 0
dpdx5(10) = x1*x4
dpdx5(11) = x6**2+2*x5*x6+2*x3*x5+x3**2
dpdx5(12) = 2*x4*x5+2*x1*x5
dpdx5(13) = x4**2+x1**2
dpdx5(14) = 0
dpdx5(15) = x3*x4*x6+x2*x3*x6+x1*x3*x6
dpdx5(16) = x1*x2*x6+x2*x3*x4
dpdx5(17) = x2*x4*x6+x1*x4*x6+x1*x3*x4+x1*x2*x3
dpdx5(18) = x1*x2*x4
dpdx5(19) = x3*x6**2+2*x3*x5*x6+x3**2*x6
dpdx5(20) = x1*x6**2+2*x2*x5*x6+2*x2*x3*x5+x3**2*x4
dpdx5(21) = x2*x6**2+2*x1*x5*x6+2*x3*x4*x5+x2*x3**2
dpdx5(22) = x4*x6**2+2*x4*x5*x6+2*x1*x3*x5+x1*x3**2
dpdx5(23) = 2*x2*x4*x5+2*x1*x2*x5
dpdx5(24) = 2*x1*x4*x5
dpdx5(25) = x2**2*x6+x1**2*x6+x3*x4**2+x2**2*x3
dpdx5(26) = x4**2*x6+x1**2*x3
dpdx5(27) = x2**2*x4+x1*x2**2
dpdx5(28) = x2*x4**2+x1**2*x2
dpdx5(29) = 0
dpdx5(30) = x1*x4**2+x1**2*x4
dpdx5(31) = 2*x5*x6**2+2*x3**2*x5
dpdx5(32) = x6**3+3*x5**2*x6+3*x3*x5**2+x3**3
dpdx5(33) = 3*x4*x5**2+3*x1*x5**2
dpdx5(34) = 2*x4**2*x5+2*x1**2*x5
dpdx5(35) = 0
dpdx5(36) = x4**3+x1**3
dpdx5(37) = 0
dpdx5(38) = x2*x3*x4*x6+x1*x3*x4*x6+x1*x2*x3*x6
dpdx5(39) = x1*x2*x4*x6+x1*x2*x3*x4
dpdx5(40) = x1*x3*x6**2+2*x2*x3*x5*x6+x3**2*x4*x6
dpdx5(41) = x3*x4*x6**2+x2*x3*x6**2+2*x3*x4*x5*x6+2*x1*x3*x5*x6+x&
&2*x3**2*x6+x1*x3**2*x6
dpdx5(42) = x1*x2*x6**2+2*x1*x2*x5*x6+2*x2*x3*x4*x5+x2*x3**2*x4
dpdx5(43) = x1*x4*x6**2+2*x2*x4*x5*x6+2*x1*x2*x3*x5+x1*x3**2*x4
dpdx5(44) = x2*x4*x6**2+2*x1*x4*x5*x6+2*x1*x3*x4*x5+x1*x2*x3**2
dpdx5(45) = 2*x1*x2*x4*x5
dpdx5(46) = x3*x4**2*x6+x2**2*x3*x6+x1**2*x3*x6
dpdx5(47) = x1*x2**2*x6+x1**2*x2*x6+x2*x3*x4**2+x2**2*x3*x4
dpdx5(48) = x1*x2**2*x4
dpdx5(49) = x2**2*x4*x6+x1**2*x4*x6+x1*x3*x4**2+x1*x2**2*x3
dpdx5(50) = x2*x4**2*x6+x1*x4**2*x6+x1**2*x3*x4+x1**2*x2*x3
dpdx5(51) = x1*x2*x4**2+x1**2*x2*x4
dpdx5(52) = 2*x3*x5*x6**2+x3**2*x6**2+2*x3**2*x5*x6
dpdx5(53) = x3*x6**3+3*x3*x5**2*x6+x3**3*x6
dpdx5(54) = x1*x6**3+3*x2*x5**2*x6+3*x2*x3*x5**2+x3**3*x4
dpdx5(55) = 2*x2*x5*x6**2+2*x1*x5*x6**2+2*x3**2*x4*x5+2*x2*x3**2*&
&x5
dpdx5(56) = 2*x4*x5*x6**2+2*x1*x3**2*x5
dpdx5(57) = x2*x6**3+3*x1*x5**2*x6+3*x3*x4*x5**2+x2*x3**3
dpdx5(58) = x4*x6**3+3*x4*x5**2*x6+3*x1*x3*x5**2+x1*x3**3
dpdx5(59) = 3*x2*x4*x5**2+3*x1*x2*x5**2
dpdx5(60) = 3*x1*x4*x5**2
dpdx5(61) = x1**2*x6**2+2*x2**2*x5*x6+2*x2**2*x3*x5+x3**2*x4**2
dpdx5(62) = x2**2*x6**2+2*x1**2*x5*x6+2*x3*x4**2*x5+x2**2*x3**2
dpdx5(63) = x4**2*x6**2+2*x4**2*x5*x6+2*x1**2*x3*x5+x1**2*x3**2
dpdx5(64) = 2*x2**2*x4*x5+2*x1*x2**2*x5
dpdx5(65) = 2*x2*x4**2*x5+2*x1**2*x2*x5
dpdx5(66) = 2*x1*x4**2*x5+2*x1**2*x4*x5
dpdx5(67) = x2**2*x4**2+x1**2*x2**2
dpdx5(68) = 0
dpdx5(69) = x1**2*x4**2
dpdx5(70) = x2**3*x6+x1**3*x6+x3*x4**3+x2**3*x3
dpdx5(71) = x4**3*x6+x1**3*x3
dpdx5(72) = x2**3*x4+x1*x2**3
dpdx5(73) = x2*x4**3+x1**3*x2
dpdx5(74) = 0
dpdx5(75) = x1*x4**3+x1**3*x4
dpdx5(76) = 2*x5*x6**3+3*x5**2*x6**2+3*x3**2*x5**2+2*x3**3*x5
dpdx5(77) = x6**4+4*x5**3*x6+4*x3*x5**3+x3**4
dpdx5(78) = 4*x4*x5**3+4*x1*x5**3
dpdx5(79) = 3*x4**2*x5**2+3*x1**2*x5**2
dpdx5(80) = 2*x4**3*x5+2*x1**3*x5
dpdx5(81) = 0
dpdx5(82) = x4**4+x1**4
dpdx5(83) = 0
dpdx5(84) = x1*x2*x3*x4*x6
dpdx5(85) = x1*x3*x4*x6**2+x1*x2*x3*x6**2+2*x2*x3*x4*x5*x6+2*x1*x&
&2*x3*x5*x6+x2*x3**2*x4*x6+x1*x3**2*x4*x6
dpdx5(86) = x2*x3*x4*x6**2+2*x1*x3*x4*x5*x6+x1*x2*x3**2*x6
dpdx5(87) = x1*x2*x4*x6**2+2*x1*x2*x4*x5*x6+2*x1*x2*x3*x4*x5+x1*x&
&2*x3**2*x4
dpdx5(88) = x2*x3*x4**2*x6+x1*x3*x4**2*x6+x2**2*x3*x4*x6+x1**2*x3&
&*x4*x6+x1*x2**2*x3*x6+x1**2*x2*x3*x6
dpdx5(89) = x1*x2**2*x4*x6+x1**2*x2*x4*x6+x1*x2*x3*x4**2+x1*x2**2&
&*x3*x4
dpdx5(90) = x1*x2*x4**2*x6+x1**2*x2*x3*x4
dpdx5(91) = x1*x3*x6**3+3*x2*x3*x5**2*x6+x3**3*x4*x6
dpdx5(92) = 2*x2*x3*x5*x6**2+2*x1*x3*x5*x6**2+x3**2*x4*x6**2+x1*x&
&3**2*x6**2+2*x3**2*x4*x5*x6+2*x2*x3**2*x5*x6
dpdx5(93) = 2*x3*x4*x5*x6**2+x2*x3**2*x6**2+2*x1*x3**2*x5*x6
dpdx5(94) = x3*x4*x6**3+x2*x3*x6**3+3*x3*x4*x5**2*x6+3*x1*x3*x5**&
&2*x6+x2*x3**3*x6+x1*x3**3*x6
dpdx5(95) = 2*x1*x2*x5*x6**2+2*x2*x3**2*x4*x5
dpdx5(96) = x1*x2*x6**3+3*x1*x2*x5**2*x6+3*x2*x3*x4*x5**2+x2*x3**&
&3*x4
dpdx5(97) = x1*x4*x6**3+3*x2*x4*x5**2*x6+3*x1*x2*x3*x5**2+x1*x3**&
&3*x4
dpdx5(98) = 2*x2*x4*x5*x6**2+2*x1*x4*x5*x6**2+2*x1*x3**2*x4*x5+2*&
&x1*x2*x3**2*x5
dpdx5(99) = x2*x4*x6**3+3*x1*x4*x5**2*x6+3*x1*x3*x4*x5**2+x1*x2*x&
&3**3
dpdx5(100) = 3*x1*x2*x4*x5**2
dpdx5(101) = x1**2*x3*x6**2+2*x2**2*x3*x5*x6+x3**2*x4**2*x6
dpdx5(102) = x3*x4**2*x6**2+x2**2*x3*x6**2+2*x3*x4**2*x5*x6+2*x1*&
&*2*x3*x5*x6+x2**2*x3**2*x6+x1**2*x3**2*x6
dpdx5(103) = x1**2*x2*x6**2+2*x1*x2**2*x5*x6+2*x2**2*x3*x4*x5+x2*&
&x3**2*x4**2
dpdx5(104) = x1*x2**2*x6**2+2*x1**2*x2*x5*x6+2*x2*x3*x4**2*x5+x2*&
&*2*x3**2*x4
dpdx5(105) = 2*x1*x2**2*x4*x5
dpdx5(106) = x1**2*x4*x6**2+2*x2**2*x4*x5*x6+2*x1*x2**2*x3*x5+x1*&
&x3**2*x4**2
dpdx5(107) = x1*x4**2*x6**2+2*x2*x4**2*x5*x6+2*x1**2*x2*x3*x5+x1*&
&*2*x3**2*x4
dpdx5(108) = x2**2*x4*x6**2+2*x1**2*x4*x5*x6+2*x1*x3*x4**2*x5+x1*&
&x2**2*x3**2
dpdx5(109) = x2*x4**2*x6**2+2*x1*x4**2*x5*x6+2*x1**2*x3*x4*x5+x1*&
&*2*x2*x3**2
dpdx5(110) = 2*x1*x2*x4**2*x5+2*x1**2*x2*x4*x5
dpdx5(111) = x1**2*x2**2*x6+x2**2*x3*x4**2
dpdx5(112) = x1*x2**2*x4**2+x1**2*x2**2*x4
dpdx5(113) = x2**2*x4**2*x6+x1**2*x4**2*x6+x1**2*x3*x4**2+x1**2*x&
&2**2*x3
dpdx5(114) = x1**2*x2*x4**2
dpdx5(115) = x3*x4**3*x6+x2**3*x3*x6+x1**3*x3*x6
dpdx5(116) = x1*x2**3*x6+x1**3*x2*x6+x2*x3*x4**3+x2**3*x3*x4
dpdx5(117) = x1*x2**3*x4
dpdx5(118) = x2**3*x4*x6+x1**3*x4*x6+x1*x3*x4**3+x1*x2**3*x3
dpdx5(119) = x2*x4**3*x6+x1*x4**3*x6+x1**3*x3*x4+x1**3*x2*x3
dpdx5(120) = x1*x2*x4**3+x1**3*x2*x4
dpdx5(121) = 2*x3**2*x5*x6**2
dpdx5(122) = 2*x3*x5*x6**3+x3**2*x6**3+3*x3*x5**2*x6**2+x3**3*x6*&
&*2+3*x3**2*x5**2*x6+2*x3**3*x5*x6
dpdx5(123) = x3*x6**4+4*x3*x5**3*x6+x3**4*x6
dpdx5(124) = x1*x6**4+4*x2*x5**3*x6+4*x2*x3*x5**3+x3**4*x4
dpdx5(125) = 2*x1*x5*x6**3+3*x2*x5**2*x6**2+3*x2*x3**2*x5**2+2*x3&
&**3*x4*x5
dpdx5(126) = 2*x2*x5*x6**3+3*x1*x5**2*x6**2+3*x3**2*x4*x5**2+2*x2&
&*x3**3*x5
dpdx5(127) = 2*x4*x5*x6**3+3*x4*x5**2*x6**2+3*x1*x3**2*x5**2+2*x1&
&*x3**3*x5
dpdx5(128) = x2*x6**4+4*x1*x5**3*x6+4*x3*x4*x5**3+x2*x3**4
dpdx5(129) = x4*x6**4+4*x4*x5**3*x6+4*x1*x3*x5**3+x1*x3**4
dpdx5(130) = 4*x2*x4*x5**3+4*x1*x2*x5**3
dpdx5(131) = 4*x1*x4*x5**3
dpdx5(132) = x1**2*x6**3+3*x2**2*x5**2*x6+3*x2**2*x3*x5**2+x3**3*&
&x4**2
dpdx5(133) = 2*x2**2*x5*x6**2+2*x1**2*x5*x6**2+2*x3**2*x4**2*x5+2&
&*x2**2*x3**2*x5
dpdx5(134) = 2*x4**2*x5*x6**2+2*x1**2*x3**2*x5
dpdx5(135) = x2**2*x6**3+3*x1**2*x5**2*x6+3*x3*x4**2*x5**2+x2**2*&
&x3**3
dpdx5(136) = x4**2*x6**3+3*x4**2*x5**2*x6+3*x1**2*x3*x5**2+x1**2*&
&x3**3
dpdx5(137) = 3*x2**2*x4*x5**2+3*x1*x2**2*x5**2
dpdx5(138) = 3*x2*x4**2*x5**2+3*x1**2*x2*x5**2
dpdx5(139) = 3*x1*x4**2*x5**2+3*x1**2*x4*x5**2
dpdx5(140) = 2*x2**2*x4**2*x5+2*x1**2*x2**2*x5
dpdx5(141) = 0
dpdx5(142) = 2*x1**2*x4**2*x5
dpdx5(143) = x1**3*x6**2+2*x2**3*x5*x6+2*x2**3*x3*x5+x3**2*x4**3
dpdx5(144) = x2**3*x6**2+2*x1**3*x5*x6+2*x3*x4**3*x5+x2**3*x3**2
dpdx5(145) = x4**3*x6**2+2*x4**3*x5*x6+2*x1**3*x3*x5+x1**3*x3**2
dpdx5(146) = 2*x2**3*x4*x5+2*x1*x2**3*x5
dpdx5(147) = 2*x2*x4**3*x5+2*x1**3*x2*x5
dpdx5(148) = 2*x1*x4**3*x5+2*x1**3*x4*x5
dpdx5(149) = x2**3*x4**2+x1**2*x2**3
dpdx5(150) = x2**2*x4**3+x1**3*x2**2
dpdx5(151) = 0
dpdx5(152) = x1**2*x4**3+x1**3*x4**2
dpdx5(153) = x2**4*x6+x1**4*x6+x3*x4**4+x2**4*x3
dpdx5(154) = x4**4*x6+x1**4*x3
dpdx5(155) = x2**4*x4+x1*x2**4
dpdx5(156) = x2*x4**4+x1**4*x2
dpdx5(157) = 0
dpdx5(158) = x1*x4**4+x1**4*x4
dpdx5(159) = 3*x5**2*x6**3+3*x3**3*x5**2
dpdx5(160) = 2*x5*x6**4+4*x5**3*x6**2+4*x3**2*x5**3+2*x3**4*x5
dpdx5(161) = x6**5+5*x5**4*x6+5*x3*x5**4+x3**5
dpdx5(162) = 5*x4*x5**4+5*x1*x5**4
dpdx5(163) = 4*x4**2*x5**3+4*x1**2*x5**3
dpdx5(164) = 3*x4**3*x5**2+3*x1**3*x5**2
dpdx5(165) = 0
dpdx5(166) = 2*x4**4*x5+2*x1**4*x5
dpdx5(167) = 0
dpdx5(168) = x4**5+x1**5
dpdx5(169) = 0
dpdx5(170) = x1*x2*x3*x4*x6**2+2*x1*x2*x3*x4*x5*x6+x1*x2*x3**2*x4&
&*x6
dpdx5(171) = x1*x2*x3*x4**2*x6+x1*x2**2*x3*x4*x6+x1**2*x2*x3*x4*x&
&6
dpdx5(172) = 2*x1*x2*x3*x5*x6**2+x1*x3**2*x4*x6**2+2*x2*x3**2*x4*&
&x5*x6
dpdx5(173) = x1*x3*x4*x6**3+x1*x2*x3*x6**3+3*x2*x3*x4*x5**2*x6+3*&
&x1*x2*x3*x5**2*x6+x2*x3**3*x4*x6+x1*x3**3*x4*x6
dpdx5(174) = 2*x2*x3*x4*x5*x6**2+2*x1*x3*x4*x5*x6**2+x2*x3**2*x4*&
&x6**2+x1*x2*x3**2*x6**2+2*x1*x3**2*x4*x5*x6+2*x1*x2*x3**2*x5*x6
dpdx5(175) = 2*x1*x2*x4*x5*x6**2+2*x1*x2*x3**2*x4*x5
dpdx5(176) = x2*x3*x4*x6**3+3*x1*x3*x4*x5**2*x6+x1*x2*x3**3*x6
dpdx5(177) = x1*x2*x4*x6**3+3*x1*x2*x4*x5**2*x6+3*x1*x2*x3*x4*x5*&
&*2+x1*x2*x3**3*x4
dpdx5(178) = x1**2*x3*x4*x6**2+x1**2*x2*x3*x6**2+2*x2**2*x3*x4*x5&
&*x6+2*x1*x2**2*x3*x5*x6+x2*x3**2*x4**2*x6+x1*x3**2*x4**2*x6
dpdx5(179) = x1*x3*x4**2*x6**2+x1*x2**2*x3*x6**2+2*x2*x3*x4**2*x5&
&*x6+2*x1**2*x2*x3*x5*x6+x2**2*x3**2*x4*x6+x1**2*x3**2*x4*x6
dpdx5(180) = x1**2*x2*x4*x6**2+2*x1*x2**2*x4*x5*x6+2*x1*x2**2*x3*&
&x4*x5+x1*x2*x3**2*x4**2
dpdx5(181) = x2*x3*x4**2*x6**2+x2**2*x3*x4*x6**2+2*x1*x3*x4**2*x5&
&*x6+2*x1**2*x3*x4*x5*x6+x1*x2**2*x3**2*x6+x1**2*x2*x3**2*x6
dpdx5(182) = x1*x2**2*x4*x6**2+2*x1**2*x2*x4*x5*x6+2*x1*x2*x3*x4*&
&*2*x5+x1*x2**2*x3**2*x4
dpdx5(183) = x1*x2*x4**2*x6**2+2*x1*x2*x4**2*x5*x6+2*x1**2*x2*x3*&
&x4*x5+x1**2*x2*x3**2*x4
dpdx5(184) = x1**2*x2**2*x4*x6+x1*x2**2*x3*x4**2
dpdx5(185) = x2**2*x3*x4**2*x6+x1**2*x3*x4**2*x6+x1**2*x2**2*x3*x&
&6
dpdx5(186) = x1*x2**2*x4**2*x6+x1**2*x2*x4**2*x6+x1**2*x2*x3*x4**&
&2+x1**2*x2**2*x3*x4
dpdx5(187) = x2*x3*x4**3*x6+x1*x3*x4**3*x6+x2**3*x3*x4*x6+x1**3*x&
&3*x4*x6+x1*x2**3*x3*x6+x1**3*x2*x3*x6
dpdx5(188) = x1*x2**3*x4*x6+x1**3*x2*x4*x6+x1*x2*x3*x4**3+x1*x2**&
&3*x3*x4
dpdx5(189) = x1*x2*x4**3*x6+x1**3*x2*x3*x4
dpdx5(190) = x1*x3*x6**4+4*x2*x3*x5**3*x6+x3**4*x4*x6
dpdx5(191) = 2*x1*x3*x5*x6**3+x1*x3**2*x6**3+3*x2*x3*x5**2*x6**2+&
&x3**3*x4*x6**2+3*x2*x3**2*x5**2*x6+2*x3**3*x4*x5*x6
dpdx5(192) = 2*x3**2*x4*x5*x6**2+2*x2*x3**2*x5*x6**2+2*x1*x3**2*x&
&5*x6**2
dpdx5(193) = 2*x2*x3*x5*x6**3+x3**2*x4*x6**3+3*x1*x3*x5**2*x6**2+&
&x1*x3**3*x6**2+3*x3**2*x4*x5**2*x6+2*x2*x3**3*x5*x6
dpdx5(194) = 2*x3*x4*x5*x6**3+x2*x3**2*x6**3+3*x3*x4*x5**2*x6**2+&
&x2*x3**3*x6**2+3*x1*x3**2*x5**2*x6+2*x1*x3**3*x5*x6
dpdx5(195) = x3*x4*x6**4+x2*x3*x6**4+4*x3*x4*x5**3*x6+4*x1*x3*x5*&
&*3*x6+x2*x3**4*x6+x1*x3**4*x6
dpdx5(196) = 2*x1*x2*x5*x6**3+3*x1*x2*x5**2*x6**2+3*x2*x3**2*x4*x&
&5**2+2*x2*x3**3*x4*x5
dpdx5(197) = x1*x2*x6**4+4*x1*x2*x5**3*x6+4*x2*x3*x4*x5**3+x2*x3*&
&*4*x4
dpdx5(198) = x1*x4*x6**4+4*x2*x4*x5**3*x6+4*x1*x2*x3*x5**3+x1*x3*&
&*4*x4
dpdx5(199) = 2*x1*x4*x5*x6**3+3*x2*x4*x5**2*x6**2+3*x1*x2*x3**2*x&
&5**2+2*x1*x3**3*x4*x5
dpdx5(200) = 2*x2*x4*x5*x6**3+3*x1*x4*x5**2*x6**2+3*x1*x3**2*x4*x&
&5**2+2*x1*x2*x3**3*x5
dpdx5(201) = x2*x4*x6**4+4*x1*x4*x5**3*x6+4*x1*x3*x4*x5**3+x1*x2*&
&x3**4
dpdx5(202) = 4*x1*x2*x4*x5**3
dpdx5(203) = x1**2*x3*x6**3+3*x2**2*x3*x5**2*x6+x3**3*x4**2*x6
dpdx5(204) = 2*x2**2*x3*x5*x6**2+2*x1**2*x3*x5*x6**2+x3**2*x4**2*&
&x6**2+x1**2*x3**2*x6**2+2*x3**2*x4**2*x5*x6+2*x2**2*x3**2*x5*x6
dpdx5(205) = 2*x3*x4**2*x5*x6**2+x2**2*x3**2*x6**2+2*x1**2*x3**2*&
&x5*x6
dpdx5(206) = x3*x4**2*x6**3+x2**2*x3*x6**3+3*x3*x4**2*x5**2*x6+3*&
&x1**2*x3*x5**2*x6+x2**2*x3**3*x6+x1**2*x3**3*x6
dpdx5(207) = x1**2*x2*x6**3+3*x1*x2**2*x5**2*x6+3*x2**2*x3*x4*x5*&
&*2+x2*x3**3*x4**2
dpdx5(208) = 2*x1*x2**2*x5*x6**2+2*x1**2*x2*x5*x6**2+2*x2*x3**2*x&
&4**2*x5+2*x2**2*x3**2*x4*x5
dpdx5(209) = x1*x2**2*x6**3+3*x1**2*x2*x5**2*x6+3*x2*x3*x4**2*x5*&
&*2+x2**2*x3**3*x4
dpdx5(210) = 3*x1*x2**2*x4*x5**2
dpdx5(211) = x1**2*x4*x6**3+3*x2**2*x4*x5**2*x6+3*x1*x2**2*x3*x5*&
&*2+x1*x3**3*x4**2
dpdx5(212) = x1*x4**2*x6**3+3*x2*x4**2*x5**2*x6+3*x1**2*x2*x3*x5*&
&*2+x1**2*x3**3*x4
dpdx5(213) = 2*x2**2*x4*x5*x6**2+2*x1**2*x4*x5*x6**2+2*x1*x3**2*x&
&4**2*x5+2*x1*x2**2*x3**2*x5
dpdx5(214) = 2*x2*x4**2*x5*x6**2+2*x1*x4**2*x5*x6**2+2*x1**2*x3**&
&2*x4*x5+2*x1**2*x2*x3**2*x5
dpdx5(215) = x2**2*x4*x6**3+3*x1**2*x4*x5**2*x6+3*x1*x3*x4**2*x5*&
&*2+x1*x2**2*x3**3
dpdx5(216) = x2*x4**2*x6**3+3*x1*x4**2*x5**2*x6+3*x1**2*x3*x4*x5*&
&*2+x1**2*x2*x3**3
dpdx5(217) = 3*x1*x2*x4**2*x5**2+3*x1**2*x2*x4*x5**2
dpdx5(218) = x1**2*x2**2*x6**2+2*x1**2*x2**2*x5*x6+2*x2**2*x3*x4*&
&*2*x5+x2**2*x3**2*x4**2
dpdx5(219) = 2*x1*x2**2*x4**2*x5+2*x1**2*x2**2*x4*x5
dpdx5(220) = x1**2*x4**2*x6**2+2*x2**2*x4**2*x5*x6+2*x1**2*x2**2*&
&x3*x5+x1**2*x3**2*x4**2
dpdx5(221) = x1**2*x2**2*x4**2
dpdx5(222) = x2**2*x4**2*x6**2+2*x1**2*x4**2*x5*x6+2*x1**2*x3*x4*&
&*2*x5+x1**2*x2**2*x3**2
dpdx5(223) = 2*x1**2*x2*x4**2*x5
dpdx5(224) = x1**3*x3*x6**2+2*x2**3*x3*x5*x6+x3**2*x4**3*x6
dpdx5(225) = x3*x4**3*x6**2+x2**3*x3*x6**2+2*x3*x4**3*x5*x6+2*x1*&
&*3*x3*x5*x6+x2**3*x3**2*x6+x1**3*x3**2*x6
dpdx5(226) = x1**3*x2*x6**2+2*x1*x2**3*x5*x6+2*x2**3*x3*x4*x5+x2*&
&x3**2*x4**3
dpdx5(227) = x1*x2**3*x6**2+2*x1**3*x2*x5*x6+2*x2*x3*x4**3*x5+x2*&
&*3*x3**2*x4
dpdx5(228) = 2*x1*x2**3*x4*x5
dpdx5(229) = x1**3*x4*x6**2+2*x2**3*x4*x5*x6+2*x1*x2**3*x3*x5+x1*&
&x3**2*x4**3
dpdx5(230) = x1*x4**3*x6**2+2*x2*x4**3*x5*x6+2*x1**3*x2*x3*x5+x1*&
&*3*x3**2*x4
dpdx5(231) = x2**3*x4*x6**2+2*x1**3*x4*x5*x6+2*x1*x3*x4**3*x5+x1*&
&x2**3*x3**2
dpdx5(232) = x2*x4**3*x6**2+2*x1*x4**3*x5*x6+2*x1**3*x3*x4*x5+x1*&
&*3*x2*x3**2
dpdx5(233) = 2*x1*x2*x4**3*x5+2*x1**3*x2*x4*x5
dpdx5(234) = x1**2*x2**3*x6+x1**3*x2**2*x6+x2**2*x3*x4**3+x2**3*x&
&3*x4**2
dpdx5(235) = x1*x2**3*x4**2+x1**2*x2**3*x4
dpdx5(236) = x1*x2**2*x4**3+x1**3*x2**2*x4
dpdx5(237) = x2**3*x4**2*x6+x1**3*x4**2*x6+x1**2*x3*x4**3+x1**2*x&
&2**3*x3
dpdx5(238) = x2**2*x4**3*x6+x1**2*x4**3*x6+x1**3*x3*x4**2+x1**3*x&
&2**2*x3
dpdx5(239) = x1**2*x2*x4**3+x1**3*x2*x4**2
dpdx5(240) = x3*x4**4*x6+x2**4*x3*x6+x1**4*x3*x6
dpdx5(241) = x1*x2**4*x6+x1**4*x2*x6+x2*x3*x4**4+x2**4*x3*x4
dpdx5(242) = x1*x2**4*x4
dpdx5(243) = x2**4*x4*x6+x1**4*x4*x6+x1*x3*x4**4+x1*x2**4*x3
dpdx5(244) = x2*x4**4*x6+x1*x4**4*x6+x1**4*x3*x4+x1**4*x2*x3
dpdx5(245) = x1*x2*x4**4+x1**4*x2*x4
dpdx5(246) = 2*x3**2*x5*x6**3+3*x3**2*x5**2*x6**2+2*x3**3*x5*x6**&
&2
dpdx5(247) = 3*x3*x5**2*x6**3+x3**3*x6**3+3*x3**3*x5**2*x6
dpdx5(248) = 2*x3*x5*x6**4+x3**2*x6**4+4*x3*x5**3*x6**2+x3**4*x6*&
&*2+4*x3**2*x5**3*x6+2*x3**4*x5*x6
dpdx5(249) = x3*x6**5+5*x3*x5**4*x6+x3**5*x6
dpdx5(250) = x1*x6**5+5*x2*x5**4*x6+5*x2*x3*x5**4+x3**5*x4
dpdx5(251) = 2*x1*x5*x6**4+4*x2*x5**3*x6**2+4*x2*x3**2*x5**3+2*x3&
&**4*x4*x5
dpdx5(252) = 3*x2*x5**2*x6**3+3*x1*x5**2*x6**3+3*x3**3*x4*x5**2+3&
&*x2*x3**3*x5**2
dpdx5(253) = 3*x4*x5**2*x6**3+3*x1*x3**3*x5**2
dpdx5(254) = 2*x2*x5*x6**4+4*x1*x5**3*x6**2+4*x3**2*x4*x5**3+2*x2&
&*x3**4*x5
dpdx5(255) = 2*x4*x5*x6**4+4*x4*x5**3*x6**2+4*x1*x3**2*x5**3+2*x1&
&*x3**4*x5
dpdx5(256) = x2*x6**5+5*x1*x5**4*x6+5*x3*x4*x5**4+x2*x3**5
dpdx5(257) = x4*x6**5+5*x4*x5**4*x6+5*x1*x3*x5**4+x1*x3**5
dpdx5(258) = 5*x2*x4*x5**4+5*x1*x2*x5**4
dpdx5(259) = 5*x1*x4*x5**4
dpdx5(260) = x1**2*x6**4+4*x2**2*x5**3*x6+4*x2**2*x3*x5**3+x3**4*&
&x4**2
dpdx5(261) = 2*x1**2*x5*x6**3+3*x2**2*x5**2*x6**2+3*x2**2*x3**2*x&
&5**2+2*x3**3*x4**2*x5
dpdx5(262) = 2*x2**2*x5*x6**3+3*x1**2*x5**2*x6**2+3*x3**2*x4**2*x&
&5**2+2*x2**2*x3**3*x5
dpdx5(263) = 2*x4**2*x5*x6**3+3*x4**2*x5**2*x6**2+3*x1**2*x3**2*x&
&5**2+2*x1**2*x3**3*x5
dpdx5(264) = x2**2*x6**4+4*x1**2*x5**3*x6+4*x3*x4**2*x5**3+x2**2*&
&x3**4
dpdx5(265) = x4**2*x6**4+4*x4**2*x5**3*x6+4*x1**2*x3*x5**3+x1**2*&
&x3**4
dpdx5(266) = 4*x2**2*x4*x5**3+4*x1*x2**2*x5**3
dpdx5(267) = 4*x2*x4**2*x5**3+4*x1**2*x2*x5**3
dpdx5(268) = 4*x1*x4**2*x5**3+4*x1**2*x4*x5**3
dpdx5(269) = 3*x2**2*x4**2*x5**2+3*x1**2*x2**2*x5**2
dpdx5(270) = 3*x1**2*x4**2*x5**2
dpdx5(271) = x1**3*x6**3+3*x2**3*x5**2*x6+3*x2**3*x3*x5**2+x3**3*&
&x4**3
dpdx5(272) = 2*x2**3*x5*x6**2+2*x1**3*x5*x6**2+2*x3**2*x4**3*x5+2&
&*x2**3*x3**2*x5
dpdx5(273) = 2*x4**3*x5*x6**2+2*x1**3*x3**2*x5
dpdx5(274) = x2**3*x6**3+3*x1**3*x5**2*x6+3*x3*x4**3*x5**2+x2**3*&
&x3**3
dpdx5(275) = x4**3*x6**3+3*x4**3*x5**2*x6+3*x1**3*x3*x5**2+x1**3*&
&x3**3
dpdx5(276) = 3*x2**3*x4*x5**2+3*x1*x2**3*x5**2
dpdx5(277) = 3*x2*x4**3*x5**2+3*x1**3*x2*x5**2
dpdx5(278) = 3*x1*x4**3*x5**2+3*x1**3*x4*x5**2
dpdx5(279) = 2*x2**3*x4**2*x5+2*x1**2*x2**3*x5
dpdx5(280) = 2*x2**2*x4**3*x5+2*x1**3*x2**2*x5
dpdx5(281) = 0
dpdx5(282) = 2*x1**2*x4**3*x5+2*x1**3*x4**2*x5
dpdx5(283) = x2**3*x4**3+x1**3*x2**3
dpdx5(284) = 0
dpdx5(285) = x1**3*x4**3
dpdx5(286) = x1**4*x6**2+2*x2**4*x5*x6+2*x2**4*x3*x5+x3**2*x4**4
dpdx5(287) = x2**4*x6**2+2*x1**4*x5*x6+2*x3*x4**4*x5+x2**4*x3**2
dpdx5(288) = x4**4*x6**2+2*x4**4*x5*x6+2*x1**4*x3*x5+x1**4*x3**2
dpdx5(289) = 2*x2**4*x4*x5+2*x1*x2**4*x5
dpdx5(290) = 2*x2*x4**4*x5+2*x1**4*x2*x5
dpdx5(291) = 2*x1*x4**4*x5+2*x1**4*x4*x5
dpdx5(292) = x2**4*x4**2+x1**2*x2**4
dpdx5(293) = x2**2*x4**4+x1**4*x2**2
dpdx5(294) = 0
dpdx5(295) = x1**2*x4**4+x1**4*x4**2
dpdx5(296) = x2**5*x6+x1**5*x6+x3*x4**5+x2**5*x3
dpdx5(297) = x4**5*x6+x1**5*x3
dpdx5(298) = x2**5*x4+x1*x2**5
dpdx5(299) = x2*x4**5+x1**5*x2
dpdx5(300) = 0
dpdx5(301) = x1*x4**5+x1**5*x4
dpdx5(302) = 3*x5**2*x6**4+4*x5**3*x6**3+4*x3**3*x5**3+3*x3**4*x5&
&**2
dpdx5(303) = 2*x5*x6**5+5*x5**4*x6**2+5*x3**2*x5**4+2*x3**5*x5
dpdx5(304) = x6**6+6*x5**5*x6+6*x3*x5**5+x3**6
dpdx5(305) = 6*x4*x5**5+6*x1*x5**5
dpdx5(306) = 5*x4**2*x5**4+5*x1**2*x5**4
dpdx5(307) = 4*x4**3*x5**3+4*x1**3*x5**3
dpdx5(308) = 3*x4**4*x5**2+3*x1**4*x5**2
dpdx5(309) = 0
dpdx5(310) = 2*x4**5*x5+2*x1**5*x5
dpdx5(311) = 0
dpdx5(312) = x4**6+x1**6
dpdx5(313) = 0
dpdx5(314) = 2*x1*x2*x3*x4*x5*x6**2+x1*x2*x3**2*x4*x6**2+2*x1*x2*&
&x3**2*x4*x5*x6
dpdx5(315) = x1*x2*x3*x4*x6**3+3*x1*x2*x3*x4*x5**2*x6+x1*x2*x3**3&
&*x4*x6
dpdx5(316) = x1**2*x2*x3*x4*x6**2+2*x1*x2**2*x3*x4*x5*x6+x1*x2*x3&
&**2*x4**2*x6
dpdx5(317) = x1*x2*x3*x4**2*x6**2+x1*x2**2*x3*x4*x6**2+2*x1*x2*x3&
&*x4**2*x5*x6+2*x1**2*x2*x3*x4*x5*x6+x1*x2**2*x3**2*x4*x6+x1**2*x2&
&*x3**2*x4*x6
dpdx5(318) = x1*x2**2*x3*x4**2*x6+x1**2*x2*x3*x4**2*x6+x1**2*x2**&
&2*x3*x4*x6
dpdx5(319) = x1*x2*x3*x4**3*x6+x1*x2**3*x3*x4*x6+x1**3*x2*x3*x4*x&
&6
dpdx5(320) = 2*x1*x2*x3*x5*x6**3+x1*x3**2*x4*x6**3+3*x1*x2*x3*x5*&
&*2*x6**2+x1*x3**3*x4*x6**2+3*x2*x3**2*x4*x5**2*x6+2*x2*x3**3*x4*x&
&5*x6
dpdx5(321) = x1*x3*x4*x6**4+x1*x2*x3*x6**4+4*x2*x3*x4*x5**3*x6+4*&
&x1*x2*x3*x5**3*x6+x2*x3**4*x4*x6+x1*x3**4*x4*x6
dpdx5(322) = 2*x2*x3**2*x4*x5*x6**2+2*x1*x3**2*x4*x5*x6**2+2*x1*x&
&2*x3**2*x5*x6**2
dpdx5(323) = 2*x1*x3*x4*x5*x6**3+x1*x2*x3**2*x6**3+3*x2*x3*x4*x5*&
&*2*x6**2+x2*x3**3*x4*x6**2+3*x1*x2*x3**2*x5**2*x6+2*x1*x3**3*x4*x&
&5*x6
dpdx5(324) = 2*x2*x3*x4*x5*x6**3+x2*x3**2*x4*x6**3+3*x1*x3*x4*x5*&
&*2*x6**2+x1*x2*x3**3*x6**2+3*x1*x3**2*x4*x5**2*x6+2*x1*x2*x3**3*x&
&5*x6
dpdx5(325) = 2*x1*x2*x4*x5*x6**3+3*x1*x2*x4*x5**2*x6**2+3*x1*x2*x&
&3**2*x4*x5**2+2*x1*x2*x3**3*x4*x5
dpdx5(326) = x2*x3*x4*x6**4+4*x1*x3*x4*x5**3*x6+x1*x2*x3**4*x6
dpdx5(327) = x1*x2*x4*x6**4+4*x1*x2*x4*x5**3*x6+4*x1*x2*x3*x4*x5*&
&*3+x1*x2*x3**4*x4
dpdx5(328) = x1**2*x3*x4*x6**3+x1**2*x2*x3*x6**3+3*x2**2*x3*x4*x5&
&**2*x6+3*x1*x2**2*x3*x5**2*x6+x2*x3**3*x4**2*x6+x1*x3**3*x4**2*x6
dpdx5(329) = 2*x1*x2**2*x3*x5*x6**2+2*x1**2*x2*x3*x5*x6**2+x1*x3*&
&*2*x4**2*x6**2+x1**2*x3**2*x4*x6**2+2*x2*x3**2*x4**2*x5*x6+2*x2**&
&2*x3**2*x4*x5*x6
dpdx5(330) = x1*x3*x4**2*x6**3+x1*x2**2*x3*x6**3+3*x2*x3*x4**2*x5&
&**2*x6+3*x1**2*x2*x3*x5**2*x6+x2**2*x3**3*x4*x6+x1**2*x3**3*x4*x6
dpdx5(331) = x1**2*x2*x4*x6**3+3*x1*x2**2*x4*x5**2*x6+3*x1*x2**2*&
&x3*x4*x5**2+x1*x2*x3**3*x4**2
dpdx5(332) = 2*x2**2*x3*x4*x5*x6**2+2*x1**2*x3*x4*x5*x6**2+x2*x3*&
&*2*x4**2*x6**2+x1**2*x2*x3**2*x6**2+2*x1*x3**2*x4**2*x5*x6+2*x1*x&
&2**2*x3**2*x5*x6
dpdx5(333) = 2*x2*x3*x4**2*x5*x6**2+2*x1*x3*x4**2*x5*x6**2+x2**2*&
&x3**2*x4*x6**2+x1*x2**2*x3**2*x6**2+2*x1**2*x3**2*x4*x5*x6+2*x1**&
&2*x2*x3**2*x5*x6
dpdx5(334) = 2*x1*x2**2*x4*x5*x6**2+2*x1**2*x2*x4*x5*x6**2+2*x1*x&
&2*x3**2*x4**2*x5+2*x1*x2**2*x3**2*x4*x5
dpdx5(335) = 2*x1*x2*x4**2*x5*x6**2+2*x1**2*x2*x3**2*x4*x5
dpdx5(336) = x2*x3*x4**2*x6**3+x2**2*x3*x4*x6**3+3*x1*x3*x4**2*x5&
&**2*x6+3*x1**2*x3*x4*x5**2*x6+x1*x2**2*x3**3*x6+x1**2*x2*x3**3*x6
dpdx5(337) = x1*x2**2*x4*x6**3+3*x1**2*x2*x4*x5**2*x6+3*x1*x2*x3*&
&x4**2*x5**2+x1*x2**2*x3**3*x4
dpdx5(338) = x1*x2*x4**2*x6**3+3*x1*x2*x4**2*x5**2*x6+3*x1**2*x2*&
&x3*x4*x5**2+x1**2*x2*x3**3*x4
dpdx5(339) = x1**2*x2**2*x4*x6**2+2*x1**2*x2**2*x4*x5*x6+2*x1*x2*&
&*2*x3*x4**2*x5+x1*x2**2*x3**2*x4**2
dpdx5(340) = x1**2*x3*x4**2*x6**2+x1**2*x2**2*x3*x6**2+2*x2**2*x3&
&*x4**2*x5*x6+2*x1**2*x2**2*x3*x5*x6+x2**2*x3**2*x4**2*x6+x1**2*x3&
&**2*x4**2*x6
dpdx5(341) = x1**2*x2*x4**2*x6**2+2*x1*x2**2*x4**2*x5*x6+2*x1**2*&
&x2**2*x3*x4*x5+x1**2*x2*x3**2*x4**2
dpdx5(342) = x1**2*x2**2*x4**2*x6+x1**2*x2**2*x3*x4**2
dpdx5(343) = x2**2*x3*x4**2*x6**2+2*x1**2*x3*x4**2*x5*x6+x1**2*x2&
&**2*x3**2*x6
dpdx5(344) = x1*x2**2*x4**2*x6**2+2*x1**2*x2*x4**2*x5*x6+2*x1**2*&
&x2*x3*x4**2*x5+x1**2*x2**2*x3**2*x4
dpdx5(345) = x1**3*x3*x4*x6**2+x1**3*x2*x3*x6**2+2*x2**3*x3*x4*x5&
&*x6+2*x1*x2**3*x3*x5*x6+x2*x3**2*x4**3*x6+x1*x3**2*x4**3*x6
dpdx5(346) = x1*x3*x4**3*x6**2+x1*x2**3*x3*x6**2+2*x2*x3*x4**3*x5&
&*x6+2*x1**3*x2*x3*x5*x6+x2**3*x3**2*x4*x6+x1**3*x3**2*x4*x6
dpdx5(347) = x1**3*x2*x4*x6**2+2*x1*x2**3*x4*x5*x6+2*x1*x2**3*x3*&
&x4*x5+x1*x2*x3**2*x4**3
dpdx5(348) = x2*x3*x4**3*x6**2+x2**3*x3*x4*x6**2+2*x1*x3*x4**3*x5&
&*x6+2*x1**3*x3*x4*x5*x6+x1*x2**3*x3**2*x6+x1**3*x2*x3**2*x6
dpdx5(349) = x1*x2**3*x4*x6**2+2*x1**3*x2*x4*x5*x6+2*x1*x2*x3*x4*&
&*3*x5+x1*x2**3*x3**2*x4
dpdx5(350) = x1*x2*x4**3*x6**2+2*x1*x2*x4**3*x5*x6+2*x1**3*x2*x3*&
&x4*x5+x1**3*x2*x3**2*x4
dpdx5(351) = x1**2*x2**3*x4*x6+x1**3*x2**2*x4*x6+x1*x2**2*x3*x4**&
&3+x1*x2**3*x3*x4**2
dpdx5(352) = x2**2*x3*x4**3*x6+x1**2*x3*x4**3*x6+x2**3*x3*x4**2*x&
&6+x1**3*x3*x4**2*x6+x1**2*x2**3*x3*x6+x1**3*x2**2*x3*x6
dpdx5(353) = x1*x2**3*x4**2*x6+x1**3*x2*x4**2*x6+x1**2*x2*x3*x4**&
&3+x1**2*x2**3*x3*x4
dpdx5(354) = x1*x2**2*x4**3*x6+x1**2*x2*x4**3*x6+x1**3*x2*x3*x4**&
&2+x1**3*x2**2*x3*x4
dpdx5(355) = x2*x3*x4**4*x6+x1*x3*x4**4*x6+x2**4*x3*x4*x6+x1**4*x&
&3*x4*x6+x1*x2**4*x3*x6+x1**4*x2*x3*x6
dpdx5(356) = x1*x2**4*x4*x6+x1**4*x2*x4*x6+x1*x2*x3*x4**4+x1*x2**&
&4*x3*x4
dpdx5(357) = x1*x2*x4**4*x6+x1**4*x2*x3*x4
dpdx5(358) = x1*x3*x6**5+5*x2*x3*x5**4*x6+x3**5*x4*x6
dpdx5(359) = 2*x1*x3*x5*x6**4+x1*x3**2*x6**4+4*x2*x3*x5**3*x6**2+&
&x3**4*x4*x6**2+4*x2*x3**2*x5**3*x6+2*x3**4*x4*x5*x6
dpdx5(360) = 2*x1*x3**2*x5*x6**3+3*x2*x3**2*x5**2*x6**2+2*x3**3*x&
&4*x5*x6**2
dpdx5(361) = 3*x2*x3*x5**2*x6**3+3*x1*x3*x5**2*x6**3+x3**3*x4*x6*&
&*3+x1*x3**3*x6**3+3*x3**3*x4*x5**2*x6+3*x2*x3**3*x5**2*x6
dpdx5(362) = 2*x3**2*x4*x5*x6**3+2*x2*x3**2*x5*x6**3+3*x3**2*x4*x&
&5**2*x6**2+3*x1*x3**2*x5**2*x6**2+2*x2*x3**3*x5*x6**2+2*x1*x3**3*&
&x5*x6**2
dpdx5(363) = 3*x3*x4*x5**2*x6**3+x2*x3**3*x6**3+3*x1*x3**3*x5**2*&
&x6
dpdx5(364) = 2*x2*x3*x5*x6**4+x3**2*x4*x6**4+4*x1*x3*x5**3*x6**2+&
&x1*x3**4*x6**2+4*x3**2*x4*x5**3*x6+2*x2*x3**4*x5*x6
dpdx5(365) = 2*x3*x4*x5*x6**4+x2*x3**2*x6**4+4*x3*x4*x5**3*x6**2+&
&x2*x3**4*x6**2+4*x1*x3**2*x5**3*x6+2*x1*x3**4*x5*x6
dpdx5(366) = x3*x4*x6**5+x2*x3*x6**5+5*x3*x4*x5**4*x6+5*x1*x3*x5*&
&*4*x6+x2*x3**5*x6+x1*x3**5*x6
dpdx5(367) = 3*x1*x2*x5**2*x6**3+3*x2*x3**3*x4*x5**2
dpdx5(368) = 2*x1*x2*x5*x6**4+4*x1*x2*x5**3*x6**2+4*x2*x3**2*x4*x&
&5**3+2*x2*x3**4*x4*x5
dpdx5(369) = x1*x2*x6**5+5*x1*x2*x5**4*x6+5*x2*x3*x4*x5**4+x2*x3*&
&*5*x4
dpdx5(370) = x1*x4*x6**5+5*x2*x4*x5**4*x6+5*x1*x2*x3*x5**4+x1*x3*&
&*5*x4
dpdx5(371) = 2*x1*x4*x5*x6**4+4*x2*x4*x5**3*x6**2+4*x1*x2*x3**2*x&
&5**3+2*x1*x3**4*x4*x5
dpdx5(372) = 3*x2*x4*x5**2*x6**3+3*x1*x4*x5**2*x6**3+3*x1*x3**3*x&
&4*x5**2+3*x1*x2*x3**3*x5**2
dpdx5(373) = 2*x2*x4*x5*x6**4+4*x1*x4*x5**3*x6**2+4*x1*x3**2*x4*x&
&5**3+2*x1*x2*x3**4*x5
dpdx5(374) = x2*x4*x6**5+5*x1*x4*x5**4*x6+5*x1*x3*x4*x5**4+x1*x2*&
&x3**5
dpdx5(375) = 5*x1*x2*x4*x5**4
dpdx5(376) = x1**2*x3*x6**4+4*x2**2*x3*x5**3*x6+x3**4*x4**2*x6
dpdx5(377) = 2*x1**2*x3*x5*x6**3+x1**2*x3**2*x6**3+3*x2**2*x3*x5*&
&*2*x6**2+x3**3*x4**2*x6**2+3*x2**2*x3**2*x5**2*x6+2*x3**3*x4**2*x&
&5*x6
dpdx5(378) = 2*x3**2*x4**2*x5*x6**2+2*x2**2*x3**2*x5*x6**2+2*x1**&
&2*x3**2*x5*x6**2
dpdx5(379) = 2*x2**2*x3*x5*x6**3+x3**2*x4**2*x6**3+3*x1**2*x3*x5*&
&*2*x6**2+x1**2*x3**3*x6**2+3*x3**2*x4**2*x5**2*x6+2*x2**2*x3**3*x&
&5*x6
dpdx5(380) = 2*x3*x4**2*x5*x6**3+x2**2*x3**2*x6**3+3*x3*x4**2*x5*&
&*2*x6**2+x2**2*x3**3*x6**2+3*x1**2*x3**2*x5**2*x6+2*x1**2*x3**3*x&
&5*x6
dpdx5(381) = x3*x4**2*x6**4+x2**2*x3*x6**4+4*x3*x4**2*x5**3*x6+4*&
&x1**2*x3*x5**3*x6+x2**2*x3**4*x6+x1**2*x3**4*x6
dpdx5(382) = x1**2*x2*x6**4+4*x1*x2**2*x5**3*x6+4*x2**2*x3*x4*x5*&
&*3+x2*x3**4*x4**2
dpdx5(383) = 2*x1**2*x2*x5*x6**3+3*x1*x2**2*x5**2*x6**2+3*x2**2*x&
&3**2*x4*x5**2+2*x2*x3**3*x4**2*x5
dpdx5(384) = 2*x1*x2**2*x5*x6**3+3*x1**2*x2*x5**2*x6**2+3*x2*x3**&
&2*x4**2*x5**2+2*x2**2*x3**3*x4*x5
dpdx5(385) = x1*x2**2*x6**4+4*x1**2*x2*x5**3*x6+4*x2*x3*x4**2*x5*&
&*3+x2**2*x3**4*x4
dpdx5(386) = 4*x1*x2**2*x4*x5**3
dpdx5(387) = x1**2*x4*x6**4+4*x2**2*x4*x5**3*x6+4*x1*x2**2*x3*x5*&
&*3+x1*x3**4*x4**2
dpdx5(388) = x1*x4**2*x6**4+4*x2*x4**2*x5**3*x6+4*x1**2*x2*x3*x5*&
&*3+x1**2*x3**4*x4
dpdx5(389) = 2*x1**2*x4*x5*x6**3+3*x2**2*x4*x5**2*x6**2+3*x1*x2**&
&2*x3**2*x5**2+2*x1*x3**3*x4**2*x5
dpdx5(390) = 2*x1*x4**2*x5*x6**3+3*x2*x4**2*x5**2*x6**2+3*x1**2*x&
&2*x3**2*x5**2+2*x1**2*x3**3*x4*x5
dpdx5(391) = 2*x2**2*x4*x5*x6**3+3*x1**2*x4*x5**2*x6**2+3*x1*x3**&
&2*x4**2*x5**2+2*x1*x2**2*x3**3*x5
dpdx5(392) = 2*x2*x4**2*x5*x6**3+3*x1*x4**2*x5**2*x6**2+3*x1**2*x&
&3**2*x4*x5**2+2*x1**2*x2*x3**3*x5
dpdx5(393) = x2**2*x4*x6**4+4*x1**2*x4*x5**3*x6+4*x1*x3*x4**2*x5*&
&*3+x1*x2**2*x3**4
dpdx5(394) = x2*x4**2*x6**4+4*x1*x4**2*x5**3*x6+4*x1**2*x3*x4*x5*&
&*3+x1**2*x2*x3**4
dpdx5(395) = 4*x1*x2*x4**2*x5**3+4*x1**2*x2*x4*x5**3
dpdx5(396) = 2*x1**2*x2**2*x5*x6**2+2*x2**2*x3**2*x4**2*x5
dpdx5(397) = x1**2*x2**2*x6**3+3*x1**2*x2**2*x5**2*x6+3*x2**2*x3*&
&x4**2*x5**2+x2**2*x3**3*x4**2
dpdx5(398) = 3*x1*x2**2*x4**2*x5**2+3*x1**2*x2**2*x4*x5**2
dpdx5(399) = x1**2*x4**2*x6**3+3*x2**2*x4**2*x5**2*x6+3*x1**2*x2*&
&*2*x3*x5**2+x1**2*x3**3*x4**2
dpdx5(400) = 2*x2**2*x4**2*x5*x6**2+2*x1**2*x4**2*x5*x6**2+2*x1**&
&2*x3**2*x4**2*x5+2*x1**2*x2**2*x3**2*x5
dpdx5(401) = 2*x1**2*x2**2*x4**2*x5
dpdx5(402) = x2**2*x4**2*x6**3+3*x1**2*x4**2*x5**2*x6+3*x1**2*x3*&
&x4**2*x5**2+x1**2*x2**2*x3**3
dpdx5(403) = 3*x1**2*x2*x4**2*x5**2
dpdx5(404) = x1**3*x3*x6**3+3*x2**3*x3*x5**2*x6+x3**3*x4**3*x6
dpdx5(405) = 2*x2**3*x3*x5*x6**2+2*x1**3*x3*x5*x6**2+x3**2*x4**3*&
&x6**2+x1**3*x3**2*x6**2+2*x3**2*x4**3*x5*x6+2*x2**3*x3**2*x5*x6
dpdx5(406) = 2*x3*x4**3*x5*x6**2+x2**3*x3**2*x6**2+2*x1**3*x3**2*&
&x5*x6
dpdx5(407) = x3*x4**3*x6**3+x2**3*x3*x6**3+3*x3*x4**3*x5**2*x6+3*&
&x1**3*x3*x5**2*x6+x2**3*x3**3*x6+x1**3*x3**3*x6
dpdx5(408) = x1**3*x2*x6**3+3*x1*x2**3*x5**2*x6+3*x2**3*x3*x4*x5*&
&*2+x2*x3**3*x4**3
dpdx5(409) = 2*x1*x2**3*x5*x6**2+2*x1**3*x2*x5*x6**2+2*x2*x3**2*x&
&4**3*x5+2*x2**3*x3**2*x4*x5
dpdx5(410) = x1*x2**3*x6**3+3*x1**3*x2*x5**2*x6+3*x2*x3*x4**3*x5*&
&*2+x2**3*x3**3*x4
dpdx5(411) = 3*x1*x2**3*x4*x5**2
dpdx5(412) = x1**3*x4*x6**3+3*x2**3*x4*x5**2*x6+3*x1*x2**3*x3*x5*&
&*2+x1*x3**3*x4**3
dpdx5(413) = x1*x4**3*x6**3+3*x2*x4**3*x5**2*x6+3*x1**3*x2*x3*x5*&
&*2+x1**3*x3**3*x4
dpdx5(414) = 2*x2**3*x4*x5*x6**2+2*x1**3*x4*x5*x6**2+2*x1*x3**2*x&
&4**3*x5+2*x1*x2**3*x3**2*x5
dpdx5(415) = 2*x2*x4**3*x5*x6**2+2*x1*x4**3*x5*x6**2+2*x1**3*x3**&
&2*x4*x5+2*x1**3*x2*x3**2*x5
dpdx5(416) = x2**3*x4*x6**3+3*x1**3*x4*x5**2*x6+3*x1*x3*x4**3*x5*&
&*2+x1*x2**3*x3**3
dpdx5(417) = x2*x4**3*x6**3+3*x1*x4**3*x5**2*x6+3*x1**3*x3*x4*x5*&
&*2+x1**3*x2*x3**3
dpdx5(418) = 3*x1*x2*x4**3*x5**2+3*x1**3*x2*x4*x5**2
dpdx5(419) = x1**3*x2**2*x6**2+2*x1**2*x2**3*x5*x6+2*x2**3*x3*x4*&
&*2*x5+x2**2*x3**2*x4**3
dpdx5(420) = x1**2*x2**3*x6**2+2*x1**3*x2**2*x5*x6+2*x2**2*x3*x4*&
&*3*x5+x2**3*x3**2*x4**2
dpdx5(421) = 2*x1*x2**3*x4**2*x5+2*x1**2*x2**3*x4*x5
dpdx5(422) = 2*x1*x2**2*x4**3*x5+2*x1**3*x2**2*x4*x5
dpdx5(423) = x1**2*x2**3*x4**2
dpdx5(424) = x1**3*x4**2*x6**2+2*x2**3*x4**2*x5*x6+2*x1**2*x2**3*&
&x3*x5+x1**2*x3**2*x4**3
dpdx5(425) = x1**2*x4**3*x6**2+2*x2**2*x4**3*x5*x6+2*x1**3*x2**2*&
&x3*x5+x1**3*x3**2*x4**2
dpdx5(426) = x1**2*x2**2*x4**3+x1**3*x2**2*x4**2
dpdx5(427) = x2**3*x4**2*x6**2+2*x1**3*x4**2*x5*x6+2*x1**2*x3*x4*&
&*3*x5+x1**2*x2**3*x3**2
dpdx5(428) = x2**2*x4**3*x6**2+2*x1**2*x4**3*x5*x6+2*x1**3*x3*x4*&
&*2*x5+x1**3*x2**2*x3**2
dpdx5(429) = 2*x1**2*x2*x4**3*x5+2*x1**3*x2*x4**2*x5
dpdx5(430) = x1**3*x2**3*x6+x2**3*x3*x4**3
dpdx5(431) = x1*x2**3*x4**3+x1**3*x2**3*x4
dpdx5(432) = x2**3*x4**3*x6+x1**3*x4**3*x6+x1**3*x3*x4**3+x1**3*x&
&2**3*x3
dpdx5(433) = x1**3*x2*x4**3
dpdx5(434) = x1**4*x3*x6**2+2*x2**4*x3*x5*x6+x3**2*x4**4*x6
dpdx5(435) = x3*x4**4*x6**2+x2**4*x3*x6**2+2*x3*x4**4*x5*x6+2*x1*&
&*4*x3*x5*x6+x2**4*x3**2*x6+x1**4*x3**2*x6
dpdx5(436) = x1**4*x2*x6**2+2*x1*x2**4*x5*x6+2*x2**4*x3*x4*x5+x2*&
&x3**2*x4**4
dpdx5(437) = x1*x2**4*x6**2+2*x1**4*x2*x5*x6+2*x2*x3*x4**4*x5+x2*&
&*4*x3**2*x4
dpdx5(438) = 2*x1*x2**4*x4*x5
dpdx5(439) = x1**4*x4*x6**2+2*x2**4*x4*x5*x6+2*x1*x2**4*x3*x5+x1*&
&x3**2*x4**4
dpdx5(440) = x1*x4**4*x6**2+2*x2*x4**4*x5*x6+2*x1**4*x2*x3*x5+x1*&
&*4*x3**2*x4
dpdx5(441) = x2**4*x4*x6**2+2*x1**4*x4*x5*x6+2*x1*x3*x4**4*x5+x1*&
&x2**4*x3**2
dpdx5(442) = x2*x4**4*x6**2+2*x1*x4**4*x5*x6+2*x1**4*x3*x4*x5+x1*&
&*4*x2*x3**2
dpdx5(443) = 2*x1*x2*x4**4*x5+2*x1**4*x2*x4*x5
dpdx5(444) = x1**2*x2**4*x6+x1**4*x2**2*x6+x2**2*x3*x4**4+x2**4*x&
&3*x4**2
dpdx5(445) = x1*x2**4*x4**2+x1**2*x2**4*x4
dpdx5(446) = x1*x2**2*x4**4+x1**4*x2**2*x4
dpdx5(447) = x2**4*x4**2*x6+x1**4*x4**2*x6+x1**2*x3*x4**4+x1**2*x&
&2**4*x3
dpdx5(448) = x2**2*x4**4*x6+x1**2*x4**4*x6+x1**4*x3*x4**2+x1**4*x&
&2**2*x3
dpdx5(449) = x1**2*x2*x4**4+x1**4*x2*x4**2
dpdx5(450) = x3*x4**5*x6+x2**5*x3*x6+x1**5*x3*x6
dpdx5(451) = x1*x2**5*x6+x1**5*x2*x6+x2*x3*x4**5+x2**5*x3*x4
dpdx5(452) = x1*x2**5*x4
dpdx5(453) = x2**5*x4*x6+x1**5*x4*x6+x1*x3*x4**5+x1*x2**5*x3
dpdx5(454) = x2*x4**5*x6+x1*x4**5*x6+x1**5*x3*x4+x1**5*x2*x3
dpdx5(455) = x1*x2*x4**5+x1**5*x2*x4
dpdx5(456) = 3*x3**2*x5**2*x6**3+2*x3**3*x5*x6**3+3*x3**3*x5**2*x&
&6**2
dpdx5(457) = 2*x3**2*x5*x6**4+4*x3**2*x5**3*x6**2+2*x3**4*x5*x6**&
&2
dpdx5(458) = 3*x3*x5**2*x6**4+x3**3*x6**4+4*x3*x5**3*x6**3+x3**4*&
&x6**3+4*x3**3*x5**3*x6+3*x3**4*x5**2*x6
dpdx5(459) = 2*x3*x5*x6**5+x3**2*x6**5+5*x3*x5**4*x6**2+x3**5*x6*&
&*2+5*x3**2*x5**4*x6+2*x3**5*x5*x6
dpdx5(460) = x3*x6**6+6*x3*x5**5*x6+x3**6*x6
dpdx5(461) = x1*x6**6+6*x2*x5**5*x6+6*x2*x3*x5**5+x3**6*x4
dpdx5(462) = 2*x1*x5*x6**5+5*x2*x5**4*x6**2+5*x2*x3**2*x5**4+2*x3&
&**5*x4*x5
dpdx5(463) = 3*x1*x5**2*x6**4+4*x2*x5**3*x6**3+4*x2*x3**3*x5**3+3&
&*x3**4*x4*x5**2
dpdx5(464) = 3*x2*x5**2*x6**4+4*x1*x5**3*x6**3+4*x3**3*x4*x5**3+3&
&*x2*x3**4*x5**2
dpdx5(465) = 3*x4*x5**2*x6**4+4*x4*x5**3*x6**3+4*x1*x3**3*x5**3+3&
&*x1*x3**4*x5**2
dpdx5(466) = 2*x2*x5*x6**5+5*x1*x5**4*x6**2+5*x3**2*x4*x5**4+2*x2&
&*x3**5*x5
dpdx5(467) = 2*x4*x5*x6**5+5*x4*x5**4*x6**2+5*x1*x3**2*x5**4+2*x1&
&*x3**5*x5
dpdx5(468) = x2*x6**6+6*x1*x5**5*x6+6*x3*x4*x5**5+x2*x3**6
dpdx5(469) = x4*x6**6+6*x4*x5**5*x6+6*x1*x3*x5**5+x1*x3**6
dpdx5(470) = 6*x2*x4*x5**5+6*x1*x2*x5**5
dpdx5(471) = 6*x1*x4*x5**5
dpdx5(472) = x1**2*x6**5+5*x2**2*x5**4*x6+5*x2**2*x3*x5**4+x3**5*&
&x4**2
dpdx5(473) = 2*x1**2*x5*x6**4+4*x2**2*x5**3*x6**2+4*x2**2*x3**2*x&
&5**3+2*x3**4*x4**2*x5
dpdx5(474) = 3*x2**2*x5**2*x6**3+3*x1**2*x5**2*x6**3+3*x3**3*x4**&
&2*x5**2+3*x2**2*x3**3*x5**2
dpdx5(475) = 3*x4**2*x5**2*x6**3+3*x1**2*x3**3*x5**2
dpdx5(476) = 2*x2**2*x5*x6**4+4*x1**2*x5**3*x6**2+4*x3**2*x4**2*x&
&5**3+2*x2**2*x3**4*x5
dpdx5(477) = 2*x4**2*x5*x6**4+4*x4**2*x5**3*x6**2+4*x1**2*x3**2*x&
&5**3+2*x1**2*x3**4*x5
dpdx5(478) = x2**2*x6**5+5*x1**2*x5**4*x6+5*x3*x4**2*x5**4+x2**2*&
&x3**5
dpdx5(479) = x4**2*x6**5+5*x4**2*x5**4*x6+5*x1**2*x3*x5**4+x1**2*&
&x3**5
dpdx5(480) = 5*x2**2*x4*x5**4+5*x1*x2**2*x5**4
dpdx5(481) = 5*x2*x4**2*x5**4+5*x1**2*x2*x5**4
dpdx5(482) = 5*x1*x4**2*x5**4+5*x1**2*x4*x5**4
dpdx5(483) = 4*x2**2*x4**2*x5**3+4*x1**2*x2**2*x5**3
dpdx5(484) = 4*x1**2*x4**2*x5**3
dpdx5(485) = x1**3*x6**4+4*x2**3*x5**3*x6+4*x2**3*x3*x5**3+x3**4*&
&x4**3
dpdx5(486) = 2*x1**3*x5*x6**3+3*x2**3*x5**2*x6**2+3*x2**3*x3**2*x&
&5**2+2*x3**3*x4**3*x5
dpdx5(487) = 2*x2**3*x5*x6**3+3*x1**3*x5**2*x6**2+3*x3**2*x4**3*x&
&5**2+2*x2**3*x3**3*x5
dpdx5(488) = 2*x4**3*x5*x6**3+3*x4**3*x5**2*x6**2+3*x1**3*x3**2*x&
&5**2+2*x1**3*x3**3*x5
dpdx5(489) = x2**3*x6**4+4*x1**3*x5**3*x6+4*x3*x4**3*x5**3+x2**3*&
&x3**4
dpdx5(490) = x4**3*x6**4+4*x4**3*x5**3*x6+4*x1**3*x3*x5**3+x1**3*&
&x3**4
dpdx5(491) = 4*x2**3*x4*x5**3+4*x1*x2**3*x5**3
dpdx5(492) = 4*x2*x4**3*x5**3+4*x1**3*x2*x5**3
dpdx5(493) = 4*x1*x4**3*x5**3+4*x1**3*x4*x5**3
dpdx5(494) = 3*x2**3*x4**2*x5**2+3*x1**2*x2**3*x5**2
dpdx5(495) = 3*x2**2*x4**3*x5**2+3*x1**3*x2**2*x5**2
dpdx5(496) = 3*x1**2*x4**3*x5**2+3*x1**3*x4**2*x5**2
dpdx5(497) = 2*x2**3*x4**3*x5+2*x1**3*x2**3*x5
dpdx5(498) = 0
dpdx5(499) = 2*x1**3*x4**3*x5
dpdx5(500) = x1**4*x6**3+3*x2**4*x5**2*x6+3*x2**4*x3*x5**2+x3**3*&
&x4**4
dpdx5(501) = 2*x2**4*x5*x6**2+2*x1**4*x5*x6**2+2*x3**2*x4**4*x5+2&
&*x2**4*x3**2*x5
dpdx5(502) = 2*x4**4*x5*x6**2+2*x1**4*x3**2*x5
dpdx5(503) = x2**4*x6**3+3*x1**4*x5**2*x6+3*x3*x4**4*x5**2+x2**4*&
&x3**3
dpdx5(504) = x4**4*x6**3+3*x4**4*x5**2*x6+3*x1**4*x3*x5**2+x1**4*&
&x3**3
dpdx5(505) = 3*x2**4*x4*x5**2+3*x1*x2**4*x5**2
dpdx5(506) = 3*x2*x4**4*x5**2+3*x1**4*x2*x5**2
dpdx5(507) = 3*x1*x4**4*x5**2+3*x1**4*x4*x5**2
dpdx5(508) = 2*x2**4*x4**2*x5+2*x1**2*x2**4*x5
dpdx5(509) = 2*x2**2*x4**4*x5+2*x1**4*x2**2*x5
dpdx5(510) = 0
dpdx5(511) = 2*x1**2*x4**4*x5+2*x1**4*x4**2*x5
dpdx5(512) = x2**4*x4**3+x1**3*x2**4
dpdx5(513) = x2**3*x4**4+x1**4*x2**3
dpdx5(514) = 0
dpdx5(515) = x1**3*x4**4+x1**4*x4**3
dpdx5(516) = x1**5*x6**2+2*x2**5*x5*x6+2*x2**5*x3*x5+x3**2*x4**5
dpdx5(517) = x2**5*x6**2+2*x1**5*x5*x6+2*x3*x4**5*x5+x2**5*x3**2
dpdx5(518) = x4**5*x6**2+2*x4**5*x5*x6+2*x1**5*x3*x5+x1**5*x3**2
dpdx5(519) = 2*x2**5*x4*x5+2*x1*x2**5*x5
dpdx5(520) = 2*x2*x4**5*x5+2*x1**5*x2*x5
dpdx5(521) = 2*x1*x4**5*x5+2*x1**5*x4*x5
dpdx5(522) = x2**5*x4**2+x1**2*x2**5
dpdx5(523) = x2**2*x4**5+x1**5*x2**2
dpdx5(524) = 0
dpdx5(525) = x1**2*x4**5+x1**5*x4**2
dpdx5(526) = x2**6*x6+x1**6*x6+x3*x4**6+x2**6*x3
dpdx5(527) = x4**6*x6+x1**6*x3
dpdx5(528) = x2**6*x4+x1*x2**6
dpdx5(529) = x2*x4**6+x1**6*x2
dpdx5(530) = 0
dpdx5(531) = x1*x4**6+x1**6*x4
dpdx5(532) = 4*x5**3*x6**4+4*x3**4*x5**3
dpdx5(533) = 3*x5**2*x6**5+5*x5**4*x6**3+5*x3**3*x5**4+3*x3**5*x5&
&**2
dpdx5(534) = 2*x5*x6**6+6*x5**5*x6**2+6*x3**2*x5**5+2*x3**6*x5
dpdx5(535) = x6**7+7*x5**6*x6+7*x3*x5**6+x3**7
dpdx5(536) = 7*x4*x5**6+7*x1*x5**6
dpdx5(537) = 6*x4**2*x5**5+6*x1**2*x5**5
dpdx5(538) = 5*x4**3*x5**4+5*x1**3*x5**4
dpdx5(539) = 4*x4**4*x5**3+4*x1**4*x5**3
dpdx5(540) = 0
dpdx5(541) = 3*x4**5*x5**2+3*x1**5*x5**2
dpdx5(542) = 0
dpdx5(543) = 2*x4**6*x5+2*x1**6*x5
dpdx5(544) = 0
dpdx5(545) = x4**7+x1**7
dpdx5(546) = 0
dpdx6(1) = 0
dpdx6(2) = x5+x3
dpdx6(3) = x4+x2
dpdx6(4) = 0
dpdx6(5) = x3*x5
dpdx6(6) = x2*x5+x1*x5+x3*x4+x1*x3
dpdx6(7) = x4*x5+x2*x3
dpdx6(8) = x1*x4+x1*x2
dpdx6(9) = 0
dpdx6(10) = x2*x4
dpdx6(11) = 2*x5*x6+2*x3*x6+x5**2+x3**2
dpdx6(12) = 2*x4*x6+2*x2*x6
dpdx6(13) = x4**2+x2**2
dpdx6(14) = 0
dpdx6(15) = x3*x4*x5+x2*x3*x5+x1*x3*x5
dpdx6(16) = x1*x2*x5+x1*x3*x4
dpdx6(17) = x2*x4*x5+x1*x4*x5+x2*x3*x4+x1*x2*x3
dpdx6(18) = x1*x2*x4
dpdx6(19) = 2*x3*x5*x6+x3*x5**2+x3**2*x5
dpdx6(20) = 2*x1*x5*x6+2*x1*x3*x6+x2*x5**2+x3**2*x4
dpdx6(21) = 2*x2*x5*x6+2*x3*x4*x6+x1*x5**2+x1*x3**2
dpdx6(22) = 2*x4*x5*x6+2*x2*x3*x6+x4*x5**2+x2*x3**2
dpdx6(23) = 2*x1*x4*x6+2*x1*x2*x6
dpdx6(24) = 2*x2*x4*x6
dpdx6(25) = x2**2*x5+x1**2*x5+x3*x4**2+x1**2*x3
dpdx6(26) = x4**2*x5+x2**2*x3
dpdx6(27) = x1**2*x4+x1**2*x2
dpdx6(28) = x1*x4**2+x1*x2**2
dpdx6(29) = 0
dpdx6(30) = x2*x4**2+x2**2*x4
dpdx6(31) = 2*x5**2*x6+2*x3**2*x6
dpdx6(32) = 3*x5*x6**2+3*x3*x6**2+x5**3+x3**3
dpdx6(33) = 3*x4*x6**2+3*x2*x6**2
dpdx6(34) = 2*x4**2*x6+2*x2**2*x6
dpdx6(35) = 0
dpdx6(36) = x4**3+x2**3
dpdx6(37) = 0
dpdx6(38) = x2*x3*x4*x5+x1*x3*x4*x5+x1*x2*x3*x5
dpdx6(39) = x1*x2*x4*x5+x1*x2*x3*x4
dpdx6(40) = 2*x1*x3*x5*x6+x2*x3*x5**2+x3**2*x4*x5
dpdx6(41) = 2*x3*x4*x5*x6+2*x2*x3*x5*x6+x3*x4*x5**2+x1*x3*x5**2+x&
&2*x3**2*x5+x1*x3**2*x5
dpdx6(42) = 2*x1*x2*x5*x6+2*x1*x3*x4*x6+x1*x2*x5**2+x1*x3**2*x4
dpdx6(43) = 2*x1*x4*x5*x6+2*x1*x2*x3*x6+x2*x4*x5**2+x2*x3**2*x4
dpdx6(44) = 2*x2*x4*x5*x6+2*x2*x3*x4*x6+x1*x4*x5**2+x1*x2*x3**2
dpdx6(45) = 2*x1*x2*x4*x6
dpdx6(46) = x3*x4**2*x5+x2**2*x3*x5+x1**2*x3*x5
dpdx6(47) = x1*x2**2*x5+x1**2*x2*x5+x1*x3*x4**2+x1**2*x3*x4
dpdx6(48) = x1**2*x2*x4
dpdx6(49) = x2**2*x4*x5+x1**2*x4*x5+x2*x3*x4**2+x1**2*x2*x3
dpdx6(50) = x2*x4**2*x5+x1*x4**2*x5+x2**2*x3*x4+x1*x2**2*x3
dpdx6(51) = x1*x2*x4**2+x1*x2**2*x4
dpdx6(52) = 2*x3*x5**2*x6+2*x3**2*x5*x6+x3**2*x5**2
dpdx6(53) = 3*x3*x5*x6**2+x3*x5**3+x3**3*x5
dpdx6(54) = 3*x1*x5*x6**2+3*x1*x3*x6**2+x2*x5**3+x3**3*x4
dpdx6(55) = 2*x2*x5**2*x6+2*x1*x5**2*x6+2*x3**2*x4*x6+2*x1*x3**2*&
&x6
dpdx6(56) = 2*x4*x5**2*x6+2*x2*x3**2*x6
dpdx6(57) = 3*x2*x5*x6**2+3*x3*x4*x6**2+x1*x5**3+x1*x3**3
dpdx6(58) = 3*x4*x5*x6**2+3*x2*x3*x6**2+x4*x5**3+x2*x3**3
dpdx6(59) = 3*x1*x4*x6**2+3*x1*x2*x6**2
dpdx6(60) = 3*x2*x4*x6**2
dpdx6(61) = 2*x1**2*x5*x6+2*x1**2*x3*x6+x2**2*x5**2+x3**2*x4**2
dpdx6(62) = 2*x2**2*x5*x6+2*x3*x4**2*x6+x1**2*x5**2+x1**2*x3**2
dpdx6(63) = 2*x4**2*x5*x6+2*x2**2*x3*x6+x4**2*x5**2+x2**2*x3**2
dpdx6(64) = 2*x1**2*x4*x6+2*x1**2*x2*x6
dpdx6(65) = 2*x1*x4**2*x6+2*x1*x2**2*x6
dpdx6(66) = 2*x2*x4**2*x6+2*x2**2*x4*x6
dpdx6(67) = x1**2*x4**2+x1**2*x2**2
dpdx6(68) = 0
dpdx6(69) = x2**2*x4**2
dpdx6(70) = x2**3*x5+x1**3*x5+x3*x4**3+x1**3*x3
dpdx6(71) = x4**3*x5+x2**3*x3
dpdx6(72) = x1**3*x4+x1**3*x2
dpdx6(73) = x1*x4**3+x1*x2**3
dpdx6(74) = 0
dpdx6(75) = x2*x4**3+x2**3*x4
dpdx6(76) = 3*x5**2*x6**2+3*x3**2*x6**2+2*x5**3*x6+2*x3**3*x6
dpdx6(77) = 4*x5*x6**3+4*x3*x6**3+x5**4+x3**4
dpdx6(78) = 4*x4*x6**3+4*x2*x6**3
dpdx6(79) = 3*x4**2*x6**2+3*x2**2*x6**2
dpdx6(80) = 2*x4**3*x6+2*x2**3*x6
dpdx6(81) = 0
dpdx6(82) = x4**4+x2**4
dpdx6(83) = 0
dpdx6(84) = x1*x2*x3*x4*x5
dpdx6(85) = 2*x1*x3*x4*x5*x6+2*x1*x2*x3*x5*x6+x2*x3*x4*x5**2+x1*x&
&2*x3*x5**2+x2*x3**2*x4*x5+x1*x3**2*x4*x5
dpdx6(86) = 2*x2*x3*x4*x5*x6+x1*x3*x4*x5**2+x1*x2*x3**2*x5
dpdx6(87) = 2*x1*x2*x4*x5*x6+2*x1*x2*x3*x4*x6+x1*x2*x4*x5**2+x1*x&
&2*x3**2*x4
dpdx6(88) = x2*x3*x4**2*x5+x1*x3*x4**2*x5+x2**2*x3*x4*x5+x1**2*x3&
&*x4*x5+x1*x2**2*x3*x5+x1**2*x2*x3*x5
dpdx6(89) = x1*x2**2*x4*x5+x1**2*x2*x4*x5+x1*x2*x3*x4**2+x1**2*x2&
&*x3*x4
dpdx6(90) = x1*x2*x4**2*x5+x1*x2**2*x3*x4
dpdx6(91) = 3*x1*x3*x5*x6**2+x2*x3*x5**3+x3**3*x4*x5
dpdx6(92) = 2*x2*x3*x5**2*x6+2*x1*x3*x5**2*x6+2*x3**2*x4*x5*x6+2*&
&x1*x3**2*x5*x6+x3**2*x4*x5**2+x2*x3**2*x5**2
dpdx6(93) = 2*x3*x4*x5**2*x6+2*x2*x3**2*x5*x6+x1*x3**2*x5**2
dpdx6(94) = 3*x3*x4*x5*x6**2+3*x2*x3*x5*x6**2+x3*x4*x5**3+x1*x3*x&
&5**3+x2*x3**3*x5+x1*x3**3*x5
dpdx6(95) = 2*x1*x2*x5**2*x6+2*x1*x3**2*x4*x6
dpdx6(96) = 3*x1*x2*x5*x6**2+3*x1*x3*x4*x6**2+x1*x2*x5**3+x1*x3**&
&3*x4
dpdx6(97) = 3*x1*x4*x5*x6**2+3*x1*x2*x3*x6**2+x2*x4*x5**3+x2*x3**&
&3*x4
dpdx6(98) = 2*x2*x4*x5**2*x6+2*x1*x4*x5**2*x6+2*x2*x3**2*x4*x6+2*&
&x1*x2*x3**2*x6
dpdx6(99) = 3*x2*x4*x5*x6**2+3*x2*x3*x4*x6**2+x1*x4*x5**3+x1*x2*x&
&3**3
dpdx6(100) = 3*x1*x2*x4*x6**2
dpdx6(101) = 2*x1**2*x3*x5*x6+x2**2*x3*x5**2+x3**2*x4**2*x5
dpdx6(102) = 2*x3*x4**2*x5*x6+2*x2**2*x3*x5*x6+x3*x4**2*x5**2+x1*&
&*2*x3*x5**2+x2**2*x3**2*x5+x1**2*x3**2*x5
dpdx6(103) = 2*x1**2*x2*x5*x6+2*x1**2*x3*x4*x6+x1*x2**2*x5**2+x1*&
&x3**2*x4**2
dpdx6(104) = 2*x1*x2**2*x5*x6+2*x1*x3*x4**2*x6+x1**2*x2*x5**2+x1*&
&*2*x3**2*x4
dpdx6(105) = 2*x1**2*x2*x4*x6
dpdx6(106) = 2*x1**2*x4*x5*x6+2*x1**2*x2*x3*x6+x2**2*x4*x5**2+x2*&
&x3**2*x4**2
dpdx6(107) = 2*x1*x4**2*x5*x6+2*x1*x2**2*x3*x6+x2*x4**2*x5**2+x2*&
&*2*x3**2*x4
dpdx6(108) = 2*x2**2*x4*x5*x6+2*x2*x3*x4**2*x6+x1**2*x4*x5**2+x1*&
&*2*x2*x3**2
dpdx6(109) = 2*x2*x4**2*x5*x6+2*x2**2*x3*x4*x6+x1*x4**2*x5**2+x1*&
&x2**2*x3**2
dpdx6(110) = 2*x1*x2*x4**2*x6+2*x1*x2**2*x4*x6
dpdx6(111) = x1**2*x2**2*x5+x1**2*x3*x4**2
dpdx6(112) = x1**2*x2*x4**2+x1**2*x2**2*x4
dpdx6(113) = x2**2*x4**2*x5+x1**2*x4**2*x5+x2**2*x3*x4**2+x1**2*x&
&2**2*x3
dpdx6(114) = x1*x2**2*x4**2
dpdx6(115) = x3*x4**3*x5+x2**3*x3*x5+x1**3*x3*x5
dpdx6(116) = x1*x2**3*x5+x1**3*x2*x5+x1*x3*x4**3+x1**3*x3*x4
dpdx6(117) = x1**3*x2*x4
dpdx6(118) = x2**3*x4*x5+x1**3*x4*x5+x2*x3*x4**3+x1**3*x2*x3
dpdx6(119) = x2*x4**3*x5+x1*x4**3*x5+x2**3*x3*x4+x1*x2**3*x3
dpdx6(120) = x1*x2*x4**3+x1*x2**3*x4
dpdx6(121) = 2*x3**2*x5**2*x6
dpdx6(122) = 3*x3*x5**2*x6**2+3*x3**2*x5*x6**2+2*x3*x5**3*x6+2*x3&
&**3*x5*x6+x3**2*x5**3+x3**3*x5**2
dpdx6(123) = 4*x3*x5*x6**3+x3*x5**4+x3**4*x5
dpdx6(124) = 4*x1*x5*x6**3+4*x1*x3*x6**3+x2*x5**4+x3**4*x4
dpdx6(125) = 3*x1*x5**2*x6**2+3*x1*x3**2*x6**2+2*x2*x5**3*x6+2*x3&
&**3*x4*x6
dpdx6(126) = 3*x2*x5**2*x6**2+3*x3**2*x4*x6**2+2*x1*x5**3*x6+2*x1&
&*x3**3*x6
dpdx6(127) = 3*x4*x5**2*x6**2+3*x2*x3**2*x6**2+2*x4*x5**3*x6+2*x2&
&*x3**3*x6
dpdx6(128) = 4*x2*x5*x6**3+4*x3*x4*x6**3+x1*x5**4+x1*x3**4
dpdx6(129) = 4*x4*x5*x6**3+4*x2*x3*x6**3+x4*x5**4+x2*x3**4
dpdx6(130) = 4*x1*x4*x6**3+4*x1*x2*x6**3
dpdx6(131) = 4*x2*x4*x6**3
dpdx6(132) = 3*x1**2*x5*x6**2+3*x1**2*x3*x6**2+x2**2*x5**3+x3**3*&
&x4**2
dpdx6(133) = 2*x2**2*x5**2*x6+2*x1**2*x5**2*x6+2*x3**2*x4**2*x6+2&
&*x1**2*x3**2*x6
dpdx6(134) = 2*x4**2*x5**2*x6+2*x2**2*x3**2*x6
dpdx6(135) = 3*x2**2*x5*x6**2+3*x3*x4**2*x6**2+x1**2*x5**3+x1**2*&
&x3**3
dpdx6(136) = 3*x4**2*x5*x6**2+3*x2**2*x3*x6**2+x4**2*x5**3+x2**2*&
&x3**3
dpdx6(137) = 3*x1**2*x4*x6**2+3*x1**2*x2*x6**2
dpdx6(138) = 3*x1*x4**2*x6**2+3*x1*x2**2*x6**2
dpdx6(139) = 3*x2*x4**2*x6**2+3*x2**2*x4*x6**2
dpdx6(140) = 2*x1**2*x4**2*x6+2*x1**2*x2**2*x6
dpdx6(141) = 0
dpdx6(142) = 2*x2**2*x4**2*x6
dpdx6(143) = 2*x1**3*x5*x6+2*x1**3*x3*x6+x2**3*x5**2+x3**2*x4**3
dpdx6(144) = 2*x2**3*x5*x6+2*x3*x4**3*x6+x1**3*x5**2+x1**3*x3**2
dpdx6(145) = 2*x4**3*x5*x6+2*x2**3*x3*x6+x4**3*x5**2+x2**3*x3**2
dpdx6(146) = 2*x1**3*x4*x6+2*x1**3*x2*x6
dpdx6(147) = 2*x1*x4**3*x6+2*x1*x2**3*x6
dpdx6(148) = 2*x2*x4**3*x6+2*x2**3*x4*x6
dpdx6(149) = x1**3*x4**2+x1**3*x2**2
dpdx6(150) = x1**2*x4**3+x1**2*x2**3
dpdx6(151) = 0
dpdx6(152) = x2**2*x4**3+x2**3*x4**2
dpdx6(153) = x2**4*x5+x1**4*x5+x3*x4**4+x1**4*x3
dpdx6(154) = x4**4*x5+x2**4*x3
dpdx6(155) = x1**4*x4+x1**4*x2
dpdx6(156) = x1*x4**4+x1*x2**4
dpdx6(157) = 0
dpdx6(158) = x2*x4**4+x2**4*x4
dpdx6(159) = 3*x5**3*x6**2+3*x3**3*x6**2
dpdx6(160) = 4*x5**2*x6**3+4*x3**2*x6**3+2*x5**4*x6+2*x3**4*x6
dpdx6(161) = 5*x5*x6**4+5*x3*x6**4+x5**5+x3**5
dpdx6(162) = 5*x4*x6**4+5*x2*x6**4
dpdx6(163) = 4*x4**2*x6**3+4*x2**2*x6**3
dpdx6(164) = 3*x4**3*x6**2+3*x2**3*x6**2
dpdx6(165) = 0
dpdx6(166) = 2*x4**4*x6+2*x2**4*x6
dpdx6(167) = 0
dpdx6(168) = x4**5+x2**5
dpdx6(169) = 0
dpdx6(170) = 2*x1*x2*x3*x4*x5*x6+x1*x2*x3*x4*x5**2+x1*x2*x3**2*x4&
&*x5
dpdx6(171) = x1*x2*x3*x4**2*x5+x1*x2**2*x3*x4*x5+x1**2*x2*x3*x4*x&
&5
dpdx6(172) = 2*x1*x2*x3*x5**2*x6+2*x1*x3**2*x4*x5*x6+x2*x3**2*x4*&
&x5**2
dpdx6(173) = 3*x1*x3*x4*x5*x6**2+3*x1*x2*x3*x5*x6**2+x2*x3*x4*x5*&
&*3+x1*x2*x3*x5**3+x2*x3**3*x4*x5+x1*x3**3*x4*x5
dpdx6(174) = 2*x2*x3*x4*x5**2*x6+2*x1*x3*x4*x5**2*x6+2*x2*x3**2*x&
&4*x5*x6+2*x1*x2*x3**2*x5*x6+x1*x3**2*x4*x5**2+x1*x2*x3**2*x5**2
dpdx6(175) = 2*x1*x2*x4*x5**2*x6+2*x1*x2*x3**2*x4*x6
dpdx6(176) = 3*x2*x3*x4*x5*x6**2+x1*x3*x4*x5**3+x1*x2*x3**3*x5
dpdx6(177) = 3*x1*x2*x4*x5*x6**2+3*x1*x2*x3*x4*x6**2+x1*x2*x4*x5*&
&*3+x1*x2*x3**3*x4
dpdx6(178) = 2*x1**2*x3*x4*x5*x6+2*x1**2*x2*x3*x5*x6+x2**2*x3*x4*&
&x5**2+x1*x2**2*x3*x5**2+x2*x3**2*x4**2*x5+x1*x3**2*x4**2*x5
dpdx6(179) = 2*x1*x3*x4**2*x5*x6+2*x1*x2**2*x3*x5*x6+x2*x3*x4**2*&
&x5**2+x1**2*x2*x3*x5**2+x2**2*x3**2*x4*x5+x1**2*x3**2*x4*x5
dpdx6(180) = 2*x1**2*x2*x4*x5*x6+2*x1**2*x2*x3*x4*x6+x1*x2**2*x4*&
&x5**2+x1*x2*x3**2*x4**2
dpdx6(181) = 2*x2*x3*x4**2*x5*x6+2*x2**2*x3*x4*x5*x6+x1*x3*x4**2*&
&x5**2+x1**2*x3*x4*x5**2+x1*x2**2*x3**2*x5+x1**2*x2*x3**2*x5
dpdx6(182) = 2*x1*x2**2*x4*x5*x6+2*x1*x2*x3*x4**2*x6+x1**2*x2*x4*&
&x5**2+x1**2*x2*x3**2*x4
dpdx6(183) = 2*x1*x2*x4**2*x5*x6+2*x1*x2**2*x3*x4*x6+x1*x2*x4**2*&
&x5**2+x1*x2**2*x3**2*x4
dpdx6(184) = x1**2*x2**2*x4*x5+x1**2*x2*x3*x4**2
dpdx6(185) = x2**2*x3*x4**2*x5+x1**2*x3*x4**2*x5+x1**2*x2**2*x3*x&
&5
dpdx6(186) = x1*x2**2*x4**2*x5+x1**2*x2*x4**2*x5+x1*x2**2*x3*x4**&
&2+x1**2*x2**2*x3*x4
dpdx6(187) = x2*x3*x4**3*x5+x1*x3*x4**3*x5+x2**3*x3*x4*x5+x1**3*x&
&3*x4*x5+x1*x2**3*x3*x5+x1**3*x2*x3*x5
dpdx6(188) = x1*x2**3*x4*x5+x1**3*x2*x4*x5+x1*x2*x3*x4**3+x1**3*x&
&2*x3*x4
dpdx6(189) = x1*x2*x4**3*x5+x1*x2**3*x3*x4
dpdx6(190) = 4*x1*x3*x5*x6**3+x2*x3*x5**4+x3**4*x4*x5
dpdx6(191) = 3*x1*x3*x5**2*x6**2+3*x1*x3**2*x5*x6**2+2*x2*x3*x5**&
&3*x6+2*x3**3*x4*x5*x6+x2*x3**2*x5**3+x3**3*x4*x5**2
dpdx6(192) = 2*x3**2*x4*x5**2*x6+2*x2*x3**2*x5**2*x6+2*x1*x3**2*x&
&5**2*x6
dpdx6(193) = 3*x2*x3*x5**2*x6**2+3*x3**2*x4*x5*x6**2+2*x1*x3*x5**&
&3*x6+2*x1*x3**3*x5*x6+x3**2*x4*x5**3+x2*x3**3*x5**2
dpdx6(194) = 3*x3*x4*x5**2*x6**2+3*x2*x3**2*x5*x6**2+2*x3*x4*x5**&
&3*x6+2*x2*x3**3*x5*x6+x1*x3**2*x5**3+x1*x3**3*x5**2
dpdx6(195) = 4*x3*x4*x5*x6**3+4*x2*x3*x5*x6**3+x3*x4*x5**4+x1*x3*&
&x5**4+x2*x3**4*x5+x1*x3**4*x5
dpdx6(196) = 3*x1*x2*x5**2*x6**2+3*x1*x3**2*x4*x6**2+2*x1*x2*x5**&
&3*x6+2*x1*x3**3*x4*x6
dpdx6(197) = 4*x1*x2*x5*x6**3+4*x1*x3*x4*x6**3+x1*x2*x5**4+x1*x3*&
&*4*x4
dpdx6(198) = 4*x1*x4*x5*x6**3+4*x1*x2*x3*x6**3+x2*x4*x5**4+x2*x3*&
&*4*x4
dpdx6(199) = 3*x1*x4*x5**2*x6**2+3*x1*x2*x3**2*x6**2+2*x2*x4*x5**&
&3*x6+2*x2*x3**3*x4*x6
dpdx6(200) = 3*x2*x4*x5**2*x6**2+3*x2*x3**2*x4*x6**2+2*x1*x4*x5**&
&3*x6+2*x1*x2*x3**3*x6
dpdx6(201) = 4*x2*x4*x5*x6**3+4*x2*x3*x4*x6**3+x1*x4*x5**4+x1*x2*&
&x3**4
dpdx6(202) = 4*x1*x2*x4*x6**3
dpdx6(203) = 3*x1**2*x3*x5*x6**2+x2**2*x3*x5**3+x3**3*x4**2*x5
dpdx6(204) = 2*x2**2*x3*x5**2*x6+2*x1**2*x3*x5**2*x6+2*x3**2*x4**&
&2*x5*x6+2*x1**2*x3**2*x5*x6+x3**2*x4**2*x5**2+x2**2*x3**2*x5**2
dpdx6(205) = 2*x3*x4**2*x5**2*x6+2*x2**2*x3**2*x5*x6+x1**2*x3**2*&
&x5**2
dpdx6(206) = 3*x3*x4**2*x5*x6**2+3*x2**2*x3*x5*x6**2+x3*x4**2*x5*&
&*3+x1**2*x3*x5**3+x2**2*x3**3*x5+x1**2*x3**3*x5
dpdx6(207) = 3*x1**2*x2*x5*x6**2+3*x1**2*x3*x4*x6**2+x1*x2**2*x5*&
&*3+x1*x3**3*x4**2
dpdx6(208) = 2*x1*x2**2*x5**2*x6+2*x1**2*x2*x5**2*x6+2*x1*x3**2*x&
&4**2*x6+2*x1**2*x3**2*x4*x6
dpdx6(209) = 3*x1*x2**2*x5*x6**2+3*x1*x3*x4**2*x6**2+x1**2*x2*x5*&
&*3+x1**2*x3**3*x4
dpdx6(210) = 3*x1**2*x2*x4*x6**2
dpdx6(211) = 3*x1**2*x4*x5*x6**2+3*x1**2*x2*x3*x6**2+x2**2*x4*x5*&
&*3+x2*x3**3*x4**2
dpdx6(212) = 3*x1*x4**2*x5*x6**2+3*x1*x2**2*x3*x6**2+x2*x4**2*x5*&
&*3+x2**2*x3**3*x4
dpdx6(213) = 2*x2**2*x4*x5**2*x6+2*x1**2*x4*x5**2*x6+2*x2*x3**2*x&
&4**2*x6+2*x1**2*x2*x3**2*x6
dpdx6(214) = 2*x2*x4**2*x5**2*x6+2*x1*x4**2*x5**2*x6+2*x2**2*x3**&
&2*x4*x6+2*x1*x2**2*x3**2*x6
dpdx6(215) = 3*x2**2*x4*x5*x6**2+3*x2*x3*x4**2*x6**2+x1**2*x4*x5*&
&*3+x1**2*x2*x3**3
dpdx6(216) = 3*x2*x4**2*x5*x6**2+3*x2**2*x3*x4*x6**2+x1*x4**2*x5*&
&*3+x1*x2**2*x3**3
dpdx6(217) = 3*x1*x2*x4**2*x6**2+3*x1*x2**2*x4*x6**2
dpdx6(218) = 2*x1**2*x2**2*x5*x6+2*x1**2*x3*x4**2*x6+x1**2*x2**2*&
&x5**2+x1**2*x3**2*x4**2
dpdx6(219) = 2*x1**2*x2*x4**2*x6+2*x1**2*x2**2*x4*x6
dpdx6(220) = 2*x1**2*x4**2*x5*x6+2*x1**2*x2**2*x3*x6+x2**2*x4**2*&
&x5**2+x2**2*x3**2*x4**2
dpdx6(221) = x1**2*x2**2*x4**2
dpdx6(222) = 2*x2**2*x4**2*x5*x6+2*x2**2*x3*x4**2*x6+x1**2*x4**2*&
&x5**2+x1**2*x2**2*x3**2
dpdx6(223) = 2*x1*x2**2*x4**2*x6
dpdx6(224) = 2*x1**3*x3*x5*x6+x2**3*x3*x5**2+x3**2*x4**3*x5
dpdx6(225) = 2*x3*x4**3*x5*x6+2*x2**3*x3*x5*x6+x3*x4**3*x5**2+x1*&
&*3*x3*x5**2+x2**3*x3**2*x5+x1**3*x3**2*x5
dpdx6(226) = 2*x1**3*x2*x5*x6+2*x1**3*x3*x4*x6+x1*x2**3*x5**2+x1*&
&x3**2*x4**3
dpdx6(227) = 2*x1*x2**3*x5*x6+2*x1*x3*x4**3*x6+x1**3*x2*x5**2+x1*&
&*3*x3**2*x4
dpdx6(228) = 2*x1**3*x2*x4*x6
dpdx6(229) = 2*x1**3*x4*x5*x6+2*x1**3*x2*x3*x6+x2**3*x4*x5**2+x2*&
&x3**2*x4**3
dpdx6(230) = 2*x1*x4**3*x5*x6+2*x1*x2**3*x3*x6+x2*x4**3*x5**2+x2*&
&*3*x3**2*x4
dpdx6(231) = 2*x2**3*x4*x5*x6+2*x2*x3*x4**3*x6+x1**3*x4*x5**2+x1*&
&*3*x2*x3**2
dpdx6(232) = 2*x2*x4**3*x5*x6+2*x2**3*x3*x4*x6+x1*x4**3*x5**2+x1*&
&x2**3*x3**2
dpdx6(233) = 2*x1*x2*x4**3*x6+2*x1*x2**3*x4*x6
dpdx6(234) = x1**2*x2**3*x5+x1**3*x2**2*x5+x1**2*x3*x4**3+x1**3*x&
&3*x4**2
dpdx6(235) = x1**3*x2*x4**2+x1**3*x2**2*x4
dpdx6(236) = x1**2*x2*x4**3+x1**2*x2**3*x4
dpdx6(237) = x2**3*x4**2*x5+x1**3*x4**2*x5+x2**2*x3*x4**3+x1**3*x&
&2**2*x3
dpdx6(238) = x2**2*x4**3*x5+x1**2*x4**3*x5+x2**3*x3*x4**2+x1**2*x&
&2**3*x3
dpdx6(239) = x1*x2**2*x4**3+x1*x2**3*x4**2
dpdx6(240) = x3*x4**4*x5+x2**4*x3*x5+x1**4*x3*x5
dpdx6(241) = x1*x2**4*x5+x1**4*x2*x5+x1*x3*x4**4+x1**4*x3*x4
dpdx6(242) = x1**4*x2*x4
dpdx6(243) = x2**4*x4*x5+x1**4*x4*x5+x2*x3*x4**4+x1**4*x2*x3
dpdx6(244) = x2*x4**4*x5+x1*x4**4*x5+x2**4*x3*x4+x1*x2**4*x3
dpdx6(245) = x1*x2*x4**4+x1*x2**4*x4
dpdx6(246) = 3*x3**2*x5**2*x6**2+2*x3**2*x5**3*x6+2*x3**3*x5**2*x&
&6
dpdx6(247) = 3*x3*x5**3*x6**2+3*x3**3*x5*x6**2+x3**3*x5**3
dpdx6(248) = 4*x3*x5**2*x6**3+4*x3**2*x5*x6**3+2*x3*x5**4*x6+2*x3&
&**4*x5*x6+x3**2*x5**4+x3**4*x5**2
dpdx6(249) = 5*x3*x5*x6**4+x3*x5**5+x3**5*x5
dpdx6(250) = 5*x1*x5*x6**4+5*x1*x3*x6**4+x2*x5**5+x3**5*x4
dpdx6(251) = 4*x1*x5**2*x6**3+4*x1*x3**2*x6**3+2*x2*x5**4*x6+2*x3&
&**4*x4*x6
dpdx6(252) = 3*x2*x5**3*x6**2+3*x1*x5**3*x6**2+3*x3**3*x4*x6**2+3&
&*x1*x3**3*x6**2
dpdx6(253) = 3*x4*x5**3*x6**2+3*x2*x3**3*x6**2
dpdx6(254) = 4*x2*x5**2*x6**3+4*x3**2*x4*x6**3+2*x1*x5**4*x6+2*x1&
&*x3**4*x6
dpdx6(255) = 4*x4*x5**2*x6**3+4*x2*x3**2*x6**3+2*x4*x5**4*x6+2*x2&
&*x3**4*x6
dpdx6(256) = 5*x2*x5*x6**4+5*x3*x4*x6**4+x1*x5**5+x1*x3**5
dpdx6(257) = 5*x4*x5*x6**4+5*x2*x3*x6**4+x4*x5**5+x2*x3**5
dpdx6(258) = 5*x1*x4*x6**4+5*x1*x2*x6**4
dpdx6(259) = 5*x2*x4*x6**4
dpdx6(260) = 4*x1**2*x5*x6**3+4*x1**2*x3*x6**3+x2**2*x5**4+x3**4*&
&x4**2
dpdx6(261) = 3*x1**2*x5**2*x6**2+3*x1**2*x3**2*x6**2+2*x2**2*x5**&
&3*x6+2*x3**3*x4**2*x6
dpdx6(262) = 3*x2**2*x5**2*x6**2+3*x3**2*x4**2*x6**2+2*x1**2*x5**&
&3*x6+2*x1**2*x3**3*x6
dpdx6(263) = 3*x4**2*x5**2*x6**2+3*x2**2*x3**2*x6**2+2*x4**2*x5**&
&3*x6+2*x2**2*x3**3*x6
dpdx6(264) = 4*x2**2*x5*x6**3+4*x3*x4**2*x6**3+x1**2*x5**4+x1**2*&
&x3**4
dpdx6(265) = 4*x4**2*x5*x6**3+4*x2**2*x3*x6**3+x4**2*x5**4+x2**2*&
&x3**4
dpdx6(266) = 4*x1**2*x4*x6**3+4*x1**2*x2*x6**3
dpdx6(267) = 4*x1*x4**2*x6**3+4*x1*x2**2*x6**3
dpdx6(268) = 4*x2*x4**2*x6**3+4*x2**2*x4*x6**3
dpdx6(269) = 3*x1**2*x4**2*x6**2+3*x1**2*x2**2*x6**2
dpdx6(270) = 3*x2**2*x4**2*x6**2
dpdx6(271) = 3*x1**3*x5*x6**2+3*x1**3*x3*x6**2+x2**3*x5**3+x3**3*&
&x4**3
dpdx6(272) = 2*x2**3*x5**2*x6+2*x1**3*x5**2*x6+2*x3**2*x4**3*x6+2&
&*x1**3*x3**2*x6
dpdx6(273) = 2*x4**3*x5**2*x6+2*x2**3*x3**2*x6
dpdx6(274) = 3*x2**3*x5*x6**2+3*x3*x4**3*x6**2+x1**3*x5**3+x1**3*&
&x3**3
dpdx6(275) = 3*x4**3*x5*x6**2+3*x2**3*x3*x6**2+x4**3*x5**3+x2**3*&
&x3**3
dpdx6(276) = 3*x1**3*x4*x6**2+3*x1**3*x2*x6**2
dpdx6(277) = 3*x1*x4**3*x6**2+3*x1*x2**3*x6**2
dpdx6(278) = 3*x2*x4**3*x6**2+3*x2**3*x4*x6**2
dpdx6(279) = 2*x1**3*x4**2*x6+2*x1**3*x2**2*x6
dpdx6(280) = 2*x1**2*x4**3*x6+2*x1**2*x2**3*x6
dpdx6(281) = 0
dpdx6(282) = 2*x2**2*x4**3*x6+2*x2**3*x4**2*x6
dpdx6(283) = x1**3*x4**3+x1**3*x2**3
dpdx6(284) = 0
dpdx6(285) = x2**3*x4**3
dpdx6(286) = 2*x1**4*x5*x6+2*x1**4*x3*x6+x2**4*x5**2+x3**2*x4**4
dpdx6(287) = 2*x2**4*x5*x6+2*x3*x4**4*x6+x1**4*x5**2+x1**4*x3**2
dpdx6(288) = 2*x4**4*x5*x6+2*x2**4*x3*x6+x4**4*x5**2+x2**4*x3**2
dpdx6(289) = 2*x1**4*x4*x6+2*x1**4*x2*x6
dpdx6(290) = 2*x1*x4**4*x6+2*x1*x2**4*x6
dpdx6(291) = 2*x2*x4**4*x6+2*x2**4*x4*x6
dpdx6(292) = x1**4*x4**2+x1**4*x2**2
dpdx6(293) = x1**2*x4**4+x1**2*x2**4
dpdx6(294) = 0
dpdx6(295) = x2**2*x4**4+x2**4*x4**2
dpdx6(296) = x2**5*x5+x1**5*x5+x3*x4**5+x1**5*x3
dpdx6(297) = x4**5*x5+x2**5*x3
dpdx6(298) = x1**5*x4+x1**5*x2
dpdx6(299) = x1*x4**5+x1*x2**5
dpdx6(300) = 0
dpdx6(301) = x2*x4**5+x2**5*x4
dpdx6(302) = 4*x5**3*x6**3+4*x3**3*x6**3+3*x5**4*x6**2+3*x3**4*x6&
&**2
dpdx6(303) = 5*x5**2*x6**4+5*x3**2*x6**4+2*x5**5*x6+2*x3**5*x6
dpdx6(304) = 6*x5*x6**5+6*x3*x6**5+x5**6+x3**6
dpdx6(305) = 6*x4*x6**5+6*x2*x6**5
dpdx6(306) = 5*x4**2*x6**4+5*x2**2*x6**4
dpdx6(307) = 4*x4**3*x6**3+4*x2**3*x6**3
dpdx6(308) = 3*x4**4*x6**2+3*x2**4*x6**2
dpdx6(309) = 0
dpdx6(310) = 2*x4**5*x6+2*x2**5*x6
dpdx6(311) = 0
dpdx6(312) = x4**6+x2**6
dpdx6(313) = 0
dpdx6(314) = 2*x1*x2*x3*x4*x5**2*x6+2*x1*x2*x3**2*x4*x5*x6+x1*x2*&
&x3**2*x4*x5**2
dpdx6(315) = 3*x1*x2*x3*x4*x5*x6**2+x1*x2*x3*x4*x5**3+x1*x2*x3**3&
&*x4*x5
dpdx6(316) = 2*x1**2*x2*x3*x4*x5*x6+x1*x2**2*x3*x4*x5**2+x1*x2*x3&
&**2*x4**2*x5
dpdx6(317) = 2*x1*x2*x3*x4**2*x5*x6+2*x1*x2**2*x3*x4*x5*x6+x1*x2*&
&x3*x4**2*x5**2+x1**2*x2*x3*x4*x5**2+x1*x2**2*x3**2*x4*x5+x1**2*x2&
&*x3**2*x4*x5
dpdx6(318) = x1*x2**2*x3*x4**2*x5+x1**2*x2*x3*x4**2*x5+x1**2*x2**&
&2*x3*x4*x5
dpdx6(319) = x1*x2*x3*x4**3*x5+x1*x2**3*x3*x4*x5+x1**3*x2*x3*x4*x&
&5
dpdx6(320) = 3*x1*x2*x3*x5**2*x6**2+3*x1*x3**2*x4*x5*x6**2+2*x1*x&
&2*x3*x5**3*x6+2*x1*x3**3*x4*x5*x6+x2*x3**2*x4*x5**3+x2*x3**3*x4*x&
&5**2
dpdx6(321) = 4*x1*x3*x4*x5*x6**3+4*x1*x2*x3*x5*x6**3+x2*x3*x4*x5*&
&*4+x1*x2*x3*x5**4+x2*x3**4*x4*x5+x1*x3**4*x4*x5
dpdx6(322) = 2*x2*x3**2*x4*x5**2*x6+2*x1*x3**2*x4*x5**2*x6+2*x1*x&
&2*x3**2*x5**2*x6
dpdx6(323) = 3*x1*x3*x4*x5**2*x6**2+3*x1*x2*x3**2*x5*x6**2+2*x2*x&
&3*x4*x5**3*x6+2*x2*x3**3*x4*x5*x6+x1*x2*x3**2*x5**3+x1*x3**3*x4*x&
&5**2
dpdx6(324) = 3*x2*x3*x4*x5**2*x6**2+3*x2*x3**2*x4*x5*x6**2+2*x1*x&
&3*x4*x5**3*x6+2*x1*x2*x3**3*x5*x6+x1*x3**2*x4*x5**3+x1*x2*x3**3*x&
&5**2
dpdx6(325) = 3*x1*x2*x4*x5**2*x6**2+3*x1*x2*x3**2*x4*x6**2+2*x1*x&
&2*x4*x5**3*x6+2*x1*x2*x3**3*x4*x6
dpdx6(326) = 4*x2*x3*x4*x5*x6**3+x1*x3*x4*x5**4+x1*x2*x3**4*x5
dpdx6(327) = 4*x1*x2*x4*x5*x6**3+4*x1*x2*x3*x4*x6**3+x1*x2*x4*x5*&
&*4+x1*x2*x3**4*x4
dpdx6(328) = 3*x1**2*x3*x4*x5*x6**2+3*x1**2*x2*x3*x5*x6**2+x2**2*&
&x3*x4*x5**3+x1*x2**2*x3*x5**3+x2*x3**3*x4**2*x5+x1*x3**3*x4**2*x5
dpdx6(329) = 2*x1*x2**2*x3*x5**2*x6+2*x1**2*x2*x3*x5**2*x6+2*x1*x&
&3**2*x4**2*x5*x6+2*x1**2*x3**2*x4*x5*x6+x2*x3**2*x4**2*x5**2+x2**&
&2*x3**2*x4*x5**2
dpdx6(330) = 3*x1*x3*x4**2*x5*x6**2+3*x1*x2**2*x3*x5*x6**2+x2*x3*&
&x4**2*x5**3+x1**2*x2*x3*x5**3+x2**2*x3**3*x4*x5+x1**2*x3**3*x4*x5
dpdx6(331) = 3*x1**2*x2*x4*x5*x6**2+3*x1**2*x2*x3*x4*x6**2+x1*x2*&
&*2*x4*x5**3+x1*x2*x3**3*x4**2
dpdx6(332) = 2*x2**2*x3*x4*x5**2*x6+2*x1**2*x3*x4*x5**2*x6+2*x2*x&
&3**2*x4**2*x5*x6+2*x1**2*x2*x3**2*x5*x6+x1*x3**2*x4**2*x5**2+x1*x&
&2**2*x3**2*x5**2
dpdx6(333) = 2*x2*x3*x4**2*x5**2*x6+2*x1*x3*x4**2*x5**2*x6+2*x2**&
&2*x3**2*x4*x5*x6+2*x1*x2**2*x3**2*x5*x6+x1**2*x3**2*x4*x5**2+x1**&
&2*x2*x3**2*x5**2
dpdx6(334) = 2*x1*x2**2*x4*x5**2*x6+2*x1**2*x2*x4*x5**2*x6+2*x1*x&
&2*x3**2*x4**2*x6+2*x1**2*x2*x3**2*x4*x6
dpdx6(335) = 2*x1*x2*x4**2*x5**2*x6+2*x1*x2**2*x3**2*x4*x6
dpdx6(336) = 3*x2*x3*x4**2*x5*x6**2+3*x2**2*x3*x4*x5*x6**2+x1*x3*&
&x4**2*x5**3+x1**2*x3*x4*x5**3+x1*x2**2*x3**3*x5+x1**2*x2*x3**3*x5
dpdx6(337) = 3*x1*x2**2*x4*x5*x6**2+3*x1*x2*x3*x4**2*x6**2+x1**2*&
&x2*x4*x5**3+x1**2*x2*x3**3*x4
dpdx6(338) = 3*x1*x2*x4**2*x5*x6**2+3*x1*x2**2*x3*x4*x6**2+x1*x2*&
&x4**2*x5**3+x1*x2**2*x3**3*x4
dpdx6(339) = 2*x1**2*x2**2*x4*x5*x6+2*x1**2*x2*x3*x4**2*x6+x1**2*&
&x2**2*x4*x5**2+x1**2*x2*x3**2*x4**2
dpdx6(340) = 2*x1**2*x3*x4**2*x5*x6+2*x1**2*x2**2*x3*x5*x6+x2**2*&
&x3*x4**2*x5**2+x1**2*x2**2*x3*x5**2+x2**2*x3**2*x4**2*x5+x1**2*x3&
&**2*x4**2*x5
dpdx6(341) = 2*x1**2*x2*x4**2*x5*x6+2*x1**2*x2**2*x3*x4*x6+x1*x2*&
&*2*x4**2*x5**2+x1*x2**2*x3**2*x4**2
dpdx6(342) = x1**2*x2**2*x4**2*x5+x1**2*x2**2*x3*x4**2
dpdx6(343) = 2*x2**2*x3*x4**2*x5*x6+x1**2*x3*x4**2*x5**2+x1**2*x2&
&**2*x3**2*x5
dpdx6(344) = 2*x1*x2**2*x4**2*x5*x6+2*x1*x2**2*x3*x4**2*x6+x1**2*&
&x2*x4**2*x5**2+x1**2*x2**2*x3**2*x4
dpdx6(345) = 2*x1**3*x3*x4*x5*x6+2*x1**3*x2*x3*x5*x6+x2**3*x3*x4*&
&x5**2+x1*x2**3*x3*x5**2+x2*x3**2*x4**3*x5+x1*x3**2*x4**3*x5
dpdx6(346) = 2*x1*x3*x4**3*x5*x6+2*x1*x2**3*x3*x5*x6+x2*x3*x4**3*&
&x5**2+x1**3*x2*x3*x5**2+x2**3*x3**2*x4*x5+x1**3*x3**2*x4*x5
dpdx6(347) = 2*x1**3*x2*x4*x5*x6+2*x1**3*x2*x3*x4*x6+x1*x2**3*x4*&
&x5**2+x1*x2*x3**2*x4**3
dpdx6(348) = 2*x2*x3*x4**3*x5*x6+2*x2**3*x3*x4*x5*x6+x1*x3*x4**3*&
&x5**2+x1**3*x3*x4*x5**2+x1*x2**3*x3**2*x5+x1**3*x2*x3**2*x5
dpdx6(349) = 2*x1*x2**3*x4*x5*x6+2*x1*x2*x3*x4**3*x6+x1**3*x2*x4*&
&x5**2+x1**3*x2*x3**2*x4
dpdx6(350) = 2*x1*x2*x4**3*x5*x6+2*x1*x2**3*x3*x4*x6+x1*x2*x4**3*&
&x5**2+x1*x2**3*x3**2*x4
dpdx6(351) = x1**2*x2**3*x4*x5+x1**3*x2**2*x4*x5+x1**2*x2*x3*x4**&
&3+x1**3*x2*x3*x4**2
dpdx6(352) = x2**2*x3*x4**3*x5+x1**2*x3*x4**3*x5+x2**3*x3*x4**2*x&
&5+x1**3*x3*x4**2*x5+x1**2*x2**3*x3*x5+x1**3*x2**2*x3*x5
dpdx6(353) = x1*x2**3*x4**2*x5+x1**3*x2*x4**2*x5+x1*x2**2*x3*x4**&
&3+x1**3*x2**2*x3*x4
dpdx6(354) = x1*x2**2*x4**3*x5+x1**2*x2*x4**3*x5+x1*x2**3*x3*x4**&
&2+x1**2*x2**3*x3*x4
dpdx6(355) = x2*x3*x4**4*x5+x1*x3*x4**4*x5+x2**4*x3*x4*x5+x1**4*x&
&3*x4*x5+x1*x2**4*x3*x5+x1**4*x2*x3*x5
dpdx6(356) = x1*x2**4*x4*x5+x1**4*x2*x4*x5+x1*x2*x3*x4**4+x1**4*x&
&2*x3*x4
dpdx6(357) = x1*x2*x4**4*x5+x1*x2**4*x3*x4
dpdx6(358) = 5*x1*x3*x5*x6**4+x2*x3*x5**5+x3**5*x4*x5
dpdx6(359) = 4*x1*x3*x5**2*x6**3+4*x1*x3**2*x5*x6**3+2*x2*x3*x5**&
&4*x6+2*x3**4*x4*x5*x6+x2*x3**2*x5**4+x3**4*x4*x5**2
dpdx6(360) = 3*x1*x3**2*x5**2*x6**2+2*x2*x3**2*x5**3*x6+2*x3**3*x&
&4*x5**2*x6
dpdx6(361) = 3*x2*x3*x5**3*x6**2+3*x1*x3*x5**3*x6**2+3*x3**3*x4*x&
&5*x6**2+3*x1*x3**3*x5*x6**2+x3**3*x4*x5**3+x2*x3**3*x5**3
dpdx6(362) = 3*x3**2*x4*x5**2*x6**2+3*x2*x3**2*x5**2*x6**2+2*x3**&
&2*x4*x5**3*x6+2*x1*x3**2*x5**3*x6+2*x2*x3**3*x5**2*x6+2*x1*x3**3*&
&x5**2*x6
dpdx6(363) = 3*x3*x4*x5**3*x6**2+3*x2*x3**3*x5*x6**2+x1*x3**3*x5*&
&*3
dpdx6(364) = 4*x2*x3*x5**2*x6**3+4*x3**2*x4*x5*x6**3+2*x1*x3*x5**&
&4*x6+2*x1*x3**4*x5*x6+x3**2*x4*x5**4+x2*x3**4*x5**2
dpdx6(365) = 4*x3*x4*x5**2*x6**3+4*x2*x3**2*x5*x6**3+2*x3*x4*x5**&
&4*x6+2*x2*x3**4*x5*x6+x1*x3**2*x5**4+x1*x3**4*x5**2
dpdx6(366) = 5*x3*x4*x5*x6**4+5*x2*x3*x5*x6**4+x3*x4*x5**5+x1*x3*&
&x5**5+x2*x3**5*x5+x1*x3**5*x5
dpdx6(367) = 3*x1*x2*x5**3*x6**2+3*x1*x3**3*x4*x6**2
dpdx6(368) = 4*x1*x2*x5**2*x6**3+4*x1*x3**2*x4*x6**3+2*x1*x2*x5**&
&4*x6+2*x1*x3**4*x4*x6
dpdx6(369) = 5*x1*x2*x5*x6**4+5*x1*x3*x4*x6**4+x1*x2*x5**5+x1*x3*&
&*5*x4
dpdx6(370) = 5*x1*x4*x5*x6**4+5*x1*x2*x3*x6**4+x2*x4*x5**5+x2*x3*&
&*5*x4
dpdx6(371) = 4*x1*x4*x5**2*x6**3+4*x1*x2*x3**2*x6**3+2*x2*x4*x5**&
&4*x6+2*x2*x3**4*x4*x6
dpdx6(372) = 3*x2*x4*x5**3*x6**2+3*x1*x4*x5**3*x6**2+3*x2*x3**3*x&
&4*x6**2+3*x1*x2*x3**3*x6**2
dpdx6(373) = 4*x2*x4*x5**2*x6**3+4*x2*x3**2*x4*x6**3+2*x1*x4*x5**&
&4*x6+2*x1*x2*x3**4*x6
dpdx6(374) = 5*x2*x4*x5*x6**4+5*x2*x3*x4*x6**4+x1*x4*x5**5+x1*x2*&
&x3**5
dpdx6(375) = 5*x1*x2*x4*x6**4
dpdx6(376) = 4*x1**2*x3*x5*x6**3+x2**2*x3*x5**4+x3**4*x4**2*x5
dpdx6(377) = 3*x1**2*x3*x5**2*x6**2+3*x1**2*x3**2*x5*x6**2+2*x2**&
&2*x3*x5**3*x6+2*x3**3*x4**2*x5*x6+x2**2*x3**2*x5**3+x3**3*x4**2*x&
&5**2
dpdx6(378) = 2*x3**2*x4**2*x5**2*x6+2*x2**2*x3**2*x5**2*x6+2*x1**&
&2*x3**2*x5**2*x6
dpdx6(379) = 3*x2**2*x3*x5**2*x6**2+3*x3**2*x4**2*x5*x6**2+2*x1**&
&2*x3*x5**3*x6+2*x1**2*x3**3*x5*x6+x3**2*x4**2*x5**3+x2**2*x3**3*x&
&5**2
dpdx6(380) = 3*x3*x4**2*x5**2*x6**2+3*x2**2*x3**2*x5*x6**2+2*x3*x&
&4**2*x5**3*x6+2*x2**2*x3**3*x5*x6+x1**2*x3**2*x5**3+x1**2*x3**3*x&
&5**2
dpdx6(381) = 4*x3*x4**2*x5*x6**3+4*x2**2*x3*x5*x6**3+x3*x4**2*x5*&
&*4+x1**2*x3*x5**4+x2**2*x3**4*x5+x1**2*x3**4*x5
dpdx6(382) = 4*x1**2*x2*x5*x6**3+4*x1**2*x3*x4*x6**3+x1*x2**2*x5*&
&*4+x1*x3**4*x4**2
dpdx6(383) = 3*x1**2*x2*x5**2*x6**2+3*x1**2*x3**2*x4*x6**2+2*x1*x&
&2**2*x5**3*x6+2*x1*x3**3*x4**2*x6
dpdx6(384) = 3*x1*x2**2*x5**2*x6**2+3*x1*x3**2*x4**2*x6**2+2*x1**&
&2*x2*x5**3*x6+2*x1**2*x3**3*x4*x6
dpdx6(385) = 4*x1*x2**2*x5*x6**3+4*x1*x3*x4**2*x6**3+x1**2*x2*x5*&
&*4+x1**2*x3**4*x4
dpdx6(386) = 4*x1**2*x2*x4*x6**3
dpdx6(387) = 4*x1**2*x4*x5*x6**3+4*x1**2*x2*x3*x6**3+x2**2*x4*x5*&
&*4+x2*x3**4*x4**2
dpdx6(388) = 4*x1*x4**2*x5*x6**3+4*x1*x2**2*x3*x6**3+x2*x4**2*x5*&
&*4+x2**2*x3**4*x4
dpdx6(389) = 3*x1**2*x4*x5**2*x6**2+3*x1**2*x2*x3**2*x6**2+2*x2**&
&2*x4*x5**3*x6+2*x2*x3**3*x4**2*x6
dpdx6(390) = 3*x1*x4**2*x5**2*x6**2+3*x1*x2**2*x3**2*x6**2+2*x2*x&
&4**2*x5**3*x6+2*x2**2*x3**3*x4*x6
dpdx6(391) = 3*x2**2*x4*x5**2*x6**2+3*x2*x3**2*x4**2*x6**2+2*x1**&
&2*x4*x5**3*x6+2*x1**2*x2*x3**3*x6
dpdx6(392) = 3*x2*x4**2*x5**2*x6**2+3*x2**2*x3**2*x4*x6**2+2*x1*x&
&4**2*x5**3*x6+2*x1*x2**2*x3**3*x6
dpdx6(393) = 4*x2**2*x4*x5*x6**3+4*x2*x3*x4**2*x6**3+x1**2*x4*x5*&
&*4+x1**2*x2*x3**4
dpdx6(394) = 4*x2*x4**2*x5*x6**3+4*x2**2*x3*x4*x6**3+x1*x4**2*x5*&
&*4+x1*x2**2*x3**4
dpdx6(395) = 4*x1*x2*x4**2*x6**3+4*x1*x2**2*x4*x6**3
dpdx6(396) = 2*x1**2*x2**2*x5**2*x6+2*x1**2*x3**2*x4**2*x6
dpdx6(397) = 3*x1**2*x2**2*x5*x6**2+3*x1**2*x3*x4**2*x6**2+x1**2*&
&x2**2*x5**3+x1**2*x3**3*x4**2
dpdx6(398) = 3*x1**2*x2*x4**2*x6**2+3*x1**2*x2**2*x4*x6**2
dpdx6(399) = 3*x1**2*x4**2*x5*x6**2+3*x1**2*x2**2*x3*x6**2+x2**2*&
&x4**2*x5**3+x2**2*x3**3*x4**2
dpdx6(400) = 2*x2**2*x4**2*x5**2*x6+2*x1**2*x4**2*x5**2*x6+2*x2**&
&2*x3**2*x4**2*x6+2*x1**2*x2**2*x3**2*x6
dpdx6(401) = 2*x1**2*x2**2*x4**2*x6
dpdx6(402) = 3*x2**2*x4**2*x5*x6**2+3*x2**2*x3*x4**2*x6**2+x1**2*&
&x4**2*x5**3+x1**2*x2**2*x3**3
dpdx6(403) = 3*x1*x2**2*x4**2*x6**2
dpdx6(404) = 3*x1**3*x3*x5*x6**2+x2**3*x3*x5**3+x3**3*x4**3*x5
dpdx6(405) = 2*x2**3*x3*x5**2*x6+2*x1**3*x3*x5**2*x6+2*x3**2*x4**&
&3*x5*x6+2*x1**3*x3**2*x5*x6+x3**2*x4**3*x5**2+x2**3*x3**2*x5**2
dpdx6(406) = 2*x3*x4**3*x5**2*x6+2*x2**3*x3**2*x5*x6+x1**3*x3**2*&
&x5**2
dpdx6(407) = 3*x3*x4**3*x5*x6**2+3*x2**3*x3*x5*x6**2+x3*x4**3*x5*&
&*3+x1**3*x3*x5**3+x2**3*x3**3*x5+x1**3*x3**3*x5
dpdx6(408) = 3*x1**3*x2*x5*x6**2+3*x1**3*x3*x4*x6**2+x1*x2**3*x5*&
&*3+x1*x3**3*x4**3
dpdx6(409) = 2*x1*x2**3*x5**2*x6+2*x1**3*x2*x5**2*x6+2*x1*x3**2*x&
&4**3*x6+2*x1**3*x3**2*x4*x6
dpdx6(410) = 3*x1*x2**3*x5*x6**2+3*x1*x3*x4**3*x6**2+x1**3*x2*x5*&
&*3+x1**3*x3**3*x4
dpdx6(411) = 3*x1**3*x2*x4*x6**2
dpdx6(412) = 3*x1**3*x4*x5*x6**2+3*x1**3*x2*x3*x6**2+x2**3*x4*x5*&
&*3+x2*x3**3*x4**3
dpdx6(413) = 3*x1*x4**3*x5*x6**2+3*x1*x2**3*x3*x6**2+x2*x4**3*x5*&
&*3+x2**3*x3**3*x4
dpdx6(414) = 2*x2**3*x4*x5**2*x6+2*x1**3*x4*x5**2*x6+2*x2*x3**2*x&
&4**3*x6+2*x1**3*x2*x3**2*x6
dpdx6(415) = 2*x2*x4**3*x5**2*x6+2*x1*x4**3*x5**2*x6+2*x2**3*x3**&
&2*x4*x6+2*x1*x2**3*x3**2*x6
dpdx6(416) = 3*x2**3*x4*x5*x6**2+3*x2*x3*x4**3*x6**2+x1**3*x4*x5*&
&*3+x1**3*x2*x3**3
dpdx6(417) = 3*x2*x4**3*x5*x6**2+3*x2**3*x3*x4*x6**2+x1*x4**3*x5*&
&*3+x1*x2**3*x3**3
dpdx6(418) = 3*x1*x2*x4**3*x6**2+3*x1*x2**3*x4*x6**2
dpdx6(419) = 2*x1**3*x2**2*x5*x6+2*x1**3*x3*x4**2*x6+x1**2*x2**3*&
&x5**2+x1**2*x3**2*x4**3
dpdx6(420) = 2*x1**2*x2**3*x5*x6+2*x1**2*x3*x4**3*x6+x1**3*x2**2*&
&x5**2+x1**3*x3**2*x4**2
dpdx6(421) = 2*x1**3*x2*x4**2*x6+2*x1**3*x2**2*x4*x6
dpdx6(422) = 2*x1**2*x2*x4**3*x6+2*x1**2*x2**3*x4*x6
dpdx6(423) = x1**3*x2**2*x4**2
dpdx6(424) = 2*x1**3*x4**2*x5*x6+2*x1**3*x2**2*x3*x6+x2**3*x4**2*&
&x5**2+x2**2*x3**2*x4**3
dpdx6(425) = 2*x1**2*x4**3*x5*x6+2*x1**2*x2**3*x3*x6+x2**2*x4**3*&
&x5**2+x2**3*x3**2*x4**2
dpdx6(426) = x1**2*x2**2*x4**3+x1**2*x2**3*x4**2
dpdx6(427) = 2*x2**3*x4**2*x5*x6+2*x2**2*x3*x4**3*x6+x1**3*x4**2*&
&x5**2+x1**3*x2**2*x3**2
dpdx6(428) = 2*x2**2*x4**3*x5*x6+2*x2**3*x3*x4**2*x6+x1**2*x4**3*&
&x5**2+x1**2*x2**3*x3**2
dpdx6(429) = 2*x1*x2**2*x4**3*x6+2*x1*x2**3*x4**2*x6
dpdx6(430) = x1**3*x2**3*x5+x1**3*x3*x4**3
dpdx6(431) = x1**3*x2*x4**3+x1**3*x2**3*x4
dpdx6(432) = x2**3*x4**3*x5+x1**3*x4**3*x5+x2**3*x3*x4**3+x1**3*x&
&2**3*x3
dpdx6(433) = x1*x2**3*x4**3
dpdx6(434) = 2*x1**4*x3*x5*x6+x2**4*x3*x5**2+x3**2*x4**4*x5
dpdx6(435) = 2*x3*x4**4*x5*x6+2*x2**4*x3*x5*x6+x3*x4**4*x5**2+x1*&
&*4*x3*x5**2+x2**4*x3**2*x5+x1**4*x3**2*x5
dpdx6(436) = 2*x1**4*x2*x5*x6+2*x1**4*x3*x4*x6+x1*x2**4*x5**2+x1*&
&x3**2*x4**4
dpdx6(437) = 2*x1*x2**4*x5*x6+2*x1*x3*x4**4*x6+x1**4*x2*x5**2+x1*&
&*4*x3**2*x4
dpdx6(438) = 2*x1**4*x2*x4*x6
dpdx6(439) = 2*x1**4*x4*x5*x6+2*x1**4*x2*x3*x6+x2**4*x4*x5**2+x2*&
&x3**2*x4**4
dpdx6(440) = 2*x1*x4**4*x5*x6+2*x1*x2**4*x3*x6+x2*x4**4*x5**2+x2*&
&*4*x3**2*x4
dpdx6(441) = 2*x2**4*x4*x5*x6+2*x2*x3*x4**4*x6+x1**4*x4*x5**2+x1*&
&*4*x2*x3**2
dpdx6(442) = 2*x2*x4**4*x5*x6+2*x2**4*x3*x4*x6+x1*x4**4*x5**2+x1*&
&x2**4*x3**2
dpdx6(443) = 2*x1*x2*x4**4*x6+2*x1*x2**4*x4*x6
dpdx6(444) = x1**2*x2**4*x5+x1**4*x2**2*x5+x1**2*x3*x4**4+x1**4*x&
&3*x4**2
dpdx6(445) = x1**4*x2*x4**2+x1**4*x2**2*x4
dpdx6(446) = x1**2*x2*x4**4+x1**2*x2**4*x4
dpdx6(447) = x2**4*x4**2*x5+x1**4*x4**2*x5+x2**2*x3*x4**4+x1**4*x&
&2**2*x3
dpdx6(448) = x2**2*x4**4*x5+x1**2*x4**4*x5+x2**4*x3*x4**2+x1**2*x&
&2**4*x3
dpdx6(449) = x1*x2**2*x4**4+x1*x2**4*x4**2
dpdx6(450) = x3*x4**5*x5+x2**5*x3*x5+x1**5*x3*x5
dpdx6(451) = x1*x2**5*x5+x1**5*x2*x5+x1*x3*x4**5+x1**5*x3*x4
dpdx6(452) = x1**5*x2*x4
dpdx6(453) = x2**5*x4*x5+x1**5*x4*x5+x2*x3*x4**5+x1**5*x2*x3
dpdx6(454) = x2*x4**5*x5+x1*x4**5*x5+x2**5*x3*x4+x1*x2**5*x3
dpdx6(455) = x1*x2*x4**5+x1*x2**5*x4
dpdx6(456) = 3*x3**2*x5**3*x6**2+3*x3**3*x5**2*x6**2+2*x3**3*x5**&
&3*x6
dpdx6(457) = 4*x3**2*x5**2*x6**3+2*x3**2*x5**4*x6+2*x3**4*x5**2*x&
&6
dpdx6(458) = 4*x3*x5**3*x6**3+4*x3**3*x5*x6**3+3*x3*x5**4*x6**2+3&
&*x3**4*x5*x6**2+x3**3*x5**4+x3**4*x5**3
dpdx6(459) = 5*x3*x5**2*x6**4+5*x3**2*x5*x6**4+2*x3*x5**5*x6+2*x3&
&**5*x5*x6+x3**2*x5**5+x3**5*x5**2
dpdx6(460) = 6*x3*x5*x6**5+x3*x5**6+x3**6*x5
dpdx6(461) = 6*x1*x5*x6**5+6*x1*x3*x6**5+x2*x5**6+x3**6*x4
dpdx6(462) = 5*x1*x5**2*x6**4+5*x1*x3**2*x6**4+2*x2*x5**5*x6+2*x3&
&**5*x4*x6
dpdx6(463) = 4*x1*x5**3*x6**3+4*x1*x3**3*x6**3+3*x2*x5**4*x6**2+3&
&*x3**4*x4*x6**2
dpdx6(464) = 4*x2*x5**3*x6**3+4*x3**3*x4*x6**3+3*x1*x5**4*x6**2+3&
&*x1*x3**4*x6**2
dpdx6(465) = 4*x4*x5**3*x6**3+4*x2*x3**3*x6**3+3*x4*x5**4*x6**2+3&
&*x2*x3**4*x6**2
dpdx6(466) = 5*x2*x5**2*x6**4+5*x3**2*x4*x6**4+2*x1*x5**5*x6+2*x1&
&*x3**5*x6
dpdx6(467) = 5*x4*x5**2*x6**4+5*x2*x3**2*x6**4+2*x4*x5**5*x6+2*x2&
&*x3**5*x6
dpdx6(468) = 6*x2*x5*x6**5+6*x3*x4*x6**5+x1*x5**6+x1*x3**6
dpdx6(469) = 6*x4*x5*x6**5+6*x2*x3*x6**5+x4*x5**6+x2*x3**6
dpdx6(470) = 6*x1*x4*x6**5+6*x1*x2*x6**5
dpdx6(471) = 6*x2*x4*x6**5
dpdx6(472) = 5*x1**2*x5*x6**4+5*x1**2*x3*x6**4+x2**2*x5**5+x3**5*&
&x4**2
dpdx6(473) = 4*x1**2*x5**2*x6**3+4*x1**2*x3**2*x6**3+2*x2**2*x5**&
&4*x6+2*x3**4*x4**2*x6
dpdx6(474) = 3*x2**2*x5**3*x6**2+3*x1**2*x5**3*x6**2+3*x3**3*x4**&
&2*x6**2+3*x1**2*x3**3*x6**2
dpdx6(475) = 3*x4**2*x5**3*x6**2+3*x2**2*x3**3*x6**2
dpdx6(476) = 4*x2**2*x5**2*x6**3+4*x3**2*x4**2*x6**3+2*x1**2*x5**&
&4*x6+2*x1**2*x3**4*x6
dpdx6(477) = 4*x4**2*x5**2*x6**3+4*x2**2*x3**2*x6**3+2*x4**2*x5**&
&4*x6+2*x2**2*x3**4*x6
dpdx6(478) = 5*x2**2*x5*x6**4+5*x3*x4**2*x6**4+x1**2*x5**5+x1**2*&
&x3**5
dpdx6(479) = 5*x4**2*x5*x6**4+5*x2**2*x3*x6**4+x4**2*x5**5+x2**2*&
&x3**5
dpdx6(480) = 5*x1**2*x4*x6**4+5*x1**2*x2*x6**4
dpdx6(481) = 5*x1*x4**2*x6**4+5*x1*x2**2*x6**4
dpdx6(482) = 5*x2*x4**2*x6**4+5*x2**2*x4*x6**4
dpdx6(483) = 4*x1**2*x4**2*x6**3+4*x1**2*x2**2*x6**3
dpdx6(484) = 4*x2**2*x4**2*x6**3
dpdx6(485) = 4*x1**3*x5*x6**3+4*x1**3*x3*x6**3+x2**3*x5**4+x3**4*&
&x4**3
dpdx6(486) = 3*x1**3*x5**2*x6**2+3*x1**3*x3**2*x6**2+2*x2**3*x5**&
&3*x6+2*x3**3*x4**3*x6
dpdx6(487) = 3*x2**3*x5**2*x6**2+3*x3**2*x4**3*x6**2+2*x1**3*x5**&
&3*x6+2*x1**3*x3**3*x6
dpdx6(488) = 3*x4**3*x5**2*x6**2+3*x2**3*x3**2*x6**2+2*x4**3*x5**&
&3*x6+2*x2**3*x3**3*x6
dpdx6(489) = 4*x2**3*x5*x6**3+4*x3*x4**3*x6**3+x1**3*x5**4+x1**3*&
&x3**4
dpdx6(490) = 4*x4**3*x5*x6**3+4*x2**3*x3*x6**3+x4**3*x5**4+x2**3*&
&x3**4
dpdx6(491) = 4*x1**3*x4*x6**3+4*x1**3*x2*x6**3
dpdx6(492) = 4*x1*x4**3*x6**3+4*x1*x2**3*x6**3
dpdx6(493) = 4*x2*x4**3*x6**3+4*x2**3*x4*x6**3
dpdx6(494) = 3*x1**3*x4**2*x6**2+3*x1**3*x2**2*x6**2
dpdx6(495) = 3*x1**2*x4**3*x6**2+3*x1**2*x2**3*x6**2
dpdx6(496) = 3*x2**2*x4**3*x6**2+3*x2**3*x4**2*x6**2
dpdx6(497) = 2*x1**3*x4**3*x6+2*x1**3*x2**3*x6
dpdx6(498) = 0
dpdx6(499) = 2*x2**3*x4**3*x6
dpdx6(500) = 3*x1**4*x5*x6**2+3*x1**4*x3*x6**2+x2**4*x5**3+x3**3*&
&x4**4
dpdx6(501) = 2*x2**4*x5**2*x6+2*x1**4*x5**2*x6+2*x3**2*x4**4*x6+2&
&*x1**4*x3**2*x6
dpdx6(502) = 2*x4**4*x5**2*x6+2*x2**4*x3**2*x6
dpdx6(503) = 3*x2**4*x5*x6**2+3*x3*x4**4*x6**2+x1**4*x5**3+x1**4*&
&x3**3
dpdx6(504) = 3*x4**4*x5*x6**2+3*x2**4*x3*x6**2+x4**4*x5**3+x2**4*&
&x3**3
dpdx6(505) = 3*x1**4*x4*x6**2+3*x1**4*x2*x6**2
dpdx6(506) = 3*x1*x4**4*x6**2+3*x1*x2**4*x6**2
dpdx6(507) = 3*x2*x4**4*x6**2+3*x2**4*x4*x6**2
dpdx6(508) = 2*x1**4*x4**2*x6+2*x1**4*x2**2*x6
dpdx6(509) = 2*x1**2*x4**4*x6+2*x1**2*x2**4*x6
dpdx6(510) = 0
dpdx6(511) = 2*x2**2*x4**4*x6+2*x2**4*x4**2*x6
dpdx6(512) = x1**4*x4**3+x1**4*x2**3
dpdx6(513) = x1**3*x4**4+x1**3*x2**4
dpdx6(514) = 0
dpdx6(515) = x2**3*x4**4+x2**4*x4**3
dpdx6(516) = 2*x1**5*x5*x6+2*x1**5*x3*x6+x2**5*x5**2+x3**2*x4**5
dpdx6(517) = 2*x2**5*x5*x6+2*x3*x4**5*x6+x1**5*x5**2+x1**5*x3**2
dpdx6(518) = 2*x4**5*x5*x6+2*x2**5*x3*x6+x4**5*x5**2+x2**5*x3**2
dpdx6(519) = 2*x1**5*x4*x6+2*x1**5*x2*x6
dpdx6(520) = 2*x1*x4**5*x6+2*x1*x2**5*x6
dpdx6(521) = 2*x2*x4**5*x6+2*x2**5*x4*x6
dpdx6(522) = x1**5*x4**2+x1**5*x2**2
dpdx6(523) = x1**2*x4**5+x1**2*x2**5
dpdx6(524) = 0
dpdx6(525) = x2**2*x4**5+x2**5*x4**2
dpdx6(526) = x2**6*x5+x1**6*x5+x3*x4**6+x1**6*x3
dpdx6(527) = x4**6*x5+x2**6*x3
dpdx6(528) = x1**6*x4+x1**6*x2
dpdx6(529) = x1*x4**6+x1*x2**6
dpdx6(530) = 0
dpdx6(531) = x2*x4**6+x2**6*x4
dpdx6(532) = 4*x5**4*x6**3+4*x3**4*x6**3
dpdx6(533) = 5*x5**3*x6**4+5*x3**3*x6**4+3*x5**5*x6**2+3*x3**5*x6&
&**2
dpdx6(534) = 6*x5**2*x6**5+6*x3**2*x6**5+2*x5**6*x6+2*x3**6*x6
dpdx6(535) = 7*x5*x6**6+7*x3*x6**6+x5**7+x3**7
dpdx6(536) = 7*x4*x6**6+7*x2*x6**6
dpdx6(537) = 6*x4**2*x6**5+6*x2**2*x6**5
dpdx6(538) = 5*x4**3*x6**4+5*x2**3*x6**4
dpdx6(539) = 4*x4**4*x6**3+4*x2**4*x6**3
dpdx6(540) = 0
dpdx6(541) = 3*x4**5*x6**2+3*x2**5*x6**2
dpdx6(542) = 0
dpdx6(543) = 2*x4**6*x6+2*x2**6*x6
dpdx6(544) = 0
dpdx6(545) = x4**7+x2**7
dpdx6(546) = 0

end subroutine grad_ev_pol_con_8
